/* Generated by Cython 0.23.2 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_23_2"
#include <stddef.h>
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if !defined(CYTHON_USE_PYLONG_INTERNALS) && CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x02070000
#define CYTHON_USE_PYLONG_INTERNALS 1
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
#define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if PY_VERSION_HEX >= 0x030500B1
#define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
#elif CYTHON_COMPILING_IN_CPYTHON && PY_MAJOR_VERSION >= 3
typedef struct {
    unaryfunc am_await;
    unaryfunc am_aiter;
    unaryfunc am_anext;
} __Pyx_PyAsyncMethodsStruct;
#define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
#else
#define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#define __Pyx_void_to_None(void_result) (void_result, Py_INCREF(Py_None), Py_None)

#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__dipy__align__vector_fields
#define __PYX_HAVE_API__dipy__align__vector_fields
#include "string.h"
#include "stdio.h"
#include "stdlib.h"
#include "numpy/arrayobject.h"
#include "numpy/ufuncobject.h"
#include "dpy_math.h"
#include "pythread.h"
#include "pystate.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (_MSC_VER) && defined (_M_X64)
    #define __Pyx_sst_abs(value) _abs64(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;

#if !defined(CYTHON_CCOMPLEX)
  #if defined(__cplusplus)
    #define CYTHON_CCOMPLEX 1
  #elif defined(_Complex_I)
    #define CYTHON_CCOMPLEX 1
  #else
    #define CYTHON_CCOMPLEX 0
  #endif
#endif
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #include <complex>
  #else
    #include <complex.h>
  #endif
#endif
#if CYTHON_CCOMPLEX && !defined(__cplusplus) && defined(__sun__) && defined(__GNUC__)
  #undef _Complex_I
  #define _Complex_I 1.0fj
#endif


static const char *__pyx_f[] = {
  "dipy/align/vector_fields.pyx",
  "__init__.pxd",
  "stringsource",
  "type.pxd",
};
struct __pyx_memoryview_obj;
typedef struct {
  struct __pyx_memoryview_obj *memview;
  char *data;
  Py_ssize_t shape[8];
  Py_ssize_t strides[8];
  Py_ssize_t suboffsets[8];
} __Pyx_memviewslice;

#define IS_UNSIGNED(type) (((type) -1) > 0)
struct __Pyx_StructField_;
#define __PYX_BUF_FLAGS_PACKED_STRUCT (1 << 0)
typedef struct {
  const char* name;
  struct __Pyx_StructField_* fields;
  size_t size;
  size_t arraysize[8];
  int ndim;
  char typegroup;
  char is_unsigned;
  int flags;
} __Pyx_TypeInfo;
typedef struct __Pyx_StructField_ {
  __Pyx_TypeInfo* type;
  const char* name;
  size_t offset;
} __Pyx_StructField;
typedef struct {
  __Pyx_StructField* field;
  size_t parent_offset;
} __Pyx_BufFmt_StackElem;
typedef struct {
  __Pyx_StructField root;
  __Pyx_BufFmt_StackElem* head;
  size_t fmt_offset;
  size_t new_count, enc_count;
  size_t struct_alignment;
  int is_complex;
  char enc_type;
  char new_packmode;
  char enc_packmode;
  char is_valid_array;
} __Pyx_BufFmt_Context;

#include <pythread.h>
#ifndef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 1
#endif
#define __pyx_atomic_int_type int
#if CYTHON_ATOMICS && __GNUC__ >= 4 && (__GNUC_MINOR__ > 1 ||\
                    (__GNUC_MINOR__ == 1 && __GNUC_PATCHLEVEL >= 2)) &&\
                    !defined(__i386__)
    #define __pyx_atomic_incr_aligned(value, lock) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_decr_aligned(value, lock) __sync_fetch_and_sub(value, 1)
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using GNU atomics"
    #endif
#elif CYTHON_ATOMICS && defined(_MSC_VER) && 0
    #include <Windows.h>
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type LONG
    #define __pyx_atomic_incr_aligned(value, lock) InterlockedIncrement(value)
    #define __pyx_atomic_decr_aligned(value, lock) InterlockedDecrement(value)
    #ifdef __PYX_DEBUG_ATOMICS
        #pragma message ("Using MSVC atomics")
    #endif
#elif CYTHON_ATOMICS && (defined(__ICC) || defined(__INTEL_COMPILER)) && 0
    #define __pyx_atomic_incr_aligned(value, lock) _InterlockedIncrement(value)
    #define __pyx_atomic_decr_aligned(value, lock) _InterlockedDecrement(value)
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using Intel atomics"
    #endif
#else
    #undef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 0
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Not using atomics"
    #endif
#endif
typedef volatile __pyx_atomic_int_type __pyx_atomic_int;
#if CYTHON_ATOMICS
    #define __pyx_add_acquisition_count(memview)\
             __pyx_atomic_incr_aligned(__pyx_get_slice_count_pointer(memview), memview->lock)
    #define __pyx_sub_acquisition_count(memview)\
            __pyx_atomic_decr_aligned(__pyx_get_slice_count_pointer(memview), memview->lock)
#else
    #define __pyx_add_acquisition_count(memview)\
            __pyx_add_acquisition_count_locked(__pyx_get_slice_count_pointer(memview), memview->lock)
    #define __pyx_sub_acquisition_count(memview)\
            __pyx_sub_acquisition_count_locked(__pyx_get_slice_count_pointer(memview), memview->lock)
#endif


/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":725
 * # in Cython to enable them only on the right systems.
 * 
 * ctypedef npy_int8       int8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 */
typedef npy_int8 __pyx_t_5numpy_int8_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":726
 * 
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t
 */
typedef npy_int16 __pyx_t_5numpy_int16_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":727
 * ctypedef npy_int8       int8_t
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_int64      int64_t
 * #ctypedef npy_int96      int96_t
 */
typedef npy_int32 __pyx_t_5numpy_int32_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":728
 * ctypedef npy_int16      int16_t
 * ctypedef npy_int32      int32_t
 * ctypedef npy_int64      int64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_int96      int96_t
 * #ctypedef npy_int128     int128_t
 */
typedef npy_int64 __pyx_t_5numpy_int64_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":732
 * #ctypedef npy_int128     int128_t
 * 
 * ctypedef npy_uint8      uint8_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 */
typedef npy_uint8 __pyx_t_5numpy_uint8_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":733
 * 
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t
 */
typedef npy_uint16 __pyx_t_5numpy_uint16_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":734
 * ctypedef npy_uint8      uint8_t
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uint64     uint64_t
 * #ctypedef npy_uint96     uint96_t
 */
typedef npy_uint32 __pyx_t_5numpy_uint32_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":735
 * ctypedef npy_uint16     uint16_t
 * ctypedef npy_uint32     uint32_t
 * ctypedef npy_uint64     uint64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_uint96     uint96_t
 * #ctypedef npy_uint128    uint128_t
 */
typedef npy_uint64 __pyx_t_5numpy_uint64_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":739
 * #ctypedef npy_uint128    uint128_t
 * 
 * ctypedef npy_float32    float32_t             # <<<<<<<<<<<<<<
 * ctypedef npy_float64    float64_t
 * #ctypedef npy_float80    float80_t
 */
typedef npy_float32 __pyx_t_5numpy_float32_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":740
 * 
 * ctypedef npy_float32    float32_t
 * ctypedef npy_float64    float64_t             # <<<<<<<<<<<<<<
 * #ctypedef npy_float80    float80_t
 * #ctypedef npy_float128   float128_t
 */
typedef npy_float64 __pyx_t_5numpy_float64_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":749
 * # The int types are mapped a bit surprising --
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t
 */
typedef npy_long __pyx_t_5numpy_int_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":750
 * # numpy.int corresponds to 'l' and numpy.long to 'q'
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longlong   longlong_t
 * 
 */
typedef npy_longlong __pyx_t_5numpy_long_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":751
 * ctypedef npy_long       int_t
 * ctypedef npy_longlong   long_t
 * ctypedef npy_longlong   longlong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_ulong      uint_t
 */
typedef npy_longlong __pyx_t_5numpy_longlong_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":753
 * ctypedef npy_longlong   longlong_t
 * 
 * ctypedef npy_ulong      uint_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t
 */
typedef npy_ulong __pyx_t_5numpy_uint_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":754
 * 
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t             # <<<<<<<<<<<<<<
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 */
typedef npy_ulonglong __pyx_t_5numpy_ulong_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":755
 * ctypedef npy_ulong      uint_t
 * ctypedef npy_ulonglong  ulong_t
 * ctypedef npy_ulonglong  ulonglong_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_intp       intp_t
 */
typedef npy_ulonglong __pyx_t_5numpy_ulonglong_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":757
 * ctypedef npy_ulonglong  ulonglong_t
 * 
 * ctypedef npy_intp       intp_t             # <<<<<<<<<<<<<<
 * ctypedef npy_uintp      uintp_t
 * 
 */
typedef npy_intp __pyx_t_5numpy_intp_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":758
 * 
 * ctypedef npy_intp       intp_t
 * ctypedef npy_uintp      uintp_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_double     float_t
 */
typedef npy_uintp __pyx_t_5numpy_uintp_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":760
 * ctypedef npy_uintp      uintp_t
 * 
 * ctypedef npy_double     float_t             # <<<<<<<<<<<<<<
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t
 */
typedef npy_double __pyx_t_5numpy_float_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":761
 * 
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t             # <<<<<<<<<<<<<<
 * ctypedef npy_longdouble longdouble_t
 * 
 */
typedef npy_double __pyx_t_5numpy_double_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":762
 * ctypedef npy_double     float_t
 * ctypedef npy_double     double_t
 * ctypedef npy_longdouble longdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cfloat      cfloat_t
 */
typedef npy_longdouble __pyx_t_5numpy_longdouble_t;
#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< float > __pyx_t_float_complex;
  #else
    typedef float _Complex __pyx_t_float_complex;
  #endif
#else
    typedef struct { float real, imag; } __pyx_t_float_complex;
#endif

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    typedef ::std::complex< double > __pyx_t_double_complex;
  #else
    typedef double _Complex __pyx_t_double_complex;
  #endif
#else
    typedef struct { double real, imag; } __pyx_t_double_complex;
#endif


/*--- Type declarations ---*/
struct __pyx_array_obj;
struct __pyx_MemviewEnum_obj;
struct __pyx_memoryview_obj;
struct __pyx_memoryviewslice_obj;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":764
 * ctypedef npy_longdouble longdouble_t
 * 
 * ctypedef npy_cfloat      cfloat_t             # <<<<<<<<<<<<<<
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t
 */
typedef npy_cfloat __pyx_t_5numpy_cfloat_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":765
 * 
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t             # <<<<<<<<<<<<<<
 * ctypedef npy_clongdouble clongdouble_t
 * 
 */
typedef npy_cdouble __pyx_t_5numpy_cdouble_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":766
 * ctypedef npy_cfloat      cfloat_t
 * ctypedef npy_cdouble     cdouble_t
 * ctypedef npy_clongdouble clongdouble_t             # <<<<<<<<<<<<<<
 * 
 * ctypedef npy_cdouble     complex_t
 */
typedef npy_clongdouble __pyx_t_5numpy_clongdouble_t;

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":768
 * ctypedef npy_clongdouble clongdouble_t
 * 
 * ctypedef npy_cdouble     complex_t             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 */
typedef npy_cdouble __pyx_t_5numpy_complex_t;
struct __pyx_defaults;
typedef struct __pyx_defaults __pyx_defaults;
struct __pyx_defaults1;
typedef struct __pyx_defaults1 __pyx_defaults1;
struct __pyx_defaults2;
typedef struct __pyx_defaults2 __pyx_defaults2;
struct __pyx_defaults3;
typedef struct __pyx_defaults3 __pyx_defaults3;
struct __pyx_defaults4;
typedef struct __pyx_defaults4 __pyx_defaults4;
struct __pyx_defaults5;
typedef struct __pyx_defaults5 __pyx_defaults5;
struct __pyx_defaults6;
typedef struct __pyx_defaults6 __pyx_defaults6;
struct __pyx_defaults7;
typedef struct __pyx_defaults7 __pyx_defaults7;
struct __pyx_defaults8;
typedef struct __pyx_defaults8 __pyx_defaults8;
struct __pyx_defaults9;
typedef struct __pyx_defaults9 __pyx_defaults9;
struct __pyx_defaults10;
typedef struct __pyx_defaults10 __pyx_defaults10;
struct __pyx_defaults11;
typedef struct __pyx_defaults11 __pyx_defaults11;
struct __pyx_defaults12;
typedef struct __pyx_defaults12 __pyx_defaults12;
struct __pyx_defaults13;
typedef struct __pyx_defaults13 __pyx_defaults13;
struct __pyx_defaults14;
typedef struct __pyx_defaults14 __pyx_defaults14;
struct __pyx_defaults15;
typedef struct __pyx_defaults15 __pyx_defaults15;
struct __pyx_defaults16;
typedef struct __pyx_defaults16 __pyx_defaults16;
struct __pyx_defaults17;
typedef struct __pyx_defaults17 __pyx_defaults17;
struct __pyx_defaults18;
typedef struct __pyx_defaults18 __pyx_defaults18;
struct __pyx_defaults19;
typedef struct __pyx_defaults19 __pyx_defaults19;
struct __pyx_defaults20;
typedef struct __pyx_defaults20 __pyx_defaults20;
struct __pyx_defaults21;
typedef struct __pyx_defaults21 __pyx_defaults21;
struct __pyx_defaults22;
typedef struct __pyx_defaults22 __pyx_defaults22;
struct __pyx_defaults23;
typedef struct __pyx_defaults23 __pyx_defaults23;
struct __pyx_defaults24;
typedef struct __pyx_defaults24 __pyx_defaults24;
struct __pyx_defaults25;
typedef struct __pyx_defaults25 __pyx_defaults25;
struct __pyx_defaults26;
typedef struct __pyx_defaults26 __pyx_defaults26;
struct __pyx_defaults27;
typedef struct __pyx_defaults27 __pyx_defaults27;
struct __pyx_defaults28;
typedef struct __pyx_defaults28 __pyx_defaults28;
struct __pyx_defaults29;
typedef struct __pyx_defaults29 __pyx_defaults29;
struct __pyx_defaults30;
typedef struct __pyx_defaults30 __pyx_defaults30;
struct __pyx_defaults31;
typedef struct __pyx_defaults31 __pyx_defaults31;
struct __pyx_defaults32;
typedef struct __pyx_defaults32 __pyx_defaults32;
struct __pyx_defaults33;
typedef struct __pyx_defaults33 __pyx_defaults33;
struct __pyx_defaults34;
typedef struct __pyx_defaults34 __pyx_defaults34;
struct __pyx_defaults35;
typedef struct __pyx_defaults35 __pyx_defaults35;
struct __pyx_defaults36;
typedef struct __pyx_defaults36 __pyx_defaults36;
struct __pyx_defaults37;
typedef struct __pyx_defaults37 __pyx_defaults37;
struct __pyx_defaults38;
typedef struct __pyx_defaults38 __pyx_defaults38;
struct __pyx_defaults39;
typedef struct __pyx_defaults39 __pyx_defaults39;
struct __pyx_defaults40;
typedef struct __pyx_defaults40 __pyx_defaults40;
struct __pyx_defaults41;
typedef struct __pyx_defaults41 __pyx_defaults41;
struct __pyx_defaults42;
typedef struct __pyx_defaults42 __pyx_defaults42;
struct __pyx_defaults43;
typedef struct __pyx_defaults43 __pyx_defaults43;
struct __pyx_defaults44;
typedef struct __pyx_defaults44 __pyx_defaults44;
struct __pyx_defaults45;
typedef struct __pyx_defaults45 __pyx_defaults45;
struct __pyx_defaults46;
typedef struct __pyx_defaults46 __pyx_defaults46;
struct __pyx_defaults47;
typedef struct __pyx_defaults47 __pyx_defaults47;
struct __pyx_defaults48;
typedef struct __pyx_defaults48 __pyx_defaults48;
struct __pyx_defaults49;
typedef struct __pyx_defaults49 __pyx_defaults49;
struct __pyx_defaults50;
typedef struct __pyx_defaults50 __pyx_defaults50;
struct __pyx_defaults51;
typedef struct __pyx_defaults51 __pyx_defaults51;
struct __pyx_defaults52;
typedef struct __pyx_defaults52 __pyx_defaults52;
struct __pyx_defaults53;
typedef struct __pyx_defaults53 __pyx_defaults53;
struct __pyx_defaults54;
typedef struct __pyx_defaults54 __pyx_defaults54;
struct __pyx_defaults55;
typedef struct __pyx_defaults55 __pyx_defaults55;
struct __pyx_defaults56;
typedef struct __pyx_defaults56 __pyx_defaults56;
struct __pyx_defaults57;
typedef struct __pyx_defaults57 __pyx_defaults57;
struct __pyx_defaults58;
typedef struct __pyx_defaults58 __pyx_defaults58;
struct __pyx_defaults59;
typedef struct __pyx_defaults59 __pyx_defaults59;
struct __pyx_defaults60;
typedef struct __pyx_defaults60 __pyx_defaults60;
struct __pyx_defaults61;
typedef struct __pyx_defaults61 __pyx_defaults61;
struct __pyx_defaults62;
typedef struct __pyx_defaults62 __pyx_defaults62;
struct __pyx_defaults63;
typedef struct __pyx_defaults63 __pyx_defaults63;
struct __pyx_defaults64;
typedef struct __pyx_defaults64 __pyx_defaults64;
struct __pyx_defaults65;
typedef struct __pyx_defaults65 __pyx_defaults65;
struct __pyx_defaults66;
typedef struct __pyx_defaults66 __pyx_defaults66;
struct __pyx_defaults67;
typedef struct __pyx_defaults67 __pyx_defaults67;
struct __pyx_defaults68;
typedef struct __pyx_defaults68 __pyx_defaults68;
struct __pyx_defaults69;
typedef struct __pyx_defaults69 __pyx_defaults69;
struct __pyx_defaults70;
typedef struct __pyx_defaults70 __pyx_defaults70;
struct __pyx_defaults71;
typedef struct __pyx_defaults71 __pyx_defaults71;
struct __pyx_defaults72;
typedef struct __pyx_defaults72 __pyx_defaults72;
struct __pyx_defaults73;
typedef struct __pyx_defaults73 __pyx_defaults73;
struct __pyx_defaults74;
typedef struct __pyx_defaults74 __pyx_defaults74;
struct __pyx_defaults75;
typedef struct __pyx_defaults75 __pyx_defaults75;
struct __pyx_defaults76;
typedef struct __pyx_defaults76 __pyx_defaults76;
struct __pyx_defaults77;
typedef struct __pyx_defaults77 __pyx_defaults77;
struct __pyx_defaults78;
typedef struct __pyx_defaults78 __pyx_defaults78;
struct __pyx_defaults79;
typedef struct __pyx_defaults79 __pyx_defaults79;
struct __pyx_defaults80;
typedef struct __pyx_defaults80 __pyx_defaults80;
struct __pyx_defaults81;
typedef struct __pyx_defaults81 __pyx_defaults81;
struct __pyx_defaults82;
typedef struct __pyx_defaults82 __pyx_defaults82;
struct __pyx_defaults83;
typedef struct __pyx_defaults83 __pyx_defaults83;
struct __pyx_defaults84;
typedef struct __pyx_defaults84 __pyx_defaults84;
struct __pyx_defaults85;
typedef struct __pyx_defaults85 __pyx_defaults85;
struct __pyx_defaults86;
typedef struct __pyx_defaults86 __pyx_defaults86;
struct __pyx_defaults87;
typedef struct __pyx_defaults87 __pyx_defaults87;
struct __pyx_defaults88;
typedef struct __pyx_defaults88 __pyx_defaults88;
struct __pyx_defaults89;
typedef struct __pyx_defaults89 __pyx_defaults89;
struct __pyx_defaults90;
typedef struct __pyx_defaults90 __pyx_defaults90;
struct __pyx_defaults91;
typedef struct __pyx_defaults91 __pyx_defaults91;
struct __pyx_defaults92;
typedef struct __pyx_defaults92 __pyx_defaults92;
struct __pyx_defaults93;
typedef struct __pyx_defaults93 __pyx_defaults93;
struct __pyx_defaults94;
typedef struct __pyx_defaults94 __pyx_defaults94;
struct __pyx_defaults95;
typedef struct __pyx_defaults95 __pyx_defaults95;
struct __pyx_defaults96;
typedef struct __pyx_defaults96 __pyx_defaults96;
struct __pyx_defaults97;
typedef struct __pyx_defaults97 __pyx_defaults97;
struct __pyx_defaults98;
typedef struct __pyx_defaults98 __pyx_defaults98;
struct __pyx_defaults99;
typedef struct __pyx_defaults99 __pyx_defaults99;
struct __pyx_defaults100;
typedef struct __pyx_defaults100 __pyx_defaults100;
struct __pyx_defaults101;
typedef struct __pyx_defaults101 __pyx_defaults101;
struct __pyx_defaults102;
typedef struct __pyx_defaults102 __pyx_defaults102;
struct __pyx_defaults103;
typedef struct __pyx_defaults103 __pyx_defaults103;
struct __pyx_defaults {
  __Pyx_memviewslice __pyx_arg_start;
};
struct __pyx_defaults1 {
  __Pyx_memviewslice __pyx_arg_start;
};
struct __pyx_defaults2 {
  __Pyx_memviewslice __pyx_arg_start;
};
struct __pyx_defaults3 {
  __Pyx_memviewslice __pyx_arg_start;
};
struct __pyx_defaults4 {
  __Pyx_memviewslice __pyx_arg_start;
};
struct __pyx_defaults5 {
  __Pyx_memviewslice __pyx_arg_start;
};
struct __pyx_defaults6 {
  __Pyx_memviewslice __pyx_arg_start;
};
struct __pyx_defaults7 {
  __Pyx_memviewslice __pyx_arg_start;
};
struct __pyx_defaults8 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults9 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults10 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults11 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults12 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults13 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults14 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults15 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults16 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults17 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults18 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults19 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults20 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults21 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults22 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults23 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults24 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults25 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults26 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults27 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults28 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults29 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults30 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults31 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults32 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults33 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults34 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults35 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults36 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults37 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults38 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults39 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults40 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults41 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults42 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults43 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults44 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults45 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults46 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults47 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults48 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults49 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults50 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults51 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults52 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults53 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults54 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults55 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults56 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults57 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults58 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults59 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults60 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults61 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults62 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults63 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults64 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults65 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults66 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults67 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults68 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults69 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults70 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults71 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults72 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults73 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults74 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults75 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults76 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults77 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults78 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults79 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults80 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults81 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults82 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults83 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults84 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults85 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults86 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults87 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults88 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults89 {
  __Pyx_memviewslice __pyx_arg_affine_idx_in;
  __Pyx_memviewslice __pyx_arg_affine_idx_out;
  __Pyx_memviewslice __pyx_arg_affine_disp;
  __Pyx_memviewslice __pyx_arg_out_shape;
};
struct __pyx_defaults90 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults91 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults92 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults93 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults94 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults95 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults96 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults97 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults98 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults99 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults100 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults101 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults102 {
  __Pyx_memviewslice __pyx_arg_affine;
};
struct __pyx_defaults103 {
  __Pyx_memviewslice __pyx_arg_affine;
};

/* "View.MemoryView":101
 * 
 * @cname("__pyx_array")
 * cdef class array:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */
struct __pyx_array_obj {
  PyObject_HEAD
  char *data;
  Py_ssize_t len;
  char *format;
  int ndim;
  Py_ssize_t *_shape;
  Py_ssize_t *_strides;
  Py_ssize_t itemsize;
  PyObject *mode;
  PyObject *_format;
  void (*callback_free_data)(void *);
  int free_data;
  int dtype_is_object;
};


/* "View.MemoryView":271
 * 
 * @cname('__pyx_MemviewEnum')
 * cdef class Enum(object):             # <<<<<<<<<<<<<<
 *     cdef object name
 *     def __init__(self, name):
 */
struct __pyx_MemviewEnum_obj {
  PyObject_HEAD
  PyObject *name;
};


/* "View.MemoryView":304
 * 
 * @cname('__pyx_memoryview')
 * cdef class memoryview(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef object obj
 */
struct __pyx_memoryview_obj {
  PyObject_HEAD
  struct __pyx_vtabstruct_memoryview *__pyx_vtab;
  PyObject *obj;
  PyObject *_size;
  PyObject *_array_interface;
  PyThread_type_lock lock;
  __pyx_atomic_int acquisition_count[2];
  __pyx_atomic_int *acquisition_count_aligned_p;
  Py_buffer view;
  int flags;
  int dtype_is_object;
  __Pyx_TypeInfo *typeinfo;
};


/* "View.MemoryView":923
 * 
 * @cname('__pyx_memoryviewslice')
 * cdef class _memoryviewslice(memoryview):             # <<<<<<<<<<<<<<
 *     "Internal class for passing memoryview slices to Python"
 * 
 */
struct __pyx_memoryviewslice_obj {
  struct __pyx_memoryview_obj __pyx_base;
  __Pyx_memviewslice from_slice;
  PyObject *from_object;
  PyObject *(*to_object_func)(char *);
  int (*to_dtype_func)(char *, PyObject *);
};



/* "View.MemoryView":304
 * 
 * @cname('__pyx_memoryview')
 * cdef class memoryview(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef object obj
 */

struct __pyx_vtabstruct_memoryview {
  char *(*get_item_pointer)(struct __pyx_memoryview_obj *, PyObject *);
  PyObject *(*is_slice)(struct __pyx_memoryview_obj *, PyObject *);
  PyObject *(*setitem_slice_assignment)(struct __pyx_memoryview_obj *, PyObject *, PyObject *);
  PyObject *(*setitem_slice_assign_scalar)(struct __pyx_memoryview_obj *, struct __pyx_memoryview_obj *, PyObject *);
  PyObject *(*setitem_indexed)(struct __pyx_memoryview_obj *, PyObject *, PyObject *);
  PyObject *(*convert_item_to_object)(struct __pyx_memoryview_obj *, char *);
  PyObject *(*assign_item_from_object)(struct __pyx_memoryview_obj *, char *, PyObject *);
};
static struct __pyx_vtabstruct_memoryview *__pyx_vtabptr_memoryview;


/* "View.MemoryView":923
 * 
 * @cname('__pyx_memoryviewslice')
 * cdef class _memoryviewslice(memoryview):             # <<<<<<<<<<<<<<
 *     "Internal class for passing memoryview slices to Python"
 * 
 */

struct __pyx_vtabstruct__memoryviewslice {
  struct __pyx_vtabstruct_memoryview __pyx_base;
};
static struct __pyx_vtabstruct__memoryviewslice *__pyx_vtabptr__memoryviewslice;

/* --- Runtime support code (head) --- */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name);

static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

static CYTHON_INLINE int  __Pyx_GetBufferAndValidate(Py_buffer* buf, PyObject* obj,
    __Pyx_TypeInfo* dtype, int flags, int nd, int cast, __Pyx_BufFmt_StackElem* stack);
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info);

#define __Pyx_BUF_MAX_NDIMS %(BUF_MAX_NDIMS)d
#define __Pyx_MEMVIEW_DIRECT   1
#define __Pyx_MEMVIEW_PTR      2
#define __Pyx_MEMVIEW_FULL     4
#define __Pyx_MEMVIEW_CONTIG   8
#define __Pyx_MEMVIEW_STRIDED  16
#define __Pyx_MEMVIEW_FOLLOW   32
#define __Pyx_IS_C_CONTIG 1
#define __Pyx_IS_F_CONTIG 2
static int __Pyx_init_memviewslice(
                struct __pyx_memoryview_obj *memview,
                int ndim,
                __Pyx_memviewslice *memviewslice,
                int memview_is_new_reference);
static CYTHON_INLINE int __pyx_add_acquisition_count_locked(
    __pyx_atomic_int *acquisition_count, PyThread_type_lock lock);
static CYTHON_INLINE int __pyx_sub_acquisition_count_locked(
    __pyx_atomic_int *acquisition_count, PyThread_type_lock lock);
#define __pyx_get_slice_count_pointer(memview) (memview->acquisition_count_aligned_p)
#define __pyx_get_slice_count(memview) (*__pyx_get_slice_count_pointer(memview))
#define __PYX_INC_MEMVIEW(slice, have_gil) __Pyx_INC_MEMVIEW(slice, have_gil, __LINE__)
#define __PYX_XDEC_MEMVIEW(slice, have_gil) __Pyx_XDEC_MEMVIEW(slice, have_gil, __LINE__)
static CYTHON_INLINE void __Pyx_INC_MEMVIEW(__Pyx_memviewslice *, int, int);
static CYTHON_INLINE void __Pyx_XDEC_MEMVIEW(__Pyx_memviewslice *, int, int);

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb);
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb);

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);

static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args))
                PyErr_SetObject(PyExc_KeyError, args);
            Py_XDECREF(args);
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#else
    #define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#endif

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb);

static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject*);

#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyObject_Ord(c)\
    (likely(PyUnicode_Check(c)) ? (long)__Pyx_PyUnicode_AsPy_UCS4(c) : __Pyx__PyObject_Ord(c))
#else
#define __Pyx_PyObject_Ord(c) __Pyx__PyObject_Ord(c)
#endif
static long __Pyx__PyObject_Ord(PyObject* c);

#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static CYTHON_INLINE int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

static CYTHON_INLINE int __Pyx_IterFinish(void);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

static CYTHON_INLINE int __Pyx_unpack_tuple2(PyObject* tuple, PyObject** value1, PyObject** value2,
                                             int is_tuple, int has_known_size, int decref_tuple);

static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx_PyFloat_SubtractObjC(PyObject *op1, PyObject *op2, double floatval, int inplace);
#else
#define __Pyx_PyFloat_SubtractObjC(op1, op2, floatval, inplace)\
    (inplace ? PyNumber_InPlaceSubtract(op1, op2) : PyNumber_Subtract(op1, op2))
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx_PyFloat_DivideCObj(PyObject *op1, PyObject *op2, double floatval, int inplace);
#else
#define __Pyx_PyFloat_DivideCObj(op1, op2, floatval, inplace)\
    ((inplace ? __Pyx_PyNumber_InPlaceDivide(op1, op2) : __Pyx_PyNumber_Divide(op1, op2)))
    #endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx_PyInt_AddCObj(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_AddCObj(op1, op2, intval, inplace)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact);

#include <string.h>

static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

#define UNARY_NEG_WOULD_OVERFLOW(x)\
        (((x) < 0) & ((unsigned long)(x) == 0-(unsigned long)(x)))

static CYTHON_UNUSED int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static PyObject *get_memview(PyObject *__pyx_v_self); /*proto*/
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);

static CYTHON_UNUSED int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static PyObject *__pyx_memoryview_transpose(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview__get__base(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_shape(PyObject *__pyx_v_self); /*proto*/
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

static PyObject *__pyx_memoryview_get_strides(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_suboffsets(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_ndim(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_itemsize(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_nbytes(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_size(PyObject *__pyx_v_self); /*proto*/
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_AddObjC(op1, op2, intval, inplace)\
    (inplace ? PyNumber_InPlaceAdd(op1, op2) : PyNumber_Add(op1, op2))
#endif

static CYTHON_INLINE int __Pyx_PyList_Extend(PyObject* L, PyObject* v) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject* none = _PyList_Extend((PyListObject*)L, v);
    if (unlikely(!none))
        return -1;
    Py_DECREF(none);
    return 0;
#else
    return PyList_SetSlice(L, PY_SSIZE_T_MAX, PY_SSIZE_T_MAX, v);
#endif
}

static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname);

#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

static PyObject *__pyx_memoryviewslice__get__base(PyObject *__pyx_v_self); /*proto*/
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

#define __Pyx_CyFunction_USED 1
#include <structmember.h>
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_NewEx(ml, flags, qualname, self, module, globals, code)\
    __Pyx_CyFunction_New(__pyx_CyFunctionType, ml, flags, qualname, self, module, globals, code)
static PyObject *__Pyx_CyFunction_New(PyTypeObject *, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

typedef struct {
    __pyx_CyFunctionObject func;
    PyObject *__signatures__;
    PyObject *type;
    PyObject *self;
} __pyx_FusedFunctionObject;
#define __pyx_FusedFunction_NewEx(ml, flags, qualname, self, module, globals, code)\
        __pyx_FusedFunction_New(__pyx_FusedFunctionType, ml, flags, qualname, self, module, globals, code)
static PyObject *__pyx_FusedFunction_New(PyTypeObject *type,
                                         PyMethodDef *ml, int flags,
                                         PyObject *qualname, PyObject *self,
                                         PyObject *module, PyObject *globals,
                                         PyObject *code);
static int __pyx_FusedFunction_clear(__pyx_FusedFunctionObject *self);
static PyTypeObject *__pyx_FusedFunctionType = NULL;
static int __pyx_FusedFunction_init(void);
#define __Pyx_FusedFunction_USED

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

typedef struct {
  Py_ssize_t shape, strides, suboffsets;
} __Pyx_Buf_DimInfo;
typedef struct {
  size_t refcount;
  Py_buffer pybuffer;
} __Pyx_Buffer;
typedef struct {
  __Pyx_Buffer *rcbuffer;
  char *data;
  __Pyx_Buf_DimInfo diminfo[8];
} __Pyx_LocalBuf_ND;

#if PY_MAJOR_VERSION < 3
    static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags);
    static void __Pyx_ReleaseBuffer(Py_buffer *view);
#else
    #define __Pyx_GetBuffer PyObject_GetBuffer
    #define __Pyx_ReleaseBuffer PyBuffer_Release
#endif


static Py_ssize_t __Pyx_zeros[] = {0, 0, 0, 0, 0, 0, 0, 0};
static Py_ssize_t __Pyx_minusones[] = {-1, -1, -1, -1, -1, -1, -1, -1};

static int __pyx_typeinfo_cmp(__Pyx_TypeInfo *a, __Pyx_TypeInfo *b);

static int __Pyx_ValidateAndInit_memviewslice(
                int *axes_specs,
                int c_or_f_flag,
                int buf_flags,
                int ndim,
                __Pyx_TypeInfo *dtype,
                __Pyx_BufFmt_StackElem stack[],
                __Pyx_memviewslice *memviewslice,
                PyObject *original_obj);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsds_float(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsds_double(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsds_short(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsds_int(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsds_long(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_float_complex(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_double_complex(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsds_short(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsds_long(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_float_complex(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_double_complex(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(PyObject *);

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    #define __Pyx_CREAL(z) ((z).real())
    #define __Pyx_CIMAG(z) ((z).imag())
  #else
    #define __Pyx_CREAL(z) (__real__(z))
    #define __Pyx_CIMAG(z) (__imag__(z))
  #endif
#else
    #define __Pyx_CREAL(z) ((z).real)
    #define __Pyx_CIMAG(z) ((z).imag)
#endif
#if (defined(_WIN32) || defined(__clang__)) && defined(__cplusplus) && CYTHON_CCOMPLEX
    #define __Pyx_SET_CREAL(z,x) ((z).real(x))
    #define __Pyx_SET_CIMAG(z,y) ((z).imag(y))
#else
    #define __Pyx_SET_CREAL(z,x) __Pyx_CREAL(z) = (x)
    #define __Pyx_SET_CIMAG(z,y) __Pyx_CIMAG(z) = (y)
#endif

static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float, float);

#if CYTHON_CCOMPLEX
    #define __Pyx_c_eqf(a, b)   ((a)==(b))
    #define __Pyx_c_sumf(a, b)  ((a)+(b))
    #define __Pyx_c_difff(a, b) ((a)-(b))
    #define __Pyx_c_prodf(a, b) ((a)*(b))
    #define __Pyx_c_quotf(a, b) ((a)/(b))
    #define __Pyx_c_negf(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zerof(z) ((z)==(float)0)
    #define __Pyx_c_conjf(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_absf(z)     (::std::abs(z))
        #define __Pyx_c_powf(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zerof(z) ((z)==0)
    #define __Pyx_c_conjf(z)    (conjf(z))
    #if 1
        #define __Pyx_c_absf(z)     (cabsf(z))
        #define __Pyx_c_powf(a, b)  (cpowf(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eqf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sumf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_difff(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prodf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quotf(__pyx_t_float_complex, __pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_negf(__pyx_t_float_complex);
    static CYTHON_INLINE int __Pyx_c_is_zerof(__pyx_t_float_complex);
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conjf(__pyx_t_float_complex);
    #if 1
        static CYTHON_INLINE float __Pyx_c_absf(__pyx_t_float_complex);
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_powf(__pyx_t_float_complex, __pyx_t_float_complex);
    #endif
#endif

static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double, double);

#if CYTHON_CCOMPLEX
    #define __Pyx_c_eq(a, b)   ((a)==(b))
    #define __Pyx_c_sum(a, b)  ((a)+(b))
    #define __Pyx_c_diff(a, b) ((a)-(b))
    #define __Pyx_c_prod(a, b) ((a)*(b))
    #define __Pyx_c_quot(a, b) ((a)/(b))
    #define __Pyx_c_neg(a)     (-(a))
  #ifdef __cplusplus
    #define __Pyx_c_is_zero(z) ((z)==(double)0)
    #define __Pyx_c_conj(z)    (::std::conj(z))
    #if 1
        #define __Pyx_c_abs(z)     (::std::abs(z))
        #define __Pyx_c_pow(a, b)  (::std::pow(a, b))
    #endif
  #else
    #define __Pyx_c_is_zero(z) ((z)==0)
    #define __Pyx_c_conj(z)    (conj(z))
    #if 1
        #define __Pyx_c_abs(z)     (cabs(z))
        #define __Pyx_c_pow(a, b)  (cpow(a, b))
    #endif
 #endif
#else
    static CYTHON_INLINE int __Pyx_c_eq(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot(__pyx_t_double_complex, __pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg(__pyx_t_double_complex);
    static CYTHON_INLINE int __Pyx_c_is_zero(__pyx_t_double_complex);
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj(__pyx_t_double_complex);
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs(__pyx_t_double_complex);
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow(__pyx_t_double_complex, __pyx_t_double_complex);
    #endif
#endif

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_double(PyObject *);

static PyObject *__pyx_memview_get_float(const char *itemp);
static int __pyx_memview_set_float(const char *itemp, PyObject *obj);

static PyObject *__pyx_memview_get_double(const char *itemp);
static int __pyx_memview_set_double(const char *itemp, PyObject *obj);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_int(PyObject *);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

static PyObject *__pyx_memview_get_int(const char *itemp);
static int __pyx_memview_set_int(const char *itemp, PyObject *obj);

static CYTHON_INLINE Py_intptr_t __Pyx_PyInt_As_Py_intptr_t(PyObject *);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

static CYTHON_INLINE int __Pyx_BytesContains(PyObject* bytes, char character);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_short(short value);

static CYTHON_INLINE short __Pyx_PyInt_As_short(PyObject *);

static PyObject *__pyx_memview_get_short(const char *itemp);
static int __pyx_memview_set_short(const char *itemp, PyObject *obj);

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

static PyObject *__pyx_memview_get_long(const char *itemp);
static int __pyx_memview_set_long(const char *itemp, PyObject *obj);

#define __pyx_PyComplex_FromComplex(z) \
        PyComplex_FromDoubles((double)__Pyx_CREAL(z), \
                              (double)__Pyx_CIMAG(z))

static __pyx_t_float_complex __Pyx_PyComplex_As___pyx_t_float_complex(PyObject*);

static PyObject *__pyx_memview_get___pyx_t_float_complex(const char *itemp);
static int __pyx_memview_set___pyx_t_float_complex(const char *itemp, PyObject *obj);

static __pyx_t_double_complex __Pyx_PyComplex_As___pyx_t_double_complex(PyObject*);

static PyObject *__pyx_memview_get___pyx_t_double_complex(const char *itemp);
static int __pyx_memview_set___pyx_t_double_complex(const char *itemp, PyObject *obj);

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value);

static int __pyx_memviewslice_is_contig(const __Pyx_memviewslice *mvs,
                                        char order, int ndim);

static int __pyx_slices_overlap(__Pyx_memviewslice *slice1,
                                __Pyx_memviewslice *slice2,
                                int ndim, size_t itemsize);

static __Pyx_memviewslice
__pyx_memoryview_copy_new_contig(const __Pyx_memviewslice *from_mvs,
                                 const char *mode, int ndim,
                                 size_t sizeof_dtype, int contig_flag,
                                 int dtype_is_object);

static CYTHON_INLINE PyObject *__pyx_capsule_create(void *p, const char *sig);

static CYTHON_INLINE char __Pyx_PyInt_As_char(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_float(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_short(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_long(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds___pyx_t_float_complex(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds___pyx_t_double_complex(PyObject *);

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_int(PyObject *);

static int __Pyx_check_binary_version(void);

#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

static PyObject *__Pyx_ImportModule(const char *name);

static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static char *__pyx_memoryview_get_item_pointer(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index); /* proto*/
static PyObject *__pyx_memoryview_is_slice(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj); /* proto*/
static PyObject *__pyx_memoryview_setitem_slice_assignment(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_dst, PyObject *__pyx_v_src); /* proto*/
static PyObject *__pyx_memoryview_setitem_slice_assign_scalar(struct __pyx_memoryview_obj *__pyx_v_self, struct __pyx_memoryview_obj *__pyx_v_dst, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryview_setitem_indexed(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryview_convert_item_to_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp); /* proto*/
static PyObject *__pyx_memoryview_assign_item_from_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value); /* proto*/
static PyObject *__pyx_memoryviewslice_convert_item_to_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp); /* proto*/
static PyObject *__pyx_memoryviewslice_assign_item_from_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value); /* proto*/

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'libc.stdlib' */

/* Module declarations from 'numpy' */

/* Module declarations from 'numpy' */
static PyTypeObject *__pyx_ptype_5numpy_dtype = 0;
static PyTypeObject *__pyx_ptype_5numpy_flatiter = 0;
static PyTypeObject *__pyx_ptype_5numpy_broadcast = 0;
static PyTypeObject *__pyx_ptype_5numpy_ndarray = 0;
static PyTypeObject *__pyx_ptype_5numpy_ufunc = 0;
static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *, char *, char *, int *); /*proto*/

/* Module declarations from 'cython.view' */

/* Module declarations from 'cython' */

/* Module declarations from 'dipy.align.fused_types' */

/* Module declarations from 'dipy.align.vector_fields' */
static PyTypeObject *__pyx_array_type = 0;
static PyTypeObject *__pyx_MemviewEnum_type = 0;
static PyTypeObject *__pyx_memoryview_type = 0;
static PyTypeObject *__pyx_memoryviewslice_type = 0;
static PyObject *generic = 0;
static PyObject *strided = 0;
static PyObject *indirect = 0;
static PyObject *contiguous = 0;
static PyObject *indirect_contiguous = 0;
static CYTHON_INLINE double __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(double, double, double, double, __Pyx_memviewslice); /*proto*/
static CYTHON_INLINE double __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(double, double, double, double, __Pyx_memviewslice); /*proto*/
static CYTHON_INLINE double __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(double, double, double, double, __Pyx_memviewslice); /*proto*/
static CYTHON_INLINE double __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(double, double, double, __Pyx_memviewslice); /*proto*/
static CYTHON_INLINE double __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(double, double, double, __Pyx_memviewslice); /*proto*/
static CYTHON_INLINE int __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__Pyx_memviewslice, double, double, __Pyx_memviewslice); /*proto*/
static CYTHON_INLINE int __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__Pyx_memviewslice, double, double, __Pyx_memviewslice); /*proto*/
static CYTHON_INLINE int __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__Pyx_memviewslice, double, double, float *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__Pyx_memviewslice, double, double, double *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__Pyx_memviewslice, double, double, short *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__Pyx_memviewslice, double, double, int *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_2__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__Pyx_memviewslice, double, double, long *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_3__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__Pyx_memviewslice, double, double, float *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_4__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__Pyx_memviewslice, double, double, double *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_5__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__Pyx_memviewslice, double, double, __pyx_t_float_complex *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_6__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__Pyx_memviewslice, double, double, __pyx_t_double_complex *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__Pyx_memviewslice, double, double, double, short *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__Pyx_memviewslice, double, double, double, int *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_2__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__Pyx_memviewslice, double, double, double, long *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_3__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__Pyx_memviewslice, double, double, double, float *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_4__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__Pyx_memviewslice, double, double, double, double *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_5__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__Pyx_memviewslice, double, double, double, __pyx_t_float_complex *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_6__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__Pyx_memviewslice, double, double, double, __pyx_t_double_complex *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__Pyx_memviewslice, double, double, double, float *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__Pyx_memviewslice, double, double, double, double *); /*proto*/
static CYTHON_INLINE int __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__Pyx_memviewslice, double, double, double, __Pyx_memviewslice); /*proto*/
static CYTHON_INLINE int __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__Pyx_memviewslice, double, double, double, __Pyx_memviewslice); /*proto*/
static void __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__compose_vector_fields_2d(__Pyx_memviewslice, __Pyx_memviewslice, __Pyx_memviewslice, __Pyx_memviewslice, double, __Pyx_memviewslice, __Pyx_memviewslice); /*proto*/
static void __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__compose_vector_fields_2d(__Pyx_memviewslice, __Pyx_memviewslice, __Pyx_memviewslice, __Pyx_memviewslice, double, __Pyx_memviewslice, __Pyx_memviewslice); /*proto*/
static void __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__compose_vector_fields_3d(__Pyx_memviewslice, __Pyx_memviewslice, __Pyx_memviewslice, __Pyx_memviewslice, double, __Pyx_memviewslice, __Pyx_memviewslice); /*proto*/
static void __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__compose_vector_fields_3d(__Pyx_memviewslice, __Pyx_memviewslice, __Pyx_memviewslice, __Pyx_memviewslice, double, __Pyx_memviewslice, __Pyx_memviewslice); /*proto*/
static struct __pyx_array_obj *__pyx_array_new(PyObject *, Py_ssize_t, char *, char *, char *); /*proto*/
static void *__pyx_align_pointer(void *, size_t); /*proto*/
static PyObject *__pyx_memoryview_new(PyObject *, int, int, __Pyx_TypeInfo *); /*proto*/
static CYTHON_INLINE int __pyx_memoryview_check(PyObject *); /*proto*/
static PyObject *_unellipsify(PyObject *, int); /*proto*/
static PyObject *assert_direct_dimensions(Py_ssize_t *, int); /*proto*/
static struct __pyx_memoryview_obj *__pyx_memview_slice(struct __pyx_memoryview_obj *, PyObject *); /*proto*/
static int __pyx_memoryview_slice_memviewslice(__Pyx_memviewslice *, Py_ssize_t, Py_ssize_t, Py_ssize_t, int, int, int *, Py_ssize_t, Py_ssize_t, Py_ssize_t, int, int, int, int); /*proto*/
static char *__pyx_pybuffer_index(Py_buffer *, char *, Py_ssize_t, Py_ssize_t); /*proto*/
static int __pyx_memslice_transpose(__Pyx_memviewslice *); /*proto*/
static PyObject *__pyx_memoryview_fromslice(__Pyx_memviewslice, int, PyObject *(*)(char *), int (*)(char *, PyObject *), int); /*proto*/
static __Pyx_memviewslice *__pyx_memoryview_get_slice_from_memoryview(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static void __pyx_memoryview_slice_copy(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static PyObject *__pyx_memoryview_copy_object(struct __pyx_memoryview_obj *); /*proto*/
static PyObject *__pyx_memoryview_copy_object_from_slice(struct __pyx_memoryview_obj *, __Pyx_memviewslice *); /*proto*/
static Py_ssize_t abs_py_ssize_t(Py_ssize_t); /*proto*/
static char __pyx_get_best_slice_order(__Pyx_memviewslice *, int); /*proto*/
static void _copy_strided_to_strided(char *, Py_ssize_t *, char *, Py_ssize_t *, Py_ssize_t *, Py_ssize_t *, int, size_t); /*proto*/
static void copy_strided_to_strided(__Pyx_memviewslice *, __Pyx_memviewslice *, int, size_t); /*proto*/
static Py_ssize_t __pyx_memoryview_slice_get_size(__Pyx_memviewslice *, int); /*proto*/
static Py_ssize_t __pyx_fill_contig_strides_array(Py_ssize_t *, Py_ssize_t *, Py_ssize_t, int, char); /*proto*/
static void *__pyx_memoryview_copy_data_to_temp(__Pyx_memviewslice *, __Pyx_memviewslice *, char, int); /*proto*/
static int __pyx_memoryview_err_extents(int, Py_ssize_t, Py_ssize_t); /*proto*/
static int __pyx_memoryview_err_dim(PyObject *, char *, int); /*proto*/
static int __pyx_memoryview_err(PyObject *, char *); /*proto*/
static int __pyx_memoryview_copy_contents(__Pyx_memviewslice, __Pyx_memviewslice, int, int, int); /*proto*/
static void __pyx_memoryview_broadcast_leading(__Pyx_memviewslice *, int, int); /*proto*/
static void __pyx_memoryview_refcount_copying(__Pyx_memviewslice *, int, int, int); /*proto*/
static void __pyx_memoryview_refcount_objects_in_slice_with_gil(char *, Py_ssize_t *, Py_ssize_t *, int, int); /*proto*/
static void __pyx_memoryview_refcount_objects_in_slice(char *, Py_ssize_t *, Py_ssize_t *, int, int); /*proto*/
static void __pyx_memoryview_slice_assign_scalar(__Pyx_memviewslice *, int, size_t, void *, int); /*proto*/
static void __pyx_memoryview__slice_assign_scalar(char *, Py_ssize_t *, Py_ssize_t *, int, size_t, void *); /*proto*/
static __Pyx_TypeInfo __Pyx_TypeInfo_float = { "float", NULL, sizeof(float), { 0 }, 0, 'R', 0, 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo_double = { "double", NULL, sizeof(double), { 0 }, 0, 'R', 0, 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo_short = { "short", NULL, sizeof(short), { 0 }, 0, IS_UNSIGNED(short) ? 'U' : 'I', IS_UNSIGNED(short), 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo_int = { "int", NULL, sizeof(int), { 0 }, 0, IS_UNSIGNED(int) ? 'U' : 'I', IS_UNSIGNED(int), 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo_long = { "long", NULL, sizeof(long), { 0 }, 0, IS_UNSIGNED(long) ? 'U' : 'I', IS_UNSIGNED(long), 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo___pyx_t_float_complex = { "float complex", NULL, sizeof(__pyx_t_float_complex), { 0 }, 0, 'C', 0, 0 };
static __Pyx_TypeInfo __Pyx_TypeInfo___pyx_t_double_complex = { "double complex", NULL, sizeof(__pyx_t_double_complex), { 0 }, 0, 'C', 0, 0 };
#define __Pyx_MODULE_NAME "dipy.align.vector_fields"
int __pyx_module_is_main_dipy__align__vector_fields = 0;

/* Implementation of 'dipy.align.vector_fields' */
static PyObject *__pyx_builtin_ImportError;
static PyObject *__pyx_builtin_AttributeError;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_zip;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_RuntimeError;
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_enumerate;
static PyObject *__pyx_builtin_Ellipsis;
static PyObject *__pyx_builtin_id;
static PyObject *__pyx_builtin_IndexError;
static char __pyx_k_[] = "()";
static char __pyx_k_B[] = "B";
static char __pyx_k_H[] = "H";
static char __pyx_k_I[] = "I";
static char __pyx_k_L[] = "L";
static char __pyx_k_M[] = "M";
static char __pyx_k_O[] = "O";
static char __pyx_k_Q[] = "Q";
static char __pyx_k_b[] = "b";
static char __pyx_k_c[] = "c";
static char __pyx_k_d[] = "d";
static char __pyx_k_f[] = "f";
static char __pyx_k_g[] = "g";
static char __pyx_k_h[] = "h";
static char __pyx_k_i[] = "i";
static char __pyx_k_j[] = "j";
static char __pyx_k_k[] = "k";
static char __pyx_k_l[] = "l";
static char __pyx_k_m[] = "m";
static char __pyx_k_n[] = "n";
static char __pyx_k_p[] = "p";
static char __pyx_k_q[] = "q";
static char __pyx_k_r[] = "r";
static char __pyx_k_s[] = "s";
static char __pyx_k_x[] = "x";
static char __pyx_k_Zd[] = "Zd";
static char __pyx_k_Zf[] = "Zf";
static char __pyx_k_Zg[] = "Zg";
static char __pyx_k__3[] = "|";
static char __pyx_k_d1[] = "d1";
static char __pyx_k_d2[] = "d2";
static char __pyx_k_di[] = "di";
static char __pyx_k_dj[] = "dj";
static char __pyx_k_dk[] = "dk";
static char __pyx_k_dx[] = "dx";
static char __pyx_k_id[] = "id";
static char __pyx_k_ii[] = "ii";
static char __pyx_k_jj[] = "jj";
static char __pyx_k_kk[] = "kk";
static char __pyx_k_nc[] = "nc";
static char __pyx_k_np[] = "np";
static char __pyx_k_nr[] = "nr";
static char __pyx_k_ns[] = "ns";
static char __pyx_k_ri[] = "ri";
static char __pyx_k_rj[] = "rj";
static char __pyx_k_rk[] = "rk";
static char __pyx_k_sc[] = "sc";
static char __pyx_k_sr[] = "sr";
static char __pyx_k_ss[] = "ss";
static char __pyx_k_all[] = "all";
static char __pyx_k_cnt[] = "cnt";
static char __pyx_k_cos[] = "cos";
static char __pyx_k_dii[] = "dii";
static char __pyx_k_dim[] = "dim";
static char __pyx_k_djj[] = "djj";
static char __pyx_k_dkk[] = "dkk";
static char __pyx_k_img[] = "img";
static char __pyx_k_int[] = "int";
static char __pyx_k_inv[] = "inv";
static char __pyx_k_mag[] = "mag";
static char __pyx_k_nnc[] = "nnc";
static char __pyx_k_nnr[] = "nnr";
static char __pyx_k_nns[] = "nns";
static char __pyx_k_obj[] = "obj";
static char __pyx_k_out[] = "out";
static char __pyx_k_tmp[] = "tmp";
static char __pyx_k_tol[] = "tol";
static char __pyx_k_zip[] = "zip";
static char __pyx_k_args[] = "args";
static char __pyx_k_base[] = "base";
static char __pyx_k_beta[] = "beta";
static char __pyx_k_comp[] = "comp";
static char __pyx_k_down[] = "down";
static char __pyx_k_flag[] = "flag";
static char __pyx_k_kind[] = "kind";
static char __pyx_k_long[] = "long";
static char __pyx_k_main[] = "__main__";
static char __pyx_k_mode[] = "mode";
static char __pyx_k_name[] = "name";
static char __pyx_k_ndim[] = "ndim";
static char __pyx_k_pack[] = "pack";
static char __pyx_k_size[] = "size";
static char __pyx_k_sqrt[] = "sqrt";
static char __pyx_k_step[] = "step";
static char __pyx_k_stop[] = "stop";
static char __pyx_k_test[] = "__test__";
static char __pyx_k_tmp0[] = "tmp0";
static char __pyx_k_tmp1[] = "tmp1";
static char __pyx_k_type[] = "type";
static char __pyx_k_ASCII[] = "ASCII";
static char __pyx_k_alpha[] = "alpha";
static char __pyx_k_cbeta[] = "cbeta";
static char __pyx_k_class[] = "__class__";
static char __pyx_k_dtype[] = "dtype";
static char __pyx_k_empty[] = "empty";
static char __pyx_k_error[] = "error";
static char __pyx_k_field[] = "field";
static char __pyx_k_flags[] = "flags";
static char __pyx_k_float[] = "float";
static char __pyx_k_ftype[] = "ftype";
static char __pyx_k_gamma[] = "gamma";
static char __pyx_k_image[] = "image";
static char __pyx_k_int32[] = "int32";
static char __pyx_k_ncVol[] = "ncVol";
static char __pyx_k_ncols[] = "ncols";
static char __pyx_k_norms[] = "norms";
static char __pyx_k_nrVol[] = "nrVol";
static char __pyx_k_nrows[] = "nrows";
static char __pyx_k_nsVol[] = "nsVol";
static char __pyx_k_numpy[] = "numpy";
static char __pyx_k_range[] = "range";
static char __pyx_k_shape[] = "shape";
static char __pyx_k_short[] = "short";
static char __pyx_k_split[] = "split";
static char __pyx_k_start[] = "start";
static char __pyx_k_stats[] = "stats";
static char __pyx_k_strip[] = "strip";
static char __pyx_k_tcols[] = "tcols";
static char __pyx_k_theta[] = "theta";
static char __pyx_k_trows[] = "trows";
static char __pyx_k_zeros[] = "zeros";
static char __pyx_k_affine[] = "affine";
static char __pyx_k_astype[] = "astype";
static char __pyx_k_calpha[] = "calpha";
static char __pyx_k_cgamma[] = "cgamma";
static char __pyx_k_difmag[] = "difmag";
static char __pyx_k_double[] = "double";
static char __pyx_k_encode[] = "encode";
static char __pyx_k_format[] = "format";
static char __pyx_k_import[] = "__import__";
static char __pyx_k_inside[] = "inside";
static char __pyx_k_kwargs[] = "kwargs";
static char __pyx_k_maxlen[] = "maxlen";
static char __pyx_k_name_2[] = "__name__";
static char __pyx_k_output[] = "output";
static char __pyx_k_radius[] = "radius";
static char __pyx_k_random[] = "random";
static char __pyx_k_struct[] = "struct";
static char __pyx_k_unpack[] = "unpack";
static char __pyx_k_volume[] = "volume";
static char __pyx_k_warped[] = "warped";
static char __pyx_k_arctan2[] = "arctan2";
static char __pyx_k_asarray[] = "asarray";
static char __pyx_k_current[] = "current";
static char __pyx_k_epsilon[] = "epsilon";
static char __pyx_k_factors[] = "factors";
static char __pyx_k_float64[] = "float64";
static char __pyx_k_fortran[] = "fortran";
static char __pyx_k_in_flag[] = "in_flag";
static char __pyx_k_jd_grad[] = "jd_grad";
static char __pyx_k_memview[] = "memview";
static char __pyx_k_mid_col[] = "mid_col";
static char __pyx_k_mid_row[] = "mid_row";
static char __pyx_k_ndarray[] = "ndarray";
static char __pyx_k_nslices[] = "nslices";
static char __pyx_k_randint[] = "randint";
static char __pyx_k_spacing[] = "spacing";
static char __pyx_k_tslices[] = "tslices";
static char __pyx_k_warp_2d[] = "warp_2d";
static char __pyx_k_warp_3d[] = "warp_3d";
static char __pyx_k_Ellipsis[] = "Ellipsis";
static char __pyx_k_defaults[] = "defaults";
static char __pyx_k_dom_size[] = "dom_size";
static char __pyx_k_expanded[] = "expanded";
static char __pyx_k_gradient[] = "gradient";
static char __pyx_k_isfinite[] = "isfinite";
static char __pyx_k_itemsize[] = "itemsize";
static char __pyx_k_max_iter[] = "max_iter";
static char __pyx_k_substats[] = "substats";
static char __pyx_k_to_shape[] = "to_shape";
static char __pyx_k_TypeError[] = "TypeError";
static char __pyx_k_enumerate[] = "enumerate";
static char __pyx_k_int_field[] = "int_field";
static char __pyx_k_int_float[] = "int|float";
static char __pyx_k_locations[] = "locations";
static char __pyx_k_mid_slice[] = "mid_slice";
static char __pyx_k_out_shape[] = "out_shape";
static char __pyx_k_ref_shape[] = "ref_shape";
static char __pyx_k_tolerance[] = "tolerance";
static char __pyx_k_IndexError[] = "IndexError";
static char __pyx_k_ValueError[] = "ValueError";
static char __pyx_k_from_shape[] = "from_shape";
static char __pyx_k_int_double[] = "int|double";
static char __pyx_k_iter_count[] = "iter_count";
static char __pyx_k_long_float[] = "long|float";
static char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static char __pyx_k_signatures[] = "signatures";
static char __pyx_k_warp_2d_nn[] = "warp_2d_nn";
static char __pyx_k_warp_3d_nn[] = "warp_3d_nn";
static char __pyx_k_zeros_like[] = "zeros_like";
static char __pyx_k_ImportError[] = "ImportError";
static char __pyx_k_MemoryError[] = "MemoryError";
static char __pyx_k_affine_disp[] = "affine_disp";
static char __pyx_k_float_float[] = "float|float";
static char __pyx_k_gradient_2d[] = "_gradient_2d";
static char __pyx_k_gradient_3d[] = "_gradient_3d";
static char __pyx_k_img_spacing[] = "img_spacing";
static char __pyx_k_long_double[] = "long|double";
static char __pyx_k_short_float[] = "short|float";
static char __pyx_k_step_factor[] = "step_factor";
static char __pyx_k_RuntimeError[] = "RuntimeError";
static char __pyx_k_d_world2grid[] = "d_world2grid";
static char __pyx_k_double_float[] = "double|float";
static char __pyx_k_float_double[] = "float|double";
static char __pyx_k_premult_disp[] = "premult_disp";
static char __pyx_k_short_double[] = "short|double";
static char __pyx_k_time_scaling[] = "time_scaling";
static char __pyx_k_affine_idx_in[] = "affine_idx_in";
static char __pyx_k_create_circle[] = "create_circle";
static char __pyx_k_create_sphere[] = "create_sphere";
static char __pyx_k_double_double[] = "double|double";
static char __pyx_k_float_complex[] = "float complex";
static char __pyx_k_premult_index[] = "premult_index";
static char __pyx_k_pyx_getbuffer[] = "__pyx_getbuffer";
static char __pyx_k_sample_points[] = "sample_points";
static char __pyx_k_to_grid2world[] = "to_grid2world";
static char __pyx_k_AttributeError[] = "AttributeError";
static char __pyx_k_affine_idx_out[] = "affine_idx_out";
static char __pyx_k_double_complex[] = "double complex";
static char __pyx_k_img_world2grid[] = "img_world2grid";
static char __pyx_k_out_grid2world[] = "out_grid2world";
static char __pyx_k_allocate_buffer[] = "allocate_buffer";
static char __pyx_k_dtype_is_object[] = "dtype_is_object";
static char __pyx_k_from_grid2world[] = "from_grid2world";
static char __pyx_k_is_valid_affine[] = "is_valid_affine";
static char __pyx_k_sparse_gradient[] = "sparse_gradient";
static char __pyx_k_Invalid_spacings[] = "Invalid spacings";
static char __pyx_k_sparse_gradient_2d[] = "_sparse_gradient_2d";
static char __pyx_k_sparse_gradient_3d[] = "_sparse_gradient_3d";
static char __pyx_k_strided_and_direct[] = "<strided and direct>";
static char __pyx_k_float_complex_float[] = "float complex|float";
static char __pyx_k_transform_2d_affine[] = "transform_2d_affine";
static char __pyx_k_transform_3d_affine[] = "transform_3d_affine";
static char __pyx_k_double_complex_float[] = "double complex|float";
static char __pyx_k_float_complex_double[] = "float complex|double";
static char __pyx_k_strided_and_indirect[] = "<strided and indirect>";
static char __pyx_k_contiguous_and_direct[] = "<contiguous and direct>";
static char __pyx_k_double_complex_double[] = "double complex|double";
static char __pyx_k_interpolate_scalar_2d[] = "interpolate_scalar_2d";
static char __pyx_k_interpolate_scalar_3d[] = "interpolate_scalar_3d";
static char __pyx_k_interpolate_vector_2d[] = "interpolate_vector_2d";
static char __pyx_k_interpolate_vector_3d[] = "interpolate_vector_3d";
static char __pyx_k_MemoryView_of_r_object[] = "<MemoryView of %r object>";
static char __pyx_k_transform_2d_affine_nn[] = "transform_2d_affine_nn";
static char __pyx_k_transform_3d_affine_nn[] = "transform_3d_affine_nn";
static char __pyx_k_MemoryView_of_r_at_0x_x[] = "<MemoryView of %r at 0x%x>";
static char __pyx_k_contiguous_and_indirect[] = "<contiguous and indirect>";
static char __pyx_k_Cannot_index_with_type_s[] = "Cannot index with type '%s'";
static char __pyx_k_compose_vector_fields_2d[] = "compose_vector_fields_2d";
static char __pyx_k_compose_vector_fields_3d[] = "compose_vector_fields_3d";
static char __pyx_k_dipy_align_vector_fields[] = "dipy.align.vector_fields";
static char __pyx_k_getbuffer_obj_view_flags[] = "getbuffer(obj, view, flags)";
static char __pyx_k_interpolate_scalar_nn_2d[] = "interpolate_scalar_nn_2d";
static char __pyx_k_interpolate_scalar_nn_3d[] = "interpolate_scalar_nn_3d";
static char __pyx_k_reorient_vector_field_2d[] = "reorient_vector_field_2d";
static char __pyx_k_reorient_vector_field_3d[] = "reorient_vector_field_3d";
static char __pyx_k_Dimension_d_is_not_direct[] = "Dimension %d is not direct";
static char __pyx_k_Invalid_shape_in_axis_d_d[] = "Invalid shape in axis %d: %d.";
static char __pyx_k_create_harmonic_fields_2d[] = "create_harmonic_fields_2d";
static char __pyx_k_create_harmonic_fields_3d[] = "create_harmonic_fields_3d";
static char __pyx_k_simplify_warp_function_2d[] = "simplify_warp_function_2d";
static char __pyx_k_simplify_warp_function_3d[] = "simplify_warp_function_3d";
static char __pyx_k_Index_out_of_bounds_axis_d[] = "Index out of bounds (axis %d)";
static char __pyx_k_downsample_scalar_field_2d[] = "downsample_scalar_field_2d";
static char __pyx_k_downsample_scalar_field_3d[] = "downsample_scalar_field_3d";
static char __pyx_k_No_matching_signature_found[] = "No matching signature found";
static char __pyx_k_Step_may_not_be_zero_axis_d[] = "Step may not be zero (axis %d)";
static char __pyx_k_itemsize_0_for_cython_array[] = "itemsize <= 0 for cython.array";
static char __pyx_k_ndarray_is_not_C_contiguous[] = "ndarray is not C contiguous";
static char __pyx_k_Expected_at_least_d_arguments[] = "Expected at least %d arguments";
static char __pyx_k_create_random_displacement_2d[] = "create_random_displacement_2d";
static char __pyx_k_create_random_displacement_3d[] = "create_random_displacement_3d";
static char __pyx_k_unable_to_allocate_array_data[] = "unable to allocate array data.";
static char __pyx_k_Invalid_image_affine_transform[] = "Invalid image affine transform";
static char __pyx_k_resample_displacement_field_2d[] = "resample_displacement_field_2d";
static char __pyx_k_resample_displacement_field_3d[] = "resample_displacement_field_3d";
static char __pyx_k_strided_and_direct_or_indirect[] = "<strided and direct or indirect>";
static char __pyx_k_Invalid_affine_transform_matrix[] = "Invalid affine transform matrix";
static char __pyx_k_home_yoh_deb_gits_pkg_exppsy_di[] = "/home/yoh/deb/gits/pkg-exppsy/dipy/dipy/align/vector_fields.pyx";
static char __pyx_k_invert_vector_field_fixed_point[] = "invert_vector_field_fixed_point_2d";
static char __pyx_k_unknown_dtype_code_in_numpy_pxd[] = "unknown dtype code in numpy.pxd (%d)";
static char __pyx_k_All_dimensions_preceding_dimensi[] = "All dimensions preceding dimension %d must be indexed and not sliced";
static char __pyx_k_Buffer_view_does_not_expose_stri[] = "Buffer view does not expose strides";
static char __pyx_k_Can_only_create_a_buffer_that_is[] = "Can only create a buffer that is contiguous in memory.";
static char __pyx_k_Cannot_transpose_memoryview_with[] = "Cannot transpose memoryview with indirect dimensions";
static char __pyx_k_Empty_shape_tuple_for_cython_arr[] = "Empty shape tuple for cython.array";
static char __pyx_k_Format_string_allocated_too_shor[] = "Format string allocated too short, see comment in numpy.pxd";
static char __pyx_k_Function_call_with_ambiguous_arg[] = "Function call with ambiguous argument types";
static char __pyx_k_Indirect_dimensions_not_supporte[] = "Indirect dimensions not supported";
static char __pyx_k_Invalid_displacement_multiplicat[] = "Invalid displacement multiplication matrix";
static char __pyx_k_Invalid_displacement_pre_multipl[] = "Invalid displacement pre-multiplication matrix";
static char __pyx_k_Invalid_from_affine_transform_ma[] = "Invalid 'from' affine transform matrix";
static char __pyx_k_Invalid_index_multiplication_mat[] = "Invalid index multiplication matrix";
static char __pyx_k_Invalid_index_pre_multiplication[] = "Invalid index pre-multiplication matrix";
static char __pyx_k_Invalid_inner_index_multiplicati[] = "Invalid inner index multiplication matrix";
static char __pyx_k_Invalid_mode_expected_c_or_fortr[] = "Invalid mode, expected 'c' or 'fortran', got %s";
static char __pyx_k_Invalid_outer_index_multiplicati[] = "Invalid outer index multiplication matrix";
static char __pyx_k_Invalid_sampling_grid_affine_tra[] = "Invalid sampling grid affine transform";
static char __pyx_k_Invalid_to_affine_transform_matr[] = "Invalid 'to' affine transform matrix";
static char __pyx_k_Invalid_world_to_image_transform[] = "Invalid world-to-image transform";
static char __pyx_k_Non_native_byte_order_not_suppor[] = "Non-native byte order not supported";
static char __pyx_k_Out_of_bounds_on_buffer_access_a[] = "Out of bounds on buffer access (axis %d)";
static char __pyx_k_Unable_to_convert_item_to_object[] = "Unable to convert item to object";
static char __pyx_k_Undefined_gradient_for_image_dim[] = "Undefined gradient for image dimension %d";
static char __pyx_k_downsample_displacement_field_2d[] = "downsample_displacement_field_2d";
static char __pyx_k_downsample_displacement_field_3d[] = "downsample_displacement_field_3d";
static char __pyx_k_got_differing_extents_in_dimensi[] = "got differing extents in dimension %d (got %d and %d)";
static char __pyx_k_ndarray_is_not_Fortran_contiguou[] = "ndarray is not Fortran contiguous";
static char __pyx_k_unable_to_allocate_shape_and_str[] = "unable to allocate shape and strides.";
static char __pyx_k_invert_vector_field_fixed_point_2[] = "invert_vector_field_fixed_point_3d";
static char __pyx_k_Format_string_allocated_too_shor_2[] = "Format string allocated too short.";
static PyObject *__pyx_kp_s_;
static PyObject *__pyx_n_s_ASCII;
static PyObject *__pyx_n_s_AttributeError;
static PyObject *__pyx_kp_s_Buffer_view_does_not_expose_stri;
static PyObject *__pyx_kp_s_Can_only_create_a_buffer_that_is;
static PyObject *__pyx_kp_s_Cannot_index_with_type_s;
static PyObject *__pyx_n_s_Ellipsis;
static PyObject *__pyx_kp_s_Empty_shape_tuple_for_cython_arr;
static PyObject *__pyx_kp_s_Expected_at_least_d_arguments;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor;
static PyObject *__pyx_kp_u_Format_string_allocated_too_shor_2;
static PyObject *__pyx_kp_s_Function_call_with_ambiguous_arg;
static PyObject *__pyx_n_s_ImportError;
static PyObject *__pyx_n_s_IndexError;
static PyObject *__pyx_kp_s_Indirect_dimensions_not_supporte;
static PyObject *__pyx_kp_s_Invalid_affine_transform_matrix;
static PyObject *__pyx_kp_s_Invalid_displacement_multiplicat;
static PyObject *__pyx_kp_s_Invalid_displacement_pre_multipl;
static PyObject *__pyx_kp_s_Invalid_from_affine_transform_ma;
static PyObject *__pyx_kp_s_Invalid_image_affine_transform;
static PyObject *__pyx_kp_s_Invalid_index_multiplication_mat;
static PyObject *__pyx_kp_s_Invalid_index_pre_multiplication;
static PyObject *__pyx_kp_s_Invalid_inner_index_multiplicati;
static PyObject *__pyx_kp_s_Invalid_mode_expected_c_or_fortr;
static PyObject *__pyx_kp_s_Invalid_outer_index_multiplicati;
static PyObject *__pyx_kp_s_Invalid_sampling_grid_affine_tra;
static PyObject *__pyx_kp_s_Invalid_shape_in_axis_d_d;
static PyObject *__pyx_kp_s_Invalid_spacings;
static PyObject *__pyx_kp_s_Invalid_to_affine_transform_matr;
static PyObject *__pyx_kp_s_Invalid_world_to_image_transform;
static PyObject *__pyx_n_s_M;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_kp_s_MemoryView_of_r_at_0x_x;
static PyObject *__pyx_kp_s_MemoryView_of_r_object;
static PyObject *__pyx_kp_s_No_matching_signature_found;
static PyObject *__pyx_kp_u_Non_native_byte_order_not_suppor;
static PyObject *__pyx_n_b_O;
static PyObject *__pyx_kp_s_Out_of_bounds_on_buffer_access_a;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_s_Unable_to_convert_item_to_object;
static PyObject *__pyx_kp_s_Undefined_gradient_for_image_dim;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_kp_s__3;
static PyObject *__pyx_n_s_affine;
static PyObject *__pyx_n_s_affine_disp;
static PyObject *__pyx_n_s_affine_idx_in;
static PyObject *__pyx_n_s_affine_idx_out;
static PyObject *__pyx_n_s_all;
static PyObject *__pyx_n_s_allocate_buffer;
static PyObject *__pyx_n_s_alpha;
static PyObject *__pyx_n_s_arctan2;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_asarray;
static PyObject *__pyx_n_s_astype;
static PyObject *__pyx_n_s_b;
static PyObject *__pyx_n_s_base;
static PyObject *__pyx_n_s_beta;
static PyObject *__pyx_n_s_c;
static PyObject *__pyx_n_u_c;
static PyObject *__pyx_n_s_calpha;
static PyObject *__pyx_n_s_cbeta;
static PyObject *__pyx_n_s_cgamma;
static PyObject *__pyx_n_s_class;
static PyObject *__pyx_n_s_cnt;
static PyObject *__pyx_n_s_comp;
static PyObject *__pyx_n_s_compose_vector_fields_2d;
static PyObject *__pyx_n_s_compose_vector_fields_3d;
static PyObject *__pyx_kp_s_contiguous_and_direct;
static PyObject *__pyx_kp_s_contiguous_and_indirect;
static PyObject *__pyx_n_s_cos;
static PyObject *__pyx_n_s_create_circle;
static PyObject *__pyx_n_s_create_harmonic_fields_2d;
static PyObject *__pyx_n_s_create_harmonic_fields_3d;
static PyObject *__pyx_n_s_create_random_displacement_2d;
static PyObject *__pyx_n_s_create_random_displacement_3d;
static PyObject *__pyx_n_s_create_sphere;
static PyObject *__pyx_n_s_current;
static PyObject *__pyx_n_s_d;
static PyObject *__pyx_n_s_d1;
static PyObject *__pyx_n_s_d2;
static PyObject *__pyx_n_s_d_world2grid;
static PyObject *__pyx_n_s_defaults;
static PyObject *__pyx_n_s_di;
static PyObject *__pyx_n_s_difmag;
static PyObject *__pyx_n_s_dii;
static PyObject *__pyx_n_s_dim;
static PyObject *__pyx_n_s_dipy_align_vector_fields;
static PyObject *__pyx_n_s_dj;
static PyObject *__pyx_n_s_djj;
static PyObject *__pyx_n_s_dk;
static PyObject *__pyx_n_s_dkk;
static PyObject *__pyx_n_s_dom_size;
static PyObject *__pyx_n_s_double;
static PyObject *__pyx_kp_s_double_complex;
static PyObject *__pyx_kp_s_double_complex_double;
static PyObject *__pyx_kp_s_double_complex_float;
static PyObject *__pyx_kp_s_double_double;
static PyObject *__pyx_kp_s_double_float;
static PyObject *__pyx_n_s_down;
static PyObject *__pyx_n_s_downsample_displacement_field_2d;
static PyObject *__pyx_n_s_downsample_displacement_field_3d;
static PyObject *__pyx_n_s_downsample_scalar_field_2d;
static PyObject *__pyx_n_s_downsample_scalar_field_3d;
static PyObject *__pyx_n_s_dtype;
static PyObject *__pyx_n_s_dtype_is_object;
static PyObject *__pyx_n_s_dx;
static PyObject *__pyx_n_s_empty;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_s_enumerate;
static PyObject *__pyx_n_s_epsilon;
static PyObject *__pyx_n_s_error;
static PyObject *__pyx_n_s_expanded;
static PyObject *__pyx_n_s_factors;
static PyObject *__pyx_n_s_field;
static PyObject *__pyx_n_s_flag;
static PyObject *__pyx_n_s_flags;
static PyObject *__pyx_n_s_float;
static PyObject *__pyx_n_s_float64;
static PyObject *__pyx_kp_s_float_complex;
static PyObject *__pyx_kp_s_float_complex_double;
static PyObject *__pyx_kp_s_float_complex_float;
static PyObject *__pyx_kp_s_float_double;
static PyObject *__pyx_kp_s_float_float;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_fortran;
static PyObject *__pyx_n_u_fortran;
static PyObject *__pyx_n_s_from_grid2world;
static PyObject *__pyx_n_s_from_shape;
static PyObject *__pyx_n_s_ftype;
static PyObject *__pyx_n_s_gamma;
static PyObject *__pyx_kp_s_got_differing_extents_in_dimensi;
static PyObject *__pyx_n_s_gradient;
static PyObject *__pyx_n_s_gradient_2d;
static PyObject *__pyx_n_s_gradient_3d;
static PyObject *__pyx_n_s_h;
static PyObject *__pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_s_id;
static PyObject *__pyx_n_s_ii;
static PyObject *__pyx_n_s_image;
static PyObject *__pyx_n_s_img;
static PyObject *__pyx_n_s_img_spacing;
static PyObject *__pyx_n_s_img_world2grid;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_in_flag;
static PyObject *__pyx_n_s_inside;
static PyObject *__pyx_n_s_int;
static PyObject *__pyx_n_s_int32;
static PyObject *__pyx_kp_s_int_double;
static PyObject *__pyx_n_s_int_field;
static PyObject *__pyx_kp_s_int_float;
static PyObject *__pyx_n_s_interpolate_scalar_2d;
static PyObject *__pyx_n_s_interpolate_scalar_3d;
static PyObject *__pyx_n_s_interpolate_scalar_nn_2d;
static PyObject *__pyx_n_s_interpolate_scalar_nn_3d;
static PyObject *__pyx_n_s_interpolate_vector_2d;
static PyObject *__pyx_n_s_interpolate_vector_3d;
static PyObject *__pyx_n_s_inv;
static PyObject *__pyx_n_s_invert_vector_field_fixed_point;
static PyObject *__pyx_n_s_invert_vector_field_fixed_point_2;
static PyObject *__pyx_n_s_is_valid_affine;
static PyObject *__pyx_n_s_isfinite;
static PyObject *__pyx_n_s_itemsize;
static PyObject *__pyx_kp_s_itemsize_0_for_cython_array;
static PyObject *__pyx_n_s_iter_count;
static PyObject *__pyx_n_s_j;
static PyObject *__pyx_n_s_jd_grad;
static PyObject *__pyx_n_s_jj;
static PyObject *__pyx_n_s_k;
static PyObject *__pyx_n_s_kind;
static PyObject *__pyx_n_s_kk;
static PyObject *__pyx_n_s_kwargs;
static PyObject *__pyx_n_s_locations;
static PyObject *__pyx_n_s_long;
static PyObject *__pyx_kp_s_long_double;
static PyObject *__pyx_kp_s_long_float;
static PyObject *__pyx_n_s_m;
static PyObject *__pyx_n_s_mag;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_max_iter;
static PyObject *__pyx_n_s_maxlen;
static PyObject *__pyx_n_s_memview;
static PyObject *__pyx_n_s_mid_col;
static PyObject *__pyx_n_s_mid_row;
static PyObject *__pyx_n_s_mid_slice;
static PyObject *__pyx_n_s_mode;
static PyObject *__pyx_n_s_n;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_nc;
static PyObject *__pyx_n_s_ncVol;
static PyObject *__pyx_n_s_ncols;
static PyObject *__pyx_n_s_ndarray;
static PyObject *__pyx_kp_u_ndarray_is_not_C_contiguous;
static PyObject *__pyx_kp_u_ndarray_is_not_Fortran_contiguou;
static PyObject *__pyx_n_s_ndim;
static PyObject *__pyx_n_s_nnc;
static PyObject *__pyx_n_s_nnr;
static PyObject *__pyx_n_s_nns;
static PyObject *__pyx_n_s_norms;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_nr;
static PyObject *__pyx_n_s_nrVol;
static PyObject *__pyx_n_s_nrows;
static PyObject *__pyx_n_s_ns;
static PyObject *__pyx_n_s_nsVol;
static PyObject *__pyx_n_s_nslices;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_n_s_obj;
static PyObject *__pyx_n_s_out;
static PyObject *__pyx_n_s_out_grid2world;
static PyObject *__pyx_n_s_out_shape;
static PyObject *__pyx_n_s_output;
static PyObject *__pyx_n_s_p;
static PyObject *__pyx_n_s_pack;
static PyObject *__pyx_n_s_premult_disp;
static PyObject *__pyx_n_s_premult_index;
static PyObject *__pyx_n_s_pyx_getbuffer;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_q;
static PyObject *__pyx_n_s_r;
static PyObject *__pyx_n_s_radius;
static PyObject *__pyx_n_s_randint;
static PyObject *__pyx_n_s_random;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_ref_shape;
static PyObject *__pyx_n_s_reorient_vector_field_2d;
static PyObject *__pyx_n_s_reorient_vector_field_3d;
static PyObject *__pyx_n_s_resample_displacement_field_2d;
static PyObject *__pyx_n_s_resample_displacement_field_3d;
static PyObject *__pyx_n_s_ri;
static PyObject *__pyx_n_s_rj;
static PyObject *__pyx_n_s_rk;
static PyObject *__pyx_n_s_s;
static PyObject *__pyx_n_s_sample_points;
static PyObject *__pyx_n_s_sc;
static PyObject *__pyx_n_s_shape;
static PyObject *__pyx_n_s_short;
static PyObject *__pyx_kp_s_short_double;
static PyObject *__pyx_kp_s_short_float;
static PyObject *__pyx_n_s_signatures;
static PyObject *__pyx_n_s_simplify_warp_function_2d;
static PyObject *__pyx_n_s_simplify_warp_function_3d;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_spacing;
static PyObject *__pyx_n_s_sparse_gradient;
static PyObject *__pyx_n_s_sparse_gradient_2d;
static PyObject *__pyx_n_s_sparse_gradient_3d;
static PyObject *__pyx_n_s_split;
static PyObject *__pyx_n_s_sqrt;
static PyObject *__pyx_n_s_sr;
static PyObject *__pyx_n_s_ss;
static PyObject *__pyx_n_s_start;
static PyObject *__pyx_n_s_stats;
static PyObject *__pyx_n_s_step;
static PyObject *__pyx_n_s_step_factor;
static PyObject *__pyx_n_s_stop;
static PyObject *__pyx_kp_s_strided_and_direct;
static PyObject *__pyx_kp_s_strided_and_direct_or_indirect;
static PyObject *__pyx_kp_s_strided_and_indirect;
static PyObject *__pyx_n_s_strip;
static PyObject *__pyx_n_s_struct;
static PyObject *__pyx_n_s_substats;
static PyObject *__pyx_n_s_tcols;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_theta;
static PyObject *__pyx_n_s_time_scaling;
static PyObject *__pyx_n_s_tmp;
static PyObject *__pyx_n_s_tmp0;
static PyObject *__pyx_n_s_tmp1;
static PyObject *__pyx_n_s_to_grid2world;
static PyObject *__pyx_n_s_to_shape;
static PyObject *__pyx_n_s_tol;
static PyObject *__pyx_n_s_tolerance;
static PyObject *__pyx_n_s_transform_2d_affine;
static PyObject *__pyx_n_s_transform_2d_affine_nn;
static PyObject *__pyx_n_s_transform_3d_affine;
static PyObject *__pyx_n_s_transform_3d_affine_nn;
static PyObject *__pyx_n_s_trows;
static PyObject *__pyx_n_s_tslices;
static PyObject *__pyx_n_s_type;
static PyObject *__pyx_kp_s_unable_to_allocate_array_data;
static PyObject *__pyx_kp_s_unable_to_allocate_shape_and_str;
static PyObject *__pyx_kp_u_unknown_dtype_code_in_numpy_pxd;
static PyObject *__pyx_n_s_unpack;
static PyObject *__pyx_n_s_volume;
static PyObject *__pyx_n_s_warp_2d;
static PyObject *__pyx_n_s_warp_2d_nn;
static PyObject *__pyx_n_s_warp_3d;
static PyObject *__pyx_n_s_warp_3d_nn;
static PyObject *__pyx_n_s_warped;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_n_s_zeros;
static PyObject *__pyx_n_s_zeros_like;
static PyObject *__pyx_n_s_zip;
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_is_valid_affine(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_M, int __pyx_v_dim); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_2interpolate_vector_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_82interpolate_vector_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_84interpolate_vector_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_4interpolate_scalar_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_88interpolate_scalar_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_90interpolate_scalar_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_6interpolate_scalar_nn_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_94interpolate_scalar_nn_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_96interpolate_scalar_nn_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_98interpolate_scalar_nn_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_100interpolate_scalar_nn_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_102interpolate_scalar_nn_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_104interpolate_scalar_nn_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_106interpolate_scalar_nn_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_8interpolate_scalar_nn_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_110interpolate_scalar_nn_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_112interpolate_scalar_nn_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_114interpolate_scalar_nn_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_116interpolate_scalar_nn_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_118interpolate_scalar_nn_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_120interpolate_scalar_nn_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_122interpolate_scalar_nn_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_10interpolate_scalar_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_126interpolate_scalar_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_128interpolate_scalar_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_12interpolate_vector_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_132interpolate_vector_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_134interpolate_vector_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field, __Pyx_memviewslice __pyx_v_locations); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_14compose_vector_fields_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_138compose_vector_fields_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_d2, __Pyx_memviewslice __pyx_v_premult_index, __Pyx_memviewslice __pyx_v_premult_disp, double __pyx_v_time_scaling, __Pyx_memviewslice __pyx_v_comp); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_140compose_vector_fields_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_d2, __Pyx_memviewslice __pyx_v_premult_index, __Pyx_memviewslice __pyx_v_premult_disp, double __pyx_v_time_scaling, __Pyx_memviewslice __pyx_v_comp); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_16compose_vector_fields_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_144compose_vector_fields_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_d2, __Pyx_memviewslice __pyx_v_premult_index, __Pyx_memviewslice __pyx_v_premult_disp, double __pyx_v_time_scaling, __Pyx_memviewslice __pyx_v_comp); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_146compose_vector_fields_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_d2, __Pyx_memviewslice __pyx_v_premult_index, __Pyx_memviewslice __pyx_v_premult_disp, double __pyx_v_time_scaling, __Pyx_memviewslice __pyx_v_comp); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_18invert_vector_field_fixed_point_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_366__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_150invert_vector_field_fixed_point_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_d_world2grid, __Pyx_memviewslice __pyx_v_spacing, int __pyx_v_max_iter, double __pyx_v_tolerance, __Pyx_memviewslice __pyx_v_start); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_368__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_152invert_vector_field_fixed_point_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_d_world2grid, __Pyx_memviewslice __pyx_v_spacing, int __pyx_v_max_iter, double __pyx_v_tolerance, __Pyx_memviewslice __pyx_v_start); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_20invert_vector_field_fixed_point_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_374__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_156invert_vector_field_fixed_point_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_d_world2grid, __Pyx_memviewslice __pyx_v_spacing, int __pyx_v_max_iter, double __pyx_v_tol, __Pyx_memviewslice __pyx_v_start); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_376__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_158invert_vector_field_fixed_point_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_d_world2grid, __Pyx_memviewslice __pyx_v_spacing, int __pyx_v_max_iter, double __pyx_v_tol, __Pyx_memviewslice __pyx_v_start); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_22simplify_warp_function_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_162simplify_warp_function_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_164simplify_warp_function_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_24simplify_warp_function_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_168simplify_warp_function_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_170simplify_warp_function_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_26reorient_vector_field_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_174reorient_vector_field_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_176reorient_vector_field_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_28reorient_vector_field_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_180reorient_vector_field_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_182reorient_vector_field_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_30downsample_scalar_field_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_186downsample_scalar_field_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_188downsample_scalar_field_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_32downsample_displacement_field_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_192downsample_displacement_field_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_194downsample_displacement_field_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_34downsample_scalar_field_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_198downsample_scalar_field_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_200downsample_scalar_field_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_36downsample_displacement_field_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_204downsample_displacement_field_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_206downsample_displacement_field_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_38warp_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_382__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_210warp_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_384__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_212warp_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_40transform_3d_affine(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_216transform_3d_affine(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_218transform_3d_affine(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_42warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_414__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_222warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_416__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_224warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_418__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_226warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_420__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_228warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_422__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_230warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_424__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_232warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_426__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_234warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_428__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_236warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_430__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_238warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_432__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_240warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_434__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_242warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_436__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_244warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_438__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_246warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_440__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_248warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_44transform_3d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_456__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_252transform_3d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_458__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_254transform_3d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_460__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_256transform_3d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_462__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_258transform_3d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_464__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_260transform_3d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_466__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_262transform_3d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_468__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_264transform_3d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_46warp_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_474__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_268warp_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_476__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_270warp_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_48transform_2d_affine(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_482__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_274transform_2d_affine(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_484__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_276transform_2d_affine(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_50warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_514__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_280warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_516__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_282warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_518__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_284warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_520__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_286warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_522__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_288warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_524__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_290warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_526__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_292warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_528__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_294warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_530__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_296warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_532__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_298warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_534__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_300warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_536__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_302warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_538__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_304warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_540__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_306warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_52transform_2d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_556__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_310transform_2d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_558__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_312transform_2d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_560__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_314transform_2d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_562__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_316transform_2d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_564__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_318transform_2d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_566__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_320transform_2d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_568__defaults__(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_322transform_2d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_54resample_displacement_field_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_326resample_displacement_field_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field, __Pyx_memviewslice __pyx_v_factors, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_328resample_displacement_field_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field, __Pyx_memviewslice __pyx_v_factors, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_56resample_displacement_field_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_332resample_displacement_field_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field, __Pyx_memviewslice __pyx_v_factors, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_334resample_displacement_field_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field, __Pyx_memviewslice __pyx_v_factors, __Pyx_memviewslice __pyx_v_out_shape); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_58create_random_displacement_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_from_shape, __Pyx_memviewslice __pyx_v_from_grid2world, __Pyx_memviewslice __pyx_v_to_shape, __Pyx_memviewslice __pyx_v_to_grid2world); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_60create_random_displacement_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_from_shape, __Pyx_memviewslice __pyx_v_from_grid2world, __Pyx_memviewslice __pyx_v_to_shape, __Pyx_memviewslice __pyx_v_to_grid2world); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_62create_harmonic_fields_2d(CYTHON_UNUSED PyObject *__pyx_self, npy_intp __pyx_v_nrows, npy_intp __pyx_v_ncols, double __pyx_v_b, double __pyx_v_m); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_64create_harmonic_fields_3d(CYTHON_UNUSED PyObject *__pyx_self, int __pyx_v_nslices, npy_intp __pyx_v_nrows, npy_intp __pyx_v_ncols, double __pyx_v_b, double __pyx_v_m); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_66create_circle(CYTHON_UNUSED PyObject *__pyx_self, npy_intp __pyx_v_nrows, npy_intp __pyx_v_ncols, npy_intp __pyx_v_radius); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_68create_sphere(CYTHON_UNUSED PyObject *__pyx_self, npy_intp __pyx_v_nslices, npy_intp __pyx_v_nrows, npy_intp __pyx_v_ncols, npy_intp __pyx_v_radius); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_70_gradient_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_338_gradient_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_img, __Pyx_memviewslice __pyx_v_img_world2grid, __Pyx_memviewslice __pyx_v_img_spacing, __Pyx_memviewslice __pyx_v_out_grid2world, __Pyx_memviewslice __pyx_v_out, __Pyx_memviewslice __pyx_v_inside); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_340_gradient_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_img, __Pyx_memviewslice __pyx_v_img_world2grid, __Pyx_memviewslice __pyx_v_img_spacing, __Pyx_memviewslice __pyx_v_out_grid2world, __Pyx_memviewslice __pyx_v_out, __Pyx_memviewslice __pyx_v_inside); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_72_sparse_gradient_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_344_sparse_gradient_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_img, __Pyx_memviewslice __pyx_v_img_world2grid, __Pyx_memviewslice __pyx_v_img_spacing, __Pyx_memviewslice __pyx_v_sample_points, __Pyx_memviewslice __pyx_v_out, __Pyx_memviewslice __pyx_v_inside); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_346_sparse_gradient_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_img, __Pyx_memviewslice __pyx_v_img_world2grid, __Pyx_memviewslice __pyx_v_img_spacing, __Pyx_memviewslice __pyx_v_sample_points, __Pyx_memviewslice __pyx_v_out, __Pyx_memviewslice __pyx_v_inside); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_74_gradient_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_350_gradient_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_img, __Pyx_memviewslice __pyx_v_img_world2grid, __Pyx_memviewslice __pyx_v_img_spacing, __Pyx_memviewslice __pyx_v_out_grid2world, __Pyx_memviewslice __pyx_v_out, __Pyx_memviewslice __pyx_v_inside); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_352_gradient_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_img, __Pyx_memviewslice __pyx_v_img_world2grid, __Pyx_memviewslice __pyx_v_img_spacing, __Pyx_memviewslice __pyx_v_out_grid2world, __Pyx_memviewslice __pyx_v_out, __Pyx_memviewslice __pyx_v_inside); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_76_sparse_gradient_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_356_sparse_gradient_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_img, __Pyx_memviewslice __pyx_v_img_world2grid, __Pyx_memviewslice __pyx_v_img_spacing, __Pyx_memviewslice __pyx_v_sample_points, __Pyx_memviewslice __pyx_v_out, __Pyx_memviewslice __pyx_v_inside); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_358_sparse_gradient_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_img, __Pyx_memviewslice __pyx_v_img_world2grid, __Pyx_memviewslice __pyx_v_img_spacing, __Pyx_memviewslice __pyx_v_sample_points, __Pyx_memviewslice __pyx_v_out, __Pyx_memviewslice __pyx_v_inside); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_78gradient(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_img, PyObject *__pyx_v_img_world2grid, PyObject *__pyx_v_img_spacing, PyObject *__pyx_v_out_shape, PyObject *__pyx_v_out_grid2world); /* proto */
static PyObject *__pyx_pf_4dipy_5align_13vector_fields_80sparse_gradient(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_img, PyObject *__pyx_v_img_world2grid, PyObject *__pyx_v_img_spacing, PyObject *__pyx_v_sample_points); /* proto */
static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info); /* proto */
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array___cinit__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, PyObject *__pyx_v_format, PyObject *__pyx_v_mode, int __pyx_v_allocate_buffer); /* proto */
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_2__getbuffer__(struct __pyx_array_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static void __pyx_array___pyx_pf_15View_dot_MemoryView_5array_4__dealloc__(struct __pyx_array_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_5array_7memview___get__(struct __pyx_array_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_6__getattr__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_attr); /* proto */
static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_8__getitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item); /* proto */
static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_10__setitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value); /* proto */
static int __pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum___init__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum_2__repr__(struct __pyx_MemviewEnum_obj *__pyx_v_self); /* proto */
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview___cinit__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_v_flags, int __pyx_v_dtype_is_object); /* proto */
static void __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_2__dealloc__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_4__getitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index); /* proto */
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_6__setitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /* proto */
static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_8__getbuffer__(struct __pyx_memoryview_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_1T___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4base___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_5shape___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_7strides___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_10suboffsets___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4ndim___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_8itemsize___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_6nbytes___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4size___get__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_10__len__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_12__repr__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_14__str__(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_16is_c_contig(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_18is_f_contig(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_20copy(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_22copy_fortran(struct __pyx_memoryview_obj *__pyx_v_self); /* proto */
static void __pyx_memoryviewslice___pyx_pf_15View_dot_MemoryView_16_memoryviewslice___dealloc__(struct __pyx_memoryviewslice_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_15View_dot_MemoryView_16_memoryviewslice_4base___get__(struct __pyx_memoryviewslice_obj *__pyx_v_self); /* proto */
static PyObject *__pyx_tp_new_array(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_Enum(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_memoryview(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new__memoryviewslice(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_float_1_0;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_3;
static PyObject *__pyx_int_neg_1;
static PyObject *__pyx_k__123;
static PyObject *__pyx_k__124;
static PyObject *__pyx_k__143;
static PyObject *__pyx_k__144;
static PyObject *__pyx_k__205;
static PyObject *__pyx_k__217;
static PyObject *__pyx_k__218;
static PyObject *__pyx_k__231;
static PyObject *__pyx_k__238;
static PyObject *__pyx_k__239;
static PyObject *__pyx_k__300;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__32;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__34;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_tuple__38;
static PyObject *__pyx_tuple__39;
static PyObject *__pyx_tuple__40;
static PyObject *__pyx_tuple__41;
static PyObject *__pyx_tuple__42;
static PyObject *__pyx_tuple__43;
static PyObject *__pyx_tuple__44;
static PyObject *__pyx_tuple__45;
static PyObject *__pyx_tuple__46;
static PyObject *__pyx_tuple__47;
static PyObject *__pyx_tuple__48;
static PyObject *__pyx_tuple__49;
static PyObject *__pyx_tuple__50;
static PyObject *__pyx_tuple__51;
static PyObject *__pyx_tuple__52;
static PyObject *__pyx_tuple__53;
static PyObject *__pyx_tuple__54;
static PyObject *__pyx_tuple__55;
static PyObject *__pyx_tuple__56;
static PyObject *__pyx_tuple__57;
static PyObject *__pyx_tuple__58;
static PyObject *__pyx_tuple__59;
static PyObject *__pyx_tuple__60;
static PyObject *__pyx_tuple__61;
static PyObject *__pyx_tuple__62;
static PyObject *__pyx_tuple__63;
static PyObject *__pyx_tuple__64;
static PyObject *__pyx_tuple__65;
static PyObject *__pyx_tuple__66;
static PyObject *__pyx_tuple__67;
static PyObject *__pyx_tuple__68;
static PyObject *__pyx_tuple__69;
static PyObject *__pyx_tuple__70;
static PyObject *__pyx_tuple__71;
static PyObject *__pyx_tuple__72;
static PyObject *__pyx_tuple__73;
static PyObject *__pyx_tuple__74;
static PyObject *__pyx_tuple__75;
static PyObject *__pyx_tuple__76;
static PyObject *__pyx_tuple__77;
static PyObject *__pyx_tuple__78;
static PyObject *__pyx_tuple__79;
static PyObject *__pyx_tuple__80;
static PyObject *__pyx_tuple__81;
static PyObject *__pyx_tuple__82;
static PyObject *__pyx_tuple__83;
static PyObject *__pyx_tuple__84;
static PyObject *__pyx_tuple__85;
static PyObject *__pyx_tuple__86;
static PyObject *__pyx_tuple__87;
static PyObject *__pyx_tuple__88;
static PyObject *__pyx_tuple__89;
static PyObject *__pyx_tuple__90;
static PyObject *__pyx_tuple__91;
static PyObject *__pyx_tuple__92;
static PyObject *__pyx_tuple__93;
static PyObject *__pyx_tuple__94;
static PyObject *__pyx_tuple__95;
static PyObject *__pyx_tuple__96;
static PyObject *__pyx_tuple__97;
static PyObject *__pyx_tuple__98;
static PyObject *__pyx_tuple__99;
static PyObject *__pyx_slice__392;
static PyObject *__pyx_slice__393;
static PyObject *__pyx_slice__394;
static PyObject *__pyx_tuple__100;
static PyObject *__pyx_tuple__101;
static PyObject *__pyx_tuple__102;
static PyObject *__pyx_tuple__103;
static PyObject *__pyx_tuple__104;
static PyObject *__pyx_tuple__105;
static PyObject *__pyx_tuple__106;
static PyObject *__pyx_tuple__107;
static PyObject *__pyx_tuple__108;
static PyObject *__pyx_tuple__109;
static PyObject *__pyx_tuple__110;
static PyObject *__pyx_tuple__111;
static PyObject *__pyx_tuple__112;
static PyObject *__pyx_tuple__113;
static PyObject *__pyx_tuple__114;
static PyObject *__pyx_tuple__115;
static PyObject *__pyx_tuple__116;
static PyObject *__pyx_tuple__117;
static PyObject *__pyx_tuple__118;
static PyObject *__pyx_tuple__119;
static PyObject *__pyx_tuple__120;
static PyObject *__pyx_tuple__121;
static PyObject *__pyx_tuple__122;
static PyObject *__pyx_tuple__125;
static PyObject *__pyx_tuple__126;
static PyObject *__pyx_tuple__127;
static PyObject *__pyx_tuple__128;
static PyObject *__pyx_tuple__129;
static PyObject *__pyx_tuple__130;
static PyObject *__pyx_tuple__131;
static PyObject *__pyx_tuple__132;
static PyObject *__pyx_tuple__133;
static PyObject *__pyx_tuple__134;
static PyObject *__pyx_tuple__135;
static PyObject *__pyx_tuple__136;
static PyObject *__pyx_tuple__137;
static PyObject *__pyx_tuple__138;
static PyObject *__pyx_tuple__139;
static PyObject *__pyx_tuple__140;
static PyObject *__pyx_tuple__141;
static PyObject *__pyx_tuple__142;
static PyObject *__pyx_tuple__145;
static PyObject *__pyx_tuple__146;
static PyObject *__pyx_tuple__147;
static PyObject *__pyx_tuple__148;
static PyObject *__pyx_tuple__149;
static PyObject *__pyx_tuple__150;
static PyObject *__pyx_tuple__151;
static PyObject *__pyx_tuple__152;
static PyObject *__pyx_tuple__153;
static PyObject *__pyx_tuple__154;
static PyObject *__pyx_tuple__155;
static PyObject *__pyx_tuple__156;
static PyObject *__pyx_tuple__157;
static PyObject *__pyx_tuple__158;
static PyObject *__pyx_tuple__159;
static PyObject *__pyx_tuple__160;
static PyObject *__pyx_tuple__161;
static PyObject *__pyx_tuple__162;
static PyObject *__pyx_tuple__163;
static PyObject *__pyx_tuple__164;
static PyObject *__pyx_tuple__165;
static PyObject *__pyx_tuple__166;
static PyObject *__pyx_tuple__167;
static PyObject *__pyx_tuple__168;
static PyObject *__pyx_tuple__169;
static PyObject *__pyx_tuple__170;
static PyObject *__pyx_tuple__171;
static PyObject *__pyx_tuple__172;
static PyObject *__pyx_tuple__173;
static PyObject *__pyx_tuple__174;
static PyObject *__pyx_tuple__175;
static PyObject *__pyx_tuple__176;
static PyObject *__pyx_tuple__177;
static PyObject *__pyx_tuple__178;
static PyObject *__pyx_tuple__179;
static PyObject *__pyx_tuple__180;
static PyObject *__pyx_tuple__181;
static PyObject *__pyx_tuple__182;
static PyObject *__pyx_tuple__183;
static PyObject *__pyx_tuple__184;
static PyObject *__pyx_tuple__185;
static PyObject *__pyx_tuple__186;
static PyObject *__pyx_tuple__187;
static PyObject *__pyx_tuple__188;
static PyObject *__pyx_tuple__189;
static PyObject *__pyx_tuple__190;
static PyObject *__pyx_tuple__191;
static PyObject *__pyx_tuple__192;
static PyObject *__pyx_tuple__193;
static PyObject *__pyx_tuple__194;
static PyObject *__pyx_tuple__195;
static PyObject *__pyx_tuple__196;
static PyObject *__pyx_tuple__197;
static PyObject *__pyx_tuple__198;
static PyObject *__pyx_tuple__199;
static PyObject *__pyx_tuple__200;
static PyObject *__pyx_tuple__201;
static PyObject *__pyx_tuple__202;
static PyObject *__pyx_tuple__203;
static PyObject *__pyx_tuple__204;
static PyObject *__pyx_tuple__206;
static PyObject *__pyx_tuple__207;
static PyObject *__pyx_tuple__208;
static PyObject *__pyx_tuple__209;
static PyObject *__pyx_tuple__210;
static PyObject *__pyx_tuple__211;
static PyObject *__pyx_tuple__212;
static PyObject *__pyx_tuple__213;
static PyObject *__pyx_tuple__214;
static PyObject *__pyx_tuple__215;
static PyObject *__pyx_tuple__216;
static PyObject *__pyx_tuple__219;
static PyObject *__pyx_tuple__220;
static PyObject *__pyx_tuple__221;
static PyObject *__pyx_tuple__222;
static PyObject *__pyx_tuple__223;
static PyObject *__pyx_tuple__224;
static PyObject *__pyx_tuple__225;
static PyObject *__pyx_tuple__226;
static PyObject *__pyx_tuple__227;
static PyObject *__pyx_tuple__228;
static PyObject *__pyx_tuple__229;
static PyObject *__pyx_tuple__230;
static PyObject *__pyx_tuple__232;
static PyObject *__pyx_tuple__233;
static PyObject *__pyx_tuple__234;
static PyObject *__pyx_tuple__235;
static PyObject *__pyx_tuple__236;
static PyObject *__pyx_tuple__237;
static PyObject *__pyx_tuple__240;
static PyObject *__pyx_tuple__241;
static PyObject *__pyx_tuple__242;
static PyObject *__pyx_tuple__243;
static PyObject *__pyx_tuple__244;
static PyObject *__pyx_tuple__245;
static PyObject *__pyx_tuple__246;
static PyObject *__pyx_tuple__247;
static PyObject *__pyx_tuple__248;
static PyObject *__pyx_tuple__249;
static PyObject *__pyx_tuple__250;
static PyObject *__pyx_tuple__251;
static PyObject *__pyx_tuple__252;
static PyObject *__pyx_tuple__253;
static PyObject *__pyx_tuple__254;
static PyObject *__pyx_tuple__255;
static PyObject *__pyx_tuple__256;
static PyObject *__pyx_tuple__257;
static PyObject *__pyx_tuple__258;
static PyObject *__pyx_tuple__259;
static PyObject *__pyx_tuple__260;
static PyObject *__pyx_tuple__261;
static PyObject *__pyx_tuple__262;
static PyObject *__pyx_tuple__263;
static PyObject *__pyx_tuple__264;
static PyObject *__pyx_tuple__265;
static PyObject *__pyx_tuple__266;
static PyObject *__pyx_tuple__267;
static PyObject *__pyx_tuple__268;
static PyObject *__pyx_tuple__269;
static PyObject *__pyx_tuple__270;
static PyObject *__pyx_tuple__271;
static PyObject *__pyx_tuple__272;
static PyObject *__pyx_tuple__273;
static PyObject *__pyx_tuple__274;
static PyObject *__pyx_tuple__275;
static PyObject *__pyx_tuple__276;
static PyObject *__pyx_tuple__277;
static PyObject *__pyx_tuple__278;
static PyObject *__pyx_tuple__279;
static PyObject *__pyx_tuple__280;
static PyObject *__pyx_tuple__281;
static PyObject *__pyx_tuple__282;
static PyObject *__pyx_tuple__283;
static PyObject *__pyx_tuple__284;
static PyObject *__pyx_tuple__285;
static PyObject *__pyx_tuple__286;
static PyObject *__pyx_tuple__287;
static PyObject *__pyx_tuple__288;
static PyObject *__pyx_tuple__289;
static PyObject *__pyx_tuple__290;
static PyObject *__pyx_tuple__291;
static PyObject *__pyx_tuple__292;
static PyObject *__pyx_tuple__293;
static PyObject *__pyx_tuple__294;
static PyObject *__pyx_tuple__295;
static PyObject *__pyx_tuple__296;
static PyObject *__pyx_tuple__297;
static PyObject *__pyx_tuple__298;
static PyObject *__pyx_tuple__299;
static PyObject *__pyx_tuple__301;
static PyObject *__pyx_tuple__302;
static PyObject *__pyx_tuple__303;
static PyObject *__pyx_tuple__304;
static PyObject *__pyx_tuple__305;
static PyObject *__pyx_tuple__306;
static PyObject *__pyx_tuple__307;
static PyObject *__pyx_tuple__308;
static PyObject *__pyx_tuple__309;
static PyObject *__pyx_tuple__310;
static PyObject *__pyx_tuple__311;
static PyObject *__pyx_tuple__312;
static PyObject *__pyx_tuple__313;
static PyObject *__pyx_tuple__314;
static PyObject *__pyx_tuple__315;
static PyObject *__pyx_tuple__316;
static PyObject *__pyx_tuple__317;
static PyObject *__pyx_tuple__318;
static PyObject *__pyx_tuple__319;
static PyObject *__pyx_tuple__320;
static PyObject *__pyx_tuple__321;
static PyObject *__pyx_tuple__322;
static PyObject *__pyx_tuple__323;
static PyObject *__pyx_tuple__324;
static PyObject *__pyx_tuple__325;
static PyObject *__pyx_tuple__326;
static PyObject *__pyx_tuple__327;
static PyObject *__pyx_tuple__328;
static PyObject *__pyx_tuple__329;
static PyObject *__pyx_tuple__330;
static PyObject *__pyx_tuple__331;
static PyObject *__pyx_tuple__332;
static PyObject *__pyx_tuple__333;
static PyObject *__pyx_tuple__334;
static PyObject *__pyx_tuple__335;
static PyObject *__pyx_tuple__336;
static PyObject *__pyx_tuple__337;
static PyObject *__pyx_tuple__338;
static PyObject *__pyx_tuple__339;
static PyObject *__pyx_tuple__340;
static PyObject *__pyx_tuple__341;
static PyObject *__pyx_tuple__342;
static PyObject *__pyx_tuple__343;
static PyObject *__pyx_tuple__344;
static PyObject *__pyx_tuple__345;
static PyObject *__pyx_tuple__346;
static PyObject *__pyx_tuple__347;
static PyObject *__pyx_tuple__348;
static PyObject *__pyx_tuple__349;
static PyObject *__pyx_tuple__350;
static PyObject *__pyx_tuple__351;
static PyObject *__pyx_tuple__352;
static PyObject *__pyx_tuple__353;
static PyObject *__pyx_tuple__354;
static PyObject *__pyx_tuple__355;
static PyObject *__pyx_tuple__356;
static PyObject *__pyx_tuple__357;
static PyObject *__pyx_tuple__358;
static PyObject *__pyx_tuple__359;
static PyObject *__pyx_tuple__360;
static PyObject *__pyx_tuple__361;
static PyObject *__pyx_tuple__362;
static PyObject *__pyx_tuple__363;
static PyObject *__pyx_tuple__364;
static PyObject *__pyx_tuple__365;
static PyObject *__pyx_tuple__366;
static PyObject *__pyx_tuple__367;
static PyObject *__pyx_tuple__368;
static PyObject *__pyx_tuple__369;
static PyObject *__pyx_tuple__370;
static PyObject *__pyx_tuple__371;
static PyObject *__pyx_tuple__372;
static PyObject *__pyx_tuple__373;
static PyObject *__pyx_tuple__374;
static PyObject *__pyx_tuple__375;
static PyObject *__pyx_tuple__376;
static PyObject *__pyx_tuple__377;
static PyObject *__pyx_tuple__378;
static PyObject *__pyx_tuple__379;
static PyObject *__pyx_tuple__380;
static PyObject *__pyx_tuple__381;
static PyObject *__pyx_tuple__382;
static PyObject *__pyx_tuple__383;
static PyObject *__pyx_tuple__384;
static PyObject *__pyx_tuple__385;
static PyObject *__pyx_tuple__386;
static PyObject *__pyx_tuple__387;
static PyObject *__pyx_tuple__388;
static PyObject *__pyx_tuple__389;
static PyObject *__pyx_tuple__390;
static PyObject *__pyx_tuple__391;
static PyObject *__pyx_tuple__395;
static PyObject *__pyx_tuple__396;
static PyObject *__pyx_tuple__398;
static PyObject *__pyx_tuple__400;
static PyObject *__pyx_tuple__402;
static PyObject *__pyx_tuple__404;
static PyObject *__pyx_tuple__406;
static PyObject *__pyx_tuple__408;
static PyObject *__pyx_tuple__410;
static PyObject *__pyx_tuple__412;
static PyObject *__pyx_tuple__414;
static PyObject *__pyx_tuple__416;
static PyObject *__pyx_tuple__418;
static PyObject *__pyx_tuple__420;
static PyObject *__pyx_tuple__422;
static PyObject *__pyx_tuple__424;
static PyObject *__pyx_tuple__426;
static PyObject *__pyx_tuple__428;
static PyObject *__pyx_tuple__430;
static PyObject *__pyx_tuple__432;
static PyObject *__pyx_tuple__434;
static PyObject *__pyx_tuple__436;
static PyObject *__pyx_tuple__438;
static PyObject *__pyx_tuple__440;
static PyObject *__pyx_tuple__442;
static PyObject *__pyx_tuple__444;
static PyObject *__pyx_tuple__446;
static PyObject *__pyx_tuple__448;
static PyObject *__pyx_tuple__450;
static PyObject *__pyx_tuple__452;
static PyObject *__pyx_tuple__454;
static PyObject *__pyx_tuple__456;
static PyObject *__pyx_tuple__458;
static PyObject *__pyx_tuple__460;
static PyObject *__pyx_tuple__462;
static PyObject *__pyx_tuple__464;
static PyObject *__pyx_tuple__466;
static PyObject *__pyx_tuple__468;
static PyObject *__pyx_tuple__470;
static PyObject *__pyx_tuple__472;
static PyObject *__pyx_tuple__474;
static PyObject *__pyx_tuple__476;
static PyObject *__pyx_tuple__478;
static PyObject *__pyx_tuple__479;
static PyObject *__pyx_tuple__480;
static PyObject *__pyx_tuple__481;
static PyObject *__pyx_tuple__482;
static PyObject *__pyx_codeobj__397;
static PyObject *__pyx_codeobj__399;
static PyObject *__pyx_codeobj__401;
static PyObject *__pyx_codeobj__403;
static PyObject *__pyx_codeobj__405;
static PyObject *__pyx_codeobj__407;
static PyObject *__pyx_codeobj__409;
static PyObject *__pyx_codeobj__411;
static PyObject *__pyx_codeobj__413;
static PyObject *__pyx_codeobj__415;
static PyObject *__pyx_codeobj__417;
static PyObject *__pyx_codeobj__419;
static PyObject *__pyx_codeobj__421;
static PyObject *__pyx_codeobj__423;
static PyObject *__pyx_codeobj__425;
static PyObject *__pyx_codeobj__427;
static PyObject *__pyx_codeobj__429;
static PyObject *__pyx_codeobj__431;
static PyObject *__pyx_codeobj__433;
static PyObject *__pyx_codeobj__435;
static PyObject *__pyx_codeobj__437;
static PyObject *__pyx_codeobj__439;
static PyObject *__pyx_codeobj__441;
static PyObject *__pyx_codeobj__443;
static PyObject *__pyx_codeobj__445;
static PyObject *__pyx_codeobj__447;
static PyObject *__pyx_codeobj__449;
static PyObject *__pyx_codeobj__451;
static PyObject *__pyx_codeobj__453;
static PyObject *__pyx_codeobj__455;
static PyObject *__pyx_codeobj__457;
static PyObject *__pyx_codeobj__459;
static PyObject *__pyx_codeobj__461;
static PyObject *__pyx_codeobj__463;
static PyObject *__pyx_codeobj__465;
static PyObject *__pyx_codeobj__467;
static PyObject *__pyx_codeobj__469;
static PyObject *__pyx_codeobj__471;
static PyObject *__pyx_codeobj__473;
static PyObject *__pyx_codeobj__475;
static PyObject *__pyx_codeobj__477;

/* "dipy/align/vector_fields.pyx":15
 * 
 * 
 * def is_valid_affine(double[:, :] M, int dim):             # <<<<<<<<<<<<<<
 *     if M is None:
 *         return True
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_1is_valid_affine(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_1is_valid_affine = {"is_valid_affine", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_1is_valid_affine, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_1is_valid_affine(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_M = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_v_dim;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_valid_affine (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_M,&__pyx_n_s_dim,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_M)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dim)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("is_valid_affine", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 15; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "is_valid_affine") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 15; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_M = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[0]); if (unlikely(!__pyx_v_M.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 15; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_dim = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_dim == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 15; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("is_valid_affine", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 15; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.is_valid_affine", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_is_valid_affine(__pyx_self, __pyx_v_M, __pyx_v_dim);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_is_valid_affine(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_M, int __pyx_v_dim) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_valid_affine", 0);

  /* "dipy/align/vector_fields.pyx":16
 * 
 * def is_valid_affine(double[:, :] M, int dim):
 *     if M is None:             # <<<<<<<<<<<<<<
 *         return True
 *     if M.shape[0] < dim or M.shape[1] < dim + 1:
 */
  __pyx_t_1 = ((((PyObject *) __pyx_v_M.memview) == Py_None) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":17
 * def is_valid_affine(double[:, :] M, int dim):
 *     if M is None:
 *         return True             # <<<<<<<<<<<<<<
 *     if M.shape[0] < dim or M.shape[1] < dim + 1:
 *         return False
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_True);
    __pyx_r = Py_True;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":16
 * 
 * def is_valid_affine(double[:, :] M, int dim):
 *     if M is None:             # <<<<<<<<<<<<<<
 *         return True
 *     if M.shape[0] < dim or M.shape[1] < dim + 1:
 */
  }

  /* "dipy/align/vector_fields.pyx":18
 *     if M is None:
 *         return True
 *     if M.shape[0] < dim or M.shape[1] < dim + 1:             # <<<<<<<<<<<<<<
 *         return False
 *     if not np.all(np.isfinite(M)):
 */
  __pyx_t_2 = (((__pyx_v_M.shape[0]) < __pyx_v_dim) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_2 = (((__pyx_v_M.shape[1]) < (__pyx_v_dim + 1)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":19
 *         return True
 *     if M.shape[0] < dim or M.shape[1] < dim + 1:
 *         return False             # <<<<<<<<<<<<<<
 *     if not np.all(np.isfinite(M)):
 *         return False
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":18
 *     if M is None:
 *         return True
 *     if M.shape[0] < dim or M.shape[1] < dim + 1:             # <<<<<<<<<<<<<<
 *         return False
 *     if not np.all(np.isfinite(M)):
 */
  }

  /* "dipy/align/vector_fields.pyx":20
 *     if M.shape[0] < dim or M.shape[1] < dim + 1:
 *         return False
 *     if not np.all(np.isfinite(M)):             # <<<<<<<<<<<<<<
 *         return False
 *     return True
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_all); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_isfinite); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_M, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_8 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
    }
  }
  if (!__pyx_t_8) {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_4);
  } else {
    __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_9, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  if (!__pyx_t_7) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_9, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 20; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_2 = ((!__pyx_t_1) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":21
 *         return False
 *     if not np.all(np.isfinite(M)):
 *         return False             # <<<<<<<<<<<<<<
 *     return True
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":20
 *     if M.shape[0] < dim or M.shape[1] < dim + 1:
 *         return False
 *     if not np.all(np.isfinite(M)):             # <<<<<<<<<<<<<<
 *         return False
 *     return True
 */
  }

  /* "dipy/align/vector_fields.pyx":22
 *     if not np.all(np.isfinite(M)):
 *         return False
 *     return True             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_True);
  __pyx_r = Py_True;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":15
 * 
 * 
 * def is_valid_affine(double[:, :] M, int dim):             # <<<<<<<<<<<<<<
 *     if M is None:
 *         return True
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("dipy.align.vector_fields.is_valid_affine", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_M, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":25
 * 
 * 
 * def interpolate_vector_2d(floating[:, :, :] field, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Bilinear interpolation of a 2D vector field
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_3interpolate_vector_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_2interpolate_vector_2d[] = "Bilinear interpolation of a 2D vector field\n\n    Interpolates the 2D vector field at the given locations. This function is\n    a wrapper for _interpolate_vector_2d for testing purposes, it is\n    equivalent to using scipy.ndimage.interpolation.map_coordinates with\n    bilinear interpolation at each vector component\n\n    Parameters\n    ----------\n    field : array, shape (S, R, 2)\n        the 2D vector field to be interpolated\n    locations : array, shape (n, 2)\n        (locations[i,0], locations[i,1]), 0<=i<n must contain the row and\n        column coordinates to interpolate the vector field at\n\n    Returns\n    -------\n    out : array, shape (n, 2)\n        out[i,:], 0<=i<n will be the interpolated vector at coordinates\n        locations[i,:], or (0,0) if locations[i,:] is outside the field\n    inside : array, (n,)\n        if (locations[i,0], locations[i,1]) is inside the vector field\n        then inside[i]=1, else inside[i]=0\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_3interpolate_vector_2d = {"interpolate_vector_2d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_3interpolate_vector_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_2interpolate_vector_2d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_3interpolate_vector_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_2interpolate_vector_2d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_2interpolate_vector_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("interpolate_vector_2d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_field, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_field); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_83interpolate_vector_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_83interpolate_vector_2d = {"__pyx_fuse_0interpolate_vector_2d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_83interpolate_vector_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_2interpolate_vector_2d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_83interpolate_vector_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_vector_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_field,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_field)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_vector_2d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_vector_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_field = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[0]); if (unlikely(!__pyx_v_field.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_vector_2d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_vector_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_82interpolate_vector_2d(__pyx_self, __pyx_v_field, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_82interpolate_vector_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0interpolate_vector_2d", 0);

  /* "dipy/align/vector_fields.pyx":50
 *         then inside[i]=1, else inside[i]=0
 *     """
 *     ftype = np.asarray(field).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_field, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":52
 *     ftype = np.asarray(field).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         floating[:, :] out = np.zeros(shape=(n, 2), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":53
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         floating[:, :] out = np.zeros(shape=(n, 2), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_int_2);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":54
 *         cnp.npy_intp n = locations.shape[0]
 *         floating[:, :] out = np.zeros(shape=(n, 2), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":55
 *         floating[:, :] out = np.zeros(shape=(n, 2), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_vector_2d[floating](field,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":56
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_vector_2d[floating](field,
 *                 locations[i, 0], locations[i, 1], out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":58
 *         for i in range(n):
 *             inside[i] = _interpolate_vector_2d[floating](field,
 *                 locations[i, 0], locations[i, 1], out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14.data = __pyx_v_out.data;
          __pyx_t_14.memview = __pyx_v_out.memview;
          __PYX_INC_MEMVIEW(&__pyx_t_14, 0);
          {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_out.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_out.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 58; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
    }
        __pyx_t_14.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_14.shape[0] = __pyx_v_out.shape[1];
__pyx_t_14.strides[0] = __pyx_v_out.strides[1];
    __pyx_t_14.suboffsets[0] = -1;

__pyx_t_15 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":57
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_vector_2d[floating](field,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], out[i])
 *     return out, inside
 */
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_15 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_field, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), __pyx_t_14);
          __PYX_XDEC_MEMVIEW(&__pyx_t_14, 0);
        }
      }

      /* "dipy/align/vector_fields.pyx":55
 *         floating[:, :] out = np.zeros(shape=(n, 2), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_vector_2d[floating](field,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L4_error: {
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L1_error;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":59
 *             inside[i] = _interpolate_vector_2d[floating](field,
 *                 locations[i, 0], locations[i, 1], out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 59; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 59; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 59; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":25
 * 
 * 
 * def interpolate_vector_2d(floating[:, :, :] field, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Bilinear interpolation of a 2D vector field
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_vector_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_field, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_85interpolate_vector_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_85interpolate_vector_2d = {"__pyx_fuse_1interpolate_vector_2d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_85interpolate_vector_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_2interpolate_vector_2d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_85interpolate_vector_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_vector_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_field,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_field)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_vector_2d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_vector_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_field = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[0]); if (unlikely(!__pyx_v_field.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_vector_2d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_vector_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_84interpolate_vector_2d(__pyx_self, __pyx_v_field, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_84interpolate_vector_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1interpolate_vector_2d", 0);

  /* "dipy/align/vector_fields.pyx":50
 *         then inside[i]=1, else inside[i]=0
 *     """
 *     ftype = np.asarray(field).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_field, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":52
 *     ftype = np.asarray(field).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         floating[:, :] out = np.zeros(shape=(n, 2), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":53
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         floating[:, :] out = np.zeros(shape=(n, 2), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_int_2);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 53; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":54
 *         cnp.npy_intp n = locations.shape[0]
 *         floating[:, :] out = np.zeros(shape=(n, 2), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 54; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":55
 *         floating[:, :] out = np.zeros(shape=(n, 2), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_vector_2d[floating](field,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":56
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_vector_2d[floating](field,
 *                 locations[i, 0], locations[i, 1], out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":58
 *         for i in range(n):
 *             inside[i] = _interpolate_vector_2d[floating](field,
 *                 locations[i, 0], locations[i, 1], out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14.data = __pyx_v_out.data;
          __pyx_t_14.memview = __pyx_v_out.memview;
          __PYX_INC_MEMVIEW(&__pyx_t_14, 0);
          {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_out.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_out.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 58; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
    }
        __pyx_t_14.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_14.shape[0] = __pyx_v_out.shape[1];
__pyx_t_14.strides[0] = __pyx_v_out.strides[1];
    __pyx_t_14.suboffsets[0] = -1;

__pyx_t_15 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":57
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_vector_2d[floating](field,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], out[i])
 *     return out, inside
 */
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_15 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_field, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), __pyx_t_14);
          __PYX_XDEC_MEMVIEW(&__pyx_t_14, 0);
        }
      }

      /* "dipy/align/vector_fields.pyx":55
 *         floating[:, :] out = np.zeros(shape=(n, 2), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_vector_2d[floating](field,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L4_error: {
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L1_error;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":59
 *             inside[i] = _interpolate_vector_2d[floating](field,
 *                 locations[i, 0], locations[i, 1], out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 59; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 59; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 59; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":25
 * 
 * 
 * def interpolate_vector_2d(floating[:, :, :] field, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Bilinear interpolation of a 2D vector field
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_vector_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_field, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":62
 * 
 * 
 * cdef inline int _interpolate_vector_2d(floating[:, :, :] field, double dii,             # <<<<<<<<<<<<<<
 *                                        double djj, floating[:] out) nogil:
 *     r"""Bilinear interpolation of a 2D vector field
 */

static CYTHON_INLINE int __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__Pyx_memviewslice __pyx_v_field, double __pyx_v_dii, double __pyx_v_djj, __Pyx_memviewslice __pyx_v_out) {
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  int __pyx_v_inside;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  int __pyx_t_39;

  /* "dipy/align/vector_fields.pyx":88
 *     """
 *     cdef:
 *         cnp.npy_intp nr = field.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = field.shape[1]
 *         cnp.npy_intp ii, jj
 */
  __pyx_v_nr = (__pyx_v_field.shape[0]);

  /* "dipy/align/vector_fields.pyx":89
 *     cdef:
 *         cnp.npy_intp nr = field.shape[0]
 *         cnp.npy_intp nc = field.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 */
  __pyx_v_nc = (__pyx_v_field.shape[1]);

  /* "dipy/align/vector_fields.pyx":93
 *         double alpha, beta, calpha, cbeta
 *         int inside
 *     if((dii <= -1) or (djj <= -1) or (dii >= nr) or (djj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         out[1] = 0
 */
  __pyx_t_2 = ((__pyx_v_dii <= -1.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj <= -1.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_dii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":94
 *         int inside
 *     if((dii <= -1) or (djj <= -1) or (dii >= nr) or (djj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         out[1] = 0
 *         return 0
 */
    __pyx_t_3 = 0;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_3 * __pyx_v_out.strides[0]) )) = 0.0;

    /* "dipy/align/vector_fields.pyx":95
 *     if((dii <= -1) or (djj <= -1) or (dii >= nr) or (djj >= nc)):
 *         out[0] = 0
 *         out[1] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # ---top-left
 */
    __pyx_t_4 = 1;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_4 * __pyx_v_out.strides[0]) )) = 0.0;

    /* "dipy/align/vector_fields.pyx":96
 *         out[0] = 0
 *         out[1] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # ---top-left
 *     ii = <int>floor(dii)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":93
 *         double alpha, beta, calpha, cbeta
 *         int inside
 *     if((dii <= -1) or (djj <= -1) or (dii >= nr) or (djj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         out[1] = 0
 */
  }

  /* "dipy/align/vector_fields.pyx":98
 *         return 0
 *     # ---top-left
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 * 
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":99
 *     # ---top-left
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 * 
 *     calpha = dii - ii
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":101
 *     jj = <int>floor(djj)
 * 
 *     calpha = dii - ii             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":102
 * 
 *     calpha = dii - ii
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":103
 *     calpha = dii - ii
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 * 
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":104
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 * 
 *     inside = 0
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":106
 *     beta = 1 - cbeta
 * 
 *     inside = 0             # <<<<<<<<<<<<<<
 *     if (ii >= 0) and (jj >= 0):
 *         out[0] = alpha * beta * field[ii, jj, 0]
 */
  __pyx_v_inside = 0;

  /* "dipy/align/vector_fields.pyx":107
 * 
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0):             # <<<<<<<<<<<<<<
 *         out[0] = alpha * beta * field[ii, jj, 0]
 *         out[1] = alpha * beta * field[ii, jj, 1]
 */
  __pyx_t_2 = ((__pyx_v_ii >= 0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= 0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":108
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0):
 *         out[0] = alpha * beta * field[ii, jj, 0]             # <<<<<<<<<<<<<<
 *         out[1] = alpha * beta * field[ii, jj, 1]
 *         inside += 1
 */
    __pyx_t_5 = __pyx_v_ii;
    __pyx_t_6 = __pyx_v_jj;
    __pyx_t_7 = 0;
    __pyx_t_8 = 0;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_8 * __pyx_v_out.strides[0]) )) = ((__pyx_v_alpha * __pyx_v_beta) * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_5 * __pyx_v_field.strides[0]) ) + __pyx_t_6 * __pyx_v_field.strides[1]) ) + __pyx_t_7 * __pyx_v_field.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":109
 *     if (ii >= 0) and (jj >= 0):
 *         out[0] = alpha * beta * field[ii, jj, 0]
 *         out[1] = alpha * beta * field[ii, jj, 1]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     else:
 */
    __pyx_t_9 = __pyx_v_ii;
    __pyx_t_10 = __pyx_v_jj;
    __pyx_t_11 = 1;
    __pyx_t_12 = 1;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_12 * __pyx_v_out.strides[0]) )) = ((__pyx_v_alpha * __pyx_v_beta) * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_9 * __pyx_v_field.strides[0]) ) + __pyx_t_10 * __pyx_v_field.strides[1]) ) + __pyx_t_11 * __pyx_v_field.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":110
 *         out[0] = alpha * beta * field[ii, jj, 0]
 *         out[1] = alpha * beta * field[ii, jj, 1]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     else:
 *         out[0] = 0
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":107
 * 
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0):             # <<<<<<<<<<<<<<
 *         out[0] = alpha * beta * field[ii, jj, 0]
 *         out[1] = alpha * beta * field[ii, jj, 1]
 */
    goto __pyx_L8;
  }

  /* "dipy/align/vector_fields.pyx":112
 *         inside += 1
 *     else:
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         out[1] = 0
 *     # ---top-right
 */
  /*else*/ {
    __pyx_t_13 = 0;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_13 * __pyx_v_out.strides[0]) )) = 0.0;

    /* "dipy/align/vector_fields.pyx":113
 *     else:
 *         out[0] = 0
 *         out[1] = 0             # <<<<<<<<<<<<<<
 *     # ---top-right
 *     jj += 1
 */
    __pyx_t_14 = 1;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_14 * __pyx_v_out.strides[0]) )) = 0.0;
  }
  __pyx_L8:;

  /* "dipy/align/vector_fields.pyx":115
 *         out[1] = 0
 *     # ---top-right
 *     jj += 1             # <<<<<<<<<<<<<<
 *     if (jj < nc) and (ii >= 0):
 *         out[0] += alpha * cbeta * field[ii, jj, 0]
 */
  __pyx_v_jj = (__pyx_v_jj + 1);

  /* "dipy/align/vector_fields.pyx":116
 *     # ---top-right
 *     jj += 1
 *     if (jj < nc) and (ii >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += alpha * cbeta * field[ii, jj, 0]
 *         out[1] += alpha * cbeta * field[ii, jj, 1]
 */
  __pyx_t_2 = ((__pyx_v_jj < __pyx_v_nc) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= 0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L12_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":117
 *     jj += 1
 *     if (jj < nc) and (ii >= 0):
 *         out[0] += alpha * cbeta * field[ii, jj, 0]             # <<<<<<<<<<<<<<
 *         out[1] += alpha * cbeta * field[ii, jj, 1]
 *         inside += 1
 */
    __pyx_t_15 = __pyx_v_ii;
    __pyx_t_16 = __pyx_v_jj;
    __pyx_t_17 = 0;
    __pyx_t_18 = 0;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_18 * __pyx_v_out.strides[0]) )) += ((__pyx_v_alpha * __pyx_v_cbeta) * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_15 * __pyx_v_field.strides[0]) ) + __pyx_t_16 * __pyx_v_field.strides[1]) ) + __pyx_t_17 * __pyx_v_field.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":118
 *     if (jj < nc) and (ii >= 0):
 *         out[0] += alpha * cbeta * field[ii, jj, 0]
 *         out[1] += alpha * cbeta * field[ii, jj, 1]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     # ---bottom-right
 */
    __pyx_t_19 = __pyx_v_ii;
    __pyx_t_20 = __pyx_v_jj;
    __pyx_t_21 = 1;
    __pyx_t_22 = 1;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_22 * __pyx_v_out.strides[0]) )) += ((__pyx_v_alpha * __pyx_v_cbeta) * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_19 * __pyx_v_field.strides[0]) ) + __pyx_t_20 * __pyx_v_field.strides[1]) ) + __pyx_t_21 * __pyx_v_field.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":119
 *         out[0] += alpha * cbeta * field[ii, jj, 0]
 *         out[1] += alpha * cbeta * field[ii, jj, 1]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     # ---bottom-right
 *     ii += 1
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":116
 *     # ---top-right
 *     jj += 1
 *     if (jj < nc) and (ii >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += alpha * cbeta * field[ii, jj, 0]
 *         out[1] += alpha * cbeta * field[ii, jj, 1]
 */
  }

  /* "dipy/align/vector_fields.pyx":121
 *         inside += 1
 *     # ---bottom-right
 *     ii += 1             # <<<<<<<<<<<<<<
 *     if (jj < nc) and (ii < nr):
 *         out[0] += calpha * cbeta * field[ii, jj, 0]
 */
  __pyx_v_ii = (__pyx_v_ii + 1);

  /* "dipy/align/vector_fields.pyx":122
 *     # ---bottom-right
 *     ii += 1
 *     if (jj < nc) and (ii < nr):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * cbeta * field[ii, jj, 0]
 *         out[1] += calpha * cbeta * field[ii, jj, 1]
 */
  __pyx_t_2 = ((__pyx_v_jj < __pyx_v_nc) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii < __pyx_v_nr) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L15_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":123
 *     ii += 1
 *     if (jj < nc) and (ii < nr):
 *         out[0] += calpha * cbeta * field[ii, jj, 0]             # <<<<<<<<<<<<<<
 *         out[1] += calpha * cbeta * field[ii, jj, 1]
 *         inside += 1
 */
    __pyx_t_23 = __pyx_v_ii;
    __pyx_t_24 = __pyx_v_jj;
    __pyx_t_25 = 0;
    __pyx_t_26 = 0;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_26 * __pyx_v_out.strides[0]) )) += ((__pyx_v_calpha * __pyx_v_cbeta) * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_23 * __pyx_v_field.strides[0]) ) + __pyx_t_24 * __pyx_v_field.strides[1]) ) + __pyx_t_25 * __pyx_v_field.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":124
 *     if (jj < nc) and (ii < nr):
 *         out[0] += calpha * cbeta * field[ii, jj, 0]
 *         out[1] += calpha * cbeta * field[ii, jj, 1]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     # ---bottom-left
 */
    __pyx_t_27 = __pyx_v_ii;
    __pyx_t_28 = __pyx_v_jj;
    __pyx_t_29 = 1;
    __pyx_t_30 = 1;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_30 * __pyx_v_out.strides[0]) )) += ((__pyx_v_calpha * __pyx_v_cbeta) * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_27 * __pyx_v_field.strides[0]) ) + __pyx_t_28 * __pyx_v_field.strides[1]) ) + __pyx_t_29 * __pyx_v_field.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":125
 *         out[0] += calpha * cbeta * field[ii, jj, 0]
 *         out[1] += calpha * cbeta * field[ii, jj, 1]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     # ---bottom-left
 *     jj -= 1
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":122
 *     # ---bottom-right
 *     ii += 1
 *     if (jj < nc) and (ii < nr):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * cbeta * field[ii, jj, 0]
 *         out[1] += calpha * cbeta * field[ii, jj, 1]
 */
  }

  /* "dipy/align/vector_fields.pyx":127
 *         inside += 1
 *     # ---bottom-left
 *     jj -= 1             # <<<<<<<<<<<<<<
 *     if (jj >= 0) and (ii < nr):
 *         out[0] += calpha * beta * field[ii, jj, 0]
 */
  __pyx_v_jj = (__pyx_v_jj - 1);

  /* "dipy/align/vector_fields.pyx":128
 *     # ---bottom-left
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * beta * field[ii, jj, 0]
 *         out[1] += calpha * beta * field[ii, jj, 1]
 */
  __pyx_t_2 = ((__pyx_v_jj >= 0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L18_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii < __pyx_v_nr) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L18_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":129
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr):
 *         out[0] += calpha * beta * field[ii, jj, 0]             # <<<<<<<<<<<<<<
 *         out[1] += calpha * beta * field[ii, jj, 1]
 *         inside += 1
 */
    __pyx_t_31 = __pyx_v_ii;
    __pyx_t_32 = __pyx_v_jj;
    __pyx_t_33 = 0;
    __pyx_t_34 = 0;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_34 * __pyx_v_out.strides[0]) )) += ((__pyx_v_calpha * __pyx_v_beta) * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_31 * __pyx_v_field.strides[0]) ) + __pyx_t_32 * __pyx_v_field.strides[1]) ) + __pyx_t_33 * __pyx_v_field.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":130
 *     if (jj >= 0) and (ii < nr):
 *         out[0] += calpha * beta * field[ii, jj, 0]
 *         out[1] += calpha * beta * field[ii, jj, 1]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     return 1 if inside == 4 else 0
 */
    __pyx_t_35 = __pyx_v_ii;
    __pyx_t_36 = __pyx_v_jj;
    __pyx_t_37 = 1;
    __pyx_t_38 = 1;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_38 * __pyx_v_out.strides[0]) )) += ((__pyx_v_calpha * __pyx_v_beta) * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_35 * __pyx_v_field.strides[0]) ) + __pyx_t_36 * __pyx_v_field.strides[1]) ) + __pyx_t_37 * __pyx_v_field.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":131
 *         out[0] += calpha * beta * field[ii, jj, 0]
 *         out[1] += calpha * beta * field[ii, jj, 1]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     return 1 if inside == 4 else 0
 * 
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":128
 *     # ---bottom-left
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * beta * field[ii, jj, 0]
 *         out[1] += calpha * beta * field[ii, jj, 1]
 */
  }

  /* "dipy/align/vector_fields.pyx":132
 *         out[1] += calpha * beta * field[ii, jj, 1]
 *         inside += 1
 *     return 1 if inside == 4 else 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (((__pyx_v_inside == 4) != 0)) {
    __pyx_t_39 = 1;
  } else {
    __pyx_t_39 = 0;
  }
  __pyx_r = __pyx_t_39;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":62
 * 
 * 
 * cdef inline int _interpolate_vector_2d(floating[:, :, :] field, double dii,             # <<<<<<<<<<<<<<
 *                                        double djj, floating[:] out) nogil:
 *     r"""Bilinear interpolation of a 2D vector field
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__Pyx_memviewslice __pyx_v_field, double __pyx_v_dii, double __pyx_v_djj, __Pyx_memviewslice __pyx_v_out) {
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  int __pyx_v_inside;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  int __pyx_t_39;

  /* "dipy/align/vector_fields.pyx":88
 *     """
 *     cdef:
 *         cnp.npy_intp nr = field.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = field.shape[1]
 *         cnp.npy_intp ii, jj
 */
  __pyx_v_nr = (__pyx_v_field.shape[0]);

  /* "dipy/align/vector_fields.pyx":89
 *     cdef:
 *         cnp.npy_intp nr = field.shape[0]
 *         cnp.npy_intp nc = field.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 */
  __pyx_v_nc = (__pyx_v_field.shape[1]);

  /* "dipy/align/vector_fields.pyx":93
 *         double alpha, beta, calpha, cbeta
 *         int inside
 *     if((dii <= -1) or (djj <= -1) or (dii >= nr) or (djj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         out[1] = 0
 */
  __pyx_t_2 = ((__pyx_v_dii <= -1.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj <= -1.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_dii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":94
 *         int inside
 *     if((dii <= -1) or (djj <= -1) or (dii >= nr) or (djj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         out[1] = 0
 *         return 0
 */
    __pyx_t_3 = 0;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_3 * __pyx_v_out.strides[0]) )) = 0.0;

    /* "dipy/align/vector_fields.pyx":95
 *     if((dii <= -1) or (djj <= -1) or (dii >= nr) or (djj >= nc)):
 *         out[0] = 0
 *         out[1] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # ---top-left
 */
    __pyx_t_4 = 1;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_4 * __pyx_v_out.strides[0]) )) = 0.0;

    /* "dipy/align/vector_fields.pyx":96
 *         out[0] = 0
 *         out[1] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # ---top-left
 *     ii = <int>floor(dii)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":93
 *         double alpha, beta, calpha, cbeta
 *         int inside
 *     if((dii <= -1) or (djj <= -1) or (dii >= nr) or (djj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         out[1] = 0
 */
  }

  /* "dipy/align/vector_fields.pyx":98
 *         return 0
 *     # ---top-left
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 * 
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":99
 *     # ---top-left
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 * 
 *     calpha = dii - ii
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":101
 *     jj = <int>floor(djj)
 * 
 *     calpha = dii - ii             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":102
 * 
 *     calpha = dii - ii
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":103
 *     calpha = dii - ii
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 * 
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":104
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 * 
 *     inside = 0
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":106
 *     beta = 1 - cbeta
 * 
 *     inside = 0             # <<<<<<<<<<<<<<
 *     if (ii >= 0) and (jj >= 0):
 *         out[0] = alpha * beta * field[ii, jj, 0]
 */
  __pyx_v_inside = 0;

  /* "dipy/align/vector_fields.pyx":107
 * 
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0):             # <<<<<<<<<<<<<<
 *         out[0] = alpha * beta * field[ii, jj, 0]
 *         out[1] = alpha * beta * field[ii, jj, 1]
 */
  __pyx_t_2 = ((__pyx_v_ii >= 0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= 0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":108
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0):
 *         out[0] = alpha * beta * field[ii, jj, 0]             # <<<<<<<<<<<<<<
 *         out[1] = alpha * beta * field[ii, jj, 1]
 *         inside += 1
 */
    __pyx_t_5 = __pyx_v_ii;
    __pyx_t_6 = __pyx_v_jj;
    __pyx_t_7 = 0;
    __pyx_t_8 = 0;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_8 * __pyx_v_out.strides[0]) )) = ((__pyx_v_alpha * __pyx_v_beta) * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_5 * __pyx_v_field.strides[0]) ) + __pyx_t_6 * __pyx_v_field.strides[1]) ) + __pyx_t_7 * __pyx_v_field.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":109
 *     if (ii >= 0) and (jj >= 0):
 *         out[0] = alpha * beta * field[ii, jj, 0]
 *         out[1] = alpha * beta * field[ii, jj, 1]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     else:
 */
    __pyx_t_9 = __pyx_v_ii;
    __pyx_t_10 = __pyx_v_jj;
    __pyx_t_11 = 1;
    __pyx_t_12 = 1;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_12 * __pyx_v_out.strides[0]) )) = ((__pyx_v_alpha * __pyx_v_beta) * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_9 * __pyx_v_field.strides[0]) ) + __pyx_t_10 * __pyx_v_field.strides[1]) ) + __pyx_t_11 * __pyx_v_field.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":110
 *         out[0] = alpha * beta * field[ii, jj, 0]
 *         out[1] = alpha * beta * field[ii, jj, 1]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     else:
 *         out[0] = 0
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":107
 * 
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0):             # <<<<<<<<<<<<<<
 *         out[0] = alpha * beta * field[ii, jj, 0]
 *         out[1] = alpha * beta * field[ii, jj, 1]
 */
    goto __pyx_L8;
  }

  /* "dipy/align/vector_fields.pyx":112
 *         inside += 1
 *     else:
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         out[1] = 0
 *     # ---top-right
 */
  /*else*/ {
    __pyx_t_13 = 0;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_13 * __pyx_v_out.strides[0]) )) = 0.0;

    /* "dipy/align/vector_fields.pyx":113
 *     else:
 *         out[0] = 0
 *         out[1] = 0             # <<<<<<<<<<<<<<
 *     # ---top-right
 *     jj += 1
 */
    __pyx_t_14 = 1;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_14 * __pyx_v_out.strides[0]) )) = 0.0;
  }
  __pyx_L8:;

  /* "dipy/align/vector_fields.pyx":115
 *         out[1] = 0
 *     # ---top-right
 *     jj += 1             # <<<<<<<<<<<<<<
 *     if (jj < nc) and (ii >= 0):
 *         out[0] += alpha * cbeta * field[ii, jj, 0]
 */
  __pyx_v_jj = (__pyx_v_jj + 1);

  /* "dipy/align/vector_fields.pyx":116
 *     # ---top-right
 *     jj += 1
 *     if (jj < nc) and (ii >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += alpha * cbeta * field[ii, jj, 0]
 *         out[1] += alpha * cbeta * field[ii, jj, 1]
 */
  __pyx_t_2 = ((__pyx_v_jj < __pyx_v_nc) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= 0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L12_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":117
 *     jj += 1
 *     if (jj < nc) and (ii >= 0):
 *         out[0] += alpha * cbeta * field[ii, jj, 0]             # <<<<<<<<<<<<<<
 *         out[1] += alpha * cbeta * field[ii, jj, 1]
 *         inside += 1
 */
    __pyx_t_15 = __pyx_v_ii;
    __pyx_t_16 = __pyx_v_jj;
    __pyx_t_17 = 0;
    __pyx_t_18 = 0;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_18 * __pyx_v_out.strides[0]) )) += ((__pyx_v_alpha * __pyx_v_cbeta) * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_15 * __pyx_v_field.strides[0]) ) + __pyx_t_16 * __pyx_v_field.strides[1]) ) + __pyx_t_17 * __pyx_v_field.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":118
 *     if (jj < nc) and (ii >= 0):
 *         out[0] += alpha * cbeta * field[ii, jj, 0]
 *         out[1] += alpha * cbeta * field[ii, jj, 1]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     # ---bottom-right
 */
    __pyx_t_19 = __pyx_v_ii;
    __pyx_t_20 = __pyx_v_jj;
    __pyx_t_21 = 1;
    __pyx_t_22 = 1;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_22 * __pyx_v_out.strides[0]) )) += ((__pyx_v_alpha * __pyx_v_cbeta) * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_19 * __pyx_v_field.strides[0]) ) + __pyx_t_20 * __pyx_v_field.strides[1]) ) + __pyx_t_21 * __pyx_v_field.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":119
 *         out[0] += alpha * cbeta * field[ii, jj, 0]
 *         out[1] += alpha * cbeta * field[ii, jj, 1]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     # ---bottom-right
 *     ii += 1
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":116
 *     # ---top-right
 *     jj += 1
 *     if (jj < nc) and (ii >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += alpha * cbeta * field[ii, jj, 0]
 *         out[1] += alpha * cbeta * field[ii, jj, 1]
 */
  }

  /* "dipy/align/vector_fields.pyx":121
 *         inside += 1
 *     # ---bottom-right
 *     ii += 1             # <<<<<<<<<<<<<<
 *     if (jj < nc) and (ii < nr):
 *         out[0] += calpha * cbeta * field[ii, jj, 0]
 */
  __pyx_v_ii = (__pyx_v_ii + 1);

  /* "dipy/align/vector_fields.pyx":122
 *     # ---bottom-right
 *     ii += 1
 *     if (jj < nc) and (ii < nr):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * cbeta * field[ii, jj, 0]
 *         out[1] += calpha * cbeta * field[ii, jj, 1]
 */
  __pyx_t_2 = ((__pyx_v_jj < __pyx_v_nc) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii < __pyx_v_nr) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L15_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":123
 *     ii += 1
 *     if (jj < nc) and (ii < nr):
 *         out[0] += calpha * cbeta * field[ii, jj, 0]             # <<<<<<<<<<<<<<
 *         out[1] += calpha * cbeta * field[ii, jj, 1]
 *         inside += 1
 */
    __pyx_t_23 = __pyx_v_ii;
    __pyx_t_24 = __pyx_v_jj;
    __pyx_t_25 = 0;
    __pyx_t_26 = 0;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_26 * __pyx_v_out.strides[0]) )) += ((__pyx_v_calpha * __pyx_v_cbeta) * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_23 * __pyx_v_field.strides[0]) ) + __pyx_t_24 * __pyx_v_field.strides[1]) ) + __pyx_t_25 * __pyx_v_field.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":124
 *     if (jj < nc) and (ii < nr):
 *         out[0] += calpha * cbeta * field[ii, jj, 0]
 *         out[1] += calpha * cbeta * field[ii, jj, 1]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     # ---bottom-left
 */
    __pyx_t_27 = __pyx_v_ii;
    __pyx_t_28 = __pyx_v_jj;
    __pyx_t_29 = 1;
    __pyx_t_30 = 1;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_30 * __pyx_v_out.strides[0]) )) += ((__pyx_v_calpha * __pyx_v_cbeta) * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_27 * __pyx_v_field.strides[0]) ) + __pyx_t_28 * __pyx_v_field.strides[1]) ) + __pyx_t_29 * __pyx_v_field.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":125
 *         out[0] += calpha * cbeta * field[ii, jj, 0]
 *         out[1] += calpha * cbeta * field[ii, jj, 1]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     # ---bottom-left
 *     jj -= 1
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":122
 *     # ---bottom-right
 *     ii += 1
 *     if (jj < nc) and (ii < nr):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * cbeta * field[ii, jj, 0]
 *         out[1] += calpha * cbeta * field[ii, jj, 1]
 */
  }

  /* "dipy/align/vector_fields.pyx":127
 *         inside += 1
 *     # ---bottom-left
 *     jj -= 1             # <<<<<<<<<<<<<<
 *     if (jj >= 0) and (ii < nr):
 *         out[0] += calpha * beta * field[ii, jj, 0]
 */
  __pyx_v_jj = (__pyx_v_jj - 1);

  /* "dipy/align/vector_fields.pyx":128
 *     # ---bottom-left
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * beta * field[ii, jj, 0]
 *         out[1] += calpha * beta * field[ii, jj, 1]
 */
  __pyx_t_2 = ((__pyx_v_jj >= 0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L18_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii < __pyx_v_nr) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L18_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":129
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr):
 *         out[0] += calpha * beta * field[ii, jj, 0]             # <<<<<<<<<<<<<<
 *         out[1] += calpha * beta * field[ii, jj, 1]
 *         inside += 1
 */
    __pyx_t_31 = __pyx_v_ii;
    __pyx_t_32 = __pyx_v_jj;
    __pyx_t_33 = 0;
    __pyx_t_34 = 0;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_34 * __pyx_v_out.strides[0]) )) += ((__pyx_v_calpha * __pyx_v_beta) * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_31 * __pyx_v_field.strides[0]) ) + __pyx_t_32 * __pyx_v_field.strides[1]) ) + __pyx_t_33 * __pyx_v_field.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":130
 *     if (jj >= 0) and (ii < nr):
 *         out[0] += calpha * beta * field[ii, jj, 0]
 *         out[1] += calpha * beta * field[ii, jj, 1]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     return 1 if inside == 4 else 0
 */
    __pyx_t_35 = __pyx_v_ii;
    __pyx_t_36 = __pyx_v_jj;
    __pyx_t_37 = 1;
    __pyx_t_38 = 1;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_38 * __pyx_v_out.strides[0]) )) += ((__pyx_v_calpha * __pyx_v_beta) * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_35 * __pyx_v_field.strides[0]) ) + __pyx_t_36 * __pyx_v_field.strides[1]) ) + __pyx_t_37 * __pyx_v_field.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":131
 *         out[0] += calpha * beta * field[ii, jj, 0]
 *         out[1] += calpha * beta * field[ii, jj, 1]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     return 1 if inside == 4 else 0
 * 
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":128
 *     # ---bottom-left
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * beta * field[ii, jj, 0]
 *         out[1] += calpha * beta * field[ii, jj, 1]
 */
  }

  /* "dipy/align/vector_fields.pyx":132
 *         out[1] += calpha * beta * field[ii, jj, 1]
 *         inside += 1
 *     return 1 if inside == 4 else 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (((__pyx_v_inside == 4) != 0)) {
    __pyx_t_39 = 1;
  } else {
    __pyx_t_39 = 0;
  }
  __pyx_r = __pyx_t_39;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":62
 * 
 * 
 * cdef inline int _interpolate_vector_2d(floating[:, :, :] field, double dii,             # <<<<<<<<<<<<<<
 *                                        double djj, floating[:] out) nogil:
 *     r"""Bilinear interpolation of a 2D vector field
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":135
 * 
 * 
 * def interpolate_scalar_2d(floating[:, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Bilinear interpolation of a 2D scalar image
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_5interpolate_scalar_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_4interpolate_scalar_2d[] = "Bilinear interpolation of a 2D scalar image\n\n    Interpolates the 2D image at the given locations. This function is\n    a wrapper for _interpolate_scalar_2d for testing purposes, it is\n    equivalent to scipy.ndimage.interpolation.map_coordinates with\n    bilinear interpolation\n\n    Parameters\n    ----------\n    field : array, shape (S, R)\n        the 2D image to be interpolated\n    locations : array, shape (n, 2)\n        (locations[i,0], locations[i,1]), 0<=i<n must contain the row and\n        column coordinates to interpolate the image at\n\n    Returns\n    -------\n    out : array, shape (n,)\n        out[i], 0<=i<n will be the interpolated scalar at coordinates\n        locations[i,:], or 0 if locations[i,:] is outside the image\n    inside : array, (n,)\n        if locations[i:] is inside the image then inside[i]=1, else\n        inside[i]=0\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_5interpolate_scalar_2d = {"interpolate_scalar_2d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_5interpolate_scalar_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_4interpolate_scalar_2d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_5interpolate_scalar_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_4interpolate_scalar_2d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_4interpolate_scalar_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("interpolate_scalar_2d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_image, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_image); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_89interpolate_scalar_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_89interpolate_scalar_2d = {"__pyx_fuse_0interpolate_scalar_2d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_89interpolate_scalar_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_4interpolate_scalar_2d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_89interpolate_scalar_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_2d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_2d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_88interpolate_scalar_2d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_88interpolate_scalar_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0interpolate_scalar_2d", 0);

  /* "dipy/align/vector_fields.pyx":160
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":162
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":163
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":164
 *         cnp.npy_intp n = locations.shape[0]
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":165
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_2d[floating](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":166
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_2d[floating](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":168
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_2d[floating](image,
 *                 locations[i, 0], locations[i, 1], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":167
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_2d[floating](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside
 */
          __pyx_t_15 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_15 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (&(*((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_14 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":165
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_2d[floating](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":169
 *             inside[i] = _interpolate_scalar_2d[floating](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":135
 * 
 * 
 * def interpolate_scalar_2d(floating[:, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Bilinear interpolation of a 2D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_91interpolate_scalar_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_91interpolate_scalar_2d = {"__pyx_fuse_1interpolate_scalar_2d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_91interpolate_scalar_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_4interpolate_scalar_2d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_91interpolate_scalar_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_2d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_2d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_90interpolate_scalar_2d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_90interpolate_scalar_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1interpolate_scalar_2d", 0);

  /* "dipy/align/vector_fields.pyx":160
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":162
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":163
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":164
 *         cnp.npy_intp n = locations.shape[0]
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":165
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_2d[floating](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":166
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_2d[floating](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":168
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_2d[floating](image,
 *                 locations[i, 0], locations[i, 1], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":167
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_2d[floating](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside
 */
          __pyx_t_15 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_15 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (&(*((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_14 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":165
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_2d[floating](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":169
 *             inside[i] = _interpolate_scalar_2d[floating](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":135
 * 
 * 
 * def interpolate_scalar_2d(floating[:, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Bilinear interpolation of a 2D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":172
 * 
 * 
 * cdef inline int _interpolate_scalar_2d(floating[:, :] image, double dii,             # <<<<<<<<<<<<<<
 *                                        double djj, floating *out) nogil:
 *     r"""Bilinear interpolation of a 2D scalar image
 */

static CYTHON_INLINE int __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__Pyx_memviewslice __pyx_v_image, double __pyx_v_dii, double __pyx_v_djj, float *__pyx_v_out) {
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  int __pyx_v_inside;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  long __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  int __pyx_t_12;

  /* "dipy/align/vector_fields.pyx":198
 *     """
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = image.shape[1]
 *         cnp.npy_intp ii, jj
 */
  __pyx_v_nr = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":199
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]
 *         cnp.npy_intp nc = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ii, jj
 *         int inside
 */
  __pyx_v_nc = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":203
 *         int inside
 *         double alpha, beta, calpha, cbeta
 *     if((dii <= -1) or (djj <= -1) or (dii >= nr) or (djj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_dii <= -1.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj <= -1.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_dii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":204
 *         double alpha, beta, calpha, cbeta
 *     if((dii <= -1) or (djj <= -1) or (dii >= nr) or (djj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # ---top-left
 */
    (__pyx_v_out[0]) = 0.0;

    /* "dipy/align/vector_fields.pyx":205
 *     if((dii <= -1) or (djj <= -1) or (dii >= nr) or (djj >= nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # ---top-left
 *     ii = <int>floor(dii)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":203
 *         int inside
 *         double alpha, beta, calpha, cbeta
 *     if((dii <= -1) or (djj <= -1) or (dii >= nr) or (djj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":207
 *         return 0
 *     # ---top-left
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 * 
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":208
 *     # ---top-left
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 * 
 *     calpha = dii - ii
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":210
 *     jj = <int>floor(djj)
 * 
 *     calpha = dii - ii             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":211
 * 
 *     calpha = dii - ii
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":212
 *     calpha = dii - ii
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 * 
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":213
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 * 
 *     inside = 0
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":215
 *     beta = 1 - cbeta
 * 
 *     inside = 0             # <<<<<<<<<<<<<<
 *     if (ii >= 0) and (jj >= 0):
 *         out[0] = alpha * beta * image[ii, jj]
 */
  __pyx_v_inside = 0;

  /* "dipy/align/vector_fields.pyx":216
 * 
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0):             # <<<<<<<<<<<<<<
 *         out[0] = alpha * beta * image[ii, jj]
 *         inside += 1
 */
  __pyx_t_2 = ((__pyx_v_ii >= 0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= 0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":217
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0):
 *         out[0] = alpha * beta * image[ii, jj]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     else:
 */
    __pyx_t_3 = __pyx_v_ii;
    __pyx_t_4 = __pyx_v_jj;
    (__pyx_v_out[0]) = ((__pyx_v_alpha * __pyx_v_beta) * (*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_image.data + __pyx_t_3 * __pyx_v_image.strides[0]) ) + __pyx_t_4 * __pyx_v_image.strides[1]) ))));

    /* "dipy/align/vector_fields.pyx":218
 *     if (ii >= 0) and (jj >= 0):
 *         out[0] = alpha * beta * image[ii, jj]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     else:
 *         out[0] = 0
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":216
 * 
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0):             # <<<<<<<<<<<<<<
 *         out[0] = alpha * beta * image[ii, jj]
 *         inside += 1
 */
    goto __pyx_L8;
  }

  /* "dipy/align/vector_fields.pyx":220
 *         inside += 1
 *     else:
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *     # ---top-right
 *     jj += 1
 */
  /*else*/ {
    (__pyx_v_out[0]) = 0.0;
  }
  __pyx_L8:;

  /* "dipy/align/vector_fields.pyx":222
 *         out[0] = 0
 *     # ---top-right
 *     jj += 1             # <<<<<<<<<<<<<<
 *     if (jj < nc) and (ii >= 0):
 *         out[0] += alpha * cbeta * image[ii, jj]
 */
  __pyx_v_jj = (__pyx_v_jj + 1);

  /* "dipy/align/vector_fields.pyx":223
 *     # ---top-right
 *     jj += 1
 *     if (jj < nc) and (ii >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += alpha * cbeta * image[ii, jj]
 *         inside += 1
 */
  __pyx_t_2 = ((__pyx_v_jj < __pyx_v_nc) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= 0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L12_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":224
 *     jj += 1
 *     if (jj < nc) and (ii >= 0):
 *         out[0] += alpha * cbeta * image[ii, jj]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     # ---bottom-right
 */
    __pyx_t_5 = 0;
    __pyx_t_6 = __pyx_v_ii;
    __pyx_t_7 = __pyx_v_jj;
    (__pyx_v_out[__pyx_t_5]) = ((__pyx_v_out[__pyx_t_5]) + ((__pyx_v_alpha * __pyx_v_cbeta) * (*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_image.data + __pyx_t_6 * __pyx_v_image.strides[0]) ) + __pyx_t_7 * __pyx_v_image.strides[1]) )))));

    /* "dipy/align/vector_fields.pyx":225
 *     if (jj < nc) and (ii >= 0):
 *         out[0] += alpha * cbeta * image[ii, jj]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     # ---bottom-right
 *     ii += 1
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":223
 *     # ---top-right
 *     jj += 1
 *     if (jj < nc) and (ii >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += alpha * cbeta * image[ii, jj]
 *         inside += 1
 */
  }

  /* "dipy/align/vector_fields.pyx":227
 *         inside += 1
 *     # ---bottom-right
 *     ii += 1             # <<<<<<<<<<<<<<
 *     if (jj < nc) and (ii < nr):
 *         out[0] += calpha * cbeta * image[ii, jj]
 */
  __pyx_v_ii = (__pyx_v_ii + 1);

  /* "dipy/align/vector_fields.pyx":228
 *     # ---bottom-right
 *     ii += 1
 *     if (jj < nc) and (ii < nr):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * cbeta * image[ii, jj]
 *         inside += 1
 */
  __pyx_t_2 = ((__pyx_v_jj < __pyx_v_nc) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii < __pyx_v_nr) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L15_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":229
 *     ii += 1
 *     if (jj < nc) and (ii < nr):
 *         out[0] += calpha * cbeta * image[ii, jj]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     # ---bottom-left
 */
    __pyx_t_5 = 0;
    __pyx_t_8 = __pyx_v_ii;
    __pyx_t_9 = __pyx_v_jj;
    (__pyx_v_out[__pyx_t_5]) = ((__pyx_v_out[__pyx_t_5]) + ((__pyx_v_calpha * __pyx_v_cbeta) * (*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_image.data + __pyx_t_8 * __pyx_v_image.strides[0]) ) + __pyx_t_9 * __pyx_v_image.strides[1]) )))));

    /* "dipy/align/vector_fields.pyx":230
 *     if (jj < nc) and (ii < nr):
 *         out[0] += calpha * cbeta * image[ii, jj]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     # ---bottom-left
 *     jj -= 1
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":228
 *     # ---bottom-right
 *     ii += 1
 *     if (jj < nc) and (ii < nr):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * cbeta * image[ii, jj]
 *         inside += 1
 */
  }

  /* "dipy/align/vector_fields.pyx":232
 *         inside += 1
 *     # ---bottom-left
 *     jj -= 1             # <<<<<<<<<<<<<<
 *     if (jj >= 0) and (ii < nr):
 *         out[0] += calpha * beta * image[ii, jj]
 */
  __pyx_v_jj = (__pyx_v_jj - 1);

  /* "dipy/align/vector_fields.pyx":233
 *     # ---bottom-left
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * beta * image[ii, jj]
 *         inside += 1
 */
  __pyx_t_2 = ((__pyx_v_jj >= 0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L18_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii < __pyx_v_nr) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L18_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":234
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr):
 *         out[0] += calpha * beta * image[ii, jj]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     return 1 if inside == 4 else 0
 */
    __pyx_t_5 = 0;
    __pyx_t_10 = __pyx_v_ii;
    __pyx_t_11 = __pyx_v_jj;
    (__pyx_v_out[__pyx_t_5]) = ((__pyx_v_out[__pyx_t_5]) + ((__pyx_v_calpha * __pyx_v_beta) * (*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_image.data + __pyx_t_10 * __pyx_v_image.strides[0]) ) + __pyx_t_11 * __pyx_v_image.strides[1]) )))));

    /* "dipy/align/vector_fields.pyx":235
 *     if (jj >= 0) and (ii < nr):
 *         out[0] += calpha * beta * image[ii, jj]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     return 1 if inside == 4 else 0
 * 
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":233
 *     # ---bottom-left
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * beta * image[ii, jj]
 *         inside += 1
 */
  }

  /* "dipy/align/vector_fields.pyx":236
 *         out[0] += calpha * beta * image[ii, jj]
 *         inside += 1
 *     return 1 if inside == 4 else 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (((__pyx_v_inside == 4) != 0)) {
    __pyx_t_12 = 1;
  } else {
    __pyx_t_12 = 0;
  }
  __pyx_r = __pyx_t_12;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":172
 * 
 * 
 * cdef inline int _interpolate_scalar_2d(floating[:, :] image, double dii,             # <<<<<<<<<<<<<<
 *                                        double djj, floating *out) nogil:
 *     r"""Bilinear interpolation of a 2D scalar image
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__Pyx_memviewslice __pyx_v_image, double __pyx_v_dii, double __pyx_v_djj, double *__pyx_v_out) {
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  int __pyx_v_inside;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  long __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  int __pyx_t_12;

  /* "dipy/align/vector_fields.pyx":198
 *     """
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = image.shape[1]
 *         cnp.npy_intp ii, jj
 */
  __pyx_v_nr = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":199
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]
 *         cnp.npy_intp nc = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ii, jj
 *         int inside
 */
  __pyx_v_nc = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":203
 *         int inside
 *         double alpha, beta, calpha, cbeta
 *     if((dii <= -1) or (djj <= -1) or (dii >= nr) or (djj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_dii <= -1.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj <= -1.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_dii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":204
 *         double alpha, beta, calpha, cbeta
 *     if((dii <= -1) or (djj <= -1) or (dii >= nr) or (djj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # ---top-left
 */
    (__pyx_v_out[0]) = 0.0;

    /* "dipy/align/vector_fields.pyx":205
 *     if((dii <= -1) or (djj <= -1) or (dii >= nr) or (djj >= nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # ---top-left
 *     ii = <int>floor(dii)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":203
 *         int inside
 *         double alpha, beta, calpha, cbeta
 *     if((dii <= -1) or (djj <= -1) or (dii >= nr) or (djj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":207
 *         return 0
 *     # ---top-left
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 * 
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":208
 *     # ---top-left
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 * 
 *     calpha = dii - ii
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":210
 *     jj = <int>floor(djj)
 * 
 *     calpha = dii - ii             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":211
 * 
 *     calpha = dii - ii
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":212
 *     calpha = dii - ii
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 * 
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":213
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 * 
 *     inside = 0
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":215
 *     beta = 1 - cbeta
 * 
 *     inside = 0             # <<<<<<<<<<<<<<
 *     if (ii >= 0) and (jj >= 0):
 *         out[0] = alpha * beta * image[ii, jj]
 */
  __pyx_v_inside = 0;

  /* "dipy/align/vector_fields.pyx":216
 * 
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0):             # <<<<<<<<<<<<<<
 *         out[0] = alpha * beta * image[ii, jj]
 *         inside += 1
 */
  __pyx_t_2 = ((__pyx_v_ii >= 0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= 0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":217
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0):
 *         out[0] = alpha * beta * image[ii, jj]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     else:
 */
    __pyx_t_3 = __pyx_v_ii;
    __pyx_t_4 = __pyx_v_jj;
    (__pyx_v_out[0]) = ((__pyx_v_alpha * __pyx_v_beta) * (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_image.data + __pyx_t_3 * __pyx_v_image.strides[0]) ) + __pyx_t_4 * __pyx_v_image.strides[1]) ))));

    /* "dipy/align/vector_fields.pyx":218
 *     if (ii >= 0) and (jj >= 0):
 *         out[0] = alpha * beta * image[ii, jj]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     else:
 *         out[0] = 0
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":216
 * 
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0):             # <<<<<<<<<<<<<<
 *         out[0] = alpha * beta * image[ii, jj]
 *         inside += 1
 */
    goto __pyx_L8;
  }

  /* "dipy/align/vector_fields.pyx":220
 *         inside += 1
 *     else:
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *     # ---top-right
 *     jj += 1
 */
  /*else*/ {
    (__pyx_v_out[0]) = 0.0;
  }
  __pyx_L8:;

  /* "dipy/align/vector_fields.pyx":222
 *         out[0] = 0
 *     # ---top-right
 *     jj += 1             # <<<<<<<<<<<<<<
 *     if (jj < nc) and (ii >= 0):
 *         out[0] += alpha * cbeta * image[ii, jj]
 */
  __pyx_v_jj = (__pyx_v_jj + 1);

  /* "dipy/align/vector_fields.pyx":223
 *     # ---top-right
 *     jj += 1
 *     if (jj < nc) and (ii >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += alpha * cbeta * image[ii, jj]
 *         inside += 1
 */
  __pyx_t_2 = ((__pyx_v_jj < __pyx_v_nc) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= 0) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L12_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":224
 *     jj += 1
 *     if (jj < nc) and (ii >= 0):
 *         out[0] += alpha * cbeta * image[ii, jj]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     # ---bottom-right
 */
    __pyx_t_5 = 0;
    __pyx_t_6 = __pyx_v_ii;
    __pyx_t_7 = __pyx_v_jj;
    (__pyx_v_out[__pyx_t_5]) = ((__pyx_v_out[__pyx_t_5]) + ((__pyx_v_alpha * __pyx_v_cbeta) * (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_image.data + __pyx_t_6 * __pyx_v_image.strides[0]) ) + __pyx_t_7 * __pyx_v_image.strides[1]) )))));

    /* "dipy/align/vector_fields.pyx":225
 *     if (jj < nc) and (ii >= 0):
 *         out[0] += alpha * cbeta * image[ii, jj]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     # ---bottom-right
 *     ii += 1
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":223
 *     # ---top-right
 *     jj += 1
 *     if (jj < nc) and (ii >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += alpha * cbeta * image[ii, jj]
 *         inside += 1
 */
  }

  /* "dipy/align/vector_fields.pyx":227
 *         inside += 1
 *     # ---bottom-right
 *     ii += 1             # <<<<<<<<<<<<<<
 *     if (jj < nc) and (ii < nr):
 *         out[0] += calpha * cbeta * image[ii, jj]
 */
  __pyx_v_ii = (__pyx_v_ii + 1);

  /* "dipy/align/vector_fields.pyx":228
 *     # ---bottom-right
 *     ii += 1
 *     if (jj < nc) and (ii < nr):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * cbeta * image[ii, jj]
 *         inside += 1
 */
  __pyx_t_2 = ((__pyx_v_jj < __pyx_v_nc) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii < __pyx_v_nr) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L15_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":229
 *     ii += 1
 *     if (jj < nc) and (ii < nr):
 *         out[0] += calpha * cbeta * image[ii, jj]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     # ---bottom-left
 */
    __pyx_t_5 = 0;
    __pyx_t_8 = __pyx_v_ii;
    __pyx_t_9 = __pyx_v_jj;
    (__pyx_v_out[__pyx_t_5]) = ((__pyx_v_out[__pyx_t_5]) + ((__pyx_v_calpha * __pyx_v_cbeta) * (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_image.data + __pyx_t_8 * __pyx_v_image.strides[0]) ) + __pyx_t_9 * __pyx_v_image.strides[1]) )))));

    /* "dipy/align/vector_fields.pyx":230
 *     if (jj < nc) and (ii < nr):
 *         out[0] += calpha * cbeta * image[ii, jj]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     # ---bottom-left
 *     jj -= 1
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":228
 *     # ---bottom-right
 *     ii += 1
 *     if (jj < nc) and (ii < nr):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * cbeta * image[ii, jj]
 *         inside += 1
 */
  }

  /* "dipy/align/vector_fields.pyx":232
 *         inside += 1
 *     # ---bottom-left
 *     jj -= 1             # <<<<<<<<<<<<<<
 *     if (jj >= 0) and (ii < nr):
 *         out[0] += calpha * beta * image[ii, jj]
 */
  __pyx_v_jj = (__pyx_v_jj - 1);

  /* "dipy/align/vector_fields.pyx":233
 *     # ---bottom-left
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * beta * image[ii, jj]
 *         inside += 1
 */
  __pyx_t_2 = ((__pyx_v_jj >= 0) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L18_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii < __pyx_v_nr) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L18_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":234
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr):
 *         out[0] += calpha * beta * image[ii, jj]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     return 1 if inside == 4 else 0
 */
    __pyx_t_5 = 0;
    __pyx_t_10 = __pyx_v_ii;
    __pyx_t_11 = __pyx_v_jj;
    (__pyx_v_out[__pyx_t_5]) = ((__pyx_v_out[__pyx_t_5]) + ((__pyx_v_calpha * __pyx_v_beta) * (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_image.data + __pyx_t_10 * __pyx_v_image.strides[0]) ) + __pyx_t_11 * __pyx_v_image.strides[1]) )))));

    /* "dipy/align/vector_fields.pyx":235
 *     if (jj >= 0) and (ii < nr):
 *         out[0] += calpha * beta * image[ii, jj]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     return 1 if inside == 4 else 0
 * 
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":233
 *     # ---bottom-left
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * beta * image[ii, jj]
 *         inside += 1
 */
  }

  /* "dipy/align/vector_fields.pyx":236
 *         out[0] += calpha * beta * image[ii, jj]
 *         inside += 1
 *     return 1 if inside == 4 else 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (((__pyx_v_inside == 4) != 0)) {
    __pyx_t_12 = 1;
  } else {
    __pyx_t_12 = 0;
  }
  __pyx_r = __pyx_t_12;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":172
 * 
 * 
 * cdef inline int _interpolate_scalar_2d(floating[:, :] image, double dii,             # <<<<<<<<<<<<<<
 *                                        double djj, floating *out) nogil:
 *     r"""Bilinear interpolation of a 2D scalar image
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":239
 * 
 * 
 * def interpolate_scalar_nn_2d(number[:, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 2D scalar image
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_7interpolate_scalar_nn_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_6interpolate_scalar_nn_2d[] = "Nearest neighbor interpolation of a 2D scalar image\n\n    Interpolates the 2D image at the given locations. This function is\n    a wrapper for _interpolate_scalar_nn_2d for testing purposes, it is\n    equivalent to scipy.ndimage.interpolation.map_coordinates with\n    nearest neighbor interpolation\n\n    Parameters\n    ----------\n    image : array, shape (S, R)\n        the 2D image to be interpolated\n    locations : array, shape (n, 2)\n        (locations[i,0], locations[i,1]), 0<=i<n must contain the row and\n        column coordinates to interpolate the image at\n\n    Returns\n    -------\n    out : array, shape (n,)\n        out[i], 0<=i<n will be the interpolated scalar at coordinates\n        locations[i,:], or 0 if locations[i,:] is outside the image\n    inside : array, (n,)\n        if locations[i:] is inside the image then inside[i]=1, else\n        inside[i]=0\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_7interpolate_scalar_nn_2d = {"interpolate_scalar_nn_2d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_7interpolate_scalar_nn_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_6interpolate_scalar_nn_2d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_7interpolate_scalar_nn_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_6interpolate_scalar_nn_2d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_6interpolate_scalar_nn_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  int __pyx_v_short_is_signed;
  int __pyx_v_long_is_signed;
  int __pyx_v_int_is_signed;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("interpolate_scalar_nn_2d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  __pyx_v_short_is_signed = (((short)-1L) < 0);
  __pyx_v_long_is_signed = (((long)-1L) < 0);
  __pyx_v_int_is_signed = (((int)-1L) < 0);
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_image, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_image); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          __pyx_t_3 = (((sizeof(short)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_short_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_short, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(int)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L26_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L26_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_int_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L26_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_int, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(long)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L30_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L30_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_long_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L30_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_long, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L34_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L34_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L37_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L37_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          __pyx_t_3 = (((sizeof(__pyx_t_float_complex)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L40_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L40_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_float_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(__pyx_t_double_complex)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L43_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L43_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_double_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L46_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(short))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L46_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_short(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_short, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L50_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(int))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L50_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_int(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_int, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L54_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(long))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L54_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_long(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_long, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L58_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L58_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L62_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L62_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L66_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(__pyx_t_float_complex))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L66_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_float_complex(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_float_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L70_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(__pyx_t_double_complex))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L70_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_double_complex(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_double_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L77_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L77_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L78_unpacking_done;
        __pyx_L77_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L78_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L80;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L76_break;
        }
        __pyx_L80:;
      }
    }
    __pyx_L76_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_95interpolate_scalar_nn_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_95interpolate_scalar_nn_2d = {"__pyx_fuse_0interpolate_scalar_nn_2d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_95interpolate_scalar_nn_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_6interpolate_scalar_nn_2d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_95interpolate_scalar_nn_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_nn_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_2d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_nn_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_short(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_2d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_94interpolate_scalar_nn_2d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_94interpolate_scalar_nn_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0interpolate_scalar_nn_2d", 0);

  /* "dipy/align/vector_fields.pyx":264
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_short, (int (*)(char *, PyObject *)) __pyx_memview_set_short, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":266
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":267
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds_short(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":268
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":269
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":270
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":272
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":271
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside
 */
          __pyx_t_15 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_15 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (&(*((short *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_14 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":269
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":273
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get_short, (int (*)(char *, PyObject *)) __pyx_memview_set_short, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":239
 * 
 * 
 * def interpolate_scalar_nn_2d(number[:, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 2D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_97interpolate_scalar_nn_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_97interpolate_scalar_nn_2d = {"__pyx_fuse_1interpolate_scalar_nn_2d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_97interpolate_scalar_nn_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_6interpolate_scalar_nn_2d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_97interpolate_scalar_nn_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_nn_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_2d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_nn_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_int(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_2d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_96interpolate_scalar_nn_2d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_96interpolate_scalar_nn_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_7;
  npy_intp __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1interpolate_scalar_nn_2d", 0);

  /* "dipy/align/vector_fields.pyx":264
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":266
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":267
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":268
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":269
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":270
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 */
        __pyx_t_7 = __pyx_v_n;
        for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
          __pyx_v_i = __pyx_t_8;

          /* "dipy/align/vector_fields.pyx":272
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_9 = __pyx_v_i;
          __pyx_t_10 = 0;
          __pyx_t_11 = __pyx_v_i;
          __pyx_t_12 = 1;
          __pyx_t_13 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":271
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside
 */
          __pyx_t_14 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_14 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_9 * __pyx_v_locations.strides[0]) ) + __pyx_t_10 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_11 * __pyx_v_locations.strides[0]) ) + __pyx_t_12 * __pyx_v_locations.strides[1]) ))), (&(*((int *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_13 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":269
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":273
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":239
 * 
 * 
 * def interpolate_scalar_nn_2d(number[:, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 2D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_2__pyx_pw_4dipy_5align_13vector_fields_99interpolate_scalar_nn_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_2__pyx_mdef_4dipy_5align_13vector_fields_99interpolate_scalar_nn_2d = {"__pyx_fuse_2interpolate_scalar_nn_2d", (PyCFunction)__pyx_fuse_2__pyx_pw_4dipy_5align_13vector_fields_99interpolate_scalar_nn_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_6interpolate_scalar_nn_2d};
static PyObject *__pyx_fuse_2__pyx_pw_4dipy_5align_13vector_fields_99interpolate_scalar_nn_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_nn_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_2d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_nn_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_long(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_2d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_98interpolate_scalar_nn_2d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_98interpolate_scalar_nn_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_2interpolate_scalar_nn_2d", 0);

  /* "dipy/align/vector_fields.pyx":264
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_long, (int (*)(char *, PyObject *)) __pyx_memview_set_long, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":266
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":267
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds_long(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":268
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":269
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":270
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":272
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":271
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside
 */
          __pyx_t_15 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_15 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_2__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (&(*((long *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_14 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":269
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":273
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get_long, (int (*)(char *, PyObject *)) __pyx_memview_set_long, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":239
 * 
 * 
 * def interpolate_scalar_nn_2d(number[:, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 2D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_3__pyx_pw_4dipy_5align_13vector_fields_101interpolate_scalar_nn_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_3__pyx_mdef_4dipy_5align_13vector_fields_101interpolate_scalar_nn_2d = {"__pyx_fuse_3interpolate_scalar_nn_2d", (PyCFunction)__pyx_fuse_3__pyx_pw_4dipy_5align_13vector_fields_101interpolate_scalar_nn_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_6interpolate_scalar_nn_2d};
static PyObject *__pyx_fuse_3__pyx_pw_4dipy_5align_13vector_fields_101interpolate_scalar_nn_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_nn_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_2d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_nn_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_2d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_100interpolate_scalar_nn_2d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_100interpolate_scalar_nn_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_3interpolate_scalar_nn_2d", 0);

  /* "dipy/align/vector_fields.pyx":264
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":266
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":267
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":268
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":269
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":270
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":272
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":271
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside
 */
          __pyx_t_15 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_15 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_3__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (&(*((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_14 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":269
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":273
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":239
 * 
 * 
 * def interpolate_scalar_nn_2d(number[:, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 2D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_4__pyx_pw_4dipy_5align_13vector_fields_103interpolate_scalar_nn_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_4__pyx_mdef_4dipy_5align_13vector_fields_103interpolate_scalar_nn_2d = {"__pyx_fuse_4interpolate_scalar_nn_2d", (PyCFunction)__pyx_fuse_4__pyx_pw_4dipy_5align_13vector_fields_103interpolate_scalar_nn_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_6interpolate_scalar_nn_2d};
static PyObject *__pyx_fuse_4__pyx_pw_4dipy_5align_13vector_fields_103interpolate_scalar_nn_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_nn_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_2d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_nn_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_2d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_102interpolate_scalar_nn_2d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_102interpolate_scalar_nn_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_4interpolate_scalar_nn_2d", 0);

  /* "dipy/align/vector_fields.pyx":264
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":266
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":267
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":268
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":269
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":270
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":272
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":271
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside
 */
          __pyx_t_15 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_15 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_4__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (&(*((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_14 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":269
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":273
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":239
 * 
 * 
 * def interpolate_scalar_nn_2d(number[:, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 2D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_5__pyx_pw_4dipy_5align_13vector_fields_105interpolate_scalar_nn_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_5__pyx_mdef_4dipy_5align_13vector_fields_105interpolate_scalar_nn_2d = {"__pyx_fuse_5interpolate_scalar_nn_2d", (PyCFunction)__pyx_fuse_5__pyx_pw_4dipy_5align_13vector_fields_105interpolate_scalar_nn_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_6interpolate_scalar_nn_2d};
static PyObject *__pyx_fuse_5__pyx_pw_4dipy_5align_13vector_fields_105interpolate_scalar_nn_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_nn_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_2d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_nn_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_float_complex(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_2d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_104interpolate_scalar_nn_2d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_104interpolate_scalar_nn_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_5interpolate_scalar_nn_2d", 0);

  /* "dipy/align/vector_fields.pyx":264
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_float_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_float_complex, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":266
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":267
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds___pyx_t_float_complex(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":268
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":269
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":270
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":272
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":271
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside
 */
          __pyx_t_15 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_15 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_5__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (&(*((__pyx_t_float_complex *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_14 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":269
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":273
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_float_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_float_complex, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":239
 * 
 * 
 * def interpolate_scalar_nn_2d(number[:, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 2D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_6__pyx_pw_4dipy_5align_13vector_fields_107interpolate_scalar_nn_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_6__pyx_mdef_4dipy_5align_13vector_fields_107interpolate_scalar_nn_2d = {"__pyx_fuse_6interpolate_scalar_nn_2d", (PyCFunction)__pyx_fuse_6__pyx_pw_4dipy_5align_13vector_fields_107interpolate_scalar_nn_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_6interpolate_scalar_nn_2d};
static PyObject *__pyx_fuse_6__pyx_pw_4dipy_5align_13vector_fields_107interpolate_scalar_nn_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_nn_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_2d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_nn_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_double_complex(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_2d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_106interpolate_scalar_nn_2d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_106interpolate_scalar_nn_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_6interpolate_scalar_nn_2d", 0);

  /* "dipy/align/vector_fields.pyx":264
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_double_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_double_complex, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":266
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":267
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds___pyx_t_double_complex(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 267; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":268
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 268; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":269
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":270
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":272
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":271
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside
 */
          __pyx_t_15 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_15 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_6__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (&(*((__pyx_t_double_complex *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_14 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":269
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":273
 *             inside[i] = _interpolate_scalar_nn_2d[number](image,
 *                 locations[i, 0], locations[i, 1], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_double_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_double_complex, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":239
 * 
 * 
 * def interpolate_scalar_nn_2d(number[:, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 2D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":276
 * 
 * 
 * cdef inline int _interpolate_scalar_nn_2d(number[:, :] image, double dii,             # <<<<<<<<<<<<<<
 *                                           double djj, number *out) nogil:
 *     r"""Nearest-neighbor interpolation of a 2D scalar image
 */

static CYTHON_INLINE int __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__Pyx_memviewslice __pyx_v_image, double __pyx_v_dii, double __pyx_v_djj, short *__pyx_v_out) {
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;

  /* "dipy/align/vector_fields.pyx":302
 *     """
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = image.shape[1]
 *         cnp.npy_intp ii, jj
 */
  __pyx_v_nr = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":303
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]
 *         cnp.npy_intp nc = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 */
  __pyx_v_nc = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":306
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_dii < 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj < 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_dii > (__pyx_v_nr - 1)) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj > (__pyx_v_nc - 1)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":307
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # find the top left index and the interpolation coefficients
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":308
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":306
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":310
 *         return 0
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 *     # no one is affected
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":311
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":313
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_ii < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":314
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":315
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":313
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":316
 *         out[0] = 0
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":317
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":318
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     if(alpha < calpha):
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":319
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     if(alpha < calpha):
 *         ii += 1
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":320
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  __pyx_t_1 = ((__pyx_v_alpha < __pyx_v_calpha) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":321
 *     beta = 1 - cbeta
 *     if(alpha < calpha):
 *         ii += 1             # <<<<<<<<<<<<<<
 *     if(beta < cbeta):
 *         jj += 1
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":320
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  }

  /* "dipy/align/vector_fields.pyx":322
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  __pyx_t_1 = ((__pyx_v_beta < __pyx_v_cbeta) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":323
 *         ii += 1
 *     if(beta < cbeta):
 *         jj += 1             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":322
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  }

  /* "dipy/align/vector_fields.pyx":325
 *         jj += 1
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_ii < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L16_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":326
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     out[0] = image[ii, jj]
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":327
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     out[0] = image[ii, jj]
 *     return 1
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":325
 *         jj += 1
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":328
 *         out[0] = 0
 *         return 0
 *     out[0] = image[ii, jj]             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
  __pyx_t_3 = __pyx_v_ii;
  __pyx_t_4 = __pyx_v_jj;
  (__pyx_v_out[0]) = (*((short *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_image.data + __pyx_t_3 * __pyx_v_image.strides[0]) ) + __pyx_t_4 * __pyx_v_image.strides[1]) )));

  /* "dipy/align/vector_fields.pyx":329
 *         return 0
 *     out[0] = image[ii, jj]
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":276
 * 
 * 
 * cdef inline int _interpolate_scalar_nn_2d(number[:, :] image, double dii,             # <<<<<<<<<<<<<<
 *                                           double djj, number *out) nogil:
 *     r"""Nearest-neighbor interpolation of a 2D scalar image
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__Pyx_memviewslice __pyx_v_image, double __pyx_v_dii, double __pyx_v_djj, int *__pyx_v_out) {
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;

  /* "dipy/align/vector_fields.pyx":302
 *     """
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = image.shape[1]
 *         cnp.npy_intp ii, jj
 */
  __pyx_v_nr = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":303
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]
 *         cnp.npy_intp nc = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 */
  __pyx_v_nc = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":306
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_dii < 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj < 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_dii > (__pyx_v_nr - 1)) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj > (__pyx_v_nc - 1)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":307
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # find the top left index and the interpolation coefficients
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":308
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":306
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":310
 *         return 0
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 *     # no one is affected
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":311
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":313
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_ii < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":314
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":315
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":313
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":316
 *         out[0] = 0
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":317
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":318
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     if(alpha < calpha):
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":319
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     if(alpha < calpha):
 *         ii += 1
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":320
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  __pyx_t_1 = ((__pyx_v_alpha < __pyx_v_calpha) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":321
 *     beta = 1 - cbeta
 *     if(alpha < calpha):
 *         ii += 1             # <<<<<<<<<<<<<<
 *     if(beta < cbeta):
 *         jj += 1
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":320
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  }

  /* "dipy/align/vector_fields.pyx":322
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  __pyx_t_1 = ((__pyx_v_beta < __pyx_v_cbeta) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":323
 *         ii += 1
 *     if(beta < cbeta):
 *         jj += 1             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":322
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  }

  /* "dipy/align/vector_fields.pyx":325
 *         jj += 1
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_ii < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L16_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":326
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     out[0] = image[ii, jj]
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":327
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     out[0] = image[ii, jj]
 *     return 1
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":325
 *         jj += 1
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":328
 *         out[0] = 0
 *         return 0
 *     out[0] = image[ii, jj]             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
  __pyx_t_3 = __pyx_v_ii;
  __pyx_t_4 = __pyx_v_jj;
  (__pyx_v_out[0]) = (*((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_image.data + __pyx_t_3 * __pyx_v_image.strides[0]) ) + __pyx_t_4 * __pyx_v_image.strides[1]) )));

  /* "dipy/align/vector_fields.pyx":329
 *         return 0
 *     out[0] = image[ii, jj]
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":276
 * 
 * 
 * cdef inline int _interpolate_scalar_nn_2d(number[:, :] image, double dii,             # <<<<<<<<<<<<<<
 *                                           double djj, number *out) nogil:
 *     r"""Nearest-neighbor interpolation of a 2D scalar image
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_2__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__Pyx_memviewslice __pyx_v_image, double __pyx_v_dii, double __pyx_v_djj, long *__pyx_v_out) {
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;

  /* "dipy/align/vector_fields.pyx":302
 *     """
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = image.shape[1]
 *         cnp.npy_intp ii, jj
 */
  __pyx_v_nr = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":303
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]
 *         cnp.npy_intp nc = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 */
  __pyx_v_nc = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":306
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_dii < 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj < 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_dii > (__pyx_v_nr - 1)) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj > (__pyx_v_nc - 1)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":307
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # find the top left index and the interpolation coefficients
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":308
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":306
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":310
 *         return 0
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 *     # no one is affected
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":311
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":313
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_ii < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":314
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":315
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":313
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":316
 *         out[0] = 0
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":317
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":318
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     if(alpha < calpha):
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":319
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     if(alpha < calpha):
 *         ii += 1
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":320
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  __pyx_t_1 = ((__pyx_v_alpha < __pyx_v_calpha) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":321
 *     beta = 1 - cbeta
 *     if(alpha < calpha):
 *         ii += 1             # <<<<<<<<<<<<<<
 *     if(beta < cbeta):
 *         jj += 1
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":320
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  }

  /* "dipy/align/vector_fields.pyx":322
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  __pyx_t_1 = ((__pyx_v_beta < __pyx_v_cbeta) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":323
 *         ii += 1
 *     if(beta < cbeta):
 *         jj += 1             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":322
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  }

  /* "dipy/align/vector_fields.pyx":325
 *         jj += 1
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_ii < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L16_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":326
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     out[0] = image[ii, jj]
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":327
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     out[0] = image[ii, jj]
 *     return 1
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":325
 *         jj += 1
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":328
 *         out[0] = 0
 *         return 0
 *     out[0] = image[ii, jj]             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
  __pyx_t_3 = __pyx_v_ii;
  __pyx_t_4 = __pyx_v_jj;
  (__pyx_v_out[0]) = (*((long *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_image.data + __pyx_t_3 * __pyx_v_image.strides[0]) ) + __pyx_t_4 * __pyx_v_image.strides[1]) )));

  /* "dipy/align/vector_fields.pyx":329
 *         return 0
 *     out[0] = image[ii, jj]
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":276
 * 
 * 
 * cdef inline int _interpolate_scalar_nn_2d(number[:, :] image, double dii,             # <<<<<<<<<<<<<<
 *                                           double djj, number *out) nogil:
 *     r"""Nearest-neighbor interpolation of a 2D scalar image
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_3__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__Pyx_memviewslice __pyx_v_image, double __pyx_v_dii, double __pyx_v_djj, float *__pyx_v_out) {
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;

  /* "dipy/align/vector_fields.pyx":302
 *     """
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = image.shape[1]
 *         cnp.npy_intp ii, jj
 */
  __pyx_v_nr = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":303
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]
 *         cnp.npy_intp nc = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 */
  __pyx_v_nc = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":306
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_dii < 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj < 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_dii > (__pyx_v_nr - 1)) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj > (__pyx_v_nc - 1)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":307
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # find the top left index and the interpolation coefficients
 */
    (__pyx_v_out[0]) = 0.0;

    /* "dipy/align/vector_fields.pyx":308
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":306
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":310
 *         return 0
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 *     # no one is affected
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":311
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":313
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_ii < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":314
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative
 */
    (__pyx_v_out[0]) = 0.0;

    /* "dipy/align/vector_fields.pyx":315
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":313
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":316
 *         out[0] = 0
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":317
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":318
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     if(alpha < calpha):
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":319
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     if(alpha < calpha):
 *         ii += 1
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":320
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  __pyx_t_1 = ((__pyx_v_alpha < __pyx_v_calpha) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":321
 *     beta = 1 - cbeta
 *     if(alpha < calpha):
 *         ii += 1             # <<<<<<<<<<<<<<
 *     if(beta < cbeta):
 *         jj += 1
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":320
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  }

  /* "dipy/align/vector_fields.pyx":322
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  __pyx_t_1 = ((__pyx_v_beta < __pyx_v_cbeta) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":323
 *         ii += 1
 *     if(beta < cbeta):
 *         jj += 1             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":322
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  }

  /* "dipy/align/vector_fields.pyx":325
 *         jj += 1
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_ii < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L16_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":326
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     out[0] = image[ii, jj]
 */
    (__pyx_v_out[0]) = 0.0;

    /* "dipy/align/vector_fields.pyx":327
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     out[0] = image[ii, jj]
 *     return 1
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":325
 *         jj += 1
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":328
 *         out[0] = 0
 *         return 0
 *     out[0] = image[ii, jj]             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
  __pyx_t_3 = __pyx_v_ii;
  __pyx_t_4 = __pyx_v_jj;
  (__pyx_v_out[0]) = (*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_image.data + __pyx_t_3 * __pyx_v_image.strides[0]) ) + __pyx_t_4 * __pyx_v_image.strides[1]) )));

  /* "dipy/align/vector_fields.pyx":329
 *         return 0
 *     out[0] = image[ii, jj]
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":276
 * 
 * 
 * cdef inline int _interpolate_scalar_nn_2d(number[:, :] image, double dii,             # <<<<<<<<<<<<<<
 *                                           double djj, number *out) nogil:
 *     r"""Nearest-neighbor interpolation of a 2D scalar image
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_4__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__Pyx_memviewslice __pyx_v_image, double __pyx_v_dii, double __pyx_v_djj, double *__pyx_v_out) {
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;

  /* "dipy/align/vector_fields.pyx":302
 *     """
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = image.shape[1]
 *         cnp.npy_intp ii, jj
 */
  __pyx_v_nr = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":303
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]
 *         cnp.npy_intp nc = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 */
  __pyx_v_nc = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":306
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_dii < 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj < 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_dii > (__pyx_v_nr - 1)) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj > (__pyx_v_nc - 1)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":307
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # find the top left index and the interpolation coefficients
 */
    (__pyx_v_out[0]) = 0.0;

    /* "dipy/align/vector_fields.pyx":308
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":306
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":310
 *         return 0
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 *     # no one is affected
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":311
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":313
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_ii < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":314
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative
 */
    (__pyx_v_out[0]) = 0.0;

    /* "dipy/align/vector_fields.pyx":315
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":313
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":316
 *         out[0] = 0
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":317
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":318
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     if(alpha < calpha):
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":319
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     if(alpha < calpha):
 *         ii += 1
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":320
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  __pyx_t_1 = ((__pyx_v_alpha < __pyx_v_calpha) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":321
 *     beta = 1 - cbeta
 *     if(alpha < calpha):
 *         ii += 1             # <<<<<<<<<<<<<<
 *     if(beta < cbeta):
 *         jj += 1
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":320
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  }

  /* "dipy/align/vector_fields.pyx":322
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  __pyx_t_1 = ((__pyx_v_beta < __pyx_v_cbeta) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":323
 *         ii += 1
 *     if(beta < cbeta):
 *         jj += 1             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":322
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  }

  /* "dipy/align/vector_fields.pyx":325
 *         jj += 1
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_ii < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L16_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":326
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     out[0] = image[ii, jj]
 */
    (__pyx_v_out[0]) = 0.0;

    /* "dipy/align/vector_fields.pyx":327
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     out[0] = image[ii, jj]
 *     return 1
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":325
 *         jj += 1
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":328
 *         out[0] = 0
 *         return 0
 *     out[0] = image[ii, jj]             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
  __pyx_t_3 = __pyx_v_ii;
  __pyx_t_4 = __pyx_v_jj;
  (__pyx_v_out[0]) = (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_image.data + __pyx_t_3 * __pyx_v_image.strides[0]) ) + __pyx_t_4 * __pyx_v_image.strides[1]) )));

  /* "dipy/align/vector_fields.pyx":329
 *         return 0
 *     out[0] = image[ii, jj]
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":276
 * 
 * 
 * cdef inline int _interpolate_scalar_nn_2d(number[:, :] image, double dii,             # <<<<<<<<<<<<<<
 *                                           double djj, number *out) nogil:
 *     r"""Nearest-neighbor interpolation of a 2D scalar image
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_5__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__Pyx_memviewslice __pyx_v_image, double __pyx_v_dii, double __pyx_v_djj, __pyx_t_float_complex *__pyx_v_out) {
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;

  /* "dipy/align/vector_fields.pyx":302
 *     """
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = image.shape[1]
 *         cnp.npy_intp ii, jj
 */
  __pyx_v_nr = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":303
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]
 *         cnp.npy_intp nc = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 */
  __pyx_v_nc = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":306
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_dii < 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj < 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_dii > (__pyx_v_nr - 1)) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj > (__pyx_v_nc - 1)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":307
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # find the top left index and the interpolation coefficients
 */
    (__pyx_v_out[0]) = __pyx_t_float_complex_from_parts(0, 0);

    /* "dipy/align/vector_fields.pyx":308
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":306
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":310
 *         return 0
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 *     # no one is affected
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":311
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":313
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_ii < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":314
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative
 */
    (__pyx_v_out[0]) = __pyx_t_float_complex_from_parts(0, 0);

    /* "dipy/align/vector_fields.pyx":315
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":313
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":316
 *         out[0] = 0
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":317
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":318
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     if(alpha < calpha):
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":319
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     if(alpha < calpha):
 *         ii += 1
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":320
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  __pyx_t_1 = ((__pyx_v_alpha < __pyx_v_calpha) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":321
 *     beta = 1 - cbeta
 *     if(alpha < calpha):
 *         ii += 1             # <<<<<<<<<<<<<<
 *     if(beta < cbeta):
 *         jj += 1
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":320
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  }

  /* "dipy/align/vector_fields.pyx":322
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  __pyx_t_1 = ((__pyx_v_beta < __pyx_v_cbeta) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":323
 *         ii += 1
 *     if(beta < cbeta):
 *         jj += 1             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":322
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  }

  /* "dipy/align/vector_fields.pyx":325
 *         jj += 1
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_ii < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L16_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":326
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     out[0] = image[ii, jj]
 */
    (__pyx_v_out[0]) = __pyx_t_float_complex_from_parts(0, 0);

    /* "dipy/align/vector_fields.pyx":327
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     out[0] = image[ii, jj]
 *     return 1
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":325
 *         jj += 1
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":328
 *         out[0] = 0
 *         return 0
 *     out[0] = image[ii, jj]             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
  __pyx_t_3 = __pyx_v_ii;
  __pyx_t_4 = __pyx_v_jj;
  (__pyx_v_out[0]) = (*((__pyx_t_float_complex *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_image.data + __pyx_t_3 * __pyx_v_image.strides[0]) ) + __pyx_t_4 * __pyx_v_image.strides[1]) )));

  /* "dipy/align/vector_fields.pyx":329
 *         return 0
 *     out[0] = image[ii, jj]
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":276
 * 
 * 
 * cdef inline int _interpolate_scalar_nn_2d(number[:, :] image, double dii,             # <<<<<<<<<<<<<<
 *                                           double djj, number *out) nogil:
 *     r"""Nearest-neighbor interpolation of a 2D scalar image
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_6__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__Pyx_memviewslice __pyx_v_image, double __pyx_v_dii, double __pyx_v_djj, __pyx_t_double_complex *__pyx_v_out) {
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;

  /* "dipy/align/vector_fields.pyx":302
 *     """
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = image.shape[1]
 *         cnp.npy_intp ii, jj
 */
  __pyx_v_nr = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":303
 *     cdef:
 *         cnp.npy_intp nr = image.shape[0]
 *         cnp.npy_intp nc = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 */
  __pyx_v_nc = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":306
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_dii < 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj < 0.0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_dii > (__pyx_v_nr - 1)) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_djj > (__pyx_v_nc - 1)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":307
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # find the top left index and the interpolation coefficients
 */
    (__pyx_v_out[0]) = __pyx_t_double_complex_from_parts(0, 0);

    /* "dipy/align/vector_fields.pyx":308
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":306
 *         cnp.npy_intp ii, jj
 *         double alpha, beta, calpha, cbeta
 *     if((dii < 0) or (djj < 0) or (dii > nr - 1) or (djj > nc - 1)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":310
 *         return 0
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 *     # no one is affected
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":311
 *     # find the top left index and the interpolation coefficients
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":313
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_ii < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":314
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative
 */
    (__pyx_v_out[0]) = __pyx_t_double_complex_from_parts(0, 0);

    /* "dipy/align/vector_fields.pyx":315
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":313
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":316
 *         out[0] = 0
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":317
 *         return 0
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":318
 *     calpha = dii - ii  # by definition these factors are nonnegative
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     if(alpha < calpha):
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":319
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     if(alpha < calpha):
 *         ii += 1
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":320
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  __pyx_t_1 = ((__pyx_v_alpha < __pyx_v_calpha) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":321
 *     beta = 1 - cbeta
 *     if(alpha < calpha):
 *         ii += 1             # <<<<<<<<<<<<<<
 *     if(beta < cbeta):
 *         jj += 1
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":320
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  }

  /* "dipy/align/vector_fields.pyx":322
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  __pyx_t_1 = ((__pyx_v_beta < __pyx_v_cbeta) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":323
 *         ii += 1
 *     if(beta < cbeta):
 *         jj += 1             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":322
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  }

  /* "dipy/align/vector_fields.pyx":325
 *         jj += 1
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = ((__pyx_v_ii < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj < 0) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_ii >= __pyx_v_nr) != 0);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_2 = ((__pyx_v_jj >= __pyx_v_nc) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L16_bool_binop_done:;
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":326
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     out[0] = image[ii, jj]
 */
    (__pyx_v_out[0]) = __pyx_t_double_complex_from_parts(0, 0);

    /* "dipy/align/vector_fields.pyx":327
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     out[0] = image[ii, jj]
 *     return 1
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":325
 *         jj += 1
 *     # no one is affected
 *     if((ii < 0) or (jj < 0) or (ii >= nr) or (jj >= nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":328
 *         out[0] = 0
 *         return 0
 *     out[0] = image[ii, jj]             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
  __pyx_t_3 = __pyx_v_ii;
  __pyx_t_4 = __pyx_v_jj;
  (__pyx_v_out[0]) = (*((__pyx_t_double_complex *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_image.data + __pyx_t_3 * __pyx_v_image.strides[0]) ) + __pyx_t_4 * __pyx_v_image.strides[1]) )));

  /* "dipy/align/vector_fields.pyx":329
 *         return 0
 *     out[0] = image[ii, jj]
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":276
 * 
 * 
 * cdef inline int _interpolate_scalar_nn_2d(number[:, :] image, double dii,             # <<<<<<<<<<<<<<
 *                                           double djj, number *out) nogil:
 *     r"""Nearest-neighbor interpolation of a 2D scalar image
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":332
 * 
 * 
 * def interpolate_scalar_nn_3d(number[:, :, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 3D scalar image
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_9interpolate_scalar_nn_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_8interpolate_scalar_nn_3d[] = "Nearest neighbor interpolation of a 3D scalar image\n\n    Interpolates the 3D image at the given locations. This function is\n    a wrapper for _interpolate_scalar_nn_3d for testing purposes, it is\n    equivalent to scipy.ndimage.interpolation.map_coordinates with\n    nearest neighbor interpolation\n\n    Parameters\n    ----------\n    image : array, shape (S, R, C)\n        the 3D image to be interpolated\n    locations : array, shape (n, 3)\n        (locations[i,0], locations[i,1], locations[i,2), 0<=i<n must contain\n        the coordinates to interpolate the image at\n\n    Returns\n    -------\n    out : array, shape (n,)\n        out[i], 0<=i<n will be the interpolated scalar at coordinates\n        locations[i,:], or 0 if locations[i,:] is outside the image\n    inside : array, (n,)\n        if locations[i,:] is inside the image then inside[i]=1, else\n        inside[i]=0\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_9interpolate_scalar_nn_3d = {"interpolate_scalar_nn_3d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_9interpolate_scalar_nn_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_8interpolate_scalar_nn_3d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_9interpolate_scalar_nn_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_8interpolate_scalar_nn_3d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_8interpolate_scalar_nn_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  int __pyx_v_short_is_signed;
  int __pyx_v_long_is_signed;
  int __pyx_v_int_is_signed;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("interpolate_scalar_nn_3d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  __pyx_v_short_is_signed = (((short)-1L) < 0);
  __pyx_v_long_is_signed = (((long)-1L) < 0);
  __pyx_v_int_is_signed = (((int)-1L) < 0);
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_image, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_image); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          __pyx_t_3 = (((sizeof(short)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_short_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_short, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(int)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L26_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L26_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_int_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L26_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_int, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(long)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L30_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L30_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_long_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L30_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_long, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L34_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L34_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L37_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L37_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          __pyx_t_3 = (((sizeof(__pyx_t_float_complex)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L40_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L40_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_float_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(__pyx_t_double_complex)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L43_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L43_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_double_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L46_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(short))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L46_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_short(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_short, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L50_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(int))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L50_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_int, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L54_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(long))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L54_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_long(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_long, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L58_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L58_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L62_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L62_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L66_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(__pyx_t_float_complex))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L66_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_float_complex(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_float_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L70_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(__pyx_t_double_complex))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L70_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_double_complex(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_double_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L77_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L77_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L78_unpacking_done;
        __pyx_L77_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L78_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L80;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L76_break;
        }
        __pyx_L80:;
      }
    }
    __pyx_L76_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_111interpolate_scalar_nn_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_111interpolate_scalar_nn_3d = {"__pyx_fuse_0interpolate_scalar_nn_3d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_111interpolate_scalar_nn_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_8interpolate_scalar_nn_3d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_111interpolate_scalar_nn_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_nn_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_3d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_nn_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_short(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_3d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_110interpolate_scalar_nn_3d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_110interpolate_scalar_nn_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0interpolate_scalar_nn_3d", 0);

  /* "dipy/align/vector_fields.pyx":357
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 3, (PyObject *(*)(char *)) __pyx_memview_get_short, (int (*)(char *, PyObject *)) __pyx_memview_set_short, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":359
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":360
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds_short(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":361
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":362
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":363
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":365
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;
          __pyx_t_15 = 2;
          __pyx_t_16 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":364
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside
 */
          __pyx_t_17 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_17 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_14 * __pyx_v_locations.strides[0]) ) + __pyx_t_15 * __pyx_v_locations.strides[1]) ))), (&(*((short *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_16 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":362
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":366
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get_short, (int (*)(char *, PyObject *)) __pyx_memview_set_short, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":332
 * 
 * 
 * def interpolate_scalar_nn_3d(number[:, :, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 3D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_113interpolate_scalar_nn_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_113interpolate_scalar_nn_3d = {"__pyx_fuse_1interpolate_scalar_nn_3d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_113interpolate_scalar_nn_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_8interpolate_scalar_nn_3d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_113interpolate_scalar_nn_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_nn_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_3d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_nn_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_3d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_112interpolate_scalar_nn_3d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_112interpolate_scalar_nn_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_7;
  npy_intp __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1interpolate_scalar_nn_3d", 0);

  /* "dipy/align/vector_fields.pyx":357
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 3, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":359
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":360
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":361
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":362
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":363
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 */
        __pyx_t_7 = __pyx_v_n;
        for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
          __pyx_v_i = __pyx_t_8;

          /* "dipy/align/vector_fields.pyx":365
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_9 = __pyx_v_i;
          __pyx_t_10 = 0;
          __pyx_t_11 = __pyx_v_i;
          __pyx_t_12 = 1;
          __pyx_t_13 = __pyx_v_i;
          __pyx_t_14 = 2;
          __pyx_t_15 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":364
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside
 */
          __pyx_t_16 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_16 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_9 * __pyx_v_locations.strides[0]) ) + __pyx_t_10 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_11 * __pyx_v_locations.strides[0]) ) + __pyx_t_12 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_13 * __pyx_v_locations.strides[0]) ) + __pyx_t_14 * __pyx_v_locations.strides[1]) ))), (&(*((int *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_15 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":362
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":366
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":332
 * 
 * 
 * def interpolate_scalar_nn_3d(number[:, :, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 3D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_2__pyx_pw_4dipy_5align_13vector_fields_115interpolate_scalar_nn_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_2__pyx_mdef_4dipy_5align_13vector_fields_115interpolate_scalar_nn_3d = {"__pyx_fuse_2interpolate_scalar_nn_3d", (PyCFunction)__pyx_fuse_2__pyx_pw_4dipy_5align_13vector_fields_115interpolate_scalar_nn_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_8interpolate_scalar_nn_3d};
static PyObject *__pyx_fuse_2__pyx_pw_4dipy_5align_13vector_fields_115interpolate_scalar_nn_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_nn_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_3d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_nn_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_long(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_3d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_114interpolate_scalar_nn_3d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_114interpolate_scalar_nn_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_2interpolate_scalar_nn_3d", 0);

  /* "dipy/align/vector_fields.pyx":357
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 3, (PyObject *(*)(char *)) __pyx_memview_get_long, (int (*)(char *, PyObject *)) __pyx_memview_set_long, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":359
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":360
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds_long(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":361
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":362
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":363
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":365
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;
          __pyx_t_15 = 2;
          __pyx_t_16 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":364
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside
 */
          __pyx_t_17 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_17 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_2__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_14 * __pyx_v_locations.strides[0]) ) + __pyx_t_15 * __pyx_v_locations.strides[1]) ))), (&(*((long *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_16 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":362
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":366
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get_long, (int (*)(char *, PyObject *)) __pyx_memview_set_long, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":332
 * 
 * 
 * def interpolate_scalar_nn_3d(number[:, :, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 3D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_3__pyx_pw_4dipy_5align_13vector_fields_117interpolate_scalar_nn_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_3__pyx_mdef_4dipy_5align_13vector_fields_117interpolate_scalar_nn_3d = {"__pyx_fuse_3interpolate_scalar_nn_3d", (PyCFunction)__pyx_fuse_3__pyx_pw_4dipy_5align_13vector_fields_117interpolate_scalar_nn_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_8interpolate_scalar_nn_3d};
static PyObject *__pyx_fuse_3__pyx_pw_4dipy_5align_13vector_fields_117interpolate_scalar_nn_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_nn_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_3d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_nn_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_3d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_116interpolate_scalar_nn_3d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_116interpolate_scalar_nn_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_3interpolate_scalar_nn_3d", 0);

  /* "dipy/align/vector_fields.pyx":357
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":359
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":360
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":361
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":362
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":363
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":365
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;
          __pyx_t_15 = 2;
          __pyx_t_16 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":364
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside
 */
          __pyx_t_17 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_17 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_3__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_14 * __pyx_v_locations.strides[0]) ) + __pyx_t_15 * __pyx_v_locations.strides[1]) ))), (&(*((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_16 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":362
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":366
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":332
 * 
 * 
 * def interpolate_scalar_nn_3d(number[:, :, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 3D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_4__pyx_pw_4dipy_5align_13vector_fields_119interpolate_scalar_nn_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_4__pyx_mdef_4dipy_5align_13vector_fields_119interpolate_scalar_nn_3d = {"__pyx_fuse_4interpolate_scalar_nn_3d", (PyCFunction)__pyx_fuse_4__pyx_pw_4dipy_5align_13vector_fields_119interpolate_scalar_nn_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_8interpolate_scalar_nn_3d};
static PyObject *__pyx_fuse_4__pyx_pw_4dipy_5align_13vector_fields_119interpolate_scalar_nn_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_nn_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_3d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_nn_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_3d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_118interpolate_scalar_nn_3d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_118interpolate_scalar_nn_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_4interpolate_scalar_nn_3d", 0);

  /* "dipy/align/vector_fields.pyx":357
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":359
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":360
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":361
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":362
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":363
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":365
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;
          __pyx_t_15 = 2;
          __pyx_t_16 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":364
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside
 */
          __pyx_t_17 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_17 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_4__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_14 * __pyx_v_locations.strides[0]) ) + __pyx_t_15 * __pyx_v_locations.strides[1]) ))), (&(*((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_16 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":362
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":366
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":332
 * 
 * 
 * def interpolate_scalar_nn_3d(number[:, :, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 3D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_5__pyx_pw_4dipy_5align_13vector_fields_121interpolate_scalar_nn_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_5__pyx_mdef_4dipy_5align_13vector_fields_121interpolate_scalar_nn_3d = {"__pyx_fuse_5interpolate_scalar_nn_3d", (PyCFunction)__pyx_fuse_5__pyx_pw_4dipy_5align_13vector_fields_121interpolate_scalar_nn_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_8interpolate_scalar_nn_3d};
static PyObject *__pyx_fuse_5__pyx_pw_4dipy_5align_13vector_fields_121interpolate_scalar_nn_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_nn_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_3d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_nn_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_float_complex(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_3d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_120interpolate_scalar_nn_3d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_120interpolate_scalar_nn_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_5interpolate_scalar_nn_3d", 0);

  /* "dipy/align/vector_fields.pyx":357
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 3, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_float_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_float_complex, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":359
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":360
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds___pyx_t_float_complex(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":361
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":362
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":363
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":365
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;
          __pyx_t_15 = 2;
          __pyx_t_16 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":364
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside
 */
          __pyx_t_17 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_17 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_5__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_14 * __pyx_v_locations.strides[0]) ) + __pyx_t_15 * __pyx_v_locations.strides[1]) ))), (&(*((__pyx_t_float_complex *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_16 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":362
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":366
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_float_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_float_complex, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":332
 * 
 * 
 * def interpolate_scalar_nn_3d(number[:, :, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 3D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_6__pyx_pw_4dipy_5align_13vector_fields_123interpolate_scalar_nn_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_6__pyx_mdef_4dipy_5align_13vector_fields_123interpolate_scalar_nn_3d = {"__pyx_fuse_6interpolate_scalar_nn_3d", (PyCFunction)__pyx_fuse_6__pyx_pw_4dipy_5align_13vector_fields_123interpolate_scalar_nn_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_8interpolate_scalar_nn_3d};
static PyObject *__pyx_fuse_6__pyx_pw_4dipy_5align_13vector_fields_123interpolate_scalar_nn_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_nn_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_3d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_nn_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_double_complex(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_nn_3d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_122interpolate_scalar_nn_3d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_122interpolate_scalar_nn_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_6interpolate_scalar_nn_3d", 0);

  /* "dipy/align/vector_fields.pyx":357
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 3, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_double_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_double_complex, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":359
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":360
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds___pyx_t_double_complex(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 360; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":361
 *         cnp.npy_intp n = locations.shape[0]
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 361; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":362
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":363
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":365
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;
          __pyx_t_15 = 2;
          __pyx_t_16 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":364
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside
 */
          __pyx_t_17 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_17 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_6__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_14 * __pyx_v_locations.strides[0]) ) + __pyx_t_15 * __pyx_v_locations.strides[1]) ))), (&(*((__pyx_t_double_complex *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_16 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":362
 *         number[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":366
 *             inside[i] = _interpolate_scalar_nn_3d[number](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_double_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_double_complex, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":332
 * 
 * 
 * def interpolate_scalar_nn_3d(number[:, :, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 3D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_nn_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":369
 * 
 * 
 * cdef inline int _interpolate_scalar_nn_3d(number[:, :, :] volume, double dkk,             # <<<<<<<<<<<<<<
 *                                          double dii, double djj,
 *                                          number *out) nogil:
 */

static CYTHON_INLINE int __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__Pyx_memviewslice __pyx_v_volume, double __pyx_v_dkk, double __pyx_v_dii, double __pyx_v_djj, short *__pyx_v_out) {
  npy_intp __pyx_v_ns;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_kk;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  double __pyx_v_gamma;
  double __pyx_v_cgamma;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;

  /* "dipy/align/vector_fields.pyx":398
 *     """
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]
 */
  __pyx_v_ns = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":399
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = volume.shape[2]
 *         cnp.npy_intp kk, ii, jj
 */
  __pyx_v_nr = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":400
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 */
  __pyx_v_nc = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":403
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = (0.0 <= __pyx_v_dkk);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_dkk <= (__pyx_v_ns - 1));
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (0.0 <= __pyx_v_dii);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_dii <= (__pyx_v_nr - 1));
  }
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (0.0 <= __pyx_v_djj);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_djj <= (__pyx_v_nc - 1));
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  __pyx_t_3 = ((!__pyx_t_1) != 0);
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":404
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # find the top left index and the interpolation coefficients
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":405
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":403
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":407
 *         return 0
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)             # <<<<<<<<<<<<<<
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)
 */
  __pyx_v_kk = ((int)floor(__pyx_v_dkk));

  /* "dipy/align/vector_fields.pyx":408
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 *     # no one is affected
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":409
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":411
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_1 = (0 <= __pyx_v_kk);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_kk < __pyx_v_ns);
  }
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_3 = __pyx_t_2;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_2 = (0 <= __pyx_v_ii);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_ii < __pyx_v_nr);
  }
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_1 = (0 <= __pyx_v_jj);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_jj < __pyx_v_nc);
  }
  __pyx_t_2 = (__pyx_t_1 != 0);
  __pyx_t_3 = __pyx_t_2;
  __pyx_L8_bool_binop_done:;
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":412
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     cgamma = dkk - kk
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":413
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":411
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":414
 *         out[0] = 0
 *         return 0
 *     cgamma = dkk - kk             # <<<<<<<<<<<<<<
 *     calpha = dii - ii
 *     cbeta = djj - jj
 */
  __pyx_v_cgamma = (__pyx_v_dkk - __pyx_v_kk);

  /* "dipy/align/vector_fields.pyx":415
 *         return 0
 *     cgamma = dkk - kk
 *     calpha = dii - ii             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":416
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":417
 *     calpha = dii - ii
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":418
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":419
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma             # <<<<<<<<<<<<<<
 *     if(gamma < cgamma):
 *         kk += 1
 */
  __pyx_v_gamma = (1.0 - __pyx_v_cgamma);

  /* "dipy/align/vector_fields.pyx":420
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):             # <<<<<<<<<<<<<<
 *         kk += 1
 *     if(alpha < calpha):
 */
  __pyx_t_2 = ((__pyx_v_gamma < __pyx_v_cgamma) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":421
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):
 *         kk += 1             # <<<<<<<<<<<<<<
 *     if(alpha < calpha):
 *         ii += 1
 */
    __pyx_v_kk = (__pyx_v_kk + 1);

    /* "dipy/align/vector_fields.pyx":420
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):             # <<<<<<<<<<<<<<
 *         kk += 1
 *     if(alpha < calpha):
 */
  }

  /* "dipy/align/vector_fields.pyx":422
 *     if(gamma < cgamma):
 *         kk += 1
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  __pyx_t_2 = ((__pyx_v_alpha < __pyx_v_calpha) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":423
 *         kk += 1
 *     if(alpha < calpha):
 *         ii += 1             # <<<<<<<<<<<<<<
 *     if(beta < cbeta):
 *         jj += 1
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":422
 *     if(gamma < cgamma):
 *         kk += 1
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  }

  /* "dipy/align/vector_fields.pyx":424
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  __pyx_t_2 = ((__pyx_v_beta < __pyx_v_cbeta) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":425
 *         ii += 1
 *     if(beta < cbeta):
 *         jj += 1             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":424
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  }

  /* "dipy/align/vector_fields.pyx":427
 *         jj += 1
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_3 = (0 <= __pyx_v_kk);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_kk < __pyx_v_ns);
  }
  __pyx_t_1 = (__pyx_t_3 != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_2 = __pyx_t_1;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_1 = (0 <= __pyx_v_ii);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_ii < __pyx_v_nr);
  }
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_3 = (0 <= __pyx_v_jj);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_jj < __pyx_v_nc);
  }
  __pyx_t_1 = (__pyx_t_3 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L15_bool_binop_done:;
  __pyx_t_1 = ((!__pyx_t_2) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":428
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     out[0] = volume[kk, ii, jj]
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":429
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     out[0] = volume[kk, ii, jj]
 *     return 1
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":427
 *         jj += 1
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":430
 *         out[0] = 0
 *         return 0
 *     out[0] = volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
  __pyx_t_4 = __pyx_v_kk;
  __pyx_t_5 = __pyx_v_ii;
  __pyx_t_6 = __pyx_v_jj;
  (__pyx_v_out[0]) = (*((short *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_4 * __pyx_v_volume.strides[0]) ) + __pyx_t_5 * __pyx_v_volume.strides[1]) ) + __pyx_t_6 * __pyx_v_volume.strides[2]) )));

  /* "dipy/align/vector_fields.pyx":431
 *         return 0
 *     out[0] = volume[kk, ii, jj]
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":369
 * 
 * 
 * cdef inline int _interpolate_scalar_nn_3d(number[:, :, :] volume, double dkk,             # <<<<<<<<<<<<<<
 *                                          double dii, double djj,
 *                                          number *out) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__Pyx_memviewslice __pyx_v_volume, double __pyx_v_dkk, double __pyx_v_dii, double __pyx_v_djj, int *__pyx_v_out) {
  npy_intp __pyx_v_ns;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_kk;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  double __pyx_v_gamma;
  double __pyx_v_cgamma;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;

  /* "dipy/align/vector_fields.pyx":398
 *     """
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]
 */
  __pyx_v_ns = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":399
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = volume.shape[2]
 *         cnp.npy_intp kk, ii, jj
 */
  __pyx_v_nr = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":400
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 */
  __pyx_v_nc = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":403
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = (0.0 <= __pyx_v_dkk);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_dkk <= (__pyx_v_ns - 1));
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (0.0 <= __pyx_v_dii);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_dii <= (__pyx_v_nr - 1));
  }
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (0.0 <= __pyx_v_djj);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_djj <= (__pyx_v_nc - 1));
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  __pyx_t_3 = ((!__pyx_t_1) != 0);
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":404
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # find the top left index and the interpolation coefficients
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":405
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":403
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":407
 *         return 0
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)             # <<<<<<<<<<<<<<
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)
 */
  __pyx_v_kk = ((int)floor(__pyx_v_dkk));

  /* "dipy/align/vector_fields.pyx":408
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 *     # no one is affected
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":409
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":411
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_1 = (0 <= __pyx_v_kk);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_kk < __pyx_v_ns);
  }
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_3 = __pyx_t_2;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_2 = (0 <= __pyx_v_ii);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_ii < __pyx_v_nr);
  }
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_1 = (0 <= __pyx_v_jj);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_jj < __pyx_v_nc);
  }
  __pyx_t_2 = (__pyx_t_1 != 0);
  __pyx_t_3 = __pyx_t_2;
  __pyx_L8_bool_binop_done:;
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":412
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     cgamma = dkk - kk
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":413
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":411
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":414
 *         out[0] = 0
 *         return 0
 *     cgamma = dkk - kk             # <<<<<<<<<<<<<<
 *     calpha = dii - ii
 *     cbeta = djj - jj
 */
  __pyx_v_cgamma = (__pyx_v_dkk - __pyx_v_kk);

  /* "dipy/align/vector_fields.pyx":415
 *         return 0
 *     cgamma = dkk - kk
 *     calpha = dii - ii             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":416
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":417
 *     calpha = dii - ii
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":418
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":419
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma             # <<<<<<<<<<<<<<
 *     if(gamma < cgamma):
 *         kk += 1
 */
  __pyx_v_gamma = (1.0 - __pyx_v_cgamma);

  /* "dipy/align/vector_fields.pyx":420
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):             # <<<<<<<<<<<<<<
 *         kk += 1
 *     if(alpha < calpha):
 */
  __pyx_t_2 = ((__pyx_v_gamma < __pyx_v_cgamma) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":421
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):
 *         kk += 1             # <<<<<<<<<<<<<<
 *     if(alpha < calpha):
 *         ii += 1
 */
    __pyx_v_kk = (__pyx_v_kk + 1);

    /* "dipy/align/vector_fields.pyx":420
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):             # <<<<<<<<<<<<<<
 *         kk += 1
 *     if(alpha < calpha):
 */
  }

  /* "dipy/align/vector_fields.pyx":422
 *     if(gamma < cgamma):
 *         kk += 1
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  __pyx_t_2 = ((__pyx_v_alpha < __pyx_v_calpha) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":423
 *         kk += 1
 *     if(alpha < calpha):
 *         ii += 1             # <<<<<<<<<<<<<<
 *     if(beta < cbeta):
 *         jj += 1
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":422
 *     if(gamma < cgamma):
 *         kk += 1
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  }

  /* "dipy/align/vector_fields.pyx":424
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  __pyx_t_2 = ((__pyx_v_beta < __pyx_v_cbeta) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":425
 *         ii += 1
 *     if(beta < cbeta):
 *         jj += 1             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":424
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  }

  /* "dipy/align/vector_fields.pyx":427
 *         jj += 1
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_3 = (0 <= __pyx_v_kk);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_kk < __pyx_v_ns);
  }
  __pyx_t_1 = (__pyx_t_3 != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_2 = __pyx_t_1;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_1 = (0 <= __pyx_v_ii);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_ii < __pyx_v_nr);
  }
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_3 = (0 <= __pyx_v_jj);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_jj < __pyx_v_nc);
  }
  __pyx_t_1 = (__pyx_t_3 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L15_bool_binop_done:;
  __pyx_t_1 = ((!__pyx_t_2) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":428
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     out[0] = volume[kk, ii, jj]
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":429
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     out[0] = volume[kk, ii, jj]
 *     return 1
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":427
 *         jj += 1
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":430
 *         out[0] = 0
 *         return 0
 *     out[0] = volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
  __pyx_t_4 = __pyx_v_kk;
  __pyx_t_5 = __pyx_v_ii;
  __pyx_t_6 = __pyx_v_jj;
  (__pyx_v_out[0]) = (*((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_4 * __pyx_v_volume.strides[0]) ) + __pyx_t_5 * __pyx_v_volume.strides[1]) ) + __pyx_t_6 * __pyx_v_volume.strides[2]) )));

  /* "dipy/align/vector_fields.pyx":431
 *         return 0
 *     out[0] = volume[kk, ii, jj]
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":369
 * 
 * 
 * cdef inline int _interpolate_scalar_nn_3d(number[:, :, :] volume, double dkk,             # <<<<<<<<<<<<<<
 *                                          double dii, double djj,
 *                                          number *out) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_2__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__Pyx_memviewslice __pyx_v_volume, double __pyx_v_dkk, double __pyx_v_dii, double __pyx_v_djj, long *__pyx_v_out) {
  npy_intp __pyx_v_ns;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_kk;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  double __pyx_v_gamma;
  double __pyx_v_cgamma;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;

  /* "dipy/align/vector_fields.pyx":398
 *     """
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]
 */
  __pyx_v_ns = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":399
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = volume.shape[2]
 *         cnp.npy_intp kk, ii, jj
 */
  __pyx_v_nr = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":400
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 */
  __pyx_v_nc = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":403
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = (0.0 <= __pyx_v_dkk);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_dkk <= (__pyx_v_ns - 1));
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (0.0 <= __pyx_v_dii);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_dii <= (__pyx_v_nr - 1));
  }
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (0.0 <= __pyx_v_djj);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_djj <= (__pyx_v_nc - 1));
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  __pyx_t_3 = ((!__pyx_t_1) != 0);
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":404
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # find the top left index and the interpolation coefficients
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":405
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":403
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":407
 *         return 0
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)             # <<<<<<<<<<<<<<
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)
 */
  __pyx_v_kk = ((int)floor(__pyx_v_dkk));

  /* "dipy/align/vector_fields.pyx":408
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 *     # no one is affected
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":409
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":411
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_1 = (0 <= __pyx_v_kk);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_kk < __pyx_v_ns);
  }
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_3 = __pyx_t_2;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_2 = (0 <= __pyx_v_ii);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_ii < __pyx_v_nr);
  }
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_1 = (0 <= __pyx_v_jj);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_jj < __pyx_v_nc);
  }
  __pyx_t_2 = (__pyx_t_1 != 0);
  __pyx_t_3 = __pyx_t_2;
  __pyx_L8_bool_binop_done:;
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":412
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     cgamma = dkk - kk
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":413
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":411
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":414
 *         out[0] = 0
 *         return 0
 *     cgamma = dkk - kk             # <<<<<<<<<<<<<<
 *     calpha = dii - ii
 *     cbeta = djj - jj
 */
  __pyx_v_cgamma = (__pyx_v_dkk - __pyx_v_kk);

  /* "dipy/align/vector_fields.pyx":415
 *         return 0
 *     cgamma = dkk - kk
 *     calpha = dii - ii             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":416
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":417
 *     calpha = dii - ii
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":418
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":419
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma             # <<<<<<<<<<<<<<
 *     if(gamma < cgamma):
 *         kk += 1
 */
  __pyx_v_gamma = (1.0 - __pyx_v_cgamma);

  /* "dipy/align/vector_fields.pyx":420
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):             # <<<<<<<<<<<<<<
 *         kk += 1
 *     if(alpha < calpha):
 */
  __pyx_t_2 = ((__pyx_v_gamma < __pyx_v_cgamma) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":421
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):
 *         kk += 1             # <<<<<<<<<<<<<<
 *     if(alpha < calpha):
 *         ii += 1
 */
    __pyx_v_kk = (__pyx_v_kk + 1);

    /* "dipy/align/vector_fields.pyx":420
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):             # <<<<<<<<<<<<<<
 *         kk += 1
 *     if(alpha < calpha):
 */
  }

  /* "dipy/align/vector_fields.pyx":422
 *     if(gamma < cgamma):
 *         kk += 1
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  __pyx_t_2 = ((__pyx_v_alpha < __pyx_v_calpha) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":423
 *         kk += 1
 *     if(alpha < calpha):
 *         ii += 1             # <<<<<<<<<<<<<<
 *     if(beta < cbeta):
 *         jj += 1
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":422
 *     if(gamma < cgamma):
 *         kk += 1
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  }

  /* "dipy/align/vector_fields.pyx":424
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  __pyx_t_2 = ((__pyx_v_beta < __pyx_v_cbeta) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":425
 *         ii += 1
 *     if(beta < cbeta):
 *         jj += 1             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":424
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  }

  /* "dipy/align/vector_fields.pyx":427
 *         jj += 1
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_3 = (0 <= __pyx_v_kk);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_kk < __pyx_v_ns);
  }
  __pyx_t_1 = (__pyx_t_3 != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_2 = __pyx_t_1;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_1 = (0 <= __pyx_v_ii);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_ii < __pyx_v_nr);
  }
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_3 = (0 <= __pyx_v_jj);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_jj < __pyx_v_nc);
  }
  __pyx_t_1 = (__pyx_t_3 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L15_bool_binop_done:;
  __pyx_t_1 = ((!__pyx_t_2) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":428
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     out[0] = volume[kk, ii, jj]
 */
    (__pyx_v_out[0]) = 0;

    /* "dipy/align/vector_fields.pyx":429
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     out[0] = volume[kk, ii, jj]
 *     return 1
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":427
 *         jj += 1
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":430
 *         out[0] = 0
 *         return 0
 *     out[0] = volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
  __pyx_t_4 = __pyx_v_kk;
  __pyx_t_5 = __pyx_v_ii;
  __pyx_t_6 = __pyx_v_jj;
  (__pyx_v_out[0]) = (*((long *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_4 * __pyx_v_volume.strides[0]) ) + __pyx_t_5 * __pyx_v_volume.strides[1]) ) + __pyx_t_6 * __pyx_v_volume.strides[2]) )));

  /* "dipy/align/vector_fields.pyx":431
 *         return 0
 *     out[0] = volume[kk, ii, jj]
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":369
 * 
 * 
 * cdef inline int _interpolate_scalar_nn_3d(number[:, :, :] volume, double dkk,             # <<<<<<<<<<<<<<
 *                                          double dii, double djj,
 *                                          number *out) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_3__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__Pyx_memviewslice __pyx_v_volume, double __pyx_v_dkk, double __pyx_v_dii, double __pyx_v_djj, float *__pyx_v_out) {
  npy_intp __pyx_v_ns;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_kk;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  double __pyx_v_gamma;
  double __pyx_v_cgamma;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;

  /* "dipy/align/vector_fields.pyx":398
 *     """
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]
 */
  __pyx_v_ns = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":399
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = volume.shape[2]
 *         cnp.npy_intp kk, ii, jj
 */
  __pyx_v_nr = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":400
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 */
  __pyx_v_nc = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":403
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = (0.0 <= __pyx_v_dkk);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_dkk <= (__pyx_v_ns - 1));
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (0.0 <= __pyx_v_dii);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_dii <= (__pyx_v_nr - 1));
  }
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (0.0 <= __pyx_v_djj);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_djj <= (__pyx_v_nc - 1));
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  __pyx_t_3 = ((!__pyx_t_1) != 0);
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":404
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # find the top left index and the interpolation coefficients
 */
    (__pyx_v_out[0]) = 0.0;

    /* "dipy/align/vector_fields.pyx":405
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":403
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":407
 *         return 0
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)             # <<<<<<<<<<<<<<
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)
 */
  __pyx_v_kk = ((int)floor(__pyx_v_dkk));

  /* "dipy/align/vector_fields.pyx":408
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 *     # no one is affected
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":409
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":411
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_1 = (0 <= __pyx_v_kk);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_kk < __pyx_v_ns);
  }
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_3 = __pyx_t_2;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_2 = (0 <= __pyx_v_ii);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_ii < __pyx_v_nr);
  }
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_1 = (0 <= __pyx_v_jj);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_jj < __pyx_v_nc);
  }
  __pyx_t_2 = (__pyx_t_1 != 0);
  __pyx_t_3 = __pyx_t_2;
  __pyx_L8_bool_binop_done:;
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":412
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     cgamma = dkk - kk
 */
    (__pyx_v_out[0]) = 0.0;

    /* "dipy/align/vector_fields.pyx":413
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":411
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":414
 *         out[0] = 0
 *         return 0
 *     cgamma = dkk - kk             # <<<<<<<<<<<<<<
 *     calpha = dii - ii
 *     cbeta = djj - jj
 */
  __pyx_v_cgamma = (__pyx_v_dkk - __pyx_v_kk);

  /* "dipy/align/vector_fields.pyx":415
 *         return 0
 *     cgamma = dkk - kk
 *     calpha = dii - ii             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":416
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":417
 *     calpha = dii - ii
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":418
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":419
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma             # <<<<<<<<<<<<<<
 *     if(gamma < cgamma):
 *         kk += 1
 */
  __pyx_v_gamma = (1.0 - __pyx_v_cgamma);

  /* "dipy/align/vector_fields.pyx":420
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):             # <<<<<<<<<<<<<<
 *         kk += 1
 *     if(alpha < calpha):
 */
  __pyx_t_2 = ((__pyx_v_gamma < __pyx_v_cgamma) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":421
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):
 *         kk += 1             # <<<<<<<<<<<<<<
 *     if(alpha < calpha):
 *         ii += 1
 */
    __pyx_v_kk = (__pyx_v_kk + 1);

    /* "dipy/align/vector_fields.pyx":420
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):             # <<<<<<<<<<<<<<
 *         kk += 1
 *     if(alpha < calpha):
 */
  }

  /* "dipy/align/vector_fields.pyx":422
 *     if(gamma < cgamma):
 *         kk += 1
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  __pyx_t_2 = ((__pyx_v_alpha < __pyx_v_calpha) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":423
 *         kk += 1
 *     if(alpha < calpha):
 *         ii += 1             # <<<<<<<<<<<<<<
 *     if(beta < cbeta):
 *         jj += 1
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":422
 *     if(gamma < cgamma):
 *         kk += 1
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  }

  /* "dipy/align/vector_fields.pyx":424
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  __pyx_t_2 = ((__pyx_v_beta < __pyx_v_cbeta) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":425
 *         ii += 1
 *     if(beta < cbeta):
 *         jj += 1             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":424
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  }

  /* "dipy/align/vector_fields.pyx":427
 *         jj += 1
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_3 = (0 <= __pyx_v_kk);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_kk < __pyx_v_ns);
  }
  __pyx_t_1 = (__pyx_t_3 != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_2 = __pyx_t_1;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_1 = (0 <= __pyx_v_ii);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_ii < __pyx_v_nr);
  }
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_3 = (0 <= __pyx_v_jj);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_jj < __pyx_v_nc);
  }
  __pyx_t_1 = (__pyx_t_3 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L15_bool_binop_done:;
  __pyx_t_1 = ((!__pyx_t_2) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":428
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     out[0] = volume[kk, ii, jj]
 */
    (__pyx_v_out[0]) = 0.0;

    /* "dipy/align/vector_fields.pyx":429
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     out[0] = volume[kk, ii, jj]
 *     return 1
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":427
 *         jj += 1
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":430
 *         out[0] = 0
 *         return 0
 *     out[0] = volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
  __pyx_t_4 = __pyx_v_kk;
  __pyx_t_5 = __pyx_v_ii;
  __pyx_t_6 = __pyx_v_jj;
  (__pyx_v_out[0]) = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_4 * __pyx_v_volume.strides[0]) ) + __pyx_t_5 * __pyx_v_volume.strides[1]) ) + __pyx_t_6 * __pyx_v_volume.strides[2]) )));

  /* "dipy/align/vector_fields.pyx":431
 *         return 0
 *     out[0] = volume[kk, ii, jj]
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":369
 * 
 * 
 * cdef inline int _interpolate_scalar_nn_3d(number[:, :, :] volume, double dkk,             # <<<<<<<<<<<<<<
 *                                          double dii, double djj,
 *                                          number *out) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_4__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__Pyx_memviewslice __pyx_v_volume, double __pyx_v_dkk, double __pyx_v_dii, double __pyx_v_djj, double *__pyx_v_out) {
  npy_intp __pyx_v_ns;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_kk;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  double __pyx_v_gamma;
  double __pyx_v_cgamma;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;

  /* "dipy/align/vector_fields.pyx":398
 *     """
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]
 */
  __pyx_v_ns = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":399
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = volume.shape[2]
 *         cnp.npy_intp kk, ii, jj
 */
  __pyx_v_nr = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":400
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 */
  __pyx_v_nc = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":403
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = (0.0 <= __pyx_v_dkk);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_dkk <= (__pyx_v_ns - 1));
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (0.0 <= __pyx_v_dii);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_dii <= (__pyx_v_nr - 1));
  }
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (0.0 <= __pyx_v_djj);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_djj <= (__pyx_v_nc - 1));
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  __pyx_t_3 = ((!__pyx_t_1) != 0);
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":404
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # find the top left index and the interpolation coefficients
 */
    (__pyx_v_out[0]) = 0.0;

    /* "dipy/align/vector_fields.pyx":405
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":403
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":407
 *         return 0
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)             # <<<<<<<<<<<<<<
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)
 */
  __pyx_v_kk = ((int)floor(__pyx_v_dkk));

  /* "dipy/align/vector_fields.pyx":408
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 *     # no one is affected
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":409
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":411
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_1 = (0 <= __pyx_v_kk);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_kk < __pyx_v_ns);
  }
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_3 = __pyx_t_2;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_2 = (0 <= __pyx_v_ii);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_ii < __pyx_v_nr);
  }
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_1 = (0 <= __pyx_v_jj);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_jj < __pyx_v_nc);
  }
  __pyx_t_2 = (__pyx_t_1 != 0);
  __pyx_t_3 = __pyx_t_2;
  __pyx_L8_bool_binop_done:;
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":412
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     cgamma = dkk - kk
 */
    (__pyx_v_out[0]) = 0.0;

    /* "dipy/align/vector_fields.pyx":413
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":411
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":414
 *         out[0] = 0
 *         return 0
 *     cgamma = dkk - kk             # <<<<<<<<<<<<<<
 *     calpha = dii - ii
 *     cbeta = djj - jj
 */
  __pyx_v_cgamma = (__pyx_v_dkk - __pyx_v_kk);

  /* "dipy/align/vector_fields.pyx":415
 *         return 0
 *     cgamma = dkk - kk
 *     calpha = dii - ii             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":416
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":417
 *     calpha = dii - ii
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":418
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":419
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma             # <<<<<<<<<<<<<<
 *     if(gamma < cgamma):
 *         kk += 1
 */
  __pyx_v_gamma = (1.0 - __pyx_v_cgamma);

  /* "dipy/align/vector_fields.pyx":420
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):             # <<<<<<<<<<<<<<
 *         kk += 1
 *     if(alpha < calpha):
 */
  __pyx_t_2 = ((__pyx_v_gamma < __pyx_v_cgamma) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":421
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):
 *         kk += 1             # <<<<<<<<<<<<<<
 *     if(alpha < calpha):
 *         ii += 1
 */
    __pyx_v_kk = (__pyx_v_kk + 1);

    /* "dipy/align/vector_fields.pyx":420
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):             # <<<<<<<<<<<<<<
 *         kk += 1
 *     if(alpha < calpha):
 */
  }

  /* "dipy/align/vector_fields.pyx":422
 *     if(gamma < cgamma):
 *         kk += 1
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  __pyx_t_2 = ((__pyx_v_alpha < __pyx_v_calpha) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":423
 *         kk += 1
 *     if(alpha < calpha):
 *         ii += 1             # <<<<<<<<<<<<<<
 *     if(beta < cbeta):
 *         jj += 1
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":422
 *     if(gamma < cgamma):
 *         kk += 1
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  }

  /* "dipy/align/vector_fields.pyx":424
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  __pyx_t_2 = ((__pyx_v_beta < __pyx_v_cbeta) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":425
 *         ii += 1
 *     if(beta < cbeta):
 *         jj += 1             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":424
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  }

  /* "dipy/align/vector_fields.pyx":427
 *         jj += 1
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_3 = (0 <= __pyx_v_kk);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_kk < __pyx_v_ns);
  }
  __pyx_t_1 = (__pyx_t_3 != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_2 = __pyx_t_1;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_1 = (0 <= __pyx_v_ii);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_ii < __pyx_v_nr);
  }
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_3 = (0 <= __pyx_v_jj);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_jj < __pyx_v_nc);
  }
  __pyx_t_1 = (__pyx_t_3 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L15_bool_binop_done:;
  __pyx_t_1 = ((!__pyx_t_2) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":428
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     out[0] = volume[kk, ii, jj]
 */
    (__pyx_v_out[0]) = 0.0;

    /* "dipy/align/vector_fields.pyx":429
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     out[0] = volume[kk, ii, jj]
 *     return 1
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":427
 *         jj += 1
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":430
 *         out[0] = 0
 *         return 0
 *     out[0] = volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
  __pyx_t_4 = __pyx_v_kk;
  __pyx_t_5 = __pyx_v_ii;
  __pyx_t_6 = __pyx_v_jj;
  (__pyx_v_out[0]) = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_4 * __pyx_v_volume.strides[0]) ) + __pyx_t_5 * __pyx_v_volume.strides[1]) ) + __pyx_t_6 * __pyx_v_volume.strides[2]) )));

  /* "dipy/align/vector_fields.pyx":431
 *         return 0
 *     out[0] = volume[kk, ii, jj]
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":369
 * 
 * 
 * cdef inline int _interpolate_scalar_nn_3d(number[:, :, :] volume, double dkk,             # <<<<<<<<<<<<<<
 *                                          double dii, double djj,
 *                                          number *out) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_5__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__Pyx_memviewslice __pyx_v_volume, double __pyx_v_dkk, double __pyx_v_dii, double __pyx_v_djj, __pyx_t_float_complex *__pyx_v_out) {
  npy_intp __pyx_v_ns;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_kk;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  double __pyx_v_gamma;
  double __pyx_v_cgamma;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;

  /* "dipy/align/vector_fields.pyx":398
 *     """
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]
 */
  __pyx_v_ns = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":399
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = volume.shape[2]
 *         cnp.npy_intp kk, ii, jj
 */
  __pyx_v_nr = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":400
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 */
  __pyx_v_nc = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":403
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = (0.0 <= __pyx_v_dkk);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_dkk <= (__pyx_v_ns - 1));
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (0.0 <= __pyx_v_dii);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_dii <= (__pyx_v_nr - 1));
  }
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (0.0 <= __pyx_v_djj);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_djj <= (__pyx_v_nc - 1));
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  __pyx_t_3 = ((!__pyx_t_1) != 0);
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":404
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # find the top left index and the interpolation coefficients
 */
    (__pyx_v_out[0]) = __pyx_t_float_complex_from_parts(0, 0);

    /* "dipy/align/vector_fields.pyx":405
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":403
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":407
 *         return 0
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)             # <<<<<<<<<<<<<<
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)
 */
  __pyx_v_kk = ((int)floor(__pyx_v_dkk));

  /* "dipy/align/vector_fields.pyx":408
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 *     # no one is affected
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":409
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":411
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_1 = (0 <= __pyx_v_kk);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_kk < __pyx_v_ns);
  }
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_3 = __pyx_t_2;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_2 = (0 <= __pyx_v_ii);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_ii < __pyx_v_nr);
  }
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_1 = (0 <= __pyx_v_jj);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_jj < __pyx_v_nc);
  }
  __pyx_t_2 = (__pyx_t_1 != 0);
  __pyx_t_3 = __pyx_t_2;
  __pyx_L8_bool_binop_done:;
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":412
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     cgamma = dkk - kk
 */
    (__pyx_v_out[0]) = __pyx_t_float_complex_from_parts(0, 0);

    /* "dipy/align/vector_fields.pyx":413
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":411
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":414
 *         out[0] = 0
 *         return 0
 *     cgamma = dkk - kk             # <<<<<<<<<<<<<<
 *     calpha = dii - ii
 *     cbeta = djj - jj
 */
  __pyx_v_cgamma = (__pyx_v_dkk - __pyx_v_kk);

  /* "dipy/align/vector_fields.pyx":415
 *         return 0
 *     cgamma = dkk - kk
 *     calpha = dii - ii             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":416
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":417
 *     calpha = dii - ii
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":418
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":419
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma             # <<<<<<<<<<<<<<
 *     if(gamma < cgamma):
 *         kk += 1
 */
  __pyx_v_gamma = (1.0 - __pyx_v_cgamma);

  /* "dipy/align/vector_fields.pyx":420
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):             # <<<<<<<<<<<<<<
 *         kk += 1
 *     if(alpha < calpha):
 */
  __pyx_t_2 = ((__pyx_v_gamma < __pyx_v_cgamma) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":421
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):
 *         kk += 1             # <<<<<<<<<<<<<<
 *     if(alpha < calpha):
 *         ii += 1
 */
    __pyx_v_kk = (__pyx_v_kk + 1);

    /* "dipy/align/vector_fields.pyx":420
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):             # <<<<<<<<<<<<<<
 *         kk += 1
 *     if(alpha < calpha):
 */
  }

  /* "dipy/align/vector_fields.pyx":422
 *     if(gamma < cgamma):
 *         kk += 1
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  __pyx_t_2 = ((__pyx_v_alpha < __pyx_v_calpha) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":423
 *         kk += 1
 *     if(alpha < calpha):
 *         ii += 1             # <<<<<<<<<<<<<<
 *     if(beta < cbeta):
 *         jj += 1
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":422
 *     if(gamma < cgamma):
 *         kk += 1
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  }

  /* "dipy/align/vector_fields.pyx":424
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  __pyx_t_2 = ((__pyx_v_beta < __pyx_v_cbeta) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":425
 *         ii += 1
 *     if(beta < cbeta):
 *         jj += 1             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":424
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  }

  /* "dipy/align/vector_fields.pyx":427
 *         jj += 1
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_3 = (0 <= __pyx_v_kk);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_kk < __pyx_v_ns);
  }
  __pyx_t_1 = (__pyx_t_3 != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_2 = __pyx_t_1;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_1 = (0 <= __pyx_v_ii);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_ii < __pyx_v_nr);
  }
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_3 = (0 <= __pyx_v_jj);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_jj < __pyx_v_nc);
  }
  __pyx_t_1 = (__pyx_t_3 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L15_bool_binop_done:;
  __pyx_t_1 = ((!__pyx_t_2) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":428
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     out[0] = volume[kk, ii, jj]
 */
    (__pyx_v_out[0]) = __pyx_t_float_complex_from_parts(0, 0);

    /* "dipy/align/vector_fields.pyx":429
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     out[0] = volume[kk, ii, jj]
 *     return 1
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":427
 *         jj += 1
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":430
 *         out[0] = 0
 *         return 0
 *     out[0] = volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
  __pyx_t_4 = __pyx_v_kk;
  __pyx_t_5 = __pyx_v_ii;
  __pyx_t_6 = __pyx_v_jj;
  (__pyx_v_out[0]) = (*((__pyx_t_float_complex *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_4 * __pyx_v_volume.strides[0]) ) + __pyx_t_5 * __pyx_v_volume.strides[1]) ) + __pyx_t_6 * __pyx_v_volume.strides[2]) )));

  /* "dipy/align/vector_fields.pyx":431
 *         return 0
 *     out[0] = volume[kk, ii, jj]
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":369
 * 
 * 
 * cdef inline int _interpolate_scalar_nn_3d(number[:, :, :] volume, double dkk,             # <<<<<<<<<<<<<<
 *                                          double dii, double djj,
 *                                          number *out) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_6__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__Pyx_memviewslice __pyx_v_volume, double __pyx_v_dkk, double __pyx_v_dii, double __pyx_v_djj, __pyx_t_double_complex *__pyx_v_out) {
  npy_intp __pyx_v_ns;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_kk;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  double __pyx_v_gamma;
  double __pyx_v_cgamma;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;

  /* "dipy/align/vector_fields.pyx":398
 *     """
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]
 */
  __pyx_v_ns = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":399
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = volume.shape[2]
 *         cnp.npy_intp kk, ii, jj
 */
  __pyx_v_nr = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":400
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 */
  __pyx_v_nc = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":403
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = (0.0 <= __pyx_v_dkk);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_dkk <= (__pyx_v_ns - 1));
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (0.0 <= __pyx_v_dii);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_dii <= (__pyx_v_nr - 1));
  }
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (0.0 <= __pyx_v_djj);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_djj <= (__pyx_v_nc - 1));
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  __pyx_t_3 = ((!__pyx_t_1) != 0);
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":404
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # find the top left index and the interpolation coefficients
 */
    (__pyx_v_out[0]) = __pyx_t_double_complex_from_parts(0, 0);

    /* "dipy/align/vector_fields.pyx":405
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":403
 *         cnp.npy_intp kk, ii, jj
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (0 <= dkk <= ns - 1 and 0 <= dii <= nr - 1 and 0 <= djj <= nc - 1):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":407
 *         return 0
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)             # <<<<<<<<<<<<<<
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)
 */
  __pyx_v_kk = ((int)floor(__pyx_v_dkk));

  /* "dipy/align/vector_fields.pyx":408
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 *     # no one is affected
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":409
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":411
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_1 = (0 <= __pyx_v_kk);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_kk < __pyx_v_ns);
  }
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_3 = __pyx_t_2;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_2 = (0 <= __pyx_v_ii);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_ii < __pyx_v_nr);
  }
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_1 = (0 <= __pyx_v_jj);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_jj < __pyx_v_nc);
  }
  __pyx_t_2 = (__pyx_t_1 != 0);
  __pyx_t_3 = __pyx_t_2;
  __pyx_L8_bool_binop_done:;
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":412
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     cgamma = dkk - kk
 */
    (__pyx_v_out[0]) = __pyx_t_double_complex_from_parts(0, 0);

    /* "dipy/align/vector_fields.pyx":413
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":411
 *     jj = <int>floor(djj)
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":414
 *         out[0] = 0
 *         return 0
 *     cgamma = dkk - kk             # <<<<<<<<<<<<<<
 *     calpha = dii - ii
 *     cbeta = djj - jj
 */
  __pyx_v_cgamma = (__pyx_v_dkk - __pyx_v_kk);

  /* "dipy/align/vector_fields.pyx":415
 *         return 0
 *     cgamma = dkk - kk
 *     calpha = dii - ii             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":416
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":417
 *     calpha = dii - ii
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":418
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":419
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma             # <<<<<<<<<<<<<<
 *     if(gamma < cgamma):
 *         kk += 1
 */
  __pyx_v_gamma = (1.0 - __pyx_v_cgamma);

  /* "dipy/align/vector_fields.pyx":420
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):             # <<<<<<<<<<<<<<
 *         kk += 1
 *     if(alpha < calpha):
 */
  __pyx_t_2 = ((__pyx_v_gamma < __pyx_v_cgamma) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":421
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):
 *         kk += 1             # <<<<<<<<<<<<<<
 *     if(alpha < calpha):
 *         ii += 1
 */
    __pyx_v_kk = (__pyx_v_kk + 1);

    /* "dipy/align/vector_fields.pyx":420
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 *     if(gamma < cgamma):             # <<<<<<<<<<<<<<
 *         kk += 1
 *     if(alpha < calpha):
 */
  }

  /* "dipy/align/vector_fields.pyx":422
 *     if(gamma < cgamma):
 *         kk += 1
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  __pyx_t_2 = ((__pyx_v_alpha < __pyx_v_calpha) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":423
 *         kk += 1
 *     if(alpha < calpha):
 *         ii += 1             # <<<<<<<<<<<<<<
 *     if(beta < cbeta):
 *         jj += 1
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":422
 *     if(gamma < cgamma):
 *         kk += 1
 *     if(alpha < calpha):             # <<<<<<<<<<<<<<
 *         ii += 1
 *     if(beta < cbeta):
 */
  }

  /* "dipy/align/vector_fields.pyx":424
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  __pyx_t_2 = ((__pyx_v_beta < __pyx_v_cbeta) != 0);
  if (__pyx_t_2) {

    /* "dipy/align/vector_fields.pyx":425
 *         ii += 1
 *     if(beta < cbeta):
 *         jj += 1             # <<<<<<<<<<<<<<
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":424
 *     if(alpha < calpha):
 *         ii += 1
 *     if(beta < cbeta):             # <<<<<<<<<<<<<<
 *         jj += 1
 *     # no one is affected
 */
  }

  /* "dipy/align/vector_fields.pyx":427
 *         jj += 1
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_3 = (0 <= __pyx_v_kk);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_kk < __pyx_v_ns);
  }
  __pyx_t_1 = (__pyx_t_3 != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_2 = __pyx_t_1;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_1 = (0 <= __pyx_v_ii);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_ii < __pyx_v_nr);
  }
  __pyx_t_3 = (__pyx_t_1 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_3 = (0 <= __pyx_v_jj);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_jj < __pyx_v_nc);
  }
  __pyx_t_1 = (__pyx_t_3 != 0);
  __pyx_t_2 = __pyx_t_1;
  __pyx_L15_bool_binop_done:;
  __pyx_t_1 = ((!__pyx_t_2) != 0);
  if (__pyx_t_1) {

    /* "dipy/align/vector_fields.pyx":428
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     out[0] = volume[kk, ii, jj]
 */
    (__pyx_v_out[0]) = __pyx_t_double_complex_from_parts(0, 0);

    /* "dipy/align/vector_fields.pyx":429
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     out[0] = volume[kk, ii, jj]
 *     return 1
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":427
 *         jj += 1
 *     # no one is affected
 *     if not ((0 <= kk < ns) and (0 <= ii < nr) and (0 <= jj < nc)):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":430
 *         out[0] = 0
 *         return 0
 *     out[0] = volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *     return 1
 * 
 */
  __pyx_t_4 = __pyx_v_kk;
  __pyx_t_5 = __pyx_v_ii;
  __pyx_t_6 = __pyx_v_jj;
  (__pyx_v_out[0]) = (*((__pyx_t_double_complex *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_4 * __pyx_v_volume.strides[0]) ) + __pyx_t_5 * __pyx_v_volume.strides[1]) ) + __pyx_t_6 * __pyx_v_volume.strides[2]) )));

  /* "dipy/align/vector_fields.pyx":431
 *         return 0
 *     out[0] = volume[kk, ii, jj]
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":369
 * 
 * 
 * cdef inline int _interpolate_scalar_nn_3d(number[:, :, :] volume, double dkk,             # <<<<<<<<<<<<<<
 *                                          double dii, double djj,
 *                                          number *out) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":434
 * 
 * 
 * def interpolate_scalar_3d(floating[:, :, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Trilinear interpolation of a 3D scalar image
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_11interpolate_scalar_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_10interpolate_scalar_3d[] = "Trilinear interpolation of a 3D scalar image\n\n    Interpolates the 3D image at the given locations. This function is\n    a wrapper for _interpolate_scalar_3d for testing purposes, it is\n    equivalent to scipy.ndimage.interpolation.map_coordinates with\n    trilinear interpolation\n\n    Parameters\n    ----------\n    field : array, shape (S, R, C)\n        the 3D image to be interpolated\n    locations : array, shape (n, 3)\n        (locations[i,0], locations[i,1], locations[i,2), 0<=i<n must contain\n        the coordinates to interpolate the image at\n\n    Returns\n    -------\n    out : array, shape (n,)\n        out[i], 0<=i<n will be the interpolated scalar at coordinates\n        locations[i,:], or 0 if locations[i,:] is outside the image\n    inside : array, (n,)\n        if locations[i,:] is inside the image then inside[i]=1, else\n        inside[i]=0\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_11interpolate_scalar_3d = {"interpolate_scalar_3d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_11interpolate_scalar_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_10interpolate_scalar_3d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_11interpolate_scalar_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_10interpolate_scalar_3d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_10interpolate_scalar_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("interpolate_scalar_3d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_image, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_image); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__20, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_127interpolate_scalar_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_127interpolate_scalar_3d = {"__pyx_fuse_0interpolate_scalar_3d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_127interpolate_scalar_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_10interpolate_scalar_3d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_127interpolate_scalar_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_3d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_3d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_126interpolate_scalar_3d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_126interpolate_scalar_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0interpolate_scalar_3d", 0);

  /* "dipy/align/vector_fields.pyx":459
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":461
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":462
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":463
 *         cnp.npy_intp n = locations.shape[0]
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":464
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_3d[floating](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":465
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_3d[floating](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":467
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_3d[floating](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;
          __pyx_t_15 = 2;
          __pyx_t_16 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":466
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_3d[floating](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside
 */
          __pyx_t_17 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_17 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_14 * __pyx_v_locations.strides[0]) ) + __pyx_t_15 * __pyx_v_locations.strides[1]) ))), (&(*((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_16 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":464
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_3d[floating](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":468
 *             inside[i] = _interpolate_scalar_3d[floating](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 468; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 468; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 468; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":434
 * 
 * 
 * def interpolate_scalar_3d(floating[:, :, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Trilinear interpolation of a 3D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_129interpolate_scalar_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_129interpolate_scalar_3d = {"__pyx_fuse_1interpolate_scalar_3d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_129interpolate_scalar_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_10interpolate_scalar_3d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_129interpolate_scalar_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_scalar_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_scalar_3d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_scalar_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_scalar_3d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_128interpolate_scalar_3d(__pyx_self, __pyx_v_image, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_128interpolate_scalar_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1interpolate_scalar_3d", 0);

  /* "dipy/align/vector_fields.pyx":459
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(image).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_image, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 459; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":461
 *     ftype = np.asarray(image).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":462
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":463
 *         cnp.npy_intp n = locations.shape[0]
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 463; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":464
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_3d[floating](image,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":465
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_scalar_3d[floating](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":467
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_3d[floating](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;
          __pyx_t_15 = 2;
          __pyx_t_16 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":466
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_3d[floating](image,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside
 */
          __pyx_t_17 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_17 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__pyx_v_image, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_14 * __pyx_v_locations.strides[0]) ) + __pyx_t_15 * __pyx_v_locations.strides[1]) ))), (&(*((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_16 * __pyx_v_out.strides[0]) )))));
        }
      }

      /* "dipy/align/vector_fields.pyx":464
 *         floating[:] out = np.zeros(shape=(n,), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_scalar_3d[floating](image,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":468
 *             inside[i] = _interpolate_scalar_3d[floating](image,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], &out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 1, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 468; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 468; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 468; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":434
 * 
 * 
 * def interpolate_scalar_3d(floating[:, :, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Trilinear interpolation of a 3D scalar image
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_scalar_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":471
 * 
 * 
 * cdef inline int _interpolate_scalar_3d(floating[:, :, :] volume,             # <<<<<<<<<<<<<<
 *                                        double dkk, double dii, double djj,
 *                                        floating *out) nogil:
 */

static CYTHON_INLINE int __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__Pyx_memviewslice __pyx_v_volume, double __pyx_v_dkk, double __pyx_v_dii, double __pyx_v_djj, float *__pyx_v_out) {
  npy_intp __pyx_v_ns;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_kk;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  int __pyx_v_inside;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  double __pyx_v_gamma;
  double __pyx_v_cgamma;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  long __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_t_29;

  /* "dipy/align/vector_fields.pyx":500
 *     """
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]
 */
  __pyx_v_ns = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":501
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = volume.shape[2]
 *         cnp.npy_intp kk, ii, jj
 */
  __pyx_v_nr = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":502
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp kk, ii, jj
 *         int inside
 */
  __pyx_v_nc = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":506
 *         int inside
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (-1 < dkk < ns and -1 < dii < nr and -1 < djj < nc):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = (-1.0 < __pyx_v_dkk);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_dkk < __pyx_v_ns);
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (-1.0 < __pyx_v_dii);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_dii < __pyx_v_nr);
  }
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (-1.0 < __pyx_v_djj);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_djj < __pyx_v_nc);
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  __pyx_t_3 = ((!__pyx_t_1) != 0);
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":507
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (-1 < dkk < ns and -1 < dii < nr and -1 < djj < nc):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # find the top left index and the interpolation coefficients
 */
    (__pyx_v_out[0]) = 0.0;

    /* "dipy/align/vector_fields.pyx":508
 *     if not (-1 < dkk < ns and -1 < dii < nr and -1 < djj < nc):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":506
 *         int inside
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (-1 < dkk < ns and -1 < dii < nr and -1 < djj < nc):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":510
 *         return 0
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)             # <<<<<<<<<<<<<<
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)
 */
  __pyx_v_kk = ((int)floor(__pyx_v_dkk));

  /* "dipy/align/vector_fields.pyx":511
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 *     # no one is affected
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":512
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 *     # no one is affected
 * 
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":515
 *     # no one is affected
 * 
 *     cgamma = dkk - kk             # <<<<<<<<<<<<<<
 *     calpha = dii - ii
 *     cbeta = djj - jj
 */
  __pyx_v_cgamma = (__pyx_v_dkk - __pyx_v_kk);

  /* "dipy/align/vector_fields.pyx":516
 * 
 *     cgamma = dkk - kk
 *     calpha = dii - ii             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":517
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":518
 *     calpha = dii - ii
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":519
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     gamma = 1 - cgamma
 * 
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":520
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma             # <<<<<<<<<<<<<<
 * 
 *     inside = 0
 */
  __pyx_v_gamma = (1.0 - __pyx_v_cgamma);

  /* "dipy/align/vector_fields.pyx":522
 *     gamma = 1 - cgamma
 * 
 *     inside = 0             # <<<<<<<<<<<<<<
 *     # ---top-left
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):
 */
  __pyx_v_inside = 0;

  /* "dipy/align/vector_fields.pyx":524
 *     inside = 0
 *     # ---top-left
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] = alpha * beta * gamma * volume[kk, ii, jj]
 *         inside += 1
 */
  __pyx_t_1 = ((__pyx_v_ii >= 0) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_jj >= 0) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_kk >= 0) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L8_bool_binop_done:;
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":525
 *     # ---top-left
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):
 *         out[0] = alpha * beta * gamma * volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     else:
 */
    __pyx_t_4 = __pyx_v_kk;
    __pyx_t_5 = __pyx_v_ii;
    __pyx_t_6 = __pyx_v_jj;
    (__pyx_v_out[0]) = (((__pyx_v_alpha * __pyx_v_beta) * __pyx_v_gamma) * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_4 * __pyx_v_volume.strides[0]) ) + __pyx_t_5 * __pyx_v_volume.strides[1]) ) + __pyx_t_6 * __pyx_v_volume.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":526
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):
 *         out[0] = alpha * beta * gamma * volume[kk, ii, jj]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     else:
 *         out[0] = 0
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":524
 *     inside = 0
 *     # ---top-left
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] = alpha * beta * gamma * volume[kk, ii, jj]
 *         inside += 1
 */
    goto __pyx_L7;
  }

  /* "dipy/align/vector_fields.pyx":528
 *         inside += 1
 *     else:
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *     # ---top-right
 *     jj += 1
 */
  /*else*/ {
    (__pyx_v_out[0]) = 0.0;
  }
  __pyx_L7:;

  /* "dipy/align/vector_fields.pyx":530
 *         out[0] = 0
 *     # ---top-right
 *     jj += 1             # <<<<<<<<<<<<<<
 *     if (ii >= 0) and (jj < nc) and (kk >= 0):
 *         out[0] += alpha * cbeta * gamma * volume[kk, ii, jj]
 */
  __pyx_v_jj = (__pyx_v_jj + 1);

  /* "dipy/align/vector_fields.pyx":531
 *     # ---top-right
 *     jj += 1
 *     if (ii >= 0) and (jj < nc) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += alpha * cbeta * gamma * volume[kk, ii, jj]
 *         inside += 1
 */
  __pyx_t_1 = ((__pyx_v_ii >= 0) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_jj < __pyx_v_nc) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_kk >= 0) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L12_bool_binop_done:;
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":532
 *     jj += 1
 *     if (ii >= 0) and (jj < nc) and (kk >= 0):
 *         out[0] += alpha * cbeta * gamma * volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     # ---bottom-right
 */
    __pyx_t_7 = 0;
    __pyx_t_8 = __pyx_v_kk;
    __pyx_t_9 = __pyx_v_ii;
    __pyx_t_10 = __pyx_v_jj;
    (__pyx_v_out[__pyx_t_7]) = ((__pyx_v_out[__pyx_t_7]) + (((__pyx_v_alpha * __pyx_v_cbeta) * __pyx_v_gamma) * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_8 * __pyx_v_volume.strides[0]) ) + __pyx_t_9 * __pyx_v_volume.strides[1]) ) + __pyx_t_10 * __pyx_v_volume.strides[2]) )))));

    /* "dipy/align/vector_fields.pyx":533
 *     if (ii >= 0) and (jj < nc) and (kk >= 0):
 *         out[0] += alpha * cbeta * gamma * volume[kk, ii, jj]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     # ---bottom-right
 *     ii += 1
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":531
 *     # ---top-right
 *     jj += 1
 *     if (ii >= 0) and (jj < nc) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += alpha * cbeta * gamma * volume[kk, ii, jj]
 *         inside += 1
 */
  }

  /* "dipy/align/vector_fields.pyx":535
 *         inside += 1
 *     # ---bottom-right
 *     ii += 1             # <<<<<<<<<<<<<<
 *     if (ii < nr) and (jj < nc) and (kk >= 0):
 *         out[0] += calpha * cbeta * gamma * volume[kk, ii, jj]
 */
  __pyx_v_ii = (__pyx_v_ii + 1);

  /* "dipy/align/vector_fields.pyx":536
 *     # ---bottom-right
 *     ii += 1
 *     if (ii < nr) and (jj < nc) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * cbeta * gamma * volume[kk, ii, jj]
 *         inside += 1
 */
  __pyx_t_1 = ((__pyx_v_ii < __pyx_v_nr) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_jj < __pyx_v_nc) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_kk >= 0) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L16_bool_binop_done:;
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":537
 *     ii += 1
 *     if (ii < nr) and (jj < nc) and (kk >= 0):
 *         out[0] += calpha * cbeta * gamma * volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     # ---bottom-left
 */
    __pyx_t_7 = 0;
    __pyx_t_11 = __pyx_v_kk;
    __pyx_t_12 = __pyx_v_ii;
    __pyx_t_13 = __pyx_v_jj;
    (__pyx_v_out[__pyx_t_7]) = ((__pyx_v_out[__pyx_t_7]) + (((__pyx_v_calpha * __pyx_v_cbeta) * __pyx_v_gamma) * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_11 * __pyx_v_volume.strides[0]) ) + __pyx_t_12 * __pyx_v_volume.strides[1]) ) + __pyx_t_13 * __pyx_v_volume.strides[2]) )))));

    /* "dipy/align/vector_fields.pyx":538
 *     if (ii < nr) and (jj < nc) and (kk >= 0):
 *         out[0] += calpha * cbeta * gamma * volume[kk, ii, jj]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     # ---bottom-left
 *     jj -= 1
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":536
 *     # ---bottom-right
 *     ii += 1
 *     if (ii < nr) and (jj < nc) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * cbeta * gamma * volume[kk, ii, jj]
 *         inside += 1
 */
  }

  /* "dipy/align/vector_fields.pyx":540
 *         inside += 1
 *     # ---bottom-left
 *     jj -= 1             # <<<<<<<<<<<<<<
 *     if (ii < nr) and (jj >= 0) and (kk >= 0):
 *         out[0] += calpha * beta * gamma * volume[kk, ii, jj]
 */
  __pyx_v_jj = (__pyx_v_jj - 1);

  /* "dipy/align/vector_fields.pyx":541
 *     # ---bottom-left
 *     jj -= 1
 *     if (ii < nr) and (jj >= 0) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * beta * gamma * volume[kk, ii, jj]
 *         inside += 1
 */
  __pyx_t_1 = ((__pyx_v_ii < __pyx_v_nr) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L20_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_jj >= 0) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L20_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_kk >= 0) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L20_bool_binop_done:;
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":542
 *     jj -= 1
 *     if (ii < nr) and (jj >= 0) and (kk >= 0):
 *         out[0] += calpha * beta * gamma * volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     kk += 1
 */
    __pyx_t_7 = 0;
    __pyx_t_14 = __pyx_v_kk;
    __pyx_t_15 = __pyx_v_ii;
    __pyx_t_16 = __pyx_v_jj;
    (__pyx_v_out[__pyx_t_7]) = ((__pyx_v_out[__pyx_t_7]) + (((__pyx_v_calpha * __pyx_v_beta) * __pyx_v_gamma) * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_14 * __pyx_v_volume.strides[0]) ) + __pyx_t_15 * __pyx_v_volume.strides[1]) ) + __pyx_t_16 * __pyx_v_volume.strides[2]) )))));

    /* "dipy/align/vector_fields.pyx":543
 *     if (ii < nr) and (jj >= 0) and (kk >= 0):
 *         out[0] += calpha * beta * gamma * volume[kk, ii, jj]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     kk += 1
 *     if(kk < ns):
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":541
 *     # ---bottom-left
 *     jj -= 1
 *     if (ii < nr) and (jj >= 0) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * beta * gamma * volume[kk, ii, jj]
 *         inside += 1
 */
  }

  /* "dipy/align/vector_fields.pyx":544
 *         out[0] += calpha * beta * gamma * volume[kk, ii, jj]
 *         inside += 1
 *     kk += 1             # <<<<<<<<<<<<<<
 *     if(kk < ns):
 *         ii -= 1
 */
  __pyx_v_kk = (__pyx_v_kk + 1);

  /* "dipy/align/vector_fields.pyx":545
 *         inside += 1
 *     kk += 1
 *     if(kk < ns):             # <<<<<<<<<<<<<<
 *         ii -= 1
 *         if (ii >= 0) and (jj >= 0):
 */
  __pyx_t_3 = ((__pyx_v_kk < __pyx_v_ns) != 0);
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":546
 *     kk += 1
 *     if(kk < ns):
 *         ii -= 1             # <<<<<<<<<<<<<<
 *         if (ii >= 0) and (jj >= 0):
 *             out[0] += alpha * beta * cgamma * volume[kk, ii, jj]
 */
    __pyx_v_ii = (__pyx_v_ii - 1);

    /* "dipy/align/vector_fields.pyx":547
 *     if(kk < ns):
 *         ii -= 1
 *         if (ii >= 0) and (jj >= 0):             # <<<<<<<<<<<<<<
 *             out[0] += alpha * beta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 */
    __pyx_t_1 = ((__pyx_v_ii >= 0) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_3 = __pyx_t_1;
      goto __pyx_L25_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_jj >= 0) != 0);
    __pyx_t_3 = __pyx_t_1;
    __pyx_L25_bool_binop_done:;
    if (__pyx_t_3) {

      /* "dipy/align/vector_fields.pyx":548
 *         ii -= 1
 *         if (ii >= 0) and (jj >= 0):
 *             out[0] += alpha * beta * cgamma * volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *             inside += 1
 *         jj += 1
 */
      __pyx_t_7 = 0;
      __pyx_t_17 = __pyx_v_kk;
      __pyx_t_18 = __pyx_v_ii;
      __pyx_t_19 = __pyx_v_jj;
      (__pyx_v_out[__pyx_t_7]) = ((__pyx_v_out[__pyx_t_7]) + (((__pyx_v_alpha * __pyx_v_beta) * __pyx_v_cgamma) * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_17 * __pyx_v_volume.strides[0]) ) + __pyx_t_18 * __pyx_v_volume.strides[1]) ) + __pyx_t_19 * __pyx_v_volume.strides[2]) )))));

      /* "dipy/align/vector_fields.pyx":549
 *         if (ii >= 0) and (jj >= 0):
 *             out[0] += alpha * beta * cgamma * volume[kk, ii, jj]
 *             inside += 1             # <<<<<<<<<<<<<<
 *         jj += 1
 *         if (ii >= 0) and (jj < nc):
 */
      __pyx_v_inside = (__pyx_v_inside + 1);

      /* "dipy/align/vector_fields.pyx":547
 *     if(kk < ns):
 *         ii -= 1
 *         if (ii >= 0) and (jj >= 0):             # <<<<<<<<<<<<<<
 *             out[0] += alpha * beta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 */
    }

    /* "dipy/align/vector_fields.pyx":550
 *             out[0] += alpha * beta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 *         jj += 1             # <<<<<<<<<<<<<<
 *         if (ii >= 0) and (jj < nc):
 *             out[0] += alpha * cbeta * cgamma * volume[kk, ii, jj]
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":551
 *             inside += 1
 *         jj += 1
 *         if (ii >= 0) and (jj < nc):             # <<<<<<<<<<<<<<
 *             out[0] += alpha * cbeta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 */
    __pyx_t_1 = ((__pyx_v_ii >= 0) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_3 = __pyx_t_1;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_jj < __pyx_v_nc) != 0);
    __pyx_t_3 = __pyx_t_1;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_3) {

      /* "dipy/align/vector_fields.pyx":552
 *         jj += 1
 *         if (ii >= 0) and (jj < nc):
 *             out[0] += alpha * cbeta * cgamma * volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *             inside += 1
 *         # ---bottom-right
 */
      __pyx_t_7 = 0;
      __pyx_t_20 = __pyx_v_kk;
      __pyx_t_21 = __pyx_v_ii;
      __pyx_t_22 = __pyx_v_jj;
      (__pyx_v_out[__pyx_t_7]) = ((__pyx_v_out[__pyx_t_7]) + (((__pyx_v_alpha * __pyx_v_cbeta) * __pyx_v_cgamma) * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_20 * __pyx_v_volume.strides[0]) ) + __pyx_t_21 * __pyx_v_volume.strides[1]) ) + __pyx_t_22 * __pyx_v_volume.strides[2]) )))));

      /* "dipy/align/vector_fields.pyx":553
 *         if (ii >= 0) and (jj < nc):
 *             out[0] += alpha * cbeta * cgamma * volume[kk, ii, jj]
 *             inside += 1             # <<<<<<<<<<<<<<
 *         # ---bottom-right
 *         ii += 1
 */
      __pyx_v_inside = (__pyx_v_inside + 1);

      /* "dipy/align/vector_fields.pyx":551
 *             inside += 1
 *         jj += 1
 *         if (ii >= 0) and (jj < nc):             # <<<<<<<<<<<<<<
 *             out[0] += alpha * cbeta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 */
    }

    /* "dipy/align/vector_fields.pyx":555
 *             inside += 1
 *         # ---bottom-right
 *         ii += 1             # <<<<<<<<<<<<<<
 *         if (ii < nr) and (jj < nc):
 *             out[0] += calpha * cbeta * cgamma * volume[kk, ii, jj]
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":556
 *         # ---bottom-right
 *         ii += 1
 *         if (ii < nr) and (jj < nc):             # <<<<<<<<<<<<<<
 *             out[0] += calpha * cbeta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 */
    __pyx_t_1 = ((__pyx_v_ii < __pyx_v_nr) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_3 = __pyx_t_1;
      goto __pyx_L31_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_jj < __pyx_v_nc) != 0);
    __pyx_t_3 = __pyx_t_1;
    __pyx_L31_bool_binop_done:;
    if (__pyx_t_3) {

      /* "dipy/align/vector_fields.pyx":557
 *         ii += 1
 *         if (ii < nr) and (jj < nc):
 *             out[0] += calpha * cbeta * cgamma * volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *             inside += 1
 *         # ---bottom-left
 */
      __pyx_t_7 = 0;
      __pyx_t_23 = __pyx_v_kk;
      __pyx_t_24 = __pyx_v_ii;
      __pyx_t_25 = __pyx_v_jj;
      (__pyx_v_out[__pyx_t_7]) = ((__pyx_v_out[__pyx_t_7]) + (((__pyx_v_calpha * __pyx_v_cbeta) * __pyx_v_cgamma) * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_23 * __pyx_v_volume.strides[0]) ) + __pyx_t_24 * __pyx_v_volume.strides[1]) ) + __pyx_t_25 * __pyx_v_volume.strides[2]) )))));

      /* "dipy/align/vector_fields.pyx":558
 *         if (ii < nr) and (jj < nc):
 *             out[0] += calpha * cbeta * cgamma * volume[kk, ii, jj]
 *             inside += 1             # <<<<<<<<<<<<<<
 *         # ---bottom-left
 *         jj -= 1
 */
      __pyx_v_inside = (__pyx_v_inside + 1);

      /* "dipy/align/vector_fields.pyx":556
 *         # ---bottom-right
 *         ii += 1
 *         if (ii < nr) and (jj < nc):             # <<<<<<<<<<<<<<
 *             out[0] += calpha * cbeta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 */
    }

    /* "dipy/align/vector_fields.pyx":560
 *             inside += 1
 *         # ---bottom-left
 *         jj -= 1             # <<<<<<<<<<<<<<
 *         if (ii < nr) and (jj >= 0):
 *             out[0] += calpha * beta * cgamma * volume[kk, ii, jj]
 */
    __pyx_v_jj = (__pyx_v_jj - 1);

    /* "dipy/align/vector_fields.pyx":561
 *         # ---bottom-left
 *         jj -= 1
 *         if (ii < nr) and (jj >= 0):             # <<<<<<<<<<<<<<
 *             out[0] += calpha * beta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 */
    __pyx_t_1 = ((__pyx_v_ii < __pyx_v_nr) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_3 = __pyx_t_1;
      goto __pyx_L34_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_jj >= 0) != 0);
    __pyx_t_3 = __pyx_t_1;
    __pyx_L34_bool_binop_done:;
    if (__pyx_t_3) {

      /* "dipy/align/vector_fields.pyx":562
 *         jj -= 1
 *         if (ii < nr) and (jj >= 0):
 *             out[0] += calpha * beta * cgamma * volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *             inside += 1
 *     return 1 if inside == 8 else 0
 */
      __pyx_t_7 = 0;
      __pyx_t_26 = __pyx_v_kk;
      __pyx_t_27 = __pyx_v_ii;
      __pyx_t_28 = __pyx_v_jj;
      (__pyx_v_out[__pyx_t_7]) = ((__pyx_v_out[__pyx_t_7]) + (((__pyx_v_calpha * __pyx_v_beta) * __pyx_v_cgamma) * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_26 * __pyx_v_volume.strides[0]) ) + __pyx_t_27 * __pyx_v_volume.strides[1]) ) + __pyx_t_28 * __pyx_v_volume.strides[2]) )))));

      /* "dipy/align/vector_fields.pyx":563
 *         if (ii < nr) and (jj >= 0):
 *             out[0] += calpha * beta * cgamma * volume[kk, ii, jj]
 *             inside += 1             # <<<<<<<<<<<<<<
 *     return 1 if inside == 8 else 0
 * 
 */
      __pyx_v_inside = (__pyx_v_inside + 1);

      /* "dipy/align/vector_fields.pyx":561
 *         # ---bottom-left
 *         jj -= 1
 *         if (ii < nr) and (jj >= 0):             # <<<<<<<<<<<<<<
 *             out[0] += calpha * beta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 */
    }

    /* "dipy/align/vector_fields.pyx":545
 *         inside += 1
 *     kk += 1
 *     if(kk < ns):             # <<<<<<<<<<<<<<
 *         ii -= 1
 *         if (ii >= 0) and (jj >= 0):
 */
  }

  /* "dipy/align/vector_fields.pyx":564
 *             out[0] += calpha * beta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 *     return 1 if inside == 8 else 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (((__pyx_v_inside == 8) != 0)) {
    __pyx_t_29 = 1;
  } else {
    __pyx_t_29 = 0;
  }
  __pyx_r = __pyx_t_29;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":471
 * 
 * 
 * cdef inline int _interpolate_scalar_3d(floating[:, :, :] volume,             # <<<<<<<<<<<<<<
 *                                        double dkk, double dii, double djj,
 *                                        floating *out) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__Pyx_memviewslice __pyx_v_volume, double __pyx_v_dkk, double __pyx_v_dii, double __pyx_v_djj, double *__pyx_v_out) {
  npy_intp __pyx_v_ns;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_kk;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  int __pyx_v_inside;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  double __pyx_v_gamma;
  double __pyx_v_cgamma;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  long __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_t_29;

  /* "dipy/align/vector_fields.pyx":500
 *     """
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]
 */
  __pyx_v_ns = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":501
 *     cdef:
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = volume.shape[2]
 *         cnp.npy_intp kk, ii, jj
 */
  __pyx_v_nr = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":502
 *         cnp.npy_intp ns = volume.shape[0]
 *         cnp.npy_intp nr = volume.shape[1]
 *         cnp.npy_intp nc = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp kk, ii, jj
 *         int inside
 */
  __pyx_v_nc = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":506
 *         int inside
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (-1 < dkk < ns and -1 < dii < nr and -1 < djj < nc):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  __pyx_t_2 = (-1.0 < __pyx_v_dkk);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_dkk < __pyx_v_ns);
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (-1.0 < __pyx_v_dii);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_dii < __pyx_v_nr);
  }
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (-1.0 < __pyx_v_djj);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_djj < __pyx_v_nc);
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  __pyx_t_3 = ((!__pyx_t_1) != 0);
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":507
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (-1 < dkk < ns and -1 < dii < nr and -1 < djj < nc):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     # find the top left index and the interpolation coefficients
 */
    (__pyx_v_out[0]) = 0.0;

    /* "dipy/align/vector_fields.pyx":508
 *     if not (-1 < dkk < ns and -1 < dii < nr and -1 < djj < nc):
 *         out[0] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":506
 *         int inside
 *         double alpha, beta, calpha, cbeta, gamma, cgamma
 *     if not (-1 < dkk < ns and -1 < dii < nr and -1 < djj < nc):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         return 0
 */
  }

  /* "dipy/align/vector_fields.pyx":510
 *         return 0
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)             # <<<<<<<<<<<<<<
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)
 */
  __pyx_v_kk = ((int)floor(__pyx_v_dkk));

  /* "dipy/align/vector_fields.pyx":511
 *     # find the top left index and the interpolation coefficients
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 *     # no one is affected
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":512
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 *     # no one is affected
 * 
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":515
 *     # no one is affected
 * 
 *     cgamma = dkk - kk             # <<<<<<<<<<<<<<
 *     calpha = dii - ii
 *     cbeta = djj - jj
 */
  __pyx_v_cgamma = (__pyx_v_dkk - __pyx_v_kk);

  /* "dipy/align/vector_fields.pyx":516
 * 
 *     cgamma = dkk - kk
 *     calpha = dii - ii             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":517
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":518
 *     calpha = dii - ii
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":519
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     gamma = 1 - cgamma
 * 
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":520
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma             # <<<<<<<<<<<<<<
 * 
 *     inside = 0
 */
  __pyx_v_gamma = (1.0 - __pyx_v_cgamma);

  /* "dipy/align/vector_fields.pyx":522
 *     gamma = 1 - cgamma
 * 
 *     inside = 0             # <<<<<<<<<<<<<<
 *     # ---top-left
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):
 */
  __pyx_v_inside = 0;

  /* "dipy/align/vector_fields.pyx":524
 *     inside = 0
 *     # ---top-left
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] = alpha * beta * gamma * volume[kk, ii, jj]
 *         inside += 1
 */
  __pyx_t_1 = ((__pyx_v_ii >= 0) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_jj >= 0) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_kk >= 0) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L8_bool_binop_done:;
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":525
 *     # ---top-left
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):
 *         out[0] = alpha * beta * gamma * volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     else:
 */
    __pyx_t_4 = __pyx_v_kk;
    __pyx_t_5 = __pyx_v_ii;
    __pyx_t_6 = __pyx_v_jj;
    (__pyx_v_out[0]) = (((__pyx_v_alpha * __pyx_v_beta) * __pyx_v_gamma) * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_4 * __pyx_v_volume.strides[0]) ) + __pyx_t_5 * __pyx_v_volume.strides[1]) ) + __pyx_t_6 * __pyx_v_volume.strides[2]) ))));

    /* "dipy/align/vector_fields.pyx":526
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):
 *         out[0] = alpha * beta * gamma * volume[kk, ii, jj]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     else:
 *         out[0] = 0
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":524
 *     inside = 0
 *     # ---top-left
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] = alpha * beta * gamma * volume[kk, ii, jj]
 *         inside += 1
 */
    goto __pyx_L7;
  }

  /* "dipy/align/vector_fields.pyx":528
 *         inside += 1
 *     else:
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *     # ---top-right
 *     jj += 1
 */
  /*else*/ {
    (__pyx_v_out[0]) = 0.0;
  }
  __pyx_L7:;

  /* "dipy/align/vector_fields.pyx":530
 *         out[0] = 0
 *     # ---top-right
 *     jj += 1             # <<<<<<<<<<<<<<
 *     if (ii >= 0) and (jj < nc) and (kk >= 0):
 *         out[0] += alpha * cbeta * gamma * volume[kk, ii, jj]
 */
  __pyx_v_jj = (__pyx_v_jj + 1);

  /* "dipy/align/vector_fields.pyx":531
 *     # ---top-right
 *     jj += 1
 *     if (ii >= 0) and (jj < nc) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += alpha * cbeta * gamma * volume[kk, ii, jj]
 *         inside += 1
 */
  __pyx_t_1 = ((__pyx_v_ii >= 0) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_jj < __pyx_v_nc) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_kk >= 0) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L12_bool_binop_done:;
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":532
 *     jj += 1
 *     if (ii >= 0) and (jj < nc) and (kk >= 0):
 *         out[0] += alpha * cbeta * gamma * volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     # ---bottom-right
 */
    __pyx_t_7 = 0;
    __pyx_t_8 = __pyx_v_kk;
    __pyx_t_9 = __pyx_v_ii;
    __pyx_t_10 = __pyx_v_jj;
    (__pyx_v_out[__pyx_t_7]) = ((__pyx_v_out[__pyx_t_7]) + (((__pyx_v_alpha * __pyx_v_cbeta) * __pyx_v_gamma) * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_8 * __pyx_v_volume.strides[0]) ) + __pyx_t_9 * __pyx_v_volume.strides[1]) ) + __pyx_t_10 * __pyx_v_volume.strides[2]) )))));

    /* "dipy/align/vector_fields.pyx":533
 *     if (ii >= 0) and (jj < nc) and (kk >= 0):
 *         out[0] += alpha * cbeta * gamma * volume[kk, ii, jj]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     # ---bottom-right
 *     ii += 1
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":531
 *     # ---top-right
 *     jj += 1
 *     if (ii >= 0) and (jj < nc) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += alpha * cbeta * gamma * volume[kk, ii, jj]
 *         inside += 1
 */
  }

  /* "dipy/align/vector_fields.pyx":535
 *         inside += 1
 *     # ---bottom-right
 *     ii += 1             # <<<<<<<<<<<<<<
 *     if (ii < nr) and (jj < nc) and (kk >= 0):
 *         out[0] += calpha * cbeta * gamma * volume[kk, ii, jj]
 */
  __pyx_v_ii = (__pyx_v_ii + 1);

  /* "dipy/align/vector_fields.pyx":536
 *     # ---bottom-right
 *     ii += 1
 *     if (ii < nr) and (jj < nc) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * cbeta * gamma * volume[kk, ii, jj]
 *         inside += 1
 */
  __pyx_t_1 = ((__pyx_v_ii < __pyx_v_nr) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_jj < __pyx_v_nc) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_kk >= 0) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L16_bool_binop_done:;
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":537
 *     ii += 1
 *     if (ii < nr) and (jj < nc) and (kk >= 0):
 *         out[0] += calpha * cbeta * gamma * volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     # ---bottom-left
 */
    __pyx_t_7 = 0;
    __pyx_t_11 = __pyx_v_kk;
    __pyx_t_12 = __pyx_v_ii;
    __pyx_t_13 = __pyx_v_jj;
    (__pyx_v_out[__pyx_t_7]) = ((__pyx_v_out[__pyx_t_7]) + (((__pyx_v_calpha * __pyx_v_cbeta) * __pyx_v_gamma) * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_11 * __pyx_v_volume.strides[0]) ) + __pyx_t_12 * __pyx_v_volume.strides[1]) ) + __pyx_t_13 * __pyx_v_volume.strides[2]) )))));

    /* "dipy/align/vector_fields.pyx":538
 *     if (ii < nr) and (jj < nc) and (kk >= 0):
 *         out[0] += calpha * cbeta * gamma * volume[kk, ii, jj]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     # ---bottom-left
 *     jj -= 1
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":536
 *     # ---bottom-right
 *     ii += 1
 *     if (ii < nr) and (jj < nc) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * cbeta * gamma * volume[kk, ii, jj]
 *         inside += 1
 */
  }

  /* "dipy/align/vector_fields.pyx":540
 *         inside += 1
 *     # ---bottom-left
 *     jj -= 1             # <<<<<<<<<<<<<<
 *     if (ii < nr) and (jj >= 0) and (kk >= 0):
 *         out[0] += calpha * beta * gamma * volume[kk, ii, jj]
 */
  __pyx_v_jj = (__pyx_v_jj - 1);

  /* "dipy/align/vector_fields.pyx":541
 *     # ---bottom-left
 *     jj -= 1
 *     if (ii < nr) and (jj >= 0) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * beta * gamma * volume[kk, ii, jj]
 *         inside += 1
 */
  __pyx_t_1 = ((__pyx_v_ii < __pyx_v_nr) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L20_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_jj >= 0) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L20_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_kk >= 0) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L20_bool_binop_done:;
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":542
 *     jj -= 1
 *     if (ii < nr) and (jj >= 0) and (kk >= 0):
 *         out[0] += calpha * beta * gamma * volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     kk += 1
 */
    __pyx_t_7 = 0;
    __pyx_t_14 = __pyx_v_kk;
    __pyx_t_15 = __pyx_v_ii;
    __pyx_t_16 = __pyx_v_jj;
    (__pyx_v_out[__pyx_t_7]) = ((__pyx_v_out[__pyx_t_7]) + (((__pyx_v_calpha * __pyx_v_beta) * __pyx_v_gamma) * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_14 * __pyx_v_volume.strides[0]) ) + __pyx_t_15 * __pyx_v_volume.strides[1]) ) + __pyx_t_16 * __pyx_v_volume.strides[2]) )))));

    /* "dipy/align/vector_fields.pyx":543
 *     if (ii < nr) and (jj >= 0) and (kk >= 0):
 *         out[0] += calpha * beta * gamma * volume[kk, ii, jj]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     kk += 1
 *     if(kk < ns):
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":541
 *     # ---bottom-left
 *     jj -= 1
 *     if (ii < nr) and (jj >= 0) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * beta * gamma * volume[kk, ii, jj]
 *         inside += 1
 */
  }

  /* "dipy/align/vector_fields.pyx":544
 *         out[0] += calpha * beta * gamma * volume[kk, ii, jj]
 *         inside += 1
 *     kk += 1             # <<<<<<<<<<<<<<
 *     if(kk < ns):
 *         ii -= 1
 */
  __pyx_v_kk = (__pyx_v_kk + 1);

  /* "dipy/align/vector_fields.pyx":545
 *         inside += 1
 *     kk += 1
 *     if(kk < ns):             # <<<<<<<<<<<<<<
 *         ii -= 1
 *         if (ii >= 0) and (jj >= 0):
 */
  __pyx_t_3 = ((__pyx_v_kk < __pyx_v_ns) != 0);
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":546
 *     kk += 1
 *     if(kk < ns):
 *         ii -= 1             # <<<<<<<<<<<<<<
 *         if (ii >= 0) and (jj >= 0):
 *             out[0] += alpha * beta * cgamma * volume[kk, ii, jj]
 */
    __pyx_v_ii = (__pyx_v_ii - 1);

    /* "dipy/align/vector_fields.pyx":547
 *     if(kk < ns):
 *         ii -= 1
 *         if (ii >= 0) and (jj >= 0):             # <<<<<<<<<<<<<<
 *             out[0] += alpha * beta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 */
    __pyx_t_1 = ((__pyx_v_ii >= 0) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_3 = __pyx_t_1;
      goto __pyx_L25_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_jj >= 0) != 0);
    __pyx_t_3 = __pyx_t_1;
    __pyx_L25_bool_binop_done:;
    if (__pyx_t_3) {

      /* "dipy/align/vector_fields.pyx":548
 *         ii -= 1
 *         if (ii >= 0) and (jj >= 0):
 *             out[0] += alpha * beta * cgamma * volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *             inside += 1
 *         jj += 1
 */
      __pyx_t_7 = 0;
      __pyx_t_17 = __pyx_v_kk;
      __pyx_t_18 = __pyx_v_ii;
      __pyx_t_19 = __pyx_v_jj;
      (__pyx_v_out[__pyx_t_7]) = ((__pyx_v_out[__pyx_t_7]) + (((__pyx_v_alpha * __pyx_v_beta) * __pyx_v_cgamma) * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_17 * __pyx_v_volume.strides[0]) ) + __pyx_t_18 * __pyx_v_volume.strides[1]) ) + __pyx_t_19 * __pyx_v_volume.strides[2]) )))));

      /* "dipy/align/vector_fields.pyx":549
 *         if (ii >= 0) and (jj >= 0):
 *             out[0] += alpha * beta * cgamma * volume[kk, ii, jj]
 *             inside += 1             # <<<<<<<<<<<<<<
 *         jj += 1
 *         if (ii >= 0) and (jj < nc):
 */
      __pyx_v_inside = (__pyx_v_inside + 1);

      /* "dipy/align/vector_fields.pyx":547
 *     if(kk < ns):
 *         ii -= 1
 *         if (ii >= 0) and (jj >= 0):             # <<<<<<<<<<<<<<
 *             out[0] += alpha * beta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 */
    }

    /* "dipy/align/vector_fields.pyx":550
 *             out[0] += alpha * beta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 *         jj += 1             # <<<<<<<<<<<<<<
 *         if (ii >= 0) and (jj < nc):
 *             out[0] += alpha * cbeta * cgamma * volume[kk, ii, jj]
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":551
 *             inside += 1
 *         jj += 1
 *         if (ii >= 0) and (jj < nc):             # <<<<<<<<<<<<<<
 *             out[0] += alpha * cbeta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 */
    __pyx_t_1 = ((__pyx_v_ii >= 0) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_3 = __pyx_t_1;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_jj < __pyx_v_nc) != 0);
    __pyx_t_3 = __pyx_t_1;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_3) {

      /* "dipy/align/vector_fields.pyx":552
 *         jj += 1
 *         if (ii >= 0) and (jj < nc):
 *             out[0] += alpha * cbeta * cgamma * volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *             inside += 1
 *         # ---bottom-right
 */
      __pyx_t_7 = 0;
      __pyx_t_20 = __pyx_v_kk;
      __pyx_t_21 = __pyx_v_ii;
      __pyx_t_22 = __pyx_v_jj;
      (__pyx_v_out[__pyx_t_7]) = ((__pyx_v_out[__pyx_t_7]) + (((__pyx_v_alpha * __pyx_v_cbeta) * __pyx_v_cgamma) * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_20 * __pyx_v_volume.strides[0]) ) + __pyx_t_21 * __pyx_v_volume.strides[1]) ) + __pyx_t_22 * __pyx_v_volume.strides[2]) )))));

      /* "dipy/align/vector_fields.pyx":553
 *         if (ii >= 0) and (jj < nc):
 *             out[0] += alpha * cbeta * cgamma * volume[kk, ii, jj]
 *             inside += 1             # <<<<<<<<<<<<<<
 *         # ---bottom-right
 *         ii += 1
 */
      __pyx_v_inside = (__pyx_v_inside + 1);

      /* "dipy/align/vector_fields.pyx":551
 *             inside += 1
 *         jj += 1
 *         if (ii >= 0) and (jj < nc):             # <<<<<<<<<<<<<<
 *             out[0] += alpha * cbeta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 */
    }

    /* "dipy/align/vector_fields.pyx":555
 *             inside += 1
 *         # ---bottom-right
 *         ii += 1             # <<<<<<<<<<<<<<
 *         if (ii < nr) and (jj < nc):
 *             out[0] += calpha * cbeta * cgamma * volume[kk, ii, jj]
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":556
 *         # ---bottom-right
 *         ii += 1
 *         if (ii < nr) and (jj < nc):             # <<<<<<<<<<<<<<
 *             out[0] += calpha * cbeta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 */
    __pyx_t_1 = ((__pyx_v_ii < __pyx_v_nr) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_3 = __pyx_t_1;
      goto __pyx_L31_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_jj < __pyx_v_nc) != 0);
    __pyx_t_3 = __pyx_t_1;
    __pyx_L31_bool_binop_done:;
    if (__pyx_t_3) {

      /* "dipy/align/vector_fields.pyx":557
 *         ii += 1
 *         if (ii < nr) and (jj < nc):
 *             out[0] += calpha * cbeta * cgamma * volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *             inside += 1
 *         # ---bottom-left
 */
      __pyx_t_7 = 0;
      __pyx_t_23 = __pyx_v_kk;
      __pyx_t_24 = __pyx_v_ii;
      __pyx_t_25 = __pyx_v_jj;
      (__pyx_v_out[__pyx_t_7]) = ((__pyx_v_out[__pyx_t_7]) + (((__pyx_v_calpha * __pyx_v_cbeta) * __pyx_v_cgamma) * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_23 * __pyx_v_volume.strides[0]) ) + __pyx_t_24 * __pyx_v_volume.strides[1]) ) + __pyx_t_25 * __pyx_v_volume.strides[2]) )))));

      /* "dipy/align/vector_fields.pyx":558
 *         if (ii < nr) and (jj < nc):
 *             out[0] += calpha * cbeta * cgamma * volume[kk, ii, jj]
 *             inside += 1             # <<<<<<<<<<<<<<
 *         # ---bottom-left
 *         jj -= 1
 */
      __pyx_v_inside = (__pyx_v_inside + 1);

      /* "dipy/align/vector_fields.pyx":556
 *         # ---bottom-right
 *         ii += 1
 *         if (ii < nr) and (jj < nc):             # <<<<<<<<<<<<<<
 *             out[0] += calpha * cbeta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 */
    }

    /* "dipy/align/vector_fields.pyx":560
 *             inside += 1
 *         # ---bottom-left
 *         jj -= 1             # <<<<<<<<<<<<<<
 *         if (ii < nr) and (jj >= 0):
 *             out[0] += calpha * beta * cgamma * volume[kk, ii, jj]
 */
    __pyx_v_jj = (__pyx_v_jj - 1);

    /* "dipy/align/vector_fields.pyx":561
 *         # ---bottom-left
 *         jj -= 1
 *         if (ii < nr) and (jj >= 0):             # <<<<<<<<<<<<<<
 *             out[0] += calpha * beta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 */
    __pyx_t_1 = ((__pyx_v_ii < __pyx_v_nr) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_3 = __pyx_t_1;
      goto __pyx_L34_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_jj >= 0) != 0);
    __pyx_t_3 = __pyx_t_1;
    __pyx_L34_bool_binop_done:;
    if (__pyx_t_3) {

      /* "dipy/align/vector_fields.pyx":562
 *         jj -= 1
 *         if (ii < nr) and (jj >= 0):
 *             out[0] += calpha * beta * cgamma * volume[kk, ii, jj]             # <<<<<<<<<<<<<<
 *             inside += 1
 *     return 1 if inside == 8 else 0
 */
      __pyx_t_7 = 0;
      __pyx_t_26 = __pyx_v_kk;
      __pyx_t_27 = __pyx_v_ii;
      __pyx_t_28 = __pyx_v_jj;
      (__pyx_v_out[__pyx_t_7]) = ((__pyx_v_out[__pyx_t_7]) + (((__pyx_v_calpha * __pyx_v_beta) * __pyx_v_cgamma) * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_volume.data + __pyx_t_26 * __pyx_v_volume.strides[0]) ) + __pyx_t_27 * __pyx_v_volume.strides[1]) ) + __pyx_t_28 * __pyx_v_volume.strides[2]) )))));

      /* "dipy/align/vector_fields.pyx":563
 *         if (ii < nr) and (jj >= 0):
 *             out[0] += calpha * beta * cgamma * volume[kk, ii, jj]
 *             inside += 1             # <<<<<<<<<<<<<<
 *     return 1 if inside == 8 else 0
 * 
 */
      __pyx_v_inside = (__pyx_v_inside + 1);

      /* "dipy/align/vector_fields.pyx":561
 *         # ---bottom-left
 *         jj -= 1
 *         if (ii < nr) and (jj >= 0):             # <<<<<<<<<<<<<<
 *             out[0] += calpha * beta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 */
    }

    /* "dipy/align/vector_fields.pyx":545
 *         inside += 1
 *     kk += 1
 *     if(kk < ns):             # <<<<<<<<<<<<<<
 *         ii -= 1
 *         if (ii >= 0) and (jj >= 0):
 */
  }

  /* "dipy/align/vector_fields.pyx":564
 *             out[0] += calpha * beta * cgamma * volume[kk, ii, jj]
 *             inside += 1
 *     return 1 if inside == 8 else 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (((__pyx_v_inside == 8) != 0)) {
    __pyx_t_29 = 1;
  } else {
    __pyx_t_29 = 0;
  }
  __pyx_r = __pyx_t_29;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":471
 * 
 * 
 * cdef inline int _interpolate_scalar_3d(floating[:, :, :] volume,             # <<<<<<<<<<<<<<
 *                                        double dkk, double dii, double djj,
 *                                        floating *out) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":567
 * 
 * 
 * def interpolate_vector_3d(floating[:, :, :, :] field, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Trilinear interpolation of a 3D vector field
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_13interpolate_vector_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_12interpolate_vector_3d[] = "Trilinear interpolation of a 3D vector field\n\n    Interpolates the 3D vector field at the given locations. This function is\n    a wrapper for _interpolate_vector_3d for testing purposes, it is\n    equivalent to using scipy.ndimage.interpolation.map_coordinates with\n    trilinear interpolation at each vector component\n\n    Parameters\n    ----------\n    field : array, shape (S, R, C, 3)\n        the 3D vector field to be interpolated\n    locations : array, shape (n, 3)\n        (locations[i,0], locations[i,1], locations[i,2), 0<=i<n must contain\n        the coordinates to interpolate the vector field at\n\n    Returns\n    -------\n    out : array, shape (n, 3)\n        out[i,:], 0<=i<n will be the interpolated vector at coordinates\n        locations[i,:], or (0,0,0) if locations[i,:] is outside the field\n    inside : array, (n,)\n        if locations[i,:] is inside the vector field then inside[i]=1, else\n        inside[i]=0\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_13interpolate_vector_3d = {"interpolate_vector_3d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_13interpolate_vector_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_12interpolate_vector_3d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_13interpolate_vector_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_12interpolate_vector_3d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_12interpolate_vector_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("interpolate_vector_3d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_field, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_field); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 4) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 4) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__23, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__24, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__25, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_133interpolate_vector_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_133interpolate_vector_3d = {"__pyx_fuse_0interpolate_vector_3d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_133interpolate_vector_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_12interpolate_vector_3d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_133interpolate_vector_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_vector_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_field,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_field)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_vector_3d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_vector_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_field = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[0]); if (unlikely(!__pyx_v_field.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_vector_3d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_vector_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_132interpolate_vector_3d(__pyx_self, __pyx_v_field, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_132interpolate_vector_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  __Pyx_memviewslice __pyx_t_16 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0interpolate_vector_3d", 0);

  /* "dipy/align/vector_fields.pyx":592
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(field).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_field, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":594
 *     ftype = np.asarray(field).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         floating[:, :] out = np.zeros(shape=(n, 3), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":595
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         floating[:, :] out = np.zeros(shape=(n, 3), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_int_3);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":596
 *         cnp.npy_intp n = locations.shape[0]
 *         floating[:, :] out = np.zeros(shape=(n, 3), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":597
 *         floating[:, :] out = np.zeros(shape=(n, 3), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_vector_3d[floating](field,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":598
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_vector_3d[floating](field,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":600
 *         for i in range(n):
 *             inside[i] = _interpolate_vector_3d[floating](field,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;
          __pyx_t_15 = 2;
          __pyx_t_16.data = __pyx_v_out.data;
          __pyx_t_16.memview = __pyx_v_out.memview;
          __PYX_INC_MEMVIEW(&__pyx_t_16, 0);
          {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_out.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_out.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 600; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
    }
        __pyx_t_16.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_16.shape[0] = __pyx_v_out.shape[1];
__pyx_t_16.strides[0] = __pyx_v_out.strides[1];
    __pyx_t_16.suboffsets[0] = -1;

__pyx_t_17 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":599
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_vector_3d[floating](field,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], locations[i, 2], out[i])
 *     return out, inside
 */
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_17 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_field, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_14 * __pyx_v_locations.strides[0]) ) + __pyx_t_15 * __pyx_v_locations.strides[1]) ))), __pyx_t_16);
          __PYX_XDEC_MEMVIEW(&__pyx_t_16, 0);
        }
      }

      /* "dipy/align/vector_fields.pyx":597
 *         floating[:, :] out = np.zeros(shape=(n, 3), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_vector_3d[floating](field,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L4_error: {
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L1_error;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":601
 *             inside[i] = _interpolate_vector_3d[floating](field,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 601; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 601; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 601; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":567
 * 
 * 
 * def interpolate_vector_3d(floating[:, :, :, :] field, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Trilinear interpolation of a 3D vector field
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_16, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_vector_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_field, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_135interpolate_vector_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_135interpolate_vector_3d = {"__pyx_fuse_1interpolate_vector_3d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_135interpolate_vector_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_12interpolate_vector_3d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_135interpolate_vector_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_locations = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("interpolate_vector_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_field,&__pyx_n_s_locations,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_field)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_locations)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("interpolate_vector_3d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "interpolate_vector_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_field = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[0]); if (unlikely(!__pyx_v_field.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_locations = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_locations.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("interpolate_vector_3d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_vector_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_134interpolate_vector_3d(__pyx_self, __pyx_v_field, __pyx_v_locations);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_134interpolate_vector_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field, __Pyx_memviewslice __pyx_v_locations) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_n;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  __Pyx_memviewslice __pyx_t_16 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1interpolate_vector_3d", 0);

  /* "dipy/align/vector_fields.pyx":592
 *         inside[i]=0
 *     """
 *     ftype = np.asarray(field).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_field, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 592; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":594
 *     ftype = np.asarray(field).dtype
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]             # <<<<<<<<<<<<<<
 *         floating[:, :] out = np.zeros(shape=(n, 3), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_v_n = (__pyx_v_locations.shape[0]);

  /* "dipy/align/vector_fields.pyx":595
 *     cdef:
 *         cnp.npy_intp n = locations.shape[0]
 *         floating[:, :] out = np.zeros(shape=(n, 3), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_int_3);
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_t_2);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 595; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_out = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":596
 *         cnp.npy_intp n = locations.shape[0]
 *         floating[:, :] out = np.zeros(shape=(n, 3), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         for i in range(n):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_n); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(__pyx_t_1);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 596; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inside = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":597
 *         floating[:, :] out = np.zeros(shape=(n, 3), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_vector_3d[floating](field,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":598
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = _interpolate_vector_3d[floating](field,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], out[i])
 */
        __pyx_t_8 = __pyx_v_n;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":600
 *         for i in range(n):
 *             inside[i] = _interpolate_vector_3d[floating](field,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], out[i])             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
          __pyx_t_10 = __pyx_v_i;
          __pyx_t_11 = 0;
          __pyx_t_12 = __pyx_v_i;
          __pyx_t_13 = 1;
          __pyx_t_14 = __pyx_v_i;
          __pyx_t_15 = 2;
          __pyx_t_16.data = __pyx_v_out.data;
          __pyx_t_16.memview = __pyx_v_out.memview;
          __PYX_INC_MEMVIEW(&__pyx_t_16, 0);
          {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_out.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_out.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 600; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
    }
        __pyx_t_16.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_16.shape[0] = __pyx_v_out.shape[1];
__pyx_t_16.strides[0] = __pyx_v_out.strides[1];
    __pyx_t_16.suboffsets[0] = -1;

__pyx_t_17 = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":599
 *     with nogil:
 *         for i in range(n):
 *             inside[i] = _interpolate_vector_3d[floating](field,             # <<<<<<<<<<<<<<
 *                 locations[i, 0], locations[i, 1], locations[i, 2], out[i])
 *     return out, inside
 */
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_17 * __pyx_v_inside.strides[0]) )) = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_field, (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_10 * __pyx_v_locations.strides[0]) ) + __pyx_t_11 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_12 * __pyx_v_locations.strides[0]) ) + __pyx_t_13 * __pyx_v_locations.strides[1]) ))), (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_locations.data + __pyx_t_14 * __pyx_v_locations.strides[0]) ) + __pyx_t_15 * __pyx_v_locations.strides[1]) ))), __pyx_t_16);
          __PYX_XDEC_MEMVIEW(&__pyx_t_16, 0);
        }
      }

      /* "dipy/align/vector_fields.pyx":597
 *         floating[:, :] out = np.zeros(shape=(n, 3), dtype=ftype)
 *         int[:] inside = np.empty(shape=(n,), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = _interpolate_vector_3d[floating](field,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L4_error: {
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L1_error;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":601
 *             inside[i] = _interpolate_vector_3d[floating](field,
 *                 locations[i, 0], locations[i, 1], locations[i, 2], out[i])
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 601; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_inside, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 601; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 601; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":567
 * 
 * 
 * def interpolate_vector_3d(floating[:, :, :, :] field, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Trilinear interpolation of a 3D vector field
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_16, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.interpolate_vector_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_field, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_locations, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":604
 * 
 * 
 * cdef inline int _interpolate_vector_3d(floating[:, :, :, :] field, double dkk,             # <<<<<<<<<<<<<<
 *                                        double dii, double djj,
 *                                        floating[:] out) nogil:
 */

static CYTHON_INLINE int __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__Pyx_memviewslice __pyx_v_field, double __pyx_v_dkk, double __pyx_v_dii, double __pyx_v_djj, __Pyx_memviewslice __pyx_v_out) {
  npy_intp __pyx_v_ns;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_kk;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  int __pyx_v_inside;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_gamma;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  double __pyx_v_cgamma;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  Py_ssize_t __pyx_t_60;
  Py_ssize_t __pyx_t_61;
  Py_ssize_t __pyx_t_62;
  Py_ssize_t __pyx_t_63;
  Py_ssize_t __pyx_t_64;
  Py_ssize_t __pyx_t_65;
  Py_ssize_t __pyx_t_66;
  Py_ssize_t __pyx_t_67;
  Py_ssize_t __pyx_t_68;
  Py_ssize_t __pyx_t_69;
  Py_ssize_t __pyx_t_70;
  Py_ssize_t __pyx_t_71;
  Py_ssize_t __pyx_t_72;
  Py_ssize_t __pyx_t_73;
  Py_ssize_t __pyx_t_74;
  Py_ssize_t __pyx_t_75;
  Py_ssize_t __pyx_t_76;
  Py_ssize_t __pyx_t_77;
  Py_ssize_t __pyx_t_78;
  Py_ssize_t __pyx_t_79;
  Py_ssize_t __pyx_t_80;
  Py_ssize_t __pyx_t_81;
  Py_ssize_t __pyx_t_82;
  Py_ssize_t __pyx_t_83;
  Py_ssize_t __pyx_t_84;
  Py_ssize_t __pyx_t_85;
  Py_ssize_t __pyx_t_86;
  Py_ssize_t __pyx_t_87;
  Py_ssize_t __pyx_t_88;
  Py_ssize_t __pyx_t_89;
  Py_ssize_t __pyx_t_90;
  Py_ssize_t __pyx_t_91;
  Py_ssize_t __pyx_t_92;
  Py_ssize_t __pyx_t_93;
  Py_ssize_t __pyx_t_94;
  Py_ssize_t __pyx_t_95;
  Py_ssize_t __pyx_t_96;
  Py_ssize_t __pyx_t_97;
  Py_ssize_t __pyx_t_98;
  Py_ssize_t __pyx_t_99;
  Py_ssize_t __pyx_t_100;
  Py_ssize_t __pyx_t_101;
  Py_ssize_t __pyx_t_102;
  Py_ssize_t __pyx_t_103;
  Py_ssize_t __pyx_t_104;
  Py_ssize_t __pyx_t_105;
  Py_ssize_t __pyx_t_106;
  Py_ssize_t __pyx_t_107;
  Py_ssize_t __pyx_t_108;
  Py_ssize_t __pyx_t_109;
  Py_ssize_t __pyx_t_110;
  Py_ssize_t __pyx_t_111;
  Py_ssize_t __pyx_t_112;
  Py_ssize_t __pyx_t_113;
  Py_ssize_t __pyx_t_114;
  Py_ssize_t __pyx_t_115;
  Py_ssize_t __pyx_t_116;
  Py_ssize_t __pyx_t_117;
  Py_ssize_t __pyx_t_118;
  Py_ssize_t __pyx_t_119;
  Py_ssize_t __pyx_t_120;
  Py_ssize_t __pyx_t_121;
  Py_ssize_t __pyx_t_122;
  Py_ssize_t __pyx_t_123;
  Py_ssize_t __pyx_t_124;
  Py_ssize_t __pyx_t_125;
  Py_ssize_t __pyx_t_126;
  Py_ssize_t __pyx_t_127;
  Py_ssize_t __pyx_t_128;
  Py_ssize_t __pyx_t_129;
  int __pyx_t_130;

  /* "dipy/align/vector_fields.pyx":633
 *     """
 *     cdef:
 *         cnp.npy_intp ns = field.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr = field.shape[1]
 *         cnp.npy_intp nc = field.shape[2]
 */
  __pyx_v_ns = (__pyx_v_field.shape[0]);

  /* "dipy/align/vector_fields.pyx":634
 *     cdef:
 *         cnp.npy_intp ns = field.shape[0]
 *         cnp.npy_intp nr = field.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = field.shape[2]
 *         cnp.npy_intp kk, ii, jj
 */
  __pyx_v_nr = (__pyx_v_field.shape[1]);

  /* "dipy/align/vector_fields.pyx":635
 *         cnp.npy_intp ns = field.shape[0]
 *         cnp.npy_intp nr = field.shape[1]
 *         cnp.npy_intp nc = field.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp kk, ii, jj
 *         int inside
 */
  __pyx_v_nc = (__pyx_v_field.shape[2]);

  /* "dipy/align/vector_fields.pyx":639
 *         int inside
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *     if not (-1 < dkk < ns and -1 < dii < nr and -1 < djj < nc):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         out[1] = 0
 */
  __pyx_t_2 = (-1.0 < __pyx_v_dkk);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_dkk < __pyx_v_ns);
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (-1.0 < __pyx_v_dii);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_dii < __pyx_v_nr);
  }
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (-1.0 < __pyx_v_djj);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_djj < __pyx_v_nc);
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  __pyx_t_3 = ((!__pyx_t_1) != 0);
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":640
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *     if not (-1 < dkk < ns and -1 < dii < nr and -1 < djj < nc):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         out[1] = 0
 *         out[2] = 0
 */
    __pyx_t_4 = 0;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_4 * __pyx_v_out.strides[0]) )) = 0.0;

    /* "dipy/align/vector_fields.pyx":641
 *     if not (-1 < dkk < ns and -1 < dii < nr and -1 < djj < nc):
 *         out[0] = 0
 *         out[1] = 0             # <<<<<<<<<<<<<<
 *         out[2] = 0
 *         return 0
 */
    __pyx_t_5 = 1;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_5 * __pyx_v_out.strides[0]) )) = 0.0;

    /* "dipy/align/vector_fields.pyx":642
 *         out[0] = 0
 *         out[1] = 0
 *         out[2] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     #---top-left
 */
    __pyx_t_6 = 2;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_6 * __pyx_v_out.strides[0]) )) = 0.0;

    /* "dipy/align/vector_fields.pyx":643
 *         out[1] = 0
 *         out[2] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     #---top-left
 *     kk = <int>floor(dkk)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":639
 *         int inside
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *     if not (-1 < dkk < ns and -1 < dii < nr and -1 < djj < nc):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         out[1] = 0
 */
  }

  /* "dipy/align/vector_fields.pyx":645
 *         return 0
 *     #---top-left
 *     kk = <int>floor(dkk)             # <<<<<<<<<<<<<<
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)
 */
  __pyx_v_kk = ((int)floor(__pyx_v_dkk));

  /* "dipy/align/vector_fields.pyx":646
 *     #---top-left
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 * 
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":647
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 * 
 *     cgamma = dkk - kk
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":649
 *     jj = <int>floor(djj)
 * 
 *     cgamma = dkk - kk             # <<<<<<<<<<<<<<
 *     calpha = dii - ii
 *     cbeta = djj - jj
 */
  __pyx_v_cgamma = (__pyx_v_dkk - __pyx_v_kk);

  /* "dipy/align/vector_fields.pyx":650
 * 
 *     cgamma = dkk - kk
 *     calpha = dii - ii             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":651
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":652
 *     calpha = dii - ii
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":653
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     gamma = 1 - cgamma
 * 
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":654
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma             # <<<<<<<<<<<<<<
 * 
 *     inside = 0
 */
  __pyx_v_gamma = (1.0 - __pyx_v_cgamma);

  /* "dipy/align/vector_fields.pyx":656
 *     gamma = 1 - cgamma
 * 
 *     inside = 0             # <<<<<<<<<<<<<<
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):
 *         out[0] = alpha * beta * gamma * field[kk, ii, jj, 0]
 */
  __pyx_v_inside = 0;

  /* "dipy/align/vector_fields.pyx":657
 * 
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] = alpha * beta * gamma * field[kk, ii, jj, 0]
 *         out[1] = alpha * beta * gamma * field[kk, ii, jj, 1]
 */
  __pyx_t_1 = ((__pyx_v_ii >= 0) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_jj >= 0) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_kk >= 0) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L8_bool_binop_done:;
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":658
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):
 *         out[0] = alpha * beta * gamma * field[kk, ii, jj, 0]             # <<<<<<<<<<<<<<
 *         out[1] = alpha * beta * gamma * field[kk, ii, jj, 1]
 *         out[2] = alpha * beta * gamma * field[kk, ii, jj, 2]
 */
    __pyx_t_7 = __pyx_v_kk;
    __pyx_t_8 = __pyx_v_ii;
    __pyx_t_9 = __pyx_v_jj;
    __pyx_t_10 = 0;
    __pyx_t_11 = 0;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_11 * __pyx_v_out.strides[0]) )) = (((__pyx_v_alpha * __pyx_v_beta) * __pyx_v_gamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_7 * __pyx_v_field.strides[0]) ) + __pyx_t_8 * __pyx_v_field.strides[1]) ) + __pyx_t_9 * __pyx_v_field.strides[2]) ) + __pyx_t_10 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":659
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):
 *         out[0] = alpha * beta * gamma * field[kk, ii, jj, 0]
 *         out[1] = alpha * beta * gamma * field[kk, ii, jj, 1]             # <<<<<<<<<<<<<<
 *         out[2] = alpha * beta * gamma * field[kk, ii, jj, 2]
 *         inside += 1
 */
    __pyx_t_12 = __pyx_v_kk;
    __pyx_t_13 = __pyx_v_ii;
    __pyx_t_14 = __pyx_v_jj;
    __pyx_t_15 = 1;
    __pyx_t_16 = 1;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_16 * __pyx_v_out.strides[0]) )) = (((__pyx_v_alpha * __pyx_v_beta) * __pyx_v_gamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_12 * __pyx_v_field.strides[0]) ) + __pyx_t_13 * __pyx_v_field.strides[1]) ) + __pyx_t_14 * __pyx_v_field.strides[2]) ) + __pyx_t_15 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":660
 *         out[0] = alpha * beta * gamma * field[kk, ii, jj, 0]
 *         out[1] = alpha * beta * gamma * field[kk, ii, jj, 1]
 *         out[2] = alpha * beta * gamma * field[kk, ii, jj, 2]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     else:
 */
    __pyx_t_17 = __pyx_v_kk;
    __pyx_t_18 = __pyx_v_ii;
    __pyx_t_19 = __pyx_v_jj;
    __pyx_t_20 = 2;
    __pyx_t_21 = 2;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_21 * __pyx_v_out.strides[0]) )) = (((__pyx_v_alpha * __pyx_v_beta) * __pyx_v_gamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_17 * __pyx_v_field.strides[0]) ) + __pyx_t_18 * __pyx_v_field.strides[1]) ) + __pyx_t_19 * __pyx_v_field.strides[2]) ) + __pyx_t_20 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":661
 *         out[1] = alpha * beta * gamma * field[kk, ii, jj, 1]
 *         out[2] = alpha * beta * gamma * field[kk, ii, jj, 2]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     else:
 *         out[0] = 0
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":657
 * 
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] = alpha * beta * gamma * field[kk, ii, jj, 0]
 *         out[1] = alpha * beta * gamma * field[kk, ii, jj, 1]
 */
    goto __pyx_L7;
  }

  /* "dipy/align/vector_fields.pyx":663
 *         inside += 1
 *     else:
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         out[1] = 0
 *         out[2] = 0
 */
  /*else*/ {
    __pyx_t_22 = 0;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_22 * __pyx_v_out.strides[0]) )) = 0.0;

    /* "dipy/align/vector_fields.pyx":664
 *     else:
 *         out[0] = 0
 *         out[1] = 0             # <<<<<<<<<<<<<<
 *         out[2] = 0
 *     # ---top-right
 */
    __pyx_t_23 = 1;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_23 * __pyx_v_out.strides[0]) )) = 0.0;

    /* "dipy/align/vector_fields.pyx":665
 *         out[0] = 0
 *         out[1] = 0
 *         out[2] = 0             # <<<<<<<<<<<<<<
 *     # ---top-right
 *     jj += 1
 */
    __pyx_t_24 = 2;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_24 * __pyx_v_out.strides[0]) )) = 0.0;
  }
  __pyx_L7:;

  /* "dipy/align/vector_fields.pyx":667
 *         out[2] = 0
 *     # ---top-right
 *     jj += 1             # <<<<<<<<<<<<<<
 *     if (jj < nc) and (ii >= 0) and (kk >= 0):
 *         out[0] += alpha * cbeta * gamma * field[kk, ii, jj, 0]
 */
  __pyx_v_jj = (__pyx_v_jj + 1);

  /* "dipy/align/vector_fields.pyx":668
 *     # ---top-right
 *     jj += 1
 *     if (jj < nc) and (ii >= 0) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += alpha * cbeta * gamma * field[kk, ii, jj, 0]
 *         out[1] += alpha * cbeta * gamma * field[kk, ii, jj, 1]
 */
  __pyx_t_1 = ((__pyx_v_jj < __pyx_v_nc) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_ii >= 0) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_kk >= 0) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L12_bool_binop_done:;
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":669
 *     jj += 1
 *     if (jj < nc) and (ii >= 0) and (kk >= 0):
 *         out[0] += alpha * cbeta * gamma * field[kk, ii, jj, 0]             # <<<<<<<<<<<<<<
 *         out[1] += alpha * cbeta * gamma * field[kk, ii, jj, 1]
 *         out[2] += alpha * cbeta * gamma * field[kk, ii, jj, 2]
 */
    __pyx_t_25 = __pyx_v_kk;
    __pyx_t_26 = __pyx_v_ii;
    __pyx_t_27 = __pyx_v_jj;
    __pyx_t_28 = 0;
    __pyx_t_29 = 0;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_29 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_cbeta) * __pyx_v_gamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_25 * __pyx_v_field.strides[0]) ) + __pyx_t_26 * __pyx_v_field.strides[1]) ) + __pyx_t_27 * __pyx_v_field.strides[2]) ) + __pyx_t_28 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":670
 *     if (jj < nc) and (ii >= 0) and (kk >= 0):
 *         out[0] += alpha * cbeta * gamma * field[kk, ii, jj, 0]
 *         out[1] += alpha * cbeta * gamma * field[kk, ii, jj, 1]             # <<<<<<<<<<<<<<
 *         out[2] += alpha * cbeta * gamma * field[kk, ii, jj, 2]
 *         inside += 1
 */
    __pyx_t_30 = __pyx_v_kk;
    __pyx_t_31 = __pyx_v_ii;
    __pyx_t_32 = __pyx_v_jj;
    __pyx_t_33 = 1;
    __pyx_t_34 = 1;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_34 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_cbeta) * __pyx_v_gamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_30 * __pyx_v_field.strides[0]) ) + __pyx_t_31 * __pyx_v_field.strides[1]) ) + __pyx_t_32 * __pyx_v_field.strides[2]) ) + __pyx_t_33 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":671
 *         out[0] += alpha * cbeta * gamma * field[kk, ii, jj, 0]
 *         out[1] += alpha * cbeta * gamma * field[kk, ii, jj, 1]
 *         out[2] += alpha * cbeta * gamma * field[kk, ii, jj, 2]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     # ---bottom-right
 */
    __pyx_t_35 = __pyx_v_kk;
    __pyx_t_36 = __pyx_v_ii;
    __pyx_t_37 = __pyx_v_jj;
    __pyx_t_38 = 2;
    __pyx_t_39 = 2;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_39 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_cbeta) * __pyx_v_gamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_35 * __pyx_v_field.strides[0]) ) + __pyx_t_36 * __pyx_v_field.strides[1]) ) + __pyx_t_37 * __pyx_v_field.strides[2]) ) + __pyx_t_38 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":672
 *         out[1] += alpha * cbeta * gamma * field[kk, ii, jj, 1]
 *         out[2] += alpha * cbeta * gamma * field[kk, ii, jj, 2]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     # ---bottom-right
 *     ii += 1
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":668
 *     # ---top-right
 *     jj += 1
 *     if (jj < nc) and (ii >= 0) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += alpha * cbeta * gamma * field[kk, ii, jj, 0]
 *         out[1] += alpha * cbeta * gamma * field[kk, ii, jj, 1]
 */
  }

  /* "dipy/align/vector_fields.pyx":674
 *         inside += 1
 *     # ---bottom-right
 *     ii += 1             # <<<<<<<<<<<<<<
 *     if (jj < nc) and (ii < nr) and (kk >= 0):
 *         out[0] += calpha * cbeta * gamma * field[kk, ii, jj, 0]
 */
  __pyx_v_ii = (__pyx_v_ii + 1);

  /* "dipy/align/vector_fields.pyx":675
 *     # ---bottom-right
 *     ii += 1
 *     if (jj < nc) and (ii < nr) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * cbeta * gamma * field[kk, ii, jj, 0]
 *         out[1] += calpha * cbeta * gamma * field[kk, ii, jj, 1]
 */
  __pyx_t_1 = ((__pyx_v_jj < __pyx_v_nc) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_ii < __pyx_v_nr) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_kk >= 0) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L16_bool_binop_done:;
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":676
 *     ii += 1
 *     if (jj < nc) and (ii < nr) and (kk >= 0):
 *         out[0] += calpha * cbeta * gamma * field[kk, ii, jj, 0]             # <<<<<<<<<<<<<<
 *         out[1] += calpha * cbeta * gamma * field[kk, ii, jj, 1]
 *         out[2] += calpha * cbeta * gamma * field[kk, ii, jj, 2]
 */
    __pyx_t_40 = __pyx_v_kk;
    __pyx_t_41 = __pyx_v_ii;
    __pyx_t_42 = __pyx_v_jj;
    __pyx_t_43 = 0;
    __pyx_t_44 = 0;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_44 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_cbeta) * __pyx_v_gamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_40 * __pyx_v_field.strides[0]) ) + __pyx_t_41 * __pyx_v_field.strides[1]) ) + __pyx_t_42 * __pyx_v_field.strides[2]) ) + __pyx_t_43 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":677
 *     if (jj < nc) and (ii < nr) and (kk >= 0):
 *         out[0] += calpha * cbeta * gamma * field[kk, ii, jj, 0]
 *         out[1] += calpha * cbeta * gamma * field[kk, ii, jj, 1]             # <<<<<<<<<<<<<<
 *         out[2] += calpha * cbeta * gamma * field[kk, ii, jj, 2]
 *         inside += 1
 */
    __pyx_t_45 = __pyx_v_kk;
    __pyx_t_46 = __pyx_v_ii;
    __pyx_t_47 = __pyx_v_jj;
    __pyx_t_48 = 1;
    __pyx_t_49 = 1;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_49 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_cbeta) * __pyx_v_gamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_45 * __pyx_v_field.strides[0]) ) + __pyx_t_46 * __pyx_v_field.strides[1]) ) + __pyx_t_47 * __pyx_v_field.strides[2]) ) + __pyx_t_48 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":678
 *         out[0] += calpha * cbeta * gamma * field[kk, ii, jj, 0]
 *         out[1] += calpha * cbeta * gamma * field[kk, ii, jj, 1]
 *         out[2] += calpha * cbeta * gamma * field[kk, ii, jj, 2]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     # ---bottom-left
 */
    __pyx_t_50 = __pyx_v_kk;
    __pyx_t_51 = __pyx_v_ii;
    __pyx_t_52 = __pyx_v_jj;
    __pyx_t_53 = 2;
    __pyx_t_54 = 2;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_54 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_cbeta) * __pyx_v_gamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_50 * __pyx_v_field.strides[0]) ) + __pyx_t_51 * __pyx_v_field.strides[1]) ) + __pyx_t_52 * __pyx_v_field.strides[2]) ) + __pyx_t_53 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":679
 *         out[1] += calpha * cbeta * gamma * field[kk, ii, jj, 1]
 *         out[2] += calpha * cbeta * gamma * field[kk, ii, jj, 2]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     # ---bottom-left
 *     jj -= 1
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":675
 *     # ---bottom-right
 *     ii += 1
 *     if (jj < nc) and (ii < nr) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * cbeta * gamma * field[kk, ii, jj, 0]
 *         out[1] += calpha * cbeta * gamma * field[kk, ii, jj, 1]
 */
  }

  /* "dipy/align/vector_fields.pyx":681
 *         inside += 1
 *     # ---bottom-left
 *     jj -= 1             # <<<<<<<<<<<<<<
 *     if (jj >= 0) and (ii < nr) and (kk >= 0):
 *         out[0] += calpha * beta * gamma * field[kk, ii, jj, 0]
 */
  __pyx_v_jj = (__pyx_v_jj - 1);

  /* "dipy/align/vector_fields.pyx":682
 *     # ---bottom-left
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * beta * gamma * field[kk, ii, jj, 0]
 *         out[1] += calpha * beta * gamma * field[kk, ii, jj, 1]
 */
  __pyx_t_1 = ((__pyx_v_jj >= 0) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L20_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_ii < __pyx_v_nr) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L20_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_kk >= 0) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L20_bool_binop_done:;
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":683
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr) and (kk >= 0):
 *         out[0] += calpha * beta * gamma * field[kk, ii, jj, 0]             # <<<<<<<<<<<<<<
 *         out[1] += calpha * beta * gamma * field[kk, ii, jj, 1]
 *         out[2] += calpha * beta * gamma * field[kk, ii, jj, 2]
 */
    __pyx_t_55 = __pyx_v_kk;
    __pyx_t_56 = __pyx_v_ii;
    __pyx_t_57 = __pyx_v_jj;
    __pyx_t_58 = 0;
    __pyx_t_59 = 0;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_59 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_beta) * __pyx_v_gamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_55 * __pyx_v_field.strides[0]) ) + __pyx_t_56 * __pyx_v_field.strides[1]) ) + __pyx_t_57 * __pyx_v_field.strides[2]) ) + __pyx_t_58 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":684
 *     if (jj >= 0) and (ii < nr) and (kk >= 0):
 *         out[0] += calpha * beta * gamma * field[kk, ii, jj, 0]
 *         out[1] += calpha * beta * gamma * field[kk, ii, jj, 1]             # <<<<<<<<<<<<<<
 *         out[2] += calpha * beta * gamma * field[kk, ii, jj, 2]
 *         inside += 1
 */
    __pyx_t_60 = __pyx_v_kk;
    __pyx_t_61 = __pyx_v_ii;
    __pyx_t_62 = __pyx_v_jj;
    __pyx_t_63 = 1;
    __pyx_t_64 = 1;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_64 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_beta) * __pyx_v_gamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_60 * __pyx_v_field.strides[0]) ) + __pyx_t_61 * __pyx_v_field.strides[1]) ) + __pyx_t_62 * __pyx_v_field.strides[2]) ) + __pyx_t_63 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":685
 *         out[0] += calpha * beta * gamma * field[kk, ii, jj, 0]
 *         out[1] += calpha * beta * gamma * field[kk, ii, jj, 1]
 *         out[2] += calpha * beta * gamma * field[kk, ii, jj, 2]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     kk += 1
 */
    __pyx_t_65 = __pyx_v_kk;
    __pyx_t_66 = __pyx_v_ii;
    __pyx_t_67 = __pyx_v_jj;
    __pyx_t_68 = 2;
    __pyx_t_69 = 2;
    *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_69 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_beta) * __pyx_v_gamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_65 * __pyx_v_field.strides[0]) ) + __pyx_t_66 * __pyx_v_field.strides[1]) ) + __pyx_t_67 * __pyx_v_field.strides[2]) ) + __pyx_t_68 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":686
 *         out[1] += calpha * beta * gamma * field[kk, ii, jj, 1]
 *         out[2] += calpha * beta * gamma * field[kk, ii, jj, 2]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     kk += 1
 *     if (kk < ns):
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":682
 *     # ---bottom-left
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * beta * gamma * field[kk, ii, jj, 0]
 *         out[1] += calpha * beta * gamma * field[kk, ii, jj, 1]
 */
  }

  /* "dipy/align/vector_fields.pyx":687
 *         out[2] += calpha * beta * gamma * field[kk, ii, jj, 2]
 *         inside += 1
 *     kk += 1             # <<<<<<<<<<<<<<
 *     if (kk < ns):
 *         ii -= 1
 */
  __pyx_v_kk = (__pyx_v_kk + 1);

  /* "dipy/align/vector_fields.pyx":688
 *         inside += 1
 *     kk += 1
 *     if (kk < ns):             # <<<<<<<<<<<<<<
 *         ii -= 1
 *         if (jj >= 0) and (ii >= 0):
 */
  __pyx_t_3 = ((__pyx_v_kk < __pyx_v_ns) != 0);
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":689
 *     kk += 1
 *     if (kk < ns):
 *         ii -= 1             # <<<<<<<<<<<<<<
 *         if (jj >= 0) and (ii >= 0):
 *             out[0] += alpha * beta * cgamma * field[kk, ii, jj, 0]
 */
    __pyx_v_ii = (__pyx_v_ii - 1);

    /* "dipy/align/vector_fields.pyx":690
 *     if (kk < ns):
 *         ii -= 1
 *         if (jj >= 0) and (ii >= 0):             # <<<<<<<<<<<<<<
 *             out[0] += alpha * beta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += alpha * beta * cgamma * field[kk, ii, jj, 1]
 */
    __pyx_t_1 = ((__pyx_v_jj >= 0) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_3 = __pyx_t_1;
      goto __pyx_L25_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_ii >= 0) != 0);
    __pyx_t_3 = __pyx_t_1;
    __pyx_L25_bool_binop_done:;
    if (__pyx_t_3) {

      /* "dipy/align/vector_fields.pyx":691
 *         ii -= 1
 *         if (jj >= 0) and (ii >= 0):
 *             out[0] += alpha * beta * cgamma * field[kk, ii, jj, 0]             # <<<<<<<<<<<<<<
 *             out[1] += alpha * beta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += alpha * beta * cgamma * field[kk, ii, jj, 2]
 */
      __pyx_t_70 = __pyx_v_kk;
      __pyx_t_71 = __pyx_v_ii;
      __pyx_t_72 = __pyx_v_jj;
      __pyx_t_73 = 0;
      __pyx_t_74 = 0;
      *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_74 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_beta) * __pyx_v_cgamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_70 * __pyx_v_field.strides[0]) ) + __pyx_t_71 * __pyx_v_field.strides[1]) ) + __pyx_t_72 * __pyx_v_field.strides[2]) ) + __pyx_t_73 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":692
 *         if (jj >= 0) and (ii >= 0):
 *             out[0] += alpha * beta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += alpha * beta * cgamma * field[kk, ii, jj, 1]             # <<<<<<<<<<<<<<
 *             out[2] += alpha * beta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1
 */
      __pyx_t_75 = __pyx_v_kk;
      __pyx_t_76 = __pyx_v_ii;
      __pyx_t_77 = __pyx_v_jj;
      __pyx_t_78 = 1;
      __pyx_t_79 = 1;
      *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_79 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_beta) * __pyx_v_cgamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_75 * __pyx_v_field.strides[0]) ) + __pyx_t_76 * __pyx_v_field.strides[1]) ) + __pyx_t_77 * __pyx_v_field.strides[2]) ) + __pyx_t_78 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":693
 *             out[0] += alpha * beta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += alpha * beta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += alpha * beta * cgamma * field[kk, ii, jj, 2]             # <<<<<<<<<<<<<<
 *             inside += 1
 *         jj += 1
 */
      __pyx_t_80 = __pyx_v_kk;
      __pyx_t_81 = __pyx_v_ii;
      __pyx_t_82 = __pyx_v_jj;
      __pyx_t_83 = 2;
      __pyx_t_84 = 2;
      *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_84 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_beta) * __pyx_v_cgamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_80 * __pyx_v_field.strides[0]) ) + __pyx_t_81 * __pyx_v_field.strides[1]) ) + __pyx_t_82 * __pyx_v_field.strides[2]) ) + __pyx_t_83 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":694
 *             out[1] += alpha * beta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += alpha * beta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1             # <<<<<<<<<<<<<<
 *         jj += 1
 *         if (jj < nc) and (ii >= 0):
 */
      __pyx_v_inside = (__pyx_v_inside + 1);

      /* "dipy/align/vector_fields.pyx":690
 *     if (kk < ns):
 *         ii -= 1
 *         if (jj >= 0) and (ii >= 0):             # <<<<<<<<<<<<<<
 *             out[0] += alpha * beta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += alpha * beta * cgamma * field[kk, ii, jj, 1]
 */
    }

    /* "dipy/align/vector_fields.pyx":695
 *             out[2] += alpha * beta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1
 *         jj += 1             # <<<<<<<<<<<<<<
 *         if (jj < nc) and (ii >= 0):
 *             out[0] += alpha * cbeta * cgamma * field[kk, ii, jj, 0]
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":696
 *             inside += 1
 *         jj += 1
 *         if (jj < nc) and (ii >= 0):             # <<<<<<<<<<<<<<
 *             out[0] += alpha * cbeta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += alpha * cbeta * cgamma * field[kk, ii, jj, 1]
 */
    __pyx_t_1 = ((__pyx_v_jj < __pyx_v_nc) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_3 = __pyx_t_1;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_ii >= 0) != 0);
    __pyx_t_3 = __pyx_t_1;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_3) {

      /* "dipy/align/vector_fields.pyx":697
 *         jj += 1
 *         if (jj < nc) and (ii >= 0):
 *             out[0] += alpha * cbeta * cgamma * field[kk, ii, jj, 0]             # <<<<<<<<<<<<<<
 *             out[1] += alpha * cbeta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += alpha * cbeta * cgamma * field[kk, ii, jj, 2]
 */
      __pyx_t_85 = __pyx_v_kk;
      __pyx_t_86 = __pyx_v_ii;
      __pyx_t_87 = __pyx_v_jj;
      __pyx_t_88 = 0;
      __pyx_t_89 = 0;
      *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_89 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_cbeta) * __pyx_v_cgamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_85 * __pyx_v_field.strides[0]) ) + __pyx_t_86 * __pyx_v_field.strides[1]) ) + __pyx_t_87 * __pyx_v_field.strides[2]) ) + __pyx_t_88 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":698
 *         if (jj < nc) and (ii >= 0):
 *             out[0] += alpha * cbeta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += alpha * cbeta * cgamma * field[kk, ii, jj, 1]             # <<<<<<<<<<<<<<
 *             out[2] += alpha * cbeta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1
 */
      __pyx_t_90 = __pyx_v_kk;
      __pyx_t_91 = __pyx_v_ii;
      __pyx_t_92 = __pyx_v_jj;
      __pyx_t_93 = 1;
      __pyx_t_94 = 1;
      *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_94 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_cbeta) * __pyx_v_cgamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_90 * __pyx_v_field.strides[0]) ) + __pyx_t_91 * __pyx_v_field.strides[1]) ) + __pyx_t_92 * __pyx_v_field.strides[2]) ) + __pyx_t_93 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":699
 *             out[0] += alpha * cbeta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += alpha * cbeta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += alpha * cbeta * cgamma * field[kk, ii, jj, 2]             # <<<<<<<<<<<<<<
 *             inside += 1
 *         # ---bottom-right
 */
      __pyx_t_95 = __pyx_v_kk;
      __pyx_t_96 = __pyx_v_ii;
      __pyx_t_97 = __pyx_v_jj;
      __pyx_t_98 = 2;
      __pyx_t_99 = 2;
      *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_99 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_cbeta) * __pyx_v_cgamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_95 * __pyx_v_field.strides[0]) ) + __pyx_t_96 * __pyx_v_field.strides[1]) ) + __pyx_t_97 * __pyx_v_field.strides[2]) ) + __pyx_t_98 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":700
 *             out[1] += alpha * cbeta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += alpha * cbeta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1             # <<<<<<<<<<<<<<
 *         # ---bottom-right
 *         ii += 1
 */
      __pyx_v_inside = (__pyx_v_inside + 1);

      /* "dipy/align/vector_fields.pyx":696
 *             inside += 1
 *         jj += 1
 *         if (jj < nc) and (ii >= 0):             # <<<<<<<<<<<<<<
 *             out[0] += alpha * cbeta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += alpha * cbeta * cgamma * field[kk, ii, jj, 1]
 */
    }

    /* "dipy/align/vector_fields.pyx":702
 *             inside += 1
 *         # ---bottom-right
 *         ii += 1             # <<<<<<<<<<<<<<
 *         if (jj < nc) and (ii < nr):
 *             out[0] += calpha * cbeta * cgamma * field[kk, ii, jj, 0]
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":703
 *         # ---bottom-right
 *         ii += 1
 *         if (jj < nc) and (ii < nr):             # <<<<<<<<<<<<<<
 *             out[0] += calpha * cbeta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += calpha * cbeta * cgamma * field[kk, ii, jj, 1]
 */
    __pyx_t_1 = ((__pyx_v_jj < __pyx_v_nc) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_3 = __pyx_t_1;
      goto __pyx_L31_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_ii < __pyx_v_nr) != 0);
    __pyx_t_3 = __pyx_t_1;
    __pyx_L31_bool_binop_done:;
    if (__pyx_t_3) {

      /* "dipy/align/vector_fields.pyx":704
 *         ii += 1
 *         if (jj < nc) and (ii < nr):
 *             out[0] += calpha * cbeta * cgamma * field[kk, ii, jj, 0]             # <<<<<<<<<<<<<<
 *             out[1] += calpha * cbeta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += calpha * cbeta * cgamma * field[kk, ii, jj, 2]
 */
      __pyx_t_100 = __pyx_v_kk;
      __pyx_t_101 = __pyx_v_ii;
      __pyx_t_102 = __pyx_v_jj;
      __pyx_t_103 = 0;
      __pyx_t_104 = 0;
      *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_104 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_cbeta) * __pyx_v_cgamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_100 * __pyx_v_field.strides[0]) ) + __pyx_t_101 * __pyx_v_field.strides[1]) ) + __pyx_t_102 * __pyx_v_field.strides[2]) ) + __pyx_t_103 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":705
 *         if (jj < nc) and (ii < nr):
 *             out[0] += calpha * cbeta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += calpha * cbeta * cgamma * field[kk, ii, jj, 1]             # <<<<<<<<<<<<<<
 *             out[2] += calpha * cbeta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1
 */
      __pyx_t_105 = __pyx_v_kk;
      __pyx_t_106 = __pyx_v_ii;
      __pyx_t_107 = __pyx_v_jj;
      __pyx_t_108 = 1;
      __pyx_t_109 = 1;
      *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_109 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_cbeta) * __pyx_v_cgamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_105 * __pyx_v_field.strides[0]) ) + __pyx_t_106 * __pyx_v_field.strides[1]) ) + __pyx_t_107 * __pyx_v_field.strides[2]) ) + __pyx_t_108 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":706
 *             out[0] += calpha * cbeta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += calpha * cbeta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += calpha * cbeta * cgamma * field[kk, ii, jj, 2]             # <<<<<<<<<<<<<<
 *             inside += 1
 *         # ---bottom-left
 */
      __pyx_t_110 = __pyx_v_kk;
      __pyx_t_111 = __pyx_v_ii;
      __pyx_t_112 = __pyx_v_jj;
      __pyx_t_113 = 2;
      __pyx_t_114 = 2;
      *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_114 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_cbeta) * __pyx_v_cgamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_110 * __pyx_v_field.strides[0]) ) + __pyx_t_111 * __pyx_v_field.strides[1]) ) + __pyx_t_112 * __pyx_v_field.strides[2]) ) + __pyx_t_113 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":707
 *             out[1] += calpha * cbeta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += calpha * cbeta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1             # <<<<<<<<<<<<<<
 *         # ---bottom-left
 *         jj -= 1
 */
      __pyx_v_inside = (__pyx_v_inside + 1);

      /* "dipy/align/vector_fields.pyx":703
 *         # ---bottom-right
 *         ii += 1
 *         if (jj < nc) and (ii < nr):             # <<<<<<<<<<<<<<
 *             out[0] += calpha * cbeta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += calpha * cbeta * cgamma * field[kk, ii, jj, 1]
 */
    }

    /* "dipy/align/vector_fields.pyx":709
 *             inside += 1
 *         # ---bottom-left
 *         jj -= 1             # <<<<<<<<<<<<<<
 *         if (jj >= 0) and (ii < nr):
 *             out[0] += calpha * beta * cgamma * field[kk, ii, jj, 0]
 */
    __pyx_v_jj = (__pyx_v_jj - 1);

    /* "dipy/align/vector_fields.pyx":710
 *         # ---bottom-left
 *         jj -= 1
 *         if (jj >= 0) and (ii < nr):             # <<<<<<<<<<<<<<
 *             out[0] += calpha * beta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += calpha * beta * cgamma * field[kk, ii, jj, 1]
 */
    __pyx_t_1 = ((__pyx_v_jj >= 0) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_3 = __pyx_t_1;
      goto __pyx_L34_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_ii < __pyx_v_nr) != 0);
    __pyx_t_3 = __pyx_t_1;
    __pyx_L34_bool_binop_done:;
    if (__pyx_t_3) {

      /* "dipy/align/vector_fields.pyx":711
 *         jj -= 1
 *         if (jj >= 0) and (ii < nr):
 *             out[0] += calpha * beta * cgamma * field[kk, ii, jj, 0]             # <<<<<<<<<<<<<<
 *             out[1] += calpha * beta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += calpha * beta * cgamma * field[kk, ii, jj, 2]
 */
      __pyx_t_115 = __pyx_v_kk;
      __pyx_t_116 = __pyx_v_ii;
      __pyx_t_117 = __pyx_v_jj;
      __pyx_t_118 = 0;
      __pyx_t_119 = 0;
      *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_119 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_beta) * __pyx_v_cgamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_115 * __pyx_v_field.strides[0]) ) + __pyx_t_116 * __pyx_v_field.strides[1]) ) + __pyx_t_117 * __pyx_v_field.strides[2]) ) + __pyx_t_118 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":712
 *         if (jj >= 0) and (ii < nr):
 *             out[0] += calpha * beta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += calpha * beta * cgamma * field[kk, ii, jj, 1]             # <<<<<<<<<<<<<<
 *             out[2] += calpha * beta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1
 */
      __pyx_t_120 = __pyx_v_kk;
      __pyx_t_121 = __pyx_v_ii;
      __pyx_t_122 = __pyx_v_jj;
      __pyx_t_123 = 1;
      __pyx_t_124 = 1;
      *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_124 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_beta) * __pyx_v_cgamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_120 * __pyx_v_field.strides[0]) ) + __pyx_t_121 * __pyx_v_field.strides[1]) ) + __pyx_t_122 * __pyx_v_field.strides[2]) ) + __pyx_t_123 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":713
 *             out[0] += calpha * beta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += calpha * beta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += calpha * beta * cgamma * field[kk, ii, jj, 2]             # <<<<<<<<<<<<<<
 *             inside += 1
 *     return 1 if inside == 8 else 0
 */
      __pyx_t_125 = __pyx_v_kk;
      __pyx_t_126 = __pyx_v_ii;
      __pyx_t_127 = __pyx_v_jj;
      __pyx_t_128 = 2;
      __pyx_t_129 = 2;
      *((float *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_129 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_beta) * __pyx_v_cgamma) * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_125 * __pyx_v_field.strides[0]) ) + __pyx_t_126 * __pyx_v_field.strides[1]) ) + __pyx_t_127 * __pyx_v_field.strides[2]) ) + __pyx_t_128 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":714
 *             out[1] += calpha * beta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += calpha * beta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1             # <<<<<<<<<<<<<<
 *     return 1 if inside == 8 else 0
 * 
 */
      __pyx_v_inside = (__pyx_v_inside + 1);

      /* "dipy/align/vector_fields.pyx":710
 *         # ---bottom-left
 *         jj -= 1
 *         if (jj >= 0) and (ii < nr):             # <<<<<<<<<<<<<<
 *             out[0] += calpha * beta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += calpha * beta * cgamma * field[kk, ii, jj, 1]
 */
    }

    /* "dipy/align/vector_fields.pyx":688
 *         inside += 1
 *     kk += 1
 *     if (kk < ns):             # <<<<<<<<<<<<<<
 *         ii -= 1
 *         if (jj >= 0) and (ii >= 0):
 */
  }

  /* "dipy/align/vector_fields.pyx":715
 *             out[2] += calpha * beta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1
 *     return 1 if inside == 8 else 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (((__pyx_v_inside == 8) != 0)) {
    __pyx_t_130 = 1;
  } else {
    __pyx_t_130 = 0;
  }
  __pyx_r = __pyx_t_130;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":604
 * 
 * 
 * cdef inline int _interpolate_vector_3d(floating[:, :, :, :] field, double dkk,             # <<<<<<<<<<<<<<
 *                                        double dii, double djj,
 *                                        floating[:] out) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

static CYTHON_INLINE int __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__Pyx_memviewslice __pyx_v_field, double __pyx_v_dkk, double __pyx_v_dii, double __pyx_v_djj, __Pyx_memviewslice __pyx_v_out) {
  npy_intp __pyx_v_ns;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_kk;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  int __pyx_v_inside;
  double __pyx_v_alpha;
  double __pyx_v_beta;
  double __pyx_v_gamma;
  double __pyx_v_calpha;
  double __pyx_v_cbeta;
  double __pyx_v_cgamma;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  Py_ssize_t __pyx_t_60;
  Py_ssize_t __pyx_t_61;
  Py_ssize_t __pyx_t_62;
  Py_ssize_t __pyx_t_63;
  Py_ssize_t __pyx_t_64;
  Py_ssize_t __pyx_t_65;
  Py_ssize_t __pyx_t_66;
  Py_ssize_t __pyx_t_67;
  Py_ssize_t __pyx_t_68;
  Py_ssize_t __pyx_t_69;
  Py_ssize_t __pyx_t_70;
  Py_ssize_t __pyx_t_71;
  Py_ssize_t __pyx_t_72;
  Py_ssize_t __pyx_t_73;
  Py_ssize_t __pyx_t_74;
  Py_ssize_t __pyx_t_75;
  Py_ssize_t __pyx_t_76;
  Py_ssize_t __pyx_t_77;
  Py_ssize_t __pyx_t_78;
  Py_ssize_t __pyx_t_79;
  Py_ssize_t __pyx_t_80;
  Py_ssize_t __pyx_t_81;
  Py_ssize_t __pyx_t_82;
  Py_ssize_t __pyx_t_83;
  Py_ssize_t __pyx_t_84;
  Py_ssize_t __pyx_t_85;
  Py_ssize_t __pyx_t_86;
  Py_ssize_t __pyx_t_87;
  Py_ssize_t __pyx_t_88;
  Py_ssize_t __pyx_t_89;
  Py_ssize_t __pyx_t_90;
  Py_ssize_t __pyx_t_91;
  Py_ssize_t __pyx_t_92;
  Py_ssize_t __pyx_t_93;
  Py_ssize_t __pyx_t_94;
  Py_ssize_t __pyx_t_95;
  Py_ssize_t __pyx_t_96;
  Py_ssize_t __pyx_t_97;
  Py_ssize_t __pyx_t_98;
  Py_ssize_t __pyx_t_99;
  Py_ssize_t __pyx_t_100;
  Py_ssize_t __pyx_t_101;
  Py_ssize_t __pyx_t_102;
  Py_ssize_t __pyx_t_103;
  Py_ssize_t __pyx_t_104;
  Py_ssize_t __pyx_t_105;
  Py_ssize_t __pyx_t_106;
  Py_ssize_t __pyx_t_107;
  Py_ssize_t __pyx_t_108;
  Py_ssize_t __pyx_t_109;
  Py_ssize_t __pyx_t_110;
  Py_ssize_t __pyx_t_111;
  Py_ssize_t __pyx_t_112;
  Py_ssize_t __pyx_t_113;
  Py_ssize_t __pyx_t_114;
  Py_ssize_t __pyx_t_115;
  Py_ssize_t __pyx_t_116;
  Py_ssize_t __pyx_t_117;
  Py_ssize_t __pyx_t_118;
  Py_ssize_t __pyx_t_119;
  Py_ssize_t __pyx_t_120;
  Py_ssize_t __pyx_t_121;
  Py_ssize_t __pyx_t_122;
  Py_ssize_t __pyx_t_123;
  Py_ssize_t __pyx_t_124;
  Py_ssize_t __pyx_t_125;
  Py_ssize_t __pyx_t_126;
  Py_ssize_t __pyx_t_127;
  Py_ssize_t __pyx_t_128;
  Py_ssize_t __pyx_t_129;
  int __pyx_t_130;

  /* "dipy/align/vector_fields.pyx":633
 *     """
 *     cdef:
 *         cnp.npy_intp ns = field.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr = field.shape[1]
 *         cnp.npy_intp nc = field.shape[2]
 */
  __pyx_v_ns = (__pyx_v_field.shape[0]);

  /* "dipy/align/vector_fields.pyx":634
 *     cdef:
 *         cnp.npy_intp ns = field.shape[0]
 *         cnp.npy_intp nr = field.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = field.shape[2]
 *         cnp.npy_intp kk, ii, jj
 */
  __pyx_v_nr = (__pyx_v_field.shape[1]);

  /* "dipy/align/vector_fields.pyx":635
 *         cnp.npy_intp ns = field.shape[0]
 *         cnp.npy_intp nr = field.shape[1]
 *         cnp.npy_intp nc = field.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp kk, ii, jj
 *         int inside
 */
  __pyx_v_nc = (__pyx_v_field.shape[2]);

  /* "dipy/align/vector_fields.pyx":639
 *         int inside
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *     if not (-1 < dkk < ns and -1 < dii < nr and -1 < djj < nc):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         out[1] = 0
 */
  __pyx_t_2 = (-1.0 < __pyx_v_dkk);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_dkk < __pyx_v_ns);
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (-1.0 < __pyx_v_dii);
  if (__pyx_t_3) {
    __pyx_t_3 = (__pyx_v_dii < __pyx_v_nr);
  }
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (-1.0 < __pyx_v_djj);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_djj < __pyx_v_nc);
  }
  __pyx_t_3 = (__pyx_t_2 != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  __pyx_t_3 = ((!__pyx_t_1) != 0);
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":640
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *     if not (-1 < dkk < ns and -1 < dii < nr and -1 < djj < nc):
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         out[1] = 0
 *         out[2] = 0
 */
    __pyx_t_4 = 0;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_4 * __pyx_v_out.strides[0]) )) = 0.0;

    /* "dipy/align/vector_fields.pyx":641
 *     if not (-1 < dkk < ns and -1 < dii < nr and -1 < djj < nc):
 *         out[0] = 0
 *         out[1] = 0             # <<<<<<<<<<<<<<
 *         out[2] = 0
 *         return 0
 */
    __pyx_t_5 = 1;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_5 * __pyx_v_out.strides[0]) )) = 0.0;

    /* "dipy/align/vector_fields.pyx":642
 *         out[0] = 0
 *         out[1] = 0
 *         out[2] = 0             # <<<<<<<<<<<<<<
 *         return 0
 *     #---top-left
 */
    __pyx_t_6 = 2;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_6 * __pyx_v_out.strides[0]) )) = 0.0;

    /* "dipy/align/vector_fields.pyx":643
 *         out[1] = 0
 *         out[2] = 0
 *         return 0             # <<<<<<<<<<<<<<
 *     #---top-left
 *     kk = <int>floor(dkk)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":639
 *         int inside
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *     if not (-1 < dkk < ns and -1 < dii < nr and -1 < djj < nc):             # <<<<<<<<<<<<<<
 *         out[0] = 0
 *         out[1] = 0
 */
  }

  /* "dipy/align/vector_fields.pyx":645
 *         return 0
 *     #---top-left
 *     kk = <int>floor(dkk)             # <<<<<<<<<<<<<<
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)
 */
  __pyx_v_kk = ((int)floor(__pyx_v_dkk));

  /* "dipy/align/vector_fields.pyx":646
 *     #---top-left
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)             # <<<<<<<<<<<<<<
 *     jj = <int>floor(djj)
 * 
 */
  __pyx_v_ii = ((int)floor(__pyx_v_dii));

  /* "dipy/align/vector_fields.pyx":647
 *     kk = <int>floor(dkk)
 *     ii = <int>floor(dii)
 *     jj = <int>floor(djj)             # <<<<<<<<<<<<<<
 * 
 *     cgamma = dkk - kk
 */
  __pyx_v_jj = ((int)floor(__pyx_v_djj));

  /* "dipy/align/vector_fields.pyx":649
 *     jj = <int>floor(djj)
 * 
 *     cgamma = dkk - kk             # <<<<<<<<<<<<<<
 *     calpha = dii - ii
 *     cbeta = djj - jj
 */
  __pyx_v_cgamma = (__pyx_v_dkk - __pyx_v_kk);

  /* "dipy/align/vector_fields.pyx":650
 * 
 *     cgamma = dkk - kk
 *     calpha = dii - ii             # <<<<<<<<<<<<<<
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 */
  __pyx_v_calpha = (__pyx_v_dii - __pyx_v_ii);

  /* "dipy/align/vector_fields.pyx":651
 *     cgamma = dkk - kk
 *     calpha = dii - ii
 *     cbeta = djj - jj             # <<<<<<<<<<<<<<
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 */
  __pyx_v_cbeta = (__pyx_v_djj - __pyx_v_jj);

  /* "dipy/align/vector_fields.pyx":652
 *     calpha = dii - ii
 *     cbeta = djj - jj
 *     alpha = 1 - calpha             # <<<<<<<<<<<<<<
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma
 */
  __pyx_v_alpha = (1.0 - __pyx_v_calpha);

  /* "dipy/align/vector_fields.pyx":653
 *     cbeta = djj - jj
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta             # <<<<<<<<<<<<<<
 *     gamma = 1 - cgamma
 * 
 */
  __pyx_v_beta = (1.0 - __pyx_v_cbeta);

  /* "dipy/align/vector_fields.pyx":654
 *     alpha = 1 - calpha
 *     beta = 1 - cbeta
 *     gamma = 1 - cgamma             # <<<<<<<<<<<<<<
 * 
 *     inside = 0
 */
  __pyx_v_gamma = (1.0 - __pyx_v_cgamma);

  /* "dipy/align/vector_fields.pyx":656
 *     gamma = 1 - cgamma
 * 
 *     inside = 0             # <<<<<<<<<<<<<<
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):
 *         out[0] = alpha * beta * gamma * field[kk, ii, jj, 0]
 */
  __pyx_v_inside = 0;

  /* "dipy/align/vector_fields.pyx":657
 * 
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] = alpha * beta * gamma * field[kk, ii, jj, 0]
 *         out[1] = alpha * beta * gamma * field[kk, ii, jj, 1]
 */
  __pyx_t_1 = ((__pyx_v_ii >= 0) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_jj >= 0) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L8_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_kk >= 0) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L8_bool_binop_done:;
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":658
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):
 *         out[0] = alpha * beta * gamma * field[kk, ii, jj, 0]             # <<<<<<<<<<<<<<
 *         out[1] = alpha * beta * gamma * field[kk, ii, jj, 1]
 *         out[2] = alpha * beta * gamma * field[kk, ii, jj, 2]
 */
    __pyx_t_7 = __pyx_v_kk;
    __pyx_t_8 = __pyx_v_ii;
    __pyx_t_9 = __pyx_v_jj;
    __pyx_t_10 = 0;
    __pyx_t_11 = 0;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_11 * __pyx_v_out.strides[0]) )) = (((__pyx_v_alpha * __pyx_v_beta) * __pyx_v_gamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_7 * __pyx_v_field.strides[0]) ) + __pyx_t_8 * __pyx_v_field.strides[1]) ) + __pyx_t_9 * __pyx_v_field.strides[2]) ) + __pyx_t_10 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":659
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):
 *         out[0] = alpha * beta * gamma * field[kk, ii, jj, 0]
 *         out[1] = alpha * beta * gamma * field[kk, ii, jj, 1]             # <<<<<<<<<<<<<<
 *         out[2] = alpha * beta * gamma * field[kk, ii, jj, 2]
 *         inside += 1
 */
    __pyx_t_12 = __pyx_v_kk;
    __pyx_t_13 = __pyx_v_ii;
    __pyx_t_14 = __pyx_v_jj;
    __pyx_t_15 = 1;
    __pyx_t_16 = 1;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_16 * __pyx_v_out.strides[0]) )) = (((__pyx_v_alpha * __pyx_v_beta) * __pyx_v_gamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_12 * __pyx_v_field.strides[0]) ) + __pyx_t_13 * __pyx_v_field.strides[1]) ) + __pyx_t_14 * __pyx_v_field.strides[2]) ) + __pyx_t_15 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":660
 *         out[0] = alpha * beta * gamma * field[kk, ii, jj, 0]
 *         out[1] = alpha * beta * gamma * field[kk, ii, jj, 1]
 *         out[2] = alpha * beta * gamma * field[kk, ii, jj, 2]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     else:
 */
    __pyx_t_17 = __pyx_v_kk;
    __pyx_t_18 = __pyx_v_ii;
    __pyx_t_19 = __pyx_v_jj;
    __pyx_t_20 = 2;
    __pyx_t_21 = 2;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_21 * __pyx_v_out.strides[0]) )) = (((__pyx_v_alpha * __pyx_v_beta) * __pyx_v_gamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_17 * __pyx_v_field.strides[0]) ) + __pyx_t_18 * __pyx_v_field.strides[1]) ) + __pyx_t_19 * __pyx_v_field.strides[2]) ) + __pyx_t_20 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":661
 *         out[1] = alpha * beta * gamma * field[kk, ii, jj, 1]
 *         out[2] = alpha * beta * gamma * field[kk, ii, jj, 2]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     else:
 *         out[0] = 0
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":657
 * 
 *     inside = 0
 *     if (ii >= 0) and (jj >= 0) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] = alpha * beta * gamma * field[kk, ii, jj, 0]
 *         out[1] = alpha * beta * gamma * field[kk, ii, jj, 1]
 */
    goto __pyx_L7;
  }

  /* "dipy/align/vector_fields.pyx":663
 *         inside += 1
 *     else:
 *         out[0] = 0             # <<<<<<<<<<<<<<
 *         out[1] = 0
 *         out[2] = 0
 */
  /*else*/ {
    __pyx_t_22 = 0;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_22 * __pyx_v_out.strides[0]) )) = 0.0;

    /* "dipy/align/vector_fields.pyx":664
 *     else:
 *         out[0] = 0
 *         out[1] = 0             # <<<<<<<<<<<<<<
 *         out[2] = 0
 *     # ---top-right
 */
    __pyx_t_23 = 1;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_23 * __pyx_v_out.strides[0]) )) = 0.0;

    /* "dipy/align/vector_fields.pyx":665
 *         out[0] = 0
 *         out[1] = 0
 *         out[2] = 0             # <<<<<<<<<<<<<<
 *     # ---top-right
 *     jj += 1
 */
    __pyx_t_24 = 2;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_24 * __pyx_v_out.strides[0]) )) = 0.0;
  }
  __pyx_L7:;

  /* "dipy/align/vector_fields.pyx":667
 *         out[2] = 0
 *     # ---top-right
 *     jj += 1             # <<<<<<<<<<<<<<
 *     if (jj < nc) and (ii >= 0) and (kk >= 0):
 *         out[0] += alpha * cbeta * gamma * field[kk, ii, jj, 0]
 */
  __pyx_v_jj = (__pyx_v_jj + 1);

  /* "dipy/align/vector_fields.pyx":668
 *     # ---top-right
 *     jj += 1
 *     if (jj < nc) and (ii >= 0) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += alpha * cbeta * gamma * field[kk, ii, jj, 0]
 *         out[1] += alpha * cbeta * gamma * field[kk, ii, jj, 1]
 */
  __pyx_t_1 = ((__pyx_v_jj < __pyx_v_nc) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_ii >= 0) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_kk >= 0) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L12_bool_binop_done:;
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":669
 *     jj += 1
 *     if (jj < nc) and (ii >= 0) and (kk >= 0):
 *         out[0] += alpha * cbeta * gamma * field[kk, ii, jj, 0]             # <<<<<<<<<<<<<<
 *         out[1] += alpha * cbeta * gamma * field[kk, ii, jj, 1]
 *         out[2] += alpha * cbeta * gamma * field[kk, ii, jj, 2]
 */
    __pyx_t_25 = __pyx_v_kk;
    __pyx_t_26 = __pyx_v_ii;
    __pyx_t_27 = __pyx_v_jj;
    __pyx_t_28 = 0;
    __pyx_t_29 = 0;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_29 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_cbeta) * __pyx_v_gamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_25 * __pyx_v_field.strides[0]) ) + __pyx_t_26 * __pyx_v_field.strides[1]) ) + __pyx_t_27 * __pyx_v_field.strides[2]) ) + __pyx_t_28 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":670
 *     if (jj < nc) and (ii >= 0) and (kk >= 0):
 *         out[0] += alpha * cbeta * gamma * field[kk, ii, jj, 0]
 *         out[1] += alpha * cbeta * gamma * field[kk, ii, jj, 1]             # <<<<<<<<<<<<<<
 *         out[2] += alpha * cbeta * gamma * field[kk, ii, jj, 2]
 *         inside += 1
 */
    __pyx_t_30 = __pyx_v_kk;
    __pyx_t_31 = __pyx_v_ii;
    __pyx_t_32 = __pyx_v_jj;
    __pyx_t_33 = 1;
    __pyx_t_34 = 1;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_34 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_cbeta) * __pyx_v_gamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_30 * __pyx_v_field.strides[0]) ) + __pyx_t_31 * __pyx_v_field.strides[1]) ) + __pyx_t_32 * __pyx_v_field.strides[2]) ) + __pyx_t_33 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":671
 *         out[0] += alpha * cbeta * gamma * field[kk, ii, jj, 0]
 *         out[1] += alpha * cbeta * gamma * field[kk, ii, jj, 1]
 *         out[2] += alpha * cbeta * gamma * field[kk, ii, jj, 2]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     # ---bottom-right
 */
    __pyx_t_35 = __pyx_v_kk;
    __pyx_t_36 = __pyx_v_ii;
    __pyx_t_37 = __pyx_v_jj;
    __pyx_t_38 = 2;
    __pyx_t_39 = 2;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_39 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_cbeta) * __pyx_v_gamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_35 * __pyx_v_field.strides[0]) ) + __pyx_t_36 * __pyx_v_field.strides[1]) ) + __pyx_t_37 * __pyx_v_field.strides[2]) ) + __pyx_t_38 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":672
 *         out[1] += alpha * cbeta * gamma * field[kk, ii, jj, 1]
 *         out[2] += alpha * cbeta * gamma * field[kk, ii, jj, 2]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     # ---bottom-right
 *     ii += 1
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":668
 *     # ---top-right
 *     jj += 1
 *     if (jj < nc) and (ii >= 0) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += alpha * cbeta * gamma * field[kk, ii, jj, 0]
 *         out[1] += alpha * cbeta * gamma * field[kk, ii, jj, 1]
 */
  }

  /* "dipy/align/vector_fields.pyx":674
 *         inside += 1
 *     # ---bottom-right
 *     ii += 1             # <<<<<<<<<<<<<<
 *     if (jj < nc) and (ii < nr) and (kk >= 0):
 *         out[0] += calpha * cbeta * gamma * field[kk, ii, jj, 0]
 */
  __pyx_v_ii = (__pyx_v_ii + 1);

  /* "dipy/align/vector_fields.pyx":675
 *     # ---bottom-right
 *     ii += 1
 *     if (jj < nc) and (ii < nr) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * cbeta * gamma * field[kk, ii, jj, 0]
 *         out[1] += calpha * cbeta * gamma * field[kk, ii, jj, 1]
 */
  __pyx_t_1 = ((__pyx_v_jj < __pyx_v_nc) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_ii < __pyx_v_nr) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_kk >= 0) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L16_bool_binop_done:;
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":676
 *     ii += 1
 *     if (jj < nc) and (ii < nr) and (kk >= 0):
 *         out[0] += calpha * cbeta * gamma * field[kk, ii, jj, 0]             # <<<<<<<<<<<<<<
 *         out[1] += calpha * cbeta * gamma * field[kk, ii, jj, 1]
 *         out[2] += calpha * cbeta * gamma * field[kk, ii, jj, 2]
 */
    __pyx_t_40 = __pyx_v_kk;
    __pyx_t_41 = __pyx_v_ii;
    __pyx_t_42 = __pyx_v_jj;
    __pyx_t_43 = 0;
    __pyx_t_44 = 0;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_44 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_cbeta) * __pyx_v_gamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_40 * __pyx_v_field.strides[0]) ) + __pyx_t_41 * __pyx_v_field.strides[1]) ) + __pyx_t_42 * __pyx_v_field.strides[2]) ) + __pyx_t_43 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":677
 *     if (jj < nc) and (ii < nr) and (kk >= 0):
 *         out[0] += calpha * cbeta * gamma * field[kk, ii, jj, 0]
 *         out[1] += calpha * cbeta * gamma * field[kk, ii, jj, 1]             # <<<<<<<<<<<<<<
 *         out[2] += calpha * cbeta * gamma * field[kk, ii, jj, 2]
 *         inside += 1
 */
    __pyx_t_45 = __pyx_v_kk;
    __pyx_t_46 = __pyx_v_ii;
    __pyx_t_47 = __pyx_v_jj;
    __pyx_t_48 = 1;
    __pyx_t_49 = 1;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_49 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_cbeta) * __pyx_v_gamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_45 * __pyx_v_field.strides[0]) ) + __pyx_t_46 * __pyx_v_field.strides[1]) ) + __pyx_t_47 * __pyx_v_field.strides[2]) ) + __pyx_t_48 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":678
 *         out[0] += calpha * cbeta * gamma * field[kk, ii, jj, 0]
 *         out[1] += calpha * cbeta * gamma * field[kk, ii, jj, 1]
 *         out[2] += calpha * cbeta * gamma * field[kk, ii, jj, 2]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     # ---bottom-left
 */
    __pyx_t_50 = __pyx_v_kk;
    __pyx_t_51 = __pyx_v_ii;
    __pyx_t_52 = __pyx_v_jj;
    __pyx_t_53 = 2;
    __pyx_t_54 = 2;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_54 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_cbeta) * __pyx_v_gamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_50 * __pyx_v_field.strides[0]) ) + __pyx_t_51 * __pyx_v_field.strides[1]) ) + __pyx_t_52 * __pyx_v_field.strides[2]) ) + __pyx_t_53 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":679
 *         out[1] += calpha * cbeta * gamma * field[kk, ii, jj, 1]
 *         out[2] += calpha * cbeta * gamma * field[kk, ii, jj, 2]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     # ---bottom-left
 *     jj -= 1
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":675
 *     # ---bottom-right
 *     ii += 1
 *     if (jj < nc) and (ii < nr) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * cbeta * gamma * field[kk, ii, jj, 0]
 *         out[1] += calpha * cbeta * gamma * field[kk, ii, jj, 1]
 */
  }

  /* "dipy/align/vector_fields.pyx":681
 *         inside += 1
 *     # ---bottom-left
 *     jj -= 1             # <<<<<<<<<<<<<<
 *     if (jj >= 0) and (ii < nr) and (kk >= 0):
 *         out[0] += calpha * beta * gamma * field[kk, ii, jj, 0]
 */
  __pyx_v_jj = (__pyx_v_jj - 1);

  /* "dipy/align/vector_fields.pyx":682
 *     # ---bottom-left
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * beta * gamma * field[kk, ii, jj, 0]
 *         out[1] += calpha * beta * gamma * field[kk, ii, jj, 1]
 */
  __pyx_t_1 = ((__pyx_v_jj >= 0) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L20_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_ii < __pyx_v_nr) != 0);
  if (__pyx_t_1) {
  } else {
    __pyx_t_3 = __pyx_t_1;
    goto __pyx_L20_bool_binop_done;
  }
  __pyx_t_1 = ((__pyx_v_kk >= 0) != 0);
  __pyx_t_3 = __pyx_t_1;
  __pyx_L20_bool_binop_done:;
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":683
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr) and (kk >= 0):
 *         out[0] += calpha * beta * gamma * field[kk, ii, jj, 0]             # <<<<<<<<<<<<<<
 *         out[1] += calpha * beta * gamma * field[kk, ii, jj, 1]
 *         out[2] += calpha * beta * gamma * field[kk, ii, jj, 2]
 */
    __pyx_t_55 = __pyx_v_kk;
    __pyx_t_56 = __pyx_v_ii;
    __pyx_t_57 = __pyx_v_jj;
    __pyx_t_58 = 0;
    __pyx_t_59 = 0;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_59 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_beta) * __pyx_v_gamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_55 * __pyx_v_field.strides[0]) ) + __pyx_t_56 * __pyx_v_field.strides[1]) ) + __pyx_t_57 * __pyx_v_field.strides[2]) ) + __pyx_t_58 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":684
 *     if (jj >= 0) and (ii < nr) and (kk >= 0):
 *         out[0] += calpha * beta * gamma * field[kk, ii, jj, 0]
 *         out[1] += calpha * beta * gamma * field[kk, ii, jj, 1]             # <<<<<<<<<<<<<<
 *         out[2] += calpha * beta * gamma * field[kk, ii, jj, 2]
 *         inside += 1
 */
    __pyx_t_60 = __pyx_v_kk;
    __pyx_t_61 = __pyx_v_ii;
    __pyx_t_62 = __pyx_v_jj;
    __pyx_t_63 = 1;
    __pyx_t_64 = 1;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_64 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_beta) * __pyx_v_gamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_60 * __pyx_v_field.strides[0]) ) + __pyx_t_61 * __pyx_v_field.strides[1]) ) + __pyx_t_62 * __pyx_v_field.strides[2]) ) + __pyx_t_63 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":685
 *         out[0] += calpha * beta * gamma * field[kk, ii, jj, 0]
 *         out[1] += calpha * beta * gamma * field[kk, ii, jj, 1]
 *         out[2] += calpha * beta * gamma * field[kk, ii, jj, 2]             # <<<<<<<<<<<<<<
 *         inside += 1
 *     kk += 1
 */
    __pyx_t_65 = __pyx_v_kk;
    __pyx_t_66 = __pyx_v_ii;
    __pyx_t_67 = __pyx_v_jj;
    __pyx_t_68 = 2;
    __pyx_t_69 = 2;
    *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_69 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_beta) * __pyx_v_gamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_65 * __pyx_v_field.strides[0]) ) + __pyx_t_66 * __pyx_v_field.strides[1]) ) + __pyx_t_67 * __pyx_v_field.strides[2]) ) + __pyx_t_68 * __pyx_v_field.strides[3]) ))));

    /* "dipy/align/vector_fields.pyx":686
 *         out[1] += calpha * beta * gamma * field[kk, ii, jj, 1]
 *         out[2] += calpha * beta * gamma * field[kk, ii, jj, 2]
 *         inside += 1             # <<<<<<<<<<<<<<
 *     kk += 1
 *     if (kk < ns):
 */
    __pyx_v_inside = (__pyx_v_inside + 1);

    /* "dipy/align/vector_fields.pyx":682
 *     # ---bottom-left
 *     jj -= 1
 *     if (jj >= 0) and (ii < nr) and (kk >= 0):             # <<<<<<<<<<<<<<
 *         out[0] += calpha * beta * gamma * field[kk, ii, jj, 0]
 *         out[1] += calpha * beta * gamma * field[kk, ii, jj, 1]
 */
  }

  /* "dipy/align/vector_fields.pyx":687
 *         out[2] += calpha * beta * gamma * field[kk, ii, jj, 2]
 *         inside += 1
 *     kk += 1             # <<<<<<<<<<<<<<
 *     if (kk < ns):
 *         ii -= 1
 */
  __pyx_v_kk = (__pyx_v_kk + 1);

  /* "dipy/align/vector_fields.pyx":688
 *         inside += 1
 *     kk += 1
 *     if (kk < ns):             # <<<<<<<<<<<<<<
 *         ii -= 1
 *         if (jj >= 0) and (ii >= 0):
 */
  __pyx_t_3 = ((__pyx_v_kk < __pyx_v_ns) != 0);
  if (__pyx_t_3) {

    /* "dipy/align/vector_fields.pyx":689
 *     kk += 1
 *     if (kk < ns):
 *         ii -= 1             # <<<<<<<<<<<<<<
 *         if (jj >= 0) and (ii >= 0):
 *             out[0] += alpha * beta * cgamma * field[kk, ii, jj, 0]
 */
    __pyx_v_ii = (__pyx_v_ii - 1);

    /* "dipy/align/vector_fields.pyx":690
 *     if (kk < ns):
 *         ii -= 1
 *         if (jj >= 0) and (ii >= 0):             # <<<<<<<<<<<<<<
 *             out[0] += alpha * beta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += alpha * beta * cgamma * field[kk, ii, jj, 1]
 */
    __pyx_t_1 = ((__pyx_v_jj >= 0) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_3 = __pyx_t_1;
      goto __pyx_L25_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_ii >= 0) != 0);
    __pyx_t_3 = __pyx_t_1;
    __pyx_L25_bool_binop_done:;
    if (__pyx_t_3) {

      /* "dipy/align/vector_fields.pyx":691
 *         ii -= 1
 *         if (jj >= 0) and (ii >= 0):
 *             out[0] += alpha * beta * cgamma * field[kk, ii, jj, 0]             # <<<<<<<<<<<<<<
 *             out[1] += alpha * beta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += alpha * beta * cgamma * field[kk, ii, jj, 2]
 */
      __pyx_t_70 = __pyx_v_kk;
      __pyx_t_71 = __pyx_v_ii;
      __pyx_t_72 = __pyx_v_jj;
      __pyx_t_73 = 0;
      __pyx_t_74 = 0;
      *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_74 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_beta) * __pyx_v_cgamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_70 * __pyx_v_field.strides[0]) ) + __pyx_t_71 * __pyx_v_field.strides[1]) ) + __pyx_t_72 * __pyx_v_field.strides[2]) ) + __pyx_t_73 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":692
 *         if (jj >= 0) and (ii >= 0):
 *             out[0] += alpha * beta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += alpha * beta * cgamma * field[kk, ii, jj, 1]             # <<<<<<<<<<<<<<
 *             out[2] += alpha * beta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1
 */
      __pyx_t_75 = __pyx_v_kk;
      __pyx_t_76 = __pyx_v_ii;
      __pyx_t_77 = __pyx_v_jj;
      __pyx_t_78 = 1;
      __pyx_t_79 = 1;
      *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_79 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_beta) * __pyx_v_cgamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_75 * __pyx_v_field.strides[0]) ) + __pyx_t_76 * __pyx_v_field.strides[1]) ) + __pyx_t_77 * __pyx_v_field.strides[2]) ) + __pyx_t_78 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":693
 *             out[0] += alpha * beta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += alpha * beta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += alpha * beta * cgamma * field[kk, ii, jj, 2]             # <<<<<<<<<<<<<<
 *             inside += 1
 *         jj += 1
 */
      __pyx_t_80 = __pyx_v_kk;
      __pyx_t_81 = __pyx_v_ii;
      __pyx_t_82 = __pyx_v_jj;
      __pyx_t_83 = 2;
      __pyx_t_84 = 2;
      *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_84 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_beta) * __pyx_v_cgamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_80 * __pyx_v_field.strides[0]) ) + __pyx_t_81 * __pyx_v_field.strides[1]) ) + __pyx_t_82 * __pyx_v_field.strides[2]) ) + __pyx_t_83 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":694
 *             out[1] += alpha * beta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += alpha * beta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1             # <<<<<<<<<<<<<<
 *         jj += 1
 *         if (jj < nc) and (ii >= 0):
 */
      __pyx_v_inside = (__pyx_v_inside + 1);

      /* "dipy/align/vector_fields.pyx":690
 *     if (kk < ns):
 *         ii -= 1
 *         if (jj >= 0) and (ii >= 0):             # <<<<<<<<<<<<<<
 *             out[0] += alpha * beta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += alpha * beta * cgamma * field[kk, ii, jj, 1]
 */
    }

    /* "dipy/align/vector_fields.pyx":695
 *             out[2] += alpha * beta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1
 *         jj += 1             # <<<<<<<<<<<<<<
 *         if (jj < nc) and (ii >= 0):
 *             out[0] += alpha * cbeta * cgamma * field[kk, ii, jj, 0]
 */
    __pyx_v_jj = (__pyx_v_jj + 1);

    /* "dipy/align/vector_fields.pyx":696
 *             inside += 1
 *         jj += 1
 *         if (jj < nc) and (ii >= 0):             # <<<<<<<<<<<<<<
 *             out[0] += alpha * cbeta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += alpha * cbeta * cgamma * field[kk, ii, jj, 1]
 */
    __pyx_t_1 = ((__pyx_v_jj < __pyx_v_nc) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_3 = __pyx_t_1;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_ii >= 0) != 0);
    __pyx_t_3 = __pyx_t_1;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_3) {

      /* "dipy/align/vector_fields.pyx":697
 *         jj += 1
 *         if (jj < nc) and (ii >= 0):
 *             out[0] += alpha * cbeta * cgamma * field[kk, ii, jj, 0]             # <<<<<<<<<<<<<<
 *             out[1] += alpha * cbeta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += alpha * cbeta * cgamma * field[kk, ii, jj, 2]
 */
      __pyx_t_85 = __pyx_v_kk;
      __pyx_t_86 = __pyx_v_ii;
      __pyx_t_87 = __pyx_v_jj;
      __pyx_t_88 = 0;
      __pyx_t_89 = 0;
      *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_89 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_cbeta) * __pyx_v_cgamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_85 * __pyx_v_field.strides[0]) ) + __pyx_t_86 * __pyx_v_field.strides[1]) ) + __pyx_t_87 * __pyx_v_field.strides[2]) ) + __pyx_t_88 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":698
 *         if (jj < nc) and (ii >= 0):
 *             out[0] += alpha * cbeta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += alpha * cbeta * cgamma * field[kk, ii, jj, 1]             # <<<<<<<<<<<<<<
 *             out[2] += alpha * cbeta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1
 */
      __pyx_t_90 = __pyx_v_kk;
      __pyx_t_91 = __pyx_v_ii;
      __pyx_t_92 = __pyx_v_jj;
      __pyx_t_93 = 1;
      __pyx_t_94 = 1;
      *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_94 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_cbeta) * __pyx_v_cgamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_90 * __pyx_v_field.strides[0]) ) + __pyx_t_91 * __pyx_v_field.strides[1]) ) + __pyx_t_92 * __pyx_v_field.strides[2]) ) + __pyx_t_93 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":699
 *             out[0] += alpha * cbeta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += alpha * cbeta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += alpha * cbeta * cgamma * field[kk, ii, jj, 2]             # <<<<<<<<<<<<<<
 *             inside += 1
 *         # ---bottom-right
 */
      __pyx_t_95 = __pyx_v_kk;
      __pyx_t_96 = __pyx_v_ii;
      __pyx_t_97 = __pyx_v_jj;
      __pyx_t_98 = 2;
      __pyx_t_99 = 2;
      *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_99 * __pyx_v_out.strides[0]) )) += (((__pyx_v_alpha * __pyx_v_cbeta) * __pyx_v_cgamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_95 * __pyx_v_field.strides[0]) ) + __pyx_t_96 * __pyx_v_field.strides[1]) ) + __pyx_t_97 * __pyx_v_field.strides[2]) ) + __pyx_t_98 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":700
 *             out[1] += alpha * cbeta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += alpha * cbeta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1             # <<<<<<<<<<<<<<
 *         # ---bottom-right
 *         ii += 1
 */
      __pyx_v_inside = (__pyx_v_inside + 1);

      /* "dipy/align/vector_fields.pyx":696
 *             inside += 1
 *         jj += 1
 *         if (jj < nc) and (ii >= 0):             # <<<<<<<<<<<<<<
 *             out[0] += alpha * cbeta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += alpha * cbeta * cgamma * field[kk, ii, jj, 1]
 */
    }

    /* "dipy/align/vector_fields.pyx":702
 *             inside += 1
 *         # ---bottom-right
 *         ii += 1             # <<<<<<<<<<<<<<
 *         if (jj < nc) and (ii < nr):
 *             out[0] += calpha * cbeta * cgamma * field[kk, ii, jj, 0]
 */
    __pyx_v_ii = (__pyx_v_ii + 1);

    /* "dipy/align/vector_fields.pyx":703
 *         # ---bottom-right
 *         ii += 1
 *         if (jj < nc) and (ii < nr):             # <<<<<<<<<<<<<<
 *             out[0] += calpha * cbeta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += calpha * cbeta * cgamma * field[kk, ii, jj, 1]
 */
    __pyx_t_1 = ((__pyx_v_jj < __pyx_v_nc) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_3 = __pyx_t_1;
      goto __pyx_L31_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_ii < __pyx_v_nr) != 0);
    __pyx_t_3 = __pyx_t_1;
    __pyx_L31_bool_binop_done:;
    if (__pyx_t_3) {

      /* "dipy/align/vector_fields.pyx":704
 *         ii += 1
 *         if (jj < nc) and (ii < nr):
 *             out[0] += calpha * cbeta * cgamma * field[kk, ii, jj, 0]             # <<<<<<<<<<<<<<
 *             out[1] += calpha * cbeta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += calpha * cbeta * cgamma * field[kk, ii, jj, 2]
 */
      __pyx_t_100 = __pyx_v_kk;
      __pyx_t_101 = __pyx_v_ii;
      __pyx_t_102 = __pyx_v_jj;
      __pyx_t_103 = 0;
      __pyx_t_104 = 0;
      *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_104 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_cbeta) * __pyx_v_cgamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_100 * __pyx_v_field.strides[0]) ) + __pyx_t_101 * __pyx_v_field.strides[1]) ) + __pyx_t_102 * __pyx_v_field.strides[2]) ) + __pyx_t_103 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":705
 *         if (jj < nc) and (ii < nr):
 *             out[0] += calpha * cbeta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += calpha * cbeta * cgamma * field[kk, ii, jj, 1]             # <<<<<<<<<<<<<<
 *             out[2] += calpha * cbeta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1
 */
      __pyx_t_105 = __pyx_v_kk;
      __pyx_t_106 = __pyx_v_ii;
      __pyx_t_107 = __pyx_v_jj;
      __pyx_t_108 = 1;
      __pyx_t_109 = 1;
      *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_109 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_cbeta) * __pyx_v_cgamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_105 * __pyx_v_field.strides[0]) ) + __pyx_t_106 * __pyx_v_field.strides[1]) ) + __pyx_t_107 * __pyx_v_field.strides[2]) ) + __pyx_t_108 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":706
 *             out[0] += calpha * cbeta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += calpha * cbeta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += calpha * cbeta * cgamma * field[kk, ii, jj, 2]             # <<<<<<<<<<<<<<
 *             inside += 1
 *         # ---bottom-left
 */
      __pyx_t_110 = __pyx_v_kk;
      __pyx_t_111 = __pyx_v_ii;
      __pyx_t_112 = __pyx_v_jj;
      __pyx_t_113 = 2;
      __pyx_t_114 = 2;
      *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_114 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_cbeta) * __pyx_v_cgamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_110 * __pyx_v_field.strides[0]) ) + __pyx_t_111 * __pyx_v_field.strides[1]) ) + __pyx_t_112 * __pyx_v_field.strides[2]) ) + __pyx_t_113 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":707
 *             out[1] += calpha * cbeta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += calpha * cbeta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1             # <<<<<<<<<<<<<<
 *         # ---bottom-left
 *         jj -= 1
 */
      __pyx_v_inside = (__pyx_v_inside + 1);

      /* "dipy/align/vector_fields.pyx":703
 *         # ---bottom-right
 *         ii += 1
 *         if (jj < nc) and (ii < nr):             # <<<<<<<<<<<<<<
 *             out[0] += calpha * cbeta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += calpha * cbeta * cgamma * field[kk, ii, jj, 1]
 */
    }

    /* "dipy/align/vector_fields.pyx":709
 *             inside += 1
 *         # ---bottom-left
 *         jj -= 1             # <<<<<<<<<<<<<<
 *         if (jj >= 0) and (ii < nr):
 *             out[0] += calpha * beta * cgamma * field[kk, ii, jj, 0]
 */
    __pyx_v_jj = (__pyx_v_jj - 1);

    /* "dipy/align/vector_fields.pyx":710
 *         # ---bottom-left
 *         jj -= 1
 *         if (jj >= 0) and (ii < nr):             # <<<<<<<<<<<<<<
 *             out[0] += calpha * beta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += calpha * beta * cgamma * field[kk, ii, jj, 1]
 */
    __pyx_t_1 = ((__pyx_v_jj >= 0) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_3 = __pyx_t_1;
      goto __pyx_L34_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_ii < __pyx_v_nr) != 0);
    __pyx_t_3 = __pyx_t_1;
    __pyx_L34_bool_binop_done:;
    if (__pyx_t_3) {

      /* "dipy/align/vector_fields.pyx":711
 *         jj -= 1
 *         if (jj >= 0) and (ii < nr):
 *             out[0] += calpha * beta * cgamma * field[kk, ii, jj, 0]             # <<<<<<<<<<<<<<
 *             out[1] += calpha * beta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += calpha * beta * cgamma * field[kk, ii, jj, 2]
 */
      __pyx_t_115 = __pyx_v_kk;
      __pyx_t_116 = __pyx_v_ii;
      __pyx_t_117 = __pyx_v_jj;
      __pyx_t_118 = 0;
      __pyx_t_119 = 0;
      *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_119 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_beta) * __pyx_v_cgamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_115 * __pyx_v_field.strides[0]) ) + __pyx_t_116 * __pyx_v_field.strides[1]) ) + __pyx_t_117 * __pyx_v_field.strides[2]) ) + __pyx_t_118 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":712
 *         if (jj >= 0) and (ii < nr):
 *             out[0] += calpha * beta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += calpha * beta * cgamma * field[kk, ii, jj, 1]             # <<<<<<<<<<<<<<
 *             out[2] += calpha * beta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1
 */
      __pyx_t_120 = __pyx_v_kk;
      __pyx_t_121 = __pyx_v_ii;
      __pyx_t_122 = __pyx_v_jj;
      __pyx_t_123 = 1;
      __pyx_t_124 = 1;
      *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_124 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_beta) * __pyx_v_cgamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_120 * __pyx_v_field.strides[0]) ) + __pyx_t_121 * __pyx_v_field.strides[1]) ) + __pyx_t_122 * __pyx_v_field.strides[2]) ) + __pyx_t_123 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":713
 *             out[0] += calpha * beta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += calpha * beta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += calpha * beta * cgamma * field[kk, ii, jj, 2]             # <<<<<<<<<<<<<<
 *             inside += 1
 *     return 1 if inside == 8 else 0
 */
      __pyx_t_125 = __pyx_v_kk;
      __pyx_t_126 = __pyx_v_ii;
      __pyx_t_127 = __pyx_v_jj;
      __pyx_t_128 = 2;
      __pyx_t_129 = 2;
      *((double *) ( /* dim=0 */ (__pyx_v_out.data + __pyx_t_129 * __pyx_v_out.strides[0]) )) += (((__pyx_v_calpha * __pyx_v_beta) * __pyx_v_cgamma) * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_125 * __pyx_v_field.strides[0]) ) + __pyx_t_126 * __pyx_v_field.strides[1]) ) + __pyx_t_127 * __pyx_v_field.strides[2]) ) + __pyx_t_128 * __pyx_v_field.strides[3]) ))));

      /* "dipy/align/vector_fields.pyx":714
 *             out[1] += calpha * beta * cgamma * field[kk, ii, jj, 1]
 *             out[2] += calpha * beta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1             # <<<<<<<<<<<<<<
 *     return 1 if inside == 8 else 0
 * 
 */
      __pyx_v_inside = (__pyx_v_inside + 1);

      /* "dipy/align/vector_fields.pyx":710
 *         # ---bottom-left
 *         jj -= 1
 *         if (jj >= 0) and (ii < nr):             # <<<<<<<<<<<<<<
 *             out[0] += calpha * beta * cgamma * field[kk, ii, jj, 0]
 *             out[1] += calpha * beta * cgamma * field[kk, ii, jj, 1]
 */
    }

    /* "dipy/align/vector_fields.pyx":688
 *         inside += 1
 *     kk += 1
 *     if (kk < ns):             # <<<<<<<<<<<<<<
 *         ii -= 1
 *         if (jj >= 0) and (ii >= 0):
 */
  }

  /* "dipy/align/vector_fields.pyx":715
 *             out[2] += calpha * beta * cgamma * field[kk, ii, jj, 2]
 *             inside += 1
 *     return 1 if inside == 8 else 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (((__pyx_v_inside == 8) != 0)) {
    __pyx_t_130 = 1;
  } else {
    __pyx_t_130 = 0;
  }
  __pyx_r = __pyx_t_130;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":604
 * 
 * 
 * cdef inline int _interpolate_vector_3d(floating[:, :, :, :] field, double dkk,             # <<<<<<<<<<<<<<
 *                                        double dii, double djj,
 *                                        floating[:] out) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":718
 * 
 * 
 * cdef void _compose_vector_fields_2d(floating[:, :, :] d1, floating[:, :, :] d2,             # <<<<<<<<<<<<<<
 *                                     double[:, :] premult_index,
 *                                     double[:, :] premult_disp,
 */

static void __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__compose_vector_fields_2d(__Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_d2, __Pyx_memviewslice __pyx_v_premult_index, __Pyx_memviewslice __pyx_v_premult_disp, double __pyx_v_time_scaling, __Pyx_memviewslice __pyx_v_comp, __Pyx_memviewslice __pyx_v_stats) {
  npy_intp __pyx_v_nr1;
  npy_intp __pyx_v_nc1;
  CYTHON_UNUSED npy_intp __pyx_v_nr2;
  CYTHON_UNUSED npy_intp __pyx_v_nc2;
  int __pyx_v_inside;
  int __pyx_v_cnt;
  double __pyx_v_maxNorm;
  double __pyx_v_meanNorm;
  double __pyx_v_stdNorm;
  double __pyx_v_nn;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_diii;
  double __pyx_v_djjj;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;
  npy_intp __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  __Pyx_memviewslice __pyx_t_12 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "dipy/align/vector_fields.pyx":786
 *     """
 *     cdef:
 *         cnp.npy_intp nr1 = d1.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc1 = d1.shape[1]
 *         cnp.npy_intp nr2 = d2.shape[0]
 */
  __pyx_v_nr1 = (__pyx_v_d1.shape[0]);

  /* "dipy/align/vector_fields.pyx":787
 *     cdef:
 *         cnp.npy_intp nr1 = d1.shape[0]
 *         cnp.npy_intp nc1 = d1.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr2 = d2.shape[0]
 *         cnp.npy_intp nc2 = d2.shape[1]
 */
  __pyx_v_nc1 = (__pyx_v_d1.shape[1]);

  /* "dipy/align/vector_fields.pyx":788
 *         cnp.npy_intp nr1 = d1.shape[0]
 *         cnp.npy_intp nc1 = d1.shape[1]
 *         cnp.npy_intp nr2 = d2.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc2 = d2.shape[1]
 *         int inside, cnt = 0
 */
  __pyx_v_nr2 = (__pyx_v_d2.shape[0]);

  /* "dipy/align/vector_fields.pyx":789
 *         cnp.npy_intp nc1 = d1.shape[1]
 *         cnp.npy_intp nr2 = d2.shape[0]
 *         cnp.npy_intp nc2 = d2.shape[1]             # <<<<<<<<<<<<<<
 *         int inside, cnt = 0
 *         double maxNorm = 0
 */
  __pyx_v_nc2 = (__pyx_v_d2.shape[1]);

  /* "dipy/align/vector_fields.pyx":790
 *         cnp.npy_intp nr2 = d2.shape[0]
 *         cnp.npy_intp nc2 = d2.shape[1]
 *         int inside, cnt = 0             # <<<<<<<<<<<<<<
 *         double maxNorm = 0
 *         double meanNorm = 0
 */
  __pyx_v_cnt = 0;

  /* "dipy/align/vector_fields.pyx":791
 *         cnp.npy_intp nc2 = d2.shape[1]
 *         int inside, cnt = 0
 *         double maxNorm = 0             # <<<<<<<<<<<<<<
 *         double meanNorm = 0
 *         double stdNorm = 0
 */
  __pyx_v_maxNorm = 0.0;

  /* "dipy/align/vector_fields.pyx":792
 *         int inside, cnt = 0
 *         double maxNorm = 0
 *         double meanNorm = 0             # <<<<<<<<<<<<<<
 *         double stdNorm = 0
 *         double nn
 */
  __pyx_v_meanNorm = 0.0;

  /* "dipy/align/vector_fields.pyx":793
 *         double maxNorm = 0
 *         double meanNorm = 0
 *         double stdNorm = 0             # <<<<<<<<<<<<<<
 *         double nn
 *         cnp.npy_intp i, j
 */
  __pyx_v_stdNorm = 0.0;

  /* "dipy/align/vector_fields.pyx":798
 *         double di, dj, dii, djj, diii, djjj
 * 
 *     for i in range(nr1):             # <<<<<<<<<<<<<<
 *         for j in range(nc1):
 * 
 */
  __pyx_t_1 = __pyx_v_nr1;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "dipy/align/vector_fields.pyx":799
 * 
 *     for i in range(nr1):
 *         for j in range(nc1):             # <<<<<<<<<<<<<<
 * 
 *             # This is the only place we access d1[i, j]
 */
    __pyx_t_3 = __pyx_v_nc1;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_j = __pyx_t_4;

      /* "dipy/align/vector_fields.pyx":802
 * 
 *             # This is the only place we access d1[i, j]
 *             dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *             djj = d1[i, j, 1]
 * 
 */
      __pyx_t_5 = __pyx_v_i;
      __pyx_t_6 = __pyx_v_j;
      __pyx_t_7 = 0;
      __pyx_v_dii = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_5 * __pyx_v_d1.strides[0]) ) + __pyx_t_6 * __pyx_v_d1.strides[1]) ) + __pyx_t_7 * __pyx_v_d1.strides[2]) )));

      /* "dipy/align/vector_fields.pyx":803
 *             # This is the only place we access d1[i, j]
 *             dii = d1[i, j, 0]
 *             djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 * 
 *             if premult_disp is None:
 */
      __pyx_t_8 = __pyx_v_i;
      __pyx_t_9 = __pyx_v_j;
      __pyx_t_10 = 1;
      __pyx_v_djj = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_8 * __pyx_v_d1.strides[0]) ) + __pyx_t_9 * __pyx_v_d1.strides[1]) ) + __pyx_t_10 * __pyx_v_d1.strides[2]) )));

      /* "dipy/align/vector_fields.pyx":805
 *             djj = d1[i, j, 1]
 * 
 *             if premult_disp is None:             # <<<<<<<<<<<<<<
 *                 di = dii
 *                 dj = djj
 */
      __pyx_t_11 = ((((PyObject *) __pyx_v_premult_disp.memview) == Py_None) != 0);
      if (__pyx_t_11) {

        /* "dipy/align/vector_fields.pyx":806
 * 
 *             if premult_disp is None:
 *                 di = dii             # <<<<<<<<<<<<<<
 *                 dj = djj
 *             else:
 */
        __pyx_v_di = __pyx_v_dii;

        /* "dipy/align/vector_fields.pyx":807
 *             if premult_disp is None:
 *                 di = dii
 *                 dj = djj             # <<<<<<<<<<<<<<
 *             else:
 *                 di = _apply_affine_2d_x0(dii, djj, 0, premult_disp)
 */
        __pyx_v_dj = __pyx_v_djj;

        /* "dipy/align/vector_fields.pyx":805
 *             djj = d1[i, j, 1]
 * 
 *             if premult_disp is None:             # <<<<<<<<<<<<<<
 *                 di = dii
 *                 dj = djj
 */
        goto __pyx_L7;
      }

      /* "dipy/align/vector_fields.pyx":809
 *                 dj = djj
 *             else:
 *                 di = _apply_affine_2d_x0(dii, djj, 0, premult_disp)             # <<<<<<<<<<<<<<
 *                 dj = _apply_affine_2d_x1(dii, djj, 0, premult_disp)
 * 
 */
      /*else*/ {
        __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_premult_disp);

        /* "dipy/align/vector_fields.pyx":810
 *             else:
 *                 di = _apply_affine_2d_x0(dii, djj, 0, premult_disp)
 *                 dj = _apply_affine_2d_x1(dii, djj, 0, premult_disp)             # <<<<<<<<<<<<<<
 * 
 *             if premult_index is None:
 */
        __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_premult_disp);
      }
      __pyx_L7:;

      /* "dipy/align/vector_fields.pyx":812
 *                 dj = _apply_affine_2d_x1(dii, djj, 0, premult_disp)
 * 
 *             if premult_index is None:             # <<<<<<<<<<<<<<
 *                 diii = i
 *                 djjj = j
 */
      __pyx_t_11 = ((((PyObject *) __pyx_v_premult_index.memview) == Py_None) != 0);
      if (__pyx_t_11) {

        /* "dipy/align/vector_fields.pyx":813
 * 
 *             if premult_index is None:
 *                 diii = i             # <<<<<<<<<<<<<<
 *                 djjj = j
 *             else:
 */
        __pyx_v_diii = __pyx_v_i;

        /* "dipy/align/vector_fields.pyx":814
 *             if premult_index is None:
 *                 diii = i
 *                 djjj = j             # <<<<<<<<<<<<<<
 *             else:
 *                 diii = _apply_affine_2d_x0(i, j, 1, premult_index)
 */
        __pyx_v_djjj = __pyx_v_j;

        /* "dipy/align/vector_fields.pyx":812
 *                 dj = _apply_affine_2d_x1(dii, djj, 0, premult_disp)
 * 
 *             if premult_index is None:             # <<<<<<<<<<<<<<
 *                 diii = i
 *                 djjj = j
 */
        goto __pyx_L8;
      }

      /* "dipy/align/vector_fields.pyx":816
 *                 djjj = j
 *             else:
 *                 diii = _apply_affine_2d_x0(i, j, 1, premult_index)             # <<<<<<<<<<<<<<
 *                 djjj = _apply_affine_2d_x1(i, j, 1, premult_index)
 * 
 */
      /*else*/ {
        __pyx_v_diii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_premult_index);

        /* "dipy/align/vector_fields.pyx":817
 *             else:
 *                 diii = _apply_affine_2d_x0(i, j, 1, premult_index)
 *                 djjj = _apply_affine_2d_x1(i, j, 1, premult_index)             # <<<<<<<<<<<<<<
 * 
 *             diii += di
 */
        __pyx_v_djjj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_premult_index);
      }
      __pyx_L8:;

      /* "dipy/align/vector_fields.pyx":819
 *                 djjj = _apply_affine_2d_x1(i, j, 1, premult_index)
 * 
 *             diii += di             # <<<<<<<<<<<<<<
 *             djjj += dj
 * 
 */
      __pyx_v_diii = (__pyx_v_diii + __pyx_v_di);

      /* "dipy/align/vector_fields.pyx":820
 * 
 *             diii += di
 *             djjj += dj             # <<<<<<<<<<<<<<
 * 
 *             # If d1 and comp are the same array, this will correctly update
 */
      __pyx_v_djjj = (__pyx_v_djjj + __pyx_v_dj);

      /* "dipy/align/vector_fields.pyx":828
 *             # which may be problematic
 *             inside = _interpolate_vector_2d[floating](d2, diii, djjj,
 *                                                       comp[i, j])             # <<<<<<<<<<<<<<
 * 
 *             if inside == 1:
 */
      __pyx_t_12.data = __pyx_v_comp.data;
      __pyx_t_12.memview = __pyx_v_comp.memview;
      __PYX_INC_MEMVIEW(&__pyx_t_12, 0);
      {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_12.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_j;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[1];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[1];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 1)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_12.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_12.shape[0] = __pyx_v_comp.shape[2];
__pyx_t_12.strides[0] = __pyx_v_comp.strides[2];
    __pyx_t_12.suboffsets[0] = -1;

__pyx_v_inside = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d2, __pyx_v_diii, __pyx_v_djjj, __pyx_t_12);

      /* "dipy/align/vector_fields.pyx":827
 *             # in the neighborhood of a previously updated vector from d2,
 *             # which may be problematic
 *             inside = _interpolate_vector_2d[floating](d2, diii, djjj,             # <<<<<<<<<<<<<<
 *                                                       comp[i, j])
 * 
 */
      __PYX_XDEC_MEMVIEW(&__pyx_t_12, 0);

      /* "dipy/align/vector_fields.pyx":830
 *                                                       comp[i, j])
 * 
 *             if inside == 1:             # <<<<<<<<<<<<<<
 *                 comp[i, j, 0] = time_scaling * comp[i, j, 0] + dii
 *                 comp[i, j, 1] = time_scaling * comp[i, j, 1] + djj
 */
      __pyx_t_11 = ((__pyx_v_inside == 1) != 0);
      if (__pyx_t_11) {

        /* "dipy/align/vector_fields.pyx":831
 * 
 *             if inside == 1:
 *                 comp[i, j, 0] = time_scaling * comp[i, j, 0] + dii             # <<<<<<<<<<<<<<
 *                 comp[i, j, 1] = time_scaling * comp[i, j, 1] + djj
 *                 nn = comp[i, j, 0] ** 2 + comp[i, j, 1] ** 2
 */
        __pyx_t_13 = __pyx_v_i;
        __pyx_t_14 = __pyx_v_j;
        __pyx_t_15 = 0;
        __pyx_t_16 = __pyx_v_i;
        __pyx_t_17 = __pyx_v_j;
        __pyx_t_18 = 0;
        *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_16 * __pyx_v_comp.strides[0]) ) + __pyx_t_17 * __pyx_v_comp.strides[1]) ) + __pyx_t_18 * __pyx_v_comp.strides[2]) )) = ((__pyx_v_time_scaling * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_13 * __pyx_v_comp.strides[0]) ) + __pyx_t_14 * __pyx_v_comp.strides[1]) ) + __pyx_t_15 * __pyx_v_comp.strides[2]) )))) + __pyx_v_dii);

        /* "dipy/align/vector_fields.pyx":832
 *             if inside == 1:
 *                 comp[i, j, 0] = time_scaling * comp[i, j, 0] + dii
 *                 comp[i, j, 1] = time_scaling * comp[i, j, 1] + djj             # <<<<<<<<<<<<<<
 *                 nn = comp[i, j, 0] ** 2 + comp[i, j, 1] ** 2
 *                 meanNorm += nn
 */
        __pyx_t_19 = __pyx_v_i;
        __pyx_t_20 = __pyx_v_j;
        __pyx_t_21 = 1;
        __pyx_t_22 = __pyx_v_i;
        __pyx_t_23 = __pyx_v_j;
        __pyx_t_24 = 1;
        *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_22 * __pyx_v_comp.strides[0]) ) + __pyx_t_23 * __pyx_v_comp.strides[1]) ) + __pyx_t_24 * __pyx_v_comp.strides[2]) )) = ((__pyx_v_time_scaling * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_19 * __pyx_v_comp.strides[0]) ) + __pyx_t_20 * __pyx_v_comp.strides[1]) ) + __pyx_t_21 * __pyx_v_comp.strides[2]) )))) + __pyx_v_djj);

        /* "dipy/align/vector_fields.pyx":833
 *                 comp[i, j, 0] = time_scaling * comp[i, j, 0] + dii
 *                 comp[i, j, 1] = time_scaling * comp[i, j, 1] + djj
 *                 nn = comp[i, j, 0] ** 2 + comp[i, j, 1] ** 2             # <<<<<<<<<<<<<<
 *                 meanNorm += nn
 *                 stdNorm += nn * nn
 */
        __pyx_t_25 = __pyx_v_i;
        __pyx_t_26 = __pyx_v_j;
        __pyx_t_27 = 0;
        __pyx_t_28 = __pyx_v_i;
        __pyx_t_29 = __pyx_v_j;
        __pyx_t_30 = 1;
        __pyx_v_nn = (powf((*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_25 * __pyx_v_comp.strides[0]) ) + __pyx_t_26 * __pyx_v_comp.strides[1]) ) + __pyx_t_27 * __pyx_v_comp.strides[2]) ))), 2.0) + powf((*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_28 * __pyx_v_comp.strides[0]) ) + __pyx_t_29 * __pyx_v_comp.strides[1]) ) + __pyx_t_30 * __pyx_v_comp.strides[2]) ))), 2.0));

        /* "dipy/align/vector_fields.pyx":834
 *                 comp[i, j, 1] = time_scaling * comp[i, j, 1] + djj
 *                 nn = comp[i, j, 0] ** 2 + comp[i, j, 1] ** 2
 *                 meanNorm += nn             # <<<<<<<<<<<<<<
 *                 stdNorm += nn * nn
 *                 cnt += 1
 */
        __pyx_v_meanNorm = (__pyx_v_meanNorm + __pyx_v_nn);

        /* "dipy/align/vector_fields.pyx":835
 *                 nn = comp[i, j, 0] ** 2 + comp[i, j, 1] ** 2
 *                 meanNorm += nn
 *                 stdNorm += nn * nn             # <<<<<<<<<<<<<<
 *                 cnt += 1
 *                 if(maxNorm < nn):
 */
        __pyx_v_stdNorm = (__pyx_v_stdNorm + (__pyx_v_nn * __pyx_v_nn));

        /* "dipy/align/vector_fields.pyx":836
 *                 meanNorm += nn
 *                 stdNorm += nn * nn
 *                 cnt += 1             # <<<<<<<<<<<<<<
 *                 if(maxNorm < nn):
 *                     maxNorm = nn
 */
        __pyx_v_cnt = (__pyx_v_cnt + 1);

        /* "dipy/align/vector_fields.pyx":837
 *                 stdNorm += nn * nn
 *                 cnt += 1
 *                 if(maxNorm < nn):             # <<<<<<<<<<<<<<
 *                     maxNorm = nn
 *             else:
 */
        __pyx_t_11 = ((__pyx_v_maxNorm < __pyx_v_nn) != 0);
        if (__pyx_t_11) {

          /* "dipy/align/vector_fields.pyx":838
 *                 cnt += 1
 *                 if(maxNorm < nn):
 *                     maxNorm = nn             # <<<<<<<<<<<<<<
 *             else:
 *                 comp[i, j, :] = 0
 */
          __pyx_v_maxNorm = __pyx_v_nn;

          /* "dipy/align/vector_fields.pyx":837
 *                 stdNorm += nn * nn
 *                 cnt += 1
 *                 if(maxNorm < nn):             # <<<<<<<<<<<<<<
 *                     maxNorm = nn
 *             else:
 */
        }

        /* "dipy/align/vector_fields.pyx":830
 *                                                       comp[i, j])
 * 
 *             if inside == 1:             # <<<<<<<<<<<<<<
 *                 comp[i, j, 0] = time_scaling * comp[i, j, 0] + dii
 *                 comp[i, j, 1] = time_scaling * comp[i, j, 1] + djj
 */
        goto __pyx_L9;
      }

      /* "dipy/align/vector_fields.pyx":840
 *                     maxNorm = nn
 *             else:
 *                 comp[i, j, :] = 0             # <<<<<<<<<<<<<<
 *     meanNorm /= cnt
 *     stats[0] = sqrt(maxNorm)
 */
      /*else*/ {
        __pyx_t_12.data = __pyx_v_comp.data;
        __pyx_t_12.memview = __pyx_v_comp.memview;
        __PYX_INC_MEMVIEW(&__pyx_t_12, 0);
        {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_12.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_j;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[1];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[1];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 1)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_12.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_12.shape[0] = __pyx_v_comp.shape[2];
__pyx_t_12.strides[0] = __pyx_v_comp.strides[2];
    __pyx_t_12.suboffsets[0] = -1;

{
            float __pyx_temp_scalar = 0.0;
            {
                Py_ssize_t __pyx_temp_extent_0 = __pyx_t_12.shape[0];
                Py_ssize_t __pyx_temp_stride_0 = __pyx_t_12.strides[0];
                char *__pyx_temp_pointer_0;
                Py_ssize_t __pyx_temp_idx_0;
                __pyx_temp_pointer_0 = __pyx_t_12.data;
                for (__pyx_temp_idx_0 = 0; __pyx_temp_idx_0 < __pyx_temp_extent_0; __pyx_temp_idx_0++) {
                  *((float *) __pyx_temp_pointer_0) = __pyx_temp_scalar;
                  __pyx_temp_pointer_0 += __pyx_temp_stride_0;
                }
            }
        }
        __PYX_XDEC_MEMVIEW(&__pyx_t_12, 0);
      }
      __pyx_L9:;
    }
  }

  /* "dipy/align/vector_fields.pyx":841
 *             else:
 *                 comp[i, j, :] = 0
 *     meanNorm /= cnt             # <<<<<<<<<<<<<<
 *     stats[0] = sqrt(maxNorm)
 *     stats[1] = sqrt(meanNorm)
 */
  __pyx_v_meanNorm = (__pyx_v_meanNorm / __pyx_v_cnt);

  /* "dipy/align/vector_fields.pyx":842
 *                 comp[i, j, :] = 0
 *     meanNorm /= cnt
 *     stats[0] = sqrt(maxNorm)             # <<<<<<<<<<<<<<
 *     stats[1] = sqrt(meanNorm)
 *     stats[2] = sqrt(stdNorm / cnt - meanNorm * meanNorm)
 */
  __pyx_t_31 = 0;
  *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_31 * __pyx_v_stats.strides[0]) )) = sqrt(__pyx_v_maxNorm);

  /* "dipy/align/vector_fields.pyx":843
 *     meanNorm /= cnt
 *     stats[0] = sqrt(maxNorm)
 *     stats[1] = sqrt(meanNorm)             # <<<<<<<<<<<<<<
 *     stats[2] = sqrt(stdNorm / cnt - meanNorm * meanNorm)
 * 
 */
  __pyx_t_32 = 1;
  *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_32 * __pyx_v_stats.strides[0]) )) = sqrt(__pyx_v_meanNorm);

  /* "dipy/align/vector_fields.pyx":844
 *     stats[0] = sqrt(maxNorm)
 *     stats[1] = sqrt(meanNorm)
 *     stats[2] = sqrt(stdNorm / cnt - meanNorm * meanNorm)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_33 = 2;
  *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_33 * __pyx_v_stats.strides[0]) )) = sqrt(((__pyx_v_stdNorm / __pyx_v_cnt) - (__pyx_v_meanNorm * __pyx_v_meanNorm)));

  /* "dipy/align/vector_fields.pyx":718
 * 
 * 
 * cdef void _compose_vector_fields_2d(floating[:, :, :] d1, floating[:, :, :] d2,             # <<<<<<<<<<<<<<
 *                                     double[:, :] premult_index,
 *                                     double[:, :] premult_disp,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __PYX_XDEC_MEMVIEW(&__pyx_t_12, 0);
  __Pyx_WriteUnraisable("dipy.align.vector_fields._compose_vector_fields_2d", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 1);
  __pyx_L0:;
}

static void __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__compose_vector_fields_2d(__Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_d2, __Pyx_memviewslice __pyx_v_premult_index, __Pyx_memviewslice __pyx_v_premult_disp, double __pyx_v_time_scaling, __Pyx_memviewslice __pyx_v_comp, __Pyx_memviewslice __pyx_v_stats) {
  npy_intp __pyx_v_nr1;
  npy_intp __pyx_v_nc1;
  CYTHON_UNUSED npy_intp __pyx_v_nr2;
  CYTHON_UNUSED npy_intp __pyx_v_nc2;
  int __pyx_v_inside;
  int __pyx_v_cnt;
  double __pyx_v_maxNorm;
  double __pyx_v_meanNorm;
  double __pyx_v_stdNorm;
  double __pyx_v_nn;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_diii;
  double __pyx_v_djjj;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;
  npy_intp __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  __Pyx_memviewslice __pyx_t_12 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "dipy/align/vector_fields.pyx":786
 *     """
 *     cdef:
 *         cnp.npy_intp nr1 = d1.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc1 = d1.shape[1]
 *         cnp.npy_intp nr2 = d2.shape[0]
 */
  __pyx_v_nr1 = (__pyx_v_d1.shape[0]);

  /* "dipy/align/vector_fields.pyx":787
 *     cdef:
 *         cnp.npy_intp nr1 = d1.shape[0]
 *         cnp.npy_intp nc1 = d1.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr2 = d2.shape[0]
 *         cnp.npy_intp nc2 = d2.shape[1]
 */
  __pyx_v_nc1 = (__pyx_v_d1.shape[1]);

  /* "dipy/align/vector_fields.pyx":788
 *         cnp.npy_intp nr1 = d1.shape[0]
 *         cnp.npy_intp nc1 = d1.shape[1]
 *         cnp.npy_intp nr2 = d2.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc2 = d2.shape[1]
 *         int inside, cnt = 0
 */
  __pyx_v_nr2 = (__pyx_v_d2.shape[0]);

  /* "dipy/align/vector_fields.pyx":789
 *         cnp.npy_intp nc1 = d1.shape[1]
 *         cnp.npy_intp nr2 = d2.shape[0]
 *         cnp.npy_intp nc2 = d2.shape[1]             # <<<<<<<<<<<<<<
 *         int inside, cnt = 0
 *         double maxNorm = 0
 */
  __pyx_v_nc2 = (__pyx_v_d2.shape[1]);

  /* "dipy/align/vector_fields.pyx":790
 *         cnp.npy_intp nr2 = d2.shape[0]
 *         cnp.npy_intp nc2 = d2.shape[1]
 *         int inside, cnt = 0             # <<<<<<<<<<<<<<
 *         double maxNorm = 0
 *         double meanNorm = 0
 */
  __pyx_v_cnt = 0;

  /* "dipy/align/vector_fields.pyx":791
 *         cnp.npy_intp nc2 = d2.shape[1]
 *         int inside, cnt = 0
 *         double maxNorm = 0             # <<<<<<<<<<<<<<
 *         double meanNorm = 0
 *         double stdNorm = 0
 */
  __pyx_v_maxNorm = 0.0;

  /* "dipy/align/vector_fields.pyx":792
 *         int inside, cnt = 0
 *         double maxNorm = 0
 *         double meanNorm = 0             # <<<<<<<<<<<<<<
 *         double stdNorm = 0
 *         double nn
 */
  __pyx_v_meanNorm = 0.0;

  /* "dipy/align/vector_fields.pyx":793
 *         double maxNorm = 0
 *         double meanNorm = 0
 *         double stdNorm = 0             # <<<<<<<<<<<<<<
 *         double nn
 *         cnp.npy_intp i, j
 */
  __pyx_v_stdNorm = 0.0;

  /* "dipy/align/vector_fields.pyx":798
 *         double di, dj, dii, djj, diii, djjj
 * 
 *     for i in range(nr1):             # <<<<<<<<<<<<<<
 *         for j in range(nc1):
 * 
 */
  __pyx_t_1 = __pyx_v_nr1;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "dipy/align/vector_fields.pyx":799
 * 
 *     for i in range(nr1):
 *         for j in range(nc1):             # <<<<<<<<<<<<<<
 * 
 *             # This is the only place we access d1[i, j]
 */
    __pyx_t_3 = __pyx_v_nc1;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_j = __pyx_t_4;

      /* "dipy/align/vector_fields.pyx":802
 * 
 *             # This is the only place we access d1[i, j]
 *             dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *             djj = d1[i, j, 1]
 * 
 */
      __pyx_t_5 = __pyx_v_i;
      __pyx_t_6 = __pyx_v_j;
      __pyx_t_7 = 0;
      __pyx_v_dii = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_5 * __pyx_v_d1.strides[0]) ) + __pyx_t_6 * __pyx_v_d1.strides[1]) ) + __pyx_t_7 * __pyx_v_d1.strides[2]) )));

      /* "dipy/align/vector_fields.pyx":803
 *             # This is the only place we access d1[i, j]
 *             dii = d1[i, j, 0]
 *             djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 * 
 *             if premult_disp is None:
 */
      __pyx_t_8 = __pyx_v_i;
      __pyx_t_9 = __pyx_v_j;
      __pyx_t_10 = 1;
      __pyx_v_djj = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_8 * __pyx_v_d1.strides[0]) ) + __pyx_t_9 * __pyx_v_d1.strides[1]) ) + __pyx_t_10 * __pyx_v_d1.strides[2]) )));

      /* "dipy/align/vector_fields.pyx":805
 *             djj = d1[i, j, 1]
 * 
 *             if premult_disp is None:             # <<<<<<<<<<<<<<
 *                 di = dii
 *                 dj = djj
 */
      __pyx_t_11 = ((((PyObject *) __pyx_v_premult_disp.memview) == Py_None) != 0);
      if (__pyx_t_11) {

        /* "dipy/align/vector_fields.pyx":806
 * 
 *             if premult_disp is None:
 *                 di = dii             # <<<<<<<<<<<<<<
 *                 dj = djj
 *             else:
 */
        __pyx_v_di = __pyx_v_dii;

        /* "dipy/align/vector_fields.pyx":807
 *             if premult_disp is None:
 *                 di = dii
 *                 dj = djj             # <<<<<<<<<<<<<<
 *             else:
 *                 di = _apply_affine_2d_x0(dii, djj, 0, premult_disp)
 */
        __pyx_v_dj = __pyx_v_djj;

        /* "dipy/align/vector_fields.pyx":805
 *             djj = d1[i, j, 1]
 * 
 *             if premult_disp is None:             # <<<<<<<<<<<<<<
 *                 di = dii
 *                 dj = djj
 */
        goto __pyx_L7;
      }

      /* "dipy/align/vector_fields.pyx":809
 *                 dj = djj
 *             else:
 *                 di = _apply_affine_2d_x0(dii, djj, 0, premult_disp)             # <<<<<<<<<<<<<<
 *                 dj = _apply_affine_2d_x1(dii, djj, 0, premult_disp)
 * 
 */
      /*else*/ {
        __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_premult_disp);

        /* "dipy/align/vector_fields.pyx":810
 *             else:
 *                 di = _apply_affine_2d_x0(dii, djj, 0, premult_disp)
 *                 dj = _apply_affine_2d_x1(dii, djj, 0, premult_disp)             # <<<<<<<<<<<<<<
 * 
 *             if premult_index is None:
 */
        __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_premult_disp);
      }
      __pyx_L7:;

      /* "dipy/align/vector_fields.pyx":812
 *                 dj = _apply_affine_2d_x1(dii, djj, 0, premult_disp)
 * 
 *             if premult_index is None:             # <<<<<<<<<<<<<<
 *                 diii = i
 *                 djjj = j
 */
      __pyx_t_11 = ((((PyObject *) __pyx_v_premult_index.memview) == Py_None) != 0);
      if (__pyx_t_11) {

        /* "dipy/align/vector_fields.pyx":813
 * 
 *             if premult_index is None:
 *                 diii = i             # <<<<<<<<<<<<<<
 *                 djjj = j
 *             else:
 */
        __pyx_v_diii = __pyx_v_i;

        /* "dipy/align/vector_fields.pyx":814
 *             if premult_index is None:
 *                 diii = i
 *                 djjj = j             # <<<<<<<<<<<<<<
 *             else:
 *                 diii = _apply_affine_2d_x0(i, j, 1, premult_index)
 */
        __pyx_v_djjj = __pyx_v_j;

        /* "dipy/align/vector_fields.pyx":812
 *                 dj = _apply_affine_2d_x1(dii, djj, 0, premult_disp)
 * 
 *             if premult_index is None:             # <<<<<<<<<<<<<<
 *                 diii = i
 *                 djjj = j
 */
        goto __pyx_L8;
      }

      /* "dipy/align/vector_fields.pyx":816
 *                 djjj = j
 *             else:
 *                 diii = _apply_affine_2d_x0(i, j, 1, premult_index)             # <<<<<<<<<<<<<<
 *                 djjj = _apply_affine_2d_x1(i, j, 1, premult_index)
 * 
 */
      /*else*/ {
        __pyx_v_diii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_premult_index);

        /* "dipy/align/vector_fields.pyx":817
 *             else:
 *                 diii = _apply_affine_2d_x0(i, j, 1, premult_index)
 *                 djjj = _apply_affine_2d_x1(i, j, 1, premult_index)             # <<<<<<<<<<<<<<
 * 
 *             diii += di
 */
        __pyx_v_djjj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_premult_index);
      }
      __pyx_L8:;

      /* "dipy/align/vector_fields.pyx":819
 *                 djjj = _apply_affine_2d_x1(i, j, 1, premult_index)
 * 
 *             diii += di             # <<<<<<<<<<<<<<
 *             djjj += dj
 * 
 */
      __pyx_v_diii = (__pyx_v_diii + __pyx_v_di);

      /* "dipy/align/vector_fields.pyx":820
 * 
 *             diii += di
 *             djjj += dj             # <<<<<<<<<<<<<<
 * 
 *             # If d1 and comp are the same array, this will correctly update
 */
      __pyx_v_djjj = (__pyx_v_djjj + __pyx_v_dj);

      /* "dipy/align/vector_fields.pyx":828
 *             # which may be problematic
 *             inside = _interpolate_vector_2d[floating](d2, diii, djjj,
 *                                                       comp[i, j])             # <<<<<<<<<<<<<<
 * 
 *             if inside == 1:
 */
      __pyx_t_12.data = __pyx_v_comp.data;
      __pyx_t_12.memview = __pyx_v_comp.memview;
      __PYX_INC_MEMVIEW(&__pyx_t_12, 0);
      {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_12.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_j;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[1];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[1];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 1)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_12.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_12.shape[0] = __pyx_v_comp.shape[2];
__pyx_t_12.strides[0] = __pyx_v_comp.strides[2];
    __pyx_t_12.suboffsets[0] = -1;

__pyx_v_inside = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d2, __pyx_v_diii, __pyx_v_djjj, __pyx_t_12);

      /* "dipy/align/vector_fields.pyx":827
 *             # in the neighborhood of a previously updated vector from d2,
 *             # which may be problematic
 *             inside = _interpolate_vector_2d[floating](d2, diii, djjj,             # <<<<<<<<<<<<<<
 *                                                       comp[i, j])
 * 
 */
      __PYX_XDEC_MEMVIEW(&__pyx_t_12, 0);

      /* "dipy/align/vector_fields.pyx":830
 *                                                       comp[i, j])
 * 
 *             if inside == 1:             # <<<<<<<<<<<<<<
 *                 comp[i, j, 0] = time_scaling * comp[i, j, 0] + dii
 *                 comp[i, j, 1] = time_scaling * comp[i, j, 1] + djj
 */
      __pyx_t_11 = ((__pyx_v_inside == 1) != 0);
      if (__pyx_t_11) {

        /* "dipy/align/vector_fields.pyx":831
 * 
 *             if inside == 1:
 *                 comp[i, j, 0] = time_scaling * comp[i, j, 0] + dii             # <<<<<<<<<<<<<<
 *                 comp[i, j, 1] = time_scaling * comp[i, j, 1] + djj
 *                 nn = comp[i, j, 0] ** 2 + comp[i, j, 1] ** 2
 */
        __pyx_t_13 = __pyx_v_i;
        __pyx_t_14 = __pyx_v_j;
        __pyx_t_15 = 0;
        __pyx_t_16 = __pyx_v_i;
        __pyx_t_17 = __pyx_v_j;
        __pyx_t_18 = 0;
        *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_16 * __pyx_v_comp.strides[0]) ) + __pyx_t_17 * __pyx_v_comp.strides[1]) ) + __pyx_t_18 * __pyx_v_comp.strides[2]) )) = ((__pyx_v_time_scaling * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_13 * __pyx_v_comp.strides[0]) ) + __pyx_t_14 * __pyx_v_comp.strides[1]) ) + __pyx_t_15 * __pyx_v_comp.strides[2]) )))) + __pyx_v_dii);

        /* "dipy/align/vector_fields.pyx":832
 *             if inside == 1:
 *                 comp[i, j, 0] = time_scaling * comp[i, j, 0] + dii
 *                 comp[i, j, 1] = time_scaling * comp[i, j, 1] + djj             # <<<<<<<<<<<<<<
 *                 nn = comp[i, j, 0] ** 2 + comp[i, j, 1] ** 2
 *                 meanNorm += nn
 */
        __pyx_t_19 = __pyx_v_i;
        __pyx_t_20 = __pyx_v_j;
        __pyx_t_21 = 1;
        __pyx_t_22 = __pyx_v_i;
        __pyx_t_23 = __pyx_v_j;
        __pyx_t_24 = 1;
        *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_22 * __pyx_v_comp.strides[0]) ) + __pyx_t_23 * __pyx_v_comp.strides[1]) ) + __pyx_t_24 * __pyx_v_comp.strides[2]) )) = ((__pyx_v_time_scaling * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_19 * __pyx_v_comp.strides[0]) ) + __pyx_t_20 * __pyx_v_comp.strides[1]) ) + __pyx_t_21 * __pyx_v_comp.strides[2]) )))) + __pyx_v_djj);

        /* "dipy/align/vector_fields.pyx":833
 *                 comp[i, j, 0] = time_scaling * comp[i, j, 0] + dii
 *                 comp[i, j, 1] = time_scaling * comp[i, j, 1] + djj
 *                 nn = comp[i, j, 0] ** 2 + comp[i, j, 1] ** 2             # <<<<<<<<<<<<<<
 *                 meanNorm += nn
 *                 stdNorm += nn * nn
 */
        __pyx_t_25 = __pyx_v_i;
        __pyx_t_26 = __pyx_v_j;
        __pyx_t_27 = 0;
        __pyx_t_28 = __pyx_v_i;
        __pyx_t_29 = __pyx_v_j;
        __pyx_t_30 = 1;
        __pyx_v_nn = (pow((*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_25 * __pyx_v_comp.strides[0]) ) + __pyx_t_26 * __pyx_v_comp.strides[1]) ) + __pyx_t_27 * __pyx_v_comp.strides[2]) ))), 2.0) + pow((*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_28 * __pyx_v_comp.strides[0]) ) + __pyx_t_29 * __pyx_v_comp.strides[1]) ) + __pyx_t_30 * __pyx_v_comp.strides[2]) ))), 2.0));

        /* "dipy/align/vector_fields.pyx":834
 *                 comp[i, j, 1] = time_scaling * comp[i, j, 1] + djj
 *                 nn = comp[i, j, 0] ** 2 + comp[i, j, 1] ** 2
 *                 meanNorm += nn             # <<<<<<<<<<<<<<
 *                 stdNorm += nn * nn
 *                 cnt += 1
 */
        __pyx_v_meanNorm = (__pyx_v_meanNorm + __pyx_v_nn);

        /* "dipy/align/vector_fields.pyx":835
 *                 nn = comp[i, j, 0] ** 2 + comp[i, j, 1] ** 2
 *                 meanNorm += nn
 *                 stdNorm += nn * nn             # <<<<<<<<<<<<<<
 *                 cnt += 1
 *                 if(maxNorm < nn):
 */
        __pyx_v_stdNorm = (__pyx_v_stdNorm + (__pyx_v_nn * __pyx_v_nn));

        /* "dipy/align/vector_fields.pyx":836
 *                 meanNorm += nn
 *                 stdNorm += nn * nn
 *                 cnt += 1             # <<<<<<<<<<<<<<
 *                 if(maxNorm < nn):
 *                     maxNorm = nn
 */
        __pyx_v_cnt = (__pyx_v_cnt + 1);

        /* "dipy/align/vector_fields.pyx":837
 *                 stdNorm += nn * nn
 *                 cnt += 1
 *                 if(maxNorm < nn):             # <<<<<<<<<<<<<<
 *                     maxNorm = nn
 *             else:
 */
        __pyx_t_11 = ((__pyx_v_maxNorm < __pyx_v_nn) != 0);
        if (__pyx_t_11) {

          /* "dipy/align/vector_fields.pyx":838
 *                 cnt += 1
 *                 if(maxNorm < nn):
 *                     maxNorm = nn             # <<<<<<<<<<<<<<
 *             else:
 *                 comp[i, j, :] = 0
 */
          __pyx_v_maxNorm = __pyx_v_nn;

          /* "dipy/align/vector_fields.pyx":837
 *                 stdNorm += nn * nn
 *                 cnt += 1
 *                 if(maxNorm < nn):             # <<<<<<<<<<<<<<
 *                     maxNorm = nn
 *             else:
 */
        }

        /* "dipy/align/vector_fields.pyx":830
 *                                                       comp[i, j])
 * 
 *             if inside == 1:             # <<<<<<<<<<<<<<
 *                 comp[i, j, 0] = time_scaling * comp[i, j, 0] + dii
 *                 comp[i, j, 1] = time_scaling * comp[i, j, 1] + djj
 */
        goto __pyx_L9;
      }

      /* "dipy/align/vector_fields.pyx":840
 *                     maxNorm = nn
 *             else:
 *                 comp[i, j, :] = 0             # <<<<<<<<<<<<<<
 *     meanNorm /= cnt
 *     stats[0] = sqrt(maxNorm)
 */
      /*else*/ {
        __pyx_t_12.data = __pyx_v_comp.data;
        __pyx_t_12.memview = __pyx_v_comp.memview;
        __PYX_INC_MEMVIEW(&__pyx_t_12, 0);
        {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_12.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_j;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[1];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[1];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 1)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_12.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_12.shape[0] = __pyx_v_comp.shape[2];
__pyx_t_12.strides[0] = __pyx_v_comp.strides[2];
    __pyx_t_12.suboffsets[0] = -1;

{
            double __pyx_temp_scalar = 0.0;
            {
                Py_ssize_t __pyx_temp_extent_0 = __pyx_t_12.shape[0];
                Py_ssize_t __pyx_temp_stride_0 = __pyx_t_12.strides[0];
                char *__pyx_temp_pointer_0;
                Py_ssize_t __pyx_temp_idx_0;
                __pyx_temp_pointer_0 = __pyx_t_12.data;
                for (__pyx_temp_idx_0 = 0; __pyx_temp_idx_0 < __pyx_temp_extent_0; __pyx_temp_idx_0++) {
                  *((double *) __pyx_temp_pointer_0) = __pyx_temp_scalar;
                  __pyx_temp_pointer_0 += __pyx_temp_stride_0;
                }
            }
        }
        __PYX_XDEC_MEMVIEW(&__pyx_t_12, 0);
      }
      __pyx_L9:;
    }
  }

  /* "dipy/align/vector_fields.pyx":841
 *             else:
 *                 comp[i, j, :] = 0
 *     meanNorm /= cnt             # <<<<<<<<<<<<<<
 *     stats[0] = sqrt(maxNorm)
 *     stats[1] = sqrt(meanNorm)
 */
  __pyx_v_meanNorm = (__pyx_v_meanNorm / __pyx_v_cnt);

  /* "dipy/align/vector_fields.pyx":842
 *                 comp[i, j, :] = 0
 *     meanNorm /= cnt
 *     stats[0] = sqrt(maxNorm)             # <<<<<<<<<<<<<<
 *     stats[1] = sqrt(meanNorm)
 *     stats[2] = sqrt(stdNorm / cnt - meanNorm * meanNorm)
 */
  __pyx_t_31 = 0;
  *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_31 * __pyx_v_stats.strides[0]) )) = sqrt(__pyx_v_maxNorm);

  /* "dipy/align/vector_fields.pyx":843
 *     meanNorm /= cnt
 *     stats[0] = sqrt(maxNorm)
 *     stats[1] = sqrt(meanNorm)             # <<<<<<<<<<<<<<
 *     stats[2] = sqrt(stdNorm / cnt - meanNorm * meanNorm)
 * 
 */
  __pyx_t_32 = 1;
  *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_32 * __pyx_v_stats.strides[0]) )) = sqrt(__pyx_v_meanNorm);

  /* "dipy/align/vector_fields.pyx":844
 *     stats[0] = sqrt(maxNorm)
 *     stats[1] = sqrt(meanNorm)
 *     stats[2] = sqrt(stdNorm / cnt - meanNorm * meanNorm)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_33 = 2;
  *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_33 * __pyx_v_stats.strides[0]) )) = sqrt(((__pyx_v_stdNorm / __pyx_v_cnt) - (__pyx_v_meanNorm * __pyx_v_meanNorm)));

  /* "dipy/align/vector_fields.pyx":718
 * 
 * 
 * cdef void _compose_vector_fields_2d(floating[:, :, :] d1, floating[:, :, :] d2,             # <<<<<<<<<<<<<<
 *                                     double[:, :] premult_index,
 *                                     double[:, :] premult_disp,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __PYX_XDEC_MEMVIEW(&__pyx_t_12, 0);
  __Pyx_WriteUnraisable("dipy.align.vector_fields._compose_vector_fields_2d", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 1);
  __pyx_L0:;
}

/* "dipy/align/vector_fields.pyx":847
 * 
 * 
 * def compose_vector_fields_2d(floating[:, :, :] d1, floating[:, :, :] d2,             # <<<<<<<<<<<<<<
 *                              double[:, :] premult_index,
 *                              double[:, :] premult_disp,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_15compose_vector_fields_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_14compose_vector_fields_2d[] = "Computes the composition of two 2D displacement fields\n\n    Computes the composition of the two 2-D displacements d1 and d2. The\n    evaluation of d2 at non-lattice points is computed using tri-linear\n    interpolation. The actual composition is computed as:\n\n    comp[i] = d1[i] + t * d2[ A * i + B * d1[i] ]\n\n    where t = time_scaling, A = premult_index and B=premult_disp and i denotes\n    the voxel coordinates of a voxel in d1's grid. Using this parameters it is\n    possible to compose vector fields with arbitrary discretizations: let R and\n    S be the voxel-to-space transformation associated to d1 and d2,\n    respectively then the composition at a voxel with coordinates i in d1's\n    grid is given by:\n\n    comp[i] = d1[i] + R*i + d2[Sinv*(R*i + d1[i])] - R*i\n\n    (the R*i terms cancel each other) where Sinv = S^{-1}\n    we can then define A = Sinv * R and B = Sinv to compute the composition\n    using this function.\n\n    Parameters\n    ----------\n    d1 : array, shape (R, C, 2)\n        first displacement field to be applied. R, C are the number of rows\n        and columns of the displacement field, respectively.\n    d2 : array, shape (R', C', 2)\n        second displacement field to be applied. R', C' are the number of rows\n        and columns of the displacement field, respectively.\n    premult_index : array, shape (3, 3)\n        the matrix A in the explanation above\n    premult_disp : array, shape (3, 3)\n        the matrix B in the explanation above\n    time_scaling : float\n        this corresponds to the time scaling 't' in the above explanation\n    comp : array, shape (R, C, 2)\n        the buffer to write the composition to. If None, the buffer is created\n        internally\n\n    Returns\n    -------\n    comp : array, shape (R, C, 2), same dimension as d1\n        on output, this array will contain the composition of the two fields\n    stats : array, shape (3,)\n        on output, this array will contain three statistics"" of the vector norms\n        of the composition (maximum, mean, standard_deviation)\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_15compose_vector_fields_2d = {"compose_vector_fields_2d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_15compose_vector_fields_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_14compose_vector_fields_2d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_15compose_vector_fields_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_14compose_vector_fields_2d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_14compose_vector_fields_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("compose_vector_fields_2d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_d1, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_d1); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__27, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__28, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__29, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__30, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_139compose_vector_fields_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_139compose_vector_fields_2d = {"__pyx_fuse_0compose_vector_fields_2d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_139compose_vector_fields_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_14compose_vector_fields_2d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_139compose_vector_fields_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d2 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_premult_index = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_premult_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  double __pyx_v_time_scaling;
  __Pyx_memviewslice __pyx_v_comp = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compose_vector_fields_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_d1,&__pyx_n_s_d2,&__pyx_n_s_premult_index,&__pyx_n_s_premult_disp,&__pyx_n_s_time_scaling,&__pyx_n_s_comp,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_2d", 1, 6, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_premult_index)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_2d", 1, 6, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_premult_disp)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_2d", 1, 6, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_time_scaling)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_2d", 1, 6, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_comp)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_2d", 1, 6, 6, 5); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "compose_vector_fields_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[0]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d2 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[1]); if (unlikely(!__pyx_v_d2.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_premult_index = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_premult_index.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 848; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_premult_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_premult_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_time_scaling = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_time_scaling == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 850; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_comp = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[5]); if (unlikely(!__pyx_v_comp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 851; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("compose_vector_fields_2d", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.compose_vector_fields_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_138compose_vector_fields_2d(__pyx_self, __pyx_v_d1, __pyx_v_d2, __pyx_v_premult_index, __pyx_v_premult_disp, __pyx_v_time_scaling, __pyx_v_comp);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_138compose_vector_fields_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_d2, __Pyx_memviewslice __pyx_v_premult_index, __Pyx_memviewslice __pyx_v_premult_disp, double __pyx_v_time_scaling, __Pyx_memviewslice __pyx_v_comp) {
  __Pyx_memviewslice __pyx_v_stats = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_memviewslice __pyx_t_8 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0compose_vector_fields_2d", 0);

  /* "dipy/align/vector_fields.pyx":900
 *     """
 *     cdef:
 *         double[:] stats = np.zeros(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 * 
 *     if comp is None:
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_tuple__31) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_float64); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_stats = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":902
 *         double[:] stats = np.zeros(shape=(3,), dtype=np.float64)
 * 
 *     if comp is None:             # <<<<<<<<<<<<<<
 *         comp = np.zeros_like(d1)
 * 
 */
  __pyx_t_6 = ((((PyObject *) __pyx_v_comp.memview) == Py_None) != 0);
  if (__pyx_t_6) {

    /* "dipy/align/vector_fields.pyx":903
 * 
 *     if comp is None:
 *         comp = np.zeros_like(d1)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(premult_index, 2):
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros_like); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_3) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else {
      __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_t_4);
    if (unlikely(!__pyx_t_8.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_XDEC_MEMVIEW(&__pyx_v_comp, 1);
    __pyx_v_comp = __pyx_t_8;
    __pyx_t_8.memview = NULL;
    __pyx_t_8.data = NULL;

    /* "dipy/align/vector_fields.pyx":902
 *         double[:] stats = np.zeros(shape=(3,), dtype=np.float64)
 * 
 *     if comp is None:             # <<<<<<<<<<<<<<
 *         comp = np.zeros_like(d1)
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":905
 *         comp = np.zeros_like(d1)
 * 
 *     if not is_valid_affine(premult_index, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid index multiplication matrix")
 *     if not is_valid_affine(premult_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 905; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_premult_index, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 905; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = NULL;
  __pyx_t_9 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_9 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 905; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_1) {
    __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1); __pyx_t_1 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_9, __pyx_t_7);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_9, __pyx_int_2);
  __pyx_t_7 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 905; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 905; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_10 = ((!__pyx_t_6) != 0);
  if (__pyx_t_10) {

    /* "dipy/align/vector_fields.pyx":906
 * 
 *     if not is_valid_affine(premult_index, 2):
 *         raise ValueError("Invalid index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(premult_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__32, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 906; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 906; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":905
 *         comp = np.zeros_like(d1)
 * 
 *     if not is_valid_affine(premult_index, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid index multiplication matrix")
 *     if not is_valid_affine(premult_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":907
 *     if not is_valid_affine(premult_index, 2):
 *         raise ValueError("Invalid index multiplication matrix")
 *     if not is_valid_affine(premult_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 907; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_premult_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 907; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  __pyx_t_9 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_9 = 1;
    }
  }
  __pyx_t_1 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 907; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (__pyx_t_7) {
    __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_7); __pyx_t_7 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_9, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_9, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 907; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_10 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 907; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = ((!__pyx_t_10) != 0);
  if (__pyx_t_6) {

    /* "dipy/align/vector_fields.pyx":908
 *         raise ValueError("Invalid index multiplication matrix")
 *     if not is_valid_affine(premult_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     _compose_vector_fields_2d[floating](d1, d2, premult_index, premult_disp,
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__33, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 908; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 908; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":907
 *     if not is_valid_affine(premult_index, 2):
 *         raise ValueError("Invalid index multiplication matrix")
 *     if not is_valid_affine(premult_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":910
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     _compose_vector_fields_2d[floating](d1, d2, premult_index, premult_disp,             # <<<<<<<<<<<<<<
 *                                         time_scaling, comp, stats)
 *     return comp, stats
 */
  __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__compose_vector_fields_2d(__pyx_v_d1, __pyx_v_d2, __pyx_v_premult_index, __pyx_v_premult_disp, __pyx_v_time_scaling, __pyx_v_comp, __pyx_v_stats);

  /* "dipy/align/vector_fields.pyx":912
 *     _compose_vector_fields_2d[floating](d1, d2, premult_index, premult_disp,
 *                                         time_scaling, comp, stats)
 *     return comp, stats             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_comp, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_stats, 1, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
  __pyx_t_4 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":847
 * 
 * 
 * def compose_vector_fields_2d(floating[:, :, :] d1, floating[:, :, :] d2,             # <<<<<<<<<<<<<<
 *                              double[:, :] premult_index,
 *                              double[:, :] premult_disp,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __Pyx_XDECREF(__pyx_t_7);
  __PYX_XDEC_MEMVIEW(&__pyx_t_8, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.compose_vector_fields_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_stats, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d2, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_premult_index, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_premult_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_comp, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_141compose_vector_fields_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_141compose_vector_fields_2d = {"__pyx_fuse_1compose_vector_fields_2d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_141compose_vector_fields_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_14compose_vector_fields_2d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_141compose_vector_fields_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d2 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_premult_index = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_premult_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  double __pyx_v_time_scaling;
  __Pyx_memviewslice __pyx_v_comp = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compose_vector_fields_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_d1,&__pyx_n_s_d2,&__pyx_n_s_premult_index,&__pyx_n_s_premult_disp,&__pyx_n_s_time_scaling,&__pyx_n_s_comp,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_2d", 1, 6, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_premult_index)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_2d", 1, 6, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_premult_disp)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_2d", 1, 6, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_time_scaling)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_2d", 1, 6, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_comp)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_2d", 1, 6, 6, 5); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "compose_vector_fields_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[0]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d2 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[1]); if (unlikely(!__pyx_v_d2.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_premult_index = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_premult_index.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 848; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_premult_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_premult_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_time_scaling = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_time_scaling == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 850; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_comp = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[5]); if (unlikely(!__pyx_v_comp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 851; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("compose_vector_fields_2d", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.compose_vector_fields_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_140compose_vector_fields_2d(__pyx_self, __pyx_v_d1, __pyx_v_d2, __pyx_v_premult_index, __pyx_v_premult_disp, __pyx_v_time_scaling, __pyx_v_comp);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_140compose_vector_fields_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_d2, __Pyx_memviewslice __pyx_v_premult_index, __Pyx_memviewslice __pyx_v_premult_disp, double __pyx_v_time_scaling, __Pyx_memviewslice __pyx_v_comp) {
  __Pyx_memviewslice __pyx_v_stats = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_memviewslice __pyx_t_8 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1compose_vector_fields_2d", 0);

  /* "dipy/align/vector_fields.pyx":900
 *     """
 *     cdef:
 *         double[:] stats = np.zeros(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 * 
 *     if comp is None:
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_tuple__34) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_float64); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_stats = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":902
 *         double[:] stats = np.zeros(shape=(3,), dtype=np.float64)
 * 
 *     if comp is None:             # <<<<<<<<<<<<<<
 *         comp = np.zeros_like(d1)
 * 
 */
  __pyx_t_6 = ((((PyObject *) __pyx_v_comp.memview) == Py_None) != 0);
  if (__pyx_t_6) {

    /* "dipy/align/vector_fields.pyx":903
 * 
 *     if comp is None:
 *         comp = np.zeros_like(d1)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(premult_index, 2):
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros_like); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_3) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else {
      __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_4);
    if (unlikely(!__pyx_t_8.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 903; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_XDEC_MEMVIEW(&__pyx_v_comp, 1);
    __pyx_v_comp = __pyx_t_8;
    __pyx_t_8.memview = NULL;
    __pyx_t_8.data = NULL;

    /* "dipy/align/vector_fields.pyx":902
 *         double[:] stats = np.zeros(shape=(3,), dtype=np.float64)
 * 
 *     if comp is None:             # <<<<<<<<<<<<<<
 *         comp = np.zeros_like(d1)
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":905
 *         comp = np.zeros_like(d1)
 * 
 *     if not is_valid_affine(premult_index, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid index multiplication matrix")
 *     if not is_valid_affine(premult_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 905; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_premult_index, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 905; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = NULL;
  __pyx_t_9 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_9 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 905; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_1) {
    __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1); __pyx_t_1 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_9, __pyx_t_7);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_9, __pyx_int_2);
  __pyx_t_7 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 905; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 905; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_10 = ((!__pyx_t_6) != 0);
  if (__pyx_t_10) {

    /* "dipy/align/vector_fields.pyx":906
 * 
 *     if not is_valid_affine(premult_index, 2):
 *         raise ValueError("Invalid index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(premult_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__35, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 906; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 906; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":905
 *         comp = np.zeros_like(d1)
 * 
 *     if not is_valid_affine(premult_index, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid index multiplication matrix")
 *     if not is_valid_affine(premult_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":907
 *     if not is_valid_affine(premult_index, 2):
 *         raise ValueError("Invalid index multiplication matrix")
 *     if not is_valid_affine(premult_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 907; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_premult_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 907; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  __pyx_t_9 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_9 = 1;
    }
  }
  __pyx_t_1 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 907; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (__pyx_t_7) {
    __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_7); __pyx_t_7 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_9, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_9, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 907; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_10 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 907; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = ((!__pyx_t_10) != 0);
  if (__pyx_t_6) {

    /* "dipy/align/vector_fields.pyx":908
 *         raise ValueError("Invalid index multiplication matrix")
 *     if not is_valid_affine(premult_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     _compose_vector_fields_2d[floating](d1, d2, premult_index, premult_disp,
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__36, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 908; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 908; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":907
 *     if not is_valid_affine(premult_index, 2):
 *         raise ValueError("Invalid index multiplication matrix")
 *     if not is_valid_affine(premult_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":910
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     _compose_vector_fields_2d[floating](d1, d2, premult_index, premult_disp,             # <<<<<<<<<<<<<<
 *                                         time_scaling, comp, stats)
 *     return comp, stats
 */
  __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__compose_vector_fields_2d(__pyx_v_d1, __pyx_v_d2, __pyx_v_premult_index, __pyx_v_premult_disp, __pyx_v_time_scaling, __pyx_v_comp, __pyx_v_stats);

  /* "dipy/align/vector_fields.pyx":912
 *     _compose_vector_fields_2d[floating](d1, d2, premult_index, premult_disp,
 *                                         time_scaling, comp, stats)
 *     return comp, stats             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_comp, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_stats, 1, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
  __pyx_t_4 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":847
 * 
 * 
 * def compose_vector_fields_2d(floating[:, :, :] d1, floating[:, :, :] d2,             # <<<<<<<<<<<<<<
 *                              double[:, :] premult_index,
 *                              double[:, :] premult_disp,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __Pyx_XDECREF(__pyx_t_7);
  __PYX_XDEC_MEMVIEW(&__pyx_t_8, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.compose_vector_fields_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_stats, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d2, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_premult_index, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_premult_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_comp, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":915
 * 
 * 
 * cdef void _compose_vector_fields_3d(floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                                     floating[:, :, :, :] d2,
 *                                     double[:, :] premult_index,
 */

static void __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__compose_vector_fields_3d(__Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_d2, __Pyx_memviewslice __pyx_v_premult_index, __Pyx_memviewslice __pyx_v_premult_disp, double __pyx_v_t, __Pyx_memviewslice __pyx_v_comp, __Pyx_memviewslice __pyx_v_stats) {
  npy_intp __pyx_v_ns1;
  npy_intp __pyx_v_nr1;
  npy_intp __pyx_v_nc1;
  CYTHON_UNUSED npy_intp __pyx_v_ns2;
  CYTHON_UNUSED npy_intp __pyx_v_nr2;
  CYTHON_UNUSED npy_intp __pyx_v_nc2;
  int __pyx_v_inside;
  int __pyx_v_cnt;
  double __pyx_v_maxNorm;
  double __pyx_v_meanNorm;
  double __pyx_v_stdNorm;
  double __pyx_v_nn;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dkk;
  double __pyx_v_diii;
  double __pyx_v_djjj;
  double __pyx_v_dkkk;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;
  npy_intp __pyx_t_4;
  npy_intp __pyx_t_5;
  npy_intp __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  int __pyx_t_19;
  __Pyx_memviewslice __pyx_t_20 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "dipy/align/vector_fields.pyx":983
 *     """
 *     cdef:
 *         cnp.npy_intp ns1 = d1.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr1 = d1.shape[1]
 *         cnp.npy_intp nc1 = d1.shape[2]
 */
  __pyx_v_ns1 = (__pyx_v_d1.shape[0]);

  /* "dipy/align/vector_fields.pyx":984
 *     cdef:
 *         cnp.npy_intp ns1 = d1.shape[0]
 *         cnp.npy_intp nr1 = d1.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc1 = d1.shape[2]
 *         cnp.npy_intp ns2 = d2.shape[0]
 */
  __pyx_v_nr1 = (__pyx_v_d1.shape[1]);

  /* "dipy/align/vector_fields.pyx":985
 *         cnp.npy_intp ns1 = d1.shape[0]
 *         cnp.npy_intp nr1 = d1.shape[1]
 *         cnp.npy_intp nc1 = d1.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ns2 = d2.shape[0]
 *         cnp.npy_intp nr2 = d2.shape[1]
 */
  __pyx_v_nc1 = (__pyx_v_d1.shape[2]);

  /* "dipy/align/vector_fields.pyx":986
 *         cnp.npy_intp nr1 = d1.shape[1]
 *         cnp.npy_intp nc1 = d1.shape[2]
 *         cnp.npy_intp ns2 = d2.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr2 = d2.shape[1]
 *         cnp.npy_intp nc2 = d2.shape[2]
 */
  __pyx_v_ns2 = (__pyx_v_d2.shape[0]);

  /* "dipy/align/vector_fields.pyx":987
 *         cnp.npy_intp nc1 = d1.shape[2]
 *         cnp.npy_intp ns2 = d2.shape[0]
 *         cnp.npy_intp nr2 = d2.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc2 = d2.shape[2]
 *         int inside, cnt = 0
 */
  __pyx_v_nr2 = (__pyx_v_d2.shape[1]);

  /* "dipy/align/vector_fields.pyx":988
 *         cnp.npy_intp ns2 = d2.shape[0]
 *         cnp.npy_intp nr2 = d2.shape[1]
 *         cnp.npy_intp nc2 = d2.shape[2]             # <<<<<<<<<<<<<<
 *         int inside, cnt = 0
 *         double maxNorm = 0
 */
  __pyx_v_nc2 = (__pyx_v_d2.shape[2]);

  /* "dipy/align/vector_fields.pyx":989
 *         cnp.npy_intp nr2 = d2.shape[1]
 *         cnp.npy_intp nc2 = d2.shape[2]
 *         int inside, cnt = 0             # <<<<<<<<<<<<<<
 *         double maxNorm = 0
 *         double meanNorm = 0
 */
  __pyx_v_cnt = 0;

  /* "dipy/align/vector_fields.pyx":990
 *         cnp.npy_intp nc2 = d2.shape[2]
 *         int inside, cnt = 0
 *         double maxNorm = 0             # <<<<<<<<<<<<<<
 *         double meanNorm = 0
 *         double stdNorm = 0
 */
  __pyx_v_maxNorm = 0.0;

  /* "dipy/align/vector_fields.pyx":991
 *         int inside, cnt = 0
 *         double maxNorm = 0
 *         double meanNorm = 0             # <<<<<<<<<<<<<<
 *         double stdNorm = 0
 *         double nn
 */
  __pyx_v_meanNorm = 0.0;

  /* "dipy/align/vector_fields.pyx":992
 *         double maxNorm = 0
 *         double meanNorm = 0
 *         double stdNorm = 0             # <<<<<<<<<<<<<<
 *         double nn
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_stdNorm = 0.0;

  /* "dipy/align/vector_fields.pyx":996
 *         cnp.npy_intp i, j, k
 *         double di, dj, dk, dii, djj, dkk, diii, djjj, dkkk
 *     for k in range(ns1):             # <<<<<<<<<<<<<<
 *         for i in range(nr1):
 *             for j in range(nc1):
 */
  __pyx_t_1 = __pyx_v_ns1;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_k = __pyx_t_2;

    /* "dipy/align/vector_fields.pyx":997
 *         double di, dj, dk, dii, djj, dkk, diii, djjj, dkkk
 *     for k in range(ns1):
 *         for i in range(nr1):             # <<<<<<<<<<<<<<
 *             for j in range(nc1):
 * 
 */
    __pyx_t_3 = __pyx_v_nr1;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "dipy/align/vector_fields.pyx":998
 *     for k in range(ns1):
 *         for i in range(nr1):
 *             for j in range(nc1):             # <<<<<<<<<<<<<<
 * 
 *                 # This is the only place we access d1[k, i, j]
 */
      __pyx_t_5 = __pyx_v_nc1;
      for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
        __pyx_v_j = __pyx_t_6;

        /* "dipy/align/vector_fields.pyx":1001
 * 
 *                 # This is the only place we access d1[k, i, j]
 *                 dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                 dii = d1[k, i, j, 1]
 *                 djj = d1[k, i, j, 2]
 */
        __pyx_t_7 = __pyx_v_k;
        __pyx_t_8 = __pyx_v_i;
        __pyx_t_9 = __pyx_v_j;
        __pyx_t_10 = 0;
        __pyx_v_dkk = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_7 * __pyx_v_d1.strides[0]) ) + __pyx_t_8 * __pyx_v_d1.strides[1]) ) + __pyx_t_9 * __pyx_v_d1.strides[2]) ) + __pyx_t_10 * __pyx_v_d1.strides[3]) )));

        /* "dipy/align/vector_fields.pyx":1002
 *                 # This is the only place we access d1[k, i, j]
 *                 dkk = d1[k, i, j, 0]
 *                 dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                 djj = d1[k, i, j, 2]
 * 
 */
        __pyx_t_11 = __pyx_v_k;
        __pyx_t_12 = __pyx_v_i;
        __pyx_t_13 = __pyx_v_j;
        __pyx_t_14 = 1;
        __pyx_v_dii = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_11 * __pyx_v_d1.strides[0]) ) + __pyx_t_12 * __pyx_v_d1.strides[1]) ) + __pyx_t_13 * __pyx_v_d1.strides[2]) ) + __pyx_t_14 * __pyx_v_d1.strides[3]) )));

        /* "dipy/align/vector_fields.pyx":1003
 *                 dkk = d1[k, i, j, 0]
 *                 dii = d1[k, i, j, 1]
 *                 djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 * 
 *                 if premult_disp is None:
 */
        __pyx_t_15 = __pyx_v_k;
        __pyx_t_16 = __pyx_v_i;
        __pyx_t_17 = __pyx_v_j;
        __pyx_t_18 = 2;
        __pyx_v_djj = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_15 * __pyx_v_d1.strides[0]) ) + __pyx_t_16 * __pyx_v_d1.strides[1]) ) + __pyx_t_17 * __pyx_v_d1.strides[2]) ) + __pyx_t_18 * __pyx_v_d1.strides[3]) )));

        /* "dipy/align/vector_fields.pyx":1005
 *                 djj = d1[k, i, j, 2]
 * 
 *                 if premult_disp is None:             # <<<<<<<<<<<<<<
 *                     dk = dkk
 *                     di = dii
 */
        __pyx_t_19 = ((((PyObject *) __pyx_v_premult_disp.memview) == Py_None) != 0);
        if (__pyx_t_19) {

          /* "dipy/align/vector_fields.pyx":1006
 * 
 *                 if premult_disp is None:
 *                     dk = dkk             # <<<<<<<<<<<<<<
 *                     di = dii
 *                     dj = djj
 */
          __pyx_v_dk = __pyx_v_dkk;

          /* "dipy/align/vector_fields.pyx":1007
 *                 if premult_disp is None:
 *                     dk = dkk
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 *                 else:
 */
          __pyx_v_di = __pyx_v_dii;

          /* "dipy/align/vector_fields.pyx":1008
 *                     dk = dkk
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 *                 else:
 *                     dk = _apply_affine_3d_x0(dkk, dii, djj, 0, premult_disp)
 */
          __pyx_v_dj = __pyx_v_djj;

          /* "dipy/align/vector_fields.pyx":1005
 *                 djj = d1[k, i, j, 2]
 * 
 *                 if premult_disp is None:             # <<<<<<<<<<<<<<
 *                     dk = dkk
 *                     di = dii
 */
          goto __pyx_L9;
        }

        /* "dipy/align/vector_fields.pyx":1010
 *                     dj = djj
 *                 else:
 *                     dk = _apply_affine_3d_x0(dkk, dii, djj, 0, premult_disp)             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_3d_x1(dkk, dii, djj, 0, premult_disp)
 *                     dj = _apply_affine_3d_x2(dkk, dii, djj, 0, premult_disp)
 */
        /*else*/ {
          __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_premult_disp);

          /* "dipy/align/vector_fields.pyx":1011
 *                 else:
 *                     dk = _apply_affine_3d_x0(dkk, dii, djj, 0, premult_disp)
 *                     di = _apply_affine_3d_x1(dkk, dii, djj, 0, premult_disp)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_3d_x2(dkk, dii, djj, 0, premult_disp)
 * 
 */
          __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_premult_disp);

          /* "dipy/align/vector_fields.pyx":1012
 *                     dk = _apply_affine_3d_x0(dkk, dii, djj, 0, premult_disp)
 *                     di = _apply_affine_3d_x1(dkk, dii, djj, 0, premult_disp)
 *                     dj = _apply_affine_3d_x2(dkk, dii, djj, 0, premult_disp)             # <<<<<<<<<<<<<<
 * 
 *                 if premult_index is None:
 */
          __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_premult_disp);
        }
        __pyx_L9:;

        /* "dipy/align/vector_fields.pyx":1014
 *                     dj = _apply_affine_3d_x2(dkk, dii, djj, 0, premult_disp)
 * 
 *                 if premult_index is None:             # <<<<<<<<<<<<<<
 *                     dkkk = k
 *                     diii = i
 */
        __pyx_t_19 = ((((PyObject *) __pyx_v_premult_index.memview) == Py_None) != 0);
        if (__pyx_t_19) {

          /* "dipy/align/vector_fields.pyx":1015
 * 
 *                 if premult_index is None:
 *                     dkkk = k             # <<<<<<<<<<<<<<
 *                     diii = i
 *                     djjj = j
 */
          __pyx_v_dkkk = __pyx_v_k;

          /* "dipy/align/vector_fields.pyx":1016
 *                 if premult_index is None:
 *                     dkkk = k
 *                     diii = i             # <<<<<<<<<<<<<<
 *                     djjj = j
 *                 else:
 */
          __pyx_v_diii = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":1017
 *                     dkkk = k
 *                     diii = i
 *                     djjj = j             # <<<<<<<<<<<<<<
 *                 else:
 *                     dkkk = _apply_affine_3d_x0(k, i, j, 1, premult_index)
 */
          __pyx_v_djjj = __pyx_v_j;

          /* "dipy/align/vector_fields.pyx":1014
 *                     dj = _apply_affine_3d_x2(dkk, dii, djj, 0, premult_disp)
 * 
 *                 if premult_index is None:             # <<<<<<<<<<<<<<
 *                     dkkk = k
 *                     diii = i
 */
          goto __pyx_L10;
        }

        /* "dipy/align/vector_fields.pyx":1019
 *                     djjj = j
 *                 else:
 *                     dkkk = _apply_affine_3d_x0(k, i, j, 1, premult_index)             # <<<<<<<<<<<<<<
 *                     diii = _apply_affine_3d_x1(k, i, j, 1, premult_index)
 *                     djjj = _apply_affine_3d_x2(k, i, j, 1, premult_index)
 */
        /*else*/ {
          __pyx_v_dkkk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_premult_index);

          /* "dipy/align/vector_fields.pyx":1020
 *                 else:
 *                     dkkk = _apply_affine_3d_x0(k, i, j, 1, premult_index)
 *                     diii = _apply_affine_3d_x1(k, i, j, 1, premult_index)             # <<<<<<<<<<<<<<
 *                     djjj = _apply_affine_3d_x2(k, i, j, 1, premult_index)
 * 
 */
          __pyx_v_diii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_premult_index);

          /* "dipy/align/vector_fields.pyx":1021
 *                     dkkk = _apply_affine_3d_x0(k, i, j, 1, premult_index)
 *                     diii = _apply_affine_3d_x1(k, i, j, 1, premult_index)
 *                     djjj = _apply_affine_3d_x2(k, i, j, 1, premult_index)             # <<<<<<<<<<<<<<
 * 
 *                 dkkk += dk
 */
          __pyx_v_djjj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_premult_index);
        }
        __pyx_L10:;

        /* "dipy/align/vector_fields.pyx":1023
 *                     djjj = _apply_affine_3d_x2(k, i, j, 1, premult_index)
 * 
 *                 dkkk += dk             # <<<<<<<<<<<<<<
 *                 diii += di
 *                 djjj += dj
 */
        __pyx_v_dkkk = (__pyx_v_dkkk + __pyx_v_dk);

        /* "dipy/align/vector_fields.pyx":1024
 * 
 *                 dkkk += dk
 *                 diii += di             # <<<<<<<<<<<<<<
 *                 djjj += dj
 * 
 */
        __pyx_v_diii = (__pyx_v_diii + __pyx_v_di);

        /* "dipy/align/vector_fields.pyx":1025
 *                 dkkk += dk
 *                 diii += di
 *                 djjj += dj             # <<<<<<<<<<<<<<
 * 
 *                 # If d1 and comp are the same array, this will correctly update
 */
        __pyx_v_djjj = (__pyx_v_djjj + __pyx_v_dj);

        /* "dipy/align/vector_fields.pyx":1033
 *                 # from d2, which may be problematic
 *                 inside = _interpolate_vector_3d[floating](d2, dkkk, diii,
 *                                                           djjj, comp[k, i, j])             # <<<<<<<<<<<<<<
 * 
 *                 if inside == 1:
 */
        __pyx_t_20.data = __pyx_v_comp.data;
        __pyx_t_20.memview = __pyx_v_comp.memview;
        __PYX_INC_MEMVIEW(&__pyx_t_20, 0);
        {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_k;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1033; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_20.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[1];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[1];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 1)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1033; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_20.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_j;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[2];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[2];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 2)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1033; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_20.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_20.shape[0] = __pyx_v_comp.shape[3];
__pyx_t_20.strides[0] = __pyx_v_comp.strides[3];
    __pyx_t_20.suboffsets[0] = -1;

__pyx_v_inside = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d2, __pyx_v_dkkk, __pyx_v_diii, __pyx_v_djjj, __pyx_t_20);

        /* "dipy/align/vector_fields.pyx":1032
 *                 # may be in the neighborhood of a previously updated vector
 *                 # from d2, which may be problematic
 *                 inside = _interpolate_vector_3d[floating](d2, dkkk, diii,             # <<<<<<<<<<<<<<
 *                                                           djjj, comp[k, i, j])
 * 
 */
        __PYX_XDEC_MEMVIEW(&__pyx_t_20, 0);

        /* "dipy/align/vector_fields.pyx":1035
 *                                                           djjj, comp[k, i, j])
 * 
 *                 if inside == 1:             # <<<<<<<<<<<<<<
 *                     comp[k, i, j, 0] = t * comp[k, i, j, 0] + dkk
 *                     comp[k, i, j, 1] = t * comp[k, i, j, 1] + dii
 */
        __pyx_t_19 = ((__pyx_v_inside == 1) != 0);
        if (__pyx_t_19) {

          /* "dipy/align/vector_fields.pyx":1036
 * 
 *                 if inside == 1:
 *                     comp[k, i, j, 0] = t * comp[k, i, j, 0] + dkk             # <<<<<<<<<<<<<<
 *                     comp[k, i, j, 1] = t * comp[k, i, j, 1] + dii
 *                     comp[k, i, j, 2] = t * comp[k, i, j, 2] + djj
 */
          __pyx_t_21 = __pyx_v_k;
          __pyx_t_22 = __pyx_v_i;
          __pyx_t_23 = __pyx_v_j;
          __pyx_t_24 = 0;
          __pyx_t_25 = __pyx_v_k;
          __pyx_t_26 = __pyx_v_i;
          __pyx_t_27 = __pyx_v_j;
          __pyx_t_28 = 0;
          *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_25 * __pyx_v_comp.strides[0]) ) + __pyx_t_26 * __pyx_v_comp.strides[1]) ) + __pyx_t_27 * __pyx_v_comp.strides[2]) ) + __pyx_t_28 * __pyx_v_comp.strides[3]) )) = ((__pyx_v_t * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_21 * __pyx_v_comp.strides[0]) ) + __pyx_t_22 * __pyx_v_comp.strides[1]) ) + __pyx_t_23 * __pyx_v_comp.strides[2]) ) + __pyx_t_24 * __pyx_v_comp.strides[3]) )))) + __pyx_v_dkk);

          /* "dipy/align/vector_fields.pyx":1037
 *                 if inside == 1:
 *                     comp[k, i, j, 0] = t * comp[k, i, j, 0] + dkk
 *                     comp[k, i, j, 1] = t * comp[k, i, j, 1] + dii             # <<<<<<<<<<<<<<
 *                     comp[k, i, j, 2] = t * comp[k, i, j, 2] + djj
 *                     nn = (comp[k, i, j, 0] ** 2 + comp[k, i, j, 1] ** 2 +
 */
          __pyx_t_29 = __pyx_v_k;
          __pyx_t_30 = __pyx_v_i;
          __pyx_t_31 = __pyx_v_j;
          __pyx_t_32 = 1;
          __pyx_t_33 = __pyx_v_k;
          __pyx_t_34 = __pyx_v_i;
          __pyx_t_35 = __pyx_v_j;
          __pyx_t_36 = 1;
          *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_33 * __pyx_v_comp.strides[0]) ) + __pyx_t_34 * __pyx_v_comp.strides[1]) ) + __pyx_t_35 * __pyx_v_comp.strides[2]) ) + __pyx_t_36 * __pyx_v_comp.strides[3]) )) = ((__pyx_v_t * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_29 * __pyx_v_comp.strides[0]) ) + __pyx_t_30 * __pyx_v_comp.strides[1]) ) + __pyx_t_31 * __pyx_v_comp.strides[2]) ) + __pyx_t_32 * __pyx_v_comp.strides[3]) )))) + __pyx_v_dii);

          /* "dipy/align/vector_fields.pyx":1038
 *                     comp[k, i, j, 0] = t * comp[k, i, j, 0] + dkk
 *                     comp[k, i, j, 1] = t * comp[k, i, j, 1] + dii
 *                     comp[k, i, j, 2] = t * comp[k, i, j, 2] + djj             # <<<<<<<<<<<<<<
 *                     nn = (comp[k, i, j, 0] ** 2 + comp[k, i, j, 1] ** 2 +
 *                           comp[k, i, j, 2]**2)
 */
          __pyx_t_37 = __pyx_v_k;
          __pyx_t_38 = __pyx_v_i;
          __pyx_t_39 = __pyx_v_j;
          __pyx_t_40 = 2;
          __pyx_t_41 = __pyx_v_k;
          __pyx_t_42 = __pyx_v_i;
          __pyx_t_43 = __pyx_v_j;
          __pyx_t_44 = 2;
          *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_41 * __pyx_v_comp.strides[0]) ) + __pyx_t_42 * __pyx_v_comp.strides[1]) ) + __pyx_t_43 * __pyx_v_comp.strides[2]) ) + __pyx_t_44 * __pyx_v_comp.strides[3]) )) = ((__pyx_v_t * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_37 * __pyx_v_comp.strides[0]) ) + __pyx_t_38 * __pyx_v_comp.strides[1]) ) + __pyx_t_39 * __pyx_v_comp.strides[2]) ) + __pyx_t_40 * __pyx_v_comp.strides[3]) )))) + __pyx_v_djj);

          /* "dipy/align/vector_fields.pyx":1039
 *                     comp[k, i, j, 1] = t * comp[k, i, j, 1] + dii
 *                     comp[k, i, j, 2] = t * comp[k, i, j, 2] + djj
 *                     nn = (comp[k, i, j, 0] ** 2 + comp[k, i, j, 1] ** 2 +             # <<<<<<<<<<<<<<
 *                           comp[k, i, j, 2]**2)
 *                     meanNorm += nn
 */
          __pyx_t_45 = __pyx_v_k;
          __pyx_t_46 = __pyx_v_i;
          __pyx_t_47 = __pyx_v_j;
          __pyx_t_48 = 0;
          __pyx_t_49 = __pyx_v_k;
          __pyx_t_50 = __pyx_v_i;
          __pyx_t_51 = __pyx_v_j;
          __pyx_t_52 = 1;

          /* "dipy/align/vector_fields.pyx":1040
 *                     comp[k, i, j, 2] = t * comp[k, i, j, 2] + djj
 *                     nn = (comp[k, i, j, 0] ** 2 + comp[k, i, j, 1] ** 2 +
 *                           comp[k, i, j, 2]**2)             # <<<<<<<<<<<<<<
 *                     meanNorm += nn
 *                     stdNorm += nn * nn
 */
          __pyx_t_53 = __pyx_v_k;
          __pyx_t_54 = __pyx_v_i;
          __pyx_t_55 = __pyx_v_j;
          __pyx_t_56 = 2;

          /* "dipy/align/vector_fields.pyx":1039
 *                     comp[k, i, j, 1] = t * comp[k, i, j, 1] + dii
 *                     comp[k, i, j, 2] = t * comp[k, i, j, 2] + djj
 *                     nn = (comp[k, i, j, 0] ** 2 + comp[k, i, j, 1] ** 2 +             # <<<<<<<<<<<<<<
 *                           comp[k, i, j, 2]**2)
 *                     meanNorm += nn
 */
          __pyx_v_nn = ((powf((*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_45 * __pyx_v_comp.strides[0]) ) + __pyx_t_46 * __pyx_v_comp.strides[1]) ) + __pyx_t_47 * __pyx_v_comp.strides[2]) ) + __pyx_t_48 * __pyx_v_comp.strides[3]) ))), 2.0) + powf((*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_49 * __pyx_v_comp.strides[0]) ) + __pyx_t_50 * __pyx_v_comp.strides[1]) ) + __pyx_t_51 * __pyx_v_comp.strides[2]) ) + __pyx_t_52 * __pyx_v_comp.strides[3]) ))), 2.0)) + powf((*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_53 * __pyx_v_comp.strides[0]) ) + __pyx_t_54 * __pyx_v_comp.strides[1]) ) + __pyx_t_55 * __pyx_v_comp.strides[2]) ) + __pyx_t_56 * __pyx_v_comp.strides[3]) ))), 2.0));

          /* "dipy/align/vector_fields.pyx":1041
 *                     nn = (comp[k, i, j, 0] ** 2 + comp[k, i, j, 1] ** 2 +
 *                           comp[k, i, j, 2]**2)
 *                     meanNorm += nn             # <<<<<<<<<<<<<<
 *                     stdNorm += nn * nn
 *                     cnt += 1
 */
          __pyx_v_meanNorm = (__pyx_v_meanNorm + __pyx_v_nn);

          /* "dipy/align/vector_fields.pyx":1042
 *                           comp[k, i, j, 2]**2)
 *                     meanNorm += nn
 *                     stdNorm += nn * nn             # <<<<<<<<<<<<<<
 *                     cnt += 1
 *                     if(maxNorm < nn):
 */
          __pyx_v_stdNorm = (__pyx_v_stdNorm + (__pyx_v_nn * __pyx_v_nn));

          /* "dipy/align/vector_fields.pyx":1043
 *                     meanNorm += nn
 *                     stdNorm += nn * nn
 *                     cnt += 1             # <<<<<<<<<<<<<<
 *                     if(maxNorm < nn):
 *                         maxNorm = nn
 */
          __pyx_v_cnt = (__pyx_v_cnt + 1);

          /* "dipy/align/vector_fields.pyx":1044
 *                     stdNorm += nn * nn
 *                     cnt += 1
 *                     if(maxNorm < nn):             # <<<<<<<<<<<<<<
 *                         maxNorm = nn
 *                 else:
 */
          __pyx_t_19 = ((__pyx_v_maxNorm < __pyx_v_nn) != 0);
          if (__pyx_t_19) {

            /* "dipy/align/vector_fields.pyx":1045
 *                     cnt += 1
 *                     if(maxNorm < nn):
 *                         maxNorm = nn             # <<<<<<<<<<<<<<
 *                 else:
 *                     comp[k, i, j, :] = 0
 */
            __pyx_v_maxNorm = __pyx_v_nn;

            /* "dipy/align/vector_fields.pyx":1044
 *                     stdNorm += nn * nn
 *                     cnt += 1
 *                     if(maxNorm < nn):             # <<<<<<<<<<<<<<
 *                         maxNorm = nn
 *                 else:
 */
          }

          /* "dipy/align/vector_fields.pyx":1035
 *                                                           djjj, comp[k, i, j])
 * 
 *                 if inside == 1:             # <<<<<<<<<<<<<<
 *                     comp[k, i, j, 0] = t * comp[k, i, j, 0] + dkk
 *                     comp[k, i, j, 1] = t * comp[k, i, j, 1] + dii
 */
          goto __pyx_L11;
        }

        /* "dipy/align/vector_fields.pyx":1047
 *                         maxNorm = nn
 *                 else:
 *                     comp[k, i, j, :] = 0             # <<<<<<<<<<<<<<
 *     meanNorm /= cnt
 *     stats[0] = sqrt(maxNorm)
 */
        /*else*/ {
          __pyx_t_20.data = __pyx_v_comp.data;
          __pyx_t_20.memview = __pyx_v_comp.memview;
          __PYX_INC_MEMVIEW(&__pyx_t_20, 0);
          {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_k;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1047; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_20.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[1];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[1];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 1)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1047; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_20.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_j;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[2];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[2];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 2)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1047; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_20.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_20.shape[0] = __pyx_v_comp.shape[3];
__pyx_t_20.strides[0] = __pyx_v_comp.strides[3];
    __pyx_t_20.suboffsets[0] = -1;

{
              float __pyx_temp_scalar = 0.0;
              {
                  Py_ssize_t __pyx_temp_extent_0 = __pyx_t_20.shape[0];
                  Py_ssize_t __pyx_temp_stride_0 = __pyx_t_20.strides[0];
                  char *__pyx_temp_pointer_0;
                  Py_ssize_t __pyx_temp_idx_0;
                  __pyx_temp_pointer_0 = __pyx_t_20.data;
                  for (__pyx_temp_idx_0 = 0; __pyx_temp_idx_0 < __pyx_temp_extent_0; __pyx_temp_idx_0++) {
                    *((float *) __pyx_temp_pointer_0) = __pyx_temp_scalar;
                    __pyx_temp_pointer_0 += __pyx_temp_stride_0;
                  }
              }
          }
          __PYX_XDEC_MEMVIEW(&__pyx_t_20, 0);
        }
        __pyx_L11:;
      }
    }
  }

  /* "dipy/align/vector_fields.pyx":1048
 *                 else:
 *                     comp[k, i, j, :] = 0
 *     meanNorm /= cnt             # <<<<<<<<<<<<<<
 *     stats[0] = sqrt(maxNorm)
 *     stats[1] = sqrt(meanNorm)
 */
  __pyx_v_meanNorm = (__pyx_v_meanNorm / __pyx_v_cnt);

  /* "dipy/align/vector_fields.pyx":1049
 *                     comp[k, i, j, :] = 0
 *     meanNorm /= cnt
 *     stats[0] = sqrt(maxNorm)             # <<<<<<<<<<<<<<
 *     stats[1] = sqrt(meanNorm)
 *     stats[2] = sqrt(stdNorm / cnt - meanNorm * meanNorm)
 */
  __pyx_t_57 = 0;
  *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_57 * __pyx_v_stats.strides[0]) )) = sqrt(__pyx_v_maxNorm);

  /* "dipy/align/vector_fields.pyx":1050
 *     meanNorm /= cnt
 *     stats[0] = sqrt(maxNorm)
 *     stats[1] = sqrt(meanNorm)             # <<<<<<<<<<<<<<
 *     stats[2] = sqrt(stdNorm / cnt - meanNorm * meanNorm)
 * 
 */
  __pyx_t_58 = 1;
  *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_58 * __pyx_v_stats.strides[0]) )) = sqrt(__pyx_v_meanNorm);

  /* "dipy/align/vector_fields.pyx":1051
 *     stats[0] = sqrt(maxNorm)
 *     stats[1] = sqrt(meanNorm)
 *     stats[2] = sqrt(stdNorm / cnt - meanNorm * meanNorm)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_59 = 2;
  *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_59 * __pyx_v_stats.strides[0]) )) = sqrt(((__pyx_v_stdNorm / __pyx_v_cnt) - (__pyx_v_meanNorm * __pyx_v_meanNorm)));

  /* "dipy/align/vector_fields.pyx":915
 * 
 * 
 * cdef void _compose_vector_fields_3d(floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                                     floating[:, :, :, :] d2,
 *                                     double[:, :] premult_index,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __PYX_XDEC_MEMVIEW(&__pyx_t_20, 0);
  __Pyx_WriteUnraisable("dipy.align.vector_fields._compose_vector_fields_3d", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 1);
  __pyx_L0:;
}

static void __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__compose_vector_fields_3d(__Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_d2, __Pyx_memviewslice __pyx_v_premult_index, __Pyx_memviewslice __pyx_v_premult_disp, double __pyx_v_t, __Pyx_memviewslice __pyx_v_comp, __Pyx_memviewslice __pyx_v_stats) {
  npy_intp __pyx_v_ns1;
  npy_intp __pyx_v_nr1;
  npy_intp __pyx_v_nc1;
  CYTHON_UNUSED npy_intp __pyx_v_ns2;
  CYTHON_UNUSED npy_intp __pyx_v_nr2;
  CYTHON_UNUSED npy_intp __pyx_v_nc2;
  int __pyx_v_inside;
  int __pyx_v_cnt;
  double __pyx_v_maxNorm;
  double __pyx_v_meanNorm;
  double __pyx_v_stdNorm;
  double __pyx_v_nn;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dkk;
  double __pyx_v_diii;
  double __pyx_v_djjj;
  double __pyx_v_dkkk;
  npy_intp __pyx_t_1;
  npy_intp __pyx_t_2;
  npy_intp __pyx_t_3;
  npy_intp __pyx_t_4;
  npy_intp __pyx_t_5;
  npy_intp __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  int __pyx_t_19;
  __Pyx_memviewslice __pyx_t_20 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "dipy/align/vector_fields.pyx":983
 *     """
 *     cdef:
 *         cnp.npy_intp ns1 = d1.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr1 = d1.shape[1]
 *         cnp.npy_intp nc1 = d1.shape[2]
 */
  __pyx_v_ns1 = (__pyx_v_d1.shape[0]);

  /* "dipy/align/vector_fields.pyx":984
 *     cdef:
 *         cnp.npy_intp ns1 = d1.shape[0]
 *         cnp.npy_intp nr1 = d1.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc1 = d1.shape[2]
 *         cnp.npy_intp ns2 = d2.shape[0]
 */
  __pyx_v_nr1 = (__pyx_v_d1.shape[1]);

  /* "dipy/align/vector_fields.pyx":985
 *         cnp.npy_intp ns1 = d1.shape[0]
 *         cnp.npy_intp nr1 = d1.shape[1]
 *         cnp.npy_intp nc1 = d1.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ns2 = d2.shape[0]
 *         cnp.npy_intp nr2 = d2.shape[1]
 */
  __pyx_v_nc1 = (__pyx_v_d1.shape[2]);

  /* "dipy/align/vector_fields.pyx":986
 *         cnp.npy_intp nr1 = d1.shape[1]
 *         cnp.npy_intp nc1 = d1.shape[2]
 *         cnp.npy_intp ns2 = d2.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr2 = d2.shape[1]
 *         cnp.npy_intp nc2 = d2.shape[2]
 */
  __pyx_v_ns2 = (__pyx_v_d2.shape[0]);

  /* "dipy/align/vector_fields.pyx":987
 *         cnp.npy_intp nc1 = d1.shape[2]
 *         cnp.npy_intp ns2 = d2.shape[0]
 *         cnp.npy_intp nr2 = d2.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc2 = d2.shape[2]
 *         int inside, cnt = 0
 */
  __pyx_v_nr2 = (__pyx_v_d2.shape[1]);

  /* "dipy/align/vector_fields.pyx":988
 *         cnp.npy_intp ns2 = d2.shape[0]
 *         cnp.npy_intp nr2 = d2.shape[1]
 *         cnp.npy_intp nc2 = d2.shape[2]             # <<<<<<<<<<<<<<
 *         int inside, cnt = 0
 *         double maxNorm = 0
 */
  __pyx_v_nc2 = (__pyx_v_d2.shape[2]);

  /* "dipy/align/vector_fields.pyx":989
 *         cnp.npy_intp nr2 = d2.shape[1]
 *         cnp.npy_intp nc2 = d2.shape[2]
 *         int inside, cnt = 0             # <<<<<<<<<<<<<<
 *         double maxNorm = 0
 *         double meanNorm = 0
 */
  __pyx_v_cnt = 0;

  /* "dipy/align/vector_fields.pyx":990
 *         cnp.npy_intp nc2 = d2.shape[2]
 *         int inside, cnt = 0
 *         double maxNorm = 0             # <<<<<<<<<<<<<<
 *         double meanNorm = 0
 *         double stdNorm = 0
 */
  __pyx_v_maxNorm = 0.0;

  /* "dipy/align/vector_fields.pyx":991
 *         int inside, cnt = 0
 *         double maxNorm = 0
 *         double meanNorm = 0             # <<<<<<<<<<<<<<
 *         double stdNorm = 0
 *         double nn
 */
  __pyx_v_meanNorm = 0.0;

  /* "dipy/align/vector_fields.pyx":992
 *         double maxNorm = 0
 *         double meanNorm = 0
 *         double stdNorm = 0             # <<<<<<<<<<<<<<
 *         double nn
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_stdNorm = 0.0;

  /* "dipy/align/vector_fields.pyx":996
 *         cnp.npy_intp i, j, k
 *         double di, dj, dk, dii, djj, dkk, diii, djjj, dkkk
 *     for k in range(ns1):             # <<<<<<<<<<<<<<
 *         for i in range(nr1):
 *             for j in range(nc1):
 */
  __pyx_t_1 = __pyx_v_ns1;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_k = __pyx_t_2;

    /* "dipy/align/vector_fields.pyx":997
 *         double di, dj, dk, dii, djj, dkk, diii, djjj, dkkk
 *     for k in range(ns1):
 *         for i in range(nr1):             # <<<<<<<<<<<<<<
 *             for j in range(nc1):
 * 
 */
    __pyx_t_3 = __pyx_v_nr1;
    for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
      __pyx_v_i = __pyx_t_4;

      /* "dipy/align/vector_fields.pyx":998
 *     for k in range(ns1):
 *         for i in range(nr1):
 *             for j in range(nc1):             # <<<<<<<<<<<<<<
 * 
 *                 # This is the only place we access d1[k, i, j]
 */
      __pyx_t_5 = __pyx_v_nc1;
      for (__pyx_t_6 = 0; __pyx_t_6 < __pyx_t_5; __pyx_t_6+=1) {
        __pyx_v_j = __pyx_t_6;

        /* "dipy/align/vector_fields.pyx":1001
 * 
 *                 # This is the only place we access d1[k, i, j]
 *                 dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                 dii = d1[k, i, j, 1]
 *                 djj = d1[k, i, j, 2]
 */
        __pyx_t_7 = __pyx_v_k;
        __pyx_t_8 = __pyx_v_i;
        __pyx_t_9 = __pyx_v_j;
        __pyx_t_10 = 0;
        __pyx_v_dkk = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_7 * __pyx_v_d1.strides[0]) ) + __pyx_t_8 * __pyx_v_d1.strides[1]) ) + __pyx_t_9 * __pyx_v_d1.strides[2]) ) + __pyx_t_10 * __pyx_v_d1.strides[3]) )));

        /* "dipy/align/vector_fields.pyx":1002
 *                 # This is the only place we access d1[k, i, j]
 *                 dkk = d1[k, i, j, 0]
 *                 dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                 djj = d1[k, i, j, 2]
 * 
 */
        __pyx_t_11 = __pyx_v_k;
        __pyx_t_12 = __pyx_v_i;
        __pyx_t_13 = __pyx_v_j;
        __pyx_t_14 = 1;
        __pyx_v_dii = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_11 * __pyx_v_d1.strides[0]) ) + __pyx_t_12 * __pyx_v_d1.strides[1]) ) + __pyx_t_13 * __pyx_v_d1.strides[2]) ) + __pyx_t_14 * __pyx_v_d1.strides[3]) )));

        /* "dipy/align/vector_fields.pyx":1003
 *                 dkk = d1[k, i, j, 0]
 *                 dii = d1[k, i, j, 1]
 *                 djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 * 
 *                 if premult_disp is None:
 */
        __pyx_t_15 = __pyx_v_k;
        __pyx_t_16 = __pyx_v_i;
        __pyx_t_17 = __pyx_v_j;
        __pyx_t_18 = 2;
        __pyx_v_djj = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_15 * __pyx_v_d1.strides[0]) ) + __pyx_t_16 * __pyx_v_d1.strides[1]) ) + __pyx_t_17 * __pyx_v_d1.strides[2]) ) + __pyx_t_18 * __pyx_v_d1.strides[3]) )));

        /* "dipy/align/vector_fields.pyx":1005
 *                 djj = d1[k, i, j, 2]
 * 
 *                 if premult_disp is None:             # <<<<<<<<<<<<<<
 *                     dk = dkk
 *                     di = dii
 */
        __pyx_t_19 = ((((PyObject *) __pyx_v_premult_disp.memview) == Py_None) != 0);
        if (__pyx_t_19) {

          /* "dipy/align/vector_fields.pyx":1006
 * 
 *                 if premult_disp is None:
 *                     dk = dkk             # <<<<<<<<<<<<<<
 *                     di = dii
 *                     dj = djj
 */
          __pyx_v_dk = __pyx_v_dkk;

          /* "dipy/align/vector_fields.pyx":1007
 *                 if premult_disp is None:
 *                     dk = dkk
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 *                 else:
 */
          __pyx_v_di = __pyx_v_dii;

          /* "dipy/align/vector_fields.pyx":1008
 *                     dk = dkk
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 *                 else:
 *                     dk = _apply_affine_3d_x0(dkk, dii, djj, 0, premult_disp)
 */
          __pyx_v_dj = __pyx_v_djj;

          /* "dipy/align/vector_fields.pyx":1005
 *                 djj = d1[k, i, j, 2]
 * 
 *                 if premult_disp is None:             # <<<<<<<<<<<<<<
 *                     dk = dkk
 *                     di = dii
 */
          goto __pyx_L9;
        }

        /* "dipy/align/vector_fields.pyx":1010
 *                     dj = djj
 *                 else:
 *                     dk = _apply_affine_3d_x0(dkk, dii, djj, 0, premult_disp)             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_3d_x1(dkk, dii, djj, 0, premult_disp)
 *                     dj = _apply_affine_3d_x2(dkk, dii, djj, 0, premult_disp)
 */
        /*else*/ {
          __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_premult_disp);

          /* "dipy/align/vector_fields.pyx":1011
 *                 else:
 *                     dk = _apply_affine_3d_x0(dkk, dii, djj, 0, premult_disp)
 *                     di = _apply_affine_3d_x1(dkk, dii, djj, 0, premult_disp)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_3d_x2(dkk, dii, djj, 0, premult_disp)
 * 
 */
          __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_premult_disp);

          /* "dipy/align/vector_fields.pyx":1012
 *                     dk = _apply_affine_3d_x0(dkk, dii, djj, 0, premult_disp)
 *                     di = _apply_affine_3d_x1(dkk, dii, djj, 0, premult_disp)
 *                     dj = _apply_affine_3d_x2(dkk, dii, djj, 0, premult_disp)             # <<<<<<<<<<<<<<
 * 
 *                 if premult_index is None:
 */
          __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_premult_disp);
        }
        __pyx_L9:;

        /* "dipy/align/vector_fields.pyx":1014
 *                     dj = _apply_affine_3d_x2(dkk, dii, djj, 0, premult_disp)
 * 
 *                 if premult_index is None:             # <<<<<<<<<<<<<<
 *                     dkkk = k
 *                     diii = i
 */
        __pyx_t_19 = ((((PyObject *) __pyx_v_premult_index.memview) == Py_None) != 0);
        if (__pyx_t_19) {

          /* "dipy/align/vector_fields.pyx":1015
 * 
 *                 if premult_index is None:
 *                     dkkk = k             # <<<<<<<<<<<<<<
 *                     diii = i
 *                     djjj = j
 */
          __pyx_v_dkkk = __pyx_v_k;

          /* "dipy/align/vector_fields.pyx":1016
 *                 if premult_index is None:
 *                     dkkk = k
 *                     diii = i             # <<<<<<<<<<<<<<
 *                     djjj = j
 *                 else:
 */
          __pyx_v_diii = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":1017
 *                     dkkk = k
 *                     diii = i
 *                     djjj = j             # <<<<<<<<<<<<<<
 *                 else:
 *                     dkkk = _apply_affine_3d_x0(k, i, j, 1, premult_index)
 */
          __pyx_v_djjj = __pyx_v_j;

          /* "dipy/align/vector_fields.pyx":1014
 *                     dj = _apply_affine_3d_x2(dkk, dii, djj, 0, premult_disp)
 * 
 *                 if premult_index is None:             # <<<<<<<<<<<<<<
 *                     dkkk = k
 *                     diii = i
 */
          goto __pyx_L10;
        }

        /* "dipy/align/vector_fields.pyx":1019
 *                     djjj = j
 *                 else:
 *                     dkkk = _apply_affine_3d_x0(k, i, j, 1, premult_index)             # <<<<<<<<<<<<<<
 *                     diii = _apply_affine_3d_x1(k, i, j, 1, premult_index)
 *                     djjj = _apply_affine_3d_x2(k, i, j, 1, premult_index)
 */
        /*else*/ {
          __pyx_v_dkkk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_premult_index);

          /* "dipy/align/vector_fields.pyx":1020
 *                 else:
 *                     dkkk = _apply_affine_3d_x0(k, i, j, 1, premult_index)
 *                     diii = _apply_affine_3d_x1(k, i, j, 1, premult_index)             # <<<<<<<<<<<<<<
 *                     djjj = _apply_affine_3d_x2(k, i, j, 1, premult_index)
 * 
 */
          __pyx_v_diii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_premult_index);

          /* "dipy/align/vector_fields.pyx":1021
 *                     dkkk = _apply_affine_3d_x0(k, i, j, 1, premult_index)
 *                     diii = _apply_affine_3d_x1(k, i, j, 1, premult_index)
 *                     djjj = _apply_affine_3d_x2(k, i, j, 1, premult_index)             # <<<<<<<<<<<<<<
 * 
 *                 dkkk += dk
 */
          __pyx_v_djjj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_premult_index);
        }
        __pyx_L10:;

        /* "dipy/align/vector_fields.pyx":1023
 *                     djjj = _apply_affine_3d_x2(k, i, j, 1, premult_index)
 * 
 *                 dkkk += dk             # <<<<<<<<<<<<<<
 *                 diii += di
 *                 djjj += dj
 */
        __pyx_v_dkkk = (__pyx_v_dkkk + __pyx_v_dk);

        /* "dipy/align/vector_fields.pyx":1024
 * 
 *                 dkkk += dk
 *                 diii += di             # <<<<<<<<<<<<<<
 *                 djjj += dj
 * 
 */
        __pyx_v_diii = (__pyx_v_diii + __pyx_v_di);

        /* "dipy/align/vector_fields.pyx":1025
 *                 dkkk += dk
 *                 diii += di
 *                 djjj += dj             # <<<<<<<<<<<<<<
 * 
 *                 # If d1 and comp are the same array, this will correctly update
 */
        __pyx_v_djjj = (__pyx_v_djjj + __pyx_v_dj);

        /* "dipy/align/vector_fields.pyx":1033
 *                 # from d2, which may be problematic
 *                 inside = _interpolate_vector_3d[floating](d2, dkkk, diii,
 *                                                           djjj, comp[k, i, j])             # <<<<<<<<<<<<<<
 * 
 *                 if inside == 1:
 */
        __pyx_t_20.data = __pyx_v_comp.data;
        __pyx_t_20.memview = __pyx_v_comp.memview;
        __PYX_INC_MEMVIEW(&__pyx_t_20, 0);
        {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_k;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1033; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_20.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[1];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[1];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 1)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1033; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_20.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_j;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[2];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[2];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 2)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1033; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_20.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_20.shape[0] = __pyx_v_comp.shape[3];
__pyx_t_20.strides[0] = __pyx_v_comp.strides[3];
    __pyx_t_20.suboffsets[0] = -1;

__pyx_v_inside = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d2, __pyx_v_dkkk, __pyx_v_diii, __pyx_v_djjj, __pyx_t_20);

        /* "dipy/align/vector_fields.pyx":1032
 *                 # may be in the neighborhood of a previously updated vector
 *                 # from d2, which may be problematic
 *                 inside = _interpolate_vector_3d[floating](d2, dkkk, diii,             # <<<<<<<<<<<<<<
 *                                                           djjj, comp[k, i, j])
 * 
 */
        __PYX_XDEC_MEMVIEW(&__pyx_t_20, 0);

        /* "dipy/align/vector_fields.pyx":1035
 *                                                           djjj, comp[k, i, j])
 * 
 *                 if inside == 1:             # <<<<<<<<<<<<<<
 *                     comp[k, i, j, 0] = t * comp[k, i, j, 0] + dkk
 *                     comp[k, i, j, 1] = t * comp[k, i, j, 1] + dii
 */
        __pyx_t_19 = ((__pyx_v_inside == 1) != 0);
        if (__pyx_t_19) {

          /* "dipy/align/vector_fields.pyx":1036
 * 
 *                 if inside == 1:
 *                     comp[k, i, j, 0] = t * comp[k, i, j, 0] + dkk             # <<<<<<<<<<<<<<
 *                     comp[k, i, j, 1] = t * comp[k, i, j, 1] + dii
 *                     comp[k, i, j, 2] = t * comp[k, i, j, 2] + djj
 */
          __pyx_t_21 = __pyx_v_k;
          __pyx_t_22 = __pyx_v_i;
          __pyx_t_23 = __pyx_v_j;
          __pyx_t_24 = 0;
          __pyx_t_25 = __pyx_v_k;
          __pyx_t_26 = __pyx_v_i;
          __pyx_t_27 = __pyx_v_j;
          __pyx_t_28 = 0;
          *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_25 * __pyx_v_comp.strides[0]) ) + __pyx_t_26 * __pyx_v_comp.strides[1]) ) + __pyx_t_27 * __pyx_v_comp.strides[2]) ) + __pyx_t_28 * __pyx_v_comp.strides[3]) )) = ((__pyx_v_t * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_21 * __pyx_v_comp.strides[0]) ) + __pyx_t_22 * __pyx_v_comp.strides[1]) ) + __pyx_t_23 * __pyx_v_comp.strides[2]) ) + __pyx_t_24 * __pyx_v_comp.strides[3]) )))) + __pyx_v_dkk);

          /* "dipy/align/vector_fields.pyx":1037
 *                 if inside == 1:
 *                     comp[k, i, j, 0] = t * comp[k, i, j, 0] + dkk
 *                     comp[k, i, j, 1] = t * comp[k, i, j, 1] + dii             # <<<<<<<<<<<<<<
 *                     comp[k, i, j, 2] = t * comp[k, i, j, 2] + djj
 *                     nn = (comp[k, i, j, 0] ** 2 + comp[k, i, j, 1] ** 2 +
 */
          __pyx_t_29 = __pyx_v_k;
          __pyx_t_30 = __pyx_v_i;
          __pyx_t_31 = __pyx_v_j;
          __pyx_t_32 = 1;
          __pyx_t_33 = __pyx_v_k;
          __pyx_t_34 = __pyx_v_i;
          __pyx_t_35 = __pyx_v_j;
          __pyx_t_36 = 1;
          *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_33 * __pyx_v_comp.strides[0]) ) + __pyx_t_34 * __pyx_v_comp.strides[1]) ) + __pyx_t_35 * __pyx_v_comp.strides[2]) ) + __pyx_t_36 * __pyx_v_comp.strides[3]) )) = ((__pyx_v_t * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_29 * __pyx_v_comp.strides[0]) ) + __pyx_t_30 * __pyx_v_comp.strides[1]) ) + __pyx_t_31 * __pyx_v_comp.strides[2]) ) + __pyx_t_32 * __pyx_v_comp.strides[3]) )))) + __pyx_v_dii);

          /* "dipy/align/vector_fields.pyx":1038
 *                     comp[k, i, j, 0] = t * comp[k, i, j, 0] + dkk
 *                     comp[k, i, j, 1] = t * comp[k, i, j, 1] + dii
 *                     comp[k, i, j, 2] = t * comp[k, i, j, 2] + djj             # <<<<<<<<<<<<<<
 *                     nn = (comp[k, i, j, 0] ** 2 + comp[k, i, j, 1] ** 2 +
 *                           comp[k, i, j, 2]**2)
 */
          __pyx_t_37 = __pyx_v_k;
          __pyx_t_38 = __pyx_v_i;
          __pyx_t_39 = __pyx_v_j;
          __pyx_t_40 = 2;
          __pyx_t_41 = __pyx_v_k;
          __pyx_t_42 = __pyx_v_i;
          __pyx_t_43 = __pyx_v_j;
          __pyx_t_44 = 2;
          *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_41 * __pyx_v_comp.strides[0]) ) + __pyx_t_42 * __pyx_v_comp.strides[1]) ) + __pyx_t_43 * __pyx_v_comp.strides[2]) ) + __pyx_t_44 * __pyx_v_comp.strides[3]) )) = ((__pyx_v_t * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_37 * __pyx_v_comp.strides[0]) ) + __pyx_t_38 * __pyx_v_comp.strides[1]) ) + __pyx_t_39 * __pyx_v_comp.strides[2]) ) + __pyx_t_40 * __pyx_v_comp.strides[3]) )))) + __pyx_v_djj);

          /* "dipy/align/vector_fields.pyx":1039
 *                     comp[k, i, j, 1] = t * comp[k, i, j, 1] + dii
 *                     comp[k, i, j, 2] = t * comp[k, i, j, 2] + djj
 *                     nn = (comp[k, i, j, 0] ** 2 + comp[k, i, j, 1] ** 2 +             # <<<<<<<<<<<<<<
 *                           comp[k, i, j, 2]**2)
 *                     meanNorm += nn
 */
          __pyx_t_45 = __pyx_v_k;
          __pyx_t_46 = __pyx_v_i;
          __pyx_t_47 = __pyx_v_j;
          __pyx_t_48 = 0;
          __pyx_t_49 = __pyx_v_k;
          __pyx_t_50 = __pyx_v_i;
          __pyx_t_51 = __pyx_v_j;
          __pyx_t_52 = 1;

          /* "dipy/align/vector_fields.pyx":1040
 *                     comp[k, i, j, 2] = t * comp[k, i, j, 2] + djj
 *                     nn = (comp[k, i, j, 0] ** 2 + comp[k, i, j, 1] ** 2 +
 *                           comp[k, i, j, 2]**2)             # <<<<<<<<<<<<<<
 *                     meanNorm += nn
 *                     stdNorm += nn * nn
 */
          __pyx_t_53 = __pyx_v_k;
          __pyx_t_54 = __pyx_v_i;
          __pyx_t_55 = __pyx_v_j;
          __pyx_t_56 = 2;

          /* "dipy/align/vector_fields.pyx":1039
 *                     comp[k, i, j, 1] = t * comp[k, i, j, 1] + dii
 *                     comp[k, i, j, 2] = t * comp[k, i, j, 2] + djj
 *                     nn = (comp[k, i, j, 0] ** 2 + comp[k, i, j, 1] ** 2 +             # <<<<<<<<<<<<<<
 *                           comp[k, i, j, 2]**2)
 *                     meanNorm += nn
 */
          __pyx_v_nn = ((pow((*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_45 * __pyx_v_comp.strides[0]) ) + __pyx_t_46 * __pyx_v_comp.strides[1]) ) + __pyx_t_47 * __pyx_v_comp.strides[2]) ) + __pyx_t_48 * __pyx_v_comp.strides[3]) ))), 2.0) + pow((*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_49 * __pyx_v_comp.strides[0]) ) + __pyx_t_50 * __pyx_v_comp.strides[1]) ) + __pyx_t_51 * __pyx_v_comp.strides[2]) ) + __pyx_t_52 * __pyx_v_comp.strides[3]) ))), 2.0)) + pow((*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_comp.data + __pyx_t_53 * __pyx_v_comp.strides[0]) ) + __pyx_t_54 * __pyx_v_comp.strides[1]) ) + __pyx_t_55 * __pyx_v_comp.strides[2]) ) + __pyx_t_56 * __pyx_v_comp.strides[3]) ))), 2.0));

          /* "dipy/align/vector_fields.pyx":1041
 *                     nn = (comp[k, i, j, 0] ** 2 + comp[k, i, j, 1] ** 2 +
 *                           comp[k, i, j, 2]**2)
 *                     meanNorm += nn             # <<<<<<<<<<<<<<
 *                     stdNorm += nn * nn
 *                     cnt += 1
 */
          __pyx_v_meanNorm = (__pyx_v_meanNorm + __pyx_v_nn);

          /* "dipy/align/vector_fields.pyx":1042
 *                           comp[k, i, j, 2]**2)
 *                     meanNorm += nn
 *                     stdNorm += nn * nn             # <<<<<<<<<<<<<<
 *                     cnt += 1
 *                     if(maxNorm < nn):
 */
          __pyx_v_stdNorm = (__pyx_v_stdNorm + (__pyx_v_nn * __pyx_v_nn));

          /* "dipy/align/vector_fields.pyx":1043
 *                     meanNorm += nn
 *                     stdNorm += nn * nn
 *                     cnt += 1             # <<<<<<<<<<<<<<
 *                     if(maxNorm < nn):
 *                         maxNorm = nn
 */
          __pyx_v_cnt = (__pyx_v_cnt + 1);

          /* "dipy/align/vector_fields.pyx":1044
 *                     stdNorm += nn * nn
 *                     cnt += 1
 *                     if(maxNorm < nn):             # <<<<<<<<<<<<<<
 *                         maxNorm = nn
 *                 else:
 */
          __pyx_t_19 = ((__pyx_v_maxNorm < __pyx_v_nn) != 0);
          if (__pyx_t_19) {

            /* "dipy/align/vector_fields.pyx":1045
 *                     cnt += 1
 *                     if(maxNorm < nn):
 *                         maxNorm = nn             # <<<<<<<<<<<<<<
 *                 else:
 *                     comp[k, i, j, :] = 0
 */
            __pyx_v_maxNorm = __pyx_v_nn;

            /* "dipy/align/vector_fields.pyx":1044
 *                     stdNorm += nn * nn
 *                     cnt += 1
 *                     if(maxNorm < nn):             # <<<<<<<<<<<<<<
 *                         maxNorm = nn
 *                 else:
 */
          }

          /* "dipy/align/vector_fields.pyx":1035
 *                                                           djjj, comp[k, i, j])
 * 
 *                 if inside == 1:             # <<<<<<<<<<<<<<
 *                     comp[k, i, j, 0] = t * comp[k, i, j, 0] + dkk
 *                     comp[k, i, j, 1] = t * comp[k, i, j, 1] + dii
 */
          goto __pyx_L11;
        }

        /* "dipy/align/vector_fields.pyx":1047
 *                         maxNorm = nn
 *                 else:
 *                     comp[k, i, j, :] = 0             # <<<<<<<<<<<<<<
 *     meanNorm /= cnt
 *     stats[0] = sqrt(maxNorm)
 */
        /*else*/ {
          __pyx_t_20.data = __pyx_v_comp.data;
          __pyx_t_20.memview = __pyx_v_comp.memview;
          __PYX_INC_MEMVIEW(&__pyx_t_20, 0);
          {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_k;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1047; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_20.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[1];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[1];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 1)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1047; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_20.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_j;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_comp.shape[2];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_comp.strides[2];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 2)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1047; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_20.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_20.shape[0] = __pyx_v_comp.shape[3];
__pyx_t_20.strides[0] = __pyx_v_comp.strides[3];
    __pyx_t_20.suboffsets[0] = -1;

{
              double __pyx_temp_scalar = 0.0;
              {
                  Py_ssize_t __pyx_temp_extent_0 = __pyx_t_20.shape[0];
                  Py_ssize_t __pyx_temp_stride_0 = __pyx_t_20.strides[0];
                  char *__pyx_temp_pointer_0;
                  Py_ssize_t __pyx_temp_idx_0;
                  __pyx_temp_pointer_0 = __pyx_t_20.data;
                  for (__pyx_temp_idx_0 = 0; __pyx_temp_idx_0 < __pyx_temp_extent_0; __pyx_temp_idx_0++) {
                    *((double *) __pyx_temp_pointer_0) = __pyx_temp_scalar;
                    __pyx_temp_pointer_0 += __pyx_temp_stride_0;
                  }
              }
          }
          __PYX_XDEC_MEMVIEW(&__pyx_t_20, 0);
        }
        __pyx_L11:;
      }
    }
  }

  /* "dipy/align/vector_fields.pyx":1048
 *                 else:
 *                     comp[k, i, j, :] = 0
 *     meanNorm /= cnt             # <<<<<<<<<<<<<<
 *     stats[0] = sqrt(maxNorm)
 *     stats[1] = sqrt(meanNorm)
 */
  __pyx_v_meanNorm = (__pyx_v_meanNorm / __pyx_v_cnt);

  /* "dipy/align/vector_fields.pyx":1049
 *                     comp[k, i, j, :] = 0
 *     meanNorm /= cnt
 *     stats[0] = sqrt(maxNorm)             # <<<<<<<<<<<<<<
 *     stats[1] = sqrt(meanNorm)
 *     stats[2] = sqrt(stdNorm / cnt - meanNorm * meanNorm)
 */
  __pyx_t_57 = 0;
  *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_57 * __pyx_v_stats.strides[0]) )) = sqrt(__pyx_v_maxNorm);

  /* "dipy/align/vector_fields.pyx":1050
 *     meanNorm /= cnt
 *     stats[0] = sqrt(maxNorm)
 *     stats[1] = sqrt(meanNorm)             # <<<<<<<<<<<<<<
 *     stats[2] = sqrt(stdNorm / cnt - meanNorm * meanNorm)
 * 
 */
  __pyx_t_58 = 1;
  *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_58 * __pyx_v_stats.strides[0]) )) = sqrt(__pyx_v_meanNorm);

  /* "dipy/align/vector_fields.pyx":1051
 *     stats[0] = sqrt(maxNorm)
 *     stats[1] = sqrt(meanNorm)
 *     stats[2] = sqrt(stdNorm / cnt - meanNorm * meanNorm)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_59 = 2;
  *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_59 * __pyx_v_stats.strides[0]) )) = sqrt(((__pyx_v_stdNorm / __pyx_v_cnt) - (__pyx_v_meanNorm * __pyx_v_meanNorm)));

  /* "dipy/align/vector_fields.pyx":915
 * 
 * 
 * cdef void _compose_vector_fields_3d(floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                                     floating[:, :, :, :] d2,
 *                                     double[:, :] premult_index,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __PYX_XDEC_MEMVIEW(&__pyx_t_20, 0);
  __Pyx_WriteUnraisable("dipy.align.vector_fields._compose_vector_fields_3d", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 1);
  __pyx_L0:;
}

/* "dipy/align/vector_fields.pyx":1054
 * 
 * 
 * def compose_vector_fields_3d(floating[:, :, :, :] d1, floating[:, :, :, :] d2,             # <<<<<<<<<<<<<<
 *                              double[:, :] premult_index,
 *                              double[:, :] premult_disp,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_17compose_vector_fields_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_16compose_vector_fields_3d[] = "Computes the composition of two 3D displacement fields\n\n    Computes the composition of the two 3-D displacements d1 and d2. The\n    evaluation of d2 at non-lattice points is computed using tri-linear\n    interpolation. The actual composition is computed as:\n\n    comp[i] = d1[i] + t * d2[ A * i + B * d1[i] ]\n\n    where t = time_scaling, A = premult_index and B=premult_disp and i denotes\n    the voxel coordinates of a voxel in d1's grid. Using this parameters it is\n    possible to compose vector fields with arbitrary discretization: let R and\n    S be the voxel-to-space transformation associated to d1 and d2,\n    respectively then the composition at a voxel with coordinates i in d1's\n    grid is given by:\n\n    comp[i] = d1[i] + R*i + d2[Sinv*(R*i + d1[i])] - R*i\n\n    (the R*i terms cancel each other) where Sinv = S^{-1}\n    we can then define A = Sinv * R and B = Sinv to compute the composition\n    using this function.\n\n    Parameters\n    ----------\n    d1 : array, shape (S, R, C, 3)\n        first displacement field to be applied. S, R, C are the number of\n        slices, rows and columns of the displacement field, respectively.\n    d2 : array, shape (S', R', C', 3)\n        second displacement field to be applied. R', C' are the number of rows\n        and columns of the displacement field, respectively.\n    premult_index : array, shape (4, 4)\n        the matrix A in the explanation above\n    premult_disp : array, shape (4, 4)\n        the matrix B in the explanation above\n    time_scaling : float\n        this corresponds to the time scaling 't' in the above explanation\n    comp : array, shape (S, R, C, 3), same dimension as d1\n        the buffer to write the composition to. If None, the buffer will be\n        created internally\n\n    Returns\n    -------\n    comp : array, shape (S, R, C, 3), same dimension as d1\n        on output, this array will contain the composition of the two fields\n    stats : array, shape (3,)\n        o""n output, this array will contain three statistics of the vector norms\n        of the composition (maximum, mean, standard_deviation)\n\n    Notes\n    -----\n    If d1[s,r,c] lies outside the domain of d2, then comp[s,r,c] will contain\n    a zero vector.\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_17compose_vector_fields_3d = {"compose_vector_fields_3d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_17compose_vector_fields_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_16compose_vector_fields_3d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_17compose_vector_fields_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_16compose_vector_fields_3d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_16compose_vector_fields_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("compose_vector_fields_3d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_d1, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_d1); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 4) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 4) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__37, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__38, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__39, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__40, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_145compose_vector_fields_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_145compose_vector_fields_3d = {"__pyx_fuse_0compose_vector_fields_3d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_145compose_vector_fields_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_16compose_vector_fields_3d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_145compose_vector_fields_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d2 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_premult_index = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_premult_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  double __pyx_v_time_scaling;
  __Pyx_memviewslice __pyx_v_comp = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compose_vector_fields_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_d1,&__pyx_n_s_d2,&__pyx_n_s_premult_index,&__pyx_n_s_premult_disp,&__pyx_n_s_time_scaling,&__pyx_n_s_comp,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_3d", 1, 6, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_premult_index)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_3d", 1, 6, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_premult_disp)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_3d", 1, 6, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_time_scaling)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_3d", 1, 6, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_comp)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_3d", 1, 6, 6, 5); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "compose_vector_fields_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[0]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d2 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[1]); if (unlikely(!__pyx_v_d2.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_premult_index = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_premult_index.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1055; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_premult_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_premult_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1056; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_time_scaling = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_time_scaling == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1057; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_comp = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[5]); if (unlikely(!__pyx_v_comp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1058; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("compose_vector_fields_3d", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.compose_vector_fields_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_144compose_vector_fields_3d(__pyx_self, __pyx_v_d1, __pyx_v_d2, __pyx_v_premult_index, __pyx_v_premult_disp, __pyx_v_time_scaling, __pyx_v_comp);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_144compose_vector_fields_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_d2, __Pyx_memviewslice __pyx_v_premult_index, __Pyx_memviewslice __pyx_v_premult_disp, double __pyx_v_time_scaling, __Pyx_memviewslice __pyx_v_comp) {
  __Pyx_memviewslice __pyx_v_stats = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_memviewslice __pyx_t_8 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0compose_vector_fields_3d", 0);

  /* "dipy/align/vector_fields.pyx":1112
 *     """
 *     cdef:
 *         double[:] stats = np.zeros(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 * 
 *     if comp is None:
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_tuple__41) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_float64); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_stats = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":1114
 *         double[:] stats = np.zeros(shape=(3,), dtype=np.float64)
 * 
 *     if comp is None:             # <<<<<<<<<<<<<<
 *         comp = np.zeros_like(d1)
 * 
 */
  __pyx_t_6 = ((((PyObject *) __pyx_v_comp.memview) == Py_None) != 0);
  if (__pyx_t_6) {

    /* "dipy/align/vector_fields.pyx":1115
 * 
 *     if comp is None:
 *         comp = np.zeros_like(d1)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(premult_index, 3):
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros_like); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_3) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else {
      __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(__pyx_t_4);
    if (unlikely(!__pyx_t_8.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_XDEC_MEMVIEW(&__pyx_v_comp, 1);
    __pyx_v_comp = __pyx_t_8;
    __pyx_t_8.memview = NULL;
    __pyx_t_8.data = NULL;

    /* "dipy/align/vector_fields.pyx":1114
 *         double[:] stats = np.zeros(shape=(3,), dtype=np.float64)
 * 
 *     if comp is None:             # <<<<<<<<<<<<<<
 *         comp = np.zeros_like(d1)
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1117
 *         comp = np.zeros_like(d1)
 * 
 *     if not is_valid_affine(premult_index, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid index pre-multiplication matrix")
 *     if not is_valid_affine(premult_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_premult_index, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = NULL;
  __pyx_t_9 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_9 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_1) {
    __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1); __pyx_t_1 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_9, __pyx_t_7);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_9, __pyx_int_3);
  __pyx_t_7 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_10 = ((!__pyx_t_6) != 0);
  if (__pyx_t_10) {

    /* "dipy/align/vector_fields.pyx":1118
 * 
 *     if not is_valid_affine(premult_index, 3):
 *         raise ValueError("Invalid index pre-multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(premult_disp, 3):
 *         raise ValueError("Invalid displacement pre-multiplication matrix")
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__42, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1118; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1118; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1117
 *         comp = np.zeros_like(d1)
 * 
 *     if not is_valid_affine(premult_index, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid index pre-multiplication matrix")
 *     if not is_valid_affine(premult_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":1119
 *     if not is_valid_affine(premult_index, 3):
 *         raise ValueError("Invalid index pre-multiplication matrix")
 *     if not is_valid_affine(premult_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement pre-multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1119; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_premult_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1119; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  __pyx_t_9 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_9 = 1;
    }
  }
  __pyx_t_1 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1119; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (__pyx_t_7) {
    __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_7); __pyx_t_7 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_9, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_9, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1119; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_10 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1119; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = ((!__pyx_t_10) != 0);
  if (__pyx_t_6) {

    /* "dipy/align/vector_fields.pyx":1120
 *         raise ValueError("Invalid index pre-multiplication matrix")
 *     if not is_valid_affine(premult_disp, 3):
 *         raise ValueError("Invalid displacement pre-multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     _compose_vector_fields_3d[floating](d1, d2, premult_index, premult_disp,
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__43, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1120; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1120; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1119
 *     if not is_valid_affine(premult_index, 3):
 *         raise ValueError("Invalid index pre-multiplication matrix")
 *     if not is_valid_affine(premult_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement pre-multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1122
 *         raise ValueError("Invalid displacement pre-multiplication matrix")
 * 
 *     _compose_vector_fields_3d[floating](d1, d2, premult_index, premult_disp,             # <<<<<<<<<<<<<<
 *                                         time_scaling, comp, stats)
 *     return comp, stats
 */
  __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__compose_vector_fields_3d(__pyx_v_d1, __pyx_v_d2, __pyx_v_premult_index, __pyx_v_premult_disp, __pyx_v_time_scaling, __pyx_v_comp, __pyx_v_stats);

  /* "dipy/align/vector_fields.pyx":1124
 *     _compose_vector_fields_3d[floating](d1, d2, premult_index, premult_disp,
 *                                         time_scaling, comp, stats)
 *     return comp, stats             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_comp, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_stats, 1, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
  __pyx_t_4 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1054
 * 
 * 
 * def compose_vector_fields_3d(floating[:, :, :, :] d1, floating[:, :, :, :] d2,             # <<<<<<<<<<<<<<
 *                              double[:, :] premult_index,
 *                              double[:, :] premult_disp,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __Pyx_XDECREF(__pyx_t_7);
  __PYX_XDEC_MEMVIEW(&__pyx_t_8, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.compose_vector_fields_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_stats, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d2, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_premult_index, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_premult_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_comp, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_147compose_vector_fields_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_147compose_vector_fields_3d = {"__pyx_fuse_1compose_vector_fields_3d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_147compose_vector_fields_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_16compose_vector_fields_3d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_147compose_vector_fields_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d2 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_premult_index = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_premult_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  double __pyx_v_time_scaling;
  __Pyx_memviewslice __pyx_v_comp = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compose_vector_fields_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_d1,&__pyx_n_s_d2,&__pyx_n_s_premult_index,&__pyx_n_s_premult_disp,&__pyx_n_s_time_scaling,&__pyx_n_s_comp,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d2)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_3d", 1, 6, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_premult_index)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_3d", 1, 6, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_premult_disp)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_3d", 1, 6, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_time_scaling)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_3d", 1, 6, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_comp)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("compose_vector_fields_3d", 1, 6, 6, 5); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "compose_vector_fields_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[0]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d2 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[1]); if (unlikely(!__pyx_v_d2.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_premult_index = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_premult_index.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1055; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_premult_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_premult_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1056; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_time_scaling = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_time_scaling == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1057; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_comp = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[5]); if (unlikely(!__pyx_v_comp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1058; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("compose_vector_fields_3d", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.compose_vector_fields_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_146compose_vector_fields_3d(__pyx_self, __pyx_v_d1, __pyx_v_d2, __pyx_v_premult_index, __pyx_v_premult_disp, __pyx_v_time_scaling, __pyx_v_comp);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_146compose_vector_fields_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_d2, __Pyx_memviewslice __pyx_v_premult_index, __Pyx_memviewslice __pyx_v_premult_disp, double __pyx_v_time_scaling, __Pyx_memviewslice __pyx_v_comp) {
  __Pyx_memviewslice __pyx_v_stats = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_memviewslice __pyx_t_8 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1compose_vector_fields_3d", 0);

  /* "dipy/align/vector_fields.pyx":1112
 *     """
 *     cdef:
 *         double[:] stats = np.zeros(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 * 
 *     if comp is None:
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_tuple__44) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_float64); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_stats = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":1114
 *         double[:] stats = np.zeros(shape=(3,), dtype=np.float64)
 * 
 *     if comp is None:             # <<<<<<<<<<<<<<
 *         comp = np.zeros_like(d1)
 * 
 */
  __pyx_t_6 = ((((PyObject *) __pyx_v_comp.memview) == Py_None) != 0);
  if (__pyx_t_6) {

    /* "dipy/align/vector_fields.pyx":1115
 * 
 *     if comp is None:
 *         comp = np.zeros_like(d1)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(premult_index, 3):
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros_like); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_3) {
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else {
      __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_t_4);
    if (unlikely(!__pyx_t_8.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1115; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_XDEC_MEMVIEW(&__pyx_v_comp, 1);
    __pyx_v_comp = __pyx_t_8;
    __pyx_t_8.memview = NULL;
    __pyx_t_8.data = NULL;

    /* "dipy/align/vector_fields.pyx":1114
 *         double[:] stats = np.zeros(shape=(3,), dtype=np.float64)
 * 
 *     if comp is None:             # <<<<<<<<<<<<<<
 *         comp = np.zeros_like(d1)
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1117
 *         comp = np.zeros_like(d1)
 * 
 *     if not is_valid_affine(premult_index, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid index pre-multiplication matrix")
 *     if not is_valid_affine(premult_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_premult_index, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = NULL;
  __pyx_t_9 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_9 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_1) {
    __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1); __pyx_t_1 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_9, __pyx_t_7);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_9, __pyx_int_3);
  __pyx_t_7 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1117; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_10 = ((!__pyx_t_6) != 0);
  if (__pyx_t_10) {

    /* "dipy/align/vector_fields.pyx":1118
 * 
 *     if not is_valid_affine(premult_index, 3):
 *         raise ValueError("Invalid index pre-multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(premult_disp, 3):
 *         raise ValueError("Invalid displacement pre-multiplication matrix")
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__45, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1118; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1118; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1117
 *         comp = np.zeros_like(d1)
 * 
 *     if not is_valid_affine(premult_index, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid index pre-multiplication matrix")
 *     if not is_valid_affine(premult_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":1119
 *     if not is_valid_affine(premult_index, 3):
 *         raise ValueError("Invalid index pre-multiplication matrix")
 *     if not is_valid_affine(premult_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement pre-multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1119; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_premult_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1119; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = NULL;
  __pyx_t_9 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_9 = 1;
    }
  }
  __pyx_t_1 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1119; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (__pyx_t_7) {
    __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_7); __pyx_t_7 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_9, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_9, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1119; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_10 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1119; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = ((!__pyx_t_10) != 0);
  if (__pyx_t_6) {

    /* "dipy/align/vector_fields.pyx":1120
 *         raise ValueError("Invalid index pre-multiplication matrix")
 *     if not is_valid_affine(premult_disp, 3):
 *         raise ValueError("Invalid displacement pre-multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     _compose_vector_fields_3d[floating](d1, d2, premult_index, premult_disp,
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__46, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1120; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1120; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1119
 *     if not is_valid_affine(premult_index, 3):
 *         raise ValueError("Invalid index pre-multiplication matrix")
 *     if not is_valid_affine(premult_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement pre-multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1122
 *         raise ValueError("Invalid displacement pre-multiplication matrix")
 * 
 *     _compose_vector_fields_3d[floating](d1, d2, premult_index, premult_disp,             # <<<<<<<<<<<<<<
 *                                         time_scaling, comp, stats)
 *     return comp, stats
 */
  __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__compose_vector_fields_3d(__pyx_v_d1, __pyx_v_d2, __pyx_v_premult_index, __pyx_v_premult_disp, __pyx_v_time_scaling, __pyx_v_comp, __pyx_v_stats);

  /* "dipy/align/vector_fields.pyx":1124
 *     _compose_vector_fields_3d[floating](d1, d2, premult_index, premult_disp,
 *                                         time_scaling, comp, stats)
 *     return comp, stats             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_comp, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_stats, 1, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
  __pyx_t_4 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1054
 * 
 * 
 * def compose_vector_fields_3d(floating[:, :, :, :] d1, floating[:, :, :, :] d2,             # <<<<<<<<<<<<<<
 *                              double[:, :] premult_index,
 *                              double[:, :] premult_disp,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __Pyx_XDECREF(__pyx_t_7);
  __PYX_XDEC_MEMVIEW(&__pyx_t_8, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.compose_vector_fields_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_stats, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d2, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_premult_index, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_premult_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_comp, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":1127
 * 
 * 
 * def invert_vector_field_fixed_point_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                                        double[:, :] d_world2grid,
 *                                        double[:] spacing,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_19invert_vector_field_fixed_point_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_18invert_vector_field_fixed_point_2d[] = "Computes the inverse of a 2D displacement fields\n\n    Computes the inverse of the given 2-D displacement field d using the\n    fixed-point algorithm [1].\n\n    [1] Chen, M., Lu, W., Chen, Q., Ruchala, K. J., & Olivera, G. H. (2008).\n        A simple fixed-point approach to invert a deformation field.\n        Medical Physics, 35(1), 81. doi:10.1118/1.2816107\n\n    Parameters\n    ----------\n    d : array, shape (R, C, 2)\n        the 2-D displacement field to be inverted\n    d_world2grid : array, shape (3, 3)\n        the space-to-grid transformation associated to the displacement field\n        d (transforming physical space coordinates to voxel coordinates of the\n        displacement field grid)\n    spacing :array, shape (2,)\n        the spacing between voxels (voxel size along each axis)\n    max_iter : int\n        maximum number of iterations to be performed\n    tolerance : float\n        maximum tolerated inversion error\n    start : array, shape (R, C)\n        an approximation to the inverse displacement field (if no approximation\n        is available, None can be provided and the start displacement field\n        will be zero)\n\n    Returns\n    -------\n    p : array, shape (R, C, 2)\n        the inverse displacement field\n\n    Notes\n    -----\n    We assume that the displacement field is an endomorphism so that the shape\n    and voxel-to-space transformation of the inverse field's discretization is\n    the same as those of the input displacement field. The 'inversion error' at\n    iteration t is defined as the mean norm of the displacement vectors of the\n    input displacement field composed with the inverse at iteration t.\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_19invert_vector_field_fixed_point_2d = {"invert_vector_field_fixed_point_2d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_19invert_vector_field_fixed_point_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_18invert_vector_field_fixed_point_2d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_19invert_vector_field_fixed_point_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_18invert_vector_field_fixed_point_2d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_18invert_vector_field_fixed_point_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("invert_vector_field_fixed_point_2d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_d, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_d); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__47, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__48, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__49, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__50, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_366__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults2, __pyx_self)->__pyx_arg_start, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_151invert_vector_field_fixed_point_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_151invert_vector_field_fixed_point_2d = {"__pyx_fuse_0invert_vector_field_fixed_point_2d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_151invert_vector_field_fixed_point_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_18invert_vector_field_fixed_point_2d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_151invert_vector_field_fixed_point_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_d = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d_world2grid = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_spacing = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_v_max_iter;
  double __pyx_v_tolerance;
  __Pyx_memviewslice __pyx_v_start = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("invert_vector_field_fixed_point_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_d,&__pyx_n_s_d_world2grid,&__pyx_n_s_spacing,&__pyx_n_s_max_iter,&__pyx_n_s_tolerance,&__pyx_n_s_start,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults2 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults2, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d_world2grid)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_2d", 0, 5, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_spacing)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_2d", 0, 5, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_max_iter)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_2d", 0, 5, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tolerance)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_2d", 0, 5, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_start);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "invert_vector_field_fixed_point_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_d = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[0]); if (unlikely(!__pyx_v_d.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d_world2grid = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_d_world2grid.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1128; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_spacing = __Pyx_PyObject_to_MemoryviewSlice_ds_double(values[2]); if (unlikely(!__pyx_v_spacing.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1129; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_max_iter = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_max_iter == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1130; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_tolerance = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_tolerance == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1130; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[5]) {
      __pyx_v_start = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[5]); if (unlikely(!__pyx_v_start.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1131; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_start = __pyx_dynamic_args->__pyx_arg_start;
      __PYX_INC_MEMVIEW(&__pyx_v_start, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_2d", 0, 5, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.invert_vector_field_fixed_point_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_150invert_vector_field_fixed_point_2d(__pyx_self, __pyx_v_d, __pyx_v_d_world2grid, __pyx_v_spacing, __pyx_v_max_iter, __pyx_v_tolerance, __pyx_v_start);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_150invert_vector_field_fixed_point_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_d_world2grid, __Pyx_memviewslice __pyx_v_spacing, int __pyx_v_max_iter, double __pyx_v_tolerance, __Pyx_memviewslice __pyx_v_start) {
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  int __pyx_v_iter_count;
  double __pyx_v_difmag;
  double __pyx_v_mag;
  double __pyx_v_maxlen;
  double __pyx_v_step_factor;
  double __pyx_v_epsilon;
  double __pyx_v_error;
  double __pyx_v_sr;
  double __pyx_v_sc;
  PyObject *__pyx_v_ftype = NULL;
  __Pyx_memviewslice __pyx_v_stats = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_substats = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_norms = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_p = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_q = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_memviewslice __pyx_t_8 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_9 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_10 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  npy_intp __pyx_t_14;
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0invert_vector_field_fixed_point_2d", 0);

  /* "dipy/align/vector_fields.pyx":1174
 *     """
 *     cdef:
 *         cnp.npy_intp nr = d.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = d.shape[1]
 *         int iter_count, current, flag
 */
  __pyx_v_nr = (__pyx_v_d.shape[0]);

  /* "dipy/align/vector_fields.pyx":1175
 *     cdef:
 *         cnp.npy_intp nr = d.shape[0]
 *         cnp.npy_intp nc = d.shape[1]             # <<<<<<<<<<<<<<
 *         int iter_count, current, flag
 *         double difmag, mag, maxlen, step_factor
 */
  __pyx_v_nc = (__pyx_v_d.shape[1]);

  /* "dipy/align/vector_fields.pyx":1179
 *         double difmag, mag, maxlen, step_factor
 *         double epsilon
 *         double error = 1 + tolerance             # <<<<<<<<<<<<<<
 *         double di, dj, dii, djj
 *         double sr = spacing[0], sc = spacing[1]
 */
  __pyx_v_error = (1.0 + __pyx_v_tolerance);

  /* "dipy/align/vector_fields.pyx":1181
 *         double error = 1 + tolerance
 *         double di, dj, dii, djj
 *         double sr = spacing[0], sc = spacing[1]             # <<<<<<<<<<<<<<
 * 
 *     ftype = np.asarray(d).dtype
 */
  __pyx_t_1 = 0;
  __pyx_v_sr = (*((double *) ( /* dim=0 */ (__pyx_v_spacing.data + __pyx_t_1 * __pyx_v_spacing.strides[0]) )));
  __pyx_t_2 = 1;
  __pyx_v_sc = (*((double *) ( /* dim=0 */ (__pyx_v_spacing.data + __pyx_t_2 * __pyx_v_spacing.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":1183
 *         double sr = spacing[0], sc = spacing[1]
 * 
 *     ftype = np.asarray(d).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_d, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_dtype); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_ftype = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "dipy/align/vector_fields.pyx":1185
 *     ftype = np.asarray(d).dtype
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] substats = np.empty(shape=(3,), dtype=np.float64)
 *         double[:, :] norms = np.zeros(shape=(nr, nc), dtype=np.float64)
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_zeros); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_shape, __pyx_tuple__51) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_float64); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_8.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_stats = __pyx_t_8;
  __pyx_t_8.memview = NULL;
  __pyx_t_8.data = NULL;

  /* "dipy/align/vector_fields.pyx":1186
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)
 *         double[:] substats = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:, :] norms = np.zeros(shape=(nr, nc), dtype=np.float64)
 *         floating[:, :, :] p = np.zeros(shape=(nr, nc, 2), dtype=ftype)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_empty); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__52) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_float64); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_7);
  if (unlikely(!__pyx_t_8.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_substats = __pyx_t_8;
  __pyx_t_8.memview = NULL;
  __pyx_t_8.data = NULL;

  /* "dipy/align/vector_fields.pyx":1187
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)
 *         double[:] substats = np.empty(shape=(3,), dtype=np.float64)
 *         double[:, :] norms = np.zeros(shape=(nr, nc), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         floating[:, :, :] p = np.zeros(shape=(nr, nc, 2), dtype=ftype)
 *         floating[:, :, :] q = np.zeros(shape=(nr, nc, 2), dtype=ftype)
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyDict_New(); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nr); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nc); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_3);
  __pyx_t_5 = 0;
  __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_float64); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_7); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_9 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_t_3);
  if (unlikely(!__pyx_t_9.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_norms = __pyx_t_9;
  __pyx_t_9.memview = NULL;
  __pyx_t_9.data = NULL;

  /* "dipy/align/vector_fields.pyx":1188
 *         double[:] substats = np.empty(shape=(3,), dtype=np.float64)
 *         double[:, :] norms = np.zeros(shape=(nr, nc), dtype=np.float64)
 *         floating[:, :, :] p = np.zeros(shape=(nr, nc, 2), dtype=ftype)             # <<<<<<<<<<<<<<
 *         floating[:, :, :] q = np.zeros(shape=(nr, nc, 2), dtype=ftype)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nr); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nc); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_10 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_t_5);
  if (unlikely(!__pyx_t_10.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_p = __pyx_t_10;
  __pyx_t_10.memview = NULL;
  __pyx_t_10.data = NULL;

  /* "dipy/align/vector_fields.pyx":1189
 *         double[:, :] norms = np.zeros(shape=(nr, nc), dtype=np.float64)
 *         floating[:, :, :] p = np.zeros(shape=(nr, nc, 2), dtype=ftype)
 *         floating[:, :, :] q = np.zeros(shape=(nr, nc, 2), dtype=ftype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(d_world2grid, 2):
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_zeros); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_7 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nr); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nc); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_int_2);
  __pyx_t_7 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_shape, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_10 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_t_4);
  if (unlikely(!__pyx_t_10.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_q = __pyx_t_10;
  __pyx_t_10.memview = NULL;
  __pyx_t_10.data = NULL;

  /* "dipy/align/vector_fields.pyx":1191
 *         floating[:, :, :] q = np.zeros(shape=(nr, nc, 2), dtype=ftype)
 * 
 *     if not is_valid_affine(d_world2grid, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid world-to-image transform")
 * 
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_d_world2grid, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = NULL;
  __pyx_t_11 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_11 = 1;
    }
  }
  __pyx_t_7 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_11, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_11, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_12 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_13 = ((!__pyx_t_12) != 0);
  if (__pyx_t_13) {

    /* "dipy/align/vector_fields.pyx":1192
 * 
 *     if not is_valid_affine(d_world2grid, 2):
 *         raise ValueError("Invalid world-to-image transform")             # <<<<<<<<<<<<<<
 * 
 *     if start is not None:
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__53, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1192; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1192; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1191
 *         floating[:, :, :] q = np.zeros(shape=(nr, nc, 2), dtype=ftype)
 * 
 *     if not is_valid_affine(d_world2grid, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid world-to-image transform")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1194
 *         raise ValueError("Invalid world-to-image transform")
 * 
 *     if start is not None:             # <<<<<<<<<<<<<<
 *         p[...] = start
 * 
 */
  __pyx_t_13 = ((((PyObject *) __pyx_v_start.memview) != Py_None) != 0);
  if (__pyx_t_13) {

    /* "dipy/align/vector_fields.pyx":1195
 * 
 *     if start is not None:
 *         p[...] = start             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    if (unlikely(__pyx_memoryview_copy_contents(__pyx_v_start, __pyx_v_p, 3, 3, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1195; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1194
 *         raise ValueError("Invalid world-to-image transform")
 * 
 *     if start is not None:             # <<<<<<<<<<<<<<
 *         p[...] = start
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1197
 *         p[...] = start
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         iter_count = 0
 *         while (iter_count < max_iter) and (tolerance < error):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1198
 * 
 *     with nogil:
 *         iter_count = 0             # <<<<<<<<<<<<<<
 *         while (iter_count < max_iter) and (tolerance < error):
 *             if iter_count == 0:
 */
        __pyx_v_iter_count = 0;

        /* "dipy/align/vector_fields.pyx":1199
 *     with nogil:
 *         iter_count = 0
 *         while (iter_count < max_iter) and (tolerance < error):             # <<<<<<<<<<<<<<
 *             if iter_count == 0:
 *                 epsilon = 0.75
 */
        while (1) {
          __pyx_t_12 = ((__pyx_v_iter_count < __pyx_v_max_iter) != 0);
          if (__pyx_t_12) {
          } else {
            __pyx_t_13 = __pyx_t_12;
            goto __pyx_L10_bool_binop_done;
          }
          __pyx_t_12 = ((__pyx_v_tolerance < __pyx_v_error) != 0);
          __pyx_t_13 = __pyx_t_12;
          __pyx_L10_bool_binop_done:;
          if (!__pyx_t_13) break;

          /* "dipy/align/vector_fields.pyx":1200
 *         iter_count = 0
 *         while (iter_count < max_iter) and (tolerance < error):
 *             if iter_count == 0:             # <<<<<<<<<<<<<<
 *                 epsilon = 0.75
 *             else:
 */
          __pyx_t_13 = ((__pyx_v_iter_count == 0) != 0);
          if (__pyx_t_13) {

            /* "dipy/align/vector_fields.pyx":1201
 *         while (iter_count < max_iter) and (tolerance < error):
 *             if iter_count == 0:
 *                 epsilon = 0.75             # <<<<<<<<<<<<<<
 *             else:
 *                 epsilon = 0.5
 */
            __pyx_v_epsilon = 0.75;

            /* "dipy/align/vector_fields.pyx":1200
 *         iter_count = 0
 *         while (iter_count < max_iter) and (tolerance < error):
 *             if iter_count == 0:             # <<<<<<<<<<<<<<
 *                 epsilon = 0.75
 *             else:
 */
            goto __pyx_L12;
          }

          /* "dipy/align/vector_fields.pyx":1203
 *                 epsilon = 0.75
 *             else:
 *                 epsilon = 0.5             # <<<<<<<<<<<<<<
 *             _compose_vector_fields_2d[floating](p, d, None, d_world2grid,
 *                                                 1.0, q, substats)
 */
          /*else*/ {
            __pyx_v_epsilon = 0.5;
          }
          __pyx_L12:;

          /* "dipy/align/vector_fields.pyx":1204
 *             else:
 *                 epsilon = 0.5
 *             _compose_vector_fields_2d[floating](p, d, None, d_world2grid,             # <<<<<<<<<<<<<<
 *                                                 1.0, q, substats)
 *             difmag = 0
 */
          __pyx_t_9 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
          if (unlikely(!__pyx_t_9.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1204; __pyx_clineno = __LINE__; goto __pyx_L6_error;}

          /* "dipy/align/vector_fields.pyx":1205
 *                 epsilon = 0.5
 *             _compose_vector_fields_2d[floating](p, d, None, d_world2grid,
 *                                                 1.0, q, substats)             # <<<<<<<<<<<<<<
 *             difmag = 0
 *             error = 0
 */
          __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__compose_vector_fields_2d(__pyx_v_p, __pyx_v_d, __pyx_t_9, __pyx_v_d_world2grid, 1.0, __pyx_v_q, __pyx_v_substats);
          __PYX_XDEC_MEMVIEW(&__pyx_t_9, 0);

          /* "dipy/align/vector_fields.pyx":1206
 *             _compose_vector_fields_2d[floating](p, d, None, d_world2grid,
 *                                                 1.0, q, substats)
 *             difmag = 0             # <<<<<<<<<<<<<<
 *             error = 0
 *             for i in range(nr):
 */
          __pyx_v_difmag = 0.0;

          /* "dipy/align/vector_fields.pyx":1207
 *                                                 1.0, q, substats)
 *             difmag = 0
 *             error = 0             # <<<<<<<<<<<<<<
 *             for i in range(nr):
 *                 for j in range(nc):
 */
          __pyx_v_error = 0.0;

          /* "dipy/align/vector_fields.pyx":1208
 *             difmag = 0
 *             error = 0
 *             for i in range(nr):             # <<<<<<<<<<<<<<
 *                 for j in range(nc):
 *                     mag = sqrt((q[i, j, 0]/sr) ** 2 + (q[i, j, 1]/sc) ** 2)
 */
          __pyx_t_14 = __pyx_v_nr;
          for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
            __pyx_v_i = __pyx_t_15;

            /* "dipy/align/vector_fields.pyx":1209
 *             error = 0
 *             for i in range(nr):
 *                 for j in range(nc):             # <<<<<<<<<<<<<<
 *                     mag = sqrt((q[i, j, 0]/sr) ** 2 + (q[i, j, 1]/sc) ** 2)
 *                     norms[i, j] = mag
 */
            __pyx_t_16 = __pyx_v_nc;
            for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
              __pyx_v_j = __pyx_t_17;

              /* "dipy/align/vector_fields.pyx":1210
 *             for i in range(nr):
 *                 for j in range(nc):
 *                     mag = sqrt((q[i, j, 0]/sr) ** 2 + (q[i, j, 1]/sc) ** 2)             # <<<<<<<<<<<<<<
 *                     norms[i, j] = mag
 *                     error += mag
 */
              __pyx_t_18 = __pyx_v_i;
              __pyx_t_19 = __pyx_v_j;
              __pyx_t_20 = 0;
              __pyx_t_21 = __pyx_v_i;
              __pyx_t_22 = __pyx_v_j;
              __pyx_t_23 = 1;
              __pyx_v_mag = sqrt((pow(((*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_18 * __pyx_v_q.strides[0]) ) + __pyx_t_19 * __pyx_v_q.strides[1]) ) + __pyx_t_20 * __pyx_v_q.strides[2]) ))) / __pyx_v_sr), 2.0) + pow(((*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_21 * __pyx_v_q.strides[0]) ) + __pyx_t_22 * __pyx_v_q.strides[1]) ) + __pyx_t_23 * __pyx_v_q.strides[2]) ))) / __pyx_v_sc), 2.0)));

              /* "dipy/align/vector_fields.pyx":1211
 *                 for j in range(nc):
 *                     mag = sqrt((q[i, j, 0]/sr) ** 2 + (q[i, j, 1]/sc) ** 2)
 *                     norms[i, j] = mag             # <<<<<<<<<<<<<<
 *                     error += mag
 *                     if(difmag < mag):
 */
              __pyx_t_24 = __pyx_v_i;
              __pyx_t_25 = __pyx_v_j;
              *((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_norms.data + __pyx_t_24 * __pyx_v_norms.strides[0]) ) + __pyx_t_25 * __pyx_v_norms.strides[1]) )) = __pyx_v_mag;

              /* "dipy/align/vector_fields.pyx":1212
 *                     mag = sqrt((q[i, j, 0]/sr) ** 2 + (q[i, j, 1]/sc) ** 2)
 *                     norms[i, j] = mag
 *                     error += mag             # <<<<<<<<<<<<<<
 *                     if(difmag < mag):
 *                         difmag = mag
 */
              __pyx_v_error = (__pyx_v_error + __pyx_v_mag);

              /* "dipy/align/vector_fields.pyx":1213
 *                     norms[i, j] = mag
 *                     error += mag
 *                     if(difmag < mag):             # <<<<<<<<<<<<<<
 *                         difmag = mag
 *             maxlen = difmag * epsilon
 */
              __pyx_t_13 = ((__pyx_v_difmag < __pyx_v_mag) != 0);
              if (__pyx_t_13) {

                /* "dipy/align/vector_fields.pyx":1214
 *                     error += mag
 *                     if(difmag < mag):
 *                         difmag = mag             # <<<<<<<<<<<<<<
 *             maxlen = difmag * epsilon
 *             for i in range(nr):
 */
                __pyx_v_difmag = __pyx_v_mag;

                /* "dipy/align/vector_fields.pyx":1213
 *                     norms[i, j] = mag
 *                     error += mag
 *                     if(difmag < mag):             # <<<<<<<<<<<<<<
 *                         difmag = mag
 *             maxlen = difmag * epsilon
 */
              }
            }
          }

          /* "dipy/align/vector_fields.pyx":1215
 *                     if(difmag < mag):
 *                         difmag = mag
 *             maxlen = difmag * epsilon             # <<<<<<<<<<<<<<
 *             for i in range(nr):
 *                 for j in range(nc):
 */
          __pyx_v_maxlen = (__pyx_v_difmag * __pyx_v_epsilon);

          /* "dipy/align/vector_fields.pyx":1216
 *                         difmag = mag
 *             maxlen = difmag * epsilon
 *             for i in range(nr):             # <<<<<<<<<<<<<<
 *                 for j in range(nc):
 *                     if norms[i, j] > maxlen:
 */
          __pyx_t_14 = __pyx_v_nr;
          for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
            __pyx_v_i = __pyx_t_15;

            /* "dipy/align/vector_fields.pyx":1217
 *             maxlen = difmag * epsilon
 *             for i in range(nr):
 *                 for j in range(nc):             # <<<<<<<<<<<<<<
 *                     if norms[i, j] > maxlen:
 *                         step_factor = epsilon * maxlen / norms[i, j]
 */
            __pyx_t_16 = __pyx_v_nc;
            for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
              __pyx_v_j = __pyx_t_17;

              /* "dipy/align/vector_fields.pyx":1218
 *             for i in range(nr):
 *                 for j in range(nc):
 *                     if norms[i, j] > maxlen:             # <<<<<<<<<<<<<<
 *                         step_factor = epsilon * maxlen / norms[i, j]
 *                     else:
 */
              __pyx_t_26 = __pyx_v_i;
              __pyx_t_27 = __pyx_v_j;
              __pyx_t_13 = (((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_norms.data + __pyx_t_26 * __pyx_v_norms.strides[0]) ) + __pyx_t_27 * __pyx_v_norms.strides[1]) ))) > __pyx_v_maxlen) != 0);
              if (__pyx_t_13) {

                /* "dipy/align/vector_fields.pyx":1219
 *                 for j in range(nc):
 *                     if norms[i, j] > maxlen:
 *                         step_factor = epsilon * maxlen / norms[i, j]             # <<<<<<<<<<<<<<
 *                     else:
 *                         step_factor = epsilon
 */
                __pyx_t_28 = __pyx_v_i;
                __pyx_t_29 = __pyx_v_j;
                __pyx_v_step_factor = ((__pyx_v_epsilon * __pyx_v_maxlen) / (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_norms.data + __pyx_t_28 * __pyx_v_norms.strides[0]) ) + __pyx_t_29 * __pyx_v_norms.strides[1]) ))));

                /* "dipy/align/vector_fields.pyx":1218
 *             for i in range(nr):
 *                 for j in range(nc):
 *                     if norms[i, j] > maxlen:             # <<<<<<<<<<<<<<
 *                         step_factor = epsilon * maxlen / norms[i, j]
 *                     else:
 */
                goto __pyx_L22;
              }

              /* "dipy/align/vector_fields.pyx":1221
 *                         step_factor = epsilon * maxlen / norms[i, j]
 *                     else:
 *                         step_factor = epsilon             # <<<<<<<<<<<<<<
 *                     p[i, j, 0] = p[i, j, 0] - step_factor * q[i, j, 0]
 *                     p[i, j, 1] = p[i, j, 1] - step_factor * q[i, j, 1]
 */
              /*else*/ {
                __pyx_v_step_factor = __pyx_v_epsilon;
              }
              __pyx_L22:;

              /* "dipy/align/vector_fields.pyx":1222
 *                     else:
 *                         step_factor = epsilon
 *                     p[i, j, 0] = p[i, j, 0] - step_factor * q[i, j, 0]             # <<<<<<<<<<<<<<
 *                     p[i, j, 1] = p[i, j, 1] - step_factor * q[i, j, 1]
 *             error /= (nr * nc)
 */
              __pyx_t_30 = __pyx_v_i;
              __pyx_t_31 = __pyx_v_j;
              __pyx_t_32 = 0;
              __pyx_t_33 = __pyx_v_i;
              __pyx_t_34 = __pyx_v_j;
              __pyx_t_35 = 0;
              __pyx_t_36 = __pyx_v_i;
              __pyx_t_37 = __pyx_v_j;
              __pyx_t_38 = 0;
              *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_36 * __pyx_v_p.strides[0]) ) + __pyx_t_37 * __pyx_v_p.strides[1]) ) + __pyx_t_38 * __pyx_v_p.strides[2]) )) = ((*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_30 * __pyx_v_p.strides[0]) ) + __pyx_t_31 * __pyx_v_p.strides[1]) ) + __pyx_t_32 * __pyx_v_p.strides[2]) ))) - (__pyx_v_step_factor * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_33 * __pyx_v_q.strides[0]) ) + __pyx_t_34 * __pyx_v_q.strides[1]) ) + __pyx_t_35 * __pyx_v_q.strides[2]) )))));

              /* "dipy/align/vector_fields.pyx":1223
 *                         step_factor = epsilon
 *                     p[i, j, 0] = p[i, j, 0] - step_factor * q[i, j, 0]
 *                     p[i, j, 1] = p[i, j, 1] - step_factor * q[i, j, 1]             # <<<<<<<<<<<<<<
 *             error /= (nr * nc)
 *             iter_count += 1
 */
              __pyx_t_39 = __pyx_v_i;
              __pyx_t_40 = __pyx_v_j;
              __pyx_t_41 = 1;
              __pyx_t_42 = __pyx_v_i;
              __pyx_t_43 = __pyx_v_j;
              __pyx_t_44 = 1;
              __pyx_t_45 = __pyx_v_i;
              __pyx_t_46 = __pyx_v_j;
              __pyx_t_47 = 1;
              *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_45 * __pyx_v_p.strides[0]) ) + __pyx_t_46 * __pyx_v_p.strides[1]) ) + __pyx_t_47 * __pyx_v_p.strides[2]) )) = ((*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_39 * __pyx_v_p.strides[0]) ) + __pyx_t_40 * __pyx_v_p.strides[1]) ) + __pyx_t_41 * __pyx_v_p.strides[2]) ))) - (__pyx_v_step_factor * (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_42 * __pyx_v_q.strides[0]) ) + __pyx_t_43 * __pyx_v_q.strides[1]) ) + __pyx_t_44 * __pyx_v_q.strides[2]) )))));
            }
          }

          /* "dipy/align/vector_fields.pyx":1224
 *                     p[i, j, 0] = p[i, j, 0] - step_factor * q[i, j, 0]
 *                     p[i, j, 1] = p[i, j, 1] - step_factor * q[i, j, 1]
 *             error /= (nr * nc)             # <<<<<<<<<<<<<<
 *             iter_count += 1
 *         stats[0] = substats[1]
 */
          __pyx_v_error = (__pyx_v_error / (__pyx_v_nr * __pyx_v_nc));

          /* "dipy/align/vector_fields.pyx":1225
 *                     p[i, j, 1] = p[i, j, 1] - step_factor * q[i, j, 1]
 *             error /= (nr * nc)
 *             iter_count += 1             # <<<<<<<<<<<<<<
 *         stats[0] = substats[1]
 *         stats[1] = iter_count
 */
          __pyx_v_iter_count = (__pyx_v_iter_count + 1);
        }

        /* "dipy/align/vector_fields.pyx":1226
 *             error /= (nr * nc)
 *             iter_count += 1
 *         stats[0] = substats[1]             # <<<<<<<<<<<<<<
 *         stats[1] = iter_count
 *     return p
 */
        __pyx_t_48 = 1;
        __pyx_t_49 = 0;
        *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_49 * __pyx_v_stats.strides[0]) )) = (*((double *) ( /* dim=0 */ (__pyx_v_substats.data + __pyx_t_48 * __pyx_v_substats.strides[0]) )));

        /* "dipy/align/vector_fields.pyx":1227
 *             iter_count += 1
 *         stats[0] = substats[1]
 *         stats[1] = iter_count             # <<<<<<<<<<<<<<
 *     return p
 * 
 */
        __pyx_t_50 = 1;
        *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_50 * __pyx_v_stats.strides[0]) )) = __pyx_v_iter_count;
      }

      /* "dipy/align/vector_fields.pyx":1197
 *         p[...] = start
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         iter_count = 0
 *         while (iter_count < max_iter) and (tolerance < error):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L7;
        }
        __pyx_L6_error: {
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L1_error;
        }
        __pyx_L7:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1228
 *         stats[0] = substats[1]
 *         stats[1] = iter_count
 *     return p             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_p, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1228; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1127
 * 
 * 
 * def invert_vector_field_fixed_point_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                                        double[:, :] d_world2grid,
 *                                        double[:] spacing,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __PYX_XDEC_MEMVIEW(&__pyx_t_8, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_9, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_10, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.invert_vector_field_fixed_point_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_stats, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_substats, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_norms, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_p, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_q, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d_world2grid, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_spacing, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_start, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_368__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults3, __pyx_self)->__pyx_arg_start, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_153invert_vector_field_fixed_point_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_153invert_vector_field_fixed_point_2d = {"__pyx_fuse_1invert_vector_field_fixed_point_2d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_153invert_vector_field_fixed_point_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_18invert_vector_field_fixed_point_2d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_153invert_vector_field_fixed_point_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_d = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d_world2grid = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_spacing = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_v_max_iter;
  double __pyx_v_tolerance;
  __Pyx_memviewslice __pyx_v_start = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("invert_vector_field_fixed_point_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_d,&__pyx_n_s_d_world2grid,&__pyx_n_s_spacing,&__pyx_n_s_max_iter,&__pyx_n_s_tolerance,&__pyx_n_s_start,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults3 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults3, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d_world2grid)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_2d", 0, 5, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_spacing)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_2d", 0, 5, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_max_iter)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_2d", 0, 5, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tolerance)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_2d", 0, 5, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_start);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "invert_vector_field_fixed_point_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_d = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[0]); if (unlikely(!__pyx_v_d.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d_world2grid = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_d_world2grid.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1128; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_spacing = __Pyx_PyObject_to_MemoryviewSlice_ds_double(values[2]); if (unlikely(!__pyx_v_spacing.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1129; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_max_iter = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_max_iter == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1130; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_tolerance = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_tolerance == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1130; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[5]) {
      __pyx_v_start = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[5]); if (unlikely(!__pyx_v_start.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1131; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_start = __pyx_dynamic_args->__pyx_arg_start;
      __PYX_INC_MEMVIEW(&__pyx_v_start, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_2d", 0, 5, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.invert_vector_field_fixed_point_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_152invert_vector_field_fixed_point_2d(__pyx_self, __pyx_v_d, __pyx_v_d_world2grid, __pyx_v_spacing, __pyx_v_max_iter, __pyx_v_tolerance, __pyx_v_start);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_152invert_vector_field_fixed_point_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_d_world2grid, __Pyx_memviewslice __pyx_v_spacing, int __pyx_v_max_iter, double __pyx_v_tolerance, __Pyx_memviewslice __pyx_v_start) {
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  int __pyx_v_iter_count;
  double __pyx_v_difmag;
  double __pyx_v_mag;
  double __pyx_v_maxlen;
  double __pyx_v_step_factor;
  double __pyx_v_epsilon;
  double __pyx_v_error;
  double __pyx_v_sr;
  double __pyx_v_sc;
  PyObject *__pyx_v_ftype = NULL;
  __Pyx_memviewslice __pyx_v_stats = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_substats = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_norms = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_p = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_q = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_memviewslice __pyx_t_8 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_9 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_10 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  npy_intp __pyx_t_14;
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1invert_vector_field_fixed_point_2d", 0);

  /* "dipy/align/vector_fields.pyx":1174
 *     """
 *     cdef:
 *         cnp.npy_intp nr = d.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = d.shape[1]
 *         int iter_count, current, flag
 */
  __pyx_v_nr = (__pyx_v_d.shape[0]);

  /* "dipy/align/vector_fields.pyx":1175
 *     cdef:
 *         cnp.npy_intp nr = d.shape[0]
 *         cnp.npy_intp nc = d.shape[1]             # <<<<<<<<<<<<<<
 *         int iter_count, current, flag
 *         double difmag, mag, maxlen, step_factor
 */
  __pyx_v_nc = (__pyx_v_d.shape[1]);

  /* "dipy/align/vector_fields.pyx":1179
 *         double difmag, mag, maxlen, step_factor
 *         double epsilon
 *         double error = 1 + tolerance             # <<<<<<<<<<<<<<
 *         double di, dj, dii, djj
 *         double sr = spacing[0], sc = spacing[1]
 */
  __pyx_v_error = (1.0 + __pyx_v_tolerance);

  /* "dipy/align/vector_fields.pyx":1181
 *         double error = 1 + tolerance
 *         double di, dj, dii, djj
 *         double sr = spacing[0], sc = spacing[1]             # <<<<<<<<<<<<<<
 * 
 *     ftype = np.asarray(d).dtype
 */
  __pyx_t_1 = 0;
  __pyx_v_sr = (*((double *) ( /* dim=0 */ (__pyx_v_spacing.data + __pyx_t_1 * __pyx_v_spacing.strides[0]) )));
  __pyx_t_2 = 1;
  __pyx_v_sc = (*((double *) ( /* dim=0 */ (__pyx_v_spacing.data + __pyx_t_2 * __pyx_v_spacing.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":1183
 *         double sr = spacing[0], sc = spacing[1]
 * 
 *     ftype = np.asarray(d).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_d, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_dtype); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_ftype = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "dipy/align/vector_fields.pyx":1185
 *     ftype = np.asarray(d).dtype
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] substats = np.empty(shape=(3,), dtype=np.float64)
 *         double[:, :] norms = np.zeros(shape=(nr, nc), dtype=np.float64)
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_zeros); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_shape, __pyx_tuple__54) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_float64); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_8.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_stats = __pyx_t_8;
  __pyx_t_8.memview = NULL;
  __pyx_t_8.data = NULL;

  /* "dipy/align/vector_fields.pyx":1186
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)
 *         double[:] substats = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:, :] norms = np.zeros(shape=(nr, nc), dtype=np.float64)
 *         floating[:, :, :] p = np.zeros(shape=(nr, nc, 2), dtype=ftype)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_empty); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__55) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_float64); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_7);
  if (unlikely(!__pyx_t_8.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_substats = __pyx_t_8;
  __pyx_t_8.memview = NULL;
  __pyx_t_8.data = NULL;

  /* "dipy/align/vector_fields.pyx":1187
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)
 *         double[:] substats = np.empty(shape=(3,), dtype=np.float64)
 *         double[:, :] norms = np.zeros(shape=(nr, nc), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         floating[:, :, :] p = np.zeros(shape=(nr, nc, 2), dtype=ftype)
 *         floating[:, :, :] q = np.zeros(shape=(nr, nc, 2), dtype=ftype)
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyDict_New(); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nr); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nc); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_3);
  __pyx_t_5 = 0;
  __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_float64); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_7); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_9 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_t_3);
  if (unlikely(!__pyx_t_9.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1187; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_norms = __pyx_t_9;
  __pyx_t_9.memview = NULL;
  __pyx_t_9.data = NULL;

  /* "dipy/align/vector_fields.pyx":1188
 *         double[:] substats = np.empty(shape=(3,), dtype=np.float64)
 *         double[:, :] norms = np.zeros(shape=(nr, nc), dtype=np.float64)
 *         floating[:, :, :] p = np.zeros(shape=(nr, nc, 2), dtype=ftype)             # <<<<<<<<<<<<<<
 *         floating[:, :, :] q = np.zeros(shape=(nr, nc, 2), dtype=ftype)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nr); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nc); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_10 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_5);
  if (unlikely(!__pyx_t_10.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_p = __pyx_t_10;
  __pyx_t_10.memview = NULL;
  __pyx_t_10.data = NULL;

  /* "dipy/align/vector_fields.pyx":1189
 *         double[:, :] norms = np.zeros(shape=(nr, nc), dtype=np.float64)
 *         floating[:, :, :] p = np.zeros(shape=(nr, nc, 2), dtype=ftype)
 *         floating[:, :, :] q = np.zeros(shape=(nr, nc, 2), dtype=ftype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(d_world2grid, 2):
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_zeros); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_7 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nr); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nc); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_int_2);
  __pyx_t_7 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_shape, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_10 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_10.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_q = __pyx_t_10;
  __pyx_t_10.memview = NULL;
  __pyx_t_10.data = NULL;

  /* "dipy/align/vector_fields.pyx":1191
 *         floating[:, :, :] q = np.zeros(shape=(nr, nc, 2), dtype=ftype)
 * 
 *     if not is_valid_affine(d_world2grid, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid world-to-image transform")
 * 
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_d_world2grid, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = NULL;
  __pyx_t_11 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_11 = 1;
    }
  }
  __pyx_t_7 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_11, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_11, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_12 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_13 = ((!__pyx_t_12) != 0);
  if (__pyx_t_13) {

    /* "dipy/align/vector_fields.pyx":1192
 * 
 *     if not is_valid_affine(d_world2grid, 2):
 *         raise ValueError("Invalid world-to-image transform")             # <<<<<<<<<<<<<<
 * 
 *     if start is not None:
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__56, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1192; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1192; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1191
 *         floating[:, :, :] q = np.zeros(shape=(nr, nc, 2), dtype=ftype)
 * 
 *     if not is_valid_affine(d_world2grid, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid world-to-image transform")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1194
 *         raise ValueError("Invalid world-to-image transform")
 * 
 *     if start is not None:             # <<<<<<<<<<<<<<
 *         p[...] = start
 * 
 */
  __pyx_t_13 = ((((PyObject *) __pyx_v_start.memview) != Py_None) != 0);
  if (__pyx_t_13) {

    /* "dipy/align/vector_fields.pyx":1195
 * 
 *     if start is not None:
 *         p[...] = start             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    if (unlikely(__pyx_memoryview_copy_contents(__pyx_v_start, __pyx_v_p, 3, 3, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1195; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1194
 *         raise ValueError("Invalid world-to-image transform")
 * 
 *     if start is not None:             # <<<<<<<<<<<<<<
 *         p[...] = start
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1197
 *         p[...] = start
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         iter_count = 0
 *         while (iter_count < max_iter) and (tolerance < error):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1198
 * 
 *     with nogil:
 *         iter_count = 0             # <<<<<<<<<<<<<<
 *         while (iter_count < max_iter) and (tolerance < error):
 *             if iter_count == 0:
 */
        __pyx_v_iter_count = 0;

        /* "dipy/align/vector_fields.pyx":1199
 *     with nogil:
 *         iter_count = 0
 *         while (iter_count < max_iter) and (tolerance < error):             # <<<<<<<<<<<<<<
 *             if iter_count == 0:
 *                 epsilon = 0.75
 */
        while (1) {
          __pyx_t_12 = ((__pyx_v_iter_count < __pyx_v_max_iter) != 0);
          if (__pyx_t_12) {
          } else {
            __pyx_t_13 = __pyx_t_12;
            goto __pyx_L10_bool_binop_done;
          }
          __pyx_t_12 = ((__pyx_v_tolerance < __pyx_v_error) != 0);
          __pyx_t_13 = __pyx_t_12;
          __pyx_L10_bool_binop_done:;
          if (!__pyx_t_13) break;

          /* "dipy/align/vector_fields.pyx":1200
 *         iter_count = 0
 *         while (iter_count < max_iter) and (tolerance < error):
 *             if iter_count == 0:             # <<<<<<<<<<<<<<
 *                 epsilon = 0.75
 *             else:
 */
          __pyx_t_13 = ((__pyx_v_iter_count == 0) != 0);
          if (__pyx_t_13) {

            /* "dipy/align/vector_fields.pyx":1201
 *         while (iter_count < max_iter) and (tolerance < error):
 *             if iter_count == 0:
 *                 epsilon = 0.75             # <<<<<<<<<<<<<<
 *             else:
 *                 epsilon = 0.5
 */
            __pyx_v_epsilon = 0.75;

            /* "dipy/align/vector_fields.pyx":1200
 *         iter_count = 0
 *         while (iter_count < max_iter) and (tolerance < error):
 *             if iter_count == 0:             # <<<<<<<<<<<<<<
 *                 epsilon = 0.75
 *             else:
 */
            goto __pyx_L12;
          }

          /* "dipy/align/vector_fields.pyx":1203
 *                 epsilon = 0.75
 *             else:
 *                 epsilon = 0.5             # <<<<<<<<<<<<<<
 *             _compose_vector_fields_2d[floating](p, d, None, d_world2grid,
 *                                                 1.0, q, substats)
 */
          /*else*/ {
            __pyx_v_epsilon = 0.5;
          }
          __pyx_L12:;

          /* "dipy/align/vector_fields.pyx":1204
 *             else:
 *                 epsilon = 0.5
 *             _compose_vector_fields_2d[floating](p, d, None, d_world2grid,             # <<<<<<<<<<<<<<
 *                                                 1.0, q, substats)
 *             difmag = 0
 */
          __pyx_t_9 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
          if (unlikely(!__pyx_t_9.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1204; __pyx_clineno = __LINE__; goto __pyx_L6_error;}

          /* "dipy/align/vector_fields.pyx":1205
 *                 epsilon = 0.5
 *             _compose_vector_fields_2d[floating](p, d, None, d_world2grid,
 *                                                 1.0, q, substats)             # <<<<<<<<<<<<<<
 *             difmag = 0
 *             error = 0
 */
          __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__compose_vector_fields_2d(__pyx_v_p, __pyx_v_d, __pyx_t_9, __pyx_v_d_world2grid, 1.0, __pyx_v_q, __pyx_v_substats);
          __PYX_XDEC_MEMVIEW(&__pyx_t_9, 0);

          /* "dipy/align/vector_fields.pyx":1206
 *             _compose_vector_fields_2d[floating](p, d, None, d_world2grid,
 *                                                 1.0, q, substats)
 *             difmag = 0             # <<<<<<<<<<<<<<
 *             error = 0
 *             for i in range(nr):
 */
          __pyx_v_difmag = 0.0;

          /* "dipy/align/vector_fields.pyx":1207
 *                                                 1.0, q, substats)
 *             difmag = 0
 *             error = 0             # <<<<<<<<<<<<<<
 *             for i in range(nr):
 *                 for j in range(nc):
 */
          __pyx_v_error = 0.0;

          /* "dipy/align/vector_fields.pyx":1208
 *             difmag = 0
 *             error = 0
 *             for i in range(nr):             # <<<<<<<<<<<<<<
 *                 for j in range(nc):
 *                     mag = sqrt((q[i, j, 0]/sr) ** 2 + (q[i, j, 1]/sc) ** 2)
 */
          __pyx_t_14 = __pyx_v_nr;
          for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
            __pyx_v_i = __pyx_t_15;

            /* "dipy/align/vector_fields.pyx":1209
 *             error = 0
 *             for i in range(nr):
 *                 for j in range(nc):             # <<<<<<<<<<<<<<
 *                     mag = sqrt((q[i, j, 0]/sr) ** 2 + (q[i, j, 1]/sc) ** 2)
 *                     norms[i, j] = mag
 */
            __pyx_t_16 = __pyx_v_nc;
            for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
              __pyx_v_j = __pyx_t_17;

              /* "dipy/align/vector_fields.pyx":1210
 *             for i in range(nr):
 *                 for j in range(nc):
 *                     mag = sqrt((q[i, j, 0]/sr) ** 2 + (q[i, j, 1]/sc) ** 2)             # <<<<<<<<<<<<<<
 *                     norms[i, j] = mag
 *                     error += mag
 */
              __pyx_t_18 = __pyx_v_i;
              __pyx_t_19 = __pyx_v_j;
              __pyx_t_20 = 0;
              __pyx_t_21 = __pyx_v_i;
              __pyx_t_22 = __pyx_v_j;
              __pyx_t_23 = 1;
              __pyx_v_mag = sqrt((pow(((*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_18 * __pyx_v_q.strides[0]) ) + __pyx_t_19 * __pyx_v_q.strides[1]) ) + __pyx_t_20 * __pyx_v_q.strides[2]) ))) / __pyx_v_sr), 2.0) + pow(((*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_21 * __pyx_v_q.strides[0]) ) + __pyx_t_22 * __pyx_v_q.strides[1]) ) + __pyx_t_23 * __pyx_v_q.strides[2]) ))) / __pyx_v_sc), 2.0)));

              /* "dipy/align/vector_fields.pyx":1211
 *                 for j in range(nc):
 *                     mag = sqrt((q[i, j, 0]/sr) ** 2 + (q[i, j, 1]/sc) ** 2)
 *                     norms[i, j] = mag             # <<<<<<<<<<<<<<
 *                     error += mag
 *                     if(difmag < mag):
 */
              __pyx_t_24 = __pyx_v_i;
              __pyx_t_25 = __pyx_v_j;
              *((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_norms.data + __pyx_t_24 * __pyx_v_norms.strides[0]) ) + __pyx_t_25 * __pyx_v_norms.strides[1]) )) = __pyx_v_mag;

              /* "dipy/align/vector_fields.pyx":1212
 *                     mag = sqrt((q[i, j, 0]/sr) ** 2 + (q[i, j, 1]/sc) ** 2)
 *                     norms[i, j] = mag
 *                     error += mag             # <<<<<<<<<<<<<<
 *                     if(difmag < mag):
 *                         difmag = mag
 */
              __pyx_v_error = (__pyx_v_error + __pyx_v_mag);

              /* "dipy/align/vector_fields.pyx":1213
 *                     norms[i, j] = mag
 *                     error += mag
 *                     if(difmag < mag):             # <<<<<<<<<<<<<<
 *                         difmag = mag
 *             maxlen = difmag * epsilon
 */
              __pyx_t_13 = ((__pyx_v_difmag < __pyx_v_mag) != 0);
              if (__pyx_t_13) {

                /* "dipy/align/vector_fields.pyx":1214
 *                     error += mag
 *                     if(difmag < mag):
 *                         difmag = mag             # <<<<<<<<<<<<<<
 *             maxlen = difmag * epsilon
 *             for i in range(nr):
 */
                __pyx_v_difmag = __pyx_v_mag;

                /* "dipy/align/vector_fields.pyx":1213
 *                     norms[i, j] = mag
 *                     error += mag
 *                     if(difmag < mag):             # <<<<<<<<<<<<<<
 *                         difmag = mag
 *             maxlen = difmag * epsilon
 */
              }
            }
          }

          /* "dipy/align/vector_fields.pyx":1215
 *                     if(difmag < mag):
 *                         difmag = mag
 *             maxlen = difmag * epsilon             # <<<<<<<<<<<<<<
 *             for i in range(nr):
 *                 for j in range(nc):
 */
          __pyx_v_maxlen = (__pyx_v_difmag * __pyx_v_epsilon);

          /* "dipy/align/vector_fields.pyx":1216
 *                         difmag = mag
 *             maxlen = difmag * epsilon
 *             for i in range(nr):             # <<<<<<<<<<<<<<
 *                 for j in range(nc):
 *                     if norms[i, j] > maxlen:
 */
          __pyx_t_14 = __pyx_v_nr;
          for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
            __pyx_v_i = __pyx_t_15;

            /* "dipy/align/vector_fields.pyx":1217
 *             maxlen = difmag * epsilon
 *             for i in range(nr):
 *                 for j in range(nc):             # <<<<<<<<<<<<<<
 *                     if norms[i, j] > maxlen:
 *                         step_factor = epsilon * maxlen / norms[i, j]
 */
            __pyx_t_16 = __pyx_v_nc;
            for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
              __pyx_v_j = __pyx_t_17;

              /* "dipy/align/vector_fields.pyx":1218
 *             for i in range(nr):
 *                 for j in range(nc):
 *                     if norms[i, j] > maxlen:             # <<<<<<<<<<<<<<
 *                         step_factor = epsilon * maxlen / norms[i, j]
 *                     else:
 */
              __pyx_t_26 = __pyx_v_i;
              __pyx_t_27 = __pyx_v_j;
              __pyx_t_13 = (((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_norms.data + __pyx_t_26 * __pyx_v_norms.strides[0]) ) + __pyx_t_27 * __pyx_v_norms.strides[1]) ))) > __pyx_v_maxlen) != 0);
              if (__pyx_t_13) {

                /* "dipy/align/vector_fields.pyx":1219
 *                 for j in range(nc):
 *                     if norms[i, j] > maxlen:
 *                         step_factor = epsilon * maxlen / norms[i, j]             # <<<<<<<<<<<<<<
 *                     else:
 *                         step_factor = epsilon
 */
                __pyx_t_28 = __pyx_v_i;
                __pyx_t_29 = __pyx_v_j;
                __pyx_v_step_factor = ((__pyx_v_epsilon * __pyx_v_maxlen) / (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_norms.data + __pyx_t_28 * __pyx_v_norms.strides[0]) ) + __pyx_t_29 * __pyx_v_norms.strides[1]) ))));

                /* "dipy/align/vector_fields.pyx":1218
 *             for i in range(nr):
 *                 for j in range(nc):
 *                     if norms[i, j] > maxlen:             # <<<<<<<<<<<<<<
 *                         step_factor = epsilon * maxlen / norms[i, j]
 *                     else:
 */
                goto __pyx_L22;
              }

              /* "dipy/align/vector_fields.pyx":1221
 *                         step_factor = epsilon * maxlen / norms[i, j]
 *                     else:
 *                         step_factor = epsilon             # <<<<<<<<<<<<<<
 *                     p[i, j, 0] = p[i, j, 0] - step_factor * q[i, j, 0]
 *                     p[i, j, 1] = p[i, j, 1] - step_factor * q[i, j, 1]
 */
              /*else*/ {
                __pyx_v_step_factor = __pyx_v_epsilon;
              }
              __pyx_L22:;

              /* "dipy/align/vector_fields.pyx":1222
 *                     else:
 *                         step_factor = epsilon
 *                     p[i, j, 0] = p[i, j, 0] - step_factor * q[i, j, 0]             # <<<<<<<<<<<<<<
 *                     p[i, j, 1] = p[i, j, 1] - step_factor * q[i, j, 1]
 *             error /= (nr * nc)
 */
              __pyx_t_30 = __pyx_v_i;
              __pyx_t_31 = __pyx_v_j;
              __pyx_t_32 = 0;
              __pyx_t_33 = __pyx_v_i;
              __pyx_t_34 = __pyx_v_j;
              __pyx_t_35 = 0;
              __pyx_t_36 = __pyx_v_i;
              __pyx_t_37 = __pyx_v_j;
              __pyx_t_38 = 0;
              *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_36 * __pyx_v_p.strides[0]) ) + __pyx_t_37 * __pyx_v_p.strides[1]) ) + __pyx_t_38 * __pyx_v_p.strides[2]) )) = ((*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_30 * __pyx_v_p.strides[0]) ) + __pyx_t_31 * __pyx_v_p.strides[1]) ) + __pyx_t_32 * __pyx_v_p.strides[2]) ))) - (__pyx_v_step_factor * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_33 * __pyx_v_q.strides[0]) ) + __pyx_t_34 * __pyx_v_q.strides[1]) ) + __pyx_t_35 * __pyx_v_q.strides[2]) )))));

              /* "dipy/align/vector_fields.pyx":1223
 *                         step_factor = epsilon
 *                     p[i, j, 0] = p[i, j, 0] - step_factor * q[i, j, 0]
 *                     p[i, j, 1] = p[i, j, 1] - step_factor * q[i, j, 1]             # <<<<<<<<<<<<<<
 *             error /= (nr * nc)
 *             iter_count += 1
 */
              __pyx_t_39 = __pyx_v_i;
              __pyx_t_40 = __pyx_v_j;
              __pyx_t_41 = 1;
              __pyx_t_42 = __pyx_v_i;
              __pyx_t_43 = __pyx_v_j;
              __pyx_t_44 = 1;
              __pyx_t_45 = __pyx_v_i;
              __pyx_t_46 = __pyx_v_j;
              __pyx_t_47 = 1;
              *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_45 * __pyx_v_p.strides[0]) ) + __pyx_t_46 * __pyx_v_p.strides[1]) ) + __pyx_t_47 * __pyx_v_p.strides[2]) )) = ((*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_39 * __pyx_v_p.strides[0]) ) + __pyx_t_40 * __pyx_v_p.strides[1]) ) + __pyx_t_41 * __pyx_v_p.strides[2]) ))) - (__pyx_v_step_factor * (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_42 * __pyx_v_q.strides[0]) ) + __pyx_t_43 * __pyx_v_q.strides[1]) ) + __pyx_t_44 * __pyx_v_q.strides[2]) )))));
            }
          }

          /* "dipy/align/vector_fields.pyx":1224
 *                     p[i, j, 0] = p[i, j, 0] - step_factor * q[i, j, 0]
 *                     p[i, j, 1] = p[i, j, 1] - step_factor * q[i, j, 1]
 *             error /= (nr * nc)             # <<<<<<<<<<<<<<
 *             iter_count += 1
 *         stats[0] = substats[1]
 */
          __pyx_v_error = (__pyx_v_error / (__pyx_v_nr * __pyx_v_nc));

          /* "dipy/align/vector_fields.pyx":1225
 *                     p[i, j, 1] = p[i, j, 1] - step_factor * q[i, j, 1]
 *             error /= (nr * nc)
 *             iter_count += 1             # <<<<<<<<<<<<<<
 *         stats[0] = substats[1]
 *         stats[1] = iter_count
 */
          __pyx_v_iter_count = (__pyx_v_iter_count + 1);
        }

        /* "dipy/align/vector_fields.pyx":1226
 *             error /= (nr * nc)
 *             iter_count += 1
 *         stats[0] = substats[1]             # <<<<<<<<<<<<<<
 *         stats[1] = iter_count
 *     return p
 */
        __pyx_t_48 = 1;
        __pyx_t_49 = 0;
        *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_49 * __pyx_v_stats.strides[0]) )) = (*((double *) ( /* dim=0 */ (__pyx_v_substats.data + __pyx_t_48 * __pyx_v_substats.strides[0]) )));

        /* "dipy/align/vector_fields.pyx":1227
 *             iter_count += 1
 *         stats[0] = substats[1]
 *         stats[1] = iter_count             # <<<<<<<<<<<<<<
 *     return p
 * 
 */
        __pyx_t_50 = 1;
        *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_50 * __pyx_v_stats.strides[0]) )) = __pyx_v_iter_count;
      }

      /* "dipy/align/vector_fields.pyx":1197
 *         p[...] = start
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         iter_count = 0
 *         while (iter_count < max_iter) and (tolerance < error):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L7;
        }
        __pyx_L6_error: {
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L1_error;
        }
        __pyx_L7:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1228
 *         stats[0] = substats[1]
 *         stats[1] = iter_count
 *     return p             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_p, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1228; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1127
 * 
 * 
 * def invert_vector_field_fixed_point_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                                        double[:, :] d_world2grid,
 *                                        double[:] spacing,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __PYX_XDEC_MEMVIEW(&__pyx_t_8, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_9, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_10, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.invert_vector_field_fixed_point_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_stats, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_substats, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_norms, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_p, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_q, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d_world2grid, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_spacing, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_start, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":1231
 * 
 * 
 * def invert_vector_field_fixed_point_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                                        double[:, :] d_world2grid,
 *                                        double[:] spacing,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_21invert_vector_field_fixed_point_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_20invert_vector_field_fixed_point_3d[] = "Computes the inverse of a 3D displacement fields\n\n    Computes the inverse of the given 3-D displacement field d using the\n    fixed-point algorithm [1].\n\n    [1] Chen, M., Lu, W., Chen, Q., Ruchala, K. J., & Olivera, G. H. (2008).\n        A simple fixed-point approach to invert a deformation field.\n        Medical Physics, 35(1), 81. doi:10.1118/1.2816107\n\n    Parameters\n    ----------\n    d : array, shape (S, R, C, 3)\n        the 3-D displacement field to be inverted\n    d_world2grid : array, shape (4, 4)\n        the space-to-grid transformation associated to the displacement field\n        d (transforming physical space coordinates to voxel coordinates of the\n        displacement field grid)\n    spacing :array, shape (3,)\n        the spacing between voxels (voxel size along each axis)\n    max_iter : int\n        maximum number of iterations to be performed\n    tol : float\n        maximum tolerated inversion error\n    start : array, shape (S, R, C)\n        an approximation to the inverse displacement field (if no approximation\n        is available, None can be provided and the start displacement field\n        will be zero)\n\n    Returns\n    -------\n    p : array, shape (S, R, C, 3)\n        the inverse displacement field\n\n    Notes\n    -----\n    We assume that the displacement field is an endomorphism so that the shape\n    and voxel-to-space transformation of the inverse field's discretization is\n    the same as those of the input displacement field. The 'inversion error' at\n    iteration t is defined as the mean norm of the displacement vectors of the\n    input displacement field composed with the inverse at iteration t.\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_21invert_vector_field_fixed_point_3d = {"invert_vector_field_fixed_point_3d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_21invert_vector_field_fixed_point_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_20invert_vector_field_fixed_point_3d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_21invert_vector_field_fixed_point_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_20invert_vector_field_fixed_point_3d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_20invert_vector_field_fixed_point_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("invert_vector_field_fixed_point_3d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_d, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_d); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 4) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 4) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__57, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__58, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__59, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__60, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_374__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults6, __pyx_self)->__pyx_arg_start, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_157invert_vector_field_fixed_point_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_157invert_vector_field_fixed_point_3d = {"__pyx_fuse_0invert_vector_field_fixed_point_3d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_157invert_vector_field_fixed_point_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_20invert_vector_field_fixed_point_3d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_157invert_vector_field_fixed_point_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_d = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d_world2grid = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_spacing = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_v_max_iter;
  double __pyx_v_tol;
  __Pyx_memviewslice __pyx_v_start = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("invert_vector_field_fixed_point_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_d,&__pyx_n_s_d_world2grid,&__pyx_n_s_spacing,&__pyx_n_s_max_iter,&__pyx_n_s_tol,&__pyx_n_s_start,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults6 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults6, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d_world2grid)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_3d", 0, 5, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_spacing)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_3d", 0, 5, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_max_iter)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_3d", 0, 5, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tol)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_3d", 0, 5, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_start);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "invert_vector_field_fixed_point_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_d = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[0]); if (unlikely(!__pyx_v_d.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d_world2grid = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_d_world2grid.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1232; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_spacing = __Pyx_PyObject_to_MemoryviewSlice_ds_double(values[2]); if (unlikely(!__pyx_v_spacing.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1233; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_max_iter = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_max_iter == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1234; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_tol = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_tol == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1234; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[5]) {
      __pyx_v_start = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[5]); if (unlikely(!__pyx_v_start.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1235; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_start = __pyx_dynamic_args->__pyx_arg_start;
      __PYX_INC_MEMVIEW(&__pyx_v_start, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_3d", 0, 5, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.invert_vector_field_fixed_point_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_156invert_vector_field_fixed_point_3d(__pyx_self, __pyx_v_d, __pyx_v_d_world2grid, __pyx_v_spacing, __pyx_v_max_iter, __pyx_v_tol, __pyx_v_start);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_156invert_vector_field_fixed_point_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_d_world2grid, __Pyx_memviewslice __pyx_v_spacing, int __pyx_v_max_iter, double __pyx_v_tol, __Pyx_memviewslice __pyx_v_start) {
  npy_intp __pyx_v_ns;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  int __pyx_v_iter_count;
  double __pyx_v_difmag;
  double __pyx_v_mag;
  double __pyx_v_maxlen;
  double __pyx_v_step_factor;
  double __pyx_v_epsilon;
  double __pyx_v_error;
  double __pyx_v_ss;
  double __pyx_v_sr;
  double __pyx_v_sc;
  PyObject *__pyx_v_ftype = NULL;
  __Pyx_memviewslice __pyx_v_stats = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_substats = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_norms = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_p = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_q = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_memviewslice __pyx_t_9 = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_t_10 = NULL;
  __Pyx_memviewslice __pyx_t_11 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_12 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  __Pyx_memviewslice __pyx_t_16 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  npy_intp __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  Py_ssize_t __pyx_t_60;
  Py_ssize_t __pyx_t_61;
  Py_ssize_t __pyx_t_62;
  Py_ssize_t __pyx_t_63;
  Py_ssize_t __pyx_t_64;
  Py_ssize_t __pyx_t_65;
  Py_ssize_t __pyx_t_66;
  Py_ssize_t __pyx_t_67;
  Py_ssize_t __pyx_t_68;
  Py_ssize_t __pyx_t_69;
  Py_ssize_t __pyx_t_70;
  Py_ssize_t __pyx_t_71;
  Py_ssize_t __pyx_t_72;
  Py_ssize_t __pyx_t_73;
  Py_ssize_t __pyx_t_74;
  Py_ssize_t __pyx_t_75;
  Py_ssize_t __pyx_t_76;
  Py_ssize_t __pyx_t_77;
  Py_ssize_t __pyx_t_78;
  Py_ssize_t __pyx_t_79;
  Py_ssize_t __pyx_t_80;
  Py_ssize_t __pyx_t_81;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0invert_vector_field_fixed_point_3d", 0);

  /* "dipy/align/vector_fields.pyx":1278
 *     """
 *     cdef:
 *         cnp.npy_intp ns = d.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr = d.shape[1]
 *         cnp.npy_intp nc = d.shape[2]
 */
  __pyx_v_ns = (__pyx_v_d.shape[0]);

  /* "dipy/align/vector_fields.pyx":1279
 *     cdef:
 *         cnp.npy_intp ns = d.shape[0]
 *         cnp.npy_intp nr = d.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = d.shape[2]
 *         int iter_count, current
 */
  __pyx_v_nr = (__pyx_v_d.shape[1]);

  /* "dipy/align/vector_fields.pyx":1280
 *         cnp.npy_intp ns = d.shape[0]
 *         cnp.npy_intp nr = d.shape[1]
 *         cnp.npy_intp nc = d.shape[2]             # <<<<<<<<<<<<<<
 *         int iter_count, current
 *         double dkk, dii, djj, dk, di, dj
 */
  __pyx_v_nc = (__pyx_v_d.shape[2]);

  /* "dipy/align/vector_fields.pyx":1284
 *         double dkk, dii, djj, dk, di, dj
 *         double difmag, mag, maxlen, step_factor
 *         double epsilon = 0.5             # <<<<<<<<<<<<<<
 *         double error = 1 + tol
 *         double ss = spacing[0], sr = spacing[1], sc = spacing[2]
 */
  __pyx_v_epsilon = 0.5;

  /* "dipy/align/vector_fields.pyx":1285
 *         double difmag, mag, maxlen, step_factor
 *         double epsilon = 0.5
 *         double error = 1 + tol             # <<<<<<<<<<<<<<
 *         double ss = spacing[0], sr = spacing[1], sc = spacing[2]
 * 
 */
  __pyx_v_error = (1.0 + __pyx_v_tol);

  /* "dipy/align/vector_fields.pyx":1286
 *         double epsilon = 0.5
 *         double error = 1 + tol
 *         double ss = spacing[0], sr = spacing[1], sc = spacing[2]             # <<<<<<<<<<<<<<
 * 
 *     ftype = np.asarray(d).dtype
 */
  __pyx_t_1 = 0;
  __pyx_v_ss = (*((double *) ( /* dim=0 */ (__pyx_v_spacing.data + __pyx_t_1 * __pyx_v_spacing.strides[0]) )));
  __pyx_t_2 = 1;
  __pyx_v_sr = (*((double *) ( /* dim=0 */ (__pyx_v_spacing.data + __pyx_t_2 * __pyx_v_spacing.strides[0]) )));
  __pyx_t_3 = 2;
  __pyx_v_sc = (*((double *) ( /* dim=0 */ (__pyx_v_spacing.data + __pyx_t_3 * __pyx_v_spacing.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":1288
 *         double ss = spacing[0], sr = spacing[1], sc = spacing[2]
 * 
 *     ftype = np.asarray(d).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_asarray); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_d, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_7 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  if (!__pyx_t_7) {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_4);
  } else {
    __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7); __pyx_t_7 = NULL;
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_dtype); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_ftype = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":1290
 *     ftype = np.asarray(d).dtype
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] substats = np.zeros(shape=(3,), dtype=np.float64)
 *         double[:, :, :] norms = np.zeros(shape=(ns, nr, nc), dtype=np.float64)
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_shape, __pyx_tuple__61) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_float64); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_6); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_9 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_5);
  if (unlikely(!__pyx_t_9.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_stats = __pyx_t_9;
  __pyx_t_9.memview = NULL;
  __pyx_t_9.data = NULL;

  /* "dipy/align/vector_fields.pyx":1291
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)
 *         double[:] substats = np.zeros(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:, :, :] norms = np.zeros(shape=(ns, nr, nc), dtype=np.float64)
 *         floating[:, :, :, :] p = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_zeros); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_shape, __pyx_tuple__62) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_float64); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_empty_tuple, __pyx_t_5); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_9 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_8);
  if (unlikely(!__pyx_t_9.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_substats = __pyx_t_9;
  __pyx_t_9.memview = NULL;
  __pyx_t_9.data = NULL;

  /* "dipy/align/vector_fields.pyx":1292
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)
 *         double[:] substats = np.zeros(shape=(3,), dtype=np.float64)
 *         double[:, :, :] norms = np.zeros(shape=(ns, nr, nc), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         floating[:, :, :, :] p = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)
 *         floating[:, :, :, :] q = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = PyDict_New(); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ns); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nr); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nc); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_10 = PyTuple_New(3); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_10, 2, __pyx_t_7);
  __pyx_t_6 = 0;
  __pyx_t_4 = 0;
  __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_shape, __pyx_t_10) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_float64); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_dtype, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_empty_tuple, __pyx_t_8); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_11 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_7);
  if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_norms = __pyx_t_11;
  __pyx_t_11.memview = NULL;
  __pyx_t_11.data = NULL;

  /* "dipy/align/vector_fields.pyx":1293
 *         double[:] substats = np.zeros(shape=(3,), dtype=np.float64)
 *         double[:, :, :] norms = np.zeros(shape=(ns, nr, nc), dtype=np.float64)
 *         floating[:, :, :, :] p = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)             # <<<<<<<<<<<<<<
 *         floating[:, :, :, :] q = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)
 * 
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_zeros); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyDict_New(); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ns); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_10 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nr); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nc); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_int_3);
  __pyx_t_5 = 0;
  __pyx_t_10 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_empty_tuple, __pyx_t_7); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_12 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(__pyx_t_6);
  if (unlikely(!__pyx_t_12.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_p = __pyx_t_12;
  __pyx_t_12.memview = NULL;
  __pyx_t_12.data = NULL;

  /* "dipy/align/vector_fields.pyx":1294
 *         double[:, :, :] norms = np.zeros(shape=(ns, nr, nc), dtype=np.float64)
 *         floating[:, :, :, :] p = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)
 *         floating[:, :, :, :] q = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(d_world2grid, 3):
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_zeros); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_8 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ns); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nr); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_10 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nc); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_10);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_int_3);
  __pyx_t_8 = 0;
  __pyx_t_4 = 0;
  __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_empty_tuple, __pyx_t_6); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_12 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(__pyx_t_5);
  if (unlikely(!__pyx_t_12.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_q = __pyx_t_12;
  __pyx_t_12.memview = NULL;
  __pyx_t_12.data = NULL;

  /* "dipy/align/vector_fields.pyx":1296
 *         floating[:, :, :, :] q = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)
 * 
 *     if not is_valid_affine(d_world2grid, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid world-to-image transform")
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_d_world2grid, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_10 = NULL;
  __pyx_t_13 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
      __pyx_t_13 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_10) {
    __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_10); __pyx_t_10 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_13, __pyx_t_7);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_13, __pyx_int_3);
  __pyx_t_7 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_4, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_14 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_15 = ((!__pyx_t_14) != 0);
  if (__pyx_t_15) {

    /* "dipy/align/vector_fields.pyx":1297
 * 
 *     if not is_valid_affine(d_world2grid, 3):
 *         raise ValueError("Invalid world-to-image transform")             # <<<<<<<<<<<<<<
 * 
 *     if start is not None:
 */
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__63, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1297; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1297; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1296
 *         floating[:, :, :, :] q = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)
 * 
 *     if not is_valid_affine(d_world2grid, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid world-to-image transform")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1299
 *         raise ValueError("Invalid world-to-image transform")
 * 
 *     if start is not None:             # <<<<<<<<<<<<<<
 *         p[...] = start
 * 
 */
  __pyx_t_15 = ((((PyObject *) __pyx_v_start.memview) != Py_None) != 0);
  if (__pyx_t_15) {

    /* "dipy/align/vector_fields.pyx":1300
 * 
 *     if start is not None:
 *         p[...] = start             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    if (unlikely(__pyx_memoryview_copy_contents(__pyx_v_start, __pyx_v_p, 4, 4, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1300; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1299
 *         raise ValueError("Invalid world-to-image transform")
 * 
 *     if start is not None:             # <<<<<<<<<<<<<<
 *         p[...] = start
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1302
 *         p[...] = start
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         iter_count = 0
 *         difmag = 1
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1303
 * 
 *     with nogil:
 *         iter_count = 0             # <<<<<<<<<<<<<<
 *         difmag = 1
 *         while (0.1 < difmag) and (iter_count < max_iter) and (tol < error):
 */
        __pyx_v_iter_count = 0;

        /* "dipy/align/vector_fields.pyx":1304
 *     with nogil:
 *         iter_count = 0
 *         difmag = 1             # <<<<<<<<<<<<<<
 *         while (0.1 < difmag) and (iter_count < max_iter) and (tol < error):
 *             if iter_count == 0:
 */
        __pyx_v_difmag = 1.0;

        /* "dipy/align/vector_fields.pyx":1305
 *         iter_count = 0
 *         difmag = 1
 *         while (0.1 < difmag) and (iter_count < max_iter) and (tol < error):             # <<<<<<<<<<<<<<
 *             if iter_count == 0:
 *                 epsilon = 0.75
 */
        while (1) {
          __pyx_t_14 = ((0.1 < __pyx_v_difmag) != 0);
          if (__pyx_t_14) {
          } else {
            __pyx_t_15 = __pyx_t_14;
            goto __pyx_L10_bool_binop_done;
          }
          __pyx_t_14 = ((__pyx_v_iter_count < __pyx_v_max_iter) != 0);
          if (__pyx_t_14) {
          } else {
            __pyx_t_15 = __pyx_t_14;
            goto __pyx_L10_bool_binop_done;
          }
          __pyx_t_14 = ((__pyx_v_tol < __pyx_v_error) != 0);
          __pyx_t_15 = __pyx_t_14;
          __pyx_L10_bool_binop_done:;
          if (!__pyx_t_15) break;

          /* "dipy/align/vector_fields.pyx":1306
 *         difmag = 1
 *         while (0.1 < difmag) and (iter_count < max_iter) and (tol < error):
 *             if iter_count == 0:             # <<<<<<<<<<<<<<
 *                 epsilon = 0.75
 *             else:
 */
          __pyx_t_15 = ((__pyx_v_iter_count == 0) != 0);
          if (__pyx_t_15) {

            /* "dipy/align/vector_fields.pyx":1307
 *         while (0.1 < difmag) and (iter_count < max_iter) and (tol < error):
 *             if iter_count == 0:
 *                 epsilon = 0.75             # <<<<<<<<<<<<<<
 *             else:
 *                 epsilon = 0.5
 */
            __pyx_v_epsilon = 0.75;

            /* "dipy/align/vector_fields.pyx":1306
 *         difmag = 1
 *         while (0.1 < difmag) and (iter_count < max_iter) and (tol < error):
 *             if iter_count == 0:             # <<<<<<<<<<<<<<
 *                 epsilon = 0.75
 *             else:
 */
            goto __pyx_L13;
          }

          /* "dipy/align/vector_fields.pyx":1309
 *                 epsilon = 0.75
 *             else:
 *                 epsilon = 0.5             # <<<<<<<<<<<<<<
 *             _compose_vector_fields_3d[floating](p, d, None, d_world2grid,
 *                                                 1.0, q, substats)
 */
          /*else*/ {
            __pyx_v_epsilon = 0.5;
          }
          __pyx_L13:;

          /* "dipy/align/vector_fields.pyx":1310
 *             else:
 *                 epsilon = 0.5
 *             _compose_vector_fields_3d[floating](p, d, None, d_world2grid,             # <<<<<<<<<<<<<<
 *                                                 1.0, q, substats)
 *             difmag = 0
 */
          __pyx_t_16 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
          if (unlikely(!__pyx_t_16.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1310; __pyx_clineno = __LINE__; goto __pyx_L6_error;}

          /* "dipy/align/vector_fields.pyx":1311
 *                 epsilon = 0.5
 *             _compose_vector_fields_3d[floating](p, d, None, d_world2grid,
 *                                                 1.0, q, substats)             # <<<<<<<<<<<<<<
 *             difmag = 0
 *             error = 0
 */
          __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__compose_vector_fields_3d(__pyx_v_p, __pyx_v_d, __pyx_t_16, __pyx_v_d_world2grid, 1.0, __pyx_v_q, __pyx_v_substats);
          __PYX_XDEC_MEMVIEW(&__pyx_t_16, 0);

          /* "dipy/align/vector_fields.pyx":1312
 *             _compose_vector_fields_3d[floating](p, d, None, d_world2grid,
 *                                                 1.0, q, substats)
 *             difmag = 0             # <<<<<<<<<<<<<<
 *             error = 0
 *             for k in range(ns):
 */
          __pyx_v_difmag = 0.0;

          /* "dipy/align/vector_fields.pyx":1313
 *                                                 1.0, q, substats)
 *             difmag = 0
 *             error = 0             # <<<<<<<<<<<<<<
 *             for k in range(ns):
 *                 for i in range(nr):
 */
          __pyx_v_error = 0.0;

          /* "dipy/align/vector_fields.pyx":1314
 *             difmag = 0
 *             error = 0
 *             for k in range(ns):             # <<<<<<<<<<<<<<
 *                 for i in range(nr):
 *                     for j in range(nc):
 */
          __pyx_t_17 = __pyx_v_ns;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_k = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":1315
 *             error = 0
 *             for k in range(ns):
 *                 for i in range(nr):             # <<<<<<<<<<<<<<
 *                     for j in range(nc):
 *                         mag = sqrt((q[k, i, j, 0]/ss) ** 2 +
 */
            __pyx_t_19 = __pyx_v_nr;
            for (__pyx_t_20 = 0; __pyx_t_20 < __pyx_t_19; __pyx_t_20+=1) {
              __pyx_v_i = __pyx_t_20;

              /* "dipy/align/vector_fields.pyx":1316
 *             for k in range(ns):
 *                 for i in range(nr):
 *                     for j in range(nc):             # <<<<<<<<<<<<<<
 *                         mag = sqrt((q[k, i, j, 0]/ss) ** 2 +
 *                                    (q[k, i, j, 1]/sr) ** 2 +
 */
              __pyx_t_21 = __pyx_v_nc;
              for (__pyx_t_22 = 0; __pyx_t_22 < __pyx_t_21; __pyx_t_22+=1) {
                __pyx_v_j = __pyx_t_22;

                /* "dipy/align/vector_fields.pyx":1317
 *                 for i in range(nr):
 *                     for j in range(nc):
 *                         mag = sqrt((q[k, i, j, 0]/ss) ** 2 +             # <<<<<<<<<<<<<<
 *                                    (q[k, i, j, 1]/sr) ** 2 +
 *                                    (q[k, i, j, 2]/sc) ** 2)
 */
                __pyx_t_23 = __pyx_v_k;
                __pyx_t_24 = __pyx_v_i;
                __pyx_t_25 = __pyx_v_j;
                __pyx_t_26 = 0;

                /* "dipy/align/vector_fields.pyx":1318
 *                     for j in range(nc):
 *                         mag = sqrt((q[k, i, j, 0]/ss) ** 2 +
 *                                    (q[k, i, j, 1]/sr) ** 2 +             # <<<<<<<<<<<<<<
 *                                    (q[k, i, j, 2]/sc) ** 2)
 *                         norms[k, i, j] = mag
 */
                __pyx_t_27 = __pyx_v_k;
                __pyx_t_28 = __pyx_v_i;
                __pyx_t_29 = __pyx_v_j;
                __pyx_t_30 = 1;

                /* "dipy/align/vector_fields.pyx":1319
 *                         mag = sqrt((q[k, i, j, 0]/ss) ** 2 +
 *                                    (q[k, i, j, 1]/sr) ** 2 +
 *                                    (q[k, i, j, 2]/sc) ** 2)             # <<<<<<<<<<<<<<
 *                         norms[k, i, j] = mag
 *                         error += mag
 */
                __pyx_t_31 = __pyx_v_k;
                __pyx_t_32 = __pyx_v_i;
                __pyx_t_33 = __pyx_v_j;
                __pyx_t_34 = 2;

                /* "dipy/align/vector_fields.pyx":1317
 *                 for i in range(nr):
 *                     for j in range(nc):
 *                         mag = sqrt((q[k, i, j, 0]/ss) ** 2 +             # <<<<<<<<<<<<<<
 *                                    (q[k, i, j, 1]/sr) ** 2 +
 *                                    (q[k, i, j, 2]/sc) ** 2)
 */
                __pyx_v_mag = sqrt(((pow(((*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_23 * __pyx_v_q.strides[0]) ) + __pyx_t_24 * __pyx_v_q.strides[1]) ) + __pyx_t_25 * __pyx_v_q.strides[2]) ) + __pyx_t_26 * __pyx_v_q.strides[3]) ))) / __pyx_v_ss), 2.0) + pow(((*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_27 * __pyx_v_q.strides[0]) ) + __pyx_t_28 * __pyx_v_q.strides[1]) ) + __pyx_t_29 * __pyx_v_q.strides[2]) ) + __pyx_t_30 * __pyx_v_q.strides[3]) ))) / __pyx_v_sr), 2.0)) + pow(((*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_31 * __pyx_v_q.strides[0]) ) + __pyx_t_32 * __pyx_v_q.strides[1]) ) + __pyx_t_33 * __pyx_v_q.strides[2]) ) + __pyx_t_34 * __pyx_v_q.strides[3]) ))) / __pyx_v_sc), 2.0)));

                /* "dipy/align/vector_fields.pyx":1320
 *                                    (q[k, i, j, 1]/sr) ** 2 +
 *                                    (q[k, i, j, 2]/sc) ** 2)
 *                         norms[k, i, j] = mag             # <<<<<<<<<<<<<<
 *                         error += mag
 *                         if(difmag < mag):
 */
                __pyx_t_35 = __pyx_v_k;
                __pyx_t_36 = __pyx_v_i;
                __pyx_t_37 = __pyx_v_j;
                *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_norms.data + __pyx_t_35 * __pyx_v_norms.strides[0]) ) + __pyx_t_36 * __pyx_v_norms.strides[1]) ) + __pyx_t_37 * __pyx_v_norms.strides[2]) )) = __pyx_v_mag;

                /* "dipy/align/vector_fields.pyx":1321
 *                                    (q[k, i, j, 2]/sc) ** 2)
 *                         norms[k, i, j] = mag
 *                         error += mag             # <<<<<<<<<<<<<<
 *                         if(difmag < mag):
 *                             difmag = mag
 */
                __pyx_v_error = (__pyx_v_error + __pyx_v_mag);

                /* "dipy/align/vector_fields.pyx":1322
 *                         norms[k, i, j] = mag
 *                         error += mag
 *                         if(difmag < mag):             # <<<<<<<<<<<<<<
 *                             difmag = mag
 *             maxlen = difmag*epsilon
 */
                __pyx_t_15 = ((__pyx_v_difmag < __pyx_v_mag) != 0);
                if (__pyx_t_15) {

                  /* "dipy/align/vector_fields.pyx":1323
 *                         error += mag
 *                         if(difmag < mag):
 *                             difmag = mag             # <<<<<<<<<<<<<<
 *             maxlen = difmag*epsilon
 *             for k in range(ns):
 */
                  __pyx_v_difmag = __pyx_v_mag;

                  /* "dipy/align/vector_fields.pyx":1322
 *                         norms[k, i, j] = mag
 *                         error += mag
 *                         if(difmag < mag):             # <<<<<<<<<<<<<<
 *                             difmag = mag
 *             maxlen = difmag*epsilon
 */
                }
              }
            }
          }

          /* "dipy/align/vector_fields.pyx":1324
 *                         if(difmag < mag):
 *                             difmag = mag
 *             maxlen = difmag*epsilon             # <<<<<<<<<<<<<<
 *             for k in range(ns):
 *                 for i in range(nr):
 */
          __pyx_v_maxlen = (__pyx_v_difmag * __pyx_v_epsilon);

          /* "dipy/align/vector_fields.pyx":1325
 *                             difmag = mag
 *             maxlen = difmag*epsilon
 *             for k in range(ns):             # <<<<<<<<<<<<<<
 *                 for i in range(nr):
 *                     for j in range(nc):
 */
          __pyx_t_17 = __pyx_v_ns;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_k = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":1326
 *             maxlen = difmag*epsilon
 *             for k in range(ns):
 *                 for i in range(nr):             # <<<<<<<<<<<<<<
 *                     for j in range(nc):
 *                         if norms[k, i, j] > maxlen:
 */
            __pyx_t_19 = __pyx_v_nr;
            for (__pyx_t_20 = 0; __pyx_t_20 < __pyx_t_19; __pyx_t_20+=1) {
              __pyx_v_i = __pyx_t_20;

              /* "dipy/align/vector_fields.pyx":1327
 *             for k in range(ns):
 *                 for i in range(nr):
 *                     for j in range(nc):             # <<<<<<<<<<<<<<
 *                         if norms[k, i, j] > maxlen:
 *                             step_factor = epsilon * maxlen / norms[k, i, j]
 */
              __pyx_t_21 = __pyx_v_nc;
              for (__pyx_t_22 = 0; __pyx_t_22 < __pyx_t_21; __pyx_t_22+=1) {
                __pyx_v_j = __pyx_t_22;

                /* "dipy/align/vector_fields.pyx":1328
 *                 for i in range(nr):
 *                     for j in range(nc):
 *                         if norms[k, i, j] > maxlen:             # <<<<<<<<<<<<<<
 *                             step_factor = epsilon * maxlen / norms[k, i, j]
 *                         else:
 */
                __pyx_t_38 = __pyx_v_k;
                __pyx_t_39 = __pyx_v_i;
                __pyx_t_40 = __pyx_v_j;
                __pyx_t_15 = (((*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_norms.data + __pyx_t_38 * __pyx_v_norms.strides[0]) ) + __pyx_t_39 * __pyx_v_norms.strides[1]) ) + __pyx_t_40 * __pyx_v_norms.strides[2]) ))) > __pyx_v_maxlen) != 0);
                if (__pyx_t_15) {

                  /* "dipy/align/vector_fields.pyx":1329
 *                     for j in range(nc):
 *                         if norms[k, i, j] > maxlen:
 *                             step_factor = epsilon * maxlen / norms[k, i, j]             # <<<<<<<<<<<<<<
 *                         else:
 *                             step_factor = epsilon
 */
                  __pyx_t_41 = __pyx_v_k;
                  __pyx_t_42 = __pyx_v_i;
                  __pyx_t_43 = __pyx_v_j;
                  __pyx_v_step_factor = ((__pyx_v_epsilon * __pyx_v_maxlen) / (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_norms.data + __pyx_t_41 * __pyx_v_norms.strides[0]) ) + __pyx_t_42 * __pyx_v_norms.strides[1]) ) + __pyx_t_43 * __pyx_v_norms.strides[2]) ))));

                  /* "dipy/align/vector_fields.pyx":1328
 *                 for i in range(nr):
 *                     for j in range(nc):
 *                         if norms[k, i, j] > maxlen:             # <<<<<<<<<<<<<<
 *                             step_factor = epsilon * maxlen / norms[k, i, j]
 *                         else:
 */
                  goto __pyx_L27;
                }

                /* "dipy/align/vector_fields.pyx":1331
 *                             step_factor = epsilon * maxlen / norms[k, i, j]
 *                         else:
 *                             step_factor = epsilon             # <<<<<<<<<<<<<<
 *                         p[k, i, j, 0] = (p[k, i, j, 0] -
 *                                          step_factor * q[k, i, j, 0])
 */
                /*else*/ {
                  __pyx_v_step_factor = __pyx_v_epsilon;
                }
                __pyx_L27:;

                /* "dipy/align/vector_fields.pyx":1332
 *                         else:
 *                             step_factor = epsilon
 *                         p[k, i, j, 0] = (p[k, i, j, 0] -             # <<<<<<<<<<<<<<
 *                                          step_factor * q[k, i, j, 0])
 *                         p[k, i, j, 1] = (p[k, i, j, 1] -
 */
                __pyx_t_44 = __pyx_v_k;
                __pyx_t_45 = __pyx_v_i;
                __pyx_t_46 = __pyx_v_j;
                __pyx_t_47 = 0;

                /* "dipy/align/vector_fields.pyx":1333
 *                             step_factor = epsilon
 *                         p[k, i, j, 0] = (p[k, i, j, 0] -
 *                                          step_factor * q[k, i, j, 0])             # <<<<<<<<<<<<<<
 *                         p[k, i, j, 1] = (p[k, i, j, 1] -
 *                                          step_factor * q[k, i, j, 1])
 */
                __pyx_t_48 = __pyx_v_k;
                __pyx_t_49 = __pyx_v_i;
                __pyx_t_50 = __pyx_v_j;
                __pyx_t_51 = 0;

                /* "dipy/align/vector_fields.pyx":1332
 *                         else:
 *                             step_factor = epsilon
 *                         p[k, i, j, 0] = (p[k, i, j, 0] -             # <<<<<<<<<<<<<<
 *                                          step_factor * q[k, i, j, 0])
 *                         p[k, i, j, 1] = (p[k, i, j, 1] -
 */
                __pyx_t_52 = __pyx_v_k;
                __pyx_t_53 = __pyx_v_i;
                __pyx_t_54 = __pyx_v_j;
                __pyx_t_55 = 0;
                *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_52 * __pyx_v_p.strides[0]) ) + __pyx_t_53 * __pyx_v_p.strides[1]) ) + __pyx_t_54 * __pyx_v_p.strides[2]) ) + __pyx_t_55 * __pyx_v_p.strides[3]) )) = ((*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_44 * __pyx_v_p.strides[0]) ) + __pyx_t_45 * __pyx_v_p.strides[1]) ) + __pyx_t_46 * __pyx_v_p.strides[2]) ) + __pyx_t_47 * __pyx_v_p.strides[3]) ))) - (__pyx_v_step_factor * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_48 * __pyx_v_q.strides[0]) ) + __pyx_t_49 * __pyx_v_q.strides[1]) ) + __pyx_t_50 * __pyx_v_q.strides[2]) ) + __pyx_t_51 * __pyx_v_q.strides[3]) )))));

                /* "dipy/align/vector_fields.pyx":1334
 *                         p[k, i, j, 0] = (p[k, i, j, 0] -
 *                                          step_factor * q[k, i, j, 0])
 *                         p[k, i, j, 1] = (p[k, i, j, 1] -             # <<<<<<<<<<<<<<
 *                                          step_factor * q[k, i, j, 1])
 *                         p[k, i, j, 2] = (p[k, i, j, 2] -
 */
                __pyx_t_56 = __pyx_v_k;
                __pyx_t_57 = __pyx_v_i;
                __pyx_t_58 = __pyx_v_j;
                __pyx_t_59 = 1;

                /* "dipy/align/vector_fields.pyx":1335
 *                                          step_factor * q[k, i, j, 0])
 *                         p[k, i, j, 1] = (p[k, i, j, 1] -
 *                                          step_factor * q[k, i, j, 1])             # <<<<<<<<<<<<<<
 *                         p[k, i, j, 2] = (p[k, i, j, 2] -
 *                                          step_factor * q[k, i, j, 2])
 */
                __pyx_t_60 = __pyx_v_k;
                __pyx_t_61 = __pyx_v_i;
                __pyx_t_62 = __pyx_v_j;
                __pyx_t_63 = 1;

                /* "dipy/align/vector_fields.pyx":1334
 *                         p[k, i, j, 0] = (p[k, i, j, 0] -
 *                                          step_factor * q[k, i, j, 0])
 *                         p[k, i, j, 1] = (p[k, i, j, 1] -             # <<<<<<<<<<<<<<
 *                                          step_factor * q[k, i, j, 1])
 *                         p[k, i, j, 2] = (p[k, i, j, 2] -
 */
                __pyx_t_64 = __pyx_v_k;
                __pyx_t_65 = __pyx_v_i;
                __pyx_t_66 = __pyx_v_j;
                __pyx_t_67 = 1;
                *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_64 * __pyx_v_p.strides[0]) ) + __pyx_t_65 * __pyx_v_p.strides[1]) ) + __pyx_t_66 * __pyx_v_p.strides[2]) ) + __pyx_t_67 * __pyx_v_p.strides[3]) )) = ((*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_56 * __pyx_v_p.strides[0]) ) + __pyx_t_57 * __pyx_v_p.strides[1]) ) + __pyx_t_58 * __pyx_v_p.strides[2]) ) + __pyx_t_59 * __pyx_v_p.strides[3]) ))) - (__pyx_v_step_factor * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_60 * __pyx_v_q.strides[0]) ) + __pyx_t_61 * __pyx_v_q.strides[1]) ) + __pyx_t_62 * __pyx_v_q.strides[2]) ) + __pyx_t_63 * __pyx_v_q.strides[3]) )))));

                /* "dipy/align/vector_fields.pyx":1336
 *                         p[k, i, j, 1] = (p[k, i, j, 1] -
 *                                          step_factor * q[k, i, j, 1])
 *                         p[k, i, j, 2] = (p[k, i, j, 2] -             # <<<<<<<<<<<<<<
 *                                          step_factor * q[k, i, j, 2])
 *             error /= (ns * nr * nc)
 */
                __pyx_t_68 = __pyx_v_k;
                __pyx_t_69 = __pyx_v_i;
                __pyx_t_70 = __pyx_v_j;
                __pyx_t_71 = 2;

                /* "dipy/align/vector_fields.pyx":1337
 *                                          step_factor * q[k, i, j, 1])
 *                         p[k, i, j, 2] = (p[k, i, j, 2] -
 *                                          step_factor * q[k, i, j, 2])             # <<<<<<<<<<<<<<
 *             error /= (ns * nr * nc)
 *             iter_count += 1
 */
                __pyx_t_72 = __pyx_v_k;
                __pyx_t_73 = __pyx_v_i;
                __pyx_t_74 = __pyx_v_j;
                __pyx_t_75 = 2;

                /* "dipy/align/vector_fields.pyx":1336
 *                         p[k, i, j, 1] = (p[k, i, j, 1] -
 *                                          step_factor * q[k, i, j, 1])
 *                         p[k, i, j, 2] = (p[k, i, j, 2] -             # <<<<<<<<<<<<<<
 *                                          step_factor * q[k, i, j, 2])
 *             error /= (ns * nr * nc)
 */
                __pyx_t_76 = __pyx_v_k;
                __pyx_t_77 = __pyx_v_i;
                __pyx_t_78 = __pyx_v_j;
                __pyx_t_79 = 2;
                *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_76 * __pyx_v_p.strides[0]) ) + __pyx_t_77 * __pyx_v_p.strides[1]) ) + __pyx_t_78 * __pyx_v_p.strides[2]) ) + __pyx_t_79 * __pyx_v_p.strides[3]) )) = ((*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_68 * __pyx_v_p.strides[0]) ) + __pyx_t_69 * __pyx_v_p.strides[1]) ) + __pyx_t_70 * __pyx_v_p.strides[2]) ) + __pyx_t_71 * __pyx_v_p.strides[3]) ))) - (__pyx_v_step_factor * (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_72 * __pyx_v_q.strides[0]) ) + __pyx_t_73 * __pyx_v_q.strides[1]) ) + __pyx_t_74 * __pyx_v_q.strides[2]) ) + __pyx_t_75 * __pyx_v_q.strides[3]) )))));
              }
            }
          }

          /* "dipy/align/vector_fields.pyx":1338
 *                         p[k, i, j, 2] = (p[k, i, j, 2] -
 *                                          step_factor * q[k, i, j, 2])
 *             error /= (ns * nr * nc)             # <<<<<<<<<<<<<<
 *             iter_count += 1
 *         stats[0] = error
 */
          __pyx_v_error = (__pyx_v_error / ((__pyx_v_ns * __pyx_v_nr) * __pyx_v_nc));

          /* "dipy/align/vector_fields.pyx":1339
 *                                          step_factor * q[k, i, j, 2])
 *             error /= (ns * nr * nc)
 *             iter_count += 1             # <<<<<<<<<<<<<<
 *         stats[0] = error
 *         stats[1] = iter_count
 */
          __pyx_v_iter_count = (__pyx_v_iter_count + 1);
        }

        /* "dipy/align/vector_fields.pyx":1340
 *             error /= (ns * nr * nc)
 *             iter_count += 1
 *         stats[0] = error             # <<<<<<<<<<<<<<
 *         stats[1] = iter_count
 *     return p
 */
        __pyx_t_80 = 0;
        *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_80 * __pyx_v_stats.strides[0]) )) = __pyx_v_error;

        /* "dipy/align/vector_fields.pyx":1341
 *             iter_count += 1
 *         stats[0] = error
 *         stats[1] = iter_count             # <<<<<<<<<<<<<<
 *     return p
 * 
 */
        __pyx_t_81 = 1;
        *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_81 * __pyx_v_stats.strides[0]) )) = __pyx_v_iter_count;
      }

      /* "dipy/align/vector_fields.pyx":1302
 *         p[...] = start
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         iter_count = 0
 *         difmag = 1
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L7;
        }
        __pyx_L6_error: {
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L1_error;
        }
        __pyx_L7:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1342
 *         stats[0] = error
 *         stats[1] = iter_count
 *     return p             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_p, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1342; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1231
 * 
 * 
 * def invert_vector_field_fixed_point_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                                        double[:, :] d_world2grid,
 *                                        double[:] spacing,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __PYX_XDEC_MEMVIEW(&__pyx_t_9, 1);
  __Pyx_XDECREF(__pyx_t_10);
  __PYX_XDEC_MEMVIEW(&__pyx_t_11, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_12, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_16, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.invert_vector_field_fixed_point_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_stats, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_substats, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_norms, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_p, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_q, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d_world2grid, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_spacing, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_start, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_376__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults7, __pyx_self)->__pyx_arg_start, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_159invert_vector_field_fixed_point_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_159invert_vector_field_fixed_point_3d = {"__pyx_fuse_1invert_vector_field_fixed_point_3d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_159invert_vector_field_fixed_point_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_20invert_vector_field_fixed_point_3d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_159invert_vector_field_fixed_point_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_d = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d_world2grid = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_spacing = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_v_max_iter;
  double __pyx_v_tol;
  __Pyx_memviewslice __pyx_v_start = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("invert_vector_field_fixed_point_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_d,&__pyx_n_s_d_world2grid,&__pyx_n_s_spacing,&__pyx_n_s_max_iter,&__pyx_n_s_tol,&__pyx_n_s_start,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults7 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults7, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d_world2grid)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_3d", 0, 5, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_spacing)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_3d", 0, 5, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_max_iter)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_3d", 0, 5, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_tol)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_3d", 0, 5, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_start);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "invert_vector_field_fixed_point_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_d = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[0]); if (unlikely(!__pyx_v_d.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d_world2grid = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_d_world2grid.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1232; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_spacing = __Pyx_PyObject_to_MemoryviewSlice_ds_double(values[2]); if (unlikely(!__pyx_v_spacing.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1233; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_max_iter = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_max_iter == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1234; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_tol = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_tol == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1234; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[5]) {
      __pyx_v_start = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[5]); if (unlikely(!__pyx_v_start.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1235; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_start = __pyx_dynamic_args->__pyx_arg_start;
      __PYX_INC_MEMVIEW(&__pyx_v_start, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("invert_vector_field_fixed_point_3d", 0, 5, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.invert_vector_field_fixed_point_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_158invert_vector_field_fixed_point_3d(__pyx_self, __pyx_v_d, __pyx_v_d_world2grid, __pyx_v_spacing, __pyx_v_max_iter, __pyx_v_tol, __pyx_v_start);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_158invert_vector_field_fixed_point_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_d_world2grid, __Pyx_memviewslice __pyx_v_spacing, int __pyx_v_max_iter, double __pyx_v_tol, __Pyx_memviewslice __pyx_v_start) {
  npy_intp __pyx_v_ns;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  int __pyx_v_iter_count;
  double __pyx_v_difmag;
  double __pyx_v_mag;
  double __pyx_v_maxlen;
  double __pyx_v_step_factor;
  double __pyx_v_epsilon;
  double __pyx_v_error;
  double __pyx_v_ss;
  double __pyx_v_sr;
  double __pyx_v_sc;
  PyObject *__pyx_v_ftype = NULL;
  __Pyx_memviewslice __pyx_v_stats = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_substats = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_norms = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_p = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_q = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_memviewslice __pyx_t_9 = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_t_10 = NULL;
  __Pyx_memviewslice __pyx_t_11 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_12 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  __Pyx_memviewslice __pyx_t_16 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  npy_intp __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  Py_ssize_t __pyx_t_60;
  Py_ssize_t __pyx_t_61;
  Py_ssize_t __pyx_t_62;
  Py_ssize_t __pyx_t_63;
  Py_ssize_t __pyx_t_64;
  Py_ssize_t __pyx_t_65;
  Py_ssize_t __pyx_t_66;
  Py_ssize_t __pyx_t_67;
  Py_ssize_t __pyx_t_68;
  Py_ssize_t __pyx_t_69;
  Py_ssize_t __pyx_t_70;
  Py_ssize_t __pyx_t_71;
  Py_ssize_t __pyx_t_72;
  Py_ssize_t __pyx_t_73;
  Py_ssize_t __pyx_t_74;
  Py_ssize_t __pyx_t_75;
  Py_ssize_t __pyx_t_76;
  Py_ssize_t __pyx_t_77;
  Py_ssize_t __pyx_t_78;
  Py_ssize_t __pyx_t_79;
  Py_ssize_t __pyx_t_80;
  Py_ssize_t __pyx_t_81;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1invert_vector_field_fixed_point_3d", 0);

  /* "dipy/align/vector_fields.pyx":1278
 *     """
 *     cdef:
 *         cnp.npy_intp ns = d.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr = d.shape[1]
 *         cnp.npy_intp nc = d.shape[2]
 */
  __pyx_v_ns = (__pyx_v_d.shape[0]);

  /* "dipy/align/vector_fields.pyx":1279
 *     cdef:
 *         cnp.npy_intp ns = d.shape[0]
 *         cnp.npy_intp nr = d.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = d.shape[2]
 *         int iter_count, current
 */
  __pyx_v_nr = (__pyx_v_d.shape[1]);

  /* "dipy/align/vector_fields.pyx":1280
 *         cnp.npy_intp ns = d.shape[0]
 *         cnp.npy_intp nr = d.shape[1]
 *         cnp.npy_intp nc = d.shape[2]             # <<<<<<<<<<<<<<
 *         int iter_count, current
 *         double dkk, dii, djj, dk, di, dj
 */
  __pyx_v_nc = (__pyx_v_d.shape[2]);

  /* "dipy/align/vector_fields.pyx":1284
 *         double dkk, dii, djj, dk, di, dj
 *         double difmag, mag, maxlen, step_factor
 *         double epsilon = 0.5             # <<<<<<<<<<<<<<
 *         double error = 1 + tol
 *         double ss = spacing[0], sr = spacing[1], sc = spacing[2]
 */
  __pyx_v_epsilon = 0.5;

  /* "dipy/align/vector_fields.pyx":1285
 *         double difmag, mag, maxlen, step_factor
 *         double epsilon = 0.5
 *         double error = 1 + tol             # <<<<<<<<<<<<<<
 *         double ss = spacing[0], sr = spacing[1], sc = spacing[2]
 * 
 */
  __pyx_v_error = (1.0 + __pyx_v_tol);

  /* "dipy/align/vector_fields.pyx":1286
 *         double epsilon = 0.5
 *         double error = 1 + tol
 *         double ss = spacing[0], sr = spacing[1], sc = spacing[2]             # <<<<<<<<<<<<<<
 * 
 *     ftype = np.asarray(d).dtype
 */
  __pyx_t_1 = 0;
  __pyx_v_ss = (*((double *) ( /* dim=0 */ (__pyx_v_spacing.data + __pyx_t_1 * __pyx_v_spacing.strides[0]) )));
  __pyx_t_2 = 1;
  __pyx_v_sr = (*((double *) ( /* dim=0 */ (__pyx_v_spacing.data + __pyx_t_2 * __pyx_v_spacing.strides[0]) )));
  __pyx_t_3 = 2;
  __pyx_v_sc = (*((double *) ( /* dim=0 */ (__pyx_v_spacing.data + __pyx_t_3 * __pyx_v_spacing.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":1288
 *         double ss = spacing[0], sr = spacing[1], sc = spacing[2]
 * 
 *     ftype = np.asarray(d).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_asarray); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_d, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_7 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  if (!__pyx_t_7) {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_4);
  } else {
    __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7); __pyx_t_7 = NULL;
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_dtype); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1288; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_ftype = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":1290
 *     ftype = np.asarray(d).dtype
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] substats = np.zeros(shape=(3,), dtype=np.float64)
 *         double[:, :, :] norms = np.zeros(shape=(ns, nr, nc), dtype=np.float64)
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_shape, __pyx_tuple__64) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_float64); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_6); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_9 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_5);
  if (unlikely(!__pyx_t_9.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_stats = __pyx_t_9;
  __pyx_t_9.memview = NULL;
  __pyx_t_9.data = NULL;

  /* "dipy/align/vector_fields.pyx":1291
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)
 *         double[:] substats = np.zeros(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:, :, :] norms = np.zeros(shape=(ns, nr, nc), dtype=np.float64)
 *         floating[:, :, :, :] p = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_zeros); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_shape, __pyx_tuple__65) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_float64); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_empty_tuple, __pyx_t_5); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_9 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_8);
  if (unlikely(!__pyx_t_9.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_substats = __pyx_t_9;
  __pyx_t_9.memview = NULL;
  __pyx_t_9.data = NULL;

  /* "dipy/align/vector_fields.pyx":1292
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)
 *         double[:] substats = np.zeros(shape=(3,), dtype=np.float64)
 *         double[:, :, :] norms = np.zeros(shape=(ns, nr, nc), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         floating[:, :, :, :] p = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)
 *         floating[:, :, :, :] q = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = PyDict_New(); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ns); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nr); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nc); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_10 = PyTuple_New(3); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_10, 2, __pyx_t_7);
  __pyx_t_6 = 0;
  __pyx_t_4 = 0;
  __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_shape, __pyx_t_10) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_float64); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_dtype, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_empty_tuple, __pyx_t_8); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_11 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_7);
  if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1292; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_norms = __pyx_t_11;
  __pyx_t_11.memview = NULL;
  __pyx_t_11.data = NULL;

  /* "dipy/align/vector_fields.pyx":1293
 *         double[:] substats = np.zeros(shape=(3,), dtype=np.float64)
 *         double[:, :, :] norms = np.zeros(shape=(ns, nr, nc), dtype=np.float64)
 *         floating[:, :, :, :] p = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)             # <<<<<<<<<<<<<<
 *         floating[:, :, :, :] q = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)
 * 
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_zeros); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyDict_New(); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ns); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_10 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nr); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nc); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 3, __pyx_int_3);
  __pyx_t_5 = 0;
  __pyx_t_10 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_empty_tuple, __pyx_t_7); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_12 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_t_6);
  if (unlikely(!__pyx_t_12.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1293; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_p = __pyx_t_12;
  __pyx_t_12.memview = NULL;
  __pyx_t_12.data = NULL;

  /* "dipy/align/vector_fields.pyx":1294
 *         double[:, :, :] norms = np.zeros(shape=(ns, nr, nc), dtype=np.float64)
 *         floating[:, :, :, :] p = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)
 *         floating[:, :, :, :] q = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(d_world2grid, 3):
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_zeros); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_8 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ns); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nr); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_10 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nc); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_10);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_int_3);
  __pyx_t_8 = 0;
  __pyx_t_4 = 0;
  __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_empty_tuple, __pyx_t_6); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_12 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_t_5);
  if (unlikely(!__pyx_t_12.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1294; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_q = __pyx_t_12;
  __pyx_t_12.memview = NULL;
  __pyx_t_12.data = NULL;

  /* "dipy/align/vector_fields.pyx":1296
 *         floating[:, :, :, :] q = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)
 * 
 *     if not is_valid_affine(d_world2grid, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid world-to-image transform")
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_d_world2grid, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_10 = NULL;
  __pyx_t_13 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
      __pyx_t_13 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_10) {
    __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_10); __pyx_t_10 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_13, __pyx_t_7);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_13, __pyx_int_3);
  __pyx_t_7 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_4, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_14 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1296; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_15 = ((!__pyx_t_14) != 0);
  if (__pyx_t_15) {

    /* "dipy/align/vector_fields.pyx":1297
 * 
 *     if not is_valid_affine(d_world2grid, 3):
 *         raise ValueError("Invalid world-to-image transform")             # <<<<<<<<<<<<<<
 * 
 *     if start is not None:
 */
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__66, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1297; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1297; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1296
 *         floating[:, :, :, :] q = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)
 * 
 *     if not is_valid_affine(d_world2grid, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid world-to-image transform")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1299
 *         raise ValueError("Invalid world-to-image transform")
 * 
 *     if start is not None:             # <<<<<<<<<<<<<<
 *         p[...] = start
 * 
 */
  __pyx_t_15 = ((((PyObject *) __pyx_v_start.memview) != Py_None) != 0);
  if (__pyx_t_15) {

    /* "dipy/align/vector_fields.pyx":1300
 * 
 *     if start is not None:
 *         p[...] = start             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    if (unlikely(__pyx_memoryview_copy_contents(__pyx_v_start, __pyx_v_p, 4, 4, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1300; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1299
 *         raise ValueError("Invalid world-to-image transform")
 * 
 *     if start is not None:             # <<<<<<<<<<<<<<
 *         p[...] = start
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1302
 *         p[...] = start
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         iter_count = 0
 *         difmag = 1
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1303
 * 
 *     with nogil:
 *         iter_count = 0             # <<<<<<<<<<<<<<
 *         difmag = 1
 *         while (0.1 < difmag) and (iter_count < max_iter) and (tol < error):
 */
        __pyx_v_iter_count = 0;

        /* "dipy/align/vector_fields.pyx":1304
 *     with nogil:
 *         iter_count = 0
 *         difmag = 1             # <<<<<<<<<<<<<<
 *         while (0.1 < difmag) and (iter_count < max_iter) and (tol < error):
 *             if iter_count == 0:
 */
        __pyx_v_difmag = 1.0;

        /* "dipy/align/vector_fields.pyx":1305
 *         iter_count = 0
 *         difmag = 1
 *         while (0.1 < difmag) and (iter_count < max_iter) and (tol < error):             # <<<<<<<<<<<<<<
 *             if iter_count == 0:
 *                 epsilon = 0.75
 */
        while (1) {
          __pyx_t_14 = ((0.1 < __pyx_v_difmag) != 0);
          if (__pyx_t_14) {
          } else {
            __pyx_t_15 = __pyx_t_14;
            goto __pyx_L10_bool_binop_done;
          }
          __pyx_t_14 = ((__pyx_v_iter_count < __pyx_v_max_iter) != 0);
          if (__pyx_t_14) {
          } else {
            __pyx_t_15 = __pyx_t_14;
            goto __pyx_L10_bool_binop_done;
          }
          __pyx_t_14 = ((__pyx_v_tol < __pyx_v_error) != 0);
          __pyx_t_15 = __pyx_t_14;
          __pyx_L10_bool_binop_done:;
          if (!__pyx_t_15) break;

          /* "dipy/align/vector_fields.pyx":1306
 *         difmag = 1
 *         while (0.1 < difmag) and (iter_count < max_iter) and (tol < error):
 *             if iter_count == 0:             # <<<<<<<<<<<<<<
 *                 epsilon = 0.75
 *             else:
 */
          __pyx_t_15 = ((__pyx_v_iter_count == 0) != 0);
          if (__pyx_t_15) {

            /* "dipy/align/vector_fields.pyx":1307
 *         while (0.1 < difmag) and (iter_count < max_iter) and (tol < error):
 *             if iter_count == 0:
 *                 epsilon = 0.75             # <<<<<<<<<<<<<<
 *             else:
 *                 epsilon = 0.5
 */
            __pyx_v_epsilon = 0.75;

            /* "dipy/align/vector_fields.pyx":1306
 *         difmag = 1
 *         while (0.1 < difmag) and (iter_count < max_iter) and (tol < error):
 *             if iter_count == 0:             # <<<<<<<<<<<<<<
 *                 epsilon = 0.75
 *             else:
 */
            goto __pyx_L13;
          }

          /* "dipy/align/vector_fields.pyx":1309
 *                 epsilon = 0.75
 *             else:
 *                 epsilon = 0.5             # <<<<<<<<<<<<<<
 *             _compose_vector_fields_3d[floating](p, d, None, d_world2grid,
 *                                                 1.0, q, substats)
 */
          /*else*/ {
            __pyx_v_epsilon = 0.5;
          }
          __pyx_L13:;

          /* "dipy/align/vector_fields.pyx":1310
 *             else:
 *                 epsilon = 0.5
 *             _compose_vector_fields_3d[floating](p, d, None, d_world2grid,             # <<<<<<<<<<<<<<
 *                                                 1.0, q, substats)
 *             difmag = 0
 */
          __pyx_t_16 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
          if (unlikely(!__pyx_t_16.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1310; __pyx_clineno = __LINE__; goto __pyx_L6_error;}

          /* "dipy/align/vector_fields.pyx":1311
 *                 epsilon = 0.5
 *             _compose_vector_fields_3d[floating](p, d, None, d_world2grid,
 *                                                 1.0, q, substats)             # <<<<<<<<<<<<<<
 *             difmag = 0
 *             error = 0
 */
          __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__compose_vector_fields_3d(__pyx_v_p, __pyx_v_d, __pyx_t_16, __pyx_v_d_world2grid, 1.0, __pyx_v_q, __pyx_v_substats);
          __PYX_XDEC_MEMVIEW(&__pyx_t_16, 0);

          /* "dipy/align/vector_fields.pyx":1312
 *             _compose_vector_fields_3d[floating](p, d, None, d_world2grid,
 *                                                 1.0, q, substats)
 *             difmag = 0             # <<<<<<<<<<<<<<
 *             error = 0
 *             for k in range(ns):
 */
          __pyx_v_difmag = 0.0;

          /* "dipy/align/vector_fields.pyx":1313
 *                                                 1.0, q, substats)
 *             difmag = 0
 *             error = 0             # <<<<<<<<<<<<<<
 *             for k in range(ns):
 *                 for i in range(nr):
 */
          __pyx_v_error = 0.0;

          /* "dipy/align/vector_fields.pyx":1314
 *             difmag = 0
 *             error = 0
 *             for k in range(ns):             # <<<<<<<<<<<<<<
 *                 for i in range(nr):
 *                     for j in range(nc):
 */
          __pyx_t_17 = __pyx_v_ns;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_k = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":1315
 *             error = 0
 *             for k in range(ns):
 *                 for i in range(nr):             # <<<<<<<<<<<<<<
 *                     for j in range(nc):
 *                         mag = sqrt((q[k, i, j, 0]/ss) ** 2 +
 */
            __pyx_t_19 = __pyx_v_nr;
            for (__pyx_t_20 = 0; __pyx_t_20 < __pyx_t_19; __pyx_t_20+=1) {
              __pyx_v_i = __pyx_t_20;

              /* "dipy/align/vector_fields.pyx":1316
 *             for k in range(ns):
 *                 for i in range(nr):
 *                     for j in range(nc):             # <<<<<<<<<<<<<<
 *                         mag = sqrt((q[k, i, j, 0]/ss) ** 2 +
 *                                    (q[k, i, j, 1]/sr) ** 2 +
 */
              __pyx_t_21 = __pyx_v_nc;
              for (__pyx_t_22 = 0; __pyx_t_22 < __pyx_t_21; __pyx_t_22+=1) {
                __pyx_v_j = __pyx_t_22;

                /* "dipy/align/vector_fields.pyx":1317
 *                 for i in range(nr):
 *                     for j in range(nc):
 *                         mag = sqrt((q[k, i, j, 0]/ss) ** 2 +             # <<<<<<<<<<<<<<
 *                                    (q[k, i, j, 1]/sr) ** 2 +
 *                                    (q[k, i, j, 2]/sc) ** 2)
 */
                __pyx_t_23 = __pyx_v_k;
                __pyx_t_24 = __pyx_v_i;
                __pyx_t_25 = __pyx_v_j;
                __pyx_t_26 = 0;

                /* "dipy/align/vector_fields.pyx":1318
 *                     for j in range(nc):
 *                         mag = sqrt((q[k, i, j, 0]/ss) ** 2 +
 *                                    (q[k, i, j, 1]/sr) ** 2 +             # <<<<<<<<<<<<<<
 *                                    (q[k, i, j, 2]/sc) ** 2)
 *                         norms[k, i, j] = mag
 */
                __pyx_t_27 = __pyx_v_k;
                __pyx_t_28 = __pyx_v_i;
                __pyx_t_29 = __pyx_v_j;
                __pyx_t_30 = 1;

                /* "dipy/align/vector_fields.pyx":1319
 *                         mag = sqrt((q[k, i, j, 0]/ss) ** 2 +
 *                                    (q[k, i, j, 1]/sr) ** 2 +
 *                                    (q[k, i, j, 2]/sc) ** 2)             # <<<<<<<<<<<<<<
 *                         norms[k, i, j] = mag
 *                         error += mag
 */
                __pyx_t_31 = __pyx_v_k;
                __pyx_t_32 = __pyx_v_i;
                __pyx_t_33 = __pyx_v_j;
                __pyx_t_34 = 2;

                /* "dipy/align/vector_fields.pyx":1317
 *                 for i in range(nr):
 *                     for j in range(nc):
 *                         mag = sqrt((q[k, i, j, 0]/ss) ** 2 +             # <<<<<<<<<<<<<<
 *                                    (q[k, i, j, 1]/sr) ** 2 +
 *                                    (q[k, i, j, 2]/sc) ** 2)
 */
                __pyx_v_mag = sqrt(((pow(((*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_23 * __pyx_v_q.strides[0]) ) + __pyx_t_24 * __pyx_v_q.strides[1]) ) + __pyx_t_25 * __pyx_v_q.strides[2]) ) + __pyx_t_26 * __pyx_v_q.strides[3]) ))) / __pyx_v_ss), 2.0) + pow(((*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_27 * __pyx_v_q.strides[0]) ) + __pyx_t_28 * __pyx_v_q.strides[1]) ) + __pyx_t_29 * __pyx_v_q.strides[2]) ) + __pyx_t_30 * __pyx_v_q.strides[3]) ))) / __pyx_v_sr), 2.0)) + pow(((*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_31 * __pyx_v_q.strides[0]) ) + __pyx_t_32 * __pyx_v_q.strides[1]) ) + __pyx_t_33 * __pyx_v_q.strides[2]) ) + __pyx_t_34 * __pyx_v_q.strides[3]) ))) / __pyx_v_sc), 2.0)));

                /* "dipy/align/vector_fields.pyx":1320
 *                                    (q[k, i, j, 1]/sr) ** 2 +
 *                                    (q[k, i, j, 2]/sc) ** 2)
 *                         norms[k, i, j] = mag             # <<<<<<<<<<<<<<
 *                         error += mag
 *                         if(difmag < mag):
 */
                __pyx_t_35 = __pyx_v_k;
                __pyx_t_36 = __pyx_v_i;
                __pyx_t_37 = __pyx_v_j;
                *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_norms.data + __pyx_t_35 * __pyx_v_norms.strides[0]) ) + __pyx_t_36 * __pyx_v_norms.strides[1]) ) + __pyx_t_37 * __pyx_v_norms.strides[2]) )) = __pyx_v_mag;

                /* "dipy/align/vector_fields.pyx":1321
 *                                    (q[k, i, j, 2]/sc) ** 2)
 *                         norms[k, i, j] = mag
 *                         error += mag             # <<<<<<<<<<<<<<
 *                         if(difmag < mag):
 *                             difmag = mag
 */
                __pyx_v_error = (__pyx_v_error + __pyx_v_mag);

                /* "dipy/align/vector_fields.pyx":1322
 *                         norms[k, i, j] = mag
 *                         error += mag
 *                         if(difmag < mag):             # <<<<<<<<<<<<<<
 *                             difmag = mag
 *             maxlen = difmag*epsilon
 */
                __pyx_t_15 = ((__pyx_v_difmag < __pyx_v_mag) != 0);
                if (__pyx_t_15) {

                  /* "dipy/align/vector_fields.pyx":1323
 *                         error += mag
 *                         if(difmag < mag):
 *                             difmag = mag             # <<<<<<<<<<<<<<
 *             maxlen = difmag*epsilon
 *             for k in range(ns):
 */
                  __pyx_v_difmag = __pyx_v_mag;

                  /* "dipy/align/vector_fields.pyx":1322
 *                         norms[k, i, j] = mag
 *                         error += mag
 *                         if(difmag < mag):             # <<<<<<<<<<<<<<
 *                             difmag = mag
 *             maxlen = difmag*epsilon
 */
                }
              }
            }
          }

          /* "dipy/align/vector_fields.pyx":1324
 *                         if(difmag < mag):
 *                             difmag = mag
 *             maxlen = difmag*epsilon             # <<<<<<<<<<<<<<
 *             for k in range(ns):
 *                 for i in range(nr):
 */
          __pyx_v_maxlen = (__pyx_v_difmag * __pyx_v_epsilon);

          /* "dipy/align/vector_fields.pyx":1325
 *                             difmag = mag
 *             maxlen = difmag*epsilon
 *             for k in range(ns):             # <<<<<<<<<<<<<<
 *                 for i in range(nr):
 *                     for j in range(nc):
 */
          __pyx_t_17 = __pyx_v_ns;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_k = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":1326
 *             maxlen = difmag*epsilon
 *             for k in range(ns):
 *                 for i in range(nr):             # <<<<<<<<<<<<<<
 *                     for j in range(nc):
 *                         if norms[k, i, j] > maxlen:
 */
            __pyx_t_19 = __pyx_v_nr;
            for (__pyx_t_20 = 0; __pyx_t_20 < __pyx_t_19; __pyx_t_20+=1) {
              __pyx_v_i = __pyx_t_20;

              /* "dipy/align/vector_fields.pyx":1327
 *             for k in range(ns):
 *                 for i in range(nr):
 *                     for j in range(nc):             # <<<<<<<<<<<<<<
 *                         if norms[k, i, j] > maxlen:
 *                             step_factor = epsilon * maxlen / norms[k, i, j]
 */
              __pyx_t_21 = __pyx_v_nc;
              for (__pyx_t_22 = 0; __pyx_t_22 < __pyx_t_21; __pyx_t_22+=1) {
                __pyx_v_j = __pyx_t_22;

                /* "dipy/align/vector_fields.pyx":1328
 *                 for i in range(nr):
 *                     for j in range(nc):
 *                         if norms[k, i, j] > maxlen:             # <<<<<<<<<<<<<<
 *                             step_factor = epsilon * maxlen / norms[k, i, j]
 *                         else:
 */
                __pyx_t_38 = __pyx_v_k;
                __pyx_t_39 = __pyx_v_i;
                __pyx_t_40 = __pyx_v_j;
                __pyx_t_15 = (((*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_norms.data + __pyx_t_38 * __pyx_v_norms.strides[0]) ) + __pyx_t_39 * __pyx_v_norms.strides[1]) ) + __pyx_t_40 * __pyx_v_norms.strides[2]) ))) > __pyx_v_maxlen) != 0);
                if (__pyx_t_15) {

                  /* "dipy/align/vector_fields.pyx":1329
 *                     for j in range(nc):
 *                         if norms[k, i, j] > maxlen:
 *                             step_factor = epsilon * maxlen / norms[k, i, j]             # <<<<<<<<<<<<<<
 *                         else:
 *                             step_factor = epsilon
 */
                  __pyx_t_41 = __pyx_v_k;
                  __pyx_t_42 = __pyx_v_i;
                  __pyx_t_43 = __pyx_v_j;
                  __pyx_v_step_factor = ((__pyx_v_epsilon * __pyx_v_maxlen) / (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_norms.data + __pyx_t_41 * __pyx_v_norms.strides[0]) ) + __pyx_t_42 * __pyx_v_norms.strides[1]) ) + __pyx_t_43 * __pyx_v_norms.strides[2]) ))));

                  /* "dipy/align/vector_fields.pyx":1328
 *                 for i in range(nr):
 *                     for j in range(nc):
 *                         if norms[k, i, j] > maxlen:             # <<<<<<<<<<<<<<
 *                             step_factor = epsilon * maxlen / norms[k, i, j]
 *                         else:
 */
                  goto __pyx_L27;
                }

                /* "dipy/align/vector_fields.pyx":1331
 *                             step_factor = epsilon * maxlen / norms[k, i, j]
 *                         else:
 *                             step_factor = epsilon             # <<<<<<<<<<<<<<
 *                         p[k, i, j, 0] = (p[k, i, j, 0] -
 *                                          step_factor * q[k, i, j, 0])
 */
                /*else*/ {
                  __pyx_v_step_factor = __pyx_v_epsilon;
                }
                __pyx_L27:;

                /* "dipy/align/vector_fields.pyx":1332
 *                         else:
 *                             step_factor = epsilon
 *                         p[k, i, j, 0] = (p[k, i, j, 0] -             # <<<<<<<<<<<<<<
 *                                          step_factor * q[k, i, j, 0])
 *                         p[k, i, j, 1] = (p[k, i, j, 1] -
 */
                __pyx_t_44 = __pyx_v_k;
                __pyx_t_45 = __pyx_v_i;
                __pyx_t_46 = __pyx_v_j;
                __pyx_t_47 = 0;

                /* "dipy/align/vector_fields.pyx":1333
 *                             step_factor = epsilon
 *                         p[k, i, j, 0] = (p[k, i, j, 0] -
 *                                          step_factor * q[k, i, j, 0])             # <<<<<<<<<<<<<<
 *                         p[k, i, j, 1] = (p[k, i, j, 1] -
 *                                          step_factor * q[k, i, j, 1])
 */
                __pyx_t_48 = __pyx_v_k;
                __pyx_t_49 = __pyx_v_i;
                __pyx_t_50 = __pyx_v_j;
                __pyx_t_51 = 0;

                /* "dipy/align/vector_fields.pyx":1332
 *                         else:
 *                             step_factor = epsilon
 *                         p[k, i, j, 0] = (p[k, i, j, 0] -             # <<<<<<<<<<<<<<
 *                                          step_factor * q[k, i, j, 0])
 *                         p[k, i, j, 1] = (p[k, i, j, 1] -
 */
                __pyx_t_52 = __pyx_v_k;
                __pyx_t_53 = __pyx_v_i;
                __pyx_t_54 = __pyx_v_j;
                __pyx_t_55 = 0;
                *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_52 * __pyx_v_p.strides[0]) ) + __pyx_t_53 * __pyx_v_p.strides[1]) ) + __pyx_t_54 * __pyx_v_p.strides[2]) ) + __pyx_t_55 * __pyx_v_p.strides[3]) )) = ((*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_44 * __pyx_v_p.strides[0]) ) + __pyx_t_45 * __pyx_v_p.strides[1]) ) + __pyx_t_46 * __pyx_v_p.strides[2]) ) + __pyx_t_47 * __pyx_v_p.strides[3]) ))) - (__pyx_v_step_factor * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_48 * __pyx_v_q.strides[0]) ) + __pyx_t_49 * __pyx_v_q.strides[1]) ) + __pyx_t_50 * __pyx_v_q.strides[2]) ) + __pyx_t_51 * __pyx_v_q.strides[3]) )))));

                /* "dipy/align/vector_fields.pyx":1334
 *                         p[k, i, j, 0] = (p[k, i, j, 0] -
 *                                          step_factor * q[k, i, j, 0])
 *                         p[k, i, j, 1] = (p[k, i, j, 1] -             # <<<<<<<<<<<<<<
 *                                          step_factor * q[k, i, j, 1])
 *                         p[k, i, j, 2] = (p[k, i, j, 2] -
 */
                __pyx_t_56 = __pyx_v_k;
                __pyx_t_57 = __pyx_v_i;
                __pyx_t_58 = __pyx_v_j;
                __pyx_t_59 = 1;

                /* "dipy/align/vector_fields.pyx":1335
 *                                          step_factor * q[k, i, j, 0])
 *                         p[k, i, j, 1] = (p[k, i, j, 1] -
 *                                          step_factor * q[k, i, j, 1])             # <<<<<<<<<<<<<<
 *                         p[k, i, j, 2] = (p[k, i, j, 2] -
 *                                          step_factor * q[k, i, j, 2])
 */
                __pyx_t_60 = __pyx_v_k;
                __pyx_t_61 = __pyx_v_i;
                __pyx_t_62 = __pyx_v_j;
                __pyx_t_63 = 1;

                /* "dipy/align/vector_fields.pyx":1334
 *                         p[k, i, j, 0] = (p[k, i, j, 0] -
 *                                          step_factor * q[k, i, j, 0])
 *                         p[k, i, j, 1] = (p[k, i, j, 1] -             # <<<<<<<<<<<<<<
 *                                          step_factor * q[k, i, j, 1])
 *                         p[k, i, j, 2] = (p[k, i, j, 2] -
 */
                __pyx_t_64 = __pyx_v_k;
                __pyx_t_65 = __pyx_v_i;
                __pyx_t_66 = __pyx_v_j;
                __pyx_t_67 = 1;
                *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_64 * __pyx_v_p.strides[0]) ) + __pyx_t_65 * __pyx_v_p.strides[1]) ) + __pyx_t_66 * __pyx_v_p.strides[2]) ) + __pyx_t_67 * __pyx_v_p.strides[3]) )) = ((*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_56 * __pyx_v_p.strides[0]) ) + __pyx_t_57 * __pyx_v_p.strides[1]) ) + __pyx_t_58 * __pyx_v_p.strides[2]) ) + __pyx_t_59 * __pyx_v_p.strides[3]) ))) - (__pyx_v_step_factor * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_60 * __pyx_v_q.strides[0]) ) + __pyx_t_61 * __pyx_v_q.strides[1]) ) + __pyx_t_62 * __pyx_v_q.strides[2]) ) + __pyx_t_63 * __pyx_v_q.strides[3]) )))));

                /* "dipy/align/vector_fields.pyx":1336
 *                         p[k, i, j, 1] = (p[k, i, j, 1] -
 *                                          step_factor * q[k, i, j, 1])
 *                         p[k, i, j, 2] = (p[k, i, j, 2] -             # <<<<<<<<<<<<<<
 *                                          step_factor * q[k, i, j, 2])
 *             error /= (ns * nr * nc)
 */
                __pyx_t_68 = __pyx_v_k;
                __pyx_t_69 = __pyx_v_i;
                __pyx_t_70 = __pyx_v_j;
                __pyx_t_71 = 2;

                /* "dipy/align/vector_fields.pyx":1337
 *                                          step_factor * q[k, i, j, 1])
 *                         p[k, i, j, 2] = (p[k, i, j, 2] -
 *                                          step_factor * q[k, i, j, 2])             # <<<<<<<<<<<<<<
 *             error /= (ns * nr * nc)
 *             iter_count += 1
 */
                __pyx_t_72 = __pyx_v_k;
                __pyx_t_73 = __pyx_v_i;
                __pyx_t_74 = __pyx_v_j;
                __pyx_t_75 = 2;

                /* "dipy/align/vector_fields.pyx":1336
 *                         p[k, i, j, 1] = (p[k, i, j, 1] -
 *                                          step_factor * q[k, i, j, 1])
 *                         p[k, i, j, 2] = (p[k, i, j, 2] -             # <<<<<<<<<<<<<<
 *                                          step_factor * q[k, i, j, 2])
 *             error /= (ns * nr * nc)
 */
                __pyx_t_76 = __pyx_v_k;
                __pyx_t_77 = __pyx_v_i;
                __pyx_t_78 = __pyx_v_j;
                __pyx_t_79 = 2;
                *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_76 * __pyx_v_p.strides[0]) ) + __pyx_t_77 * __pyx_v_p.strides[1]) ) + __pyx_t_78 * __pyx_v_p.strides[2]) ) + __pyx_t_79 * __pyx_v_p.strides[3]) )) = ((*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_p.data + __pyx_t_68 * __pyx_v_p.strides[0]) ) + __pyx_t_69 * __pyx_v_p.strides[1]) ) + __pyx_t_70 * __pyx_v_p.strides[2]) ) + __pyx_t_71 * __pyx_v_p.strides[3]) ))) - (__pyx_v_step_factor * (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_q.data + __pyx_t_72 * __pyx_v_q.strides[0]) ) + __pyx_t_73 * __pyx_v_q.strides[1]) ) + __pyx_t_74 * __pyx_v_q.strides[2]) ) + __pyx_t_75 * __pyx_v_q.strides[3]) )))));
              }
            }
          }

          /* "dipy/align/vector_fields.pyx":1338
 *                         p[k, i, j, 2] = (p[k, i, j, 2] -
 *                                          step_factor * q[k, i, j, 2])
 *             error /= (ns * nr * nc)             # <<<<<<<<<<<<<<
 *             iter_count += 1
 *         stats[0] = error
 */
          __pyx_v_error = (__pyx_v_error / ((__pyx_v_ns * __pyx_v_nr) * __pyx_v_nc));

          /* "dipy/align/vector_fields.pyx":1339
 *                                          step_factor * q[k, i, j, 2])
 *             error /= (ns * nr * nc)
 *             iter_count += 1             # <<<<<<<<<<<<<<
 *         stats[0] = error
 *         stats[1] = iter_count
 */
          __pyx_v_iter_count = (__pyx_v_iter_count + 1);
        }

        /* "dipy/align/vector_fields.pyx":1340
 *             error /= (ns * nr * nc)
 *             iter_count += 1
 *         stats[0] = error             # <<<<<<<<<<<<<<
 *         stats[1] = iter_count
 *     return p
 */
        __pyx_t_80 = 0;
        *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_80 * __pyx_v_stats.strides[0]) )) = __pyx_v_error;

        /* "dipy/align/vector_fields.pyx":1341
 *             iter_count += 1
 *         stats[0] = error
 *         stats[1] = iter_count             # <<<<<<<<<<<<<<
 *     return p
 * 
 */
        __pyx_t_81 = 1;
        *((double *) ( /* dim=0 */ (__pyx_v_stats.data + __pyx_t_81 * __pyx_v_stats.strides[0]) )) = __pyx_v_iter_count;
      }

      /* "dipy/align/vector_fields.pyx":1302
 *         p[...] = start
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         iter_count = 0
 *         difmag = 1
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L7;
        }
        __pyx_L6_error: {
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L1_error;
        }
        __pyx_L7:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1342
 *         stats[0] = error
 *         stats[1] = iter_count
 *     return p             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_p, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1342; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1231
 * 
 * 
 * def invert_vector_field_fixed_point_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                                        double[:, :] d_world2grid,
 *                                        double[:] spacing,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __PYX_XDEC_MEMVIEW(&__pyx_t_9, 1);
  __Pyx_XDECREF(__pyx_t_10);
  __PYX_XDEC_MEMVIEW(&__pyx_t_11, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_12, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_16, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.invert_vector_field_fixed_point_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_stats, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_substats, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_norms, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_p, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_q, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d_world2grid, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_spacing, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_start, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":1345
 * 
 * 
 * def simplify_warp_function_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                               double[:, :] affine_idx_in,
 *                               double[:, :] affine_idx_out,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_23simplify_warp_function_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_22simplify_warp_function_2d[] = "\n    Simplifies a nonlinear warping function combined with an affine transform\n\n    Modifies the given deformation field by incorporating into it a\n    an affine transformation and voxel-to-space transforms associated to\n    the discretization of its domain and codomain.\n    The resulting transformation may be regarded as operating on the\n    image spaces given by the domain and codomain discretization.\n    More precisely, the resulting transform is of the form:\n\n    (1) T[i] = W * d[U * i] + V * i\n\n    Where U = affine_idx_in, V = affine_idx_out, W = affine_disp.\n    Both the direct and inverse transforms of a DiffeomorphicMap can be written\n    in this form:\n\n    Direct:  Let D be the voxel-to-space transform of the domain's\n             discretization, P be the pre-align matrix, Rinv the space-to-voxel\n             transform of the reference grid (the grid the displacement field\n             is defined on) and Cinv be the space-to-voxel transform of the\n             codomain's discretization. Then, for each i in the domain's grid,\n             the direct transform is given by\n\n             (2) T[i] = Cinv * d[Rinv * P * D * i] + Cinv * P * D * i\n\n             and we identify U = Rinv * P * D, V = Cinv * P * D, W = Cinv\n\n    Inverse: Let C be the voxel-to-space transform of the codomain's\n             discretization, Pinv be the inverse of the pre-align matrix, Rinv\n             the space-to-voxel transform of the reference grid (the grid the\n             displacement field is defined on) and Dinv be the space-to-voxel\n             transform of the domain's discretization. Then, for each j in the\n             codomain's grid, the inverse transform is given by\n\n             (3) Tinv[j] = Dinv * Pinv * d[Rinv * C * j] + Dinv * Pinv * C * j\n\n             and we identify U = Rinv * C, V = Dinv * Pinv * C, W = Dinv * Pinv\n\n    Parameters\n    ----------\n    d : array, shape (R', C', 2)\n        the non-linear part of the transform""ation (displacement field)\n    affine_idx_in : array, shape (3, 3)\n        the matrix U in eq. (1) above\n    affine_idx_out : array, shape (3, 3)\n        the matrix V in eq. (1) above\n    affine_disp : array, shape (3, 3)\n        the matrix W in eq. (1) above\n    out_shape : array, shape (2,)\n        the number of rows and columns of the sampling grid\n\n    Returns\n    -------\n    out : array, shape = out_shape\n        the simplified transformation given by one single displacement field\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_23simplify_warp_function_2d = {"simplify_warp_function_2d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_23simplify_warp_function_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_22simplify_warp_function_2d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_23simplify_warp_function_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_22simplify_warp_function_2d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_22simplify_warp_function_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("simplify_warp_function_2d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_d, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_d); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__67, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__68, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__69, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__70, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_163simplify_warp_function_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_163simplify_warp_function_2d = {"__pyx_fuse_0simplify_warp_function_2d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_163simplify_warp_function_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_22simplify_warp_function_2d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_163simplify_warp_function_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_d = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("simplify_warp_function_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_d,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simplify_warp_function_2d", 1, 5, 5, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simplify_warp_function_2d", 1, 5, 5, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simplify_warp_function_2d", 1, 5, 5, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simplify_warp_function_2d", 1, 5, 5, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "simplify_warp_function_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_d = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[0]); if (unlikely(!__pyx_v_d.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1346; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1347; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1348; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[4]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1349; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("simplify_warp_function_2d", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.simplify_warp_function_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_162simplify_warp_function_2d(__pyx_self, __pyx_v_d, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_162simplify_warp_function_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_memviewslice __pyx_t_10 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_11 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0simplify_warp_function_2d", 0);

  /* "dipy/align/vector_fields.pyx":1407
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = out_shape[1]
 *         cnp.npy_intp i, j
 */
  __pyx_t_1 = 0;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_1 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":1408
 *     cdef:
 *         cnp.npy_intp nrows = out_shape[0]
 *         cnp.npy_intp ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j
 *         double di, dj, dii, djj
 */
  __pyx_t_2 = 1;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_2 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":1411
 *         cnp.npy_intp i, j
 *         double di, dj, dii, djj
 *         floating[:] tmp = np.zeros((2,), dtype=np.asarray(d).dtype)             # <<<<<<<<<<<<<<
 *         floating[:, :, :] out = np.zeros(shape=(nrows, ncols, 2),
 *                                          dtype=np.asarray(d).dtype)
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_8 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
    }
  }
  if (!__pyx_t_8) {
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_5);
  } else {
    __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_9, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_dtype); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__72, __pyx_t_3); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_10 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_7);
  if (unlikely(!__pyx_t_10.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_tmp = __pyx_t_10;
  __pyx_t_10.memview = NULL;
  __pyx_t_10.data = NULL;

  /* "dipy/align/vector_fields.pyx":1412
 *         double di, dj, dii, djj
 *         floating[:] tmp = np.zeros((2,), dtype=np.asarray(d).dtype)
 *         floating[:, :, :] out = np.zeros(shape=(nrows, ncols, 2),             # <<<<<<<<<<<<<<
 *                                          dtype=np.asarray(d).dtype)
 * 
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_zeros); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyDict_New(); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_9 = PyTuple_New(3); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_shape, __pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "dipy/align/vector_fields.pyx":1413
 *         floating[:] tmp = np.zeros((2,), dtype=np.asarray(d).dtype)
 *         floating[:, :, :] out = np.zeros(shape=(nrows, ncols, 2),
 *                                          dtype=np.asarray(d).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_d, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_9 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_9);
  } else {
    __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_8, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":1412
 *         double di, dj, dii, djj
 *         floating[:] tmp = np.zeros((2,), dtype=np.asarray(d).dtype)
 *         floating[:, :, :] out = np.zeros(shape=(nrows, ncols, 2),             # <<<<<<<<<<<<<<
 *                                          dtype=np.asarray(d).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_7); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_11 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_t_4);
  if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_out = __pyx_t_11;
  __pyx_t_11.memview = NULL;
  __pyx_t_11.data = NULL;

  /* "dipy/align/vector_fields.pyx":1415
 *                                          dtype=np.asarray(d).dtype)
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_9 = NULL;
  __pyx_t_12 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
      __pyx_t_12 = 1;
    }
  }
  __pyx_t_8 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  if (__pyx_t_9) {
    __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_9); __pyx_t_9 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_12, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_12, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_13 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_14 = ((!__pyx_t_13) != 0);
  if (__pyx_t_14) {

    /* "dipy/align/vector_fields.pyx":1416
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__73, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1415
 *                                          dtype=np.asarray(d).dtype)
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":1417
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_3 = NULL;
  __pyx_t_12 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
      __pyx_t_12 = 1;
    }
  }
  __pyx_t_9 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_12, __pyx_t_8);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_12, __pyx_int_2);
  __pyx_t_8 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_9, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_14 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_13 = ((!__pyx_t_14) != 0);
  if (__pyx_t_13) {

    /* "dipy/align/vector_fields.pyx":1418
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__74, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1417
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":1419
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_8 = NULL;
  __pyx_t_12 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
      __pyx_t_12 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_8) {
    __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_8); __pyx_t_8 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_12, __pyx_t_9);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_12, __pyx_int_2);
  __pyx_t_9 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_13 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_14 = ((!__pyx_t_13) != 0);
  if (__pyx_t_14) {

    /* "dipy/align/vector_fields.pyx":1420
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__75, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1419
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1422
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1424
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":1425
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":1427
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d[i, j, 0]
 *                     djj = d[i, j, 1]
 */
            __pyx_t_14 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_14) {

              /* "dipy/align/vector_fields.pyx":1428
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_19 * __pyx_v_d.strides[0]) ) + __pyx_t_20 * __pyx_v_d.strides[1]) ) + __pyx_t_21 * __pyx_v_d.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":1429
 *                 if affine_idx_in is None:
 *                     dii = d[i, j, 0]
 *                     djj = d[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_22 * __pyx_v_d.strides[0]) ) + __pyx_t_23 * __pyx_v_d.strides[1]) ) + __pyx_t_24 * __pyx_v_d.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":1427
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d[i, j, 0]
 *                     djj = d[i, j, 1]
 */
              goto __pyx_L13;
            }

            /* "dipy/align/vector_fields.pyx":1431
 *                     djj = d[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":1432
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":1433
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":1435
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":1436
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":1437
 *                     _interpolate_vector_2d[floating](d, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L13:;

            /* "dipy/align/vector_fields.pyx":1440
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_14 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_14) {

              /* "dipy/align/vector_fields.pyx":1441
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":1443
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":1440
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":1446
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":1447
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":1450
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     out[i, j, 0] = di + _apply_affine_2d_x0(i, j, 1,
 *                                                             affine_idx_out) - i
 */
            __pyx_t_14 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_14) {

              /* "dipy/align/vector_fields.pyx":1451
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     out[i, j, 0] = di + _apply_affine_2d_x0(i, j, 1,             # <<<<<<<<<<<<<<
 *                                                             affine_idx_out) - i
 *                     out[i, j, 1] = dj + _apply_affine_2d_x1(i, j, 1,
 */
              __pyx_t_27 = __pyx_v_i;
              __pyx_t_28 = __pyx_v_j;
              __pyx_t_29 = 0;
              *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_27 * __pyx_v_out.strides[0]) ) + __pyx_t_28 * __pyx_v_out.strides[1]) ) + __pyx_t_29 * __pyx_v_out.strides[2]) )) = ((__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out)) - __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":1453
 *                     out[i, j, 0] = di + _apply_affine_2d_x0(i, j, 1,
 *                                                             affine_idx_out) - i
 *                     out[i, j, 1] = dj + _apply_affine_2d_x1(i, j, 1,             # <<<<<<<<<<<<<<
 *                                                             affine_idx_out) - j
 *                 else:
 */
              __pyx_t_30 = __pyx_v_i;
              __pyx_t_31 = __pyx_v_j;
              __pyx_t_32 = 1;
              *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_30 * __pyx_v_out.strides[0]) ) + __pyx_t_31 * __pyx_v_out.strides[1]) ) + __pyx_t_32 * __pyx_v_out.strides[2]) )) = ((__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out)) - __pyx_v_j);

              /* "dipy/align/vector_fields.pyx":1450
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     out[i, j, 0] = di + _apply_affine_2d_x0(i, j, 1,
 *                                                             affine_idx_out) - i
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":1456
 *                                                             affine_idx_out) - j
 *                 else:
 *                     out[i, j, 0] = di             # <<<<<<<<<<<<<<
 *                     out[i, j, 1] = dj
 *     return out
 */
            /*else*/ {
              __pyx_t_33 = __pyx_v_i;
              __pyx_t_34 = __pyx_v_j;
              __pyx_t_35 = 0;
              *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_33 * __pyx_v_out.strides[0]) ) + __pyx_t_34 * __pyx_v_out.strides[1]) ) + __pyx_t_35 * __pyx_v_out.strides[2]) )) = __pyx_v_di;

              /* "dipy/align/vector_fields.pyx":1457
 *                 else:
 *                     out[i, j, 0] = di
 *                     out[i, j, 1] = dj             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
              __pyx_t_36 = __pyx_v_i;
              __pyx_t_37 = __pyx_v_j;
              __pyx_t_38 = 1;
              *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_36 * __pyx_v_out.strides[0]) ) + __pyx_t_37 * __pyx_v_out.strides[1]) ) + __pyx_t_38 * __pyx_v_out.strides[2]) )) = __pyx_v_dj;
            }
            __pyx_L15:;
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1422
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L8;
        }
        __pyx_L8:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1458
 *                     out[i, j, 0] = di
 *                     out[i, j, 1] = dj
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_out, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1458; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1345
 * 
 * 
 * def simplify_warp_function_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                               double[:, :] affine_idx_in,
 *                               double[:, :] affine_idx_out,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __PYX_XDEC_MEMVIEW(&__pyx_t_10, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_11, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.simplify_warp_function_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_165simplify_warp_function_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_165simplify_warp_function_2d = {"__pyx_fuse_1simplify_warp_function_2d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_165simplify_warp_function_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_22simplify_warp_function_2d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_165simplify_warp_function_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_d = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("simplify_warp_function_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_d,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simplify_warp_function_2d", 1, 5, 5, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simplify_warp_function_2d", 1, 5, 5, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simplify_warp_function_2d", 1, 5, 5, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simplify_warp_function_2d", 1, 5, 5, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "simplify_warp_function_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_d = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[0]); if (unlikely(!__pyx_v_d.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1346; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1347; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1348; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[4]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1349; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("simplify_warp_function_2d", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.simplify_warp_function_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_164simplify_warp_function_2d(__pyx_self, __pyx_v_d, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_164simplify_warp_function_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_memviewslice __pyx_t_10 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_11 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1simplify_warp_function_2d", 0);

  /* "dipy/align/vector_fields.pyx":1407
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = out_shape[1]
 *         cnp.npy_intp i, j
 */
  __pyx_t_1 = 0;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_1 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":1408
 *     cdef:
 *         cnp.npy_intp nrows = out_shape[0]
 *         cnp.npy_intp ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j
 *         double di, dj, dii, djj
 */
  __pyx_t_2 = 1;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_2 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":1411
 *         cnp.npy_intp i, j
 *         double di, dj, dii, djj
 *         floating[:] tmp = np.zeros((2,), dtype=np.asarray(d).dtype)             # <<<<<<<<<<<<<<
 *         floating[:, :, :] out = np.zeros(shape=(nrows, ncols, 2),
 *                                          dtype=np.asarray(d).dtype)
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_8 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
    }
  }
  if (!__pyx_t_8) {
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_5);
  } else {
    __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_9, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_dtype); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_tuple__77, __pyx_t_3); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_10 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_7);
  if (unlikely(!__pyx_t_10.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_tmp = __pyx_t_10;
  __pyx_t_10.memview = NULL;
  __pyx_t_10.data = NULL;

  /* "dipy/align/vector_fields.pyx":1412
 *         double di, dj, dii, djj
 *         floating[:] tmp = np.zeros((2,), dtype=np.asarray(d).dtype)
 *         floating[:, :, :] out = np.zeros(shape=(nrows, ncols, 2),             # <<<<<<<<<<<<<<
 *                                          dtype=np.asarray(d).dtype)
 * 
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_zeros); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyDict_New(); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_9 = PyTuple_New(3); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_shape, __pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "dipy/align/vector_fields.pyx":1413
 *         floating[:] tmp = np.zeros((2,), dtype=np.asarray(d).dtype)
 *         floating[:, :, :] out = np.zeros(shape=(nrows, ncols, 2),
 *                                          dtype=np.asarray(d).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_d, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_9 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_9);
  } else {
    __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_8, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":1412
 *         double di, dj, dii, djj
 *         floating[:] tmp = np.zeros((2,), dtype=np.asarray(d).dtype)
 *         floating[:, :, :] out = np.zeros(shape=(nrows, ncols, 2),             # <<<<<<<<<<<<<<
 *                                          dtype=np.asarray(d).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_7); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_11 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1412; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_out = __pyx_t_11;
  __pyx_t_11.memview = NULL;
  __pyx_t_11.data = NULL;

  /* "dipy/align/vector_fields.pyx":1415
 *                                          dtype=np.asarray(d).dtype)
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_9 = NULL;
  __pyx_t_12 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
      __pyx_t_12 = 1;
    }
  }
  __pyx_t_8 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  if (__pyx_t_9) {
    __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_9); __pyx_t_9 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_12, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_12, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_13 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_14 = ((!__pyx_t_13) != 0);
  if (__pyx_t_14) {

    /* "dipy/align/vector_fields.pyx":1416
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__78, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1415
 *                                          dtype=np.asarray(d).dtype)
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":1417
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_3 = NULL;
  __pyx_t_12 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
      __pyx_t_12 = 1;
    }
  }
  __pyx_t_9 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_12, __pyx_t_8);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_12, __pyx_int_2);
  __pyx_t_8 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_9, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_14 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_13 = ((!__pyx_t_14) != 0);
  if (__pyx_t_13) {

    /* "dipy/align/vector_fields.pyx":1418
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__79, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1417
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":1419
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_8 = NULL;
  __pyx_t_12 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
      __pyx_t_12 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_8) {
    __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_8); __pyx_t_8 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_12, __pyx_t_9);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_12, __pyx_int_2);
  __pyx_t_9 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_13 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_14 = ((!__pyx_t_13) != 0);
  if (__pyx_t_14) {

    /* "dipy/align/vector_fields.pyx":1420
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__80, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1419
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1422
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1424
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":1425
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":1427
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d[i, j, 0]
 *                     djj = d[i, j, 1]
 */
            __pyx_t_14 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_14) {

              /* "dipy/align/vector_fields.pyx":1428
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_19 * __pyx_v_d.strides[0]) ) + __pyx_t_20 * __pyx_v_d.strides[1]) ) + __pyx_t_21 * __pyx_v_d.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":1429
 *                 if affine_idx_in is None:
 *                     dii = d[i, j, 0]
 *                     djj = d[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_22 * __pyx_v_d.strides[0]) ) + __pyx_t_23 * __pyx_v_d.strides[1]) ) + __pyx_t_24 * __pyx_v_d.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":1427
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d[i, j, 0]
 *                     djj = d[i, j, 1]
 */
              goto __pyx_L13;
            }

            /* "dipy/align/vector_fields.pyx":1431
 *                     djj = d[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":1432
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":1433
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":1435
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":1436
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":1437
 *                     _interpolate_vector_2d[floating](d, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L13:;

            /* "dipy/align/vector_fields.pyx":1440
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_14 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_14) {

              /* "dipy/align/vector_fields.pyx":1441
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":1443
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":1440
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":1446
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":1447
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":1450
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     out[i, j, 0] = di + _apply_affine_2d_x0(i, j, 1,
 *                                                             affine_idx_out) - i
 */
            __pyx_t_14 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_14) {

              /* "dipy/align/vector_fields.pyx":1451
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     out[i, j, 0] = di + _apply_affine_2d_x0(i, j, 1,             # <<<<<<<<<<<<<<
 *                                                             affine_idx_out) - i
 *                     out[i, j, 1] = dj + _apply_affine_2d_x1(i, j, 1,
 */
              __pyx_t_27 = __pyx_v_i;
              __pyx_t_28 = __pyx_v_j;
              __pyx_t_29 = 0;
              *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_27 * __pyx_v_out.strides[0]) ) + __pyx_t_28 * __pyx_v_out.strides[1]) ) + __pyx_t_29 * __pyx_v_out.strides[2]) )) = ((__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out)) - __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":1453
 *                     out[i, j, 0] = di + _apply_affine_2d_x0(i, j, 1,
 *                                                             affine_idx_out) - i
 *                     out[i, j, 1] = dj + _apply_affine_2d_x1(i, j, 1,             # <<<<<<<<<<<<<<
 *                                                             affine_idx_out) - j
 *                 else:
 */
              __pyx_t_30 = __pyx_v_i;
              __pyx_t_31 = __pyx_v_j;
              __pyx_t_32 = 1;
              *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_30 * __pyx_v_out.strides[0]) ) + __pyx_t_31 * __pyx_v_out.strides[1]) ) + __pyx_t_32 * __pyx_v_out.strides[2]) )) = ((__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out)) - __pyx_v_j);

              /* "dipy/align/vector_fields.pyx":1450
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     out[i, j, 0] = di + _apply_affine_2d_x0(i, j, 1,
 *                                                             affine_idx_out) - i
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":1456
 *                                                             affine_idx_out) - j
 *                 else:
 *                     out[i, j, 0] = di             # <<<<<<<<<<<<<<
 *                     out[i, j, 1] = dj
 *     return out
 */
            /*else*/ {
              __pyx_t_33 = __pyx_v_i;
              __pyx_t_34 = __pyx_v_j;
              __pyx_t_35 = 0;
              *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_33 * __pyx_v_out.strides[0]) ) + __pyx_t_34 * __pyx_v_out.strides[1]) ) + __pyx_t_35 * __pyx_v_out.strides[2]) )) = __pyx_v_di;

              /* "dipy/align/vector_fields.pyx":1457
 *                 else:
 *                     out[i, j, 0] = di
 *                     out[i, j, 1] = dj             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
              __pyx_t_36 = __pyx_v_i;
              __pyx_t_37 = __pyx_v_j;
              __pyx_t_38 = 1;
              *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_36 * __pyx_v_out.strides[0]) ) + __pyx_t_37 * __pyx_v_out.strides[1]) ) + __pyx_t_38 * __pyx_v_out.strides[2]) )) = __pyx_v_dj;
            }
            __pyx_L15:;
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1422
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L8;
        }
        __pyx_L8:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1458
 *                     out[i, j, 0] = di
 *                     out[i, j, 1] = dj
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_out, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1458; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1345
 * 
 * 
 * def simplify_warp_function_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                               double[:, :] affine_idx_in,
 *                               double[:, :] affine_idx_out,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __PYX_XDEC_MEMVIEW(&__pyx_t_10, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_11, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.simplify_warp_function_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":1461
 * 
 * 
 * def simplify_warp_function_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                               double[:, :] affine_idx_in,
 *                               double[:, :] affine_idx_out,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_25simplify_warp_function_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_24simplify_warp_function_3d[] = "\n    Simplifies a nonlinear warping function combined with an affine transform\n\n    Modifies the given deformation field by incorporating into it a\n    an affine transformation and voxel-to-space transforms associated to\n    the discretization of its domain and codomain.\n    The resulting transformation may be regarded as operating on the\n    image spaces given by the domain and codomain discretization.\n    More precisely, the resulting transform is of the form:\n\n    (1) T[i] = W * d[U * i] + V * i\n\n    Where U = affine_idx_in, V = affine_idx_out, W = affine_disp.\n    Both the direct and inverse transforms of a DiffeomorphicMap can be written\n    in this form:\n\n    Direct:  Let D be the voxel-to-space transform of the domain's\n             discretization, P be the pre-align matrix, Rinv the space-to-voxel\n             transform of the reference grid (the grid the displacement field\n             is defined on) and Cinv be the space-to-voxel transform of the\n             codomain's discretization. Then, for each i in the domain's grid,\n             the direct transform is given by\n\n             (2) T[i] = Cinv * d[Rinv * P * D * i] + Cinv * P * D * i\n\n             and we identify U = Rinv * P * D, V = Cinv * P * D, W = Cinv\n\n    Inverse: Let C be the voxel-to-space transform of the codomain's\n             discretization, Pinv be the inverse of the pre-align matrix, Rinv\n             the space-to-voxel transform of the reference grid (the grid the\n             displacement field is defined on) and Dinv be the space-to-voxel\n             transform of the domain's discretization. Then, for each j in the\n             codomain's grid, the inverse transform is given by\n\n             (3) Tinv[j] = Dinv * Pinv * d[Rinv * C * j] + Dinv * Pinv * C * j\n\n             and we identify U = Rinv * C, V = Dinv * Pinv * C, W = Dinv * Pinv\n\n    Parameters\n    ----------\n    d : array, shape (S', R', C', 3)\n        the non-linear part of the trans""formation (displacement field)\n    affine_idx_in : array, shape (4, 4)\n        the matrix U in eq. (1) above\n    affine_idx_out : array, shape (4, 4)\n        the matrix V in eq. (1) above\n    affine_disp : array, shape (4, 4)\n        the matrix W in eq. (1) above\n    out_shape : array, shape (3,)\n        the number of slices, rows and columns of the sampling grid\n\n    Returns\n    -------\n    out : array, shape = out_shape\n        the simplified transformation given by one single displacement field\n\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_25simplify_warp_function_3d = {"simplify_warp_function_3d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_25simplify_warp_function_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_24simplify_warp_function_3d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_25simplify_warp_function_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_24simplify_warp_function_3d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_24simplify_warp_function_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("simplify_warp_function_3d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_d, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_d); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 4) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 4) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__81, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__82, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__83, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__84, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_169simplify_warp_function_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_169simplify_warp_function_3d = {"__pyx_fuse_0simplify_warp_function_3d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_169simplify_warp_function_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_24simplify_warp_function_3d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_169simplify_warp_function_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_d = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("simplify_warp_function_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_d,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simplify_warp_function_3d", 1, 5, 5, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simplify_warp_function_3d", 1, 5, 5, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simplify_warp_function_3d", 1, 5, 5, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simplify_warp_function_3d", 1, 5, 5, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "simplify_warp_function_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_d = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[0]); if (unlikely(!__pyx_v_d.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1462; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1463; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1464; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[4]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1465; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("simplify_warp_function_3d", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.simplify_warp_function_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_168simplify_warp_function_3d(__pyx_self, __pyx_v_d, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_168simplify_warp_function_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED npy_intp __pyx_v_inside;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dkk;
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_memviewslice __pyx_t_11 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_12 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  Py_ssize_t __pyx_t_60;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0simplify_warp_function_3d", 0);

  /* "dipy/align/vector_fields.pyx":1524
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = out_shape[1]
 *         cnp.npy_intp ncols = out_shape[2]
 */
  __pyx_t_1 = 0;
  __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_1 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":1525
 *     cdef:
 *         cnp.npy_intp nslices = out_shape[0]
 *         cnp.npy_intp nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = out_shape[2]
 *         cnp.npy_intp i, j, k, inside
 */
  __pyx_t_2 = 1;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_2 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":1526
 *         cnp.npy_intp nslices = out_shape[0]
 *         cnp.npy_intp nrows = out_shape[1]
 *         cnp.npy_intp ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k, inside
 *         double di, dj, dk, dii, djj, dkk
 */
  __pyx_t_3 = 2;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_3 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":1529
 *         cnp.npy_intp i, j, k, inside
 *         double di, dj, dk, dii, djj, dkk
 *         floating[:] tmp = np.zeros((3,), dtype=np.asarray(d).dtype)             # <<<<<<<<<<<<<<
 *         floating[:, :, :, :] out = np.zeros(shape=(nslices, nrows, ncols, 3),
 *                                             dtype=np.asarray(d).dtype)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_asarray); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_d, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
    }
  }
  if (!__pyx_t_9) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_10, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__86, __pyx_t_4); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_11 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_8);
  if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_tmp = __pyx_t_11;
  __pyx_t_11.memview = NULL;
  __pyx_t_11.data = NULL;

  /* "dipy/align/vector_fields.pyx":1530
 *         double di, dj, dk, dii, djj, dkk
 *         floating[:] tmp = np.zeros((3,), dtype=np.asarray(d).dtype)
 *         floating[:, :, :, :] out = np.zeros(shape=(nslices, nrows, ncols, 3),             # <<<<<<<<<<<<<<
 *                                             dtype=np.asarray(d).dtype)
 * 
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = PyDict_New(); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_10 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_7 = PyTuple_New(4); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_t_10);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_7, 3, __pyx_int_3);
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_shape, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "dipy/align/vector_fields.pyx":1531
 *         floating[:] tmp = np.zeros((3,), dtype=np.asarray(d).dtype)
 *         floating[:, :, :, :] out = np.zeros(shape=(nslices, nrows, ncols, 3),
 *                                             dtype=np.asarray(d).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 */
  __pyx_t_10 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_asarray); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __pyx_memoryview_fromslice(__pyx_v_d, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_5 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  if (!__pyx_t_5) {
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_10); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_7);
  } else {
    __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_5); __pyx_t_5 = NULL;
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_10);
    __pyx_t_10 = 0;
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_dtype); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_dtype, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":1530
 *         double di, dj, dk, dii, djj, dkk
 *         floating[:] tmp = np.zeros((3,), dtype=np.asarray(d).dtype)
 *         floating[:, :, :, :] out = np.zeros(shape=(nslices, nrows, ncols, 3),             # <<<<<<<<<<<<<<
 *                                             dtype=np.asarray(d).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_8); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_12 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(__pyx_t_6);
  if (unlikely(!__pyx_t_12.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_out = __pyx_t_12;
  __pyx_t_12.memview = NULL;
  __pyx_t_12.data = NULL;

  /* "dipy/align/vector_fields.pyx":1533
 *                                             dtype=np.asarray(d).dtype)
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = NULL;
  __pyx_t_13 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_13 = 1;
    }
  }
  __pyx_t_9 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  if (__pyx_t_7) {
    __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_13, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_13, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_9, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_14 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_15 = ((!__pyx_t_14) != 0);
  if (__pyx_t_15) {

    /* "dipy/align/vector_fields.pyx":1534
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__87, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1534; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1534; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1533
 *                                             dtype=np.asarray(d).dtype)
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":1535
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1535; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1535; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_4 = NULL;
  __pyx_t_13 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_13 = 1;
    }
  }
  __pyx_t_7 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1535; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_13, __pyx_t_9);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_13, __pyx_int_3);
  __pyx_t_9 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_7, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1535; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_15 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_15 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1535; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_14 = ((!__pyx_t_15) != 0);
  if (__pyx_t_14) {

    /* "dipy/align/vector_fields.pyx":1536
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__88, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1536; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1536; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1535
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":1537
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1537; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1537; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = NULL;
  __pyx_t_13 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_13 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1537; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_9) {
    __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_9); __pyx_t_9 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_13, __pyx_t_7);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_13, __pyx_int_3);
  __pyx_t_7 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_4, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1537; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_14 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1537; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_15 = ((!__pyx_t_14) != 0);
  if (__pyx_t_15) {

    /* "dipy/align/vector_fields.pyx":1538
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__89, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1538; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1538; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1537
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1540
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1542
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":1543
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":1544
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":1545
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d[k, i, j, 0]
 *                         dii = d[k, i, j, 1]
 */
              __pyx_t_15 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_15) {

                /* "dipy/align/vector_fields.pyx":1546
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d[k, i, j, 1]
 *                         djj = d[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_22 * __pyx_v_d.strides[0]) ) + __pyx_t_23 * __pyx_v_d.strides[1]) ) + __pyx_t_24 * __pyx_v_d.strides[2]) ) + __pyx_t_25 * __pyx_v_d.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":1547
 *                     if affine_idx_in is None:
 *                         dkk = d[k, i, j, 0]
 *                         dii = d[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_26 * __pyx_v_d.strides[0]) ) + __pyx_t_27 * __pyx_v_d.strides[1]) ) + __pyx_t_28 * __pyx_v_d.strides[2]) ) + __pyx_t_29 * __pyx_v_d.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":1548
 *                         dkk = d[k, i, j, 0]
 *                         dii = d[k, i, j, 1]
 *                         djj = d[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_30 * __pyx_v_d.strides[0]) ) + __pyx_t_31 * __pyx_v_d.strides[1]) ) + __pyx_t_32 * __pyx_v_d.strides[2]) ) + __pyx_t_33 * __pyx_v_d.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":1545
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d[k, i, j, 0]
 *                         dii = d[k, i, j, 1]
 */
                goto __pyx_L15;
              }

              /* "dipy/align/vector_fields.pyx":1550
 *                         djj = d[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":1551
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":1552
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":1554
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":1556
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":1558
 *                         inside = _interpolate_vector_3d[floating](d, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":1559
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":1560
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L15:;

              /* "dipy/align/vector_fields.pyx":1562
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_15 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_15) {

                /* "dipy/align/vector_fields.pyx":1563
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":1565
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":1567
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":1562
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":1570
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":1571
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":1572
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":1574
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         out[k, i, j, 0] = dk +\
 *                             _apply_affine_3d_x0(k, i, j, 1, affine_idx_out) - k
 */
              __pyx_t_15 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_15) {

                /* "dipy/align/vector_fields.pyx":1575
 * 
 *                     if affine_idx_out is not None:
 *                         out[k, i, j, 0] = dk +\             # <<<<<<<<<<<<<<
 *                             _apply_affine_3d_x0(k, i, j, 1, affine_idx_out) - k
 *                         out[k, i, j, 1] = di +\
 */
                __pyx_t_37 = __pyx_v_k;
                __pyx_t_38 = __pyx_v_i;
                __pyx_t_39 = __pyx_v_j;
                __pyx_t_40 = 0;
                *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_37 * __pyx_v_out.strides[0]) ) + __pyx_t_38 * __pyx_v_out.strides[1]) ) + __pyx_t_39 * __pyx_v_out.strides[2]) ) + __pyx_t_40 * __pyx_v_out.strides[3]) )) = ((__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out)) - __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":1577
 *                         out[k, i, j, 0] = dk +\
 *                             _apply_affine_3d_x0(k, i, j, 1, affine_idx_out) - k
 *                         out[k, i, j, 1] = di +\             # <<<<<<<<<<<<<<
 *                             _apply_affine_3d_x1(k, i, j, 1, affine_idx_out) - i
 *                         out[k, i, j, 2] = dj +\
 */
                __pyx_t_41 = __pyx_v_k;
                __pyx_t_42 = __pyx_v_i;
                __pyx_t_43 = __pyx_v_j;
                __pyx_t_44 = 1;
                *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_41 * __pyx_v_out.strides[0]) ) + __pyx_t_42 * __pyx_v_out.strides[1]) ) + __pyx_t_43 * __pyx_v_out.strides[2]) ) + __pyx_t_44 * __pyx_v_out.strides[3]) )) = ((__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out)) - __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":1579
 *                         out[k, i, j, 1] = di +\
 *                             _apply_affine_3d_x1(k, i, j, 1, affine_idx_out) - i
 *                         out[k, i, j, 2] = dj +\             # <<<<<<<<<<<<<<
 *                             _apply_affine_3d_x2(k, i, j, 1, affine_idx_out) - j
 *                     else:
 */
                __pyx_t_45 = __pyx_v_k;
                __pyx_t_46 = __pyx_v_i;
                __pyx_t_47 = __pyx_v_j;
                __pyx_t_48 = 2;
                *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_45 * __pyx_v_out.strides[0]) ) + __pyx_t_46 * __pyx_v_out.strides[1]) ) + __pyx_t_47 * __pyx_v_out.strides[2]) ) + __pyx_t_48 * __pyx_v_out.strides[3]) )) = ((__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out)) - __pyx_v_j);

                /* "dipy/align/vector_fields.pyx":1574
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         out[k, i, j, 0] = dk +\
 *                             _apply_affine_3d_x0(k, i, j, 1, affine_idx_out) - k
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":1582
 *                             _apply_affine_3d_x2(k, i, j, 1, affine_idx_out) - j
 *                     else:
 *                         out[k, i, j, 0] = dk             # <<<<<<<<<<<<<<
 *                         out[k, i, j, 1] = di
 *                         out[k, i, j, 2] = dj
 */
              /*else*/ {
                __pyx_t_49 = __pyx_v_k;
                __pyx_t_50 = __pyx_v_i;
                __pyx_t_51 = __pyx_v_j;
                __pyx_t_52 = 0;
                *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_49 * __pyx_v_out.strides[0]) ) + __pyx_t_50 * __pyx_v_out.strides[1]) ) + __pyx_t_51 * __pyx_v_out.strides[2]) ) + __pyx_t_52 * __pyx_v_out.strides[3]) )) = __pyx_v_dk;

                /* "dipy/align/vector_fields.pyx":1583
 *                     else:
 *                         out[k, i, j, 0] = dk
 *                         out[k, i, j, 1] = di             # <<<<<<<<<<<<<<
 *                         out[k, i, j, 2] = dj
 *     return out
 */
                __pyx_t_53 = __pyx_v_k;
                __pyx_t_54 = __pyx_v_i;
                __pyx_t_55 = __pyx_v_j;
                __pyx_t_56 = 1;
                *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_53 * __pyx_v_out.strides[0]) ) + __pyx_t_54 * __pyx_v_out.strides[1]) ) + __pyx_t_55 * __pyx_v_out.strides[2]) ) + __pyx_t_56 * __pyx_v_out.strides[3]) )) = __pyx_v_di;

                /* "dipy/align/vector_fields.pyx":1584
 *                         out[k, i, j, 0] = dk
 *                         out[k, i, j, 1] = di
 *                         out[k, i, j, 2] = dj             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
                __pyx_t_57 = __pyx_v_k;
                __pyx_t_58 = __pyx_v_i;
                __pyx_t_59 = __pyx_v_j;
                __pyx_t_60 = 2;
                *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_57 * __pyx_v_out.strides[0]) ) + __pyx_t_58 * __pyx_v_out.strides[1]) ) + __pyx_t_59 * __pyx_v_out.strides[2]) ) + __pyx_t_60 * __pyx_v_out.strides[3]) )) = __pyx_v_dj;
              }
              __pyx_L17:;
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1540
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L8;
        }
        __pyx_L8:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1585
 *                         out[k, i, j, 1] = di
 *                         out[k, i, j, 2] = dj
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_out, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1585; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1461
 * 
 * 
 * def simplify_warp_function_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                               double[:, :] affine_idx_in,
 *                               double[:, :] affine_idx_out,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __PYX_XDEC_MEMVIEW(&__pyx_t_11, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_12, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.simplify_warp_function_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_171simplify_warp_function_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_171simplify_warp_function_3d = {"__pyx_fuse_1simplify_warp_function_3d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_171simplify_warp_function_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_24simplify_warp_function_3d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_171simplify_warp_function_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_d = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("simplify_warp_function_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_d,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simplify_warp_function_3d", 1, 5, 5, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simplify_warp_function_3d", 1, 5, 5, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simplify_warp_function_3d", 1, 5, 5, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("simplify_warp_function_3d", 1, 5, 5, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "simplify_warp_function_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_d = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[0]); if (unlikely(!__pyx_v_d.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1462; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1463; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1464; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[4]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1465; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("simplify_warp_function_3d", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.simplify_warp_function_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_170simplify_warp_function_3d(__pyx_self, __pyx_v_d, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_170simplify_warp_function_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED npy_intp __pyx_v_inside;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dkk;
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_memviewslice __pyx_t_11 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_12 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  Py_ssize_t __pyx_t_60;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1simplify_warp_function_3d", 0);

  /* "dipy/align/vector_fields.pyx":1524
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = out_shape[1]
 *         cnp.npy_intp ncols = out_shape[2]
 */
  __pyx_t_1 = 0;
  __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_1 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":1525
 *     cdef:
 *         cnp.npy_intp nslices = out_shape[0]
 *         cnp.npy_intp nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = out_shape[2]
 *         cnp.npy_intp i, j, k, inside
 */
  __pyx_t_2 = 1;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_2 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":1526
 *         cnp.npy_intp nslices = out_shape[0]
 *         cnp.npy_intp nrows = out_shape[1]
 *         cnp.npy_intp ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k, inside
 *         double di, dj, dk, dii, djj, dkk
 */
  __pyx_t_3 = 2;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_3 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":1529
 *         cnp.npy_intp i, j, k, inside
 *         double di, dj, dk, dii, djj, dkk
 *         floating[:] tmp = np.zeros((3,), dtype=np.asarray(d).dtype)             # <<<<<<<<<<<<<<
 *         floating[:, :, :, :] out = np.zeros(shape=(nslices, nrows, ncols, 3),
 *                                             dtype=np.asarray(d).dtype)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_asarray); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_d, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
    }
  }
  if (!__pyx_t_9) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_10, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__91, __pyx_t_4); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_11 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_8);
  if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_tmp = __pyx_t_11;
  __pyx_t_11.memview = NULL;
  __pyx_t_11.data = NULL;

  /* "dipy/align/vector_fields.pyx":1530
 *         double di, dj, dk, dii, djj, dkk
 *         floating[:] tmp = np.zeros((3,), dtype=np.asarray(d).dtype)
 *         floating[:, :, :, :] out = np.zeros(shape=(nslices, nrows, ncols, 3),             # <<<<<<<<<<<<<<
 *                                             dtype=np.asarray(d).dtype)
 * 
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = PyDict_New(); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_10 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_7 = PyTuple_New(4); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_10);
  PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_t_10);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_7, 3, __pyx_int_3);
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_10 = 0;
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_shape, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "dipy/align/vector_fields.pyx":1531
 *         floating[:] tmp = np.zeros((3,), dtype=np.asarray(d).dtype)
 *         floating[:, :, :, :] out = np.zeros(shape=(nslices, nrows, ncols, 3),
 *                                             dtype=np.asarray(d).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 */
  __pyx_t_10 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_asarray); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __pyx_memoryview_fromslice(__pyx_v_d, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_5 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  if (!__pyx_t_5) {
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_10); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_7);
  } else {
    __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_5); __pyx_t_5 = NULL;
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_10);
    __pyx_t_10 = 0;
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_dtype); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_dtype, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":1530
 *         double di, dj, dk, dii, djj, dkk
 *         floating[:] tmp = np.zeros((3,), dtype=np.asarray(d).dtype)
 *         floating[:, :, :, :] out = np.zeros(shape=(nslices, nrows, ncols, 3),             # <<<<<<<<<<<<<<
 *                                             dtype=np.asarray(d).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_8); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_12 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_t_6);
  if (unlikely(!__pyx_t_12.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1530; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_out = __pyx_t_12;
  __pyx_t_12.memview = NULL;
  __pyx_t_12.data = NULL;

  /* "dipy/align/vector_fields.pyx":1533
 *                                             dtype=np.asarray(d).dtype)
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = NULL;
  __pyx_t_13 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_13 = 1;
    }
  }
  __pyx_t_9 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  if (__pyx_t_7) {
    __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_13, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_13, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_9, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_14 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_15 = ((!__pyx_t_14) != 0);
  if (__pyx_t_15) {

    /* "dipy/align/vector_fields.pyx":1534
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__92, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1534; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1534; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1533
 *                                             dtype=np.asarray(d).dtype)
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":1535
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1535; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1535; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_4 = NULL;
  __pyx_t_13 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_13 = 1;
    }
  }
  __pyx_t_7 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1535; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_13, __pyx_t_9);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_13, __pyx_int_3);
  __pyx_t_9 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_7, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1535; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_15 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_15 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1535; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_14 = ((!__pyx_t_15) != 0);
  if (__pyx_t_14) {

    /* "dipy/align/vector_fields.pyx":1536
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__93, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1536; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1536; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1535
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":1537
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1537; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1537; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = NULL;
  __pyx_t_13 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_13 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1537; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_9) {
    __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_9); __pyx_t_9 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_13, __pyx_t_7);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_13, __pyx_int_3);
  __pyx_t_7 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_4, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1537; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_14 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1537; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_15 = ((!__pyx_t_14) != 0);
  if (__pyx_t_15) {

    /* "dipy/align/vector_fields.pyx":1538
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__94, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1538; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1538; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1537
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1540
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1542
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":1543
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":1544
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":1545
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d[k, i, j, 0]
 *                         dii = d[k, i, j, 1]
 */
              __pyx_t_15 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_15) {

                /* "dipy/align/vector_fields.pyx":1546
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d[k, i, j, 1]
 *                         djj = d[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_22 * __pyx_v_d.strides[0]) ) + __pyx_t_23 * __pyx_v_d.strides[1]) ) + __pyx_t_24 * __pyx_v_d.strides[2]) ) + __pyx_t_25 * __pyx_v_d.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":1547
 *                     if affine_idx_in is None:
 *                         dkk = d[k, i, j, 0]
 *                         dii = d[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_26 * __pyx_v_d.strides[0]) ) + __pyx_t_27 * __pyx_v_d.strides[1]) ) + __pyx_t_28 * __pyx_v_d.strides[2]) ) + __pyx_t_29 * __pyx_v_d.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":1548
 *                         dkk = d[k, i, j, 0]
 *                         dii = d[k, i, j, 1]
 *                         djj = d[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_30 * __pyx_v_d.strides[0]) ) + __pyx_t_31 * __pyx_v_d.strides[1]) ) + __pyx_t_32 * __pyx_v_d.strides[2]) ) + __pyx_t_33 * __pyx_v_d.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":1545
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d[k, i, j, 0]
 *                         dii = d[k, i, j, 1]
 */
                goto __pyx_L15;
              }

              /* "dipy/align/vector_fields.pyx":1550
 *                         djj = d[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":1551
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":1552
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":1554
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":1556
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":1558
 *                         inside = _interpolate_vector_3d[floating](d, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":1559
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":1560
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L15:;

              /* "dipy/align/vector_fields.pyx":1562
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_15 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_15) {

                /* "dipy/align/vector_fields.pyx":1563
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":1565
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":1567
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":1562
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":1570
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":1571
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":1572
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":1574
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         out[k, i, j, 0] = dk +\
 *                             _apply_affine_3d_x0(k, i, j, 1, affine_idx_out) - k
 */
              __pyx_t_15 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_15) {

                /* "dipy/align/vector_fields.pyx":1575
 * 
 *                     if affine_idx_out is not None:
 *                         out[k, i, j, 0] = dk +\             # <<<<<<<<<<<<<<
 *                             _apply_affine_3d_x0(k, i, j, 1, affine_idx_out) - k
 *                         out[k, i, j, 1] = di +\
 */
                __pyx_t_37 = __pyx_v_k;
                __pyx_t_38 = __pyx_v_i;
                __pyx_t_39 = __pyx_v_j;
                __pyx_t_40 = 0;
                *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_37 * __pyx_v_out.strides[0]) ) + __pyx_t_38 * __pyx_v_out.strides[1]) ) + __pyx_t_39 * __pyx_v_out.strides[2]) ) + __pyx_t_40 * __pyx_v_out.strides[3]) )) = ((__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out)) - __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":1577
 *                         out[k, i, j, 0] = dk +\
 *                             _apply_affine_3d_x0(k, i, j, 1, affine_idx_out) - k
 *                         out[k, i, j, 1] = di +\             # <<<<<<<<<<<<<<
 *                             _apply_affine_3d_x1(k, i, j, 1, affine_idx_out) - i
 *                         out[k, i, j, 2] = dj +\
 */
                __pyx_t_41 = __pyx_v_k;
                __pyx_t_42 = __pyx_v_i;
                __pyx_t_43 = __pyx_v_j;
                __pyx_t_44 = 1;
                *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_41 * __pyx_v_out.strides[0]) ) + __pyx_t_42 * __pyx_v_out.strides[1]) ) + __pyx_t_43 * __pyx_v_out.strides[2]) ) + __pyx_t_44 * __pyx_v_out.strides[3]) )) = ((__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out)) - __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":1579
 *                         out[k, i, j, 1] = di +\
 *                             _apply_affine_3d_x1(k, i, j, 1, affine_idx_out) - i
 *                         out[k, i, j, 2] = dj +\             # <<<<<<<<<<<<<<
 *                             _apply_affine_3d_x2(k, i, j, 1, affine_idx_out) - j
 *                     else:
 */
                __pyx_t_45 = __pyx_v_k;
                __pyx_t_46 = __pyx_v_i;
                __pyx_t_47 = __pyx_v_j;
                __pyx_t_48 = 2;
                *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_45 * __pyx_v_out.strides[0]) ) + __pyx_t_46 * __pyx_v_out.strides[1]) ) + __pyx_t_47 * __pyx_v_out.strides[2]) ) + __pyx_t_48 * __pyx_v_out.strides[3]) )) = ((__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out)) - __pyx_v_j);

                /* "dipy/align/vector_fields.pyx":1574
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         out[k, i, j, 0] = dk +\
 *                             _apply_affine_3d_x0(k, i, j, 1, affine_idx_out) - k
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":1582
 *                             _apply_affine_3d_x2(k, i, j, 1, affine_idx_out) - j
 *                     else:
 *                         out[k, i, j, 0] = dk             # <<<<<<<<<<<<<<
 *                         out[k, i, j, 1] = di
 *                         out[k, i, j, 2] = dj
 */
              /*else*/ {
                __pyx_t_49 = __pyx_v_k;
                __pyx_t_50 = __pyx_v_i;
                __pyx_t_51 = __pyx_v_j;
                __pyx_t_52 = 0;
                *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_49 * __pyx_v_out.strides[0]) ) + __pyx_t_50 * __pyx_v_out.strides[1]) ) + __pyx_t_51 * __pyx_v_out.strides[2]) ) + __pyx_t_52 * __pyx_v_out.strides[3]) )) = __pyx_v_dk;

                /* "dipy/align/vector_fields.pyx":1583
 *                     else:
 *                         out[k, i, j, 0] = dk
 *                         out[k, i, j, 1] = di             # <<<<<<<<<<<<<<
 *                         out[k, i, j, 2] = dj
 *     return out
 */
                __pyx_t_53 = __pyx_v_k;
                __pyx_t_54 = __pyx_v_i;
                __pyx_t_55 = __pyx_v_j;
                __pyx_t_56 = 1;
                *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_53 * __pyx_v_out.strides[0]) ) + __pyx_t_54 * __pyx_v_out.strides[1]) ) + __pyx_t_55 * __pyx_v_out.strides[2]) ) + __pyx_t_56 * __pyx_v_out.strides[3]) )) = __pyx_v_di;

                /* "dipy/align/vector_fields.pyx":1584
 *                         out[k, i, j, 0] = dk
 *                         out[k, i, j, 1] = di
 *                         out[k, i, j, 2] = dj             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
                __pyx_t_57 = __pyx_v_k;
                __pyx_t_58 = __pyx_v_i;
                __pyx_t_59 = __pyx_v_j;
                __pyx_t_60 = 2;
                *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_57 * __pyx_v_out.strides[0]) ) + __pyx_t_58 * __pyx_v_out.strides[1]) ) + __pyx_t_59 * __pyx_v_out.strides[2]) ) + __pyx_t_60 * __pyx_v_out.strides[3]) )) = __pyx_v_dj;
              }
              __pyx_L17:;
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1540
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L8;
        }
        __pyx_L8:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1585
 *                         out[k, i, j, 1] = di
 *                         out[k, i, j, 2] = dj
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_out, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1585; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1461
 * 
 * 
 * def simplify_warp_function_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                               double[:, :] affine_idx_in,
 *                               double[:, :] affine_idx_out,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __PYX_XDEC_MEMVIEW(&__pyx_t_11, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_12, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.simplify_warp_function_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":1588
 * 
 * 
 * def reorient_vector_field_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                              double[:, :] affine):
 *     r"""Linearly transforms all vectors of a 2D displacement field
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_27reorient_vector_field_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_26reorient_vector_field_2d[] = "Linearly transforms all vectors of a 2D displacement field\n\n    Modifies the input displacement field by multiplying each displacement\n    vector by the given matrix. Note that the elements of the displacement\n    field are vectors, not points, so their last homogeneous coordinate is\n    zero, not one, and therefore the translation component of the affine\n    transform will not have any effect on them.\n\n    Parameters\n    ----------\n    d : array, shape (R, C, 2)\n        the displacement field to be re-oriented\n    affine: array, shape (3, 3)\n        the matrix to be applied\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_27reorient_vector_field_2d = {"reorient_vector_field_2d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_27reorient_vector_field_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_26reorient_vector_field_2d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_27reorient_vector_field_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_26reorient_vector_field_2d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_26reorient_vector_field_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("reorient_vector_field_2d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_d, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_d); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__95, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__96, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__97, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__98, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_175reorient_vector_field_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_175reorient_vector_field_2d = {"__pyx_fuse_0reorient_vector_field_2d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_175reorient_vector_field_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_26reorient_vector_field_2d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_175reorient_vector_field_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_d = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("reorient_vector_field_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_d,&__pyx_n_s_affine,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("reorient_vector_field_2d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "reorient_vector_field_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_d = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[0]); if (unlikely(!__pyx_v_d.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1589; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("reorient_vector_field_2d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.reorient_vector_field_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_174reorient_vector_field_2d(__pyx_self, __pyx_v_d, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_174reorient_vector_field_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  npy_intp __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0reorient_vector_field_2d", 0);

  /* "dipy/align/vector_fields.pyx":1606
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = d.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = d.shape[1]
 *         cnp.npy_intp i, j
 */
  __pyx_v_nrows = (__pyx_v_d.shape[0]);

  /* "dipy/align/vector_fields.pyx":1607
 *     cdef:
 *         cnp.npy_intp nrows = d.shape[0]
 *         cnp.npy_intp ncols = d.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j
 *         double di, dj
 */
  __pyx_v_ncols = (__pyx_v_d.shape[1]);

  /* "dipy/align/vector_fields.pyx":1611
 *         double di, dj
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1611; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1611; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1611; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1611; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1611; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":1612
 * 
 *     if not is_valid_affine(affine, 2):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     if affine is None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__99, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1611
 *         double di, dj
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1614
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     if affine is None:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_affine.memview) == Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":1615
 * 
 *     if affine is None:
 *         return             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":1614
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     if affine is None:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1617
 *         return
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(nrows):
 *             for j in range(ncols):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1618
 * 
 *     with nogil:
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 di = d[i, j, 0]
 */
        __pyx_t_9 = __pyx_v_nrows;
        for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
          __pyx_v_i = __pyx_t_10;

          /* "dipy/align/vector_fields.pyx":1619
 *     with nogil:
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 di = d[i, j, 0]
 *                 dj = d[i, j, 1]
 */
          __pyx_t_11 = __pyx_v_ncols;
          for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
            __pyx_v_j = __pyx_t_12;

            /* "dipy/align/vector_fields.pyx":1620
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 di = d[i, j, 0]             # <<<<<<<<<<<<<<
 *                 dj = d[i, j, 1]
 *                 d[i, j, 0] = _apply_affine_2d_x0(di, dj, 0, affine)
 */
            __pyx_t_13 = __pyx_v_i;
            __pyx_t_14 = __pyx_v_j;
            __pyx_t_15 = 0;
            __pyx_v_di = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_13 * __pyx_v_d.strides[0]) ) + __pyx_t_14 * __pyx_v_d.strides[1]) ) + __pyx_t_15 * __pyx_v_d.strides[2]) )));

            /* "dipy/align/vector_fields.pyx":1621
 *             for j in range(ncols):
 *                 di = d[i, j, 0]
 *                 dj = d[i, j, 1]             # <<<<<<<<<<<<<<
 *                 d[i, j, 0] = _apply_affine_2d_x0(di, dj, 0, affine)
 *                 d[i, j, 1] = _apply_affine_2d_x1(di, dj, 0, affine)
 */
            __pyx_t_16 = __pyx_v_i;
            __pyx_t_17 = __pyx_v_j;
            __pyx_t_18 = 1;
            __pyx_v_dj = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_16 * __pyx_v_d.strides[0]) ) + __pyx_t_17 * __pyx_v_d.strides[1]) ) + __pyx_t_18 * __pyx_v_d.strides[2]) )));

            /* "dipy/align/vector_fields.pyx":1622
 *                 di = d[i, j, 0]
 *                 dj = d[i, j, 1]
 *                 d[i, j, 0] = _apply_affine_2d_x0(di, dj, 0, affine)             # <<<<<<<<<<<<<<
 *                 d[i, j, 1] = _apply_affine_2d_x1(di, dj, 0, affine)
 * 
 */
            __pyx_t_19 = __pyx_v_i;
            __pyx_t_20 = __pyx_v_j;
            __pyx_t_21 = 0;
            *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_19 * __pyx_v_d.strides[0]) ) + __pyx_t_20 * __pyx_v_d.strides[1]) ) + __pyx_t_21 * __pyx_v_d.strides[2]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_di, __pyx_v_dj, 0.0, __pyx_v_affine);

            /* "dipy/align/vector_fields.pyx":1623
 *                 dj = d[i, j, 1]
 *                 d[i, j, 0] = _apply_affine_2d_x0(di, dj, 0, affine)
 *                 d[i, j, 1] = _apply_affine_2d_x1(di, dj, 0, affine)             # <<<<<<<<<<<<<<
 * 
 * 
 */
            __pyx_t_22 = __pyx_v_i;
            __pyx_t_23 = __pyx_v_j;
            __pyx_t_24 = 1;
            *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_22 * __pyx_v_d.strides[0]) ) + __pyx_t_23 * __pyx_v_d.strides[1]) ) + __pyx_t_24 * __pyx_v_d.strides[2]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_di, __pyx_v_dj, 0.0, __pyx_v_affine);
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1617
 *         return
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(nrows):
 *             for j in range(ncols):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1588
 * 
 * 
 * def reorient_vector_field_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                              double[:, :] affine):
 *     r"""Linearly transforms all vectors of a 2D displacement field
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("dipy.align.vector_fields.reorient_vector_field_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_d, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_177reorient_vector_field_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_177reorient_vector_field_2d = {"__pyx_fuse_1reorient_vector_field_2d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_177reorient_vector_field_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_26reorient_vector_field_2d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_177reorient_vector_field_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_d = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("reorient_vector_field_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_d,&__pyx_n_s_affine,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("reorient_vector_field_2d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "reorient_vector_field_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_d = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[0]); if (unlikely(!__pyx_v_d.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1589; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("reorient_vector_field_2d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.reorient_vector_field_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_176reorient_vector_field_2d(__pyx_self, __pyx_v_d, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_176reorient_vector_field_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  npy_intp __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1reorient_vector_field_2d", 0);

  /* "dipy/align/vector_fields.pyx":1606
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = d.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = d.shape[1]
 *         cnp.npy_intp i, j
 */
  __pyx_v_nrows = (__pyx_v_d.shape[0]);

  /* "dipy/align/vector_fields.pyx":1607
 *     cdef:
 *         cnp.npy_intp nrows = d.shape[0]
 *         cnp.npy_intp ncols = d.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j
 *         double di, dj
 */
  __pyx_v_ncols = (__pyx_v_d.shape[1]);

  /* "dipy/align/vector_fields.pyx":1611
 *         double di, dj
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1611; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1611; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1611; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1611; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1611; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":1612
 * 
 *     if not is_valid_affine(affine, 2):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     if affine is None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__100, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1611
 *         double di, dj
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1614
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     if affine is None:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_affine.memview) == Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":1615
 * 
 *     if affine is None:
 *         return             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":1614
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     if affine is None:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1617
 *         return
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(nrows):
 *             for j in range(ncols):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1618
 * 
 *     with nogil:
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 di = d[i, j, 0]
 */
        __pyx_t_9 = __pyx_v_nrows;
        for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
          __pyx_v_i = __pyx_t_10;

          /* "dipy/align/vector_fields.pyx":1619
 *     with nogil:
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 di = d[i, j, 0]
 *                 dj = d[i, j, 1]
 */
          __pyx_t_11 = __pyx_v_ncols;
          for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
            __pyx_v_j = __pyx_t_12;

            /* "dipy/align/vector_fields.pyx":1620
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 di = d[i, j, 0]             # <<<<<<<<<<<<<<
 *                 dj = d[i, j, 1]
 *                 d[i, j, 0] = _apply_affine_2d_x0(di, dj, 0, affine)
 */
            __pyx_t_13 = __pyx_v_i;
            __pyx_t_14 = __pyx_v_j;
            __pyx_t_15 = 0;
            __pyx_v_di = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_13 * __pyx_v_d.strides[0]) ) + __pyx_t_14 * __pyx_v_d.strides[1]) ) + __pyx_t_15 * __pyx_v_d.strides[2]) )));

            /* "dipy/align/vector_fields.pyx":1621
 *             for j in range(ncols):
 *                 di = d[i, j, 0]
 *                 dj = d[i, j, 1]             # <<<<<<<<<<<<<<
 *                 d[i, j, 0] = _apply_affine_2d_x0(di, dj, 0, affine)
 *                 d[i, j, 1] = _apply_affine_2d_x1(di, dj, 0, affine)
 */
            __pyx_t_16 = __pyx_v_i;
            __pyx_t_17 = __pyx_v_j;
            __pyx_t_18 = 1;
            __pyx_v_dj = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_16 * __pyx_v_d.strides[0]) ) + __pyx_t_17 * __pyx_v_d.strides[1]) ) + __pyx_t_18 * __pyx_v_d.strides[2]) )));

            /* "dipy/align/vector_fields.pyx":1622
 *                 di = d[i, j, 0]
 *                 dj = d[i, j, 1]
 *                 d[i, j, 0] = _apply_affine_2d_x0(di, dj, 0, affine)             # <<<<<<<<<<<<<<
 *                 d[i, j, 1] = _apply_affine_2d_x1(di, dj, 0, affine)
 * 
 */
            __pyx_t_19 = __pyx_v_i;
            __pyx_t_20 = __pyx_v_j;
            __pyx_t_21 = 0;
            *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_19 * __pyx_v_d.strides[0]) ) + __pyx_t_20 * __pyx_v_d.strides[1]) ) + __pyx_t_21 * __pyx_v_d.strides[2]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_di, __pyx_v_dj, 0.0, __pyx_v_affine);

            /* "dipy/align/vector_fields.pyx":1623
 *                 dj = d[i, j, 1]
 *                 d[i, j, 0] = _apply_affine_2d_x0(di, dj, 0, affine)
 *                 d[i, j, 1] = _apply_affine_2d_x1(di, dj, 0, affine)             # <<<<<<<<<<<<<<
 * 
 * 
 */
            __pyx_t_22 = __pyx_v_i;
            __pyx_t_23 = __pyx_v_j;
            __pyx_t_24 = 1;
            *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_22 * __pyx_v_d.strides[0]) ) + __pyx_t_23 * __pyx_v_d.strides[1]) ) + __pyx_t_24 * __pyx_v_d.strides[2]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_di, __pyx_v_dj, 0.0, __pyx_v_affine);
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1617
 *         return
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for i in range(nrows):
 *             for j in range(ncols):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1588
 * 
 * 
 * def reorient_vector_field_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                              double[:, :] affine):
 *     r"""Linearly transforms all vectors of a 2D displacement field
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("dipy.align.vector_fields.reorient_vector_field_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_d, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":1626
 * 
 * 
 * def reorient_vector_field_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                              double[:, :] affine):
 *     r"""Linearly transforms all vectors of a 3D displacement field
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_29reorient_vector_field_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_28reorient_vector_field_3d[] = "Linearly transforms all vectors of a 3D displacement field\n\n    Modifies the input displacement field by multiplying each displacement\n    vector by the given matrix. Note that the elements of the displacement\n    field are vectors, not points, so their last homogeneous coordinate is\n    zero, not one, and therefore the translation component of the affine\n    transform will not have any effect on them.\n\n    Parameters\n    ----------\n    d : array, shape (S, R, C, 3)\n        the displacement field to be re-oriented\n    affine : array, shape (4, 4)\n        the matrix to be applied\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_29reorient_vector_field_3d = {"reorient_vector_field_3d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_29reorient_vector_field_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_28reorient_vector_field_3d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_29reorient_vector_field_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_28reorient_vector_field_3d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_28reorient_vector_field_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("reorient_vector_field_3d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_d, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_d); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 4) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 4) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__101, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__102, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__103, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__104, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_181reorient_vector_field_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_181reorient_vector_field_3d = {"__pyx_fuse_0reorient_vector_field_3d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_181reorient_vector_field_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_28reorient_vector_field_3d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_181reorient_vector_field_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_d = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("reorient_vector_field_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_d,&__pyx_n_s_affine,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("reorient_vector_field_3d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "reorient_vector_field_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_d = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[0]); if (unlikely(!__pyx_v_d.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1627; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("reorient_vector_field_3d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.reorient_vector_field_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_180reorient_vector_field_3d(__pyx_self, __pyx_v_d, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_180reorient_vector_field_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dk;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  npy_intp __pyx_t_12;
  npy_intp __pyx_t_13;
  npy_intp __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0reorient_vector_field_3d", 0);

  /* "dipy/align/vector_fields.pyx":1644
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = d.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = d.shape[1]
 *         cnp.npy_intp ncols = d.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_d.shape[0]);

  /* "dipy/align/vector_fields.pyx":1645
 *     cdef:
 *         cnp.npy_intp nslices = d.shape[0]
 *         cnp.npy_intp nrows = d.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = d.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrows = (__pyx_v_d.shape[1]);

  /* "dipy/align/vector_fields.pyx":1646
 *         cnp.npy_intp nslices = d.shape[0]
 *         cnp.npy_intp nrows = d.shape[1]
 *         cnp.npy_intp ncols = d.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         double di, dj, dk
 */
  __pyx_v_ncols = (__pyx_v_d.shape[2]);

  /* "dipy/align/vector_fields.pyx":1650
 *         double di, dj, dk
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1650; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1650; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1650; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1650; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1650; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":1651
 * 
 *     if not is_valid_affine(affine, 3):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     if affine is None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__105, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1651; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1651; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1650
 *         double di, dj, dk
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1653
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     if affine is None:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_affine.memview) == Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":1654
 * 
 *     if affine is None:
 *         return             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":1653
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     if affine is None:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1656
 *         return
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for k in range(nslices):
 *             for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1657
 * 
 *     with nogil:
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_9 = __pyx_v_nslices;
        for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
          __pyx_v_k = __pyx_t_10;

          /* "dipy/align/vector_fields.pyx":1658
 *     with nogil:
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     dk = d[k, i, j, 0]
 */
          __pyx_t_11 = __pyx_v_nrows;
          for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
            __pyx_v_i = __pyx_t_12;

            /* "dipy/align/vector_fields.pyx":1659
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     dk = d[k, i, j, 0]
 *                     di = d[k, i, j, 1]
 */
            __pyx_t_13 = __pyx_v_ncols;
            for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
              __pyx_v_j = __pyx_t_14;

              /* "dipy/align/vector_fields.pyx":1660
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     dk = d[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                     di = d[k, i, j, 1]
 *                     dj = d[k, i, j, 2]
 */
              __pyx_t_15 = __pyx_v_k;
              __pyx_t_16 = __pyx_v_i;
              __pyx_t_17 = __pyx_v_j;
              __pyx_t_18 = 0;
              __pyx_v_dk = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_15 * __pyx_v_d.strides[0]) ) + __pyx_t_16 * __pyx_v_d.strides[1]) ) + __pyx_t_17 * __pyx_v_d.strides[2]) ) + __pyx_t_18 * __pyx_v_d.strides[3]) )));

              /* "dipy/align/vector_fields.pyx":1661
 *                 for j in range(ncols):
 *                     dk = d[k, i, j, 0]
 *                     di = d[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                     dj = d[k, i, j, 2]
 *                     d[k, i, j, 0] = _apply_affine_3d_x0(dk, di, dj, 0, affine)
 */
              __pyx_t_19 = __pyx_v_k;
              __pyx_t_20 = __pyx_v_i;
              __pyx_t_21 = __pyx_v_j;
              __pyx_t_22 = 1;
              __pyx_v_di = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_19 * __pyx_v_d.strides[0]) ) + __pyx_t_20 * __pyx_v_d.strides[1]) ) + __pyx_t_21 * __pyx_v_d.strides[2]) ) + __pyx_t_22 * __pyx_v_d.strides[3]) )));

              /* "dipy/align/vector_fields.pyx":1662
 *                     dk = d[k, i, j, 0]
 *                     di = d[k, i, j, 1]
 *                     dj = d[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     d[k, i, j, 0] = _apply_affine_3d_x0(dk, di, dj, 0, affine)
 *                     d[k, i, j, 1] = _apply_affine_3d_x1(dk, di, dj, 0, affine)
 */
              __pyx_t_23 = __pyx_v_k;
              __pyx_t_24 = __pyx_v_i;
              __pyx_t_25 = __pyx_v_j;
              __pyx_t_26 = 2;
              __pyx_v_dj = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_23 * __pyx_v_d.strides[0]) ) + __pyx_t_24 * __pyx_v_d.strides[1]) ) + __pyx_t_25 * __pyx_v_d.strides[2]) ) + __pyx_t_26 * __pyx_v_d.strides[3]) )));

              /* "dipy/align/vector_fields.pyx":1663
 *                     di = d[k, i, j, 1]
 *                     dj = d[k, i, j, 2]
 *                     d[k, i, j, 0] = _apply_affine_3d_x0(dk, di, dj, 0, affine)             # <<<<<<<<<<<<<<
 *                     d[k, i, j, 1] = _apply_affine_3d_x1(dk, di, dj, 0, affine)
 *                     d[k, i, j, 2] = _apply_affine_3d_x2(dk, di, dj, 0, affine)
 */
              __pyx_t_27 = __pyx_v_k;
              __pyx_t_28 = __pyx_v_i;
              __pyx_t_29 = __pyx_v_j;
              __pyx_t_30 = 0;
              *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_27 * __pyx_v_d.strides[0]) ) + __pyx_t_28 * __pyx_v_d.strides[1]) ) + __pyx_t_29 * __pyx_v_d.strides[2]) ) + __pyx_t_30 * __pyx_v_d.strides[3]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dk, __pyx_v_di, __pyx_v_dj, 0.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":1664
 *                     dj = d[k, i, j, 2]
 *                     d[k, i, j, 0] = _apply_affine_3d_x0(dk, di, dj, 0, affine)
 *                     d[k, i, j, 1] = _apply_affine_3d_x1(dk, di, dj, 0, affine)             # <<<<<<<<<<<<<<
 *                     d[k, i, j, 2] = _apply_affine_3d_x2(dk, di, dj, 0, affine)
 * 
 */
              __pyx_t_31 = __pyx_v_k;
              __pyx_t_32 = __pyx_v_i;
              __pyx_t_33 = __pyx_v_j;
              __pyx_t_34 = 1;
              *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_31 * __pyx_v_d.strides[0]) ) + __pyx_t_32 * __pyx_v_d.strides[1]) ) + __pyx_t_33 * __pyx_v_d.strides[2]) ) + __pyx_t_34 * __pyx_v_d.strides[3]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dk, __pyx_v_di, __pyx_v_dj, 0.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":1665
 *                     d[k, i, j, 0] = _apply_affine_3d_x0(dk, di, dj, 0, affine)
 *                     d[k, i, j, 1] = _apply_affine_3d_x1(dk, di, dj, 0, affine)
 *                     d[k, i, j, 2] = _apply_affine_3d_x2(dk, di, dj, 0, affine)             # <<<<<<<<<<<<<<
 * 
 * 
 */
              __pyx_t_35 = __pyx_v_k;
              __pyx_t_36 = __pyx_v_i;
              __pyx_t_37 = __pyx_v_j;
              __pyx_t_38 = 2;
              *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_35 * __pyx_v_d.strides[0]) ) + __pyx_t_36 * __pyx_v_d.strides[1]) ) + __pyx_t_37 * __pyx_v_d.strides[2]) ) + __pyx_t_38 * __pyx_v_d.strides[3]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dk, __pyx_v_di, __pyx_v_dj, 0.0, __pyx_v_affine);
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1656
 *         return
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for k in range(nslices):
 *             for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1626
 * 
 * 
 * def reorient_vector_field_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                              double[:, :] affine):
 *     r"""Linearly transforms all vectors of a 3D displacement field
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("dipy.align.vector_fields.reorient_vector_field_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_d, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_183reorient_vector_field_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_183reorient_vector_field_3d = {"__pyx_fuse_1reorient_vector_field_3d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_183reorient_vector_field_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_28reorient_vector_field_3d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_183reorient_vector_field_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_d = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("reorient_vector_field_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_d,&__pyx_n_s_affine,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("reorient_vector_field_3d", 1, 2, 2, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "reorient_vector_field_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_d = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[0]); if (unlikely(!__pyx_v_d.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1627; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("reorient_vector_field_3d", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.reorient_vector_field_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_182reorient_vector_field_3d(__pyx_self, __pyx_v_d, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_182reorient_vector_field_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_d, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dk;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  npy_intp __pyx_t_12;
  npy_intp __pyx_t_13;
  npy_intp __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1reorient_vector_field_3d", 0);

  /* "dipy/align/vector_fields.pyx":1644
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = d.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = d.shape[1]
 *         cnp.npy_intp ncols = d.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_d.shape[0]);

  /* "dipy/align/vector_fields.pyx":1645
 *     cdef:
 *         cnp.npy_intp nslices = d.shape[0]
 *         cnp.npy_intp nrows = d.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = d.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrows = (__pyx_v_d.shape[1]);

  /* "dipy/align/vector_fields.pyx":1646
 *         cnp.npy_intp nslices = d.shape[0]
 *         cnp.npy_intp nrows = d.shape[1]
 *         cnp.npy_intp ncols = d.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         double di, dj, dk
 */
  __pyx_v_ncols = (__pyx_v_d.shape[2]);

  /* "dipy/align/vector_fields.pyx":1650
 *         double di, dj, dk
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1650; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1650; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1650; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1650; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1650; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":1651
 * 
 *     if not is_valid_affine(affine, 3):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     if affine is None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__106, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1651; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1651; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1650
 *         double di, dj, dk
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1653
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     if affine is None:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_affine.memview) == Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":1654
 * 
 *     if affine is None:
 *         return             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "dipy/align/vector_fields.pyx":1653
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     if affine is None:             # <<<<<<<<<<<<<<
 *         return
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1656
 *         return
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for k in range(nslices):
 *             for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1657
 * 
 *     with nogil:
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_9 = __pyx_v_nslices;
        for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
          __pyx_v_k = __pyx_t_10;

          /* "dipy/align/vector_fields.pyx":1658
 *     with nogil:
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     dk = d[k, i, j, 0]
 */
          __pyx_t_11 = __pyx_v_nrows;
          for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
            __pyx_v_i = __pyx_t_12;

            /* "dipy/align/vector_fields.pyx":1659
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     dk = d[k, i, j, 0]
 *                     di = d[k, i, j, 1]
 */
            __pyx_t_13 = __pyx_v_ncols;
            for (__pyx_t_14 = 0; __pyx_t_14 < __pyx_t_13; __pyx_t_14+=1) {
              __pyx_v_j = __pyx_t_14;

              /* "dipy/align/vector_fields.pyx":1660
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     dk = d[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                     di = d[k, i, j, 1]
 *                     dj = d[k, i, j, 2]
 */
              __pyx_t_15 = __pyx_v_k;
              __pyx_t_16 = __pyx_v_i;
              __pyx_t_17 = __pyx_v_j;
              __pyx_t_18 = 0;
              __pyx_v_dk = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_15 * __pyx_v_d.strides[0]) ) + __pyx_t_16 * __pyx_v_d.strides[1]) ) + __pyx_t_17 * __pyx_v_d.strides[2]) ) + __pyx_t_18 * __pyx_v_d.strides[3]) )));

              /* "dipy/align/vector_fields.pyx":1661
 *                 for j in range(ncols):
 *                     dk = d[k, i, j, 0]
 *                     di = d[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                     dj = d[k, i, j, 2]
 *                     d[k, i, j, 0] = _apply_affine_3d_x0(dk, di, dj, 0, affine)
 */
              __pyx_t_19 = __pyx_v_k;
              __pyx_t_20 = __pyx_v_i;
              __pyx_t_21 = __pyx_v_j;
              __pyx_t_22 = 1;
              __pyx_v_di = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_19 * __pyx_v_d.strides[0]) ) + __pyx_t_20 * __pyx_v_d.strides[1]) ) + __pyx_t_21 * __pyx_v_d.strides[2]) ) + __pyx_t_22 * __pyx_v_d.strides[3]) )));

              /* "dipy/align/vector_fields.pyx":1662
 *                     dk = d[k, i, j, 0]
 *                     di = d[k, i, j, 1]
 *                     dj = d[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     d[k, i, j, 0] = _apply_affine_3d_x0(dk, di, dj, 0, affine)
 *                     d[k, i, j, 1] = _apply_affine_3d_x1(dk, di, dj, 0, affine)
 */
              __pyx_t_23 = __pyx_v_k;
              __pyx_t_24 = __pyx_v_i;
              __pyx_t_25 = __pyx_v_j;
              __pyx_t_26 = 2;
              __pyx_v_dj = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_23 * __pyx_v_d.strides[0]) ) + __pyx_t_24 * __pyx_v_d.strides[1]) ) + __pyx_t_25 * __pyx_v_d.strides[2]) ) + __pyx_t_26 * __pyx_v_d.strides[3]) )));

              /* "dipy/align/vector_fields.pyx":1663
 *                     di = d[k, i, j, 1]
 *                     dj = d[k, i, j, 2]
 *                     d[k, i, j, 0] = _apply_affine_3d_x0(dk, di, dj, 0, affine)             # <<<<<<<<<<<<<<
 *                     d[k, i, j, 1] = _apply_affine_3d_x1(dk, di, dj, 0, affine)
 *                     d[k, i, j, 2] = _apply_affine_3d_x2(dk, di, dj, 0, affine)
 */
              __pyx_t_27 = __pyx_v_k;
              __pyx_t_28 = __pyx_v_i;
              __pyx_t_29 = __pyx_v_j;
              __pyx_t_30 = 0;
              *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_27 * __pyx_v_d.strides[0]) ) + __pyx_t_28 * __pyx_v_d.strides[1]) ) + __pyx_t_29 * __pyx_v_d.strides[2]) ) + __pyx_t_30 * __pyx_v_d.strides[3]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dk, __pyx_v_di, __pyx_v_dj, 0.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":1664
 *                     dj = d[k, i, j, 2]
 *                     d[k, i, j, 0] = _apply_affine_3d_x0(dk, di, dj, 0, affine)
 *                     d[k, i, j, 1] = _apply_affine_3d_x1(dk, di, dj, 0, affine)             # <<<<<<<<<<<<<<
 *                     d[k, i, j, 2] = _apply_affine_3d_x2(dk, di, dj, 0, affine)
 * 
 */
              __pyx_t_31 = __pyx_v_k;
              __pyx_t_32 = __pyx_v_i;
              __pyx_t_33 = __pyx_v_j;
              __pyx_t_34 = 1;
              *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_31 * __pyx_v_d.strides[0]) ) + __pyx_t_32 * __pyx_v_d.strides[1]) ) + __pyx_t_33 * __pyx_v_d.strides[2]) ) + __pyx_t_34 * __pyx_v_d.strides[3]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dk, __pyx_v_di, __pyx_v_dj, 0.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":1665
 *                     d[k, i, j, 0] = _apply_affine_3d_x0(dk, di, dj, 0, affine)
 *                     d[k, i, j, 1] = _apply_affine_3d_x1(dk, di, dj, 0, affine)
 *                     d[k, i, j, 2] = _apply_affine_3d_x2(dk, di, dj, 0, affine)             # <<<<<<<<<<<<<<
 * 
 * 
 */
              __pyx_t_35 = __pyx_v_k;
              __pyx_t_36 = __pyx_v_i;
              __pyx_t_37 = __pyx_v_j;
              __pyx_t_38 = 2;
              *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_35 * __pyx_v_d.strides[0]) ) + __pyx_t_36 * __pyx_v_d.strides[1]) ) + __pyx_t_37 * __pyx_v_d.strides[2]) ) + __pyx_t_38 * __pyx_v_d.strides[3]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dk, __pyx_v_di, __pyx_v_dj, 0.0, __pyx_v_affine);
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1656
 *         return
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for k in range(nslices):
 *             for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1626
 * 
 * 
 * def reorient_vector_field_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                              double[:, :] affine):
 *     r"""Linearly transforms all vectors of a 3D displacement field
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("dipy.align.vector_fields.reorient_vector_field_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_d, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":1668
 * 
 * 
 * def downsample_scalar_field_3d(floating[:, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input volume by a factor of 2
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_31downsample_scalar_field_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_30downsample_scalar_field_3d[] = "Down-samples the input volume by a factor of 2\n\n    Down-samples the input volume by a factor of 2. The value at each voxel\n    of the resulting volume is the average of its surrounding voxels in the\n    original volume.\n\n    Parameters\n    ----------\n    field : array, shape (S, R, C)\n        the volume to be down-sampled\n\n    Returns\n    -------\n    down : array, shape (S', R', C')\n        the down-sampled displacement field, where S' = ceil(S/2),\n        R'= ceil(R/2), C'=ceil(C/2)\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_31downsample_scalar_field_3d = {"downsample_scalar_field_3d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_31downsample_scalar_field_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_30downsample_scalar_field_3d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_31downsample_scalar_field_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_30downsample_scalar_field_3d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_30downsample_scalar_field_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("downsample_scalar_field_3d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_field, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_field); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__107, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__108, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__109, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__110, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_187downsample_scalar_field_3d(PyObject *__pyx_self, PyObject *__pyx_arg_field); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_187downsample_scalar_field_3d = {"__pyx_fuse_0downsample_scalar_field_3d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_187downsample_scalar_field_3d, METH_O, __pyx_doc_4dipy_5align_13vector_fields_30downsample_scalar_field_3d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_187downsample_scalar_field_3d(PyObject *__pyx_self, PyObject *__pyx_arg_field) {
  __Pyx_memviewslice __pyx_v_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("downsample_scalar_field_3d (wrapper)", 0);
  assert(__pyx_arg_field); {
    __pyx_v_field = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_arg_field); if (unlikely(!__pyx_v_field.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.downsample_scalar_field_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_186downsample_scalar_field_3d(__pyx_self, __pyx_v_field);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_186downsample_scalar_field_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_ns;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_nns;
  npy_intp __pyx_v_nnr;
  npy_intp __pyx_v_nnc;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  npy_intp __pyx_v_kk;
  __Pyx_memviewslice __pyx_v_down = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_cnt = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  npy_intp __pyx_t_12;
  npy_intp __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  int __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0downsample_scalar_field_3d", 0);

  /* "dipy/align/vector_fields.pyx":1686
 *         R'= ceil(R/2), C'=ceil(C/2)
 *     """
 *     ftype = np.asarray(field).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp ns = field.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1686; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1686; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_field, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1686; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1686; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1686; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1686; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1686; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":1688
 *     ftype = np.asarray(field).dtype
 *     cdef:
 *         cnp.npy_intp ns = field.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr = field.shape[1]
 *         cnp.npy_intp nc = field.shape[2]
 */
  __pyx_v_ns = (__pyx_v_field.shape[0]);

  /* "dipy/align/vector_fields.pyx":1689
 *     cdef:
 *         cnp.npy_intp ns = field.shape[0]
 *         cnp.npy_intp nr = field.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = field.shape[2]
 *         cnp.npy_intp nns = (ns + 1) // 2
 */
  __pyx_v_nr = (__pyx_v_field.shape[1]);

  /* "dipy/align/vector_fields.pyx":1690
 *         cnp.npy_intp ns = field.shape[0]
 *         cnp.npy_intp nr = field.shape[1]
 *         cnp.npy_intp nc = field.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nns = (ns + 1) // 2
 *         cnp.npy_intp nnr = (nr + 1) // 2
 */
  __pyx_v_nc = (__pyx_v_field.shape[2]);

  /* "dipy/align/vector_fields.pyx":1691
 *         cnp.npy_intp nr = field.shape[1]
 *         cnp.npy_intp nc = field.shape[2]
 *         cnp.npy_intp nns = (ns + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nnr = (nr + 1) // 2
 *         cnp.npy_intp nnc = (nc + 1) // 2
 */
  __pyx_v_nns = ((__pyx_v_ns + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1692
 *         cnp.npy_intp nc = field.shape[2]
 *         cnp.npy_intp nns = (ns + 1) // 2
 *         cnp.npy_intp nnr = (nr + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nnc = (nc + 1) // 2
 *         cnp.npy_intp i, j, k, ii, jj, kk
 */
  __pyx_v_nnr = ((__pyx_v_nr + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1693
 *         cnp.npy_intp nns = (ns + 1) // 2
 *         cnp.npy_intp nnr = (nr + 1) // 2
 *         cnp.npy_intp nnc = (nc + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         floating[:, :, :] down = np.zeros((nns, nnr, nnc), dtype=ftype)
 */
  __pyx_v_nnc = ((__pyx_v_nc + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1695
 *         cnp.npy_intp nnc = (nc + 1) // 2
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         floating[:, :, :] down = np.zeros((nns, nnr, nnc), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:, :, :] cnt = np.zeros((nns, nnr, nnc), dtype=np.int32)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nns); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnr); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnc); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_2);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_t_5);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_down = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":1696
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         floating[:, :, :] down = np.zeros((nns, nnr, nnc), dtype=ftype)
 *         int[:, :, :] cnt = np.zeros((nns, nnr, nnc), dtype=np.int32)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nns); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnr); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnc); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_1);
  __pyx_t_5 = 0;
  __pyx_t_2 = 0;
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_int32); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(__pyx_t_5);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_cnt = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":1698
 *         int[:, :, :] cnt = np.zeros((nns, nnr, nnc), dtype=np.int32)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for k in range(ns):
 *             for i in range(nr):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1699
 * 
 *     with nogil:
 *         for k in range(ns):             # <<<<<<<<<<<<<<
 *             for i in range(nr):
 *                 for j in range(nc):
 */
        __pyx_t_8 = __pyx_v_ns;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_k = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":1700
 *     with nogil:
 *         for k in range(ns):
 *             for i in range(nr):             # <<<<<<<<<<<<<<
 *                 for j in range(nc):
 *                     kk = k // 2
 */
          __pyx_t_10 = __pyx_v_nr;
          for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
            __pyx_v_i = __pyx_t_11;

            /* "dipy/align/vector_fields.pyx":1701
 *         for k in range(ns):
 *             for i in range(nr):
 *                 for j in range(nc):             # <<<<<<<<<<<<<<
 *                     kk = k // 2
 *                     ii = i // 2
 */
            __pyx_t_12 = __pyx_v_nc;
            for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
              __pyx_v_j = __pyx_t_13;

              /* "dipy/align/vector_fields.pyx":1702
 *             for i in range(nr):
 *                 for j in range(nc):
 *                     kk = k // 2             # <<<<<<<<<<<<<<
 *                     ii = i // 2
 *                     jj = j // 2
 */
              __pyx_v_kk = (__pyx_v_k / 2);

              /* "dipy/align/vector_fields.pyx":1703
 *                 for j in range(nc):
 *                     kk = k // 2
 *                     ii = i // 2             # <<<<<<<<<<<<<<
 *                     jj = j // 2
 *                     down[kk, ii, jj] += field[k, i, j]
 */
              __pyx_v_ii = (__pyx_v_i / 2);

              /* "dipy/align/vector_fields.pyx":1704
 *                     kk = k // 2
 *                     ii = i // 2
 *                     jj = j // 2             # <<<<<<<<<<<<<<
 *                     down[kk, ii, jj] += field[k, i, j]
 *                     cnt[kk, ii, jj] += 1
 */
              __pyx_v_jj = (__pyx_v_j / 2);

              /* "dipy/align/vector_fields.pyx":1705
 *                     ii = i // 2
 *                     jj = j // 2
 *                     down[kk, ii, jj] += field[k, i, j]             # <<<<<<<<<<<<<<
 *                     cnt[kk, ii, jj] += 1
 *         for k in range(nns):
 */
              __pyx_t_14 = __pyx_v_k;
              __pyx_t_15 = __pyx_v_i;
              __pyx_t_16 = __pyx_v_j;
              __pyx_t_17 = __pyx_v_kk;
              __pyx_t_18 = __pyx_v_ii;
              __pyx_t_19 = __pyx_v_jj;
              *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_17 * __pyx_v_down.strides[0]) ) + __pyx_t_18 * __pyx_v_down.strides[1]) ) + __pyx_t_19 * __pyx_v_down.strides[2]) )) += (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_14 * __pyx_v_field.strides[0]) ) + __pyx_t_15 * __pyx_v_field.strides[1]) ) + __pyx_t_16 * __pyx_v_field.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":1706
 *                     jj = j // 2
 *                     down[kk, ii, jj] += field[k, i, j]
 *                     cnt[kk, ii, jj] += 1             # <<<<<<<<<<<<<<
 *         for k in range(nns):
 *             for i in range(nnr):
 */
              __pyx_t_20 = __pyx_v_kk;
              __pyx_t_21 = __pyx_v_ii;
              __pyx_t_22 = __pyx_v_jj;
              *((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_20 * __pyx_v_cnt.strides[0]) ) + __pyx_t_21 * __pyx_v_cnt.strides[1]) ) + __pyx_t_22 * __pyx_v_cnt.strides[2]) )) += 1;
            }
          }
        }

        /* "dipy/align/vector_fields.pyx":1707
 *                     down[kk, ii, jj] += field[k, i, j]
 *                     cnt[kk, ii, jj] += 1
 *         for k in range(nns):             # <<<<<<<<<<<<<<
 *             for i in range(nnr):
 *                 for j in range(nnc):
 */
        __pyx_t_8 = __pyx_v_nns;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_k = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":1708
 *                     cnt[kk, ii, jj] += 1
 *         for k in range(nns):
 *             for i in range(nnr):             # <<<<<<<<<<<<<<
 *                 for j in range(nnc):
 *                     if cnt[k, i, j] > 0:
 */
          __pyx_t_10 = __pyx_v_nnr;
          for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
            __pyx_v_i = __pyx_t_11;

            /* "dipy/align/vector_fields.pyx":1709
 *         for k in range(nns):
 *             for i in range(nnr):
 *                 for j in range(nnc):             # <<<<<<<<<<<<<<
 *                     if cnt[k, i, j] > 0:
 *                         down[k, i, j] /= cnt[k, i, j]
 */
            __pyx_t_12 = __pyx_v_nnc;
            for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
              __pyx_v_j = __pyx_t_13;

              /* "dipy/align/vector_fields.pyx":1710
 *             for i in range(nnr):
 *                 for j in range(nnc):
 *                     if cnt[k, i, j] > 0:             # <<<<<<<<<<<<<<
 *                         down[k, i, j] /= cnt[k, i, j]
 *     return down
 */
              __pyx_t_23 = __pyx_v_k;
              __pyx_t_24 = __pyx_v_i;
              __pyx_t_25 = __pyx_v_j;
              __pyx_t_26 = (((*((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_23 * __pyx_v_cnt.strides[0]) ) + __pyx_t_24 * __pyx_v_cnt.strides[1]) ) + __pyx_t_25 * __pyx_v_cnt.strides[2]) ))) > 0) != 0);
              if (__pyx_t_26) {

                /* "dipy/align/vector_fields.pyx":1711
 *                 for j in range(nnc):
 *                     if cnt[k, i, j] > 0:
 *                         down[k, i, j] /= cnt[k, i, j]             # <<<<<<<<<<<<<<
 *     return down
 * 
 */
                __pyx_t_27 = __pyx_v_k;
                __pyx_t_28 = __pyx_v_i;
                __pyx_t_29 = __pyx_v_j;
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_30 * __pyx_v_down.strides[0]) ) + __pyx_t_31 * __pyx_v_down.strides[1]) ) + __pyx_t_32 * __pyx_v_down.strides[2]) )) /= (*((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_27 * __pyx_v_cnt.strides[0]) ) + __pyx_t_28 * __pyx_v_cnt.strides[1]) ) + __pyx_t_29 * __pyx_v_cnt.strides[2]) )));

                /* "dipy/align/vector_fields.pyx":1710
 *             for i in range(nnr):
 *                 for j in range(nnc):
 *                     if cnt[k, i, j] > 0:             # <<<<<<<<<<<<<<
 *                         down[k, i, j] /= cnt[k, i, j]
 *     return down
 */
              }
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1698
 *         int[:, :, :] cnt = np.zeros((nns, nnr, nnc), dtype=np.int32)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for k in range(ns):
 *             for i in range(nr):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1712
 *                     if cnt[k, i, j] > 0:
 *                         down[k, i, j] /= cnt[k, i, j]
 *     return down             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_down, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1712; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1668
 * 
 * 
 * def downsample_scalar_field_3d(floating[:, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input volume by a factor of 2
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.downsample_scalar_field_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_field, 1);
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_down, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_cnt, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_189downsample_scalar_field_3d(PyObject *__pyx_self, PyObject *__pyx_arg_field); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_189downsample_scalar_field_3d = {"__pyx_fuse_1downsample_scalar_field_3d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_189downsample_scalar_field_3d, METH_O, __pyx_doc_4dipy_5align_13vector_fields_30downsample_scalar_field_3d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_189downsample_scalar_field_3d(PyObject *__pyx_self, PyObject *__pyx_arg_field) {
  __Pyx_memviewslice __pyx_v_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("downsample_scalar_field_3d (wrapper)", 0);
  assert(__pyx_arg_field); {
    __pyx_v_field = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_arg_field); if (unlikely(!__pyx_v_field.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.downsample_scalar_field_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_188downsample_scalar_field_3d(__pyx_self, __pyx_v_field);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_188downsample_scalar_field_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_ns;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_nns;
  npy_intp __pyx_v_nnr;
  npy_intp __pyx_v_nnc;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  npy_intp __pyx_v_kk;
  __Pyx_memviewslice __pyx_v_down = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_cnt = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  npy_intp __pyx_t_12;
  npy_intp __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  int __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1downsample_scalar_field_3d", 0);

  /* "dipy/align/vector_fields.pyx":1686
 *         R'= ceil(R/2), C'=ceil(C/2)
 *     """
 *     ftype = np.asarray(field).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp ns = field.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1686; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1686; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_field, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1686; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1686; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1686; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1686; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1686; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":1688
 *     ftype = np.asarray(field).dtype
 *     cdef:
 *         cnp.npy_intp ns = field.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr = field.shape[1]
 *         cnp.npy_intp nc = field.shape[2]
 */
  __pyx_v_ns = (__pyx_v_field.shape[0]);

  /* "dipy/align/vector_fields.pyx":1689
 *     cdef:
 *         cnp.npy_intp ns = field.shape[0]
 *         cnp.npy_intp nr = field.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = field.shape[2]
 *         cnp.npy_intp nns = (ns + 1) // 2
 */
  __pyx_v_nr = (__pyx_v_field.shape[1]);

  /* "dipy/align/vector_fields.pyx":1690
 *         cnp.npy_intp ns = field.shape[0]
 *         cnp.npy_intp nr = field.shape[1]
 *         cnp.npy_intp nc = field.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nns = (ns + 1) // 2
 *         cnp.npy_intp nnr = (nr + 1) // 2
 */
  __pyx_v_nc = (__pyx_v_field.shape[2]);

  /* "dipy/align/vector_fields.pyx":1691
 *         cnp.npy_intp nr = field.shape[1]
 *         cnp.npy_intp nc = field.shape[2]
 *         cnp.npy_intp nns = (ns + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nnr = (nr + 1) // 2
 *         cnp.npy_intp nnc = (nc + 1) // 2
 */
  __pyx_v_nns = ((__pyx_v_ns + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1692
 *         cnp.npy_intp nc = field.shape[2]
 *         cnp.npy_intp nns = (ns + 1) // 2
 *         cnp.npy_intp nnr = (nr + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nnc = (nc + 1) // 2
 *         cnp.npy_intp i, j, k, ii, jj, kk
 */
  __pyx_v_nnr = ((__pyx_v_nr + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1693
 *         cnp.npy_intp nns = (ns + 1) // 2
 *         cnp.npy_intp nnr = (nr + 1) // 2
 *         cnp.npy_intp nnc = (nc + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         floating[:, :, :] down = np.zeros((nns, nnr, nnc), dtype=ftype)
 */
  __pyx_v_nnc = ((__pyx_v_nc + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1695
 *         cnp.npy_intp nnc = (nc + 1) // 2
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         floating[:, :, :] down = np.zeros((nns, nnr, nnc), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:, :, :] cnt = np.zeros((nns, nnr, nnc), dtype=np.int32)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nns); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnr); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnc); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_2);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_5);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1695; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_down = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":1696
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         floating[:, :, :] down = np.zeros((nns, nnr, nnc), dtype=ftype)
 *         int[:, :, :] cnt = np.zeros((nns, nnr, nnc), dtype=np.int32)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nns); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnr); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnc); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_1);
  __pyx_t_5 = 0;
  __pyx_t_2 = 0;
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_int32); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(__pyx_t_5);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1696; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_cnt = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":1698
 *         int[:, :, :] cnt = np.zeros((nns, nnr, nnc), dtype=np.int32)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for k in range(ns):
 *             for i in range(nr):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1699
 * 
 *     with nogil:
 *         for k in range(ns):             # <<<<<<<<<<<<<<
 *             for i in range(nr):
 *                 for j in range(nc):
 */
        __pyx_t_8 = __pyx_v_ns;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_k = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":1700
 *     with nogil:
 *         for k in range(ns):
 *             for i in range(nr):             # <<<<<<<<<<<<<<
 *                 for j in range(nc):
 *                     kk = k // 2
 */
          __pyx_t_10 = __pyx_v_nr;
          for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
            __pyx_v_i = __pyx_t_11;

            /* "dipy/align/vector_fields.pyx":1701
 *         for k in range(ns):
 *             for i in range(nr):
 *                 for j in range(nc):             # <<<<<<<<<<<<<<
 *                     kk = k // 2
 *                     ii = i // 2
 */
            __pyx_t_12 = __pyx_v_nc;
            for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
              __pyx_v_j = __pyx_t_13;

              /* "dipy/align/vector_fields.pyx":1702
 *             for i in range(nr):
 *                 for j in range(nc):
 *                     kk = k // 2             # <<<<<<<<<<<<<<
 *                     ii = i // 2
 *                     jj = j // 2
 */
              __pyx_v_kk = (__pyx_v_k / 2);

              /* "dipy/align/vector_fields.pyx":1703
 *                 for j in range(nc):
 *                     kk = k // 2
 *                     ii = i // 2             # <<<<<<<<<<<<<<
 *                     jj = j // 2
 *                     down[kk, ii, jj] += field[k, i, j]
 */
              __pyx_v_ii = (__pyx_v_i / 2);

              /* "dipy/align/vector_fields.pyx":1704
 *                     kk = k // 2
 *                     ii = i // 2
 *                     jj = j // 2             # <<<<<<<<<<<<<<
 *                     down[kk, ii, jj] += field[k, i, j]
 *                     cnt[kk, ii, jj] += 1
 */
              __pyx_v_jj = (__pyx_v_j / 2);

              /* "dipy/align/vector_fields.pyx":1705
 *                     ii = i // 2
 *                     jj = j // 2
 *                     down[kk, ii, jj] += field[k, i, j]             # <<<<<<<<<<<<<<
 *                     cnt[kk, ii, jj] += 1
 *         for k in range(nns):
 */
              __pyx_t_14 = __pyx_v_k;
              __pyx_t_15 = __pyx_v_i;
              __pyx_t_16 = __pyx_v_j;
              __pyx_t_17 = __pyx_v_kk;
              __pyx_t_18 = __pyx_v_ii;
              __pyx_t_19 = __pyx_v_jj;
              *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_17 * __pyx_v_down.strides[0]) ) + __pyx_t_18 * __pyx_v_down.strides[1]) ) + __pyx_t_19 * __pyx_v_down.strides[2]) )) += (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_14 * __pyx_v_field.strides[0]) ) + __pyx_t_15 * __pyx_v_field.strides[1]) ) + __pyx_t_16 * __pyx_v_field.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":1706
 *                     jj = j // 2
 *                     down[kk, ii, jj] += field[k, i, j]
 *                     cnt[kk, ii, jj] += 1             # <<<<<<<<<<<<<<
 *         for k in range(nns):
 *             for i in range(nnr):
 */
              __pyx_t_20 = __pyx_v_kk;
              __pyx_t_21 = __pyx_v_ii;
              __pyx_t_22 = __pyx_v_jj;
              *((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_20 * __pyx_v_cnt.strides[0]) ) + __pyx_t_21 * __pyx_v_cnt.strides[1]) ) + __pyx_t_22 * __pyx_v_cnt.strides[2]) )) += 1;
            }
          }
        }

        /* "dipy/align/vector_fields.pyx":1707
 *                     down[kk, ii, jj] += field[k, i, j]
 *                     cnt[kk, ii, jj] += 1
 *         for k in range(nns):             # <<<<<<<<<<<<<<
 *             for i in range(nnr):
 *                 for j in range(nnc):
 */
        __pyx_t_8 = __pyx_v_nns;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_k = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":1708
 *                     cnt[kk, ii, jj] += 1
 *         for k in range(nns):
 *             for i in range(nnr):             # <<<<<<<<<<<<<<
 *                 for j in range(nnc):
 *                     if cnt[k, i, j] > 0:
 */
          __pyx_t_10 = __pyx_v_nnr;
          for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
            __pyx_v_i = __pyx_t_11;

            /* "dipy/align/vector_fields.pyx":1709
 *         for k in range(nns):
 *             for i in range(nnr):
 *                 for j in range(nnc):             # <<<<<<<<<<<<<<
 *                     if cnt[k, i, j] > 0:
 *                         down[k, i, j] /= cnt[k, i, j]
 */
            __pyx_t_12 = __pyx_v_nnc;
            for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
              __pyx_v_j = __pyx_t_13;

              /* "dipy/align/vector_fields.pyx":1710
 *             for i in range(nnr):
 *                 for j in range(nnc):
 *                     if cnt[k, i, j] > 0:             # <<<<<<<<<<<<<<
 *                         down[k, i, j] /= cnt[k, i, j]
 *     return down
 */
              __pyx_t_23 = __pyx_v_k;
              __pyx_t_24 = __pyx_v_i;
              __pyx_t_25 = __pyx_v_j;
              __pyx_t_26 = (((*((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_23 * __pyx_v_cnt.strides[0]) ) + __pyx_t_24 * __pyx_v_cnt.strides[1]) ) + __pyx_t_25 * __pyx_v_cnt.strides[2]) ))) > 0) != 0);
              if (__pyx_t_26) {

                /* "dipy/align/vector_fields.pyx":1711
 *                 for j in range(nnc):
 *                     if cnt[k, i, j] > 0:
 *                         down[k, i, j] /= cnt[k, i, j]             # <<<<<<<<<<<<<<
 *     return down
 * 
 */
                __pyx_t_27 = __pyx_v_k;
                __pyx_t_28 = __pyx_v_i;
                __pyx_t_29 = __pyx_v_j;
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_30 * __pyx_v_down.strides[0]) ) + __pyx_t_31 * __pyx_v_down.strides[1]) ) + __pyx_t_32 * __pyx_v_down.strides[2]) )) /= (*((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_27 * __pyx_v_cnt.strides[0]) ) + __pyx_t_28 * __pyx_v_cnt.strides[1]) ) + __pyx_t_29 * __pyx_v_cnt.strides[2]) )));

                /* "dipy/align/vector_fields.pyx":1710
 *             for i in range(nnr):
 *                 for j in range(nnc):
 *                     if cnt[k, i, j] > 0:             # <<<<<<<<<<<<<<
 *                         down[k, i, j] /= cnt[k, i, j]
 *     return down
 */
              }
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1698
 *         int[:, :, :] cnt = np.zeros((nns, nnr, nnc), dtype=np.int32)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         for k in range(ns):
 *             for i in range(nr):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1712
 *                     if cnt[k, i, j] > 0:
 *                         down[k, i, j] /= cnt[k, i, j]
 *     return down             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_down, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1712; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1668
 * 
 * 
 * def downsample_scalar_field_3d(floating[:, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input volume by a factor of 2
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.downsample_scalar_field_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_field, 1);
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_down, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_cnt, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":1715
 * 
 * 
 * def downsample_displacement_field_3d(floating[:, :, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input 3D vector field by a factor of 2
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_33downsample_displacement_field_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_32downsample_displacement_field_3d[] = "Down-samples the input 3D vector field by a factor of 2\n\n    Down-samples the input vector field by a factor of 2. This operation\n    is equivalent to dividing the input image into 2x2x2 cubes and averaging\n    the 8 vectors. The resulting field consists of these average vectors.\n\n    Parameters\n    ----------\n    field : array, shape (S, R, C)\n        the vector field to be down-sampled\n\n    Returns\n    -------\n    down : array, shape (S', R', C')\n        the down-sampled displacement field, where S' = ceil(S/2),\n        R'= ceil(R/2), C'=ceil(C/2)\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_33downsample_displacement_field_3d = {"downsample_displacement_field_3d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_33downsample_displacement_field_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_32downsample_displacement_field_3d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_33downsample_displacement_field_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_32downsample_displacement_field_3d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_32downsample_displacement_field_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("downsample_displacement_field_3d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_field, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_field); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 4) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 4) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__111, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__112, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__113, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__114, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_193downsample_displacement_field_3d(PyObject *__pyx_self, PyObject *__pyx_arg_field); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_193downsample_displacement_field_3d = {"__pyx_fuse_0downsample_displacement_field_3d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_193downsample_displacement_field_3d, METH_O, __pyx_doc_4dipy_5align_13vector_fields_32downsample_displacement_field_3d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_193downsample_displacement_field_3d(PyObject *__pyx_self, PyObject *__pyx_arg_field) {
  __Pyx_memviewslice __pyx_v_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("downsample_displacement_field_3d (wrapper)", 0);
  assert(__pyx_arg_field); {
    __pyx_v_field = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(__pyx_arg_field); if (unlikely(!__pyx_v_field.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.downsample_displacement_field_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_192downsample_displacement_field_3d(__pyx_self, __pyx_v_field);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_192downsample_displacement_field_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_ns;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_nns;
  npy_intp __pyx_v_nnr;
  npy_intp __pyx_v_nnc;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  npy_intp __pyx_v_kk;
  __Pyx_memviewslice __pyx_v_down = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_cnt = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  npy_intp __pyx_t_12;
  npy_intp __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  int __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  Py_ssize_t __pyx_t_60;
  Py_ssize_t __pyx_t_61;
  Py_ssize_t __pyx_t_62;
  Py_ssize_t __pyx_t_63;
  Py_ssize_t __pyx_t_64;
  Py_ssize_t __pyx_t_65;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0downsample_displacement_field_3d", 0);

  /* "dipy/align/vector_fields.pyx":1733
 *         R'= ceil(R/2), C'=ceil(C/2)
 *     """
 *     ftype = np.asarray(field).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp ns = field.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1733; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1733; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_field, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1733; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1733; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1733; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1733; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1733; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":1735
 *     ftype = np.asarray(field).dtype
 *     cdef:
 *         cnp.npy_intp ns = field.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr = field.shape[1]
 *         cnp.npy_intp nc = field.shape[2]
 */
  __pyx_v_ns = (__pyx_v_field.shape[0]);

  /* "dipy/align/vector_fields.pyx":1736
 *     cdef:
 *         cnp.npy_intp ns = field.shape[0]
 *         cnp.npy_intp nr = field.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = field.shape[2]
 *         cnp.npy_intp nns = (ns + 1) // 2
 */
  __pyx_v_nr = (__pyx_v_field.shape[1]);

  /* "dipy/align/vector_fields.pyx":1737
 *         cnp.npy_intp ns = field.shape[0]
 *         cnp.npy_intp nr = field.shape[1]
 *         cnp.npy_intp nc = field.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nns = (ns + 1) // 2
 *         cnp.npy_intp nnr = (nr + 1) // 2
 */
  __pyx_v_nc = (__pyx_v_field.shape[2]);

  /* "dipy/align/vector_fields.pyx":1738
 *         cnp.npy_intp nr = field.shape[1]
 *         cnp.npy_intp nc = field.shape[2]
 *         cnp.npy_intp nns = (ns + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nnr = (nr + 1) // 2
 *         cnp.npy_intp nnc = (nc + 1) // 2
 */
  __pyx_v_nns = ((__pyx_v_ns + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1739
 *         cnp.npy_intp nc = field.shape[2]
 *         cnp.npy_intp nns = (ns + 1) // 2
 *         cnp.npy_intp nnr = (nr + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nnc = (nc + 1) // 2
 *         cnp.npy_intp i, j, k, ii, jj, kk
 */
  __pyx_v_nnr = ((__pyx_v_nr + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1740
 *         cnp.npy_intp nns = (ns + 1) // 2
 *         cnp.npy_intp nnr = (nr + 1) // 2
 *         cnp.npy_intp nnc = (nc + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         floating[:, :, :, :] down = np.zeros((nns, nnr, nnc, 3), dtype=ftype)
 */
  __pyx_v_nnc = ((__pyx_v_nc + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1742
 *         cnp.npy_intp nnc = (nc + 1) // 2
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         floating[:, :, :, :] down = np.zeros((nns, nnr, nnc, 3), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:, :, :] cnt = np.zeros((nns, nnr, nnc), dtype=np.int32)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nns); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnr); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnc); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_2);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(__pyx_t_5);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_down = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":1743
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         floating[:, :, :, :] down = np.zeros((nns, nnr, nnc, 3), dtype=ftype)
 *         int[:, :, :] cnt = np.zeros((nns, nnr, nnc), dtype=np.int32)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nns); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnr); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnc); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_1);
  __pyx_t_5 = 0;
  __pyx_t_2 = 0;
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_int32); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(__pyx_t_5);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_cnt = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":1745
 *         int[:, :, :] cnt = np.zeros((nns, nnr, nnc), dtype=np.int32)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(ns):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1747
 *     with nogil:
 * 
 *         for k in range(ns):             # <<<<<<<<<<<<<<
 *             for i in range(nr):
 *                 for j in range(nc):
 */
        __pyx_t_8 = __pyx_v_ns;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_k = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":1748
 * 
 *         for k in range(ns):
 *             for i in range(nr):             # <<<<<<<<<<<<<<
 *                 for j in range(nc):
 *                     kk = k // 2
 */
          __pyx_t_10 = __pyx_v_nr;
          for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
            __pyx_v_i = __pyx_t_11;

            /* "dipy/align/vector_fields.pyx":1749
 *         for k in range(ns):
 *             for i in range(nr):
 *                 for j in range(nc):             # <<<<<<<<<<<<<<
 *                     kk = k // 2
 *                     ii = i // 2
 */
            __pyx_t_12 = __pyx_v_nc;
            for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
              __pyx_v_j = __pyx_t_13;

              /* "dipy/align/vector_fields.pyx":1750
 *             for i in range(nr):
 *                 for j in range(nc):
 *                     kk = k // 2             # <<<<<<<<<<<<<<
 *                     ii = i // 2
 *                     jj = j // 2
 */
              __pyx_v_kk = (__pyx_v_k / 2);

              /* "dipy/align/vector_fields.pyx":1751
 *                 for j in range(nc):
 *                     kk = k // 2
 *                     ii = i // 2             # <<<<<<<<<<<<<<
 *                     jj = j // 2
 *                     down[kk, ii, jj, 0] += field[k, i, j, 0]
 */
              __pyx_v_ii = (__pyx_v_i / 2);

              /* "dipy/align/vector_fields.pyx":1752
 *                     kk = k // 2
 *                     ii = i // 2
 *                     jj = j // 2             # <<<<<<<<<<<<<<
 *                     down[kk, ii, jj, 0] += field[k, i, j, 0]
 *                     down[kk, ii, jj, 1] += field[k, i, j, 1]
 */
              __pyx_v_jj = (__pyx_v_j / 2);

              /* "dipy/align/vector_fields.pyx":1753
 *                     ii = i // 2
 *                     jj = j // 2
 *                     down[kk, ii, jj, 0] += field[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                     down[kk, ii, jj, 1] += field[k, i, j, 1]
 *                     down[kk, ii, jj, 2] += field[k, i, j, 2]
 */
              __pyx_t_14 = __pyx_v_k;
              __pyx_t_15 = __pyx_v_i;
              __pyx_t_16 = __pyx_v_j;
              __pyx_t_17 = 0;
              __pyx_t_18 = __pyx_v_kk;
              __pyx_t_19 = __pyx_v_ii;
              __pyx_t_20 = __pyx_v_jj;
              __pyx_t_21 = 0;
              *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_18 * __pyx_v_down.strides[0]) ) + __pyx_t_19 * __pyx_v_down.strides[1]) ) + __pyx_t_20 * __pyx_v_down.strides[2]) ) + __pyx_t_21 * __pyx_v_down.strides[3]) )) += (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_14 * __pyx_v_field.strides[0]) ) + __pyx_t_15 * __pyx_v_field.strides[1]) ) + __pyx_t_16 * __pyx_v_field.strides[2]) ) + __pyx_t_17 * __pyx_v_field.strides[3]) )));

              /* "dipy/align/vector_fields.pyx":1754
 *                     jj = j // 2
 *                     down[kk, ii, jj, 0] += field[k, i, j, 0]
 *                     down[kk, ii, jj, 1] += field[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                     down[kk, ii, jj, 2] += field[k, i, j, 2]
 *                     cnt[kk, ii, jj] += 1
 */
              __pyx_t_22 = __pyx_v_k;
              __pyx_t_23 = __pyx_v_i;
              __pyx_t_24 = __pyx_v_j;
              __pyx_t_25 = 1;
              __pyx_t_26 = __pyx_v_kk;
              __pyx_t_27 = __pyx_v_ii;
              __pyx_t_28 = __pyx_v_jj;
              __pyx_t_29 = 1;
              *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_26 * __pyx_v_down.strides[0]) ) + __pyx_t_27 * __pyx_v_down.strides[1]) ) + __pyx_t_28 * __pyx_v_down.strides[2]) ) + __pyx_t_29 * __pyx_v_down.strides[3]) )) += (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_22 * __pyx_v_field.strides[0]) ) + __pyx_t_23 * __pyx_v_field.strides[1]) ) + __pyx_t_24 * __pyx_v_field.strides[2]) ) + __pyx_t_25 * __pyx_v_field.strides[3]) )));

              /* "dipy/align/vector_fields.pyx":1755
 *                     down[kk, ii, jj, 0] += field[k, i, j, 0]
 *                     down[kk, ii, jj, 1] += field[k, i, j, 1]
 *                     down[kk, ii, jj, 2] += field[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     cnt[kk, ii, jj] += 1
 *         for k in range(nns):
 */
              __pyx_t_30 = __pyx_v_k;
              __pyx_t_31 = __pyx_v_i;
              __pyx_t_32 = __pyx_v_j;
              __pyx_t_33 = 2;
              __pyx_t_34 = __pyx_v_kk;
              __pyx_t_35 = __pyx_v_ii;
              __pyx_t_36 = __pyx_v_jj;
              __pyx_t_37 = 2;
              *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_34 * __pyx_v_down.strides[0]) ) + __pyx_t_35 * __pyx_v_down.strides[1]) ) + __pyx_t_36 * __pyx_v_down.strides[2]) ) + __pyx_t_37 * __pyx_v_down.strides[3]) )) += (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_30 * __pyx_v_field.strides[0]) ) + __pyx_t_31 * __pyx_v_field.strides[1]) ) + __pyx_t_32 * __pyx_v_field.strides[2]) ) + __pyx_t_33 * __pyx_v_field.strides[3]) )));

              /* "dipy/align/vector_fields.pyx":1756
 *                     down[kk, ii, jj, 1] += field[k, i, j, 1]
 *                     down[kk, ii, jj, 2] += field[k, i, j, 2]
 *                     cnt[kk, ii, jj] += 1             # <<<<<<<<<<<<<<
 *         for k in range(nns):
 *             for i in range(nnr):
 */
              __pyx_t_38 = __pyx_v_kk;
              __pyx_t_39 = __pyx_v_ii;
              __pyx_t_40 = __pyx_v_jj;
              *((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_38 * __pyx_v_cnt.strides[0]) ) + __pyx_t_39 * __pyx_v_cnt.strides[1]) ) + __pyx_t_40 * __pyx_v_cnt.strides[2]) )) += 1;
            }
          }
        }

        /* "dipy/align/vector_fields.pyx":1757
 *                     down[kk, ii, jj, 2] += field[k, i, j, 2]
 *                     cnt[kk, ii, jj] += 1
 *         for k in range(nns):             # <<<<<<<<<<<<<<
 *             for i in range(nnr):
 *                 for j in range(nnc):
 */
        __pyx_t_8 = __pyx_v_nns;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_k = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":1758
 *                     cnt[kk, ii, jj] += 1
 *         for k in range(nns):
 *             for i in range(nnr):             # <<<<<<<<<<<<<<
 *                 for j in range(nnc):
 *                     if cnt[k, i, j] > 0:
 */
          __pyx_t_10 = __pyx_v_nnr;
          for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
            __pyx_v_i = __pyx_t_11;

            /* "dipy/align/vector_fields.pyx":1759
 *         for k in range(nns):
 *             for i in range(nnr):
 *                 for j in range(nnc):             # <<<<<<<<<<<<<<
 *                     if cnt[k, i, j] > 0:
 *                         down[k, i, j, 0] /= cnt[k, i, j]
 */
            __pyx_t_12 = __pyx_v_nnc;
            for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
              __pyx_v_j = __pyx_t_13;

              /* "dipy/align/vector_fields.pyx":1760
 *             for i in range(nnr):
 *                 for j in range(nnc):
 *                     if cnt[k, i, j] > 0:             # <<<<<<<<<<<<<<
 *                         down[k, i, j, 0] /= cnt[k, i, j]
 *                         down[k, i, j, 1] /= cnt[k, i, j]
 */
              __pyx_t_41 = __pyx_v_k;
              __pyx_t_42 = __pyx_v_i;
              __pyx_t_43 = __pyx_v_j;
              __pyx_t_44 = (((*((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_41 * __pyx_v_cnt.strides[0]) ) + __pyx_t_42 * __pyx_v_cnt.strides[1]) ) + __pyx_t_43 * __pyx_v_cnt.strides[2]) ))) > 0) != 0);
              if (__pyx_t_44) {

                /* "dipy/align/vector_fields.pyx":1761
 *                 for j in range(nnc):
 *                     if cnt[k, i, j] > 0:
 *                         down[k, i, j, 0] /= cnt[k, i, j]             # <<<<<<<<<<<<<<
 *                         down[k, i, j, 1] /= cnt[k, i, j]
 *                         down[k, i, j, 2] /= cnt[k, i, j]
 */
                __pyx_t_45 = __pyx_v_k;
                __pyx_t_46 = __pyx_v_i;
                __pyx_t_47 = __pyx_v_j;
                __pyx_t_48 = __pyx_v_k;
                __pyx_t_49 = __pyx_v_i;
                __pyx_t_50 = __pyx_v_j;
                __pyx_t_51 = 0;
                *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_48 * __pyx_v_down.strides[0]) ) + __pyx_t_49 * __pyx_v_down.strides[1]) ) + __pyx_t_50 * __pyx_v_down.strides[2]) ) + __pyx_t_51 * __pyx_v_down.strides[3]) )) /= (*((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_45 * __pyx_v_cnt.strides[0]) ) + __pyx_t_46 * __pyx_v_cnt.strides[1]) ) + __pyx_t_47 * __pyx_v_cnt.strides[2]) )));

                /* "dipy/align/vector_fields.pyx":1762
 *                     if cnt[k, i, j] > 0:
 *                         down[k, i, j, 0] /= cnt[k, i, j]
 *                         down[k, i, j, 1] /= cnt[k, i, j]             # <<<<<<<<<<<<<<
 *                         down[k, i, j, 2] /= cnt[k, i, j]
 *     return down
 */
                __pyx_t_52 = __pyx_v_k;
                __pyx_t_53 = __pyx_v_i;
                __pyx_t_54 = __pyx_v_j;
                __pyx_t_55 = __pyx_v_k;
                __pyx_t_56 = __pyx_v_i;
                __pyx_t_57 = __pyx_v_j;
                __pyx_t_58 = 1;
                *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_55 * __pyx_v_down.strides[0]) ) + __pyx_t_56 * __pyx_v_down.strides[1]) ) + __pyx_t_57 * __pyx_v_down.strides[2]) ) + __pyx_t_58 * __pyx_v_down.strides[3]) )) /= (*((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_52 * __pyx_v_cnt.strides[0]) ) + __pyx_t_53 * __pyx_v_cnt.strides[1]) ) + __pyx_t_54 * __pyx_v_cnt.strides[2]) )));

                /* "dipy/align/vector_fields.pyx":1763
 *                         down[k, i, j, 0] /= cnt[k, i, j]
 *                         down[k, i, j, 1] /= cnt[k, i, j]
 *                         down[k, i, j, 2] /= cnt[k, i, j]             # <<<<<<<<<<<<<<
 *     return down
 * 
 */
                __pyx_t_59 = __pyx_v_k;
                __pyx_t_60 = __pyx_v_i;
                __pyx_t_61 = __pyx_v_j;
                __pyx_t_62 = __pyx_v_k;
                __pyx_t_63 = __pyx_v_i;
                __pyx_t_64 = __pyx_v_j;
                __pyx_t_65 = 2;
                *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_62 * __pyx_v_down.strides[0]) ) + __pyx_t_63 * __pyx_v_down.strides[1]) ) + __pyx_t_64 * __pyx_v_down.strides[2]) ) + __pyx_t_65 * __pyx_v_down.strides[3]) )) /= (*((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_59 * __pyx_v_cnt.strides[0]) ) + __pyx_t_60 * __pyx_v_cnt.strides[1]) ) + __pyx_t_61 * __pyx_v_cnt.strides[2]) )));

                /* "dipy/align/vector_fields.pyx":1760
 *             for i in range(nnr):
 *                 for j in range(nnc):
 *                     if cnt[k, i, j] > 0:             # <<<<<<<<<<<<<<
 *                         down[k, i, j, 0] /= cnt[k, i, j]
 *                         down[k, i, j, 1] /= cnt[k, i, j]
 */
              }
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1745
 *         int[:, :, :] cnt = np.zeros((nns, nnr, nnc), dtype=np.int32)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(ns):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1764
 *                         down[k, i, j, 1] /= cnt[k, i, j]
 *                         down[k, i, j, 2] /= cnt[k, i, j]
 *     return down             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_down, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1764; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1715
 * 
 * 
 * def downsample_displacement_field_3d(floating[:, :, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input 3D vector field by a factor of 2
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.downsample_displacement_field_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_field, 1);
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_down, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_cnt, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_195downsample_displacement_field_3d(PyObject *__pyx_self, PyObject *__pyx_arg_field); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_195downsample_displacement_field_3d = {"__pyx_fuse_1downsample_displacement_field_3d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_195downsample_displacement_field_3d, METH_O, __pyx_doc_4dipy_5align_13vector_fields_32downsample_displacement_field_3d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_195downsample_displacement_field_3d(PyObject *__pyx_self, PyObject *__pyx_arg_field) {
  __Pyx_memviewslice __pyx_v_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("downsample_displacement_field_3d (wrapper)", 0);
  assert(__pyx_arg_field); {
    __pyx_v_field = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_arg_field); if (unlikely(!__pyx_v_field.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.downsample_displacement_field_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_194downsample_displacement_field_3d(__pyx_self, __pyx_v_field);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_194downsample_displacement_field_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_ns;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_nns;
  npy_intp __pyx_v_nnr;
  npy_intp __pyx_v_nnc;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  npy_intp __pyx_v_kk;
  __Pyx_memviewslice __pyx_v_down = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_cnt = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  npy_intp __pyx_t_12;
  npy_intp __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  int __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  Py_ssize_t __pyx_t_60;
  Py_ssize_t __pyx_t_61;
  Py_ssize_t __pyx_t_62;
  Py_ssize_t __pyx_t_63;
  Py_ssize_t __pyx_t_64;
  Py_ssize_t __pyx_t_65;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1downsample_displacement_field_3d", 0);

  /* "dipy/align/vector_fields.pyx":1733
 *         R'= ceil(R/2), C'=ceil(C/2)
 *     """
 *     ftype = np.asarray(field).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp ns = field.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1733; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1733; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_field, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1733; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1733; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1733; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1733; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1733; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":1735
 *     ftype = np.asarray(field).dtype
 *     cdef:
 *         cnp.npy_intp ns = field.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nr = field.shape[1]
 *         cnp.npy_intp nc = field.shape[2]
 */
  __pyx_v_ns = (__pyx_v_field.shape[0]);

  /* "dipy/align/vector_fields.pyx":1736
 *     cdef:
 *         cnp.npy_intp ns = field.shape[0]
 *         cnp.npy_intp nr = field.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = field.shape[2]
 *         cnp.npy_intp nns = (ns + 1) // 2
 */
  __pyx_v_nr = (__pyx_v_field.shape[1]);

  /* "dipy/align/vector_fields.pyx":1737
 *         cnp.npy_intp ns = field.shape[0]
 *         cnp.npy_intp nr = field.shape[1]
 *         cnp.npy_intp nc = field.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nns = (ns + 1) // 2
 *         cnp.npy_intp nnr = (nr + 1) // 2
 */
  __pyx_v_nc = (__pyx_v_field.shape[2]);

  /* "dipy/align/vector_fields.pyx":1738
 *         cnp.npy_intp nr = field.shape[1]
 *         cnp.npy_intp nc = field.shape[2]
 *         cnp.npy_intp nns = (ns + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nnr = (nr + 1) // 2
 *         cnp.npy_intp nnc = (nc + 1) // 2
 */
  __pyx_v_nns = ((__pyx_v_ns + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1739
 *         cnp.npy_intp nc = field.shape[2]
 *         cnp.npy_intp nns = (ns + 1) // 2
 *         cnp.npy_intp nnr = (nr + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nnc = (nc + 1) // 2
 *         cnp.npy_intp i, j, k, ii, jj, kk
 */
  __pyx_v_nnr = ((__pyx_v_nr + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1740
 *         cnp.npy_intp nns = (ns + 1) // 2
 *         cnp.npy_intp nnr = (nr + 1) // 2
 *         cnp.npy_intp nnc = (nc + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         floating[:, :, :, :] down = np.zeros((nns, nnr, nnc, 3), dtype=ftype)
 */
  __pyx_v_nnc = ((__pyx_v_nc + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1742
 *         cnp.npy_intp nnc = (nc + 1) // 2
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         floating[:, :, :, :] down = np.zeros((nns, nnr, nnc, 3), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:, :, :] cnt = np.zeros((nns, nnr, nnc), dtype=np.int32)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nns); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnr); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnc); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_2);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_t_5);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1742; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_down = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":1743
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         floating[:, :, :, :] down = np.zeros((nns, nnr, nnc, 3), dtype=ftype)
 *         int[:, :, :] cnt = np.zeros((nns, nnr, nnc), dtype=np.int32)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nns); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnr); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnc); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_1);
  __pyx_t_5 = 0;
  __pyx_t_2 = 0;
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_int32); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(__pyx_t_5);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1743; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_cnt = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":1745
 *         int[:, :, :] cnt = np.zeros((nns, nnr, nnc), dtype=np.int32)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(ns):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1747
 *     with nogil:
 * 
 *         for k in range(ns):             # <<<<<<<<<<<<<<
 *             for i in range(nr):
 *                 for j in range(nc):
 */
        __pyx_t_8 = __pyx_v_ns;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_k = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":1748
 * 
 *         for k in range(ns):
 *             for i in range(nr):             # <<<<<<<<<<<<<<
 *                 for j in range(nc):
 *                     kk = k // 2
 */
          __pyx_t_10 = __pyx_v_nr;
          for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
            __pyx_v_i = __pyx_t_11;

            /* "dipy/align/vector_fields.pyx":1749
 *         for k in range(ns):
 *             for i in range(nr):
 *                 for j in range(nc):             # <<<<<<<<<<<<<<
 *                     kk = k // 2
 *                     ii = i // 2
 */
            __pyx_t_12 = __pyx_v_nc;
            for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
              __pyx_v_j = __pyx_t_13;

              /* "dipy/align/vector_fields.pyx":1750
 *             for i in range(nr):
 *                 for j in range(nc):
 *                     kk = k // 2             # <<<<<<<<<<<<<<
 *                     ii = i // 2
 *                     jj = j // 2
 */
              __pyx_v_kk = (__pyx_v_k / 2);

              /* "dipy/align/vector_fields.pyx":1751
 *                 for j in range(nc):
 *                     kk = k // 2
 *                     ii = i // 2             # <<<<<<<<<<<<<<
 *                     jj = j // 2
 *                     down[kk, ii, jj, 0] += field[k, i, j, 0]
 */
              __pyx_v_ii = (__pyx_v_i / 2);

              /* "dipy/align/vector_fields.pyx":1752
 *                     kk = k // 2
 *                     ii = i // 2
 *                     jj = j // 2             # <<<<<<<<<<<<<<
 *                     down[kk, ii, jj, 0] += field[k, i, j, 0]
 *                     down[kk, ii, jj, 1] += field[k, i, j, 1]
 */
              __pyx_v_jj = (__pyx_v_j / 2);

              /* "dipy/align/vector_fields.pyx":1753
 *                     ii = i // 2
 *                     jj = j // 2
 *                     down[kk, ii, jj, 0] += field[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                     down[kk, ii, jj, 1] += field[k, i, j, 1]
 *                     down[kk, ii, jj, 2] += field[k, i, j, 2]
 */
              __pyx_t_14 = __pyx_v_k;
              __pyx_t_15 = __pyx_v_i;
              __pyx_t_16 = __pyx_v_j;
              __pyx_t_17 = 0;
              __pyx_t_18 = __pyx_v_kk;
              __pyx_t_19 = __pyx_v_ii;
              __pyx_t_20 = __pyx_v_jj;
              __pyx_t_21 = 0;
              *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_18 * __pyx_v_down.strides[0]) ) + __pyx_t_19 * __pyx_v_down.strides[1]) ) + __pyx_t_20 * __pyx_v_down.strides[2]) ) + __pyx_t_21 * __pyx_v_down.strides[3]) )) += (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_14 * __pyx_v_field.strides[0]) ) + __pyx_t_15 * __pyx_v_field.strides[1]) ) + __pyx_t_16 * __pyx_v_field.strides[2]) ) + __pyx_t_17 * __pyx_v_field.strides[3]) )));

              /* "dipy/align/vector_fields.pyx":1754
 *                     jj = j // 2
 *                     down[kk, ii, jj, 0] += field[k, i, j, 0]
 *                     down[kk, ii, jj, 1] += field[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                     down[kk, ii, jj, 2] += field[k, i, j, 2]
 *                     cnt[kk, ii, jj] += 1
 */
              __pyx_t_22 = __pyx_v_k;
              __pyx_t_23 = __pyx_v_i;
              __pyx_t_24 = __pyx_v_j;
              __pyx_t_25 = 1;
              __pyx_t_26 = __pyx_v_kk;
              __pyx_t_27 = __pyx_v_ii;
              __pyx_t_28 = __pyx_v_jj;
              __pyx_t_29 = 1;
              *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_26 * __pyx_v_down.strides[0]) ) + __pyx_t_27 * __pyx_v_down.strides[1]) ) + __pyx_t_28 * __pyx_v_down.strides[2]) ) + __pyx_t_29 * __pyx_v_down.strides[3]) )) += (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_22 * __pyx_v_field.strides[0]) ) + __pyx_t_23 * __pyx_v_field.strides[1]) ) + __pyx_t_24 * __pyx_v_field.strides[2]) ) + __pyx_t_25 * __pyx_v_field.strides[3]) )));

              /* "dipy/align/vector_fields.pyx":1755
 *                     down[kk, ii, jj, 0] += field[k, i, j, 0]
 *                     down[kk, ii, jj, 1] += field[k, i, j, 1]
 *                     down[kk, ii, jj, 2] += field[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     cnt[kk, ii, jj] += 1
 *         for k in range(nns):
 */
              __pyx_t_30 = __pyx_v_k;
              __pyx_t_31 = __pyx_v_i;
              __pyx_t_32 = __pyx_v_j;
              __pyx_t_33 = 2;
              __pyx_t_34 = __pyx_v_kk;
              __pyx_t_35 = __pyx_v_ii;
              __pyx_t_36 = __pyx_v_jj;
              __pyx_t_37 = 2;
              *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_34 * __pyx_v_down.strides[0]) ) + __pyx_t_35 * __pyx_v_down.strides[1]) ) + __pyx_t_36 * __pyx_v_down.strides[2]) ) + __pyx_t_37 * __pyx_v_down.strides[3]) )) += (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_30 * __pyx_v_field.strides[0]) ) + __pyx_t_31 * __pyx_v_field.strides[1]) ) + __pyx_t_32 * __pyx_v_field.strides[2]) ) + __pyx_t_33 * __pyx_v_field.strides[3]) )));

              /* "dipy/align/vector_fields.pyx":1756
 *                     down[kk, ii, jj, 1] += field[k, i, j, 1]
 *                     down[kk, ii, jj, 2] += field[k, i, j, 2]
 *                     cnt[kk, ii, jj] += 1             # <<<<<<<<<<<<<<
 *         for k in range(nns):
 *             for i in range(nnr):
 */
              __pyx_t_38 = __pyx_v_kk;
              __pyx_t_39 = __pyx_v_ii;
              __pyx_t_40 = __pyx_v_jj;
              *((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_38 * __pyx_v_cnt.strides[0]) ) + __pyx_t_39 * __pyx_v_cnt.strides[1]) ) + __pyx_t_40 * __pyx_v_cnt.strides[2]) )) += 1;
            }
          }
        }

        /* "dipy/align/vector_fields.pyx":1757
 *                     down[kk, ii, jj, 2] += field[k, i, j, 2]
 *                     cnt[kk, ii, jj] += 1
 *         for k in range(nns):             # <<<<<<<<<<<<<<
 *             for i in range(nnr):
 *                 for j in range(nnc):
 */
        __pyx_t_8 = __pyx_v_nns;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_k = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":1758
 *                     cnt[kk, ii, jj] += 1
 *         for k in range(nns):
 *             for i in range(nnr):             # <<<<<<<<<<<<<<
 *                 for j in range(nnc):
 *                     if cnt[k, i, j] > 0:
 */
          __pyx_t_10 = __pyx_v_nnr;
          for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
            __pyx_v_i = __pyx_t_11;

            /* "dipy/align/vector_fields.pyx":1759
 *         for k in range(nns):
 *             for i in range(nnr):
 *                 for j in range(nnc):             # <<<<<<<<<<<<<<
 *                     if cnt[k, i, j] > 0:
 *                         down[k, i, j, 0] /= cnt[k, i, j]
 */
            __pyx_t_12 = __pyx_v_nnc;
            for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
              __pyx_v_j = __pyx_t_13;

              /* "dipy/align/vector_fields.pyx":1760
 *             for i in range(nnr):
 *                 for j in range(nnc):
 *                     if cnt[k, i, j] > 0:             # <<<<<<<<<<<<<<
 *                         down[k, i, j, 0] /= cnt[k, i, j]
 *                         down[k, i, j, 1] /= cnt[k, i, j]
 */
              __pyx_t_41 = __pyx_v_k;
              __pyx_t_42 = __pyx_v_i;
              __pyx_t_43 = __pyx_v_j;
              __pyx_t_44 = (((*((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_41 * __pyx_v_cnt.strides[0]) ) + __pyx_t_42 * __pyx_v_cnt.strides[1]) ) + __pyx_t_43 * __pyx_v_cnt.strides[2]) ))) > 0) != 0);
              if (__pyx_t_44) {

                /* "dipy/align/vector_fields.pyx":1761
 *                 for j in range(nnc):
 *                     if cnt[k, i, j] > 0:
 *                         down[k, i, j, 0] /= cnt[k, i, j]             # <<<<<<<<<<<<<<
 *                         down[k, i, j, 1] /= cnt[k, i, j]
 *                         down[k, i, j, 2] /= cnt[k, i, j]
 */
                __pyx_t_45 = __pyx_v_k;
                __pyx_t_46 = __pyx_v_i;
                __pyx_t_47 = __pyx_v_j;
                __pyx_t_48 = __pyx_v_k;
                __pyx_t_49 = __pyx_v_i;
                __pyx_t_50 = __pyx_v_j;
                __pyx_t_51 = 0;
                *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_48 * __pyx_v_down.strides[0]) ) + __pyx_t_49 * __pyx_v_down.strides[1]) ) + __pyx_t_50 * __pyx_v_down.strides[2]) ) + __pyx_t_51 * __pyx_v_down.strides[3]) )) /= (*((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_45 * __pyx_v_cnt.strides[0]) ) + __pyx_t_46 * __pyx_v_cnt.strides[1]) ) + __pyx_t_47 * __pyx_v_cnt.strides[2]) )));

                /* "dipy/align/vector_fields.pyx":1762
 *                     if cnt[k, i, j] > 0:
 *                         down[k, i, j, 0] /= cnt[k, i, j]
 *                         down[k, i, j, 1] /= cnt[k, i, j]             # <<<<<<<<<<<<<<
 *                         down[k, i, j, 2] /= cnt[k, i, j]
 *     return down
 */
                __pyx_t_52 = __pyx_v_k;
                __pyx_t_53 = __pyx_v_i;
                __pyx_t_54 = __pyx_v_j;
                __pyx_t_55 = __pyx_v_k;
                __pyx_t_56 = __pyx_v_i;
                __pyx_t_57 = __pyx_v_j;
                __pyx_t_58 = 1;
                *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_55 * __pyx_v_down.strides[0]) ) + __pyx_t_56 * __pyx_v_down.strides[1]) ) + __pyx_t_57 * __pyx_v_down.strides[2]) ) + __pyx_t_58 * __pyx_v_down.strides[3]) )) /= (*((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_52 * __pyx_v_cnt.strides[0]) ) + __pyx_t_53 * __pyx_v_cnt.strides[1]) ) + __pyx_t_54 * __pyx_v_cnt.strides[2]) )));

                /* "dipy/align/vector_fields.pyx":1763
 *                         down[k, i, j, 0] /= cnt[k, i, j]
 *                         down[k, i, j, 1] /= cnt[k, i, j]
 *                         down[k, i, j, 2] /= cnt[k, i, j]             # <<<<<<<<<<<<<<
 *     return down
 * 
 */
                __pyx_t_59 = __pyx_v_k;
                __pyx_t_60 = __pyx_v_i;
                __pyx_t_61 = __pyx_v_j;
                __pyx_t_62 = __pyx_v_k;
                __pyx_t_63 = __pyx_v_i;
                __pyx_t_64 = __pyx_v_j;
                __pyx_t_65 = 2;
                *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_62 * __pyx_v_down.strides[0]) ) + __pyx_t_63 * __pyx_v_down.strides[1]) ) + __pyx_t_64 * __pyx_v_down.strides[2]) ) + __pyx_t_65 * __pyx_v_down.strides[3]) )) /= (*((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_59 * __pyx_v_cnt.strides[0]) ) + __pyx_t_60 * __pyx_v_cnt.strides[1]) ) + __pyx_t_61 * __pyx_v_cnt.strides[2]) )));

                /* "dipy/align/vector_fields.pyx":1760
 *             for i in range(nnr):
 *                 for j in range(nnc):
 *                     if cnt[k, i, j] > 0:             # <<<<<<<<<<<<<<
 *                         down[k, i, j, 0] /= cnt[k, i, j]
 *                         down[k, i, j, 1] /= cnt[k, i, j]
 */
              }
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1745
 *         int[:, :, :] cnt = np.zeros((nns, nnr, nnc), dtype=np.int32)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(ns):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1764
 *                         down[k, i, j, 1] /= cnt[k, i, j]
 *                         down[k, i, j, 2] /= cnt[k, i, j]
 *     return down             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_down, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1764; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1715
 * 
 * 
 * def downsample_displacement_field_3d(floating[:, :, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input 3D vector field by a factor of 2
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.downsample_displacement_field_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_field, 1);
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_down, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_cnt, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":1767
 * 
 * 
 * def downsample_scalar_field_2d(floating[:, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input 2D image by a factor of 2
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_35downsample_scalar_field_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_34downsample_scalar_field_2d[] = "Down-samples the input 2D image by a factor of 2\n\n    Down-samples the input image by a factor of 2. The value at each pixel\n    of the resulting image is the average of its surrounding pixels in the\n    original image.\n\n    Parameters\n    ----------\n    field : array, shape (R, C)\n        the image to be down-sampled\n\n    Returns\n    -------\n    down : array, shape (R', C')\n        the down-sampled displacement field, where R'= ceil(R/2), C'=ceil(C/2)\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_35downsample_scalar_field_2d = {"downsample_scalar_field_2d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_35downsample_scalar_field_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_34downsample_scalar_field_2d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_35downsample_scalar_field_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_34downsample_scalar_field_2d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_34downsample_scalar_field_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("downsample_scalar_field_2d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_field, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_field); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__115, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__116, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__117, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__118, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_199downsample_scalar_field_2d(PyObject *__pyx_self, PyObject *__pyx_arg_field); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_199downsample_scalar_field_2d = {"__pyx_fuse_0downsample_scalar_field_2d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_199downsample_scalar_field_2d, METH_O, __pyx_doc_4dipy_5align_13vector_fields_34downsample_scalar_field_2d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_199downsample_scalar_field_2d(PyObject *__pyx_self, PyObject *__pyx_arg_field) {
  __Pyx_memviewslice __pyx_v_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("downsample_scalar_field_2d (wrapper)", 0);
  assert(__pyx_arg_field); {
    __pyx_v_field = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_arg_field); if (unlikely(!__pyx_v_field.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.downsample_scalar_field_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_198downsample_scalar_field_2d(__pyx_self, __pyx_v_field);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_198downsample_scalar_field_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_nnr;
  npy_intp __pyx_v_nnc;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  __Pyx_memviewslice __pyx_v_down = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_cnt = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  int __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0downsample_scalar_field_2d", 0);

  /* "dipy/align/vector_fields.pyx":1784
 *         the down-sampled displacement field, where R'= ceil(R/2), C'=ceil(C/2)
 *     """
 *     ftype = np.asarray(field).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp nr = field.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_field, 2, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":1786
 *     ftype = np.asarray(field).dtype
 *     cdef:
 *         cnp.npy_intp nr = field.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = field.shape[1]
 *         cnp.npy_intp nnr = (nr + 1) // 2
 */
  __pyx_v_nr = (__pyx_v_field.shape[0]);

  /* "dipy/align/vector_fields.pyx":1787
 *     cdef:
 *         cnp.npy_intp nr = field.shape[0]
 *         cnp.npy_intp nc = field.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nnr = (nr + 1) // 2
 *         cnp.npy_intp nnc = (nc + 1) // 2
 */
  __pyx_v_nc = (__pyx_v_field.shape[1]);

  /* "dipy/align/vector_fields.pyx":1788
 *         cnp.npy_intp nr = field.shape[0]
 *         cnp.npy_intp nc = field.shape[1]
 *         cnp.npy_intp nnr = (nr + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nnc = (nc + 1) // 2
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nnr = ((__pyx_v_nr + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1789
 *         cnp.npy_intp nc = field.shape[1]
 *         cnp.npy_intp nnr = (nr + 1) // 2
 *         cnp.npy_intp nnc = (nc + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         floating[:, :] down = np.zeros(shape=(nnr, nnc), dtype=ftype)
 */
  __pyx_v_nnc = ((__pyx_v_nc + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1791
 *         cnp.npy_intp nnc = (nc + 1) // 2
 *         cnp.npy_intp i, j, ii, jj
 *         floating[:, :] down = np.zeros(shape=(nnr, nnc), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:, :] cnt = np.zeros(shape=(nnr, nnc), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnr); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnc); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __pyx_t_5 = 0;
  __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_t_4);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_down = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":1792
 *         cnp.npy_intp i, j, ii, jj
 *         floating[:, :] down = np.zeros(shape=(nnr, nnc), dtype=ftype)
 *         int[:, :] cnt = np.zeros(shape=(nnr, nnc), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnr); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnc); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_dsds_int(__pyx_t_2);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_cnt = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":1793
 *         floating[:, :] down = np.zeros(shape=(nnr, nnc), dtype=ftype)
 *         int[:, :] cnt = np.zeros(shape=(nnr, nnc), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nr):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1795
 *     with nogil:
 * 
 *         for i in range(nr):             # <<<<<<<<<<<<<<
 *             for j in range(nc):
 *                 ii = i // 2
 */
        __pyx_t_8 = __pyx_v_nr;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":1796
 * 
 *         for i in range(nr):
 *             for j in range(nc):             # <<<<<<<<<<<<<<
 *                 ii = i // 2
 *                 jj = j // 2
 */
          __pyx_t_10 = __pyx_v_nc;
          for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
            __pyx_v_j = __pyx_t_11;

            /* "dipy/align/vector_fields.pyx":1797
 *         for i in range(nr):
 *             for j in range(nc):
 *                 ii = i // 2             # <<<<<<<<<<<<<<
 *                 jj = j // 2
 *                 down[ii, jj] += field[i, j]
 */
            __pyx_v_ii = (__pyx_v_i / 2);

            /* "dipy/align/vector_fields.pyx":1798
 *             for j in range(nc):
 *                 ii = i // 2
 *                 jj = j // 2             # <<<<<<<<<<<<<<
 *                 down[ii, jj] += field[i, j]
 *                 cnt[ii, jj] += 1
 */
            __pyx_v_jj = (__pyx_v_j / 2);

            /* "dipy/align/vector_fields.pyx":1799
 *                 ii = i // 2
 *                 jj = j // 2
 *                 down[ii, jj] += field[i, j]             # <<<<<<<<<<<<<<
 *                 cnt[ii, jj] += 1
 *         for i in range(nnr):
 */
            __pyx_t_12 = __pyx_v_i;
            __pyx_t_13 = __pyx_v_j;
            __pyx_t_14 = __pyx_v_ii;
            __pyx_t_15 = __pyx_v_jj;
            *((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_14 * __pyx_v_down.strides[0]) ) + __pyx_t_15 * __pyx_v_down.strides[1]) )) += (*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_12 * __pyx_v_field.strides[0]) ) + __pyx_t_13 * __pyx_v_field.strides[1]) )));

            /* "dipy/align/vector_fields.pyx":1800
 *                 jj = j // 2
 *                 down[ii, jj] += field[i, j]
 *                 cnt[ii, jj] += 1             # <<<<<<<<<<<<<<
 *         for i in range(nnr):
 *             for j in range(nnc):
 */
            __pyx_t_16 = __pyx_v_ii;
            __pyx_t_17 = __pyx_v_jj;
            *((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_16 * __pyx_v_cnt.strides[0]) ) + __pyx_t_17 * __pyx_v_cnt.strides[1]) )) += 1;
          }
        }

        /* "dipy/align/vector_fields.pyx":1801
 *                 down[ii, jj] += field[i, j]
 *                 cnt[ii, jj] += 1
 *         for i in range(nnr):             # <<<<<<<<<<<<<<
 *             for j in range(nnc):
 *                 if cnt[i, j] > 0:
 */
        __pyx_t_8 = __pyx_v_nnr;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":1802
 *                 cnt[ii, jj] += 1
 *         for i in range(nnr):
 *             for j in range(nnc):             # <<<<<<<<<<<<<<
 *                 if cnt[i, j] > 0:
 *                     down[i, j] /= cnt[i, j]
 */
          __pyx_t_10 = __pyx_v_nnc;
          for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
            __pyx_v_j = __pyx_t_11;

            /* "dipy/align/vector_fields.pyx":1803
 *         for i in range(nnr):
 *             for j in range(nnc):
 *                 if cnt[i, j] > 0:             # <<<<<<<<<<<<<<
 *                     down[i, j] /= cnt[i, j]
 *     return down
 */
            __pyx_t_18 = __pyx_v_i;
            __pyx_t_19 = __pyx_v_j;
            __pyx_t_20 = (((*((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_18 * __pyx_v_cnt.strides[0]) ) + __pyx_t_19 * __pyx_v_cnt.strides[1]) ))) > 0) != 0);
            if (__pyx_t_20) {

              /* "dipy/align/vector_fields.pyx":1804
 *             for j in range(nnc):
 *                 if cnt[i, j] > 0:
 *                     down[i, j] /= cnt[i, j]             # <<<<<<<<<<<<<<
 *     return down
 * 
 */
              __pyx_t_21 = __pyx_v_i;
              __pyx_t_22 = __pyx_v_j;
              __pyx_t_23 = __pyx_v_i;
              __pyx_t_24 = __pyx_v_j;
              *((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_23 * __pyx_v_down.strides[0]) ) + __pyx_t_24 * __pyx_v_down.strides[1]) )) /= (*((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_21 * __pyx_v_cnt.strides[0]) ) + __pyx_t_22 * __pyx_v_cnt.strides[1]) )));

              /* "dipy/align/vector_fields.pyx":1803
 *         for i in range(nnr):
 *             for j in range(nnc):
 *                 if cnt[i, j] > 0:             # <<<<<<<<<<<<<<
 *                     down[i, j] /= cnt[i, j]
 *     return down
 */
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1793
 *         floating[:, :] down = np.zeros(shape=(nnr, nnc), dtype=ftype)
 *         int[:, :] cnt = np.zeros(shape=(nnr, nnc), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nr):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1805
 *                 if cnt[i, j] > 0:
 *                     down[i, j] /= cnt[i, j]
 *     return down             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_down, 2, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1805; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1767
 * 
 * 
 * def downsample_scalar_field_2d(floating[:, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input 2D image by a factor of 2
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.downsample_scalar_field_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_field, 1);
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_down, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_cnt, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_201downsample_scalar_field_2d(PyObject *__pyx_self, PyObject *__pyx_arg_field); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_201downsample_scalar_field_2d = {"__pyx_fuse_1downsample_scalar_field_2d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_201downsample_scalar_field_2d, METH_O, __pyx_doc_4dipy_5align_13vector_fields_34downsample_scalar_field_2d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_201downsample_scalar_field_2d(PyObject *__pyx_self, PyObject *__pyx_arg_field) {
  __Pyx_memviewslice __pyx_v_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("downsample_scalar_field_2d (wrapper)", 0);
  assert(__pyx_arg_field); {
    __pyx_v_field = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_arg_field); if (unlikely(!__pyx_v_field.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.downsample_scalar_field_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_200downsample_scalar_field_2d(__pyx_self, __pyx_v_field);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_200downsample_scalar_field_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_nnr;
  npy_intp __pyx_v_nnc;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  __Pyx_memviewslice __pyx_v_down = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_cnt = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  int __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1downsample_scalar_field_2d", 0);

  /* "dipy/align/vector_fields.pyx":1784
 *         the down-sampled displacement field, where R'= ceil(R/2), C'=ceil(C/2)
 *     """
 *     ftype = np.asarray(field).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp nr = field.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_field, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1784; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":1786
 *     ftype = np.asarray(field).dtype
 *     cdef:
 *         cnp.npy_intp nr = field.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = field.shape[1]
 *         cnp.npy_intp nnr = (nr + 1) // 2
 */
  __pyx_v_nr = (__pyx_v_field.shape[0]);

  /* "dipy/align/vector_fields.pyx":1787
 *     cdef:
 *         cnp.npy_intp nr = field.shape[0]
 *         cnp.npy_intp nc = field.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nnr = (nr + 1) // 2
 *         cnp.npy_intp nnc = (nc + 1) // 2
 */
  __pyx_v_nc = (__pyx_v_field.shape[1]);

  /* "dipy/align/vector_fields.pyx":1788
 *         cnp.npy_intp nr = field.shape[0]
 *         cnp.npy_intp nc = field.shape[1]
 *         cnp.npy_intp nnr = (nr + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nnc = (nc + 1) // 2
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nnr = ((__pyx_v_nr + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1789
 *         cnp.npy_intp nc = field.shape[1]
 *         cnp.npy_intp nnr = (nr + 1) // 2
 *         cnp.npy_intp nnc = (nc + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         floating[:, :] down = np.zeros(shape=(nnr, nnc), dtype=ftype)
 */
  __pyx_v_nnc = ((__pyx_v_nc + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1791
 *         cnp.npy_intp nnc = (nc + 1) // 2
 *         cnp.npy_intp i, j, ii, jj
 *         floating[:, :] down = np.zeros(shape=(nnr, nnc), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:, :] cnt = np.zeros(shape=(nnr, nnc), dtype=np.int32)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnr); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnc); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __pyx_t_5 = 0;
  __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1791; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_down = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":1792
 *         cnp.npy_intp i, j, ii, jj
 *         floating[:, :] down = np.zeros(shape=(nnr, nnc), dtype=ftype)
 *         int[:, :] cnt = np.zeros(shape=(nnr, nnc), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     with nogil:
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnr); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnc); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_int32); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_dsds_int(__pyx_t_2);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1792; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_cnt = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":1793
 *         floating[:, :] down = np.zeros(shape=(nnr, nnc), dtype=ftype)
 *         int[:, :] cnt = np.zeros(shape=(nnr, nnc), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nr):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1795
 *     with nogil:
 * 
 *         for i in range(nr):             # <<<<<<<<<<<<<<
 *             for j in range(nc):
 *                 ii = i // 2
 */
        __pyx_t_8 = __pyx_v_nr;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":1796
 * 
 *         for i in range(nr):
 *             for j in range(nc):             # <<<<<<<<<<<<<<
 *                 ii = i // 2
 *                 jj = j // 2
 */
          __pyx_t_10 = __pyx_v_nc;
          for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
            __pyx_v_j = __pyx_t_11;

            /* "dipy/align/vector_fields.pyx":1797
 *         for i in range(nr):
 *             for j in range(nc):
 *                 ii = i // 2             # <<<<<<<<<<<<<<
 *                 jj = j // 2
 *                 down[ii, jj] += field[i, j]
 */
            __pyx_v_ii = (__pyx_v_i / 2);

            /* "dipy/align/vector_fields.pyx":1798
 *             for j in range(nc):
 *                 ii = i // 2
 *                 jj = j // 2             # <<<<<<<<<<<<<<
 *                 down[ii, jj] += field[i, j]
 *                 cnt[ii, jj] += 1
 */
            __pyx_v_jj = (__pyx_v_j / 2);

            /* "dipy/align/vector_fields.pyx":1799
 *                 ii = i // 2
 *                 jj = j // 2
 *                 down[ii, jj] += field[i, j]             # <<<<<<<<<<<<<<
 *                 cnt[ii, jj] += 1
 *         for i in range(nnr):
 */
            __pyx_t_12 = __pyx_v_i;
            __pyx_t_13 = __pyx_v_j;
            __pyx_t_14 = __pyx_v_ii;
            __pyx_t_15 = __pyx_v_jj;
            *((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_14 * __pyx_v_down.strides[0]) ) + __pyx_t_15 * __pyx_v_down.strides[1]) )) += (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_12 * __pyx_v_field.strides[0]) ) + __pyx_t_13 * __pyx_v_field.strides[1]) )));

            /* "dipy/align/vector_fields.pyx":1800
 *                 jj = j // 2
 *                 down[ii, jj] += field[i, j]
 *                 cnt[ii, jj] += 1             # <<<<<<<<<<<<<<
 *         for i in range(nnr):
 *             for j in range(nnc):
 */
            __pyx_t_16 = __pyx_v_ii;
            __pyx_t_17 = __pyx_v_jj;
            *((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_16 * __pyx_v_cnt.strides[0]) ) + __pyx_t_17 * __pyx_v_cnt.strides[1]) )) += 1;
          }
        }

        /* "dipy/align/vector_fields.pyx":1801
 *                 down[ii, jj] += field[i, j]
 *                 cnt[ii, jj] += 1
 *         for i in range(nnr):             # <<<<<<<<<<<<<<
 *             for j in range(nnc):
 *                 if cnt[i, j] > 0:
 */
        __pyx_t_8 = __pyx_v_nnr;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":1802
 *                 cnt[ii, jj] += 1
 *         for i in range(nnr):
 *             for j in range(nnc):             # <<<<<<<<<<<<<<
 *                 if cnt[i, j] > 0:
 *                     down[i, j] /= cnt[i, j]
 */
          __pyx_t_10 = __pyx_v_nnc;
          for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
            __pyx_v_j = __pyx_t_11;

            /* "dipy/align/vector_fields.pyx":1803
 *         for i in range(nnr):
 *             for j in range(nnc):
 *                 if cnt[i, j] > 0:             # <<<<<<<<<<<<<<
 *                     down[i, j] /= cnt[i, j]
 *     return down
 */
            __pyx_t_18 = __pyx_v_i;
            __pyx_t_19 = __pyx_v_j;
            __pyx_t_20 = (((*((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_18 * __pyx_v_cnt.strides[0]) ) + __pyx_t_19 * __pyx_v_cnt.strides[1]) ))) > 0) != 0);
            if (__pyx_t_20) {

              /* "dipy/align/vector_fields.pyx":1804
 *             for j in range(nnc):
 *                 if cnt[i, j] > 0:
 *                     down[i, j] /= cnt[i, j]             # <<<<<<<<<<<<<<
 *     return down
 * 
 */
              __pyx_t_21 = __pyx_v_i;
              __pyx_t_22 = __pyx_v_j;
              __pyx_t_23 = __pyx_v_i;
              __pyx_t_24 = __pyx_v_j;
              *((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_23 * __pyx_v_down.strides[0]) ) + __pyx_t_24 * __pyx_v_down.strides[1]) )) /= (*((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_21 * __pyx_v_cnt.strides[0]) ) + __pyx_t_22 * __pyx_v_cnt.strides[1]) )));

              /* "dipy/align/vector_fields.pyx":1803
 *         for i in range(nnr):
 *             for j in range(nnc):
 *                 if cnt[i, j] > 0:             # <<<<<<<<<<<<<<
 *                     down[i, j] /= cnt[i, j]
 *     return down
 */
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1793
 *         floating[:, :] down = np.zeros(shape=(nnr, nnc), dtype=ftype)
 *         int[:, :] cnt = np.zeros(shape=(nnr, nnc), dtype=np.int32)
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nr):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1805
 *                 if cnt[i, j] > 0:
 *                     down[i, j] /= cnt[i, j]
 *     return down             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_down, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1805; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1767
 * 
 * 
 * def downsample_scalar_field_2d(floating[:, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input 2D image by a factor of 2
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.downsample_scalar_field_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_field, 1);
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_down, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_cnt, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":1808
 * 
 * 
 * def downsample_displacement_field_2d(floating[:, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the 2D input vector field by a factor of 2
 *     Down-samples the input vector field by a factor of 2. The value at each
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_37downsample_displacement_field_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_36downsample_displacement_field_2d[] = "Down-samples the 2D input vector field by a factor of 2\n    Down-samples the input vector field by a factor of 2. The value at each\n    pixel of the resulting field is the average of its surrounding pixels in\n    the original field.\n\n    Parameters\n    ----------\n    field : array, shape (R, C)\n        the vector field to be down-sampled\n\n    Returns\n    -------\n    down : array, shape (R', C')\n        the down-sampled displacement field, where R'= ceil(R/2), C'=ceil(C/2),\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_37downsample_displacement_field_2d = {"downsample_displacement_field_2d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_37downsample_displacement_field_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_36downsample_displacement_field_2d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_37downsample_displacement_field_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_36downsample_displacement_field_2d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_36downsample_displacement_field_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("downsample_displacement_field_2d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_field, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_field); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__119, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__120, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__121, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__122, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_205downsample_displacement_field_2d(PyObject *__pyx_self, PyObject *__pyx_arg_field); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_205downsample_displacement_field_2d = {"__pyx_fuse_0downsample_displacement_field_2d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_205downsample_displacement_field_2d, METH_O, __pyx_doc_4dipy_5align_13vector_fields_36downsample_displacement_field_2d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_205downsample_displacement_field_2d(PyObject *__pyx_self, PyObject *__pyx_arg_field) {
  __Pyx_memviewslice __pyx_v_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("downsample_displacement_field_2d (wrapper)", 0);
  assert(__pyx_arg_field); {
    __pyx_v_field = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_arg_field); if (unlikely(!__pyx_v_field.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.downsample_displacement_field_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_204downsample_displacement_field_2d(__pyx_self, __pyx_v_field);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_204downsample_displacement_field_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_nnr;
  npy_intp __pyx_v_nnc;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  __Pyx_memviewslice __pyx_v_down = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_cnt = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  int __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0downsample_displacement_field_2d", 0);

  /* "dipy/align/vector_fields.pyx":1824
 *         the down-sampled displacement field, where R'= ceil(R/2), C'=ceil(C/2),
 *     """
 *     ftype = np.asarray(field).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp nr = field.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1824; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1824; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_field, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1824; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1824; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1824; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1824; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1824; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":1826
 *     ftype = np.asarray(field).dtype
 *     cdef:
 *         cnp.npy_intp nr = field.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = field.shape[1]
 *         cnp.npy_intp nnr = (nr + 1) // 2
 */
  __pyx_v_nr = (__pyx_v_field.shape[0]);

  /* "dipy/align/vector_fields.pyx":1827
 *     cdef:
 *         cnp.npy_intp nr = field.shape[0]
 *         cnp.npy_intp nc = field.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nnr = (nr + 1) // 2
 *         cnp.npy_intp nnc = (nc + 1) // 2
 */
  __pyx_v_nc = (__pyx_v_field.shape[1]);

  /* "dipy/align/vector_fields.pyx":1828
 *         cnp.npy_intp nr = field.shape[0]
 *         cnp.npy_intp nc = field.shape[1]
 *         cnp.npy_intp nnr = (nr + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nnc = (nc + 1) // 2
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nnr = ((__pyx_v_nr + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1829
 *         cnp.npy_intp nc = field.shape[1]
 *         cnp.npy_intp nnr = (nr + 1) // 2
 *         cnp.npy_intp nnc = (nc + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         floating[:, :, :] down = np.zeros((nnr, nnc, 2), dtype=ftype)
 */
  __pyx_v_nnc = ((__pyx_v_nc + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1831
 *         cnp.npy_intp nnc = (nc + 1) // 2
 *         cnp.npy_intp i, j, ii, jj
 *         floating[:, :, :] down = np.zeros((nnr, nnc, 2), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:, :] cnt = np.zeros((nnr, nnc), dtype=np.int32)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnr); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnc); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_t_3);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_down = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":1832
 *         cnp.npy_intp i, j, ii, jj
 *         floating[:, :, :] down = np.zeros((nnr, nnc, 2), dtype=ftype)
 *         int[:, :] cnt = np.zeros((nnr, nnc), dtype=np.int32)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnr); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnc); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_5);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_int32); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_dsds_int(__pyx_t_4);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_cnt = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":1834
 *         int[:, :] cnt = np.zeros((nnr, nnc), dtype=np.int32)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nr):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1836
 *     with nogil:
 * 
 *         for i in range(nr):             # <<<<<<<<<<<<<<
 *             for j in range(nc):
 *                 ii = i // 2
 */
        __pyx_t_8 = __pyx_v_nr;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":1837
 * 
 *         for i in range(nr):
 *             for j in range(nc):             # <<<<<<<<<<<<<<
 *                 ii = i // 2
 *                 jj = j // 2
 */
          __pyx_t_10 = __pyx_v_nc;
          for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
            __pyx_v_j = __pyx_t_11;

            /* "dipy/align/vector_fields.pyx":1838
 *         for i in range(nr):
 *             for j in range(nc):
 *                 ii = i // 2             # <<<<<<<<<<<<<<
 *                 jj = j // 2
 *                 down[ii, jj, 0] += field[i, j, 0]
 */
            __pyx_v_ii = (__pyx_v_i / 2);

            /* "dipy/align/vector_fields.pyx":1839
 *             for j in range(nc):
 *                 ii = i // 2
 *                 jj = j // 2             # <<<<<<<<<<<<<<
 *                 down[ii, jj, 0] += field[i, j, 0]
 *                 down[ii, jj, 1] += field[i, j, 1]
 */
            __pyx_v_jj = (__pyx_v_j / 2);

            /* "dipy/align/vector_fields.pyx":1840
 *                 ii = i // 2
 *                 jj = j // 2
 *                 down[ii, jj, 0] += field[i, j, 0]             # <<<<<<<<<<<<<<
 *                 down[ii, jj, 1] += field[i, j, 1]
 *                 cnt[ii, jj] += 1
 */
            __pyx_t_12 = __pyx_v_i;
            __pyx_t_13 = __pyx_v_j;
            __pyx_t_14 = 0;
            __pyx_t_15 = __pyx_v_ii;
            __pyx_t_16 = __pyx_v_jj;
            __pyx_t_17 = 0;
            *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_15 * __pyx_v_down.strides[0]) ) + __pyx_t_16 * __pyx_v_down.strides[1]) ) + __pyx_t_17 * __pyx_v_down.strides[2]) )) += (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_12 * __pyx_v_field.strides[0]) ) + __pyx_t_13 * __pyx_v_field.strides[1]) ) + __pyx_t_14 * __pyx_v_field.strides[2]) )));

            /* "dipy/align/vector_fields.pyx":1841
 *                 jj = j // 2
 *                 down[ii, jj, 0] += field[i, j, 0]
 *                 down[ii, jj, 1] += field[i, j, 1]             # <<<<<<<<<<<<<<
 *                 cnt[ii, jj] += 1
 *         for i in range(nnr):
 */
            __pyx_t_18 = __pyx_v_i;
            __pyx_t_19 = __pyx_v_j;
            __pyx_t_20 = 1;
            __pyx_t_21 = __pyx_v_ii;
            __pyx_t_22 = __pyx_v_jj;
            __pyx_t_23 = 1;
            *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_21 * __pyx_v_down.strides[0]) ) + __pyx_t_22 * __pyx_v_down.strides[1]) ) + __pyx_t_23 * __pyx_v_down.strides[2]) )) += (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_18 * __pyx_v_field.strides[0]) ) + __pyx_t_19 * __pyx_v_field.strides[1]) ) + __pyx_t_20 * __pyx_v_field.strides[2]) )));

            /* "dipy/align/vector_fields.pyx":1842
 *                 down[ii, jj, 0] += field[i, j, 0]
 *                 down[ii, jj, 1] += field[i, j, 1]
 *                 cnt[ii, jj] += 1             # <<<<<<<<<<<<<<
 *         for i in range(nnr):
 *             for j in range(nnc):
 */
            __pyx_t_24 = __pyx_v_ii;
            __pyx_t_25 = __pyx_v_jj;
            *((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_24 * __pyx_v_cnt.strides[0]) ) + __pyx_t_25 * __pyx_v_cnt.strides[1]) )) += 1;
          }
        }

        /* "dipy/align/vector_fields.pyx":1843
 *                 down[ii, jj, 1] += field[i, j, 1]
 *                 cnt[ii, jj] += 1
 *         for i in range(nnr):             # <<<<<<<<<<<<<<
 *             for j in range(nnc):
 *                 if cnt[i, j] > 0:
 */
        __pyx_t_8 = __pyx_v_nnr;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":1844
 *                 cnt[ii, jj] += 1
 *         for i in range(nnr):
 *             for j in range(nnc):             # <<<<<<<<<<<<<<
 *                 if cnt[i, j] > 0:
 *                     down[i, j, 0] /= cnt[i, j]
 */
          __pyx_t_10 = __pyx_v_nnc;
          for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
            __pyx_v_j = __pyx_t_11;

            /* "dipy/align/vector_fields.pyx":1845
 *         for i in range(nnr):
 *             for j in range(nnc):
 *                 if cnt[i, j] > 0:             # <<<<<<<<<<<<<<
 *                     down[i, j, 0] /= cnt[i, j]
 *                     down[i, j, 1] /= cnt[i, j]
 */
            __pyx_t_26 = __pyx_v_i;
            __pyx_t_27 = __pyx_v_j;
            __pyx_t_28 = (((*((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_26 * __pyx_v_cnt.strides[0]) ) + __pyx_t_27 * __pyx_v_cnt.strides[1]) ))) > 0) != 0);
            if (__pyx_t_28) {

              /* "dipy/align/vector_fields.pyx":1846
 *             for j in range(nnc):
 *                 if cnt[i, j] > 0:
 *                     down[i, j, 0] /= cnt[i, j]             # <<<<<<<<<<<<<<
 *                     down[i, j, 1] /= cnt[i, j]
 *     return down
 */
              __pyx_t_29 = __pyx_v_i;
              __pyx_t_30 = __pyx_v_j;
              __pyx_t_31 = __pyx_v_i;
              __pyx_t_32 = __pyx_v_j;
              __pyx_t_33 = 0;
              *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_31 * __pyx_v_down.strides[0]) ) + __pyx_t_32 * __pyx_v_down.strides[1]) ) + __pyx_t_33 * __pyx_v_down.strides[2]) )) /= (*((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_29 * __pyx_v_cnt.strides[0]) ) + __pyx_t_30 * __pyx_v_cnt.strides[1]) )));

              /* "dipy/align/vector_fields.pyx":1847
 *                 if cnt[i, j] > 0:
 *                     down[i, j, 0] /= cnt[i, j]
 *                     down[i, j, 1] /= cnt[i, j]             # <<<<<<<<<<<<<<
 *     return down
 * 
 */
              __pyx_t_34 = __pyx_v_i;
              __pyx_t_35 = __pyx_v_j;
              __pyx_t_36 = __pyx_v_i;
              __pyx_t_37 = __pyx_v_j;
              __pyx_t_38 = 1;
              *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_36 * __pyx_v_down.strides[0]) ) + __pyx_t_37 * __pyx_v_down.strides[1]) ) + __pyx_t_38 * __pyx_v_down.strides[2]) )) /= (*((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_34 * __pyx_v_cnt.strides[0]) ) + __pyx_t_35 * __pyx_v_cnt.strides[1]) )));

              /* "dipy/align/vector_fields.pyx":1845
 *         for i in range(nnr):
 *             for j in range(nnc):
 *                 if cnt[i, j] > 0:             # <<<<<<<<<<<<<<
 *                     down[i, j, 0] /= cnt[i, j]
 *                     down[i, j, 1] /= cnt[i, j]
 */
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1834
 *         int[:, :] cnt = np.zeros((nnr, nnc), dtype=np.int32)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nr):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1848
 *                     down[i, j, 0] /= cnt[i, j]
 *                     down[i, j, 1] /= cnt[i, j]
 *     return down             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_down, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1848; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1808
 * 
 * 
 * def downsample_displacement_field_2d(floating[:, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the 2D input vector field by a factor of 2
 *     Down-samples the input vector field by a factor of 2. The value at each
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.downsample_displacement_field_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_field, 1);
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_down, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_cnt, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_207downsample_displacement_field_2d(PyObject *__pyx_self, PyObject *__pyx_arg_field); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_207downsample_displacement_field_2d = {"__pyx_fuse_1downsample_displacement_field_2d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_207downsample_displacement_field_2d, METH_O, __pyx_doc_4dipy_5align_13vector_fields_36downsample_displacement_field_2d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_207downsample_displacement_field_2d(PyObject *__pyx_self, PyObject *__pyx_arg_field) {
  __Pyx_memviewslice __pyx_v_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("downsample_displacement_field_2d (wrapper)", 0);
  assert(__pyx_arg_field); {
    __pyx_v_field = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_arg_field); if (unlikely(!__pyx_v_field.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.downsample_displacement_field_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_206downsample_displacement_field_2d(__pyx_self, __pyx_v_field);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_206downsample_displacement_field_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_nr;
  npy_intp __pyx_v_nc;
  npy_intp __pyx_v_nnr;
  npy_intp __pyx_v_nnc;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  __Pyx_memviewslice __pyx_v_down = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_cnt = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  int __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1downsample_displacement_field_2d", 0);

  /* "dipy/align/vector_fields.pyx":1824
 *         the down-sampled displacement field, where R'= ceil(R/2), C'=ceil(C/2),
 *     """
 *     ftype = np.asarray(field).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp nr = field.shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1824; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1824; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_field, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1824; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1824; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1824; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1824; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1824; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":1826
 *     ftype = np.asarray(field).dtype
 *     cdef:
 *         cnp.npy_intp nr = field.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nc = field.shape[1]
 *         cnp.npy_intp nnr = (nr + 1) // 2
 */
  __pyx_v_nr = (__pyx_v_field.shape[0]);

  /* "dipy/align/vector_fields.pyx":1827
 *     cdef:
 *         cnp.npy_intp nr = field.shape[0]
 *         cnp.npy_intp nc = field.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nnr = (nr + 1) // 2
 *         cnp.npy_intp nnc = (nc + 1) // 2
 */
  __pyx_v_nc = (__pyx_v_field.shape[1]);

  /* "dipy/align/vector_fields.pyx":1828
 *         cnp.npy_intp nr = field.shape[0]
 *         cnp.npy_intp nc = field.shape[1]
 *         cnp.npy_intp nnr = (nr + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nnc = (nc + 1) // 2
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nnr = ((__pyx_v_nr + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1829
 *         cnp.npy_intp nc = field.shape[1]
 *         cnp.npy_intp nnr = (nr + 1) // 2
 *         cnp.npy_intp nnc = (nc + 1) // 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         floating[:, :, :] down = np.zeros((nnr, nnc, 2), dtype=ftype)
 */
  __pyx_v_nnc = ((__pyx_v_nc + 1) / 2);

  /* "dipy/align/vector_fields.pyx":1831
 *         cnp.npy_intp nnc = (nc + 1) // 2
 *         cnp.npy_intp i, j, ii, jj
 *         floating[:, :, :] down = np.zeros((nnr, nnc, 2), dtype=ftype)             # <<<<<<<<<<<<<<
 *         int[:, :] cnt = np.zeros((nnr, nnc), dtype=np.int32)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnr); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnc); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_3);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_down = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":1832
 *         cnp.npy_intp i, j, ii, jj
 *         floating[:, :, :] down = np.zeros((nnr, nnc, 2), dtype=ftype)
 *         int[:, :] cnt = np.zeros((nnr, nnc), dtype=np.int32)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnr); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nnc); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_5);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_int32); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_dsds_int(__pyx_t_4);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_cnt = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":1834
 *         int[:, :] cnt = np.zeros((nnr, nnc), dtype=np.int32)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nr):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1836
 *     with nogil:
 * 
 *         for i in range(nr):             # <<<<<<<<<<<<<<
 *             for j in range(nc):
 *                 ii = i // 2
 */
        __pyx_t_8 = __pyx_v_nr;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":1837
 * 
 *         for i in range(nr):
 *             for j in range(nc):             # <<<<<<<<<<<<<<
 *                 ii = i // 2
 *                 jj = j // 2
 */
          __pyx_t_10 = __pyx_v_nc;
          for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
            __pyx_v_j = __pyx_t_11;

            /* "dipy/align/vector_fields.pyx":1838
 *         for i in range(nr):
 *             for j in range(nc):
 *                 ii = i // 2             # <<<<<<<<<<<<<<
 *                 jj = j // 2
 *                 down[ii, jj, 0] += field[i, j, 0]
 */
            __pyx_v_ii = (__pyx_v_i / 2);

            /* "dipy/align/vector_fields.pyx":1839
 *             for j in range(nc):
 *                 ii = i // 2
 *                 jj = j // 2             # <<<<<<<<<<<<<<
 *                 down[ii, jj, 0] += field[i, j, 0]
 *                 down[ii, jj, 1] += field[i, j, 1]
 */
            __pyx_v_jj = (__pyx_v_j / 2);

            /* "dipy/align/vector_fields.pyx":1840
 *                 ii = i // 2
 *                 jj = j // 2
 *                 down[ii, jj, 0] += field[i, j, 0]             # <<<<<<<<<<<<<<
 *                 down[ii, jj, 1] += field[i, j, 1]
 *                 cnt[ii, jj] += 1
 */
            __pyx_t_12 = __pyx_v_i;
            __pyx_t_13 = __pyx_v_j;
            __pyx_t_14 = 0;
            __pyx_t_15 = __pyx_v_ii;
            __pyx_t_16 = __pyx_v_jj;
            __pyx_t_17 = 0;
            *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_15 * __pyx_v_down.strides[0]) ) + __pyx_t_16 * __pyx_v_down.strides[1]) ) + __pyx_t_17 * __pyx_v_down.strides[2]) )) += (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_12 * __pyx_v_field.strides[0]) ) + __pyx_t_13 * __pyx_v_field.strides[1]) ) + __pyx_t_14 * __pyx_v_field.strides[2]) )));

            /* "dipy/align/vector_fields.pyx":1841
 *                 jj = j // 2
 *                 down[ii, jj, 0] += field[i, j, 0]
 *                 down[ii, jj, 1] += field[i, j, 1]             # <<<<<<<<<<<<<<
 *                 cnt[ii, jj] += 1
 *         for i in range(nnr):
 */
            __pyx_t_18 = __pyx_v_i;
            __pyx_t_19 = __pyx_v_j;
            __pyx_t_20 = 1;
            __pyx_t_21 = __pyx_v_ii;
            __pyx_t_22 = __pyx_v_jj;
            __pyx_t_23 = 1;
            *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_21 * __pyx_v_down.strides[0]) ) + __pyx_t_22 * __pyx_v_down.strides[1]) ) + __pyx_t_23 * __pyx_v_down.strides[2]) )) += (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_field.data + __pyx_t_18 * __pyx_v_field.strides[0]) ) + __pyx_t_19 * __pyx_v_field.strides[1]) ) + __pyx_t_20 * __pyx_v_field.strides[2]) )));

            /* "dipy/align/vector_fields.pyx":1842
 *                 down[ii, jj, 0] += field[i, j, 0]
 *                 down[ii, jj, 1] += field[i, j, 1]
 *                 cnt[ii, jj] += 1             # <<<<<<<<<<<<<<
 *         for i in range(nnr):
 *             for j in range(nnc):
 */
            __pyx_t_24 = __pyx_v_ii;
            __pyx_t_25 = __pyx_v_jj;
            *((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_24 * __pyx_v_cnt.strides[0]) ) + __pyx_t_25 * __pyx_v_cnt.strides[1]) )) += 1;
          }
        }

        /* "dipy/align/vector_fields.pyx":1843
 *                 down[ii, jj, 1] += field[i, j, 1]
 *                 cnt[ii, jj] += 1
 *         for i in range(nnr):             # <<<<<<<<<<<<<<
 *             for j in range(nnc):
 *                 if cnt[i, j] > 0:
 */
        __pyx_t_8 = __pyx_v_nnr;
        for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
          __pyx_v_i = __pyx_t_9;

          /* "dipy/align/vector_fields.pyx":1844
 *                 cnt[ii, jj] += 1
 *         for i in range(nnr):
 *             for j in range(nnc):             # <<<<<<<<<<<<<<
 *                 if cnt[i, j] > 0:
 *                     down[i, j, 0] /= cnt[i, j]
 */
          __pyx_t_10 = __pyx_v_nnc;
          for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
            __pyx_v_j = __pyx_t_11;

            /* "dipy/align/vector_fields.pyx":1845
 *         for i in range(nnr):
 *             for j in range(nnc):
 *                 if cnt[i, j] > 0:             # <<<<<<<<<<<<<<
 *                     down[i, j, 0] /= cnt[i, j]
 *                     down[i, j, 1] /= cnt[i, j]
 */
            __pyx_t_26 = __pyx_v_i;
            __pyx_t_27 = __pyx_v_j;
            __pyx_t_28 = (((*((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_26 * __pyx_v_cnt.strides[0]) ) + __pyx_t_27 * __pyx_v_cnt.strides[1]) ))) > 0) != 0);
            if (__pyx_t_28) {

              /* "dipy/align/vector_fields.pyx":1846
 *             for j in range(nnc):
 *                 if cnt[i, j] > 0:
 *                     down[i, j, 0] /= cnt[i, j]             # <<<<<<<<<<<<<<
 *                     down[i, j, 1] /= cnt[i, j]
 *     return down
 */
              __pyx_t_29 = __pyx_v_i;
              __pyx_t_30 = __pyx_v_j;
              __pyx_t_31 = __pyx_v_i;
              __pyx_t_32 = __pyx_v_j;
              __pyx_t_33 = 0;
              *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_31 * __pyx_v_down.strides[0]) ) + __pyx_t_32 * __pyx_v_down.strides[1]) ) + __pyx_t_33 * __pyx_v_down.strides[2]) )) /= (*((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_29 * __pyx_v_cnt.strides[0]) ) + __pyx_t_30 * __pyx_v_cnt.strides[1]) )));

              /* "dipy/align/vector_fields.pyx":1847
 *                 if cnt[i, j] > 0:
 *                     down[i, j, 0] /= cnt[i, j]
 *                     down[i, j, 1] /= cnt[i, j]             # <<<<<<<<<<<<<<
 *     return down
 * 
 */
              __pyx_t_34 = __pyx_v_i;
              __pyx_t_35 = __pyx_v_j;
              __pyx_t_36 = __pyx_v_i;
              __pyx_t_37 = __pyx_v_j;
              __pyx_t_38 = 1;
              *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_down.data + __pyx_t_36 * __pyx_v_down.strides[0]) ) + __pyx_t_37 * __pyx_v_down.strides[1]) ) + __pyx_t_38 * __pyx_v_down.strides[2]) )) /= (*((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_cnt.data + __pyx_t_34 * __pyx_v_cnt.strides[0]) ) + __pyx_t_35 * __pyx_v_cnt.strides[1]) )));

              /* "dipy/align/vector_fields.pyx":1845
 *         for i in range(nnr):
 *             for j in range(nnc):
 *                 if cnt[i, j] > 0:             # <<<<<<<<<<<<<<
 *                     down[i, j, 0] /= cnt[i, j]
 *                     down[i, j, 1] /= cnt[i, j]
 */
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1834
 *         int[:, :] cnt = np.zeros((nnr, nnc), dtype=np.int32)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nr):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1848
 *                     down[i, j, 0] /= cnt[i, j]
 *                     down[i, j, 1] /= cnt[i, j]
 *     return down             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_down, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1848; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1808
 * 
 * 
 * def downsample_displacement_field_2d(floating[:, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the 2D input vector field by a factor of 2
 *     Down-samples the input vector field by a factor of 2. The value at each
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.downsample_displacement_field_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_field, 1);
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_down, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_cnt, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":1851
 * 
 * 
 * def warp_3d(floating[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_39warp_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_38warp_3d[] = "Warps a 3D volume using trilinear interpolation\n\n    Deforms the input volume under the given transformation. The warped volume\n    is computed using tri-linear interpolation and is given by:\n\n    (1) warped[i] = volume[ C * d1[A*i] + B*i ]\n\n    where A = affine_idx_in, B = affine_idx_out, C = affine_disp and i denotes\n    the discrete coordinates of a voxel in the sampling grid of\n    shape = out_shape. To illustrate the use of this function, consider a\n    displacement field d1 with grid-to-space transformation R, a volume with\n    grid-to-space transformation T and let's say we want to sample the warped\n    volume on a grid with grid-to-space transformation S (sampling grid). For\n    each voxel in the sampling grid with discrete coordinates i, the warped\n    volume is given by:\n\n    (2) warped[i] = volume[Tinv * ( d1[Rinv * S * i] + S * i ) ]\n\n    where Tinv = T^{-1} and Rinv = R^{-1}. By identifying A = Rinv * S,\n    B = Tinv * S, C = Tinv we can use this function to efficiently warp the\n    input image.\n\n\n    Parameters\n    ----------\n    volume : array, shape (S, R, C)\n        the input volume to be transformed\n    d1 : array, shape (S', R', C', 3)\n        the displacement field driving the transformation\n    affine_idx_in : array, shape (4, 4)\n        the matrix A in eq. (1) above\n    affine_idx_out : array, shape (4, 4)\n        the matrix B in eq. (1) above\n    affine_disp : array, shape (4, 4)\n        the matrix C in eq. (1) above\n    out_shape : array, shape (3,)\n        the number of slices, rows and columns of the sampling grid\n\n    Returns\n    -------\n    warped : array, shape = out_shape\n        the transformed volume\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_39warp_3d = {"warp_3d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_39warp_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_38warp_3d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_39warp_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    values[2] = __pyx_k__123;
    values[3] = __pyx_k__124;
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_38warp_3d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_38warp_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("warp_3d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_volume, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_volume); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__125, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__126, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__127, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__128, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_382__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults10, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults10, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults10, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults10, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_211warp_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_211warp_3d = {"__pyx_fuse_0warp_3d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_211warp_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_38warp_3d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_211warp_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults10 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults10, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_3d", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1852; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1853; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1854; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1855; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_3d", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_210warp_3d(__pyx_self, __pyx_v_volume, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_210warp_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dk;
  double __pyx_v_di;
  double __pyx_v_dj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0warp_3d", 0);

  /* "dipy/align/vector_fields.pyx":1900
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":1901
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":1902
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":1903
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":1904
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":1905
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":1910
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":1911
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__129, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1911; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1911; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1910
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":1912
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":1913
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__130, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1913; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1913; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1912
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":1914
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1914; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1914; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1914; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1914; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1914; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":1915
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__131, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1914
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1917
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":1918
 * 
 *     if out_shape is not None:
 *         nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 */
    __pyx_t_9 = 0;
    __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":1919
 *     if out_shape is not None:
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 */
    __pyx_t_10 = 1;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":1920
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 */
    __pyx_t_11 = 2;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_11 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":1917
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":1921
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":1922
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 *         nslices = d1.shape[0]             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]
 */
    __pyx_v_nslices = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":1923
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[2]
 * 
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":1924
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]             # <<<<<<<<<<<<<<
 * 
 *     cdef floating[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[2]);

    /* "dipy/align/vector_fields.pyx":1921
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":1926
 *         ncols = d1.shape[2]
 * 
 *     cdef floating[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                              dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "dipy/align/vector_fields.pyx":1927
 * 
 *     cdef floating[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                              dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1927; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1927; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1927; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1927; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_12);
  } else {
    __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1927; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1927; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1927; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":1926
 *         ncols = d1.shape[2]
 * 
 *     cdef floating[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                              dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_t_4);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_warped = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":1928
 *     cdef floating[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                              dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__132) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_asarray); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_12);
    __pyx_t_12 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_13) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_15 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_13);
  if (unlikely(!__pyx_t_15.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_tmp = __pyx_t_15;
  __pyx_t_15.memview = NULL;
  __pyx_t_15.data = NULL;

  /* "dipy/align/vector_fields.pyx":1930
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1932
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":1933
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":1934
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":1935
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":1936
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) ) + __pyx_t_25 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":1937
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d1[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_26 * __pyx_v_d1.strides[0]) ) + __pyx_t_27 * __pyx_v_d1.strides[1]) ) + __pyx_t_28 * __pyx_v_d1.strides[2]) ) + __pyx_t_29 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":1938
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_30 * __pyx_v_d1.strides[0]) ) + __pyx_t_31 * __pyx_v_d1.strides[1]) ) + __pyx_t_32 * __pyx_v_d1.strides[2]) ) + __pyx_t_33 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":1935
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":1940
 *                         djj = d1[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":1941
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":1942
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":1944
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":1946
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d1, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":1948
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":1949
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":1950
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":1952
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":1953
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":1955
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":1957
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":1952
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":1960
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":1961
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":1962
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L17:;

              /* "dipy/align/vector_fields.pyx":1964
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":1965
 * 
 *                     if affine_idx_out is not None:
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 */
                __pyx_v_dkk = (__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":1967
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":1969
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                     else:
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":1964
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
                goto __pyx_L18;
              }

              /* "dipy/align/vector_fields.pyx":1972
 *                                                        affine_idx_out)
 *                     else:
 *                         dkk = dk + k             # <<<<<<<<<<<<<<
 *                         dii = di + i
 *                         djj = dj + j
 */
              /*else*/ {
                __pyx_v_dkk = (__pyx_v_dk + __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":1973
 *                     else:
 *                         dkk = dk + k
 *                         dii = di + i             # <<<<<<<<<<<<<<
 *                         djj = dj + j
 * 
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":1974
 *                         dkk = dk + k
 *                         dii = di + i
 *                         djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                     inside = _interpolate_scalar_3d[floating](volume, dkk,
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
              }
              __pyx_L18:;

              /* "dipy/align/vector_fields.pyx":1978
 *                     inside = _interpolate_scalar_3d[floating](volume, dkk,
 *                                                               dii, djj,
 *                                                               &warped[k,i,j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
              __pyx_t_37 = __pyx_v_k;
              __pyx_t_38 = __pyx_v_i;
              __pyx_t_39 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":1976
 *                         djj = dj + j
 * 
 *                     inside = _interpolate_scalar_3d[floating](volume, dkk,             # <<<<<<<<<<<<<<
 *                                                               dii, djj,
 *                                                               &warped[k,i,j])
 */
              __pyx_v_inside = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_37 * __pyx_v_warped.strides[0]) ) + __pyx_t_38 * __pyx_v_warped.strides[1]) ) + __pyx_t_39 * __pyx_v_warped.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1930
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1979
 *                                                               dii, djj,
 *                                                               &warped[k,i,j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_13 = __pyx_memoryview_fromslice(__pyx_v_warped, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1979; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1851
 * 
 * 
 * def warp_3d(floating[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_384__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults11, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults11, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults11, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults11, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_213warp_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_213warp_3d = {"__pyx_fuse_1warp_3d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_213warp_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_38warp_3d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_213warp_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults11 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults11, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_3d", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1852; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1853; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1854; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1855; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_3d", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_212warp_3d(__pyx_self, __pyx_v_volume, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_212warp_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dk;
  double __pyx_v_di;
  double __pyx_v_dj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1warp_3d", 0);

  /* "dipy/align/vector_fields.pyx":1900
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":1901
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":1902
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":1903
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":1904
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":1905
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":1910
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1910; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":1911
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__133, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1911; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1911; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1910
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":1912
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1912; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":1913
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__134, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1913; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1913; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1912
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":1914
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1914; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1914; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1914; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1914; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1914; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":1915
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__135, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":1914
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":1917
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":1918
 * 
 *     if out_shape is not None:
 *         nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 */
    __pyx_t_9 = 0;
    __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":1919
 *     if out_shape is not None:
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 */
    __pyx_t_10 = 1;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":1920
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 */
    __pyx_t_11 = 2;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_11 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":1917
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":1921
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":1922
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 *         nslices = d1.shape[0]             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]
 */
    __pyx_v_nslices = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":1923
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[2]
 * 
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":1924
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]             # <<<<<<<<<<<<<<
 * 
 *     cdef floating[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[2]);

    /* "dipy/align/vector_fields.pyx":1921
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":1926
 *         ncols = d1.shape[2]
 * 
 *     cdef floating[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                              dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "dipy/align/vector_fields.pyx":1927
 * 
 *     cdef floating[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                              dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1927; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1927; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1927; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1927; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_12);
  } else {
    __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1927; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1927; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1927; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":1926
 *         ncols = d1.shape[2]
 * 
 *     cdef floating[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                              dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1926; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_warped = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":1928
 *     cdef floating[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                              dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__136) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_asarray); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_12);
    __pyx_t_12 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_13) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_15 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_13);
  if (unlikely(!__pyx_t_15.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_tmp = __pyx_t_15;
  __pyx_t_15.memview = NULL;
  __pyx_t_15.data = NULL;

  /* "dipy/align/vector_fields.pyx":1930
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":1932
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":1933
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":1934
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":1935
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":1936
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) ) + __pyx_t_25 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":1937
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d1[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_26 * __pyx_v_d1.strides[0]) ) + __pyx_t_27 * __pyx_v_d1.strides[1]) ) + __pyx_t_28 * __pyx_v_d1.strides[2]) ) + __pyx_t_29 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":1938
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_30 * __pyx_v_d1.strides[0]) ) + __pyx_t_31 * __pyx_v_d1.strides[1]) ) + __pyx_t_32 * __pyx_v_d1.strides[2]) ) + __pyx_t_33 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":1935
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":1940
 *                         djj = d1[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":1941
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":1942
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":1944
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":1946
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d1, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":1948
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":1949
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":1950
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":1952
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":1953
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":1955
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":1957
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":1952
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":1960
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":1961
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":1962
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L17:;

              /* "dipy/align/vector_fields.pyx":1964
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":1965
 * 
 *                     if affine_idx_out is not None:
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 */
                __pyx_v_dkk = (__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":1967
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":1969
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                     else:
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":1964
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
                goto __pyx_L18;
              }

              /* "dipy/align/vector_fields.pyx":1972
 *                                                        affine_idx_out)
 *                     else:
 *                         dkk = dk + k             # <<<<<<<<<<<<<<
 *                         dii = di + i
 *                         djj = dj + j
 */
              /*else*/ {
                __pyx_v_dkk = (__pyx_v_dk + __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":1973
 *                     else:
 *                         dkk = dk + k
 *                         dii = di + i             # <<<<<<<<<<<<<<
 *                         djj = dj + j
 * 
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":1974
 *                         dkk = dk + k
 *                         dii = di + i
 *                         djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                     inside = _interpolate_scalar_3d[floating](volume, dkk,
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
              }
              __pyx_L18:;

              /* "dipy/align/vector_fields.pyx":1978
 *                     inside = _interpolate_scalar_3d[floating](volume, dkk,
 *                                                               dii, djj,
 *                                                               &warped[k,i,j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
              __pyx_t_37 = __pyx_v_k;
              __pyx_t_38 = __pyx_v_i;
              __pyx_t_39 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":1976
 *                         djj = dj + j
 * 
 *                     inside = _interpolate_scalar_3d[floating](volume, dkk,             # <<<<<<<<<<<<<<
 *                                                               dii, djj,
 *                                                               &warped[k,i,j])
 */
              __pyx_v_inside = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_37 * __pyx_v_warped.strides[0]) ) + __pyx_t_38 * __pyx_v_warped.strides[1]) ) + __pyx_t_39 * __pyx_v_warped.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":1930
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":1979
 *                                                               dii, djj,
 *                                                               &warped[k,i,j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_13 = __pyx_memoryview_fromslice(__pyx_v_warped, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1979; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1851
 * 
 * 
 * def warp_3d(floating[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":1982
 * 
 * 
 * def transform_3d_affine(floating[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                         double[:, :] affine):
 *     r"""Transforms a 3D volume by an affine transform with trilinear interp.
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_41transform_3d_affine(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_40transform_3d_affine[] = "Transforms a 3D volume by an affine transform with trilinear interp.\n\n    Deforms the input volume under the given affine transformation using\n    tri-linear interpolation. The shape of the resulting transformation\n    is given by ref_shape. If the affine matrix is None, it is taken as the\n    identity.\n\n    Parameters\n    ----------\n    volume : array, shape (S, R, C)\n        the input volume to be transformed\n    ref_shape : array, shape (3,)\n        the shape of the resulting volume\n    affine : array, shape (4, 4)\n        the affine transform to be applied\n\n    Returns\n    -------\n    out : array, shape (S', R', C')\n        the transformed volume\n\n    Notes\n    -----\n    The reason it is necessary to provide the intended shape of the resulting\n    volume is because the affine transformation is defined on all R^{3}\n    but we must sample a finite lattice. Also the resulting shape may not be\n    necessarily equal to the input shape, unless we are interested on\n    endomorphisms only and not general diffeomorphisms.\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_41transform_3d_affine = {"transform_3d_affine", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_41transform_3d_affine, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_40transform_3d_affine};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_41transform_3d_affine(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_40transform_3d_affine(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_40transform_3d_affine(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("transform_3d_affine", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_volume, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_volume); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__137, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__138, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__139, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__140, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_217transform_3d_affine(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_217transform_3d_affine = {"__pyx_fuse_0transform_3d_affine", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_217transform_3d_affine, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_40transform_3d_affine};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_217transform_3d_affine(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_3d_affine (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_3d_affine", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_3d_affine", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_3d_affine") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1983; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_3d_affine", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_216transform_3d_affine(__pyx_self, __pyx_v_volume, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_216transform_3d_affine(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_memviewslice __pyx_t_11 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0transform_3d_affine", 0);

  /* "dipy/align/vector_fields.pyx":2014
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 */
  __pyx_t_1 = 0;
  __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2015
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_t_2 = 1;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2016
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_t_3 = 2;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_3 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2017
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2018
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k, ii, jj, kk
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2019
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2024
 *         double dkk, dii, djj, tmp0, tmp1
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         floating[:, :, :] out = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                          dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = PyTuple_New(3); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_t_8);
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "dipy/align/vector_fields.pyx":2025
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         floating[:, :, :] out = np.zeros(shape=(nslices, nrows, ncols),
 *                                          dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 3):
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2025; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_asarray); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2025; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2025; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_9 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_8); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2025; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_GOTREF(__pyx_t_9);
  } else {
    __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2025; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_10, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2025; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_dtype); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2025; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "dipy/align/vector_fields.pyx":2024
 *         double dkk, dii, djj, tmp0, tmp1
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         floating[:, :, :] out = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                          dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_11 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_t_7);
  if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_out = __pyx_t_11;
  __pyx_t_11.memview = NULL;
  __pyx_t_11.data = NULL;

  /* "dipy/align/vector_fields.pyx":2027
 *                                          dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2027; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2027; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_9 = NULL;
  __pyx_t_12 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_12 = 1;
    }
  }
  __pyx_t_10 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2027; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  if (__pyx_t_9) {
    __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_12, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_12, __pyx_int_3);
  __pyx_t_5 = 0;
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_10, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2027; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely(__pyx_t_13 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2027; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_14 = ((!__pyx_t_13) != 0);
  if (__pyx_t_14) {

    /* "dipy/align/vector_fields.pyx":2028
 * 
 *     if not is_valid_affine(affine, 3):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__141, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2028; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_Raise(__pyx_t_7, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2028; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2027
 *                                          dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2030
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2032
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_15 = __pyx_v_nslices;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_k = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2033
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine is not None:
 */
          __pyx_t_17 = __pyx_v_nrows;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_i = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2034
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 */
            __pyx_t_19 = __pyx_v_ncols;
            for (__pyx_t_20 = 0; __pyx_t_20 < __pyx_t_19; __pyx_t_20+=1) {
              __pyx_v_j = __pyx_t_20;

              /* "dipy/align/vector_fields.pyx":2035
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
              __pyx_t_14 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
              if (__pyx_t_14) {

                /* "dipy/align/vector_fields.pyx":2036
 *                 for j in range(ncols):
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 */
                __pyx_v_dkk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2037
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 */
                __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2038
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     else:
 *                         dkk = k
 */
                __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2035
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
                goto __pyx_L13;
              }

              /* "dipy/align/vector_fields.pyx":2040
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 *                         dkk = k             # <<<<<<<<<<<<<<
 *                         dii = i
 *                         djj = j
 */
              /*else*/ {
                __pyx_v_dkk = __pyx_v_k;

                /* "dipy/align/vector_fields.pyx":2041
 *                     else:
 *                         dkk = k
 *                         dii = i             # <<<<<<<<<<<<<<
 *                         djj = j
 *                     inside = _interpolate_scalar_3d[floating](volume, dkk,
 */
                __pyx_v_dii = __pyx_v_i;

                /* "dipy/align/vector_fields.pyx":2042
 *                         dkk = k
 *                         dii = i
 *                         djj = j             # <<<<<<<<<<<<<<
 *                     inside = _interpolate_scalar_3d[floating](volume, dkk,
 *                         dii, djj, &out[k,i,j])
 */
                __pyx_v_djj = __pyx_v_j;
              }
              __pyx_L13:;

              /* "dipy/align/vector_fields.pyx":2044
 *                         djj = j
 *                     inside = _interpolate_scalar_3d[floating](volume, dkk,
 *                         dii, djj, &out[k,i,j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
              __pyx_t_21 = __pyx_v_k;
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2043
 *                         dii = i
 *                         djj = j
 *                     inside = _interpolate_scalar_3d[floating](volume, dkk,             # <<<<<<<<<<<<<<
 *                         dii, djj, &out[k,i,j])
 *     return out
 */
              __pyx_v_inside = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_21 * __pyx_v_out.strides[0]) ) + __pyx_t_22 * __pyx_v_out.strides[1]) ) + __pyx_t_23 * __pyx_v_out.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2030
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2045
 *                     inside = _interpolate_scalar_3d[floating](volume, dkk,
 *                         dii, djj, &out[k,i,j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_out, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2045; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_r = __pyx_t_7;
  __pyx_t_7 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1982
 * 
 * 
 * def transform_3d_affine(floating[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                         double[:, :] affine):
 *     r"""Transforms a 3D volume by an affine transform with trilinear interp.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __PYX_XDEC_MEMVIEW(&__pyx_t_11, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_219transform_3d_affine(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_219transform_3d_affine = {"__pyx_fuse_1transform_3d_affine", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_219transform_3d_affine, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_40transform_3d_affine};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_219transform_3d_affine(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_3d_affine (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_3d_affine", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_3d_affine", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_3d_affine") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1983; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_3d_affine", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_218transform_3d_affine(__pyx_self, __pyx_v_volume, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_218transform_3d_affine(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_memviewslice __pyx_t_11 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1transform_3d_affine", 0);

  /* "dipy/align/vector_fields.pyx":2014
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 */
  __pyx_t_1 = 0;
  __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2015
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_t_2 = 1;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2016
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_t_3 = 2;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_3 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2017
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2018
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k, ii, jj, kk
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2019
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2024
 *         double dkk, dii, djj, tmp0, tmp1
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         floating[:, :, :] out = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                          dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = PyTuple_New(3); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_t_8);
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "dipy/align/vector_fields.pyx":2025
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         floating[:, :, :] out = np.zeros(shape=(nslices, nrows, ncols),
 *                                          dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 3):
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2025; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_asarray); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2025; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2025; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_9 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_8); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2025; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_GOTREF(__pyx_t_9);
  } else {
    __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2025; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_10, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2025; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_dtype); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2025; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "dipy/align/vector_fields.pyx":2024
 *         double dkk, dii, djj, tmp0, tmp1
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         floating[:, :, :] out = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                          dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_11 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_7);
  if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2024; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_out = __pyx_t_11;
  __pyx_t_11.memview = NULL;
  __pyx_t_11.data = NULL;

  /* "dipy/align/vector_fields.pyx":2027
 *                                          dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2027; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2027; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_9 = NULL;
  __pyx_t_12 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
      __pyx_t_12 = 1;
    }
  }
  __pyx_t_10 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2027; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  if (__pyx_t_9) {
    __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_12, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_12, __pyx_int_3);
  __pyx_t_5 = 0;
  __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_10, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2027; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely(__pyx_t_13 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2027; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_14 = ((!__pyx_t_13) != 0);
  if (__pyx_t_14) {

    /* "dipy/align/vector_fields.pyx":2028
 * 
 *     if not is_valid_affine(affine, 3):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__142, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2028; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_Raise(__pyx_t_7, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2028; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2027
 *                                          dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2030
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2032
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_15 = __pyx_v_nslices;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_k = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2033
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine is not None:
 */
          __pyx_t_17 = __pyx_v_nrows;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_i = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2034
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 */
            __pyx_t_19 = __pyx_v_ncols;
            for (__pyx_t_20 = 0; __pyx_t_20 < __pyx_t_19; __pyx_t_20+=1) {
              __pyx_v_j = __pyx_t_20;

              /* "dipy/align/vector_fields.pyx":2035
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
              __pyx_t_14 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
              if (__pyx_t_14) {

                /* "dipy/align/vector_fields.pyx":2036
 *                 for j in range(ncols):
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 */
                __pyx_v_dkk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2037
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 */
                __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2038
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     else:
 *                         dkk = k
 */
                __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2035
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
                goto __pyx_L13;
              }

              /* "dipy/align/vector_fields.pyx":2040
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 *                         dkk = k             # <<<<<<<<<<<<<<
 *                         dii = i
 *                         djj = j
 */
              /*else*/ {
                __pyx_v_dkk = __pyx_v_k;

                /* "dipy/align/vector_fields.pyx":2041
 *                     else:
 *                         dkk = k
 *                         dii = i             # <<<<<<<<<<<<<<
 *                         djj = j
 *                     inside = _interpolate_scalar_3d[floating](volume, dkk,
 */
                __pyx_v_dii = __pyx_v_i;

                /* "dipy/align/vector_fields.pyx":2042
 *                         dkk = k
 *                         dii = i
 *                         djj = j             # <<<<<<<<<<<<<<
 *                     inside = _interpolate_scalar_3d[floating](volume, dkk,
 *                         dii, djj, &out[k,i,j])
 */
                __pyx_v_djj = __pyx_v_j;
              }
              __pyx_L13:;

              /* "dipy/align/vector_fields.pyx":2044
 *                         djj = j
 *                     inside = _interpolate_scalar_3d[floating](volume, dkk,
 *                         dii, djj, &out[k,i,j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
              __pyx_t_21 = __pyx_v_k;
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2043
 *                         dii = i
 *                         djj = j
 *                     inside = _interpolate_scalar_3d[floating](volume, dkk,             # <<<<<<<<<<<<<<
 *                         dii, djj, &out[k,i,j])
 *     return out
 */
              __pyx_v_inside = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_21 * __pyx_v_out.strides[0]) ) + __pyx_t_22 * __pyx_v_out.strides[1]) ) + __pyx_t_23 * __pyx_v_out.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2030
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2045
 *                     inside = _interpolate_scalar_3d[floating](volume, dkk,
 *                         dii, djj, &out[k,i,j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_out, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2045; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_r = __pyx_t_7;
  __pyx_t_7 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":1982
 * 
 * 
 * def transform_3d_affine(floating[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                         double[:, :] affine):
 *     r"""Transforms a 3D volume by an affine transform with trilinear interp.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __PYX_XDEC_MEMVIEW(&__pyx_t_11, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_43warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_42warp_3d_nn[] = "Warps a 3D volume using using nearest-neighbor interpolation\n\n    Deforms the input volume under the given transformation. The warped volume\n    is computed using nearest-neighbor interpolation and is given by:\n\n    (1) warped[i] = volume[ C * d1[A*i] + B*i ]\n\n    where A = affine_idx_in, B = affine_idx_out, C = affine_disp and i denotes\n    the discrete coordinates of a voxel in the sampling grid of\n    shape = out_shape. To illustrate the use of this function, consider a\n    displacement field d1 with grid-to-space transformation R, a volume with\n    grid-to-space transformation T and let's say we want to sample the warped\n    volume on a grid with grid-to-space transformation S (sampling grid). For\n    each voxel in the sampling grid with discrete coordinates i, the warped\n    volume is given by:\n\n    (2) warped[i] = volume[Tinv * ( d1[Rinv * S * i] + S * i ) ]\n\n    where Tinv = T^{-1} and Rinv = R^{-1}. By identifying A = Rinv * S,\n    B = Tinv * S, C = Tinv we can use this function to efficiently warp the\n    input image.\n\n\n    Parameters\n    ----------\n    volume : array, shape (S, R, C)\n        the input volume to be transformed\n    d1 : array, shape (S', R', C', 3)\n        the displacement field driving the transformation\n    affine_idx_in : array, shape (4, 4)\n        the matrix A in eq. (1) above\n    affine_idx_out : array, shape (4, 4)\n        the matrix B in eq. (1) above\n    affine_disp : array, shape (4, 4)\n        the matrix C in eq. (1) above\n    out_shape : array, shape (3,)\n        the number of slices, rows and columns of the sampling grid\n\n    Returns\n    -------\n    warped : array, shape = out_shape\n        the transformed volume\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_43warp_3d_nn = {"warp_3d_nn", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_43warp_3d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_42warp_3d_nn};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_43warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    values[2] = __pyx_k__143;
    values[3] = __pyx_k__144;
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_42warp_3d_nn(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_42warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  int __pyx_v_short_is_signed;
  int __pyx_v_long_is_signed;
  int __pyx_v_int_is_signed;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("warp_3d_nn", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1 * 2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  { Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < 2; __pyx_temp++) {
      __Pyx_INCREF(Py_None);
      __Pyx_GIVEREF(Py_None);
      PyList_SET_ITEM(__pyx_t_1, __pyx_temp, Py_None);
    }
  }
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  __pyx_v_short_is_signed = (((short)-1L) < 0);
  __pyx_v_long_is_signed = (((long)-1L) < 0);
  __pyx_v_int_is_signed = (((int)-1L) < 0);
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_volume, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_volume); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          __pyx_t_3 = (((sizeof(short)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_short_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_short, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(int)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L26_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L26_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_int_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L26_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_int, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(long)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L30_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L30_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_long_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L30_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_long, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L34_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L34_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L37_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L37_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          __pyx_t_3 = (((sizeof(__pyx_t_float_complex)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L40_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L40_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_float_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(__pyx_t_double_complex)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L43_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L43_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_double_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L46_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(short))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L46_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_short(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_short, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L50_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(int))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L50_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_int, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L54_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(long))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L54_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_long(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_long, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L58_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L58_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L62_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L62_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L66_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(__pyx_t_float_complex))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L66_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_float_complex(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_float_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L70_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(__pyx_t_double_complex))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L70_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_double_complex(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_double_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((1 < __pyx_t_10) != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 1);
    __Pyx_INCREF(__pyx_t_8);
    __Pyx_DECREF_SET(__pyx_v_arg, __pyx_t_8);
    __pyx_t_8 = 0;
    goto __pyx_L73;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_2 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_d1, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_d1); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF_SET(__pyx_v_arg, __pyx_t_8);
    __pyx_t_8 = 0;
    goto __pyx_L73;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_8 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_8); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_8, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_Raise(__pyx_t_9, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L73:;
  while (1) {
    __pyx_t_3 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_2 = (__pyx_t_3 != 0);
    if (__pyx_t_2) {
      __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_XDECREF_SET(__pyx_v_dtype, __pyx_t_9);
        __pyx_t_9 = 0;
        goto __pyx_L77;
      }
      __pyx_t_3 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_3) {
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_XDECREF_SET(__pyx_v_arg_base, __pyx_t_9);
        __pyx_t_9 = 0;
        __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_2 = (__pyx_t_3 != 0);
        if (__pyx_t_2) {
          __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_XDECREF_SET(__pyx_v_dtype, __pyx_t_9);
          __pyx_t_9 = 0;
          goto __pyx_L78;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __Pyx_XDECREF_SET(__pyx_v_dtype, Py_None);
        }
        __pyx_L78:;
        goto __pyx_L77;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __Pyx_XDECREF_SET(__pyx_v_dtype, Py_None);
      }
      __pyx_L77:;
      __pyx_v_itemsize = -1L;
      __pyx_t_2 = (__pyx_v_dtype != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_9); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_2 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_2) {
          } else {
            __pyx_t_3 = __pyx_t_2;
            goto __pyx_L81_bool_binop_done;
          }
          __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __pyx_t_2 = ((((Py_ssize_t)__pyx_t_10) == 4) != 0);
          __pyx_t_3 = __pyx_t_2;
          __pyx_L81_bool_binop_done:;
          if (__pyx_t_3) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 1, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L75_break;
          }
          __pyx_t_2 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_2) {
          } else {
            __pyx_t_3 = __pyx_t_2;
            goto __pyx_L84_bool_binop_done;
          }
          __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __pyx_t_2 = ((((Py_ssize_t)__pyx_t_10) == 4) != 0);
          __pyx_t_3 = __pyx_t_2;
          __pyx_L84_bool_binop_done:;
          if (__pyx_t_3) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 1, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L75_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_2 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_3 = __pyx_t_2;
      goto __pyx_L87_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_3 = __pyx_t_2;
    __pyx_L87_bool_binop_done:;
    if (__pyx_t_3) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(__pyx_v_arg);
      __pyx_t_3 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_3) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 1, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L75_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_2 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_3 = __pyx_t_2;
      goto __pyx_L91_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_3 = __pyx_t_2;
    __pyx_L91_bool_binop_done:;
    if (__pyx_t_3) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_v_arg);
      __pyx_t_3 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_3) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 1, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L75_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 1, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L75_break;
  }
  __pyx_L75_break:;
  __pyx_t_9 = PyList_New(0); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_v_candidates = ((PyObject*)__pyx_t_9);
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_8 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __pyx_t_9 = __pyx_t_8;
  __pyx_t_8 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_9, __pyx_t_12, &__pyx_t_10, &__pyx_t_8, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_tuple__145, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_tuple__146, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_8 = __pyx_t_1; __Pyx_INCREF(__pyx_t_8); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_8 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_15 = Py_TYPE(__pyx_t_8)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_8))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_8)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_8, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_8)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_8, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_8);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L98_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L98_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L99_unpacking_done;
        __pyx_L98_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L99_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_3 = (__pyx_v_dst_type != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_2) {
          __pyx_v_match_found = 1;
          goto __pyx_L101;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L97_break;
        }
        __pyx_L101:;
      }
    }
    __pyx_L97_break:;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_2 = (__pyx_v_match_found != 0);
    if (__pyx_t_2) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_2 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_3 = ((!__pyx_t_2) != 0);
  if (__pyx_t_3) {
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__147, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_Raise(__pyx_t_9, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_3) {
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__148, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_Raise(__pyx_t_9, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_r = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_414__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults26, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults26, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults26, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults26, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0_0__pyx_pw_4dipy_5align_13vector_fields_223warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0_0__pyx_mdef_4dipy_5align_13vector_fields_223warp_3d_nn = {"__pyx_fuse_0_0warp_3d_nn", (PyCFunction)__pyx_fuse_0_0__pyx_pw_4dipy_5align_13vector_fields_223warp_3d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_42warp_3d_nn};
static PyObject *__pyx_fuse_0_0__pyx_pw_4dipy_5align_13vector_fields_223warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_3d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults26 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults26, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_3d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_short(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_222warp_3d_nn(__pyx_self, __pyx_v_volume, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_222warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dk;
  double __pyx_v_di;
  double __pyx_v_dj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0_0warp_3d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2097
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2098
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2099
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2100
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2101
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2102
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__149, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__150, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__151, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2115
 * 
 *     if out_shape is not None:
 *         nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 */
    __pyx_t_9 = 0;
    __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2116
 *     if out_shape is not None:
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 */
    __pyx_t_10 = 1;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2117
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 */
    __pyx_t_11 = 2;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_11 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2119
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 *         nslices = d1.shape[0]             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]
 */
    __pyx_v_nslices = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2120
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[2]
 * 
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2121
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]             # <<<<<<<<<<<<<<
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[2]);

    /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "dipy/align/vector_fields.pyx":2124
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_short, (int (*)(char *, PyObject *)) __pyx_memview_set_short, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_12);
  } else {
    __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_short(__pyx_t_4);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_warped = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__152) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_asarray); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_12);
    __pyx_t_12 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_13) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_15 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_13);
  if (unlikely(!__pyx_t_15.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_tmp = __pyx_t_15;
  __pyx_t_15.memview = NULL;
  __pyx_t_15.data = NULL;

  /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2129
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2130
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2131
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2133
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) ) + __pyx_t_25 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2134
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d1[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_26 * __pyx_v_d1.strides[0]) ) + __pyx_t_27 * __pyx_v_d1.strides[1]) ) + __pyx_t_28 * __pyx_v_d1.strides[2]) ) + __pyx_t_29 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2135
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_30 * __pyx_v_d1.strides[0]) ) + __pyx_t_31 * __pyx_v_d1.strides[1]) ) + __pyx_t_32 * __pyx_v_d1.strides[2]) ) + __pyx_t_33 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":2137
 *                         djj = d1[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":2138
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2139
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2141
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2143
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d1, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":2145
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2146
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2147
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2150
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2152
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2154
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":2157
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":2158
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":2159
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L17:;

              /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2162
 * 
 *                     if affine_idx_out is not None:
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 */
                __pyx_v_dkk = (__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2164
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2166
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                     else:
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
                goto __pyx_L18;
              }

              /* "dipy/align/vector_fields.pyx":2169
 *                                                        affine_idx_out)
 *                     else:
 *                         dkk = dk + k             # <<<<<<<<<<<<<<
 *                         dii = di + i
 *                         djj = dj + j
 */
              /*else*/ {
                __pyx_v_dkk = (__pyx_v_dk + __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":2170
 *                     else:
 *                         dkk = dk + k
 *                         dii = di + i             # <<<<<<<<<<<<<<
 *                         djj = dj + j
 * 
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":2171
 *                         dkk = dk + k
 *                         dii = di + i
 *                         djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
              }
              __pyx_L18:;

              /* "dipy/align/vector_fields.pyx":2174
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
              __pyx_t_37 = __pyx_v_k;
              __pyx_t_38 = __pyx_v_i;
              __pyx_t_39 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2173
 *                         djj = dj + j
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                        &warped[k,i,j])
 *     return warped
 */
              __pyx_v_inside = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((short *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_37 * __pyx_v_warped.strides[0]) ) + __pyx_t_38 * __pyx_v_warped.strides[1]) ) + __pyx_t_39 * __pyx_v_warped.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2175
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_13 = __pyx_memoryview_fromslice(__pyx_v_warped, 3, (PyObject *(*)(char *)) __pyx_memview_get_short, (int (*)(char *, PyObject *)) __pyx_memview_set_short, 0);; if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2175; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_416__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults27, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults27, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults27, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults27, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0_1__pyx_pw_4dipy_5align_13vector_fields_225warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0_1__pyx_mdef_4dipy_5align_13vector_fields_225warp_3d_nn = {"__pyx_fuse_0_1warp_3d_nn", (PyCFunction)__pyx_fuse_0_1__pyx_pw_4dipy_5align_13vector_fields_225warp_3d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_42warp_3d_nn};
static PyObject *__pyx_fuse_0_1__pyx_pw_4dipy_5align_13vector_fields_225warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_3d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults27 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults27, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_3d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_short(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_224warp_3d_nn(__pyx_self, __pyx_v_volume, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_224warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dk;
  double __pyx_v_di;
  double __pyx_v_dj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0_1warp_3d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2097
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2098
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2099
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2100
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2101
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2102
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__153, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__154, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__155, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2115
 * 
 *     if out_shape is not None:
 *         nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 */
    __pyx_t_9 = 0;
    __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2116
 *     if out_shape is not None:
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 */
    __pyx_t_10 = 1;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2117
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 */
    __pyx_t_11 = 2;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_11 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2119
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 *         nslices = d1.shape[0]             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]
 */
    __pyx_v_nslices = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2120
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[2]
 * 
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2121
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]             # <<<<<<<<<<<<<<
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[2]);

    /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "dipy/align/vector_fields.pyx":2124
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_short, (int (*)(char *, PyObject *)) __pyx_memview_set_short, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_12);
  } else {
    __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_short(__pyx_t_4);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_warped = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__156) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_asarray); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_12);
    __pyx_t_12 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_13) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_15 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_13);
  if (unlikely(!__pyx_t_15.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_tmp = __pyx_t_15;
  __pyx_t_15.memview = NULL;
  __pyx_t_15.data = NULL;

  /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2129
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2130
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2131
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2133
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) ) + __pyx_t_25 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2134
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d1[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_26 * __pyx_v_d1.strides[0]) ) + __pyx_t_27 * __pyx_v_d1.strides[1]) ) + __pyx_t_28 * __pyx_v_d1.strides[2]) ) + __pyx_t_29 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2135
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_30 * __pyx_v_d1.strides[0]) ) + __pyx_t_31 * __pyx_v_d1.strides[1]) ) + __pyx_t_32 * __pyx_v_d1.strides[2]) ) + __pyx_t_33 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":2137
 *                         djj = d1[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":2138
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2139
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2141
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2143
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d1, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":2145
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2146
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2147
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2150
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2152
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2154
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":2157
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":2158
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":2159
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L17:;

              /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2162
 * 
 *                     if affine_idx_out is not None:
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 */
                __pyx_v_dkk = (__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2164
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2166
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                     else:
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
                goto __pyx_L18;
              }

              /* "dipy/align/vector_fields.pyx":2169
 *                                                        affine_idx_out)
 *                     else:
 *                         dkk = dk + k             # <<<<<<<<<<<<<<
 *                         dii = di + i
 *                         djj = dj + j
 */
              /*else*/ {
                __pyx_v_dkk = (__pyx_v_dk + __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":2170
 *                     else:
 *                         dkk = dk + k
 *                         dii = di + i             # <<<<<<<<<<<<<<
 *                         djj = dj + j
 * 
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":2171
 *                         dkk = dk + k
 *                         dii = di + i
 *                         djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
              }
              __pyx_L18:;

              /* "dipy/align/vector_fields.pyx":2174
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
              __pyx_t_37 = __pyx_v_k;
              __pyx_t_38 = __pyx_v_i;
              __pyx_t_39 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2173
 *                         djj = dj + j
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                        &warped[k,i,j])
 *     return warped
 */
              __pyx_v_inside = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((short *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_37 * __pyx_v_warped.strides[0]) ) + __pyx_t_38 * __pyx_v_warped.strides[1]) ) + __pyx_t_39 * __pyx_v_warped.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2175
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_13 = __pyx_memoryview_fromslice(__pyx_v_warped, 3, (PyObject *(*)(char *)) __pyx_memview_get_short, (int (*)(char *, PyObject *)) __pyx_memview_set_short, 0);; if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2175; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_418__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults28, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults28, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults28, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults28, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1_0__pyx_pw_4dipy_5align_13vector_fields_227warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1_0__pyx_mdef_4dipy_5align_13vector_fields_227warp_3d_nn = {"__pyx_fuse_1_0warp_3d_nn", (PyCFunction)__pyx_fuse_1_0__pyx_pw_4dipy_5align_13vector_fields_227warp_3d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_42warp_3d_nn};
static PyObject *__pyx_fuse_1_0__pyx_pw_4dipy_5align_13vector_fields_227warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_3d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults28 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults28, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_3d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_226warp_3d_nn(__pyx_self, __pyx_v_volume, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_226warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dk;
  double __pyx_v_di;
  double __pyx_v_dj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1_0warp_3d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2097
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2098
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2099
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2100
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2101
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2102
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__157, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__158, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__159, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2115
 * 
 *     if out_shape is not None:
 *         nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 */
    __pyx_t_9 = 0;
    __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2116
 *     if out_shape is not None:
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 */
    __pyx_t_10 = 1;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2117
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 */
    __pyx_t_11 = 2;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_11 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2119
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 *         nslices = d1.shape[0]             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]
 */
    __pyx_v_nslices = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2120
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[2]
 * 
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2121
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]             # <<<<<<<<<<<<<<
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[2]);

    /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "dipy/align/vector_fields.pyx":2124
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_12);
  } else {
    __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(__pyx_t_4);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_warped = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__160) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_asarray); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_12);
    __pyx_t_12 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_13) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_15 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_13);
  if (unlikely(!__pyx_t_15.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_tmp = __pyx_t_15;
  __pyx_t_15.memview = NULL;
  __pyx_t_15.data = NULL;

  /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2129
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2130
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2131
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2133
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) ) + __pyx_t_25 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2134
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d1[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_26 * __pyx_v_d1.strides[0]) ) + __pyx_t_27 * __pyx_v_d1.strides[1]) ) + __pyx_t_28 * __pyx_v_d1.strides[2]) ) + __pyx_t_29 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2135
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_30 * __pyx_v_d1.strides[0]) ) + __pyx_t_31 * __pyx_v_d1.strides[1]) ) + __pyx_t_32 * __pyx_v_d1.strides[2]) ) + __pyx_t_33 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":2137
 *                         djj = d1[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":2138
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2139
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2141
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2143
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d1, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":2145
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2146
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2147
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2150
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2152
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2154
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":2157
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":2158
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":2159
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L17:;

              /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2162
 * 
 *                     if affine_idx_out is not None:
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 */
                __pyx_v_dkk = (__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2164
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2166
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                     else:
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
                goto __pyx_L18;
              }

              /* "dipy/align/vector_fields.pyx":2169
 *                                                        affine_idx_out)
 *                     else:
 *                         dkk = dk + k             # <<<<<<<<<<<<<<
 *                         dii = di + i
 *                         djj = dj + j
 */
              /*else*/ {
                __pyx_v_dkk = (__pyx_v_dk + __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":2170
 *                     else:
 *                         dkk = dk + k
 *                         dii = di + i             # <<<<<<<<<<<<<<
 *                         djj = dj + j
 * 
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":2171
 *                         dkk = dk + k
 *                         dii = di + i
 *                         djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
              }
              __pyx_L18:;

              /* "dipy/align/vector_fields.pyx":2174
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
              __pyx_t_37 = __pyx_v_k;
              __pyx_t_38 = __pyx_v_i;
              __pyx_t_39 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2173
 *                         djj = dj + j
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                        &warped[k,i,j])
 *     return warped
 */
              __pyx_v_inside = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_37 * __pyx_v_warped.strides[0]) ) + __pyx_t_38 * __pyx_v_warped.strides[1]) ) + __pyx_t_39 * __pyx_v_warped.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2175
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_13 = __pyx_memoryview_fromslice(__pyx_v_warped, 3, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2175; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_420__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults29, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults29, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults29, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults29, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1_1__pyx_pw_4dipy_5align_13vector_fields_229warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1_1__pyx_mdef_4dipy_5align_13vector_fields_229warp_3d_nn = {"__pyx_fuse_1_1warp_3d_nn", (PyCFunction)__pyx_fuse_1_1__pyx_pw_4dipy_5align_13vector_fields_229warp_3d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_42warp_3d_nn};
static PyObject *__pyx_fuse_1_1__pyx_pw_4dipy_5align_13vector_fields_229warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_3d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults29 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults29, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_3d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_228warp_3d_nn(__pyx_self, __pyx_v_volume, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_228warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dk;
  double __pyx_v_di;
  double __pyx_v_dj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1_1warp_3d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2097
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2098
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2099
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2100
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2101
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2102
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__161, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__162, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__163, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2115
 * 
 *     if out_shape is not None:
 *         nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 */
    __pyx_t_9 = 0;
    __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2116
 *     if out_shape is not None:
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 */
    __pyx_t_10 = 1;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2117
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 */
    __pyx_t_11 = 2;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_11 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2119
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 *         nslices = d1.shape[0]             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]
 */
    __pyx_v_nslices = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2120
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[2]
 * 
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2121
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]             # <<<<<<<<<<<<<<
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[2]);

    /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "dipy/align/vector_fields.pyx":2124
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_12);
  } else {
    __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(__pyx_t_4);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_warped = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__164) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_asarray); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_12);
    __pyx_t_12 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_13) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_15 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_13);
  if (unlikely(!__pyx_t_15.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_tmp = __pyx_t_15;
  __pyx_t_15.memview = NULL;
  __pyx_t_15.data = NULL;

  /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2129
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2130
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2131
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2133
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) ) + __pyx_t_25 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2134
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d1[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_26 * __pyx_v_d1.strides[0]) ) + __pyx_t_27 * __pyx_v_d1.strides[1]) ) + __pyx_t_28 * __pyx_v_d1.strides[2]) ) + __pyx_t_29 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2135
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_30 * __pyx_v_d1.strides[0]) ) + __pyx_t_31 * __pyx_v_d1.strides[1]) ) + __pyx_t_32 * __pyx_v_d1.strides[2]) ) + __pyx_t_33 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":2137
 *                         djj = d1[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":2138
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2139
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2141
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2143
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d1, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":2145
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2146
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2147
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2150
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2152
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2154
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":2157
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":2158
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":2159
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L17:;

              /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2162
 * 
 *                     if affine_idx_out is not None:
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 */
                __pyx_v_dkk = (__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2164
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2166
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                     else:
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
                goto __pyx_L18;
              }

              /* "dipy/align/vector_fields.pyx":2169
 *                                                        affine_idx_out)
 *                     else:
 *                         dkk = dk + k             # <<<<<<<<<<<<<<
 *                         dii = di + i
 *                         djj = dj + j
 */
              /*else*/ {
                __pyx_v_dkk = (__pyx_v_dk + __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":2170
 *                     else:
 *                         dkk = dk + k
 *                         dii = di + i             # <<<<<<<<<<<<<<
 *                         djj = dj + j
 * 
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":2171
 *                         dkk = dk + k
 *                         dii = di + i
 *                         djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
              }
              __pyx_L18:;

              /* "dipy/align/vector_fields.pyx":2174
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
              __pyx_t_37 = __pyx_v_k;
              __pyx_t_38 = __pyx_v_i;
              __pyx_t_39 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2173
 *                         djj = dj + j
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                        &warped[k,i,j])
 *     return warped
 */
              __pyx_v_inside = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_37 * __pyx_v_warped.strides[0]) ) + __pyx_t_38 * __pyx_v_warped.strides[1]) ) + __pyx_t_39 * __pyx_v_warped.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2175
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_13 = __pyx_memoryview_fromslice(__pyx_v_warped, 3, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2175; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_422__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults30, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults30, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults30, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults30, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_2_0__pyx_pw_4dipy_5align_13vector_fields_231warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_2_0__pyx_mdef_4dipy_5align_13vector_fields_231warp_3d_nn = {"__pyx_fuse_2_0warp_3d_nn", (PyCFunction)__pyx_fuse_2_0__pyx_pw_4dipy_5align_13vector_fields_231warp_3d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_42warp_3d_nn};
static PyObject *__pyx_fuse_2_0__pyx_pw_4dipy_5align_13vector_fields_231warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_3d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults30 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults30, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_3d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_long(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_230warp_3d_nn(__pyx_self, __pyx_v_volume, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_230warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dk;
  double __pyx_v_di;
  double __pyx_v_dj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_2_0warp_3d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2097
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2098
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2099
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2100
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2101
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2102
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__165, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__166, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__167, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2115
 * 
 *     if out_shape is not None:
 *         nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 */
    __pyx_t_9 = 0;
    __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2116
 *     if out_shape is not None:
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 */
    __pyx_t_10 = 1;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2117
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 */
    __pyx_t_11 = 2;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_11 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2119
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 *         nslices = d1.shape[0]             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]
 */
    __pyx_v_nslices = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2120
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[2]
 * 
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2121
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]             # <<<<<<<<<<<<<<
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[2]);

    /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "dipy/align/vector_fields.pyx":2124
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_long, (int (*)(char *, PyObject *)) __pyx_memview_set_long, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_12);
  } else {
    __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_long(__pyx_t_4);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_warped = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__168) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_asarray); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_12);
    __pyx_t_12 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_13) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_15 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_13);
  if (unlikely(!__pyx_t_15.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_tmp = __pyx_t_15;
  __pyx_t_15.memview = NULL;
  __pyx_t_15.data = NULL;

  /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2129
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2130
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2131
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2133
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) ) + __pyx_t_25 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2134
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d1[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_26 * __pyx_v_d1.strides[0]) ) + __pyx_t_27 * __pyx_v_d1.strides[1]) ) + __pyx_t_28 * __pyx_v_d1.strides[2]) ) + __pyx_t_29 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2135
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_30 * __pyx_v_d1.strides[0]) ) + __pyx_t_31 * __pyx_v_d1.strides[1]) ) + __pyx_t_32 * __pyx_v_d1.strides[2]) ) + __pyx_t_33 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":2137
 *                         djj = d1[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":2138
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2139
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2141
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2143
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d1, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":2145
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2146
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2147
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2150
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2152
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2154
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":2157
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":2158
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":2159
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L17:;

              /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2162
 * 
 *                     if affine_idx_out is not None:
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 */
                __pyx_v_dkk = (__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2164
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2166
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                     else:
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
                goto __pyx_L18;
              }

              /* "dipy/align/vector_fields.pyx":2169
 *                                                        affine_idx_out)
 *                     else:
 *                         dkk = dk + k             # <<<<<<<<<<<<<<
 *                         dii = di + i
 *                         djj = dj + j
 */
              /*else*/ {
                __pyx_v_dkk = (__pyx_v_dk + __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":2170
 *                     else:
 *                         dkk = dk + k
 *                         dii = di + i             # <<<<<<<<<<<<<<
 *                         djj = dj + j
 * 
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":2171
 *                         dkk = dk + k
 *                         dii = di + i
 *                         djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
              }
              __pyx_L18:;

              /* "dipy/align/vector_fields.pyx":2174
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
              __pyx_t_37 = __pyx_v_k;
              __pyx_t_38 = __pyx_v_i;
              __pyx_t_39 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2173
 *                         djj = dj + j
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                        &warped[k,i,j])
 *     return warped
 */
              __pyx_v_inside = __pyx_fuse_2__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((long *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_37 * __pyx_v_warped.strides[0]) ) + __pyx_t_38 * __pyx_v_warped.strides[1]) ) + __pyx_t_39 * __pyx_v_warped.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2175
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_13 = __pyx_memoryview_fromslice(__pyx_v_warped, 3, (PyObject *(*)(char *)) __pyx_memview_get_long, (int (*)(char *, PyObject *)) __pyx_memview_set_long, 0);; if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2175; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_424__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults31, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults31, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults31, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults31, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_2_1__pyx_pw_4dipy_5align_13vector_fields_233warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_2_1__pyx_mdef_4dipy_5align_13vector_fields_233warp_3d_nn = {"__pyx_fuse_2_1warp_3d_nn", (PyCFunction)__pyx_fuse_2_1__pyx_pw_4dipy_5align_13vector_fields_233warp_3d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_42warp_3d_nn};
static PyObject *__pyx_fuse_2_1__pyx_pw_4dipy_5align_13vector_fields_233warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_3d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults31 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults31, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_3d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_long(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_232warp_3d_nn(__pyx_self, __pyx_v_volume, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_232warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dk;
  double __pyx_v_di;
  double __pyx_v_dj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_2_1warp_3d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2097
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2098
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2099
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2100
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2101
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2102
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__169, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__170, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__171, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2115
 * 
 *     if out_shape is not None:
 *         nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 */
    __pyx_t_9 = 0;
    __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2116
 *     if out_shape is not None:
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 */
    __pyx_t_10 = 1;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2117
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 */
    __pyx_t_11 = 2;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_11 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2119
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 *         nslices = d1.shape[0]             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]
 */
    __pyx_v_nslices = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2120
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[2]
 * 
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2121
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]             # <<<<<<<<<<<<<<
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[2]);

    /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "dipy/align/vector_fields.pyx":2124
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_long, (int (*)(char *, PyObject *)) __pyx_memview_set_long, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_12);
  } else {
    __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_long(__pyx_t_4);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_warped = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__172) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_asarray); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_12);
    __pyx_t_12 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_13) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_15 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_13);
  if (unlikely(!__pyx_t_15.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_tmp = __pyx_t_15;
  __pyx_t_15.memview = NULL;
  __pyx_t_15.data = NULL;

  /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2129
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2130
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2131
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2133
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) ) + __pyx_t_25 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2134
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d1[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_26 * __pyx_v_d1.strides[0]) ) + __pyx_t_27 * __pyx_v_d1.strides[1]) ) + __pyx_t_28 * __pyx_v_d1.strides[2]) ) + __pyx_t_29 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2135
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_30 * __pyx_v_d1.strides[0]) ) + __pyx_t_31 * __pyx_v_d1.strides[1]) ) + __pyx_t_32 * __pyx_v_d1.strides[2]) ) + __pyx_t_33 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":2137
 *                         djj = d1[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":2138
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2139
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2141
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2143
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d1, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":2145
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2146
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2147
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2150
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2152
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2154
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":2157
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":2158
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":2159
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L17:;

              /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2162
 * 
 *                     if affine_idx_out is not None:
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 */
                __pyx_v_dkk = (__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2164
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2166
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                     else:
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
                goto __pyx_L18;
              }

              /* "dipy/align/vector_fields.pyx":2169
 *                                                        affine_idx_out)
 *                     else:
 *                         dkk = dk + k             # <<<<<<<<<<<<<<
 *                         dii = di + i
 *                         djj = dj + j
 */
              /*else*/ {
                __pyx_v_dkk = (__pyx_v_dk + __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":2170
 *                     else:
 *                         dkk = dk + k
 *                         dii = di + i             # <<<<<<<<<<<<<<
 *                         djj = dj + j
 * 
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":2171
 *                         dkk = dk + k
 *                         dii = di + i
 *                         djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
              }
              __pyx_L18:;

              /* "dipy/align/vector_fields.pyx":2174
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
              __pyx_t_37 = __pyx_v_k;
              __pyx_t_38 = __pyx_v_i;
              __pyx_t_39 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2173
 *                         djj = dj + j
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                        &warped[k,i,j])
 *     return warped
 */
              __pyx_v_inside = __pyx_fuse_2__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((long *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_37 * __pyx_v_warped.strides[0]) ) + __pyx_t_38 * __pyx_v_warped.strides[1]) ) + __pyx_t_39 * __pyx_v_warped.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2175
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_13 = __pyx_memoryview_fromslice(__pyx_v_warped, 3, (PyObject *(*)(char *)) __pyx_memview_get_long, (int (*)(char *, PyObject *)) __pyx_memview_set_long, 0);; if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2175; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_426__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults32, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults32, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults32, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults32, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_3_0__pyx_pw_4dipy_5align_13vector_fields_235warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_3_0__pyx_mdef_4dipy_5align_13vector_fields_235warp_3d_nn = {"__pyx_fuse_3_0warp_3d_nn", (PyCFunction)__pyx_fuse_3_0__pyx_pw_4dipy_5align_13vector_fields_235warp_3d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_42warp_3d_nn};
static PyObject *__pyx_fuse_3_0__pyx_pw_4dipy_5align_13vector_fields_235warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_3d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults32 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults32, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_3d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_234warp_3d_nn(__pyx_self, __pyx_v_volume, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_234warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dk;
  double __pyx_v_di;
  double __pyx_v_dj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_3_0warp_3d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2097
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2098
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2099
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2100
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2101
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2102
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__173, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__174, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__175, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2115
 * 
 *     if out_shape is not None:
 *         nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 */
    __pyx_t_9 = 0;
    __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2116
 *     if out_shape is not None:
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 */
    __pyx_t_10 = 1;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2117
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 */
    __pyx_t_11 = 2;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_11 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2119
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 *         nslices = d1.shape[0]             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]
 */
    __pyx_v_nslices = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2120
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[2]
 * 
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2121
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]             # <<<<<<<<<<<<<<
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[2]);

    /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "dipy/align/vector_fields.pyx":2124
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_12);
  } else {
    __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_t_4);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_warped = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__176) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_asarray); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_12);
    __pyx_t_12 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_13) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_15 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_13);
  if (unlikely(!__pyx_t_15.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_tmp = __pyx_t_15;
  __pyx_t_15.memview = NULL;
  __pyx_t_15.data = NULL;

  /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2129
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2130
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2131
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2133
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) ) + __pyx_t_25 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2134
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d1[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_26 * __pyx_v_d1.strides[0]) ) + __pyx_t_27 * __pyx_v_d1.strides[1]) ) + __pyx_t_28 * __pyx_v_d1.strides[2]) ) + __pyx_t_29 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2135
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_30 * __pyx_v_d1.strides[0]) ) + __pyx_t_31 * __pyx_v_d1.strides[1]) ) + __pyx_t_32 * __pyx_v_d1.strides[2]) ) + __pyx_t_33 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":2137
 *                         djj = d1[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":2138
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2139
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2141
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2143
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d1, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":2145
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2146
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2147
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2150
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2152
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2154
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":2157
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":2158
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":2159
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L17:;

              /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2162
 * 
 *                     if affine_idx_out is not None:
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 */
                __pyx_v_dkk = (__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2164
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2166
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                     else:
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
                goto __pyx_L18;
              }

              /* "dipy/align/vector_fields.pyx":2169
 *                                                        affine_idx_out)
 *                     else:
 *                         dkk = dk + k             # <<<<<<<<<<<<<<
 *                         dii = di + i
 *                         djj = dj + j
 */
              /*else*/ {
                __pyx_v_dkk = (__pyx_v_dk + __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":2170
 *                     else:
 *                         dkk = dk + k
 *                         dii = di + i             # <<<<<<<<<<<<<<
 *                         djj = dj + j
 * 
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":2171
 *                         dkk = dk + k
 *                         dii = di + i
 *                         djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
              }
              __pyx_L18:;

              /* "dipy/align/vector_fields.pyx":2174
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
              __pyx_t_37 = __pyx_v_k;
              __pyx_t_38 = __pyx_v_i;
              __pyx_t_39 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2173
 *                         djj = dj + j
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                        &warped[k,i,j])
 *     return warped
 */
              __pyx_v_inside = __pyx_fuse_3__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_37 * __pyx_v_warped.strides[0]) ) + __pyx_t_38 * __pyx_v_warped.strides[1]) ) + __pyx_t_39 * __pyx_v_warped.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2175
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_13 = __pyx_memoryview_fromslice(__pyx_v_warped, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2175; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_428__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults33, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults33, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults33, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults33, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_3_1__pyx_pw_4dipy_5align_13vector_fields_237warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_3_1__pyx_mdef_4dipy_5align_13vector_fields_237warp_3d_nn = {"__pyx_fuse_3_1warp_3d_nn", (PyCFunction)__pyx_fuse_3_1__pyx_pw_4dipy_5align_13vector_fields_237warp_3d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_42warp_3d_nn};
static PyObject *__pyx_fuse_3_1__pyx_pw_4dipy_5align_13vector_fields_237warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_3d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults33 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults33, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_3d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_236warp_3d_nn(__pyx_self, __pyx_v_volume, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_236warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dk;
  double __pyx_v_di;
  double __pyx_v_dj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_3_1warp_3d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2097
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2098
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2099
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2100
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2101
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2102
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__177, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__178, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__179, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2115
 * 
 *     if out_shape is not None:
 *         nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 */
    __pyx_t_9 = 0;
    __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2116
 *     if out_shape is not None:
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 */
    __pyx_t_10 = 1;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2117
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 */
    __pyx_t_11 = 2;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_11 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2119
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 *         nslices = d1.shape[0]             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]
 */
    __pyx_v_nslices = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2120
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[2]
 * 
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2121
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]             # <<<<<<<<<<<<<<
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[2]);

    /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "dipy/align/vector_fields.pyx":2124
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_12);
  } else {
    __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_t_4);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_warped = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__180) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_asarray); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_12);
    __pyx_t_12 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_13) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_15 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_13);
  if (unlikely(!__pyx_t_15.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_tmp = __pyx_t_15;
  __pyx_t_15.memview = NULL;
  __pyx_t_15.data = NULL;

  /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2129
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2130
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2131
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2133
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) ) + __pyx_t_25 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2134
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d1[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_26 * __pyx_v_d1.strides[0]) ) + __pyx_t_27 * __pyx_v_d1.strides[1]) ) + __pyx_t_28 * __pyx_v_d1.strides[2]) ) + __pyx_t_29 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2135
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_30 * __pyx_v_d1.strides[0]) ) + __pyx_t_31 * __pyx_v_d1.strides[1]) ) + __pyx_t_32 * __pyx_v_d1.strides[2]) ) + __pyx_t_33 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":2137
 *                         djj = d1[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":2138
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2139
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2141
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2143
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d1, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":2145
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2146
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2147
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2150
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2152
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2154
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":2157
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":2158
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":2159
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L17:;

              /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2162
 * 
 *                     if affine_idx_out is not None:
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 */
                __pyx_v_dkk = (__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2164
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2166
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                     else:
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
                goto __pyx_L18;
              }

              /* "dipy/align/vector_fields.pyx":2169
 *                                                        affine_idx_out)
 *                     else:
 *                         dkk = dk + k             # <<<<<<<<<<<<<<
 *                         dii = di + i
 *                         djj = dj + j
 */
              /*else*/ {
                __pyx_v_dkk = (__pyx_v_dk + __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":2170
 *                     else:
 *                         dkk = dk + k
 *                         dii = di + i             # <<<<<<<<<<<<<<
 *                         djj = dj + j
 * 
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":2171
 *                         dkk = dk + k
 *                         dii = di + i
 *                         djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
              }
              __pyx_L18:;

              /* "dipy/align/vector_fields.pyx":2174
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
              __pyx_t_37 = __pyx_v_k;
              __pyx_t_38 = __pyx_v_i;
              __pyx_t_39 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2173
 *                         djj = dj + j
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                        &warped[k,i,j])
 *     return warped
 */
              __pyx_v_inside = __pyx_fuse_3__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_37 * __pyx_v_warped.strides[0]) ) + __pyx_t_38 * __pyx_v_warped.strides[1]) ) + __pyx_t_39 * __pyx_v_warped.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2175
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_13 = __pyx_memoryview_fromslice(__pyx_v_warped, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2175; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_430__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults34, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults34, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults34, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults34, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_4_0__pyx_pw_4dipy_5align_13vector_fields_239warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_4_0__pyx_mdef_4dipy_5align_13vector_fields_239warp_3d_nn = {"__pyx_fuse_4_0warp_3d_nn", (PyCFunction)__pyx_fuse_4_0__pyx_pw_4dipy_5align_13vector_fields_239warp_3d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_42warp_3d_nn};
static PyObject *__pyx_fuse_4_0__pyx_pw_4dipy_5align_13vector_fields_239warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_3d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults34 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults34, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_3d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_238warp_3d_nn(__pyx_self, __pyx_v_volume, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_238warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dk;
  double __pyx_v_di;
  double __pyx_v_dj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_4_0warp_3d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2097
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2098
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2099
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2100
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2101
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2102
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__181, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__182, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__183, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2115
 * 
 *     if out_shape is not None:
 *         nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 */
    __pyx_t_9 = 0;
    __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2116
 *     if out_shape is not None:
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 */
    __pyx_t_10 = 1;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2117
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 */
    __pyx_t_11 = 2;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_11 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2119
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 *         nslices = d1.shape[0]             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]
 */
    __pyx_v_nslices = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2120
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[2]
 * 
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2121
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]             # <<<<<<<<<<<<<<
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[2]);

    /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "dipy/align/vector_fields.pyx":2124
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_12);
  } else {
    __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_warped = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__184) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_asarray); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_12);
    __pyx_t_12 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_13) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_15 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_13);
  if (unlikely(!__pyx_t_15.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_tmp = __pyx_t_15;
  __pyx_t_15.memview = NULL;
  __pyx_t_15.data = NULL;

  /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2129
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2130
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2131
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2133
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) ) + __pyx_t_25 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2134
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d1[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_26 * __pyx_v_d1.strides[0]) ) + __pyx_t_27 * __pyx_v_d1.strides[1]) ) + __pyx_t_28 * __pyx_v_d1.strides[2]) ) + __pyx_t_29 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2135
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_30 * __pyx_v_d1.strides[0]) ) + __pyx_t_31 * __pyx_v_d1.strides[1]) ) + __pyx_t_32 * __pyx_v_d1.strides[2]) ) + __pyx_t_33 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":2137
 *                         djj = d1[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":2138
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2139
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2141
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2143
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d1, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":2145
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2146
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2147
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2150
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2152
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2154
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":2157
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":2158
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":2159
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L17:;

              /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2162
 * 
 *                     if affine_idx_out is not None:
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 */
                __pyx_v_dkk = (__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2164
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2166
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                     else:
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
                goto __pyx_L18;
              }

              /* "dipy/align/vector_fields.pyx":2169
 *                                                        affine_idx_out)
 *                     else:
 *                         dkk = dk + k             # <<<<<<<<<<<<<<
 *                         dii = di + i
 *                         djj = dj + j
 */
              /*else*/ {
                __pyx_v_dkk = (__pyx_v_dk + __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":2170
 *                     else:
 *                         dkk = dk + k
 *                         dii = di + i             # <<<<<<<<<<<<<<
 *                         djj = dj + j
 * 
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":2171
 *                         dkk = dk + k
 *                         dii = di + i
 *                         djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
              }
              __pyx_L18:;

              /* "dipy/align/vector_fields.pyx":2174
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
              __pyx_t_37 = __pyx_v_k;
              __pyx_t_38 = __pyx_v_i;
              __pyx_t_39 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2173
 *                         djj = dj + j
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                        &warped[k,i,j])
 *     return warped
 */
              __pyx_v_inside = __pyx_fuse_4__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_37 * __pyx_v_warped.strides[0]) ) + __pyx_t_38 * __pyx_v_warped.strides[1]) ) + __pyx_t_39 * __pyx_v_warped.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2175
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_13 = __pyx_memoryview_fromslice(__pyx_v_warped, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2175; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_432__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults35, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults35, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults35, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults35, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_4_1__pyx_pw_4dipy_5align_13vector_fields_241warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_4_1__pyx_mdef_4dipy_5align_13vector_fields_241warp_3d_nn = {"__pyx_fuse_4_1warp_3d_nn", (PyCFunction)__pyx_fuse_4_1__pyx_pw_4dipy_5align_13vector_fields_241warp_3d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_42warp_3d_nn};
static PyObject *__pyx_fuse_4_1__pyx_pw_4dipy_5align_13vector_fields_241warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_3d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults35 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults35, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_3d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_240warp_3d_nn(__pyx_self, __pyx_v_volume, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_240warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dk;
  double __pyx_v_di;
  double __pyx_v_dj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_4_1warp_3d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2097
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2098
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2099
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2100
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2101
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2102
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__185, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__186, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__187, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2115
 * 
 *     if out_shape is not None:
 *         nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 */
    __pyx_t_9 = 0;
    __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2116
 *     if out_shape is not None:
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 */
    __pyx_t_10 = 1;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2117
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 */
    __pyx_t_11 = 2;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_11 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2119
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 *         nslices = d1.shape[0]             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]
 */
    __pyx_v_nslices = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2120
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[2]
 * 
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2121
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]             # <<<<<<<<<<<<<<
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[2]);

    /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "dipy/align/vector_fields.pyx":2124
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_12);
  } else {
    __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_warped = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__188) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_asarray); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_12);
    __pyx_t_12 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_13) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_15 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_13);
  if (unlikely(!__pyx_t_15.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_tmp = __pyx_t_15;
  __pyx_t_15.memview = NULL;
  __pyx_t_15.data = NULL;

  /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2129
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2130
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2131
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2133
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) ) + __pyx_t_25 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2134
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d1[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_26 * __pyx_v_d1.strides[0]) ) + __pyx_t_27 * __pyx_v_d1.strides[1]) ) + __pyx_t_28 * __pyx_v_d1.strides[2]) ) + __pyx_t_29 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2135
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_30 * __pyx_v_d1.strides[0]) ) + __pyx_t_31 * __pyx_v_d1.strides[1]) ) + __pyx_t_32 * __pyx_v_d1.strides[2]) ) + __pyx_t_33 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":2137
 *                         djj = d1[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":2138
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2139
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2141
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2143
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d1, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":2145
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2146
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2147
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2150
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2152
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2154
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":2157
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":2158
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":2159
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L17:;

              /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2162
 * 
 *                     if affine_idx_out is not None:
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 */
                __pyx_v_dkk = (__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2164
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2166
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                     else:
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
                goto __pyx_L18;
              }

              /* "dipy/align/vector_fields.pyx":2169
 *                                                        affine_idx_out)
 *                     else:
 *                         dkk = dk + k             # <<<<<<<<<<<<<<
 *                         dii = di + i
 *                         djj = dj + j
 */
              /*else*/ {
                __pyx_v_dkk = (__pyx_v_dk + __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":2170
 *                     else:
 *                         dkk = dk + k
 *                         dii = di + i             # <<<<<<<<<<<<<<
 *                         djj = dj + j
 * 
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":2171
 *                         dkk = dk + k
 *                         dii = di + i
 *                         djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
              }
              __pyx_L18:;

              /* "dipy/align/vector_fields.pyx":2174
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
              __pyx_t_37 = __pyx_v_k;
              __pyx_t_38 = __pyx_v_i;
              __pyx_t_39 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2173
 *                         djj = dj + j
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                        &warped[k,i,j])
 *     return warped
 */
              __pyx_v_inside = __pyx_fuse_4__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_37 * __pyx_v_warped.strides[0]) ) + __pyx_t_38 * __pyx_v_warped.strides[1]) ) + __pyx_t_39 * __pyx_v_warped.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2175
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_13 = __pyx_memoryview_fromslice(__pyx_v_warped, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2175; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_434__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults36, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults36, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults36, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults36, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_5_0__pyx_pw_4dipy_5align_13vector_fields_243warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_5_0__pyx_mdef_4dipy_5align_13vector_fields_243warp_3d_nn = {"__pyx_fuse_5_0warp_3d_nn", (PyCFunction)__pyx_fuse_5_0__pyx_pw_4dipy_5align_13vector_fields_243warp_3d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_42warp_3d_nn};
static PyObject *__pyx_fuse_5_0__pyx_pw_4dipy_5align_13vector_fields_243warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_3d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults36 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults36, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_3d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_float_complex(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_242warp_3d_nn(__pyx_self, __pyx_v_volume, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_242warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dk;
  double __pyx_v_di;
  double __pyx_v_dj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_5_0warp_3d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2097
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2098
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2099
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2100
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2101
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2102
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__189, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__190, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__191, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2115
 * 
 *     if out_shape is not None:
 *         nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 */
    __pyx_t_9 = 0;
    __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2116
 *     if out_shape is not None:
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 */
    __pyx_t_10 = 1;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2117
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 */
    __pyx_t_11 = 2;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_11 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2119
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 *         nslices = d1.shape[0]             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]
 */
    __pyx_v_nslices = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2120
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[2]
 * 
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2121
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]             # <<<<<<<<<<<<<<
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[2]);

    /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "dipy/align/vector_fields.pyx":2124
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_float_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_float_complex, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_12);
  } else {
    __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_float_complex(__pyx_t_4);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_warped = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__192) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_asarray); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_12);
    __pyx_t_12 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_13) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_15 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_13);
  if (unlikely(!__pyx_t_15.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_tmp = __pyx_t_15;
  __pyx_t_15.memview = NULL;
  __pyx_t_15.data = NULL;

  /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2129
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2130
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2131
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2133
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) ) + __pyx_t_25 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2134
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d1[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_26 * __pyx_v_d1.strides[0]) ) + __pyx_t_27 * __pyx_v_d1.strides[1]) ) + __pyx_t_28 * __pyx_v_d1.strides[2]) ) + __pyx_t_29 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2135
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_30 * __pyx_v_d1.strides[0]) ) + __pyx_t_31 * __pyx_v_d1.strides[1]) ) + __pyx_t_32 * __pyx_v_d1.strides[2]) ) + __pyx_t_33 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":2137
 *                         djj = d1[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":2138
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2139
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2141
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2143
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d1, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":2145
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2146
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2147
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2150
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2152
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2154
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":2157
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":2158
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":2159
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L17:;

              /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2162
 * 
 *                     if affine_idx_out is not None:
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 */
                __pyx_v_dkk = (__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2164
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2166
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                     else:
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
                goto __pyx_L18;
              }

              /* "dipy/align/vector_fields.pyx":2169
 *                                                        affine_idx_out)
 *                     else:
 *                         dkk = dk + k             # <<<<<<<<<<<<<<
 *                         dii = di + i
 *                         djj = dj + j
 */
              /*else*/ {
                __pyx_v_dkk = (__pyx_v_dk + __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":2170
 *                     else:
 *                         dkk = dk + k
 *                         dii = di + i             # <<<<<<<<<<<<<<
 *                         djj = dj + j
 * 
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":2171
 *                         dkk = dk + k
 *                         dii = di + i
 *                         djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
              }
              __pyx_L18:;

              /* "dipy/align/vector_fields.pyx":2174
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
              __pyx_t_37 = __pyx_v_k;
              __pyx_t_38 = __pyx_v_i;
              __pyx_t_39 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2173
 *                         djj = dj + j
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                        &warped[k,i,j])
 *     return warped
 */
              __pyx_v_inside = __pyx_fuse_5__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((__pyx_t_float_complex *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_37 * __pyx_v_warped.strides[0]) ) + __pyx_t_38 * __pyx_v_warped.strides[1]) ) + __pyx_t_39 * __pyx_v_warped.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2175
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_13 = __pyx_memoryview_fromslice(__pyx_v_warped, 3, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_float_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_float_complex, 0);; if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2175; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_436__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults37, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults37, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults37, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults37, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_5_1__pyx_pw_4dipy_5align_13vector_fields_245warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_5_1__pyx_mdef_4dipy_5align_13vector_fields_245warp_3d_nn = {"__pyx_fuse_5_1warp_3d_nn", (PyCFunction)__pyx_fuse_5_1__pyx_pw_4dipy_5align_13vector_fields_245warp_3d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_42warp_3d_nn};
static PyObject *__pyx_fuse_5_1__pyx_pw_4dipy_5align_13vector_fields_245warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_3d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults37 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults37, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_3d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_float_complex(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_244warp_3d_nn(__pyx_self, __pyx_v_volume, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_244warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dk;
  double __pyx_v_di;
  double __pyx_v_dj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_5_1warp_3d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2097
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2098
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2099
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2100
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2101
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2102
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__193, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__194, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__195, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2115
 * 
 *     if out_shape is not None:
 *         nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 */
    __pyx_t_9 = 0;
    __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2116
 *     if out_shape is not None:
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 */
    __pyx_t_10 = 1;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2117
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 */
    __pyx_t_11 = 2;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_11 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2119
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 *         nslices = d1.shape[0]             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]
 */
    __pyx_v_nslices = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2120
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[2]
 * 
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2121
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]             # <<<<<<<<<<<<<<
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[2]);

    /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "dipy/align/vector_fields.pyx":2124
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_float_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_float_complex, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_12);
  } else {
    __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_float_complex(__pyx_t_4);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_warped = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__196) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_asarray); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_12);
    __pyx_t_12 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_13) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_15 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_13);
  if (unlikely(!__pyx_t_15.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_tmp = __pyx_t_15;
  __pyx_t_15.memview = NULL;
  __pyx_t_15.data = NULL;

  /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2129
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2130
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2131
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2133
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) ) + __pyx_t_25 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2134
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d1[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_26 * __pyx_v_d1.strides[0]) ) + __pyx_t_27 * __pyx_v_d1.strides[1]) ) + __pyx_t_28 * __pyx_v_d1.strides[2]) ) + __pyx_t_29 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2135
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_30 * __pyx_v_d1.strides[0]) ) + __pyx_t_31 * __pyx_v_d1.strides[1]) ) + __pyx_t_32 * __pyx_v_d1.strides[2]) ) + __pyx_t_33 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":2137
 *                         djj = d1[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":2138
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2139
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2141
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2143
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d1, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":2145
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2146
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2147
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2150
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2152
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2154
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":2157
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":2158
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":2159
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L17:;

              /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2162
 * 
 *                     if affine_idx_out is not None:
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 */
                __pyx_v_dkk = (__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2164
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2166
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                     else:
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
                goto __pyx_L18;
              }

              /* "dipy/align/vector_fields.pyx":2169
 *                                                        affine_idx_out)
 *                     else:
 *                         dkk = dk + k             # <<<<<<<<<<<<<<
 *                         dii = di + i
 *                         djj = dj + j
 */
              /*else*/ {
                __pyx_v_dkk = (__pyx_v_dk + __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":2170
 *                     else:
 *                         dkk = dk + k
 *                         dii = di + i             # <<<<<<<<<<<<<<
 *                         djj = dj + j
 * 
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":2171
 *                         dkk = dk + k
 *                         dii = di + i
 *                         djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
              }
              __pyx_L18:;

              /* "dipy/align/vector_fields.pyx":2174
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
              __pyx_t_37 = __pyx_v_k;
              __pyx_t_38 = __pyx_v_i;
              __pyx_t_39 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2173
 *                         djj = dj + j
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                        &warped[k,i,j])
 *     return warped
 */
              __pyx_v_inside = __pyx_fuse_5__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((__pyx_t_float_complex *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_37 * __pyx_v_warped.strides[0]) ) + __pyx_t_38 * __pyx_v_warped.strides[1]) ) + __pyx_t_39 * __pyx_v_warped.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2175
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_13 = __pyx_memoryview_fromslice(__pyx_v_warped, 3, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_float_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_float_complex, 0);; if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2175; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_438__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults38, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults38, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults38, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults38, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_6_0__pyx_pw_4dipy_5align_13vector_fields_247warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_6_0__pyx_mdef_4dipy_5align_13vector_fields_247warp_3d_nn = {"__pyx_fuse_6_0warp_3d_nn", (PyCFunction)__pyx_fuse_6_0__pyx_pw_4dipy_5align_13vector_fields_247warp_3d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_42warp_3d_nn};
static PyObject *__pyx_fuse_6_0__pyx_pw_4dipy_5align_13vector_fields_247warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_3d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults38 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults38, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_3d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_double_complex(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_246warp_3d_nn(__pyx_self, __pyx_v_volume, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_246warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dk;
  double __pyx_v_di;
  double __pyx_v_dj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_6_0warp_3d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2097
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2098
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2099
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2100
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2101
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2102
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__197, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__198, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__199, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2115
 * 
 *     if out_shape is not None:
 *         nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 */
    __pyx_t_9 = 0;
    __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2116
 *     if out_shape is not None:
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 */
    __pyx_t_10 = 1;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2117
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 */
    __pyx_t_11 = 2;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_11 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2119
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 *         nslices = d1.shape[0]             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]
 */
    __pyx_v_nslices = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2120
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[2]
 * 
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2121
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]             # <<<<<<<<<<<<<<
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[2]);

    /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "dipy/align/vector_fields.pyx":2124
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_double_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_double_complex, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_12);
  } else {
    __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_double_complex(__pyx_t_4);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_warped = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__200) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_asarray); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_12);
    __pyx_t_12 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_13) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_15 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_13);
  if (unlikely(!__pyx_t_15.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_tmp = __pyx_t_15;
  __pyx_t_15.memview = NULL;
  __pyx_t_15.data = NULL;

  /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2129
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2130
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2131
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2133
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) ) + __pyx_t_25 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2134
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d1[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_26 * __pyx_v_d1.strides[0]) ) + __pyx_t_27 * __pyx_v_d1.strides[1]) ) + __pyx_t_28 * __pyx_v_d1.strides[2]) ) + __pyx_t_29 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2135
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_30 * __pyx_v_d1.strides[0]) ) + __pyx_t_31 * __pyx_v_d1.strides[1]) ) + __pyx_t_32 * __pyx_v_d1.strides[2]) ) + __pyx_t_33 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":2137
 *                         djj = d1[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":2138
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2139
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2141
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2143
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d1, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":2145
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2146
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2147
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2150
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2152
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2154
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":2157
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":2158
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":2159
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L17:;

              /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2162
 * 
 *                     if affine_idx_out is not None:
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 */
                __pyx_v_dkk = (__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2164
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2166
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                     else:
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
                goto __pyx_L18;
              }

              /* "dipy/align/vector_fields.pyx":2169
 *                                                        affine_idx_out)
 *                     else:
 *                         dkk = dk + k             # <<<<<<<<<<<<<<
 *                         dii = di + i
 *                         djj = dj + j
 */
              /*else*/ {
                __pyx_v_dkk = (__pyx_v_dk + __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":2170
 *                     else:
 *                         dkk = dk + k
 *                         dii = di + i             # <<<<<<<<<<<<<<
 *                         djj = dj + j
 * 
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":2171
 *                         dkk = dk + k
 *                         dii = di + i
 *                         djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
              }
              __pyx_L18:;

              /* "dipy/align/vector_fields.pyx":2174
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
              __pyx_t_37 = __pyx_v_k;
              __pyx_t_38 = __pyx_v_i;
              __pyx_t_39 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2173
 *                         djj = dj + j
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                        &warped[k,i,j])
 *     return warped
 */
              __pyx_v_inside = __pyx_fuse_6__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((__pyx_t_double_complex *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_37 * __pyx_v_warped.strides[0]) ) + __pyx_t_38 * __pyx_v_warped.strides[1]) ) + __pyx_t_39 * __pyx_v_warped.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2175
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_13 = __pyx_memoryview_fromslice(__pyx_v_warped, 3, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_double_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_double_complex, 0);; if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2175; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_440__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults39, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults39, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults39, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults39, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_6_1__pyx_pw_4dipy_5align_13vector_fields_249warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_6_1__pyx_mdef_4dipy_5align_13vector_fields_249warp_3d_nn = {"__pyx_fuse_6_1warp_3d_nn", (PyCFunction)__pyx_fuse_6_1__pyx_pw_4dipy_5align_13vector_fields_249warp_3d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_42warp_3d_nn};
static PyObject *__pyx_fuse_6_1__pyx_pw_4dipy_5align_13vector_fields_249warp_3d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_3d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults39 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults39, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_3d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_double_complex(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_3d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_248warp_3d_nn(__pyx_self, __pyx_v_volume, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_248warp_3d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  double __pyx_v_dk;
  double __pyx_v_di;
  double __pyx_v_dj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_6_1warp_3d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2097
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2098
 *     cdef:
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2099
 *         cnp.npy_intp nslices = volume.shape[0]
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2100
 *         cnp.npy_intp nrows = volume.shape[1]
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2101
 *         cnp.npy_intp ncols = volume.shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         cnp.npy_intp i, j, k
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2102
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k
 *         int inside
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2107; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__201, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2107
 *         double dkk, dii, djj, dk, di, dj
 * 
 *     if not is_valid_affine(affine_idx_in, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2109; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__202, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2109
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_3);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2111; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__203, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2111
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2115
 * 
 *     if out_shape is not None:
 *         nslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 */
    __pyx_t_9 = 0;
    __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2116
 *     if out_shape is not None:
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 */
    __pyx_t_10 = 1;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2117
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 */
    __pyx_t_11 = 2;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_11 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2114
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nslices = out_shape[0]
 *         nrows = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2119
 *         ncols = out_shape[2]
 *     elif d1 is not None:
 *         nslices = d1.shape[0]             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]
 */
    __pyx_v_nslices = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2120
 *     elif d1 is not None:
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[2]
 * 
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2121
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 *         ncols = d1.shape[2]             # <<<<<<<<<<<<<<
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[2]);

    /* "dipy/align/vector_fields.pyx":2118
 *         nrows = out_shape[1]
 *         ncols = out_shape[2]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nslices = d1.shape[0]
 *         nrows = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = PyTuple_New(3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_12, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_12, 2, __pyx_t_6);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "dipy/align/vector_fields.pyx":2124
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_double_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_double_complex, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_6); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_12);
  } else {
    __pyx_t_13 = PyTuple_New(1+1); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_13, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_13, NULL); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_dtype); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2124; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2123
 *         ncols = d1.shape[2]
 * 
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_double_complex(__pyx_t_4);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2123; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_warped = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__204) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_asarray); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_d1, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_6 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
    __Pyx_GIVEREF(__pyx_t_12);
    PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_12);
    __pyx_t_12 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_13, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_13) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_15 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_13);
  if (unlikely(!__pyx_t_15.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_tmp = __pyx_t_15;
  __pyx_t_15.memview = NULL;
  __pyx_t_15.data = NULL;

  /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2129
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2130
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2131
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2133
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]             # <<<<<<<<<<<<<<
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]
 */
                __pyx_t_22 = __pyx_v_k;
                __pyx_t_23 = __pyx_v_i;
                __pyx_t_24 = __pyx_v_j;
                __pyx_t_25 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) ) + __pyx_t_25 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2134
 *                     if affine_idx_in is None:
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]             # <<<<<<<<<<<<<<
 *                         djj = d1[k, i, j, 2]
 *                     else:
 */
                __pyx_t_26 = __pyx_v_k;
                __pyx_t_27 = __pyx_v_i;
                __pyx_t_28 = __pyx_v_j;
                __pyx_t_29 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_26 * __pyx_v_d1.strides[0]) ) + __pyx_t_27 * __pyx_v_d1.strides[1]) ) + __pyx_t_28 * __pyx_v_d1.strides[2]) ) + __pyx_t_29 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2135
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 *                         djj = d1[k, i, j, 2]             # <<<<<<<<<<<<<<
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 */
                __pyx_t_30 = __pyx_v_k;
                __pyx_t_31 = __pyx_v_i;
                __pyx_t_32 = __pyx_v_j;
                __pyx_t_33 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_30 * __pyx_v_d1.strides[0]) ) + __pyx_t_31 * __pyx_v_d1.strides[1]) ) + __pyx_t_32 * __pyx_v_d1.strides[2]) ) + __pyx_t_33 * __pyx_v_d1.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":2132
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                         dkk = d1[k, i, j, 0]
 *                         dii = d1[k, i, j, 1]
 */
                goto __pyx_L16;
              }

              /* "dipy/align/vector_fields.pyx":2137
 *                         djj = d1[k, i, j, 2]
 *                     else:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(
 */
              /*else*/ {

                /* "dipy/align/vector_fields.pyx":2138
 *                     else:
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2139
 *                         dk = _apply_affine_3d_x0(
 *                             k, i, j, 1, affine_idx_in)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2141
 *                         di = _apply_affine_3d_x1(
 *                             k, i, j, 1, affine_idx_in)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

                /* "dipy/align/vector_fields.pyx":2143
 *                         dj = _apply_affine_3d_x2(
 *                             k, i, j, 1, affine_idx_in)
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,             # <<<<<<<<<<<<<<
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 */
                __pyx_v_inside = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_d1, __pyx_v_dk, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

                /* "dipy/align/vector_fields.pyx":2145
 *                         inside = _interpolate_vector_3d[floating](d1, dk, di,
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]             # <<<<<<<<<<<<<<
 *                         dii = tmp[1]
 *                         djj = tmp[2]
 */
                __pyx_t_34 = 0;
                __pyx_v_dkk = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_34 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2146
 *                                                                   dj, tmp)
 *                         dkk = tmp[0]
 *                         dii = tmp[1]             # <<<<<<<<<<<<<<
 *                         djj = tmp[2]
 * 
 */
                __pyx_t_35 = 1;
                __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_35 * __pyx_v_tmp.strides[0]) )));

                /* "dipy/align/vector_fields.pyx":2147
 *                         dkk = tmp[0]
 *                         dii = tmp[1]
 *                         djj = tmp[2]             # <<<<<<<<<<<<<<
 * 
 *                     if affine_disp is not None:
 */
                __pyx_t_36 = 2;
                __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_36 * __pyx_v_tmp.strides[0]) )));
              }
              __pyx_L16:;

              /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2150
 * 
 *                     if affine_disp is not None:
 *                         dk = _apply_affine_3d_x0(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(
 */
                __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2152
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         di = _apply_affine_3d_x1(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(
 */
                __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2154
 *                         di = _apply_affine_3d_x1(
 *                             dkk, dii, djj, 0, affine_disp)
 *                         dj = _apply_affine_3d_x2(             # <<<<<<<<<<<<<<
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 */
                __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

                /* "dipy/align/vector_fields.pyx":2149
 *                         djj = tmp[2]
 * 
 *                     if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                         dk = _apply_affine_3d_x0(
 *                             dkk, dii, djj, 0, affine_disp)
 */
                goto __pyx_L17;
              }

              /* "dipy/align/vector_fields.pyx":2157
 *                             dkk, dii, djj, 0, affine_disp)
 *                     else:
 *                         dk = dkk             # <<<<<<<<<<<<<<
 *                         di = dii
 *                         dj = djj
 */
              /*else*/ {
                __pyx_v_dk = __pyx_v_dkk;

                /* "dipy/align/vector_fields.pyx":2158
 *                     else:
 *                         dk = dkk
 *                         di = dii             # <<<<<<<<<<<<<<
 *                         dj = djj
 * 
 */
                __pyx_v_di = __pyx_v_dii;

                /* "dipy/align/vector_fields.pyx":2159
 *                         dk = dkk
 *                         di = dii
 *                         dj = djj             # <<<<<<<<<<<<<<
 * 
 *                     if affine_idx_out is not None:
 */
                __pyx_v_dj = __pyx_v_djj;
              }
              __pyx_L17:;

              /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
              __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
              if (__pyx_t_8) {

                /* "dipy/align/vector_fields.pyx":2162
 * 
 *                     if affine_idx_out is not None:
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 */
                __pyx_v_dkk = (__pyx_v_dk + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2164
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2166
 *                         dii = di + _apply_affine_3d_x1(k, i, j, 1,
 *                                                        affine_idx_out)
 *                         djj = dj + _apply_affine_3d_x2(k, i, j, 1,             # <<<<<<<<<<<<<<
 *                                                        affine_idx_out)
 *                     else:
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

                /* "dipy/align/vector_fields.pyx":2161
 *                         dj = djj
 * 
 *                     if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                         dkk = dk + _apply_affine_3d_x0(k, i, j, 1,
 *                                                        affine_idx_out)
 */
                goto __pyx_L18;
              }

              /* "dipy/align/vector_fields.pyx":2169
 *                                                        affine_idx_out)
 *                     else:
 *                         dkk = dk + k             # <<<<<<<<<<<<<<
 *                         dii = di + i
 *                         djj = dj + j
 */
              /*else*/ {
                __pyx_v_dkk = (__pyx_v_dk + __pyx_v_k);

                /* "dipy/align/vector_fields.pyx":2170
 *                     else:
 *                         dkk = dk + k
 *                         dii = di + i             # <<<<<<<<<<<<<<
 *                         djj = dj + j
 * 
 */
                __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

                /* "dipy/align/vector_fields.pyx":2171
 *                         dkk = dk + k
 *                         dii = di + i
 *                         djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
              }
              __pyx_L18:;

              /* "dipy/align/vector_fields.pyx":2174
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
              __pyx_t_37 = __pyx_v_k;
              __pyx_t_38 = __pyx_v_i;
              __pyx_t_39 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2173
 *                         djj = dj + j
 * 
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                        &warped[k,i,j])
 *     return warped
 */
              __pyx_v_inside = __pyx_fuse_6__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((__pyx_t_double_complex *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_37 * __pyx_v_warped.strides[0]) ) + __pyx_t_38 * __pyx_v_warped.strides[1]) ) + __pyx_t_39 * __pyx_v_warped.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2127
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2175
 *                     inside = _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                        &warped[k,i,j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_13 = __pyx_memoryview_fromslice(__pyx_v_warped, 3, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_double_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_double_complex, 0);; if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2175; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_r = __pyx_t_13;
  __pyx_t_13 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_3d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_45transform_3d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_44transform_3d_affine_nn[] = "Transforms a 3D volume by an affine transform with NN interpolation\n\n    Deforms the input volume under the given affine transformation using\n    nearest neighbor interpolation. The shape of the resulting volume\n    is given by ref_shape. If the affine matrix is None, it is taken as the\n    identity.\n\n    Parameters\n    ----------\n    volume : array, shape (S, R, C)\n        the input volume to be transformed\n    ref_shape : array, shape (3,)\n        the shape of the resulting volume\n    affine : array, shape (4, 4)\n        the affine transform to be applied\n\n    Returns\n    -------\n    out : array, shape (S', R', C')\n        the transformed volume\n\n    Notes\n    -----\n    The reason it is necessary to provide the intended shape of the resulting\n    volume is because the affine transformation is defined on all R^{3}\n    but we must sample a finite lattice. Also the resulting shape may not be\n    necessarily equal to the input shape, unless we are interested on\n    endomorphisms only and not general diffeomorphisms.\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_45transform_3d_affine_nn = {"transform_3d_affine_nn", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_45transform_3d_affine_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_44transform_3d_affine_nn};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_45transform_3d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    values[2] = __pyx_k__205;
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_44transform_3d_affine_nn(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_44transform_3d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  int __pyx_v_short_is_signed;
  int __pyx_v_long_is_signed;
  int __pyx_v_int_is_signed;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("transform_3d_affine_nn", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  __pyx_v_short_is_signed = (((short)-1L) < 0);
  __pyx_v_long_is_signed = (((long)-1L) < 0);
  __pyx_v_int_is_signed = (((int)-1L) < 0);
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_volume, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_volume); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          __pyx_t_3 = (((sizeof(short)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_short_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_short, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(int)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L26_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L26_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_int_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L26_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_int, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(long)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L30_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L30_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_long_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L30_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_long, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L34_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L34_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L37_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L37_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          __pyx_t_3 = (((sizeof(__pyx_t_float_complex)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L40_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L40_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_float_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(__pyx_t_double_complex)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L43_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L43_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_double_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L46_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(short))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L46_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_short(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_short, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L50_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(int))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L50_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_int, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L54_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(long))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L54_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_long(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_long, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L58_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L58_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L62_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L62_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L66_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(__pyx_t_float_complex))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L66_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_float_complex(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_float_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L70_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(__pyx_t_double_complex))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L70_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_double_complex(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_double_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__206, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__207, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L77_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L77_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L78_unpacking_done;
        __pyx_L77_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L78_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L80;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L76_break;
        }
        __pyx_L80:;
      }
    }
    __pyx_L76_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__208, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__209, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_456__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults47, __pyx_self)->__pyx_arg_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_253transform_3d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_253transform_3d_affine_nn = {"__pyx_fuse_0transform_3d_affine_nn", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_253transform_3d_affine_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_44transform_3d_affine_nn};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_253transform_3d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_3d_affine_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    __pyx_defaults47 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults47, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_3d_affine_nn", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_3d_affine_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_short(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2179; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine = __pyx_dynamic_args->__pyx_arg_affine;
      __PYX_INC_MEMVIEW(&__pyx_v_affine, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_3d_affine_nn", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_252transform_3d_affine_nn(__pyx_self, __pyx_v_volume, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_252transform_3d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_memviewslice __pyx_t_12 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0transform_3d_affine_nn", 0);

  /* "dipy/align/vector_fields.pyx":2210
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 */
  __pyx_t_1 = 0;
  __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2211
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_t_2 = 1;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2212
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_t_3 = 2;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_3 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2213
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2214
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         double dkk, dii, djj, tmp0, tmp1
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2215
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         double dkk, dii, djj, tmp0, tmp1
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2219
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_7);
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_8);
  __pyx_t_8 = 0;

  /* "dipy/align/vector_fields.pyx":2220
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),
 *                                         dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 3):
 */
  __pyx_t_8 = PyDict_New(); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_short, (int (*)(char *, PyObject *)) __pyx_memview_set_short, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_10 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  if (!__pyx_t_10) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_11, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_dtype, __pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "dipy/align/vector_fields.pyx":2219
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, __pyx_t_8); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_12 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_short(__pyx_t_9);
  if (unlikely(!__pyx_t_12.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_out = __pyx_t_12;
  __pyx_t_12.memview = NULL;
  __pyx_t_12.data = NULL;

  /* "dipy/align/vector_fields.pyx":2222
 *                                         dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_5 = NULL;
  __pyx_t_13 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_13 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_5) {
    __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_13, __pyx_t_7);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_13, __pyx_int_3);
  __pyx_t_7 = 0;
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_6, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_14 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_15 = ((!__pyx_t_14) != 0);
  if (__pyx_t_15) {

    /* "dipy/align/vector_fields.pyx":2223
 * 
 *     if not is_valid_affine(affine, 3):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__210, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_Raise(__pyx_t_9, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2222
 *                                         dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2225
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2227
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2228
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine is not None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2229
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2230
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
              __pyx_t_15 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
              if (__pyx_t_15) {

                /* "dipy/align/vector_fields.pyx":2231
 *                 for j in range(ncols):
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 */
                __pyx_v_dkk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2232
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 */
                __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2233
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     else:
 *                         dkk = k
 */
                __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2230
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
                goto __pyx_L13;
              }

              /* "dipy/align/vector_fields.pyx":2235
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 *                         dkk = k             # <<<<<<<<<<<<<<
 *                         dii = i
 *                         djj = j
 */
              /*else*/ {
                __pyx_v_dkk = __pyx_v_k;

                /* "dipy/align/vector_fields.pyx":2236
 *                     else:
 *                         dkk = k
 *                         dii = i             # <<<<<<<<<<<<<<
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_dii = __pyx_v_i;

                /* "dipy/align/vector_fields.pyx":2237
 *                         dkk = k
 *                         dii = i
 *                         djj = j             # <<<<<<<<<<<<<<
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])
 */
                __pyx_v_djj = __pyx_v_j;
              }
              __pyx_L13:;

              /* "dipy/align/vector_fields.pyx":2239
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
              __pyx_t_22 = __pyx_v_k;
              __pyx_t_23 = __pyx_v_i;
              __pyx_t_24 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2238
 *                         dii = i
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                       &out[k,i,j])
 *     return out
 */
              __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((short *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_22 * __pyx_v_out.strides[0]) ) + __pyx_t_23 * __pyx_v_out.strides[1]) ) + __pyx_t_24 * __pyx_v_out.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2225
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2240
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_9 = __pyx_memoryview_fromslice(__pyx_v_out, 3, (PyObject *(*)(char *)) __pyx_memview_get_short, (int (*)(char *, PyObject *)) __pyx_memview_set_short, 0);; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2240; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_r = __pyx_t_9;
  __pyx_t_9 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __PYX_XDEC_MEMVIEW(&__pyx_t_12, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_458__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults48, __pyx_self)->__pyx_arg_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_255transform_3d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_255transform_3d_affine_nn = {"__pyx_fuse_1transform_3d_affine_nn", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_255transform_3d_affine_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_44transform_3d_affine_nn};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_255transform_3d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_3d_affine_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    __pyx_defaults48 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults48, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_3d_affine_nn", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_3d_affine_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2179; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine = __pyx_dynamic_args->__pyx_arg_affine;
      __PYX_INC_MEMVIEW(&__pyx_v_affine, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_3d_affine_nn", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_254transform_3d_affine_nn(__pyx_self, __pyx_v_volume, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_254transform_3d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_memviewslice __pyx_t_12 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1transform_3d_affine_nn", 0);

  /* "dipy/align/vector_fields.pyx":2210
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 */
  __pyx_t_1 = 0;
  __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2211
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_t_2 = 1;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2212
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_t_3 = 2;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_3 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2213
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2214
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         double dkk, dii, djj, tmp0, tmp1
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2215
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         double dkk, dii, djj, tmp0, tmp1
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2219
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_7);
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_8);
  __pyx_t_8 = 0;

  /* "dipy/align/vector_fields.pyx":2220
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),
 *                                         dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 3):
 */
  __pyx_t_8 = PyDict_New(); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_10 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  if (!__pyx_t_10) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_11, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_dtype, __pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "dipy/align/vector_fields.pyx":2219
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, __pyx_t_8); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_12 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(__pyx_t_9);
  if (unlikely(!__pyx_t_12.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_out = __pyx_t_12;
  __pyx_t_12.memview = NULL;
  __pyx_t_12.data = NULL;

  /* "dipy/align/vector_fields.pyx":2222
 *                                         dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_5 = NULL;
  __pyx_t_13 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_13 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_5) {
    __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_13, __pyx_t_7);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_13, __pyx_int_3);
  __pyx_t_7 = 0;
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_6, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_14 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_15 = ((!__pyx_t_14) != 0);
  if (__pyx_t_15) {

    /* "dipy/align/vector_fields.pyx":2223
 * 
 *     if not is_valid_affine(affine, 3):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__211, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_Raise(__pyx_t_9, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2222
 *                                         dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2225
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2227
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2228
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine is not None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2229
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2230
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
              __pyx_t_15 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
              if (__pyx_t_15) {

                /* "dipy/align/vector_fields.pyx":2231
 *                 for j in range(ncols):
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 */
                __pyx_v_dkk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2232
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 */
                __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2233
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     else:
 *                         dkk = k
 */
                __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2230
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
                goto __pyx_L13;
              }

              /* "dipy/align/vector_fields.pyx":2235
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 *                         dkk = k             # <<<<<<<<<<<<<<
 *                         dii = i
 *                         djj = j
 */
              /*else*/ {
                __pyx_v_dkk = __pyx_v_k;

                /* "dipy/align/vector_fields.pyx":2236
 *                     else:
 *                         dkk = k
 *                         dii = i             # <<<<<<<<<<<<<<
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_dii = __pyx_v_i;

                /* "dipy/align/vector_fields.pyx":2237
 *                         dkk = k
 *                         dii = i
 *                         djj = j             # <<<<<<<<<<<<<<
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])
 */
                __pyx_v_djj = __pyx_v_j;
              }
              __pyx_L13:;

              /* "dipy/align/vector_fields.pyx":2239
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
              __pyx_t_22 = __pyx_v_k;
              __pyx_t_23 = __pyx_v_i;
              __pyx_t_24 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2238
 *                         dii = i
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                       &out[k,i,j])
 *     return out
 */
              __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_22 * __pyx_v_out.strides[0]) ) + __pyx_t_23 * __pyx_v_out.strides[1]) ) + __pyx_t_24 * __pyx_v_out.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2225
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2240
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_9 = __pyx_memoryview_fromslice(__pyx_v_out, 3, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2240; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_r = __pyx_t_9;
  __pyx_t_9 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __PYX_XDEC_MEMVIEW(&__pyx_t_12, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_460__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults49, __pyx_self)->__pyx_arg_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_2__pyx_pw_4dipy_5align_13vector_fields_257transform_3d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_2__pyx_mdef_4dipy_5align_13vector_fields_257transform_3d_affine_nn = {"__pyx_fuse_2transform_3d_affine_nn", (PyCFunction)__pyx_fuse_2__pyx_pw_4dipy_5align_13vector_fields_257transform_3d_affine_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_44transform_3d_affine_nn};
static PyObject *__pyx_fuse_2__pyx_pw_4dipy_5align_13vector_fields_257transform_3d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_3d_affine_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    __pyx_defaults49 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults49, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_3d_affine_nn", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_3d_affine_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_long(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2179; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine = __pyx_dynamic_args->__pyx_arg_affine;
      __PYX_INC_MEMVIEW(&__pyx_v_affine, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_3d_affine_nn", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_256transform_3d_affine_nn(__pyx_self, __pyx_v_volume, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_256transform_3d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_memviewslice __pyx_t_12 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_2transform_3d_affine_nn", 0);

  /* "dipy/align/vector_fields.pyx":2210
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 */
  __pyx_t_1 = 0;
  __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2211
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_t_2 = 1;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2212
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_t_3 = 2;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_3 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2213
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2214
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         double dkk, dii, djj, tmp0, tmp1
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2215
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         double dkk, dii, djj, tmp0, tmp1
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2219
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_7);
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_8);
  __pyx_t_8 = 0;

  /* "dipy/align/vector_fields.pyx":2220
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),
 *                                         dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 3):
 */
  __pyx_t_8 = PyDict_New(); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_long, (int (*)(char *, PyObject *)) __pyx_memview_set_long, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_10 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  if (!__pyx_t_10) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_11, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_dtype, __pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "dipy/align/vector_fields.pyx":2219
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, __pyx_t_8); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_12 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_long(__pyx_t_9);
  if (unlikely(!__pyx_t_12.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_out = __pyx_t_12;
  __pyx_t_12.memview = NULL;
  __pyx_t_12.data = NULL;

  /* "dipy/align/vector_fields.pyx":2222
 *                                         dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_5 = NULL;
  __pyx_t_13 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_13 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_5) {
    __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_13, __pyx_t_7);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_13, __pyx_int_3);
  __pyx_t_7 = 0;
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_6, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_14 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_15 = ((!__pyx_t_14) != 0);
  if (__pyx_t_15) {

    /* "dipy/align/vector_fields.pyx":2223
 * 
 *     if not is_valid_affine(affine, 3):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__212, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_Raise(__pyx_t_9, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2222
 *                                         dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2225
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2227
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2228
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine is not None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2229
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2230
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
              __pyx_t_15 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
              if (__pyx_t_15) {

                /* "dipy/align/vector_fields.pyx":2231
 *                 for j in range(ncols):
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 */
                __pyx_v_dkk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2232
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 */
                __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2233
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     else:
 *                         dkk = k
 */
                __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2230
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
                goto __pyx_L13;
              }

              /* "dipy/align/vector_fields.pyx":2235
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 *                         dkk = k             # <<<<<<<<<<<<<<
 *                         dii = i
 *                         djj = j
 */
              /*else*/ {
                __pyx_v_dkk = __pyx_v_k;

                /* "dipy/align/vector_fields.pyx":2236
 *                     else:
 *                         dkk = k
 *                         dii = i             # <<<<<<<<<<<<<<
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_dii = __pyx_v_i;

                /* "dipy/align/vector_fields.pyx":2237
 *                         dkk = k
 *                         dii = i
 *                         djj = j             # <<<<<<<<<<<<<<
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])
 */
                __pyx_v_djj = __pyx_v_j;
              }
              __pyx_L13:;

              /* "dipy/align/vector_fields.pyx":2239
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
              __pyx_t_22 = __pyx_v_k;
              __pyx_t_23 = __pyx_v_i;
              __pyx_t_24 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2238
 *                         dii = i
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                       &out[k,i,j])
 *     return out
 */
              __pyx_fuse_2__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((long *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_22 * __pyx_v_out.strides[0]) ) + __pyx_t_23 * __pyx_v_out.strides[1]) ) + __pyx_t_24 * __pyx_v_out.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2225
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2240
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_9 = __pyx_memoryview_fromslice(__pyx_v_out, 3, (PyObject *(*)(char *)) __pyx_memview_get_long, (int (*)(char *, PyObject *)) __pyx_memview_set_long, 0);; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2240; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_r = __pyx_t_9;
  __pyx_t_9 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __PYX_XDEC_MEMVIEW(&__pyx_t_12, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_462__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults50, __pyx_self)->__pyx_arg_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_3__pyx_pw_4dipy_5align_13vector_fields_259transform_3d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_3__pyx_mdef_4dipy_5align_13vector_fields_259transform_3d_affine_nn = {"__pyx_fuse_3transform_3d_affine_nn", (PyCFunction)__pyx_fuse_3__pyx_pw_4dipy_5align_13vector_fields_259transform_3d_affine_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_44transform_3d_affine_nn};
static PyObject *__pyx_fuse_3__pyx_pw_4dipy_5align_13vector_fields_259transform_3d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_3d_affine_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    __pyx_defaults50 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults50, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_3d_affine_nn", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_3d_affine_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2179; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine = __pyx_dynamic_args->__pyx_arg_affine;
      __PYX_INC_MEMVIEW(&__pyx_v_affine, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_3d_affine_nn", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_258transform_3d_affine_nn(__pyx_self, __pyx_v_volume, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_258transform_3d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_memviewslice __pyx_t_12 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_3transform_3d_affine_nn", 0);

  /* "dipy/align/vector_fields.pyx":2210
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 */
  __pyx_t_1 = 0;
  __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2211
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_t_2 = 1;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2212
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_t_3 = 2;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_3 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2213
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2214
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         double dkk, dii, djj, tmp0, tmp1
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2215
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         double dkk, dii, djj, tmp0, tmp1
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2219
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_7);
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_8);
  __pyx_t_8 = 0;

  /* "dipy/align/vector_fields.pyx":2220
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),
 *                                         dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 3):
 */
  __pyx_t_8 = PyDict_New(); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_10 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  if (!__pyx_t_10) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_11, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_dtype, __pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "dipy/align/vector_fields.pyx":2219
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, __pyx_t_8); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_12 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_t_9);
  if (unlikely(!__pyx_t_12.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_out = __pyx_t_12;
  __pyx_t_12.memview = NULL;
  __pyx_t_12.data = NULL;

  /* "dipy/align/vector_fields.pyx":2222
 *                                         dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_5 = NULL;
  __pyx_t_13 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_13 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_5) {
    __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_13, __pyx_t_7);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_13, __pyx_int_3);
  __pyx_t_7 = 0;
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_6, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_14 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_15 = ((!__pyx_t_14) != 0);
  if (__pyx_t_15) {

    /* "dipy/align/vector_fields.pyx":2223
 * 
 *     if not is_valid_affine(affine, 3):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__213, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_Raise(__pyx_t_9, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2222
 *                                         dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2225
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2227
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2228
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine is not None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2229
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2230
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
              __pyx_t_15 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
              if (__pyx_t_15) {

                /* "dipy/align/vector_fields.pyx":2231
 *                 for j in range(ncols):
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 */
                __pyx_v_dkk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2232
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 */
                __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2233
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     else:
 *                         dkk = k
 */
                __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2230
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
                goto __pyx_L13;
              }

              /* "dipy/align/vector_fields.pyx":2235
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 *                         dkk = k             # <<<<<<<<<<<<<<
 *                         dii = i
 *                         djj = j
 */
              /*else*/ {
                __pyx_v_dkk = __pyx_v_k;

                /* "dipy/align/vector_fields.pyx":2236
 *                     else:
 *                         dkk = k
 *                         dii = i             # <<<<<<<<<<<<<<
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_dii = __pyx_v_i;

                /* "dipy/align/vector_fields.pyx":2237
 *                         dkk = k
 *                         dii = i
 *                         djj = j             # <<<<<<<<<<<<<<
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])
 */
                __pyx_v_djj = __pyx_v_j;
              }
              __pyx_L13:;

              /* "dipy/align/vector_fields.pyx":2239
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
              __pyx_t_22 = __pyx_v_k;
              __pyx_t_23 = __pyx_v_i;
              __pyx_t_24 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2238
 *                         dii = i
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                       &out[k,i,j])
 *     return out
 */
              __pyx_fuse_3__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_22 * __pyx_v_out.strides[0]) ) + __pyx_t_23 * __pyx_v_out.strides[1]) ) + __pyx_t_24 * __pyx_v_out.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2225
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2240
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_9 = __pyx_memoryview_fromslice(__pyx_v_out, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2240; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_r = __pyx_t_9;
  __pyx_t_9 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __PYX_XDEC_MEMVIEW(&__pyx_t_12, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_464__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults51, __pyx_self)->__pyx_arg_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_4__pyx_pw_4dipy_5align_13vector_fields_261transform_3d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_4__pyx_mdef_4dipy_5align_13vector_fields_261transform_3d_affine_nn = {"__pyx_fuse_4transform_3d_affine_nn", (PyCFunction)__pyx_fuse_4__pyx_pw_4dipy_5align_13vector_fields_261transform_3d_affine_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_44transform_3d_affine_nn};
static PyObject *__pyx_fuse_4__pyx_pw_4dipy_5align_13vector_fields_261transform_3d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_3d_affine_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    __pyx_defaults51 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults51, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_3d_affine_nn", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_3d_affine_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2179; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine = __pyx_dynamic_args->__pyx_arg_affine;
      __PYX_INC_MEMVIEW(&__pyx_v_affine, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_3d_affine_nn", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_260transform_3d_affine_nn(__pyx_self, __pyx_v_volume, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_260transform_3d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_memviewslice __pyx_t_12 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_4transform_3d_affine_nn", 0);

  /* "dipy/align/vector_fields.pyx":2210
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 */
  __pyx_t_1 = 0;
  __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2211
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_t_2 = 1;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2212
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_t_3 = 2;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_3 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2213
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2214
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         double dkk, dii, djj, tmp0, tmp1
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2215
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         double dkk, dii, djj, tmp0, tmp1
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2219
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_7);
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_8);
  __pyx_t_8 = 0;

  /* "dipy/align/vector_fields.pyx":2220
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),
 *                                         dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 3):
 */
  __pyx_t_8 = PyDict_New(); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_10 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  if (!__pyx_t_10) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_11, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_dtype, __pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "dipy/align/vector_fields.pyx":2219
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, __pyx_t_8); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_12 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_9);
  if (unlikely(!__pyx_t_12.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_out = __pyx_t_12;
  __pyx_t_12.memview = NULL;
  __pyx_t_12.data = NULL;

  /* "dipy/align/vector_fields.pyx":2222
 *                                         dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_5 = NULL;
  __pyx_t_13 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_13 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_5) {
    __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_13, __pyx_t_7);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_13, __pyx_int_3);
  __pyx_t_7 = 0;
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_6, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_14 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_15 = ((!__pyx_t_14) != 0);
  if (__pyx_t_15) {

    /* "dipy/align/vector_fields.pyx":2223
 * 
 *     if not is_valid_affine(affine, 3):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__214, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_Raise(__pyx_t_9, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2222
 *                                         dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2225
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2227
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2228
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine is not None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2229
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2230
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
              __pyx_t_15 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
              if (__pyx_t_15) {

                /* "dipy/align/vector_fields.pyx":2231
 *                 for j in range(ncols):
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 */
                __pyx_v_dkk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2232
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 */
                __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2233
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     else:
 *                         dkk = k
 */
                __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2230
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
                goto __pyx_L13;
              }

              /* "dipy/align/vector_fields.pyx":2235
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 *                         dkk = k             # <<<<<<<<<<<<<<
 *                         dii = i
 *                         djj = j
 */
              /*else*/ {
                __pyx_v_dkk = __pyx_v_k;

                /* "dipy/align/vector_fields.pyx":2236
 *                     else:
 *                         dkk = k
 *                         dii = i             # <<<<<<<<<<<<<<
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_dii = __pyx_v_i;

                /* "dipy/align/vector_fields.pyx":2237
 *                         dkk = k
 *                         dii = i
 *                         djj = j             # <<<<<<<<<<<<<<
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])
 */
                __pyx_v_djj = __pyx_v_j;
              }
              __pyx_L13:;

              /* "dipy/align/vector_fields.pyx":2239
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
              __pyx_t_22 = __pyx_v_k;
              __pyx_t_23 = __pyx_v_i;
              __pyx_t_24 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2238
 *                         dii = i
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                       &out[k,i,j])
 *     return out
 */
              __pyx_fuse_4__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_22 * __pyx_v_out.strides[0]) ) + __pyx_t_23 * __pyx_v_out.strides[1]) ) + __pyx_t_24 * __pyx_v_out.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2225
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2240
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_9 = __pyx_memoryview_fromslice(__pyx_v_out, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2240; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_r = __pyx_t_9;
  __pyx_t_9 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __PYX_XDEC_MEMVIEW(&__pyx_t_12, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_466__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults52, __pyx_self)->__pyx_arg_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_5__pyx_pw_4dipy_5align_13vector_fields_263transform_3d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_5__pyx_mdef_4dipy_5align_13vector_fields_263transform_3d_affine_nn = {"__pyx_fuse_5transform_3d_affine_nn", (PyCFunction)__pyx_fuse_5__pyx_pw_4dipy_5align_13vector_fields_263transform_3d_affine_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_44transform_3d_affine_nn};
static PyObject *__pyx_fuse_5__pyx_pw_4dipy_5align_13vector_fields_263transform_3d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_3d_affine_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    __pyx_defaults52 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults52, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_3d_affine_nn", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_3d_affine_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_float_complex(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2179; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine = __pyx_dynamic_args->__pyx_arg_affine;
      __PYX_INC_MEMVIEW(&__pyx_v_affine, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_3d_affine_nn", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_262transform_3d_affine_nn(__pyx_self, __pyx_v_volume, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_262transform_3d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_memviewslice __pyx_t_12 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_5transform_3d_affine_nn", 0);

  /* "dipy/align/vector_fields.pyx":2210
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 */
  __pyx_t_1 = 0;
  __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2211
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_t_2 = 1;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2212
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_t_3 = 2;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_3 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2213
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2214
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         double dkk, dii, djj, tmp0, tmp1
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2215
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         double dkk, dii, djj, tmp0, tmp1
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2219
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_7);
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_8);
  __pyx_t_8 = 0;

  /* "dipy/align/vector_fields.pyx":2220
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),
 *                                         dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 3):
 */
  __pyx_t_8 = PyDict_New(); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_float_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_float_complex, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_10 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  if (!__pyx_t_10) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_11, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_dtype, __pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "dipy/align/vector_fields.pyx":2219
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, __pyx_t_8); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_12 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_float_complex(__pyx_t_9);
  if (unlikely(!__pyx_t_12.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_out = __pyx_t_12;
  __pyx_t_12.memview = NULL;
  __pyx_t_12.data = NULL;

  /* "dipy/align/vector_fields.pyx":2222
 *                                         dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_5 = NULL;
  __pyx_t_13 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_13 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_5) {
    __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_13, __pyx_t_7);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_13, __pyx_int_3);
  __pyx_t_7 = 0;
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_6, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_14 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_15 = ((!__pyx_t_14) != 0);
  if (__pyx_t_15) {

    /* "dipy/align/vector_fields.pyx":2223
 * 
 *     if not is_valid_affine(affine, 3):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__215, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_Raise(__pyx_t_9, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2222
 *                                         dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2225
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2227
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2228
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine is not None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2229
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2230
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
              __pyx_t_15 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
              if (__pyx_t_15) {

                /* "dipy/align/vector_fields.pyx":2231
 *                 for j in range(ncols):
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 */
                __pyx_v_dkk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2232
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 */
                __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2233
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     else:
 *                         dkk = k
 */
                __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2230
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
                goto __pyx_L13;
              }

              /* "dipy/align/vector_fields.pyx":2235
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 *                         dkk = k             # <<<<<<<<<<<<<<
 *                         dii = i
 *                         djj = j
 */
              /*else*/ {
                __pyx_v_dkk = __pyx_v_k;

                /* "dipy/align/vector_fields.pyx":2236
 *                     else:
 *                         dkk = k
 *                         dii = i             # <<<<<<<<<<<<<<
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_dii = __pyx_v_i;

                /* "dipy/align/vector_fields.pyx":2237
 *                         dkk = k
 *                         dii = i
 *                         djj = j             # <<<<<<<<<<<<<<
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])
 */
                __pyx_v_djj = __pyx_v_j;
              }
              __pyx_L13:;

              /* "dipy/align/vector_fields.pyx":2239
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
              __pyx_t_22 = __pyx_v_k;
              __pyx_t_23 = __pyx_v_i;
              __pyx_t_24 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2238
 *                         dii = i
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                       &out[k,i,j])
 *     return out
 */
              __pyx_fuse_5__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((__pyx_t_float_complex *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_22 * __pyx_v_out.strides[0]) ) + __pyx_t_23 * __pyx_v_out.strides[1]) ) + __pyx_t_24 * __pyx_v_out.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2225
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2240
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_9 = __pyx_memoryview_fromslice(__pyx_v_out, 3, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_float_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_float_complex, 0);; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2240; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_r = __pyx_t_9;
  __pyx_t_9 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __PYX_XDEC_MEMVIEW(&__pyx_t_12, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_468__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults53, __pyx_self)->__pyx_arg_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_6__pyx_pw_4dipy_5align_13vector_fields_265transform_3d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_6__pyx_mdef_4dipy_5align_13vector_fields_265transform_3d_affine_nn = {"__pyx_fuse_6transform_3d_affine_nn", (PyCFunction)__pyx_fuse_6__pyx_pw_4dipy_5align_13vector_fields_265transform_3d_affine_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_44transform_3d_affine_nn};
static PyObject *__pyx_fuse_6__pyx_pw_4dipy_5align_13vector_fields_265transform_3d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_volume = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_3d_affine_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_volume,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    __pyx_defaults53 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults53, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_volume)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_3d_affine_nn", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_3d_affine_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_volume = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_double_complex(values[0]); if (unlikely(!__pyx_v_volume.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2179; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine = __pyx_dynamic_args->__pyx_arg_affine;
      __PYX_INC_MEMVIEW(&__pyx_v_affine, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_3d_affine_nn", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_264transform_3d_affine_nn(__pyx_self, __pyx_v_volume, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_264transform_3d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_volume, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nsVol;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_memviewslice __pyx_t_12 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  npy_intp __pyx_t_19;
  npy_intp __pyx_t_20;
  npy_intp __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_6transform_3d_affine_nn", 0);

  /* "dipy/align/vector_fields.pyx":2210
 *     """
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 */
  __pyx_t_1 = 0;
  __pyx_v_nslices = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2211
 *     cdef:
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 */
  __pyx_t_2 = 1;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2212
 *         cnp.npy_intp nslices = ref_shape[0]
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 */
  __pyx_t_3 = 2;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_3 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2213
 *         cnp.npy_intp nrows = ref_shape[1]
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]
 */
  __pyx_v_nsVol = (__pyx_v_volume.shape[0]);

  /* "dipy/align/vector_fields.pyx":2214
 *         cnp.npy_intp ncols = ref_shape[2]
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = volume.shape[2]
 *         double dkk, dii, djj, tmp0, tmp1
 */
  __pyx_v_nrVol = (__pyx_v_volume.shape[1]);

  /* "dipy/align/vector_fields.pyx":2215
 *         cnp.npy_intp nsVol = volume.shape[0]
 *         cnp.npy_intp nrVol = volume.shape[1]
 *         cnp.npy_intp ncVol = volume.shape[2]             # <<<<<<<<<<<<<<
 *         double dkk, dii, djj, tmp0, tmp1
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 */
  __pyx_v_ncVol = (__pyx_v_volume.shape[2]);

  /* "dipy/align/vector_fields.pyx":2219
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_7);
  __pyx_t_4 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_7 = PyTuple_New(1); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_8);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_8);
  __pyx_t_8 = 0;

  /* "dipy/align/vector_fields.pyx":2220
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),
 *                                         dtype=np.asarray(volume).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 3):
 */
  __pyx_t_8 = PyDict_New(); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_volume, 3, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_double_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_double_complex, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_10 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  if (!__pyx_t_10) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_11, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_8, __pyx_n_s_dtype, __pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "dipy/align/vector_fields.pyx":2219
 *         double alpha, beta, gamma, calpha, cbeta, cgamma
 *         cnp.npy_intp k, i, j, kk, ii, jj
 *         number[:, :, :] out = np.zeros((nslices, nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(volume).dtype)
 * 
 */
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_7, __pyx_t_8); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_12 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_double_complex(__pyx_t_9);
  if (unlikely(!__pyx_t_12.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2219; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_out = __pyx_t_12;
  __pyx_t_12.memview = NULL;
  __pyx_t_12.data = NULL;

  /* "dipy/align/vector_fields.pyx":2222
 *                                         dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_8 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_5 = NULL;
  __pyx_t_13 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_13 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_5) {
    __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_7);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_13, __pyx_t_7);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_13, __pyx_int_3);
  __pyx_t_7 = 0;
  __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_6, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_14 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_15 = ((!__pyx_t_14) != 0);
  if (__pyx_t_15) {

    /* "dipy/align/vector_fields.pyx":2223
 * 
 *     if not is_valid_affine(affine, 3):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__216, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_Raise(__pyx_t_9, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2222
 *                                         dtype=np.asarray(volume).dtype)
 * 
 *     if not is_valid_affine(affine, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2225
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2227
 *     with nogil:
 * 
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_16 = __pyx_v_nslices;
        for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
          __pyx_v_k = __pyx_t_17;

          /* "dipy/align/vector_fields.pyx":2228
 * 
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     if affine is not None:
 */
          __pyx_t_18 = __pyx_v_nrows;
          for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
            __pyx_v_i = __pyx_t_19;

            /* "dipy/align/vector_fields.pyx":2229
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 */
            __pyx_t_20 = __pyx_v_ncols;
            for (__pyx_t_21 = 0; __pyx_t_21 < __pyx_t_20; __pyx_t_21+=1) {
              __pyx_v_j = __pyx_t_21;

              /* "dipy/align/vector_fields.pyx":2230
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
              __pyx_t_15 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
              if (__pyx_t_15) {

                /* "dipy/align/vector_fields.pyx":2231
 *                 for j in range(ncols):
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 */
                __pyx_v_dkk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2232
 *                     if affine is not None:
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 */
                __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2233
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     else:
 *                         dkk = k
 */
                __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

                /* "dipy/align/vector_fields.pyx":2230
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     if affine is not None:             # <<<<<<<<<<<<<<
 *                         dkk = _apply_affine_3d_x0(k, i, j, 1, affine)
 *                         dii = _apply_affine_3d_x1(k, i, j, 1, affine)
 */
                goto __pyx_L13;
              }

              /* "dipy/align/vector_fields.pyx":2235
 *                         djj = _apply_affine_3d_x2(k, i, j, 1, affine)
 *                     else:
 *                         dkk = k             # <<<<<<<<<<<<<<
 *                         dii = i
 *                         djj = j
 */
              /*else*/ {
                __pyx_v_dkk = __pyx_v_k;

                /* "dipy/align/vector_fields.pyx":2236
 *                     else:
 *                         dkk = k
 *                         dii = i             # <<<<<<<<<<<<<<
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 */
                __pyx_v_dii = __pyx_v_i;

                /* "dipy/align/vector_fields.pyx":2237
 *                         dkk = k
 *                         dii = i
 *                         djj = j             # <<<<<<<<<<<<<<
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])
 */
                __pyx_v_djj = __pyx_v_j;
              }
              __pyx_L13:;

              /* "dipy/align/vector_fields.pyx":2239
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
              __pyx_t_22 = __pyx_v_k;
              __pyx_t_23 = __pyx_v_i;
              __pyx_t_24 = __pyx_v_j;

              /* "dipy/align/vector_fields.pyx":2238
 *                         dii = i
 *                         djj = j
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                       &out[k,i,j])
 *     return out
 */
              __pyx_fuse_6__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_3d(__pyx_v_volume, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, (&(*((__pyx_t_double_complex *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_22 * __pyx_v_out.strides[0]) ) + __pyx_t_23 * __pyx_v_out.strides[1]) ) + __pyx_t_24 * __pyx_v_out.strides[2]) )))));
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2225
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for k in range(nslices):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2240
 *                     _interpolate_scalar_nn_3d[number](volume, dkk, dii, djj,
 *                                                       &out[k,i,j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_9 = __pyx_memoryview_fromslice(__pyx_v_out, 3, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_double_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_double_complex, 0);; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2240; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_r = __pyx_t_9;
  __pyx_t_9 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __PYX_XDEC_MEMVIEW(&__pyx_t_12, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_3d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_volume, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":2243
 * 
 * 
 * def warp_2d(floating[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_47warp_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_46warp_2d[] = "Warps a 2D image using bilinear interpolation\n\n    Deforms the input image under the given transformation. The warped image\n    is computed using bi-linear interpolation and is given by:\n\n    (1) warped[i] = image[ C * d1[A*i] + B*i ]\n\n    where A = affine_idx_in, B = affine_idx_out, C = affine_disp and i denotes\n    the discrete coordinates of a voxel in the sampling grid of\n    shape = out_shape. To illustrate the use of this function, consider a\n    displacement field d1 with grid-to-space transformation R, an image with\n    grid-to-space transformation T and let's say we want to sample the warped\n    image on a grid with grid-to-space transformation S (sampling grid). For\n    each voxel in the sampling grid with discrete coordinates i, the warped\n    image is given by:\n\n    (2) warped[i] = image[Tinv * ( d1[Rinv * S * i] + S * i ) ]\n\n    where Tinv = T^{-1} and Rinv = R^{-1}. By identifying A = Rinv * S,\n    B = Tinv * S, C = Tinv we can use this function to efficiently warp the\n    input image.\n\n\n    Parameters\n    ----------\n    image : array, shape (R, C)\n        the input image to be transformed\n    d1 : array, shape (R', C', 2)\n        the displacement field driving the transformation\n    affine_idx_in : array, shape (3, 3)\n        the matrix A in eq. (1) above\n    affine_idx_out : array, shape (3, 3)\n        the matrix B in eq. (1) above\n    affine_disp : array, shape (3, 3)\n        the matrix C in eq. (1) above\n    out_shape : array, shape (2,)\n        the number of rows and columns of the sampling grid\n\n    Returns\n    -------\n    warped : array, shape = out_shape\n        the transformed image\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_47warp_2d = {"warp_2d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_47warp_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_46warp_2d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_47warp_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    values[2] = __pyx_k__217;
    values[3] = __pyx_k__218;
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_46warp_2d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_46warp_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("warp_2d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_image, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_image); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__219, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__220, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__221, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__222, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_474__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults56, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults56, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults56, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults56, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_269warp_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_269warp_2d = {"__pyx_fuse_0warp_2d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_269warp_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_46warp_2d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_269warp_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults56 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults56, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_2d", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2244; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2245; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2246; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2247; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_2d", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_268warp_2d(__pyx_self, __pyx_v_image, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_268warp_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_memviewslice __pyx_t_13 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0warp_2d", 0);

  /* "dipy/align/vector_fields.pyx":2292
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2293
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2294
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2295
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double di, dj, dii, djj
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2299
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2300
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__223, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2300; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2300; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2299
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2301
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2302
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__224, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2302; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2302; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2301
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2303
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2304
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__225, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2303
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2306
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2307
 * 
 *     if out_shape is not None:
 *         nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 */
    __pyx_t_9 = 0;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2308
 *     if out_shape is not None:
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 */
    __pyx_t_10 = 1;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2306
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2309
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2310
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 *         nrows = d1.shape[0]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[1]
 *     cdef floating[:, :] warped = np.zeros(shape=(nrows, ncols),
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2311
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]             # <<<<<<<<<<<<<<
 *     cdef floating[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                           dtype=np.asarray(image).dtype)
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2309
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2312
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef floating[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                           dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2313
 *         ncols = d1.shape[1]
 *     cdef floating[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                           dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_11) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2312
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef floating[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                           dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_13 = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_t_3);
  if (unlikely(!__pyx_t_13.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_warped = __pyx_t_13;
  __pyx_t_13.memview = NULL;
  __pyx_t_13.data = NULL;

  /* "dipy/align/vector_fields.pyx":2314
 *     cdef floating[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                           dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__226) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_12);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_tmp = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2316
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2318
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2319
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2321
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2322
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d1[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_19 * __pyx_v_d1.strides[0]) ) + __pyx_t_20 * __pyx_v_d1.strides[1]) ) + __pyx_t_21 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2323
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2321
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":2325
 *                     djj = d1[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":2326
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2327
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2329
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d1, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":2330
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":2331
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":2334
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2335
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2337
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2334
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":2340
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":2341
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L15:;

            /* "dipy/align/vector_fields.pyx":2344
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2345
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 */
              __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2346
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = di + i
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2344
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
              goto __pyx_L16;
            }

            /* "dipy/align/vector_fields.pyx":2348
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 *                     dii = di + i             # <<<<<<<<<<<<<<
 *                     djj = dj + j
 * 
 */
            /*else*/ {
              __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":2349
 *                 else:
 *                     dii = di + i
 *                     djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                 # Interpolate the input image at the resulting location
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
            }
            __pyx_L16:;

            /* "dipy/align/vector_fields.pyx":2353
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_2d[floating](image, dii, djj,
 *                                                  &warped[i, j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
            __pyx_t_27 = __pyx_v_i;
            __pyx_t_28 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2352
 * 
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_2d[floating](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                  &warped[i, j])
 *     return warped
 */
            __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_27 * __pyx_v_warped.strides[0]) ) + __pyx_t_28 * __pyx_v_warped.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2316
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2354
 *                 _interpolate_scalar_2d[floating](image, dii, djj,
 *                                                  &warped[i, j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_warped, 2, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2354; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_r = __pyx_t_12;
  __pyx_t_12 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2243
 * 
 * 
 * def warp_2d(floating[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __PYX_XDEC_MEMVIEW(&__pyx_t_13, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_476__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults57, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults57, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults57, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults57, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_271warp_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_271warp_2d = {"__pyx_fuse_1warp_2d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_271warp_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_46warp_2d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_271warp_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults57 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults57, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_2d", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2244; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2245; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2246; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2247; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_2d", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_270warp_2d(__pyx_self, __pyx_v_image, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_270warp_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_memviewslice __pyx_t_13 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1warp_2d", 0);

  /* "dipy/align/vector_fields.pyx":2292
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2293
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2294
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2295
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double di, dj, dii, djj
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2299
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2299; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2300
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__227, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2300; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2300; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2299
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2301
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2301; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2302
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__228, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2302; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2302; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2301
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2303
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2303; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2304
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__229, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2303
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2306
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2307
 * 
 *     if out_shape is not None:
 *         nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 */
    __pyx_t_9 = 0;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2308
 *     if out_shape is not None:
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 */
    __pyx_t_10 = 1;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2306
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2309
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2310
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 *         nrows = d1.shape[0]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[1]
 *     cdef floating[:, :] warped = np.zeros(shape=(nrows, ncols),
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2311
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]             # <<<<<<<<<<<<<<
 *     cdef floating[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                           dtype=np.asarray(image).dtype)
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2309
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2312
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef floating[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                           dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2313
 *         ncols = d1.shape[1]
 *     cdef floating[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                           dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_11) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2313; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2312
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef floating[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                           dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_13 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_t_3);
  if (unlikely(!__pyx_t_13.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2312; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_warped = __pyx_t_13;
  __pyx_t_13.memview = NULL;
  __pyx_t_13.data = NULL;

  /* "dipy/align/vector_fields.pyx":2314
 *     cdef floating[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                           dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__230) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_12);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_tmp = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2316
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2318
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2319
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2321
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2322
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d1[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_19 * __pyx_v_d1.strides[0]) ) + __pyx_t_20 * __pyx_v_d1.strides[1]) ) + __pyx_t_21 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2323
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2321
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":2325
 *                     djj = d1[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":2326
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2327
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2329
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d1, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":2330
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":2331
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":2334
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2335
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2337
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2334
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":2340
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":2341
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L15:;

            /* "dipy/align/vector_fields.pyx":2344
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2345
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 */
              __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2346
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = di + i
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2344
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
              goto __pyx_L16;
            }

            /* "dipy/align/vector_fields.pyx":2348
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 *                     dii = di + i             # <<<<<<<<<<<<<<
 *                     djj = dj + j
 * 
 */
            /*else*/ {
              __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":2349
 *                 else:
 *                     dii = di + i
 *                     djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                 # Interpolate the input image at the resulting location
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
            }
            __pyx_L16:;

            /* "dipy/align/vector_fields.pyx":2353
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_2d[floating](image, dii, djj,
 *                                                  &warped[i, j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
            __pyx_t_27 = __pyx_v_i;
            __pyx_t_28 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2352
 * 
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_2d[floating](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                  &warped[i, j])
 *     return warped
 */
            __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_27 * __pyx_v_warped.strides[0]) ) + __pyx_t_28 * __pyx_v_warped.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2316
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2354
 *                 _interpolate_scalar_2d[floating](image, dii, djj,
 *                                                  &warped[i, j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_warped, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2354; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_r = __pyx_t_12;
  __pyx_t_12 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2243
 * 
 * 
 * def warp_2d(floating[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __PYX_XDEC_MEMVIEW(&__pyx_t_13, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":2357
 * 
 * 
 * def transform_2d_affine(floating[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                         double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with bilinear interp.
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_49transform_2d_affine(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_48transform_2d_affine[] = "Transforms a 2D image by an affine transform with bilinear interp.\n\n    Deforms the input image under the given affine transformation using\n    tri-linear interpolation. The shape of the resulting image\n    is given by ref_shape. If the affine matrix is None, it is taken as the\n    identity.\n\n    Parameters\n    ----------\n    image : array, shape (R, C)\n        the input image to be transformed\n    ref_shape : array, shape (2,)\n        the shape of the resulting image\n    affine : array, shape (3, 3)\n        the affine transform to be applied\n\n    Returns\n    -------\n    out : array, shape (R', C')\n        the transformed image\n\n    Notes\n    -----\n    The reason it is necessary to provide the intended shape of the resulting\n    image is because the affine transformation is defined on all R^{2}\n    but we must sample a finite lattice. Also the resulting shape may not be\n    necessarily equal to the input shape, unless we are interested on\n    endomorphisms only and not general diffeomorphisms.\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_49transform_2d_affine = {"transform_2d_affine", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_49transform_2d_affine, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_48transform_2d_affine};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_49transform_2d_affine(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    values[2] = __pyx_k__231;
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_48transform_2d_affine(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_48transform_2d_affine(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("transform_2d_affine", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_image, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_image); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__232, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__233, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__234, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__235, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_482__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults60, __pyx_self)->__pyx_arg_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_275transform_2d_affine(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_275transform_2d_affine = {"__pyx_fuse_0transform_2d_affine", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_275transform_2d_affine, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_48transform_2d_affine};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_275transform_2d_affine(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_2d_affine (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    __pyx_defaults60 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults60, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_2d_affine", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_2d_affine") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2358; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine = __pyx_dynamic_args->__pyx_arg_affine;
      __PYX_INC_MEMVIEW(&__pyx_v_affine, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_2d_affine", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_274transform_2d_affine(__pyx_self, __pyx_v_image, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_274transform_2d_affine(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_memviewslice __pyx_t_10 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  npy_intp __pyx_t_14;
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0transform_2d_affine", 0);

  /* "dipy/align/vector_fields.pyx":2389
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_t_1 = 0;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2390
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_t_2 = 1;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2391
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2392
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double dii, djj, tmp0
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2396
 *         double dii, djj, tmp0
 *         double alpha, beta, calpha, cbeta
 *         floating[:, :] out = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                       dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_6);
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "dipy/align/vector_fields.pyx":2397
 *         double alpha, beta, calpha, cbeta
 *         floating[:, :] out = np.zeros(shape=(nrows, ncols),
 *                                       dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 2):
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_8 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  if (!__pyx_t_8) {
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_7);
  } else {
    __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_9, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_dtype); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "dipy/align/vector_fields.pyx":2396
 *         double dii, djj, tmp0
 *         double alpha, beta, calpha, cbeta
 *         floating[:, :] out = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                       dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_10 = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_t_5);
  if (unlikely(!__pyx_t_10.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_out = __pyx_t_10;
  __pyx_t_10.memview = NULL;
  __pyx_t_10.data = NULL;

  /* "dipy/align/vector_fields.pyx":2399
 *                                       dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = NULL;
  __pyx_t_11 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_11 = 1;
    }
  }
  __pyx_t_9 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  if (__pyx_t_7) {
    __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_11, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_11, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_9, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_12 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_13 = ((!__pyx_t_12) != 0);
  if (__pyx_t_13) {

    /* "dipy/align/vector_fields.pyx":2400
 * 
 *     if not is_valid_affine(affine, 2):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__236, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2400; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2400; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2399
 *                                       dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2402
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2404
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 if affine is not None:
 */
        __pyx_t_14 = __pyx_v_nrows;
        for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
          __pyx_v_i = __pyx_t_15;

          /* "dipy/align/vector_fields.pyx":2405
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 */
          __pyx_t_16 = __pyx_v_ncols;
          for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
            __pyx_v_j = __pyx_t_17;

            /* "dipy/align/vector_fields.pyx":2406
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
            __pyx_t_13 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
            if (__pyx_t_13) {

              /* "dipy/align/vector_fields.pyx":2407
 *             for j in range(ncols):
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 */
              __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2408
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = i
 */
              __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2406
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
              goto __pyx_L11;
            }

            /* "dipy/align/vector_fields.pyx":2410
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 *                     dii = i             # <<<<<<<<<<<<<<
 *                     djj = j
 *                 _interpolate_scalar_2d[floating](image, dii, djj,
 */
            /*else*/ {
              __pyx_v_dii = __pyx_v_i;

              /* "dipy/align/vector_fields.pyx":2411
 *                 else:
 *                     dii = i
 *                     djj = j             # <<<<<<<<<<<<<<
 *                 _interpolate_scalar_2d[floating](image, dii, djj,
 *                                                  &out[i, j])
 */
              __pyx_v_djj = __pyx_v_j;
            }
            __pyx_L11:;

            /* "dipy/align/vector_fields.pyx":2413
 *                     djj = j
 *                 _interpolate_scalar_2d[floating](image, dii, djj,
 *                                                  &out[i, j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
            __pyx_t_18 = __pyx_v_i;
            __pyx_t_19 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2412
 *                     dii = i
 *                     djj = j
 *                 _interpolate_scalar_2d[floating](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                  &out[i, j])
 *     return out
 */
            __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_18 * __pyx_v_out.strides[0]) ) + __pyx_t_19 * __pyx_v_out.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2402
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2414
 *                 _interpolate_scalar_2d[floating](image, dii, djj,
 *                                                  &out[i, j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2357
 * 
 * 
 * def transform_2d_affine(floating[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                         double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with bilinear interp.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __PYX_XDEC_MEMVIEW(&__pyx_t_10, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_484__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults61, __pyx_self)->__pyx_arg_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_277transform_2d_affine(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_277transform_2d_affine = {"__pyx_fuse_1transform_2d_affine", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_277transform_2d_affine, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_48transform_2d_affine};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_277transform_2d_affine(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_2d_affine (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    __pyx_defaults61 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults61, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_2d_affine", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_2d_affine") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2358; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine = __pyx_dynamic_args->__pyx_arg_affine;
      __PYX_INC_MEMVIEW(&__pyx_v_affine, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_2d_affine", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_276transform_2d_affine(__pyx_self, __pyx_v_image, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_276transform_2d_affine(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_memviewslice __pyx_t_10 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  npy_intp __pyx_t_14;
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1transform_2d_affine", 0);

  /* "dipy/align/vector_fields.pyx":2389
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_t_1 = 0;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2390
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_t_2 = 1;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2391
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2392
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double dii, djj, tmp0
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2396
 *         double dii, djj, tmp0
 *         double alpha, beta, calpha, cbeta
 *         floating[:, :] out = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                       dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_6);
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_t_7) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "dipy/align/vector_fields.pyx":2397
 *         double alpha, beta, calpha, cbeta
 *         floating[:, :] out = np.zeros(shape=(nrows, ncols),
 *                                       dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 2):
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_8 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_8)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  if (!__pyx_t_8) {
    __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_7);
  } else {
    __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_9, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_dtype); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "dipy/align/vector_fields.pyx":2396
 *         double dii, djj, tmp0
 *         double alpha, beta, calpha, cbeta
 *         floating[:, :] out = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                       dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_10 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_t_5);
  if (unlikely(!__pyx_t_10.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2396; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_out = __pyx_t_10;
  __pyx_t_10.memview = NULL;
  __pyx_t_10.data = NULL;

  /* "dipy/align/vector_fields.pyx":2399
 *                                       dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = NULL;
  __pyx_t_11 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_11 = 1;
    }
  }
  __pyx_t_9 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  if (__pyx_t_7) {
    __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_11, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_11, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_9, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_12 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_13 = ((!__pyx_t_12) != 0);
  if (__pyx_t_13) {

    /* "dipy/align/vector_fields.pyx":2400
 * 
 *     if not is_valid_affine(affine, 2):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__237, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2400; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2400; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2399
 *                                       dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2402
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2404
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 if affine is not None:
 */
        __pyx_t_14 = __pyx_v_nrows;
        for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
          __pyx_v_i = __pyx_t_15;

          /* "dipy/align/vector_fields.pyx":2405
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 */
          __pyx_t_16 = __pyx_v_ncols;
          for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
            __pyx_v_j = __pyx_t_17;

            /* "dipy/align/vector_fields.pyx":2406
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
            __pyx_t_13 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
            if (__pyx_t_13) {

              /* "dipy/align/vector_fields.pyx":2407
 *             for j in range(ncols):
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 */
              __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2408
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = i
 */
              __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2406
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
              goto __pyx_L11;
            }

            /* "dipy/align/vector_fields.pyx":2410
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 *                     dii = i             # <<<<<<<<<<<<<<
 *                     djj = j
 *                 _interpolate_scalar_2d[floating](image, dii, djj,
 */
            /*else*/ {
              __pyx_v_dii = __pyx_v_i;

              /* "dipy/align/vector_fields.pyx":2411
 *                 else:
 *                     dii = i
 *                     djj = j             # <<<<<<<<<<<<<<
 *                 _interpolate_scalar_2d[floating](image, dii, djj,
 *                                                  &out[i, j])
 */
              __pyx_v_djj = __pyx_v_j;
            }
            __pyx_L11:;

            /* "dipy/align/vector_fields.pyx":2413
 *                     djj = j
 *                 _interpolate_scalar_2d[floating](image, dii, djj,
 *                                                  &out[i, j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
            __pyx_t_18 = __pyx_v_i;
            __pyx_t_19 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2412
 *                     dii = i
 *                     djj = j
 *                 _interpolate_scalar_2d[floating](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                  &out[i, j])
 *     return out
 */
            __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_18 * __pyx_v_out.strides[0]) ) + __pyx_t_19 * __pyx_v_out.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2402
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2414
 *                 _interpolate_scalar_2d[floating](image, dii, djj,
 *                                                  &out[i, j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2357
 * 
 * 
 * def transform_2d_affine(floating[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                         double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with bilinear interp.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __PYX_XDEC_MEMVIEW(&__pyx_t_10, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_51warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_50warp_2d_nn[] = "Warps a 2D image using nearest neighbor interpolation\n\n    Deforms the input image under the given transformation. The warped image\n    is computed using nearest-neighbor interpolation and is given by:\n\n    (1) warped[i] = image[ C * d1[A*i] + B*i ]\n\n    where A = affine_idx_in, B = affine_idx_out, C = affine_disp and i denotes\n    the discrete coordinates of a voxel in the sampling grid of\n    shape = out_shape. To illustrate the use of this function, consider a\n    displacement field d1 with grid-to-space transformation R, an image with\n    grid-to-space transformation T and let's say we want to sample the warped\n    image on a grid with grid-to-space transformation S (sampling grid). For\n    each voxel in the sampling grid with discrete coordinates i, the warped\n    image is given by:\n\n    (2) warped[i] = image[Tinv * ( d1[Rinv * S * i] + S * i ) ]\n\n    where Tinv = T^{-1} and Rinv = R^{-1}. By identifying A = Rinv * S,\n    B = Tinv * S, C = Tinv we can use this function to efficiently warp the\n    input image.\n\n\n    Parameters\n    ----------\n    image : array, shape (R, C)\n        the input image to be transformed\n    d1 : array, shape (R', C', 2)\n        the displacement field driving the transformation\n    affine_idx_in : array, shape (3, 3)\n        the matrix A in eq. (1) above\n    affine_idx_out : array, shape (3, 3)\n        the matrix B in eq. (1) above\n    affine_disp : array, shape (3, 3)\n        the matrix C in eq. (1) above\n    out_shape : array, shape (2,)\n        the number of rows and columns of the sampling grid\n\n    Returns\n    -------\n    warped : array, shape = out_shape\n        the transformed image\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_51warp_2d_nn = {"warp_2d_nn", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_51warp_2d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_50warp_2d_nn};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_51warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    values[2] = __pyx_k__238;
    values[3] = __pyx_k__239;
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_50warp_2d_nn(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_50warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  int __pyx_v_short_is_signed;
  int __pyx_v_long_is_signed;
  int __pyx_v_int_is_signed;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("warp_2d_nn", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1 * 2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  { Py_ssize_t __pyx_temp;
    for (__pyx_temp=0; __pyx_temp < 2; __pyx_temp++) {
      __Pyx_INCREF(Py_None);
      __Pyx_GIVEREF(Py_None);
      PyList_SET_ITEM(__pyx_t_1, __pyx_temp, Py_None);
    }
  }
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  __pyx_v_short_is_signed = (((short)-1L) < 0);
  __pyx_v_long_is_signed = (((long)-1L) < 0);
  __pyx_v_int_is_signed = (((int)-1L) < 0);
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_image, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_image); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          __pyx_t_3 = (((sizeof(short)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_short_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_short, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(int)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L26_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L26_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_int_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L26_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_int, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(long)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L30_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L30_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_long_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L30_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_long, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L34_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L34_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L37_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L37_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          __pyx_t_3 = (((sizeof(__pyx_t_float_complex)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L40_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L40_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_float_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(__pyx_t_double_complex)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L43_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L43_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_double_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L46_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(short))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L46_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_short(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_short, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L50_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(int))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L50_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_int(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_int, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L54_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(long))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L54_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_long(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_long, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L58_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L58_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L62_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L62_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L66_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(__pyx_t_float_complex))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L66_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_float_complex(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_float_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L70_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(__pyx_t_double_complex))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L70_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_double_complex(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_double_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((1 < __pyx_t_10) != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 1);
    __Pyx_INCREF(__pyx_t_8);
    __Pyx_DECREF_SET(__pyx_v_arg, __pyx_t_8);
    __pyx_t_8 = 0;
    goto __pyx_L73;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_2 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_d1, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_d1); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF_SET(__pyx_v_arg, __pyx_t_8);
    __pyx_t_8 = 0;
    goto __pyx_L73;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_8 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_8); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = PyTuple_New(1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_8, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_Raise(__pyx_t_9, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L73:;
  while (1) {
    __pyx_t_3 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_2 = (__pyx_t_3 != 0);
    if (__pyx_t_2) {
      __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_XDECREF_SET(__pyx_v_dtype, __pyx_t_9);
        __pyx_t_9 = 0;
        goto __pyx_L77;
      }
      __pyx_t_3 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_3) {
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_XDECREF_SET(__pyx_v_arg_base, __pyx_t_9);
        __pyx_t_9 = 0;
        __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_2 = (__pyx_t_3 != 0);
        if (__pyx_t_2) {
          __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_XDECREF_SET(__pyx_v_dtype, __pyx_t_9);
          __pyx_t_9 = 0;
          goto __pyx_L78;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __Pyx_XDECREF_SET(__pyx_v_dtype, Py_None);
        }
        __pyx_L78:;
        goto __pyx_L77;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __Pyx_XDECREF_SET(__pyx_v_dtype, Py_None);
      }
      __pyx_L77:;
      __pyx_v_itemsize = -1L;
      __pyx_t_2 = (__pyx_v_dtype != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_9); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_2 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_2) {
          } else {
            __pyx_t_3 = __pyx_t_2;
            goto __pyx_L81_bool_binop_done;
          }
          __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __pyx_t_2 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_3 = __pyx_t_2;
          __pyx_L81_bool_binop_done:;
          if (__pyx_t_3) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 1, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L75_break;
          }
          __pyx_t_2 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_2) {
          } else {
            __pyx_t_3 = __pyx_t_2;
            goto __pyx_L84_bool_binop_done;
          }
          __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __pyx_t_2 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_3 = __pyx_t_2;
          __pyx_L84_bool_binop_done:;
          if (__pyx_t_3) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 1, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L75_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_2 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_3 = __pyx_t_2;
      goto __pyx_L87_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_3 = __pyx_t_2;
    __pyx_L87_bool_binop_done:;
    if (__pyx_t_3) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_v_arg);
      __pyx_t_3 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_3) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 1, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L75_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_2 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_3 = __pyx_t_2;
      goto __pyx_L91_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_3 = __pyx_t_2;
    __pyx_L91_bool_binop_done:;
    if (__pyx_t_3) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_v_arg);
      __pyx_t_3 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_3) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 1, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L75_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 1, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L75_break;
  }
  __pyx_L75_break:;
  __pyx_t_9 = PyList_New(0); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_v_candidates = ((PyObject*)__pyx_t_9);
  __pyx_t_9 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_8 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __pyx_t_9 = __pyx_t_8;
  __pyx_t_8 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_9, __pyx_t_12, &__pyx_t_10, &__pyx_t_8, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_tuple__240, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_tuple__241, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_8 = __pyx_t_1; __Pyx_INCREF(__pyx_t_8); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_8 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_15 = Py_TYPE(__pyx_t_8)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_8))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_8)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_8, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_8, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_8)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_8, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_8, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_8);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L98_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L98_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L99_unpacking_done;
        __pyx_L98_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L99_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_3 = (__pyx_v_dst_type != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_2) {
          __pyx_v_match_found = 1;
          goto __pyx_L101;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L97_break;
        }
        __pyx_L101:;
      }
    }
    __pyx_L97_break:;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_2 = (__pyx_v_match_found != 0);
    if (__pyx_t_2) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_2 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_3 = ((!__pyx_t_2) != 0);
  if (__pyx_t_3) {
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__242, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_Raise(__pyx_t_9, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_3) {
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__243, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_Raise(__pyx_t_9, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_r = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_514__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults76, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults76, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults76, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults76, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0_0__pyx_pw_4dipy_5align_13vector_fields_281warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0_0__pyx_mdef_4dipy_5align_13vector_fields_281warp_2d_nn = {"__pyx_fuse_0_0warp_2d_nn", (PyCFunction)__pyx_fuse_0_0__pyx_pw_4dipy_5align_13vector_fields_281warp_2d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_50warp_2d_nn};
static PyObject *__pyx_fuse_0_0__pyx_pw_4dipy_5align_13vector_fields_281warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_2d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults76 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults76, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_2d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_short(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_280warp_2d_nn(__pyx_self, __pyx_v_image, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_280warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_memviewslice __pyx_t_13 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0_0warp_2d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2466
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2467
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2468
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2469
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double di, dj, dii, djj
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__244, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__245, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__246, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2481
 * 
 *     if out_shape is not None:
 *         nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 */
    __pyx_t_9 = 0;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2482
 *     if out_shape is not None:
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 */
    __pyx_t_10 = 1;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2484
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 *         nrows = d1.shape[0]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2485
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]             # <<<<<<<<<<<<<<
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2487
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_short, (int (*)(char *, PyObject *)) __pyx_memview_set_short, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_11) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_13 = __Pyx_PyObject_to_MemoryviewSlice_dsds_short(__pyx_t_3);
  if (unlikely(!__pyx_t_13.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_warped = __pyx_t_13;
  __pyx_t_13.memview = NULL;
  __pyx_t_13.data = NULL;

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__247) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_12);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_tmp = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2492
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2493
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2496
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d1[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_19 * __pyx_v_d1.strides[0]) ) + __pyx_t_20 * __pyx_v_d1.strides[1]) ) + __pyx_t_21 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2497
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":2499
 *                     djj = d1[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":2500
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2501
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2503
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d1, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":2504
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":2505
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2509
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2511
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":2514
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":2515
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L15:;

            /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2519
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 */
              __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2520
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = di + i
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
              goto __pyx_L16;
            }

            /* "dipy/align/vector_fields.pyx":2522
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 *                     dii = di + i             # <<<<<<<<<<<<<<
 *                     djj = dj + j
 * 
 */
            /*else*/ {
              __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":2523
 *                 else:
 *                     dii = di + i
 *                     djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                 # Interpolate the input image at the resulting location
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
            }
            __pyx_L16:;

            /* "dipy/align/vector_fields.pyx":2527
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
            __pyx_t_27 = __pyx_v_i;
            __pyx_t_28 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2526
 * 
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &warped[i, j])
 *     return warped
 */
            __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((short *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_27 * __pyx_v_warped.strides[0]) ) + __pyx_t_28 * __pyx_v_warped.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2528
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_warped, 2, (PyObject *(*)(char *)) __pyx_memview_get_short, (int (*)(char *, PyObject *)) __pyx_memview_set_short, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_r = __pyx_t_12;
  __pyx_t_12 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __PYX_XDEC_MEMVIEW(&__pyx_t_13, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_516__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults77, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults77, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults77, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults77, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0_1__pyx_pw_4dipy_5align_13vector_fields_283warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0_1__pyx_mdef_4dipy_5align_13vector_fields_283warp_2d_nn = {"__pyx_fuse_0_1warp_2d_nn", (PyCFunction)__pyx_fuse_0_1__pyx_pw_4dipy_5align_13vector_fields_283warp_2d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_50warp_2d_nn};
static PyObject *__pyx_fuse_0_1__pyx_pw_4dipy_5align_13vector_fields_283warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_2d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults77 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults77, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_2d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_short(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_282warp_2d_nn(__pyx_self, __pyx_v_image, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_282warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_memviewslice __pyx_t_13 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0_1warp_2d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2466
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2467
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2468
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2469
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double di, dj, dii, djj
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__248, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__249, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__250, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2481
 * 
 *     if out_shape is not None:
 *         nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 */
    __pyx_t_9 = 0;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2482
 *     if out_shape is not None:
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 */
    __pyx_t_10 = 1;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2484
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 *         nrows = d1.shape[0]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2485
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]             # <<<<<<<<<<<<<<
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2487
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_short, (int (*)(char *, PyObject *)) __pyx_memview_set_short, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_11) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_13 = __Pyx_PyObject_to_MemoryviewSlice_dsds_short(__pyx_t_3);
  if (unlikely(!__pyx_t_13.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_warped = __pyx_t_13;
  __pyx_t_13.memview = NULL;
  __pyx_t_13.data = NULL;

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__251) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_12);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_tmp = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2492
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2493
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2496
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d1[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_19 * __pyx_v_d1.strides[0]) ) + __pyx_t_20 * __pyx_v_d1.strides[1]) ) + __pyx_t_21 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2497
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":2499
 *                     djj = d1[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":2500
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2501
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2503
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d1, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":2504
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":2505
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2509
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2511
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":2514
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":2515
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L15:;

            /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2519
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 */
              __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2520
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = di + i
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
              goto __pyx_L16;
            }

            /* "dipy/align/vector_fields.pyx":2522
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 *                     dii = di + i             # <<<<<<<<<<<<<<
 *                     djj = dj + j
 * 
 */
            /*else*/ {
              __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":2523
 *                 else:
 *                     dii = di + i
 *                     djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                 # Interpolate the input image at the resulting location
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
            }
            __pyx_L16:;

            /* "dipy/align/vector_fields.pyx":2527
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
            __pyx_t_27 = __pyx_v_i;
            __pyx_t_28 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2526
 * 
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &warped[i, j])
 *     return warped
 */
            __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((short *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_27 * __pyx_v_warped.strides[0]) ) + __pyx_t_28 * __pyx_v_warped.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2528
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_warped, 2, (PyObject *(*)(char *)) __pyx_memview_get_short, (int (*)(char *, PyObject *)) __pyx_memview_set_short, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_r = __pyx_t_12;
  __pyx_t_12 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __PYX_XDEC_MEMVIEW(&__pyx_t_13, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_518__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults78, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults78, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults78, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults78, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1_0__pyx_pw_4dipy_5align_13vector_fields_285warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1_0__pyx_mdef_4dipy_5align_13vector_fields_285warp_2d_nn = {"__pyx_fuse_1_0warp_2d_nn", (PyCFunction)__pyx_fuse_1_0__pyx_pw_4dipy_5align_13vector_fields_285warp_2d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_50warp_2d_nn};
static PyObject *__pyx_fuse_1_0__pyx_pw_4dipy_5align_13vector_fields_285warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_2d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults78 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults78, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_2d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_int(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_284warp_2d_nn(__pyx_self, __pyx_v_image, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_284warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_memviewslice __pyx_t_13 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1_0warp_2d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2466
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2467
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2468
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2469
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double di, dj, dii, djj
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__252, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__253, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__254, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2481
 * 
 *     if out_shape is not None:
 *         nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 */
    __pyx_t_9 = 0;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2482
 *     if out_shape is not None:
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 */
    __pyx_t_10 = 1;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2484
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 *         nrows = d1.shape[0]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2485
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]             # <<<<<<<<<<<<<<
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2487
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_11) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_13 = __Pyx_PyObject_to_MemoryviewSlice_dsds_int(__pyx_t_3);
  if (unlikely(!__pyx_t_13.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_warped = __pyx_t_13;
  __pyx_t_13.memview = NULL;
  __pyx_t_13.data = NULL;

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__255) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_12);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_tmp = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2492
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2493
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2496
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d1[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_19 * __pyx_v_d1.strides[0]) ) + __pyx_t_20 * __pyx_v_d1.strides[1]) ) + __pyx_t_21 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2497
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":2499
 *                     djj = d1[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":2500
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2501
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2503
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d1, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":2504
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":2505
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2509
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2511
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":2514
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":2515
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L15:;

            /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2519
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 */
              __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2520
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = di + i
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
              goto __pyx_L16;
            }

            /* "dipy/align/vector_fields.pyx":2522
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 *                     dii = di + i             # <<<<<<<<<<<<<<
 *                     djj = dj + j
 * 
 */
            /*else*/ {
              __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":2523
 *                 else:
 *                     dii = di + i
 *                     djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                 # Interpolate the input image at the resulting location
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
            }
            __pyx_L16:;

            /* "dipy/align/vector_fields.pyx":2527
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
            __pyx_t_27 = __pyx_v_i;
            __pyx_t_28 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2526
 * 
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &warped[i, j])
 *     return warped
 */
            __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_27 * __pyx_v_warped.strides[0]) ) + __pyx_t_28 * __pyx_v_warped.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2528
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_warped, 2, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_r = __pyx_t_12;
  __pyx_t_12 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __PYX_XDEC_MEMVIEW(&__pyx_t_13, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_520__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults79, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults79, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults79, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults79, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1_1__pyx_pw_4dipy_5align_13vector_fields_287warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1_1__pyx_mdef_4dipy_5align_13vector_fields_287warp_2d_nn = {"__pyx_fuse_1_1warp_2d_nn", (PyCFunction)__pyx_fuse_1_1__pyx_pw_4dipy_5align_13vector_fields_287warp_2d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_50warp_2d_nn};
static PyObject *__pyx_fuse_1_1__pyx_pw_4dipy_5align_13vector_fields_287warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_2d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults79 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults79, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_2d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_int(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_286warp_2d_nn(__pyx_self, __pyx_v_image, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_286warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_memviewslice __pyx_t_13 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1_1warp_2d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2466
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2467
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2468
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2469
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double di, dj, dii, djj
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__256, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__257, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__258, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2481
 * 
 *     if out_shape is not None:
 *         nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 */
    __pyx_t_9 = 0;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2482
 *     if out_shape is not None:
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 */
    __pyx_t_10 = 1;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2484
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 *         nrows = d1.shape[0]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2485
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]             # <<<<<<<<<<<<<<
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2487
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_11) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_13 = __Pyx_PyObject_to_MemoryviewSlice_dsds_int(__pyx_t_3);
  if (unlikely(!__pyx_t_13.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_warped = __pyx_t_13;
  __pyx_t_13.memview = NULL;
  __pyx_t_13.data = NULL;

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__259) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_12);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_tmp = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2492
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2493
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2496
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d1[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_19 * __pyx_v_d1.strides[0]) ) + __pyx_t_20 * __pyx_v_d1.strides[1]) ) + __pyx_t_21 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2497
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":2499
 *                     djj = d1[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":2500
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2501
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2503
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d1, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":2504
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":2505
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2509
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2511
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":2514
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":2515
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L15:;

            /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2519
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 */
              __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2520
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = di + i
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
              goto __pyx_L16;
            }

            /* "dipy/align/vector_fields.pyx":2522
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 *                     dii = di + i             # <<<<<<<<<<<<<<
 *                     djj = dj + j
 * 
 */
            /*else*/ {
              __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":2523
 *                 else:
 *                     dii = di + i
 *                     djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                 # Interpolate the input image at the resulting location
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
            }
            __pyx_L16:;

            /* "dipy/align/vector_fields.pyx":2527
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
            __pyx_t_27 = __pyx_v_i;
            __pyx_t_28 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2526
 * 
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &warped[i, j])
 *     return warped
 */
            __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_27 * __pyx_v_warped.strides[0]) ) + __pyx_t_28 * __pyx_v_warped.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2528
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_warped, 2, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_r = __pyx_t_12;
  __pyx_t_12 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __PYX_XDEC_MEMVIEW(&__pyx_t_13, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_522__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults80, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults80, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults80, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults80, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_2_0__pyx_pw_4dipy_5align_13vector_fields_289warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_2_0__pyx_mdef_4dipy_5align_13vector_fields_289warp_2d_nn = {"__pyx_fuse_2_0warp_2d_nn", (PyCFunction)__pyx_fuse_2_0__pyx_pw_4dipy_5align_13vector_fields_289warp_2d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_50warp_2d_nn};
static PyObject *__pyx_fuse_2_0__pyx_pw_4dipy_5align_13vector_fields_289warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_2d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults80 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults80, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_2d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_long(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_288warp_2d_nn(__pyx_self, __pyx_v_image, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_288warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_memviewslice __pyx_t_13 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_2_0warp_2d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2466
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2467
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2468
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2469
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double di, dj, dii, djj
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__260, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__261, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__262, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2481
 * 
 *     if out_shape is not None:
 *         nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 */
    __pyx_t_9 = 0;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2482
 *     if out_shape is not None:
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 */
    __pyx_t_10 = 1;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2484
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 *         nrows = d1.shape[0]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2485
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]             # <<<<<<<<<<<<<<
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2487
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_long, (int (*)(char *, PyObject *)) __pyx_memview_set_long, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_11) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_13 = __Pyx_PyObject_to_MemoryviewSlice_dsds_long(__pyx_t_3);
  if (unlikely(!__pyx_t_13.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_warped = __pyx_t_13;
  __pyx_t_13.memview = NULL;
  __pyx_t_13.data = NULL;

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__263) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_12);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_tmp = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2492
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2493
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2496
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d1[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_19 * __pyx_v_d1.strides[0]) ) + __pyx_t_20 * __pyx_v_d1.strides[1]) ) + __pyx_t_21 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2497
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":2499
 *                     djj = d1[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":2500
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2501
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2503
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d1, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":2504
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":2505
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2509
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2511
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":2514
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":2515
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L15:;

            /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2519
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 */
              __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2520
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = di + i
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
              goto __pyx_L16;
            }

            /* "dipy/align/vector_fields.pyx":2522
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 *                     dii = di + i             # <<<<<<<<<<<<<<
 *                     djj = dj + j
 * 
 */
            /*else*/ {
              __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":2523
 *                 else:
 *                     dii = di + i
 *                     djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                 # Interpolate the input image at the resulting location
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
            }
            __pyx_L16:;

            /* "dipy/align/vector_fields.pyx":2527
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
            __pyx_t_27 = __pyx_v_i;
            __pyx_t_28 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2526
 * 
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &warped[i, j])
 *     return warped
 */
            __pyx_fuse_2__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((long *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_27 * __pyx_v_warped.strides[0]) ) + __pyx_t_28 * __pyx_v_warped.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2528
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_warped, 2, (PyObject *(*)(char *)) __pyx_memview_get_long, (int (*)(char *, PyObject *)) __pyx_memview_set_long, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_r = __pyx_t_12;
  __pyx_t_12 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __PYX_XDEC_MEMVIEW(&__pyx_t_13, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_524__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults81, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults81, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults81, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults81, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_2_1__pyx_pw_4dipy_5align_13vector_fields_291warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_2_1__pyx_mdef_4dipy_5align_13vector_fields_291warp_2d_nn = {"__pyx_fuse_2_1warp_2d_nn", (PyCFunction)__pyx_fuse_2_1__pyx_pw_4dipy_5align_13vector_fields_291warp_2d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_50warp_2d_nn};
static PyObject *__pyx_fuse_2_1__pyx_pw_4dipy_5align_13vector_fields_291warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_2d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults81 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults81, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_2d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_long(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_290warp_2d_nn(__pyx_self, __pyx_v_image, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_290warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_memviewslice __pyx_t_13 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_2_1warp_2d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2466
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2467
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2468
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2469
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double di, dj, dii, djj
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__264, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__265, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__266, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2481
 * 
 *     if out_shape is not None:
 *         nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 */
    __pyx_t_9 = 0;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2482
 *     if out_shape is not None:
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 */
    __pyx_t_10 = 1;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2484
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 *         nrows = d1.shape[0]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2485
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]             # <<<<<<<<<<<<<<
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2487
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_long, (int (*)(char *, PyObject *)) __pyx_memview_set_long, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_11) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_13 = __Pyx_PyObject_to_MemoryviewSlice_dsds_long(__pyx_t_3);
  if (unlikely(!__pyx_t_13.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_warped = __pyx_t_13;
  __pyx_t_13.memview = NULL;
  __pyx_t_13.data = NULL;

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__267) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_12);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_tmp = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2492
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2493
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2496
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d1[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_19 * __pyx_v_d1.strides[0]) ) + __pyx_t_20 * __pyx_v_d1.strides[1]) ) + __pyx_t_21 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2497
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":2499
 *                     djj = d1[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":2500
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2501
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2503
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d1, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":2504
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":2505
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2509
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2511
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":2514
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":2515
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L15:;

            /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2519
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 */
              __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2520
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = di + i
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
              goto __pyx_L16;
            }

            /* "dipy/align/vector_fields.pyx":2522
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 *                     dii = di + i             # <<<<<<<<<<<<<<
 *                     djj = dj + j
 * 
 */
            /*else*/ {
              __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":2523
 *                 else:
 *                     dii = di + i
 *                     djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                 # Interpolate the input image at the resulting location
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
            }
            __pyx_L16:;

            /* "dipy/align/vector_fields.pyx":2527
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
            __pyx_t_27 = __pyx_v_i;
            __pyx_t_28 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2526
 * 
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &warped[i, j])
 *     return warped
 */
            __pyx_fuse_2__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((long *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_27 * __pyx_v_warped.strides[0]) ) + __pyx_t_28 * __pyx_v_warped.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2528
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_warped, 2, (PyObject *(*)(char *)) __pyx_memview_get_long, (int (*)(char *, PyObject *)) __pyx_memview_set_long, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_r = __pyx_t_12;
  __pyx_t_12 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __PYX_XDEC_MEMVIEW(&__pyx_t_13, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_526__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults82, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults82, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults82, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults82, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_3_0__pyx_pw_4dipy_5align_13vector_fields_293warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_3_0__pyx_mdef_4dipy_5align_13vector_fields_293warp_2d_nn = {"__pyx_fuse_3_0warp_2d_nn", (PyCFunction)__pyx_fuse_3_0__pyx_pw_4dipy_5align_13vector_fields_293warp_2d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_50warp_2d_nn};
static PyObject *__pyx_fuse_3_0__pyx_pw_4dipy_5align_13vector_fields_293warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_2d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults82 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults82, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_2d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_292warp_2d_nn(__pyx_self, __pyx_v_image, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_292warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_memviewslice __pyx_t_13 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_3_0warp_2d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2466
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2467
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2468
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2469
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double di, dj, dii, djj
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__268, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__269, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__270, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2481
 * 
 *     if out_shape is not None:
 *         nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 */
    __pyx_t_9 = 0;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2482
 *     if out_shape is not None:
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 */
    __pyx_t_10 = 1;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2484
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 *         nrows = d1.shape[0]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2485
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]             # <<<<<<<<<<<<<<
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2487
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_11) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_13 = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_t_3);
  if (unlikely(!__pyx_t_13.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_warped = __pyx_t_13;
  __pyx_t_13.memview = NULL;
  __pyx_t_13.data = NULL;

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__271) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_12);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_tmp = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2492
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2493
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2496
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d1[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_19 * __pyx_v_d1.strides[0]) ) + __pyx_t_20 * __pyx_v_d1.strides[1]) ) + __pyx_t_21 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2497
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":2499
 *                     djj = d1[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":2500
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2501
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2503
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d1, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":2504
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":2505
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2509
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2511
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":2514
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":2515
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L15:;

            /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2519
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 */
              __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2520
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = di + i
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
              goto __pyx_L16;
            }

            /* "dipy/align/vector_fields.pyx":2522
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 *                     dii = di + i             # <<<<<<<<<<<<<<
 *                     djj = dj + j
 * 
 */
            /*else*/ {
              __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":2523
 *                 else:
 *                     dii = di + i
 *                     djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                 # Interpolate the input image at the resulting location
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
            }
            __pyx_L16:;

            /* "dipy/align/vector_fields.pyx":2527
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
            __pyx_t_27 = __pyx_v_i;
            __pyx_t_28 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2526
 * 
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &warped[i, j])
 *     return warped
 */
            __pyx_fuse_3__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_27 * __pyx_v_warped.strides[0]) ) + __pyx_t_28 * __pyx_v_warped.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2528
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_warped, 2, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_r = __pyx_t_12;
  __pyx_t_12 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __PYX_XDEC_MEMVIEW(&__pyx_t_13, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_528__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults83, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults83, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults83, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults83, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_3_1__pyx_pw_4dipy_5align_13vector_fields_295warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_3_1__pyx_mdef_4dipy_5align_13vector_fields_295warp_2d_nn = {"__pyx_fuse_3_1warp_2d_nn", (PyCFunction)__pyx_fuse_3_1__pyx_pw_4dipy_5align_13vector_fields_295warp_2d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_50warp_2d_nn};
static PyObject *__pyx_fuse_3_1__pyx_pw_4dipy_5align_13vector_fields_295warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_2d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults83 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults83, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_2d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_294warp_2d_nn(__pyx_self, __pyx_v_image, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_294warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_memviewslice __pyx_t_13 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_3_1warp_2d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2466
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2467
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2468
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2469
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double di, dj, dii, djj
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__272, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__273, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__274, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2481
 * 
 *     if out_shape is not None:
 *         nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 */
    __pyx_t_9 = 0;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2482
 *     if out_shape is not None:
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 */
    __pyx_t_10 = 1;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2484
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 *         nrows = d1.shape[0]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2485
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]             # <<<<<<<<<<<<<<
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2487
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_11) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_13 = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_t_3);
  if (unlikely(!__pyx_t_13.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_warped = __pyx_t_13;
  __pyx_t_13.memview = NULL;
  __pyx_t_13.data = NULL;

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__275) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_12);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_tmp = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2492
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2493
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2496
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d1[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_19 * __pyx_v_d1.strides[0]) ) + __pyx_t_20 * __pyx_v_d1.strides[1]) ) + __pyx_t_21 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2497
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":2499
 *                     djj = d1[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":2500
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2501
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2503
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d1, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":2504
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":2505
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2509
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2511
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":2514
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":2515
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L15:;

            /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2519
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 */
              __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2520
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = di + i
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
              goto __pyx_L16;
            }

            /* "dipy/align/vector_fields.pyx":2522
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 *                     dii = di + i             # <<<<<<<<<<<<<<
 *                     djj = dj + j
 * 
 */
            /*else*/ {
              __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":2523
 *                 else:
 *                     dii = di + i
 *                     djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                 # Interpolate the input image at the resulting location
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
            }
            __pyx_L16:;

            /* "dipy/align/vector_fields.pyx":2527
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
            __pyx_t_27 = __pyx_v_i;
            __pyx_t_28 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2526
 * 
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &warped[i, j])
 *     return warped
 */
            __pyx_fuse_3__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_27 * __pyx_v_warped.strides[0]) ) + __pyx_t_28 * __pyx_v_warped.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2528
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_warped, 2, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_r = __pyx_t_12;
  __pyx_t_12 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __PYX_XDEC_MEMVIEW(&__pyx_t_13, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_530__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults84, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults84, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults84, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults84, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_4_0__pyx_pw_4dipy_5align_13vector_fields_297warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_4_0__pyx_mdef_4dipy_5align_13vector_fields_297warp_2d_nn = {"__pyx_fuse_4_0warp_2d_nn", (PyCFunction)__pyx_fuse_4_0__pyx_pw_4dipy_5align_13vector_fields_297warp_2d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_50warp_2d_nn};
static PyObject *__pyx_fuse_4_0__pyx_pw_4dipy_5align_13vector_fields_297warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_2d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults84 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults84, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_2d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_296warp_2d_nn(__pyx_self, __pyx_v_image, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_296warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_memviewslice __pyx_t_13 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_4_0warp_2d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2466
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2467
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2468
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2469
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double di, dj, dii, djj
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__276, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__277, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__278, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2481
 * 
 *     if out_shape is not None:
 *         nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 */
    __pyx_t_9 = 0;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2482
 *     if out_shape is not None:
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 */
    __pyx_t_10 = 1;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2484
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 *         nrows = d1.shape[0]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2485
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]             # <<<<<<<<<<<<<<
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2487
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_11) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_13 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_t_3);
  if (unlikely(!__pyx_t_13.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_warped = __pyx_t_13;
  __pyx_t_13.memview = NULL;
  __pyx_t_13.data = NULL;

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__279) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_12);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_tmp = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2492
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2493
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2496
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d1[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_19 * __pyx_v_d1.strides[0]) ) + __pyx_t_20 * __pyx_v_d1.strides[1]) ) + __pyx_t_21 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2497
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":2499
 *                     djj = d1[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":2500
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2501
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2503
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d1, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":2504
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":2505
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2509
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2511
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":2514
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":2515
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L15:;

            /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2519
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 */
              __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2520
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = di + i
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
              goto __pyx_L16;
            }

            /* "dipy/align/vector_fields.pyx":2522
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 *                     dii = di + i             # <<<<<<<<<<<<<<
 *                     djj = dj + j
 * 
 */
            /*else*/ {
              __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":2523
 *                 else:
 *                     dii = di + i
 *                     djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                 # Interpolate the input image at the resulting location
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
            }
            __pyx_L16:;

            /* "dipy/align/vector_fields.pyx":2527
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
            __pyx_t_27 = __pyx_v_i;
            __pyx_t_28 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2526
 * 
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &warped[i, j])
 *     return warped
 */
            __pyx_fuse_4__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_27 * __pyx_v_warped.strides[0]) ) + __pyx_t_28 * __pyx_v_warped.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2528
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_warped, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_r = __pyx_t_12;
  __pyx_t_12 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __PYX_XDEC_MEMVIEW(&__pyx_t_13, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_532__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults85, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults85, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults85, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults85, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_4_1__pyx_pw_4dipy_5align_13vector_fields_299warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_4_1__pyx_mdef_4dipy_5align_13vector_fields_299warp_2d_nn = {"__pyx_fuse_4_1warp_2d_nn", (PyCFunction)__pyx_fuse_4_1__pyx_pw_4dipy_5align_13vector_fields_299warp_2d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_50warp_2d_nn};
static PyObject *__pyx_fuse_4_1__pyx_pw_4dipy_5align_13vector_fields_299warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_2d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults85 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults85, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_2d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_298warp_2d_nn(__pyx_self, __pyx_v_image, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_298warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_memviewslice __pyx_t_13 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_4_1warp_2d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2466
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2467
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2468
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2469
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double di, dj, dii, djj
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__280, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__281, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__282, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2481
 * 
 *     if out_shape is not None:
 *         nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 */
    __pyx_t_9 = 0;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2482
 *     if out_shape is not None:
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 */
    __pyx_t_10 = 1;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2484
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 *         nrows = d1.shape[0]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2485
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]             # <<<<<<<<<<<<<<
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2487
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_11) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_13 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_t_3);
  if (unlikely(!__pyx_t_13.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_warped = __pyx_t_13;
  __pyx_t_13.memview = NULL;
  __pyx_t_13.data = NULL;

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__283) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_12);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_tmp = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2492
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2493
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2496
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d1[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_19 * __pyx_v_d1.strides[0]) ) + __pyx_t_20 * __pyx_v_d1.strides[1]) ) + __pyx_t_21 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2497
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":2499
 *                     djj = d1[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":2500
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2501
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2503
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d1, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":2504
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":2505
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2509
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2511
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":2514
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":2515
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L15:;

            /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2519
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 */
              __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2520
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = di + i
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
              goto __pyx_L16;
            }

            /* "dipy/align/vector_fields.pyx":2522
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 *                     dii = di + i             # <<<<<<<<<<<<<<
 *                     djj = dj + j
 * 
 */
            /*else*/ {
              __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":2523
 *                 else:
 *                     dii = di + i
 *                     djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                 # Interpolate the input image at the resulting location
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
            }
            __pyx_L16:;

            /* "dipy/align/vector_fields.pyx":2527
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
            __pyx_t_27 = __pyx_v_i;
            __pyx_t_28 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2526
 * 
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &warped[i, j])
 *     return warped
 */
            __pyx_fuse_4__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_27 * __pyx_v_warped.strides[0]) ) + __pyx_t_28 * __pyx_v_warped.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2528
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_warped, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_r = __pyx_t_12;
  __pyx_t_12 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __PYX_XDEC_MEMVIEW(&__pyx_t_13, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_534__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults86, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults86, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults86, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults86, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_5_0__pyx_pw_4dipy_5align_13vector_fields_301warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_5_0__pyx_mdef_4dipy_5align_13vector_fields_301warp_2d_nn = {"__pyx_fuse_5_0warp_2d_nn", (PyCFunction)__pyx_fuse_5_0__pyx_pw_4dipy_5align_13vector_fields_301warp_2d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_50warp_2d_nn};
static PyObject *__pyx_fuse_5_0__pyx_pw_4dipy_5align_13vector_fields_301warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_2d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults86 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults86, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_2d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_float_complex(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_300warp_2d_nn(__pyx_self, __pyx_v_image, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_300warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_memviewslice __pyx_t_13 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_5_0warp_2d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2466
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2467
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2468
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2469
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double di, dj, dii, djj
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__284, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__285, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__286, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2481
 * 
 *     if out_shape is not None:
 *         nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 */
    __pyx_t_9 = 0;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2482
 *     if out_shape is not None:
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 */
    __pyx_t_10 = 1;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2484
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 *         nrows = d1.shape[0]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2485
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]             # <<<<<<<<<<<<<<
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2487
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_float_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_float_complex, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_11) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_13 = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_float_complex(__pyx_t_3);
  if (unlikely(!__pyx_t_13.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_warped = __pyx_t_13;
  __pyx_t_13.memview = NULL;
  __pyx_t_13.data = NULL;

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__287) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_12);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_tmp = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2492
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2493
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2496
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d1[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_19 * __pyx_v_d1.strides[0]) ) + __pyx_t_20 * __pyx_v_d1.strides[1]) ) + __pyx_t_21 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2497
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":2499
 *                     djj = d1[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":2500
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2501
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2503
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d1, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":2504
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":2505
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2509
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2511
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":2514
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":2515
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L15:;

            /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2519
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 */
              __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2520
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = di + i
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
              goto __pyx_L16;
            }

            /* "dipy/align/vector_fields.pyx":2522
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 *                     dii = di + i             # <<<<<<<<<<<<<<
 *                     djj = dj + j
 * 
 */
            /*else*/ {
              __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":2523
 *                 else:
 *                     dii = di + i
 *                     djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                 # Interpolate the input image at the resulting location
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
            }
            __pyx_L16:;

            /* "dipy/align/vector_fields.pyx":2527
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
            __pyx_t_27 = __pyx_v_i;
            __pyx_t_28 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2526
 * 
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &warped[i, j])
 *     return warped
 */
            __pyx_fuse_5__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((__pyx_t_float_complex *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_27 * __pyx_v_warped.strides[0]) ) + __pyx_t_28 * __pyx_v_warped.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2528
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_warped, 2, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_float_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_float_complex, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_r = __pyx_t_12;
  __pyx_t_12 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __PYX_XDEC_MEMVIEW(&__pyx_t_13, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_536__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults87, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults87, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults87, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults87, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_5_1__pyx_pw_4dipy_5align_13vector_fields_303warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_5_1__pyx_mdef_4dipy_5align_13vector_fields_303warp_2d_nn = {"__pyx_fuse_5_1warp_2d_nn", (PyCFunction)__pyx_fuse_5_1__pyx_pw_4dipy_5align_13vector_fields_303warp_2d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_50warp_2d_nn};
static PyObject *__pyx_fuse_5_1__pyx_pw_4dipy_5align_13vector_fields_303warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_2d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults87 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults87, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_2d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_float_complex(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_302warp_2d_nn(__pyx_self, __pyx_v_image, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_302warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_memviewslice __pyx_t_13 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_5_1warp_2d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2466
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2467
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2468
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2469
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double di, dj, dii, djj
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__288, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__289, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__290, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2481
 * 
 *     if out_shape is not None:
 *         nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 */
    __pyx_t_9 = 0;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2482
 *     if out_shape is not None:
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 */
    __pyx_t_10 = 1;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2484
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 *         nrows = d1.shape[0]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2485
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]             # <<<<<<<<<<<<<<
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2487
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_float_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_float_complex, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_11) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_13 = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_float_complex(__pyx_t_3);
  if (unlikely(!__pyx_t_13.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_warped = __pyx_t_13;
  __pyx_t_13.memview = NULL;
  __pyx_t_13.data = NULL;

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__291) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_12);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_tmp = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2492
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2493
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2496
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d1[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_19 * __pyx_v_d1.strides[0]) ) + __pyx_t_20 * __pyx_v_d1.strides[1]) ) + __pyx_t_21 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2497
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":2499
 *                     djj = d1[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":2500
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2501
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2503
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d1, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":2504
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":2505
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2509
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2511
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":2514
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":2515
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L15:;

            /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2519
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 */
              __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2520
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = di + i
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
              goto __pyx_L16;
            }

            /* "dipy/align/vector_fields.pyx":2522
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 *                     dii = di + i             # <<<<<<<<<<<<<<
 *                     djj = dj + j
 * 
 */
            /*else*/ {
              __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":2523
 *                 else:
 *                     dii = di + i
 *                     djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                 # Interpolate the input image at the resulting location
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
            }
            __pyx_L16:;

            /* "dipy/align/vector_fields.pyx":2527
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
            __pyx_t_27 = __pyx_v_i;
            __pyx_t_28 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2526
 * 
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &warped[i, j])
 *     return warped
 */
            __pyx_fuse_5__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((__pyx_t_float_complex *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_27 * __pyx_v_warped.strides[0]) ) + __pyx_t_28 * __pyx_v_warped.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2528
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_warped, 2, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_float_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_float_complex, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_r = __pyx_t_12;
  __pyx_t_12 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __PYX_XDEC_MEMVIEW(&__pyx_t_13, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_538__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults88, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults88, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults88, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults88, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_6_0__pyx_pw_4dipy_5align_13vector_fields_305warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_6_0__pyx_mdef_4dipy_5align_13vector_fields_305warp_2d_nn = {"__pyx_fuse_6_0warp_2d_nn", (PyCFunction)__pyx_fuse_6_0__pyx_pw_4dipy_5align_13vector_fields_305warp_2d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_50warp_2d_nn};
static PyObject *__pyx_fuse_6_0__pyx_pw_4dipy_5align_13vector_fields_305warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_2d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults88 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults88, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_2d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_double_complex(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_304warp_2d_nn(__pyx_self, __pyx_v_image, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_304warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_memviewslice __pyx_t_13 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_6_0warp_2d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2466
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2467
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2468
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2469
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double di, dj, dii, djj
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__292, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__293, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__294, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2481
 * 
 *     if out_shape is not None:
 *         nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 */
    __pyx_t_9 = 0;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2482
 *     if out_shape is not None:
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 */
    __pyx_t_10 = 1;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2484
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 *         nrows = d1.shape[0]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2485
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]             # <<<<<<<<<<<<<<
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2487
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_double_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_double_complex, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_11) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_13 = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_double_complex(__pyx_t_3);
  if (unlikely(!__pyx_t_13.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_warped = __pyx_t_13;
  __pyx_t_13.memview = NULL;
  __pyx_t_13.data = NULL;

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__295) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_ds_float(__pyx_t_12);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_tmp = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2492
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2493
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2496
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d1[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_19 * __pyx_v_d1.strides[0]) ) + __pyx_t_20 * __pyx_v_d1.strides[1]) ) + __pyx_t_21 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2497
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":2499
 *                     djj = d1[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":2500
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2501
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2503
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d1, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":2504
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":2505
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((float *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2509
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2511
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":2514
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":2515
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L15:;

            /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2519
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 */
              __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2520
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = di + i
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
              goto __pyx_L16;
            }

            /* "dipy/align/vector_fields.pyx":2522
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 *                     dii = di + i             # <<<<<<<<<<<<<<
 *                     djj = dj + j
 * 
 */
            /*else*/ {
              __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":2523
 *                 else:
 *                     dii = di + i
 *                     djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                 # Interpolate the input image at the resulting location
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
            }
            __pyx_L16:;

            /* "dipy/align/vector_fields.pyx":2527
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
            __pyx_t_27 = __pyx_v_i;
            __pyx_t_28 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2526
 * 
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &warped[i, j])
 *     return warped
 */
            __pyx_fuse_6__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((__pyx_t_double_complex *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_27 * __pyx_v_warped.strides[0]) ) + __pyx_t_28 * __pyx_v_warped.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2528
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_warped, 2, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_double_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_double_complex, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_r = __pyx_t_12;
  __pyx_t_12 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __PYX_XDEC_MEMVIEW(&__pyx_t_13, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_540__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults89, __pyx_self)->__pyx_arg_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults89, __pyx_self)->__pyx_arg_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults89, __pyx_self)->__pyx_arg_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults89, __pyx_self)->__pyx_arg_out_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_4, 1, Py_None);
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_6_1__pyx_pw_4dipy_5align_13vector_fields_307warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_6_1__pyx_mdef_4dipy_5align_13vector_fields_307warp_2d_nn = {"__pyx_fuse_6_1warp_2d_nn", (PyCFunction)__pyx_fuse_6_1__pyx_pw_4dipy_5align_13vector_fields_307warp_2d_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_50warp_2d_nn};
static PyObject *__pyx_fuse_6_1__pyx_pw_4dipy_5align_13vector_fields_307warp_2d_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_d1 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_in = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_idx_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine_disp = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("warp_2d_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_d1,&__pyx_n_s_affine_idx_in,&__pyx_n_s_affine_idx_out,&__pyx_n_s_affine_disp,&__pyx_n_s_out_shape,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    __pyx_defaults89 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults89, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_d1)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_in);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_idx_out);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine_disp);
          if (value) { values[4] = value; kw_args--; }
        }
        case  5:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape);
          if (value) { values[5] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "warp_2d_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_double_complex(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_d1 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[1]); if (unlikely(!__pyx_v_d1.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine_idx_in = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine_idx_in.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_in = __pyx_dynamic_args->__pyx_arg_affine_idx_in;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
    }
    if (values[3]) {
      __pyx_v_affine_idx_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_affine_idx_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_idx_out = __pyx_dynamic_args->__pyx_arg_affine_idx_out;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
    }
    if (values[4]) {
      __pyx_v_affine_disp = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_affine_disp.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine_disp = __pyx_dynamic_args->__pyx_arg_affine_disp;
      __PYX_INC_MEMVIEW(&__pyx_v_affine_disp, 1);
    }
    if (values[5]) {
      __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_out_shape = __pyx_dynamic_args->__pyx_arg_out_shape;
      __PYX_INC_MEMVIEW(&__pyx_v_out_shape, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("warp_2d_nn", 0, 2, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_306warp_2d_nn(__pyx_self, __pyx_v_image, __pyx_v_d1, __pyx_v_affine_idx_in, __pyx_v_affine_idx_out, __pyx_v_affine_disp, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_306warp_2d_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_d1, __Pyx_memviewslice __pyx_v_affine_idx_in, __Pyx_memviewslice __pyx_v_affine_idx_out, __Pyx_memviewslice __pyx_v_affine_disp, __Pyx_memviewslice __pyx_v_out_shape) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_warped = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_tmp = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_memviewslice __pyx_t_13 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_6_1warp_2d_nn", 0);

  /* "dipy/align/vector_fields.pyx":2466
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_v_nrows = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2467
 *     cdef:
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_v_ncols = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2468
 *         cnp.npy_intp nrows = image.shape[0]
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2469
 *         cnp.npy_intp ncols = image.shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double di, dj, dii, djj
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_in, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2473; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__296, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2473
 *         double di, dj, dii, djj
 * 
 *     if not is_valid_affine(affine_idx_in, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_affine_idx_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, __pyx_t_6);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2475; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__297, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2475
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_affine_disp, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_6) {
    __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6); __pyx_t_6 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_5, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_5, __pyx_int_2);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2477; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__298, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2477
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_out_shape.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2481
 * 
 *     if out_shape is not None:
 *         nrows = out_shape[0]             # <<<<<<<<<<<<<<
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 */
    __pyx_t_9 = 0;
    __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_9 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2482
 *     if out_shape is not None:
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]             # <<<<<<<<<<<<<<
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 */
    __pyx_t_10 = 1;
    __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_10 * __pyx_v_out_shape.strides[0]) )));

    /* "dipy/align/vector_fields.pyx":2480
 *         raise ValueError("Invalid displacement multiplication matrix")
 * 
 *     if out_shape is not None:             # <<<<<<<<<<<<<<
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 */
    goto __pyx_L6;
  }

  /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  __pyx_t_8 = ((((PyObject *) __pyx_v_d1.memview) != Py_None) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":2484
 *         ncols = out_shape[1]
 *     elif d1 is not None:
 *         nrows = d1.shape[0]             # <<<<<<<<<<<<<<
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 */
    __pyx_v_nrows = (__pyx_v_d1.shape[0]);

    /* "dipy/align/vector_fields.pyx":2485
 *     elif d1 is not None:
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]             # <<<<<<<<<<<<<<
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 */
    __pyx_v_ncols = (__pyx_v_d1.shape[1]);

    /* "dipy/align/vector_fields.pyx":2483
 *         nrows = out_shape[0]
 *         ncols = out_shape[1]
 *     elif d1 is not None:             # <<<<<<<<<<<<<<
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 */
  }
  __pyx_L6:;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2487
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_double_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_double_complex, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_11 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_11) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_12 = PyTuple_New(1+1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_11); __pyx_t_11 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_12, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2487; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2486
 *         nrows = d1.shape[0]
 *         ncols = d1.shape[1]
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),             # <<<<<<<<<<<<<<
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_13 = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_double_complex(__pyx_t_3);
  if (unlikely(!__pyx_t_13.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2486; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_warped = __pyx_t_13;
  __pyx_t_13.memview = NULL;
  __pyx_t_13.data = NULL;

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__299) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_asarray); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_memoryview_fromslice(__pyx_v_d1, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_6); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_dtype); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_14 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_12);
  if (unlikely(!__pyx_t_14.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_tmp = __pyx_t_14;
  __pyx_t_14.memview = NULL;
  __pyx_t_14.data = NULL;

  /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2492
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2493
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_in.memview) == Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2496
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]             # <<<<<<<<<<<<<<
 *                     djj = d1[i, j, 1]
 *                 else:
 */
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              __pyx_t_21 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_19 * __pyx_v_d1.strides[0]) ) + __pyx_t_20 * __pyx_v_d1.strides[1]) ) + __pyx_t_21 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2497
 *                 if affine_idx_in is None:
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]             # <<<<<<<<<<<<<<
 *                 else:
 *                     di = _apply_affine_2d_x0(
 */
              __pyx_t_22 = __pyx_v_i;
              __pyx_t_23 = __pyx_v_j;
              __pyx_t_24 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d1.data + __pyx_t_22 * __pyx_v_d1.strides[0]) ) + __pyx_t_23 * __pyx_v_d1.strides[1]) ) + __pyx_t_24 * __pyx_v_d1.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":2495
 *             for j in range(ncols):
 *                 # Apply inner index pre-multiplication
 *                 if affine_idx_in is None:             # <<<<<<<<<<<<<<
 *                     dii = d1[i, j, 0]
 *                     djj = d1[i, j, 1]
 */
              goto __pyx_L14;
            }

            /* "dipy/align/vector_fields.pyx":2499
 *                     djj = d1[i, j, 1]
 *                 else:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(
 */
            /*else*/ {

              /* "dipy/align/vector_fields.pyx":2500
 *                 else:
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2501
 *                     di = _apply_affine_2d_x0(
 *                         i, j, 1, affine_idx_in)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_in);

              /* "dipy/align/vector_fields.pyx":2503
 *                     dj = _apply_affine_2d_x1(
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)             # <<<<<<<<<<<<<<
 *                     dii = tmp[0]
 *                     djj = tmp[1]
 */
              __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_d1, __pyx_v_di, __pyx_v_dj, __pyx_v_tmp);

              /* "dipy/align/vector_fields.pyx":2504
 *                         i, j, 1, affine_idx_in)
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]             # <<<<<<<<<<<<<<
 *                     djj = tmp[1]
 * 
 */
              __pyx_t_25 = 0;
              __pyx_v_dii = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_25 * __pyx_v_tmp.strides[0]) )));

              /* "dipy/align/vector_fields.pyx":2505
 *                     _interpolate_vector_2d[floating](d1, di, dj, tmp)
 *                     dii = tmp[0]
 *                     djj = tmp[1]             # <<<<<<<<<<<<<<
 * 
 *                 # Apply displacement multiplication
 */
              __pyx_t_26 = 1;
              __pyx_v_djj = (*((double *) ( /* dim=0 */ (__pyx_v_tmp.data + __pyx_t_26 * __pyx_v_tmp.strides[0]) )));
            }
            __pyx_L14:;

            /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_disp.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2509
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:
 *                     di = _apply_affine_2d_x0(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(
 */
              __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2511
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 *                     dj = _apply_affine_2d_x1(             # <<<<<<<<<<<<<<
 *                         dii, djj, 0, affine_disp)
 *                 else:
 */
              __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_dii, __pyx_v_djj, 0.0, __pyx_v_affine_disp);

              /* "dipy/align/vector_fields.pyx":2508
 * 
 *                 # Apply displacement multiplication
 *                 if affine_disp is not None:             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_2d_x0(
 *                         dii, djj, 0, affine_disp)
 */
              goto __pyx_L15;
            }

            /* "dipy/align/vector_fields.pyx":2514
 *                         dii, djj, 0, affine_disp)
 *                 else:
 *                     di = dii             # <<<<<<<<<<<<<<
 *                     dj = djj
 * 
 */
            /*else*/ {
              __pyx_v_di = __pyx_v_dii;

              /* "dipy/align/vector_fields.pyx":2515
 *                 else:
 *                     di = dii
 *                     dj = djj             # <<<<<<<<<<<<<<
 * 
 *                 # Apply outer index multiplication and add the displacements
 */
              __pyx_v_dj = __pyx_v_djj;
            }
            __pyx_L15:;

            /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
            __pyx_t_8 = ((((PyObject *) __pyx_v_affine_idx_out.memview) != Py_None) != 0);
            if (__pyx_t_8) {

              /* "dipy/align/vector_fields.pyx":2519
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 */
              __pyx_v_dii = (__pyx_v_di + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2520
 *                 if affine_idx_out is not None:
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = di + i
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine_idx_out));

              /* "dipy/align/vector_fields.pyx":2518
 * 
 *                 # Apply outer index multiplication and add the displacements
 *                 if affine_idx_out is not None:             # <<<<<<<<<<<<<<
 *                     dii = di + _apply_affine_2d_x0(i, j, 1, affine_idx_out)
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 */
              goto __pyx_L16;
            }

            /* "dipy/align/vector_fields.pyx":2522
 *                     djj = dj + _apply_affine_2d_x1(i, j, 1, affine_idx_out)
 *                 else:
 *                     dii = di + i             # <<<<<<<<<<<<<<
 *                     djj = dj + j
 * 
 */
            /*else*/ {
              __pyx_v_dii = (__pyx_v_di + __pyx_v_i);

              /* "dipy/align/vector_fields.pyx":2523
 *                 else:
 *                     dii = di + i
 *                     djj = dj + j             # <<<<<<<<<<<<<<
 * 
 *                 # Interpolate the input image at the resulting location
 */
              __pyx_v_djj = (__pyx_v_dj + __pyx_v_j);
            }
            __pyx_L16:;

            /* "dipy/align/vector_fields.pyx":2527
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])             # <<<<<<<<<<<<<<
 *     return warped
 * 
 */
            __pyx_t_27 = __pyx_v_i;
            __pyx_t_28 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2526
 * 
 *                 # Interpolate the input image at the resulting location
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &warped[i, j])
 *     return warped
 */
            __pyx_fuse_6__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((__pyx_t_double_complex *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_warped.data + __pyx_t_27 * __pyx_v_warped.strides[0]) ) + __pyx_t_28 * __pyx_v_warped.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2490
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L9;
        }
        __pyx_L9:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2528
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &warped[i, j])
 *     return warped             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_12 = __pyx_memoryview_fromslice(__pyx_v_warped, 2, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_double_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_double_complex, 0);; if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2528; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_r = __pyx_t_12;
  __pyx_t_12 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __PYX_XDEC_MEMVIEW(&__pyx_t_13, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.warp_2d_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_warped, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_tmp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_d1, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_in, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_idx_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine_disp, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_53transform_2d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_52transform_2d_affine_nn[] = "Transforms a 2D image by an affine transform with NN interpolation\n\n    Deforms the input image under the given affine transformation using\n    nearest neighbor interpolation. The shape of the resulting image\n    is given by ref_shape. If the affine matrix is None, it is taken as the\n    identity.\n\n    Parameters\n    ----------\n    image : array, shape (R, C)\n        the input image to be transformed\n    ref_shape : array, shape (2,)\n        the shape of the resulting image\n    affine : array, shape (3, 3)\n        the affine transform to be applied\n\n    Returns\n    -------\n    out : array, shape (R', C')\n        the transformed image\n\n    Notes\n    -----\n    The reason it is necessary to provide the intended shape of the resulting\n    image is because the affine transformation is defined on all R^{2}\n    but we must sample a finite lattice. Also the resulting shape may not be\n    necessarily equal to the input shape, unless we are interested on\n    endomorphisms only and not general diffeomorphisms.\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_53transform_2d_affine_nn = {"transform_2d_affine_nn", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_53transform_2d_affine_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_52transform_2d_affine_nn};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_53transform_2d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    values[2] = __pyx_k__300;
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs);
          if (value) { values[2] = value; kw_args--; }
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_52transform_2d_affine_nn(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_52transform_2d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  int __pyx_v_short_is_signed;
  int __pyx_v_long_is_signed;
  int __pyx_v_int_is_signed;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("transform_2d_affine_nn", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  __pyx_v_short_is_signed = (((short)-1L) < 0);
  __pyx_v_long_is_signed = (((long)-1L) < 0);
  __pyx_v_int_is_signed = (((int)-1L) < 0);
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_image, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_image); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          __pyx_t_3 = (((sizeof(short)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_short_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_short, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(int)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L26_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L26_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_int_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L26_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_int, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(long)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L30_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L30_bool_binop_done;
          }
          __pyx_t_3 = ((!((__pyx_v_long_is_signed ^ __pyx_v_dtype_signed) != 0)) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L30_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_long, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L34_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L34_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L37_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L37_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          __pyx_t_3 = (((sizeof(__pyx_t_float_complex)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L40_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L40_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_float_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(__pyx_t_double_complex)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L43_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L43_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_double_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L46_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(short))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L46_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_short(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_short, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L50_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(int))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L50_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_int(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_int, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L54_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(long))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L54_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_long(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_long, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L58_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L58_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L62_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L62_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L66_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(__pyx_t_float_complex))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L66_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_float_complex(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_float_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L70_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(__pyx_t_double_complex))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L70_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_double_complex(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_kp_s_double_complex, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__301, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__302, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L77_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L77_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L78_unpacking_done;
        __pyx_L77_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L78_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L80;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L76_break;
        }
        __pyx_L80:;
      }
    }
    __pyx_L76_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__303, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__304, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_556__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults97, __pyx_self)->__pyx_arg_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_311transform_2d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_311transform_2d_affine_nn = {"__pyx_fuse_0transform_2d_affine_nn", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_311transform_2d_affine_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_52transform_2d_affine_nn};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_311transform_2d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_2d_affine_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    __pyx_defaults97 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults97, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_2d_affine_nn", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_2d_affine_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_short(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2532; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine = __pyx_dynamic_args->__pyx_arg_affine;
      __PYX_INC_MEMVIEW(&__pyx_v_affine, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_2d_affine_nn", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_310transform_2d_affine_nn(__pyx_self, __pyx_v_image, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_310transform_2d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  double __pyx_v_dii;
  double __pyx_v_djj;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_memviewslice __pyx_t_11 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0transform_2d_affine_nn", 0);

  /* "dipy/align/vector_fields.pyx":2563
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_t_1 = 0;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2564
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_t_2 = 1;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2565
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         double dii, djj, tmp0
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2566
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         double dii, djj, tmp0
 *         double alpha, beta, calpha, cbeta
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2570
 *         double alpha, beta, calpha, cbeta
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),             # <<<<<<<<<<<<<<
 *                                     dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_5);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_6);
  __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2571
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),
 *                                     dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 2):
 */
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_asarray); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_short, (int (*)(char *, PyObject *)) __pyx_memview_set_short, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
    }
  }
  if (!__pyx_t_9) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_10, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "dipy/align/vector_fields.pyx":2570
 *         double alpha, beta, calpha, cbeta
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),             # <<<<<<<<<<<<<<
 *                                     dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_11 = __Pyx_PyObject_to_MemoryviewSlice_dsds_short(__pyx_t_8);
  if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_out = __pyx_t_11;
  __pyx_t_11.memview = NULL;
  __pyx_t_11.data = NULL;

  /* "dipy/align/vector_fields.pyx":2573
 *                                     dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = NULL;
  __pyx_t_12 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
      __pyx_t_12 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_12, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_12, __pyx_int_2);
  __pyx_t_5 = 0;
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_3, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_13 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = ((!__pyx_t_13) != 0);
  if (__pyx_t_14) {

    /* "dipy/align/vector_fields.pyx":2574
 * 
 *     if not is_valid_affine(affine, 2):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__305, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2573
 *                                     dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2576
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2578
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 if affine is not None:
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2579
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2580
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
            __pyx_t_14 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
            if (__pyx_t_14) {

              /* "dipy/align/vector_fields.pyx":2581
 *             for j in range(ncols):
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 */
              __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2582
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = i
 */
              __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2580
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
              goto __pyx_L11;
            }

            /* "dipy/align/vector_fields.pyx":2584
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 *                     dii = i             # <<<<<<<<<<<<<<
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 */
            /*else*/ {
              __pyx_v_dii = __pyx_v_i;

              /* "dipy/align/vector_fields.pyx":2585
 *                 else:
 *                     dii = i
 *                     djj = j             # <<<<<<<<<<<<<<
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])
 */
              __pyx_v_djj = __pyx_v_j;
            }
            __pyx_L11:;

            /* "dipy/align/vector_fields.pyx":2587
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
            __pyx_t_19 = __pyx_v_i;
            __pyx_t_20 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2586
 *                     dii = i
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &out[i, j])
 *     return out
 */
            __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((short *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_19 * __pyx_v_out.strides[0]) ) + __pyx_t_20 * __pyx_v_out.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2576
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2588
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_8 = __pyx_memoryview_fromslice(__pyx_v_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_short, (int (*)(char *, PyObject *)) __pyx_memview_set_short, 0);; if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_r = __pyx_t_8;
  __pyx_t_8 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __PYX_XDEC_MEMVIEW(&__pyx_t_11, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_558__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults98, __pyx_self)->__pyx_arg_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_313transform_2d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_313transform_2d_affine_nn = {"__pyx_fuse_1transform_2d_affine_nn", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_313transform_2d_affine_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_52transform_2d_affine_nn};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_313transform_2d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_2d_affine_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    __pyx_defaults98 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults98, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_2d_affine_nn", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_2d_affine_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_int(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2532; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine = __pyx_dynamic_args->__pyx_arg_affine;
      __PYX_INC_MEMVIEW(&__pyx_v_affine, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_2d_affine_nn", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_312transform_2d_affine_nn(__pyx_self, __pyx_v_image, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_312transform_2d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  double __pyx_v_dii;
  double __pyx_v_djj;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_memviewslice __pyx_t_11 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1transform_2d_affine_nn", 0);

  /* "dipy/align/vector_fields.pyx":2563
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_t_1 = 0;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2564
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_t_2 = 1;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2565
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         double dii, djj, tmp0
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2566
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         double dii, djj, tmp0
 *         double alpha, beta, calpha, cbeta
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2570
 *         double alpha, beta, calpha, cbeta
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),             # <<<<<<<<<<<<<<
 *                                     dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_5);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_6);
  __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2571
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),
 *                                     dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 2):
 */
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_asarray); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
    }
  }
  if (!__pyx_t_9) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_10, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "dipy/align/vector_fields.pyx":2570
 *         double alpha, beta, calpha, cbeta
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),             # <<<<<<<<<<<<<<
 *                                     dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_11 = __Pyx_PyObject_to_MemoryviewSlice_dsds_int(__pyx_t_8);
  if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_out = __pyx_t_11;
  __pyx_t_11.memview = NULL;
  __pyx_t_11.data = NULL;

  /* "dipy/align/vector_fields.pyx":2573
 *                                     dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = NULL;
  __pyx_t_12 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
      __pyx_t_12 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_12, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_12, __pyx_int_2);
  __pyx_t_5 = 0;
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_3, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_13 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = ((!__pyx_t_13) != 0);
  if (__pyx_t_14) {

    /* "dipy/align/vector_fields.pyx":2574
 * 
 *     if not is_valid_affine(affine, 2):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__306, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2573
 *                                     dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2576
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2578
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 if affine is not None:
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2579
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2580
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
            __pyx_t_14 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
            if (__pyx_t_14) {

              /* "dipy/align/vector_fields.pyx":2581
 *             for j in range(ncols):
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 */
              __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2582
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = i
 */
              __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2580
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
              goto __pyx_L11;
            }

            /* "dipy/align/vector_fields.pyx":2584
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 *                     dii = i             # <<<<<<<<<<<<<<
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 */
            /*else*/ {
              __pyx_v_dii = __pyx_v_i;

              /* "dipy/align/vector_fields.pyx":2585
 *                 else:
 *                     dii = i
 *                     djj = j             # <<<<<<<<<<<<<<
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])
 */
              __pyx_v_djj = __pyx_v_j;
            }
            __pyx_L11:;

            /* "dipy/align/vector_fields.pyx":2587
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
            __pyx_t_19 = __pyx_v_i;
            __pyx_t_20 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2586
 *                     dii = i
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &out[i, j])
 *     return out
 */
            __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_19 * __pyx_v_out.strides[0]) ) + __pyx_t_20 * __pyx_v_out.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2576
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2588
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_8 = __pyx_memoryview_fromslice(__pyx_v_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_r = __pyx_t_8;
  __pyx_t_8 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __PYX_XDEC_MEMVIEW(&__pyx_t_11, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_560__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults99, __pyx_self)->__pyx_arg_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_2__pyx_pw_4dipy_5align_13vector_fields_315transform_2d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_2__pyx_mdef_4dipy_5align_13vector_fields_315transform_2d_affine_nn = {"__pyx_fuse_2transform_2d_affine_nn", (PyCFunction)__pyx_fuse_2__pyx_pw_4dipy_5align_13vector_fields_315transform_2d_affine_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_52transform_2d_affine_nn};
static PyObject *__pyx_fuse_2__pyx_pw_4dipy_5align_13vector_fields_315transform_2d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_2d_affine_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    __pyx_defaults99 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults99, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_2d_affine_nn", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_2d_affine_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_long(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2532; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine = __pyx_dynamic_args->__pyx_arg_affine;
      __PYX_INC_MEMVIEW(&__pyx_v_affine, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_2d_affine_nn", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_314transform_2d_affine_nn(__pyx_self, __pyx_v_image, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_314transform_2d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  double __pyx_v_dii;
  double __pyx_v_djj;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_memviewslice __pyx_t_11 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_2transform_2d_affine_nn", 0);

  /* "dipy/align/vector_fields.pyx":2563
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_t_1 = 0;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2564
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_t_2 = 1;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2565
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         double dii, djj, tmp0
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2566
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         double dii, djj, tmp0
 *         double alpha, beta, calpha, cbeta
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2570
 *         double alpha, beta, calpha, cbeta
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),             # <<<<<<<<<<<<<<
 *                                     dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_5);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_6);
  __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2571
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),
 *                                     dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 2):
 */
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_asarray); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_long, (int (*)(char *, PyObject *)) __pyx_memview_set_long, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
    }
  }
  if (!__pyx_t_9) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_10, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "dipy/align/vector_fields.pyx":2570
 *         double alpha, beta, calpha, cbeta
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),             # <<<<<<<<<<<<<<
 *                                     dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_11 = __Pyx_PyObject_to_MemoryviewSlice_dsds_long(__pyx_t_8);
  if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_out = __pyx_t_11;
  __pyx_t_11.memview = NULL;
  __pyx_t_11.data = NULL;

  /* "dipy/align/vector_fields.pyx":2573
 *                                     dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = NULL;
  __pyx_t_12 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
      __pyx_t_12 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_12, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_12, __pyx_int_2);
  __pyx_t_5 = 0;
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_3, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_13 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = ((!__pyx_t_13) != 0);
  if (__pyx_t_14) {

    /* "dipy/align/vector_fields.pyx":2574
 * 
 *     if not is_valid_affine(affine, 2):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__307, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2573
 *                                     dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2576
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2578
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 if affine is not None:
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2579
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2580
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
            __pyx_t_14 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
            if (__pyx_t_14) {

              /* "dipy/align/vector_fields.pyx":2581
 *             for j in range(ncols):
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 */
              __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2582
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = i
 */
              __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2580
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
              goto __pyx_L11;
            }

            /* "dipy/align/vector_fields.pyx":2584
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 *                     dii = i             # <<<<<<<<<<<<<<
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 */
            /*else*/ {
              __pyx_v_dii = __pyx_v_i;

              /* "dipy/align/vector_fields.pyx":2585
 *                 else:
 *                     dii = i
 *                     djj = j             # <<<<<<<<<<<<<<
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])
 */
              __pyx_v_djj = __pyx_v_j;
            }
            __pyx_L11:;

            /* "dipy/align/vector_fields.pyx":2587
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
            __pyx_t_19 = __pyx_v_i;
            __pyx_t_20 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2586
 *                     dii = i
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &out[i, j])
 *     return out
 */
            __pyx_fuse_2__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((long *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_19 * __pyx_v_out.strides[0]) ) + __pyx_t_20 * __pyx_v_out.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2576
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2588
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_8 = __pyx_memoryview_fromslice(__pyx_v_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_long, (int (*)(char *, PyObject *)) __pyx_memview_set_long, 0);; if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_r = __pyx_t_8;
  __pyx_t_8 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __PYX_XDEC_MEMVIEW(&__pyx_t_11, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_562__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults100, __pyx_self)->__pyx_arg_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_3__pyx_pw_4dipy_5align_13vector_fields_317transform_2d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_3__pyx_mdef_4dipy_5align_13vector_fields_317transform_2d_affine_nn = {"__pyx_fuse_3transform_2d_affine_nn", (PyCFunction)__pyx_fuse_3__pyx_pw_4dipy_5align_13vector_fields_317transform_2d_affine_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_52transform_2d_affine_nn};
static PyObject *__pyx_fuse_3__pyx_pw_4dipy_5align_13vector_fields_317transform_2d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_2d_affine_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    __pyx_defaults100 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults100, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_2d_affine_nn", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_2d_affine_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2532; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine = __pyx_dynamic_args->__pyx_arg_affine;
      __PYX_INC_MEMVIEW(&__pyx_v_affine, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_2d_affine_nn", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_316transform_2d_affine_nn(__pyx_self, __pyx_v_image, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_316transform_2d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  double __pyx_v_dii;
  double __pyx_v_djj;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_memviewslice __pyx_t_11 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_3transform_2d_affine_nn", 0);

  /* "dipy/align/vector_fields.pyx":2563
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_t_1 = 0;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2564
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_t_2 = 1;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2565
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         double dii, djj, tmp0
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2566
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         double dii, djj, tmp0
 *         double alpha, beta, calpha, cbeta
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2570
 *         double alpha, beta, calpha, cbeta
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),             # <<<<<<<<<<<<<<
 *                                     dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_5);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_6);
  __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2571
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),
 *                                     dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 2):
 */
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_asarray); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
    }
  }
  if (!__pyx_t_9) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_10, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "dipy/align/vector_fields.pyx":2570
 *         double alpha, beta, calpha, cbeta
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),             # <<<<<<<<<<<<<<
 *                                     dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_11 = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_t_8);
  if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_out = __pyx_t_11;
  __pyx_t_11.memview = NULL;
  __pyx_t_11.data = NULL;

  /* "dipy/align/vector_fields.pyx":2573
 *                                     dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = NULL;
  __pyx_t_12 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
      __pyx_t_12 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_12, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_12, __pyx_int_2);
  __pyx_t_5 = 0;
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_3, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_13 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = ((!__pyx_t_13) != 0);
  if (__pyx_t_14) {

    /* "dipy/align/vector_fields.pyx":2574
 * 
 *     if not is_valid_affine(affine, 2):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__308, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2573
 *                                     dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2576
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2578
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 if affine is not None:
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2579
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2580
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
            __pyx_t_14 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
            if (__pyx_t_14) {

              /* "dipy/align/vector_fields.pyx":2581
 *             for j in range(ncols):
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 */
              __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2582
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = i
 */
              __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2580
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
              goto __pyx_L11;
            }

            /* "dipy/align/vector_fields.pyx":2584
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 *                     dii = i             # <<<<<<<<<<<<<<
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 */
            /*else*/ {
              __pyx_v_dii = __pyx_v_i;

              /* "dipy/align/vector_fields.pyx":2585
 *                 else:
 *                     dii = i
 *                     djj = j             # <<<<<<<<<<<<<<
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])
 */
              __pyx_v_djj = __pyx_v_j;
            }
            __pyx_L11:;

            /* "dipy/align/vector_fields.pyx":2587
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
            __pyx_t_19 = __pyx_v_i;
            __pyx_t_20 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2586
 *                     dii = i
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &out[i, j])
 *     return out
 */
            __pyx_fuse_3__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_19 * __pyx_v_out.strides[0]) ) + __pyx_t_20 * __pyx_v_out.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2576
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2588
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_8 = __pyx_memoryview_fromslice(__pyx_v_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_r = __pyx_t_8;
  __pyx_t_8 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __PYX_XDEC_MEMVIEW(&__pyx_t_11, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_564__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults101, __pyx_self)->__pyx_arg_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_4__pyx_pw_4dipy_5align_13vector_fields_319transform_2d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_4__pyx_mdef_4dipy_5align_13vector_fields_319transform_2d_affine_nn = {"__pyx_fuse_4transform_2d_affine_nn", (PyCFunction)__pyx_fuse_4__pyx_pw_4dipy_5align_13vector_fields_319transform_2d_affine_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_52transform_2d_affine_nn};
static PyObject *__pyx_fuse_4__pyx_pw_4dipy_5align_13vector_fields_319transform_2d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_2d_affine_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    __pyx_defaults101 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults101, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_2d_affine_nn", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_2d_affine_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2532; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine = __pyx_dynamic_args->__pyx_arg_affine;
      __PYX_INC_MEMVIEW(&__pyx_v_affine, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_2d_affine_nn", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_318transform_2d_affine_nn(__pyx_self, __pyx_v_image, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_318transform_2d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  double __pyx_v_dii;
  double __pyx_v_djj;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_memviewslice __pyx_t_11 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_4transform_2d_affine_nn", 0);

  /* "dipy/align/vector_fields.pyx":2563
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_t_1 = 0;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2564
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_t_2 = 1;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2565
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         double dii, djj, tmp0
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2566
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         double dii, djj, tmp0
 *         double alpha, beta, calpha, cbeta
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2570
 *         double alpha, beta, calpha, cbeta
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),             # <<<<<<<<<<<<<<
 *                                     dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_5);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_6);
  __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2571
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),
 *                                     dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 2):
 */
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_asarray); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
    }
  }
  if (!__pyx_t_9) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_10, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "dipy/align/vector_fields.pyx":2570
 *         double alpha, beta, calpha, cbeta
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),             # <<<<<<<<<<<<<<
 *                                     dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_11 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_t_8);
  if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_out = __pyx_t_11;
  __pyx_t_11.memview = NULL;
  __pyx_t_11.data = NULL;

  /* "dipy/align/vector_fields.pyx":2573
 *                                     dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = NULL;
  __pyx_t_12 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
      __pyx_t_12 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_12, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_12, __pyx_int_2);
  __pyx_t_5 = 0;
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_3, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_13 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = ((!__pyx_t_13) != 0);
  if (__pyx_t_14) {

    /* "dipy/align/vector_fields.pyx":2574
 * 
 *     if not is_valid_affine(affine, 2):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__309, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2573
 *                                     dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2576
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2578
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 if affine is not None:
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2579
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2580
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
            __pyx_t_14 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
            if (__pyx_t_14) {

              /* "dipy/align/vector_fields.pyx":2581
 *             for j in range(ncols):
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 */
              __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2582
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = i
 */
              __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2580
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
              goto __pyx_L11;
            }

            /* "dipy/align/vector_fields.pyx":2584
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 *                     dii = i             # <<<<<<<<<<<<<<
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 */
            /*else*/ {
              __pyx_v_dii = __pyx_v_i;

              /* "dipy/align/vector_fields.pyx":2585
 *                 else:
 *                     dii = i
 *                     djj = j             # <<<<<<<<<<<<<<
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])
 */
              __pyx_v_djj = __pyx_v_j;
            }
            __pyx_L11:;

            /* "dipy/align/vector_fields.pyx":2587
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
            __pyx_t_19 = __pyx_v_i;
            __pyx_t_20 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2586
 *                     dii = i
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &out[i, j])
 *     return out
 */
            __pyx_fuse_4__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_19 * __pyx_v_out.strides[0]) ) + __pyx_t_20 * __pyx_v_out.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2576
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2588
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_8 = __pyx_memoryview_fromslice(__pyx_v_out, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_r = __pyx_t_8;
  __pyx_t_8 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __PYX_XDEC_MEMVIEW(&__pyx_t_11, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_566__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults102, __pyx_self)->__pyx_arg_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_5__pyx_pw_4dipy_5align_13vector_fields_321transform_2d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_5__pyx_mdef_4dipy_5align_13vector_fields_321transform_2d_affine_nn = {"__pyx_fuse_5transform_2d_affine_nn", (PyCFunction)__pyx_fuse_5__pyx_pw_4dipy_5align_13vector_fields_321transform_2d_affine_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_52transform_2d_affine_nn};
static PyObject *__pyx_fuse_5__pyx_pw_4dipy_5align_13vector_fields_321transform_2d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_2d_affine_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    __pyx_defaults102 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults102, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_2d_affine_nn", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_2d_affine_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_float_complex(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2532; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine = __pyx_dynamic_args->__pyx_arg_affine;
      __PYX_INC_MEMVIEW(&__pyx_v_affine, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_2d_affine_nn", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_320transform_2d_affine_nn(__pyx_self, __pyx_v_image, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_320transform_2d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  double __pyx_v_dii;
  double __pyx_v_djj;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_memviewslice __pyx_t_11 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_5transform_2d_affine_nn", 0);

  /* "dipy/align/vector_fields.pyx":2563
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_t_1 = 0;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2564
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_t_2 = 1;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2565
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         double dii, djj, tmp0
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2566
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         double dii, djj, tmp0
 *         double alpha, beta, calpha, cbeta
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2570
 *         double alpha, beta, calpha, cbeta
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),             # <<<<<<<<<<<<<<
 *                                     dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_5);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_6);
  __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2571
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),
 *                                     dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 2):
 */
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_asarray); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_float_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_float_complex, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
    }
  }
  if (!__pyx_t_9) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_10, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "dipy/align/vector_fields.pyx":2570
 *         double alpha, beta, calpha, cbeta
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),             # <<<<<<<<<<<<<<
 *                                     dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_11 = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_float_complex(__pyx_t_8);
  if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_out = __pyx_t_11;
  __pyx_t_11.memview = NULL;
  __pyx_t_11.data = NULL;

  /* "dipy/align/vector_fields.pyx":2573
 *                                     dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = NULL;
  __pyx_t_12 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
      __pyx_t_12 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_12, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_12, __pyx_int_2);
  __pyx_t_5 = 0;
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_3, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_13 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = ((!__pyx_t_13) != 0);
  if (__pyx_t_14) {

    /* "dipy/align/vector_fields.pyx":2574
 * 
 *     if not is_valid_affine(affine, 2):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__310, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2573
 *                                     dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2576
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2578
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 if affine is not None:
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2579
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2580
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
            __pyx_t_14 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
            if (__pyx_t_14) {

              /* "dipy/align/vector_fields.pyx":2581
 *             for j in range(ncols):
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 */
              __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2582
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = i
 */
              __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2580
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
              goto __pyx_L11;
            }

            /* "dipy/align/vector_fields.pyx":2584
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 *                     dii = i             # <<<<<<<<<<<<<<
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 */
            /*else*/ {
              __pyx_v_dii = __pyx_v_i;

              /* "dipy/align/vector_fields.pyx":2585
 *                 else:
 *                     dii = i
 *                     djj = j             # <<<<<<<<<<<<<<
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])
 */
              __pyx_v_djj = __pyx_v_j;
            }
            __pyx_L11:;

            /* "dipy/align/vector_fields.pyx":2587
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
            __pyx_t_19 = __pyx_v_i;
            __pyx_t_20 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2586
 *                     dii = i
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &out[i, j])
 *     return out
 */
            __pyx_fuse_5__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((__pyx_t_float_complex *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_19 * __pyx_v_out.strides[0]) ) + __pyx_t_20 * __pyx_v_out.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2576
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2588
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_8 = __pyx_memoryview_fromslice(__pyx_v_out, 2, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_float_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_float_complex, 0);; if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_r = __pyx_t_8;
  __pyx_t_8 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __PYX_XDEC_MEMVIEW(&__pyx_t_11, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_568__defaults__(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__defaults__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__Pyx_CyFunction_Defaults(__pyx_defaults103, __pyx_self)->__pyx_arg_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_1, 1, Py_None);
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("dipy.align.vector_fields.__defaults__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_6__pyx_pw_4dipy_5align_13vector_fields_323transform_2d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_6__pyx_mdef_4dipy_5align_13vector_fields_323transform_2d_affine_nn = {"__pyx_fuse_6transform_2d_affine_nn", (PyCFunction)__pyx_fuse_6__pyx_pw_4dipy_5align_13vector_fields_323transform_2d_affine_nn, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_52transform_2d_affine_nn};
static PyObject *__pyx_fuse_6__pyx_pw_4dipy_5align_13vector_fields_323transform_2d_affine_nn(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_image = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_ref_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_affine = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("transform_2d_affine_nn (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_image,&__pyx_n_s_ref_shape,&__pyx_n_s_affine,0};
    PyObject* values[3] = {0,0,0};
    __pyx_defaults103 *__pyx_dynamic_args = __Pyx_CyFunction_Defaults(__pyx_defaults103, __pyx_self);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_image)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ref_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("transform_2d_affine_nn", 0, 2, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_affine);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "transform_2d_affine_nn") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_image = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_double_complex(values[0]); if (unlikely(!__pyx_v_image.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ref_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[1]); if (unlikely(!__pyx_v_ref_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_affine = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[2]); if (unlikely(!__pyx_v_affine.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2532; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_affine = __pyx_dynamic_args->__pyx_arg_affine;
      __PYX_INC_MEMVIEW(&__pyx_v_affine, 1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("transform_2d_affine_nn", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_322transform_2d_affine_nn(__pyx_self, __pyx_v_image, __pyx_v_ref_shape, __pyx_v_affine);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_322transform_2d_affine_nn(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_image, __Pyx_memviewslice __pyx_v_ref_shape, __Pyx_memviewslice __pyx_v_affine) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  CYTHON_UNUSED npy_intp __pyx_v_nrVol;
  CYTHON_UNUSED npy_intp __pyx_v_ncVol;
  double __pyx_v_dii;
  double __pyx_v_djj;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_memviewslice __pyx_t_11 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  npy_intp __pyx_t_15;
  npy_intp __pyx_t_16;
  npy_intp __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_6transform_2d_affine_nn", 0);

  /* "dipy/align/vector_fields.pyx":2563
 *     """
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 */
  __pyx_t_1 = 0;
  __pyx_v_nrows = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_1 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2564
 *     cdef:
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]
 */
  __pyx_t_2 = 1;
  __pyx_v_ncols = (*((int *) ( /* dim=0 */ (__pyx_v_ref_shape.data + __pyx_t_2 * __pyx_v_ref_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2565
 *         cnp.npy_intp nrows = ref_shape[0]
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp ncVol = image.shape[1]
 *         double dii, djj, tmp0
 */
  __pyx_v_nrVol = (__pyx_v_image.shape[0]);

  /* "dipy/align/vector_fields.pyx":2566
 *         cnp.npy_intp ncols = ref_shape[1]
 *         cnp.npy_intp nrVol = image.shape[0]
 *         cnp.npy_intp ncVol = image.shape[1]             # <<<<<<<<<<<<<<
 *         double dii, djj, tmp0
 *         double alpha, beta, calpha, cbeta
 */
  __pyx_v_ncVol = (__pyx_v_image.shape[1]);

  /* "dipy/align/vector_fields.pyx":2570
 *         double alpha, beta, calpha, cbeta
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),             # <<<<<<<<<<<<<<
 *                                     dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_5);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_6);
  __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":2571
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),
 *                                     dtype=np.asarray(image).dtype)             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(affine, 2):
 */
  __pyx_t_6 = PyDict_New(); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_asarray); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __pyx_memoryview_fromslice(__pyx_v_image, 2, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_double_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_double_complex, 0);; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
    }
  }
  if (!__pyx_t_9) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9); __pyx_t_9 = NULL;
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_10, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_6, __pyx_n_s_dtype, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "dipy/align/vector_fields.pyx":2570
 *         double alpha, beta, calpha, cbeta
 *         cnp.npy_intp i, j, ii, jj
 *         number[:, :] out = np.zeros((nrows, ncols),             # <<<<<<<<<<<<<<
 *                                     dtype=np.asarray(image).dtype)
 * 
 */
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, __pyx_t_6); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_11 = __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_double_complex(__pyx_t_8);
  if (unlikely(!__pyx_t_11.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_out = __pyx_t_11;
  __pyx_t_11.memview = NULL;
  __pyx_t_11.data = NULL;

  /* "dipy/align/vector_fields.pyx":2573
 *                                     dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_affine, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = NULL;
  __pyx_t_12 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
      __pyx_t_12 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(2+__pyx_t_12); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_12, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_12, __pyx_int_2);
  __pyx_t_5 = 0;
  __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_3, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_13 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2573; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_14 = ((!__pyx_t_13) != 0);
  if (__pyx_t_14) {

    /* "dipy/align/vector_fields.pyx":2574
 * 
 *     if not is_valid_affine(affine, 2):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__311, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2573
 *                                     dtype=np.asarray(image).dtype)
 * 
 *     if not is_valid_affine(affine, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2576
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":2578
 *     with nogil:
 * 
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 if affine is not None:
 */
        __pyx_t_15 = __pyx_v_nrows;
        for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
          __pyx_v_i = __pyx_t_16;

          /* "dipy/align/vector_fields.pyx":2579
 * 
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 */
          __pyx_t_17 = __pyx_v_ncols;
          for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
            __pyx_v_j = __pyx_t_18;

            /* "dipy/align/vector_fields.pyx":2580
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
            __pyx_t_14 = ((((PyObject *) __pyx_v_affine.memview) != Py_None) != 0);
            if (__pyx_t_14) {

              /* "dipy/align/vector_fields.pyx":2581
 *             for j in range(ncols):
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 */
              __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2582
 *                 if affine is not None:
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dii = i
 */
              __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_affine);

              /* "dipy/align/vector_fields.pyx":2580
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 if affine is not None:             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_2d_x0(i, j, 1, affine)
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 */
              goto __pyx_L11;
            }

            /* "dipy/align/vector_fields.pyx":2584
 *                     djj = _apply_affine_2d_x1(i, j, 1, affine)
 *                 else:
 *                     dii = i             # <<<<<<<<<<<<<<
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 */
            /*else*/ {
              __pyx_v_dii = __pyx_v_i;

              /* "dipy/align/vector_fields.pyx":2585
 *                 else:
 *                     dii = i
 *                     djj = j             # <<<<<<<<<<<<<<
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])
 */
              __pyx_v_djj = __pyx_v_j;
            }
            __pyx_L11:;

            /* "dipy/align/vector_fields.pyx":2587
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])             # <<<<<<<<<<<<<<
 *     return out
 * 
 */
            __pyx_t_19 = __pyx_v_i;
            __pyx_t_20 = __pyx_v_j;

            /* "dipy/align/vector_fields.pyx":2586
 *                     dii = i
 *                     djj = j
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,             # <<<<<<<<<<<<<<
 *                                                   &out[i, j])
 *     return out
 */
            __pyx_fuse_6__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_nn_2d(__pyx_v_image, __pyx_v_dii, __pyx_v_djj, (&(*((__pyx_t_double_complex *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_19 * __pyx_v_out.strides[0]) ) + __pyx_t_20 * __pyx_v_out.strides[1]) )))));
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":2576
 *         raise ValueError("Invalid affine transform matrix")
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 * 
 *         for i in range(nrows):
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "dipy/align/vector_fields.pyx":2588
 *                 _interpolate_scalar_nn_2d[number](image, dii, djj,
 *                                                   &out[i, j])
 *     return out             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_8 = __pyx_memoryview_fromslice(__pyx_v_out, 2, (PyObject *(*)(char *)) __pyx_memview_get___pyx_t_double_complex, (int (*)(char *, PyObject *)) __pyx_memview_set___pyx_t_double_complex, 0);; if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_r = __pyx_t_8;
  __pyx_t_8 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __PYX_XDEC_MEMVIEW(&__pyx_t_11, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.transform_2d_affine_nn", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_image, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_ref_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_affine, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":2591
 * 
 * 
 * def resample_displacement_field_3d(floating[:, :, :, :] field,             # <<<<<<<<<<<<<<
 *                                    double[:] factors, int[:] out_shape):
 *     r"""Resamples a 3D vector field to a custom target shape
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_55resample_displacement_field_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_54resample_displacement_field_3d[] = "Resamples a 3D vector field to a custom target shape\n\n    Resamples the given 3D displacement field on a grid of the requested shape,\n    using the given scale factors. More precisely, the resulting displacement\n    field at each grid cell i is given by\n\n    D[i] = field[Diag(factors) * i]\n\n    Parameters\n    ----------\n    factors : array, shape (3,)\n        the scaling factors mapping (integer) grid coordinates in the resampled\n        grid to (floating point) grid coordinates in the original grid\n    out_shape : array, shape (3,)\n        the desired shape of the resulting grid\n\n    Returns\n    -------\n    expanded : array, shape = out_shape + (3, )\n        the resampled displacement field\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_55resample_displacement_field_3d = {"resample_displacement_field_3d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_55resample_displacement_field_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_54resample_displacement_field_3d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_55resample_displacement_field_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_54resample_displacement_field_3d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_54resample_displacement_field_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("resample_displacement_field_3d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_field, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_field); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 4) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 4) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__312, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__313, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__314, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__315, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_327resample_displacement_field_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_327resample_displacement_field_3d = {"__pyx_fuse_0resample_displacement_field_3d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_327resample_displacement_field_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_54resample_displacement_field_3d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_327resample_displacement_field_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_factors = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("resample_displacement_field_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_field,&__pyx_n_s_factors,&__pyx_n_s_out_shape,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_field)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_factors)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("resample_displacement_field_3d", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("resample_displacement_field_3d", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "resample_displacement_field_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_field = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[0]); if (unlikely(!__pyx_v_field.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_factors = __Pyx_PyObject_to_MemoryviewSlice_ds_double(values[1]); if (unlikely(!__pyx_v_factors.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2592; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[2]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2592; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("resample_displacement_field_3d", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.resample_displacement_field_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_326resample_displacement_field_3d(__pyx_self, __pyx_v_field, __pyx_v_factors, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_326resample_displacement_field_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field, __Pyx_memviewslice __pyx_v_factors, __Pyx_memviewslice __pyx_v_out_shape) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_tslices;
  npy_intp __pyx_v_trows;
  npy_intp __pyx_v_tcols;
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_expanded = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  __Pyx_memviewslice __pyx_t_9 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  npy_intp __pyx_t_12;
  npy_intp __pyx_t_13;
  npy_intp __pyx_t_14;
  npy_intp __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  __Pyx_memviewslice __pyx_t_19 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0resample_displacement_field_3d", 0);

  /* "dipy/align/vector_fields.pyx":2614
 *         the resampled displacement field
 *     """
 *     ftype = np.asarray(field).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp tslices = out_shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_field, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2616
 *     ftype = np.asarray(field).dtype
 *     cdef:
 *         cnp.npy_intp tslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp trows = out_shape[1]
 *         cnp.npy_intp tcols = out_shape[2]
 */
  __pyx_t_6 = 0;
  __pyx_v_tslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_6 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2617
 *     cdef:
 *         cnp.npy_intp tslices = out_shape[0]
 *         cnp.npy_intp trows = out_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp tcols = out_shape[2]
 *         cnp.npy_intp k, i, j
 */
  __pyx_t_7 = 1;
  __pyx_v_trows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_7 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2618
 *         cnp.npy_intp tslices = out_shape[0]
 *         cnp.npy_intp trows = out_shape[1]
 *         cnp.npy_intp tcols = out_shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp k, i, j
 *         int inside
 */
  __pyx_t_8 = 2;
  __pyx_v_tcols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_8 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2622
 *         int inside
 *         double dkk, dii, djj
 *         floating[:, :, :, :] expanded = np.zeros((tslices, trows, tcols, 3),             # <<<<<<<<<<<<<<
 *                                                  dtype=ftype)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_tslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_trows); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_tcols); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_2);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4);
  __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2623
 *         double dkk, dii, djj
 *         floating[:, :, :, :] expanded = np.zeros((tslices, trows, tcols, 3),
 *                                                  dtype=ftype)             # <<<<<<<<<<<<<<
 * 
 *     for k in range(tslices):
 */
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2623; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2623; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2622
 *         int inside
 *         double dkk, dii, djj
 *         floating[:, :, :, :] expanded = np.zeros((tslices, trows, tcols, 3),             # <<<<<<<<<<<<<<
 *                                                  dtype=ftype)
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_9 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(__pyx_t_5);
  if (unlikely(!__pyx_t_9.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_expanded = __pyx_t_9;
  __pyx_t_9.memview = NULL;
  __pyx_t_9.data = NULL;

  /* "dipy/align/vector_fields.pyx":2625
 *                                                  dtype=ftype)
 * 
 *     for k in range(tslices):             # <<<<<<<<<<<<<<
 *         for i in range(trows):
 *             for j in range(tcols):
 */
  __pyx_t_10 = __pyx_v_tslices;
  for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
    __pyx_v_k = __pyx_t_11;

    /* "dipy/align/vector_fields.pyx":2626
 * 
 *     for k in range(tslices):
 *         for i in range(trows):             # <<<<<<<<<<<<<<
 *             for j in range(tcols):
 *                 dkk = <double> k * factors[0]
 */
    __pyx_t_12 = __pyx_v_trows;
    for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
      __pyx_v_i = __pyx_t_13;

      /* "dipy/align/vector_fields.pyx":2627
 *     for k in range(tslices):
 *         for i in range(trows):
 *             for j in range(tcols):             # <<<<<<<<<<<<<<
 *                 dkk = <double> k * factors[0]
 *                 dii = <double> i * factors[1]
 */
      __pyx_t_14 = __pyx_v_tcols;
      for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
        __pyx_v_j = __pyx_t_15;

        /* "dipy/align/vector_fields.pyx":2628
 *         for i in range(trows):
 *             for j in range(tcols):
 *                 dkk = <double> k * factors[0]             # <<<<<<<<<<<<<<
 *                 dii = <double> i * factors[1]
 *                 djj = <double> j * factors[2]
 */
        __pyx_t_16 = 0;
        __pyx_v_dkk = (((double)__pyx_v_k) * (*((double *) ( /* dim=0 */ (__pyx_v_factors.data + __pyx_t_16 * __pyx_v_factors.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":2629
 *             for j in range(tcols):
 *                 dkk = <double> k * factors[0]
 *                 dii = <double> i * factors[1]             # <<<<<<<<<<<<<<
 *                 djj = <double> j * factors[2]
 *                 _interpolate_vector_3d[floating](field, dkk, dii, djj,
 */
        __pyx_t_17 = 1;
        __pyx_v_dii = (((double)__pyx_v_i) * (*((double *) ( /* dim=0 */ (__pyx_v_factors.data + __pyx_t_17 * __pyx_v_factors.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":2630
 *                 dkk = <double> k * factors[0]
 *                 dii = <double> i * factors[1]
 *                 djj = <double> j * factors[2]             # <<<<<<<<<<<<<<
 *                 _interpolate_vector_3d[floating](field, dkk, dii, djj,
 *                                                  expanded[k, i, j])
 */
        __pyx_t_18 = 2;
        __pyx_v_djj = (((double)__pyx_v_j) * (*((double *) ( /* dim=0 */ (__pyx_v_factors.data + __pyx_t_18 * __pyx_v_factors.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":2632
 *                 djj = <double> j * factors[2]
 *                 _interpolate_vector_3d[floating](field, dkk, dii, djj,
 *                                                  expanded[k, i, j])             # <<<<<<<<<<<<<<
 *     return expanded
 * 
 */
        __pyx_t_19.data = __pyx_v_expanded.data;
        __pyx_t_19.memview = __pyx_v_expanded.memview;
        __PYX_INC_MEMVIEW(&__pyx_t_19, 0);
        {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_k;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_expanded.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_expanded.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2632; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_19.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_expanded.shape[1];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_expanded.strides[1];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 1)");
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2632; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_19.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_j;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_expanded.shape[2];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_expanded.strides[2];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 2)");
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2632; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_19.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_19.shape[0] = __pyx_v_expanded.shape[3];
__pyx_t_19.strides[0] = __pyx_v_expanded.strides[3];
    __pyx_t_19.suboffsets[0] = -1;

__pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_field, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, __pyx_t_19);

        /* "dipy/align/vector_fields.pyx":2631
 *                 dii = <double> i * factors[1]
 *                 djj = <double> j * factors[2]
 *                 _interpolate_vector_3d[floating](field, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                  expanded[k, i, j])
 *     return expanded
 */
        __PYX_XDEC_MEMVIEW(&__pyx_t_19, 1);
      }
    }
  }

  /* "dipy/align/vector_fields.pyx":2633
 *                 _interpolate_vector_3d[floating](field, dkk, dii, djj,
 *                                                  expanded[k, i, j])
 *     return expanded             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_expanded, 4, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2633; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2591
 * 
 * 
 * def resample_displacement_field_3d(floating[:, :, :, :] field,             # <<<<<<<<<<<<<<
 *                                    double[:] factors, int[:] out_shape):
 *     r"""Resamples a 3D vector field to a custom target shape
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_9, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_19, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.resample_displacement_field_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_expanded, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_field, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_factors, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_329resample_displacement_field_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_329resample_displacement_field_3d = {"__pyx_fuse_1resample_displacement_field_3d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_329resample_displacement_field_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_54resample_displacement_field_3d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_329resample_displacement_field_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_factors = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("resample_displacement_field_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_field,&__pyx_n_s_factors,&__pyx_n_s_out_shape,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_field)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_factors)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("resample_displacement_field_3d", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("resample_displacement_field_3d", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "resample_displacement_field_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_field = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[0]); if (unlikely(!__pyx_v_field.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_factors = __Pyx_PyObject_to_MemoryviewSlice_ds_double(values[1]); if (unlikely(!__pyx_v_factors.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2592; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[2]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2592; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("resample_displacement_field_3d", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.resample_displacement_field_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_328resample_displacement_field_3d(__pyx_self, __pyx_v_field, __pyx_v_factors, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_328resample_displacement_field_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field, __Pyx_memviewslice __pyx_v_factors, __Pyx_memviewslice __pyx_v_out_shape) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_tslices;
  npy_intp __pyx_v_trows;
  npy_intp __pyx_v_tcols;
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  double __pyx_v_dkk;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_expanded = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  __Pyx_memviewslice __pyx_t_9 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  npy_intp __pyx_t_12;
  npy_intp __pyx_t_13;
  npy_intp __pyx_t_14;
  npy_intp __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  __Pyx_memviewslice __pyx_t_19 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1resample_displacement_field_3d", 0);

  /* "dipy/align/vector_fields.pyx":2614
 *         the resampled displacement field
 *     """
 *     ftype = np.asarray(field).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp tslices = out_shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_field, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2614; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2616
 *     ftype = np.asarray(field).dtype
 *     cdef:
 *         cnp.npy_intp tslices = out_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp trows = out_shape[1]
 *         cnp.npy_intp tcols = out_shape[2]
 */
  __pyx_t_6 = 0;
  __pyx_v_tslices = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_6 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2617
 *     cdef:
 *         cnp.npy_intp tslices = out_shape[0]
 *         cnp.npy_intp trows = out_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp tcols = out_shape[2]
 *         cnp.npy_intp k, i, j
 */
  __pyx_t_7 = 1;
  __pyx_v_trows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_7 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2618
 *         cnp.npy_intp tslices = out_shape[0]
 *         cnp.npy_intp trows = out_shape[1]
 *         cnp.npy_intp tcols = out_shape[2]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp k, i, j
 *         int inside
 */
  __pyx_t_8 = 2;
  __pyx_v_tcols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_8 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2622
 *         int inside
 *         double dkk, dii, djj
 *         floating[:, :, :, :] expanded = np.zeros((tslices, trows, tcols, 3),             # <<<<<<<<<<<<<<
 *                                                  dtype=ftype)
 * 
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_tslices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_trows); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_tcols); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_2);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4);
  __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2623
 *         double dkk, dii, djj
 *         floating[:, :, :, :] expanded = np.zeros((tslices, trows, tcols, 3),
 *                                                  dtype=ftype)             # <<<<<<<<<<<<<<
 * 
 *     for k in range(tslices):
 */
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2623; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2623; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2622
 *         int inside
 *         double dkk, dii, djj
 *         floating[:, :, :, :] expanded = np.zeros((tslices, trows, tcols, 3),             # <<<<<<<<<<<<<<
 *                                                  dtype=ftype)
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_9 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_t_5);
  if (unlikely(!__pyx_t_9.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2622; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_expanded = __pyx_t_9;
  __pyx_t_9.memview = NULL;
  __pyx_t_9.data = NULL;

  /* "dipy/align/vector_fields.pyx":2625
 *                                                  dtype=ftype)
 * 
 *     for k in range(tslices):             # <<<<<<<<<<<<<<
 *         for i in range(trows):
 *             for j in range(tcols):
 */
  __pyx_t_10 = __pyx_v_tslices;
  for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
    __pyx_v_k = __pyx_t_11;

    /* "dipy/align/vector_fields.pyx":2626
 * 
 *     for k in range(tslices):
 *         for i in range(trows):             # <<<<<<<<<<<<<<
 *             for j in range(tcols):
 *                 dkk = <double> k * factors[0]
 */
    __pyx_t_12 = __pyx_v_trows;
    for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
      __pyx_v_i = __pyx_t_13;

      /* "dipy/align/vector_fields.pyx":2627
 *     for k in range(tslices):
 *         for i in range(trows):
 *             for j in range(tcols):             # <<<<<<<<<<<<<<
 *                 dkk = <double> k * factors[0]
 *                 dii = <double> i * factors[1]
 */
      __pyx_t_14 = __pyx_v_tcols;
      for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
        __pyx_v_j = __pyx_t_15;

        /* "dipy/align/vector_fields.pyx":2628
 *         for i in range(trows):
 *             for j in range(tcols):
 *                 dkk = <double> k * factors[0]             # <<<<<<<<<<<<<<
 *                 dii = <double> i * factors[1]
 *                 djj = <double> j * factors[2]
 */
        __pyx_t_16 = 0;
        __pyx_v_dkk = (((double)__pyx_v_k) * (*((double *) ( /* dim=0 */ (__pyx_v_factors.data + __pyx_t_16 * __pyx_v_factors.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":2629
 *             for j in range(tcols):
 *                 dkk = <double> k * factors[0]
 *                 dii = <double> i * factors[1]             # <<<<<<<<<<<<<<
 *                 djj = <double> j * factors[2]
 *                 _interpolate_vector_3d[floating](field, dkk, dii, djj,
 */
        __pyx_t_17 = 1;
        __pyx_v_dii = (((double)__pyx_v_i) * (*((double *) ( /* dim=0 */ (__pyx_v_factors.data + __pyx_t_17 * __pyx_v_factors.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":2630
 *                 dkk = <double> k * factors[0]
 *                 dii = <double> i * factors[1]
 *                 djj = <double> j * factors[2]             # <<<<<<<<<<<<<<
 *                 _interpolate_vector_3d[floating](field, dkk, dii, djj,
 *                                                  expanded[k, i, j])
 */
        __pyx_t_18 = 2;
        __pyx_v_djj = (((double)__pyx_v_j) * (*((double *) ( /* dim=0 */ (__pyx_v_factors.data + __pyx_t_18 * __pyx_v_factors.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":2632
 *                 djj = <double> j * factors[2]
 *                 _interpolate_vector_3d[floating](field, dkk, dii, djj,
 *                                                  expanded[k, i, j])             # <<<<<<<<<<<<<<
 *     return expanded
 * 
 */
        __pyx_t_19.data = __pyx_v_expanded.data;
        __pyx_t_19.memview = __pyx_v_expanded.memview;
        __PYX_INC_MEMVIEW(&__pyx_t_19, 0);
        {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_k;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_expanded.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_expanded.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2632; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_19.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_expanded.shape[1];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_expanded.strides[1];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 1)");
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2632; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_19.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_j;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_expanded.shape[2];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_expanded.strides[2];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 2)");
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2632; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_19.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_19.shape[0] = __pyx_v_expanded.shape[3];
__pyx_t_19.strides[0] = __pyx_v_expanded.strides[3];
    __pyx_t_19.suboffsets[0] = -1;

__pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_3d(__pyx_v_field, __pyx_v_dkk, __pyx_v_dii, __pyx_v_djj, __pyx_t_19);

        /* "dipy/align/vector_fields.pyx":2631
 *                 dii = <double> i * factors[1]
 *                 djj = <double> j * factors[2]
 *                 _interpolate_vector_3d[floating](field, dkk, dii, djj,             # <<<<<<<<<<<<<<
 *                                                  expanded[k, i, j])
 *     return expanded
 */
        __PYX_XDEC_MEMVIEW(&__pyx_t_19, 1);
      }
    }
  }

  /* "dipy/align/vector_fields.pyx":2633
 *                 _interpolate_vector_3d[floating](field, dkk, dii, djj,
 *                                                  expanded[k, i, j])
 *     return expanded             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_expanded, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2633; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2591
 * 
 * 
 * def resample_displacement_field_3d(floating[:, :, :, :] field,             # <<<<<<<<<<<<<<
 *                                    double[:] factors, int[:] out_shape):
 *     r"""Resamples a 3D vector field to a custom target shape
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_9, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_19, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.resample_displacement_field_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_expanded, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_field, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_factors, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":2636
 * 
 * 
 * def resample_displacement_field_2d(floating[:, :, :] field, double[:] factors,             # <<<<<<<<<<<<<<
 *                                    int[:] out_shape):
 *     r"""Resamples a 2D vector field to a custom target shape
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_57resample_displacement_field_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_56resample_displacement_field_2d[] = "Resamples a 2D vector field to a custom target shape\n\n    Resamples the given 2D displacement field on a grid of the requested shape,\n    using the given scale factors. More precisely, the resulting displacement\n    field at each grid cell i is given by\n\n    D[i] = field[Diag(factors) * i]\n\n    Parameters\n    ----------\n    factors : array, shape (2,)\n        the scaling factors mapping (integer) grid coordinates in the resampled\n        grid to (floating point) grid coordinates in the original grid\n    out_shape : array, shape (2,)\n        the desired shape of the resulting grid\n\n    Returns\n    -------\n    expanded : array, shape = out_shape + (2, )\n        the resampled displacement field\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_57resample_displacement_field_2d = {"resample_displacement_field_2d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_57resample_displacement_field_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_56resample_displacement_field_2d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_57resample_displacement_field_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_56resample_displacement_field_2d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_56resample_displacement_field_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("resample_displacement_field_2d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_field, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_field); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__316, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__317, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__318, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__319, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_333resample_displacement_field_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_333resample_displacement_field_2d = {"__pyx_fuse_0resample_displacement_field_2d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_333resample_displacement_field_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_56resample_displacement_field_2d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_333resample_displacement_field_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_factors = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("resample_displacement_field_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_field,&__pyx_n_s_factors,&__pyx_n_s_out_shape,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_field)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_factors)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("resample_displacement_field_2d", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("resample_displacement_field_2d", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "resample_displacement_field_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_field = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[0]); if (unlikely(!__pyx_v_field.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_factors = __Pyx_PyObject_to_MemoryviewSlice_ds_double(values[1]); if (unlikely(!__pyx_v_factors.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[2]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2637; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("resample_displacement_field_2d", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.resample_displacement_field_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_332resample_displacement_field_2d(__pyx_self, __pyx_v_field, __pyx_v_factors, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_332resample_displacement_field_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field, __Pyx_memviewslice __pyx_v_factors, __Pyx_memviewslice __pyx_v_out_shape) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_trows;
  npy_intp __pyx_v_tcols;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_expanded = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  __Pyx_memviewslice __pyx_t_8 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  npy_intp __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0resample_displacement_field_2d", 0);

  /* "dipy/align/vector_fields.pyx":2659
 *         the resampled displacement field
 *     """
 *     ftype = np.asarray(field).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp trows = out_shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2659; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2659; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_field, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2659; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2659; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2659; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2659; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2659; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2661
 *     ftype = np.asarray(field).dtype
 *     cdef:
 *         cnp.npy_intp trows = out_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp tcols = out_shape[1]
 *         cnp.npy_intp i, j
 */
  __pyx_t_6 = 0;
  __pyx_v_trows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_6 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2662
 *     cdef:
 *         cnp.npy_intp trows = out_shape[0]
 *         cnp.npy_intp tcols = out_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j
 *         int inside
 */
  __pyx_t_7 = 1;
  __pyx_v_tcols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_7 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2666
 *         int inside
 *         double dii, djj
 *         floating[:, :, :] expanded = np.zeros((trows, tcols, 2), dtype=ftype)             # <<<<<<<<<<<<<<
 * 
 *     for i in range(trows):
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_trows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_tcols); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_t_3);
  if (unlikely(!__pyx_t_8.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_expanded = __pyx_t_8;
  __pyx_t_8.memview = NULL;
  __pyx_t_8.data = NULL;

  /* "dipy/align/vector_fields.pyx":2668
 *         floating[:, :, :] expanded = np.zeros((trows, tcols, 2), dtype=ftype)
 * 
 *     for i in range(trows):             # <<<<<<<<<<<<<<
 *         for j in range(tcols):
 *             dii = i*factors[0]
 */
  __pyx_t_9 = __pyx_v_trows;
  for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
    __pyx_v_i = __pyx_t_10;

    /* "dipy/align/vector_fields.pyx":2669
 * 
 *     for i in range(trows):
 *         for j in range(tcols):             # <<<<<<<<<<<<<<
 *             dii = i*factors[0]
 *             djj = j*factors[1]
 */
    __pyx_t_11 = __pyx_v_tcols;
    for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
      __pyx_v_j = __pyx_t_12;

      /* "dipy/align/vector_fields.pyx":2670
 *     for i in range(trows):
 *         for j in range(tcols):
 *             dii = i*factors[0]             # <<<<<<<<<<<<<<
 *             djj = j*factors[1]
 *             inside = _interpolate_vector_2d[floating](field, dii, djj,
 */
      __pyx_t_13 = 0;
      __pyx_v_dii = (__pyx_v_i * (*((double *) ( /* dim=0 */ (__pyx_v_factors.data + __pyx_t_13 * __pyx_v_factors.strides[0]) ))));

      /* "dipy/align/vector_fields.pyx":2671
 *         for j in range(tcols):
 *             dii = i*factors[0]
 *             djj = j*factors[1]             # <<<<<<<<<<<<<<
 *             inside = _interpolate_vector_2d[floating](field, dii, djj,
 *                                                       expanded[i, j])
 */
      __pyx_t_14 = 1;
      __pyx_v_djj = (__pyx_v_j * (*((double *) ( /* dim=0 */ (__pyx_v_factors.data + __pyx_t_14 * __pyx_v_factors.strides[0]) ))));

      /* "dipy/align/vector_fields.pyx":2673
 *             djj = j*factors[1]
 *             inside = _interpolate_vector_2d[floating](field, dii, djj,
 *                                                       expanded[i, j])             # <<<<<<<<<<<<<<
 *     return expanded
 * 
 */
      __pyx_t_15.data = __pyx_v_expanded.data;
      __pyx_t_15.memview = __pyx_v_expanded.memview;
      __PYX_INC_MEMVIEW(&__pyx_t_15, 0);
      {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_expanded.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_expanded.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2673; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_15.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_j;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_expanded.shape[1];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_expanded.strides[1];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 1)");
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2673; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_15.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_15.shape[0] = __pyx_v_expanded.shape[2];
__pyx_t_15.strides[0] = __pyx_v_expanded.strides[2];
    __pyx_t_15.suboffsets[0] = -1;

__pyx_v_inside = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_field, __pyx_v_dii, __pyx_v_djj, __pyx_t_15);

      /* "dipy/align/vector_fields.pyx":2672
 *             dii = i*factors[0]
 *             djj = j*factors[1]
 *             inside = _interpolate_vector_2d[floating](field, dii, djj,             # <<<<<<<<<<<<<<
 *                                                       expanded[i, j])
 *     return expanded
 */
      __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
    }
  }

  /* "dipy/align/vector_fields.pyx":2674
 *             inside = _interpolate_vector_2d[floating](field, dii, djj,
 *                                                       expanded[i, j])
 *     return expanded             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_expanded, 3, (PyObject *(*)(char *)) __pyx_memview_get_float, (int (*)(char *, PyObject *)) __pyx_memview_set_float, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2674; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2636
 * 
 * 
 * def resample_displacement_field_2d(floating[:, :, :] field, double[:] factors,             # <<<<<<<<<<<<<<
 *                                    int[:] out_shape):
 *     r"""Resamples a 2D vector field to a custom target shape
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_8, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.resample_displacement_field_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_expanded, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_field, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_factors, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_335resample_displacement_field_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_335resample_displacement_field_2d = {"__pyx_fuse_1resample_displacement_field_2d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_335resample_displacement_field_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_56resample_displacement_field_2d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_335resample_displacement_field_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_factors = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("resample_displacement_field_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_field,&__pyx_n_s_factors,&__pyx_n_s_out_shape,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_field)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_factors)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("resample_displacement_field_2d", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("resample_displacement_field_2d", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "resample_displacement_field_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_field = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[0]); if (unlikely(!__pyx_v_field.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_factors = __Pyx_PyObject_to_MemoryviewSlice_ds_double(values[1]); if (unlikely(!__pyx_v_factors.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[2]); if (unlikely(!__pyx_v_out_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2637; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("resample_displacement_field_2d", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.resample_displacement_field_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_334resample_displacement_field_2d(__pyx_self, __pyx_v_field, __pyx_v_factors, __pyx_v_out_shape);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_334resample_displacement_field_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_field, __Pyx_memviewslice __pyx_v_factors, __Pyx_memviewslice __pyx_v_out_shape) {
  PyObject *__pyx_v_ftype = NULL;
  npy_intp __pyx_v_trows;
  npy_intp __pyx_v_tcols;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  CYTHON_UNUSED int __pyx_v_inside;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_expanded = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  __Pyx_memviewslice __pyx_t_8 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  npy_intp __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  __Pyx_memviewslice __pyx_t_15 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1resample_displacement_field_2d", 0);

  /* "dipy/align/vector_fields.pyx":2659
 *         the resampled displacement field
 *     """
 *     ftype = np.asarray(field).dtype             # <<<<<<<<<<<<<<
 *     cdef:
 *         cnp.npy_intp trows = out_shape[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2659; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_asarray); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2659; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_field, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2659; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2659; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2659; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2659; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_dtype); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2659; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2661
 *     ftype = np.asarray(field).dtype
 *     cdef:
 *         cnp.npy_intp trows = out_shape[0]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp tcols = out_shape[1]
 *         cnp.npy_intp i, j
 */
  __pyx_t_6 = 0;
  __pyx_v_trows = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_6 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2662
 *     cdef:
 *         cnp.npy_intp trows = out_shape[0]
 *         cnp.npy_intp tcols = out_shape[1]             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j
 *         int inside
 */
  __pyx_t_7 = 1;
  __pyx_v_tcols = (*((int *) ( /* dim=0 */ (__pyx_v_out_shape.data + __pyx_t_7 * __pyx_v_out_shape.strides[0]) )));

  /* "dipy/align/vector_fields.pyx":2666
 *         int inside
 *         double dii, djj
 *         floating[:, :, :] expanded = np.zeros((trows, tcols, 2), dtype=ftype)             # <<<<<<<<<<<<<<
 * 
 *     for i in range(trows):
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_trows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_tcols); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_5);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_5, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_3);
  if (unlikely(!__pyx_t_8.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2666; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_expanded = __pyx_t_8;
  __pyx_t_8.memview = NULL;
  __pyx_t_8.data = NULL;

  /* "dipy/align/vector_fields.pyx":2668
 *         floating[:, :, :] expanded = np.zeros((trows, tcols, 2), dtype=ftype)
 * 
 *     for i in range(trows):             # <<<<<<<<<<<<<<
 *         for j in range(tcols):
 *             dii = i*factors[0]
 */
  __pyx_t_9 = __pyx_v_trows;
  for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
    __pyx_v_i = __pyx_t_10;

    /* "dipy/align/vector_fields.pyx":2669
 * 
 *     for i in range(trows):
 *         for j in range(tcols):             # <<<<<<<<<<<<<<
 *             dii = i*factors[0]
 *             djj = j*factors[1]
 */
    __pyx_t_11 = __pyx_v_tcols;
    for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
      __pyx_v_j = __pyx_t_12;

      /* "dipy/align/vector_fields.pyx":2670
 *     for i in range(trows):
 *         for j in range(tcols):
 *             dii = i*factors[0]             # <<<<<<<<<<<<<<
 *             djj = j*factors[1]
 *             inside = _interpolate_vector_2d[floating](field, dii, djj,
 */
      __pyx_t_13 = 0;
      __pyx_v_dii = (__pyx_v_i * (*((double *) ( /* dim=0 */ (__pyx_v_factors.data + __pyx_t_13 * __pyx_v_factors.strides[0]) ))));

      /* "dipy/align/vector_fields.pyx":2671
 *         for j in range(tcols):
 *             dii = i*factors[0]
 *             djj = j*factors[1]             # <<<<<<<<<<<<<<
 *             inside = _interpolate_vector_2d[floating](field, dii, djj,
 *                                                       expanded[i, j])
 */
      __pyx_t_14 = 1;
      __pyx_v_djj = (__pyx_v_j * (*((double *) ( /* dim=0 */ (__pyx_v_factors.data + __pyx_t_14 * __pyx_v_factors.strides[0]) ))));

      /* "dipy/align/vector_fields.pyx":2673
 *             djj = j*factors[1]
 *             inside = _interpolate_vector_2d[floating](field, dii, djj,
 *                                                       expanded[i, j])             # <<<<<<<<<<<<<<
 *     return expanded
 * 
 */
      __pyx_t_15.data = __pyx_v_expanded.data;
      __pyx_t_15.memview = __pyx_v_expanded.memview;
      __PYX_INC_MEMVIEW(&__pyx_t_15, 0);
      {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_expanded.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_expanded.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2673; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_15.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

{
    Py_ssize_t __pyx_tmp_idx = __pyx_v_j;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_expanded.shape[1];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_expanded.strides[1];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 1)");
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2673; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
        __pyx_t_15.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_15.shape[0] = __pyx_v_expanded.shape[2];
__pyx_t_15.strides[0] = __pyx_v_expanded.strides[2];
    __pyx_t_15.suboffsets[0] = -1;

__pyx_v_inside = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_vector_2d(__pyx_v_field, __pyx_v_dii, __pyx_v_djj, __pyx_t_15);

      /* "dipy/align/vector_fields.pyx":2672
 *             dii = i*factors[0]
 *             djj = j*factors[1]
 *             inside = _interpolate_vector_2d[floating](field, dii, djj,             # <<<<<<<<<<<<<<
 *                                                       expanded[i, j])
 *     return expanded
 */
      __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
    }
  }

  /* "dipy/align/vector_fields.pyx":2674
 *             inside = _interpolate_vector_2d[floating](field, dii, djj,
 *                                                       expanded[i, j])
 *     return expanded             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_expanded, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2674; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2636
 * 
 * 
 * def resample_displacement_field_2d(floating[:, :, :] field, double[:] factors,             # <<<<<<<<<<<<<<
 *                                    int[:] out_shape):
 *     r"""Resamples a 2D vector field to a custom target shape
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_8, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_15, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.resample_displacement_field_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __PYX_XDEC_MEMVIEW(&__pyx_v_expanded, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_field, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_factors, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_shape, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":2677
 * 
 * 
 * def create_random_displacement_2d(int[:] from_shape,             # <<<<<<<<<<<<<<
 *                                   double[:, :] from_grid2world,
 *                                   int[:] to_shape,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_59create_random_displacement_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_58create_random_displacement_2d[] = "Creates a random 2D displacement 'exactly' mapping points of two grids\n\n    Creates a random 2D displacement field mapping points of an input discrete\n    domain (with dimensions given by from_shape) to points of an output\n    discrete domain (with shape given by to_shape). The affine matrices\n    bringing discrete coordinates to physical space are given by from_grid2world\n    (for the displacement field discretization) and to_grid2world (for the target\n    discretization). Since this function is intended to be used for testing,\n    voxels in the input domain will never be assigned to boundary voxels on the\n    output domain.\n\n    Parameters\n    ----------\n    from_shape : array, shape (2,)\n        the grid shape where the displacement field will be defined on.\n    from_grid2world : array, shape (3,3)\n        the grid-to-space transformation of the displacement field\n    to_shape : array, shape (2,)\n        the grid shape where the deformation field will map the input grid to.\n    to_grid2world : array, shape (3,3)\n        the grid-to-space transformation of the mapped grid\n\n    Returns\n    -------\n    output : array, shape = from_shape\n        the random displacement field in the physical domain\n    int_field : array, shape = from_shape\n        the assignment of each point in the input grid to the target grid\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_59create_random_displacement_2d = {"create_random_displacement_2d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_59create_random_displacement_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_58create_random_displacement_2d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_59create_random_displacement_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_from_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_from_grid2world = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_to_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_to_grid2world = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_random_displacement_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_from_shape,&__pyx_n_s_from_grid2world,&__pyx_n_s_to_shape,&__pyx_n_s_to_grid2world,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_from_shape)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_from_grid2world)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_random_displacement_2d", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2677; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_to_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_random_displacement_2d", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2677; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_to_grid2world)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_random_displacement_2d", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2677; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_random_displacement_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2677; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_from_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[0]); if (unlikely(!__pyx_v_from_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2677; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_from_grid2world = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_from_grid2world.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2678; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_to_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[2]); if (unlikely(!__pyx_v_to_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2679; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_to_grid2world = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_to_grid2world.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2680; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_random_displacement_2d", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2677; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.create_random_displacement_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_58create_random_displacement_2d(__pyx_self, __pyx_v_from_shape, __pyx_v_from_grid2world, __pyx_v_to_shape, __pyx_v_to_grid2world);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_58create_random_displacement_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_from_shape, __Pyx_memviewslice __pyx_v_from_grid2world, __Pyx_memviewslice __pyx_v_to_shape, __Pyx_memviewslice __pyx_v_to_grid2world) {
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_ri;
  npy_intp __pyx_v_rj;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_int_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_output = { 0, 0, { 0 }, { 0 }, { 0 } };
  CYTHON_UNUSED npy_intp __pyx_v_dom_size;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  int __pyx_t_14;
  npy_intp __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  int __pyx_t_17;
  npy_intp __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  npy_intp __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create_random_displacement_2d", 0);

  /* "dipy/align/vector_fields.pyx":2713
 *         cnp.npy_intp i, j, ri, rj
 *         double di, dj, dii, djj
 *         int[:, :, :] int_field = np.empty(tuple(from_shape) + (2,),             # <<<<<<<<<<<<<<
 *                                             dtype=np.int32)
 *         double[:, :, :] output = np.zeros(tuple(from_shape) + (2,),
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_from_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PySequence_Tuple(__pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Add(__pyx_t_3, __pyx_tuple__320); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "dipy/align/vector_fields.pyx":2714
 *         double di, dj, dii, djj
 *         int[:, :, :] int_field = np.empty(tuple(from_shape) + (2,),
 *                                             dtype=np.int32)             # <<<<<<<<<<<<<<
 *         double[:, :, :] output = np.zeros(tuple(from_shape) + (2,),
 *                                           dtype=np.float64)
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2714; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2714; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_int32); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2714; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2714; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "dipy/align/vector_fields.pyx":2713
 *         cnp.npy_intp i, j, ri, rj
 *         double di, dj, dii, djj
 *         int[:, :, :] int_field = np.empty(tuple(from_shape) + (2,),             # <<<<<<<<<<<<<<
 *                                             dtype=np.int32)
 *         double[:, :, :] output = np.zeros(tuple(from_shape) + (2,),
 */
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(__pyx_t_5);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_int_field = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":2715
 *         int[:, :, :] int_field = np.empty(tuple(from_shape) + (2,),
 *                                             dtype=np.int32)
 *         double[:, :, :] output = np.zeros(tuple(from_shape) + (2,),             # <<<<<<<<<<<<<<
 *                                           dtype=np.float64)
 *         cnp.npy_intp dom_size = from_shape[0]*from_shape[1]
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_from_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = PySequence_Tuple(__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyNumber_Add(__pyx_t_3, __pyx_tuple__321); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5);
  __pyx_t_5 = 0;

  /* "dipy/align/vector_fields.pyx":2716
 *                                             dtype=np.int32)
 *         double[:, :, :] output = np.zeros(tuple(from_shape) + (2,),
 *                                           dtype=np.float64)             # <<<<<<<<<<<<<<
 *         cnp.npy_intp dom_size = from_shape[0]*from_shape[1]
 * 
 */
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2716; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2716; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_float64); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2716; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2716; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2715
 *         int[:, :, :] int_field = np.empty(tuple(from_shape) + (2,),
 *                                             dtype=np.int32)
 *         double[:, :, :] output = np.zeros(tuple(from_shape) + (2,),             # <<<<<<<<<<<<<<
 *                                           dtype=np.float64)
 *         cnp.npy_intp dom_size = from_shape[0]*from_shape[1]
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_output = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":2717
 *         double[:, :, :] output = np.zeros(tuple(from_shape) + (2,),
 *                                           dtype=np.float64)
 *         cnp.npy_intp dom_size = from_shape[0]*from_shape[1]             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(from_grid2world, 2):
 */
  __pyx_t_8 = 0;
  __pyx_t_9 = 1;
  __pyx_v_dom_size = ((*((int *) ( /* dim=0 */ (__pyx_v_from_shape.data + __pyx_t_8 * __pyx_v_from_shape.strides[0]) ))) * (*((int *) ( /* dim=0 */ (__pyx_v_from_shape.data + __pyx_t_9 * __pyx_v_from_shape.strides[0]) ))));

  /* "dipy/align/vector_fields.pyx":2719
 *         cnp.npy_intp dom_size = from_shape[0]*from_shape[1]
 * 
 *     if not is_valid_affine(from_grid2world, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid 'from' affine transform matrix")
 *     if not is_valid_affine(to_grid2world, 2):
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_from_grid2world, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = NULL;
  __pyx_t_10 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_10 = 1;
    }
  }
  __pyx_t_2 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (__pyx_t_1) {
    __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1); __pyx_t_1 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_int_2);
  __pyx_t_3 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_2, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_11 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_12 = ((!__pyx_t_11) != 0);
  if (__pyx_t_12) {

    /* "dipy/align/vector_fields.pyx":2720
 * 
 *     if not is_valid_affine(from_grid2world, 2):
 *         raise ValueError("Invalid 'from' affine transform matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(to_grid2world, 2):
 *         raise ValueError("Invalid 'to' affine transform matrix")
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__322, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2720; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2720; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2719
 *         cnp.npy_intp dom_size = from_shape[0]*from_shape[1]
 * 
 *     if not is_valid_affine(from_grid2world, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid 'from' affine transform matrix")
 *     if not is_valid_affine(to_grid2world, 2):
 */
  }

  /* "dipy/align/vector_fields.pyx":2721
 *     if not is_valid_affine(from_grid2world, 2):
 *         raise ValueError("Invalid 'from' affine transform matrix")
 *     if not is_valid_affine(to_grid2world, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid 'to' affine transform matrix")
 * 
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2721; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_to_grid2world, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2721; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_10 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_10 = 1;
    }
  }
  __pyx_t_1 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2721; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_10, __pyx_t_2);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_10, __pyx_int_2);
  __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_1, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2721; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_12 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2721; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_11 = ((!__pyx_t_12) != 0);
  if (__pyx_t_11) {

    /* "dipy/align/vector_fields.pyx":2722
 *         raise ValueError("Invalid 'from' affine transform matrix")
 *     if not is_valid_affine(to_grid2world, 2):
 *         raise ValueError("Invalid 'to' affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     # compute the actual displacement field in the physical space
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__323, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2722; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2722; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2721
 *     if not is_valid_affine(from_grid2world, 2):
 *         raise ValueError("Invalid 'from' affine transform matrix")
 *     if not is_valid_affine(to_grid2world, 2):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid 'to' affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2725
 * 
 *     # compute the actual displacement field in the physical space
 *     for i in range(from_shape[0]):             # <<<<<<<<<<<<<<
 *         for j in range(from_shape[1]):
 *             # randomly choose where each input grid point will be mapped to in
 */
  __pyx_t_13 = 0;
  __pyx_t_14 = (*((int *) ( /* dim=0 */ (__pyx_v_from_shape.data + __pyx_t_13 * __pyx_v_from_shape.strides[0]) )));
  for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
    __pyx_v_i = __pyx_t_15;

    /* "dipy/align/vector_fields.pyx":2726
 *     # compute the actual displacement field in the physical space
 *     for i in range(from_shape[0]):
 *         for j in range(from_shape[1]):             # <<<<<<<<<<<<<<
 *             # randomly choose where each input grid point will be mapped to in
 *             # the target grid
 */
    __pyx_t_16 = 1;
    __pyx_t_17 = (*((int *) ( /* dim=0 */ (__pyx_v_from_shape.data + __pyx_t_16 * __pyx_v_from_shape.strides[0]) )));
    for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
      __pyx_v_j = __pyx_t_18;

      /* "dipy/align/vector_fields.pyx":2729
 *             # randomly choose where each input grid point will be mapped to in
 *             # the target grid
 *             ri = np.random.randint(1, to_shape[0]-1)             # <<<<<<<<<<<<<<
 *             rj = np.random.randint(1, to_shape[1]-1)
 *             int_field[i, j, 0] = ri
 */
      __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2729; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_random); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2729; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_randint); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2729; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_19 = 0;
      __pyx_t_1 = __Pyx_PyInt_From_long(((*((int *) ( /* dim=0 */ (__pyx_v_to_shape.data + __pyx_t_19 * __pyx_v_to_shape.strides[0]) ))) - 1)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2729; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = NULL;
      __pyx_t_10 = 0;
      if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_10 = 1;
        }
      }
      __pyx_t_3 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2729; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      if (__pyx_t_2) {
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2); __pyx_t_2 = NULL;
      }
      __Pyx_INCREF(__pyx_int_1);
      __Pyx_GIVEREF(__pyx_int_1);
      PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_10, __pyx_int_1);
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_10, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2729; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_20 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_4); if (unlikely((__pyx_t_20 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2729; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_v_ri = __pyx_t_20;

      /* "dipy/align/vector_fields.pyx":2730
 *             # the target grid
 *             ri = np.random.randint(1, to_shape[0]-1)
 *             rj = np.random.randint(1, to_shape[1]-1)             # <<<<<<<<<<<<<<
 *             int_field[i, j, 0] = ri
 *             int_field[i, j, 1] = rj
 */
      __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2730; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_random); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2730; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_randint); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2730; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_21 = 1;
      __pyx_t_3 = __Pyx_PyInt_From_long(((*((int *) ( /* dim=0 */ (__pyx_v_to_shape.data + __pyx_t_21 * __pyx_v_to_shape.strides[0]) ))) - 1)); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2730; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_1 = NULL;
      __pyx_t_10 = 0;
      if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_10 = 1;
        }
      }
      __pyx_t_2 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2730; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_int_1);
      __Pyx_GIVEREF(__pyx_int_1);
      PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_10, __pyx_int_1);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_10, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_2, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2730; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_20 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_4); if (unlikely((__pyx_t_20 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2730; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_v_rj = __pyx_t_20;

      /* "dipy/align/vector_fields.pyx":2731
 *             ri = np.random.randint(1, to_shape[0]-1)
 *             rj = np.random.randint(1, to_shape[1]-1)
 *             int_field[i, j, 0] = ri             # <<<<<<<<<<<<<<
 *             int_field[i, j, 1] = rj
 * 
 */
      __pyx_t_22 = __pyx_v_i;
      __pyx_t_23 = __pyx_v_j;
      __pyx_t_24 = 0;
      *((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_int_field.data + __pyx_t_22 * __pyx_v_int_field.strides[0]) ) + __pyx_t_23 * __pyx_v_int_field.strides[1]) ) + __pyx_t_24 * __pyx_v_int_field.strides[2]) )) = __pyx_v_ri;

      /* "dipy/align/vector_fields.pyx":2732
 *             rj = np.random.randint(1, to_shape[1]-1)
 *             int_field[i, j, 0] = ri
 *             int_field[i, j, 1] = rj             # <<<<<<<<<<<<<<
 * 
 *             # convert the input point to physical coordinates
 */
      __pyx_t_25 = __pyx_v_i;
      __pyx_t_26 = __pyx_v_j;
      __pyx_t_27 = 1;
      *((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_int_field.data + __pyx_t_25 * __pyx_v_int_field.strides[0]) ) + __pyx_t_26 * __pyx_v_int_field.strides[1]) ) + __pyx_t_27 * __pyx_v_int_field.strides[2]) )) = __pyx_v_rj;

      /* "dipy/align/vector_fields.pyx":2735
 * 
 *             # convert the input point to physical coordinates
 *             if from_grid2world is not None:             # <<<<<<<<<<<<<<
 *                 di = _apply_affine_2d_x0(i, j, 1, from_grid2world)
 *                 dj = _apply_affine_2d_x1(i, j, 1, from_grid2world)
 */
      __pyx_t_11 = ((((PyObject *) __pyx_v_from_grid2world.memview) != Py_None) != 0);
      if (__pyx_t_11) {

        /* "dipy/align/vector_fields.pyx":2736
 *             # convert the input point to physical coordinates
 *             if from_grid2world is not None:
 *                 di = _apply_affine_2d_x0(i, j, 1, from_grid2world)             # <<<<<<<<<<<<<<
 *                 dj = _apply_affine_2d_x1(i, j, 1, from_grid2world)
 *             else:
 */
        __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_from_grid2world);

        /* "dipy/align/vector_fields.pyx":2737
 *             if from_grid2world is not None:
 *                 di = _apply_affine_2d_x0(i, j, 1, from_grid2world)
 *                 dj = _apply_affine_2d_x1(i, j, 1, from_grid2world)             # <<<<<<<<<<<<<<
 *             else:
 *                 di = i
 */
        __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_from_grid2world);

        /* "dipy/align/vector_fields.pyx":2735
 * 
 *             # convert the input point to physical coordinates
 *             if from_grid2world is not None:             # <<<<<<<<<<<<<<
 *                 di = _apply_affine_2d_x0(i, j, 1, from_grid2world)
 *                 dj = _apply_affine_2d_x1(i, j, 1, from_grid2world)
 */
        goto __pyx_L9;
      }

      /* "dipy/align/vector_fields.pyx":2739
 *                 dj = _apply_affine_2d_x1(i, j, 1, from_grid2world)
 *             else:
 *                 di = i             # <<<<<<<<<<<<<<
 *                 dj = j
 * 
 */
      /*else*/ {
        __pyx_v_di = __pyx_v_i;

        /* "dipy/align/vector_fields.pyx":2740
 *             else:
 *                 di = i
 *                 dj = j             # <<<<<<<<<<<<<<
 * 
 *             # convert the output point to physical coordinates
 */
        __pyx_v_dj = __pyx_v_j;
      }
      __pyx_L9:;

      /* "dipy/align/vector_fields.pyx":2743
 * 
 *             # convert the output point to physical coordinates
 *             if to_grid2world is not None:             # <<<<<<<<<<<<<<
 *                 dii = _apply_affine_2d_x0(ri, rj, 1, to_grid2world)
 *                 djj = _apply_affine_2d_x1(ri, rj, 1, to_grid2world)
 */
      __pyx_t_11 = ((((PyObject *) __pyx_v_to_grid2world.memview) != Py_None) != 0);
      if (__pyx_t_11) {

        /* "dipy/align/vector_fields.pyx":2744
 *             # convert the output point to physical coordinates
 *             if to_grid2world is not None:
 *                 dii = _apply_affine_2d_x0(ri, rj, 1, to_grid2world)             # <<<<<<<<<<<<<<
 *                 djj = _apply_affine_2d_x1(ri, rj, 1, to_grid2world)
 *             else:
 */
        __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_ri, __pyx_v_rj, 1.0, __pyx_v_to_grid2world);

        /* "dipy/align/vector_fields.pyx":2745
 *             if to_grid2world is not None:
 *                 dii = _apply_affine_2d_x0(ri, rj, 1, to_grid2world)
 *                 djj = _apply_affine_2d_x1(ri, rj, 1, to_grid2world)             # <<<<<<<<<<<<<<
 *             else:
 *                 dii = ri
 */
        __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_ri, __pyx_v_rj, 1.0, __pyx_v_to_grid2world);

        /* "dipy/align/vector_fields.pyx":2743
 * 
 *             # convert the output point to physical coordinates
 *             if to_grid2world is not None:             # <<<<<<<<<<<<<<
 *                 dii = _apply_affine_2d_x0(ri, rj, 1, to_grid2world)
 *                 djj = _apply_affine_2d_x1(ri, rj, 1, to_grid2world)
 */
        goto __pyx_L10;
      }

      /* "dipy/align/vector_fields.pyx":2747
 *                 djj = _apply_affine_2d_x1(ri, rj, 1, to_grid2world)
 *             else:
 *                 dii = ri             # <<<<<<<<<<<<<<
 *                 djj = rj
 * 
 */
      /*else*/ {
        __pyx_v_dii = __pyx_v_ri;

        /* "dipy/align/vector_fields.pyx":2748
 *             else:
 *                 dii = ri
 *                 djj = rj             # <<<<<<<<<<<<<<
 * 
 *             # the displacement vector at (i,j) must be the target point minus
 */
        __pyx_v_djj = __pyx_v_rj;
      }
      __pyx_L10:;

      /* "dipy/align/vector_fields.pyx":2753
 *             # the original point, both in physical space
 * 
 *             output[i, j, 0] = dii - di             # <<<<<<<<<<<<<<
 *             output[i, j, 1] = djj - dj
 * 
 */
      __pyx_t_28 = __pyx_v_i;
      __pyx_t_29 = __pyx_v_j;
      __pyx_t_30 = 0;
      *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_output.data + __pyx_t_28 * __pyx_v_output.strides[0]) ) + __pyx_t_29 * __pyx_v_output.strides[1]) ) + __pyx_t_30 * __pyx_v_output.strides[2]) )) = (__pyx_v_dii - __pyx_v_di);

      /* "dipy/align/vector_fields.pyx":2754
 * 
 *             output[i, j, 0] = dii - di
 *             output[i, j, 1] = djj - dj             # <<<<<<<<<<<<<<
 * 
 *     return output, int_field
 */
      __pyx_t_31 = __pyx_v_i;
      __pyx_t_32 = __pyx_v_j;
      __pyx_t_33 = 1;
      *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_output.data + __pyx_t_31 * __pyx_v_output.strides[0]) ) + __pyx_t_32 * __pyx_v_output.strides[1]) ) + __pyx_t_33 * __pyx_v_output.strides[2]) )) = (__pyx_v_djj - __pyx_v_dj);
    }
  }

  /* "dipy/align/vector_fields.pyx":2756
 *             output[i, j, 1] = djj - dj
 * 
 *     return output, int_field             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_output, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2756; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_int_field, 3, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2756; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2756; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_5);
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2677
 * 
 * 
 * def create_random_displacement_2d(int[:] from_shape,             # <<<<<<<<<<<<<<
 *                                   double[:, :] from_grid2world,
 *                                   int[:] to_shape,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.create_random_displacement_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_int_field, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_output, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_from_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_from_grid2world, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_to_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_to_grid2world, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":2759
 * 
 * 
 * def create_random_displacement_3d(int[:] from_shape, double[:, :] from_grid2world,             # <<<<<<<<<<<<<<
 *                                   int[:] to_shape, double[:, :] to_grid2world):
 *     r"""Creates a random 3D displacement 'exactly' mapping points of two grids
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_61create_random_displacement_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_60create_random_displacement_3d[] = "Creates a random 3D displacement 'exactly' mapping points of two grids\n\n    Creates a random 3D displacement field mapping points of an input discrete\n    domain (with dimensions given by from_shape) to points of an output\n    discrete domain (with shape given by to_shape). The affine matrices\n    bringing discrete coordinates to physical space are given by from_grid2world\n    (for the displacement field discretization) and to_grid2world (for the target\n    discretization). Since this function is intended to be used for testing,\n    voxels in the input domain will never be assigned to boundary voxels on the\n    output domain.\n\n    Parameters\n    ----------\n    from_shape : array, shape (3,)\n        the grid shape where the displacement field will be defined on.\n    from_grid2world : array, shape (4,4)\n        the grid-to-space transformation of the displacement field\n    to_shape : array, shape (3,)\n        the grid shape where the deformation field will map the input grid to.\n    to_grid2world : array, shape (4,4)\n        the grid-to-space transformation of the mapped grid\n\n    Returns\n    -------\n    output : array, shape = from_shape\n        the random displacement field in the physical domain\n    int_field : array, shape = from_shape\n        the assignment of each point in the input grid to the target grid\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_61create_random_displacement_3d = {"create_random_displacement_3d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_61create_random_displacement_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_60create_random_displacement_3d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_61create_random_displacement_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_from_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_from_grid2world = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_to_shape = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_to_grid2world = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_random_displacement_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_from_shape,&__pyx_n_s_from_grid2world,&__pyx_n_s_to_shape,&__pyx_n_s_to_grid2world,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_from_shape)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_from_grid2world)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_random_displacement_3d", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2759; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_to_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_random_displacement_3d", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2759; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_to_grid2world)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_random_displacement_3d", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2759; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_random_displacement_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2759; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_from_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[0]); if (unlikely(!__pyx_v_from_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2759; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_from_grid2world = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_from_grid2world.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2759; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_to_shape = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[2]); if (unlikely(!__pyx_v_to_shape.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2760; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_to_grid2world = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_to_grid2world.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2760; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_random_displacement_3d", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2759; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.create_random_displacement_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_60create_random_displacement_3d(__pyx_self, __pyx_v_from_shape, __pyx_v_from_grid2world, __pyx_v_to_shape, __pyx_v_to_grid2world);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_60create_random_displacement_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_from_shape, __Pyx_memviewslice __pyx_v_from_grid2world, __Pyx_memviewslice __pyx_v_to_shape, __Pyx_memviewslice __pyx_v_to_grid2world) {
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_ri;
  npy_intp __pyx_v_rj;
  npy_intp __pyx_v_rk;
  double __pyx_v_di;
  double __pyx_v_dj;
  double __pyx_v_dii;
  double __pyx_v_djj;
  __Pyx_memviewslice __pyx_v_int_field = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_output = { 0, 0, { 0 }, { 0 }, { 0 } };
  CYTHON_UNUSED npy_intp __pyx_v_dom_size;
  double __pyx_v_dk;
  double __pyx_v_dkk;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_7 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  int __pyx_t_15;
  npy_intp __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  int __pyx_t_18;
  npy_intp __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  int __pyx_t_21;
  npy_intp __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  npy_intp __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create_random_displacement_3d", 0);

  /* "dipy/align/vector_fields.pyx":2793
 *         cnp.npy_intp i, j, k, ri, rj, rk
 *         double di, dj, dii, djj
 *         int[:, :, :, :] int_field = np.empty(tuple(from_shape) + (3,),             # <<<<<<<<<<<<<<
 *                                                dtype=np.int32)
 *         double[:, :, :, :] output = np.zeros(tuple(from_shape) + (3,),
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2793; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2793; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_from_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2793; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PySequence_Tuple(__pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2793; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Add(__pyx_t_3, __pyx_tuple__324); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2793; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2793; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "dipy/align/vector_fields.pyx":2794
 *         double di, dj, dii, djj
 *         int[:, :, :, :] int_field = np.empty(tuple(from_shape) + (3,),
 *                                                dtype=np.int32)             # <<<<<<<<<<<<<<
 *         double[:, :, :, :] output = np.zeros(tuple(from_shape) + (3,),
 *                                              dtype=np.float64)
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_int32); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "dipy/align/vector_fields.pyx":2793
 *         cnp.npy_intp i, j, k, ri, rj, rk
 *         double di, dj, dii, djj
 *         int[:, :, :, :] int_field = np.empty(tuple(from_shape) + (3,),             # <<<<<<<<<<<<<<
 *                                                dtype=np.int32)
 *         double[:, :, :, :] output = np.zeros(tuple(from_shape) + (3,),
 */
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2793; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_int(__pyx_t_5);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2793; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_int_field = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":2795
 *         int[:, :, :, :] int_field = np.empty(tuple(from_shape) + (3,),
 *                                                dtype=np.int32)
 *         double[:, :, :, :] output = np.zeros(tuple(from_shape) + (3,),             # <<<<<<<<<<<<<<
 *                                              dtype=np.float64)
 *         cnp.npy_intp dom_size = from_shape[0]*from_shape[1]*from_shape[2]
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_zeros); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_from_shape, 1, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = PySequence_Tuple(__pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyNumber_Add(__pyx_t_3, __pyx_tuple__325); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5);
  __pyx_t_5 = 0;

  /* "dipy/align/vector_fields.pyx":2796
 *                                                dtype=np.int32)
 *         double[:, :, :, :] output = np.zeros(tuple(from_shape) + (3,),
 *                                              dtype=np.float64)             # <<<<<<<<<<<<<<
 *         cnp.npy_intp dom_size = from_shape[0]*from_shape[1]*from_shape[2]
 * 
 */
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_float64); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":2795
 *         int[:, :, :, :] int_field = np.empty(tuple(from_shape) + (3,),
 *                                                dtype=np.int32)
 *         double[:, :, :, :] output = np.zeros(tuple(from_shape) + (3,),             # <<<<<<<<<<<<<<
 *                                              dtype=np.float64)
 *         cnp.npy_intp dom_size = from_shape[0]*from_shape[1]*from_shape[2]
 */
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_7 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_7.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_output = __pyx_t_7;
  __pyx_t_7.memview = NULL;
  __pyx_t_7.data = NULL;

  /* "dipy/align/vector_fields.pyx":2797
 *         double[:, :, :, :] output = np.zeros(tuple(from_shape) + (3,),
 *                                              dtype=np.float64)
 *         cnp.npy_intp dom_size = from_shape[0]*from_shape[1]*from_shape[2]             # <<<<<<<<<<<<<<
 * 
 *     if not is_valid_affine(from_grid2world, 3):
 */
  __pyx_t_8 = 0;
  __pyx_t_9 = 1;
  __pyx_t_10 = 2;
  __pyx_v_dom_size = (((*((int *) ( /* dim=0 */ (__pyx_v_from_shape.data + __pyx_t_8 * __pyx_v_from_shape.strides[0]) ))) * (*((int *) ( /* dim=0 */ (__pyx_v_from_shape.data + __pyx_t_9 * __pyx_v_from_shape.strides[0]) )))) * (*((int *) ( /* dim=0 */ (__pyx_v_from_shape.data + __pyx_t_10 * __pyx_v_from_shape.strides[0]) ))));

  /* "dipy/align/vector_fields.pyx":2799
 *         cnp.npy_intp dom_size = from_shape[0]*from_shape[1]*from_shape[2]
 * 
 *     if not is_valid_affine(from_grid2world, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid 'from' affine transform matrix")
 *     if not is_valid_affine(to_grid2world, 3):
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_from_grid2world, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = NULL;
  __pyx_t_11 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_11 = 1;
    }
  }
  __pyx_t_2 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (__pyx_t_1) {
    __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1); __pyx_t_1 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_11, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_11, __pyx_int_3);
  __pyx_t_3 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_2, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_12 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_13 = ((!__pyx_t_12) != 0);
  if (__pyx_t_13) {

    /* "dipy/align/vector_fields.pyx":2800
 * 
 *     if not is_valid_affine(from_grid2world, 3):
 *         raise ValueError("Invalid 'from' affine transform matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(to_grid2world, 3):
 *         raise ValueError("Invalid 'to' affine transform matrix")
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__326, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2800; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2800; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2799
 *         cnp.npy_intp dom_size = from_shape[0]*from_shape[1]*from_shape[2]
 * 
 *     if not is_valid_affine(from_grid2world, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid 'from' affine transform matrix")
 *     if not is_valid_affine(to_grid2world, 3):
 */
  }

  /* "dipy/align/vector_fields.pyx":2801
 *     if not is_valid_affine(from_grid2world, 3):
 *         raise ValueError("Invalid 'from' affine transform matrix")
 *     if not is_valid_affine(to_grid2world, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid 'to' affine transform matrix")
 * 
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2801; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_2 = __pyx_memoryview_fromslice(__pyx_v_to_grid2world, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2801; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_11 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_11 = 1;
    }
  }
  __pyx_t_1 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2801; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (__pyx_t_3) {
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3); __pyx_t_3 = NULL;
  }
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_11, __pyx_t_2);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_11, __pyx_int_3);
  __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_1, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2801; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_13 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2801; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_12 = ((!__pyx_t_13) != 0);
  if (__pyx_t_12) {

    /* "dipy/align/vector_fields.pyx":2802
 *         raise ValueError("Invalid 'from' affine transform matrix")
 *     if not is_valid_affine(to_grid2world, 3):
 *         raise ValueError("Invalid 'to' affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     # compute the actual displacement field in the physical space
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__327, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2802; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2802; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":2801
 *     if not is_valid_affine(from_grid2world, 3):
 *         raise ValueError("Invalid 'from' affine transform matrix")
 *     if not is_valid_affine(to_grid2world, 3):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid 'to' affine transform matrix")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":2805
 * 
 *     # compute the actual displacement field in the physical space
 *     for k in range(from_shape[0]):             # <<<<<<<<<<<<<<
 *         for i in range(from_shape[1]):
 *             for j in range(from_shape[2]):
 */
  __pyx_t_14 = 0;
  __pyx_t_15 = (*((int *) ( /* dim=0 */ (__pyx_v_from_shape.data + __pyx_t_14 * __pyx_v_from_shape.strides[0]) )));
  for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
    __pyx_v_k = __pyx_t_16;

    /* "dipy/align/vector_fields.pyx":2806
 *     # compute the actual displacement field in the physical space
 *     for k in range(from_shape[0]):
 *         for i in range(from_shape[1]):             # <<<<<<<<<<<<<<
 *             for j in range(from_shape[2]):
 *                 # randomly choose the location of each point on the target grid
 */
    __pyx_t_17 = 1;
    __pyx_t_18 = (*((int *) ( /* dim=0 */ (__pyx_v_from_shape.data + __pyx_t_17 * __pyx_v_from_shape.strides[0]) )));
    for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
      __pyx_v_i = __pyx_t_19;

      /* "dipy/align/vector_fields.pyx":2807
 *     for k in range(from_shape[0]):
 *         for i in range(from_shape[1]):
 *             for j in range(from_shape[2]):             # <<<<<<<<<<<<<<
 *                 # randomly choose the location of each point on the target grid
 *                 rk = np.random.randint(1, to_shape[0]-1)
 */
      __pyx_t_20 = 2;
      __pyx_t_21 = (*((int *) ( /* dim=0 */ (__pyx_v_from_shape.data + __pyx_t_20 * __pyx_v_from_shape.strides[0]) )));
      for (__pyx_t_22 = 0; __pyx_t_22 < __pyx_t_21; __pyx_t_22+=1) {
        __pyx_v_j = __pyx_t_22;

        /* "dipy/align/vector_fields.pyx":2809
 *             for j in range(from_shape[2]):
 *                 # randomly choose the location of each point on the target grid
 *                 rk = np.random.randint(1, to_shape[0]-1)             # <<<<<<<<<<<<<<
 *                 ri = np.random.randint(1, to_shape[1]-1)
 *                 rj = np.random.randint(1, to_shape[2]-1)
 */
        __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2809; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_random); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2809; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_randint); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2809; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_23 = 0;
        __pyx_t_1 = __Pyx_PyInt_From_long(((*((int *) ( /* dim=0 */ (__pyx_v_to_shape.data + __pyx_t_23 * __pyx_v_to_shape.strides[0]) ))) - 1)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2809; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_2 = NULL;
        __pyx_t_11 = 0;
        if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_2)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_2);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_11 = 1;
          }
        }
        __pyx_t_3 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2809; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        if (__pyx_t_2) {
          __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2); __pyx_t_2 = NULL;
        }
        __Pyx_INCREF(__pyx_int_1);
        __Pyx_GIVEREF(__pyx_int_1);
        PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_11, __pyx_int_1);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_11, __pyx_t_1);
        __pyx_t_1 = 0;
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2809; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_24 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_4); if (unlikely((__pyx_t_24 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2809; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_v_rk = __pyx_t_24;

        /* "dipy/align/vector_fields.pyx":2810
 *                 # randomly choose the location of each point on the target grid
 *                 rk = np.random.randint(1, to_shape[0]-1)
 *                 ri = np.random.randint(1, to_shape[1]-1)             # <<<<<<<<<<<<<<
 *                 rj = np.random.randint(1, to_shape[2]-1)
 *                 int_field[k, i, j, 0] = rk
 */
        __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2810; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_random); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2810; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_randint); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2810; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_25 = 1;
        __pyx_t_3 = __Pyx_PyInt_From_long(((*((int *) ( /* dim=0 */ (__pyx_v_to_shape.data + __pyx_t_25 * __pyx_v_to_shape.strides[0]) ))) - 1)); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2810; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_1 = NULL;
        __pyx_t_11 = 0;
        if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_1)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_1);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_11 = 1;
          }
        }
        __pyx_t_2 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2810; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        if (__pyx_t_1) {
          __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1); __pyx_t_1 = NULL;
        }
        __Pyx_INCREF(__pyx_int_1);
        __Pyx_GIVEREF(__pyx_int_1);
        PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_11, __pyx_int_1);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_11, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_2, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2810; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_24 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_4); if (unlikely((__pyx_t_24 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2810; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_v_ri = __pyx_t_24;

        /* "dipy/align/vector_fields.pyx":2811
 *                 rk = np.random.randint(1, to_shape[0]-1)
 *                 ri = np.random.randint(1, to_shape[1]-1)
 *                 rj = np.random.randint(1, to_shape[2]-1)             # <<<<<<<<<<<<<<
 *                 int_field[k, i, j, 0] = rk
 *                 int_field[k, i, j, 1] = ri
 */
        __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2811; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_random); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2811; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_randint); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2811; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_26 = 2;
        __pyx_t_2 = __Pyx_PyInt_From_long(((*((int *) ( /* dim=0 */ (__pyx_v_to_shape.data + __pyx_t_26 * __pyx_v_to_shape.strides[0]) ))) - 1)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2811; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_3 = NULL;
        __pyx_t_11 = 0;
        if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_3)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_11 = 1;
          }
        }
        __pyx_t_1 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2811; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        if (__pyx_t_3) {
          __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3); __pyx_t_3 = NULL;
        }
        __Pyx_INCREF(__pyx_int_1);
        __Pyx_GIVEREF(__pyx_int_1);
        PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_11, __pyx_int_1);
        __Pyx_GIVEREF(__pyx_t_2);
        PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_11, __pyx_t_2);
        __pyx_t_2 = 0;
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_1, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2811; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_24 = __Pyx_PyInt_As_Py_intptr_t(__pyx_t_4); if (unlikely((__pyx_t_24 == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2811; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_v_rj = __pyx_t_24;

        /* "dipy/align/vector_fields.pyx":2812
 *                 ri = np.random.randint(1, to_shape[1]-1)
 *                 rj = np.random.randint(1, to_shape[2]-1)
 *                 int_field[k, i, j, 0] = rk             # <<<<<<<<<<<<<<
 *                 int_field[k, i, j, 1] = ri
 *                 int_field[k, i, j, 2] = rj
 */
        __pyx_t_27 = __pyx_v_k;
        __pyx_t_28 = __pyx_v_i;
        __pyx_t_29 = __pyx_v_j;
        __pyx_t_30 = 0;
        *((int *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_int_field.data + __pyx_t_27 * __pyx_v_int_field.strides[0]) ) + __pyx_t_28 * __pyx_v_int_field.strides[1]) ) + __pyx_t_29 * __pyx_v_int_field.strides[2]) ) + __pyx_t_30 * __pyx_v_int_field.strides[3]) )) = __pyx_v_rk;

        /* "dipy/align/vector_fields.pyx":2813
 *                 rj = np.random.randint(1, to_shape[2]-1)
 *                 int_field[k, i, j, 0] = rk
 *                 int_field[k, i, j, 1] = ri             # <<<<<<<<<<<<<<
 *                 int_field[k, i, j, 2] = rj
 * 
 */
        __pyx_t_31 = __pyx_v_k;
        __pyx_t_32 = __pyx_v_i;
        __pyx_t_33 = __pyx_v_j;
        __pyx_t_34 = 1;
        *((int *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_int_field.data + __pyx_t_31 * __pyx_v_int_field.strides[0]) ) + __pyx_t_32 * __pyx_v_int_field.strides[1]) ) + __pyx_t_33 * __pyx_v_int_field.strides[2]) ) + __pyx_t_34 * __pyx_v_int_field.strides[3]) )) = __pyx_v_ri;

        /* "dipy/align/vector_fields.pyx":2814
 *                 int_field[k, i, j, 0] = rk
 *                 int_field[k, i, j, 1] = ri
 *                 int_field[k, i, j, 2] = rj             # <<<<<<<<<<<<<<
 * 
 *                 # convert the input point to physical coordinates
 */
        __pyx_t_35 = __pyx_v_k;
        __pyx_t_36 = __pyx_v_i;
        __pyx_t_37 = __pyx_v_j;
        __pyx_t_38 = 2;
        *((int *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_int_field.data + __pyx_t_35 * __pyx_v_int_field.strides[0]) ) + __pyx_t_36 * __pyx_v_int_field.strides[1]) ) + __pyx_t_37 * __pyx_v_int_field.strides[2]) ) + __pyx_t_38 * __pyx_v_int_field.strides[3]) )) = __pyx_v_rj;

        /* "dipy/align/vector_fields.pyx":2817
 * 
 *                 # convert the input point to physical coordinates
 *                 if from_grid2world is not None:             # <<<<<<<<<<<<<<
 *                     dk = _apply_affine_3d_x0(k, i, j, 1, from_grid2world)
 *                     di = _apply_affine_3d_x1(k, i, j, 1, from_grid2world)
 */
        __pyx_t_12 = ((((PyObject *) __pyx_v_from_grid2world.memview) != Py_None) != 0);
        if (__pyx_t_12) {

          /* "dipy/align/vector_fields.pyx":2818
 *                 # convert the input point to physical coordinates
 *                 if from_grid2world is not None:
 *                     dk = _apply_affine_3d_x0(k, i, j, 1, from_grid2world)             # <<<<<<<<<<<<<<
 *                     di = _apply_affine_3d_x1(k, i, j, 1, from_grid2world)
 *                     dj = _apply_affine_3d_x2(k, i, j, 1, from_grid2world)
 */
          __pyx_v_dk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_from_grid2world);

          /* "dipy/align/vector_fields.pyx":2819
 *                 if from_grid2world is not None:
 *                     dk = _apply_affine_3d_x0(k, i, j, 1, from_grid2world)
 *                     di = _apply_affine_3d_x1(k, i, j, 1, from_grid2world)             # <<<<<<<<<<<<<<
 *                     dj = _apply_affine_3d_x2(k, i, j, 1, from_grid2world)
 *                 else:
 */
          __pyx_v_di = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_from_grid2world);

          /* "dipy/align/vector_fields.pyx":2820
 *                     dk = _apply_affine_3d_x0(k, i, j, 1, from_grid2world)
 *                     di = _apply_affine_3d_x1(k, i, j, 1, from_grid2world)
 *                     dj = _apply_affine_3d_x2(k, i, j, 1, from_grid2world)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dk = k
 */
          __pyx_v_dj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_from_grid2world);

          /* "dipy/align/vector_fields.pyx":2817
 * 
 *                 # convert the input point to physical coordinates
 *                 if from_grid2world is not None:             # <<<<<<<<<<<<<<
 *                     dk = _apply_affine_3d_x0(k, i, j, 1, from_grid2world)
 *                     di = _apply_affine_3d_x1(k, i, j, 1, from_grid2world)
 */
          goto __pyx_L11;
        }

        /* "dipy/align/vector_fields.pyx":2822
 *                     dj = _apply_affine_3d_x2(k, i, j, 1, from_grid2world)
 *                 else:
 *                     dk = k             # <<<<<<<<<<<<<<
 *                     di = i
 *                     dj = j
 */
        /*else*/ {
          __pyx_v_dk = __pyx_v_k;

          /* "dipy/align/vector_fields.pyx":2823
 *                 else:
 *                     dk = k
 *                     di = i             # <<<<<<<<<<<<<<
 *                     dj = j
 * 
 */
          __pyx_v_di = __pyx_v_i;

          /* "dipy/align/vector_fields.pyx":2824
 *                     dk = k
 *                     di = i
 *                     dj = j             # <<<<<<<<<<<<<<
 * 
 *                 # convert the output point to physical coordinates
 */
          __pyx_v_dj = __pyx_v_j;
        }
        __pyx_L11:;

        /* "dipy/align/vector_fields.pyx":2827
 * 
 *                 # convert the output point to physical coordinates
 *                 if to_grid2world is not None:             # <<<<<<<<<<<<<<
 *                     dkk = _apply_affine_3d_x0(rk, ri, rj, 1, to_grid2world)
 *                     dii = _apply_affine_3d_x1(rk, ri, rj, 1, to_grid2world)
 */
        __pyx_t_12 = ((((PyObject *) __pyx_v_to_grid2world.memview) != Py_None) != 0);
        if (__pyx_t_12) {

          /* "dipy/align/vector_fields.pyx":2828
 *                 # convert the output point to physical coordinates
 *                 if to_grid2world is not None:
 *                     dkk = _apply_affine_3d_x0(rk, ri, rj, 1, to_grid2world)             # <<<<<<<<<<<<<<
 *                     dii = _apply_affine_3d_x1(rk, ri, rj, 1, to_grid2world)
 *                     djj = _apply_affine_3d_x2(rk, ri, rj, 1, to_grid2world)
 */
          __pyx_v_dkk = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_rk, __pyx_v_ri, __pyx_v_rj, 1.0, __pyx_v_to_grid2world);

          /* "dipy/align/vector_fields.pyx":2829
 *                 if to_grid2world is not None:
 *                     dkk = _apply_affine_3d_x0(rk, ri, rj, 1, to_grid2world)
 *                     dii = _apply_affine_3d_x1(rk, ri, rj, 1, to_grid2world)             # <<<<<<<<<<<<<<
 *                     djj = _apply_affine_3d_x2(rk, ri, rj, 1, to_grid2world)
 *                 else:
 */
          __pyx_v_dii = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_rk, __pyx_v_ri, __pyx_v_rj, 1.0, __pyx_v_to_grid2world);

          /* "dipy/align/vector_fields.pyx":2830
 *                     dkk = _apply_affine_3d_x0(rk, ri, rj, 1, to_grid2world)
 *                     dii = _apply_affine_3d_x1(rk, ri, rj, 1, to_grid2world)
 *                     djj = _apply_affine_3d_x2(rk, ri, rj, 1, to_grid2world)             # <<<<<<<<<<<<<<
 *                 else:
 *                     dkk = rk
 */
          __pyx_v_djj = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_rk, __pyx_v_ri, __pyx_v_rj, 1.0, __pyx_v_to_grid2world);

          /* "dipy/align/vector_fields.pyx":2827
 * 
 *                 # convert the output point to physical coordinates
 *                 if to_grid2world is not None:             # <<<<<<<<<<<<<<
 *                     dkk = _apply_affine_3d_x0(rk, ri, rj, 1, to_grid2world)
 *                     dii = _apply_affine_3d_x1(rk, ri, rj, 1, to_grid2world)
 */
          goto __pyx_L12;
        }

        /* "dipy/align/vector_fields.pyx":2832
 *                     djj = _apply_affine_3d_x2(rk, ri, rj, 1, to_grid2world)
 *                 else:
 *                     dkk = rk             # <<<<<<<<<<<<<<
 *                     dii = ri
 *                     djj = rj
 */
        /*else*/ {
          __pyx_v_dkk = __pyx_v_rk;

          /* "dipy/align/vector_fields.pyx":2833
 *                 else:
 *                     dkk = rk
 *                     dii = ri             # <<<<<<<<<<<<<<
 *                     djj = rj
 * 
 */
          __pyx_v_dii = __pyx_v_ri;

          /* "dipy/align/vector_fields.pyx":2834
 *                     dkk = rk
 *                     dii = ri
 *                     djj = rj             # <<<<<<<<<<<<<<
 * 
 *                 # the displacement vector at (i,j) must be the target point minus
 */
          __pyx_v_djj = __pyx_v_rj;
        }
        __pyx_L12:;

        /* "dipy/align/vector_fields.pyx":2839
 *                 # the original point, both in physical space
 * 
 *                 output[k, i, j, 0] = dkk - dk             # <<<<<<<<<<<<<<
 *                 output[k, i, j, 1] = dii - di
 *                 output[k, i, j, 2] = djj - dj
 */
        __pyx_t_39 = __pyx_v_k;
        __pyx_t_40 = __pyx_v_i;
        __pyx_t_41 = __pyx_v_j;
        __pyx_t_42 = 0;
        *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_output.data + __pyx_t_39 * __pyx_v_output.strides[0]) ) + __pyx_t_40 * __pyx_v_output.strides[1]) ) + __pyx_t_41 * __pyx_v_output.strides[2]) ) + __pyx_t_42 * __pyx_v_output.strides[3]) )) = (__pyx_v_dkk - __pyx_v_dk);

        /* "dipy/align/vector_fields.pyx":2840
 * 
 *                 output[k, i, j, 0] = dkk - dk
 *                 output[k, i, j, 1] = dii - di             # <<<<<<<<<<<<<<
 *                 output[k, i, j, 2] = djj - dj
 * 
 */
        __pyx_t_43 = __pyx_v_k;
        __pyx_t_44 = __pyx_v_i;
        __pyx_t_45 = __pyx_v_j;
        __pyx_t_46 = 1;
        *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_output.data + __pyx_t_43 * __pyx_v_output.strides[0]) ) + __pyx_t_44 * __pyx_v_output.strides[1]) ) + __pyx_t_45 * __pyx_v_output.strides[2]) ) + __pyx_t_46 * __pyx_v_output.strides[3]) )) = (__pyx_v_dii - __pyx_v_di);

        /* "dipy/align/vector_fields.pyx":2841
 *                 output[k, i, j, 0] = dkk - dk
 *                 output[k, i, j, 1] = dii - di
 *                 output[k, i, j, 2] = djj - dj             # <<<<<<<<<<<<<<
 * 
 *     return output, int_field
 */
        __pyx_t_47 = __pyx_v_k;
        __pyx_t_48 = __pyx_v_i;
        __pyx_t_49 = __pyx_v_j;
        __pyx_t_50 = 2;
        *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_output.data + __pyx_t_47 * __pyx_v_output.strides[0]) ) + __pyx_t_48 * __pyx_v_output.strides[1]) ) + __pyx_t_49 * __pyx_v_output.strides[2]) ) + __pyx_t_50 * __pyx_v_output.strides[3]) )) = (__pyx_v_djj - __pyx_v_dj);
      }
    }
  }

  /* "dipy/align/vector_fields.pyx":2843
 *                 output[k, i, j, 2] = djj - dj
 * 
 *     return output, int_field             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_output, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2843; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_int_field, 4, (PyObject *(*)(char *)) __pyx_memview_get_int, (int (*)(char *, PyObject *)) __pyx_memview_set_int, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2843; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2843; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_5);
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2759
 * 
 * 
 * def create_random_displacement_3d(int[:] from_shape, double[:, :] from_grid2world,             # <<<<<<<<<<<<<<
 *                                   int[:] to_shape, double[:, :] to_grid2world):
 *     r"""Creates a random 3D displacement 'exactly' mapping points of two grids
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_7, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.create_random_displacement_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_int_field, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_output, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_from_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_from_grid2world, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_to_shape, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_to_grid2world, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":2846
 * 
 * 
 * def create_harmonic_fields_2d(cnp.npy_intp nrows, cnp.npy_intp ncols,             # <<<<<<<<<<<<<<
 *                               double b, double m):
 *     r"""Creates an invertible 2D displacement field
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_63create_harmonic_fields_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_62create_harmonic_fields_2d[] = "Creates an invertible 2D displacement field\n\n    Creates the invertible displacement fields used in Chen et al. eqs.\n    9 and 10 [1]\n\n    Parameters\n    ----------\n    nrows : int\n        number of rows in the resulting harmonic field\n    ncols : int\n        number of columns in the resulting harmonic field\n    b, m : float\n        parameters of the harmonic field (as in [1]). To understand the effect\n        of these parameters, please consider plotting a deformed image\n        (a circle or a grid) under the deformation field, or see examples\n        in [1]\n\n    Returns\n    -------\n    d : array, shape (nrows, ncols, 2)\n        the harmonic displacement field\n    inv : array, shape (nrows, ncols, 2)\n        the analitical inverse of the harmonic displacement field\n\n    [1] Chen, M., Lu, W., Chen, Q., Ruchala, K. J., & Olivera, G. H. (2008).\n        A simple fixed-point approach to invert a deformation field.\n        Medical Physics, 35(1), 81. doi:10.1118/1.2816107\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_63create_harmonic_fields_2d = {"create_harmonic_fields_2d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_63create_harmonic_fields_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_62create_harmonic_fields_2d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_63create_harmonic_fields_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  double __pyx_v_b;
  double __pyx_v_m;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_harmonic_fields_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_nrows,&__pyx_n_s_ncols,&__pyx_n_s_b,&__pyx_n_s_m,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_nrows)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ncols)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_harmonic_fields_2d", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2846; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_harmonic_fields_2d", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2846; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_m)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_harmonic_fields_2d", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2846; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_harmonic_fields_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2846; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_nrows = __Pyx_PyInt_As_Py_intptr_t(values[0]); if (unlikely((__pyx_v_nrows == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2846; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ncols = __Pyx_PyInt_As_Py_intptr_t(values[1]); if (unlikely((__pyx_v_ncols == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2846; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_b = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_b == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_m = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_m == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2847; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_harmonic_fields_2d", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2846; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.create_harmonic_fields_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_62create_harmonic_fields_2d(__pyx_self, __pyx_v_nrows, __pyx_v_ncols, __pyx_v_b, __pyx_v_m);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_62create_harmonic_fields_2d(CYTHON_UNUSED PyObject *__pyx_self, npy_intp __pyx_v_nrows, npy_intp __pyx_v_ncols, double __pyx_v_b, double __pyx_v_m) {
  npy_intp __pyx_v_mid_row;
  npy_intp __pyx_v_mid_col;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_theta;
  __Pyx_memviewslice __pyx_v_d = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inv = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_7;
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  double __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create_harmonic_fields_2d", 0);

  /* "dipy/align/vector_fields.pyx":2877
 *     """
 *     cdef:
 *         cnp.npy_intp mid_row = nrows/2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp mid_col = ncols/2
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_mid_row = (__pyx_v_nrows / 2);

  /* "dipy/align/vector_fields.pyx":2878
 *     cdef:
 *         cnp.npy_intp mid_row = nrows/2
 *         cnp.npy_intp mid_col = ncols/2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double theta
 */
  __pyx_v_mid_col = (__pyx_v_ncols / 2);

  /* "dipy/align/vector_fields.pyx":2881
 *         cnp.npy_intp i, j, ii, jj
 *         double theta
 *         double[:, :, :] d = np.zeros((nrows, ncols, 2), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:, :, :] inv = np.zeros((nrows, ncols, 2), dtype=np.float64)
 *     for i in range(nrows):
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2881; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2881; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2881; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2881; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2881; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_int_2);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2881; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2881; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2881; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_float64); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2881; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2881; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2881; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_5);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2881; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_d = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":2882
 *         double theta
 *         double[:, :, :] d = np.zeros((nrows, ncols, 2), dtype=np.float64)
 *         double[:, :, :] inv = np.zeros((nrows, ncols, 2), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     for i in range(nrows):
 *         for j in range(ncols):
 */
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_zeros); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_int_2);
  __pyx_t_5 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_float64); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_1);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2882; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inv = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":2883
 *         double[:, :, :] d = np.zeros((nrows, ncols, 2), dtype=np.float64)
 *         double[:, :, :] inv = np.zeros((nrows, ncols, 2), dtype=np.float64)
 *     for i in range(nrows):             # <<<<<<<<<<<<<<
 *         for j in range(ncols):
 *             ii = i - mid_row
 */
  __pyx_t_7 = __pyx_v_nrows;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
    __pyx_v_i = __pyx_t_8;

    /* "dipy/align/vector_fields.pyx":2884
 *         double[:, :, :] inv = np.zeros((nrows, ncols, 2), dtype=np.float64)
 *     for i in range(nrows):
 *         for j in range(ncols):             # <<<<<<<<<<<<<<
 *             ii = i - mid_row
 *             jj = j - mid_col
 */
    __pyx_t_9 = __pyx_v_ncols;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_v_j = __pyx_t_10;

      /* "dipy/align/vector_fields.pyx":2885
 *     for i in range(nrows):
 *         for j in range(ncols):
 *             ii = i - mid_row             # <<<<<<<<<<<<<<
 *             jj = j - mid_col
 *             theta = np.arctan2(ii, jj)
 */
      __pyx_v_ii = (__pyx_v_i - __pyx_v_mid_row);

      /* "dipy/align/vector_fields.pyx":2886
 *         for j in range(ncols):
 *             ii = i - mid_row
 *             jj = j - mid_col             # <<<<<<<<<<<<<<
 *             theta = np.arctan2(ii, jj)
 *             d[i, j, 0] = ii * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 */
      __pyx_v_jj = (__pyx_v_j - __pyx_v_mid_col);

      /* "dipy/align/vector_fields.pyx":2887
 *             ii = i - mid_row
 *             jj = j - mid_col
 *             theta = np.arctan2(ii, jj)             # <<<<<<<<<<<<<<
 *             d[i, j, 0] = ii * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 *             d[i, j, 1] = jj * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 */
      __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_arctan2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ii); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_jj); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = NULL;
      __pyx_t_11 = 0;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
          __pyx_t_11 = 1;
        }
      }
      __pyx_t_12 = PyTuple_New(2+__pyx_t_11); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_12);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_11, __pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_11, __pyx_t_4);
      __pyx_t_2 = 0;
      __pyx_t_4 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_12, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_13 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_13 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2887; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_v_theta = __pyx_t_13;

      /* "dipy/align/vector_fields.pyx":2888
 *             jj = j - mid_col
 *             theta = np.arctan2(ii, jj)
 *             d[i, j, 0] = ii * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)             # <<<<<<<<<<<<<<
 *             d[i, j, 1] = jj * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 *             inv[i, j, 0] = b * np.cos(m * theta) * ii
 */
      __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ii); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_3 = PyFloat_FromDouble(__pyx_v_b); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_cos); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = PyFloat_FromDouble((__pyx_v_m * __pyx_v_theta)); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      if (!__pyx_t_5) {
        __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_12);
      } else {
        __pyx_t_14 = PyTuple_New(1+1); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_14, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_14, NULL); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = PyNumber_Multiply(__pyx_t_3, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_12 = __Pyx_PyInt_AddCObj(__pyx_int_1, __pyx_t_2, 1, 0); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyFloat_DivideCObj(__pyx_float_1_0, __pyx_t_12, 1.0, 0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_12 = __Pyx_PyFloat_SubtractObjC(__pyx_t_2, __pyx_float_1_0, 1.0, 0); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = PyNumber_Multiply(__pyx_t_1, __pyx_t_12); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_13 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_13 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2888; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_15 = __pyx_v_i;
      __pyx_t_16 = __pyx_v_j;
      __pyx_t_17 = 0;
      *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_15 * __pyx_v_d.strides[0]) ) + __pyx_t_16 * __pyx_v_d.strides[1]) ) + __pyx_t_17 * __pyx_v_d.strides[2]) )) = __pyx_t_13;

      /* "dipy/align/vector_fields.pyx":2889
 *             theta = np.arctan2(ii, jj)
 *             d[i, j, 0] = ii * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 *             d[i, j, 1] = jj * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)             # <<<<<<<<<<<<<<
 *             inv[i, j, 0] = b * np.cos(m * theta) * ii
 *             inv[i, j, 1] = b * np.cos(m * theta) * jj
 */
      __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_jj); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_12 = PyFloat_FromDouble(__pyx_v_b); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_cos); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyFloat_FromDouble((__pyx_v_m * __pyx_v_theta)); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_14))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_14);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_14, function);
        }
      }
      if (!__pyx_t_4) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_14, __pyx_t_3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_14, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_14 = PyNumber_Multiply(__pyx_t_12, __pyx_t_1); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyInt_AddCObj(__pyx_int_1, __pyx_t_14, 1, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_14 = __Pyx_PyFloat_DivideCObj(__pyx_float_1_0, __pyx_t_1, 1.0, 0); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyFloat_SubtractObjC(__pyx_t_14, __pyx_float_1_0, 1.0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_14 = PyNumber_Multiply(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_13 = __pyx_PyFloat_AsDouble(__pyx_t_14); if (unlikely((__pyx_t_13 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_18 = __pyx_v_i;
      __pyx_t_19 = __pyx_v_j;
      __pyx_t_20 = 1;
      *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_18 * __pyx_v_d.strides[0]) ) + __pyx_t_19 * __pyx_v_d.strides[1]) ) + __pyx_t_20 * __pyx_v_d.strides[2]) )) = __pyx_t_13;

      /* "dipy/align/vector_fields.pyx":2890
 *             d[i, j, 0] = ii * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 *             d[i, j, 1] = jj * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 *             inv[i, j, 0] = b * np.cos(m * theta) * ii             # <<<<<<<<<<<<<<
 *             inv[i, j, 1] = b * np.cos(m * theta) * jj
 * 
 */
      __pyx_t_14 = PyFloat_FromDouble(__pyx_v_b); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2890; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2890; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_cos); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2890; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = PyFloat_FromDouble((__pyx_v_m * __pyx_v_theta)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2890; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_5 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_12))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_12);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_12, function);
        }
      }
      if (!__pyx_t_5) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2890; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else {
        __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2890; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_2);
        PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_2);
        __pyx_t_2 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2890; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_12 = PyNumber_Multiply(__pyx_t_14, __pyx_t_1); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2890; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ii); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2890; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_14 = PyNumber_Multiply(__pyx_t_12, __pyx_t_1); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2890; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_13 = __pyx_PyFloat_AsDouble(__pyx_t_14); if (unlikely((__pyx_t_13 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2890; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_21 = __pyx_v_i;
      __pyx_t_22 = __pyx_v_j;
      __pyx_t_23 = 0;
      *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_inv.data + __pyx_t_21 * __pyx_v_inv.strides[0]) ) + __pyx_t_22 * __pyx_v_inv.strides[1]) ) + __pyx_t_23 * __pyx_v_inv.strides[2]) )) = __pyx_t_13;

      /* "dipy/align/vector_fields.pyx":2891
 *             d[i, j, 1] = jj * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 *             inv[i, j, 0] = b * np.cos(m * theta) * ii
 *             inv[i, j, 1] = b * np.cos(m * theta) * jj             # <<<<<<<<<<<<<<
 * 
 *     return d, inv
 */
      __pyx_t_14 = PyFloat_FromDouble(__pyx_v_b); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2891; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_12 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2891; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_cos); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2891; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_12 = PyFloat_FromDouble((__pyx_v_m * __pyx_v_theta)); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2891; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_2 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_2) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_12); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2891; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2891; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
        __Pyx_GIVEREF(__pyx_t_12);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_12);
        __pyx_t_12 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2891; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyNumber_Multiply(__pyx_t_14, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2891; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_jj); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2891; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_14 = PyNumber_Multiply(__pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2891; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_13 = __pyx_PyFloat_AsDouble(__pyx_t_14); if (unlikely((__pyx_t_13 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2891; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
      __pyx_t_24 = __pyx_v_i;
      __pyx_t_25 = __pyx_v_j;
      __pyx_t_26 = 1;
      *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_inv.data + __pyx_t_24 * __pyx_v_inv.strides[0]) ) + __pyx_t_25 * __pyx_v_inv.strides[1]) ) + __pyx_t_26 * __pyx_v_inv.strides[2]) )) = __pyx_t_13;
    }
  }

  /* "dipy/align/vector_fields.pyx":2893
 *             inv[i, j, 1] = b * np.cos(m * theta) * jj
 * 
 *     return d, inv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_14 = __pyx_memoryview_fromslice(__pyx_v_d, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2893; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_inv, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2893; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2893; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_14);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_14);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);
  __pyx_t_14 = 0;
  __pyx_t_1 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2846
 * 
 * 
 * def create_harmonic_fields_2d(cnp.npy_intp nrows, cnp.npy_intp ncols,             # <<<<<<<<<<<<<<
 *                               double b, double m):
 *     r"""Creates an invertible 2D displacement field
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("dipy.align.vector_fields.create_harmonic_fields_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_d, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inv, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":2896
 * 
 * 
 * def create_harmonic_fields_3d(int nslices, cnp.npy_intp nrows,             # <<<<<<<<<<<<<<
 *                               cnp.npy_intp ncols, double b, double m):
 *     r"""Creates an invertible 3D displacement field
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_65create_harmonic_fields_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_64create_harmonic_fields_3d[] = "Creates an invertible 3D displacement field\n\n    Creates the invertible displacement fields used in Chen et al. eqs.\n    9 and 10 [1] computing the angle theta along z-slides.\n\n    Parameters\n    ----------\n    nslices : int\n        number of slices in the resulting harmonic field\n    nrows : int\n        number of rows in the resulting harmonic field\n    ncols : int\n        number of columns in the resulting harmonic field\n    b, f : float\n        parameters of the harmonic field (as in [1]). To understand the effect\n        of these parameters, please consider plotting a deformed image\n        (e.g. a circle or a grid) under the deformation field, or see examples\n        in [1]\n\n    Returns\n    -------\n    d : array, shape (nslices, nrows, ncols, 3)\n        the harmonic displacement field\n    inv : array, shape (nslices, nrows, ncols, 3)\n        the analitical inverse of the harmonic displacement field\n\n    [1] Chen, M., Lu, W., Chen, Q., Ruchala, K. J., & Olivera, G. H. (2008).\n        A simple fixed-point approach to invert a deformation field.\n        Medical Physics, 35(1), 81. doi:10.1118/1.2816107\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_65create_harmonic_fields_3d = {"create_harmonic_fields_3d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_65create_harmonic_fields_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_64create_harmonic_fields_3d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_65create_harmonic_fields_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  double __pyx_v_b;
  double __pyx_v_m;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_harmonic_fields_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_nslices,&__pyx_n_s_nrows,&__pyx_n_s_ncols,&__pyx_n_s_b,&__pyx_n_s_m,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_nslices)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_nrows)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_harmonic_fields_3d", 1, 5, 5, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2896; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ncols)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_harmonic_fields_3d", 1, 5, 5, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2896; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_b)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_harmonic_fields_3d", 1, 5, 5, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2896; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_m)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_harmonic_fields_3d", 1, 5, 5, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2896; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_harmonic_fields_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2896; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_nslices = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_nslices == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2896; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_nrows = __Pyx_PyInt_As_Py_intptr_t(values[1]); if (unlikely((__pyx_v_nrows == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2896; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ncols = __Pyx_PyInt_As_Py_intptr_t(values[2]); if (unlikely((__pyx_v_ncols == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2897; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_b = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_b == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2897; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_m = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_m == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2897; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_harmonic_fields_3d", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2896; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.create_harmonic_fields_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_64create_harmonic_fields_3d(__pyx_self, __pyx_v_nslices, __pyx_v_nrows, __pyx_v_ncols, __pyx_v_b, __pyx_v_m);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_64create_harmonic_fields_3d(CYTHON_UNUSED PyObject *__pyx_self, int __pyx_v_nslices, npy_intp __pyx_v_nrows, npy_intp __pyx_v_ncols, double __pyx_v_b, double __pyx_v_m) {
  npy_intp __pyx_v_mid_slice;
  npy_intp __pyx_v_mid_row;
  npy_intp __pyx_v_mid_col;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  npy_intp __pyx_v_kk;
  double __pyx_v_theta;
  __Pyx_memviewslice __pyx_v_d = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inv = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_t_7;
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  npy_intp __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  double __pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create_harmonic_fields_3d", 0);

  /* "dipy/align/vector_fields.pyx":2929
 *     """
 *     cdef:
 *         cnp.npy_intp mid_slice = nslices / 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp mid_row = nrows / 2
 *         cnp.npy_intp mid_col = ncols / 2
 */
  __pyx_v_mid_slice = (__pyx_v_nslices / 2);

  /* "dipy/align/vector_fields.pyx":2930
 *     cdef:
 *         cnp.npy_intp mid_slice = nslices / 2
 *         cnp.npy_intp mid_row = nrows / 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp mid_col = ncols / 2
 *         cnp.npy_intp i, j, k, ii, jj, kk
 */
  __pyx_v_mid_row = (__pyx_v_nrows / 2);

  /* "dipy/align/vector_fields.pyx":2931
 *         cnp.npy_intp mid_slice = nslices / 2
 *         cnp.npy_intp mid_row = nrows / 2
 *         cnp.npy_intp mid_col = ncols / 2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         double theta
 */
  __pyx_v_mid_col = (__pyx_v_ncols / 2);

  /* "dipy/align/vector_fields.pyx":2934
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         double theta
 *         double[:, :, :, :] d = np.zeros((nslices, nrows, ncols, 3),             # <<<<<<<<<<<<<<
 *                                         dtype=np.float64)
 *         double[:, :, :, :] inv = np.zeros((nslices, nrows, ncols, 3),
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2934; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2934; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_nslices); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2934; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2934; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2934; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2934; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_4);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_int_3);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2934; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __pyx_t_5 = 0;

  /* "dipy/align/vector_fields.pyx":2935
 *         double theta
 *         double[:, :, :, :] d = np.zeros((nslices, nrows, ncols, 3),
 *                                         dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:, :, :, :] inv = np.zeros((nslices, nrows, ncols, 3),
 *                                           dtype=np.float64)
 */
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2935; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2935; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_float64); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2935; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2935; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "dipy/align/vector_fields.pyx":2934
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         double theta
 *         double[:, :, :, :] d = np.zeros((nslices, nrows, ncols, 3),             # <<<<<<<<<<<<<<
 *                                         dtype=np.float64)
 *         double[:, :, :, :] inv = np.zeros((nslices, nrows, ncols, 3),
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2934; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_t_1);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2934; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_d = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":2936
 *         double[:, :, :, :] d = np.zeros((nslices, nrows, ncols, 3),
 *                                         dtype=np.float64)
 *         double[:, :, :, :] inv = np.zeros((nslices, nrows, ncols, 3),             # <<<<<<<<<<<<<<
 *                                           dtype=np.float64)
 *     for k in range(nslices):
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2936; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2936; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_nslices); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2936; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2936; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2936; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2936; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_int_3);
  __pyx_t_1 = 0;
  __pyx_t_4 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2936; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":2937
 *                                         dtype=np.float64)
 *         double[:, :, :, :] inv = np.zeros((nslices, nrows, ncols, 3),
 *                                           dtype=np.float64)             # <<<<<<<<<<<<<<
 *     for k in range(nslices):
 *         for i in range(nrows):
 */
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2937; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2937; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_float64); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2937; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2937; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "dipy/align/vector_fields.pyx":2936
 *         double[:, :, :, :] d = np.zeros((nslices, nrows, ncols, 3),
 *                                         dtype=np.float64)
 *         double[:, :, :, :] inv = np.zeros((nslices, nrows, ncols, 3),             # <<<<<<<<<<<<<<
 *                                           dtype=np.float64)
 *     for k in range(nslices):
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2936; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(__pyx_t_1);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2936; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inv = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":2938
 *         double[:, :, :, :] inv = np.zeros((nslices, nrows, ncols, 3),
 *                                           dtype=np.float64)
 *     for k in range(nslices):             # <<<<<<<<<<<<<<
 *         for i in range(nrows):
 *             for j in range(ncols):
 */
  __pyx_t_7 = __pyx_v_nslices;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
    __pyx_v_k = __pyx_t_8;

    /* "dipy/align/vector_fields.pyx":2939
 *                                           dtype=np.float64)
 *     for k in range(nslices):
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 kk = k - mid_slice
 */
    __pyx_t_9 = __pyx_v_nrows;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_v_i = __pyx_t_10;

      /* "dipy/align/vector_fields.pyx":2940
 *     for k in range(nslices):
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 kk = k - mid_slice
 *                 ii = i - mid_row
 */
      __pyx_t_11 = __pyx_v_ncols;
      for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
        __pyx_v_j = __pyx_t_12;

        /* "dipy/align/vector_fields.pyx":2941
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 kk = k - mid_slice             # <<<<<<<<<<<<<<
 *                 ii = i - mid_row
 *                 jj = j - mid_col
 */
        __pyx_v_kk = (__pyx_v_k - __pyx_v_mid_slice);

        /* "dipy/align/vector_fields.pyx":2942
 *             for j in range(ncols):
 *                 kk = k - mid_slice
 *                 ii = i - mid_row             # <<<<<<<<<<<<<<
 *                 jj = j - mid_col
 *                 theta = np.arctan2(ii, jj)
 */
        __pyx_v_ii = (__pyx_v_i - __pyx_v_mid_row);

        /* "dipy/align/vector_fields.pyx":2943
 *                 kk = k - mid_slice
 *                 ii = i - mid_row
 *                 jj = j - mid_col             # <<<<<<<<<<<<<<
 *                 theta = np.arctan2(ii, jj)
 *                 d[k, i, j, 0] = kk * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 */
        __pyx_v_jj = (__pyx_v_j - __pyx_v_mid_col);

        /* "dipy/align/vector_fields.pyx":2944
 *                 ii = i - mid_row
 *                 jj = j - mid_col
 *                 theta = np.arctan2(ii, jj)             # <<<<<<<<<<<<<<
 *                 d[k, i, j, 0] = kk * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 *                 d[k, i, j, 1] = ii * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 */
        __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2944; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_arctan2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2944; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ii); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2944; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_jj); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2944; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_4 = NULL;
        __pyx_t_13 = 0;
        if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_2))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_2, function);
            __pyx_t_13 = 1;
          }
        }
        __pyx_t_14 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2944; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_14);
        if (__pyx_t_4) {
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_4); __pyx_t_4 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_14, 0+__pyx_t_13, __pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_14, 1+__pyx_t_13, __pyx_t_5);
        __pyx_t_3 = 0;
        __pyx_t_5 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_14, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2944; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_15 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_15 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2944; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_v_theta = __pyx_t_15;

        /* "dipy/align/vector_fields.pyx":2945
 *                 jj = j - mid_col
 *                 theta = np.arctan2(ii, jj)
 *                 d[k, i, j, 0] = kk * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)             # <<<<<<<<<<<<<<
 *                 d[k, i, j, 1] = ii * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 *                 d[k, i, j, 2] = jj * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 */
        __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_kk); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_2 = PyFloat_FromDouble(__pyx_v_b); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_cos); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = PyFloat_FromDouble((__pyx_v_m * __pyx_v_theta)); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_4 = NULL;
        if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        if (!__pyx_t_4) {
          __pyx_t_14 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_14);
        } else {
          __pyx_t_16 = PyTuple_New(1+1); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_16);
          __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_t_4); __pyx_t_4 = NULL;
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_16, 0+1, __pyx_t_5);
          __pyx_t_5 = 0;
          __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_16, NULL); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_14);
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = PyNumber_Multiply(__pyx_t_2, __pyx_t_14); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __pyx_t_14 = __Pyx_PyInt_AddCObj(__pyx_int_1, __pyx_t_3, 1, 0); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyFloat_DivideCObj(__pyx_float_1_0, __pyx_t_14, 1.0, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __pyx_t_14 = __Pyx_PyFloat_SubtractObjC(__pyx_t_3, __pyx_float_1_0, 1.0, 0); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = PyNumber_Multiply(__pyx_t_1, __pyx_t_14); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __pyx_t_15 = __pyx_PyFloat_AsDouble(__pyx_t_3); if (unlikely((__pyx_t_15 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_17 = __pyx_v_k;
        __pyx_t_18 = __pyx_v_i;
        __pyx_t_19 = __pyx_v_j;
        __pyx_t_20 = 0;
        *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_17 * __pyx_v_d.strides[0]) ) + __pyx_t_18 * __pyx_v_d.strides[1]) ) + __pyx_t_19 * __pyx_v_d.strides[2]) ) + __pyx_t_20 * __pyx_v_d.strides[3]) )) = __pyx_t_15;

        /* "dipy/align/vector_fields.pyx":2946
 *                 theta = np.arctan2(ii, jj)
 *                 d[k, i, j, 0] = kk * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 *                 d[k, i, j, 1] = ii * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)             # <<<<<<<<<<<<<<
 *                 d[k, i, j, 2] = jj * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 *                 inv[k, i, j, 0] = b * np.cos(m * theta) * kk
 */
        __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ii); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_14 = PyFloat_FromDouble(__pyx_v_b); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_cos); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_2 = PyFloat_FromDouble((__pyx_v_m * __pyx_v_theta)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_5 = NULL;
        if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_16))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_16);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_16);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_16, function);
          }
        }
        if (!__pyx_t_5) {
          __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_16, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_GOTREF(__pyx_t_1);
        } else {
          __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
          __Pyx_GIVEREF(__pyx_t_2);
          PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_t_2);
          __pyx_t_2 = 0;
          __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_16, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        }
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __pyx_t_16 = PyNumber_Multiply(__pyx_t_14, __pyx_t_1); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = __Pyx_PyInt_AddCObj(__pyx_int_1, __pyx_t_16, 1, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __pyx_t_16 = __Pyx_PyFloat_DivideCObj(__pyx_float_1_0, __pyx_t_1, 1.0, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = __Pyx_PyFloat_SubtractObjC(__pyx_t_16, __pyx_float_1_0, 1.0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __pyx_t_16 = PyNumber_Multiply(__pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_15 = __pyx_PyFloat_AsDouble(__pyx_t_16); if (unlikely((__pyx_t_15 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2946; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __pyx_t_21 = __pyx_v_k;
        __pyx_t_22 = __pyx_v_i;
        __pyx_t_23 = __pyx_v_j;
        __pyx_t_24 = 1;
        *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_21 * __pyx_v_d.strides[0]) ) + __pyx_t_22 * __pyx_v_d.strides[1]) ) + __pyx_t_23 * __pyx_v_d.strides[2]) ) + __pyx_t_24 * __pyx_v_d.strides[3]) )) = __pyx_t_15;

        /* "dipy/align/vector_fields.pyx":2947
 *                 d[k, i, j, 0] = kk * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 *                 d[k, i, j, 1] = ii * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 *                 d[k, i, j, 2] = jj * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)             # <<<<<<<<<<<<<<
 *                 inv[k, i, j, 0] = b * np.cos(m * theta) * kk
 *                 inv[k, i, j, 1] = b * np.cos(m * theta) * ii
 */
        __pyx_t_16 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_jj); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_1 = PyFloat_FromDouble(__pyx_v_b); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_14 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_14, __pyx_n_s_cos); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __pyx_t_14 = PyFloat_FromDouble((__pyx_v_m * __pyx_v_theta)); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_2 = NULL;
        if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_2)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_2);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        if (!__pyx_t_2) {
          __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_14); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_GOTREF(__pyx_t_3);
        } else {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
          __Pyx_GIVEREF(__pyx_t_14);
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_14);
          __pyx_t_14 = 0;
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = PyNumber_Multiply(__pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyInt_AddCObj(__pyx_int_1, __pyx_t_4, 1, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = __Pyx_PyFloat_DivideCObj(__pyx_float_1_0, __pyx_t_3, 1.0, 0); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyFloat_SubtractObjC(__pyx_t_4, __pyx_float_1_0, 1.0, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_4 = PyNumber_Multiply(__pyx_t_16, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_15 = __pyx_PyFloat_AsDouble(__pyx_t_4); if (unlikely((__pyx_t_15 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_25 = __pyx_v_k;
        __pyx_t_26 = __pyx_v_i;
        __pyx_t_27 = __pyx_v_j;
        __pyx_t_28 = 2;
        *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_d.data + __pyx_t_25 * __pyx_v_d.strides[0]) ) + __pyx_t_26 * __pyx_v_d.strides[1]) ) + __pyx_t_27 * __pyx_v_d.strides[2]) ) + __pyx_t_28 * __pyx_v_d.strides[3]) )) = __pyx_t_15;

        /* "dipy/align/vector_fields.pyx":2948
 *                 d[k, i, j, 1] = ii * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 *                 d[k, i, j, 2] = jj * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 *                 inv[k, i, j, 0] = b * np.cos(m * theta) * kk             # <<<<<<<<<<<<<<
 *                 inv[k, i, j, 1] = b * np.cos(m * theta) * ii
 *                 inv[k, i, j, 2] = b * np.cos(m * theta) * jj
 */
        __pyx_t_4 = PyFloat_FromDouble(__pyx_v_b); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2948; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_16 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2948; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_16, __pyx_n_s_cos); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2948; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __pyx_t_16 = PyFloat_FromDouble((__pyx_v_m * __pyx_v_theta)); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2948; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_5 = NULL;
        if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_1))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_1, function);
          }
        }
        if (!__pyx_t_5) {
          __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_16); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2948; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
          __Pyx_GOTREF(__pyx_t_3);
        } else {
          __pyx_t_14 = PyTuple_New(1+1); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2948; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_14);
          __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_5); __pyx_t_5 = NULL;
          __Pyx_GIVEREF(__pyx_t_16);
          PyTuple_SET_ITEM(__pyx_t_14, 0+1, __pyx_t_16);
          __pyx_t_16 = 0;
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_14, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2948; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        }
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = PyNumber_Multiply(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2948; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_kk); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2948; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyNumber_Multiply(__pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2948; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_15 = __pyx_PyFloat_AsDouble(__pyx_t_4); if (unlikely((__pyx_t_15 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2948; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_29 = __pyx_v_k;
        __pyx_t_30 = __pyx_v_i;
        __pyx_t_31 = __pyx_v_j;
        __pyx_t_32 = 0;
        *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_inv.data + __pyx_t_29 * __pyx_v_inv.strides[0]) ) + __pyx_t_30 * __pyx_v_inv.strides[1]) ) + __pyx_t_31 * __pyx_v_inv.strides[2]) ) + __pyx_t_32 * __pyx_v_inv.strides[3]) )) = __pyx_t_15;

        /* "dipy/align/vector_fields.pyx":2949
 *                 d[k, i, j, 2] = jj * (1.0 / (1 + b * np.cos(m * theta)) - 1.0)
 *                 inv[k, i, j, 0] = b * np.cos(m * theta) * kk
 *                 inv[k, i, j, 1] = b * np.cos(m * theta) * ii             # <<<<<<<<<<<<<<
 *                 inv[k, i, j, 2] = b * np.cos(m * theta) * jj
 * 
 */
        __pyx_t_4 = PyFloat_FromDouble(__pyx_v_b); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_cos); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = PyFloat_FromDouble((__pyx_v_m * __pyx_v_theta)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_16 = NULL;
        if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_14))) {
          __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_14);
          if (likely(__pyx_t_16)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
            __Pyx_INCREF(__pyx_t_16);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_14, function);
          }
        }
        if (!__pyx_t_16) {
          __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_14, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_GOTREF(__pyx_t_3);
        } else {
          __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GIVEREF(__pyx_t_16); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_16); __pyx_t_16 = NULL;
          __Pyx_GIVEREF(__pyx_t_1);
          PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_1);
          __pyx_t_1 = 0;
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_14, __pyx_t_5, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        }
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __pyx_t_14 = PyNumber_Multiply(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ii); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyNumber_Multiply(__pyx_t_14, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_15 = __pyx_PyFloat_AsDouble(__pyx_t_4); if (unlikely((__pyx_t_15 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2949; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_33 = __pyx_v_k;
        __pyx_t_34 = __pyx_v_i;
        __pyx_t_35 = __pyx_v_j;
        __pyx_t_36 = 1;
        *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_inv.data + __pyx_t_33 * __pyx_v_inv.strides[0]) ) + __pyx_t_34 * __pyx_v_inv.strides[1]) ) + __pyx_t_35 * __pyx_v_inv.strides[2]) ) + __pyx_t_36 * __pyx_v_inv.strides[3]) )) = __pyx_t_15;

        /* "dipy/align/vector_fields.pyx":2950
 *                 inv[k, i, j, 0] = b * np.cos(m * theta) * kk
 *                 inv[k, i, j, 1] = b * np.cos(m * theta) * ii
 *                 inv[k, i, j, 2] = b * np.cos(m * theta) * jj             # <<<<<<<<<<<<<<
 * 
 *     return d, inv
 */
        __pyx_t_4 = PyFloat_FromDouble(__pyx_v_b); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2950; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_14 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2950; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_14, __pyx_n_s_cos); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2950; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __pyx_t_14 = PyFloat_FromDouble((__pyx_v_m * __pyx_v_theta)); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2950; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_1 = NULL;
        if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_1)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_1);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
          }
        }
        if (!__pyx_t_1) {
          __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_14); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2950; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_GOTREF(__pyx_t_3);
        } else {
          __pyx_t_16 = PyTuple_New(1+1); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2950; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_16);
          __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_16, 0, __pyx_t_1); __pyx_t_1 = NULL;
          __Pyx_GIVEREF(__pyx_t_14);
          PyTuple_SET_ITEM(__pyx_t_16, 0+1, __pyx_t_14);
          __pyx_t_14 = 0;
          __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_16, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2950; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = PyNumber_Multiply(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2950; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_jj); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2950; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyNumber_Multiply(__pyx_t_5, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2950; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_15 = __pyx_PyFloat_AsDouble(__pyx_t_4); if (unlikely((__pyx_t_15 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2950; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_37 = __pyx_v_k;
        __pyx_t_38 = __pyx_v_i;
        __pyx_t_39 = __pyx_v_j;
        __pyx_t_40 = 2;
        *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_inv.data + __pyx_t_37 * __pyx_v_inv.strides[0]) ) + __pyx_t_38 * __pyx_v_inv.strides[1]) ) + __pyx_t_39 * __pyx_v_inv.strides[2]) ) + __pyx_t_40 * __pyx_v_inv.strides[3]) )) = __pyx_t_15;
      }
    }
  }

  /* "dipy/align/vector_fields.pyx":2952
 *                 inv[k, i, j, 2] = b * np.cos(m * theta) * jj
 * 
 *     return d, inv             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __pyx_memoryview_fromslice(__pyx_v_d, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2952; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_inv, 4, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2952; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2952; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);
  __pyx_t_4 = 0;
  __pyx_t_3 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2896
 * 
 * 
 * def create_harmonic_fields_3d(int nslices, cnp.npy_intp nrows,             # <<<<<<<<<<<<<<
 *                               cnp.npy_intp ncols, double b, double m):
 *     r"""Creates an invertible 3D displacement field
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_AddTraceback("dipy.align.vector_fields.create_harmonic_fields_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_d, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inv, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":2955
 * 
 * 
 * def create_circle(cnp.npy_intp nrows, cnp.npy_intp ncols, cnp.npy_intp radius):             # <<<<<<<<<<<<<<
 *     r"""
 *     Create a binary 2D image where pixel values are 1 iff their distance
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_67create_circle(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_66create_circle[] = "\n    Create a binary 2D image where pixel values are 1 iff their distance\n    to the center of the image is less than or equal to radius.\n\n    Parameters\n    ----------\n    nrows : int\n        number of rows of the resulting image\n    ncols : int\n        number of columns of the resulting image\n    radius : int\n        the radius of the circle\n\n    Returns\n    -------\n    c : array, shape (nrows, ncols)\n        the binary image of the circle with the requested dimensions\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_67create_circle = {"create_circle", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_67create_circle, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_66create_circle};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_67create_circle(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  npy_intp __pyx_v_radius;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_circle (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_nrows,&__pyx_n_s_ncols,&__pyx_n_s_radius,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_nrows)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ncols)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_circle", 1, 3, 3, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2955; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_radius)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_circle", 1, 3, 3, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2955; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_circle") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2955; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_nrows = __Pyx_PyInt_As_Py_intptr_t(values[0]); if (unlikely((__pyx_v_nrows == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2955; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ncols = __Pyx_PyInt_As_Py_intptr_t(values[1]); if (unlikely((__pyx_v_ncols == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2955; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_radius = __Pyx_PyInt_As_Py_intptr_t(values[2]); if (unlikely((__pyx_v_radius == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2955; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_circle", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2955; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.create_circle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_66create_circle(__pyx_self, __pyx_v_nrows, __pyx_v_ncols, __pyx_v_radius);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_66create_circle(CYTHON_UNUSED PyObject *__pyx_self, npy_intp __pyx_v_nrows, npy_intp __pyx_v_ncols, npy_intp __pyx_v_radius) {
  npy_intp __pyx_v_mid_row;
  npy_intp __pyx_v_mid_col;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  double __pyx_v_r;
  __Pyx_memviewslice __pyx_v_c = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_7;
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  double __pyx_t_11;
  int __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create_circle", 0);

  /* "dipy/align/vector_fields.pyx":2975
 *     """
 *     cdef:
 *         cnp.npy_intp mid_row = nrows/2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp mid_col = ncols/2
 *         cnp.npy_intp i, j, ii, jj
 */
  __pyx_v_mid_row = (__pyx_v_nrows / 2);

  /* "dipy/align/vector_fields.pyx":2976
 *     cdef:
 *         cnp.npy_intp mid_row = nrows/2
 *         cnp.npy_intp mid_col = ncols/2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, ii, jj
 *         double r
 */
  __pyx_v_mid_col = (__pyx_v_ncols / 2);

  /* "dipy/align/vector_fields.pyx":2979
 *         cnp.npy_intp i, j, ii, jj
 *         double r
 *         double[:, :] c = np.zeros((nrows, ncols), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     for i in range(nrows):
 *         for j in range(ncols):
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2979; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2979; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2979; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2979; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2979; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2979; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2979; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2979; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_float64); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2979; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_5) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2979; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2979; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_t_5);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2979; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_c = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":2980
 *         double r
 *         double[:, :] c = np.zeros((nrows, ncols), dtype=np.float64)
 *     for i in range(nrows):             # <<<<<<<<<<<<<<
 *         for j in range(ncols):
 *             ii = i - mid_row
 */
  __pyx_t_7 = __pyx_v_nrows;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
    __pyx_v_i = __pyx_t_8;

    /* "dipy/align/vector_fields.pyx":2981
 *         double[:, :] c = np.zeros((nrows, ncols), dtype=np.float64)
 *     for i in range(nrows):
 *         for j in range(ncols):             # <<<<<<<<<<<<<<
 *             ii = i - mid_row
 *             jj = j - mid_col
 */
    __pyx_t_9 = __pyx_v_ncols;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_v_j = __pyx_t_10;

      /* "dipy/align/vector_fields.pyx":2982
 *     for i in range(nrows):
 *         for j in range(ncols):
 *             ii = i - mid_row             # <<<<<<<<<<<<<<
 *             jj = j - mid_col
 *             r = np.sqrt(ii*ii + jj*jj)
 */
      __pyx_v_ii = (__pyx_v_i - __pyx_v_mid_row);

      /* "dipy/align/vector_fields.pyx":2983
 *         for j in range(ncols):
 *             ii = i - mid_row
 *             jj = j - mid_col             # <<<<<<<<<<<<<<
 *             r = np.sqrt(ii*ii + jj*jj)
 *             if r <= radius:
 */
      __pyx_v_jj = (__pyx_v_j - __pyx_v_mid_col);

      /* "dipy/align/vector_fields.pyx":2984
 *             ii = i - mid_row
 *             jj = j - mid_col
 *             r = np.sqrt(ii*ii + jj*jj)             # <<<<<<<<<<<<<<
 *             if r <= radius:
 *                 c[i, j] = 1
 */
      __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2984; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2984; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(((__pyx_v_ii * __pyx_v_ii) + (__pyx_v_jj * __pyx_v_jj))); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2984; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_2 = NULL;
      if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (!__pyx_t_2) {
        __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2984; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_5);
      } else {
        __pyx_t_1 = PyTuple_New(1+1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2984; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2); __pyx_t_2 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_1, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_1, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2984; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_11 = __pyx_PyFloat_AsDouble(__pyx_t_5); if (unlikely((__pyx_t_11 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2984; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_v_r = __pyx_t_11;

      /* "dipy/align/vector_fields.pyx":2985
 *             jj = j - mid_col
 *             r = np.sqrt(ii*ii + jj*jj)
 *             if r <= radius:             # <<<<<<<<<<<<<<
 *                 c[i, j] = 1
 *             else:
 */
      __pyx_t_12 = ((__pyx_v_r <= __pyx_v_radius) != 0);
      if (__pyx_t_12) {

        /* "dipy/align/vector_fields.pyx":2986
 *             r = np.sqrt(ii*ii + jj*jj)
 *             if r <= radius:
 *                 c[i, j] = 1             # <<<<<<<<<<<<<<
 *             else:
 *                 c[i, j] = 0
 */
        __pyx_t_13 = __pyx_v_i;
        __pyx_t_14 = __pyx_v_j;
        *((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_c.data + __pyx_t_13 * __pyx_v_c.strides[0]) ) + __pyx_t_14 * __pyx_v_c.strides[1]) )) = 1.0;

        /* "dipy/align/vector_fields.pyx":2985
 *             jj = j - mid_col
 *             r = np.sqrt(ii*ii + jj*jj)
 *             if r <= radius:             # <<<<<<<<<<<<<<
 *                 c[i, j] = 1
 *             else:
 */
        goto __pyx_L7;
      }

      /* "dipy/align/vector_fields.pyx":2988
 *                 c[i, j] = 1
 *             else:
 *                 c[i, j] = 0             # <<<<<<<<<<<<<<
 *     return c
 * 
 */
      /*else*/ {
        __pyx_t_15 = __pyx_v_i;
        __pyx_t_16 = __pyx_v_j;
        *((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_c.data + __pyx_t_15 * __pyx_v_c.strides[0]) ) + __pyx_t_16 * __pyx_v_c.strides[1]) )) = 0.0;
      }
      __pyx_L7:;
    }
  }

  /* "dipy/align/vector_fields.pyx":2989
 *             else:
 *                 c[i, j] = 0
 *     return c             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __pyx_memoryview_fromslice(__pyx_v_c, 2, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2989; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2955
 * 
 * 
 * def create_circle(cnp.npy_intp nrows, cnp.npy_intp ncols, cnp.npy_intp radius):             # <<<<<<<<<<<<<<
 *     r"""
 *     Create a binary 2D image where pixel values are 1 iff their distance
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.create_circle", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_c, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":2992
 * 
 * 
 * def create_sphere(cnp.npy_intp nslices, cnp.npy_intp nrows,             # <<<<<<<<<<<<<<
 *                   cnp.npy_intp ncols, cnp.npy_intp radius):
 *     r"""
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_69create_sphere(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_68create_sphere[] = "\n    Create a binary 3D image where voxel values are 1 iff their distance\n    to the center of the image is less than or equal to radius.\n\n    Parameters\n    ----------\n    nslices : int\n        number if slices of the resulting image\n    nrows : int\n        number of rows of the resulting image\n    ncols : int\n        number of columns of the resulting image\n    radius : int\n        the radius of the sphere\n\n    Returns\n    -------\n    c : array, shape (nslices, nrows, ncols)\n        the binary image of the sphere with the requested dimensions\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_69create_sphere = {"create_sphere", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_69create_sphere, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_68create_sphere};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_69create_sphere(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  npy_intp __pyx_v_nslices;
  npy_intp __pyx_v_nrows;
  npy_intp __pyx_v_ncols;
  npy_intp __pyx_v_radius;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("create_sphere (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_nslices,&__pyx_n_s_nrows,&__pyx_n_s_ncols,&__pyx_n_s_radius,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_nslices)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_nrows)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_sphere", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2992; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_ncols)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_sphere", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2992; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_radius)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("create_sphere", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2992; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "create_sphere") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2992; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_nslices = __Pyx_PyInt_As_Py_intptr_t(values[0]); if (unlikely((__pyx_v_nslices == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2992; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_nrows = __Pyx_PyInt_As_Py_intptr_t(values[1]); if (unlikely((__pyx_v_nrows == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2992; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_ncols = __Pyx_PyInt_As_Py_intptr_t(values[2]); if (unlikely((__pyx_v_ncols == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2993; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_radius = __Pyx_PyInt_As_Py_intptr_t(values[3]); if (unlikely((__pyx_v_radius == (npy_intp)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2993; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("create_sphere", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2992; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.create_sphere", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_68create_sphere(__pyx_self, __pyx_v_nslices, __pyx_v_nrows, __pyx_v_ncols, __pyx_v_radius);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_68create_sphere(CYTHON_UNUSED PyObject *__pyx_self, npy_intp __pyx_v_nslices, npy_intp __pyx_v_nrows, npy_intp __pyx_v_ncols, npy_intp __pyx_v_radius) {
  npy_intp __pyx_v_mid_slice;
  npy_intp __pyx_v_mid_row;
  npy_intp __pyx_v_mid_col;
  npy_intp __pyx_v_i;
  npy_intp __pyx_v_j;
  npy_intp __pyx_v_k;
  npy_intp __pyx_v_ii;
  npy_intp __pyx_v_jj;
  npy_intp __pyx_v_kk;
  double __pyx_v_r;
  __Pyx_memviewslice __pyx_v_s = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_memviewslice __pyx_t_6 = { 0, 0, { 0 }, { 0 }, { 0 } };
  npy_intp __pyx_t_7;
  npy_intp __pyx_t_8;
  npy_intp __pyx_t_9;
  npy_intp __pyx_t_10;
  npy_intp __pyx_t_11;
  npy_intp __pyx_t_12;
  double __pyx_t_13;
  int __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create_sphere", 0);

  /* "dipy/align/vector_fields.pyx":3015
 *     """
 *     cdef:
 *         cnp.npy_intp mid_slice = nslices/2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp mid_row = nrows/2
 *         cnp.npy_intp mid_col = ncols/2
 */
  __pyx_v_mid_slice = (__pyx_v_nslices / 2);

  /* "dipy/align/vector_fields.pyx":3016
 *     cdef:
 *         cnp.npy_intp mid_slice = nslices/2
 *         cnp.npy_intp mid_row = nrows/2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp mid_col = ncols/2
 *         cnp.npy_intp i, j, k, ii, jj, kk
 */
  __pyx_v_mid_row = (__pyx_v_nrows / 2);

  /* "dipy/align/vector_fields.pyx":3017
 *         cnp.npy_intp mid_slice = nslices/2
 *         cnp.npy_intp mid_row = nrows/2
 *         cnp.npy_intp mid_col = ncols/2             # <<<<<<<<<<<<<<
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         double r
 */
  __pyx_v_mid_col = (__pyx_v_ncols / 2);

  /* "dipy/align/vector_fields.pyx":3020
 *         cnp.npy_intp i, j, k, ii, jj, kk
 *         double r
 *         double[:, :, :] s = np.zeros((nslices, nrows, ncols), dtype=np.float64)             # <<<<<<<<<<<<<<
 * 
 *     for k in range(nslices):
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3020; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_zeros); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3020; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nslices); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3020; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_nrows); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3020; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t(__pyx_v_ncols); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3020; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3020; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3020; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3020; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3020; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_float64); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3020; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3020; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3020; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_6 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_t_1);
  if (unlikely(!__pyx_t_6.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3020; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_s = __pyx_t_6;
  __pyx_t_6.memview = NULL;
  __pyx_t_6.data = NULL;

  /* "dipy/align/vector_fields.pyx":3022
 *         double[:, :, :] s = np.zeros((nslices, nrows, ncols), dtype=np.float64)
 * 
 *     for k in range(nslices):             # <<<<<<<<<<<<<<
 *         for i in range(nrows):
 *             for j in range(ncols):
 */
  __pyx_t_7 = __pyx_v_nslices;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
    __pyx_v_k = __pyx_t_8;

    /* "dipy/align/vector_fields.pyx":3023
 * 
 *     for k in range(nslices):
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 kk = k - mid_slice
 */
    __pyx_t_9 = __pyx_v_nrows;
    for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
      __pyx_v_i = __pyx_t_10;

      /* "dipy/align/vector_fields.pyx":3024
 *     for k in range(nslices):
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 kk = k - mid_slice
 *                 ii = i - mid_row
 */
      __pyx_t_11 = __pyx_v_ncols;
      for (__pyx_t_12 = 0; __pyx_t_12 < __pyx_t_11; __pyx_t_12+=1) {
        __pyx_v_j = __pyx_t_12;

        /* "dipy/align/vector_fields.pyx":3025
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 kk = k - mid_slice             # <<<<<<<<<<<<<<
 *                 ii = i - mid_row
 *                 jj = j - mid_col
 */
        __pyx_v_kk = (__pyx_v_k - __pyx_v_mid_slice);

        /* "dipy/align/vector_fields.pyx":3026
 *             for j in range(ncols):
 *                 kk = k - mid_slice
 *                 ii = i - mid_row             # <<<<<<<<<<<<<<
 *                 jj = j - mid_col
 *                 r = np.sqrt(ii*ii + jj*jj + kk*kk)
 */
        __pyx_v_ii = (__pyx_v_i - __pyx_v_mid_row);

        /* "dipy/align/vector_fields.pyx":3027
 *                 kk = k - mid_slice
 *                 ii = i - mid_row
 *                 jj = j - mid_col             # <<<<<<<<<<<<<<
 *                 r = np.sqrt(ii*ii + jj*jj + kk*kk)
 *                 if r <= radius:
 */
        __pyx_v_jj = (__pyx_v_j - __pyx_v_mid_col);

        /* "dipy/align/vector_fields.pyx":3028
 *                 ii = i - mid_row
 *                 jj = j - mid_col
 *                 r = np.sqrt(ii*ii + jj*jj + kk*kk)             # <<<<<<<<<<<<<<
 *                 if r <= radius:
 *                     s[k, i, j] = 1
 */
        __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3028; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3028; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t((((__pyx_v_ii * __pyx_v_ii) + (__pyx_v_jj * __pyx_v_jj)) + (__pyx_v_kk * __pyx_v_kk))); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3028; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_2 = NULL;
        if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_2)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_2);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        if (!__pyx_t_2) {
          __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3028; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_1);
        } else {
          __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3028; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2); __pyx_t_2 = NULL;
          __Pyx_GIVEREF(__pyx_t_5);
          PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_5);
          __pyx_t_5 = 0;
          __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3028; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_13 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_13 == (double)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3028; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_v_r = __pyx_t_13;

        /* "dipy/align/vector_fields.pyx":3029
 *                 jj = j - mid_col
 *                 r = np.sqrt(ii*ii + jj*jj + kk*kk)
 *                 if r <= radius:             # <<<<<<<<<<<<<<
 *                     s[k, i, j] = 1
 *                 else:
 */
        __pyx_t_14 = ((__pyx_v_r <= __pyx_v_radius) != 0);
        if (__pyx_t_14) {

          /* "dipy/align/vector_fields.pyx":3030
 *                 r = np.sqrt(ii*ii + jj*jj + kk*kk)
 *                 if r <= radius:
 *                     s[k, i, j] = 1             # <<<<<<<<<<<<<<
 *                 else:
 *                     s[k, i, j] = 0
 */
          __pyx_t_15 = __pyx_v_k;
          __pyx_t_16 = __pyx_v_i;
          __pyx_t_17 = __pyx_v_j;
          *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_s.data + __pyx_t_15 * __pyx_v_s.strides[0]) ) + __pyx_t_16 * __pyx_v_s.strides[1]) ) + __pyx_t_17 * __pyx_v_s.strides[2]) )) = 1.0;

          /* "dipy/align/vector_fields.pyx":3029
 *                 jj = j - mid_col
 *                 r = np.sqrt(ii*ii + jj*jj + kk*kk)
 *                 if r <= radius:             # <<<<<<<<<<<<<<
 *                     s[k, i, j] = 1
 *                 else:
 */
          goto __pyx_L9;
        }

        /* "dipy/align/vector_fields.pyx":3032
 *                     s[k, i, j] = 1
 *                 else:
 *                     s[k, i, j] = 0             # <<<<<<<<<<<<<<
 *     return s
 * 
 */
        /*else*/ {
          __pyx_t_18 = __pyx_v_k;
          __pyx_t_19 = __pyx_v_i;
          __pyx_t_20 = __pyx_v_j;
          *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_s.data + __pyx_t_18 * __pyx_v_s.strides[0]) ) + __pyx_t_19 * __pyx_v_s.strides[1]) ) + __pyx_t_20 * __pyx_v_s.strides[2]) )) = 0.0;
        }
        __pyx_L9:;
      }
    }
  }

  /* "dipy/align/vector_fields.pyx":3033
 *                 else:
 *                     s[k, i, j] = 0
 *     return s             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_fromslice(__pyx_v_s, 3, (PyObject *(*)(char *)) __pyx_memview_get_double, (int (*)(char *, PyObject *)) __pyx_memview_set_double, 0);; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3033; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":2992
 * 
 * 
 * def create_sphere(cnp.npy_intp nslices, cnp.npy_intp nrows,             # <<<<<<<<<<<<<<
 *                   cnp.npy_intp ncols, cnp.npy_intp radius):
 *     r"""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __PYX_XDEC_MEMVIEW(&__pyx_t_6, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields.create_sphere", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_s, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":3036
 * 
 * 
 * def _gradient_3d(floating[:, :, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                  double[:] img_spacing, double[:, :] out_grid2world,
 *                  floating[:, :, :, :] out, int[:, :, :] inside):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_71_gradient_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_70_gradient_3d[] = " Gradient of a 3D image in physical space coordinates\n\n    Each grid cell (i, j, k) in the sampling grid (determined by\n    out.shape) is mapped to its corresponding physical point (x, y, z) by\n    multiplying out_grid2world (its grid-to-space transform) by (i, j, k),\n    then the image is interpolated, at\n\n    P1=(x + h, y, z), Q1=(x - h, y, z)\n    P2=(x, y + h, z), Q2=(x, y - h, z)\n    P3=(x, y, z + h), Q3=(x, y, z - h)\n\n    (by mapping Pi and Qi to the grid using img_world2grid: the inverse of the\n    grid-to-space transform of img). The displacement parameter h is of\n    magnitude 0.5 (in physical space units), therefore the approximated partial\n    derivatives are given by the difference between the image interpolated at\n    Pi and Qi.\n\n    Parameters\n    ----------\n    img : array, shape (S, R, C)\n        the input volume whose gradient will be computed\n    img_world2grid : array, shape (4, 4)\n        the space-to-grid transform matrix associated to img\n    img_spacing : array, shape (3,)\n        the spacing between voxels (voxel size along each axis) of the input\n        volume\n    out_grid2world : array, shape (4, 4)\n        the grid-to-space transform associated to the sampling grid\n    out : array, shape (S', R', C', 3)\n        the buffer in which to store the image gradient\n    inside : array, shape (S', R', C')\n        the buffer in which to store the flags indicating whether the sample\n        point lies inside (=1) or outside (=0) the image grid\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_71_gradient_3d = {"_gradient_3d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_71_gradient_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_70_gradient_3d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_71_gradient_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_70_gradient_3d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_70_gradient_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_gradient_3d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_img, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_img); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__328, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__329, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__330, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__331, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_339_gradient_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_339_gradient_3d = {"__pyx_fuse_0_gradient_3d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_339_gradient_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_70_gradient_3d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_339_gradient_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_img = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_img_world2grid = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_img_spacing = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_grid2world = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_gradient_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_img,&__pyx_n_s_img_world2grid,&__pyx_n_s_img_spacing,&__pyx_n_s_out_grid2world,&__pyx_n_s_out,&__pyx_n_s_inside,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_world2grid)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_3d", 1, 6, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_spacing)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_3d", 1, 6, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_grid2world)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_3d", 1, 6, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_3d", 1, 6, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_inside)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_3d", 1, 6, 6, 5); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_gradient_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_img = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[0]); if (unlikely(!__pyx_v_img.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_img_world2grid = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_img_world2grid.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_img_spacing = __Pyx_PyObject_to_MemoryviewSlice_ds_double(values[2]); if (unlikely(!__pyx_v_img_spacing.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3037; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out_grid2world = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_out_grid2world.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3037; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(values[4]); if (unlikely(!__pyx_v_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3038; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_inside = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(values[5]); if (unlikely(!__pyx_v_inside.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3038; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_gradient_3d", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields._gradient_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_338_gradient_3d(__pyx_self, __pyx_v_img, __pyx_v_img_world2grid, __pyx_v_img_spacing, __pyx_v_out_grid2world, __pyx_v_out, __pyx_v_inside);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_338_gradient_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_img, __Pyx_memviewslice __pyx_v_img_world2grid, __Pyx_memviewslice __pyx_v_img_spacing, __Pyx_memviewslice __pyx_v_out_grid2world, __Pyx_memviewslice __pyx_v_out, __Pyx_memviewslice __pyx_v_inside) {
  int __pyx_v_nslices;
  int __pyx_v_nrows;
  int __pyx_v_ncols;
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_k;
  int __pyx_v_in_flag;
  double __pyx_v_tmp;
  __Pyx_memviewslice __pyx_v_x = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_dx = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_h = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_q = { 0, 0, { 0 }, { 0 }, { 0 } };
  long __pyx_v_p;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  int __pyx_t_16;
  int __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  __Pyx_memviewslice __pyx_t_24 = { 0, 0, { 0 }, { 0 }, { 0 } };
  long __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  int __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  Py_ssize_t __pyx_t_60;
  Py_ssize_t __pyx_t_61;
  Py_ssize_t __pyx_t_62;
  Py_ssize_t __pyx_t_63;
  Py_ssize_t __pyx_t_64;
  Py_ssize_t __pyx_t_65;
  Py_ssize_t __pyx_t_66;
  Py_ssize_t __pyx_t_67;
  Py_ssize_t __pyx_t_68;
  Py_ssize_t __pyx_t_69;
  Py_ssize_t __pyx_t_70;
  Py_ssize_t __pyx_t_71;
  Py_ssize_t __pyx_t_72;
  Py_ssize_t __pyx_t_73;
  Py_ssize_t __pyx_t_74;
  Py_ssize_t __pyx_t_75;
  Py_ssize_t __pyx_t_76;
  Py_ssize_t __pyx_t_77;
  Py_ssize_t __pyx_t_78;
  Py_ssize_t __pyx_t_79;
  Py_ssize_t __pyx_t_80;
  Py_ssize_t __pyx_t_81;
  Py_ssize_t __pyx_t_82;
  Py_ssize_t __pyx_t_83;
  Py_ssize_t __pyx_t_84;
  Py_ssize_t __pyx_t_85;
  Py_ssize_t __pyx_t_86;
  Py_ssize_t __pyx_t_87;
  Py_ssize_t __pyx_t_88;
  Py_ssize_t __pyx_t_89;
  Py_ssize_t __pyx_t_90;
  Py_ssize_t __pyx_t_91;
  Py_ssize_t __pyx_t_92;
  Py_ssize_t __pyx_t_93;
  Py_ssize_t __pyx_t_94;
  Py_ssize_t __pyx_t_95;
  Py_ssize_t __pyx_t_96;
  Py_ssize_t __pyx_t_97;
  Py_ssize_t __pyx_t_98;
  Py_ssize_t __pyx_t_99;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0_gradient_3d", 0);

  /* "dipy/align/vector_fields.pyx":3074
 *     """
 *     cdef:
 *         int nslices = out.shape[0]             # <<<<<<<<<<<<<<
 *         int nrows = out.shape[1]
 *         int ncols = out.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_out.shape[0]);

  /* "dipy/align/vector_fields.pyx":3075
 *     cdef:
 *         int nslices = out.shape[0]
 *         int nrows = out.shape[1]             # <<<<<<<<<<<<<<
 *         int ncols = out.shape[2]
 *         int i, j, k, in_flag
 */
  __pyx_v_nrows = (__pyx_v_out.shape[1]);

  /* "dipy/align/vector_fields.pyx":3076
 *         int nslices = out.shape[0]
 *         int nrows = out.shape[1]
 *         int ncols = out.shape[2]             # <<<<<<<<<<<<<<
 *         int i, j, k, in_flag
 *         double tmp
 */
  __pyx_v_ncols = (__pyx_v_out.shape[2]);

  /* "dipy/align/vector_fields.pyx":3079
 *         int i, j, k, in_flag
 *         double tmp
 *         double[:] x = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_tuple__332) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_float64); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_x = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3080
 *         double tmp
 *         double[:] x = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_empty); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__333) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_float64); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_3);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_dx = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3081
 *         double[:] x = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_empty); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__334) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_float64); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_2);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_h = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3082
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_tuple__335) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_float64); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_1);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_q = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3083
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":3084
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]             # <<<<<<<<<<<<<<
 *         h[1] = 0.5 * img_spacing[1]
 *         h[2] = 0.5 * img_spacing[2]
 */
        __pyx_t_6 = 0;
        __pyx_t_7 = 0;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_7 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_6 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3085
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]             # <<<<<<<<<<<<<<
 *         h[2] = 0.5 * img_spacing[2]
 *         for k in range(nslices):
 */
        __pyx_t_8 = 1;
        __pyx_t_9 = 1;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_9 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_8 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3086
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 *         h[2] = 0.5 * img_spacing[2]             # <<<<<<<<<<<<<<
 *         for k in range(nslices):
 *             for i in range(nrows):
 */
        __pyx_t_10 = 2;
        __pyx_t_11 = 2;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_11 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_10 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3087
 *         h[1] = 0.5 * img_spacing[1]
 *         h[2] = 0.5 * img_spacing[2]
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_12 = __pyx_v_nslices;
        for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
          __pyx_v_k = __pyx_t_13;

          /* "dipy/align/vector_fields.pyx":3088
 *         h[2] = 0.5 * img_spacing[2]
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     inside[k, i, j] = 1
 */
          __pyx_t_14 = __pyx_v_nrows;
          for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
            __pyx_v_i = __pyx_t_15;

            /* "dipy/align/vector_fields.pyx":3089
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     inside[k, i, j] = 1
 *                     # Compute coordinates of index (k, i, j) in physical space
 */
            __pyx_t_16 = __pyx_v_ncols;
            for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
              __pyx_v_j = __pyx_t_17;

              /* "dipy/align/vector_fields.pyx":3090
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     inside[k, i, j] = 1             # <<<<<<<<<<<<<<
 *                     # Compute coordinates of index (k, i, j) in physical space
 *                     x[0] = _apply_affine_3d_x0(k, i, j, 1, out_grid2world)
 */
              __pyx_t_18 = __pyx_v_k;
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              *((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_18 * __pyx_v_inside.strides[0]) ) + __pyx_t_19 * __pyx_v_inside.strides[1]) ) + __pyx_t_20 * __pyx_v_inside.strides[2]) )) = 1;

              /* "dipy/align/vector_fields.pyx":3092
 *                     inside[k, i, j] = 1
 *                     # Compute coordinates of index (k, i, j) in physical space
 *                     x[0] = _apply_affine_3d_x0(k, i, j, 1, out_grid2world)             # <<<<<<<<<<<<<<
 *                     x[1] = _apply_affine_3d_x1(k, i, j, 1, out_grid2world)
 *                     x[2] = _apply_affine_3d_x2(k, i, j, 1, out_grid2world)
 */
              __pyx_t_21 = 0;
              *((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_21 * __pyx_v_x.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_out_grid2world);

              /* "dipy/align/vector_fields.pyx":3093
 *                     # Compute coordinates of index (k, i, j) in physical space
 *                     x[0] = _apply_affine_3d_x0(k, i, j, 1, out_grid2world)
 *                     x[1] = _apply_affine_3d_x1(k, i, j, 1, out_grid2world)             # <<<<<<<<<<<<<<
 *                     x[2] = _apply_affine_3d_x2(k, i, j, 1, out_grid2world)
 *                     dx[:] = x[:]
 */
              __pyx_t_22 = 1;
              *((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_22 * __pyx_v_x.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_out_grid2world);

              /* "dipy/align/vector_fields.pyx":3094
 *                     x[0] = _apply_affine_3d_x0(k, i, j, 1, out_grid2world)
 *                     x[1] = _apply_affine_3d_x1(k, i, j, 1, out_grid2world)
 *                     x[2] = _apply_affine_3d_x2(k, i, j, 1, out_grid2world)             # <<<<<<<<<<<<<<
 *                     dx[:] = x[:]
 *                     for p in range(3):
 */
              __pyx_t_23 = 2;
              *((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_23 * __pyx_v_x.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_out_grid2world);

              /* "dipy/align/vector_fields.pyx":3095
 *                     x[1] = _apply_affine_3d_x1(k, i, j, 1, out_grid2world)
 *                     x[2] = _apply_affine_3d_x2(k, i, j, 1, out_grid2world)
 *                     dx[:] = x[:]             # <<<<<<<<<<<<<<
 *                     for p in range(3):
 *                         # Compute coordinates of point dx on img's grid
 */
              __pyx_t_5.data = __pyx_v_x.data;
              __pyx_t_5.memview = __pyx_v_x.memview;
              __PYX_INC_MEMVIEW(&__pyx_t_5, 0);
              __pyx_t_5.shape[0] = __pyx_v_x.shape[0];
__pyx_t_5.strides[0] = __pyx_v_x.strides[0];
    __pyx_t_5.suboffsets[0] = -1;

__pyx_t_24.data = __pyx_v_dx.data;
              __pyx_t_24.memview = __pyx_v_dx.memview;
              __PYX_INC_MEMVIEW(&__pyx_t_24, 0);
              __pyx_t_24.shape[0] = __pyx_v_dx.shape[0];
__pyx_t_24.strides[0] = __pyx_v_dx.strides[0];
    __pyx_t_24.suboffsets[0] = -1;

if (unlikely(__pyx_memoryview_copy_contents(__pyx_t_5, __pyx_t_24, 1, 1, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3095; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
              __PYX_XDEC_MEMVIEW(&__pyx_t_24, 0);
              __PYX_XDEC_MEMVIEW(&__pyx_t_5, 0);

              /* "dipy/align/vector_fields.pyx":3096
 *                     x[2] = _apply_affine_3d_x2(k, i, j, 1, out_grid2world)
 *                     dx[:] = x[:]
 *                     for p in range(3):             # <<<<<<<<<<<<<<
 *                         # Compute coordinates of point dx on img's grid
 *                         dx[p] = x[p] - h[p]
 */
              for (__pyx_t_25 = 0; __pyx_t_25 < 3; __pyx_t_25+=1) {
                __pyx_v_p = __pyx_t_25;

                /* "dipy/align/vector_fields.pyx":3098
 *                     for p in range(3):
 *                         # Compute coordinates of point dx on img's grid
 *                         dx[p] = x[p] - h[p]             # <<<<<<<<<<<<<<
 *                         q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 */
                __pyx_t_26 = __pyx_v_p;
                __pyx_t_27 = __pyx_v_p;
                __pyx_t_28 = __pyx_v_p;
                *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_28 * __pyx_v_dx.strides[0]) )) = ((*((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_26 * __pyx_v_x.strides[0]) ))) - (*((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_27 * __pyx_v_h.strides[0]) ))));

                /* "dipy/align/vector_fields.pyx":3099
 *                         # Compute coordinates of point dx on img's grid
 *                         dx[p] = x[p] - h[p]
 *                         q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                                    img_world2grid)
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 */
                __pyx_t_29 = 0;
                __pyx_t_30 = 1;
                __pyx_t_31 = 2;

                /* "dipy/align/vector_fields.pyx":3100
 *                         dx[p] = x[p] - h[p]
 *                         q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)             # <<<<<<<<<<<<<<
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 */
                __pyx_t_32 = 0;
                *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_32 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_29 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_30 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_31 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

                /* "dipy/align/vector_fields.pyx":3101
 *                         q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                                    img_world2grid)
 *                         q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 */
                __pyx_t_33 = 0;
                __pyx_t_34 = 1;
                __pyx_t_35 = 2;

                /* "dipy/align/vector_fields.pyx":3102
 *                                                    img_world2grid)
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)             # <<<<<<<<<<<<<<
 *                         q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 */
                __pyx_t_36 = 1;
                *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_36 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_33 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_34 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_35 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

                /* "dipy/align/vector_fields.pyx":3103
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 *                         q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                                    img_world2grid)
 *                         # Interpolate img at q
 */
                __pyx_t_37 = 0;
                __pyx_t_38 = 1;
                __pyx_t_39 = 2;

                /* "dipy/align/vector_fields.pyx":3104
 *                                                    img_world2grid)
 *                         q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)             # <<<<<<<<<<<<<<
 *                         # Interpolate img at q
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],
 */
                __pyx_t_40 = 2;
                *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_40 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_37 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_38 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_39 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

                /* "dipy/align/vector_fields.pyx":3106
 *                                                    img_world2grid)
 *                         # Interpolate img at q
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],             # <<<<<<<<<<<<<<
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:
 */
                __pyx_t_41 = 0;

                /* "dipy/align/vector_fields.pyx":3107
 *                         # Interpolate img at q
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],
 *                             q[1], q[2], &out[k, i, j, p])             # <<<<<<<<<<<<<<
 *                         if in_flag == 0:
 *                             out[k, i, j, p] = 0
 */
                __pyx_t_42 = 1;
                __pyx_t_43 = 2;
                __pyx_t_44 = __pyx_v_k;
                __pyx_t_45 = __pyx_v_i;
                __pyx_t_46 = __pyx_v_j;
                __pyx_t_47 = __pyx_v_p;

                /* "dipy/align/vector_fields.pyx":3106
 *                                                    img_world2grid)
 *                         # Interpolate img at q
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],             # <<<<<<<<<<<<<<
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:
 */
                __pyx_v_in_flag = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__pyx_v_img, (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_41 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_42 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_43 * __pyx_v_q.strides[0]) ))), (&(*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_44 * __pyx_v_out.strides[0]) ) + __pyx_t_45 * __pyx_v_out.strides[1]) ) + __pyx_t_46 * __pyx_v_out.strides[2]) ) + __pyx_t_47 * __pyx_v_out.strides[3]) )))));

                /* "dipy/align/vector_fields.pyx":3108
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:             # <<<<<<<<<<<<<<
 *                             out[k, i, j, p] = 0
 *                             inside[k, i, j] = 0
 */
                __pyx_t_48 = ((__pyx_v_in_flag == 0) != 0);
                if (__pyx_t_48) {

                  /* "dipy/align/vector_fields.pyx":3109
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:
 *                             out[k, i, j, p] = 0             # <<<<<<<<<<<<<<
 *                             inside[k, i, j] = 0
 *                             continue
 */
                  __pyx_t_49 = __pyx_v_k;
                  __pyx_t_50 = __pyx_v_i;
                  __pyx_t_51 = __pyx_v_j;
                  __pyx_t_52 = __pyx_v_p;
                  *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_49 * __pyx_v_out.strides[0]) ) + __pyx_t_50 * __pyx_v_out.strides[1]) ) + __pyx_t_51 * __pyx_v_out.strides[2]) ) + __pyx_t_52 * __pyx_v_out.strides[3]) )) = 0.0;

                  /* "dipy/align/vector_fields.pyx":3110
 *                         if in_flag == 0:
 *                             out[k, i, j, p] = 0
 *                             inside[k, i, j] = 0             # <<<<<<<<<<<<<<
 *                             continue
 *                         tmp = out[k, i, j, p]
 */
                  __pyx_t_53 = __pyx_v_k;
                  __pyx_t_54 = __pyx_v_i;
                  __pyx_t_55 = __pyx_v_j;
                  *((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_53 * __pyx_v_inside.strides[0]) ) + __pyx_t_54 * __pyx_v_inside.strides[1]) ) + __pyx_t_55 * __pyx_v_inside.strides[2]) )) = 0;

                  /* "dipy/align/vector_fields.pyx":3111
 *                             out[k, i, j, p] = 0
 *                             inside[k, i, j] = 0
 *                             continue             # <<<<<<<<<<<<<<
 *                         tmp = out[k, i, j, p]
 *                         # Compute coordinates of point dx on img's grid
 */
                  goto __pyx_L12_continue;

                  /* "dipy/align/vector_fields.pyx":3108
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:             # <<<<<<<<<<<<<<
 *                             out[k, i, j, p] = 0
 *                             inside[k, i, j] = 0
 */
                }

                /* "dipy/align/vector_fields.pyx":3112
 *                             inside[k, i, j] = 0
 *                             continue
 *                         tmp = out[k, i, j, p]             # <<<<<<<<<<<<<<
 *                         # Compute coordinates of point dx on img's grid
 *                         dx[p] = x[p] + h[p]
 */
                __pyx_t_56 = __pyx_v_k;
                __pyx_t_57 = __pyx_v_i;
                __pyx_t_58 = __pyx_v_j;
                __pyx_t_59 = __pyx_v_p;
                __pyx_v_tmp = (*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_56 * __pyx_v_out.strides[0]) ) + __pyx_t_57 * __pyx_v_out.strides[1]) ) + __pyx_t_58 * __pyx_v_out.strides[2]) ) + __pyx_t_59 * __pyx_v_out.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":3114
 *                         tmp = out[k, i, j, p]
 *                         # Compute coordinates of point dx on img's grid
 *                         dx[p] = x[p] + h[p]             # <<<<<<<<<<<<<<
 *                         q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 */
                __pyx_t_60 = __pyx_v_p;
                __pyx_t_61 = __pyx_v_p;
                __pyx_t_62 = __pyx_v_p;
                *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_62 * __pyx_v_dx.strides[0]) )) = ((*((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_60 * __pyx_v_x.strides[0]) ))) + (*((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_61 * __pyx_v_h.strides[0]) ))));

                /* "dipy/align/vector_fields.pyx":3115
 *                         # Compute coordinates of point dx on img's grid
 *                         dx[p] = x[p] + h[p]
 *                         q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                                    img_world2grid)
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 */
                __pyx_t_63 = 0;
                __pyx_t_64 = 1;
                __pyx_t_65 = 2;

                /* "dipy/align/vector_fields.pyx":3116
 *                         dx[p] = x[p] + h[p]
 *                         q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)             # <<<<<<<<<<<<<<
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 */
                __pyx_t_66 = 0;
                *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_66 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_63 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_64 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_65 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

                /* "dipy/align/vector_fields.pyx":3117
 *                         q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                                    img_world2grid)
 *                         q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 */
                __pyx_t_67 = 0;
                __pyx_t_68 = 1;
                __pyx_t_69 = 2;

                /* "dipy/align/vector_fields.pyx":3118
 *                                                    img_world2grid)
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)             # <<<<<<<<<<<<<<
 *                         q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 */
                __pyx_t_70 = 1;
                *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_70 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_67 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_68 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_69 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

                /* "dipy/align/vector_fields.pyx":3119
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 *                         q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                                    img_world2grid)
 *                         # Interpolate img at q
 */
                __pyx_t_71 = 0;
                __pyx_t_72 = 1;
                __pyx_t_73 = 2;

                /* "dipy/align/vector_fields.pyx":3120
 *                                                    img_world2grid)
 *                         q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)             # <<<<<<<<<<<<<<
 *                         # Interpolate img at q
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],
 */
                __pyx_t_74 = 2;
                *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_74 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_71 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_72 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_73 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

                /* "dipy/align/vector_fields.pyx":3122
 *                                                    img_world2grid)
 *                         # Interpolate img at q
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],             # <<<<<<<<<<<<<<
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:
 */
                __pyx_t_75 = 0;

                /* "dipy/align/vector_fields.pyx":3123
 *                         # Interpolate img at q
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],
 *                             q[1], q[2], &out[k, i, j, p])             # <<<<<<<<<<<<<<
 *                         if in_flag == 0:
 *                             out[k, i, j, p] = 0
 */
                __pyx_t_76 = 1;
                __pyx_t_77 = 2;
                __pyx_t_78 = __pyx_v_k;
                __pyx_t_79 = __pyx_v_i;
                __pyx_t_80 = __pyx_v_j;
                __pyx_t_81 = __pyx_v_p;

                /* "dipy/align/vector_fields.pyx":3122
 *                                                    img_world2grid)
 *                         # Interpolate img at q
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],             # <<<<<<<<<<<<<<
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:
 */
                __pyx_v_in_flag = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__pyx_v_img, (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_75 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_76 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_77 * __pyx_v_q.strides[0]) ))), (&(*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_78 * __pyx_v_out.strides[0]) ) + __pyx_t_79 * __pyx_v_out.strides[1]) ) + __pyx_t_80 * __pyx_v_out.strides[2]) ) + __pyx_t_81 * __pyx_v_out.strides[3]) )))));

                /* "dipy/align/vector_fields.pyx":3124
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:             # <<<<<<<<<<<<<<
 *                             out[k, i, j, p] = 0
 *                             inside[k, i, j] = 0
 */
                __pyx_t_48 = ((__pyx_v_in_flag == 0) != 0);
                if (__pyx_t_48) {

                  /* "dipy/align/vector_fields.pyx":3125
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:
 *                             out[k, i, j, p] = 0             # <<<<<<<<<<<<<<
 *                             inside[k, i, j] = 0
 *                             continue
 */
                  __pyx_t_82 = __pyx_v_k;
                  __pyx_t_83 = __pyx_v_i;
                  __pyx_t_84 = __pyx_v_j;
                  __pyx_t_85 = __pyx_v_p;
                  *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_82 * __pyx_v_out.strides[0]) ) + __pyx_t_83 * __pyx_v_out.strides[1]) ) + __pyx_t_84 * __pyx_v_out.strides[2]) ) + __pyx_t_85 * __pyx_v_out.strides[3]) )) = 0.0;

                  /* "dipy/align/vector_fields.pyx":3126
 *                         if in_flag == 0:
 *                             out[k, i, j, p] = 0
 *                             inside[k, i, j] = 0             # <<<<<<<<<<<<<<
 *                             continue
 *                         out[k, i, j, p] = (out[k, i, j, p] - tmp) / img_spacing[p]
 */
                  __pyx_t_86 = __pyx_v_k;
                  __pyx_t_87 = __pyx_v_i;
                  __pyx_t_88 = __pyx_v_j;
                  *((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_86 * __pyx_v_inside.strides[0]) ) + __pyx_t_87 * __pyx_v_inside.strides[1]) ) + __pyx_t_88 * __pyx_v_inside.strides[2]) )) = 0;

                  /* "dipy/align/vector_fields.pyx":3127
 *                             out[k, i, j, p] = 0
 *                             inside[k, i, j] = 0
 *                             continue             # <<<<<<<<<<<<<<
 *                         out[k, i, j, p] = (out[k, i, j, p] - tmp) / img_spacing[p]
 *                         dx[p] = x[p]
 */
                  goto __pyx_L12_continue;

                  /* "dipy/align/vector_fields.pyx":3124
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:             # <<<<<<<<<<<<<<
 *                             out[k, i, j, p] = 0
 *                             inside[k, i, j] = 0
 */
                }

                /* "dipy/align/vector_fields.pyx":3128
 *                             inside[k, i, j] = 0
 *                             continue
 *                         out[k, i, j, p] = (out[k, i, j, p] - tmp) / img_spacing[p]             # <<<<<<<<<<<<<<
 *                         dx[p] = x[p]
 * 
 */
                __pyx_t_89 = __pyx_v_k;
                __pyx_t_90 = __pyx_v_i;
                __pyx_t_91 = __pyx_v_j;
                __pyx_t_92 = __pyx_v_p;
                __pyx_t_93 = __pyx_v_p;
                __pyx_t_94 = __pyx_v_k;
                __pyx_t_95 = __pyx_v_i;
                __pyx_t_96 = __pyx_v_j;
                __pyx_t_97 = __pyx_v_p;
                *((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_94 * __pyx_v_out.strides[0]) ) + __pyx_t_95 * __pyx_v_out.strides[1]) ) + __pyx_t_96 * __pyx_v_out.strides[2]) ) + __pyx_t_97 * __pyx_v_out.strides[3]) )) = (((*((float *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_89 * __pyx_v_out.strides[0]) ) + __pyx_t_90 * __pyx_v_out.strides[1]) ) + __pyx_t_91 * __pyx_v_out.strides[2]) ) + __pyx_t_92 * __pyx_v_out.strides[3]) ))) - __pyx_v_tmp) / (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_93 * __pyx_v_img_spacing.strides[0]) ))));

                /* "dipy/align/vector_fields.pyx":3129
 *                             continue
 *                         out[k, i, j, p] = (out[k, i, j, p] - tmp) / img_spacing[p]
 *                         dx[p] = x[p]             # <<<<<<<<<<<<<<
 * 
 * 
 */
                __pyx_t_98 = __pyx_v_p;
                __pyx_t_99 = __pyx_v_p;
                *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_99 * __pyx_v_dx.strides[0]) )) = (*((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_98 * __pyx_v_x.strides[0]) )));
                __pyx_L12_continue:;
              }
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":3083
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L4_error: {
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L1_error;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":3036
 * 
 * 
 * def _gradient_3d(floating[:, :, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                  double[:] img_spacing, double[:, :] out_grid2world,
 *                  floating[:, :, :, :] out, int[:, :, :] inside):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_24, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields._gradient_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_x, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_dx, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_h, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_q, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img_world2grid, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img_spacing, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_grid2world, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_341_gradient_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_341_gradient_3d = {"__pyx_fuse_1_gradient_3d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_341_gradient_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_70_gradient_3d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_341_gradient_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_img = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_img_world2grid = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_img_spacing = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_grid2world = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_gradient_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_img,&__pyx_n_s_img_world2grid,&__pyx_n_s_img_spacing,&__pyx_n_s_out_grid2world,&__pyx_n_s_out,&__pyx_n_s_inside,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_world2grid)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_3d", 1, 6, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_spacing)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_3d", 1, 6, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_grid2world)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_3d", 1, 6, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_3d", 1, 6, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_inside)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_3d", 1, 6, 6, 5); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_gradient_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_img = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[0]); if (unlikely(!__pyx_v_img.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_img_world2grid = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_img_world2grid.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_img_spacing = __Pyx_PyObject_to_MemoryviewSlice_ds_double(values[2]); if (unlikely(!__pyx_v_img_spacing.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3037; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out_grid2world = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_out_grid2world.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3037; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(values[4]); if (unlikely(!__pyx_v_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3038; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_inside = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(values[5]); if (unlikely(!__pyx_v_inside.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3038; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_gradient_3d", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields._gradient_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_340_gradient_3d(__pyx_self, __pyx_v_img, __pyx_v_img_world2grid, __pyx_v_img_spacing, __pyx_v_out_grid2world, __pyx_v_out, __pyx_v_inside);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_340_gradient_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_img, __Pyx_memviewslice __pyx_v_img_world2grid, __Pyx_memviewslice __pyx_v_img_spacing, __Pyx_memviewslice __pyx_v_out_grid2world, __Pyx_memviewslice __pyx_v_out, __Pyx_memviewslice __pyx_v_inside) {
  int __pyx_v_nslices;
  int __pyx_v_nrows;
  int __pyx_v_ncols;
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_k;
  int __pyx_v_in_flag;
  double __pyx_v_tmp;
  __Pyx_memviewslice __pyx_v_x = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_dx = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_h = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_q = { 0, 0, { 0 }, { 0 }, { 0 } };
  long __pyx_v_p;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  int __pyx_t_16;
  int __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  __Pyx_memviewslice __pyx_t_24 = { 0, 0, { 0 }, { 0 }, { 0 } };
  long __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  int __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  Py_ssize_t __pyx_t_60;
  Py_ssize_t __pyx_t_61;
  Py_ssize_t __pyx_t_62;
  Py_ssize_t __pyx_t_63;
  Py_ssize_t __pyx_t_64;
  Py_ssize_t __pyx_t_65;
  Py_ssize_t __pyx_t_66;
  Py_ssize_t __pyx_t_67;
  Py_ssize_t __pyx_t_68;
  Py_ssize_t __pyx_t_69;
  Py_ssize_t __pyx_t_70;
  Py_ssize_t __pyx_t_71;
  Py_ssize_t __pyx_t_72;
  Py_ssize_t __pyx_t_73;
  Py_ssize_t __pyx_t_74;
  Py_ssize_t __pyx_t_75;
  Py_ssize_t __pyx_t_76;
  Py_ssize_t __pyx_t_77;
  Py_ssize_t __pyx_t_78;
  Py_ssize_t __pyx_t_79;
  Py_ssize_t __pyx_t_80;
  Py_ssize_t __pyx_t_81;
  Py_ssize_t __pyx_t_82;
  Py_ssize_t __pyx_t_83;
  Py_ssize_t __pyx_t_84;
  Py_ssize_t __pyx_t_85;
  Py_ssize_t __pyx_t_86;
  Py_ssize_t __pyx_t_87;
  Py_ssize_t __pyx_t_88;
  Py_ssize_t __pyx_t_89;
  Py_ssize_t __pyx_t_90;
  Py_ssize_t __pyx_t_91;
  Py_ssize_t __pyx_t_92;
  Py_ssize_t __pyx_t_93;
  Py_ssize_t __pyx_t_94;
  Py_ssize_t __pyx_t_95;
  Py_ssize_t __pyx_t_96;
  Py_ssize_t __pyx_t_97;
  Py_ssize_t __pyx_t_98;
  Py_ssize_t __pyx_t_99;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1_gradient_3d", 0);

  /* "dipy/align/vector_fields.pyx":3074
 *     """
 *     cdef:
 *         int nslices = out.shape[0]             # <<<<<<<<<<<<<<
 *         int nrows = out.shape[1]
 *         int ncols = out.shape[2]
 */
  __pyx_v_nslices = (__pyx_v_out.shape[0]);

  /* "dipy/align/vector_fields.pyx":3075
 *     cdef:
 *         int nslices = out.shape[0]
 *         int nrows = out.shape[1]             # <<<<<<<<<<<<<<
 *         int ncols = out.shape[2]
 *         int i, j, k, in_flag
 */
  __pyx_v_nrows = (__pyx_v_out.shape[1]);

  /* "dipy/align/vector_fields.pyx":3076
 *         int nslices = out.shape[0]
 *         int nrows = out.shape[1]
 *         int ncols = out.shape[2]             # <<<<<<<<<<<<<<
 *         int i, j, k, in_flag
 *         double tmp
 */
  __pyx_v_ncols = (__pyx_v_out.shape[2]);

  /* "dipy/align/vector_fields.pyx":3079
 *         int i, j, k, in_flag
 *         double tmp
 *         double[:] x = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_tuple__336) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_float64); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_x = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3080
 *         double tmp
 *         double[:] x = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_empty); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__337) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_float64); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_3);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_dx = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3081
 *         double[:] x = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_empty); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__338) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_float64); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_2);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_h = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3082
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_tuple__339) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_float64); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_1);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_q = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3083
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":3084
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]             # <<<<<<<<<<<<<<
 *         h[1] = 0.5 * img_spacing[1]
 *         h[2] = 0.5 * img_spacing[2]
 */
        __pyx_t_6 = 0;
        __pyx_t_7 = 0;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_7 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_6 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3085
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]             # <<<<<<<<<<<<<<
 *         h[2] = 0.5 * img_spacing[2]
 *         for k in range(nslices):
 */
        __pyx_t_8 = 1;
        __pyx_t_9 = 1;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_9 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_8 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3086
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 *         h[2] = 0.5 * img_spacing[2]             # <<<<<<<<<<<<<<
 *         for k in range(nslices):
 *             for i in range(nrows):
 */
        __pyx_t_10 = 2;
        __pyx_t_11 = 2;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_11 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_10 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3087
 *         h[1] = 0.5 * img_spacing[1]
 *         h[2] = 0.5 * img_spacing[2]
 *         for k in range(nslices):             # <<<<<<<<<<<<<<
 *             for i in range(nrows):
 *                 for j in range(ncols):
 */
        __pyx_t_12 = __pyx_v_nslices;
        for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
          __pyx_v_k = __pyx_t_13;

          /* "dipy/align/vector_fields.pyx":3088
 *         h[2] = 0.5 * img_spacing[2]
 *         for k in range(nslices):
 *             for i in range(nrows):             # <<<<<<<<<<<<<<
 *                 for j in range(ncols):
 *                     inside[k, i, j] = 1
 */
          __pyx_t_14 = __pyx_v_nrows;
          for (__pyx_t_15 = 0; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
            __pyx_v_i = __pyx_t_15;

            /* "dipy/align/vector_fields.pyx":3089
 *         for k in range(nslices):
 *             for i in range(nrows):
 *                 for j in range(ncols):             # <<<<<<<<<<<<<<
 *                     inside[k, i, j] = 1
 *                     # Compute coordinates of index (k, i, j) in physical space
 */
            __pyx_t_16 = __pyx_v_ncols;
            for (__pyx_t_17 = 0; __pyx_t_17 < __pyx_t_16; __pyx_t_17+=1) {
              __pyx_v_j = __pyx_t_17;

              /* "dipy/align/vector_fields.pyx":3090
 *             for i in range(nrows):
 *                 for j in range(ncols):
 *                     inside[k, i, j] = 1             # <<<<<<<<<<<<<<
 *                     # Compute coordinates of index (k, i, j) in physical space
 *                     x[0] = _apply_affine_3d_x0(k, i, j, 1, out_grid2world)
 */
              __pyx_t_18 = __pyx_v_k;
              __pyx_t_19 = __pyx_v_i;
              __pyx_t_20 = __pyx_v_j;
              *((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_18 * __pyx_v_inside.strides[0]) ) + __pyx_t_19 * __pyx_v_inside.strides[1]) ) + __pyx_t_20 * __pyx_v_inside.strides[2]) )) = 1;

              /* "dipy/align/vector_fields.pyx":3092
 *                     inside[k, i, j] = 1
 *                     # Compute coordinates of index (k, i, j) in physical space
 *                     x[0] = _apply_affine_3d_x0(k, i, j, 1, out_grid2world)             # <<<<<<<<<<<<<<
 *                     x[1] = _apply_affine_3d_x1(k, i, j, 1, out_grid2world)
 *                     x[2] = _apply_affine_3d_x2(k, i, j, 1, out_grid2world)
 */
              __pyx_t_21 = 0;
              *((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_21 * __pyx_v_x.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_out_grid2world);

              /* "dipy/align/vector_fields.pyx":3093
 *                     # Compute coordinates of index (k, i, j) in physical space
 *                     x[0] = _apply_affine_3d_x0(k, i, j, 1, out_grid2world)
 *                     x[1] = _apply_affine_3d_x1(k, i, j, 1, out_grid2world)             # <<<<<<<<<<<<<<
 *                     x[2] = _apply_affine_3d_x2(k, i, j, 1, out_grid2world)
 *                     dx[:] = x[:]
 */
              __pyx_t_22 = 1;
              *((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_22 * __pyx_v_x.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_out_grid2world);

              /* "dipy/align/vector_fields.pyx":3094
 *                     x[0] = _apply_affine_3d_x0(k, i, j, 1, out_grid2world)
 *                     x[1] = _apply_affine_3d_x1(k, i, j, 1, out_grid2world)
 *                     x[2] = _apply_affine_3d_x2(k, i, j, 1, out_grid2world)             # <<<<<<<<<<<<<<
 *                     dx[:] = x[:]
 *                     for p in range(3):
 */
              __pyx_t_23 = 2;
              *((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_23 * __pyx_v_x.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(__pyx_v_k, __pyx_v_i, __pyx_v_j, 1.0, __pyx_v_out_grid2world);

              /* "dipy/align/vector_fields.pyx":3095
 *                     x[1] = _apply_affine_3d_x1(k, i, j, 1, out_grid2world)
 *                     x[2] = _apply_affine_3d_x2(k, i, j, 1, out_grid2world)
 *                     dx[:] = x[:]             # <<<<<<<<<<<<<<
 *                     for p in range(3):
 *                         # Compute coordinates of point dx on img's grid
 */
              __pyx_t_5.data = __pyx_v_x.data;
              __pyx_t_5.memview = __pyx_v_x.memview;
              __PYX_INC_MEMVIEW(&__pyx_t_5, 0);
              __pyx_t_5.shape[0] = __pyx_v_x.shape[0];
__pyx_t_5.strides[0] = __pyx_v_x.strides[0];
    __pyx_t_5.suboffsets[0] = -1;

__pyx_t_24.data = __pyx_v_dx.data;
              __pyx_t_24.memview = __pyx_v_dx.memview;
              __PYX_INC_MEMVIEW(&__pyx_t_24, 0);
              __pyx_t_24.shape[0] = __pyx_v_dx.shape[0];
__pyx_t_24.strides[0] = __pyx_v_dx.strides[0];
    __pyx_t_24.suboffsets[0] = -1;

if (unlikely(__pyx_memoryview_copy_contents(__pyx_t_5, __pyx_t_24, 1, 1, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3095; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
              __PYX_XDEC_MEMVIEW(&__pyx_t_24, 0);
              __PYX_XDEC_MEMVIEW(&__pyx_t_5, 0);

              /* "dipy/align/vector_fields.pyx":3096
 *                     x[2] = _apply_affine_3d_x2(k, i, j, 1, out_grid2world)
 *                     dx[:] = x[:]
 *                     for p in range(3):             # <<<<<<<<<<<<<<
 *                         # Compute coordinates of point dx on img's grid
 *                         dx[p] = x[p] - h[p]
 */
              for (__pyx_t_25 = 0; __pyx_t_25 < 3; __pyx_t_25+=1) {
                __pyx_v_p = __pyx_t_25;

                /* "dipy/align/vector_fields.pyx":3098
 *                     for p in range(3):
 *                         # Compute coordinates of point dx on img's grid
 *                         dx[p] = x[p] - h[p]             # <<<<<<<<<<<<<<
 *                         q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 */
                __pyx_t_26 = __pyx_v_p;
                __pyx_t_27 = __pyx_v_p;
                __pyx_t_28 = __pyx_v_p;
                *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_28 * __pyx_v_dx.strides[0]) )) = ((*((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_26 * __pyx_v_x.strides[0]) ))) - (*((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_27 * __pyx_v_h.strides[0]) ))));

                /* "dipy/align/vector_fields.pyx":3099
 *                         # Compute coordinates of point dx on img's grid
 *                         dx[p] = x[p] - h[p]
 *                         q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                                    img_world2grid)
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 */
                __pyx_t_29 = 0;
                __pyx_t_30 = 1;
                __pyx_t_31 = 2;

                /* "dipy/align/vector_fields.pyx":3100
 *                         dx[p] = x[p] - h[p]
 *                         q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)             # <<<<<<<<<<<<<<
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 */
                __pyx_t_32 = 0;
                *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_32 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_29 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_30 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_31 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

                /* "dipy/align/vector_fields.pyx":3101
 *                         q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                                    img_world2grid)
 *                         q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 */
                __pyx_t_33 = 0;
                __pyx_t_34 = 1;
                __pyx_t_35 = 2;

                /* "dipy/align/vector_fields.pyx":3102
 *                                                    img_world2grid)
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)             # <<<<<<<<<<<<<<
 *                         q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 */
                __pyx_t_36 = 1;
                *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_36 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_33 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_34 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_35 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

                /* "dipy/align/vector_fields.pyx":3103
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 *                         q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                                    img_world2grid)
 *                         # Interpolate img at q
 */
                __pyx_t_37 = 0;
                __pyx_t_38 = 1;
                __pyx_t_39 = 2;

                /* "dipy/align/vector_fields.pyx":3104
 *                                                    img_world2grid)
 *                         q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)             # <<<<<<<<<<<<<<
 *                         # Interpolate img at q
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],
 */
                __pyx_t_40 = 2;
                *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_40 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_37 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_38 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_39 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

                /* "dipy/align/vector_fields.pyx":3106
 *                                                    img_world2grid)
 *                         # Interpolate img at q
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],             # <<<<<<<<<<<<<<
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:
 */
                __pyx_t_41 = 0;

                /* "dipy/align/vector_fields.pyx":3107
 *                         # Interpolate img at q
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],
 *                             q[1], q[2], &out[k, i, j, p])             # <<<<<<<<<<<<<<
 *                         if in_flag == 0:
 *                             out[k, i, j, p] = 0
 */
                __pyx_t_42 = 1;
                __pyx_t_43 = 2;
                __pyx_t_44 = __pyx_v_k;
                __pyx_t_45 = __pyx_v_i;
                __pyx_t_46 = __pyx_v_j;
                __pyx_t_47 = __pyx_v_p;

                /* "dipy/align/vector_fields.pyx":3106
 *                                                    img_world2grid)
 *                         # Interpolate img at q
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],             # <<<<<<<<<<<<<<
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:
 */
                __pyx_v_in_flag = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__pyx_v_img, (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_41 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_42 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_43 * __pyx_v_q.strides[0]) ))), (&(*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_44 * __pyx_v_out.strides[0]) ) + __pyx_t_45 * __pyx_v_out.strides[1]) ) + __pyx_t_46 * __pyx_v_out.strides[2]) ) + __pyx_t_47 * __pyx_v_out.strides[3]) )))));

                /* "dipy/align/vector_fields.pyx":3108
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:             # <<<<<<<<<<<<<<
 *                             out[k, i, j, p] = 0
 *                             inside[k, i, j] = 0
 */
                __pyx_t_48 = ((__pyx_v_in_flag == 0) != 0);
                if (__pyx_t_48) {

                  /* "dipy/align/vector_fields.pyx":3109
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:
 *                             out[k, i, j, p] = 0             # <<<<<<<<<<<<<<
 *                             inside[k, i, j] = 0
 *                             continue
 */
                  __pyx_t_49 = __pyx_v_k;
                  __pyx_t_50 = __pyx_v_i;
                  __pyx_t_51 = __pyx_v_j;
                  __pyx_t_52 = __pyx_v_p;
                  *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_49 * __pyx_v_out.strides[0]) ) + __pyx_t_50 * __pyx_v_out.strides[1]) ) + __pyx_t_51 * __pyx_v_out.strides[2]) ) + __pyx_t_52 * __pyx_v_out.strides[3]) )) = 0.0;

                  /* "dipy/align/vector_fields.pyx":3110
 *                         if in_flag == 0:
 *                             out[k, i, j, p] = 0
 *                             inside[k, i, j] = 0             # <<<<<<<<<<<<<<
 *                             continue
 *                         tmp = out[k, i, j, p]
 */
                  __pyx_t_53 = __pyx_v_k;
                  __pyx_t_54 = __pyx_v_i;
                  __pyx_t_55 = __pyx_v_j;
                  *((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_53 * __pyx_v_inside.strides[0]) ) + __pyx_t_54 * __pyx_v_inside.strides[1]) ) + __pyx_t_55 * __pyx_v_inside.strides[2]) )) = 0;

                  /* "dipy/align/vector_fields.pyx":3111
 *                             out[k, i, j, p] = 0
 *                             inside[k, i, j] = 0
 *                             continue             # <<<<<<<<<<<<<<
 *                         tmp = out[k, i, j, p]
 *                         # Compute coordinates of point dx on img's grid
 */
                  goto __pyx_L12_continue;

                  /* "dipy/align/vector_fields.pyx":3108
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:             # <<<<<<<<<<<<<<
 *                             out[k, i, j, p] = 0
 *                             inside[k, i, j] = 0
 */
                }

                /* "dipy/align/vector_fields.pyx":3112
 *                             inside[k, i, j] = 0
 *                             continue
 *                         tmp = out[k, i, j, p]             # <<<<<<<<<<<<<<
 *                         # Compute coordinates of point dx on img's grid
 *                         dx[p] = x[p] + h[p]
 */
                __pyx_t_56 = __pyx_v_k;
                __pyx_t_57 = __pyx_v_i;
                __pyx_t_58 = __pyx_v_j;
                __pyx_t_59 = __pyx_v_p;
                __pyx_v_tmp = (*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_56 * __pyx_v_out.strides[0]) ) + __pyx_t_57 * __pyx_v_out.strides[1]) ) + __pyx_t_58 * __pyx_v_out.strides[2]) ) + __pyx_t_59 * __pyx_v_out.strides[3]) )));

                /* "dipy/align/vector_fields.pyx":3114
 *                         tmp = out[k, i, j, p]
 *                         # Compute coordinates of point dx on img's grid
 *                         dx[p] = x[p] + h[p]             # <<<<<<<<<<<<<<
 *                         q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 */
                __pyx_t_60 = __pyx_v_p;
                __pyx_t_61 = __pyx_v_p;
                __pyx_t_62 = __pyx_v_p;
                *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_62 * __pyx_v_dx.strides[0]) )) = ((*((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_60 * __pyx_v_x.strides[0]) ))) + (*((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_61 * __pyx_v_h.strides[0]) ))));

                /* "dipy/align/vector_fields.pyx":3115
 *                         # Compute coordinates of point dx on img's grid
 *                         dx[p] = x[p] + h[p]
 *                         q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                                    img_world2grid)
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 */
                __pyx_t_63 = 0;
                __pyx_t_64 = 1;
                __pyx_t_65 = 2;

                /* "dipy/align/vector_fields.pyx":3116
 *                         dx[p] = x[p] + h[p]
 *                         q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)             # <<<<<<<<<<<<<<
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 */
                __pyx_t_66 = 0;
                *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_66 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_63 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_64 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_65 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

                /* "dipy/align/vector_fields.pyx":3117
 *                         q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                                    img_world2grid)
 *                         q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 */
                __pyx_t_67 = 0;
                __pyx_t_68 = 1;
                __pyx_t_69 = 2;

                /* "dipy/align/vector_fields.pyx":3118
 *                                                    img_world2grid)
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)             # <<<<<<<<<<<<<<
 *                         q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 */
                __pyx_t_70 = 1;
                *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_70 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_67 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_68 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_69 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

                /* "dipy/align/vector_fields.pyx":3119
 *                         q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)
 *                         q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                                    img_world2grid)
 *                         # Interpolate img at q
 */
                __pyx_t_71 = 0;
                __pyx_t_72 = 1;
                __pyx_t_73 = 2;

                /* "dipy/align/vector_fields.pyx":3120
 *                                                    img_world2grid)
 *                         q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 *                                                    img_world2grid)             # <<<<<<<<<<<<<<
 *                         # Interpolate img at q
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],
 */
                __pyx_t_74 = 2;
                *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_74 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_71 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_72 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_73 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

                /* "dipy/align/vector_fields.pyx":3122
 *                                                    img_world2grid)
 *                         # Interpolate img at q
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],             # <<<<<<<<<<<<<<
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:
 */
                __pyx_t_75 = 0;

                /* "dipy/align/vector_fields.pyx":3123
 *                         # Interpolate img at q
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],
 *                             q[1], q[2], &out[k, i, j, p])             # <<<<<<<<<<<<<<
 *                         if in_flag == 0:
 *                             out[k, i, j, p] = 0
 */
                __pyx_t_76 = 1;
                __pyx_t_77 = 2;
                __pyx_t_78 = __pyx_v_k;
                __pyx_t_79 = __pyx_v_i;
                __pyx_t_80 = __pyx_v_j;
                __pyx_t_81 = __pyx_v_p;

                /* "dipy/align/vector_fields.pyx":3122
 *                                                    img_world2grid)
 *                         # Interpolate img at q
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],             # <<<<<<<<<<<<<<
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:
 */
                __pyx_v_in_flag = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__pyx_v_img, (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_75 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_76 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_77 * __pyx_v_q.strides[0]) ))), (&(*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_78 * __pyx_v_out.strides[0]) ) + __pyx_t_79 * __pyx_v_out.strides[1]) ) + __pyx_t_80 * __pyx_v_out.strides[2]) ) + __pyx_t_81 * __pyx_v_out.strides[3]) )))));

                /* "dipy/align/vector_fields.pyx":3124
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:             # <<<<<<<<<<<<<<
 *                             out[k, i, j, p] = 0
 *                             inside[k, i, j] = 0
 */
                __pyx_t_48 = ((__pyx_v_in_flag == 0) != 0);
                if (__pyx_t_48) {

                  /* "dipy/align/vector_fields.pyx":3125
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:
 *                             out[k, i, j, p] = 0             # <<<<<<<<<<<<<<
 *                             inside[k, i, j] = 0
 *                             continue
 */
                  __pyx_t_82 = __pyx_v_k;
                  __pyx_t_83 = __pyx_v_i;
                  __pyx_t_84 = __pyx_v_j;
                  __pyx_t_85 = __pyx_v_p;
                  *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_82 * __pyx_v_out.strides[0]) ) + __pyx_t_83 * __pyx_v_out.strides[1]) ) + __pyx_t_84 * __pyx_v_out.strides[2]) ) + __pyx_t_85 * __pyx_v_out.strides[3]) )) = 0.0;

                  /* "dipy/align/vector_fields.pyx":3126
 *                         if in_flag == 0:
 *                             out[k, i, j, p] = 0
 *                             inside[k, i, j] = 0             # <<<<<<<<<<<<<<
 *                             continue
 *                         out[k, i, j, p] = (out[k, i, j, p] - tmp) / img_spacing[p]
 */
                  __pyx_t_86 = __pyx_v_k;
                  __pyx_t_87 = __pyx_v_i;
                  __pyx_t_88 = __pyx_v_j;
                  *((int *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_86 * __pyx_v_inside.strides[0]) ) + __pyx_t_87 * __pyx_v_inside.strides[1]) ) + __pyx_t_88 * __pyx_v_inside.strides[2]) )) = 0;

                  /* "dipy/align/vector_fields.pyx":3127
 *                             out[k, i, j, p] = 0
 *                             inside[k, i, j] = 0
 *                             continue             # <<<<<<<<<<<<<<
 *                         out[k, i, j, p] = (out[k, i, j, p] - tmp) / img_spacing[p]
 *                         dx[p] = x[p]
 */
                  goto __pyx_L12_continue;

                  /* "dipy/align/vector_fields.pyx":3124
 *                         in_flag = _interpolate_scalar_3d[floating](img, q[0],
 *                             q[1], q[2], &out[k, i, j, p])
 *                         if in_flag == 0:             # <<<<<<<<<<<<<<
 *                             out[k, i, j, p] = 0
 *                             inside[k, i, j] = 0
 */
                }

                /* "dipy/align/vector_fields.pyx":3128
 *                             inside[k, i, j] = 0
 *                             continue
 *                         out[k, i, j, p] = (out[k, i, j, p] - tmp) / img_spacing[p]             # <<<<<<<<<<<<<<
 *                         dx[p] = x[p]
 * 
 */
                __pyx_t_89 = __pyx_v_k;
                __pyx_t_90 = __pyx_v_i;
                __pyx_t_91 = __pyx_v_j;
                __pyx_t_92 = __pyx_v_p;
                __pyx_t_93 = __pyx_v_p;
                __pyx_t_94 = __pyx_v_k;
                __pyx_t_95 = __pyx_v_i;
                __pyx_t_96 = __pyx_v_j;
                __pyx_t_97 = __pyx_v_p;
                *((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_94 * __pyx_v_out.strides[0]) ) + __pyx_t_95 * __pyx_v_out.strides[1]) ) + __pyx_t_96 * __pyx_v_out.strides[2]) ) + __pyx_t_97 * __pyx_v_out.strides[3]) )) = (((*((double *) ( /* dim=3 */ (( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_89 * __pyx_v_out.strides[0]) ) + __pyx_t_90 * __pyx_v_out.strides[1]) ) + __pyx_t_91 * __pyx_v_out.strides[2]) ) + __pyx_t_92 * __pyx_v_out.strides[3]) ))) - __pyx_v_tmp) / (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_93 * __pyx_v_img_spacing.strides[0]) ))));

                /* "dipy/align/vector_fields.pyx":3129
 *                             continue
 *                         out[k, i, j, p] = (out[k, i, j, p] - tmp) / img_spacing[p]
 *                         dx[p] = x[p]             # <<<<<<<<<<<<<<
 * 
 * 
 */
                __pyx_t_98 = __pyx_v_p;
                __pyx_t_99 = __pyx_v_p;
                *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_99 * __pyx_v_dx.strides[0]) )) = (*((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_98 * __pyx_v_x.strides[0]) )));
                __pyx_L12_continue:;
              }
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":3083
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L4_error: {
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L1_error;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":3036
 * 
 * 
 * def _gradient_3d(floating[:, :, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                  double[:] img_spacing, double[:, :] out_grid2world,
 *                  floating[:, :, :, :] out, int[:, :, :] inside):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_24, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields._gradient_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_x, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_dx, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_h, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_q, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img_world2grid, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img_spacing, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_grid2world, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":3132
 * 
 * 
 * def _sparse_gradient_3d(floating[:, :, :] img,             # <<<<<<<<<<<<<<
 *                         double[:, :] img_world2grid,
 *                         double[:] img_spacing,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_73_sparse_gradient_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_72_sparse_gradient_3d[] = " Gradient of a 3D image evaluated at a set of points in physical space\n\n    For each row (x_i, y_i, z_i) in sample_points, the image is interpolated at\n\n    P1=(x_i + h, y_i, z_i), Q1=(x_i - h, y_i, z_i)\n    P2=(x_i, y_i + h, z_i), Q2=(x_i, y_i - h, z_i)\n    P3=(x_i, y_i, z_i + h), Q3=(x_i, y_i, z_i - h)\n\n    (by mapping Pi and Qi to the grid using img_world2grid: the inverse of the\n    grid-to-space transform of img). The displacement parameter h is of\n    magnitude 0.5 (in physical space units), therefore the approximated partial\n    derivatives are given by the difference between the image interpolated at\n    Pi and Qi.\n\n    Parameters\n    ----------\n    img : array, shape (S, R, C)\n        the input volume whose gradient will be computed\n    img_world2grid : array, shape (4, 4)\n        the space-to-grid transform matrix associated to img\n    img_spacing : array, shape (3,)\n        the spacing between voxels (voxel size along each axis) of the input\n    sample_points: array, shape (n, 3)\n        list of points where the derivative will be evaluated\n        (one point per row)\n    out : array, shape (n, 3)\n        the buffer in which to store the image gradient\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_73_sparse_gradient_3d = {"_sparse_gradient_3d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_73_sparse_gradient_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_72_sparse_gradient_3d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_73_sparse_gradient_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_72_sparse_gradient_3d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_72_sparse_gradient_3d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_sparse_gradient_3d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_img, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_img); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 3) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__340, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__341, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__342, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__343, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_345_sparse_gradient_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_345_sparse_gradient_3d = {"__pyx_fuse_0_sparse_gradient_3d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_345_sparse_gradient_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_72_sparse_gradient_3d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_345_sparse_gradient_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_img = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_img_world2grid = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_img_spacing = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_sample_points = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_sparse_gradient_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_img,&__pyx_n_s_img_world2grid,&__pyx_n_s_img_spacing,&__pyx_n_s_sample_points,&__pyx_n_s_out,&__pyx_n_s_inside,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_world2grid)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_3d", 1, 6, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_spacing)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_3d", 1, 6, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sample_points)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_3d", 1, 6, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_3d", 1, 6, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_inside)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_3d", 1, 6, 6, 5); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_sparse_gradient_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_img = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[0]); if (unlikely(!__pyx_v_img.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_img_world2grid = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_img_world2grid.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3133; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_img_spacing = __Pyx_PyObject_to_MemoryviewSlice_ds_double(values[2]); if (unlikely(!__pyx_v_img_spacing.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3134; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_sample_points = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_sample_points.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3135; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(values[4]); if (unlikely(!__pyx_v_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3136; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_inside = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_inside.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3136; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_sparse_gradient_3d", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields._sparse_gradient_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_344_sparse_gradient_3d(__pyx_self, __pyx_v_img, __pyx_v_img_world2grid, __pyx_v_img_spacing, __pyx_v_sample_points, __pyx_v_out, __pyx_v_inside);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_344_sparse_gradient_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_img, __Pyx_memviewslice __pyx_v_img_world2grid, __Pyx_memviewslice __pyx_v_img_spacing, __Pyx_memviewslice __pyx_v_sample_points, __Pyx_memviewslice __pyx_v_out, __Pyx_memviewslice __pyx_v_inside) {
  int __pyx_v_n;
  int __pyx_v_i;
  int __pyx_v_in_flag;
  double __pyx_v_tmp;
  __Pyx_memviewslice __pyx_v_dx = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_h = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_q = { 0, 0, { 0 }, { 0 }, { 0 } };
  long __pyx_v_p;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  int __pyx_t_15;
  __Pyx_memviewslice __pyx_t_16 = { 0, 0, { 0 }, { 0 }, { 0 } };
  long __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  int __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  Py_ssize_t __pyx_t_60;
  Py_ssize_t __pyx_t_61;
  Py_ssize_t __pyx_t_62;
  Py_ssize_t __pyx_t_63;
  Py_ssize_t __pyx_t_64;
  Py_ssize_t __pyx_t_65;
  Py_ssize_t __pyx_t_66;
  Py_ssize_t __pyx_t_67;
  Py_ssize_t __pyx_t_68;
  Py_ssize_t __pyx_t_69;
  Py_ssize_t __pyx_t_70;
  Py_ssize_t __pyx_t_71;
  Py_ssize_t __pyx_t_72;
  Py_ssize_t __pyx_t_73;
  Py_ssize_t __pyx_t_74;
  Py_ssize_t __pyx_t_75;
  Py_ssize_t __pyx_t_76;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0_sparse_gradient_3d", 0);

  /* "dipy/align/vector_fields.pyx":3166
 *     """
 *     cdef:
 *         int n = sample_points.shape[0]             # <<<<<<<<<<<<<<
 *         int i, in_flag
 *         double tmp
 */
  __pyx_v_n = (__pyx_v_sample_points.shape[0]);

  /* "dipy/align/vector_fields.pyx":3169
 *         int i, in_flag
 *         double tmp
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_tuple__344) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_float64); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_dx = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3170
 *         double tmp
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_empty); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__345) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_float64); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_3);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_h = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3171
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_empty); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__346) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_float64); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_2);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_q = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3172
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":3173
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]             # <<<<<<<<<<<<<<
 *         h[1] = 0.5 * img_spacing[1]
 *         h[2] = 0.5 * img_spacing[2]
 */
        __pyx_t_6 = 0;
        __pyx_t_7 = 0;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_7 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_6 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3174
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]             # <<<<<<<<<<<<<<
 *         h[2] = 0.5 * img_spacing[2]
 *         for i in range(n):
 */
        __pyx_t_8 = 1;
        __pyx_t_9 = 1;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_9 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_8 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3175
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 *         h[2] = 0.5 * img_spacing[2]             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = 1
 */
        __pyx_t_10 = 2;
        __pyx_t_11 = 2;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_11 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_10 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3176
 *         h[1] = 0.5 * img_spacing[1]
 *         h[2] = 0.5 * img_spacing[2]
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = 1
 *             dx[:] = sample_points[i, :3]
 */
        __pyx_t_12 = __pyx_v_n;
        for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
          __pyx_v_i = __pyx_t_13;

          /* "dipy/align/vector_fields.pyx":3177
 *         h[2] = 0.5 * img_spacing[2]
 *         for i in range(n):
 *             inside[i] = 1             # <<<<<<<<<<<<<<
 *             dx[:] = sample_points[i, :3]
 *             for p in range(3):
 */
          __pyx_t_14 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_14 * __pyx_v_inside.strides[0]) )) = 1;

          /* "dipy/align/vector_fields.pyx":3178
 *         for i in range(n):
 *             inside[i] = 1
 *             dx[:] = sample_points[i, :3]             # <<<<<<<<<<<<<<
 *             for p in range(3):
 *                 # Compute coordinates of point dx on img's grid
 */
          __pyx_t_5.data = __pyx_v_sample_points.data;
          __pyx_t_5.memview = __pyx_v_sample_points.memview;
          __PYX_INC_MEMVIEW(&__pyx_t_5, 0);
          {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_sample_points.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_sample_points.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3178; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
    }
        __pyx_t_5.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_15 = -1;
          if (unlikely(__pyx_memoryview_slice_memviewslice(
    &__pyx_t_5,
    __pyx_v_sample_points.shape[1], __pyx_v_sample_points.strides[1], __pyx_v_sample_points.suboffsets[1],
    1,
    0,
    &__pyx_t_15,
    0,
    3,
    0,
    0,
    1,
    0,
    1) < 0))
{
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3178; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
}

__pyx_t_16.data = __pyx_v_dx.data;
          __pyx_t_16.memview = __pyx_v_dx.memview;
          __PYX_INC_MEMVIEW(&__pyx_t_16, 0);
          __pyx_t_16.shape[0] = __pyx_v_dx.shape[0];
__pyx_t_16.strides[0] = __pyx_v_dx.strides[0];
    __pyx_t_16.suboffsets[0] = -1;

if (unlikely(__pyx_memoryview_copy_contents(__pyx_t_5, __pyx_t_16, 1, 1, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3178; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
          __PYX_XDEC_MEMVIEW(&__pyx_t_16, 0);
          __PYX_XDEC_MEMVIEW(&__pyx_t_5, 0);

          /* "dipy/align/vector_fields.pyx":3179
 *             inside[i] = 1
 *             dx[:] = sample_points[i, :3]
 *             for p in range(3):             # <<<<<<<<<<<<<<
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] - h[p]
 */
          for (__pyx_t_17 = 0; __pyx_t_17 < 3; __pyx_t_17+=1) {
            __pyx_v_p = __pyx_t_17;

            /* "dipy/align/vector_fields.pyx":3181
 *             for p in range(3):
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] - h[p]             # <<<<<<<<<<<<<<
 *                 q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 */
            __pyx_t_18 = __pyx_v_i;
            __pyx_t_19 = __pyx_v_p;
            __pyx_t_20 = __pyx_v_p;
            __pyx_t_21 = __pyx_v_p;
            *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_21 * __pyx_v_dx.strides[0]) )) = ((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_sample_points.data + __pyx_t_18 * __pyx_v_sample_points.strides[0]) ) + __pyx_t_19 * __pyx_v_sample_points.strides[1]) ))) - (*((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_20 * __pyx_v_h.strides[0]) ))));

            /* "dipy/align/vector_fields.pyx":3182
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] - h[p]
 *                 q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                            img_world2grid)
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 */
            __pyx_t_22 = 0;
            __pyx_t_23 = 1;
            __pyx_t_24 = 2;

            /* "dipy/align/vector_fields.pyx":3183
 *                 dx[p] = sample_points[i, p] - h[p]
 *                 q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)             # <<<<<<<<<<<<<<
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 */
            __pyx_t_25 = 0;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_25 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_22 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_23 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_24 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3184
 *                 q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                            img_world2grid)
 *                 q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 */
            __pyx_t_26 = 0;
            __pyx_t_27 = 1;
            __pyx_t_28 = 2;

            /* "dipy/align/vector_fields.pyx":3185
 *                                            img_world2grid)
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)             # <<<<<<<<<<<<<<
 *                 q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 */
            __pyx_t_29 = 1;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_29 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_26 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_27 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_28 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3186
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 *                 q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                            img_world2grid)
 *                 # Interpolate img at q
 */
            __pyx_t_30 = 0;
            __pyx_t_31 = 1;
            __pyx_t_32 = 2;

            /* "dipy/align/vector_fields.pyx":3187
 *                                            img_world2grid)
 *                 q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)             # <<<<<<<<<<<<<<
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1],
 */
            __pyx_t_33 = 2;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_33 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_30 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_31 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_32 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3189
 *                                            img_world2grid)
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1],             # <<<<<<<<<<<<<<
 *                     q[2], &out[i, p])
 *                 if in_flag == 0:
 */
            __pyx_t_34 = 0;
            __pyx_t_35 = 1;

            /* "dipy/align/vector_fields.pyx":3190
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1],
 *                     q[2], &out[i, p])             # <<<<<<<<<<<<<<
 *                 if in_flag == 0:
 *                     out[i, p] = 0
 */
            __pyx_t_36 = 2;
            __pyx_t_37 = __pyx_v_i;
            __pyx_t_38 = __pyx_v_p;

            /* "dipy/align/vector_fields.pyx":3189
 *                                            img_world2grid)
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1],             # <<<<<<<<<<<<<<
 *                     q[2], &out[i, p])
 *                 if in_flag == 0:
 */
            __pyx_v_in_flag = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__pyx_v_img, (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_34 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_35 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_36 * __pyx_v_q.strides[0]) ))), (&(*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_37 * __pyx_v_out.strides[0]) ) + __pyx_t_38 * __pyx_v_out.strides[1]) )))));

            /* "dipy/align/vector_fields.pyx":3191
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1],
 *                     q[2], &out[i, p])
 *                 if in_flag == 0:             # <<<<<<<<<<<<<<
 *                     out[i, p] = 0
 *                     inside[i] = 0
 */
            __pyx_t_39 = ((__pyx_v_in_flag == 0) != 0);
            if (__pyx_t_39) {

              /* "dipy/align/vector_fields.pyx":3192
 *                     q[2], &out[i, p])
 *                 if in_flag == 0:
 *                     out[i, p] = 0             # <<<<<<<<<<<<<<
 *                     inside[i] = 0
 *                     continue
 */
              __pyx_t_40 = __pyx_v_i;
              __pyx_t_41 = __pyx_v_p;
              *((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_40 * __pyx_v_out.strides[0]) ) + __pyx_t_41 * __pyx_v_out.strides[1]) )) = 0.0;

              /* "dipy/align/vector_fields.pyx":3193
 *                 if in_flag == 0:
 *                     out[i, p] = 0
 *                     inside[i] = 0             # <<<<<<<<<<<<<<
 *                     continue
 *                 tmp = out[i, p]
 */
              __pyx_t_42 = __pyx_v_i;
              *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_42 * __pyx_v_inside.strides[0]) )) = 0;

              /* "dipy/align/vector_fields.pyx":3194
 *                     out[i, p] = 0
 *                     inside[i] = 0
 *                     continue             # <<<<<<<<<<<<<<
 *                 tmp = out[i, p]
 *                 # Compute coordinates of point dx on img's grid
 */
              goto __pyx_L8_continue;

              /* "dipy/align/vector_fields.pyx":3191
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1],
 *                     q[2], &out[i, p])
 *                 if in_flag == 0:             # <<<<<<<<<<<<<<
 *                     out[i, p] = 0
 *                     inside[i] = 0
 */
            }

            /* "dipy/align/vector_fields.pyx":3195
 *                     inside[i] = 0
 *                     continue
 *                 tmp = out[i, p]             # <<<<<<<<<<<<<<
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] + h[p]
 */
            __pyx_t_43 = __pyx_v_i;
            __pyx_t_44 = __pyx_v_p;
            __pyx_v_tmp = (*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_43 * __pyx_v_out.strides[0]) ) + __pyx_t_44 * __pyx_v_out.strides[1]) )));

            /* "dipy/align/vector_fields.pyx":3197
 *                 tmp = out[i, p]
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] + h[p]             # <<<<<<<<<<<<<<
 *                 q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 */
            __pyx_t_45 = __pyx_v_i;
            __pyx_t_46 = __pyx_v_p;
            __pyx_t_47 = __pyx_v_p;
            __pyx_t_48 = __pyx_v_p;
            *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_48 * __pyx_v_dx.strides[0]) )) = ((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_sample_points.data + __pyx_t_45 * __pyx_v_sample_points.strides[0]) ) + __pyx_t_46 * __pyx_v_sample_points.strides[1]) ))) + (*((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_47 * __pyx_v_h.strides[0]) ))));

            /* "dipy/align/vector_fields.pyx":3198
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] + h[p]
 *                 q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                            img_world2grid)
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 */
            __pyx_t_49 = 0;
            __pyx_t_50 = 1;
            __pyx_t_51 = 2;

            /* "dipy/align/vector_fields.pyx":3199
 *                 dx[p] = sample_points[i, p] + h[p]
 *                 q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)             # <<<<<<<<<<<<<<
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 */
            __pyx_t_52 = 0;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_52 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_49 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_50 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_51 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3200
 *                 q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                            img_world2grid)
 *                 q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 */
            __pyx_t_53 = 0;
            __pyx_t_54 = 1;
            __pyx_t_55 = 2;

            /* "dipy/align/vector_fields.pyx":3201
 *                                            img_world2grid)
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)             # <<<<<<<<<<<<<<
 *                 q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 */
            __pyx_t_56 = 1;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_56 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_53 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_54 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_55 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3202
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 *                 q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                            img_world2grid)
 *                 # Interpolate img at q
 */
            __pyx_t_57 = 0;
            __pyx_t_58 = 1;
            __pyx_t_59 = 2;

            /* "dipy/align/vector_fields.pyx":3203
 *                                            img_world2grid)
 *                 q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)             # <<<<<<<<<<<<<<
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1], q[2],
 */
            __pyx_t_60 = 2;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_60 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_57 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_58 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_59 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3205
 *                                            img_world2grid)
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1], q[2],             # <<<<<<<<<<<<<<
 *                                                  &out[i, p])
 *                 if in_flag == 0:
 */
            __pyx_t_61 = 0;
            __pyx_t_62 = 1;
            __pyx_t_63 = 2;

            /* "dipy/align/vector_fields.pyx":3206
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1], q[2],
 *                                                  &out[i, p])             # <<<<<<<<<<<<<<
 *                 if in_flag == 0:
 *                     out[i, p] = 0
 */
            __pyx_t_64 = __pyx_v_i;
            __pyx_t_65 = __pyx_v_p;

            /* "dipy/align/vector_fields.pyx":3205
 *                                            img_world2grid)
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1], q[2],             # <<<<<<<<<<<<<<
 *                                                  &out[i, p])
 *                 if in_flag == 0:
 */
            __pyx_v_in_flag = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__pyx_v_img, (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_61 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_62 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_63 * __pyx_v_q.strides[0]) ))), (&(*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_64 * __pyx_v_out.strides[0]) ) + __pyx_t_65 * __pyx_v_out.strides[1]) )))));

            /* "dipy/align/vector_fields.pyx":3207
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1], q[2],
 *                                                  &out[i, p])
 *                 if in_flag == 0:             # <<<<<<<<<<<<<<
 *                     out[i, p] = 0
 *                     inside[i] = 0
 */
            __pyx_t_39 = ((__pyx_v_in_flag == 0) != 0);
            if (__pyx_t_39) {

              /* "dipy/align/vector_fields.pyx":3208
 *                                                  &out[i, p])
 *                 if in_flag == 0:
 *                     out[i, p] = 0             # <<<<<<<<<<<<<<
 *                     inside[i] = 0
 *                     continue
 */
              __pyx_t_66 = __pyx_v_i;
              __pyx_t_67 = __pyx_v_p;
              *((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_66 * __pyx_v_out.strides[0]) ) + __pyx_t_67 * __pyx_v_out.strides[1]) )) = 0.0;

              /* "dipy/align/vector_fields.pyx":3209
 *                 if in_flag == 0:
 *                     out[i, p] = 0
 *                     inside[i] = 0             # <<<<<<<<<<<<<<
 *                     continue
 *                 out[i, p] = (out[i, p] - tmp) / img_spacing[p]
 */
              __pyx_t_68 = __pyx_v_i;
              *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_68 * __pyx_v_inside.strides[0]) )) = 0;

              /* "dipy/align/vector_fields.pyx":3210
 *                     out[i, p] = 0
 *                     inside[i] = 0
 *                     continue             # <<<<<<<<<<<<<<
 *                 out[i, p] = (out[i, p] - tmp) / img_spacing[p]
 *                 dx[p] = sample_points[i, p]
 */
              goto __pyx_L8_continue;

              /* "dipy/align/vector_fields.pyx":3207
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1], q[2],
 *                                                  &out[i, p])
 *                 if in_flag == 0:             # <<<<<<<<<<<<<<
 *                     out[i, p] = 0
 *                     inside[i] = 0
 */
            }

            /* "dipy/align/vector_fields.pyx":3211
 *                     inside[i] = 0
 *                     continue
 *                 out[i, p] = (out[i, p] - tmp) / img_spacing[p]             # <<<<<<<<<<<<<<
 *                 dx[p] = sample_points[i, p]
 * 
 */
            __pyx_t_69 = __pyx_v_i;
            __pyx_t_70 = __pyx_v_p;
            __pyx_t_71 = __pyx_v_p;
            __pyx_t_72 = __pyx_v_i;
            __pyx_t_73 = __pyx_v_p;
            *((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_72 * __pyx_v_out.strides[0]) ) + __pyx_t_73 * __pyx_v_out.strides[1]) )) = (((*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_69 * __pyx_v_out.strides[0]) ) + __pyx_t_70 * __pyx_v_out.strides[1]) ))) - __pyx_v_tmp) / (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_71 * __pyx_v_img_spacing.strides[0]) ))));

            /* "dipy/align/vector_fields.pyx":3212
 *                     continue
 *                 out[i, p] = (out[i, p] - tmp) / img_spacing[p]
 *                 dx[p] = sample_points[i, p]             # <<<<<<<<<<<<<<
 * 
 * 
 */
            __pyx_t_74 = __pyx_v_i;
            __pyx_t_75 = __pyx_v_p;
            __pyx_t_76 = __pyx_v_p;
            *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_76 * __pyx_v_dx.strides[0]) )) = (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_sample_points.data + __pyx_t_74 * __pyx_v_sample_points.strides[0]) ) + __pyx_t_75 * __pyx_v_sample_points.strides[1]) )));
            __pyx_L8_continue:;
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":3172
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L4_error: {
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L1_error;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":3132
 * 
 * 
 * def _sparse_gradient_3d(floating[:, :, :] img,             # <<<<<<<<<<<<<<
 *                         double[:, :] img_world2grid,
 *                         double[:] img_spacing,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_16, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields._sparse_gradient_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_dx, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_h, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_q, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img_world2grid, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img_spacing, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_sample_points, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_347_sparse_gradient_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_347_sparse_gradient_3d = {"__pyx_fuse_1_sparse_gradient_3d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_347_sparse_gradient_3d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_72_sparse_gradient_3d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_347_sparse_gradient_3d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_img = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_img_world2grid = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_img_spacing = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_sample_points = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_sparse_gradient_3d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_img,&__pyx_n_s_img_world2grid,&__pyx_n_s_img_spacing,&__pyx_n_s_sample_points,&__pyx_n_s_out,&__pyx_n_s_inside,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_world2grid)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_3d", 1, 6, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_spacing)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_3d", 1, 6, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sample_points)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_3d", 1, 6, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_3d", 1, 6, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_inside)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_3d", 1, 6, 6, 5); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_sparse_gradient_3d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_img = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[0]); if (unlikely(!__pyx_v_img.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_img_world2grid = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_img_world2grid.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3133; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_img_spacing = __Pyx_PyObject_to_MemoryviewSlice_ds_double(values[2]); if (unlikely(!__pyx_v_img_spacing.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3134; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_sample_points = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_sample_points.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3135; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3136; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_inside = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_inside.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3136; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_sparse_gradient_3d", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields._sparse_gradient_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_346_sparse_gradient_3d(__pyx_self, __pyx_v_img, __pyx_v_img_world2grid, __pyx_v_img_spacing, __pyx_v_sample_points, __pyx_v_out, __pyx_v_inside);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_346_sparse_gradient_3d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_img, __Pyx_memviewslice __pyx_v_img_world2grid, __Pyx_memviewslice __pyx_v_img_spacing, __Pyx_memviewslice __pyx_v_sample_points, __Pyx_memviewslice __pyx_v_out, __Pyx_memviewslice __pyx_v_inside) {
  int __pyx_v_n;
  int __pyx_v_i;
  int __pyx_v_in_flag;
  double __pyx_v_tmp;
  __Pyx_memviewslice __pyx_v_dx = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_h = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_q = { 0, 0, { 0 }, { 0 }, { 0 } };
  long __pyx_v_p;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  Py_ssize_t __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  int __pyx_t_15;
  __Pyx_memviewslice __pyx_t_16 = { 0, 0, { 0 }, { 0 }, { 0 } };
  long __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  int __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  Py_ssize_t __pyx_t_60;
  Py_ssize_t __pyx_t_61;
  Py_ssize_t __pyx_t_62;
  Py_ssize_t __pyx_t_63;
  Py_ssize_t __pyx_t_64;
  Py_ssize_t __pyx_t_65;
  Py_ssize_t __pyx_t_66;
  Py_ssize_t __pyx_t_67;
  Py_ssize_t __pyx_t_68;
  Py_ssize_t __pyx_t_69;
  Py_ssize_t __pyx_t_70;
  Py_ssize_t __pyx_t_71;
  Py_ssize_t __pyx_t_72;
  Py_ssize_t __pyx_t_73;
  Py_ssize_t __pyx_t_74;
  Py_ssize_t __pyx_t_75;
  Py_ssize_t __pyx_t_76;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1_sparse_gradient_3d", 0);

  /* "dipy/align/vector_fields.pyx":3166
 *     """
 *     cdef:
 *         int n = sample_points.shape[0]             # <<<<<<<<<<<<<<
 *         int i, in_flag
 *         double tmp
 */
  __pyx_v_n = (__pyx_v_sample_points.shape[0]);

  /* "dipy/align/vector_fields.pyx":3169
 *         int i, in_flag
 *         double tmp
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_tuple__347) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_float64); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_dx = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3170
 *         double tmp
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_empty); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__348) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_float64); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_3);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_h = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3171
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_empty); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__349) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_float64); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_2);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_q = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3172
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":3173
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]             # <<<<<<<<<<<<<<
 *         h[1] = 0.5 * img_spacing[1]
 *         h[2] = 0.5 * img_spacing[2]
 */
        __pyx_t_6 = 0;
        __pyx_t_7 = 0;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_7 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_6 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3174
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]             # <<<<<<<<<<<<<<
 *         h[2] = 0.5 * img_spacing[2]
 *         for i in range(n):
 */
        __pyx_t_8 = 1;
        __pyx_t_9 = 1;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_9 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_8 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3175
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 *         h[2] = 0.5 * img_spacing[2]             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = 1
 */
        __pyx_t_10 = 2;
        __pyx_t_11 = 2;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_11 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_10 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3176
 *         h[1] = 0.5 * img_spacing[1]
 *         h[2] = 0.5 * img_spacing[2]
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = 1
 *             dx[:] = sample_points[i, :3]
 */
        __pyx_t_12 = __pyx_v_n;
        for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
          __pyx_v_i = __pyx_t_13;

          /* "dipy/align/vector_fields.pyx":3177
 *         h[2] = 0.5 * img_spacing[2]
 *         for i in range(n):
 *             inside[i] = 1             # <<<<<<<<<<<<<<
 *             dx[:] = sample_points[i, :3]
 *             for p in range(3):
 */
          __pyx_t_14 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_14 * __pyx_v_inside.strides[0]) )) = 1;

          /* "dipy/align/vector_fields.pyx":3178
 *         for i in range(n):
 *             inside[i] = 1
 *             dx[:] = sample_points[i, :3]             # <<<<<<<<<<<<<<
 *             for p in range(3):
 *                 # Compute coordinates of point dx on img's grid
 */
          __pyx_t_5.data = __pyx_v_sample_points.data;
          __pyx_t_5.memview = __pyx_v_sample_points.memview;
          __PYX_INC_MEMVIEW(&__pyx_t_5, 0);
          {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_sample_points.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_sample_points.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3178; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
    }
        __pyx_t_5.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_15 = -1;
          if (unlikely(__pyx_memoryview_slice_memviewslice(
    &__pyx_t_5,
    __pyx_v_sample_points.shape[1], __pyx_v_sample_points.strides[1], __pyx_v_sample_points.suboffsets[1],
    1,
    0,
    &__pyx_t_15,
    0,
    3,
    0,
    0,
    1,
    0,
    1) < 0))
{
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3178; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
}

__pyx_t_16.data = __pyx_v_dx.data;
          __pyx_t_16.memview = __pyx_v_dx.memview;
          __PYX_INC_MEMVIEW(&__pyx_t_16, 0);
          __pyx_t_16.shape[0] = __pyx_v_dx.shape[0];
__pyx_t_16.strides[0] = __pyx_v_dx.strides[0];
    __pyx_t_16.suboffsets[0] = -1;

if (unlikely(__pyx_memoryview_copy_contents(__pyx_t_5, __pyx_t_16, 1, 1, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3178; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
          __PYX_XDEC_MEMVIEW(&__pyx_t_16, 0);
          __PYX_XDEC_MEMVIEW(&__pyx_t_5, 0);

          /* "dipy/align/vector_fields.pyx":3179
 *             inside[i] = 1
 *             dx[:] = sample_points[i, :3]
 *             for p in range(3):             # <<<<<<<<<<<<<<
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] - h[p]
 */
          for (__pyx_t_17 = 0; __pyx_t_17 < 3; __pyx_t_17+=1) {
            __pyx_v_p = __pyx_t_17;

            /* "dipy/align/vector_fields.pyx":3181
 *             for p in range(3):
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] - h[p]             # <<<<<<<<<<<<<<
 *                 q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 */
            __pyx_t_18 = __pyx_v_i;
            __pyx_t_19 = __pyx_v_p;
            __pyx_t_20 = __pyx_v_p;
            __pyx_t_21 = __pyx_v_p;
            *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_21 * __pyx_v_dx.strides[0]) )) = ((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_sample_points.data + __pyx_t_18 * __pyx_v_sample_points.strides[0]) ) + __pyx_t_19 * __pyx_v_sample_points.strides[1]) ))) - (*((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_20 * __pyx_v_h.strides[0]) ))));

            /* "dipy/align/vector_fields.pyx":3182
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] - h[p]
 *                 q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                            img_world2grid)
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 */
            __pyx_t_22 = 0;
            __pyx_t_23 = 1;
            __pyx_t_24 = 2;

            /* "dipy/align/vector_fields.pyx":3183
 *                 dx[p] = sample_points[i, p] - h[p]
 *                 q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)             # <<<<<<<<<<<<<<
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 */
            __pyx_t_25 = 0;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_25 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_22 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_23 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_24 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3184
 *                 q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                            img_world2grid)
 *                 q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 */
            __pyx_t_26 = 0;
            __pyx_t_27 = 1;
            __pyx_t_28 = 2;

            /* "dipy/align/vector_fields.pyx":3185
 *                                            img_world2grid)
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)             # <<<<<<<<<<<<<<
 *                 q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 */
            __pyx_t_29 = 1;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_29 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_26 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_27 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_28 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3186
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 *                 q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                            img_world2grid)
 *                 # Interpolate img at q
 */
            __pyx_t_30 = 0;
            __pyx_t_31 = 1;
            __pyx_t_32 = 2;

            /* "dipy/align/vector_fields.pyx":3187
 *                                            img_world2grid)
 *                 q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)             # <<<<<<<<<<<<<<
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1],
 */
            __pyx_t_33 = 2;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_33 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_30 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_31 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_32 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3189
 *                                            img_world2grid)
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1],             # <<<<<<<<<<<<<<
 *                     q[2], &out[i, p])
 *                 if in_flag == 0:
 */
            __pyx_t_34 = 0;
            __pyx_t_35 = 1;

            /* "dipy/align/vector_fields.pyx":3190
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1],
 *                     q[2], &out[i, p])             # <<<<<<<<<<<<<<
 *                 if in_flag == 0:
 *                     out[i, p] = 0
 */
            __pyx_t_36 = 2;
            __pyx_t_37 = __pyx_v_i;
            __pyx_t_38 = __pyx_v_p;

            /* "dipy/align/vector_fields.pyx":3189
 *                                            img_world2grid)
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1],             # <<<<<<<<<<<<<<
 *                     q[2], &out[i, p])
 *                 if in_flag == 0:
 */
            __pyx_v_in_flag = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__pyx_v_img, (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_34 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_35 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_36 * __pyx_v_q.strides[0]) ))), (&(*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_37 * __pyx_v_out.strides[0]) ) + __pyx_t_38 * __pyx_v_out.strides[1]) )))));

            /* "dipy/align/vector_fields.pyx":3191
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1],
 *                     q[2], &out[i, p])
 *                 if in_flag == 0:             # <<<<<<<<<<<<<<
 *                     out[i, p] = 0
 *                     inside[i] = 0
 */
            __pyx_t_39 = ((__pyx_v_in_flag == 0) != 0);
            if (__pyx_t_39) {

              /* "dipy/align/vector_fields.pyx":3192
 *                     q[2], &out[i, p])
 *                 if in_flag == 0:
 *                     out[i, p] = 0             # <<<<<<<<<<<<<<
 *                     inside[i] = 0
 *                     continue
 */
              __pyx_t_40 = __pyx_v_i;
              __pyx_t_41 = __pyx_v_p;
              *((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_40 * __pyx_v_out.strides[0]) ) + __pyx_t_41 * __pyx_v_out.strides[1]) )) = 0.0;

              /* "dipy/align/vector_fields.pyx":3193
 *                 if in_flag == 0:
 *                     out[i, p] = 0
 *                     inside[i] = 0             # <<<<<<<<<<<<<<
 *                     continue
 *                 tmp = out[i, p]
 */
              __pyx_t_42 = __pyx_v_i;
              *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_42 * __pyx_v_inside.strides[0]) )) = 0;

              /* "dipy/align/vector_fields.pyx":3194
 *                     out[i, p] = 0
 *                     inside[i] = 0
 *                     continue             # <<<<<<<<<<<<<<
 *                 tmp = out[i, p]
 *                 # Compute coordinates of point dx on img's grid
 */
              goto __pyx_L8_continue;

              /* "dipy/align/vector_fields.pyx":3191
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1],
 *                     q[2], &out[i, p])
 *                 if in_flag == 0:             # <<<<<<<<<<<<<<
 *                     out[i, p] = 0
 *                     inside[i] = 0
 */
            }

            /* "dipy/align/vector_fields.pyx":3195
 *                     inside[i] = 0
 *                     continue
 *                 tmp = out[i, p]             # <<<<<<<<<<<<<<
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] + h[p]
 */
            __pyx_t_43 = __pyx_v_i;
            __pyx_t_44 = __pyx_v_p;
            __pyx_v_tmp = (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_43 * __pyx_v_out.strides[0]) ) + __pyx_t_44 * __pyx_v_out.strides[1]) )));

            /* "dipy/align/vector_fields.pyx":3197
 *                 tmp = out[i, p]
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] + h[p]             # <<<<<<<<<<<<<<
 *                 q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 */
            __pyx_t_45 = __pyx_v_i;
            __pyx_t_46 = __pyx_v_p;
            __pyx_t_47 = __pyx_v_p;
            __pyx_t_48 = __pyx_v_p;
            *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_48 * __pyx_v_dx.strides[0]) )) = ((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_sample_points.data + __pyx_t_45 * __pyx_v_sample_points.strides[0]) ) + __pyx_t_46 * __pyx_v_sample_points.strides[1]) ))) + (*((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_47 * __pyx_v_h.strides[0]) ))));

            /* "dipy/align/vector_fields.pyx":3198
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] + h[p]
 *                 q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                            img_world2grid)
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 */
            __pyx_t_49 = 0;
            __pyx_t_50 = 1;
            __pyx_t_51 = 2;

            /* "dipy/align/vector_fields.pyx":3199
 *                 dx[p] = sample_points[i, p] + h[p]
 *                 q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)             # <<<<<<<<<<<<<<
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 */
            __pyx_t_52 = 0;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_52 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_49 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_50 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_51 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3200
 *                 q[0] = _apply_affine_3d_x0(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                            img_world2grid)
 *                 q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 */
            __pyx_t_53 = 0;
            __pyx_t_54 = 1;
            __pyx_t_55 = 2;

            /* "dipy/align/vector_fields.pyx":3201
 *                                            img_world2grid)
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)             # <<<<<<<<<<<<<<
 *                 q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 */
            __pyx_t_56 = 1;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_56 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_53 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_54 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_55 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3202
 *                 q[1] = _apply_affine_3d_x1(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)
 *                 q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,             # <<<<<<<<<<<<<<
 *                                            img_world2grid)
 *                 # Interpolate img at q
 */
            __pyx_t_57 = 0;
            __pyx_t_58 = 1;
            __pyx_t_59 = 2;

            /* "dipy/align/vector_fields.pyx":3203
 *                                            img_world2grid)
 *                 q[2] = _apply_affine_3d_x2(dx[0], dx[1], dx[2], 1,
 *                                            img_world2grid)             # <<<<<<<<<<<<<<
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1], q[2],
 */
            __pyx_t_60 = 2;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_60 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_57 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_58 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_59 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3205
 *                                            img_world2grid)
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1], q[2],             # <<<<<<<<<<<<<<
 *                                                  &out[i, p])
 *                 if in_flag == 0:
 */
            __pyx_t_61 = 0;
            __pyx_t_62 = 1;
            __pyx_t_63 = 2;

            /* "dipy/align/vector_fields.pyx":3206
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1], q[2],
 *                                                  &out[i, p])             # <<<<<<<<<<<<<<
 *                 if in_flag == 0:
 *                     out[i, p] = 0
 */
            __pyx_t_64 = __pyx_v_i;
            __pyx_t_65 = __pyx_v_p;

            /* "dipy/align/vector_fields.pyx":3205
 *                                            img_world2grid)
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1], q[2],             # <<<<<<<<<<<<<<
 *                                                  &out[i, p])
 *                 if in_flag == 0:
 */
            __pyx_v_in_flag = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_3d(__pyx_v_img, (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_61 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_62 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_63 * __pyx_v_q.strides[0]) ))), (&(*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_64 * __pyx_v_out.strides[0]) ) + __pyx_t_65 * __pyx_v_out.strides[1]) )))));

            /* "dipy/align/vector_fields.pyx":3207
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1], q[2],
 *                                                  &out[i, p])
 *                 if in_flag == 0:             # <<<<<<<<<<<<<<
 *                     out[i, p] = 0
 *                     inside[i] = 0
 */
            __pyx_t_39 = ((__pyx_v_in_flag == 0) != 0);
            if (__pyx_t_39) {

              /* "dipy/align/vector_fields.pyx":3208
 *                                                  &out[i, p])
 *                 if in_flag == 0:
 *                     out[i, p] = 0             # <<<<<<<<<<<<<<
 *                     inside[i] = 0
 *                     continue
 */
              __pyx_t_66 = __pyx_v_i;
              __pyx_t_67 = __pyx_v_p;
              *((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_66 * __pyx_v_out.strides[0]) ) + __pyx_t_67 * __pyx_v_out.strides[1]) )) = 0.0;

              /* "dipy/align/vector_fields.pyx":3209
 *                 if in_flag == 0:
 *                     out[i, p] = 0
 *                     inside[i] = 0             # <<<<<<<<<<<<<<
 *                     continue
 *                 out[i, p] = (out[i, p] - tmp) / img_spacing[p]
 */
              __pyx_t_68 = __pyx_v_i;
              *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_68 * __pyx_v_inside.strides[0]) )) = 0;

              /* "dipy/align/vector_fields.pyx":3210
 *                     out[i, p] = 0
 *                     inside[i] = 0
 *                     continue             # <<<<<<<<<<<<<<
 *                 out[i, p] = (out[i, p] - tmp) / img_spacing[p]
 *                 dx[p] = sample_points[i, p]
 */
              goto __pyx_L8_continue;

              /* "dipy/align/vector_fields.pyx":3207
 *                 in_flag = _interpolate_scalar_3d[floating](img, q[0], q[1], q[2],
 *                                                  &out[i, p])
 *                 if in_flag == 0:             # <<<<<<<<<<<<<<
 *                     out[i, p] = 0
 *                     inside[i] = 0
 */
            }

            /* "dipy/align/vector_fields.pyx":3211
 *                     inside[i] = 0
 *                     continue
 *                 out[i, p] = (out[i, p] - tmp) / img_spacing[p]             # <<<<<<<<<<<<<<
 *                 dx[p] = sample_points[i, p]
 * 
 */
            __pyx_t_69 = __pyx_v_i;
            __pyx_t_70 = __pyx_v_p;
            __pyx_t_71 = __pyx_v_p;
            __pyx_t_72 = __pyx_v_i;
            __pyx_t_73 = __pyx_v_p;
            *((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_72 * __pyx_v_out.strides[0]) ) + __pyx_t_73 * __pyx_v_out.strides[1]) )) = (((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_69 * __pyx_v_out.strides[0]) ) + __pyx_t_70 * __pyx_v_out.strides[1]) ))) - __pyx_v_tmp) / (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_71 * __pyx_v_img_spacing.strides[0]) ))));

            /* "dipy/align/vector_fields.pyx":3212
 *                     continue
 *                 out[i, p] = (out[i, p] - tmp) / img_spacing[p]
 *                 dx[p] = sample_points[i, p]             # <<<<<<<<<<<<<<
 * 
 * 
 */
            __pyx_t_74 = __pyx_v_i;
            __pyx_t_75 = __pyx_v_p;
            __pyx_t_76 = __pyx_v_p;
            *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_76 * __pyx_v_dx.strides[0]) )) = (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_sample_points.data + __pyx_t_74 * __pyx_v_sample_points.strides[0]) ) + __pyx_t_75 * __pyx_v_sample_points.strides[1]) )));
            __pyx_L8_continue:;
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":3172
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L4_error: {
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L1_error;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":3132
 * 
 * 
 * def _sparse_gradient_3d(floating[:, :, :] img,             # <<<<<<<<<<<<<<
 *                         double[:, :] img_world2grid,
 *                         double[:] img_spacing,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_16, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields._sparse_gradient_3d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_dx, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_h, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_q, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img_world2grid, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img_spacing, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_sample_points, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":3215
 * 
 * 
 * def _gradient_2d(floating[:, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                  double[:] img_spacing, double[:, :] out_grid2world,
 *                  floating[:, :, :] out, int[:, :] inside):
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_75_gradient_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_74_gradient_2d[] = " Gradient of a 2D image in physical space coordinates\n\n    Each grid cell (i, j) in the sampling grid (determined by\n    out.shape) is mapped to its corresponding physical point (x, y) by\n    multiplying out_grid2world (its grid-to-space transform) by (i, j), then\n    the image is interpolated, at\n\n    P1=(x + h, y), Q1=(x - h, y)\n    P2=(x, y + h), Q2=(x, y - h)\n\n    (by mapping Pi and Qi to the grid using img_world2grid: the inverse of the\n    grid-to-space transform of img). The displacement parameter h is of\n    magnitude 0.5 (in physical space units), therefore the approximated partial\n    derivatives are given by the difference between the image interpolated at\n    Pi and Qi.\n\n    Parameters\n    ----------\n    img : array, shape (R, C)\n        the input image whose gradient will be computed\n    img_world2grid : array, shape (3, 3)\n        the space-to-grid transform matrix associated to img\n    img_spacing : array, shape (2,)\n        the spacing between pixels (pixel size along each axis) of the input\n        image\n    out_grid2world : array, shape (3, 3)\n        the grid-to-space transform associated to the sampling grid\n    out : array, shape (S', R', 2)\n        the buffer in which to store the image gradient\n    inside : array, shape (S', R')\n        the buffer in which to store the flags indicating whether the sample\n        point lies inside (=1) or outside (=0) the image grid\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_75_gradient_2d = {"_gradient_2d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_75_gradient_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_74_gradient_2d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_75_gradient_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_74_gradient_2d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_74_gradient_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_gradient_2d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_img, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_img); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__350, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__351, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__352, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__353, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_351_gradient_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_351_gradient_2d = {"__pyx_fuse_0_gradient_2d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_351_gradient_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_74_gradient_2d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_351_gradient_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_img = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_img_world2grid = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_img_spacing = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_grid2world = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_gradient_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_img,&__pyx_n_s_img_world2grid,&__pyx_n_s_img_spacing,&__pyx_n_s_out_grid2world,&__pyx_n_s_out,&__pyx_n_s_inside,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_world2grid)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_2d", 1, 6, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_spacing)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_2d", 1, 6, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_grid2world)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_2d", 1, 6, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_2d", 1, 6, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_inside)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_2d", 1, 6, 6, 5); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_gradient_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_img = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(values[0]); if (unlikely(!__pyx_v_img.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_img_world2grid = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_img_world2grid.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_img_spacing = __Pyx_PyObject_to_MemoryviewSlice_ds_double(values[2]); if (unlikely(!__pyx_v_img_spacing.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3216; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out_grid2world = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_out_grid2world.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3216; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(values[4]); if (unlikely(!__pyx_v_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3217; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_inside = __Pyx_PyObject_to_MemoryviewSlice_dsds_int(values[5]); if (unlikely(!__pyx_v_inside.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3217; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_gradient_2d", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields._gradient_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_350_gradient_2d(__pyx_self, __pyx_v_img, __pyx_v_img_world2grid, __pyx_v_img_spacing, __pyx_v_out_grid2world, __pyx_v_out, __pyx_v_inside);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_350_gradient_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_img, __Pyx_memviewslice __pyx_v_img_world2grid, __Pyx_memviewslice __pyx_v_img_spacing, __Pyx_memviewslice __pyx_v_out_grid2world, __Pyx_memviewslice __pyx_v_out, __Pyx_memviewslice __pyx_v_inside) {
  int __pyx_v_nrows;
  int __pyx_v_ncols;
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_in_flag;
  double __pyx_v_tmp;
  __Pyx_memviewslice __pyx_v_x = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_dx = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_h = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_q = { 0, 0, { 0 }, { 0 }, { 0 } };
  long __pyx_v_p;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  __Pyx_memviewslice __pyx_t_18 = { 0, 0, { 0 }, { 0 }, { 0 } };
  long __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  int __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  Py_ssize_t __pyx_t_60;
  Py_ssize_t __pyx_t_61;
  Py_ssize_t __pyx_t_62;
  Py_ssize_t __pyx_t_63;
  Py_ssize_t __pyx_t_64;
  Py_ssize_t __pyx_t_65;
  Py_ssize_t __pyx_t_66;
  Py_ssize_t __pyx_t_67;
  Py_ssize_t __pyx_t_68;
  Py_ssize_t __pyx_t_69;
  Py_ssize_t __pyx_t_70;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0_gradient_2d", 0);

  /* "dipy/align/vector_fields.pyx":3252
 *     """
 *     cdef:
 *         int nrows = out.shape[0]             # <<<<<<<<<<<<<<
 *         int ncols = out.shape[1]
 *         int i, j, k, in_flag
 */
  __pyx_v_nrows = (__pyx_v_out.shape[0]);

  /* "dipy/align/vector_fields.pyx":3253
 *     cdef:
 *         int nrows = out.shape[0]
 *         int ncols = out.shape[1]             # <<<<<<<<<<<<<<
 *         int i, j, k, in_flag
 *         double tmp
 */
  __pyx_v_ncols = (__pyx_v_out.shape[1]);

  /* "dipy/align/vector_fields.pyx":3256
 *         int i, j, k, in_flag
 *         double tmp
 *         double[:] x = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_tuple__354) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_float64); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_x = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3257
 *         double tmp
 *         double[:] x = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_empty); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__355) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_float64); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_3);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_dx = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3258
 *         double[:] x = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_empty); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__356) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_float64); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_2);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_h = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3259
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_tuple__357) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_float64); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_1);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_q = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3260
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":3261
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]             # <<<<<<<<<<<<<<
 *         h[1] = 0.5 * img_spacing[1]
 *         for i in range(nrows):
 */
        __pyx_t_6 = 0;
        __pyx_t_7 = 0;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_7 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_6 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3262
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]             # <<<<<<<<<<<<<<
 *         for i in range(nrows):
 *             for j in range(ncols):
 */
        __pyx_t_8 = 1;
        __pyx_t_9 = 1;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_9 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_8 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3263
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 inside[i, j] = 1
 */
        __pyx_t_10 = __pyx_v_nrows;
        for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
          __pyx_v_i = __pyx_t_11;

          /* "dipy/align/vector_fields.pyx":3264
 *         h[1] = 0.5 * img_spacing[1]
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 inside[i, j] = 1
 *                 # Compute coordinates of index (i, j) in physical space
 */
          __pyx_t_12 = __pyx_v_ncols;
          for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
            __pyx_v_j = __pyx_t_13;

            /* "dipy/align/vector_fields.pyx":3265
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 inside[i, j] = 1             # <<<<<<<<<<<<<<
 *                 # Compute coordinates of index (i, j) in physical space
 *                 x[0] = _apply_affine_2d_x0(i, j, 1, out_grid2world)
 */
            __pyx_t_14 = __pyx_v_i;
            __pyx_t_15 = __pyx_v_j;
            *((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_14 * __pyx_v_inside.strides[0]) ) + __pyx_t_15 * __pyx_v_inside.strides[1]) )) = 1;

            /* "dipy/align/vector_fields.pyx":3267
 *                 inside[i, j] = 1
 *                 # Compute coordinates of index (i, j) in physical space
 *                 x[0] = _apply_affine_2d_x0(i, j, 1, out_grid2world)             # <<<<<<<<<<<<<<
 *                 x[1] = _apply_affine_2d_x1(i, j, 1, out_grid2world)
 *                 dx[:] = x[:]
 */
            __pyx_t_16 = 0;
            *((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_16 * __pyx_v_x.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_out_grid2world);

            /* "dipy/align/vector_fields.pyx":3268
 *                 # Compute coordinates of index (i, j) in physical space
 *                 x[0] = _apply_affine_2d_x0(i, j, 1, out_grid2world)
 *                 x[1] = _apply_affine_2d_x1(i, j, 1, out_grid2world)             # <<<<<<<<<<<<<<
 *                 dx[:] = x[:]
 *                 for p in range(2):
 */
            __pyx_t_17 = 1;
            *((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_17 * __pyx_v_x.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_out_grid2world);

            /* "dipy/align/vector_fields.pyx":3269
 *                 x[0] = _apply_affine_2d_x0(i, j, 1, out_grid2world)
 *                 x[1] = _apply_affine_2d_x1(i, j, 1, out_grid2world)
 *                 dx[:] = x[:]             # <<<<<<<<<<<<<<
 *                 for p in range(2):
 *                     # Compute coordinates of point dx on img's grid
 */
            __pyx_t_5.data = __pyx_v_x.data;
            __pyx_t_5.memview = __pyx_v_x.memview;
            __PYX_INC_MEMVIEW(&__pyx_t_5, 0);
            __pyx_t_5.shape[0] = __pyx_v_x.shape[0];
__pyx_t_5.strides[0] = __pyx_v_x.strides[0];
    __pyx_t_5.suboffsets[0] = -1;

__pyx_t_18.data = __pyx_v_dx.data;
            __pyx_t_18.memview = __pyx_v_dx.memview;
            __PYX_INC_MEMVIEW(&__pyx_t_18, 0);
            __pyx_t_18.shape[0] = __pyx_v_dx.shape[0];
__pyx_t_18.strides[0] = __pyx_v_dx.strides[0];
    __pyx_t_18.suboffsets[0] = -1;

if (unlikely(__pyx_memoryview_copy_contents(__pyx_t_5, __pyx_t_18, 1, 1, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3269; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
            __PYX_XDEC_MEMVIEW(&__pyx_t_18, 0);
            __PYX_XDEC_MEMVIEW(&__pyx_t_5, 0);

            /* "dipy/align/vector_fields.pyx":3270
 *                 x[1] = _apply_affine_2d_x1(i, j, 1, out_grid2world)
 *                 dx[:] = x[:]
 *                 for p in range(2):             # <<<<<<<<<<<<<<
 *                     # Compute coordinates of point dx on img's grid
 *                     dx[p] = x[p] - h[p]
 */
            for (__pyx_t_19 = 0; __pyx_t_19 < 2; __pyx_t_19+=1) {
              __pyx_v_p = __pyx_t_19;

              /* "dipy/align/vector_fields.pyx":3272
 *                 for p in range(2):
 *                     # Compute coordinates of point dx on img's grid
 *                     dx[p] = x[p] - h[p]             # <<<<<<<<<<<<<<
 *                     q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 */
              __pyx_t_20 = __pyx_v_p;
              __pyx_t_21 = __pyx_v_p;
              __pyx_t_22 = __pyx_v_p;
              *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_22 * __pyx_v_dx.strides[0]) )) = ((*((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_20 * __pyx_v_x.strides[0]) ))) - (*((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_21 * __pyx_v_h.strides[0]) ))));

              /* "dipy/align/vector_fields.pyx":3273
 *                     # Compute coordinates of point dx on img's grid
 *                     dx[p] = x[p] - h[p]
 *                     q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)             # <<<<<<<<<<<<<<
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                     # Interpolate img at q
 */
              __pyx_t_23 = 0;
              __pyx_t_24 = 1;
              __pyx_t_25 = 0;
              *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_25 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_23 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_24 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

              /* "dipy/align/vector_fields.pyx":3274
 *                     dx[p] = x[p] - h[p]
 *                     q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)             # <<<<<<<<<<<<<<
 *                     # Interpolate img at q
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],
 */
              __pyx_t_26 = 0;
              __pyx_t_27 = 1;
              __pyx_t_28 = 1;
              *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_28 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_26 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_27 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

              /* "dipy/align/vector_fields.pyx":3276
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                     # Interpolate img at q
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],             # <<<<<<<<<<<<<<
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:
 */
              __pyx_t_29 = 0;

              /* "dipy/align/vector_fields.pyx":3277
 *                     # Interpolate img at q
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],
 *                         q[1], &out[i, j, p])             # <<<<<<<<<<<<<<
 *                     if in_flag == 0:
 *                         out[i, j, p] = 0
 */
              __pyx_t_30 = 1;
              __pyx_t_31 = __pyx_v_i;
              __pyx_t_32 = __pyx_v_j;
              __pyx_t_33 = __pyx_v_p;

              /* "dipy/align/vector_fields.pyx":3276
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                     # Interpolate img at q
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],             # <<<<<<<<<<<<<<
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:
 */
              __pyx_v_in_flag = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__pyx_v_img, (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_29 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_30 * __pyx_v_q.strides[0]) ))), (&(*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_31 * __pyx_v_out.strides[0]) ) + __pyx_t_32 * __pyx_v_out.strides[1]) ) + __pyx_t_33 * __pyx_v_out.strides[2]) )))));

              /* "dipy/align/vector_fields.pyx":3278
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:             # <<<<<<<<<<<<<<
 *                         out[i, j, p] = 0
 *                         inside[i, j] = 0
 */
              __pyx_t_34 = ((__pyx_v_in_flag == 0) != 0);
              if (__pyx_t_34) {

                /* "dipy/align/vector_fields.pyx":3279
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:
 *                         out[i, j, p] = 0             # <<<<<<<<<<<<<<
 *                         inside[i, j] = 0
 *                         continue
 */
                __pyx_t_35 = __pyx_v_i;
                __pyx_t_36 = __pyx_v_j;
                __pyx_t_37 = __pyx_v_p;
                *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_35 * __pyx_v_out.strides[0]) ) + __pyx_t_36 * __pyx_v_out.strides[1]) ) + __pyx_t_37 * __pyx_v_out.strides[2]) )) = 0.0;

                /* "dipy/align/vector_fields.pyx":3280
 *                     if in_flag == 0:
 *                         out[i, j, p] = 0
 *                         inside[i, j] = 0             # <<<<<<<<<<<<<<
 *                         continue
 *                     tmp = out[i, j, p]
 */
                __pyx_t_38 = __pyx_v_i;
                __pyx_t_39 = __pyx_v_j;
                *((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_38 * __pyx_v_inside.strides[0]) ) + __pyx_t_39 * __pyx_v_inside.strides[1]) )) = 0;

                /* "dipy/align/vector_fields.pyx":3281
 *                         out[i, j, p] = 0
 *                         inside[i, j] = 0
 *                         continue             # <<<<<<<<<<<<<<
 *                     tmp = out[i, j, p]
 *                     # Compute coordinates of point dx on img's grid
 */
                goto __pyx_L10_continue;

                /* "dipy/align/vector_fields.pyx":3278
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:             # <<<<<<<<<<<<<<
 *                         out[i, j, p] = 0
 *                         inside[i, j] = 0
 */
              }

              /* "dipy/align/vector_fields.pyx":3282
 *                         inside[i, j] = 0
 *                         continue
 *                     tmp = out[i, j, p]             # <<<<<<<<<<<<<<
 *                     # Compute coordinates of point dx on img's grid
 *                     dx[p] = x[p] + h[p]
 */
              __pyx_t_40 = __pyx_v_i;
              __pyx_t_41 = __pyx_v_j;
              __pyx_t_42 = __pyx_v_p;
              __pyx_v_tmp = (*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_40 * __pyx_v_out.strides[0]) ) + __pyx_t_41 * __pyx_v_out.strides[1]) ) + __pyx_t_42 * __pyx_v_out.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":3284
 *                     tmp = out[i, j, p]
 *                     # Compute coordinates of point dx on img's grid
 *                     dx[p] = x[p] + h[p]             # <<<<<<<<<<<<<<
 *                     q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 */
              __pyx_t_43 = __pyx_v_p;
              __pyx_t_44 = __pyx_v_p;
              __pyx_t_45 = __pyx_v_p;
              *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_45 * __pyx_v_dx.strides[0]) )) = ((*((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_43 * __pyx_v_x.strides[0]) ))) + (*((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_44 * __pyx_v_h.strides[0]) ))));

              /* "dipy/align/vector_fields.pyx":3285
 *                     # Compute coordinates of point dx on img's grid
 *                     dx[p] = x[p] + h[p]
 *                     q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)             # <<<<<<<<<<<<<<
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                     # Interpolate img at q
 */
              __pyx_t_46 = 0;
              __pyx_t_47 = 1;
              __pyx_t_48 = 0;
              *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_48 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_46 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_47 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

              /* "dipy/align/vector_fields.pyx":3286
 *                     dx[p] = x[p] + h[p]
 *                     q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)             # <<<<<<<<<<<<<<
 *                     # Interpolate img at q
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],
 */
              __pyx_t_49 = 0;
              __pyx_t_50 = 1;
              __pyx_t_51 = 1;
              *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_51 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_49 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_50 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

              /* "dipy/align/vector_fields.pyx":3288
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                     # Interpolate img at q
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],             # <<<<<<<<<<<<<<
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:
 */
              __pyx_t_52 = 0;

              /* "dipy/align/vector_fields.pyx":3289
 *                     # Interpolate img at q
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],
 *                         q[1], &out[i, j, p])             # <<<<<<<<<<<<<<
 *                     if in_flag == 0:
 *                         out[i, j, p] = 0
 */
              __pyx_t_53 = 1;
              __pyx_t_54 = __pyx_v_i;
              __pyx_t_55 = __pyx_v_j;
              __pyx_t_56 = __pyx_v_p;

              /* "dipy/align/vector_fields.pyx":3288
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                     # Interpolate img at q
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],             # <<<<<<<<<<<<<<
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:
 */
              __pyx_v_in_flag = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__pyx_v_img, (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_52 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_53 * __pyx_v_q.strides[0]) ))), (&(*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_54 * __pyx_v_out.strides[0]) ) + __pyx_t_55 * __pyx_v_out.strides[1]) ) + __pyx_t_56 * __pyx_v_out.strides[2]) )))));

              /* "dipy/align/vector_fields.pyx":3290
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:             # <<<<<<<<<<<<<<
 *                         out[i, j, p] = 0
 *                         inside[i, j] = 0
 */
              __pyx_t_34 = ((__pyx_v_in_flag == 0) != 0);
              if (__pyx_t_34) {

                /* "dipy/align/vector_fields.pyx":3291
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:
 *                         out[i, j, p] = 0             # <<<<<<<<<<<<<<
 *                         inside[i, j] = 0
 *                         continue
 */
                __pyx_t_57 = __pyx_v_i;
                __pyx_t_58 = __pyx_v_j;
                __pyx_t_59 = __pyx_v_p;
                *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_57 * __pyx_v_out.strides[0]) ) + __pyx_t_58 * __pyx_v_out.strides[1]) ) + __pyx_t_59 * __pyx_v_out.strides[2]) )) = 0.0;

                /* "dipy/align/vector_fields.pyx":3292
 *                     if in_flag == 0:
 *                         out[i, j, p] = 0
 *                         inside[i, j] = 0             # <<<<<<<<<<<<<<
 *                         continue
 *                     out[i, j, p] = (out[i, j, p] - tmp) / img_spacing[p]
 */
                __pyx_t_60 = __pyx_v_i;
                __pyx_t_61 = __pyx_v_j;
                *((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_60 * __pyx_v_inside.strides[0]) ) + __pyx_t_61 * __pyx_v_inside.strides[1]) )) = 0;

                /* "dipy/align/vector_fields.pyx":3293
 *                         out[i, j, p] = 0
 *                         inside[i, j] = 0
 *                         continue             # <<<<<<<<<<<<<<
 *                     out[i, j, p] = (out[i, j, p] - tmp) / img_spacing[p]
 *                     dx[p] = x[p]
 */
                goto __pyx_L10_continue;

                /* "dipy/align/vector_fields.pyx":3290
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:             # <<<<<<<<<<<<<<
 *                         out[i, j, p] = 0
 *                         inside[i, j] = 0
 */
              }

              /* "dipy/align/vector_fields.pyx":3294
 *                         inside[i, j] = 0
 *                         continue
 *                     out[i, j, p] = (out[i, j, p] - tmp) / img_spacing[p]             # <<<<<<<<<<<<<<
 *                     dx[p] = x[p]
 * 
 */
              __pyx_t_62 = __pyx_v_i;
              __pyx_t_63 = __pyx_v_j;
              __pyx_t_64 = __pyx_v_p;
              __pyx_t_65 = __pyx_v_p;
              __pyx_t_66 = __pyx_v_i;
              __pyx_t_67 = __pyx_v_j;
              __pyx_t_68 = __pyx_v_p;
              *((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_66 * __pyx_v_out.strides[0]) ) + __pyx_t_67 * __pyx_v_out.strides[1]) ) + __pyx_t_68 * __pyx_v_out.strides[2]) )) = (((*((float *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_62 * __pyx_v_out.strides[0]) ) + __pyx_t_63 * __pyx_v_out.strides[1]) ) + __pyx_t_64 * __pyx_v_out.strides[2]) ))) - __pyx_v_tmp) / (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_65 * __pyx_v_img_spacing.strides[0]) ))));

              /* "dipy/align/vector_fields.pyx":3295
 *                         continue
 *                     out[i, j, p] = (out[i, j, p] - tmp) / img_spacing[p]
 *                     dx[p] = x[p]             # <<<<<<<<<<<<<<
 * 
 * 
 */
              __pyx_t_69 = __pyx_v_p;
              __pyx_t_70 = __pyx_v_p;
              *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_70 * __pyx_v_dx.strides[0]) )) = (*((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_69 * __pyx_v_x.strides[0]) )));
              __pyx_L10_continue:;
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":3260
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L4_error: {
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L1_error;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":3215
 * 
 * 
 * def _gradient_2d(floating[:, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                  double[:] img_spacing, double[:, :] out_grid2world,
 *                  floating[:, :, :] out, int[:, :] inside):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_18, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields._gradient_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_x, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_dx, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_h, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_q, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img_world2grid, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img_spacing, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_grid2world, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_353_gradient_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_353_gradient_2d = {"__pyx_fuse_1_gradient_2d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_353_gradient_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_74_gradient_2d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_353_gradient_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_img = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_img_world2grid = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_img_spacing = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out_grid2world = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_gradient_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_img,&__pyx_n_s_img_world2grid,&__pyx_n_s_img_spacing,&__pyx_n_s_out_grid2world,&__pyx_n_s_out,&__pyx_n_s_inside,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_world2grid)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_2d", 1, 6, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_spacing)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_2d", 1, 6, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_grid2world)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_2d", 1, 6, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_2d", 1, 6, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_inside)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_gradient_2d", 1, 6, 6, 5); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_gradient_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_img = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[0]); if (unlikely(!__pyx_v_img.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_img_world2grid = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_img_world2grid.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_img_spacing = __Pyx_PyObject_to_MemoryviewSlice_ds_double(values[2]); if (unlikely(!__pyx_v_img_spacing.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3216; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out_grid2world = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_out_grid2world.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3216; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(values[4]); if (unlikely(!__pyx_v_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3217; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_inside = __Pyx_PyObject_to_MemoryviewSlice_dsds_int(values[5]); if (unlikely(!__pyx_v_inside.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3217; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_gradient_2d", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields._gradient_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_352_gradient_2d(__pyx_self, __pyx_v_img, __pyx_v_img_world2grid, __pyx_v_img_spacing, __pyx_v_out_grid2world, __pyx_v_out, __pyx_v_inside);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_352_gradient_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_img, __Pyx_memviewslice __pyx_v_img_world2grid, __Pyx_memviewslice __pyx_v_img_spacing, __Pyx_memviewslice __pyx_v_out_grid2world, __Pyx_memviewslice __pyx_v_out, __Pyx_memviewslice __pyx_v_inside) {
  int __pyx_v_nrows;
  int __pyx_v_ncols;
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_v_in_flag;
  double __pyx_v_tmp;
  __Pyx_memviewslice __pyx_v_x = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_dx = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_h = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_q = { 0, 0, { 0 }, { 0 }, { 0 } };
  long __pyx_v_p;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  __Pyx_memviewslice __pyx_t_18 = { 0, 0, { 0 }, { 0 }, { 0 } };
  long __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  Py_ssize_t __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  int __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  Py_ssize_t __pyx_t_60;
  Py_ssize_t __pyx_t_61;
  Py_ssize_t __pyx_t_62;
  Py_ssize_t __pyx_t_63;
  Py_ssize_t __pyx_t_64;
  Py_ssize_t __pyx_t_65;
  Py_ssize_t __pyx_t_66;
  Py_ssize_t __pyx_t_67;
  Py_ssize_t __pyx_t_68;
  Py_ssize_t __pyx_t_69;
  Py_ssize_t __pyx_t_70;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1_gradient_2d", 0);

  /* "dipy/align/vector_fields.pyx":3252
 *     """
 *     cdef:
 *         int nrows = out.shape[0]             # <<<<<<<<<<<<<<
 *         int ncols = out.shape[1]
 *         int i, j, k, in_flag
 */
  __pyx_v_nrows = (__pyx_v_out.shape[0]);

  /* "dipy/align/vector_fields.pyx":3253
 *     cdef:
 *         int nrows = out.shape[0]
 *         int ncols = out.shape[1]             # <<<<<<<<<<<<<<
 *         int i, j, k, in_flag
 *         double tmp
 */
  __pyx_v_ncols = (__pyx_v_out.shape[1]);

  /* "dipy/align/vector_fields.pyx":3256
 *         int i, j, k, in_flag
 *         double tmp
 *         double[:] x = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_tuple__358) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_float64); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_x = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3257
 *         double tmp
 *         double[:] x = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_empty); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__359) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_float64); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_3);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_dx = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3258
 *         double[:] x = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_empty); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__360) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_float64); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_2);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_h = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3259
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_shape, __pyx_tuple__361) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_float64); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_1);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_q = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3260
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":3261
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]             # <<<<<<<<<<<<<<
 *         h[1] = 0.5 * img_spacing[1]
 *         for i in range(nrows):
 */
        __pyx_t_6 = 0;
        __pyx_t_7 = 0;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_7 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_6 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3262
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]             # <<<<<<<<<<<<<<
 *         for i in range(nrows):
 *             for j in range(ncols):
 */
        __pyx_t_8 = 1;
        __pyx_t_9 = 1;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_9 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_8 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3263
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 *         for i in range(nrows):             # <<<<<<<<<<<<<<
 *             for j in range(ncols):
 *                 inside[i, j] = 1
 */
        __pyx_t_10 = __pyx_v_nrows;
        for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
          __pyx_v_i = __pyx_t_11;

          /* "dipy/align/vector_fields.pyx":3264
 *         h[1] = 0.5 * img_spacing[1]
 *         for i in range(nrows):
 *             for j in range(ncols):             # <<<<<<<<<<<<<<
 *                 inside[i, j] = 1
 *                 # Compute coordinates of index (i, j) in physical space
 */
          __pyx_t_12 = __pyx_v_ncols;
          for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
            __pyx_v_j = __pyx_t_13;

            /* "dipy/align/vector_fields.pyx":3265
 *         for i in range(nrows):
 *             for j in range(ncols):
 *                 inside[i, j] = 1             # <<<<<<<<<<<<<<
 *                 # Compute coordinates of index (i, j) in physical space
 *                 x[0] = _apply_affine_2d_x0(i, j, 1, out_grid2world)
 */
            __pyx_t_14 = __pyx_v_i;
            __pyx_t_15 = __pyx_v_j;
            *((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_14 * __pyx_v_inside.strides[0]) ) + __pyx_t_15 * __pyx_v_inside.strides[1]) )) = 1;

            /* "dipy/align/vector_fields.pyx":3267
 *                 inside[i, j] = 1
 *                 # Compute coordinates of index (i, j) in physical space
 *                 x[0] = _apply_affine_2d_x0(i, j, 1, out_grid2world)             # <<<<<<<<<<<<<<
 *                 x[1] = _apply_affine_2d_x1(i, j, 1, out_grid2world)
 *                 dx[:] = x[:]
 */
            __pyx_t_16 = 0;
            *((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_16 * __pyx_v_x.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_out_grid2world);

            /* "dipy/align/vector_fields.pyx":3268
 *                 # Compute coordinates of index (i, j) in physical space
 *                 x[0] = _apply_affine_2d_x0(i, j, 1, out_grid2world)
 *                 x[1] = _apply_affine_2d_x1(i, j, 1, out_grid2world)             # <<<<<<<<<<<<<<
 *                 dx[:] = x[:]
 *                 for p in range(2):
 */
            __pyx_t_17 = 1;
            *((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_17 * __pyx_v_x.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(__pyx_v_i, __pyx_v_j, 1.0, __pyx_v_out_grid2world);

            /* "dipy/align/vector_fields.pyx":3269
 *                 x[0] = _apply_affine_2d_x0(i, j, 1, out_grid2world)
 *                 x[1] = _apply_affine_2d_x1(i, j, 1, out_grid2world)
 *                 dx[:] = x[:]             # <<<<<<<<<<<<<<
 *                 for p in range(2):
 *                     # Compute coordinates of point dx on img's grid
 */
            __pyx_t_5.data = __pyx_v_x.data;
            __pyx_t_5.memview = __pyx_v_x.memview;
            __PYX_INC_MEMVIEW(&__pyx_t_5, 0);
            __pyx_t_5.shape[0] = __pyx_v_x.shape[0];
__pyx_t_5.strides[0] = __pyx_v_x.strides[0];
    __pyx_t_5.suboffsets[0] = -1;

__pyx_t_18.data = __pyx_v_dx.data;
            __pyx_t_18.memview = __pyx_v_dx.memview;
            __PYX_INC_MEMVIEW(&__pyx_t_18, 0);
            __pyx_t_18.shape[0] = __pyx_v_dx.shape[0];
__pyx_t_18.strides[0] = __pyx_v_dx.strides[0];
    __pyx_t_18.suboffsets[0] = -1;

if (unlikely(__pyx_memoryview_copy_contents(__pyx_t_5, __pyx_t_18, 1, 1, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3269; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
            __PYX_XDEC_MEMVIEW(&__pyx_t_18, 0);
            __PYX_XDEC_MEMVIEW(&__pyx_t_5, 0);

            /* "dipy/align/vector_fields.pyx":3270
 *                 x[1] = _apply_affine_2d_x1(i, j, 1, out_grid2world)
 *                 dx[:] = x[:]
 *                 for p in range(2):             # <<<<<<<<<<<<<<
 *                     # Compute coordinates of point dx on img's grid
 *                     dx[p] = x[p] - h[p]
 */
            for (__pyx_t_19 = 0; __pyx_t_19 < 2; __pyx_t_19+=1) {
              __pyx_v_p = __pyx_t_19;

              /* "dipy/align/vector_fields.pyx":3272
 *                 for p in range(2):
 *                     # Compute coordinates of point dx on img's grid
 *                     dx[p] = x[p] - h[p]             # <<<<<<<<<<<<<<
 *                     q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 */
              __pyx_t_20 = __pyx_v_p;
              __pyx_t_21 = __pyx_v_p;
              __pyx_t_22 = __pyx_v_p;
              *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_22 * __pyx_v_dx.strides[0]) )) = ((*((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_20 * __pyx_v_x.strides[0]) ))) - (*((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_21 * __pyx_v_h.strides[0]) ))));

              /* "dipy/align/vector_fields.pyx":3273
 *                     # Compute coordinates of point dx on img's grid
 *                     dx[p] = x[p] - h[p]
 *                     q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)             # <<<<<<<<<<<<<<
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                     # Interpolate img at q
 */
              __pyx_t_23 = 0;
              __pyx_t_24 = 1;
              __pyx_t_25 = 0;
              *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_25 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_23 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_24 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

              /* "dipy/align/vector_fields.pyx":3274
 *                     dx[p] = x[p] - h[p]
 *                     q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)             # <<<<<<<<<<<<<<
 *                     # Interpolate img at q
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],
 */
              __pyx_t_26 = 0;
              __pyx_t_27 = 1;
              __pyx_t_28 = 1;
              *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_28 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_26 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_27 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

              /* "dipy/align/vector_fields.pyx":3276
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                     # Interpolate img at q
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],             # <<<<<<<<<<<<<<
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:
 */
              __pyx_t_29 = 0;

              /* "dipy/align/vector_fields.pyx":3277
 *                     # Interpolate img at q
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],
 *                         q[1], &out[i, j, p])             # <<<<<<<<<<<<<<
 *                     if in_flag == 0:
 *                         out[i, j, p] = 0
 */
              __pyx_t_30 = 1;
              __pyx_t_31 = __pyx_v_i;
              __pyx_t_32 = __pyx_v_j;
              __pyx_t_33 = __pyx_v_p;

              /* "dipy/align/vector_fields.pyx":3276
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                     # Interpolate img at q
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],             # <<<<<<<<<<<<<<
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:
 */
              __pyx_v_in_flag = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__pyx_v_img, (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_29 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_30 * __pyx_v_q.strides[0]) ))), (&(*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_31 * __pyx_v_out.strides[0]) ) + __pyx_t_32 * __pyx_v_out.strides[1]) ) + __pyx_t_33 * __pyx_v_out.strides[2]) )))));

              /* "dipy/align/vector_fields.pyx":3278
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:             # <<<<<<<<<<<<<<
 *                         out[i, j, p] = 0
 *                         inside[i, j] = 0
 */
              __pyx_t_34 = ((__pyx_v_in_flag == 0) != 0);
              if (__pyx_t_34) {

                /* "dipy/align/vector_fields.pyx":3279
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:
 *                         out[i, j, p] = 0             # <<<<<<<<<<<<<<
 *                         inside[i, j] = 0
 *                         continue
 */
                __pyx_t_35 = __pyx_v_i;
                __pyx_t_36 = __pyx_v_j;
                __pyx_t_37 = __pyx_v_p;
                *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_35 * __pyx_v_out.strides[0]) ) + __pyx_t_36 * __pyx_v_out.strides[1]) ) + __pyx_t_37 * __pyx_v_out.strides[2]) )) = 0.0;

                /* "dipy/align/vector_fields.pyx":3280
 *                     if in_flag == 0:
 *                         out[i, j, p] = 0
 *                         inside[i, j] = 0             # <<<<<<<<<<<<<<
 *                         continue
 *                     tmp = out[i, j, p]
 */
                __pyx_t_38 = __pyx_v_i;
                __pyx_t_39 = __pyx_v_j;
                *((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_38 * __pyx_v_inside.strides[0]) ) + __pyx_t_39 * __pyx_v_inside.strides[1]) )) = 0;

                /* "dipy/align/vector_fields.pyx":3281
 *                         out[i, j, p] = 0
 *                         inside[i, j] = 0
 *                         continue             # <<<<<<<<<<<<<<
 *                     tmp = out[i, j, p]
 *                     # Compute coordinates of point dx on img's grid
 */
                goto __pyx_L10_continue;

                /* "dipy/align/vector_fields.pyx":3278
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:             # <<<<<<<<<<<<<<
 *                         out[i, j, p] = 0
 *                         inside[i, j] = 0
 */
              }

              /* "dipy/align/vector_fields.pyx":3282
 *                         inside[i, j] = 0
 *                         continue
 *                     tmp = out[i, j, p]             # <<<<<<<<<<<<<<
 *                     # Compute coordinates of point dx on img's grid
 *                     dx[p] = x[p] + h[p]
 */
              __pyx_t_40 = __pyx_v_i;
              __pyx_t_41 = __pyx_v_j;
              __pyx_t_42 = __pyx_v_p;
              __pyx_v_tmp = (*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_40 * __pyx_v_out.strides[0]) ) + __pyx_t_41 * __pyx_v_out.strides[1]) ) + __pyx_t_42 * __pyx_v_out.strides[2]) )));

              /* "dipy/align/vector_fields.pyx":3284
 *                     tmp = out[i, j, p]
 *                     # Compute coordinates of point dx on img's grid
 *                     dx[p] = x[p] + h[p]             # <<<<<<<<<<<<<<
 *                     q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 */
              __pyx_t_43 = __pyx_v_p;
              __pyx_t_44 = __pyx_v_p;
              __pyx_t_45 = __pyx_v_p;
              *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_45 * __pyx_v_dx.strides[0]) )) = ((*((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_43 * __pyx_v_x.strides[0]) ))) + (*((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_44 * __pyx_v_h.strides[0]) ))));

              /* "dipy/align/vector_fields.pyx":3285
 *                     # Compute coordinates of point dx on img's grid
 *                     dx[p] = x[p] + h[p]
 *                     q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)             # <<<<<<<<<<<<<<
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                     # Interpolate img at q
 */
              __pyx_t_46 = 0;
              __pyx_t_47 = 1;
              __pyx_t_48 = 0;
              *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_48 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_46 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_47 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

              /* "dipy/align/vector_fields.pyx":3286
 *                     dx[p] = x[p] + h[p]
 *                     q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)             # <<<<<<<<<<<<<<
 *                     # Interpolate img at q
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],
 */
              __pyx_t_49 = 0;
              __pyx_t_50 = 1;
              __pyx_t_51 = 1;
              *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_51 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_49 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_50 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

              /* "dipy/align/vector_fields.pyx":3288
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                     # Interpolate img at q
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],             # <<<<<<<<<<<<<<
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:
 */
              __pyx_t_52 = 0;

              /* "dipy/align/vector_fields.pyx":3289
 *                     # Interpolate img at q
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],
 *                         q[1], &out[i, j, p])             # <<<<<<<<<<<<<<
 *                     if in_flag == 0:
 *                         out[i, j, p] = 0
 */
              __pyx_t_53 = 1;
              __pyx_t_54 = __pyx_v_i;
              __pyx_t_55 = __pyx_v_j;
              __pyx_t_56 = __pyx_v_p;

              /* "dipy/align/vector_fields.pyx":3288
 *                     q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                     # Interpolate img at q
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],             # <<<<<<<<<<<<<<
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:
 */
              __pyx_v_in_flag = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__pyx_v_img, (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_52 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_53 * __pyx_v_q.strides[0]) ))), (&(*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_54 * __pyx_v_out.strides[0]) ) + __pyx_t_55 * __pyx_v_out.strides[1]) ) + __pyx_t_56 * __pyx_v_out.strides[2]) )))));

              /* "dipy/align/vector_fields.pyx":3290
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:             # <<<<<<<<<<<<<<
 *                         out[i, j, p] = 0
 *                         inside[i, j] = 0
 */
              __pyx_t_34 = ((__pyx_v_in_flag == 0) != 0);
              if (__pyx_t_34) {

                /* "dipy/align/vector_fields.pyx":3291
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:
 *                         out[i, j, p] = 0             # <<<<<<<<<<<<<<
 *                         inside[i, j] = 0
 *                         continue
 */
                __pyx_t_57 = __pyx_v_i;
                __pyx_t_58 = __pyx_v_j;
                __pyx_t_59 = __pyx_v_p;
                *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_57 * __pyx_v_out.strides[0]) ) + __pyx_t_58 * __pyx_v_out.strides[1]) ) + __pyx_t_59 * __pyx_v_out.strides[2]) )) = 0.0;

                /* "dipy/align/vector_fields.pyx":3292
 *                     if in_flag == 0:
 *                         out[i, j, p] = 0
 *                         inside[i, j] = 0             # <<<<<<<<<<<<<<
 *                         continue
 *                     out[i, j, p] = (out[i, j, p] - tmp) / img_spacing[p]
 */
                __pyx_t_60 = __pyx_v_i;
                __pyx_t_61 = __pyx_v_j;
                *((int *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_60 * __pyx_v_inside.strides[0]) ) + __pyx_t_61 * __pyx_v_inside.strides[1]) )) = 0;

                /* "dipy/align/vector_fields.pyx":3293
 *                         out[i, j, p] = 0
 *                         inside[i, j] = 0
 *                         continue             # <<<<<<<<<<<<<<
 *                     out[i, j, p] = (out[i, j, p] - tmp) / img_spacing[p]
 *                     dx[p] = x[p]
 */
                goto __pyx_L10_continue;

                /* "dipy/align/vector_fields.pyx":3290
 *                     in_flag = _interpolate_scalar_2d[floating](img, q[0],
 *                         q[1], &out[i, j, p])
 *                     if in_flag == 0:             # <<<<<<<<<<<<<<
 *                         out[i, j, p] = 0
 *                         inside[i, j] = 0
 */
              }

              /* "dipy/align/vector_fields.pyx":3294
 *                         inside[i, j] = 0
 *                         continue
 *                     out[i, j, p] = (out[i, j, p] - tmp) / img_spacing[p]             # <<<<<<<<<<<<<<
 *                     dx[p] = x[p]
 * 
 */
              __pyx_t_62 = __pyx_v_i;
              __pyx_t_63 = __pyx_v_j;
              __pyx_t_64 = __pyx_v_p;
              __pyx_t_65 = __pyx_v_p;
              __pyx_t_66 = __pyx_v_i;
              __pyx_t_67 = __pyx_v_j;
              __pyx_t_68 = __pyx_v_p;
              *((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_66 * __pyx_v_out.strides[0]) ) + __pyx_t_67 * __pyx_v_out.strides[1]) ) + __pyx_t_68 * __pyx_v_out.strides[2]) )) = (((*((double *) ( /* dim=2 */ (( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_62 * __pyx_v_out.strides[0]) ) + __pyx_t_63 * __pyx_v_out.strides[1]) ) + __pyx_t_64 * __pyx_v_out.strides[2]) ))) - __pyx_v_tmp) / (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_65 * __pyx_v_img_spacing.strides[0]) ))));

              /* "dipy/align/vector_fields.pyx":3295
 *                         continue
 *                     out[i, j, p] = (out[i, j, p] - tmp) / img_spacing[p]
 *                     dx[p] = x[p]             # <<<<<<<<<<<<<<
 * 
 * 
 */
              __pyx_t_69 = __pyx_v_p;
              __pyx_t_70 = __pyx_v_p;
              *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_70 * __pyx_v_dx.strides[0]) )) = (*((double *) ( /* dim=0 */ (__pyx_v_x.data + __pyx_t_69 * __pyx_v_x.strides[0]) )));
              __pyx_L10_continue:;
            }
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":3260
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L4_error: {
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L1_error;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":3215
 * 
 * 
 * def _gradient_2d(floating[:, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                  double[:] img_spacing, double[:, :] out_grid2world,
 *                  floating[:, :, :] out, int[:, :] inside):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_18, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields._gradient_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_x, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_dx, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_h, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_q, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img_world2grid, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img_spacing, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out_grid2world, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":3298
 * 
 * 
 * def _sparse_gradient_2d(floating[:, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                         double[:] img_spacing,
 *                         double[:, :] sample_points,
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_77_sparse_gradient_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_76_sparse_gradient_2d[] = " Gradient of a 2D image evaluated at a set of points in physical space\n\n    For each row (x_i, y_i) in sample_points, the image is interpolated at\n\n    P1=(x_i + h, y_i), Q1=(x_i - h, y_i)\n    P2=(x_i, y_i + h), Q2=(x_i, y_i - h)\n\n    (by mapping Pi and Qi to the grid using img_world2grid: the inverse of the\n    grid-to-space transform of img). The displacement parameter h is of\n    magnitude 0.5 (in physical space units), therefore the approximated partial\n    derivatives are given by the difference between the image interpolated at\n    Pi and Qi.\n\n    Parameters\n    ----------\n    img : array, shape (R, C)\n        the input volume whose gradient will be computed\n    img_world2grid : array, shape (3, 3)\n        the space-to-grid transform matrix associated to img\n    img_spacing : array, shape (2,)\n        the spacing between pixels (pixel size along each axis) of the input\n    sample_points: array, shape (n, 2)\n        list of points where the derivative will be evaluated\n        (one point per row)\n    out : array, shape (n, 2)\n        the buffer in which to store the image gradient\n    inside : array, shape (n,)\n        the buffer in which to store the flags indicating whether the sample\n        point lies inside (=1) or outside (=0) the image grid\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_77_sparse_gradient_2d = {"_sparse_gradient_2d", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_77_sparse_gradient_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_76_sparse_gradient_2d};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_77_sparse_gradient_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_signatures = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  CYTHON_UNUSED PyObject *__pyx_v_defaults = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_fused_cpdef (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_signatures,&__pyx_n_s_args,&__pyx_n_s_kwargs,&__pyx_n_s_defaults,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_signatures)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_args)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_kwargs)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_defaults)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_fused_cpdef") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_signatures = values[0];
    __pyx_v_args = values[1];
    __pyx_v_kwargs = values[2];
    __pyx_v_defaults = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_fused_cpdef", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_76_sparse_gradient_2d(__pyx_self, __pyx_v_signatures, __pyx_v_args, __pyx_v_kwargs, __pyx_v_defaults);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_76_sparse_gradient_2d(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_signatures, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs, CYTHON_UNUSED PyObject *__pyx_v_defaults) {
  PyObject *__pyx_v_dest_sig = NULL;
  PyTypeObject *__pyx_v_ndarray = 0;
  PyObject *__pyx_v_numpy = NULL;
  __Pyx_memviewslice __pyx_v_memslice;
  Py_ssize_t __pyx_v_itemsize;
  CYTHON_UNUSED int __pyx_v_dtype_signed;
  char __pyx_v_kind;
  PyObject *__pyx_v_arg = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_arg_base = NULL;
  PyObject *__pyx_v_candidates = NULL;
  PyObject *__pyx_v_sig = NULL;
  int __pyx_v_match_found;
  PyObject *__pyx_v_src_type = NULL;
  PyObject *__pyx_v_dst_type = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  long __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *(*__pyx_t_19)(PyObject *);
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_sparse_gradient_2d", 0);
  __Pyx_INCREF(__pyx_v_kwargs);
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyList_SET_ITEM(__pyx_t_1, 0, Py_None);
  __pyx_v_dest_sig = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_v_kwargs == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {
    __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_kwargs, __pyx_t_1);
    __pyx_t_1 = 0;
  }
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {
      __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_numpy = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_numpy, __pyx_n_s_ndarray); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyType_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "type", Py_TYPE(__pyx_t_1)->tp_name), 0))) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
      __pyx_v_ndarray = ((PyTypeObject*)__pyx_t_1);
      __pyx_t_1 = 0;
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = PyErr_ExceptionMatches(__pyx_builtin_ImportError) || PyErr_ExceptionMatches(__pyx_builtin_AttributeError) || PyErr_ExceptionMatches(__pyx_builtin_TypeError);
    if (__pyx_t_7) {
      __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(Py_None);
      __Pyx_XDECREF_SET(__pyx_v_ndarray, ((PyTypeObject*)Py_None));
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L5_exception_handled;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }
  __pyx_v_itemsize = -1L;
  if (unlikely(__pyx_v_args == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = ((0 < __pyx_t_10) != 0);
  if (__pyx_t_3) {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = PyTuple_GET_ITEM(((PyObject*)__pyx_v_args), 0);
    __Pyx_INCREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  if (unlikely(__pyx_v_kwargs == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_3 = (__Pyx_PyDict_ContainsTF(__pyx_n_s_img, ((PyObject*)__pyx_v_kwargs), Py_EQ)); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {
    if (unlikely(__pyx_v_kwargs == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_9 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_kwargs), __pyx_n_s_img); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_v_arg = __pyx_t_9;
    __pyx_t_9 = 0;
    goto __pyx_L14;
  }
  /*else*/ {
    if (unlikely(__pyx_v_args == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_10 = PyTuple_GET_SIZE(((PyObject*)__pyx_v_args)); if (unlikely(__pyx_t_10 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_9 = PyInt_FromSsize_t(__pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_8 = __Pyx_PyString_Format(__pyx_kp_s_Expected_at_least_d_arguments, __pyx_t_9); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_8);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8);
    __pyx_t_8 = 0;
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L14:;
  while (1) {
    __pyx_t_2 = (__pyx_v_ndarray != ((PyTypeObject*)Py_None));
    __pyx_t_3 = (__pyx_t_2 != 0);
    if (__pyx_t_3) {
      __pyx_t_3 = __Pyx_TypeCheck(__pyx_v_arg, __pyx_v_ndarray); 
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_dtype = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L18;
      }
      __pyx_t_2 = (__pyx_memoryview_check(__pyx_v_arg) != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_base); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_v_arg_base = __pyx_t_8;
        __pyx_t_8 = 0;
        __pyx_t_2 = __Pyx_TypeCheck(__pyx_v_arg_base, __pyx_v_ndarray); 
        __pyx_t_3 = (__pyx_t_2 != 0);
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg_base, __pyx_n_s_dtype); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_v_dtype = __pyx_t_8;
          __pyx_t_8 = 0;
          goto __pyx_L19;
        }
        /*else*/ {
          __Pyx_INCREF(Py_None);
          __pyx_v_dtype = Py_None;
        }
        __pyx_L19:;
        goto __pyx_L18;
      }
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __pyx_v_dtype = Py_None;
      }
      __pyx_L18:;
      __pyx_v_itemsize = -1L;
      __pyx_t_3 = (__pyx_v_dtype != Py_None);
      __pyx_t_2 = (__pyx_t_3 != 0);
      if (__pyx_t_2) {
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_itemsize); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_itemsize = __pyx_t_10;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_dtype, __pyx_n_s_kind); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_11 = __Pyx_PyObject_Ord(__pyx_t_8); if (unlikely(__pyx_t_11 == (long)(Py_UCS4)-1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_v_kind = __pyx_t_11;
        __pyx_v_dtype_signed = (__pyx_v_kind == 'i');
        switch (__pyx_v_kind) {
          case 'i':
          case 'u':
          break;
          case 'f':
          __pyx_t_3 = (((sizeof(float)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L22_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L22_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          __pyx_t_3 = (((sizeof(double)) == __pyx_v_itemsize) != 0);
          if (__pyx_t_3) {
          } else {
            __pyx_t_2 = __pyx_t_3;
            goto __pyx_L25_bool_binop_done;
          }
          __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_arg, __pyx_n_s_ndim); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_8); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_3 = ((((Py_ssize_t)__pyx_t_10) == 2) != 0);
          __pyx_t_2 = __pyx_t_3;
          __pyx_L25_bool_binop_done:;
          if (__pyx_t_2) {
            if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
            goto __pyx_L16_break;
          }
          break;
          case 'c':
          break;
          case 'O':
          break;
          default: break;
        }
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L28_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(float))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L28_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_float, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    __pyx_t_3 = ((__pyx_v_itemsize == -1L) != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L32_bool_binop_done;
    }
    __pyx_t_3 = ((__pyx_v_itemsize == (sizeof(double))) != 0);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L32_bool_binop_done:;
    if (__pyx_t_2) {
      __pyx_v_memslice = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(__pyx_v_arg);
      __pyx_t_2 = (__pyx_v_memslice.memview != 0);
      if (__pyx_t_2) {
        __PYX_XDEC_MEMVIEW((&__pyx_v_memslice), 1);
        if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, __pyx_n_s_double, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        goto __pyx_L16_break;
      }
      /*else*/ {
        PyErr_Clear();
      }
    }
    if (unlikely(__Pyx_SetItemInt(__pyx_v_dest_sig, 0, Py_None, long, 1, __Pyx_PyInt_From_long, 1, 0, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L16_break;
  }
  __pyx_L16_break:;
  __pyx_t_8 = PyList_New(0); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_v_candidates = ((PyObject*)__pyx_t_8);
  __pyx_t_8 = 0;
  __pyx_t_10 = 0;
  if (unlikely(__pyx_v_signatures == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_9 = __Pyx_dict_iterator(((PyObject*)__pyx_v_signatures), 1, ((PyObject *)NULL), (&__pyx_t_12), (&__pyx_t_7)); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_8);
  __pyx_t_8 = __pyx_t_9;
  __pyx_t_9 = 0;
  while (1) {
    __pyx_t_13 = __Pyx_dict_iter_next(__pyx_t_8, __pyx_t_12, &__pyx_t_10, &__pyx_t_9, NULL, NULL, __pyx_t_7);
    if (unlikely(__pyx_t_13 == 0)) break;
    if (unlikely(__pyx_t_13 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_XDECREF_SET(__pyx_v_sig, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_match_found = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_sig, __pyx_n_s_strip); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__362, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_split); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_tuple__363, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_dest_sig);
    __Pyx_GIVEREF(__pyx_v_dest_sig);
    PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_dest_sig);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
      __pyx_t_9 = __pyx_t_1; __Pyx_INCREF(__pyx_t_9); __pyx_t_14 = 0;
      __pyx_t_15 = NULL;
    } else {
      __pyx_t_14 = -1; __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext; if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (likely(!__pyx_t_15)) {
        if (likely(PyList_CheckExact(__pyx_t_9))) {
          if (__pyx_t_14 >= PyList_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyList_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        } else {
          if (__pyx_t_14 >= PyTuple_GET_SIZE(__pyx_t_9)) break;
          #if CYTHON_COMPILING_IN_CPYTHON
          __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_9, __pyx_t_14); __Pyx_INCREF(__pyx_t_1); __pyx_t_14++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          #else
          __pyx_t_1 = PySequence_ITEM(__pyx_t_9, __pyx_t_14); __pyx_t_14++; if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          __Pyx_GOTREF(__pyx_t_1);
          #endif
        }
      } else {
        __pyx_t_1 = __pyx_t_15(__pyx_t_9);
        if (unlikely(!__pyx_t_1)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_1);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        #if CYTHON_COMPILING_IN_CPYTHON
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        #if CYTHON_COMPILING_IN_CPYTHON
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_16 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_16 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_17 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx_t_17);
        #else
        __pyx_t_16 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_17);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_18 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_19 = Py_TYPE(__pyx_t_18)->tp_iternext;
        index = 0; __pyx_t_16 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_16)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_16);
        index = 1; __pyx_t_17 = __pyx_t_19(__pyx_t_18); if (unlikely(!__pyx_t_17)) goto __pyx_L39_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_17);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_19(__pyx_t_18), 2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_19 = NULL;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        goto __pyx_L40_unpacking_done;
        __pyx_L39_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
        __pyx_t_19 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_L40_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_src_type, __pyx_t_16);
      __pyx_t_16 = 0;
      __Pyx_XDECREF_SET(__pyx_v_dst_type, __pyx_t_17);
      __pyx_t_17 = 0;
      __pyx_t_2 = (__pyx_v_dst_type != Py_None);
      __pyx_t_3 = (__pyx_t_2 != 0);
      if (__pyx_t_3) {
        __pyx_t_1 = PyObject_RichCompare(__pyx_v_src_type, __pyx_v_dst_type, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_3 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_3) {
          __pyx_v_match_found = 1;
          goto __pyx_L42;
        }
        /*else*/ {
          __pyx_v_match_found = 0;
          goto __pyx_L38_break;
        }
        __pyx_L42:;
      }
    }
    __pyx_L38_break:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_3 = (__pyx_v_match_found != 0);
    if (__pyx_t_3) {
      __pyx_t_20 = __Pyx_PyList_Append(__pyx_v_candidates, __pyx_v_sig); if (unlikely(__pyx_t_20 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_3 = (__pyx_v_candidates != Py_None) && (PyList_GET_SIZE(__pyx_v_candidates) != 0);
  __pyx_t_2 = ((!__pyx_t_3) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__364, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_12 = PyList_GET_SIZE(__pyx_v_candidates); if (unlikely(__pyx_t_12 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = ((__pyx_t_12 > 1) != 0);
  if (__pyx_t_2) {
    __pyx_t_8 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__365, NULL); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_Raise(__pyx_t_8, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_signatures == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_8 = __Pyx_PyDict_GetItem(((PyObject*)__pyx_v_signatures), PyList_GET_ITEM(__pyx_v_candidates, 0)); if (unlikely(__pyx_t_8 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("dipy.align.vector_fields.__pyx_fused_cpdef", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dest_sig);
  __Pyx_XDECREF(__pyx_v_ndarray);
  __Pyx_XDECREF(__pyx_v_numpy);
  __Pyx_XDECREF(__pyx_v_arg);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_arg_base);
  __Pyx_XDECREF(__pyx_v_candidates);
  __Pyx_XDECREF(__pyx_v_sig);
  __Pyx_XDECREF(__pyx_v_src_type);
  __Pyx_XDECREF(__pyx_v_dst_type);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_357_sparse_gradient_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_357_sparse_gradient_2d = {"__pyx_fuse_0_sparse_gradient_2d", (PyCFunction)__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_357_sparse_gradient_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_76_sparse_gradient_2d};
static PyObject *__pyx_fuse_0__pyx_pw_4dipy_5align_13vector_fields_357_sparse_gradient_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_img = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_img_world2grid = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_img_spacing = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_sample_points = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_sparse_gradient_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_img,&__pyx_n_s_img_world2grid,&__pyx_n_s_img_spacing,&__pyx_n_s_sample_points,&__pyx_n_s_out,&__pyx_n_s_inside,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_world2grid)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_2d", 1, 6, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_spacing)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_2d", 1, 6, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sample_points)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_2d", 1, 6, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_2d", 1, 6, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_inside)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_2d", 1, 6, 6, 5); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_sparse_gradient_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_img = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(values[0]); if (unlikely(!__pyx_v_img.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_img_world2grid = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_img_world2grid.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_img_spacing = __Pyx_PyObject_to_MemoryviewSlice_ds_double(values[2]); if (unlikely(!__pyx_v_img_spacing.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3299; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_sample_points = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_sample_points.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3300; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_float(values[4]); if (unlikely(!__pyx_v_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3301; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_inside = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_inside.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3301; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_sparse_gradient_2d", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields._sparse_gradient_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_356_sparse_gradient_2d(__pyx_self, __pyx_v_img, __pyx_v_img_world2grid, __pyx_v_img_spacing, __pyx_v_sample_points, __pyx_v_out, __pyx_v_inside);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_356_sparse_gradient_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_img, __Pyx_memviewslice __pyx_v_img_world2grid, __Pyx_memviewslice __pyx_v_img_spacing, __Pyx_memviewslice __pyx_v_sample_points, __Pyx_memviewslice __pyx_v_out, __Pyx_memviewslice __pyx_v_inside) {
  int __pyx_v_n;
  int __pyx_v_i;
  int __pyx_v_in_flag;
  double __pyx_v_tmp;
  __Pyx_memviewslice __pyx_v_dx = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_h = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_q = { 0, 0, { 0 }, { 0 }, { 0 } };
  long __pyx_v_p;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  long __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  int __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  Py_ssize_t __pyx_t_60;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_0_sparse_gradient_2d", 0);

  /* "dipy/align/vector_fields.pyx":3333
 *     """
 *     cdef:
 *         int n = sample_points.shape[0]             # <<<<<<<<<<<<<<
 *         int i, in_flag
 *         double tmp
 */
  __pyx_v_n = (__pyx_v_sample_points.shape[0]);

  /* "dipy/align/vector_fields.pyx":3336
 *         int i, in_flag
 *         double tmp
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_tuple__366) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_float64); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_dx = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3337
 *         double tmp
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_empty); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__367) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_float64); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_3);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_h = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3338
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_empty); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__368) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_float64); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_2);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_q = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3339
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":3340
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]             # <<<<<<<<<<<<<<
 *         h[1] = 0.5 * img_spacing[1]
 *         for i in range(n):
 */
        __pyx_t_6 = 0;
        __pyx_t_7 = 0;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_7 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_6 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3341
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = 1
 */
        __pyx_t_8 = 1;
        __pyx_t_9 = 1;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_9 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_8 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3342
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = 1
 *             dx[:] = sample_points[i, :2]
 */
        __pyx_t_10 = __pyx_v_n;
        for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
          __pyx_v_i = __pyx_t_11;

          /* "dipy/align/vector_fields.pyx":3343
 *         h[1] = 0.5 * img_spacing[1]
 *         for i in range(n):
 *             inside[i] = 1             # <<<<<<<<<<<<<<
 *             dx[:] = sample_points[i, :2]
 *             for p in range(2):
 */
          __pyx_t_12 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_12 * __pyx_v_inside.strides[0]) )) = 1;

          /* "dipy/align/vector_fields.pyx":3344
 *         for i in range(n):
 *             inside[i] = 1
 *             dx[:] = sample_points[i, :2]             # <<<<<<<<<<<<<<
 *             for p in range(2):
 *                 # Compute coordinates of point dx on img's grid
 */
          __pyx_t_5.data = __pyx_v_sample_points.data;
          __pyx_t_5.memview = __pyx_v_sample_points.memview;
          __PYX_INC_MEMVIEW(&__pyx_t_5, 0);
          {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_sample_points.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_sample_points.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3344; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
    }
        __pyx_t_5.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_13 = -1;
          if (unlikely(__pyx_memoryview_slice_memviewslice(
    &__pyx_t_5,
    __pyx_v_sample_points.shape[1], __pyx_v_sample_points.strides[1], __pyx_v_sample_points.suboffsets[1],
    1,
    0,
    &__pyx_t_13,
    0,
    2,
    0,
    0,
    1,
    0,
    1) < 0))
{
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3344; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
}

__pyx_t_14.data = __pyx_v_dx.data;
          __pyx_t_14.memview = __pyx_v_dx.memview;
          __PYX_INC_MEMVIEW(&__pyx_t_14, 0);
          __pyx_t_14.shape[0] = __pyx_v_dx.shape[0];
__pyx_t_14.strides[0] = __pyx_v_dx.strides[0];
    __pyx_t_14.suboffsets[0] = -1;

if (unlikely(__pyx_memoryview_copy_contents(__pyx_t_5, __pyx_t_14, 1, 1, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3344; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
          __PYX_XDEC_MEMVIEW(&__pyx_t_14, 0);
          __PYX_XDEC_MEMVIEW(&__pyx_t_5, 0);

          /* "dipy/align/vector_fields.pyx":3345
 *             inside[i] = 1
 *             dx[:] = sample_points[i, :2]
 *             for p in range(2):             # <<<<<<<<<<<<<<
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] - h[p]
 */
          for (__pyx_t_15 = 0; __pyx_t_15 < 2; __pyx_t_15+=1) {
            __pyx_v_p = __pyx_t_15;

            /* "dipy/align/vector_fields.pyx":3347
 *             for p in range(2):
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] - h[p]             # <<<<<<<<<<<<<<
 *                 q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 */
            __pyx_t_16 = __pyx_v_i;
            __pyx_t_17 = __pyx_v_p;
            __pyx_t_18 = __pyx_v_p;
            __pyx_t_19 = __pyx_v_p;
            *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_19 * __pyx_v_dx.strides[0]) )) = ((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_sample_points.data + __pyx_t_16 * __pyx_v_sample_points.strides[0]) ) + __pyx_t_17 * __pyx_v_sample_points.strides[1]) ))) - (*((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_18 * __pyx_v_h.strides[0]) ))));

            /* "dipy/align/vector_fields.pyx":3348
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] - h[p]
 *                 q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)             # <<<<<<<<<<<<<<
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                 # Interpolate img at q
 */
            __pyx_t_20 = 0;
            __pyx_t_21 = 1;
            __pyx_t_22 = 0;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_22 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_20 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_21 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3349
 *                 dx[p] = sample_points[i, p] - h[p]
 *                 q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)             # <<<<<<<<<<<<<<
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],
 */
            __pyx_t_23 = 0;
            __pyx_t_24 = 1;
            __pyx_t_25 = 1;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_25 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_23 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_24 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3351
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],             # <<<<<<<<<<<<<<
 *                                                            &out[i, p])
 *                 if in_flag == 0:
 */
            __pyx_t_26 = 0;
            __pyx_t_27 = 1;

            /* "dipy/align/vector_fields.pyx":3352
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],
 *                                                            &out[i, p])             # <<<<<<<<<<<<<<
 *                 if in_flag == 0:
 *                     out[i, p] = 0
 */
            __pyx_t_28 = __pyx_v_i;
            __pyx_t_29 = __pyx_v_p;

            /* "dipy/align/vector_fields.pyx":3351
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],             # <<<<<<<<<<<<<<
 *                                                            &out[i, p])
 *                 if in_flag == 0:
 */
            __pyx_v_in_flag = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__pyx_v_img, (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_26 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_27 * __pyx_v_q.strides[0]) ))), (&(*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_28 * __pyx_v_out.strides[0]) ) + __pyx_t_29 * __pyx_v_out.strides[1]) )))));

            /* "dipy/align/vector_fields.pyx":3353
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],
 *                                                            &out[i, p])
 *                 if in_flag == 0:             # <<<<<<<<<<<<<<
 *                     out[i, p] = 0
 *                     inside[i] = 0
 */
            __pyx_t_30 = ((__pyx_v_in_flag == 0) != 0);
            if (__pyx_t_30) {

              /* "dipy/align/vector_fields.pyx":3354
 *                                                            &out[i, p])
 *                 if in_flag == 0:
 *                     out[i, p] = 0             # <<<<<<<<<<<<<<
 *                     inside[i] = 0
 *                     continue
 */
              __pyx_t_31 = __pyx_v_i;
              __pyx_t_32 = __pyx_v_p;
              *((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_31 * __pyx_v_out.strides[0]) ) + __pyx_t_32 * __pyx_v_out.strides[1]) )) = 0.0;

              /* "dipy/align/vector_fields.pyx":3355
 *                 if in_flag == 0:
 *                     out[i, p] = 0
 *                     inside[i] = 0             # <<<<<<<<<<<<<<
 *                     continue
 *                 tmp = out[i, p]
 */
              __pyx_t_33 = __pyx_v_i;
              *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_33 * __pyx_v_inside.strides[0]) )) = 0;

              /* "dipy/align/vector_fields.pyx":3356
 *                     out[i, p] = 0
 *                     inside[i] = 0
 *                     continue             # <<<<<<<<<<<<<<
 *                 tmp = out[i, p]
 *                 # Compute coordinates of point dx on img's grid
 */
              goto __pyx_L8_continue;

              /* "dipy/align/vector_fields.pyx":3353
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],
 *                                                            &out[i, p])
 *                 if in_flag == 0:             # <<<<<<<<<<<<<<
 *                     out[i, p] = 0
 *                     inside[i] = 0
 */
            }

            /* "dipy/align/vector_fields.pyx":3357
 *                     inside[i] = 0
 *                     continue
 *                 tmp = out[i, p]             # <<<<<<<<<<<<<<
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] + h[p]
 */
            __pyx_t_34 = __pyx_v_i;
            __pyx_t_35 = __pyx_v_p;
            __pyx_v_tmp = (*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_34 * __pyx_v_out.strides[0]) ) + __pyx_t_35 * __pyx_v_out.strides[1]) )));

            /* "dipy/align/vector_fields.pyx":3359
 *                 tmp = out[i, p]
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] + h[p]             # <<<<<<<<<<<<<<
 *                 q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 */
            __pyx_t_36 = __pyx_v_i;
            __pyx_t_37 = __pyx_v_p;
            __pyx_t_38 = __pyx_v_p;
            __pyx_t_39 = __pyx_v_p;
            *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_39 * __pyx_v_dx.strides[0]) )) = ((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_sample_points.data + __pyx_t_36 * __pyx_v_sample_points.strides[0]) ) + __pyx_t_37 * __pyx_v_sample_points.strides[1]) ))) + (*((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_38 * __pyx_v_h.strides[0]) ))));

            /* "dipy/align/vector_fields.pyx":3360
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] + h[p]
 *                 q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)             # <<<<<<<<<<<<<<
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                 # Interpolate img at q
 */
            __pyx_t_40 = 0;
            __pyx_t_41 = 1;
            __pyx_t_42 = 0;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_42 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_40 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_41 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3361
 *                 dx[p] = sample_points[i, p] + h[p]
 *                 q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)             # <<<<<<<<<<<<<<
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],
 */
            __pyx_t_43 = 0;
            __pyx_t_44 = 1;
            __pyx_t_45 = 1;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_45 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_43 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_44 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3363
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],             # <<<<<<<<<<<<<<
 *                                                            &out[i, p])
 *                 if in_flag == 0:
 */
            __pyx_t_46 = 0;
            __pyx_t_47 = 1;

            /* "dipy/align/vector_fields.pyx":3364
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],
 *                                                            &out[i, p])             # <<<<<<<<<<<<<<
 *                 if in_flag == 0:
 *                     out[i, p] = 0
 */
            __pyx_t_48 = __pyx_v_i;
            __pyx_t_49 = __pyx_v_p;

            /* "dipy/align/vector_fields.pyx":3363
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],             # <<<<<<<<<<<<<<
 *                                                            &out[i, p])
 *                 if in_flag == 0:
 */
            __pyx_v_in_flag = __pyx_fuse_0__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__pyx_v_img, (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_46 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_47 * __pyx_v_q.strides[0]) ))), (&(*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_48 * __pyx_v_out.strides[0]) ) + __pyx_t_49 * __pyx_v_out.strides[1]) )))));

            /* "dipy/align/vector_fields.pyx":3365
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],
 *                                                            &out[i, p])
 *                 if in_flag == 0:             # <<<<<<<<<<<<<<
 *                     out[i, p] = 0
 *                     inside[i] = 0
 */
            __pyx_t_30 = ((__pyx_v_in_flag == 0) != 0);
            if (__pyx_t_30) {

              /* "dipy/align/vector_fields.pyx":3366
 *                                                            &out[i, p])
 *                 if in_flag == 0:
 *                     out[i, p] = 0             # <<<<<<<<<<<<<<
 *                     inside[i] = 0
 *                     continue
 */
              __pyx_t_50 = __pyx_v_i;
              __pyx_t_51 = __pyx_v_p;
              *((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_50 * __pyx_v_out.strides[0]) ) + __pyx_t_51 * __pyx_v_out.strides[1]) )) = 0.0;

              /* "dipy/align/vector_fields.pyx":3367
 *                 if in_flag == 0:
 *                     out[i, p] = 0
 *                     inside[i] = 0             # <<<<<<<<<<<<<<
 *                     continue
 *                 out[i, p] = (out[i, p] - tmp) / img_spacing[p]
 */
              __pyx_t_52 = __pyx_v_i;
              *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_52 * __pyx_v_inside.strides[0]) )) = 0;

              /* "dipy/align/vector_fields.pyx":3368
 *                     out[i, p] = 0
 *                     inside[i] = 0
 *                     continue             # <<<<<<<<<<<<<<
 *                 out[i, p] = (out[i, p] - tmp) / img_spacing[p]
 *                 dx[p] = sample_points[i, p]
 */
              goto __pyx_L8_continue;

              /* "dipy/align/vector_fields.pyx":3365
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],
 *                                                            &out[i, p])
 *                 if in_flag == 0:             # <<<<<<<<<<<<<<
 *                     out[i, p] = 0
 *                     inside[i] = 0
 */
            }

            /* "dipy/align/vector_fields.pyx":3369
 *                     inside[i] = 0
 *                     continue
 *                 out[i, p] = (out[i, p] - tmp) / img_spacing[p]             # <<<<<<<<<<<<<<
 *                 dx[p] = sample_points[i, p]
 * 
 */
            __pyx_t_53 = __pyx_v_i;
            __pyx_t_54 = __pyx_v_p;
            __pyx_t_55 = __pyx_v_p;
            __pyx_t_56 = __pyx_v_i;
            __pyx_t_57 = __pyx_v_p;
            *((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_56 * __pyx_v_out.strides[0]) ) + __pyx_t_57 * __pyx_v_out.strides[1]) )) = (((*((float *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_53 * __pyx_v_out.strides[0]) ) + __pyx_t_54 * __pyx_v_out.strides[1]) ))) - __pyx_v_tmp) / (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_55 * __pyx_v_img_spacing.strides[0]) ))));

            /* "dipy/align/vector_fields.pyx":3370
 *                     continue
 *                 out[i, p] = (out[i, p] - tmp) / img_spacing[p]
 *                 dx[p] = sample_points[i, p]             # <<<<<<<<<<<<<<
 * 
 * 
 */
            __pyx_t_58 = __pyx_v_i;
            __pyx_t_59 = __pyx_v_p;
            __pyx_t_60 = __pyx_v_p;
            *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_60 * __pyx_v_dx.strides[0]) )) = (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_sample_points.data + __pyx_t_58 * __pyx_v_sample_points.strides[0]) ) + __pyx_t_59 * __pyx_v_sample_points.strides[1]) )));
            __pyx_L8_continue:;
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":3339
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L4_error: {
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L1_error;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":3298
 * 
 * 
 * def _sparse_gradient_2d(floating[:, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                         double[:] img_spacing,
 *                         double[:, :] sample_points,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields._sparse_gradient_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_dx, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_h, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_q, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img_world2grid, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img_spacing, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_sample_points, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_359_sparse_gradient_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_359_sparse_gradient_2d = {"__pyx_fuse_1_sparse_gradient_2d", (PyCFunction)__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_359_sparse_gradient_2d, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_76_sparse_gradient_2d};
static PyObject *__pyx_fuse_1__pyx_pw_4dipy_5align_13vector_fields_359_sparse_gradient_2d(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  __Pyx_memviewslice __pyx_v_img = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_img_world2grid = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_img_spacing = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_sample_points = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_out = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_inside = { 0, 0, { 0 }, { 0 }, { 0 } };
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_sparse_gradient_2d (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_img,&__pyx_n_s_img_world2grid,&__pyx_n_s_img_spacing,&__pyx_n_s_sample_points,&__pyx_n_s_out,&__pyx_n_s_inside,0};
    PyObject* values[6] = {0,0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  6: values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_world2grid)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_2d", 1, 6, 6, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_spacing)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_2d", 1, 6, 6, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sample_points)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_2d", 1, 6, 6, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_2d", 1, 6, 6, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  5:
        if (likely((values[5] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_inside)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_sparse_gradient_2d", 1, 6, 6, 5); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_sparse_gradient_2d") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 6) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
      values[5] = PyTuple_GET_ITEM(__pyx_args, 5);
    }
    __pyx_v_img = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[0]); if (unlikely(!__pyx_v_img.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_img_world2grid = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[1]); if (unlikely(!__pyx_v_img_world2grid.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_img_spacing = __Pyx_PyObject_to_MemoryviewSlice_ds_double(values[2]); if (unlikely(!__pyx_v_img_spacing.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3299; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_sample_points = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[3]); if (unlikely(!__pyx_v_sample_points.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3300; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_out = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(values[4]); if (unlikely(!__pyx_v_out.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3301; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_inside = __Pyx_PyObject_to_MemoryviewSlice_ds_int(values[5]); if (unlikely(!__pyx_v_inside.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3301; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_sparse_gradient_2d", 1, 6, 6, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields._sparse_gradient_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_358_sparse_gradient_2d(__pyx_self, __pyx_v_img, __pyx_v_img_world2grid, __pyx_v_img_spacing, __pyx_v_sample_points, __pyx_v_out, __pyx_v_inside);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_358_sparse_gradient_2d(CYTHON_UNUSED PyObject *__pyx_self, __Pyx_memviewslice __pyx_v_img, __Pyx_memviewslice __pyx_v_img_world2grid, __Pyx_memviewslice __pyx_v_img_spacing, __Pyx_memviewslice __pyx_v_sample_points, __Pyx_memviewslice __pyx_v_out, __Pyx_memviewslice __pyx_v_inside) {
  int __pyx_v_n;
  int __pyx_v_i;
  int __pyx_v_in_flag;
  double __pyx_v_tmp;
  __Pyx_memviewslice __pyx_v_dx = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_h = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_v_q = { 0, 0, { 0 }, { 0 }, { 0 } };
  long __pyx_v_p;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_memviewslice __pyx_t_5 = { 0, 0, { 0 }, { 0 }, { 0 } };
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_t_13;
  __Pyx_memviewslice __pyx_t_14 = { 0, 0, { 0 }, { 0 }, { 0 } };
  long __pyx_t_15;
  Py_ssize_t __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  Py_ssize_t __pyx_t_19;
  Py_ssize_t __pyx_t_20;
  Py_ssize_t __pyx_t_21;
  Py_ssize_t __pyx_t_22;
  Py_ssize_t __pyx_t_23;
  Py_ssize_t __pyx_t_24;
  Py_ssize_t __pyx_t_25;
  Py_ssize_t __pyx_t_26;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  Py_ssize_t __pyx_t_29;
  int __pyx_t_30;
  Py_ssize_t __pyx_t_31;
  Py_ssize_t __pyx_t_32;
  Py_ssize_t __pyx_t_33;
  Py_ssize_t __pyx_t_34;
  Py_ssize_t __pyx_t_35;
  Py_ssize_t __pyx_t_36;
  Py_ssize_t __pyx_t_37;
  Py_ssize_t __pyx_t_38;
  Py_ssize_t __pyx_t_39;
  Py_ssize_t __pyx_t_40;
  Py_ssize_t __pyx_t_41;
  Py_ssize_t __pyx_t_42;
  Py_ssize_t __pyx_t_43;
  Py_ssize_t __pyx_t_44;
  Py_ssize_t __pyx_t_45;
  Py_ssize_t __pyx_t_46;
  Py_ssize_t __pyx_t_47;
  Py_ssize_t __pyx_t_48;
  Py_ssize_t __pyx_t_49;
  Py_ssize_t __pyx_t_50;
  Py_ssize_t __pyx_t_51;
  Py_ssize_t __pyx_t_52;
  Py_ssize_t __pyx_t_53;
  Py_ssize_t __pyx_t_54;
  Py_ssize_t __pyx_t_55;
  Py_ssize_t __pyx_t_56;
  Py_ssize_t __pyx_t_57;
  Py_ssize_t __pyx_t_58;
  Py_ssize_t __pyx_t_59;
  Py_ssize_t __pyx_t_60;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_fuse_1_sparse_gradient_2d", 0);

  /* "dipy/align/vector_fields.pyx":3333
 *     """
 *     cdef:
 *         int n = sample_points.shape[0]             # <<<<<<<<<<<<<<
 *         int i, in_flag
 *         double tmp
 */
  __pyx_v_n = (__pyx_v_sample_points.shape[0]);

  /* "dipy/align/vector_fields.pyx":3336
 *         int i, in_flag
 *         double tmp
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_tuple__369) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_float64); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_4);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_dx = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3337
 *         double tmp
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_empty); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_tuple__370) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_float64); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_3);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_h = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3338
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_empty); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_shape, __pyx_tuple__371) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_float64); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __Pyx_PyObject_to_MemoryviewSlice_ds_double(__pyx_t_2);
  if (unlikely(!__pyx_t_5.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_q = __pyx_t_5;
  __pyx_t_5.memview = NULL;
  __pyx_t_5.data = NULL;

  /* "dipy/align/vector_fields.pyx":3339
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "dipy/align/vector_fields.pyx":3340
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]             # <<<<<<<<<<<<<<
 *         h[1] = 0.5 * img_spacing[1]
 *         for i in range(n):
 */
        __pyx_t_6 = 0;
        __pyx_t_7 = 0;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_7 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_6 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3341
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             inside[i] = 1
 */
        __pyx_t_8 = 1;
        __pyx_t_9 = 1;
        *((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_9 * __pyx_v_h.strides[0]) )) = (0.5 * (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_8 * __pyx_v_img_spacing.strides[0]) ))));

        /* "dipy/align/vector_fields.pyx":3342
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             inside[i] = 1
 *             dx[:] = sample_points[i, :2]
 */
        __pyx_t_10 = __pyx_v_n;
        for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
          __pyx_v_i = __pyx_t_11;

          /* "dipy/align/vector_fields.pyx":3343
 *         h[1] = 0.5 * img_spacing[1]
 *         for i in range(n):
 *             inside[i] = 1             # <<<<<<<<<<<<<<
 *             dx[:] = sample_points[i, :2]
 *             for p in range(2):
 */
          __pyx_t_12 = __pyx_v_i;
          *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_12 * __pyx_v_inside.strides[0]) )) = 1;

          /* "dipy/align/vector_fields.pyx":3344
 *         for i in range(n):
 *             inside[i] = 1
 *             dx[:] = sample_points[i, :2]             # <<<<<<<<<<<<<<
 *             for p in range(2):
 *                 # Compute coordinates of point dx on img's grid
 */
          __pyx_t_5.data = __pyx_v_sample_points.data;
          __pyx_t_5.memview = __pyx_v_sample_points.memview;
          __PYX_INC_MEMVIEW(&__pyx_t_5, 0);
          {
    Py_ssize_t __pyx_tmp_idx = __pyx_v_i;
    Py_ssize_t __pyx_tmp_shape = __pyx_v_sample_points.shape[0];
    Py_ssize_t __pyx_tmp_stride = __pyx_v_sample_points.strides[0];
    if (0 && (__pyx_tmp_idx < 0))
        __pyx_tmp_idx += __pyx_tmp_shape;
    if (0 && (__pyx_tmp_idx < 0 || __pyx_tmp_idx >= __pyx_tmp_shape)) {
            #ifdef WITH_THREAD
            PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
            #endif
        PyErr_SetString(PyExc_IndexError, "Index out of bounds (axis 0)");
            #ifdef WITH_THREAD
            PyGILState_Release(__pyx_gilstate_save);
            #endif
        {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3344; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
    }
        __pyx_t_5.data += __pyx_tmp_idx * __pyx_tmp_stride;
}

__pyx_t_13 = -1;
          if (unlikely(__pyx_memoryview_slice_memviewslice(
    &__pyx_t_5,
    __pyx_v_sample_points.shape[1], __pyx_v_sample_points.strides[1], __pyx_v_sample_points.suboffsets[1],
    1,
    0,
    &__pyx_t_13,
    0,
    2,
    0,
    0,
    1,
    0,
    1) < 0))
{
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3344; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
}

__pyx_t_14.data = __pyx_v_dx.data;
          __pyx_t_14.memview = __pyx_v_dx.memview;
          __PYX_INC_MEMVIEW(&__pyx_t_14, 0);
          __pyx_t_14.shape[0] = __pyx_v_dx.shape[0];
__pyx_t_14.strides[0] = __pyx_v_dx.strides[0];
    __pyx_t_14.suboffsets[0] = -1;

if (unlikely(__pyx_memoryview_copy_contents(__pyx_t_5, __pyx_t_14, 1, 1, 0) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3344; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
          __PYX_XDEC_MEMVIEW(&__pyx_t_14, 0);
          __PYX_XDEC_MEMVIEW(&__pyx_t_5, 0);

          /* "dipy/align/vector_fields.pyx":3345
 *             inside[i] = 1
 *             dx[:] = sample_points[i, :2]
 *             for p in range(2):             # <<<<<<<<<<<<<<
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] - h[p]
 */
          for (__pyx_t_15 = 0; __pyx_t_15 < 2; __pyx_t_15+=1) {
            __pyx_v_p = __pyx_t_15;

            /* "dipy/align/vector_fields.pyx":3347
 *             for p in range(2):
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] - h[p]             # <<<<<<<<<<<<<<
 *                 q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 */
            __pyx_t_16 = __pyx_v_i;
            __pyx_t_17 = __pyx_v_p;
            __pyx_t_18 = __pyx_v_p;
            __pyx_t_19 = __pyx_v_p;
            *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_19 * __pyx_v_dx.strides[0]) )) = ((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_sample_points.data + __pyx_t_16 * __pyx_v_sample_points.strides[0]) ) + __pyx_t_17 * __pyx_v_sample_points.strides[1]) ))) - (*((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_18 * __pyx_v_h.strides[0]) ))));

            /* "dipy/align/vector_fields.pyx":3348
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] - h[p]
 *                 q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)             # <<<<<<<<<<<<<<
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                 # Interpolate img at q
 */
            __pyx_t_20 = 0;
            __pyx_t_21 = 1;
            __pyx_t_22 = 0;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_22 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_20 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_21 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3349
 *                 dx[p] = sample_points[i, p] - h[p]
 *                 q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)             # <<<<<<<<<<<<<<
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],
 */
            __pyx_t_23 = 0;
            __pyx_t_24 = 1;
            __pyx_t_25 = 1;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_25 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_23 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_24 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3351
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],             # <<<<<<<<<<<<<<
 *                                                            &out[i, p])
 *                 if in_flag == 0:
 */
            __pyx_t_26 = 0;
            __pyx_t_27 = 1;

            /* "dipy/align/vector_fields.pyx":3352
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],
 *                                                            &out[i, p])             # <<<<<<<<<<<<<<
 *                 if in_flag == 0:
 *                     out[i, p] = 0
 */
            __pyx_t_28 = __pyx_v_i;
            __pyx_t_29 = __pyx_v_p;

            /* "dipy/align/vector_fields.pyx":3351
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],             # <<<<<<<<<<<<<<
 *                                                            &out[i, p])
 *                 if in_flag == 0:
 */
            __pyx_v_in_flag = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__pyx_v_img, (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_26 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_27 * __pyx_v_q.strides[0]) ))), (&(*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_28 * __pyx_v_out.strides[0]) ) + __pyx_t_29 * __pyx_v_out.strides[1]) )))));

            /* "dipy/align/vector_fields.pyx":3353
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],
 *                                                            &out[i, p])
 *                 if in_flag == 0:             # <<<<<<<<<<<<<<
 *                     out[i, p] = 0
 *                     inside[i] = 0
 */
            __pyx_t_30 = ((__pyx_v_in_flag == 0) != 0);
            if (__pyx_t_30) {

              /* "dipy/align/vector_fields.pyx":3354
 *                                                            &out[i, p])
 *                 if in_flag == 0:
 *                     out[i, p] = 0             # <<<<<<<<<<<<<<
 *                     inside[i] = 0
 *                     continue
 */
              __pyx_t_31 = __pyx_v_i;
              __pyx_t_32 = __pyx_v_p;
              *((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_31 * __pyx_v_out.strides[0]) ) + __pyx_t_32 * __pyx_v_out.strides[1]) )) = 0.0;

              /* "dipy/align/vector_fields.pyx":3355
 *                 if in_flag == 0:
 *                     out[i, p] = 0
 *                     inside[i] = 0             # <<<<<<<<<<<<<<
 *                     continue
 *                 tmp = out[i, p]
 */
              __pyx_t_33 = __pyx_v_i;
              *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_33 * __pyx_v_inside.strides[0]) )) = 0;

              /* "dipy/align/vector_fields.pyx":3356
 *                     out[i, p] = 0
 *                     inside[i] = 0
 *                     continue             # <<<<<<<<<<<<<<
 *                 tmp = out[i, p]
 *                 # Compute coordinates of point dx on img's grid
 */
              goto __pyx_L8_continue;

              /* "dipy/align/vector_fields.pyx":3353
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],
 *                                                            &out[i, p])
 *                 if in_flag == 0:             # <<<<<<<<<<<<<<
 *                     out[i, p] = 0
 *                     inside[i] = 0
 */
            }

            /* "dipy/align/vector_fields.pyx":3357
 *                     inside[i] = 0
 *                     continue
 *                 tmp = out[i, p]             # <<<<<<<<<<<<<<
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] + h[p]
 */
            __pyx_t_34 = __pyx_v_i;
            __pyx_t_35 = __pyx_v_p;
            __pyx_v_tmp = (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_34 * __pyx_v_out.strides[0]) ) + __pyx_t_35 * __pyx_v_out.strides[1]) )));

            /* "dipy/align/vector_fields.pyx":3359
 *                 tmp = out[i, p]
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] + h[p]             # <<<<<<<<<<<<<<
 *                 q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 */
            __pyx_t_36 = __pyx_v_i;
            __pyx_t_37 = __pyx_v_p;
            __pyx_t_38 = __pyx_v_p;
            __pyx_t_39 = __pyx_v_p;
            *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_39 * __pyx_v_dx.strides[0]) )) = ((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_sample_points.data + __pyx_t_36 * __pyx_v_sample_points.strides[0]) ) + __pyx_t_37 * __pyx_v_sample_points.strides[1]) ))) + (*((double *) ( /* dim=0 */ (__pyx_v_h.data + __pyx_t_38 * __pyx_v_h.strides[0]) ))));

            /* "dipy/align/vector_fields.pyx":3360
 *                 # Compute coordinates of point dx on img's grid
 *                 dx[p] = sample_points[i, p] + h[p]
 *                 q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)             # <<<<<<<<<<<<<<
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                 # Interpolate img at q
 */
            __pyx_t_40 = 0;
            __pyx_t_41 = 1;
            __pyx_t_42 = 0;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_42 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_40 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_41 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3361
 *                 dx[p] = sample_points[i, p] + h[p]
 *                 q[0] = _apply_affine_2d_x0(dx[0], dx[1], 1, img_world2grid)
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)             # <<<<<<<<<<<<<<
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],
 */
            __pyx_t_43 = 0;
            __pyx_t_44 = 1;
            __pyx_t_45 = 1;
            *((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_45 * __pyx_v_q.strides[0]) )) = __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1((*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_43 * __pyx_v_dx.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_44 * __pyx_v_dx.strides[0]) ))), 1.0, __pyx_v_img_world2grid);

            /* "dipy/align/vector_fields.pyx":3363
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],             # <<<<<<<<<<<<<<
 *                                                            &out[i, p])
 *                 if in_flag == 0:
 */
            __pyx_t_46 = 0;
            __pyx_t_47 = 1;

            /* "dipy/align/vector_fields.pyx":3364
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],
 *                                                            &out[i, p])             # <<<<<<<<<<<<<<
 *                 if in_flag == 0:
 *                     out[i, p] = 0
 */
            __pyx_t_48 = __pyx_v_i;
            __pyx_t_49 = __pyx_v_p;

            /* "dipy/align/vector_fields.pyx":3363
 *                 q[1] = _apply_affine_2d_x1(dx[0], dx[1], 1, img_world2grid)
 *                 # Interpolate img at q
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],             # <<<<<<<<<<<<<<
 *                                                            &out[i, p])
 *                 if in_flag == 0:
 */
            __pyx_v_in_flag = __pyx_fuse_1__pyx_f_4dipy_5align_13vector_fields__interpolate_scalar_2d(__pyx_v_img, (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_46 * __pyx_v_q.strides[0]) ))), (*((double *) ( /* dim=0 */ (__pyx_v_q.data + __pyx_t_47 * __pyx_v_q.strides[0]) ))), (&(*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_48 * __pyx_v_out.strides[0]) ) + __pyx_t_49 * __pyx_v_out.strides[1]) )))));

            /* "dipy/align/vector_fields.pyx":3365
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],
 *                                                            &out[i, p])
 *                 if in_flag == 0:             # <<<<<<<<<<<<<<
 *                     out[i, p] = 0
 *                     inside[i] = 0
 */
            __pyx_t_30 = ((__pyx_v_in_flag == 0) != 0);
            if (__pyx_t_30) {

              /* "dipy/align/vector_fields.pyx":3366
 *                                                            &out[i, p])
 *                 if in_flag == 0:
 *                     out[i, p] = 0             # <<<<<<<<<<<<<<
 *                     inside[i] = 0
 *                     continue
 */
              __pyx_t_50 = __pyx_v_i;
              __pyx_t_51 = __pyx_v_p;
              *((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_50 * __pyx_v_out.strides[0]) ) + __pyx_t_51 * __pyx_v_out.strides[1]) )) = 0.0;

              /* "dipy/align/vector_fields.pyx":3367
 *                 if in_flag == 0:
 *                     out[i, p] = 0
 *                     inside[i] = 0             # <<<<<<<<<<<<<<
 *                     continue
 *                 out[i, p] = (out[i, p] - tmp) / img_spacing[p]
 */
              __pyx_t_52 = __pyx_v_i;
              *((int *) ( /* dim=0 */ (__pyx_v_inside.data + __pyx_t_52 * __pyx_v_inside.strides[0]) )) = 0;

              /* "dipy/align/vector_fields.pyx":3368
 *                     out[i, p] = 0
 *                     inside[i] = 0
 *                     continue             # <<<<<<<<<<<<<<
 *                 out[i, p] = (out[i, p] - tmp) / img_spacing[p]
 *                 dx[p] = sample_points[i, p]
 */
              goto __pyx_L8_continue;

              /* "dipy/align/vector_fields.pyx":3365
 *                 in_flag = _interpolate_scalar_2d[floating](img, q[0], q[1],
 *                                                            &out[i, p])
 *                 if in_flag == 0:             # <<<<<<<<<<<<<<
 *                     out[i, p] = 0
 *                     inside[i] = 0
 */
            }

            /* "dipy/align/vector_fields.pyx":3369
 *                     inside[i] = 0
 *                     continue
 *                 out[i, p] = (out[i, p] - tmp) / img_spacing[p]             # <<<<<<<<<<<<<<
 *                 dx[p] = sample_points[i, p]
 * 
 */
            __pyx_t_53 = __pyx_v_i;
            __pyx_t_54 = __pyx_v_p;
            __pyx_t_55 = __pyx_v_p;
            __pyx_t_56 = __pyx_v_i;
            __pyx_t_57 = __pyx_v_p;
            *((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_56 * __pyx_v_out.strides[0]) ) + __pyx_t_57 * __pyx_v_out.strides[1]) )) = (((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_out.data + __pyx_t_53 * __pyx_v_out.strides[0]) ) + __pyx_t_54 * __pyx_v_out.strides[1]) ))) - __pyx_v_tmp) / (*((double *) ( /* dim=0 */ (__pyx_v_img_spacing.data + __pyx_t_55 * __pyx_v_img_spacing.strides[0]) ))));

            /* "dipy/align/vector_fields.pyx":3370
 *                     continue
 *                 out[i, p] = (out[i, p] - tmp) / img_spacing[p]
 *                 dx[p] = sample_points[i, p]             # <<<<<<<<<<<<<<
 * 
 * 
 */
            __pyx_t_58 = __pyx_v_i;
            __pyx_t_59 = __pyx_v_p;
            __pyx_t_60 = __pyx_v_p;
            *((double *) ( /* dim=0 */ (__pyx_v_dx.data + __pyx_t_60 * __pyx_v_dx.strides[0]) )) = (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_sample_points.data + __pyx_t_58 * __pyx_v_sample_points.strides[0]) ) + __pyx_t_59 * __pyx_v_sample_points.strides[1]) )));
            __pyx_L8_continue:;
          }
        }
      }

      /* "dipy/align/vector_fields.pyx":3339
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         h[0] = 0.5 * img_spacing[0]
 *         h[1] = 0.5 * img_spacing[1]
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L4_error: {
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L1_error;
        }
        __pyx_L5:;
      }
  }

  /* "dipy/align/vector_fields.pyx":3298
 * 
 * 
 * def _sparse_gradient_2d(floating[:, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                         double[:] img_spacing,
 *                         double[:, :] sample_points,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __PYX_XDEC_MEMVIEW(&__pyx_t_5, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_14, 1);
  __Pyx_AddTraceback("dipy.align.vector_fields._sparse_gradient_2d", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __PYX_XDEC_MEMVIEW(&__pyx_v_dx, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_h, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_q, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img_world2grid, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_img_spacing, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_sample_points, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_out, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_v_inside, 1);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":3373
 * 
 * 
 * def gradient(img, img_world2grid, img_spacing, out_shape,             # <<<<<<<<<<<<<<
 *              out_grid2world):
 *     r""" Gradient of an image in physical space
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_79gradient(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_78gradient[] = " Gradient of an image in physical space\n\n    Parameters\n    ----------\n    img : 2D or 3D array, shape (R, C) or (S, R, C)\n        the input image whose gradient will be computed\n    img_world2grid : array, shape (dim+1, dim+1)\n        the space-to-grid transform matrix associated to img\n    img_spacing : array, shape (dim,)\n        the spacing between voxels (voxel size along each axis) of the input\n        image\n    out_shape : array, shape (dim,)\n        the number of (slices), rows and columns of the sampling grid\n    out_grid2world : array, shape (dim+1, dim+1)\n        the grid-to-space transform associated to the sampling grid\n\n    Returns\n    -------\n    out : array, shape (R', C', 2) or (S', R', C', 3)\n        the buffer in which to store the image gradient, where\n        (S'), R', C' are given by out_shape\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_79gradient = {"gradient", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_79gradient, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_78gradient};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_79gradient(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_img = 0;
  PyObject *__pyx_v_img_world2grid = 0;
  PyObject *__pyx_v_img_spacing = 0;
  PyObject *__pyx_v_out_shape = 0;
  PyObject *__pyx_v_out_grid2world = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gradient (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_img,&__pyx_n_s_img_world2grid,&__pyx_n_s_img_spacing,&__pyx_n_s_out_shape,&__pyx_n_s_out_grid2world,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_world2grid)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("gradient", 1, 5, 5, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3373; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_spacing)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("gradient", 1, 5, 5, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3373; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_shape)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("gradient", 1, 5, 5, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3373; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  4:
        if (likely((values[4] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_out_grid2world)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("gradient", 1, 5, 5, 4); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3373; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "gradient") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3373; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 5) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
      values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
    }
    __pyx_v_img = values[0];
    __pyx_v_img_world2grid = values[1];
    __pyx_v_img_spacing = values[2];
    __pyx_v_out_shape = values[3];
    __pyx_v_out_grid2world = values[4];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gradient", 1, 5, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3373; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.gradient", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_78gradient(__pyx_self, __pyx_v_img, __pyx_v_img_world2grid, __pyx_v_img_spacing, __pyx_v_out_shape, __pyx_v_out_grid2world);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_78gradient(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_img, PyObject *__pyx_v_img_world2grid, PyObject *__pyx_v_img_spacing, PyObject *__pyx_v_out_shape, PyObject *__pyx_v_out_grid2world) {
  Py_ssize_t __pyx_v_dim;
  PyObject *__pyx_v_ftype = NULL;
  PyObject *__pyx_v_out = NULL;
  PyObject *__pyx_v_inside = NULL;
  PyObject *__pyx_v_jd_grad = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gradient", 0);

  /* "dipy/align/vector_fields.pyx":3397
 *         (S'), R', C' are given by out_shape
 *     """
 *     dim = len(img.shape)             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(img_world2grid, dim):
 *         raise ValueError("Invalid image affine transform")
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_img, __pyx_n_s_shape); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_dim = __pyx_t_2;

  /* "dipy/align/vector_fields.pyx":3398
 *     """
 *     dim = len(img.shape)
 *     if not is_valid_affine(img_world2grid, dim):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid image affine transform")
 *     if not is_valid_affine(out_grid2world, dim):
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3398; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3398; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_2 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_2 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3398; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_5) {
    __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
  }
  __Pyx_INCREF(__pyx_v_img_world2grid);
  __Pyx_GIVEREF(__pyx_v_img_world2grid);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_2, __pyx_v_img_world2grid);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_2, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3398; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3398; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":3399
 *     dim = len(img.shape)
 *     if not is_valid_affine(img_world2grid, dim):
 *         raise ValueError("Invalid image affine transform")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(out_grid2world, dim):
 *         raise ValueError("Invalid sampling grid affine transform")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__372, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":3398
 *     """
 *     dim = len(img.shape)
 *     if not is_valid_affine(img_world2grid, dim):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid image affine transform")
 *     if not is_valid_affine(out_grid2world, dim):
 */
  }

  /* "dipy/align/vector_fields.pyx":3400
 *     if not is_valid_affine(img_world2grid, dim):
 *         raise ValueError("Invalid image affine transform")
 *     if not is_valid_affine(out_grid2world, dim):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid sampling grid affine transform")
 *     if len(img_spacing) < dim:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3400; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3400; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = NULL;
  __pyx_t_2 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_2 = 1;
    }
  }
  __pyx_t_5 = PyTuple_New(2+__pyx_t_2); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3400; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (__pyx_t_4) {
    __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
  }
  __Pyx_INCREF(__pyx_v_out_grid2world);
  __Pyx_GIVEREF(__pyx_v_out_grid2world);
  PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_2, __pyx_v_out_grid2world);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_2, __pyx_t_6);
  __pyx_t_6 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3400; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_8 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3400; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_7 = ((!__pyx_t_8) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":3401
 *         raise ValueError("Invalid image affine transform")
 *     if not is_valid_affine(out_grid2world, dim):
 *         raise ValueError("Invalid sampling grid affine transform")             # <<<<<<<<<<<<<<
 *     if len(img_spacing) < dim:
 *         raise ValueError("Invalid spacings")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__373, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3401; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3401; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":3400
 *     if not is_valid_affine(img_world2grid, dim):
 *         raise ValueError("Invalid image affine transform")
 *     if not is_valid_affine(out_grid2world, dim):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid sampling grid affine transform")
 *     if len(img_spacing) < dim:
 */
  }

  /* "dipy/align/vector_fields.pyx":3402
 *     if not is_valid_affine(out_grid2world, dim):
 *         raise ValueError("Invalid sampling grid affine transform")
 *     if len(img_spacing) < dim:             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid spacings")
 *     ftype = img.dtype.type
 */
  __pyx_t_2 = PyObject_Length(__pyx_v_img_spacing); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3402; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_7 = ((__pyx_t_2 < __pyx_v_dim) != 0);
  if (__pyx_t_7) {

    /* "dipy/align/vector_fields.pyx":3403
 *         raise ValueError("Invalid sampling grid affine transform")
 *     if len(img_spacing) < dim:
 *         raise ValueError("Invalid spacings")             # <<<<<<<<<<<<<<
 *     ftype = img.dtype.type
 *     out = np.empty(tuple(out_shape)+(dim,), dtype=ftype)
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__374, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3403; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3403; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":3402
 *     if not is_valid_affine(out_grid2world, dim):
 *         raise ValueError("Invalid sampling grid affine transform")
 *     if len(img_spacing) < dim:             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid spacings")
 *     ftype = img.dtype.type
 */
  }

  /* "dipy/align/vector_fields.pyx":3404
 *     if len(img_spacing) < dim:
 *         raise ValueError("Invalid spacings")
 *     ftype = img.dtype.type             # <<<<<<<<<<<<<<
 *     out = np.empty(tuple(out_shape)+(dim,), dtype=ftype)
 *     inside = np.empty(tuple(out_shape), dtype=np.int32)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_img, __pyx_n_s_dtype); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3404; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_type); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3404; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":3405
 *         raise ValueError("Invalid spacings")
 *     ftype = img.dtype.type
 *     out = np.empty(tuple(out_shape)+(dim,), dtype=ftype)             # <<<<<<<<<<<<<<
 *     inside = np.empty(tuple(out_shape), dtype=np.int32)
 *     # Select joint density gradient 2D or 3D
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3405; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_empty); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3405; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PySequence_Tuple(__pyx_v_out_shape); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3405; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3405; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3405; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  __pyx_t_5 = PyNumber_Add(__pyx_t_3, __pyx_t_6); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3405; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3405; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5);
  __pyx_t_5 = 0;
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3405; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3405; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_6, __pyx_t_5); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3405; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_out = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":3406
 *     ftype = img.dtype.type
 *     out = np.empty(tuple(out_shape)+(dim,), dtype=ftype)
 *     inside = np.empty(tuple(out_shape), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     # Select joint density gradient 2D or 3D
 *     if dim == 2:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3406; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_empty); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3406; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PySequence_Tuple(__pyx_v_out_shape); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3406; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3406; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3406; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3406; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_int32); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3406; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_dtype, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3406; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_6, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3406; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_inside = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":3408
 *     inside = np.empty(tuple(out_shape), dtype=np.int32)
 *     # Select joint density gradient 2D or 3D
 *     if dim == 2:             # <<<<<<<<<<<<<<
 *         jd_grad = _gradient_2d
 *     elif dim == 3:
 */
  switch (__pyx_v_dim) {
    case 2:

    /* "dipy/align/vector_fields.pyx":3409
 *     # Select joint density gradient 2D or 3D
 *     if dim == 2:
 *         jd_grad = _gradient_2d             # <<<<<<<<<<<<<<
 *     elif dim == 3:
 *         jd_grad = _gradient_3d
 */
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_gradient_2d); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3409; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_v_jd_grad = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "dipy/align/vector_fields.pyx":3408
 *     inside = np.empty(tuple(out_shape), dtype=np.int32)
 *     # Select joint density gradient 2D or 3D
 *     if dim == 2:             # <<<<<<<<<<<<<<
 *         jd_grad = _gradient_2d
 *     elif dim == 3:
 */
    break;

    /* "dipy/align/vector_fields.pyx":3410
 *     if dim == 2:
 *         jd_grad = _gradient_2d
 *     elif dim == 3:             # <<<<<<<<<<<<<<
 *         jd_grad = _gradient_3d
 *     else:
 */
    case 3:

    /* "dipy/align/vector_fields.pyx":3411
 *         jd_grad = _gradient_2d
 *     elif dim == 3:
 *         jd_grad = _gradient_3d             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError('Undefined gradient for image dimension %d' % (dim,))
 */
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_gradient_3d); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_v_jd_grad = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "dipy/align/vector_fields.pyx":3410
 *     if dim == 2:
 *         jd_grad = _gradient_2d
 *     elif dim == 3:             # <<<<<<<<<<<<<<
 *         jd_grad = _gradient_3d
 *     else:
 */
    break;
    default:

    /* "dipy/align/vector_fields.pyx":3413
 *         jd_grad = _gradient_3d
 *     else:
 *         raise ValueError('Undefined gradient for image dimension %d' % (dim,))             # <<<<<<<<<<<<<<
 *     jd_grad(img, img_world2grid.astype(np.float64),
 *             img_spacing.astype(np.float64),
 */
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Undefined_gradient_for_image_dim, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    break;
  }

  /* "dipy/align/vector_fields.pyx":3414
 *     else:
 *         raise ValueError('Undefined gradient for image dimension %d' % (dim,))
 *     jd_grad(img, img_world2grid.astype(np.float64),             # <<<<<<<<<<<<<<
 *             img_spacing.astype(np.float64),
 *             out_grid2world.astype(np.float64), out, inside)
 */
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_img_world2grid, __pyx_n_s_astype); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_float64); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_6, function);
    }
  }
  if (!__pyx_t_5) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_5); __pyx_t_5 = NULL;
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_9, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":3415
 *         raise ValueError('Undefined gradient for image dimension %d' % (dim,))
 *     jd_grad(img, img_world2grid.astype(np.float64),
 *             img_spacing.astype(np.float64),             # <<<<<<<<<<<<<<
 *             out_grid2world.astype(np.float64), out, inside)
 *     return out, inside
 */
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_img_spacing, __pyx_n_s_astype); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_float64); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  if (!__pyx_t_1) {
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_5); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_6);
  } else {
    __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_1); __pyx_t_1 = NULL;
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_10, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3415; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "dipy/align/vector_fields.pyx":3416
 *     jd_grad(img, img_world2grid.astype(np.float64),
 *             img_spacing.astype(np.float64),
 *             out_grid2world.astype(np.float64), out, inside)             # <<<<<<<<<<<<<<
 *     return out, inside
 * 
 */
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_out_grid2world, __pyx_n_s_astype); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_float64); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
    }
  }
  if (!__pyx_t_5) {
    __pyx_t_9 = __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GOTREF(__pyx_t_9);
  } else {
    __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_5); __pyx_t_5 = NULL;
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_11, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_INCREF(__pyx_v_jd_grad);
  __pyx_t_10 = __pyx_v_jd_grad; __pyx_t_11 = NULL;
  __pyx_t_2 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
      __pyx_t_2 = 1;
    }
  }
  __pyx_t_1 = PyTuple_New(6+__pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (__pyx_t_11) {
    __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_11); __pyx_t_11 = NULL;
  }
  __Pyx_INCREF(__pyx_v_img);
  __Pyx_GIVEREF(__pyx_v_img);
  PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_2, __pyx_v_img);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_1, 2+__pyx_t_2, __pyx_t_6);
  __Pyx_GIVEREF(__pyx_t_9);
  PyTuple_SET_ITEM(__pyx_t_1, 3+__pyx_t_2, __pyx_t_9);
  __Pyx_INCREF(__pyx_v_out);
  __Pyx_GIVEREF(__pyx_v_out);
  PyTuple_SET_ITEM(__pyx_t_1, 4+__pyx_t_2, __pyx_v_out);
  __Pyx_INCREF(__pyx_v_inside);
  __Pyx_GIVEREF(__pyx_v_inside);
  PyTuple_SET_ITEM(__pyx_t_1, 5+__pyx_t_2, __pyx_v_inside);
  __pyx_t_3 = 0;
  __pyx_t_6 = 0;
  __pyx_t_9 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_1, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3414; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":3417
 *             img_spacing.astype(np.float64),
 *             out_grid2world.astype(np.float64), out, inside)
 *     return out, inside             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_out);
  __Pyx_GIVEREF(__pyx_v_out);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_out);
  __Pyx_INCREF(__pyx_v_inside);
  __Pyx_GIVEREF(__pyx_v_inside);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_v_inside);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":3373
 * 
 * 
 * def gradient(img, img_world2grid, img_spacing, out_shape,             # <<<<<<<<<<<<<<
 *              out_grid2world):
 *     r""" Gradient of an image in physical space
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("dipy.align.vector_fields.gradient", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __Pyx_XDECREF(__pyx_v_out);
  __Pyx_XDECREF(__pyx_v_inside);
  __Pyx_XDECREF(__pyx_v_jd_grad);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pyx":3420
 * 
 * 
 * def sparse_gradient(img, img_world2grid, img_spacing, sample_points):             # <<<<<<<<<<<<<<
 *     r""" Gradient of an image in physical space
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_81sparse_gradient(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_4dipy_5align_13vector_fields_80sparse_gradient[] = " Gradient of an image in physical space\n\n    Parameters\n    ----------\n    img : 2D or 3D array, shape (R, C) or (S, R, C)\n        the input image whose gradient will be computed\n    img_world2grid : array, shape (dim+1, dim+1)\n        the space-to-grid transform matrix associated to img\n    img_spacing : array, shape (dim,)\n        the spacing between voxels (voxel size along each axis) of the input\n        image\n    sample_points: array, shape (n, dim)\n        list of points where the derivative will be evaluated\n        (one point per row)\n\n    Returns\n    -------\n    out : array, shape (n, dim)\n        the gradient at each point stored at its corresponding row\n    ";
static PyMethodDef __pyx_mdef_4dipy_5align_13vector_fields_81sparse_gradient = {"sparse_gradient", (PyCFunction)__pyx_pw_4dipy_5align_13vector_fields_81sparse_gradient, METH_VARARGS|METH_KEYWORDS, __pyx_doc_4dipy_5align_13vector_fields_80sparse_gradient};
static PyObject *__pyx_pw_4dipy_5align_13vector_fields_81sparse_gradient(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_img = 0;
  PyObject *__pyx_v_img_world2grid = 0;
  PyObject *__pyx_v_img_spacing = 0;
  PyObject *__pyx_v_sample_points = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("sparse_gradient (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_img,&__pyx_n_s_img_world2grid,&__pyx_n_s_img_spacing,&__pyx_n_s_sample_points,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_world2grid)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("sparse_gradient", 1, 4, 4, 1); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_img_spacing)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("sparse_gradient", 1, 4, 4, 2); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_sample_points)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("sparse_gradient", 1, 4, 4, 3); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "sparse_gradient") < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_img = values[0];
    __pyx_v_img_world2grid = values[1];
    __pyx_v_img_spacing = values[2];
    __pyx_v_sample_points = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("sparse_gradient", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3420; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("dipy.align.vector_fields.sparse_gradient", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4dipy_5align_13vector_fields_80sparse_gradient(__pyx_self, __pyx_v_img, __pyx_v_img_world2grid, __pyx_v_img_spacing, __pyx_v_sample_points);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4dipy_5align_13vector_fields_80sparse_gradient(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_img, PyObject *__pyx_v_img_world2grid, PyObject *__pyx_v_img_spacing, PyObject *__pyx_v_sample_points) {
  Py_ssize_t __pyx_v_dim;
  PyObject *__pyx_v_ftype = NULL;
  PyObject *__pyx_v_n = NULL;
  PyObject *__pyx_v_out = NULL;
  PyObject *__pyx_v_inside = NULL;
  PyObject *__pyx_v_jd_grad = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("sparse_gradient", 0);

  /* "dipy/align/vector_fields.pyx":3441
 *         the gradient at each point stored at its corresponding row
 *     """
 *     dim = len(img.shape)             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(img_world2grid, dim):
 *         raise ValueError("Invalid affine transform matrix")
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_img, __pyx_n_s_shape); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3441; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_Length(__pyx_t_1); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3441; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_dim = __pyx_t_2;

  /* "dipy/align/vector_fields.pyx":3442
 *     """
 *     dim = len(img.shape)
 *     if not is_valid_affine(img_world2grid, dim):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 *     if len(img_spacing) < dim:
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_is_valid_affine); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  __pyx_t_2 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_2 = 1;
    }
  }
  __pyx_t_6 = PyTuple_New(2+__pyx_t_2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_5) {
    __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
  }
  __Pyx_INCREF(__pyx_v_img_world2grid);
  __Pyx_GIVEREF(__pyx_v_img_world2grid);
  PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_2, __pyx_v_img_world2grid);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_2, __pyx_t_4);
  __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_7 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3442; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = ((!__pyx_t_7) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":3443
 *     dim = len(img.shape)
 *     if not is_valid_affine(img_world2grid, dim):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 *     if len(img_spacing) < dim:
 *         raise ValueError("Invalid spacings")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__375, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3443; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3443; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":3442
 *     """
 *     dim = len(img.shape)
 *     if not is_valid_affine(img_world2grid, dim):             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid affine transform matrix")
 *     if len(img_spacing) < dim:
 */
  }

  /* "dipy/align/vector_fields.pyx":3444
 *     if not is_valid_affine(img_world2grid, dim):
 *         raise ValueError("Invalid affine transform matrix")
 *     if len(img_spacing) < dim:             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid spacings")
 * 
 */
  __pyx_t_2 = PyObject_Length(__pyx_v_img_spacing); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3444; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_8 = ((__pyx_t_2 < __pyx_v_dim) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":3445
 *         raise ValueError("Invalid affine transform matrix")
 *     if len(img_spacing) < dim:
 *         raise ValueError("Invalid spacings")             # <<<<<<<<<<<<<<
 * 
 *     ftype = img.dtype.type
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__376, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3445; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3445; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "dipy/align/vector_fields.pyx":3444
 *     if not is_valid_affine(img_world2grid, dim):
 *         raise ValueError("Invalid affine transform matrix")
 *     if len(img_spacing) < dim:             # <<<<<<<<<<<<<<
 *         raise ValueError("Invalid spacings")
 * 
 */
  }

  /* "dipy/align/vector_fields.pyx":3447
 *         raise ValueError("Invalid spacings")
 * 
 *     ftype = img.dtype.type             # <<<<<<<<<<<<<<
 *     n = sample_points.shape[0]
 *     out = np.empty(shape=(n, dim), dtype=ftype)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_img, __pyx_n_s_dtype); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3447; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_type); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3447; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ftype = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "dipy/align/vector_fields.pyx":3448
 * 
 *     ftype = img.dtype.type
 *     n = sample_points.shape[0]             # <<<<<<<<<<<<<<
 *     out = np.empty(shape=(n, dim), dtype=ftype)
 *     inside = np.empty(shape=(n,), dtype=np.int32)
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_sample_points, __pyx_n_s_shape); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3448; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_t_3, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3448; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_n = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "dipy/align/vector_fields.pyx":3449
 *     ftype = img.dtype.type
 *     n = sample_points.shape[0]
 *     out = np.empty(shape=(n, dim), dtype=ftype)             # <<<<<<<<<<<<<<
 *     inside = np.empty(shape=(n,), dtype=np.int32)
 *     # Select joint density gradient 2D or 3D
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3449; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3449; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3449; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3449; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3449; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_n);
  __Pyx_GIVEREF(__pyx_v_n);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_n);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_6);
  __pyx_t_6 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_shape, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3449; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_dtype, __pyx_v_ftype) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3449; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3449; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_out = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":3450
 *     n = sample_points.shape[0]
 *     out = np.empty(shape=(n, dim), dtype=ftype)
 *     inside = np.empty(shape=(n,), dtype=np.int32)             # <<<<<<<<<<<<<<
 *     # Select joint density gradient 2D or 3D
 *     if dim == 2:
 */
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_empty); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_n);
  __Pyx_GIVEREF(__pyx_v_n);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_n);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_shape, __pyx_t_3) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_int32); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_dtype, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3450; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_inside = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":3452
 *     inside = np.empty(shape=(n,), dtype=np.int32)
 *     # Select joint density gradient 2D or 3D
 *     if dim == 2:             # <<<<<<<<<<<<<<
 *         jd_grad = _sparse_gradient_2d
 *     else:
 */
  __pyx_t_8 = ((__pyx_v_dim == 2) != 0);
  if (__pyx_t_8) {

    /* "dipy/align/vector_fields.pyx":3453
 *     # Select joint density gradient 2D or 3D
 *     if dim == 2:
 *         jd_grad = _sparse_gradient_2d             # <<<<<<<<<<<<<<
 *     else:
 *         jd_grad = _sparse_gradient_3d
 */
    __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_sparse_gradient_2d); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3453; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_jd_grad = __pyx_t_6;
    __pyx_t_6 = 0;

    /* "dipy/align/vector_fields.pyx":3452
 *     inside = np.empty(shape=(n,), dtype=np.int32)
 *     # Select joint density gradient 2D or 3D
 *     if dim == 2:             # <<<<<<<<<<<<<<
 *         jd_grad = _sparse_gradient_2d
 *     else:
 */
    goto __pyx_L5;
  }

  /* "dipy/align/vector_fields.pyx":3455
 *         jd_grad = _sparse_gradient_2d
 *     else:
 *         jd_grad = _sparse_gradient_3d             # <<<<<<<<<<<<<<
 *     jd_grad(img, img_world2grid.astype(np.float64),
 *             img_spacing.astype(np.float64), sample_points, out, inside)
 */
  /*else*/ {
    __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_sparse_gradient_3d); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3455; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_jd_grad = __pyx_t_6;
    __pyx_t_6 = 0;
  }
  __pyx_L5:;

  /* "dipy/align/vector_fields.pyx":3456
 *     else:
 *         jd_grad = _sparse_gradient_3d
 *     jd_grad(img, img_world2grid.astype(np.float64),             # <<<<<<<<<<<<<<
 *             img_spacing.astype(np.float64), sample_points, out, inside)
 *     return out, inside
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_img_world2grid, __pyx_n_s_astype); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3456; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3456; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_float64); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3456; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3456; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_4);
  } else {
    __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3456; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_3); __pyx_t_3 = NULL;
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_9, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3456; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "dipy/align/vector_fields.pyx":3457
 *         jd_grad = _sparse_gradient_3d
 *     jd_grad(img, img_world2grid.astype(np.float64),
 *             img_spacing.astype(np.float64), sample_points, out, inside)             # <<<<<<<<<<<<<<
 *     return out, inside
 */
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_img_spacing, __pyx_n_s_astype); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3457; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_np); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3457; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_float64); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3457; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  if (!__pyx_t_5) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_3); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3457; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_10 = PyTuple_New(1+1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3457; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_5); __pyx_t_5 = NULL;
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_10, 0+1, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_10, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3457; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_INCREF(__pyx_v_jd_grad);
  __pyx_t_9 = __pyx_v_jd_grad; __pyx_t_10 = NULL;
  __pyx_t_2 = 0;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
      __pyx_t_2 = 1;
    }
  }
  __pyx_t_3 = PyTuple_New(6+__pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3456; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (__pyx_t_10) {
    __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_10); __pyx_t_10 = NULL;
  }
  __Pyx_INCREF(__pyx_v_img);
  __Pyx_GIVEREF(__pyx_v_img);
  PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_2, __pyx_v_img);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_2, __pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_2, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_sample_points);
  __Pyx_GIVEREF(__pyx_v_sample_points);
  PyTuple_SET_ITEM(__pyx_t_3, 3+__pyx_t_2, __pyx_v_sample_points);
  __Pyx_INCREF(__pyx_v_out);
  __Pyx_GIVEREF(__pyx_v_out);
  PyTuple_SET_ITEM(__pyx_t_3, 4+__pyx_t_2, __pyx_v_out);
  __Pyx_INCREF(__pyx_v_inside);
  __Pyx_GIVEREF(__pyx_v_inside);
  PyTuple_SET_ITEM(__pyx_t_3, 5+__pyx_t_2, __pyx_v_inside);
  __pyx_t_4 = 0;
  __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_3, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3456; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":3458
 *     jd_grad(img, img_world2grid.astype(np.float64),
 *             img_spacing.astype(np.float64), sample_points, out, inside)
 *     return out, inside             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3458; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_out);
  __Pyx_GIVEREF(__pyx_v_out);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_out);
  __Pyx_INCREF(__pyx_v_inside);
  __Pyx_GIVEREF(__pyx_v_inside);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_v_inside);
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pyx":3420
 * 
 * 
 * def sparse_gradient(img, img_world2grid, img_spacing, sample_points):             # <<<<<<<<<<<<<<
 *     r""" Gradient of an image in physical space
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("dipy.align.vector_fields.sparse_gradient", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ftype);
  __Pyx_XDECREF(__pyx_v_n);
  __Pyx_XDECREF(__pyx_v_out);
  __Pyx_XDECREF(__pyx_v_inside);
  __Pyx_XDECREF(__pyx_v_jd_grad);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":197
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fullfill the PEP.
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_pw_5numpy_7ndarray_1__getbuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5numpy_7ndarray___getbuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5numpy_7ndarray___getbuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_v_copy_shape;
  int __pyx_v_i;
  int __pyx_v_ndim;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  int __pyx_v_t;
  char *__pyx_v_f;
  PyArray_Descr *__pyx_v_descr = 0;
  int __pyx_v_offset;
  int __pyx_v_hasfields;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  char *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  if (__pyx_v_info != NULL) {
    __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(__pyx_v_info->obj);
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":203
 *             # of flags
 * 
 *             if info == NULL: return             # <<<<<<<<<<<<<<
 * 
 *             cdef int copy_shape, i, ndim
 */
  __pyx_t_1 = ((__pyx_v_info == NULL) != 0);
  if (__pyx_t_1) {
    __pyx_r = 0;
    goto __pyx_L0;
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":206
 * 
 *             cdef int copy_shape, i, ndim
 *             cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":207
 *             cdef int copy_shape, i, ndim
 *             cdef int endian_detector = 1
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 * 
 *             ndim = PyArray_NDIM(self)
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":209
 *             cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 * 
 *             ndim = PyArray_NDIM(self)             # <<<<<<<<<<<<<<
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_v_ndim = PyArray_NDIM(__pyx_v_self);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":211
 *             ndim = PyArray_NDIM(self)
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 copy_shape = 1
 *             else:
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":212
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 copy_shape = 1             # <<<<<<<<<<<<<<
 *             else:
 *                 copy_shape = 0
 */
    __pyx_v_copy_shape = 1;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":211
 *             ndim = PyArray_NDIM(self)
 * 
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 copy_shape = 1
 *             else:
 */
    goto __pyx_L4;
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":214
 *                 copy_shape = 1
 *             else:
 *                 copy_shape = 0             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 */
  /*else*/ {
    __pyx_v_copy_shape = 0;
  }
  __pyx_L4:;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_C_CONTIGUOUS) == PyBUF_C_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L6_bool_binop_done;
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":217
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_C_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L6_bool_binop_done:;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":218
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__377, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[1]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":216
 *                 copy_shape = 0
 * 
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")
 */
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  __pyx_t_2 = (((__pyx_v_flags & PyBUF_F_CONTIGUOUS) == PyBUF_F_CONTIGUOUS) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L9_bool_binop_done;
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":221
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):             # <<<<<<<<<<<<<<
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 */
  __pyx_t_2 = ((!(PyArray_CHKFLAGS(__pyx_v_self, NPY_F_CONTIGUOUS) != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L9_bool_binop_done:;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":222
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__378, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[1]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":220
 *                 raise ValueError(u"ndarray is not C contiguous")
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)             # <<<<<<<<<<<<<<
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 */
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":224
 *                 raise ValueError(u"ndarray is not Fortran contiguous")
 * 
 *             info.buf = PyArray_DATA(self)             # <<<<<<<<<<<<<<
 *             info.ndim = ndim
 *             if copy_shape:
 */
  __pyx_v_info->buf = PyArray_DATA(__pyx_v_self);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":225
 * 
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim             # <<<<<<<<<<<<<<
 *             if copy_shape:
 *                 # Allocate new buffer for strides and shape info.
 */
  __pyx_v_info->ndim = __pyx_v_ndim;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":226
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if copy_shape:             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
  __pyx_t_1 = (__pyx_v_copy_shape != 0);
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":229
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)             # <<<<<<<<<<<<<<
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 */
    __pyx_v_info->strides = ((Py_ssize_t *)malloc((((sizeof(Py_ssize_t)) * ((size_t)__pyx_v_ndim)) * 2)));

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":230
 *                 # This is allocated as one block, strides first.
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)
 *                 info.shape = info.strides + ndim             # <<<<<<<<<<<<<<
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 */
    __pyx_v_info->shape = (__pyx_v_info->strides + __pyx_v_ndim);

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":231
 *                 info.strides = <Py_ssize_t*>stdlib.malloc(sizeof(Py_ssize_t) * <size_t>ndim * 2)
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):             # <<<<<<<<<<<<<<
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 */
    __pyx_t_4 = __pyx_v_ndim;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_i = __pyx_t_5;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":232
 *                 info.shape = info.strides + ndim
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]             # <<<<<<<<<<<<<<
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 */
      (__pyx_v_info->strides[__pyx_v_i]) = (PyArray_STRIDES(__pyx_v_self)[__pyx_v_i]);

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":233
 *                 for i in range(ndim):
 *                     info.strides[i] = PyArray_STRIDES(self)[i]
 *                     info.shape[i] = PyArray_DIMS(self)[i]             # <<<<<<<<<<<<<<
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 */
      (__pyx_v_info->shape[__pyx_v_i]) = (PyArray_DIMS(__pyx_v_self)[__pyx_v_i]);
    }

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":226
 *             info.buf = PyArray_DATA(self)
 *             info.ndim = ndim
 *             if copy_shape:             # <<<<<<<<<<<<<<
 *                 # Allocate new buffer for strides and shape info.
 *                 # This is allocated as one block, strides first.
 */
    goto __pyx_L11;
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":235
 *                     info.shape[i] = PyArray_DIMS(self)[i]
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)             # <<<<<<<<<<<<<<
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 */
  /*else*/ {
    __pyx_v_info->strides = ((Py_ssize_t *)PyArray_STRIDES(__pyx_v_self));

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":236
 *             else:
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)             # <<<<<<<<<<<<<<
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 */
    __pyx_v_info->shape = ((Py_ssize_t *)PyArray_DIMS(__pyx_v_self));
  }
  __pyx_L11:;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":237
 *                 info.strides = <Py_ssize_t*>PyArray_STRIDES(self)
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 */
  __pyx_v_info->suboffsets = NULL;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":238
 *                 info.shape = <Py_ssize_t*>PyArray_DIMS(self)
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)             # <<<<<<<<<<<<<<
 *             info.readonly = not PyArray_ISWRITEABLE(self)
 * 
 */
  __pyx_v_info->itemsize = PyArray_ITEMSIZE(__pyx_v_self);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":239
 *             info.suboffsets = NULL
 *             info.itemsize = PyArray_ITEMSIZE(self)
 *             info.readonly = not PyArray_ISWRITEABLE(self)             # <<<<<<<<<<<<<<
 * 
 *             cdef int t
 */
  __pyx_v_info->readonly = (!(PyArray_ISWRITEABLE(__pyx_v_self) != 0));

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":242
 * 
 *             cdef int t
 *             cdef char* f = NULL             # <<<<<<<<<<<<<<
 *             cdef dtype descr = self.descr
 *             cdef int offset
 */
  __pyx_v_f = NULL;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":243
 *             cdef int t
 *             cdef char* f = NULL
 *             cdef dtype descr = self.descr             # <<<<<<<<<<<<<<
 *             cdef int offset
 * 
 */
  __pyx_t_3 = ((PyObject *)__pyx_v_self->descr);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_v_descr = ((PyArray_Descr *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":246
 *             cdef int offset
 * 
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)             # <<<<<<<<<<<<<<
 * 
 *             if not hasfields and not copy_shape:
 */
  __pyx_v_hasfields = PyDataType_HASFIELDS(__pyx_v_descr);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":248
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)
 * 
 *             if not hasfields and not copy_shape:             # <<<<<<<<<<<<<<
 *                 # do not call releasebuffer
 *                 info.obj = None
 */
  __pyx_t_2 = ((!(__pyx_v_hasfields != 0)) != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_v_copy_shape != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L15_bool_binop_done:;
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":250
 *             if not hasfields and not copy_shape:
 *                 # do not call releasebuffer
 *                 info.obj = None             # <<<<<<<<<<<<<<
 *             else:
 *                 # need to call releasebuffer
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj);
    __pyx_v_info->obj = Py_None;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":248
 *             cdef bint hasfields = PyDataType_HASFIELDS(descr)
 * 
 *             if not hasfields and not copy_shape:             # <<<<<<<<<<<<<<
 *                 # do not call releasebuffer
 *                 info.obj = None
 */
    goto __pyx_L14;
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":253
 *             else:
 *                 # need to call releasebuffer
 *                 info.obj = self             # <<<<<<<<<<<<<<
 * 
 *             if not hasfields:
 */
  /*else*/ {
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj);
    __pyx_v_info->obj = ((PyObject *)__pyx_v_self);
  }
  __pyx_L14:;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":255
 *                 info.obj = self
 * 
 *             if not hasfields:             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  __pyx_t_1 = ((!(__pyx_v_hasfields != 0)) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":256
 * 
 *             if not hasfields:
 *                 t = descr.type_num             # <<<<<<<<<<<<<<
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 */
    __pyx_t_4 = __pyx_v_descr->type_num;
    __pyx_v_t = __pyx_t_4;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '>') != 0);
    if (!__pyx_t_2) {
      goto __pyx_L20_next_or;
    } else {
    }
    __pyx_t_2 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_L20_next_or:;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":258
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 */
    __pyx_t_2 = ((__pyx_v_descr->byteorder == '<') != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_t_2 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_1 = __pyx_t_2;
    __pyx_L19_bool_binop_done:;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    if (__pyx_t_1) {

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":259
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__379, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":257
 *             if not hasfields:
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":260
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 */
    switch (__pyx_v_t) {
      case NPY_BYTE:
      __pyx_v_f = __pyx_k_b;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":261
 *                     raise ValueError(u"Non-native byte order not supported")
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 */
      case NPY_UBYTE:
      __pyx_v_f = __pyx_k_B;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":262
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 */
      case NPY_SHORT:
      __pyx_v_f = __pyx_k_h;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":263
 *                 elif t == NPY_UBYTE:       f = "B"
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 */
      case NPY_USHORT:
      __pyx_v_f = __pyx_k_H;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":264
 *                 elif t == NPY_SHORT:       f = "h"
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 */
      case NPY_INT:
      __pyx_v_f = __pyx_k_i;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":265
 *                 elif t == NPY_USHORT:      f = "H"
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 */
      case NPY_UINT:
      __pyx_v_f = __pyx_k_I;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":266
 *                 elif t == NPY_INT:         f = "i"
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 */
      case NPY_LONG:
      __pyx_v_f = __pyx_k_l;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":267
 *                 elif t == NPY_UINT:        f = "I"
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 */
      case NPY_ULONG:
      __pyx_v_f = __pyx_k_L;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":268
 *                 elif t == NPY_LONG:        f = "l"
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 */
      case NPY_LONGLONG:
      __pyx_v_f = __pyx_k_q;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":269
 *                 elif t == NPY_ULONG:       f = "L"
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 */
      case NPY_ULONGLONG:
      __pyx_v_f = __pyx_k_Q;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":270
 *                 elif t == NPY_LONGLONG:    f = "q"
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 */
      case NPY_FLOAT:
      __pyx_v_f = __pyx_k_f;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":271
 *                 elif t == NPY_ULONGLONG:   f = "Q"
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 */
      case NPY_DOUBLE:
      __pyx_v_f = __pyx_k_d;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":272
 *                 elif t == NPY_FLOAT:       f = "f"
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 */
      case NPY_LONGDOUBLE:
      __pyx_v_f = __pyx_k_g;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":273
 *                 elif t == NPY_DOUBLE:      f = "d"
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 */
      case NPY_CFLOAT:
      __pyx_v_f = __pyx_k_Zf;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":274
 *                 elif t == NPY_LONGDOUBLE:  f = "g"
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"
 */
      case NPY_CDOUBLE:
      __pyx_v_f = __pyx_k_Zd;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":275
 *                 elif t == NPY_CFLOAT:      f = "Zf"
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"             # <<<<<<<<<<<<<<
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 */
      case NPY_CLONGDOUBLE:
      __pyx_v_f = __pyx_k_Zg;
      break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":276
 *                 elif t == NPY_CDOUBLE:     f = "Zd"
 *                 elif t == NPY_CLONGDOUBLE: f = "Zg"
 *                 elif t == NPY_OBJECT:      f = "O"             # <<<<<<<<<<<<<<
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      case NPY_OBJECT:
      __pyx_v_f = __pyx_k_O;
      break;
      default:

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":278
 *                 elif t == NPY_OBJECT:      f = "O"
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *                 info.format = f
 *                 return
 */
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_t); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_t_3); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_3, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      break;
    }

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":279
 *                 else:
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f             # <<<<<<<<<<<<<<
 *                 return
 *             else:
 */
    __pyx_v_info->format = __pyx_v_f;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":280
 *                     raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *                 info.format = f
 *                 return             # <<<<<<<<<<<<<<
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":255
 *                 info.obj = self
 * 
 *             if not hasfields:             # <<<<<<<<<<<<<<
 *                 t = descr.type_num
 *                 if ((descr.byteorder == c'>' and little_endian) or
 */
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":282
 *                 return
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)             # <<<<<<<<<<<<<<
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 */
  /*else*/ {
    __pyx_v_info->format = ((char *)malloc(0xFF));

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":283
 *             else:
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment             # <<<<<<<<<<<<<<
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,
 */
    (__pyx_v_info->format[0]) = '^';

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":284
 *                 info.format = <char*>stdlib.malloc(_buffer_format_string_len)
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0             # <<<<<<<<<<<<<<
 *                 f = _util_dtypestring(descr, info.format + 1,
 *                                       info.format + _buffer_format_string_len,
 */
    __pyx_v_offset = 0;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":285
 *                 info.format[0] = c'^' # Native data types, manual alignment
 *                 offset = 0
 *                 f = _util_dtypestring(descr, info.format + 1,             # <<<<<<<<<<<<<<
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 */
    __pyx_t_7 = __pyx_f_5numpy__util_dtypestring(__pyx_v_descr, (__pyx_v_info->format + 1), (__pyx_v_info->format + 0xFF), (&__pyx_v_offset)); if (unlikely(__pyx_t_7 == NULL)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 285; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_f = __pyx_t_7;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":288
 *                                       info.format + _buffer_format_string_len,
 *                                       &offset)
 *                 f[0] = c'\0' # Terminate format string             # <<<<<<<<<<<<<<
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 */
    (__pyx_v_f[0]) = '\x00';
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":197
 *         # experimental exception made for __getbuffer__ and __releasebuffer__
 *         # -- the details of this may change.
 *         def __getbuffer__(ndarray self, Py_buffer* info, int flags):             # <<<<<<<<<<<<<<
 *             # This implementation of getbuffer is geared towards Cython
 *             # requirements, and does not yet fullfill the PEP.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("numpy.ndarray.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info != NULL && __pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = NULL;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info != NULL && __pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(Py_None);
    __Pyx_DECREF(Py_None); __pyx_v_info->obj = NULL;
  }
  __pyx_L2:;
  __Pyx_XDECREF((PyObject *)__pyx_v_descr);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":290
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 */

/* Python wrapper */
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info); /*proto*/
static CYTHON_UNUSED void __pyx_pw_5numpy_7ndarray_3__releasebuffer__(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__releasebuffer__ (wrapper)", 0);
  __pyx_pf_5numpy_7ndarray_2__releasebuffer__(((PyArrayObject *)__pyx_v_self), ((Py_buffer *)__pyx_v_info));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_5numpy_7ndarray_2__releasebuffer__(PyArrayObject *__pyx_v_self, Py_buffer *__pyx_v_info) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__releasebuffer__", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":291
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  __pyx_t_1 = (PyArray_HASFIELDS(__pyx_v_self) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":292
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)             # <<<<<<<<<<<<<<
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 stdlib.free(info.strides)
 */
    free(__pyx_v_info->format);

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":291
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):
 *             if PyArray_HASFIELDS(self):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 */
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":293
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  __pyx_t_1 = (((sizeof(npy_intp)) != (sizeof(Py_ssize_t))) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":294
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):
 *                 stdlib.free(info.strides)             # <<<<<<<<<<<<<<
 *                 # info.shape was stored after info.strides in the same block
 * 
 */
    free(__pyx_v_info->strides);

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":293
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 *             if sizeof(npy_intp) != sizeof(Py_ssize_t):             # <<<<<<<<<<<<<<
 *                 stdlib.free(info.strides)
 *                 # info.shape was stored after info.strides in the same block
 */
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":290
 *                 f[0] = c'\0' # Terminate format string
 * 
 *         def __releasebuffer__(ndarray self, Py_buffer* info):             # <<<<<<<<<<<<<<
 *             if PyArray_HASFIELDS(self):
 *                 stdlib.free(info.format)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":770
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew1(PyObject *__pyx_v_a) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew1", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":771
 * 
 * cdef inline object PyArray_MultiIterNew1(a):
 *     return PyArray_MultiIterNew(1, <void*>a)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(1, ((void *)__pyx_v_a)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 771; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":770
 * ctypedef npy_cdouble     complex_t
 * 
 * cdef inline object PyArray_MultiIterNew1(a):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew1", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":773
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew2(PyObject *__pyx_v_a, PyObject *__pyx_v_b) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew2", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":774
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(2, ((void *)__pyx_v_a), ((void *)__pyx_v_b)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 774; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":773
 *     return PyArray_MultiIterNew(1, <void*>a)
 * 
 * cdef inline object PyArray_MultiIterNew2(a, b):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":776
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew3(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew3", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":777
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(3, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 777; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":776
 *     return PyArray_MultiIterNew(2, <void*>a, <void*>b)
 * 
 * cdef inline object PyArray_MultiIterNew3(a, b, c):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":779
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew4(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew4", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":780
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)             # <<<<<<<<<<<<<<
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(4, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 780; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":779
 *     return PyArray_MultiIterNew(3, <void*>a, <void*>b, <void*> c)
 * 
 * cdef inline object PyArray_MultiIterNew4(a, b, c, d):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":782
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_PyArray_MultiIterNew5(PyObject *__pyx_v_a, PyObject *__pyx_v_b, PyObject *__pyx_v_c, PyObject *__pyx_v_d, PyObject *__pyx_v_e) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("PyArray_MultiIterNew5", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":783
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)             # <<<<<<<<<<<<<<
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyArray_MultiIterNew(5, ((void *)__pyx_v_a), ((void *)__pyx_v_b), ((void *)__pyx_v_c), ((void *)__pyx_v_d), ((void *)__pyx_v_e)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 783; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":782
 *     return PyArray_MultiIterNew(4, <void*>a, <void*>b, <void*>c, <void*> d)
 * 
 * cdef inline object PyArray_MultiIterNew5(a, b, c, d, e):             # <<<<<<<<<<<<<<
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("numpy.PyArray_MultiIterNew5", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":785
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

static CYTHON_INLINE char *__pyx_f_5numpy__util_dtypestring(PyArray_Descr *__pyx_v_descr, char *__pyx_v_f, char *__pyx_v_end, int *__pyx_v_offset) {
  PyArray_Descr *__pyx_v_child = 0;
  int __pyx_v_endian_detector;
  int __pyx_v_little_endian;
  PyObject *__pyx_v_fields = 0;
  PyObject *__pyx_v_childname = NULL;
  PyObject *__pyx_v_new_offset = NULL;
  PyObject *__pyx_v_t = NULL;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  long __pyx_t_8;
  char *__pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_util_dtypestring", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":790
 * 
 *     cdef dtype child
 *     cdef int endian_detector = 1             # <<<<<<<<<<<<<<
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)
 *     cdef tuple fields
 */
  __pyx_v_endian_detector = 1;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":791
 *     cdef dtype child
 *     cdef int endian_detector = 1
 *     cdef bint little_endian = ((<char*>&endian_detector)[0] != 0)             # <<<<<<<<<<<<<<
 *     cdef tuple fields
 * 
 */
  __pyx_v_little_endian = ((((char *)(&__pyx_v_endian_detector))[0]) != 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":794
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  if (unlikely(__pyx_v_descr->names == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    {__pyx_filename = __pyx_f[1]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = __pyx_v_descr->names; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyTuple_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 794; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_childname, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":795
 * 
 *     for childname in descr.names:
 *         fields = descr.fields[childname]             # <<<<<<<<<<<<<<
 *         child, new_offset = fields
 * 
 */
    if (unlikely(__pyx_v_descr->fields == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_descr->fields, __pyx_v_childname); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyTuple_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_t_3)->tp_name), 0))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_fields, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":796
 *     for childname in descr.names:
 *         fields = descr.fields[childname]
 *         child, new_offset = fields             # <<<<<<<<<<<<<<
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 */
    if (likely(__pyx_v_fields != Py_None)) {
      PyObject* sequence = __pyx_v_fields;
      #if CYTHON_COMPILING_IN_CPYTHON
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      #if CYTHON_COMPILING_IN_CPYTHON
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1); 
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      #else
      __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      #endif
    } else {
      __Pyx_RaiseNoneNotIterableError(); {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5numpy_dtype))))) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_XDECREF_SET(__pyx_v_child, ((PyArray_Descr *)__pyx_t_3));
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_new_offset, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":798
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    __pyx_t_4 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = PyNumber_Subtract(__pyx_v_new_offset, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 798; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = ((((__pyx_v_end - __pyx_v_f) - ((int)__pyx_t_5)) < 15) != 0);
    if (__pyx_t_6) {

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":799
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__380, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":798
 *         child, new_offset = fields
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 */
    }

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '>') != 0);
    if (!__pyx_t_7) {
      goto __pyx_L8_next_or;
    } else {
    }
    __pyx_t_7 = (__pyx_v_little_endian != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_L8_next_or:;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":802
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):             # <<<<<<<<<<<<<<
 *             raise ValueError(u"Non-native byte order not supported")
 *             # One could encode it in the format string and have Cython
 */
    __pyx_t_7 = ((__pyx_v_child->byteorder == '<') != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_7 = ((!(__pyx_v_little_endian != 0)) != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L7_bool_binop_done:;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    if (__pyx_t_6) {

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":803
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__381, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      {__pyx_filename = __pyx_f[1]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":801
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")
 * 
 *         if ((child.byteorder == c'>' and little_endian) or             # <<<<<<<<<<<<<<
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")
 */
    }

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":813
 * 
 *         # Output padding bytes
 *         while offset[0] < new_offset:             # <<<<<<<<<<<<<<
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 */
    while (1) {
      __pyx_t_3 = __Pyx_PyInt_From_int((__pyx_v_offset[0])); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_v_new_offset, Py_LT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 813; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (!__pyx_t_6) break;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":814
 *         # Output padding bytes
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte             # <<<<<<<<<<<<<<
 *             f += 1
 *             offset[0] += 1
 */
      (__pyx_v_f[0]) = 0x78;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":815
 *         while offset[0] < new_offset:
 *             f[0] = 120 # "x"; pad byte
 *             f += 1             # <<<<<<<<<<<<<<
 *             offset[0] += 1
 * 
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":816
 *             f[0] = 120 # "x"; pad byte
 *             f += 1
 *             offset[0] += 1             # <<<<<<<<<<<<<<
 * 
 *         offset[0] += child.itemsize
 */
      __pyx_t_8 = 0;
      (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + 1);
    }

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":818
 *             offset[0] += 1
 * 
 *         offset[0] += child.itemsize             # <<<<<<<<<<<<<<
 * 
 *         if not PyDataType_HASFIELDS(child):
 */
    __pyx_t_8 = 0;
    (__pyx_v_offset[__pyx_t_8]) = ((__pyx_v_offset[__pyx_t_8]) + __pyx_v_child->elsize);

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":820
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
    __pyx_t_6 = ((!(PyDataType_HASFIELDS(__pyx_v_child) != 0)) != 0);
    if (__pyx_t_6) {

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":821
 * 
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num             # <<<<<<<<<<<<<<
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")
 */
      __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_child->type_num); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 821; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_XDECREF_SET(__pyx_v_t, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":822
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      __pyx_t_6 = (((__pyx_v_end - __pyx_v_f) < 5) != 0);
      if (__pyx_t_6) {

        /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":823
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__382, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_Raise(__pyx_t_4, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        {__pyx_filename = __pyx_f[1]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

        /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":822
 *         if not PyDataType_HASFIELDS(child):
 *             t = child.type_num
 *             if end - f < 5:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError(u"Format string allocated too short.")
 * 
 */
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":826
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_BYTE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 826; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 98;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":827
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"             # <<<<<<<<<<<<<<
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UBYTE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 827; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 66;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":828
 *             if   t == NPY_BYTE:        f[0] =  98 #"b"
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"             # <<<<<<<<<<<<<<
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_SHORT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 828; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x68;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":829
 *             elif t == NPY_UBYTE:       f[0] =  66 #"B"
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"             # <<<<<<<<<<<<<<
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_USHORT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 829; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 72;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":830
 *             elif t == NPY_SHORT:       f[0] = 104 #"h"
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"             # <<<<<<<<<<<<<<
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_INT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 830; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x69;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":831
 *             elif t == NPY_USHORT:      f[0] =  72 #"H"
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_UINT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 831; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 73;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":832
 *             elif t == NPY_INT:         f[0] = 105 #"i"
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONG); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 832; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x6C;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":833
 *             elif t == NPY_UINT:        f[0] =  73 #"I"
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONG); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 833; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 76;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":834
 *             elif t == NPY_LONG:        f[0] = 108 #"l"
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGLONG); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 834; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x71;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":835
 *             elif t == NPY_ULONG:       f[0] = 76  #"L"
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"             # <<<<<<<<<<<<<<
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_ULONGLONG); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 835; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 81;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":836
 *             elif t == NPY_LONGLONG:    f[0] = 113 #"q"
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"             # <<<<<<<<<<<<<<
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_FLOAT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 836; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x66;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":837
 *             elif t == NPY_ULONGLONG:   f[0] = 81  #"Q"
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"             # <<<<<<<<<<<<<<
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_DOUBLE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 837; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x64;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":838
 *             elif t == NPY_FLOAT:       f[0] = 102 #"f"
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"             # <<<<<<<<<<<<<<
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_LONGDOUBLE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 838; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 0x67;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":839
 *             elif t == NPY_DOUBLE:      f[0] = 100 #"d"
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf             # <<<<<<<<<<<<<<
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CFLOAT); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 839; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x66;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":840
 *             elif t == NPY_LONGDOUBLE:  f[0] = 103 #"g"
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd             # <<<<<<<<<<<<<<
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CDOUBLE); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 840; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x64;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":841
 *             elif t == NPY_CFLOAT:      f[0] = 90; f[1] = 102; f += 1 # Zf
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg             # <<<<<<<<<<<<<<
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 */
      __pyx_t_3 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_CLONGDOUBLE); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyObject_RichCompare(__pyx_v_t, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 841; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 90;
        (__pyx_v_f[1]) = 0x67;
        __pyx_v_f = (__pyx_v_f + 1);
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":842
 *             elif t == NPY_CDOUBLE:     f[0] = 90; f[1] = 100; f += 1 # Zd
 *             elif t == NPY_CLONGDOUBLE: f[0] = 90; f[1] = 103; f += 1 # Zg
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 */
      __pyx_t_4 = __Pyx_PyInt_From_enum__NPY_TYPES(NPY_OBJECT); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = PyObject_RichCompare(__pyx_v_t, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 842; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (__pyx_t_6) {
        (__pyx_v_f[0]) = 79;
        goto __pyx_L15;
      }

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":844
 *             elif t == NPY_OBJECT:      f[0] = 79 #"O"
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)             # <<<<<<<<<<<<<<
 *             f += 1
 *         else:
 */
      /*else*/ {
        __pyx_t_3 = PyUnicode_Format(__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_v_t); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        {__pyx_filename = __pyx_f[1]; __pyx_lineno = 844; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_L15:;

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":845
 *             else:
 *                 raise ValueError(u"unknown dtype code in numpy.pxd (%d)" % t)
 *             f += 1             # <<<<<<<<<<<<<<
 *         else:
 *             # Cython ignores struct boundary information ("T{...}"),
 */
      __pyx_v_f = (__pyx_v_f + 1);

      /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":820
 *         offset[0] += child.itemsize
 * 
 *         if not PyDataType_HASFIELDS(child):             # <<<<<<<<<<<<<<
 *             t = child.type_num
 *             if end - f < 5:
 */
      goto __pyx_L13;
    }

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":849
 *             # Cython ignores struct boundary information ("T{...}"),
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)             # <<<<<<<<<<<<<<
 *     return f
 * 
 */
    /*else*/ {
      __pyx_t_9 = __pyx_f_5numpy__util_dtypestring(__pyx_v_child, __pyx_v_f, __pyx_v_end, __pyx_v_offset); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 849; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_f = __pyx_t_9;
    }
    __pyx_L13:;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":794
 *     cdef tuple fields
 * 
 *     for childname in descr.names:             # <<<<<<<<<<<<<<
 *         fields = descr.fields[childname]
 *         child, new_offset = fields
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":850
 *             # so don't output it
 *             f = _util_dtypestring(child, f, end, offset)
 *     return f             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_f;
  goto __pyx_L0;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":785
 *     return PyArray_MultiIterNew(5, <void*>a, <void*>b, <void*>c, <void*> d, <void*> e)
 * 
 * cdef inline char* _util_dtypestring(dtype descr, char* f, char* end, int* offset) except NULL:             # <<<<<<<<<<<<<<
 *     # Recursive utility function used in __getbuffer__ to get format
 *     # string. The new location in the format string is returned.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("numpy._util_dtypestring", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_child);
  __Pyx_XDECREF(__pyx_v_fields);
  __Pyx_XDECREF(__pyx_v_childname);
  __Pyx_XDECREF(__pyx_v_new_offset);
  __Pyx_XDECREF(__pyx_v_t);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":966
 * 
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *      cdef PyObject* baseptr
 *      if base is None:
 */

static CYTHON_INLINE void __pyx_f_5numpy_set_array_base(PyArrayObject *__pyx_v_arr, PyObject *__pyx_v_base) {
  PyObject *__pyx_v_baseptr;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("set_array_base", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":968
 * cdef inline void set_array_base(ndarray arr, object base):
 *      cdef PyObject* baseptr
 *      if base is None:             # <<<<<<<<<<<<<<
 *          baseptr = NULL
 *      else:
 */
  __pyx_t_1 = (__pyx_v_base == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":969
 *      cdef PyObject* baseptr
 *      if base is None:
 *          baseptr = NULL             # <<<<<<<<<<<<<<
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!
 */
    __pyx_v_baseptr = NULL;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":968
 * cdef inline void set_array_base(ndarray arr, object base):
 *      cdef PyObject* baseptr
 *      if base is None:             # <<<<<<<<<<<<<<
 *          baseptr = NULL
 *      else:
 */
    goto __pyx_L3;
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":971
 *          baseptr = NULL
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!             # <<<<<<<<<<<<<<
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)
 */
  /*else*/ {
    Py_INCREF(__pyx_v_base);

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":972
 *      else:
 *          Py_INCREF(base) # important to do this before decref below!
 *          baseptr = <PyObject*>base             # <<<<<<<<<<<<<<
 *      Py_XDECREF(arr.base)
 *      arr.base = baseptr
 */
    __pyx_v_baseptr = ((PyObject *)__pyx_v_base);
  }
  __pyx_L3:;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":973
 *          Py_INCREF(base) # important to do this before decref below!
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)             # <<<<<<<<<<<<<<
 *      arr.base = baseptr
 * 
 */
  Py_XDECREF(__pyx_v_arr->base);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":974
 *          baseptr = <PyObject*>base
 *      Py_XDECREF(arr.base)
 *      arr.base = baseptr             # <<<<<<<<<<<<<<
 * 
 * cdef inline object get_array_base(ndarray arr):
 */
  __pyx_v_arr->base = __pyx_v_baseptr;

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":966
 * 
 * 
 * cdef inline void set_array_base(ndarray arr, object base):             # <<<<<<<<<<<<<<
 *      cdef PyObject* baseptr
 *      if base is None:
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

static CYTHON_INLINE PyObject *__pyx_f_5numpy_get_array_base(PyArrayObject *__pyx_v_arr) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("get_array_base", 0);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":977
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_arr->base == NULL) != 0);
  if (__pyx_t_1) {

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":978
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:
 *         return None             # <<<<<<<<<<<<<<
 *     else:
 *         return <object>arr.base
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_None);
    __pyx_r = Py_None;
    goto __pyx_L0;

    /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":977
 * 
 * cdef inline object get_array_base(ndarray arr):
 *     if arr.base is NULL:             # <<<<<<<<<<<<<<
 *         return None
 *     else:
 */
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":980
 *         return None
 *     else:
 *         return <object>arr.base             # <<<<<<<<<<<<<<
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_arr->base));
    __pyx_r = ((PyObject *)__pyx_v_arr->base);
    goto __pyx_L0;
  }

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":976
 *      arr.base = baseptr
 * 
 * cdef inline object get_array_base(ndarray arr):             # <<<<<<<<<<<<<<
 *     if arr.base is NULL:
 *         return None
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "dipy/align/vector_fields.pxd":6
 * #cython: cdivision=True
 * 
 * cdef inline double _apply_affine_3d_x0(double x0, double x1, double x2,             # <<<<<<<<<<<<<<
 *                                        double h, double[:, :] aff) nogil:
 *     r"""Multiplies aff by (x0, x1, x2, h), returns the 1st element of product
 */

static CYTHON_INLINE double __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x0(double __pyx_v_x0, double __pyx_v_x1, double __pyx_v_x2, double __pyx_v_h, __Pyx_memviewslice __pyx_v_aff) {
  double __pyx_r;
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;

  /* "dipy/align/vector_fields.pxd":13
 *     (x0, x1, x2, h)
 *     """
 *     return aff[0, 0] * x0 + aff[0, 1] * x1 + aff[0, 2] * x2 + h*aff[0, 3]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 1;
  __pyx_t_5 = 0;
  __pyx_t_6 = 2;
  __pyx_t_7 = 0;
  __pyx_t_8 = 3;
  __pyx_r = (((((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_1 * __pyx_v_aff.strides[0]) ) + __pyx_t_2 * __pyx_v_aff.strides[1]) ))) * __pyx_v_x0) + ((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_3 * __pyx_v_aff.strides[0]) ) + __pyx_t_4 * __pyx_v_aff.strides[1]) ))) * __pyx_v_x1)) + ((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_5 * __pyx_v_aff.strides[0]) ) + __pyx_t_6 * __pyx_v_aff.strides[1]) ))) * __pyx_v_x2)) + (__pyx_v_h * (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_7 * __pyx_v_aff.strides[0]) ) + __pyx_t_8 * __pyx_v_aff.strides[1]) )))));
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pxd":6
 * #cython: cdivision=True
 * 
 * cdef inline double _apply_affine_3d_x0(double x0, double x1, double x2,             # <<<<<<<<<<<<<<
 *                                        double h, double[:, :] aff) nogil:
 *     r"""Multiplies aff by (x0, x1, x2, h), returns the 1st element of product
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "dipy/align/vector_fields.pxd":16
 * 
 * 
 * cdef inline double _apply_affine_3d_x1(double x0, double x1, double x2,             # <<<<<<<<<<<<<<
 *                                        double h, double[:, :] aff) nogil:
 *     r"""Multiplies aff by (x0, x1, x2, h), returns the 2nd element of product
 */

static CYTHON_INLINE double __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x1(double __pyx_v_x0, double __pyx_v_x1, double __pyx_v_x2, double __pyx_v_h, __Pyx_memviewslice __pyx_v_aff) {
  double __pyx_r;
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;

  /* "dipy/align/vector_fields.pxd":23
 *     (x0, x1, x2, h)
 *     """
 *     return aff[1, 0] * x0 + aff[1, 1] * x1 + aff[1, 2] * x2 + h*aff[1, 3]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = 1;
  __pyx_t_2 = 0;
  __pyx_t_3 = 1;
  __pyx_t_4 = 1;
  __pyx_t_5 = 1;
  __pyx_t_6 = 2;
  __pyx_t_7 = 1;
  __pyx_t_8 = 3;
  __pyx_r = (((((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_1 * __pyx_v_aff.strides[0]) ) + __pyx_t_2 * __pyx_v_aff.strides[1]) ))) * __pyx_v_x0) + ((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_3 * __pyx_v_aff.strides[0]) ) + __pyx_t_4 * __pyx_v_aff.strides[1]) ))) * __pyx_v_x1)) + ((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_5 * __pyx_v_aff.strides[0]) ) + __pyx_t_6 * __pyx_v_aff.strides[1]) ))) * __pyx_v_x2)) + (__pyx_v_h * (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_7 * __pyx_v_aff.strides[0]) ) + __pyx_t_8 * __pyx_v_aff.strides[1]) )))));
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pxd":16
 * 
 * 
 * cdef inline double _apply_affine_3d_x1(double x0, double x1, double x2,             # <<<<<<<<<<<<<<
 *                                        double h, double[:, :] aff) nogil:
 *     r"""Multiplies aff by (x0, x1, x2, h), returns the 2nd element of product
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "dipy/align/vector_fields.pxd":26
 * 
 * 
 * cdef inline double _apply_affine_3d_x2(double x0, double x1, double x2,             # <<<<<<<<<<<<<<
 *                                        double h, double[:, :] aff) nogil:
 *     r"""Multiplies aff by (x0, x1, x2, h), returns the 3d element of product
 */

static CYTHON_INLINE double __pyx_f_4dipy_5align_13vector_fields__apply_affine_3d_x2(double __pyx_v_x0, double __pyx_v_x1, double __pyx_v_x2, double __pyx_v_h, __Pyx_memviewslice __pyx_v_aff) {
  double __pyx_r;
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;

  /* "dipy/align/vector_fields.pxd":33
 *     (x0, x1, x2, h)
 *     """
 *     return aff[2, 0] * x0 + aff[2, 1] * x1 + aff[2, 2] * x2 + h*aff[2, 3]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = 2;
  __pyx_t_2 = 0;
  __pyx_t_3 = 2;
  __pyx_t_4 = 1;
  __pyx_t_5 = 2;
  __pyx_t_6 = 2;
  __pyx_t_7 = 2;
  __pyx_t_8 = 3;
  __pyx_r = (((((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_1 * __pyx_v_aff.strides[0]) ) + __pyx_t_2 * __pyx_v_aff.strides[1]) ))) * __pyx_v_x0) + ((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_3 * __pyx_v_aff.strides[0]) ) + __pyx_t_4 * __pyx_v_aff.strides[1]) ))) * __pyx_v_x1)) + ((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_5 * __pyx_v_aff.strides[0]) ) + __pyx_t_6 * __pyx_v_aff.strides[1]) ))) * __pyx_v_x2)) + (__pyx_v_h * (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_7 * __pyx_v_aff.strides[0]) ) + __pyx_t_8 * __pyx_v_aff.strides[1]) )))));
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pxd":26
 * 
 * 
 * cdef inline double _apply_affine_3d_x2(double x0, double x1, double x2,             # <<<<<<<<<<<<<<
 *                                        double h, double[:, :] aff) nogil:
 *     r"""Multiplies aff by (x0, x1, x2, h), returns the 3d element of product
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "dipy/align/vector_fields.pxd":36
 * 
 * 
 * cdef inline double _apply_affine_2d_x0(double x0, double x1, double h,             # <<<<<<<<<<<<<<
 *                                        double[:, :] aff) nogil:
 *     r"""Multiplies aff by (x0, x1, h), returns the 1st element of product
 */

static CYTHON_INLINE double __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x0(double __pyx_v_x0, double __pyx_v_x1, double __pyx_v_h, __Pyx_memviewslice __pyx_v_aff) {
  double __pyx_r;
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;

  /* "dipy/align/vector_fields.pxd":42
 *     (x0, x1, h)
 *     """
 *     return aff[0, 0] * x0 + aff[0, 1] * x1 + h*aff[0, 2]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 1;
  __pyx_t_5 = 0;
  __pyx_t_6 = 2;
  __pyx_r = ((((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_1 * __pyx_v_aff.strides[0]) ) + __pyx_t_2 * __pyx_v_aff.strides[1]) ))) * __pyx_v_x0) + ((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_3 * __pyx_v_aff.strides[0]) ) + __pyx_t_4 * __pyx_v_aff.strides[1]) ))) * __pyx_v_x1)) + (__pyx_v_h * (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_5 * __pyx_v_aff.strides[0]) ) + __pyx_t_6 * __pyx_v_aff.strides[1]) )))));
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pxd":36
 * 
 * 
 * cdef inline double _apply_affine_2d_x0(double x0, double x1, double h,             # <<<<<<<<<<<<<<
 *                                        double[:, :] aff) nogil:
 *     r"""Multiplies aff by (x0, x1, h), returns the 1st element of product
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "dipy/align/vector_fields.pxd":45
 * 
 * 
 * cdef inline double _apply_affine_2d_x1(double x0, double x1, double h,             # <<<<<<<<<<<<<<
 *                                        double[:, :] aff) nogil:
 *     r"""Multiplies aff by (x0, x1, h), returns the 2nd element of product
 */

static CYTHON_INLINE double __pyx_f_4dipy_5align_13vector_fields__apply_affine_2d_x1(double __pyx_v_x0, double __pyx_v_x1, double __pyx_v_h, __Pyx_memviewslice __pyx_v_aff) {
  double __pyx_r;
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;

  /* "dipy/align/vector_fields.pxd":52
 *     (x0, x1, h)
 *     """
 *     return aff[1, 0] * x0 + aff[1, 1] * x1 + h*aff[1, 2]             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = 1;
  __pyx_t_2 = 0;
  __pyx_t_3 = 1;
  __pyx_t_4 = 1;
  __pyx_t_5 = 1;
  __pyx_t_6 = 2;
  __pyx_r = ((((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_1 * __pyx_v_aff.strides[0]) ) + __pyx_t_2 * __pyx_v_aff.strides[1]) ))) * __pyx_v_x0) + ((*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_3 * __pyx_v_aff.strides[0]) ) + __pyx_t_4 * __pyx_v_aff.strides[1]) ))) * __pyx_v_x1)) + (__pyx_v_h * (*((double *) ( /* dim=1 */ (( /* dim=0 */ (__pyx_v_aff.data + __pyx_t_5 * __pyx_v_aff.strides[0]) ) + __pyx_t_6 * __pyx_v_aff.strides[1]) )))));
  goto __pyx_L0;

  /* "dipy/align/vector_fields.pxd":45
 * 
 * 
 * cdef inline double _apply_affine_2d_x1(double x0, double x1, double h,             # <<<<<<<<<<<<<<
 *                                        double[:, :] aff) nogil:
 *     r"""Multiplies aff by (x0, x1, h), returns the 2nd element of product
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":118
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
 */

/* Python wrapper */
static int __pyx_array___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_array___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_shape = 0;
  Py_ssize_t __pyx_v_itemsize;
  PyObject *__pyx_v_format = 0;
  PyObject *__pyx_v_mode = 0;
  int __pyx_v_allocate_buffer;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_shape,&__pyx_n_s_itemsize,&__pyx_n_s_format,&__pyx_n_s_mode,&__pyx_n_s_allocate_buffer,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[3] = ((PyObject *)__pyx_n_s_c);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_shape)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_itemsize)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 5, 1); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 118; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_format)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 5, 2); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 118; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  3:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_mode);
          if (value) { values[3] = value; kw_args--; }
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_allocate_buffer);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 118; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_shape = ((PyObject*)values[0]);
    __pyx_v_itemsize = __Pyx_PyIndex_AsSsize_t(values[1]); if (unlikely((__pyx_v_itemsize == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 118; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    __pyx_v_format = values[2];
    __pyx_v_mode = values[3];
    if (values[4]) {
      __pyx_v_allocate_buffer = __Pyx_PyObject_IsTrue(values[4]); if (unlikely((__pyx_v_allocate_buffer == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 119; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {

      /* "View.MemoryView":119
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,
 *                   mode="c", bint allocate_buffer=True):             # <<<<<<<<<<<<<<
 * 
 *         cdef int idx
 */
      __pyx_v_allocate_buffer = ((int)1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 3, 5, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 118; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("View.MemoryView.array.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_shape), (&PyTuple_Type), 1, "shape", 1))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 118; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (unlikely(((PyObject *)__pyx_v_format) == Py_None)) {
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "format"); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 118; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_r = __pyx_array___pyx_pf_15View_dot_MemoryView_5array___cinit__(((struct __pyx_array_obj *)__pyx_v_self), __pyx_v_shape, __pyx_v_itemsize, __pyx_v_format, __pyx_v_mode, __pyx_v_allocate_buffer);

  /* "View.MemoryView":118
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array___cinit__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, PyObject *__pyx_v_format, PyObject *__pyx_v_mode, int __pyx_v_allocate_buffer) {
  int __pyx_v_idx;
  Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_dim;
  PyObject **__pyx_v_p;
  char __pyx_v_order;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  char *__pyx_t_6;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);
  __Pyx_INCREF(__pyx_v_format);

  /* "View.MemoryView":125
 *         cdef PyObject **p
 * 
 *         self.ndim = <int> len(shape)             # <<<<<<<<<<<<<<
 *         self.itemsize = itemsize
 * 
 */
  if (unlikely(__pyx_v_shape == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_t_1 = PyTuple_GET_SIZE(__pyx_v_shape); if (unlikely(__pyx_t_1 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_self->ndim = ((int)__pyx_t_1);

  /* "View.MemoryView":126
 * 
 *         self.ndim = <int> len(shape)
 *         self.itemsize = itemsize             # <<<<<<<<<<<<<<
 * 
 *         if not self.ndim:
 */
  __pyx_v_self->itemsize = __pyx_v_itemsize;

  /* "View.MemoryView":128
 *         self.itemsize = itemsize
 * 
 *         if not self.ndim:             # <<<<<<<<<<<<<<
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 */
  __pyx_t_2 = ((!(__pyx_v_self->ndim != 0)) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":129
 * 
 *         if not self.ndim:
 *             raise ValueError("Empty shape tuple for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if itemsize <= 0:
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__383, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 129; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 129; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "View.MemoryView":128
 *         self.itemsize = itemsize
 * 
 *         if not self.ndim:             # <<<<<<<<<<<<<<
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 */
  }

  /* "View.MemoryView":131
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 *         if itemsize <= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 */
  __pyx_t_2 = ((__pyx_v_itemsize <= 0) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":132
 * 
 *         if itemsize <= 0:
 *             raise ValueError("itemsize <= 0 for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if not isinstance(format, bytes):
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__384, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "View.MemoryView":131
 *             raise ValueError("Empty shape tuple for cython.array")
 * 
 *         if itemsize <= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 */
  }

  /* "View.MemoryView":134
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 *         if not isinstance(format, bytes):             # <<<<<<<<<<<<<<
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string
 */
  __pyx_t_2 = PyBytes_Check(__pyx_v_format); 
  __pyx_t_4 = ((!(__pyx_t_2 != 0)) != 0);
  if (__pyx_t_4) {

    /* "View.MemoryView":135
 * 
 *         if not isinstance(format, bytes):
 *             format = format.encode('ASCII')             # <<<<<<<<<<<<<<
 *         self._format = format  # keep a reference to the byte string
 *         self.format = self._format
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_format, __pyx_n_s_encode); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__385, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_format, __pyx_t_5);
    __pyx_t_5 = 0;

    /* "View.MemoryView":134
 *             raise ValueError("itemsize <= 0 for cython.array")
 * 
 *         if not isinstance(format, bytes):             # <<<<<<<<<<<<<<
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string
 */
  }

  /* "View.MemoryView":136
 *         if not isinstance(format, bytes):
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string             # <<<<<<<<<<<<<<
 *         self.format = self._format
 * 
 */
  if (!(likely(PyBytes_CheckExact(__pyx_v_format))||((__pyx_v_format) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_v_format)->tp_name), 0))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 136; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_5 = __pyx_v_format;
  __Pyx_INCREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_5);
  __Pyx_GOTREF(__pyx_v_self->_format);
  __Pyx_DECREF(__pyx_v_self->_format);
  __pyx_v_self->_format = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "View.MemoryView":137
 *             format = format.encode('ASCII')
 *         self._format = format  # keep a reference to the byte string
 *         self.format = self._format             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_6 = __Pyx_PyObject_AsString(__pyx_v_self->_format); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 137; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_self->format = __pyx_t_6;

  /* "View.MemoryView":140
 * 
 * 
 *         self._shape = <Py_ssize_t *> PyMem_Malloc(sizeof(Py_ssize_t)*self.ndim*2)             # <<<<<<<<<<<<<<
 *         self._strides = self._shape + self.ndim
 * 
 */
  __pyx_v_self->_shape = ((Py_ssize_t *)PyMem_Malloc((((sizeof(Py_ssize_t)) * __pyx_v_self->ndim) * 2)));

  /* "View.MemoryView":141
 * 
 *         self._shape = <Py_ssize_t *> PyMem_Malloc(sizeof(Py_ssize_t)*self.ndim*2)
 *         self._strides = self._shape + self.ndim             # <<<<<<<<<<<<<<
 * 
 *         if not self._shape:
 */
  __pyx_v_self->_strides = (__pyx_v_self->_shape + __pyx_v_self->ndim);

  /* "View.MemoryView":143
 *         self._strides = self._shape + self.ndim
 * 
 *         if not self._shape:             # <<<<<<<<<<<<<<
 *             raise MemoryError("unable to allocate shape and strides.")
 * 
 */
  __pyx_t_4 = ((!(__pyx_v_self->_shape != 0)) != 0);
  if (__pyx_t_4) {

    /* "View.MemoryView":144
 * 
 *         if not self._shape:
 *             raise MemoryError("unable to allocate shape and strides.")             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__386, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "View.MemoryView":143
 *         self._strides = self._shape + self.ndim
 * 
 *         if not self._shape:             # <<<<<<<<<<<<<<
 *             raise MemoryError("unable to allocate shape and strides.")
 * 
 */
  }

  /* "View.MemoryView":147
 * 
 * 
 *         for idx, dim in enumerate(shape):             # <<<<<<<<<<<<<<
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 */
  __pyx_t_7 = 0;
  __pyx_t_5 = __pyx_v_shape; __Pyx_INCREF(__pyx_t_5); __pyx_t_1 = 0;
  for (;;) {
    if (__pyx_t_1 >= PyTuple_GET_SIZE(__pyx_t_5)) break;
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_3 = PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_1); __Pyx_INCREF(__pyx_t_3); __pyx_t_1++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_5, __pyx_t_1); __pyx_t_1++; if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __pyx_t_8 = __Pyx_PyIndex_AsSsize_t(__pyx_t_3); if (unlikely((__pyx_t_8 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_dim = __pyx_t_8;
    __pyx_v_idx = __pyx_t_7;
    __pyx_t_7 = (__pyx_t_7 + 1);

    /* "View.MemoryView":148
 * 
 *         for idx, dim in enumerate(shape):
 *             if dim <= 0:             # <<<<<<<<<<<<<<
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 *             self._shape[idx] = dim
 */
    __pyx_t_4 = ((__pyx_v_dim <= 0) != 0);
    if (__pyx_t_4) {

      /* "View.MemoryView":149
 *         for idx, dim in enumerate(shape):
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))             # <<<<<<<<<<<<<<
 *             self._shape[idx] = dim
 * 
 */
      __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_idx); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 149; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_9 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 149; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 149; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_9);
      __pyx_t_3 = 0;
      __pyx_t_9 = 0;
      __pyx_t_9 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_shape_in_axis_d_d, __pyx_t_10); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 149; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_10 = PyTuple_New(1); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 149; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_9);
      __pyx_t_9 = 0;
      __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_10, NULL); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 149; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_Raise(__pyx_t_9, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 149; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "View.MemoryView":148
 * 
 *         for idx, dim in enumerate(shape):
 *             if dim <= 0:             # <<<<<<<<<<<<<<
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 *             self._shape[idx] = dim
 */
    }

    /* "View.MemoryView":150
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 *             self._shape[idx] = dim             # <<<<<<<<<<<<<<
 * 
 *         cdef char order
 */
    (__pyx_v_self->_shape[__pyx_v_idx]) = __pyx_v_dim;

    /* "View.MemoryView":147
 * 
 * 
 *         for idx, dim in enumerate(shape):             # <<<<<<<<<<<<<<
 *             if dim <= 0:
 *                 raise ValueError("Invalid shape in axis %d: %d." % (idx, dim))
 */
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "View.MemoryView":153
 * 
 *         cdef char order
 *         if mode == 'fortran':             # <<<<<<<<<<<<<<
 *             order = b'F'
 *             self.mode = u'fortran'
 */
  __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_mode, __pyx_n_s_fortran, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 153; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_4) {

    /* "View.MemoryView":154
 *         cdef char order
 *         if mode == 'fortran':
 *             order = b'F'             # <<<<<<<<<<<<<<
 *             self.mode = u'fortran'
 *         elif mode == 'c':
 */
    __pyx_v_order = 'F';

    /* "View.MemoryView":155
 *         if mode == 'fortran':
 *             order = b'F'
 *             self.mode = u'fortran'             # <<<<<<<<<<<<<<
 *         elif mode == 'c':
 *             order = b'C'
 */
    __Pyx_INCREF(__pyx_n_u_fortran);
    __Pyx_GIVEREF(__pyx_n_u_fortran);
    __Pyx_GOTREF(__pyx_v_self->mode);
    __Pyx_DECREF(__pyx_v_self->mode);
    __pyx_v_self->mode = __pyx_n_u_fortran;

    /* "View.MemoryView":153
 * 
 *         cdef char order
 *         if mode == 'fortran':             # <<<<<<<<<<<<<<
 *             order = b'F'
 *             self.mode = u'fortran'
 */
    goto __pyx_L10;
  }

  /* "View.MemoryView":156
 *             order = b'F'
 *             self.mode = u'fortran'
 *         elif mode == 'c':             # <<<<<<<<<<<<<<
 *             order = b'C'
 *             self.mode = u'c'
 */
  __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_mode, __pyx_n_s_c, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 156; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_4) {

    /* "View.MemoryView":157
 *             self.mode = u'fortran'
 *         elif mode == 'c':
 *             order = b'C'             # <<<<<<<<<<<<<<
 *             self.mode = u'c'
 *         else:
 */
    __pyx_v_order = 'C';

    /* "View.MemoryView":158
 *         elif mode == 'c':
 *             order = b'C'
 *             self.mode = u'c'             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError("Invalid mode, expected 'c' or 'fortran', got %s" % mode)
 */
    __Pyx_INCREF(__pyx_n_u_c);
    __Pyx_GIVEREF(__pyx_n_u_c);
    __Pyx_GOTREF(__pyx_v_self->mode);
    __Pyx_DECREF(__pyx_v_self->mode);
    __pyx_v_self->mode = __pyx_n_u_c;

    /* "View.MemoryView":156
 *             order = b'F'
 *             self.mode = u'fortran'
 *         elif mode == 'c':             # <<<<<<<<<<<<<<
 *             order = b'C'
 *             self.mode = u'c'
 */
    goto __pyx_L10;
  }

  /* "View.MemoryView":160
 *             self.mode = u'c'
 *         else:
 *             raise ValueError("Invalid mode, expected 'c' or 'fortran', got %s" % mode)             # <<<<<<<<<<<<<<
 * 
 *         self.len = fill_contig_strides_array(self._shape, self._strides,
 */
  /*else*/ {
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_mode_expected_c_or_fortr, __pyx_v_mode); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_9, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L10:;

  /* "View.MemoryView":162
 *             raise ValueError("Invalid mode, expected 'c' or 'fortran', got %s" % mode)
 * 
 *         self.len = fill_contig_strides_array(self._shape, self._strides,             # <<<<<<<<<<<<<<
 *                                              itemsize, self.ndim, order)
 * 
 */
  __pyx_v_self->len = __pyx_fill_contig_strides_array(__pyx_v_self->_shape, __pyx_v_self->_strides, __pyx_v_itemsize, __pyx_v_self->ndim, __pyx_v_order);

  /* "View.MemoryView":165
 *                                              itemsize, self.ndim, order)
 * 
 *         self.free_data = allocate_buffer             # <<<<<<<<<<<<<<
 *         self.dtype_is_object = format == b'O'
 *         if allocate_buffer:
 */
  __pyx_v_self->free_data = __pyx_v_allocate_buffer;

  /* "View.MemoryView":166
 * 
 *         self.free_data = allocate_buffer
 *         self.dtype_is_object = format == b'O'             # <<<<<<<<<<<<<<
 *         if allocate_buffer:
 * 
 */
  __pyx_t_5 = PyObject_RichCompare(__pyx_v_format, __pyx_n_b_O, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 166; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 166; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_v_self->dtype_is_object = __pyx_t_4;

  /* "View.MemoryView":167
 *         self.free_data = allocate_buffer
 *         self.dtype_is_object = format == b'O'
 *         if allocate_buffer:             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_4 = (__pyx_v_allocate_buffer != 0);
  if (__pyx_t_4) {

    /* "View.MemoryView":170
 * 
 * 
 *             self.data = <char *>malloc(self.len)             # <<<<<<<<<<<<<<
 *             if not self.data:
 *                 raise MemoryError("unable to allocate array data.")
 */
    __pyx_v_self->data = ((char *)malloc(__pyx_v_self->len));

    /* "View.MemoryView":171
 * 
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:             # <<<<<<<<<<<<<<
 *                 raise MemoryError("unable to allocate array data.")
 * 
 */
    __pyx_t_4 = ((!(__pyx_v_self->data != 0)) != 0);
    if (__pyx_t_4) {

      /* "View.MemoryView":172
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:
 *                 raise MemoryError("unable to allocate array data.")             # <<<<<<<<<<<<<<
 * 
 *             if self.dtype_is_object:
 */
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__387, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "View.MemoryView":171
 * 
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:             # <<<<<<<<<<<<<<
 *                 raise MemoryError("unable to allocate array data.")
 * 
 */
    }

    /* "View.MemoryView":174
 *                 raise MemoryError("unable to allocate array data.")
 * 
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):
 */
    __pyx_t_4 = (__pyx_v_self->dtype_is_object != 0);
    if (__pyx_t_4) {

      /* "View.MemoryView":175
 * 
 *             if self.dtype_is_object:
 *                 p = <PyObject **> self.data             # <<<<<<<<<<<<<<
 *                 for i in range(self.len / itemsize):
 *                     p[i] = Py_None
 */
      __pyx_v_p = ((PyObject **)__pyx_v_self->data);

      /* "View.MemoryView":176
 *             if self.dtype_is_object:
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):             # <<<<<<<<<<<<<<
 *                     p[i] = Py_None
 *                     Py_INCREF(Py_None)
 */
      if (unlikely(__pyx_v_itemsize == 0)) {
        PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
        {__pyx_filename = __pyx_f[2]; __pyx_lineno = 176; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      else if (sizeof(Py_ssize_t) == sizeof(long) && (!(((Py_ssize_t)-1) > 0)) && unlikely(__pyx_v_itemsize == (Py_ssize_t)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_self->len))) {
        PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
        {__pyx_filename = __pyx_f[2]; __pyx_lineno = 176; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_t_1 = (__pyx_v_self->len / __pyx_v_itemsize);
      for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_1; __pyx_t_8+=1) {
        __pyx_v_i = __pyx_t_8;

        /* "View.MemoryView":177
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):
 *                     p[i] = Py_None             # <<<<<<<<<<<<<<
 *                     Py_INCREF(Py_None)
 * 
 */
        (__pyx_v_p[__pyx_v_i]) = Py_None;

        /* "View.MemoryView":178
 *                 for i in range(self.len / itemsize):
 *                     p[i] = Py_None
 *                     Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
 */
        Py_INCREF(Py_None);
      }

      /* "View.MemoryView":174
 *                 raise MemoryError("unable to allocate array data.")
 * 
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 p = <PyObject **> self.data
 *                 for i in range(self.len / itemsize):
 */
    }

    /* "View.MemoryView":167
 *         self.free_data = allocate_buffer
 *         self.dtype_is_object = format == b'O'
 *         if allocate_buffer:             # <<<<<<<<<<<<<<
 * 
 * 
 */
  }

  /* "View.MemoryView":118
 *         cdef bint dtype_is_object
 * 
 *     def __cinit__(array self, tuple shape, Py_ssize_t itemsize, format not None,             # <<<<<<<<<<<<<<
 *                   mode="c", bint allocate_buffer=True):
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("View.MemoryView.array.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_format);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":181
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         cdef int bufmode = -1
 *         if self.mode == u"c":
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_array_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_array___pyx_pf_15View_dot_MemoryView_5array_2__getbuffer__(((struct __pyx_array_obj *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_2__getbuffer__(struct __pyx_array_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_v_bufmode;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  char *__pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  Py_ssize_t *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  if (__pyx_v_info != NULL) {
    __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(__pyx_v_info->obj);
  }

  /* "View.MemoryView":182
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1             # <<<<<<<<<<<<<<
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 */
  __pyx_v_bufmode = -1;

  /* "View.MemoryView":183
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1
 *         if self.mode == u"c":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":
 */
  __pyx_t_1 = (__Pyx_PyUnicode_Equals(__pyx_v_self->mode, __pyx_n_u_c, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 183; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":184
 *         cdef int bufmode = -1
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS             # <<<<<<<<<<<<<<
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 */
    __pyx_v_bufmode = (PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS);

    /* "View.MemoryView":183
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         cdef int bufmode = -1
 *         if self.mode == u"c":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":185
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 */
  __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_v_self->mode, __pyx_n_u_fortran, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":186
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS             # <<<<<<<<<<<<<<
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 */
    __pyx_v_bufmode = (PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS);

    /* "View.MemoryView":185
 *         if self.mode == u"c":
 *             bufmode = PyBUF_C_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         elif self.mode == u"fortran":             # <<<<<<<<<<<<<<
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 */
  }
  __pyx_L3:;

  /* "View.MemoryView":187
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):             # <<<<<<<<<<<<<<
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data
 */
  __pyx_t_1 = ((!((__pyx_v_flags & __pyx_v_bufmode) != 0)) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":188
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")             # <<<<<<<<<<<<<<
 *         info.buf = self.data
 *         info.len = self.len
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__388, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "View.MemoryView":187
 *         elif self.mode == u"fortran":
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):             # <<<<<<<<<<<<<<
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data
 */
  }

  /* "View.MemoryView":189
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data             # <<<<<<<<<<<<<<
 *         info.len = self.len
 *         info.ndim = self.ndim
 */
  __pyx_t_4 = __pyx_v_self->data;
  __pyx_v_info->buf = __pyx_t_4;

  /* "View.MemoryView":190
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")
 *         info.buf = self.data
 *         info.len = self.len             # <<<<<<<<<<<<<<
 *         info.ndim = self.ndim
 *         info.shape = self._shape
 */
  __pyx_t_5 = __pyx_v_self->len;
  __pyx_v_info->len = __pyx_t_5;

  /* "View.MemoryView":191
 *         info.buf = self.data
 *         info.len = self.len
 *         info.ndim = self.ndim             # <<<<<<<<<<<<<<
 *         info.shape = self._shape
 *         info.strides = self._strides
 */
  __pyx_t_6 = __pyx_v_self->ndim;
  __pyx_v_info->ndim = __pyx_t_6;

  /* "View.MemoryView":192
 *         info.len = self.len
 *         info.ndim = self.ndim
 *         info.shape = self._shape             # <<<<<<<<<<<<<<
 *         info.strides = self._strides
 *         info.suboffsets = NULL
 */
  __pyx_t_7 = __pyx_v_self->_shape;
  __pyx_v_info->shape = __pyx_t_7;

  /* "View.MemoryView":193
 *         info.ndim = self.ndim
 *         info.shape = self._shape
 *         info.strides = self._strides             # <<<<<<<<<<<<<<
 *         info.suboffsets = NULL
 *         info.itemsize = self.itemsize
 */
  __pyx_t_7 = __pyx_v_self->_strides;
  __pyx_v_info->strides = __pyx_t_7;

  /* "View.MemoryView":194
 *         info.shape = self._shape
 *         info.strides = self._strides
 *         info.suboffsets = NULL             # <<<<<<<<<<<<<<
 *         info.itemsize = self.itemsize
 *         info.readonly = 0
 */
  __pyx_v_info->suboffsets = NULL;

  /* "View.MemoryView":195
 *         info.strides = self._strides
 *         info.suboffsets = NULL
 *         info.itemsize = self.itemsize             # <<<<<<<<<<<<<<
 *         info.readonly = 0
 * 
 */
  __pyx_t_5 = __pyx_v_self->itemsize;
  __pyx_v_info->itemsize = __pyx_t_5;

  /* "View.MemoryView":196
 *         info.suboffsets = NULL
 *         info.itemsize = self.itemsize
 *         info.readonly = 0             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_FORMAT:
 */
  __pyx_v_info->readonly = 0;

  /* "View.MemoryView":198
 *         info.readonly = 0
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.format
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":199
 * 
 *         if flags & PyBUF_FORMAT:
 *             info.format = self.format             # <<<<<<<<<<<<<<
 *         else:
 *             info.format = NULL
 */
    __pyx_t_4 = __pyx_v_self->format;
    __pyx_v_info->format = __pyx_t_4;

    /* "View.MemoryView":198
 *         info.readonly = 0
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.format
 *         else:
 */
    goto __pyx_L5;
  }

  /* "View.MemoryView":201
 *             info.format = self.format
 *         else:
 *             info.format = NULL             # <<<<<<<<<<<<<<
 * 
 *         info.obj = self
 */
  /*else*/ {
    __pyx_v_info->format = NULL;
  }
  __pyx_L5:;

  /* "View.MemoryView":203
 *             info.format = NULL
 * 
 *         info.obj = self             # <<<<<<<<<<<<<<
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  __Pyx_GOTREF(__pyx_v_info->obj);
  __Pyx_DECREF(__pyx_v_info->obj);
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "View.MemoryView":181
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         cdef int bufmode = -1
 *         if self.mode == u"c":
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.array.__getbuffer__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  if (__pyx_v_info != NULL && __pyx_v_info->obj != NULL) {
    __Pyx_GOTREF(__pyx_v_info->obj);
    __Pyx_DECREF(__pyx_v_info->obj); __pyx_v_info->obj = NULL;
  }
  goto __pyx_L2;
  __pyx_L0:;
  if (__pyx_v_info != NULL && __pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(Py_None);
    __Pyx_DECREF(Py_None); __pyx_v_info->obj = NULL;
  }
  __pyx_L2:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":207
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")
 * 
 *     def __dealloc__(array self):             # <<<<<<<<<<<<<<
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 */

/* Python wrapper */
static void __pyx_array___dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_array___dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_array___pyx_pf_15View_dot_MemoryView_5array_4__dealloc__(((struct __pyx_array_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_array___pyx_pf_15View_dot_MemoryView_5array_4__dealloc__(struct __pyx_array_obj *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "View.MemoryView":208
 * 
 *     def __dealloc__(array self):
 *         if self.callback_free_data != NULL:             # <<<<<<<<<<<<<<
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 */
  __pyx_t_1 = ((__pyx_v_self->callback_free_data != NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":209
 *     def __dealloc__(array self):
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)             # <<<<<<<<<<<<<<
 *         elif self.free_data:
 *             if self.dtype_is_object:
 */
    __pyx_v_self->callback_free_data(__pyx_v_self->data);

    /* "View.MemoryView":208
 * 
 *     def __dealloc__(array self):
 *         if self.callback_free_data != NULL:             # <<<<<<<<<<<<<<
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":210
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 *         elif self.free_data:             # <<<<<<<<<<<<<<
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape,
 */
  __pyx_t_1 = (__pyx_v_self->free_data != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":211
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 refcount_objects_in_slice(self.data, self._shape,
 *                                           self._strides, self.ndim, False)
 */
    __pyx_t_1 = (__pyx_v_self->dtype_is_object != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":212
 *         elif self.free_data:
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape,             # <<<<<<<<<<<<<<
 *                                           self._strides, self.ndim, False)
 *             free(self.data)
 */
      __pyx_memoryview_refcount_objects_in_slice(__pyx_v_self->data, __pyx_v_self->_shape, __pyx_v_self->_strides, __pyx_v_self->ndim, 0);

      /* "View.MemoryView":211
 *             self.callback_free_data(self.data)
 *         elif self.free_data:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 refcount_objects_in_slice(self.data, self._shape,
 *                                           self._strides, self.ndim, False)
 */
    }

    /* "View.MemoryView":214
 *                 refcount_objects_in_slice(self.data, self._shape,
 *                                           self._strides, self.ndim, False)
 *             free(self.data)             # <<<<<<<<<<<<<<
 *         PyMem_Free(self._shape)
 * 
 */
    free(__pyx_v_self->data);

    /* "View.MemoryView":210
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 *         elif self.free_data:             # <<<<<<<<<<<<<<
 *             if self.dtype_is_object:
 *                 refcount_objects_in_slice(self.data, self._shape,
 */
  }
  __pyx_L3:;

  /* "View.MemoryView":215
 *                                           self._strides, self.ndim, False)
 *             free(self.data)
 *         PyMem_Free(self._shape)             # <<<<<<<<<<<<<<
 * 
 *     property memview:
 */
  PyMem_Free(__pyx_v_self->_shape);

  /* "View.MemoryView":207
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")
 * 
 *     def __dealloc__(array self):             # <<<<<<<<<<<<<<
 *         if self.callback_free_data != NULL:
 *             self.callback_free_data(self.data)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "View.MemoryView":219
 *     property memview:
 *         @cname('get_memview')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 * 
 *             flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
 */

/* Python wrapper */
static PyObject *get_memview(PyObject *__pyx_v_self); /*proto*/
static PyObject *get_memview(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_5array_7memview___get__(((struct __pyx_array_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_5array_7memview___get__(struct __pyx_array_obj *__pyx_v_self) {
  int __pyx_v_flags;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":221
 *         def __get__(self):
 * 
 *             flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE             # <<<<<<<<<<<<<<
 *             return  memoryview(self, flags, self.dtype_is_object)
 * 
 */
  __pyx_v_flags = ((PyBUF_ANY_CONTIGUOUS | PyBUF_FORMAT) | PyBUF_WRITABLE);

  /* "View.MemoryView":222
 * 
 *             flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
 *             return  memoryview(self, flags, self.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_flags); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->dtype_is_object); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_memoryview_type), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":219
 *     property memview:
 *         @cname('get_memview')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 * 
 *             flags =  PyBUF_ANY_CONTIGUOUS|PyBUF_FORMAT|PyBUF_WRITABLE
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.array.memview.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":225
 * 
 * 
 *     def __getattr__(self, attr):             # <<<<<<<<<<<<<<
 *         return getattr(self.memview, attr)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_array___getattr__(PyObject *__pyx_v_self, PyObject *__pyx_v_attr); /*proto*/
static PyObject *__pyx_array___getattr__(PyObject *__pyx_v_self, PyObject *__pyx_v_attr) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getattr__ (wrapper)", 0);
  __pyx_r = __pyx_array___pyx_pf_15View_dot_MemoryView_5array_6__getattr__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_attr));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_6__getattr__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_attr) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getattr__", 0);

  /* "View.MemoryView":226
 * 
 *     def __getattr__(self, attr):
 *         return getattr(self.memview, attr)             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, item):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_memview); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 226; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetAttr(__pyx_t_1, __pyx_v_attr); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 226; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":225
 * 
 * 
 *     def __getattr__(self, attr):             # <<<<<<<<<<<<<<
 *         return getattr(self.memview, attr)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.array.__getattr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":228
 *         return getattr(self.memview, attr)
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         return self.memview[item]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_array___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item); /*proto*/
static PyObject *__pyx_array___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_array___pyx_pf_15View_dot_MemoryView_5array_8__getitem__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_item));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_array___pyx_pf_15View_dot_MemoryView_5array_8__getitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "View.MemoryView":229
 * 
 *     def __getitem__(self, item):
 *         return self.memview[item]             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, item, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_memview); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 229; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_GetItem(__pyx_t_1, __pyx_v_item); if (unlikely(__pyx_t_2 == NULL)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 229; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":228
 *         return getattr(self.memview, attr)
 * 
 *     def __getitem__(self, item):             # <<<<<<<<<<<<<<
 *         return self.memview[item]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.array.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":231
 *         return self.memview[item]
 * 
 *     def __setitem__(self, item, value):             # <<<<<<<<<<<<<<
 *         self.memview[item] = value
 * 
 */

/* Python wrapper */
static int __pyx_array___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value); /*proto*/
static int __pyx_array___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = __pyx_array___pyx_pf_15View_dot_MemoryView_5array_10__setitem__(((struct __pyx_array_obj *)__pyx_v_self), ((PyObject *)__pyx_v_item), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_array___pyx_pf_15View_dot_MemoryView_5array_10__setitem__(struct __pyx_array_obj *__pyx_v_self, PyObject *__pyx_v_item, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "View.MemoryView":232
 * 
 *     def __setitem__(self, item, value):
 *         self.memview[item] = value             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_memview); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(PyObject_SetItem(__pyx_t_1, __pyx_v_item, __pyx_v_value) < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "View.MemoryView":231
 *         return self.memview[item]
 * 
 *     def __setitem__(self, item, value):             # <<<<<<<<<<<<<<
 *         self.memview[item] = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.array.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":236
 * 
 * @cname("__pyx_array_new")
 * cdef array array_cwrapper(tuple shape, Py_ssize_t itemsize, char *format,             # <<<<<<<<<<<<<<
 *                           char *mode, char *buf):
 *     cdef array result
 */

static struct __pyx_array_obj *__pyx_array_new(PyObject *__pyx_v_shape, Py_ssize_t __pyx_v_itemsize, char *__pyx_v_format, char *__pyx_v_mode, char *__pyx_v_buf) {
  struct __pyx_array_obj *__pyx_v_result = 0;
  struct __pyx_array_obj *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("array_cwrapper", 0);

  /* "View.MemoryView":240
 *     cdef array result
 * 
 *     if buf == NULL:             # <<<<<<<<<<<<<<
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_buf == NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":241
 * 
 *     if buf == NULL:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))             # <<<<<<<<<<<<<<
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),
 */
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_itemsize); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyBytes_FromString(__pyx_v_format); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_decode_c_string(__pyx_v_mode, 0, strlen(__pyx_v_mode), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v_shape);
    __Pyx_GIVEREF(__pyx_v_shape);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_shape);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_array_type), __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 241; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_result = ((struct __pyx_array_obj *)__pyx_t_4);
    __pyx_t_4 = 0;

    /* "View.MemoryView":240
 *     cdef array result
 * 
 *     if buf == NULL:             # <<<<<<<<<<<<<<
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":243
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),             # <<<<<<<<<<<<<<
 *                        allocate_buffer=False)
 *         result.data = buf
 */
  /*else*/ {
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_itemsize); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_v_format); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __Pyx_decode_c_string(__pyx_v_mode, 0, strlen(__pyx_v_mode), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_shape);
    __Pyx_GIVEREF(__pyx_v_shape);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_shape);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_3);
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_t_3 = 0;

    /* "View.MemoryView":244
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),
 *                        allocate_buffer=False)             # <<<<<<<<<<<<<<
 *         result.data = buf
 * 
 */
    __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 244; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_allocate_buffer, Py_False) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 244; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "View.MemoryView":243
 *         result = array(shape, itemsize, format, mode.decode('ASCII'))
 *     else:
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),             # <<<<<<<<<<<<<<
 *                        allocate_buffer=False)
 *         result.data = buf
 */
    __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_array_type), __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_result = ((struct __pyx_array_obj *)__pyx_t_5);
    __pyx_t_5 = 0;

    /* "View.MemoryView":245
 *         result = array(shape, itemsize, format, mode.decode('ASCII'),
 *                        allocate_buffer=False)
 *         result.data = buf             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
    __pyx_v_result->data = __pyx_v_buf;
  }
  __pyx_L3:;

  /* "View.MemoryView":247
 *         result.data = buf
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "View.MemoryView":236
 * 
 * @cname("__pyx_array_new")
 * cdef array array_cwrapper(tuple shape, Py_ssize_t itemsize, char *format,             # <<<<<<<<<<<<<<
 *                           char *mode, char *buf):
 *     cdef array result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.array_cwrapper", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":273
 * cdef class Enum(object):
 *     cdef object name
 *     def __init__(self, name):             # <<<<<<<<<<<<<<
 *         self.name = name
 *     def __repr__(self):
 */

/* Python wrapper */
static int __pyx_MemviewEnum___init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_MemviewEnum___init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_name,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_name)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_name = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 273; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("View.MemoryView.Enum.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum___init__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self), __pyx_v_name);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum___init__(struct __pyx_MemviewEnum_obj *__pyx_v_self, PyObject *__pyx_v_name) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "View.MemoryView":274
 *     cdef object name
 *     def __init__(self, name):
 *         self.name = name             # <<<<<<<<<<<<<<
 *     def __repr__(self):
 *         return self.name
 */
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = __pyx_v_name;

  /* "View.MemoryView":273
 * cdef class Enum(object):
 *     cdef object name
 *     def __init__(self, name):             # <<<<<<<<<<<<<<
 *         self.name = name
 *     def __repr__(self):
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":275
 *     def __init__(self, name):
 *         self.name = name
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return self.name
 * 
 */

/* Python wrapper */
static PyObject *__pyx_MemviewEnum___repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_MemviewEnum___repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum_2__repr__(((struct __pyx_MemviewEnum_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_MemviewEnum___pyx_pf_15View_dot_MemoryView_4Enum_2__repr__(struct __pyx_MemviewEnum_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "View.MemoryView":276
 *         self.name = name
 *     def __repr__(self):
 *         return self.name             # <<<<<<<<<<<<<<
 * 
 * cdef generic = Enum("<strided and direct or indirect>")
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->name);
  __pyx_r = __pyx_v_self->name;
  goto __pyx_L0;

  /* "View.MemoryView":275
 *     def __init__(self, name):
 *         self.name = name
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return self.name
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":290
 * 
 * @cname('__pyx_align_pointer')
 * cdef void *align_pointer(void *memory, size_t alignment) nogil:             # <<<<<<<<<<<<<<
 *     "Align pointer memory on a given boundary"
 *     cdef Py_intptr_t aligned_p = <Py_intptr_t> memory
 */

static void *__pyx_align_pointer(void *__pyx_v_memory, size_t __pyx_v_alignment) {
  Py_intptr_t __pyx_v_aligned_p;
  size_t __pyx_v_offset;
  void *__pyx_r;
  int __pyx_t_1;

  /* "View.MemoryView":292
 * cdef void *align_pointer(void *memory, size_t alignment) nogil:
 *     "Align pointer memory on a given boundary"
 *     cdef Py_intptr_t aligned_p = <Py_intptr_t> memory             # <<<<<<<<<<<<<<
 *     cdef size_t offset
 * 
 */
  __pyx_v_aligned_p = ((Py_intptr_t)__pyx_v_memory);

  /* "View.MemoryView":296
 * 
 *     with cython.cdivision(True):
 *         offset = aligned_p % alignment             # <<<<<<<<<<<<<<
 * 
 *     if offset > 0:
 */
  __pyx_v_offset = (__pyx_v_aligned_p % __pyx_v_alignment);

  /* "View.MemoryView":298
 *         offset = aligned_p % alignment
 * 
 *     if offset > 0:             # <<<<<<<<<<<<<<
 *         aligned_p += alignment - offset
 * 
 */
  __pyx_t_1 = ((__pyx_v_offset > 0) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":299
 * 
 *     if offset > 0:
 *         aligned_p += alignment - offset             # <<<<<<<<<<<<<<
 * 
 *     return <void *> aligned_p
 */
    __pyx_v_aligned_p = (__pyx_v_aligned_p + (__pyx_v_alignment - __pyx_v_offset));

    /* "View.MemoryView":298
 *         offset = aligned_p % alignment
 * 
 *     if offset > 0:             # <<<<<<<<<<<<<<
 *         aligned_p += alignment - offset
 * 
 */
  }

  /* "View.MemoryView":301
 *         aligned_p += alignment - offset
 * 
 *     return <void *> aligned_p             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview')
 */
  __pyx_r = ((void *)__pyx_v_aligned_p);
  goto __pyx_L0;

  /* "View.MemoryView":290
 * 
 * @cname('__pyx_align_pointer')
 * cdef void *align_pointer(void *memory, size_t alignment) nogil:             # <<<<<<<<<<<<<<
 *     "Align pointer memory on a given boundary"
 *     cdef Py_intptr_t aligned_p = <Py_intptr_t> memory
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":319
 *     cdef __Pyx_TypeInfo *typeinfo
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):             # <<<<<<<<<<<<<<
 *         self.obj = obj
 *         self.flags = flags
 */

/* Python wrapper */
static int __pyx_memoryview___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_memoryview___cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_obj = 0;
  int __pyx_v_flags;
  int __pyx_v_dtype_is_object;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_obj,&__pyx_n_s_flags,&__pyx_n_s_dtype_is_object,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_obj)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_flags)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, 1); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 319; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
        }
        case  2:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dtype_is_object);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 319; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_obj = values[0];
    __pyx_v_flags = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_flags == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 319; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    if (values[2]) {
      __pyx_v_dtype_is_object = __Pyx_PyObject_IsTrue(values[2]); if (unlikely((__pyx_v_dtype_is_object == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 319; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
    } else {
      __pyx_v_dtype_is_object = ((int)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 2, 3, PyTuple_GET_SIZE(__pyx_args)); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 319; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
  __pyx_L3_error:;
  __Pyx_AddTraceback("View.MemoryView.memoryview.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview___cinit__(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_obj, __pyx_v_flags, __pyx_v_dtype_is_object);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview___cinit__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_v_flags, int __pyx_v_dtype_is_object) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "View.MemoryView":320
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):
 *         self.obj = obj             # <<<<<<<<<<<<<<
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:
 */
  __Pyx_INCREF(__pyx_v_obj);
  __Pyx_GIVEREF(__pyx_v_obj);
  __Pyx_GOTREF(__pyx_v_self->obj);
  __Pyx_DECREF(__pyx_v_self->obj);
  __pyx_v_self->obj = __pyx_v_obj;

  /* "View.MemoryView":321
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):
 *         self.obj = obj
 *         self.flags = flags             # <<<<<<<<<<<<<<
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 */
  __pyx_v_self->flags = __pyx_v_flags;

  /* "View.MemoryView":322
 *         self.obj = obj
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:
 */
  __pyx_t_2 = (((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))) == ((PyObject *)__pyx_memoryview_type));
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = (__pyx_v_obj != Py_None);
  __pyx_t_2 = (__pyx_t_3 != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "View.MemoryView":323
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)             # <<<<<<<<<<<<<<
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 */
    __pyx_t_4 = __Pyx_GetBuffer(__pyx_v_obj, (&__pyx_v_self->view), __pyx_v_flags); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 323; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "View.MemoryView":324
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:             # <<<<<<<<<<<<<<
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 *                 Py_INCREF(Py_None)
 */
    __pyx_t_1 = ((((PyObject *)__pyx_v_self->view.obj) == NULL) != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":325
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None             # <<<<<<<<<<<<<<
 *                 Py_INCREF(Py_None)
 * 
 */
      ((Py_buffer *)(&__pyx_v_self->view))->obj = Py_None;

      /* "View.MemoryView":326
 *             if <PyObject *> self.view.obj == NULL:
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 *                 Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *         self.lock = PyThread_allocate_lock()
 */
      Py_INCREF(Py_None);

      /* "View.MemoryView":324
 *         if type(self) is memoryview or obj is not None:
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:             # <<<<<<<<<<<<<<
 *                 (<__pyx_buffer *> &self.view).obj = Py_None
 *                 Py_INCREF(Py_None)
 */
    }

    /* "View.MemoryView":322
 *         self.obj = obj
 *         self.flags = flags
 *         if type(self) is memoryview or obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_GetBuffer(obj, &self.view, flags)
 *             if <PyObject *> self.view.obj == NULL:
 */
  }

  /* "View.MemoryView":328
 *                 Py_INCREF(Py_None)
 * 
 *         self.lock = PyThread_allocate_lock()             # <<<<<<<<<<<<<<
 *         if self.lock == NULL:
 *             raise MemoryError
 */
  __pyx_v_self->lock = PyThread_allocate_lock();

  /* "View.MemoryView":329
 * 
 *         self.lock = PyThread_allocate_lock()
 *         if self.lock == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->lock == NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":330
 *         self.lock = PyThread_allocate_lock()
 *         if self.lock == NULL:
 *             raise MemoryError             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_FORMAT:
 */
    PyErr_NoMemory(); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 330; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "View.MemoryView":329
 * 
 *         self.lock = PyThread_allocate_lock()
 *         if self.lock == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError
 * 
 */
  }

  /* "View.MemoryView":332
 *             raise MemoryError
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             self.dtype_is_object = self.view.format == b'O'
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":333
 * 
 *         if flags & PyBUF_FORMAT:
 *             self.dtype_is_object = self.view.format == b'O'             # <<<<<<<<<<<<<<
 *         else:
 *             self.dtype_is_object = dtype_is_object
 */
    __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 333; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyObject_RichCompare(__pyx_t_5, __pyx_n_b_O, Py_EQ); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 333; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 333; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_self->dtype_is_object = __pyx_t_1;

    /* "View.MemoryView":332
 *             raise MemoryError
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             self.dtype_is_object = self.view.format == b'O'
 *         else:
 */
    goto __pyx_L8;
  }

  /* "View.MemoryView":335
 *             self.dtype_is_object = self.view.format == b'O'
 *         else:
 *             self.dtype_is_object = dtype_is_object             # <<<<<<<<<<<<<<
 * 
 *         self.acquisition_count_aligned_p = <__pyx_atomic_int *> align_pointer(
 */
  /*else*/ {
    __pyx_v_self->dtype_is_object = __pyx_v_dtype_is_object;
  }
  __pyx_L8:;

  /* "View.MemoryView":337
 *             self.dtype_is_object = dtype_is_object
 * 
 *         self.acquisition_count_aligned_p = <__pyx_atomic_int *> align_pointer(             # <<<<<<<<<<<<<<
 *                   <void *> &self.acquisition_count[0], sizeof(__pyx_atomic_int))
 *         self.typeinfo = NULL
 */
  __pyx_v_self->acquisition_count_aligned_p = ((__pyx_atomic_int *)__pyx_align_pointer(((void *)(&(__pyx_v_self->acquisition_count[0]))), (sizeof(__pyx_atomic_int))));

  /* "View.MemoryView":339
 *         self.acquisition_count_aligned_p = <__pyx_atomic_int *> align_pointer(
 *                   <void *> &self.acquisition_count[0], sizeof(__pyx_atomic_int))
 *         self.typeinfo = NULL             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(memoryview self):
 */
  __pyx_v_self->typeinfo = NULL;

  /* "View.MemoryView":319
 *     cdef __Pyx_TypeInfo *typeinfo
 * 
 *     def __cinit__(memoryview self, object obj, int flags, bint dtype_is_object=False):             # <<<<<<<<<<<<<<
 *         self.obj = obj
 *         self.flags = flags
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":341
 *         self.typeinfo = NULL
 * 
 *     def __dealloc__(memoryview self):             # <<<<<<<<<<<<<<
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)
 */

/* Python wrapper */
static void __pyx_memoryview___dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_memoryview___dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_2__dealloc__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_2__dealloc__(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "View.MemoryView":342
 * 
 *     def __dealloc__(memoryview self):
 *         if self.obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_ReleaseBuffer(&self.view)
 * 
 */
  __pyx_t_1 = (__pyx_v_self->obj != Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":343
 *     def __dealloc__(memoryview self):
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)             # <<<<<<<<<<<<<<
 * 
 *         if self.lock != NULL:
 */
    __Pyx_ReleaseBuffer((&__pyx_v_self->view));

    /* "View.MemoryView":342
 * 
 *     def __dealloc__(memoryview self):
 *         if self.obj is not None:             # <<<<<<<<<<<<<<
 *             __Pyx_ReleaseBuffer(&self.view)
 * 
 */
  }

  /* "View.MemoryView":345
 *             __Pyx_ReleaseBuffer(&self.view)
 * 
 *         if self.lock != NULL:             # <<<<<<<<<<<<<<
 *             PyThread_free_lock(self.lock)
 * 
 */
  __pyx_t_2 = ((__pyx_v_self->lock != NULL) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":346
 * 
 *         if self.lock != NULL:
 *             PyThread_free_lock(self.lock)             # <<<<<<<<<<<<<<
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:
 */
    PyThread_free_lock(__pyx_v_self->lock);

    /* "View.MemoryView":345
 *             __Pyx_ReleaseBuffer(&self.view)
 * 
 *         if self.lock != NULL:             # <<<<<<<<<<<<<<
 *             PyThread_free_lock(self.lock)
 * 
 */
  }

  /* "View.MemoryView":341
 *         self.typeinfo = NULL
 * 
 *     def __dealloc__(memoryview self):             # <<<<<<<<<<<<<<
 *         if self.obj is not None:
 *             __Pyx_ReleaseBuffer(&self.view)
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "View.MemoryView":348
 *             PyThread_free_lock(self.lock)
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf
 */

static char *__pyx_memoryview_get_item_pointer(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index) {
  Py_ssize_t __pyx_v_dim;
  char *__pyx_v_itemp;
  PyObject *__pyx_v_idx = NULL;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  char *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_item_pointer", 0);

  /* "View.MemoryView":350
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf             # <<<<<<<<<<<<<<
 * 
 *         for dim, idx in enumerate(index):
 */
  __pyx_v_itemp = ((char *)__pyx_v_self->view.buf);

  /* "View.MemoryView":352
 *         cdef char *itemp = <char *> self.view.buf
 * 
 *         for dim, idx in enumerate(index):             # <<<<<<<<<<<<<<
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
 */
  __pyx_t_1 = 0;
  if (likely(PyList_CheckExact(__pyx_v_index)) || PyTuple_CheckExact(__pyx_v_index)) {
    __pyx_t_2 = __pyx_v_index; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_v_index); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 352; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 352; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_5 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 352; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 352; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_5 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_5); __pyx_t_3++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 352; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_5 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 352; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_5);
        #endif
      }
    } else {
      __pyx_t_5 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_5)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[2]; __pyx_lineno = 352; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_XDECREF_SET(__pyx_v_idx, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_v_dim = __pyx_t_1;
    __pyx_t_1 = (__pyx_t_1 + 1);

    /* "View.MemoryView":353
 * 
 *         for dim, idx in enumerate(index):
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)             # <<<<<<<<<<<<<<
 * 
 *         return itemp
 */
    __pyx_t_6 = __Pyx_PyIndex_AsSsize_t(__pyx_v_idx); if (unlikely((__pyx_t_6 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 353; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_7 = __pyx_pybuffer_index((&__pyx_v_self->view), __pyx_v_itemp, __pyx_t_6, __pyx_v_dim); if (unlikely(__pyx_t_7 == NULL)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 353; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_itemp = __pyx_t_7;

    /* "View.MemoryView":352
 *         cdef char *itemp = <char *> self.view.buf
 * 
 *         for dim, idx in enumerate(index):             # <<<<<<<<<<<<<<
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":355
 *             itemp = pybuffer_index(&self.view, itemp, idx, dim)
 * 
 *         return itemp             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_itemp;
  goto __pyx_L0;

  /* "View.MemoryView":348
 *             PyThread_free_lock(self.lock)
 * 
 *     cdef char *get_item_pointer(memoryview self, object index) except NULL:             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t dim
 *         cdef char *itemp = <char *> self.view.buf
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.memoryview.get_item_pointer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_idx);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":358
 * 
 * 
 *     def __getitem__(memoryview self, object index):             # <<<<<<<<<<<<<<
 *         if index is Ellipsis:
 *             return self
 */

/* Python wrapper */
static PyObject *__pyx_memoryview___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index); /*proto*/
static PyObject *__pyx_memoryview___getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_4__getitem__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((PyObject *)__pyx_v_index));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_4__getitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_v_have_slices = NULL;
  PyObject *__pyx_v_indices = NULL;
  char *__pyx_v_itemp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  char *__pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "View.MemoryView":359
 * 
 *     def __getitem__(memoryview self, object index):
 *         if index is Ellipsis:             # <<<<<<<<<<<<<<
 *             return self
 * 
 */
  __pyx_t_1 = (__pyx_v_index == __pyx_builtin_Ellipsis);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":360
 *     def __getitem__(memoryview self, object index):
 *         if index is Ellipsis:
 *             return self             # <<<<<<<<<<<<<<
 * 
 *         have_slices, indices = _unellipsify(index, self.view.ndim)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __pyx_r = ((PyObject *)__pyx_v_self);
    goto __pyx_L0;

    /* "View.MemoryView":359
 * 
 *     def __getitem__(memoryview self, object index):
 *         if index is Ellipsis:             # <<<<<<<<<<<<<<
 *             return self
 * 
 */
  }

  /* "View.MemoryView":362
 *             return self
 * 
 *         have_slices, indices = _unellipsify(index, self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *         cdef char *itemp
 */
  __pyx_t_3 = _unellipsify(__pyx_v_index, __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 362; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  if (likely(__pyx_t_3 != Py_None)) {
    PyObject* sequence = __pyx_t_3;
    #if CYTHON_COMPILING_IN_CPYTHON
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 362; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0); 
    __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1); 
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_5);
    #else
    __pyx_t_4 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 362; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 362; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    #endif
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __Pyx_RaiseNoneNotIterableError(); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 362; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_v_have_slices = __pyx_t_4;
  __pyx_t_4 = 0;
  __pyx_v_indices = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "View.MemoryView":365
 * 
 *         cdef char *itemp
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             return memview_slice(self, indices)
 *         else:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_have_slices); if (unlikely(__pyx_t_2 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 365; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_2) {

    /* "View.MemoryView":366
 *         cdef char *itemp
 *         if have_slices:
 *             return memview_slice(self, indices)             # <<<<<<<<<<<<<<
 *         else:
 *             itemp = self.get_item_pointer(indices)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = ((PyObject *)__pyx_memview_slice(__pyx_v_self, __pyx_v_indices)); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 366; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "View.MemoryView":365
 * 
 *         cdef char *itemp
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             return memview_slice(self, indices)
 *         else:
 */
  }

  /* "View.MemoryView":368
 *             return memview_slice(self, indices)
 *         else:
 *             itemp = self.get_item_pointer(indices)             # <<<<<<<<<<<<<<
 *             return self.convert_item_to_object(itemp)
 * 
 */
  /*else*/ {
    __pyx_t_6 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->get_item_pointer(__pyx_v_self, __pyx_v_indices); if (unlikely(__pyx_t_6 == NULL)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 368; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_itemp = __pyx_t_6;

    /* "View.MemoryView":369
 *         else:
 *             itemp = self.get_item_pointer(indices)
 *             return self.convert_item_to_object(itemp)             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(memoryview self, object index, object value):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->convert_item_to_object(__pyx_v_self, __pyx_v_itemp); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 369; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "View.MemoryView":358
 * 
 * 
 *     def __getitem__(memoryview self, object index):             # <<<<<<<<<<<<<<
 *         if index is Ellipsis:
 *             return self
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_have_slices);
  __Pyx_XDECREF(__pyx_v_indices);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":371
 *             return self.convert_item_to_object(itemp)
 * 
 *     def __setitem__(memoryview self, object index, object value):             # <<<<<<<<<<<<<<
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 * 
 */

/* Python wrapper */
static int __pyx_memoryview___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /*proto*/
static int __pyx_memoryview___setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_6__setitem__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((PyObject *)__pyx_v_index), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_6__setitem__(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_have_slices = NULL;
  PyObject *__pyx_v_obj = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);
  __Pyx_INCREF(__pyx_v_index);

  /* "View.MemoryView":372
 * 
 *     def __setitem__(memoryview self, object index, object value):
 *         have_slices, index = _unellipsify(index, self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *         if have_slices:
 */
  __pyx_t_1 = _unellipsify(__pyx_v_index, __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 372; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(__pyx_t_1 != Py_None)) {
    PyObject* sequence = __pyx_t_1;
    #if CYTHON_COMPILING_IN_CPYTHON
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 372; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    #if CYTHON_COMPILING_IN_CPYTHON
    __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
    __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 372; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 372; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    __Pyx_RaiseNoneNotIterableError(); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 372; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_v_have_slices = __pyx_t_2;
  __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_index, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "View.MemoryView":374
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 * 
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             obj = self.is_slice(value)
 *             if obj:
 */
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_have_slices); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 374; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_t_4) {

    /* "View.MemoryView":375
 * 
 *         if have_slices:
 *             obj = self.is_slice(value)             # <<<<<<<<<<<<<<
 *             if obj:
 *                 self.setitem_slice_assignment(self[index], obj)
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->is_slice(__pyx_v_self, __pyx_v_value); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 375; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_obj = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "View.MemoryView":376
 *         if have_slices:
 *             obj = self.is_slice(value)
 *             if obj:             # <<<<<<<<<<<<<<
 *                 self.setitem_slice_assignment(self[index], obj)
 *             else:
 */
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_v_obj); if (unlikely(__pyx_t_4 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 376; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (__pyx_t_4) {

      /* "View.MemoryView":377
 *             obj = self.is_slice(value)
 *             if obj:
 *                 self.setitem_slice_assignment(self[index], obj)             # <<<<<<<<<<<<<<
 *             else:
 *                 self.setitem_slice_assign_scalar(self[index], value)
 */
      __pyx_t_1 = PyObject_GetItem(((PyObject *)__pyx_v_self), __pyx_v_index); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 377; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_3 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_slice_assignment(__pyx_v_self, __pyx_t_1, __pyx_v_obj); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 377; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "View.MemoryView":376
 *         if have_slices:
 *             obj = self.is_slice(value)
 *             if obj:             # <<<<<<<<<<<<<<
 *                 self.setitem_slice_assignment(self[index], obj)
 *             else:
 */
      goto __pyx_L4;
    }

    /* "View.MemoryView":379
 *                 self.setitem_slice_assignment(self[index], obj)
 *             else:
 *                 self.setitem_slice_assign_scalar(self[index], value)             # <<<<<<<<<<<<<<
 *         else:
 *             self.setitem_indexed(index, value)
 */
    /*else*/ {
      __pyx_t_3 = PyObject_GetItem(((PyObject *)__pyx_v_self), __pyx_v_index); if (unlikely(__pyx_t_3 == NULL)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 379; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
      __Pyx_GOTREF(__pyx_t_3);
      if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_memoryview_type))))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 379; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_t_1 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_slice_assign_scalar(__pyx_v_self, ((struct __pyx_memoryview_obj *)__pyx_t_3), __pyx_v_value); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 379; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __pyx_L4:;

    /* "View.MemoryView":374
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 * 
 *         if have_slices:             # <<<<<<<<<<<<<<
 *             obj = self.is_slice(value)
 *             if obj:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":381
 *                 self.setitem_slice_assign_scalar(self[index], value)
 *         else:
 *             self.setitem_indexed(index, value)             # <<<<<<<<<<<<<<
 * 
 *     cdef is_slice(self, obj):
 */
  /*else*/ {
    __pyx_t_1 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->setitem_indexed(__pyx_v_self, __pyx_v_index, __pyx_v_value); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 381; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "View.MemoryView":371
 *             return self.convert_item_to_object(itemp)
 * 
 *     def __setitem__(memoryview self, object index, object value):             # <<<<<<<<<<<<<<
 *         have_slices, index = _unellipsify(index, self.view.ndim)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_have_slices);
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":383
 *             self.setitem_indexed(index, value)
 * 
 *     cdef is_slice(self, obj):             # <<<<<<<<<<<<<<
 *         if not isinstance(obj, memoryview):
 *             try:
 */

static PyObject *__pyx_memoryview_is_slice(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_obj) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_slice", 0);
  __Pyx_INCREF(__pyx_v_obj);

  /* "View.MemoryView":384
 * 
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):             # <<<<<<<<<<<<<<
 *             try:
 *                 obj = memoryview(obj, self.flags|PyBUF_ANY_CONTIGUOUS,
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_obj, __pyx_memoryview_type); 
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":385
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):
 *             try:             # <<<<<<<<<<<<<<
 *                 obj = memoryview(obj, self.flags|PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 */
    {
      __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      /*try:*/ {

        /* "View.MemoryView":386
 *         if not isinstance(obj, memoryview):
 *             try:
 *                 obj = memoryview(obj, self.flags|PyBUF_ANY_CONTIGUOUS,             # <<<<<<<<<<<<<<
 *                                  self.dtype_is_object)
 *             except TypeError:
 */
        __pyx_t_6 = __Pyx_PyInt_From_int((__pyx_v_self->flags | PyBUF_ANY_CONTIGUOUS)); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 386; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
        __Pyx_GOTREF(__pyx_t_6);

        /* "View.MemoryView":387
 *             try:
 *                 obj = memoryview(obj, self.flags|PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)             # <<<<<<<<<<<<<<
 *             except TypeError:
 *                 return None
 */
        __pyx_t_7 = __Pyx_PyBool_FromLong(__pyx_v_self->dtype_is_object); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 387; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
        __Pyx_GOTREF(__pyx_t_7);

        /* "View.MemoryView":386
 *         if not isinstance(obj, memoryview):
 *             try:
 *                 obj = memoryview(obj, self.flags|PyBUF_ANY_CONTIGUOUS,             # <<<<<<<<<<<<<<
 *                                  self.dtype_is_object)
 *             except TypeError:
 */
        __pyx_t_8 = PyTuple_New(3); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 386; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_INCREF(__pyx_v_obj);
        __Pyx_GIVEREF(__pyx_v_obj);
        PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_v_obj);
        __Pyx_GIVEREF(__pyx_t_6);
        PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_7);
        PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_7);
        __pyx_t_6 = 0;
        __pyx_t_7 = 0;
        __pyx_t_7 = __Pyx_PyObject_Call(((PyObject *)__pyx_memoryview_type), __pyx_t_8, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 386; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF_SET(__pyx_v_obj, __pyx_t_7);
        __pyx_t_7 = 0;

        /* "View.MemoryView":385
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):
 *             try:             # <<<<<<<<<<<<<<
 *                 obj = memoryview(obj, self.flags|PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 */
      }
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      goto __pyx_L11_try_end;
      __pyx_L4_error:;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;

      /* "View.MemoryView":388
 *                 obj = memoryview(obj, self.flags|PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 *             except TypeError:             # <<<<<<<<<<<<<<
 *                 return None
 * 
 */
      __pyx_t_9 = PyErr_ExceptionMatches(__pyx_builtin_TypeError);
      if (__pyx_t_9) {
        __Pyx_AddTraceback("View.MemoryView.memoryview.is_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_7, &__pyx_t_8, &__pyx_t_6) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 388; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_GOTREF(__pyx_t_6);

        /* "View.MemoryView":389
 *                                  self.dtype_is_object)
 *             except TypeError:
 *                 return None             # <<<<<<<<<<<<<<
 * 
 *         return obj
 */
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(Py_None);
        __pyx_r = Py_None;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L7_except_return;
      }
      goto __pyx_L6_except_error;
      __pyx_L6_except_error:;

      /* "View.MemoryView":385
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):
 *             try:             # <<<<<<<<<<<<<<
 *                 obj = memoryview(obj, self.flags|PyBUF_ANY_CONTIGUOUS,
 *                                  self.dtype_is_object)
 */
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L1_error;
      __pyx_L7_except_return:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L0;
      __pyx_L11_try_end:;
    }

    /* "View.MemoryView":384
 * 
 *     cdef is_slice(self, obj):
 *         if not isinstance(obj, memoryview):             # <<<<<<<<<<<<<<
 *             try:
 *                 obj = memoryview(obj, self.flags|PyBUF_ANY_CONTIGUOUS,
 */
  }

  /* "View.MemoryView":391
 *                 return None
 * 
 *         return obj             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_slice_assignment(self, dst, src):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_obj);
  __pyx_r = __pyx_v_obj;
  goto __pyx_L0;

  /* "View.MemoryView":383
 *             self.setitem_indexed(index, value)
 * 
 *     cdef is_slice(self, obj):             # <<<<<<<<<<<<<<
 *         if not isinstance(obj, memoryview):
 *             try:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":393
 *         return obj
 * 
 *     cdef setitem_slice_assignment(self, dst, src):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice dst_slice
 *         cdef __Pyx_memviewslice src_slice
 */

static PyObject *__pyx_memoryview_setitem_slice_assignment(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_dst, PyObject *__pyx_v_src) {
  __Pyx_memviewslice __pyx_v_dst_slice;
  __Pyx_memviewslice __pyx_v_src_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setitem_slice_assignment", 0);

  /* "View.MemoryView":397
 *         cdef __Pyx_memviewslice src_slice
 * 
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],             # <<<<<<<<<<<<<<
 *                                  get_slice_from_memview(dst, &dst_slice)[0],
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 */
  if (!(likely(((__pyx_v_src) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_src, __pyx_memoryview_type))))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "View.MemoryView":398
 * 
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],
 *                                  get_slice_from_memview(dst, &dst_slice)[0],             # <<<<<<<<<<<<<<
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 * 
 */
  if (!(likely(((__pyx_v_dst) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_dst, __pyx_memoryview_type))))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 398; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "View.MemoryView":399
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],
 *                                  get_slice_from_memview(dst, &dst_slice)[0],
 *                                  src.ndim, dst.ndim, self.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_src, __pyx_n_s_ndim); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_dst, __pyx_n_s_ndim); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "View.MemoryView":397
 *         cdef __Pyx_memviewslice src_slice
 * 
 *         memoryview_copy_contents(get_slice_from_memview(src, &src_slice)[0],             # <<<<<<<<<<<<<<
 *                                  get_slice_from_memview(dst, &dst_slice)[0],
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 */
  __pyx_t_4 = __pyx_memoryview_copy_contents((__pyx_memoryview_get_slice_from_memoryview(((struct __pyx_memoryview_obj *)__pyx_v_src), (&__pyx_v_src_slice))[0]), (__pyx_memoryview_get_slice_from_memoryview(((struct __pyx_memoryview_obj *)__pyx_v_dst), (&__pyx_v_dst_slice))[0]), __pyx_t_2, __pyx_t_3, __pyx_v_self->dtype_is_object); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 397; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "View.MemoryView":393
 *         return obj
 * 
 *     cdef setitem_slice_assignment(self, dst, src):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice dst_slice
 *         cdef __Pyx_memviewslice src_slice
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_slice_assignment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":401
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):             # <<<<<<<<<<<<<<
 *         cdef int array[128]
 *         cdef void *tmp = NULL
 */

static PyObject *__pyx_memoryview_setitem_slice_assign_scalar(struct __pyx_memoryview_obj *__pyx_v_self, struct __pyx_memoryview_obj *__pyx_v_dst, PyObject *__pyx_v_value) {
  int __pyx_v_array[0x80];
  void *__pyx_v_tmp;
  void *__pyx_v_item;
  __Pyx_memviewslice *__pyx_v_dst_slice;
  __Pyx_memviewslice __pyx_v_tmp_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  char const *__pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setitem_slice_assign_scalar", 0);

  /* "View.MemoryView":403
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):
 *         cdef int array[128]
 *         cdef void *tmp = NULL             # <<<<<<<<<<<<<<
 *         cdef void *item
 * 
 */
  __pyx_v_tmp = NULL;

  /* "View.MemoryView":408
 *         cdef __Pyx_memviewslice *dst_slice
 *         cdef __Pyx_memviewslice tmp_slice
 *         dst_slice = get_slice_from_memview(dst, &tmp_slice)             # <<<<<<<<<<<<<<
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):
 */
  __pyx_v_dst_slice = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_dst, (&__pyx_v_tmp_slice));

  /* "View.MemoryView":410
 *         dst_slice = get_slice_from_memview(dst, &tmp_slice)
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):             # <<<<<<<<<<<<<<
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:
 */
  __pyx_t_1 = ((((size_t)__pyx_v_self->view.itemsize) > (sizeof(__pyx_v_array))) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":411
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):
 *             tmp = PyMem_Malloc(self.view.itemsize)             # <<<<<<<<<<<<<<
 *             if tmp == NULL:
 *                 raise MemoryError
 */
    __pyx_v_tmp = PyMem_Malloc(__pyx_v_self->view.itemsize);

    /* "View.MemoryView":412
 *         if <size_t>self.view.itemsize > sizeof(array):
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 *             item = tmp
 */
    __pyx_t_1 = ((__pyx_v_tmp == NULL) != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":413
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:
 *                 raise MemoryError             # <<<<<<<<<<<<<<
 *             item = tmp
 *         else:
 */
      PyErr_NoMemory(); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 413; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "View.MemoryView":412
 *         if <size_t>self.view.itemsize > sizeof(array):
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:             # <<<<<<<<<<<<<<
 *                 raise MemoryError
 *             item = tmp
 */
    }

    /* "View.MemoryView":414
 *             if tmp == NULL:
 *                 raise MemoryError
 *             item = tmp             # <<<<<<<<<<<<<<
 *         else:
 *             item = <void *> array
 */
    __pyx_v_item = __pyx_v_tmp;

    /* "View.MemoryView":410
 *         dst_slice = get_slice_from_memview(dst, &tmp_slice)
 * 
 *         if <size_t>self.view.itemsize > sizeof(array):             # <<<<<<<<<<<<<<
 *             tmp = PyMem_Malloc(self.view.itemsize)
 *             if tmp == NULL:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":416
 *             item = tmp
 *         else:
 *             item = <void *> array             # <<<<<<<<<<<<<<
 * 
 *         try:
 */
  /*else*/ {
    __pyx_v_item = ((void *)__pyx_v_array);
  }
  __pyx_L3:;

  /* "View.MemoryView":418
 *             item = <void *> array
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             if self.dtype_is_object:
 *                 (<PyObject **> item)[0] = <PyObject *> value
 */
  /*try:*/ {

    /* "View.MemoryView":419
 * 
 *         try:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 (<PyObject **> item)[0] = <PyObject *> value
 *             else:
 */
    __pyx_t_1 = (__pyx_v_self->dtype_is_object != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":420
 *         try:
 *             if self.dtype_is_object:
 *                 (<PyObject **> item)[0] = <PyObject *> value             # <<<<<<<<<<<<<<
 *             else:
 *                 self.assign_item_from_object(<char *> item, value)
 */
      (((PyObject **)__pyx_v_item)[0]) = ((PyObject *)__pyx_v_value);

      /* "View.MemoryView":419
 * 
 *         try:
 *             if self.dtype_is_object:             # <<<<<<<<<<<<<<
 *                 (<PyObject **> item)[0] = <PyObject *> value
 *             else:
 */
      goto __pyx_L8;
    }

    /* "View.MemoryView":422
 *                 (<PyObject **> item)[0] = <PyObject *> value
 *             else:
 *                 self.assign_item_from_object(<char *> item, value)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    /*else*/ {
      __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->assign_item_from_object(__pyx_v_self, ((char *)__pyx_v_item), __pyx_v_value); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 422; __pyx_clineno = __LINE__; goto __pyx_L6_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_L8:;

    /* "View.MemoryView":426
 * 
 * 
 *             if self.view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,
 */
    __pyx_t_1 = ((__pyx_v_self->view.suboffsets != NULL) != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":427
 * 
 *             if self.view.suboffsets != NULL:
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)             # <<<<<<<<<<<<<<
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,
 *                                 item, self.dtype_is_object)
 */
      __pyx_t_2 = assert_direct_dimensions(__pyx_v_self->view.suboffsets, __pyx_v_self->view.ndim); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 427; __pyx_clineno = __LINE__; goto __pyx_L6_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "View.MemoryView":426
 * 
 * 
 *             if self.view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,
 */
    }

    /* "View.MemoryView":428
 *             if self.view.suboffsets != NULL:
 *                 assert_direct_dimensions(self.view.suboffsets, self.view.ndim)
 *             slice_assign_scalar(dst_slice, dst.view.ndim, self.view.itemsize,             # <<<<<<<<<<<<<<
 *                                 item, self.dtype_is_object)
 *         finally:
 */
    __pyx_memoryview_slice_assign_scalar(__pyx_v_dst_slice, __pyx_v_dst->view.ndim, __pyx_v_self->view.itemsize, __pyx_v_item, __pyx_v_self->dtype_is_object);
  }

  /* "View.MemoryView":431
 *                                 item, self.dtype_is_object)
 *         finally:
 *             PyMem_Free(tmp)             # <<<<<<<<<<<<<<
 * 
 *     cdef setitem_indexed(self, index, value):
 */
  /*finally:*/ {
    /*normal exit:*/{
      PyMem_Free(__pyx_v_tmp);
      goto __pyx_L7;
    }
    /*exception exit:*/{
      __pyx_L6_error:;
      __pyx_t_6 = 0; __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
      if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8) < 0)) __Pyx_ErrFetch(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_11);
      __pyx_t_3 = __pyx_lineno; __pyx_t_4 = __pyx_clineno; __pyx_t_5 = __pyx_filename;
      {
        PyMem_Free(__pyx_v_tmp);
      }
      if (PY_MAJOR_VERSION >= 3) {
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
      }
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_ErrRestore(__pyx_t_6, __pyx_t_7, __pyx_t_8);
      __pyx_t_6 = 0; __pyx_t_7 = 0; __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; __pyx_t_11 = 0;
      __pyx_lineno = __pyx_t_3; __pyx_clineno = __pyx_t_4; __pyx_filename = __pyx_t_5;
      goto __pyx_L1_error;
    }
    __pyx_L7:;
  }

  /* "View.MemoryView":401
 *                                  src.ndim, dst.ndim, self.dtype_is_object)
 * 
 *     cdef setitem_slice_assign_scalar(self, memoryview dst, value):             # <<<<<<<<<<<<<<
 *         cdef int array[128]
 *         cdef void *tmp = NULL
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_slice_assign_scalar", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":433
 *             PyMem_Free(tmp)
 * 
 *     cdef setitem_indexed(self, index, value):             # <<<<<<<<<<<<<<
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)
 */

static PyObject *__pyx_memoryview_setitem_indexed(struct __pyx_memoryview_obj *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  char *__pyx_v_itemp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  char *__pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("setitem_indexed", 0);

  /* "View.MemoryView":434
 * 
 *     cdef setitem_indexed(self, index, value):
 *         cdef char *itemp = self.get_item_pointer(index)             # <<<<<<<<<<<<<<
 *         self.assign_item_from_object(itemp, value)
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->get_item_pointer(__pyx_v_self, __pyx_v_index); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_itemp = __pyx_t_1;

  /* "View.MemoryView":435
 *     cdef setitem_indexed(self, index, value):
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)             # <<<<<<<<<<<<<<
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_memoryview *)__pyx_v_self->__pyx_vtab)->assign_item_from_object(__pyx_v_self, __pyx_v_itemp, __pyx_v_value); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 435; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":433
 *             PyMem_Free(tmp)
 * 
 *     cdef setitem_indexed(self, index, value):             # <<<<<<<<<<<<<<
 *         cdef char *itemp = self.get_item_pointer(index)
 *         self.assign_item_from_object(itemp, value)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.setitem_indexed", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":437
 *         self.assign_item_from_object(itemp, value)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

static PyObject *__pyx_memoryview_convert_item_to_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp) {
  PyObject *__pyx_v_struct = NULL;
  PyObject *__pyx_v_bytesitem = 0;
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("convert_item_to_object", 0);

  /* "View.MemoryView":440
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 *         import struct             # <<<<<<<<<<<<<<
 *         cdef bytes bytesitem
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_struct, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 440; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "View.MemoryView":443
 *         cdef bytes bytesitem
 * 
 *         bytesitem = itemp[:self.view.itemsize]             # <<<<<<<<<<<<<<
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)
 */
  __pyx_t_1 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_itemp + 0, __pyx_v_self->view.itemsize - 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 443; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_bytesitem = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "View.MemoryView":444
 * 
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:             # <<<<<<<<<<<<<<
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 */
  {
    __Pyx_ExceptionSave(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_2);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    /*try:*/ {

      /* "View.MemoryView":445
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)             # <<<<<<<<<<<<<<
 *         except struct.error:
 *             raise ValueError("Unable to convert item to object")
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_unpack); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_7 = NULL;
      __pyx_t_8 = 0;
      if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_8 = 1;
        }
      }
      __pyx_t_9 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      __Pyx_GOTREF(__pyx_t_9);
      if (__pyx_t_7) {
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_t_6);
      __Pyx_INCREF(__pyx_v_bytesitem);
      __Pyx_GIVEREF(__pyx_v_bytesitem);
      PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_v_bytesitem);
      __pyx_t_6 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 445; __pyx_clineno = __LINE__; goto __pyx_L3_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_v_result = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "View.MemoryView":444
 * 
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:             # <<<<<<<<<<<<<<
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 */
    }

    /* "View.MemoryView":449
 *             raise ValueError("Unable to convert item to object")
 *         else:
 *             if len(self.view.format) == 1:             # <<<<<<<<<<<<<<
 *                 return result[0]
 *             return result
 */
    /*else:*/ {
      __pyx_t_10 = strlen(__pyx_v_self->view.format); 
      __pyx_t_11 = ((__pyx_t_10 == 1) != 0);
      if (__pyx_t_11) {

        /* "View.MemoryView":450
 *         else:
 *             if len(self.view.format) == 1:
 *                 return result[0]             # <<<<<<<<<<<<<<
 *             return result
 * 
 */
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_result, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 0); if (unlikely(__pyx_t_1 == NULL)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 450; __pyx_clineno = __LINE__; goto __pyx_L5_except_error;};
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_r = __pyx_t_1;
        __pyx_t_1 = 0;
        goto __pyx_L6_except_return;

        /* "View.MemoryView":449
 *             raise ValueError("Unable to convert item to object")
 *         else:
 *             if len(self.view.format) == 1:             # <<<<<<<<<<<<<<
 *                 return result[0]
 *             return result
 */
      }

      /* "View.MemoryView":451
 *             if len(self.view.format) == 1:
 *                 return result[0]
 *             return result             # <<<<<<<<<<<<<<
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 */
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_v_result);
      __pyx_r = __pyx_v_result;
      goto __pyx_L6_except_return;
    }
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "View.MemoryView":446
 *         try:
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:             # <<<<<<<<<<<<<<
 *             raise ValueError("Unable to convert item to object")
 *         else:
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_error); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 446; __pyx_clineno = __LINE__; goto __pyx_L5_except_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_12 = PyErr_ExceptionMatches(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_12) {
      __Pyx_AddTraceback("View.MemoryView.memoryview.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_5, &__pyx_t_9) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 446; __pyx_clineno = __LINE__; goto __pyx_L5_except_error;}
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GOTREF(__pyx_t_9);

      /* "View.MemoryView":447
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 *             raise ValueError("Unable to convert item to object")             # <<<<<<<<<<<<<<
 *         else:
 *             if len(self.view.format) == 1:
 */
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__389, NULL); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 447; __pyx_clineno = __LINE__; goto __pyx_L5_except_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 447; __pyx_clineno = __LINE__; goto __pyx_L5_except_error;}
    }
    goto __pyx_L5_except_error;
    __pyx_L5_except_error:;

    /* "View.MemoryView":444
 * 
 *         bytesitem = itemp[:self.view.itemsize]
 *         try:             # <<<<<<<<<<<<<<
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 */
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L1_error;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_2);
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_ExceptionReset(__pyx_t_2, __pyx_t_3, __pyx_t_4);
    goto __pyx_L0;
  }

  /* "View.MemoryView":437
 *         self.assign_item_from_object(itemp, value)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("View.MemoryView.memoryview.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_bytesitem);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":453
 *             return result
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

static PyObject *__pyx_memoryview_assign_item_from_object(struct __pyx_memoryview_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_struct = NULL;
  char __pyx_v_c;
  PyObject *__pyx_v_bytesvalue = 0;
  Py_ssize_t __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  char *__pyx_t_10;
  char *__pyx_t_11;
  char *__pyx_t_12;
  char *__pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("assign_item_from_object", 0);

  /* "View.MemoryView":456
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 *         import struct             # <<<<<<<<<<<<<<
 *         cdef char c
 *         cdef bytes bytesvalue
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_struct, 0, 0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 456; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_struct = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "View.MemoryView":461
 *         cdef Py_ssize_t i
 * 
 *         if isinstance(value, tuple):             # <<<<<<<<<<<<<<
 *             bytesvalue = struct.pack(self.view.format, *value)
 *         else:
 */
  __pyx_t_2 = PyTuple_Check(__pyx_v_value); 
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "View.MemoryView":462
 * 
 *         if isinstance(value, tuple):
 *             bytesvalue = struct.pack(self.view.format, *value)             # <<<<<<<<<<<<<<
 *         else:
 *             bytesvalue = struct.pack(self.view.format, value)
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = PySequence_Tuple(__pyx_v_value); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = PyNumber_Add(__pyx_t_5, __pyx_t_4); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_6, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (!(likely(PyBytes_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_4)->tp_name), 0))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 462; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_bytesvalue = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;

    /* "View.MemoryView":461
 *         cdef Py_ssize_t i
 * 
 *         if isinstance(value, tuple):             # <<<<<<<<<<<<<<
 *             bytesvalue = struct.pack(self.view.format, *value)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":464
 *             bytesvalue = struct.pack(self.view.format, *value)
 *         else:
 *             bytesvalue = struct.pack(self.view.format, value)             # <<<<<<<<<<<<<<
 * 
 *         for i, c in enumerate(bytesvalue):
 */
  /*else*/ {
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_struct, __pyx_n_s_pack); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 464; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_self->view.format); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 464; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = NULL;
    __pyx_t_7 = 0;
    if (CYTHON_COMPILING_IN_CPYTHON && likely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_6, function);
        __pyx_t_7 = 1;
      }
    }
    __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 464; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_value);
    __Pyx_GIVEREF(__pyx_v_value);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_v_value);
    __pyx_t_1 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 464; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (!(likely(PyBytes_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_4)->tp_name), 0))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 464; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_bytesvalue = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;
  }
  __pyx_L3:;

  /* "View.MemoryView":466
 *             bytesvalue = struct.pack(self.view.format, value)
 * 
 *         for i, c in enumerate(bytesvalue):             # <<<<<<<<<<<<<<
 *             itemp[i] = c
 * 
 */
  __pyx_t_7 = 0;
  if (unlikely(__pyx_v_bytesvalue == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' is not iterable");
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 466; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __Pyx_INCREF(__pyx_v_bytesvalue);
  __pyx_t_9 = __pyx_v_bytesvalue;
  __pyx_t_11 = PyBytes_AS_STRING(__pyx_t_9);
  __pyx_t_12 = (__pyx_t_11 + PyBytes_GET_SIZE(__pyx_t_9));
  for (__pyx_t_13 = __pyx_t_11; __pyx_t_13 < __pyx_t_12; __pyx_t_13++) {
    __pyx_t_10 = __pyx_t_13;
    __pyx_v_c = (__pyx_t_10[0]);

    /* "View.MemoryView":467
 * 
 *         for i, c in enumerate(bytesvalue):
 *             itemp[i] = c             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
 */
    __pyx_v_i = __pyx_t_7;

    /* "View.MemoryView":466
 *             bytesvalue = struct.pack(self.view.format, value)
 * 
 *         for i, c in enumerate(bytesvalue):             # <<<<<<<<<<<<<<
 *             itemp[i] = c
 * 
 */
    __pyx_t_7 = (__pyx_t_7 + 1);

    /* "View.MemoryView":467
 * 
 *         for i, c in enumerate(bytesvalue):
 *             itemp[i] = c             # <<<<<<<<<<<<<<
 * 
 *     @cname('getbuffer')
 */
    (__pyx_v_itemp[__pyx_v_i]) = __pyx_v_c;
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

  /* "View.MemoryView":453
 *             return result
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         """Only used if instantiated manually by the user, or if Cython doesn't
 *         know how to convert the type"""
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("View.MemoryView.memoryview.assign_item_from_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_struct);
  __Pyx_XDECREF(__pyx_v_bytesvalue);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":470
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         if flags & PyBUF_STRIDES:
 *             info.shape = self.view.shape
 */

/* Python wrapper */
static CYTHON_UNUSED int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags); /*proto*/
static CYTHON_UNUSED int __pyx_memoryview_getbuffer(PyObject *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getbuffer__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_8__getbuffer__(((struct __pyx_memoryview_obj *)__pyx_v_self), ((Py_buffer *)__pyx_v_info), ((int)__pyx_v_flags));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_8__getbuffer__(struct __pyx_memoryview_obj *__pyx_v_self, Py_buffer *__pyx_v_info, int __pyx_v_flags) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  Py_ssize_t *__pyx_t_2;
  char *__pyx_t_3;
  void *__pyx_t_4;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  __Pyx_RefNannySetupContext("__getbuffer__", 0);
  if (__pyx_v_info != NULL) {
    __pyx_v_info->obj = Py_None; __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(__pyx_v_info->obj);
  }

  /* "View.MemoryView":471
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_STRIDES:             # <<<<<<<<<<<<<<
 *             info.shape = self.view.shape
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_STRIDES) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":472
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_STRIDES:
 *             info.shape = self.view.shape             # <<<<<<<<<<<<<<
 *         else:
 *             info.shape = NULL
 */
    __pyx_t_2 = __pyx_v_self->view.shape;
    __pyx_v_info->shape = __pyx_t_2;

    /* "View.MemoryView":471
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):
 *         if flags & PyBUF_STRIDES:             # <<<<<<<<<<<<<<
 *             info.shape = self.view.shape
 *         else:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":474
 *             info.shape = self.view.shape
 *         else:
 *             info.shape = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_STRIDES:
 */
  /*else*/ {
    __pyx_v_info->shape = NULL;
  }
  __pyx_L3:;

  /* "View.MemoryView":476
 *             info.shape = NULL
 * 
 *         if flags & PyBUF_STRIDES:             # <<<<<<<<<<<<<<
 *             info.strides = self.view.strides
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_STRIDES) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":477
 * 
 *         if flags & PyBUF_STRIDES:
 *             info.strides = self.view.strides             # <<<<<<<<<<<<<<
 *         else:
 *             info.strides = NULL
 */
    __pyx_t_2 = __pyx_v_self->view.strides;
    __pyx_v_info->strides = __pyx_t_2;

    /* "View.MemoryView":476
 *             info.shape = NULL
 * 
 *         if flags & PyBUF_STRIDES:             # <<<<<<<<<<<<<<
 *             info.strides = self.view.strides
 *         else:
 */
    goto __pyx_L4;
  }

  /* "View.MemoryView":479
 *             info.strides = self.view.strides
 *         else:
 *             info.strides = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_INDIRECT:
 */
  /*else*/ {
    __pyx_v_info->strides = NULL;
  }
  __pyx_L4:;

  /* "View.MemoryView":481
 *             info.strides = NULL
 * 
 *         if flags & PyBUF_INDIRECT:             # <<<<<<<<<<<<<<
 *             info.suboffsets = self.view.suboffsets
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_INDIRECT) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":482
 * 
 *         if flags & PyBUF_INDIRECT:
 *             info.suboffsets = self.view.suboffsets             # <<<<<<<<<<<<<<
 *         else:
 *             info.suboffsets = NULL
 */
    __pyx_t_2 = __pyx_v_self->view.suboffsets;
    __pyx_v_info->suboffsets = __pyx_t_2;

    /* "View.MemoryView":481
 *             info.strides = NULL
 * 
 *         if flags & PyBUF_INDIRECT:             # <<<<<<<<<<<<<<
 *             info.suboffsets = self.view.suboffsets
 *         else:
 */
    goto __pyx_L5;
  }

  /* "View.MemoryView":484
 *             info.suboffsets = self.view.suboffsets
 *         else:
 *             info.suboffsets = NULL             # <<<<<<<<<<<<<<
 * 
 *         if flags & PyBUF_FORMAT:
 */
  /*else*/ {
    __pyx_v_info->suboffsets = NULL;
  }
  __pyx_L5:;

  /* "View.MemoryView":486
 *             info.suboffsets = NULL
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.view.format
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_flags & PyBUF_FORMAT) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":487
 * 
 *         if flags & PyBUF_FORMAT:
 *             info.format = self.view.format             # <<<<<<<<<<<<<<
 *         else:
 *             info.format = NULL
 */
    __pyx_t_3 = __pyx_v_self->view.format;
    __pyx_v_info->format = __pyx_t_3;

    /* "View.MemoryView":486
 *             info.suboffsets = NULL
 * 
 *         if flags & PyBUF_FORMAT:             # <<<<<<<<<<<<<<
 *             info.format = self.view.format
 *         else:
 */
    goto __pyx_L6;
  }

  /* "View.MemoryView":489
 *             info.format = self.view.format
 *         else:
 *             info.format = NULL             # <<<<<<<<<<<<<<
 * 
 *         info.buf = self.view.buf
 */
  /*else*/ {
    __pyx_v_info->format = NULL;
  }
  __pyx_L6:;

  /* "View.MemoryView":491
 *             info.format = NULL
 * 
 *         info.buf = self.view.buf             # <<<<<<<<<<<<<<
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize
 */
  __pyx_t_4 = __pyx_v_self->view.buf;
  __pyx_v_info->buf = __pyx_t_4;

  /* "View.MemoryView":492
 * 
 *         info.buf = self.view.buf
 *         info.ndim = self.view.ndim             # <<<<<<<<<<<<<<
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len
 */
  __pyx_t_5 = __pyx_v_self->view.ndim;
  __pyx_v_info->ndim = __pyx_t_5;

  /* "View.MemoryView":493
 *         info.buf = self.view.buf
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize             # <<<<<<<<<<<<<<
 *         info.len = self.view.len
 *         info.readonly = 0
 */
  __pyx_t_6 = __pyx_v_self->view.itemsize;
  __pyx_v_info->itemsize = __pyx_t_6;

  /* "View.MemoryView":494
 *         info.ndim = self.view.ndim
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len             # <<<<<<<<<<<<<<
 *         info.readonly = 0
 *         info.obj = self
 */
  __pyx_t_6 = __pyx_v_self->view.len;
  __pyx_v_info->len = __pyx_t_6;

  /* "View.MemoryView":495
 *         info.itemsize = self.view.itemsize
 *         info.len = self.view.len
 *         info.readonly = 0             # <<<<<<<<<<<<<<
 *         info.obj = self
 * 
 */
  __pyx_v_info->readonly = 0;

  /* "View.MemoryView":496
 *         info.len = self.view.len
 *         info.readonly = 0
 *         info.obj = self             # <<<<<<<<<<<<<<
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  __Pyx_GOTREF(__pyx_v_info->obj);
  __Pyx_DECREF(__pyx_v_info->obj);
  __pyx_v_info->obj = ((PyObject *)__pyx_v_self);

  /* "View.MemoryView":470
 * 
 *     @cname('getbuffer')
 *     def __getbuffer__(self, Py_buffer *info, int flags):             # <<<<<<<<<<<<<<
 *         if flags & PyBUF_STRIDES:
 *             info.shape = self.view.shape
 */

  /* function exit code */
  __pyx_r = 0;
  if (__pyx_v_info != NULL && __pyx_v_info->obj == Py_None) {
    __Pyx_GOTREF(Py_None);
    __Pyx_DECREF(Py_None); __pyx_v_info->obj = NULL;
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":503
 *     property T:
 *         @cname('__pyx_memoryview_transpose')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             cdef _memoryviewslice result = memoryview_copy(self)
 *             transpose_memslice(&result.from_slice)
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_transpose(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_transpose(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_1T___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_1T___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  struct __pyx_memoryviewslice_obj *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":504
 *         @cname('__pyx_memoryview_transpose')
 *         def __get__(self):
 *             cdef _memoryviewslice result = memoryview_copy(self)             # <<<<<<<<<<<<<<
 *             transpose_memslice(&result.from_slice)
 *             return result
 */
  __pyx_t_1 = __pyx_memoryview_copy_object(__pyx_v_self); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 504; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_memoryviewslice_type))))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 504; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_result = ((struct __pyx_memoryviewslice_obj *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "View.MemoryView":505
 *         def __get__(self):
 *             cdef _memoryviewslice result = memoryview_copy(self)
 *             transpose_memslice(&result.from_slice)             # <<<<<<<<<<<<<<
 *             return result
 * 
 */
  __pyx_t_2 = __pyx_memslice_transpose((&__pyx_v_result->from_slice)); if (unlikely(__pyx_t_2 == 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 505; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "View.MemoryView":506
 *             cdef _memoryviewslice result = memoryview_copy(self)
 *             transpose_memslice(&result.from_slice)
 *             return result             # <<<<<<<<<<<<<<
 * 
 *     property base:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "View.MemoryView":503
 *     property T:
 *         @cname('__pyx_memoryview_transpose')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             cdef _memoryviewslice result = memoryview_copy(self)
 *             transpose_memslice(&result.from_slice)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.T.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":510
 *     property base:
 *         @cname('__pyx_memoryview__get__base')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.obj
 * 
 */

/* Python wrapper */
static PyObject *__pyx_memoryview__get__base(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview__get__base(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_4base___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4base___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":511
 *         @cname('__pyx_memoryview__get__base')
 *         def __get__(self):
 *             return self.obj             # <<<<<<<<<<<<<<
 * 
 *     property shape:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->obj);
  __pyx_r = __pyx_v_self->obj;
  goto __pyx_L0;

  /* "View.MemoryView":510
 *     property base:
 *         @cname('__pyx_memoryview__get__base')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.obj
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":515
 *     property shape:
 *         @cname('__pyx_memoryview_get_shape')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return tuple([length for length in self.view.shape[:self.view.ndim]])
 * 
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_get_shape(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_shape(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_5shape___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_5shape___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_v_length;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t *__pyx_t_2;
  Py_ssize_t *__pyx_t_3;
  Py_ssize_t *__pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":516
 *         @cname('__pyx_memoryview_get_shape')
 *         def __get__(self):
 *             return tuple([length for length in self.view.shape[:self.view.ndim]])             # <<<<<<<<<<<<<<
 * 
 *     property strides:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 516; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = (__pyx_v_self->view.shape + __pyx_v_self->view.ndim);
  for (__pyx_t_4 = __pyx_v_self->view.shape; __pyx_t_4 < __pyx_t_3; __pyx_t_4++) {
    __pyx_t_2 = __pyx_t_4;
    __pyx_v_length = (__pyx_t_2[0]);
    __pyx_t_5 = PyInt_FromSsize_t(__pyx_v_length); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 516; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_5))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 516; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __pyx_t_5 = PyList_AsTuple(((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 516; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":515
 *     property shape:
 *         @cname('__pyx_memoryview_get_shape')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return tuple([length for length in self.view.shape[:self.view.ndim]])
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.memoryview.shape.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":520
 *     property strides:
 *         @cname('__pyx_memoryview_get_strides')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.view.strides == NULL:
 * 
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_get_strides(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_strides(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_7strides___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_7strides___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_v_stride;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t *__pyx_t_3;
  Py_ssize_t *__pyx_t_4;
  Py_ssize_t *__pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":521
 *         @cname('__pyx_memoryview_get_strides')
 *         def __get__(self):
 *             if self.view.strides == NULL:             # <<<<<<<<<<<<<<
 * 
 *                 raise ValueError("Buffer view does not expose strides")
 */
  __pyx_t_1 = ((__pyx_v_self->view.strides == NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":523
 *             if self.view.strides == NULL:
 * 
 *                 raise ValueError("Buffer view does not expose strides")             # <<<<<<<<<<<<<<
 * 
 *             return tuple([stride for stride in self.view.strides[:self.view.ndim]])
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__390, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 523; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 523; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "View.MemoryView":521
 *         @cname('__pyx_memoryview_get_strides')
 *         def __get__(self):
 *             if self.view.strides == NULL:             # <<<<<<<<<<<<<<
 * 
 *                 raise ValueError("Buffer view does not expose strides")
 */
  }

  /* "View.MemoryView":525
 *                 raise ValueError("Buffer view does not expose strides")
 * 
 *             return tuple([stride for stride in self.view.strides[:self.view.ndim]])             # <<<<<<<<<<<<<<
 * 
 *     property suboffsets:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 525; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = (__pyx_v_self->view.strides + __pyx_v_self->view.ndim);
  for (__pyx_t_5 = __pyx_v_self->view.strides; __pyx_t_5 < __pyx_t_4; __pyx_t_5++) {
    __pyx_t_3 = __pyx_t_5;
    __pyx_v_stride = (__pyx_t_3[0]);
    __pyx_t_6 = PyInt_FromSsize_t(__pyx_v_stride); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 525; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_6);
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_6))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 525; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __pyx_t_6 = PyList_AsTuple(((PyObject*)__pyx_t_2)); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 525; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":520
 *     property strides:
 *         @cname('__pyx_memoryview_get_strides')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.view.strides == NULL:
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("View.MemoryView.memoryview.strides.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":529
 *     property suboffsets:
 *         @cname('__pyx_memoryview_get_suboffsets')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.view.suboffsets == NULL:
 *                 return (-1,) * self.view.ndim
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_get_suboffsets(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_suboffsets(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_10suboffsets___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_10suboffsets___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_v_suboffset;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_ssize_t *__pyx_t_4;
  Py_ssize_t *__pyx_t_5;
  Py_ssize_t *__pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":530
 *         @cname('__pyx_memoryview_get_suboffsets')
 *         def __get__(self):
 *             if self.view.suboffsets == NULL:             # <<<<<<<<<<<<<<
 *                 return (-1,) * self.view.ndim
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->view.suboffsets == NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":531
 *         def __get__(self):
 *             if self.view.suboffsets == NULL:
 *                 return (-1,) * self.view.ndim             # <<<<<<<<<<<<<<
 * 
 *             return tuple([suboffset for suboffset in self.view.suboffsets[:self.view.ndim]])
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->view.ndim); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyNumber_Multiply(__pyx_tuple__391, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "View.MemoryView":530
 *         @cname('__pyx_memoryview_get_suboffsets')
 *         def __get__(self):
 *             if self.view.suboffsets == NULL:             # <<<<<<<<<<<<<<
 *                 return (-1,) * self.view.ndim
 * 
 */
  }

  /* "View.MemoryView":533
 *                 return (-1,) * self.view.ndim
 * 
 *             return tuple([suboffset for suboffset in self.view.suboffsets[:self.view.ndim]])             # <<<<<<<<<<<<<<
 * 
 *     property ndim:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = (__pyx_v_self->view.suboffsets + __pyx_v_self->view.ndim);
  for (__pyx_t_6 = __pyx_v_self->view.suboffsets; __pyx_t_6 < __pyx_t_5; __pyx_t_6++) {
    __pyx_t_4 = __pyx_t_6;
    __pyx_v_suboffset = (__pyx_t_4[0]);
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_suboffset); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_3, (PyObject*)__pyx_t_2))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_t_2 = PyList_AsTuple(((PyObject*)__pyx_t_3)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 533; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":529
 *     property suboffsets:
 *         @cname('__pyx_memoryview_get_suboffsets')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self.view.suboffsets == NULL:
 *                 return (-1,) * self.view.ndim
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview.suboffsets.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":537
 *     property ndim:
 *         @cname('__pyx_memoryview_get_ndim')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.view.ndim
 * 
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_get_ndim(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_ndim(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_4ndim___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4ndim___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":538
 *         @cname('__pyx_memoryview_get_ndim')
 *         def __get__(self):
 *             return self.view.ndim             # <<<<<<<<<<<<<<
 * 
 *     property itemsize:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->view.ndim); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 538; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":537
 *     property ndim:
 *         @cname('__pyx_memoryview_get_ndim')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.view.ndim
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.ndim.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":542
 *     property itemsize:
 *         @cname('__pyx_memoryview_get_itemsize')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.view.itemsize
 * 
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_get_itemsize(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_itemsize(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_8itemsize___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_8itemsize___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":543
 *         @cname('__pyx_memoryview_get_itemsize')
 *         def __get__(self):
 *             return self.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     property nbytes:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyInt_FromSsize_t(__pyx_v_self->view.itemsize); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 543; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":542
 *     property itemsize:
 *         @cname('__pyx_memoryview_get_itemsize')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.view.itemsize
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.itemsize.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":547
 *     property nbytes:
 *         @cname('__pyx_memoryview_get_nbytes')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.size * self.view.itemsize
 * 
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_get_nbytes(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_nbytes(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_6nbytes___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_6nbytes___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":548
 *         @cname('__pyx_memoryview_get_nbytes')
 *         def __get__(self):
 *             return self.size * self.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     property size:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_size); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 548; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_self->view.itemsize); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 548; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 548; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":547
 *     property nbytes:
 *         @cname('__pyx_memoryview_get_nbytes')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.size * self.view.itemsize
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview.nbytes.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":552
 *     property size:
 *         @cname('__pyx_memoryview_get_size')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self._size is None:
 *                 result = 1
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_get_size(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview_get_size(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_10memoryview_4size___get__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_10memoryview_4size___get__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_v_result = NULL;
  PyObject *__pyx_v_length = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t *__pyx_t_3;
  Py_ssize_t *__pyx_t_4;
  Py_ssize_t *__pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":553
 *         @cname('__pyx_memoryview_get_size')
 *         def __get__(self):
 *             if self._size is None:             # <<<<<<<<<<<<<<
 *                 result = 1
 * 
 */
  __pyx_t_1 = (__pyx_v_self->_size == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":554
 *         def __get__(self):
 *             if self._size is None:
 *                 result = 1             # <<<<<<<<<<<<<<
 * 
 *                 for length in self.view.shape[:self.view.ndim]:
 */
    __Pyx_INCREF(__pyx_int_1);
    __pyx_v_result = __pyx_int_1;

    /* "View.MemoryView":556
 *                 result = 1
 * 
 *                 for length in self.view.shape[:self.view.ndim]:             # <<<<<<<<<<<<<<
 *                     result *= length
 * 
 */
    __pyx_t_4 = (__pyx_v_self->view.shape + __pyx_v_self->view.ndim);
    for (__pyx_t_5 = __pyx_v_self->view.shape; __pyx_t_5 < __pyx_t_4; __pyx_t_5++) {
      __pyx_t_3 = __pyx_t_5;
      __pyx_t_6 = PyInt_FromSsize_t((__pyx_t_3[0])); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 556; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_XDECREF_SET(__pyx_v_length, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "View.MemoryView":557
 * 
 *                 for length in self.view.shape[:self.view.ndim]:
 *                     result *= length             # <<<<<<<<<<<<<<
 * 
 *                 self._size = result
 */
      __pyx_t_6 = PyNumber_InPlaceMultiply(__pyx_v_result, __pyx_v_length); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 557; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF_SET(__pyx_v_result, __pyx_t_6);
      __pyx_t_6 = 0;
    }

    /* "View.MemoryView":559
 *                     result *= length
 * 
 *                 self._size = result             # <<<<<<<<<<<<<<
 * 
 *             return self._size
 */
    __Pyx_INCREF(__pyx_v_result);
    __Pyx_GIVEREF(__pyx_v_result);
    __Pyx_GOTREF(__pyx_v_self->_size);
    __Pyx_DECREF(__pyx_v_self->_size);
    __pyx_v_self->_size = __pyx_v_result;

    /* "View.MemoryView":553
 *         @cname('__pyx_memoryview_get_size')
 *         def __get__(self):
 *             if self._size is None:             # <<<<<<<<<<<<<<
 *                 result = 1
 * 
 */
  }

  /* "View.MemoryView":561
 *                 self._size = result
 * 
 *             return self._size             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_size);
  __pyx_r = __pyx_v_self->_size;
  goto __pyx_L0;

  /* "View.MemoryView":552
 *     property size:
 *         @cname('__pyx_memoryview_get_size')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             if self._size is None:
 *                 result = 1
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("View.MemoryView.memoryview.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_length);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":563
 *             return self._size
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]
 */

/* Python wrapper */
static Py_ssize_t __pyx_memoryview___len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_memoryview___len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_10__len__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_10__len__(struct __pyx_memoryview_obj *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "View.MemoryView":564
 * 
 *     def __len__(self):
 *         if self.view.ndim >= 1:             # <<<<<<<<<<<<<<
 *             return self.view.shape[0]
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->view.ndim >= 1) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":565
 *     def __len__(self):
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
    __pyx_r = (__pyx_v_self->view.shape[0]);
    goto __pyx_L0;

    /* "View.MemoryView":564
 * 
 *     def __len__(self):
 *         if self.view.ndim >= 1:             # <<<<<<<<<<<<<<
 *             return self.view.shape[0]
 * 
 */
  }

  /* "View.MemoryView":567
 *             return self.view.shape[0]
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "View.MemoryView":563
 *             return self._size
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         if self.view.ndim >= 1:
 *             return self.view.shape[0]
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":569
 *         return 0
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))
 */

/* Python wrapper */
static PyObject *__pyx_memoryview___repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview___repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_12__repr__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_12__repr__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "View.MemoryView":570
 * 
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,             # <<<<<<<<<<<<<<
 *                                                id(self))
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_base); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_class); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_name_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":571
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_id, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "View.MemoryView":570
 * 
 *     def __repr__(self):
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,             # <<<<<<<<<<<<<<
 *                                                id(self))
 * 
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_MemoryView_of_r_at_0x_x, __pyx_t_2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 570; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":569
 *         return 0
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r at 0x%x>" % (self.base.__class__.__name__,
 *                                                id(self))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":573
 *                                                id(self))
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_memoryview___str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryview___str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_14__str__(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_14__str__(struct __pyx_memoryview_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "View.MemoryView":574
 * 
 *     def __str__(self):
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_base); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_class); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_name_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_MemoryView_of_r_object, __pyx_t_2); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":573
 *                                                id(self))
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return "<MemoryView of %r object>" % (self.base.__class__.__name__,)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":577
 * 
 * 
 *     def is_c_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_is_c_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_memoryview_is_c_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_c_contig (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_16is_c_contig(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_16is_c_contig(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice *__pyx_v_mslice;
  __Pyx_memviewslice __pyx_v_tmp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_c_contig", 0);

  /* "View.MemoryView":580
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)             # <<<<<<<<<<<<<<
 *         return slice_is_contig(mslice, 'C', self.view.ndim)
 * 
 */
  __pyx_v_mslice = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_self, (&__pyx_v_tmp));

  /* "View.MemoryView":581
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)
 *         return slice_is_contig(mslice, 'C', self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *     def is_f_contig(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_memviewslice_is_contig(__pyx_v_mslice, 'C', __pyx_v_self->view.ndim)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 581; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":577
 * 
 * 
 *     def is_c_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_c_contig", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":583
 *         return slice_is_contig(mslice, 'C', self.view.ndim)
 * 
 *     def is_f_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_is_f_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_memoryview_is_f_contig(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_f_contig (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_18is_f_contig(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_18is_f_contig(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice *__pyx_v_mslice;
  __Pyx_memviewslice __pyx_v_tmp;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_f_contig", 0);

  /* "View.MemoryView":586
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)             # <<<<<<<<<<<<<<
 *         return slice_is_contig(mslice, 'F', self.view.ndim)
 * 
 */
  __pyx_v_mslice = __pyx_memoryview_get_slice_from_memoryview(__pyx_v_self, (&__pyx_v_tmp));

  /* "View.MemoryView":587
 *         cdef __Pyx_memviewslice tmp
 *         mslice = get_slice_from_memview(self, &tmp)
 *         return slice_is_contig(mslice, 'F', self.view.ndim)             # <<<<<<<<<<<<<<
 * 
 *     def copy(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_memviewslice_is_contig(__pyx_v_mslice, 'F', __pyx_v_self->view.ndim)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 587; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":583
 *         return slice_is_contig(mslice, 'C', self.view.ndim)
 * 
 *     def is_f_contig(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice *mslice
 *         cdef __Pyx_memviewslice tmp
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview.is_f_contig", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":589
 *         return slice_is_contig(mslice, 'F', self.view.ndim)
 * 
 *     def copy(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_memoryview_copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_20copy(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_20copy(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice __pyx_v_mslice;
  int __pyx_v_flags;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy", 0);

  /* "View.MemoryView":591
 *     def copy(self):
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS             # <<<<<<<<<<<<<<
 * 
 *         slice_copy(self, &mslice)
 */
  __pyx_v_flags = (__pyx_v_self->flags & (~PyBUF_F_CONTIGUOUS));

  /* "View.MemoryView":593
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
 * 
 *         slice_copy(self, &mslice)             # <<<<<<<<<<<<<<
 *         mslice = slice_copy_contig(&mslice, "c", self.view.ndim,
 *                                    self.view.itemsize,
 */
  __pyx_memoryview_slice_copy(__pyx_v_self, (&__pyx_v_mslice));

  /* "View.MemoryView":594
 * 
 *         slice_copy(self, &mslice)
 *         mslice = slice_copy_contig(&mslice, "c", self.view.ndim,             # <<<<<<<<<<<<<<
 *                                    self.view.itemsize,
 *                                    flags|PyBUF_C_CONTIGUOUS,
 */
  __pyx_t_1 = __pyx_memoryview_copy_new_contig((&__pyx_v_mslice), __pyx_k_c, __pyx_v_self->view.ndim, __pyx_v_self->view.itemsize, (__pyx_v_flags | PyBUF_C_CONTIGUOUS), __pyx_v_self->dtype_is_object); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 594; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_mslice = __pyx_t_1;

  /* "View.MemoryView":599
 *                                    self.dtype_is_object)
 * 
 *         return memoryview_copy_from_slice(self, &mslice)             # <<<<<<<<<<<<<<
 * 
 *     def copy_fortran(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_memoryview_copy_object_from_slice(__pyx_v_self, (&__pyx_v_mslice)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 599; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":589
 *         return slice_is_contig(mslice, 'F', self.view.ndim)
 * 
 *     def copy(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice mslice
 *         cdef int flags = self.flags & ~PyBUF_F_CONTIGUOUS
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":601
 *         return memoryview_copy_from_slice(self, &mslice)
 * 
 *     def copy_fortran(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
 */

/* Python wrapper */
static PyObject *__pyx_memoryview_copy_fortran(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_memoryview_copy_fortran(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy_fortran (wrapper)", 0);
  __pyx_r = __pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_22copy_fortran(((struct __pyx_memoryview_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_memoryview___pyx_pf_15View_dot_MemoryView_10memoryview_22copy_fortran(struct __pyx_memoryview_obj *__pyx_v_self) {
  __Pyx_memviewslice __pyx_v_src;
  __Pyx_memviewslice __pyx_v_dst;
  int __pyx_v_flags;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_memviewslice __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy_fortran", 0);

  /* "View.MemoryView":603
 *     def copy_fortran(self):
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS             # <<<<<<<<<<<<<<
 * 
 *         slice_copy(self, &src)
 */
  __pyx_v_flags = (__pyx_v_self->flags & (~PyBUF_C_CONTIGUOUS));

  /* "View.MemoryView":605
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
 * 
 *         slice_copy(self, &src)             # <<<<<<<<<<<<<<
 *         dst = slice_copy_contig(&src, "fortran", self.view.ndim,
 *                                 self.view.itemsize,
 */
  __pyx_memoryview_slice_copy(__pyx_v_self, (&__pyx_v_src));

  /* "View.MemoryView":606
 * 
 *         slice_copy(self, &src)
 *         dst = slice_copy_contig(&src, "fortran", self.view.ndim,             # <<<<<<<<<<<<<<
 *                                 self.view.itemsize,
 *                                 flags|PyBUF_F_CONTIGUOUS,
 */
  __pyx_t_1 = __pyx_memoryview_copy_new_contig((&__pyx_v_src), __pyx_k_fortran, __pyx_v_self->view.ndim, __pyx_v_self->view.itemsize, (__pyx_v_flags | PyBUF_F_CONTIGUOUS), __pyx_v_self->dtype_is_object); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 606; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_dst = __pyx_t_1;

  /* "View.MemoryView":611
 *                                 self.dtype_is_object)
 * 
 *         return memoryview_copy_from_slice(self, &dst)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_memoryview_copy_object_from_slice(__pyx_v_self, (&__pyx_v_dst)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 611; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":601
 *         return memoryview_copy_from_slice(self, &mslice)
 * 
 *     def copy_fortran(self):             # <<<<<<<<<<<<<<
 *         cdef __Pyx_memviewslice src, dst
 *         cdef int flags = self.flags & ~PyBUF_C_CONTIGUOUS
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView.memoryview.copy_fortran", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":615
 * 
 * @cname('__pyx_memoryview_new')
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):             # <<<<<<<<<<<<<<
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo
 */

static PyObject *__pyx_memoryview_new(PyObject *__pyx_v_o, int __pyx_v_flags, int __pyx_v_dtype_is_object, __Pyx_TypeInfo *__pyx_v_typeinfo) {
  struct __pyx_memoryview_obj *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memoryview_cwrapper", 0);

  /* "View.MemoryView":616
 * @cname('__pyx_memoryview_new')
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)             # <<<<<<<<<<<<<<
 *     result.typeinfo = typeinfo
 *     return result
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_flags); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 616; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_dtype_is_object); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 616; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 616; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_o);
  __Pyx_GIVEREF(__pyx_v_o);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_o);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_memoryview_type), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 616; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_result = ((struct __pyx_memoryview_obj *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "View.MemoryView":617
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo             # <<<<<<<<<<<<<<
 *     return result
 * 
 */
  __pyx_v_result->typeinfo = __pyx_v_typeinfo;

  /* "View.MemoryView":618
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo
 *     return result             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_check')
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "View.MemoryView":615
 * 
 * @cname('__pyx_memoryview_new')
 * cdef memoryview_cwrapper(object o, int flags, bint dtype_is_object, __Pyx_TypeInfo *typeinfo):             # <<<<<<<<<<<<<<
 *     cdef memoryview result = memoryview(o, flags, dtype_is_object)
 *     result.typeinfo = typeinfo
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview_cwrapper", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":621
 * 
 * @cname('__pyx_memoryview_check')
 * cdef inline bint memoryview_check(object o):             # <<<<<<<<<<<<<<
 *     return isinstance(o, memoryview)
 * 
 */

static CYTHON_INLINE int __pyx_memoryview_check(PyObject *__pyx_v_o) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("memoryview_check", 0);

  /* "View.MemoryView":622
 * @cname('__pyx_memoryview_check')
 * cdef inline bint memoryview_check(object o):
 *     return isinstance(o, memoryview)             # <<<<<<<<<<<<<<
 * 
 * cdef tuple _unellipsify(object index, int ndim):
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_o, __pyx_memoryview_type); 
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "View.MemoryView":621
 * 
 * @cname('__pyx_memoryview_check')
 * cdef inline bint memoryview_check(object o):             # <<<<<<<<<<<<<<
 *     return isinstance(o, memoryview)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":624
 *     return isinstance(o, memoryview)
 * 
 * cdef tuple _unellipsify(object index, int ndim):             # <<<<<<<<<<<<<<
 *     """
 *     Replace all ellipses with full slices and fill incomplete indices with
 */

static PyObject *_unellipsify(PyObject *__pyx_v_index, int __pyx_v_ndim) {
  PyObject *__pyx_v_tup = NULL;
  PyObject *__pyx_v_result = NULL;
  int __pyx_v_have_slices;
  int __pyx_v_seen_ellipsis;
  CYTHON_UNUSED PyObject *__pyx_v_idx = NULL;
  PyObject *__pyx_v_item = NULL;
  Py_ssize_t __pyx_v_nslices;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7 = NULL;
  Py_ssize_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_unellipsify", 0);

  /* "View.MemoryView":629
 *     full slices.
 *     """
 *     if not isinstance(index, tuple):             # <<<<<<<<<<<<<<
 *         tup = (index,)
 *     else:
 */
  __pyx_t_1 = PyTuple_Check(__pyx_v_index); 
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":630
 *     """
 *     if not isinstance(index, tuple):
 *         tup = (index,)             # <<<<<<<<<<<<<<
 *     else:
 *         tup = index
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 630; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_index);
    __Pyx_GIVEREF(__pyx_v_index);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_index);
    __pyx_v_tup = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "View.MemoryView":629
 *     full slices.
 *     """
 *     if not isinstance(index, tuple):             # <<<<<<<<<<<<<<
 *         tup = (index,)
 *     else:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":632
 *         tup = (index,)
 *     else:
 *         tup = index             # <<<<<<<<<<<<<<
 * 
 *     result = []
 */
  /*else*/ {
    __Pyx_INCREF(__pyx_v_index);
    __pyx_v_tup = __pyx_v_index;
  }
  __pyx_L3:;

  /* "View.MemoryView":634
 *         tup = index
 * 
 *     result = []             # <<<<<<<<<<<<<<
 *     have_slices = False
 *     seen_ellipsis = False
 */
  __pyx_t_3 = PyList_New(0); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 634; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_result = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "View.MemoryView":635
 * 
 *     result = []
 *     have_slices = False             # <<<<<<<<<<<<<<
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):
 */
  __pyx_v_have_slices = 0;

  /* "View.MemoryView":636
 *     result = []
 *     have_slices = False
 *     seen_ellipsis = False             # <<<<<<<<<<<<<<
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:
 */
  __pyx_v_seen_ellipsis = 0;

  /* "View.MemoryView":637
 *     have_slices = False
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):             # <<<<<<<<<<<<<<
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 */
  __Pyx_INCREF(__pyx_int_0);
  __pyx_t_3 = __pyx_int_0;
  if (likely(PyList_CheckExact(__pyx_v_tup)) || PyTuple_CheckExact(__pyx_v_tup)) {
    __pyx_t_4 = __pyx_v_tup; __Pyx_INCREF(__pyx_t_4); __pyx_t_5 = 0;
    __pyx_t_6 = NULL;
  } else {
    __pyx_t_5 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_v_tup); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 637; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_6 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 637; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  for (;;) {
    if (likely(!__pyx_t_6)) {
      if (likely(PyList_CheckExact(__pyx_t_4))) {
        if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_7 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 637; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 637; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_7);
        #endif
      } else {
        if (__pyx_t_5 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_7 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_5); __Pyx_INCREF(__pyx_t_7); __pyx_t_5++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 637; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_7 = PySequence_ITEM(__pyx_t_4, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 637; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_7);
        #endif
      }
    } else {
      __pyx_t_7 = __pyx_t_6(__pyx_t_4);
      if (unlikely(!__pyx_t_7)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[2]; __pyx_lineno = 637; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_7);
    }
    __Pyx_XDECREF_SET(__pyx_v_item, __pyx_t_7);
    __pyx_t_7 = 0;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_idx, __pyx_t_3);
    __pyx_t_7 = __Pyx_PyInt_AddObjC(__pyx_t_3, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 637; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3);
    __pyx_t_3 = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "View.MemoryView":638
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:             # <<<<<<<<<<<<<<
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 */
    __pyx_t_2 = (__pyx_v_item == __pyx_builtin_Ellipsis);
    __pyx_t_1 = (__pyx_t_2 != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":639
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:
 *             if not seen_ellipsis:             # <<<<<<<<<<<<<<
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 *                 seen_ellipsis = True
 */
      __pyx_t_1 = ((!(__pyx_v_seen_ellipsis != 0)) != 0);
      if (__pyx_t_1) {

        /* "View.MemoryView":640
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))             # <<<<<<<<<<<<<<
 *                 seen_ellipsis = True
 *             else:
 */
        __pyx_t_8 = PyObject_Length(__pyx_v_tup); if (unlikely(__pyx_t_8 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 640; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_7 = PyList_New(1 * ((((__pyx_v_ndim - __pyx_t_8) + 1)<0) ? 0:((__pyx_v_ndim - __pyx_t_8) + 1))); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 640; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_7);
        { Py_ssize_t __pyx_temp;
          for (__pyx_temp=0; __pyx_temp < ((__pyx_v_ndim - __pyx_t_8) + 1); __pyx_temp++) {
            __Pyx_INCREF(__pyx_slice__392);
            __Pyx_GIVEREF(__pyx_slice__392);
            PyList_SET_ITEM(__pyx_t_7, __pyx_temp, __pyx_slice__392);
          }
        }
        __pyx_t_9 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_t_7); if (unlikely(__pyx_t_9 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 640; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

        /* "View.MemoryView":641
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 *                 seen_ellipsis = True             # <<<<<<<<<<<<<<
 *             else:
 *                 result.append(slice(None))
 */
        __pyx_v_seen_ellipsis = 1;

        /* "View.MemoryView":639
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:
 *             if not seen_ellipsis:             # <<<<<<<<<<<<<<
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 *                 seen_ellipsis = True
 */
        goto __pyx_L7;
      }

      /* "View.MemoryView":643
 *                 seen_ellipsis = True
 *             else:
 *                 result.append(slice(None))             # <<<<<<<<<<<<<<
 *             have_slices = True
 *         else:
 */
      /*else*/ {
        __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_result, __pyx_slice__393); if (unlikely(__pyx_t_9 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 643; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_L7:;

      /* "View.MemoryView":644
 *             else:
 *                 result.append(slice(None))
 *             have_slices = True             # <<<<<<<<<<<<<<
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):
 */
      __pyx_v_have_slices = 1;

      /* "View.MemoryView":638
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):
 *         if item is Ellipsis:             # <<<<<<<<<<<<<<
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))
 */
      goto __pyx_L6;
    }

    /* "View.MemoryView":646
 *             have_slices = True
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):             # <<<<<<<<<<<<<<
 *                 raise TypeError("Cannot index with type '%s'" % type(item))
 * 
 */
    /*else*/ {
      __pyx_t_2 = PySlice_Check(__pyx_v_item); 
      __pyx_t_10 = ((!(__pyx_t_2 != 0)) != 0);
      if (__pyx_t_10) {
      } else {
        __pyx_t_1 = __pyx_t_10;
        goto __pyx_L9_bool_binop_done;
      }
      __pyx_t_10 = ((!(PyIndex_Check(__pyx_v_item) != 0)) != 0);
      __pyx_t_1 = __pyx_t_10;
      __pyx_L9_bool_binop_done:;
      if (__pyx_t_1) {

        /* "View.MemoryView":647
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):
 *                 raise TypeError("Cannot index with type '%s'" % type(item))             # <<<<<<<<<<<<<<
 * 
 *             have_slices = have_slices or isinstance(item, slice)
 */
        __pyx_t_7 = __Pyx_PyString_Format(__pyx_kp_s_Cannot_index_with_type_s, ((PyObject *)Py_TYPE(__pyx_v_item))); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 647; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_11 = PyTuple_New(1); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 647; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_GIVEREF(__pyx_t_7);
        PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_7);
        __pyx_t_7 = 0;
        __pyx_t_7 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_t_11, NULL); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 647; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_Raise(__pyx_t_7, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        {__pyx_filename = __pyx_f[2]; __pyx_lineno = 647; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

        /* "View.MemoryView":646
 *             have_slices = True
 *         else:
 *             if not isinstance(item, slice) and not PyIndex_Check(item):             # <<<<<<<<<<<<<<
 *                 raise TypeError("Cannot index with type '%s'" % type(item))
 * 
 */
      }

      /* "View.MemoryView":649
 *                 raise TypeError("Cannot index with type '%s'" % type(item))
 * 
 *             have_slices = have_slices or isinstance(item, slice)             # <<<<<<<<<<<<<<
 *             result.append(item)
 * 
 */
      __pyx_t_10 = (__pyx_v_have_slices != 0);
      if (!__pyx_t_10) {
      } else {
        __pyx_t_1 = __pyx_t_10;
        goto __pyx_L11_bool_binop_done;
      }
      __pyx_t_10 = PySlice_Check(__pyx_v_item); 
      __pyx_t_2 = (__pyx_t_10 != 0);
      __pyx_t_1 = __pyx_t_2;
      __pyx_L11_bool_binop_done:;
      __pyx_v_have_slices = __pyx_t_1;

      /* "View.MemoryView":650
 * 
 *             have_slices = have_slices or isinstance(item, slice)
 *             result.append(item)             # <<<<<<<<<<<<<<
 * 
 *     nslices = ndim - len(result)
 */
      __pyx_t_9 = __Pyx_PyList_Append(__pyx_v_result, __pyx_v_item); if (unlikely(__pyx_t_9 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 650; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_L6:;

    /* "View.MemoryView":637
 *     have_slices = False
 *     seen_ellipsis = False
 *     for idx, item in enumerate(tup):             # <<<<<<<<<<<<<<
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 */
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "View.MemoryView":652
 *             result.append(item)
 * 
 *     nslices = ndim - len(result)             # <<<<<<<<<<<<<<
 *     if nslices:
 *         result.extend([slice(None)] * nslices)
 */
  __pyx_t_5 = PyList_GET_SIZE(__pyx_v_result); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 652; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_nslices = (__pyx_v_ndim - __pyx_t_5);

  /* "View.MemoryView":653
 * 
 *     nslices = ndim - len(result)
 *     if nslices:             # <<<<<<<<<<<<<<
 *         result.extend([slice(None)] * nslices)
 * 
 */
  __pyx_t_1 = (__pyx_v_nslices != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":654
 *     nslices = ndim - len(result)
 *     if nslices:
 *         result.extend([slice(None)] * nslices)             # <<<<<<<<<<<<<<
 * 
 *     return have_slices or nslices, tuple(result)
 */
    __pyx_t_3 = PyList_New(1 * ((__pyx_v_nslices<0) ? 0:__pyx_v_nslices)); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 654; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    { Py_ssize_t __pyx_temp;
      for (__pyx_temp=0; __pyx_temp < __pyx_v_nslices; __pyx_temp++) {
        __Pyx_INCREF(__pyx_slice__394);
        __Pyx_GIVEREF(__pyx_slice__394);
        PyList_SET_ITEM(__pyx_t_3, __pyx_temp, __pyx_slice__394);
      }
    }
    __pyx_t_9 = __Pyx_PyList_Extend(__pyx_v_result, __pyx_t_3); if (unlikely(__pyx_t_9 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 654; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "View.MemoryView":653
 * 
 *     nslices = ndim - len(result)
 *     if nslices:             # <<<<<<<<<<<<<<
 *         result.extend([slice(None)] * nslices)
 * 
 */
  }

  /* "View.MemoryView":656
 *         result.extend([slice(None)] * nslices)
 * 
 *     return have_slices or nslices, tuple(result)             # <<<<<<<<<<<<<<
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 */
  __Pyx_XDECREF(__pyx_r);
  if (!__pyx_v_have_slices) {
  } else {
    __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_have_slices); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 656; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L14_bool_binop_done;
  }
  __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_nslices); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 656; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __pyx_t_4;
  __pyx_t_4 = 0;
  __pyx_L14_bool_binop_done:;
  __pyx_t_4 = PyList_AsTuple(__pyx_v_result); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 656; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 656; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_4);
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_r = ((PyObject*)__pyx_t_7);
  __pyx_t_7 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":624
 *     return isinstance(o, memoryview)
 * 
 * cdef tuple _unellipsify(object index, int ndim):             # <<<<<<<<<<<<<<
 *     """
 *     Replace all ellipses with full slices and fill incomplete indices with
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("View.MemoryView._unellipsify", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_tup);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_idx);
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":658
 *     return have_slices or nslices, tuple(result)
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):             # <<<<<<<<<<<<<<
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
 */

static PyObject *assert_direct_dimensions(Py_ssize_t *__pyx_v_suboffsets, int __pyx_v_ndim) {
  Py_ssize_t __pyx_v_suboffset;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t *__pyx_t_1;
  Py_ssize_t *__pyx_t_2;
  Py_ssize_t *__pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("assert_direct_dimensions", 0);

  /* "View.MemoryView":659
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 *     for suboffset in suboffsets[:ndim]:             # <<<<<<<<<<<<<<
 *         if suboffset >= 0:
 *             raise ValueError("Indirect dimensions not supported")
 */
  __pyx_t_2 = (__pyx_v_suboffsets + __pyx_v_ndim);
  for (__pyx_t_3 = __pyx_v_suboffsets; __pyx_t_3 < __pyx_t_2; __pyx_t_3++) {
    __pyx_t_1 = __pyx_t_3;
    __pyx_v_suboffset = (__pyx_t_1[0]);

    /* "View.MemoryView":660
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("Indirect dimensions not supported")
 * 
 */
    __pyx_t_4 = ((__pyx_v_suboffset >= 0) != 0);
    if (__pyx_t_4) {

      /* "View.MemoryView":661
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
 *             raise ValueError("Indirect dimensions not supported")             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__395, NULL); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 661; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 661; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "View.MemoryView":660
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             raise ValueError("Indirect dimensions not supported")
 * 
 */
    }
  }

  /* "View.MemoryView":658
 *     return have_slices or nslices, tuple(result)
 * 
 * cdef assert_direct_dimensions(Py_ssize_t *suboffsets, int ndim):             # <<<<<<<<<<<<<<
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.assert_direct_dimensions", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":668
 * 
 * @cname('__pyx_memview_slice')
 * cdef memoryview memview_slice(memoryview memview, object indices):             # <<<<<<<<<<<<<<
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim
 *     cdef bint negative_step
 */

static struct __pyx_memoryview_obj *__pyx_memview_slice(struct __pyx_memoryview_obj *__pyx_v_memview, PyObject *__pyx_v_indices) {
  int __pyx_v_new_ndim;
  int __pyx_v_suboffset_dim;
  int __pyx_v_dim;
  __Pyx_memviewslice __pyx_v_src;
  __Pyx_memviewslice __pyx_v_dst;
  __Pyx_memviewslice *__pyx_v_p_src;
  struct __pyx_memoryviewslice_obj *__pyx_v_memviewsliceobj = 0;
  __Pyx_memviewslice *__pyx_v_p_dst;
  int *__pyx_v_p_suboffset_dim;
  Py_ssize_t __pyx_v_start;
  Py_ssize_t __pyx_v_stop;
  Py_ssize_t __pyx_v_step;
  int __pyx_v_have_start;
  int __pyx_v_have_stop;
  int __pyx_v_have_step;
  PyObject *__pyx_v_index = NULL;
  struct __pyx_memoryview_obj *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  struct __pyx_memoryview_obj *__pyx_t_4;
  char *__pyx_t_5;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  Py_ssize_t __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memview_slice", 0);

  /* "View.MemoryView":669
 * @cname('__pyx_memview_slice')
 * cdef memoryview memview_slice(memoryview memview, object indices):
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim             # <<<<<<<<<<<<<<
 *     cdef bint negative_step
 *     cdef __Pyx_memviewslice src, dst
 */
  __pyx_v_new_ndim = 0;
  __pyx_v_suboffset_dim = -1;

  /* "View.MemoryView":676
 * 
 * 
 *     memset(&dst, 0, sizeof(dst))             # <<<<<<<<<<<<<<
 * 
 *     cdef _memoryviewslice memviewsliceobj
 */
  memset((&__pyx_v_dst), 0, (sizeof(__pyx_v_dst)));

  /* "View.MemoryView":680
 *     cdef _memoryviewslice memviewsliceobj
 * 
 *     assert memview.view.ndim > 0             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(memview, _memoryviewslice):
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_memview->view.ndim > 0) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 680; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif

  /* "View.MemoryView":682
 *     assert memview.view.ndim > 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         memviewsliceobj = memview
 *         p_src = &memviewsliceobj.from_slice
 */
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":683
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         memviewsliceobj = memview             # <<<<<<<<<<<<<<
 *         p_src = &memviewsliceobj.from_slice
 *     else:
 */
    if (!(likely(((((PyObject *)__pyx_v_memview)) == Py_None) || likely(__Pyx_TypeTest(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type))))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 683; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = ((PyObject *)__pyx_v_memview);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_memviewsliceobj = ((struct __pyx_memoryviewslice_obj *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "View.MemoryView":684
 *     if isinstance(memview, _memoryviewslice):
 *         memviewsliceobj = memview
 *         p_src = &memviewsliceobj.from_slice             # <<<<<<<<<<<<<<
 *     else:
 *         slice_copy(memview, &src)
 */
    __pyx_v_p_src = (&__pyx_v_memviewsliceobj->from_slice);

    /* "View.MemoryView":682
 *     assert memview.view.ndim > 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         memviewsliceobj = memview
 *         p_src = &memviewsliceobj.from_slice
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":686
 *         p_src = &memviewsliceobj.from_slice
 *     else:
 *         slice_copy(memview, &src)             # <<<<<<<<<<<<<<
 *         p_src = &src
 * 
 */
  /*else*/ {
    __pyx_memoryview_slice_copy(__pyx_v_memview, (&__pyx_v_src));

    /* "View.MemoryView":687
 *     else:
 *         slice_copy(memview, &src)
 *         p_src = &src             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_p_src = (&__pyx_v_src);
  }
  __pyx_L3:;

  /* "View.MemoryView":693
 * 
 * 
 *     dst.memview = p_src.memview             # <<<<<<<<<<<<<<
 *     dst.data = p_src.data
 * 
 */
  __pyx_t_4 = __pyx_v_p_src->memview;
  __pyx_v_dst.memview = __pyx_t_4;

  /* "View.MemoryView":694
 * 
 *     dst.memview = p_src.memview
 *     dst.data = p_src.data             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_5 = __pyx_v_p_src->data;
  __pyx_v_dst.data = __pyx_t_5;

  /* "View.MemoryView":699
 * 
 * 
 *     cdef __Pyx_memviewslice *p_dst = &dst             # <<<<<<<<<<<<<<
 *     cdef int *p_suboffset_dim = &suboffset_dim
 *     cdef Py_ssize_t start, stop, step
 */
  __pyx_v_p_dst = (&__pyx_v_dst);

  /* "View.MemoryView":700
 * 
 *     cdef __Pyx_memviewslice *p_dst = &dst
 *     cdef int *p_suboffset_dim = &suboffset_dim             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t start, stop, step
 *     cdef bint have_start, have_stop, have_step
 */
  __pyx_v_p_suboffset_dim = (&__pyx_v_suboffset_dim);

  /* "View.MemoryView":704
 *     cdef bint have_start, have_stop, have_step
 * 
 *     for dim, index in enumerate(indices):             # <<<<<<<<<<<<<<
 *         if PyIndex_Check(index):
 *             slice_memviewslice(
 */
  __pyx_t_6 = 0;
  if (likely(PyList_CheckExact(__pyx_v_indices)) || PyTuple_CheckExact(__pyx_v_indices)) {
    __pyx_t_3 = __pyx_v_indices; __Pyx_INCREF(__pyx_t_3); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_3 = PyObject_GetIter(__pyx_v_indices); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 704; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_8 = Py_TYPE(__pyx_t_3)->tp_iternext; if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 704; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_3))) {
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_9 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_7); __Pyx_INCREF(__pyx_t_9); __pyx_t_7++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 704; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_9 = PySequence_ITEM(__pyx_t_3, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 704; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_9);
        #endif
      } else {
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_3)) break;
        #if CYTHON_COMPILING_IN_CPYTHON
        __pyx_t_9 = PyTuple_GET_ITEM(__pyx_t_3, __pyx_t_7); __Pyx_INCREF(__pyx_t_9); __pyx_t_7++; if (unlikely(0 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 704; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        #else
        __pyx_t_9 = PySequence_ITEM(__pyx_t_3, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 704; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __Pyx_GOTREF(__pyx_t_9);
        #endif
      }
    } else {
      __pyx_t_9 = __pyx_t_8(__pyx_t_3);
      if (unlikely(!__pyx_t_9)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else {__pyx_filename = __pyx_f[2]; __pyx_lineno = 704; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_9);
    }
    __Pyx_XDECREF_SET(__pyx_v_index, __pyx_t_9);
    __pyx_t_9 = 0;
    __pyx_v_dim = __pyx_t_6;
    __pyx_t_6 = (__pyx_t_6 + 1);

    /* "View.MemoryView":705
 * 
 *     for dim, index in enumerate(indices):
 *         if PyIndex_Check(index):             # <<<<<<<<<<<<<<
 *             slice_memviewslice(
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 */
    __pyx_t_2 = (PyIndex_Check(__pyx_v_index) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":709
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 *                 dim, new_ndim, p_suboffset_dim,
 *                 index, 0, 0, # start, stop, step             # <<<<<<<<<<<<<<
 *                 0, 0, 0, # have_{start,stop,step}
 *                 False)
 */
      __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_v_index); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 709; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "View.MemoryView":706
 *     for dim, index in enumerate(indices):
 *         if PyIndex_Check(index):
 *             slice_memviewslice(             # <<<<<<<<<<<<<<
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 *                 dim, new_ndim, p_suboffset_dim,
 */
      __pyx_t_11 = __pyx_memoryview_slice_memviewslice(__pyx_v_p_dst, (__pyx_v_p_src->shape[__pyx_v_dim]), (__pyx_v_p_src->strides[__pyx_v_dim]), (__pyx_v_p_src->suboffsets[__pyx_v_dim]), __pyx_v_dim, __pyx_v_new_ndim, __pyx_v_p_suboffset_dim, __pyx_t_10, 0, 0, 0, 0, 0, 0); if (unlikely(__pyx_t_11 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 706; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "View.MemoryView":705
 * 
 *     for dim, index in enumerate(indices):
 *         if PyIndex_Check(index):             # <<<<<<<<<<<<<<
 *             slice_memviewslice(
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 */
      goto __pyx_L6;
    }

    /* "View.MemoryView":712
 *                 0, 0, 0, # have_{start,stop,step}
 *                 False)
 *         elif index is None:             # <<<<<<<<<<<<<<
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0
 */
    __pyx_t_2 = (__pyx_v_index == Py_None);
    __pyx_t_1 = (__pyx_t_2 != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":713
 *                 False)
 *         elif index is None:
 *             p_dst.shape[new_ndim] = 1             # <<<<<<<<<<<<<<
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1
 */
      (__pyx_v_p_dst->shape[__pyx_v_new_ndim]) = 1;

      /* "View.MemoryView":714
 *         elif index is None:
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0             # <<<<<<<<<<<<<<
 *             p_dst.suboffsets[new_ndim] = -1
 *             new_ndim += 1
 */
      (__pyx_v_p_dst->strides[__pyx_v_new_ndim]) = 0;

      /* "View.MemoryView":715
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1             # <<<<<<<<<<<<<<
 *             new_ndim += 1
 *         else:
 */
      (__pyx_v_p_dst->suboffsets[__pyx_v_new_ndim]) = -1L;

      /* "View.MemoryView":716
 *             p_dst.strides[new_ndim] = 0
 *             p_dst.suboffsets[new_ndim] = -1
 *             new_ndim += 1             # <<<<<<<<<<<<<<
 *         else:
 *             start = index.start or 0
 */
      __pyx_v_new_ndim = (__pyx_v_new_ndim + 1);

      /* "View.MemoryView":712
 *                 0, 0, 0, # have_{start,stop,step}
 *                 False)
 *         elif index is None:             # <<<<<<<<<<<<<<
 *             p_dst.shape[new_ndim] = 1
 *             p_dst.strides[new_ndim] = 0
 */
      goto __pyx_L6;
    }

    /* "View.MemoryView":718
 *             new_ndim += 1
 *         else:
 *             start = index.start or 0             # <<<<<<<<<<<<<<
 *             stop = index.stop or 0
 *             step = index.step or 0
 */
    /*else*/ {
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_start); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      if (!__pyx_t_1) {
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else {
        __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 718; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_10 = __pyx_t_12;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L7_bool_binop_done;
      }
      __pyx_t_10 = 0;
      __pyx_L7_bool_binop_done:;
      __pyx_v_start = __pyx_t_10;

      /* "View.MemoryView":719
 *         else:
 *             start = index.start or 0
 *             stop = index.stop or 0             # <<<<<<<<<<<<<<
 *             step = index.step or 0
 * 
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_stop); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      if (!__pyx_t_1) {
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else {
        __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 719; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_10 = __pyx_t_12;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L9_bool_binop_done;
      }
      __pyx_t_10 = 0;
      __pyx_L9_bool_binop_done:;
      __pyx_v_stop = __pyx_t_10;

      /* "View.MemoryView":720
 *             start = index.start or 0
 *             stop = index.stop or 0
 *             step = index.step or 0             # <<<<<<<<<<<<<<
 * 
 *             have_start = index.start is not None
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_step); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 720; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_9); if (unlikely(__pyx_t_1 < 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 720; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      if (!__pyx_t_1) {
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else {
        __pyx_t_12 = __Pyx_PyIndex_AsSsize_t(__pyx_t_9); if (unlikely((__pyx_t_12 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 720; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
        __pyx_t_10 = __pyx_t_12;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L11_bool_binop_done;
      }
      __pyx_t_10 = 0;
      __pyx_L11_bool_binop_done:;
      __pyx_v_step = __pyx_t_10;

      /* "View.MemoryView":722
 *             step = index.step or 0
 * 
 *             have_start = index.start is not None             # <<<<<<<<<<<<<<
 *             have_stop = index.stop is not None
 *             have_step = index.step is not None
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_start); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 722; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = (__pyx_t_9 != Py_None);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_v_have_start = __pyx_t_1;

      /* "View.MemoryView":723
 * 
 *             have_start = index.start is not None
 *             have_stop = index.stop is not None             # <<<<<<<<<<<<<<
 *             have_step = index.step is not None
 * 
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_stop); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 723; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = (__pyx_t_9 != Py_None);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_v_have_stop = __pyx_t_1;

      /* "View.MemoryView":724
 *             have_start = index.start is not None
 *             have_stop = index.stop is not None
 *             have_step = index.step is not None             # <<<<<<<<<<<<<<
 * 
 *             slice_memviewslice(
 */
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_index, __pyx_n_s_step); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 724; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_1 = (__pyx_t_9 != Py_None);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_v_have_step = __pyx_t_1;

      /* "View.MemoryView":726
 *             have_step = index.step is not None
 * 
 *             slice_memviewslice(             # <<<<<<<<<<<<<<
 *                 p_dst, p_src.shape[dim], p_src.strides[dim], p_src.suboffsets[dim],
 *                 dim, new_ndim, p_suboffset_dim,
 */
      __pyx_t_11 = __pyx_memoryview_slice_memviewslice(__pyx_v_p_dst, (__pyx_v_p_src->shape[__pyx_v_dim]), (__pyx_v_p_src->strides[__pyx_v_dim]), (__pyx_v_p_src->suboffsets[__pyx_v_dim]), __pyx_v_dim, __pyx_v_new_ndim, __pyx_v_p_suboffset_dim, __pyx_v_start, __pyx_v_stop, __pyx_v_step, __pyx_v_have_start, __pyx_v_have_stop, __pyx_v_have_step, 1); if (unlikely(__pyx_t_11 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 726; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "View.MemoryView":732
 *                 have_start, have_stop, have_step,
 *                 True)
 *             new_ndim += 1             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(memview, _memoryviewslice):
 */
      __pyx_v_new_ndim = (__pyx_v_new_ndim + 1);
    }
    __pyx_L6:;

    /* "View.MemoryView":704
 *     cdef bint have_start, have_stop, have_step
 * 
 *     for dim, index in enumerate(indices):             # <<<<<<<<<<<<<<
 *         if PyIndex_Check(index):
 *             slice_memviewslice(
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "View.MemoryView":734
 *             new_ndim += 1
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,
 */
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":735
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));

    /* "View.MemoryView":736
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_dtype_func,
 *                                     memview.dtype_is_object)
 */
    if (unlikely(!__pyx_v_memviewsliceobj)) { __Pyx_RaiseUnboundLocalError("memviewsliceobj"); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 736; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }

    /* "View.MemoryView":737
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 *     else:
 */
    if (unlikely(!__pyx_v_memviewsliceobj)) { __Pyx_RaiseUnboundLocalError("memviewsliceobj"); {__pyx_filename = __pyx_f[2]; __pyx_lineno = 737; __pyx_clineno = __LINE__; goto __pyx_L1_error;} }

    /* "View.MemoryView":735
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         return memoryview_fromslice(dst, new_ndim,             # <<<<<<<<<<<<<<
 *                                     memviewsliceobj.to_object_func,
 *                                     memviewsliceobj.to_dtype_func,
 */
    __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_dst, __pyx_v_new_ndim, __pyx_v_memviewsliceobj->to_object_func, __pyx_v_memviewsliceobj->to_dtype_func, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 735; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_memoryview_type))))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 735; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_r = ((struct __pyx_memoryview_obj *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "View.MemoryView":734
 *             new_ndim += 1
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         return memoryview_fromslice(dst, new_ndim,
 *                                     memviewsliceobj.to_object_func,
 */
  }

  /* "View.MemoryView":740
 *                                     memview.dtype_is_object)
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 * 
 */
  /*else*/ {
    __Pyx_XDECREF(((PyObject *)__pyx_r));

    /* "View.MemoryView":741
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,
 *                                     memview.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_3 = __pyx_memoryview_fromslice(__pyx_v_dst, __pyx_v_new_ndim, NULL, NULL, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 740; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);

    /* "View.MemoryView":740
 *                                     memview.dtype_is_object)
 *     else:
 *         return memoryview_fromslice(dst, new_ndim, NULL, NULL,             # <<<<<<<<<<<<<<
 *                                     memview.dtype_is_object)
 * 
 */
    if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_memoryview_type))))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 740; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_r = ((struct __pyx_memoryview_obj *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "View.MemoryView":668
 * 
 * @cname('__pyx_memview_slice')
 * cdef memoryview memview_slice(memoryview memview, object indices):             # <<<<<<<<<<<<<<
 *     cdef int new_ndim = 0, suboffset_dim = -1, dim
 *     cdef bint negative_step
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("View.MemoryView.memview_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_memviewsliceobj);
  __Pyx_XDECREF(__pyx_v_index);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":765
 * 
 * @cname('__pyx_memoryview_slice_memviewslice')
 * cdef int slice_memviewslice(             # <<<<<<<<<<<<<<
 *         __Pyx_memviewslice *dst,
 *         Py_ssize_t shape, Py_ssize_t stride, Py_ssize_t suboffset,
 */

static int __pyx_memoryview_slice_memviewslice(__Pyx_memviewslice *__pyx_v_dst, Py_ssize_t __pyx_v_shape, Py_ssize_t __pyx_v_stride, Py_ssize_t __pyx_v_suboffset, int __pyx_v_dim, int __pyx_v_new_ndim, int *__pyx_v_suboffset_dim, Py_ssize_t __pyx_v_start, Py_ssize_t __pyx_v_stop, Py_ssize_t __pyx_v_step, int __pyx_v_have_start, int __pyx_v_have_stop, int __pyx_v_have_step, int __pyx_v_is_slice) {
  Py_ssize_t __pyx_v_new_shape;
  int __pyx_v_negative_step;
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "View.MemoryView":785
 *     cdef bint negative_step
 * 
 *     if not is_slice:             # <<<<<<<<<<<<<<
 * 
 *         if start < 0:
 */
  __pyx_t_1 = ((!(__pyx_v_is_slice != 0)) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":787
 *     if not is_slice:
 * 
 *         if start < 0:             # <<<<<<<<<<<<<<
 *             start += shape
 *         if not 0 <= start < shape:
 */
    __pyx_t_1 = ((__pyx_v_start < 0) != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":788
 * 
 *         if start < 0:
 *             start += shape             # <<<<<<<<<<<<<<
 *         if not 0 <= start < shape:
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)
 */
      __pyx_v_start = (__pyx_v_start + __pyx_v_shape);

      /* "View.MemoryView":787
 *     if not is_slice:
 * 
 *         if start < 0:             # <<<<<<<<<<<<<<
 *             start += shape
 *         if not 0 <= start < shape:
 */
    }

    /* "View.MemoryView":789
 *         if start < 0:
 *             start += shape
 *         if not 0 <= start < shape:             # <<<<<<<<<<<<<<
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)
 *     else:
 */
    __pyx_t_1 = (0 <= __pyx_v_start);
    if (__pyx_t_1) {
      __pyx_t_1 = (__pyx_v_start < __pyx_v_shape);
    }
    __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":790
 *             start += shape
 *         if not 0 <= start < shape:
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)             # <<<<<<<<<<<<<<
 *     else:
 * 
 */
      __pyx_t_3 = __pyx_memoryview_err_dim(__pyx_builtin_IndexError, __pyx_k_Index_out_of_bounds_axis_d, __pyx_v_dim); if (unlikely(__pyx_t_3 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 790; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "View.MemoryView":789
 *         if start < 0:
 *             start += shape
 *         if not 0 <= start < shape:             # <<<<<<<<<<<<<<
 *             _err_dim(IndexError, "Index out of bounds (axis %d)", dim)
 *     else:
 */
    }

    /* "View.MemoryView":785
 *     cdef bint negative_step
 * 
 *     if not is_slice:             # <<<<<<<<<<<<<<
 * 
 *         if start < 0:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":793
 *     else:
 * 
 *         negative_step = have_step != 0 and step < 0             # <<<<<<<<<<<<<<
 * 
 *         if have_step and step == 0:
 */
  /*else*/ {
    __pyx_t_1 = ((__pyx_v_have_step != 0) != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_2 = __pyx_t_1;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_step < 0) != 0);
    __pyx_t_2 = __pyx_t_1;
    __pyx_L6_bool_binop_done:;
    __pyx_v_negative_step = __pyx_t_2;

    /* "View.MemoryView":795
 *         negative_step = have_step != 0 and step < 0
 * 
 *         if have_step and step == 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Step may not be zero (axis %d)", dim)
 * 
 */
    __pyx_t_1 = (__pyx_v_have_step != 0);
    if (__pyx_t_1) {
    } else {
      __pyx_t_2 = __pyx_t_1;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_1 = ((__pyx_v_step == 0) != 0);
    __pyx_t_2 = __pyx_t_1;
    __pyx_L9_bool_binop_done:;
    if (__pyx_t_2) {

      /* "View.MemoryView":796
 * 
 *         if have_step and step == 0:
 *             _err_dim(ValueError, "Step may not be zero (axis %d)", dim)             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_t_3 = __pyx_memoryview_err_dim(__pyx_builtin_ValueError, __pyx_k_Step_may_not_be_zero_axis_d, __pyx_v_dim); if (unlikely(__pyx_t_3 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 796; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "View.MemoryView":795
 *         negative_step = have_step != 0 and step < 0
 * 
 *         if have_step and step == 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Step may not be zero (axis %d)", dim)
 * 
 */
    }

    /* "View.MemoryView":799
 * 
 * 
 *         if have_start:             # <<<<<<<<<<<<<<
 *             if start < 0:
 *                 start += shape
 */
    __pyx_t_2 = (__pyx_v_have_start != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":800
 * 
 *         if have_start:
 *             if start < 0:             # <<<<<<<<<<<<<<
 *                 start += shape
 *                 if start < 0:
 */
      __pyx_t_2 = ((__pyx_v_start < 0) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":801
 *         if have_start:
 *             if start < 0:
 *                 start += shape             # <<<<<<<<<<<<<<
 *                 if start < 0:
 *                     start = 0
 */
        __pyx_v_start = (__pyx_v_start + __pyx_v_shape);

        /* "View.MemoryView":802
 *             if start < 0:
 *                 start += shape
 *                 if start < 0:             # <<<<<<<<<<<<<<
 *                     start = 0
 *             elif start >= shape:
 */
        __pyx_t_2 = ((__pyx_v_start < 0) != 0);
        if (__pyx_t_2) {

          /* "View.MemoryView":803
 *                 start += shape
 *                 if start < 0:
 *                     start = 0             # <<<<<<<<<<<<<<
 *             elif start >= shape:
 *                 if negative_step:
 */
          __pyx_v_start = 0;

          /* "View.MemoryView":802
 *             if start < 0:
 *                 start += shape
 *                 if start < 0:             # <<<<<<<<<<<<<<
 *                     start = 0
 *             elif start >= shape:
 */
        }

        /* "View.MemoryView":800
 * 
 *         if have_start:
 *             if start < 0:             # <<<<<<<<<<<<<<
 *                 start += shape
 *                 if start < 0:
 */
        goto __pyx_L12;
      }

      /* "View.MemoryView":804
 *                 if start < 0:
 *                     start = 0
 *             elif start >= shape:             # <<<<<<<<<<<<<<
 *                 if negative_step:
 *                     start = shape - 1
 */
      __pyx_t_2 = ((__pyx_v_start >= __pyx_v_shape) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":805
 *                     start = 0
 *             elif start >= shape:
 *                 if negative_step:             # <<<<<<<<<<<<<<
 *                     start = shape - 1
 *                 else:
 */
        __pyx_t_2 = (__pyx_v_negative_step != 0);
        if (__pyx_t_2) {

          /* "View.MemoryView":806
 *             elif start >= shape:
 *                 if negative_step:
 *                     start = shape - 1             # <<<<<<<<<<<<<<
 *                 else:
 *                     start = shape
 */
          __pyx_v_start = (__pyx_v_shape - 1);

          /* "View.MemoryView":805
 *                     start = 0
 *             elif start >= shape:
 *                 if negative_step:             # <<<<<<<<<<<<<<
 *                     start = shape - 1
 *                 else:
 */
          goto __pyx_L14;
        }

        /* "View.MemoryView":808
 *                     start = shape - 1
 *                 else:
 *                     start = shape             # <<<<<<<<<<<<<<
 *         else:
 *             if negative_step:
 */
        /*else*/ {
          __pyx_v_start = __pyx_v_shape;
        }
        __pyx_L14:;

        /* "View.MemoryView":804
 *                 if start < 0:
 *                     start = 0
 *             elif start >= shape:             # <<<<<<<<<<<<<<
 *                 if negative_step:
 *                     start = shape - 1
 */
      }
      __pyx_L12:;

      /* "View.MemoryView":799
 * 
 * 
 *         if have_start:             # <<<<<<<<<<<<<<
 *             if start < 0:
 *                 start += shape
 */
      goto __pyx_L11;
    }

    /* "View.MemoryView":810
 *                     start = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 start = shape - 1
 *             else:
 */
    /*else*/ {
      __pyx_t_2 = (__pyx_v_negative_step != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":811
 *         else:
 *             if negative_step:
 *                 start = shape - 1             # <<<<<<<<<<<<<<
 *             else:
 *                 start = 0
 */
        __pyx_v_start = (__pyx_v_shape - 1);

        /* "View.MemoryView":810
 *                     start = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 start = shape - 1
 *             else:
 */
        goto __pyx_L15;
      }

      /* "View.MemoryView":813
 *                 start = shape - 1
 *             else:
 *                 start = 0             # <<<<<<<<<<<<<<
 * 
 *         if have_stop:
 */
      /*else*/ {
        __pyx_v_start = 0;
      }
      __pyx_L15:;
    }
    __pyx_L11:;

    /* "View.MemoryView":815
 *                 start = 0
 * 
 *         if have_stop:             # <<<<<<<<<<<<<<
 *             if stop < 0:
 *                 stop += shape
 */
    __pyx_t_2 = (__pyx_v_have_stop != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":816
 * 
 *         if have_stop:
 *             if stop < 0:             # <<<<<<<<<<<<<<
 *                 stop += shape
 *                 if stop < 0:
 */
      __pyx_t_2 = ((__pyx_v_stop < 0) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":817
 *         if have_stop:
 *             if stop < 0:
 *                 stop += shape             # <<<<<<<<<<<<<<
 *                 if stop < 0:
 *                     stop = 0
 */
        __pyx_v_stop = (__pyx_v_stop + __pyx_v_shape);

        /* "View.MemoryView":818
 *             if stop < 0:
 *                 stop += shape
 *                 if stop < 0:             # <<<<<<<<<<<<<<
 *                     stop = 0
 *             elif stop > shape:
 */
        __pyx_t_2 = ((__pyx_v_stop < 0) != 0);
        if (__pyx_t_2) {

          /* "View.MemoryView":819
 *                 stop += shape
 *                 if stop < 0:
 *                     stop = 0             # <<<<<<<<<<<<<<
 *             elif stop > shape:
 *                 stop = shape
 */
          __pyx_v_stop = 0;

          /* "View.MemoryView":818
 *             if stop < 0:
 *                 stop += shape
 *                 if stop < 0:             # <<<<<<<<<<<<<<
 *                     stop = 0
 *             elif stop > shape:
 */
        }

        /* "View.MemoryView":816
 * 
 *         if have_stop:
 *             if stop < 0:             # <<<<<<<<<<<<<<
 *                 stop += shape
 *                 if stop < 0:
 */
        goto __pyx_L17;
      }

      /* "View.MemoryView":820
 *                 if stop < 0:
 *                     stop = 0
 *             elif stop > shape:             # <<<<<<<<<<<<<<
 *                 stop = shape
 *         else:
 */
      __pyx_t_2 = ((__pyx_v_stop > __pyx_v_shape) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":821
 *                     stop = 0
 *             elif stop > shape:
 *                 stop = shape             # <<<<<<<<<<<<<<
 *         else:
 *             if negative_step:
 */
        __pyx_v_stop = __pyx_v_shape;

        /* "View.MemoryView":820
 *                 if stop < 0:
 *                     stop = 0
 *             elif stop > shape:             # <<<<<<<<<<<<<<
 *                 stop = shape
 *         else:
 */
      }
      __pyx_L17:;

      /* "View.MemoryView":815
 *                 start = 0
 * 
 *         if have_stop:             # <<<<<<<<<<<<<<
 *             if stop < 0:
 *                 stop += shape
 */
      goto __pyx_L16;
    }

    /* "View.MemoryView":823
 *                 stop = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 stop = -1
 *             else:
 */
    /*else*/ {
      __pyx_t_2 = (__pyx_v_negative_step != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":824
 *         else:
 *             if negative_step:
 *                 stop = -1             # <<<<<<<<<<<<<<
 *             else:
 *                 stop = shape
 */
        __pyx_v_stop = -1L;

        /* "View.MemoryView":823
 *                 stop = shape
 *         else:
 *             if negative_step:             # <<<<<<<<<<<<<<
 *                 stop = -1
 *             else:
 */
        goto __pyx_L19;
      }

      /* "View.MemoryView":826
 *                 stop = -1
 *             else:
 *                 stop = shape             # <<<<<<<<<<<<<<
 * 
 *         if not have_step:
 */
      /*else*/ {
        __pyx_v_stop = __pyx_v_shape;
      }
      __pyx_L19:;
    }
    __pyx_L16:;

    /* "View.MemoryView":828
 *                 stop = shape
 * 
 *         if not have_step:             # <<<<<<<<<<<<<<
 *             step = 1
 * 
 */
    __pyx_t_2 = ((!(__pyx_v_have_step != 0)) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":829
 * 
 *         if not have_step:
 *             step = 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_v_step = 1;

      /* "View.MemoryView":828
 *                 stop = shape
 * 
 *         if not have_step:             # <<<<<<<<<<<<<<
 *             step = 1
 * 
 */
    }

    /* "View.MemoryView":833
 * 
 *         with cython.cdivision(True):
 *             new_shape = (stop - start) // step             # <<<<<<<<<<<<<<
 * 
 *             if (stop - start) - step * new_shape:
 */
    __pyx_v_new_shape = ((__pyx_v_stop - __pyx_v_start) / __pyx_v_step);

    /* "View.MemoryView":835
 *             new_shape = (stop - start) // step
 * 
 *             if (stop - start) - step * new_shape:             # <<<<<<<<<<<<<<
 *                 new_shape += 1
 * 
 */
    __pyx_t_2 = (((__pyx_v_stop - __pyx_v_start) - (__pyx_v_step * __pyx_v_new_shape)) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":836
 * 
 *             if (stop - start) - step * new_shape:
 *                 new_shape += 1             # <<<<<<<<<<<<<<
 * 
 *         if new_shape < 0:
 */
      __pyx_v_new_shape = (__pyx_v_new_shape + 1);

      /* "View.MemoryView":835
 *             new_shape = (stop - start) // step
 * 
 *             if (stop - start) - step * new_shape:             # <<<<<<<<<<<<<<
 *                 new_shape += 1
 * 
 */
    }

    /* "View.MemoryView":838
 *                 new_shape += 1
 * 
 *         if new_shape < 0:             # <<<<<<<<<<<<<<
 *             new_shape = 0
 * 
 */
    __pyx_t_2 = ((__pyx_v_new_shape < 0) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":839
 * 
 *         if new_shape < 0:
 *             new_shape = 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_v_new_shape = 0;

      /* "View.MemoryView":838
 *                 new_shape += 1
 * 
 *         if new_shape < 0:             # <<<<<<<<<<<<<<
 *             new_shape = 0
 * 
 */
    }

    /* "View.MemoryView":842
 * 
 * 
 *         dst.strides[new_ndim] = stride * step             # <<<<<<<<<<<<<<
 *         dst.shape[new_ndim] = new_shape
 *         dst.suboffsets[new_ndim] = suboffset
 */
    (__pyx_v_dst->strides[__pyx_v_new_ndim]) = (__pyx_v_stride * __pyx_v_step);

    /* "View.MemoryView":843
 * 
 *         dst.strides[new_ndim] = stride * step
 *         dst.shape[new_ndim] = new_shape             # <<<<<<<<<<<<<<
 *         dst.suboffsets[new_ndim] = suboffset
 * 
 */
    (__pyx_v_dst->shape[__pyx_v_new_ndim]) = __pyx_v_new_shape;

    /* "View.MemoryView":844
 *         dst.strides[new_ndim] = stride * step
 *         dst.shape[new_ndim] = new_shape
 *         dst.suboffsets[new_ndim] = suboffset             # <<<<<<<<<<<<<<
 * 
 * 
 */
    (__pyx_v_dst->suboffsets[__pyx_v_new_ndim]) = __pyx_v_suboffset;
  }
  __pyx_L3:;

  /* "View.MemoryView":847
 * 
 * 
 *     if suboffset_dim[0] < 0:             # <<<<<<<<<<<<<<
 *         dst.data += start * stride
 *     else:
 */
  __pyx_t_2 = (((__pyx_v_suboffset_dim[0]) < 0) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":848
 * 
 *     if suboffset_dim[0] < 0:
 *         dst.data += start * stride             # <<<<<<<<<<<<<<
 *     else:
 *         dst.suboffsets[suboffset_dim[0]] += start * stride
 */
    __pyx_v_dst->data = (__pyx_v_dst->data + (__pyx_v_start * __pyx_v_stride));

    /* "View.MemoryView":847
 * 
 * 
 *     if suboffset_dim[0] < 0:             # <<<<<<<<<<<<<<
 *         dst.data += start * stride
 *     else:
 */
    goto __pyx_L23;
  }

  /* "View.MemoryView":850
 *         dst.data += start * stride
 *     else:
 *         dst.suboffsets[suboffset_dim[0]] += start * stride             # <<<<<<<<<<<<<<
 * 
 *     if suboffset >= 0:
 */
  /*else*/ {
    __pyx_t_3 = (__pyx_v_suboffset_dim[0]);
    (__pyx_v_dst->suboffsets[__pyx_t_3]) = ((__pyx_v_dst->suboffsets[__pyx_t_3]) + (__pyx_v_start * __pyx_v_stride));
  }
  __pyx_L23:;

  /* "View.MemoryView":852
 *         dst.suboffsets[suboffset_dim[0]] += start * stride
 * 
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         if not is_slice:
 *             if new_ndim == 0:
 */
  __pyx_t_2 = ((__pyx_v_suboffset >= 0) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":853
 * 
 *     if suboffset >= 0:
 *         if not is_slice:             # <<<<<<<<<<<<<<
 *             if new_ndim == 0:
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 */
    __pyx_t_2 = ((!(__pyx_v_is_slice != 0)) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":854
 *     if suboffset >= 0:
 *         if not is_slice:
 *             if new_ndim == 0:             # <<<<<<<<<<<<<<
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 *             else:
 */
      __pyx_t_2 = ((__pyx_v_new_ndim == 0) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":855
 *         if not is_slice:
 *             if new_ndim == 0:
 *                 dst.data = (<char **> dst.data)[0] + suboffset             # <<<<<<<<<<<<<<
 *             else:
 *                 _err_dim(IndexError, "All dimensions preceding dimension %d "
 */
        __pyx_v_dst->data = ((((char **)__pyx_v_dst->data)[0]) + __pyx_v_suboffset);

        /* "View.MemoryView":854
 *     if suboffset >= 0:
 *         if not is_slice:
 *             if new_ndim == 0:             # <<<<<<<<<<<<<<
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 *             else:
 */
        goto __pyx_L26;
      }

      /* "View.MemoryView":857
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 *             else:
 *                 _err_dim(IndexError, "All dimensions preceding dimension %d "             # <<<<<<<<<<<<<<
 *                                      "must be indexed and not sliced", dim)
 *         else:
 */
      /*else*/ {

        /* "View.MemoryView":858
 *             else:
 *                 _err_dim(IndexError, "All dimensions preceding dimension %d "
 *                                      "must be indexed and not sliced", dim)             # <<<<<<<<<<<<<<
 *         else:
 *             suboffset_dim[0] = new_ndim
 */
        __pyx_t_3 = __pyx_memoryview_err_dim(__pyx_builtin_IndexError, __pyx_k_All_dimensions_preceding_dimensi, __pyx_v_dim); if (unlikely(__pyx_t_3 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 857; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_L26:;

      /* "View.MemoryView":853
 * 
 *     if suboffset >= 0:
 *         if not is_slice:             # <<<<<<<<<<<<<<
 *             if new_ndim == 0:
 *                 dst.data = (<char **> dst.data)[0] + suboffset
 */
      goto __pyx_L25;
    }

    /* "View.MemoryView":860
 *                                      "must be indexed and not sliced", dim)
 *         else:
 *             suboffset_dim[0] = new_ndim             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    /*else*/ {
      (__pyx_v_suboffset_dim[0]) = __pyx_v_new_ndim;
    }
    __pyx_L25:;

    /* "View.MemoryView":852
 *         dst.suboffsets[suboffset_dim[0]] += start * stride
 * 
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         if not is_slice:
 *             if new_ndim == 0:
 */
  }

  /* "View.MemoryView":862
 *             suboffset_dim[0] = new_ndim
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "View.MemoryView":765
 * 
 * @cname('__pyx_memoryview_slice_memviewslice')
 * cdef int slice_memviewslice(             # <<<<<<<<<<<<<<
 *         __Pyx_memviewslice *dst,
 *         Py_ssize_t shape, Py_ssize_t stride, Py_ssize_t suboffset,
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("View.MemoryView.slice_memviewslice", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":868
 * 
 * @cname('__pyx_pybuffer_index')
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,             # <<<<<<<<<<<<<<
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1
 */

static char *__pyx_pybuffer_index(Py_buffer *__pyx_v_view, char *__pyx_v_bufp, Py_ssize_t __pyx_v_index, Py_ssize_t __pyx_v_dim) {
  Py_ssize_t __pyx_v_shape;
  Py_ssize_t __pyx_v_stride;
  Py_ssize_t __pyx_v_suboffset;
  Py_ssize_t __pyx_v_itemsize;
  char *__pyx_v_resultp;
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pybuffer_index", 0);

  /* "View.MemoryView":870
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t itemsize = view.itemsize
 *     cdef char *resultp
 */
  __pyx_v_suboffset = -1L;

  /* "View.MemoryView":871
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1
 *     cdef Py_ssize_t itemsize = view.itemsize             # <<<<<<<<<<<<<<
 *     cdef char *resultp
 * 
 */
  __pyx_t_1 = __pyx_v_view->itemsize;
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":874
 *     cdef char *resultp
 * 
 *     if view.ndim == 0:             # <<<<<<<<<<<<<<
 *         shape = view.len / itemsize
 *         stride = itemsize
 */
  __pyx_t_2 = ((__pyx_v_view->ndim == 0) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":875
 * 
 *     if view.ndim == 0:
 *         shape = view.len / itemsize             # <<<<<<<<<<<<<<
 *         stride = itemsize
 *     else:
 */
    if (unlikely(__pyx_v_itemsize == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "integer division or modulo by zero");
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 875; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    else if (sizeof(Py_ssize_t) == sizeof(long) && (!(((Py_ssize_t)-1) > 0)) && unlikely(__pyx_v_itemsize == (Py_ssize_t)-1)  && unlikely(UNARY_NEG_WOULD_OVERFLOW(__pyx_v_view->len))) {
      PyErr_SetString(PyExc_OverflowError, "value too large to perform division");
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 875; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
    __pyx_v_shape = (__pyx_v_view->len / __pyx_v_itemsize);

    /* "View.MemoryView":876
 *     if view.ndim == 0:
 *         shape = view.len / itemsize
 *         stride = itemsize             # <<<<<<<<<<<<<<
 *     else:
 *         shape = view.shape[dim]
 */
    __pyx_v_stride = __pyx_v_itemsize;

    /* "View.MemoryView":874
 *     cdef char *resultp
 * 
 *     if view.ndim == 0:             # <<<<<<<<<<<<<<
 *         shape = view.len / itemsize
 *         stride = itemsize
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":878
 *         stride = itemsize
 *     else:
 *         shape = view.shape[dim]             # <<<<<<<<<<<<<<
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:
 */
  /*else*/ {
    __pyx_v_shape = (__pyx_v_view->shape[__pyx_v_dim]);

    /* "View.MemoryView":879
 *     else:
 *         shape = view.shape[dim]
 *         stride = view.strides[dim]             # <<<<<<<<<<<<<<
 *         if view.suboffsets != NULL:
 *             suboffset = view.suboffsets[dim]
 */
    __pyx_v_stride = (__pyx_v_view->strides[__pyx_v_dim]);

    /* "View.MemoryView":880
 *         shape = view.shape[dim]
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *             suboffset = view.suboffsets[dim]
 * 
 */
    __pyx_t_2 = ((__pyx_v_view->suboffsets != NULL) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":881
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:
 *             suboffset = view.suboffsets[dim]             # <<<<<<<<<<<<<<
 * 
 *     if index < 0:
 */
      __pyx_v_suboffset = (__pyx_v_view->suboffsets[__pyx_v_dim]);

      /* "View.MemoryView":880
 *         shape = view.shape[dim]
 *         stride = view.strides[dim]
 *         if view.suboffsets != NULL:             # <<<<<<<<<<<<<<
 *             suboffset = view.suboffsets[dim]
 * 
 */
    }
  }
  __pyx_L3:;

  /* "View.MemoryView":883
 *             suboffset = view.suboffsets[dim]
 * 
 *     if index < 0:             # <<<<<<<<<<<<<<
 *         index += view.shape[dim]
 *         if index < 0:
 */
  __pyx_t_2 = ((__pyx_v_index < 0) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":884
 * 
 *     if index < 0:
 *         index += view.shape[dim]             # <<<<<<<<<<<<<<
 *         if index < 0:
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 */
    __pyx_v_index = (__pyx_v_index + (__pyx_v_view->shape[__pyx_v_dim]));

    /* "View.MemoryView":885
 *     if index < 0:
 *         index += view.shape[dim]
 *         if index < 0:             # <<<<<<<<<<<<<<
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */
    __pyx_t_2 = ((__pyx_v_index < 0) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":886
 *         index += view.shape[dim]
 *         if index < 0:
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)             # <<<<<<<<<<<<<<
 * 
 *     if index >= shape:
 */
      __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 886; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Out_of_bounds_on_buffer_access_a, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 886; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 886; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_4);
      PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_IndexError, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 886; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_Raise(__pyx_t_4, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      {__pyx_filename = __pyx_f[2]; __pyx_lineno = 886; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "View.MemoryView":885
 *     if index < 0:
 *         index += view.shape[dim]
 *         if index < 0:             # <<<<<<<<<<<<<<
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */
    }

    /* "View.MemoryView":883
 *             suboffset = view.suboffsets[dim]
 * 
 *     if index < 0:             # <<<<<<<<<<<<<<
 *         index += view.shape[dim]
 *         if index < 0:
 */
  }

  /* "View.MemoryView":888
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 *     if index >= shape:             # <<<<<<<<<<<<<<
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */
  __pyx_t_2 = ((__pyx_v_index >= __pyx_v_shape) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":889
 * 
 *     if index >= shape:
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)             # <<<<<<<<<<<<<<
 * 
 *     resultp = bufp + index * stride
 */
    __pyx_t_4 = PyInt_FromSsize_t(__pyx_v_dim); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_Out_of_bounds_on_buffer_access_a, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_IndexError, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 889; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "View.MemoryView":888
 *             raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 *     if index >= shape:             # <<<<<<<<<<<<<<
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 */
  }

  /* "View.MemoryView":891
 *         raise IndexError("Out of bounds on buffer access (axis %d)" % dim)
 * 
 *     resultp = bufp + index * stride             # <<<<<<<<<<<<<<
 *     if suboffset >= 0:
 *         resultp = (<char **> resultp)[0] + suboffset
 */
  __pyx_v_resultp = (__pyx_v_bufp + (__pyx_v_index * __pyx_v_stride));

  /* "View.MemoryView":892
 * 
 *     resultp = bufp + index * stride
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         resultp = (<char **> resultp)[0] + suboffset
 * 
 */
  __pyx_t_2 = ((__pyx_v_suboffset >= 0) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":893
 *     resultp = bufp + index * stride
 *     if suboffset >= 0:
 *         resultp = (<char **> resultp)[0] + suboffset             # <<<<<<<<<<<<<<
 * 
 *     return resultp
 */
    __pyx_v_resultp = ((((char **)__pyx_v_resultp)[0]) + __pyx_v_suboffset);

    /* "View.MemoryView":892
 * 
 *     resultp = bufp + index * stride
 *     if suboffset >= 0:             # <<<<<<<<<<<<<<
 *         resultp = (<char **> resultp)[0] + suboffset
 * 
 */
  }

  /* "View.MemoryView":895
 *         resultp = (<char **> resultp)[0] + suboffset
 * 
 *     return resultp             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_resultp;
  goto __pyx_L0;

  /* "View.MemoryView":868
 * 
 * @cname('__pyx_pybuffer_index')
 * cdef char *pybuffer_index(Py_buffer *view, char *bufp, Py_ssize_t index,             # <<<<<<<<<<<<<<
 *                           Py_ssize_t dim) except NULL:
 *     cdef Py_ssize_t shape, stride, suboffset = -1
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("View.MemoryView.pybuffer_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":901
 * 
 * @cname('__pyx_memslice_transpose')
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) nogil except 0:             # <<<<<<<<<<<<<<
 *     cdef int ndim = memslice.memview.view.ndim
 * 
 */

static int __pyx_memslice_transpose(__Pyx_memviewslice *__pyx_v_memslice) {
  int __pyx_v_ndim;
  Py_ssize_t *__pyx_v_shape;
  Py_ssize_t *__pyx_v_strides;
  int __pyx_v_i;
  int __pyx_v_j;
  int __pyx_r;
  int __pyx_t_1;
  Py_ssize_t *__pyx_t_2;
  long __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "View.MemoryView":902
 * @cname('__pyx_memslice_transpose')
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) nogil except 0:
 *     cdef int ndim = memslice.memview.view.ndim             # <<<<<<<<<<<<<<
 * 
 *     cdef Py_ssize_t *shape = memslice.shape
 */
  __pyx_t_1 = __pyx_v_memslice->memview->view.ndim;
  __pyx_v_ndim = __pyx_t_1;

  /* "View.MemoryView":904
 *     cdef int ndim = memslice.memview.view.ndim
 * 
 *     cdef Py_ssize_t *shape = memslice.shape             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t *strides = memslice.strides
 * 
 */
  __pyx_t_2 = __pyx_v_memslice->shape;
  __pyx_v_shape = __pyx_t_2;

  /* "View.MemoryView":905
 * 
 *     cdef Py_ssize_t *shape = memslice.shape
 *     cdef Py_ssize_t *strides = memslice.strides             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __pyx_v_memslice->strides;
  __pyx_v_strides = __pyx_t_2;

  /* "View.MemoryView":909
 * 
 *     cdef int i, j
 *     for i in range(ndim / 2):             # <<<<<<<<<<<<<<
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]
 */
  __pyx_t_3 = (__pyx_v_ndim / 2);
  for (__pyx_t_1 = 0; __pyx_t_1 < __pyx_t_3; __pyx_t_1+=1) {
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":910
 *     cdef int i, j
 *     for i in range(ndim / 2):
 *         j = ndim - 1 - i             # <<<<<<<<<<<<<<
 *         strides[i], strides[j] = strides[j], strides[i]
 *         shape[i], shape[j] = shape[j], shape[i]
 */
    __pyx_v_j = ((__pyx_v_ndim - 1) - __pyx_v_i);

    /* "View.MemoryView":911
 *     for i in range(ndim / 2):
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]             # <<<<<<<<<<<<<<
 *         shape[i], shape[j] = shape[j], shape[i]
 * 
 */
    __pyx_t_4 = (__pyx_v_strides[__pyx_v_j]);
    __pyx_t_5 = (__pyx_v_strides[__pyx_v_i]);
    (__pyx_v_strides[__pyx_v_i]) = __pyx_t_4;
    (__pyx_v_strides[__pyx_v_j]) = __pyx_t_5;

    /* "View.MemoryView":912
 *         j = ndim - 1 - i
 *         strides[i], strides[j] = strides[j], strides[i]
 *         shape[i], shape[j] = shape[j], shape[i]             # <<<<<<<<<<<<<<
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:
 */
    __pyx_t_5 = (__pyx_v_shape[__pyx_v_j]);
    __pyx_t_4 = (__pyx_v_shape[__pyx_v_i]);
    (__pyx_v_shape[__pyx_v_i]) = __pyx_t_5;
    (__pyx_v_shape[__pyx_v_j]) = __pyx_t_4;

    /* "View.MemoryView":914
 *         shape[i], shape[j] = shape[j], shape[i]
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:             # <<<<<<<<<<<<<<
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")
 * 
 */
    __pyx_t_7 = (((__pyx_v_memslice->suboffsets[__pyx_v_i]) >= 0) != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_7 = (((__pyx_v_memslice->suboffsets[__pyx_v_j]) >= 0) != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_6) {

      /* "View.MemoryView":915
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")             # <<<<<<<<<<<<<<
 * 
 *     return 1
 */
      __pyx_t_8 = __pyx_memoryview_err(__pyx_builtin_ValueError, __pyx_k_Cannot_transpose_memoryview_with); if (unlikely(__pyx_t_8 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "View.MemoryView":914
 *         shape[i], shape[j] = shape[j], shape[i]
 * 
 *         if memslice.suboffsets[i] >= 0 or memslice.suboffsets[j] >= 0:             # <<<<<<<<<<<<<<
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")
 * 
 */
    }
  }

  /* "View.MemoryView":917
 *             _err(ValueError, "Cannot transpose memoryview with indirect dimensions")
 * 
 *     return 1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "View.MemoryView":901
 * 
 * @cname('__pyx_memslice_transpose')
 * cdef int transpose_memslice(__Pyx_memviewslice *memslice) nogil except 0:             # <<<<<<<<<<<<<<
 *     cdef int ndim = memslice.memview.view.ndim
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("View.MemoryView.transpose_memslice", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = 0;
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":934
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 */

/* Python wrapper */
static void __pyx_memoryviewslice___dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_memoryviewslice___dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_memoryviewslice___pyx_pf_15View_dot_MemoryView_16_memoryviewslice___dealloc__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_memoryviewslice___pyx_pf_15View_dot_MemoryView_16_memoryviewslice___dealloc__(struct __pyx_memoryviewslice_obj *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "View.MemoryView":935
 * 
 *     def __dealloc__(self):
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)             # <<<<<<<<<<<<<<
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 */
  __PYX_XDEC_MEMVIEW((&__pyx_v_self->from_slice), 1);

  /* "View.MemoryView":934
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "View.MemoryView":937
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)
 */

static PyObject *__pyx_memoryviewslice_convert_item_to_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("convert_item_to_object", 0);

  /* "View.MemoryView":938
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 *         if self.to_object_func != NULL:             # <<<<<<<<<<<<<<
 *             return self.to_object_func(itemp)
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_self->to_object_func != NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":939
 *     cdef convert_item_to_object(self, char *itemp):
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)             # <<<<<<<<<<<<<<
 *         else:
 *             return memoryview.convert_item_to_object(self, itemp)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_v_self->to_object_func(__pyx_v_itemp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 939; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "View.MemoryView":938
 * 
 *     cdef convert_item_to_object(self, char *itemp):
 *         if self.to_object_func != NULL:             # <<<<<<<<<<<<<<
 *             return self.to_object_func(itemp)
 *         else:
 */
  }

  /* "View.MemoryView":941
 *             return self.to_object_func(itemp)
 *         else:
 *             return memoryview.convert_item_to_object(self, itemp)             # <<<<<<<<<<<<<<
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_memoryview_convert_item_to_object(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_itemp); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 941; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "View.MemoryView":937
 *         __PYX_XDEC_MEMVIEW(&self.from_slice, 1)
 * 
 *     cdef convert_item_to_object(self, char *itemp):             # <<<<<<<<<<<<<<
 *         if self.to_object_func != NULL:
 *             return self.to_object_func(itemp)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.convert_item_to_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":943
 *             return memoryview.convert_item_to_object(self, itemp)
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)
 */

static PyObject *__pyx_memoryviewslice_assign_item_from_object(struct __pyx_memoryviewslice_obj *__pyx_v_self, char *__pyx_v_itemp, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("assign_item_from_object", 0);

  /* "View.MemoryView":944
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 *         if self.to_dtype_func != NULL:             # <<<<<<<<<<<<<<
 *             self.to_dtype_func(itemp, value)
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_self->to_dtype_func != NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":945
 *     cdef assign_item_from_object(self, char *itemp, object value):
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)             # <<<<<<<<<<<<<<
 *         else:
 *             memoryview.assign_item_from_object(self, itemp, value)
 */
    __pyx_t_2 = __pyx_v_self->to_dtype_func(__pyx_v_itemp, __pyx_v_value); if (unlikely(__pyx_t_2 == 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 945; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "View.MemoryView":944
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):
 *         if self.to_dtype_func != NULL:             # <<<<<<<<<<<<<<
 *             self.to_dtype_func(itemp, value)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":947
 *             self.to_dtype_func(itemp, value)
 *         else:
 *             memoryview.assign_item_from_object(self, itemp, value)             # <<<<<<<<<<<<<<
 * 
 *     property base:
 */
  /*else*/ {
    __pyx_t_3 = __pyx_memoryview_assign_item_from_object(((struct __pyx_memoryview_obj *)__pyx_v_self), __pyx_v_itemp, __pyx_v_value); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 947; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "View.MemoryView":943
 *             return memoryview.convert_item_to_object(self, itemp)
 * 
 *     cdef assign_item_from_object(self, char *itemp, object value):             # <<<<<<<<<<<<<<
 *         if self.to_dtype_func != NULL:
 *             self.to_dtype_func(itemp, value)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView._memoryviewslice.assign_item_from_object", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":951
 *     property base:
 *         @cname('__pyx_memoryviewslice__get__base')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.from_object
 * 
 */

/* Python wrapper */
static PyObject *__pyx_memoryviewslice__get__base(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_memoryviewslice__get__base(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_15View_dot_MemoryView_16_memoryviewslice_4base___get__(((struct __pyx_memoryviewslice_obj *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_15View_dot_MemoryView_16_memoryviewslice_4base___get__(struct __pyx_memoryviewslice_obj *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "View.MemoryView":952
 *         @cname('__pyx_memoryviewslice__get__base')
 *         def __get__(self):
 *             return self.from_object             # <<<<<<<<<<<<<<
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->from_object);
  __pyx_r = __pyx_v_self->from_object;
  goto __pyx_L0;

  /* "View.MemoryView":951
 *     property base:
 *         @cname('__pyx_memoryviewslice__get__base')
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.from_object
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":958
 * 
 * @cname('__pyx_memoryview_fromslice')
 * cdef memoryview_fromslice(__Pyx_memviewslice memviewslice,             # <<<<<<<<<<<<<<
 *                           int ndim,
 *                           object (*to_object_func)(char *),
 */

static PyObject *__pyx_memoryview_fromslice(__Pyx_memviewslice __pyx_v_memviewslice, int __pyx_v_ndim, PyObject *(*__pyx_v_to_object_func)(char *), int (*__pyx_v_to_dtype_func)(char *, PyObject *), int __pyx_v_dtype_is_object) {
  struct __pyx_memoryviewslice_obj *__pyx_v_result = 0;
  Py_ssize_t __pyx_v_suboffset;
  PyObject *__pyx_v_length = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_TypeInfo *__pyx_t_4;
  Py_buffer __pyx_t_5;
  Py_ssize_t *__pyx_t_6;
  Py_ssize_t *__pyx_t_7;
  Py_ssize_t *__pyx_t_8;
  Py_ssize_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memoryview_fromslice", 0);

  /* "View.MemoryView":966
 *     cdef _memoryviewslice result
 * 
 *     if <PyObject *> memviewslice.memview == Py_None:             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
  __pyx_t_1 = ((((PyObject *)__pyx_v_memviewslice.memview) == Py_None) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":967
 * 
 *     if <PyObject *> memviewslice.memview == Py_None:
 *         return None             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_None);
    __pyx_r = Py_None;
    goto __pyx_L0;

    /* "View.MemoryView":966
 *     cdef _memoryviewslice result
 * 
 *     if <PyObject *> memviewslice.memview == Py_None:             # <<<<<<<<<<<<<<
 *         return None
 * 
 */
  }

  /* "View.MemoryView":972
 * 
 * 
 *     result = _memoryviewslice(None, 0, dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 *     result.from_slice = memviewslice
 */
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_dtype_is_object); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 972; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 972; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_3, 0, Py_None);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_0);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_memoryviewslice_type), __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 972; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_result = ((struct __pyx_memoryviewslice_obj *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "View.MemoryView":974
 *     result = _memoryviewslice(None, 0, dtype_is_object)
 * 
 *     result.from_slice = memviewslice             # <<<<<<<<<<<<<<
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)
 * 
 */
  __pyx_v_result->from_slice = __pyx_v_memviewslice;

  /* "View.MemoryView":975
 * 
 *     result.from_slice = memviewslice
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)             # <<<<<<<<<<<<<<
 * 
 *     result.from_object = (<memoryview> memviewslice.memview).base
 */
  __PYX_INC_MEMVIEW((&__pyx_v_memviewslice), 1);

  /* "View.MemoryView":977
 *     __PYX_INC_MEMVIEW(&memviewslice, 1)
 * 
 *     result.from_object = (<memoryview> memviewslice.memview).base             # <<<<<<<<<<<<<<
 *     result.typeinfo = memviewslice.memview.typeinfo
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_memviewslice.memview), __pyx_n_s_base); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 977; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_result->from_object);
  __Pyx_DECREF(__pyx_v_result->from_object);
  __pyx_v_result->from_object = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "View.MemoryView":978
 * 
 *     result.from_object = (<memoryview> memviewslice.memview).base
 *     result.typeinfo = memviewslice.memview.typeinfo             # <<<<<<<<<<<<<<
 * 
 *     result.view = memviewslice.memview.view
 */
  __pyx_t_4 = __pyx_v_memviewslice.memview->typeinfo;
  __pyx_v_result->__pyx_base.typeinfo = __pyx_t_4;

  /* "View.MemoryView":980
 *     result.typeinfo = memviewslice.memview.typeinfo
 * 
 *     result.view = memviewslice.memview.view             # <<<<<<<<<<<<<<
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim
 */
  __pyx_t_5 = __pyx_v_memviewslice.memview->view;
  __pyx_v_result->__pyx_base.view = __pyx_t_5;

  /* "View.MemoryView":981
 * 
 *     result.view = memviewslice.memview.view
 *     result.view.buf = <void *> memviewslice.data             # <<<<<<<<<<<<<<
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None
 */
  __pyx_v_result->__pyx_base.view.buf = ((void *)__pyx_v_memviewslice.data);

  /* "View.MemoryView":982
 *     result.view = memviewslice.memview.view
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim             # <<<<<<<<<<<<<<
 *     (<__pyx_buffer *> &result.view).obj = Py_None
 *     Py_INCREF(Py_None)
 */
  __pyx_v_result->__pyx_base.view.ndim = __pyx_v_ndim;

  /* "View.MemoryView":983
 *     result.view.buf = <void *> memviewslice.data
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None             # <<<<<<<<<<<<<<
 *     Py_INCREF(Py_None)
 * 
 */
  ((Py_buffer *)(&__pyx_v_result->__pyx_base.view))->obj = Py_None;

  /* "View.MemoryView":984
 *     result.view.ndim = ndim
 *     (<__pyx_buffer *> &result.view).obj = Py_None
 *     Py_INCREF(Py_None)             # <<<<<<<<<<<<<<
 * 
 *     result.flags = PyBUF_RECORDS
 */
  Py_INCREF(Py_None);

  /* "View.MemoryView":986
 *     Py_INCREF(Py_None)
 * 
 *     result.flags = PyBUF_RECORDS             # <<<<<<<<<<<<<<
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape
 */
  __pyx_v_result->__pyx_base.flags = PyBUF_RECORDS;

  /* "View.MemoryView":988
 *     result.flags = PyBUF_RECORDS
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape             # <<<<<<<<<<<<<<
 *     result.view.strides = <Py_ssize_t *> result.from_slice.strides
 * 
 */
  __pyx_v_result->__pyx_base.view.shape = ((Py_ssize_t *)__pyx_v_result->from_slice.shape);

  /* "View.MemoryView":989
 * 
 *     result.view.shape = <Py_ssize_t *> result.from_slice.shape
 *     result.view.strides = <Py_ssize_t *> result.from_slice.strides             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_result->__pyx_base.view.strides = ((Py_ssize_t *)__pyx_v_result->from_slice.strides);

  /* "View.MemoryView":992
 * 
 * 
 *     result.view.suboffsets = NULL             # <<<<<<<<<<<<<<
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:
 */
  __pyx_v_result->__pyx_base.view.suboffsets = NULL;

  /* "View.MemoryView":993
 * 
 *     result.view.suboffsets = NULL
 *     for suboffset in result.from_slice.suboffsets[:ndim]:             # <<<<<<<<<<<<<<
 *         if suboffset >= 0:
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 */
  __pyx_t_7 = (__pyx_v_result->from_slice.suboffsets + __pyx_v_ndim);
  for (__pyx_t_8 = __pyx_v_result->from_slice.suboffsets; __pyx_t_8 < __pyx_t_7; __pyx_t_8++) {
    __pyx_t_6 = __pyx_t_8;
    __pyx_v_suboffset = (__pyx_t_6[0]);

    /* "View.MemoryView":994
 *     result.view.suboffsets = NULL
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 *             break
 */
    __pyx_t_1 = ((__pyx_v_suboffset >= 0) != 0);
    if (__pyx_t_1) {

      /* "View.MemoryView":995
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets             # <<<<<<<<<<<<<<
 *             break
 * 
 */
      __pyx_v_result->__pyx_base.view.suboffsets = ((Py_ssize_t *)__pyx_v_result->from_slice.suboffsets);

      /* "View.MemoryView":996
 *         if suboffset >= 0:
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 *             break             # <<<<<<<<<<<<<<
 * 
 *     result.view.len = result.view.itemsize
 */
      goto __pyx_L5_break;

      /* "View.MemoryView":994
 *     result.view.suboffsets = NULL
 *     for suboffset in result.from_slice.suboffsets[:ndim]:
 *         if suboffset >= 0:             # <<<<<<<<<<<<<<
 *             result.view.suboffsets = <Py_ssize_t *> result.from_slice.suboffsets
 *             break
 */
    }
  }
  __pyx_L5_break:;

  /* "View.MemoryView":998
 *             break
 * 
 *     result.view.len = result.view.itemsize             # <<<<<<<<<<<<<<
 *     for length in result.view.shape[:ndim]:
 *         result.view.len *= length
 */
  __pyx_t_9 = __pyx_v_result->__pyx_base.view.itemsize;
  __pyx_v_result->__pyx_base.view.len = __pyx_t_9;

  /* "View.MemoryView":999
 * 
 *     result.view.len = result.view.itemsize
 *     for length in result.view.shape[:ndim]:             # <<<<<<<<<<<<<<
 *         result.view.len *= length
 * 
 */
  __pyx_t_7 = (__pyx_v_result->__pyx_base.view.shape + __pyx_v_ndim);
  for (__pyx_t_8 = __pyx_v_result->__pyx_base.view.shape; __pyx_t_8 < __pyx_t_7; __pyx_t_8++) {
    __pyx_t_6 = __pyx_t_8;
    __pyx_t_2 = PyInt_FromSsize_t((__pyx_t_6[0])); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 999; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_length, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "View.MemoryView":1000
 *     result.view.len = result.view.itemsize
 *     for length in result.view.shape[:ndim]:
 *         result.view.len *= length             # <<<<<<<<<<<<<<
 * 
 *     result.to_object_func = to_object_func
 */
    __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_result->__pyx_base.view.len); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1000; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyNumber_InPlaceMultiply(__pyx_t_2, __pyx_v_length); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1000; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __Pyx_PyIndex_AsSsize_t(__pyx_t_3); if (unlikely((__pyx_t_9 == (Py_ssize_t)-1) && PyErr_Occurred())) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1000; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_result->__pyx_base.view.len = __pyx_t_9;
  }

  /* "View.MemoryView":1002
 *         result.view.len *= length
 * 
 *     result.to_object_func = to_object_func             # <<<<<<<<<<<<<<
 *     result.to_dtype_func = to_dtype_func
 * 
 */
  __pyx_v_result->to_object_func = __pyx_v_to_object_func;

  /* "View.MemoryView":1003
 * 
 *     result.to_object_func = to_object_func
 *     result.to_dtype_func = to_dtype_func             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
  __pyx_v_result->to_dtype_func = __pyx_v_to_dtype_func;

  /* "View.MemoryView":1005
 *     result.to_dtype_func = to_dtype_func
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "View.MemoryView":958
 * 
 * @cname('__pyx_memoryview_fromslice')
 * cdef memoryview_fromslice(__Pyx_memviewslice memviewslice,             # <<<<<<<<<<<<<<
 *                           int ndim,
 *                           object (*to_object_func)(char *),
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("View.MemoryView.memoryview_fromslice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_length);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":1008
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')
 * cdef __Pyx_memviewslice *get_slice_from_memview(memoryview memview,             # <<<<<<<<<<<<<<
 *                                                    __Pyx_memviewslice *mslice):
 *     cdef _memoryviewslice obj
 */

static __Pyx_memviewslice *__pyx_memoryview_get_slice_from_memoryview(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_mslice) {
  struct __pyx_memoryviewslice_obj *__pyx_v_obj = 0;
  __Pyx_memviewslice *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_slice_from_memview", 0);

  /* "View.MemoryView":1011
 *                                                    __Pyx_memviewslice *mslice):
 *     cdef _memoryviewslice obj
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         obj = memview
 *         return &obj.from_slice
 */
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1012
 *     cdef _memoryviewslice obj
 *     if isinstance(memview, _memoryviewslice):
 *         obj = memview             # <<<<<<<<<<<<<<
 *         return &obj.from_slice
 *     else:
 */
    if (!(likely(((((PyObject *)__pyx_v_memview)) == Py_None) || likely(__Pyx_TypeTest(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type))))) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1012; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = ((PyObject *)__pyx_v_memview);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_obj = ((struct __pyx_memoryviewslice_obj *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "View.MemoryView":1013
 *     if isinstance(memview, _memoryviewslice):
 *         obj = memview
 *         return &obj.from_slice             # <<<<<<<<<<<<<<
 *     else:
 *         slice_copy(memview, mslice)
 */
    __pyx_r = (&__pyx_v_obj->from_slice);
    goto __pyx_L0;

    /* "View.MemoryView":1011
 *                                                    __Pyx_memviewslice *mslice):
 *     cdef _memoryviewslice obj
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         obj = memview
 *         return &obj.from_slice
 */
  }

  /* "View.MemoryView":1015
 *         return &obj.from_slice
 *     else:
 *         slice_copy(memview, mslice)             # <<<<<<<<<<<<<<
 *         return mslice
 * 
 */
  /*else*/ {
    __pyx_memoryview_slice_copy(__pyx_v_memview, __pyx_v_mslice);

    /* "View.MemoryView":1016
 *     else:
 *         slice_copy(memview, mslice)
 *         return mslice             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_slice_copy')
 */
    __pyx_r = __pyx_v_mslice;
    goto __pyx_L0;
  }

  /* "View.MemoryView":1008
 * 
 * @cname('__pyx_memoryview_get_slice_from_memoryview')
 * cdef __Pyx_memviewslice *get_slice_from_memview(memoryview memview,             # <<<<<<<<<<<<<<
 *                                                    __Pyx_memviewslice *mslice):
 *     cdef _memoryviewslice obj
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_WriteUnraisable("View.MemoryView.get_slice_from_memview", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_obj);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":1019
 * 
 * @cname('__pyx_memoryview_slice_copy')
 * cdef void slice_copy(memoryview memview, __Pyx_memviewslice *dst):             # <<<<<<<<<<<<<<
 *     cdef int dim
 *     cdef (Py_ssize_t*) shape, strides, suboffsets
 */

static void __pyx_memoryview_slice_copy(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_dst) {
  int __pyx_v_dim;
  Py_ssize_t *__pyx_v_shape;
  Py_ssize_t *__pyx_v_strides;
  Py_ssize_t *__pyx_v_suboffsets;
  __Pyx_RefNannyDeclarations
  Py_ssize_t *__pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  __Pyx_RefNannySetupContext("slice_copy", 0);

  /* "View.MemoryView":1023
 *     cdef (Py_ssize_t*) shape, strides, suboffsets
 * 
 *     shape = memview.view.shape             # <<<<<<<<<<<<<<
 *     strides = memview.view.strides
 *     suboffsets = memview.view.suboffsets
 */
  __pyx_t_1 = __pyx_v_memview->view.shape;
  __pyx_v_shape = __pyx_t_1;

  /* "View.MemoryView":1024
 * 
 *     shape = memview.view.shape
 *     strides = memview.view.strides             # <<<<<<<<<<<<<<
 *     suboffsets = memview.view.suboffsets
 * 
 */
  __pyx_t_1 = __pyx_v_memview->view.strides;
  __pyx_v_strides = __pyx_t_1;

  /* "View.MemoryView":1025
 *     shape = memview.view.shape
 *     strides = memview.view.strides
 *     suboffsets = memview.view.suboffsets             # <<<<<<<<<<<<<<
 * 
 *     dst.memview = <__pyx_memoryview *> memview
 */
  __pyx_t_1 = __pyx_v_memview->view.suboffsets;
  __pyx_v_suboffsets = __pyx_t_1;

  /* "View.MemoryView":1027
 *     suboffsets = memview.view.suboffsets
 * 
 *     dst.memview = <__pyx_memoryview *> memview             # <<<<<<<<<<<<<<
 *     dst.data = <char *> memview.view.buf
 * 
 */
  __pyx_v_dst->memview = ((struct __pyx_memoryview_obj *)__pyx_v_memview);

  /* "View.MemoryView":1028
 * 
 *     dst.memview = <__pyx_memoryview *> memview
 *     dst.data = <char *> memview.view.buf             # <<<<<<<<<<<<<<
 * 
 *     for dim in range(memview.view.ndim):
 */
  __pyx_v_dst->data = ((char *)__pyx_v_memview->view.buf);

  /* "View.MemoryView":1030
 *     dst.data = <char *> memview.view.buf
 * 
 *     for dim in range(memview.view.ndim):             # <<<<<<<<<<<<<<
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]
 */
  __pyx_t_2 = __pyx_v_memview->view.ndim;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_dim = __pyx_t_3;

    /* "View.MemoryView":1031
 * 
 *     for dim in range(memview.view.ndim):
 *         dst.shape[dim] = shape[dim]             # <<<<<<<<<<<<<<
 *         dst.strides[dim] = strides[dim]
 *         dst.suboffsets[dim] = suboffsets[dim] if suboffsets else -1
 */
    (__pyx_v_dst->shape[__pyx_v_dim]) = (__pyx_v_shape[__pyx_v_dim]);

    /* "View.MemoryView":1032
 *     for dim in range(memview.view.ndim):
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]             # <<<<<<<<<<<<<<
 *         dst.suboffsets[dim] = suboffsets[dim] if suboffsets else -1
 * 
 */
    (__pyx_v_dst->strides[__pyx_v_dim]) = (__pyx_v_strides[__pyx_v_dim]);

    /* "View.MemoryView":1033
 *         dst.shape[dim] = shape[dim]
 *         dst.strides[dim] = strides[dim]
 *         dst.suboffsets[dim] = suboffsets[dim] if suboffsets else -1             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_object')
 */
    if ((__pyx_v_suboffsets != 0)) {
      __pyx_t_4 = (__pyx_v_suboffsets[__pyx_v_dim]);
    } else {
      __pyx_t_4 = -1L;
    }
    (__pyx_v_dst->suboffsets[__pyx_v_dim]) = __pyx_t_4;
  }

  /* "View.MemoryView":1019
 * 
 * @cname('__pyx_memoryview_slice_copy')
 * cdef void slice_copy(memoryview memview, __Pyx_memviewslice *dst):             # <<<<<<<<<<<<<<
 *     cdef int dim
 *     cdef (Py_ssize_t*) shape, strides, suboffsets
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "View.MemoryView":1036
 * 
 * @cname('__pyx_memoryview_copy_object')
 * cdef memoryview_copy(memoryview memview):             # <<<<<<<<<<<<<<
 *     "Create a new memoryview object"
 *     cdef __Pyx_memviewslice memviewslice
 */

static PyObject *__pyx_memoryview_copy_object(struct __pyx_memoryview_obj *__pyx_v_memview) {
  __Pyx_memviewslice __pyx_v_memviewslice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memoryview_copy", 0);

  /* "View.MemoryView":1039
 *     "Create a new memoryview object"
 *     cdef __Pyx_memviewslice memviewslice
 *     slice_copy(memview, &memviewslice)             # <<<<<<<<<<<<<<
 *     return memoryview_copy_from_slice(memview, &memviewslice)
 * 
 */
  __pyx_memoryview_slice_copy(__pyx_v_memview, (&__pyx_v_memviewslice));

  /* "View.MemoryView":1040
 *     cdef __Pyx_memviewslice memviewslice
 *     slice_copy(memview, &memviewslice)
 *     return memoryview_copy_from_slice(memview, &memviewslice)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_memoryview_copy_object_from_slice(__pyx_v_memview, (&__pyx_v_memviewslice)); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1040; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":1036
 * 
 * @cname('__pyx_memoryview_copy_object')
 * cdef memoryview_copy(memoryview memview):             # <<<<<<<<<<<<<<
 *     "Create a new memoryview object"
 *     cdef __Pyx_memviewslice memviewslice
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("View.MemoryView.memoryview_copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":1043
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')
 * cdef memoryview_copy_from_slice(memoryview memview, __Pyx_memviewslice *memviewslice):             # <<<<<<<<<<<<<<
 *     """
 *     Create a new memoryview object from a given memoryview object and slice.
 */

static PyObject *__pyx_memoryview_copy_object_from_slice(struct __pyx_memoryview_obj *__pyx_v_memview, __Pyx_memviewslice *__pyx_v_memviewslice) {
  PyObject *(*__pyx_v_to_object_func)(char *);
  int (*__pyx_v_to_dtype_func)(char *, PyObject *);
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *(*__pyx_t_3)(char *);
  int (*__pyx_t_4)(char *, PyObject *);
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("memoryview_copy_from_slice", 0);

  /* "View.MemoryView":1050
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         to_object_func = (<_memoryviewslice> memview).to_object_func
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 */
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_memview), __pyx_memoryviewslice_type); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1051
 * 
 *     if isinstance(memview, _memoryviewslice):
 *         to_object_func = (<_memoryviewslice> memview).to_object_func             # <<<<<<<<<<<<<<
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 *     else:
 */
    __pyx_t_3 = ((struct __pyx_memoryviewslice_obj *)__pyx_v_memview)->to_object_func;
    __pyx_v_to_object_func = __pyx_t_3;

    /* "View.MemoryView":1052
 *     if isinstance(memview, _memoryviewslice):
 *         to_object_func = (<_memoryviewslice> memview).to_object_func
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func             # <<<<<<<<<<<<<<
 *     else:
 *         to_object_func = NULL
 */
    __pyx_t_4 = ((struct __pyx_memoryviewslice_obj *)__pyx_v_memview)->to_dtype_func;
    __pyx_v_to_dtype_func = __pyx_t_4;

    /* "View.MemoryView":1050
 *     cdef int (*to_dtype_func)(char *, object) except 0
 * 
 *     if isinstance(memview, _memoryviewslice):             # <<<<<<<<<<<<<<
 *         to_object_func = (<_memoryviewslice> memview).to_object_func
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":1054
 *         to_dtype_func = (<_memoryviewslice> memview).to_dtype_func
 *     else:
 *         to_object_func = NULL             # <<<<<<<<<<<<<<
 *         to_dtype_func = NULL
 * 
 */
  /*else*/ {
    __pyx_v_to_object_func = NULL;

    /* "View.MemoryView":1055
 *     else:
 *         to_object_func = NULL
 *         to_dtype_func = NULL             # <<<<<<<<<<<<<<
 * 
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,
 */
    __pyx_v_to_dtype_func = NULL;
  }
  __pyx_L3:;

  /* "View.MemoryView":1057
 *         to_dtype_func = NULL
 * 
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,             # <<<<<<<<<<<<<<
 *                                 to_object_func, to_dtype_func,
 *                                 memview.dtype_is_object)
 */
  __Pyx_XDECREF(__pyx_r);

  /* "View.MemoryView":1059
 *     return memoryview_fromslice(memviewslice[0], memview.view.ndim,
 *                                 to_object_func, to_dtype_func,
 *                                 memview.dtype_is_object)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_5 = __pyx_memoryview_fromslice((__pyx_v_memviewslice[0]), __pyx_v_memview->view.ndim, __pyx_v_to_object_func, __pyx_v_to_dtype_func, __pyx_v_memview->dtype_is_object); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1057; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "View.MemoryView":1043
 * 
 * @cname('__pyx_memoryview_copy_object_from_slice')
 * cdef memoryview_copy_from_slice(memoryview memview, __Pyx_memviewslice *memviewslice):             # <<<<<<<<<<<<<<
 *     """
 *     Create a new memoryview object from a given memoryview object and slice.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView.memoryview_copy_from_slice", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "View.MemoryView":1065
 * 
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:             # <<<<<<<<<<<<<<
 *     if arg < 0:
 *         return -arg
 */

static Py_ssize_t abs_py_ssize_t(Py_ssize_t __pyx_v_arg) {
  Py_ssize_t __pyx_r;
  int __pyx_t_1;

  /* "View.MemoryView":1066
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:
 *     if arg < 0:             # <<<<<<<<<<<<<<
 *         return -arg
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_arg < 0) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1067
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:
 *     if arg < 0:
 *         return -arg             # <<<<<<<<<<<<<<
 *     else:
 *         return arg
 */
    __pyx_r = (-__pyx_v_arg);
    goto __pyx_L0;

    /* "View.MemoryView":1066
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:
 *     if arg < 0:             # <<<<<<<<<<<<<<
 *         return -arg
 *     else:
 */
  }

  /* "View.MemoryView":1069
 *         return -arg
 *     else:
 *         return arg             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_get_best_slice_order')
 */
  /*else*/ {
    __pyx_r = __pyx_v_arg;
    goto __pyx_L0;
  }

  /* "View.MemoryView":1065
 * 
 * 
 * cdef Py_ssize_t abs_py_ssize_t(Py_ssize_t arg) nogil:             # <<<<<<<<<<<<<<
 *     if arg < 0:
 *         return -arg
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1072
 * 
 * @cname('__pyx_get_best_slice_order')
 * cdef char get_best_order(__Pyx_memviewslice *mslice, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Figure out the best memory access order for a given slice.
 */

static char __pyx_get_best_slice_order(__Pyx_memviewslice *__pyx_v_mslice, int __pyx_v_ndim) {
  int __pyx_v_i;
  Py_ssize_t __pyx_v_c_stride;
  Py_ssize_t __pyx_v_f_stride;
  char __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "View.MemoryView":1077
 *     """
 *     cdef int i
 *     cdef Py_ssize_t c_stride = 0             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t f_stride = 0
 * 
 */
  __pyx_v_c_stride = 0;

  /* "View.MemoryView":1078
 *     cdef int i
 *     cdef Py_ssize_t c_stride = 0
 *     cdef Py_ssize_t f_stride = 0             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim - 1, -1, -1):
 */
  __pyx_v_f_stride = 0;

  /* "View.MemoryView":1080
 *     cdef Py_ssize_t f_stride = 0
 * 
 *     for i in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]
 */
  for (__pyx_t_1 = (__pyx_v_ndim - 1); __pyx_t_1 > -1L; __pyx_t_1-=1) {
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":1081
 * 
 *     for i in range(ndim - 1, -1, -1):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             c_stride = mslice.strides[i]
 *             break
 */
    __pyx_t_2 = (((__pyx_v_mslice->shape[__pyx_v_i]) > 1) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1082
 *     for i in range(ndim - 1, -1, -1):
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]             # <<<<<<<<<<<<<<
 *             break
 * 
 */
      __pyx_v_c_stride = (__pyx_v_mslice->strides[__pyx_v_i]);

      /* "View.MemoryView":1083
 *         if mslice.shape[i] > 1:
 *             c_stride = mslice.strides[i]
 *             break             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim):
 */
      goto __pyx_L4_break;

      /* "View.MemoryView":1081
 * 
 *     for i in range(ndim - 1, -1, -1):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             c_stride = mslice.strides[i]
 *             break
 */
    }
  }
  __pyx_L4_break:;

  /* "View.MemoryView":1085
 *             break
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]
 */
  __pyx_t_1 = __pyx_v_ndim;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_1; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "View.MemoryView":1086
 * 
 *     for i in range(ndim):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             f_stride = mslice.strides[i]
 *             break
 */
    __pyx_t_2 = (((__pyx_v_mslice->shape[__pyx_v_i]) > 1) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1087
 *     for i in range(ndim):
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]             # <<<<<<<<<<<<<<
 *             break
 * 
 */
      __pyx_v_f_stride = (__pyx_v_mslice->strides[__pyx_v_i]);

      /* "View.MemoryView":1088
 *         if mslice.shape[i] > 1:
 *             f_stride = mslice.strides[i]
 *             break             # <<<<<<<<<<<<<<
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):
 */
      goto __pyx_L7_break;

      /* "View.MemoryView":1086
 * 
 *     for i in range(ndim):
 *         if mslice.shape[i] > 1:             # <<<<<<<<<<<<<<
 *             f_stride = mslice.strides[i]
 *             break
 */
    }
  }
  __pyx_L7_break:;

  /* "View.MemoryView":1090
 *             break
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):             # <<<<<<<<<<<<<<
 *         return 'C'
 *     else:
 */
  __pyx_t_2 = ((abs_py_ssize_t(__pyx_v_c_stride) <= abs_py_ssize_t(__pyx_v_f_stride)) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1091
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):
 *         return 'C'             # <<<<<<<<<<<<<<
 *     else:
 *         return 'F'
 */
    __pyx_r = 'C';
    goto __pyx_L0;

    /* "View.MemoryView":1090
 *             break
 * 
 *     if abs_py_ssize_t(c_stride) <= abs_py_ssize_t(f_stride):             # <<<<<<<<<<<<<<
 *         return 'C'
 *     else:
 */
  }

  /* "View.MemoryView":1093
 *         return 'C'
 *     else:
 *         return 'F'             # <<<<<<<<<<<<<<
 * 
 * @cython.cdivision(True)
 */
  /*else*/ {
    __pyx_r = 'F';
    goto __pyx_L0;
  }

  /* "View.MemoryView":1072
 * 
 * @cname('__pyx_get_best_slice_order')
 * cdef char get_best_order(__Pyx_memviewslice *mslice, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     """
 *     Figure out the best memory access order for a given slice.
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1096
 * 
 * @cython.cdivision(True)
 * cdef void _copy_strided_to_strided(char *src_data, Py_ssize_t *src_strides,             # <<<<<<<<<<<<<<
 *                                    char *dst_data, Py_ssize_t *dst_strides,
 *                                    Py_ssize_t *src_shape, Py_ssize_t *dst_shape,
 */

static void _copy_strided_to_strided(char *__pyx_v_src_data, Py_ssize_t *__pyx_v_src_strides, char *__pyx_v_dst_data, Py_ssize_t *__pyx_v_dst_strides, Py_ssize_t *__pyx_v_src_shape, Py_ssize_t *__pyx_v_dst_shape, int __pyx_v_ndim, size_t __pyx_v_itemsize) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i;
  CYTHON_UNUSED Py_ssize_t __pyx_v_src_extent;
  Py_ssize_t __pyx_v_dst_extent;
  Py_ssize_t __pyx_v_src_stride;
  Py_ssize_t __pyx_v_dst_stride;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;

  /* "View.MemoryView":1103
 * 
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t src_extent = src_shape[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]
 */
  __pyx_v_src_extent = (__pyx_v_src_shape[0]);

  /* "View.MemoryView":1104
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t src_extent = src_shape[0]
 *     cdef Py_ssize_t dst_extent = dst_shape[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t src_stride = src_strides[0]
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 */
  __pyx_v_dst_extent = (__pyx_v_dst_shape[0]);

  /* "View.MemoryView":1105
 *     cdef Py_ssize_t src_extent = src_shape[0]
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 * 
 */
  __pyx_v_src_stride = (__pyx_v_src_strides[0]);

  /* "View.MemoryView":1106
 *     cdef Py_ssize_t dst_extent = dst_shape[0]
 *     cdef Py_ssize_t src_stride = src_strides[0]
 *     cdef Py_ssize_t dst_stride = dst_strides[0]             # <<<<<<<<<<<<<<
 * 
 *     if ndim == 1:
 */
  __pyx_v_dst_stride = (__pyx_v_dst_strides[0]);

  /* "View.MemoryView":1108
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 */
  __pyx_t_1 = ((__pyx_v_ndim == 1) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1109
 * 
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and             # <<<<<<<<<<<<<<
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 */
    __pyx_t_2 = ((__pyx_v_src_stride > 0) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_dst_stride > 0) != 0);
    if (__pyx_t_2) {
    } else {
      __pyx_t_1 = __pyx_t_2;
      goto __pyx_L5_bool_binop_done;
    }

    /* "View.MemoryView":1110
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):             # <<<<<<<<<<<<<<
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 *        else:
 */
    __pyx_t_2 = (((size_t)__pyx_v_src_stride) == __pyx_v_itemsize);
    if (__pyx_t_2) {
      __pyx_t_2 = (__pyx_v_itemsize == ((size_t)__pyx_v_dst_stride));
    }
    __pyx_t_3 = (__pyx_t_2 != 0);
    __pyx_t_1 = __pyx_t_3;
    __pyx_L5_bool_binop_done:;

    /* "View.MemoryView":1109
 * 
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and             # <<<<<<<<<<<<<<
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 */
    if (__pyx_t_1) {

      /* "View.MemoryView":1111
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)             # <<<<<<<<<<<<<<
 *        else:
 *            for i in range(dst_extent):
 */
      memcpy(__pyx_v_dst_data, __pyx_v_src_data, (__pyx_v_itemsize * __pyx_v_dst_extent));

      /* "View.MemoryView":1109
 * 
 *     if ndim == 1:
 *        if (src_stride > 0 and dst_stride > 0 and             # <<<<<<<<<<<<<<
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 */
      goto __pyx_L4;
    }

    /* "View.MemoryView":1113
 *            memcpy(dst_data, src_data, itemsize * dst_extent)
 *        else:
 *            for i in range(dst_extent):             # <<<<<<<<<<<<<<
 *                memcpy(dst_data, src_data, itemsize)
 *                src_data += src_stride
 */
    /*else*/ {
      __pyx_t_4 = __pyx_v_dst_extent;
      for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
        __pyx_v_i = __pyx_t_5;

        /* "View.MemoryView":1114
 *        else:
 *            for i in range(dst_extent):
 *                memcpy(dst_data, src_data, itemsize)             # <<<<<<<<<<<<<<
 *                src_data += src_stride
 *                dst_data += dst_stride
 */
        memcpy(__pyx_v_dst_data, __pyx_v_src_data, __pyx_v_itemsize);

        /* "View.MemoryView":1115
 *            for i in range(dst_extent):
 *                memcpy(dst_data, src_data, itemsize)
 *                src_data += src_stride             # <<<<<<<<<<<<<<
 *                dst_data += dst_stride
 *     else:
 */
        __pyx_v_src_data = (__pyx_v_src_data + __pyx_v_src_stride);

        /* "View.MemoryView":1116
 *                memcpy(dst_data, src_data, itemsize)
 *                src_data += src_stride
 *                dst_data += dst_stride             # <<<<<<<<<<<<<<
 *     else:
 *         for i in range(dst_extent):
 */
        __pyx_v_dst_data = (__pyx_v_dst_data + __pyx_v_dst_stride);
      }
    }
    __pyx_L4:;

    /* "View.MemoryView":1108
 *     cdef Py_ssize_t dst_stride = dst_strides[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *        if (src_stride > 0 and dst_stride > 0 and
 *            <size_t> src_stride == itemsize == <size_t> dst_stride):
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":1118
 *                dst_data += dst_stride
 *     else:
 *         for i in range(dst_extent):             # <<<<<<<<<<<<<<
 *             _copy_strided_to_strided(src_data, src_strides + 1,
 *                                      dst_data, dst_strides + 1,
 */
  /*else*/ {
    __pyx_t_4 = __pyx_v_dst_extent;
    for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
      __pyx_v_i = __pyx_t_5;

      /* "View.MemoryView":1119
 *     else:
 *         for i in range(dst_extent):
 *             _copy_strided_to_strided(src_data, src_strides + 1,             # <<<<<<<<<<<<<<
 *                                      dst_data, dst_strides + 1,
 *                                      src_shape + 1, dst_shape + 1,
 */
      _copy_strided_to_strided(__pyx_v_src_data, (__pyx_v_src_strides + 1), __pyx_v_dst_data, (__pyx_v_dst_strides + 1), (__pyx_v_src_shape + 1), (__pyx_v_dst_shape + 1), (__pyx_v_ndim - 1), __pyx_v_itemsize);

      /* "View.MemoryView":1123
 *                                      src_shape + 1, dst_shape + 1,
 *                                      ndim - 1, itemsize)
 *             src_data += src_stride             # <<<<<<<<<<<<<<
 *             dst_data += dst_stride
 * 
 */
      __pyx_v_src_data = (__pyx_v_src_data + __pyx_v_src_stride);

      /* "View.MemoryView":1124
 *                                      ndim - 1, itemsize)
 *             src_data += src_stride
 *             dst_data += dst_stride             # <<<<<<<<<<<<<<
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,
 */
      __pyx_v_dst_data = (__pyx_v_dst_data + __pyx_v_dst_stride);
    }
  }
  __pyx_L3:;

  /* "View.MemoryView":1096
 * 
 * @cython.cdivision(True)
 * cdef void _copy_strided_to_strided(char *src_data, Py_ssize_t *src_strides,             # <<<<<<<<<<<<<<
 *                                    char *dst_data, Py_ssize_t *dst_strides,
 *                                    Py_ssize_t *src_shape, Py_ssize_t *dst_shape,
 */

  /* function exit code */
}

/* "View.MemoryView":1126
 *             dst_data += dst_stride
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) nogil:
 */

static void copy_strided_to_strided(__Pyx_memviewslice *__pyx_v_src, __Pyx_memviewslice *__pyx_v_dst, int __pyx_v_ndim, size_t __pyx_v_itemsize) {

  /* "View.MemoryView":1129
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) nogil:
 *     _copy_strided_to_strided(src.data, src.strides, dst.data, dst.strides,             # <<<<<<<<<<<<<<
 *                              src.shape, dst.shape, ndim, itemsize)
 * 
 */
  _copy_strided_to_strided(__pyx_v_src->data, __pyx_v_src->strides, __pyx_v_dst->data, __pyx_v_dst->strides, __pyx_v_src->shape, __pyx_v_dst->shape, __pyx_v_ndim, __pyx_v_itemsize);

  /* "View.MemoryView":1126
 *             dst_data += dst_stride
 * 
 * cdef void copy_strided_to_strided(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice *dst,
 *                                   int ndim, size_t itemsize) nogil:
 */

  /* function exit code */
}

/* "View.MemoryView":1133
 * 
 * @cname('__pyx_memoryview_slice_get_size')
 * cdef Py_ssize_t slice_get_size(__Pyx_memviewslice *src, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     "Return the size of the memory occupied by the slice in number of bytes"
 *     cdef int i
 */

static Py_ssize_t __pyx_memoryview_slice_get_size(__Pyx_memviewslice *__pyx_v_src, int __pyx_v_ndim) {
  int __pyx_v_i;
  Py_ssize_t __pyx_v_size;
  Py_ssize_t __pyx_r;
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "View.MemoryView":1136
 *     "Return the size of the memory occupied by the slice in number of bytes"
 *     cdef int i
 *     cdef Py_ssize_t size = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim):
 */
  __pyx_t_1 = __pyx_v_src->memview->view.itemsize;
  __pyx_v_size = __pyx_t_1;

  /* "View.MemoryView":1138
 *     cdef Py_ssize_t size = src.memview.view.itemsize
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         size *= src.shape[i]
 * 
 */
  __pyx_t_2 = __pyx_v_ndim;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "View.MemoryView":1139
 * 
 *     for i in range(ndim):
 *         size *= src.shape[i]             # <<<<<<<<<<<<<<
 * 
 *     return size
 */
    __pyx_v_size = (__pyx_v_size * (__pyx_v_src->shape[__pyx_v_i]));
  }

  /* "View.MemoryView":1141
 *         size *= src.shape[i]
 * 
 *     return size             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_fill_contig_strides_array')
 */
  __pyx_r = __pyx_v_size;
  goto __pyx_L0;

  /* "View.MemoryView":1133
 * 
 * @cname('__pyx_memoryview_slice_get_size')
 * cdef Py_ssize_t slice_get_size(__Pyx_memviewslice *src, int ndim) nogil:             # <<<<<<<<<<<<<<
 *     "Return the size of the memory occupied by the slice in number of bytes"
 *     cdef int i
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1144
 * 
 * @cname('__pyx_fill_contig_strides_array')
 * cdef Py_ssize_t fill_contig_strides_array(             # <<<<<<<<<<<<<<
 *                 Py_ssize_t *shape, Py_ssize_t *strides, Py_ssize_t stride,
 *                 int ndim, char order) nogil:
 */

static Py_ssize_t __pyx_fill_contig_strides_array(Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, Py_ssize_t __pyx_v_stride, int __pyx_v_ndim, char __pyx_v_order) {
  int __pyx_v_idx;
  Py_ssize_t __pyx_r;
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;

  /* "View.MemoryView":1153
 *     cdef int idx
 * 
 *     if order == 'F':             # <<<<<<<<<<<<<<
 *         for idx in range(ndim):
 *             strides[idx] = stride
 */
  __pyx_t_1 = ((__pyx_v_order == 'F') != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1154
 * 
 *     if order == 'F':
 *         for idx in range(ndim):             # <<<<<<<<<<<<<<
 *             strides[idx] = stride
 *             stride = stride * shape[idx]
 */
    __pyx_t_2 = __pyx_v_ndim;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_idx = __pyx_t_3;

      /* "View.MemoryView":1155
 *     if order == 'F':
 *         for idx in range(ndim):
 *             strides[idx] = stride             # <<<<<<<<<<<<<<
 *             stride = stride * shape[idx]
 *     else:
 */
      (__pyx_v_strides[__pyx_v_idx]) = __pyx_v_stride;

      /* "View.MemoryView":1156
 *         for idx in range(ndim):
 *             strides[idx] = stride
 *             stride = stride * shape[idx]             # <<<<<<<<<<<<<<
 *     else:
 *         for idx in range(ndim - 1, -1, -1):
 */
      __pyx_v_stride = (__pyx_v_stride * (__pyx_v_shape[__pyx_v_idx]));
    }

    /* "View.MemoryView":1153
 *     cdef int idx
 * 
 *     if order == 'F':             # <<<<<<<<<<<<<<
 *         for idx in range(ndim):
 *             strides[idx] = stride
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":1158
 *             stride = stride * shape[idx]
 *     else:
 *         for idx in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *             strides[idx] = stride
 *             stride = stride * shape[idx]
 */
  /*else*/ {
    for (__pyx_t_2 = (__pyx_v_ndim - 1); __pyx_t_2 > -1L; __pyx_t_2-=1) {
      __pyx_v_idx = __pyx_t_2;

      /* "View.MemoryView":1159
 *     else:
 *         for idx in range(ndim - 1, -1, -1):
 *             strides[idx] = stride             # <<<<<<<<<<<<<<
 *             stride = stride * shape[idx]
 * 
 */
      (__pyx_v_strides[__pyx_v_idx]) = __pyx_v_stride;

      /* "View.MemoryView":1160
 *         for idx in range(ndim - 1, -1, -1):
 *             strides[idx] = stride
 *             stride = stride * shape[idx]             # <<<<<<<<<<<<<<
 * 
 *     return stride
 */
      __pyx_v_stride = (__pyx_v_stride * (__pyx_v_shape[__pyx_v_idx]));
    }
  }
  __pyx_L3:;

  /* "View.MemoryView":1162
 *             stride = stride * shape[idx]
 * 
 *     return stride             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')
 */
  __pyx_r = __pyx_v_stride;
  goto __pyx_L0;

  /* "View.MemoryView":1144
 * 
 * @cname('__pyx_fill_contig_strides_array')
 * cdef Py_ssize_t fill_contig_strides_array(             # <<<<<<<<<<<<<<
 *                 Py_ssize_t *shape, Py_ssize_t *strides, Py_ssize_t stride,
 *                 int ndim, char order) nogil:
 */

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1165
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')
 * cdef void *copy_data_to_temp(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                              __Pyx_memviewslice *tmpslice,
 *                              char order,
 */

static void *__pyx_memoryview_copy_data_to_temp(__Pyx_memviewslice *__pyx_v_src, __Pyx_memviewslice *__pyx_v_tmpslice, char __pyx_v_order, int __pyx_v_ndim) {
  int __pyx_v_i;
  void *__pyx_v_result;
  size_t __pyx_v_itemsize;
  size_t __pyx_v_size;
  void *__pyx_r;
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  struct __pyx_memoryview_obj *__pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "View.MemoryView":1176
 *     cdef void *result
 * 
 *     cdef size_t itemsize = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 *     cdef size_t size = slice_get_size(src, ndim)
 * 
 */
  __pyx_t_1 = __pyx_v_src->memview->view.itemsize;
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":1177
 * 
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef size_t size = slice_get_size(src, ndim)             # <<<<<<<<<<<<<<
 * 
 *     result = malloc(size)
 */
  __pyx_v_size = __pyx_memoryview_slice_get_size(__pyx_v_src, __pyx_v_ndim);

  /* "View.MemoryView":1179
 *     cdef size_t size = slice_get_size(src, ndim)
 * 
 *     result = malloc(size)             # <<<<<<<<<<<<<<
 *     if not result:
 *         _err(MemoryError, NULL)
 */
  __pyx_v_result = malloc(__pyx_v_size);

  /* "View.MemoryView":1180
 * 
 *     result = malloc(size)
 *     if not result:             # <<<<<<<<<<<<<<
 *         _err(MemoryError, NULL)
 * 
 */
  __pyx_t_2 = ((!(__pyx_v_result != 0)) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1181
 *     result = malloc(size)
 *     if not result:
 *         _err(MemoryError, NULL)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_t_3 = __pyx_memoryview_err(__pyx_builtin_MemoryError, NULL); if (unlikely(__pyx_t_3 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "View.MemoryView":1180
 * 
 *     result = malloc(size)
 *     if not result:             # <<<<<<<<<<<<<<
 *         _err(MemoryError, NULL)
 * 
 */
  }

  /* "View.MemoryView":1184
 * 
 * 
 *     tmpslice.data = <char *> result             # <<<<<<<<<<<<<<
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):
 */
  __pyx_v_tmpslice->data = ((char *)__pyx_v_result);

  /* "View.MemoryView":1185
 * 
 *     tmpslice.data = <char *> result
 *     tmpslice.memview = src.memview             # <<<<<<<<<<<<<<
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]
 */
  __pyx_t_4 = __pyx_v_src->memview;
  __pyx_v_tmpslice->memview = __pyx_t_4;

  /* "View.MemoryView":1186
 *     tmpslice.data = <char *> result
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         tmpslice.shape[i] = src.shape[i]
 *         tmpslice.suboffsets[i] = -1
 */
  __pyx_t_3 = __pyx_v_ndim;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_3; __pyx_t_5+=1) {
    __pyx_v_i = __pyx_t_5;

    /* "View.MemoryView":1187
 *     tmpslice.memview = src.memview
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]             # <<<<<<<<<<<<<<
 *         tmpslice.suboffsets[i] = -1
 * 
 */
    (__pyx_v_tmpslice->shape[__pyx_v_i]) = (__pyx_v_src->shape[__pyx_v_i]);

    /* "View.MemoryView":1188
 *     for i in range(ndim):
 *         tmpslice.shape[i] = src.shape[i]
 *         tmpslice.suboffsets[i] = -1             # <<<<<<<<<<<<<<
 * 
 *     fill_contig_strides_array(&tmpslice.shape[0], &tmpslice.strides[0], itemsize,
 */
    (__pyx_v_tmpslice->suboffsets[__pyx_v_i]) = -1L;
  }

  /* "View.MemoryView":1190
 *         tmpslice.suboffsets[i] = -1
 * 
 *     fill_contig_strides_array(&tmpslice.shape[0], &tmpslice.strides[0], itemsize,             # <<<<<<<<<<<<<<
 *                               ndim, order)
 * 
 */
  __pyx_fill_contig_strides_array((&(__pyx_v_tmpslice->shape[0])), (&(__pyx_v_tmpslice->strides[0])), __pyx_v_itemsize, __pyx_v_ndim, __pyx_v_order);

  /* "View.MemoryView":1194
 * 
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if tmpslice.shape[i] == 1:
 *             tmpslice.strides[i] = 0
 */
  __pyx_t_3 = __pyx_v_ndim;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_3; __pyx_t_5+=1) {
    __pyx_v_i = __pyx_t_5;

    /* "View.MemoryView":1195
 * 
 *     for i in range(ndim):
 *         if tmpslice.shape[i] == 1:             # <<<<<<<<<<<<<<
 *             tmpslice.strides[i] = 0
 * 
 */
    __pyx_t_2 = (((__pyx_v_tmpslice->shape[__pyx_v_i]) == 1) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1196
 *     for i in range(ndim):
 *         if tmpslice.shape[i] == 1:
 *             tmpslice.strides[i] = 0             # <<<<<<<<<<<<<<
 * 
 *     if slice_is_contig(src, order, ndim):
 */
      (__pyx_v_tmpslice->strides[__pyx_v_i]) = 0;

      /* "View.MemoryView":1195
 * 
 *     for i in range(ndim):
 *         if tmpslice.shape[i] == 1:             # <<<<<<<<<<<<<<
 *             tmpslice.strides[i] = 0
 * 
 */
    }
  }

  /* "View.MemoryView":1198
 *             tmpslice.strides[i] = 0
 * 
 *     if slice_is_contig(src, order, ndim):             # <<<<<<<<<<<<<<
 *         memcpy(result, src.data, size)
 *     else:
 */
  __pyx_t_2 = (__pyx_memviewslice_is_contig(__pyx_v_src, __pyx_v_order, __pyx_v_ndim) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1199
 * 
 *     if slice_is_contig(src, order, ndim):
 *         memcpy(result, src.data, size)             # <<<<<<<<<<<<<<
 *     else:
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)
 */
    memcpy(__pyx_v_result, __pyx_v_src->data, __pyx_v_size);

    /* "View.MemoryView":1198
 *             tmpslice.strides[i] = 0
 * 
 *     if slice_is_contig(src, order, ndim):             # <<<<<<<<<<<<<<
 *         memcpy(result, src.data, size)
 *     else:
 */
    goto __pyx_L9;
  }

  /* "View.MemoryView":1201
 *         memcpy(result, src.data, size)
 *     else:
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
  /*else*/ {
    copy_strided_to_strided(__pyx_v_src, __pyx_v_tmpslice, __pyx_v_ndim, __pyx_v_itemsize);
  }
  __pyx_L9:;

  /* "View.MemoryView":1203
 *         copy_strided_to_strided(src, tmpslice, ndim, itemsize)
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "View.MemoryView":1165
 * 
 * @cname('__pyx_memoryview_copy_data_to_temp')
 * cdef void *copy_data_to_temp(__Pyx_memviewslice *src,             # <<<<<<<<<<<<<<
 *                              __Pyx_memviewslice *tmpslice,
 *                              char order,
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("View.MemoryView.copy_data_to_temp", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = NULL;
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1208
 * 
 * @cname('__pyx_memoryview_err_extents')
 * cdef int _err_extents(int i, Py_ssize_t extent1,             # <<<<<<<<<<<<<<
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %
 */

static int __pyx_memoryview_err_extents(int __pyx_v_i, Py_ssize_t __pyx_v_extent1, Py_ssize_t __pyx_v_extent2) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_err_extents", 0);

  /* "View.MemoryView":1211
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %
 *                                                         (i, extent1, extent2))             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_err_dim')
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_i); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1211; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_extent1); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1211; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_extent2); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1211; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1211; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;

  /* "View.MemoryView":1210
 * cdef int _err_extents(int i, Py_ssize_t extent1,
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %             # <<<<<<<<<<<<<<
 *                                                         (i, extent1, extent2))
 * 
 */
  __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_got_differing_extents_in_dimensi, __pyx_t_4); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1210; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1210; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1210; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_Raise(__pyx_t_3, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1210; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "View.MemoryView":1208
 * 
 * @cname('__pyx_memoryview_err_extents')
 * cdef int _err_extents(int i, Py_ssize_t extent1,             # <<<<<<<<<<<<<<
 *                              Py_ssize_t extent2) except -1 with gil:
 *     raise ValueError("got differing extents in dimension %d (got %d and %d)" %
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("View.MemoryView._err_extents", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "View.MemoryView":1214
 * 
 * @cname('__pyx_memoryview_err_dim')
 * cdef int _err_dim(object error, char *msg, int dim) except -1 with gil:             # <<<<<<<<<<<<<<
 *     raise error(msg.decode('ascii') % dim)
 * 
 */

static int __pyx_memoryview_err_dim(PyObject *__pyx_v_error, char *__pyx_v_msg, int __pyx_v_dim) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_err_dim", 0);
  __Pyx_INCREF(__pyx_v_error);

  /* "View.MemoryView":1215
 * @cname('__pyx_memoryview_err_dim')
 * cdef int _err_dim(object error, char *msg, int dim) except -1 with gil:
 *     raise error(msg.decode('ascii') % dim)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_err')
 */
  __pyx_t_2 = __Pyx_decode_c_string(__pyx_v_msg, 0, strlen(__pyx_v_msg), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_dim); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyUnicode_Format(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_INCREF(__pyx_v_error);
  __pyx_t_3 = __pyx_v_error; __pyx_t_2 = NULL;
  if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_2) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2); __pyx_t_2 = NULL;
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "View.MemoryView":1214
 * 
 * @cname('__pyx_memoryview_err_dim')
 * cdef int _err_dim(object error, char *msg, int dim) except -1 with gil:             # <<<<<<<<<<<<<<
 *     raise error(msg.decode('ascii') % dim)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("View.MemoryView._err_dim", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_XDECREF(__pyx_v_error);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "View.MemoryView":1218
 * 
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:             # <<<<<<<<<<<<<<
 *     if msg != NULL:
 *         raise error(msg.decode('ascii'))
 */

static int __pyx_memoryview_err(PyObject *__pyx_v_error, char *__pyx_v_msg) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("_err", 0);
  __Pyx_INCREF(__pyx_v_error);

  /* "View.MemoryView":1219
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:
 *     if msg != NULL:             # <<<<<<<<<<<<<<
 *         raise error(msg.decode('ascii'))
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_msg != NULL) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1220
 * cdef int _err(object error, char *msg) except -1 with gil:
 *     if msg != NULL:
 *         raise error(msg.decode('ascii'))             # <<<<<<<<<<<<<<
 *     else:
 *         raise error
 */
    __pyx_t_3 = __Pyx_decode_c_string(__pyx_v_msg, 0, strlen(__pyx_v_msg), NULL, NULL, PyUnicode_DecodeASCII); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_error);
    __pyx_t_4 = __pyx_v_error; __pyx_t_5 = NULL;
    if (CYTHON_COMPILING_IN_CPYTHON && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1220; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "View.MemoryView":1219
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:
 *     if msg != NULL:             # <<<<<<<<<<<<<<
 *         raise error(msg.decode('ascii'))
 *     else:
 */
  }

  /* "View.MemoryView":1222
 *         raise error(msg.decode('ascii'))
 *     else:
 *         raise error             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_copy_contents')
 */
  /*else*/ {
    __Pyx_Raise(__pyx_v_error, 0, 0, 0);
    {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }

  /* "View.MemoryView":1218
 * 
 * @cname('__pyx_memoryview_err')
 * cdef int _err(object error, char *msg) except -1 with gil:             # <<<<<<<<<<<<<<
 *     if msg != NULL:
 *         raise error(msg.decode('ascii'))
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("View.MemoryView._err", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_XDECREF(__pyx_v_error);
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  PyGILState_Release(__pyx_gilstate_save);
  #endif
  return __pyx_r;
}

/* "View.MemoryView":1225
 * 
 * @cname('__pyx_memoryview_copy_contents')
 * cdef int memoryview_copy_contents(__Pyx_memviewslice src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice dst,
 *                                   int src_ndim, int dst_ndim,
 */

static int __pyx_memoryview_copy_contents(__Pyx_memviewslice __pyx_v_src, __Pyx_memviewslice __pyx_v_dst, int __pyx_v_src_ndim, int __pyx_v_dst_ndim, int __pyx_v_dtype_is_object) {
  void *__pyx_v_tmpdata;
  size_t __pyx_v_itemsize;
  int __pyx_v_i;
  char __pyx_v_order;
  int __pyx_v_broadcasting;
  int __pyx_v_direct_copy;
  __Pyx_memviewslice __pyx_v_tmp;
  int __pyx_v_ndim;
  int __pyx_r;
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  void *__pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "View.MemoryView":1233
 *     Check for overlapping memory and verify the shapes.
 *     """
 *     cdef void *tmpdata = NULL             # <<<<<<<<<<<<<<
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef int i
 */
  __pyx_v_tmpdata = NULL;

  /* "View.MemoryView":1234
 *     """
 *     cdef void *tmpdata = NULL
 *     cdef size_t itemsize = src.memview.view.itemsize             # <<<<<<<<<<<<<<
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)
 */
  __pyx_t_1 = __pyx_v_src.memview->view.itemsize;
  __pyx_v_itemsize = __pyx_t_1;

  /* "View.MemoryView":1236
 *     cdef size_t itemsize = src.memview.view.itemsize
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)             # <<<<<<<<<<<<<<
 *     cdef bint broadcasting = False
 *     cdef bint direct_copy = False
 */
  __pyx_v_order = __pyx_get_best_slice_order((&__pyx_v_src), __pyx_v_src_ndim);

  /* "View.MemoryView":1237
 *     cdef int i
 *     cdef char order = get_best_order(&src, src_ndim)
 *     cdef bint broadcasting = False             # <<<<<<<<<<<<<<
 *     cdef bint direct_copy = False
 *     cdef __Pyx_memviewslice tmp
 */
  __pyx_v_broadcasting = 0;

  /* "View.MemoryView":1238
 *     cdef char order = get_best_order(&src, src_ndim)
 *     cdef bint broadcasting = False
 *     cdef bint direct_copy = False             # <<<<<<<<<<<<<<
 *     cdef __Pyx_memviewslice tmp
 * 
 */
  __pyx_v_direct_copy = 0;

  /* "View.MemoryView":1241
 *     cdef __Pyx_memviewslice tmp
 * 
 *     if src_ndim < dst_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:
 */
  __pyx_t_2 = ((__pyx_v_src_ndim < __pyx_v_dst_ndim) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1242
 * 
 *     if src_ndim < dst_ndim:
 *         broadcast_leading(&src, src_ndim, dst_ndim)             # <<<<<<<<<<<<<<
 *     elif dst_ndim < src_ndim:
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 */
    __pyx_memoryview_broadcast_leading((&__pyx_v_src), __pyx_v_src_ndim, __pyx_v_dst_ndim);

    /* "View.MemoryView":1241
 *     cdef __Pyx_memviewslice tmp
 * 
 *     if src_ndim < dst_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":1243
 *     if src_ndim < dst_ndim:
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 * 
 */
  __pyx_t_2 = ((__pyx_v_dst_ndim < __pyx_v_src_ndim) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1244
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:
 *         broadcast_leading(&dst, dst_ndim, src_ndim)             # <<<<<<<<<<<<<<
 * 
 *     cdef int ndim = max(src_ndim, dst_ndim)
 */
    __pyx_memoryview_broadcast_leading((&__pyx_v_dst), __pyx_v_dst_ndim, __pyx_v_src_ndim);

    /* "View.MemoryView":1243
 *     if src_ndim < dst_ndim:
 *         broadcast_leading(&src, src_ndim, dst_ndim)
 *     elif dst_ndim < src_ndim:             # <<<<<<<<<<<<<<
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 * 
 */
  }
  __pyx_L3:;

  /* "View.MemoryView":1246
 *         broadcast_leading(&dst, dst_ndim, src_ndim)
 * 
 *     cdef int ndim = max(src_ndim, dst_ndim)             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim):
 */
  __pyx_t_3 = __pyx_v_dst_ndim;
  __pyx_t_4 = __pyx_v_src_ndim;
  if (((__pyx_t_3 > __pyx_t_4) != 0)) {
    __pyx_t_5 = __pyx_t_3;
  } else {
    __pyx_t_5 = __pyx_t_4;
  }
  __pyx_v_ndim = __pyx_t_5;

  /* "View.MemoryView":1248
 *     cdef int ndim = max(src_ndim, dst_ndim)
 * 
 *     for i in range(ndim):             # <<<<<<<<<<<<<<
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:
 */
  __pyx_t_5 = __pyx_v_ndim;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_5; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "View.MemoryView":1249
 * 
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:             # <<<<<<<<<<<<<<
 *             if src.shape[i] == 1:
 *                 broadcasting = True
 */
    __pyx_t_2 = (((__pyx_v_src.shape[__pyx_v_i]) != (__pyx_v_dst.shape[__pyx_v_i])) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1250
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:             # <<<<<<<<<<<<<<
 *                 broadcasting = True
 *                 src.strides[i] = 0
 */
      __pyx_t_2 = (((__pyx_v_src.shape[__pyx_v_i]) == 1) != 0);
      if (__pyx_t_2) {

        /* "View.MemoryView":1251
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:
 *                 broadcasting = True             # <<<<<<<<<<<<<<
 *                 src.strides[i] = 0
 *             else:
 */
        __pyx_v_broadcasting = 1;

        /* "View.MemoryView":1252
 *             if src.shape[i] == 1:
 *                 broadcasting = True
 *                 src.strides[i] = 0             # <<<<<<<<<<<<<<
 *             else:
 *                 _err_extents(i, dst.shape[i], src.shape[i])
 */
        (__pyx_v_src.strides[__pyx_v_i]) = 0;

        /* "View.MemoryView":1250
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:
 *             if src.shape[i] == 1:             # <<<<<<<<<<<<<<
 *                 broadcasting = True
 *                 src.strides[i] = 0
 */
        goto __pyx_L7;
      }

      /* "View.MemoryView":1254
 *                 src.strides[i] = 0
 *             else:
 *                 _err_extents(i, dst.shape[i], src.shape[i])             # <<<<<<<<<<<<<<
 * 
 *         if src.suboffsets[i] >= 0:
 */
      /*else*/ {
        __pyx_t_4 = __pyx_memoryview_err_extents(__pyx_v_i, (__pyx_v_dst.shape[__pyx_v_i]), (__pyx_v_src.shape[__pyx_v_i])); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1254; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      }
      __pyx_L7:;

      /* "View.MemoryView":1249
 * 
 *     for i in range(ndim):
 *         if src.shape[i] != dst.shape[i]:             # <<<<<<<<<<<<<<
 *             if src.shape[i] == 1:
 *                 broadcasting = True
 */
    }

    /* "View.MemoryView":1256
 *                 _err_extents(i, dst.shape[i], src.shape[i])
 * 
 *         if src.suboffsets[i] >= 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 */
    __pyx_t_2 = (((__pyx_v_src.suboffsets[__pyx_v_i]) >= 0) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1257
 * 
 *         if src.suboffsets[i] >= 0:
 *             _err_dim(ValueError, "Dimension %d is not direct", i)             # <<<<<<<<<<<<<<
 * 
 *     if slices_overlap(&src, &dst, ndim, itemsize):
 */
      __pyx_t_4 = __pyx_memoryview_err_dim(__pyx_builtin_ValueError, __pyx_k_Dimension_d_is_not_direct, __pyx_v_i); if (unlikely(__pyx_t_4 == -1)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

      /* "View.MemoryView":1256
 *                 _err_extents(i, dst.shape[i], src.shape[i])
 * 
 *         if src.suboffsets[i] >= 0:             # <<<<<<<<<<<<<<
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 */
    }
  }

  /* "View.MemoryView":1259
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 *     if slices_overlap(&src, &dst, ndim, itemsize):             # <<<<<<<<<<<<<<
 * 
 *         if not slice_is_contig(&src, order, ndim):
 */
  __pyx_t_2 = (__pyx_slices_overlap((&__pyx_v_src), (&__pyx_v_dst), __pyx_v_ndim, __pyx_v_itemsize) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1261
 *     if slices_overlap(&src, &dst, ndim, itemsize):
 * 
 *         if not slice_is_contig(&src, order, ndim):             # <<<<<<<<<<<<<<
 *             order = get_best_order(&dst, ndim)
 * 
 */
    __pyx_t_2 = ((!(__pyx_memviewslice_is_contig((&__pyx_v_src), __pyx_v_order, __pyx_v_ndim) != 0)) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1262
 * 
 *         if not slice_is_contig(&src, order, ndim):
 *             order = get_best_order(&dst, ndim)             # <<<<<<<<<<<<<<
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)
 */
      __pyx_v_order = __pyx_get_best_slice_order((&__pyx_v_dst), __pyx_v_ndim);

      /* "View.MemoryView":1261
 *     if slices_overlap(&src, &dst, ndim, itemsize):
 * 
 *         if not slice_is_contig(&src, order, ndim):             # <<<<<<<<<<<<<<
 *             order = get_best_order(&dst, ndim)
 * 
 */
    }

    /* "View.MemoryView":1264
 *             order = get_best_order(&dst, ndim)
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)             # <<<<<<<<<<<<<<
 *         src = tmp
 * 
 */
    __pyx_t_6 = __pyx_memoryview_copy_data_to_temp((&__pyx_v_src), (&__pyx_v_tmp), __pyx_v_order, __pyx_v_ndim); if (unlikely(__pyx_t_6 == NULL)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1264; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_tmpdata = __pyx_t_6;

    /* "View.MemoryView":1265
 * 
 *         tmpdata = copy_data_to_temp(&src, &tmp, order, ndim)
 *         src = tmp             # <<<<<<<<<<<<<<
 * 
 *     if not broadcasting:
 */
    __pyx_v_src = __pyx_v_tmp;

    /* "View.MemoryView":1259
 *             _err_dim(ValueError, "Dimension %d is not direct", i)
 * 
 *     if slices_overlap(&src, &dst, ndim, itemsize):             # <<<<<<<<<<<<<<
 * 
 *         if not slice_is_contig(&src, order, ndim):
 */
  }

  /* "View.MemoryView":1267
 *         src = tmp
 * 
 *     if not broadcasting:             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = ((!(__pyx_v_broadcasting != 0)) != 0);
  if (__pyx_t_2) {

    /* "View.MemoryView":1270
 * 
 * 
 *         if slice_is_contig(&src, 'C', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(&dst, 'C', ndim)
 *         elif slice_is_contig(&src, 'F', ndim):
 */
    __pyx_t_2 = (__pyx_memviewslice_is_contig((&__pyx_v_src), 'C', __pyx_v_ndim) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1271
 * 
 *         if slice_is_contig(&src, 'C', ndim):
 *             direct_copy = slice_is_contig(&dst, 'C', ndim)             # <<<<<<<<<<<<<<
 *         elif slice_is_contig(&src, 'F', ndim):
 *             direct_copy = slice_is_contig(&dst, 'F', ndim)
 */
      __pyx_v_direct_copy = __pyx_memviewslice_is_contig((&__pyx_v_dst), 'C', __pyx_v_ndim);

      /* "View.MemoryView":1270
 * 
 * 
 *         if slice_is_contig(&src, 'C', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(&dst, 'C', ndim)
 *         elif slice_is_contig(&src, 'F', ndim):
 */
      goto __pyx_L12;
    }

    /* "View.MemoryView":1272
 *         if slice_is_contig(&src, 'C', ndim):
 *             direct_copy = slice_is_contig(&dst, 'C', ndim)
 *         elif slice_is_contig(&src, 'F', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(&dst, 'F', ndim)
 * 
 */
    __pyx_t_2 = (__pyx_memviewslice_is_contig((&__pyx_v_src), 'F', __pyx_v_ndim) != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1273
 *             direct_copy = slice_is_contig(&dst, 'C', ndim)
 *         elif slice_is_contig(&src, 'F', ndim):
 *             direct_copy = slice_is_contig(&dst, 'F', ndim)             # <<<<<<<<<<<<<<
 * 
 *         if direct_copy:
 */
      __pyx_v_direct_copy = __pyx_memviewslice_is_contig((&__pyx_v_dst), 'F', __pyx_v_ndim);

      /* "View.MemoryView":1272
 *         if slice_is_contig(&src, 'C', ndim):
 *             direct_copy = slice_is_contig(&dst, 'C', ndim)
 *         elif slice_is_contig(&src, 'F', ndim):             # <<<<<<<<<<<<<<
 *             direct_copy = slice_is_contig(&dst, 'F', ndim)
 * 
 */
    }
    __pyx_L12:;

    /* "View.MemoryView":1275
 *             direct_copy = slice_is_contig(&dst, 'F', ndim)
 * 
 *         if direct_copy:             # <<<<<<<<<<<<<<
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 */
    __pyx_t_2 = (__pyx_v_direct_copy != 0);
    if (__pyx_t_2) {

      /* "View.MemoryView":1277
 *         if direct_copy:
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)             # <<<<<<<<<<<<<<
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 */
      __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

      /* "View.MemoryView":1278
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))             # <<<<<<<<<<<<<<
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 *             free(tmpdata)
 */
      memcpy(__pyx_v_dst.data, __pyx_v_src.data, __pyx_memoryview_slice_get_size((&__pyx_v_src), __pyx_v_ndim));

      /* "View.MemoryView":1279
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, True)             # <<<<<<<<<<<<<<
 *             free(tmpdata)
 *             return 0
 */
      __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

      /* "View.MemoryView":1280
 *             memcpy(dst.data, src.data, slice_get_size(&src, ndim))
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 *             free(tmpdata)             # <<<<<<<<<<<<<<
 *             return 0
 * 
 */
      free(__pyx_v_tmpdata);

      /* "View.MemoryView":1281
 *             refcount_copying(&dst, dtype_is_object, ndim, True)
 *             free(tmpdata)
 *             return 0             # <<<<<<<<<<<<<<
 * 
 *     if order == 'F' == get_best_order(&dst, ndim):
 */
      __pyx_r = 0;
      goto __pyx_L0;

      /* "View.MemoryView":1275
 *             direct_copy = slice_is_contig(&dst, 'F', ndim)
 * 
 *         if direct_copy:             # <<<<<<<<<<<<<<
 * 
 *             refcount_copying(&dst, dtype_is_object, ndim, False)
 */
    }

    /* "View.MemoryView":1267
 *         src = tmp
 * 
 *     if not broadcasting:             # <<<<<<<<<<<<<<
 * 
 * 
 */
  }

  /* "View.MemoryView":1283
 *             return 0
 * 
 *     if order == 'F' == get_best_order(&dst, ndim):             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = (__pyx_v_order == 'F');
  if (__pyx_t_2) {
    __pyx_t_2 = ('F' == __pyx_get_best_slice_order((&__pyx_v_dst), __pyx_v_ndim));
  }
  __pyx_t_7 = (__pyx_t_2 != 0);
  if (__pyx_t_7) {

    /* "View.MemoryView":1286
 * 
 * 
 *         transpose_memslice(&src)             # <<<<<<<<<<<<<<
 *         transpose_memslice(&dst)
 * 
 */
    __pyx_t_5 = __pyx_memslice_transpose((&__pyx_v_src)); if (unlikely(__pyx_t_5 == 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1286; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "View.MemoryView":1287
 * 
 *         transpose_memslice(&src)
 *         transpose_memslice(&dst)             # <<<<<<<<<<<<<<
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, False)
 */
    __pyx_t_5 = __pyx_memslice_transpose((&__pyx_v_dst)); if (unlikely(__pyx_t_5 == 0)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 1287; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "View.MemoryView":1283
 *             return 0
 * 
 *     if order == 'F' == get_best_order(&dst, ndim):             # <<<<<<<<<<<<<<
 * 
 * 
 */
  }

  /* "View.MemoryView":1289
 *         transpose_memslice(&dst)
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, False)             # <<<<<<<<<<<<<<
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)
 *     refcount_copying(&dst, dtype_is_object, ndim, True)
 */
  __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

  /* "View.MemoryView":1290
 * 
 *     refcount_copying(&dst, dtype_is_object, ndim, False)
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)             # <<<<<<<<<<<<<<
 *     refcount_copying(&dst, dtype_is_object, ndim, True)
 * 
 */
  copy_strided_to_strided((&__pyx_v_src), (&__pyx_v_dst), __pyx_v_ndim, __pyx_v_itemsize);

  /* "View.MemoryView":1291
 *     refcount_copying(&dst, dtype_is_object, ndim, False)
 *     copy_strided_to_strided(&src, &dst, ndim, itemsize)
 *     refcount_copying(&dst, dtype_is_object, ndim, True)             # <<<<<<<<<<<<<<
 * 
 *     free(tmpdata)
 */
  __pyx_memoryview_refcount_copying((&__pyx_v_dst), __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

  /* "View.MemoryView":1293
 *     refcount_copying(&dst, dtype_is_object, ndim, True)
 * 
 *     free(tmpdata)             # <<<<<<<<<<<<<<
 *     return 0
 * 
 */
  free(__pyx_v_tmpdata);

  /* "View.MemoryView":1294
 * 
 *     free(tmpdata)
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_broadcast_leading')
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "View.MemoryView":1225
 * 
 * @cname('__pyx_memoryview_copy_contents')
 * cdef int memoryview_copy_contents(__Pyx_memviewslice src,             # <<<<<<<<<<<<<<
 *                                   __Pyx_memviewslice dst,
 *                                   int src_ndim, int dst_ndim,
 */

  /* function exit code */
  __pyx_L1_error:;
  {
    #ifdef WITH_THREAD
    PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
    #endif
    __Pyx_AddTraceback("View.MemoryView.memoryview_copy_contents", __pyx_clineno, __pyx_lineno, __pyx_filename);
    #ifdef WITH_THREAD
    PyGILState_Release(__pyx_gilstate_save);
    #endif
  }
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "View.MemoryView":1297
 * 
 * @cname('__pyx_memoryview_broadcast_leading')
 * cdef void broadcast_leading(__Pyx_memviewslice *mslice,             # <<<<<<<<<<<<<<
 *                             int ndim,
 *                             int ndim_other) nogil:
 */

static void __pyx_memoryview_broadcast_leading(__Pyx_memviewslice *__pyx_v_mslice, int __pyx_v_ndim, int __pyx_v_ndim_other) {
  int __pyx_v_i;
  int __pyx_v_offset;
  int __pyx_t_1;
  int __pyx_t_2;

  /* "View.MemoryView":1301
 *                             int ndim_other) nogil:
 *     cdef int i
 *     cdef int offset = ndim_other - ndim             # <<<<<<<<<<<<<<
 * 
 *     for i in range(ndim - 1, -1, -1):
 */
  __pyx_v_offset = (__pyx_v_ndim_other - __pyx_v_ndim);

  /* "View.MemoryView":1303
 *     cdef int offset = ndim_other - ndim
 * 
 *     for i in range(ndim - 1, -1, -1):             # <<<<<<<<<<<<<<
 *         mslice.shape[i + offset] = mslice.shape[i]
 *         mslice.strides[i + offset] = mslice.strides[i]
 */
  for (__pyx_t_1 = (__pyx_v_ndim - 1); __pyx_t_1 > -1L; __pyx_t_1-=1) {
    __pyx_v_i = __pyx_t_1;

    /* "View.MemoryView":1304
 * 
 *     for i in range(ndim - 1, -1, -1):
 *         mslice.shape[i + offset] = mslice.shape[i]             # <<<<<<<<<<<<<<
 *         mslice.strides[i + offset] = mslice.strides[i]
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]
 */
    (__pyx_v_mslice->shape[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_mslice->shape[__pyx_v_i]);

    /* "View.MemoryView":1305
 *     for i in range(ndim - 1, -1, -1):
 *         mslice.shape[i + offset] = mslice.shape[i]
 *         mslice.strides[i + offset] = mslice.strides[i]             # <<<<<<<<<<<<<<
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]
 * 
 */
    (__pyx_v_mslice->strides[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_mslice->strides[__pyx_v_i]);

    /* "View.MemoryView":1306
 *         mslice.shape[i + offset] = mslice.shape[i]
 *         mslice.strides[i + offset] = mslice.strides[i]
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]             # <<<<<<<<<<<<<<
 * 
 *     for i in range(offset):
 */
    (__pyx_v_mslice->suboffsets[(__pyx_v_i + __pyx_v_offset)]) = (__pyx_v_mslice->suboffsets[__pyx_v_i]);
  }

  /* "View.MemoryView":1308
 *         mslice.suboffsets[i + offset] = mslice.suboffsets[i]
 * 
 *     for i in range(offset):             # <<<<<<<<<<<<<<
 *         mslice.shape[i] = 1
 *         mslice.strides[i] = mslice.strides[0]
 */
  __pyx_t_1 = __pyx_v_offset;
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "View.MemoryView":1309
 * 
 *     for i in range(offset):
 *         mslice.shape[i] = 1             # <<<<<<<<<<<<<<
 *         mslice.strides[i] = mslice.strides[0]
 *         mslice.suboffsets[i] = -1
 */
    (__pyx_v_mslice->shape[__pyx_v_i]) = 1;

    /* "View.MemoryView":1310
 *     for i in range(offset):
 *         mslice.shape[i] = 1
 *         mslice.strides[i] = mslice.strides[0]             # <<<<<<<<<<<<<<
 *         mslice.suboffsets[i] = -1
 * 
 */
    (__pyx_v_mslice->strides[__pyx_v_i]) = (__pyx_v_mslice->strides[0]);

    /* "View.MemoryView":1311
 *         mslice.shape[i] = 1
 *         mslice.strides[i] = mslice.strides[0]
 *         mslice.suboffsets[i] = -1             # <<<<<<<<<<<<<<
 * 
 * 
 */
    (__pyx_v_mslice->suboffsets[__pyx_v_i]) = -1L;
  }

  /* "View.MemoryView":1297
 * 
 * @cname('__pyx_memoryview_broadcast_leading')
 * cdef void broadcast_leading(__Pyx_memviewslice *mslice,             # <<<<<<<<<<<<<<
 *                             int ndim,
 *                             int ndim_other) nogil:
 */

  /* function exit code */
}

/* "View.MemoryView":1319
 * 
 * @cname('__pyx_memoryview_refcount_copying')
 * cdef void refcount_copying(__Pyx_memviewslice *dst, bint dtype_is_object,             # <<<<<<<<<<<<<<
 *                            int ndim, bint inc) nogil:
 * 
 */

static void __pyx_memoryview_refcount_copying(__Pyx_memviewslice *__pyx_v_dst, int __pyx_v_dtype_is_object, int __pyx_v_ndim, int __pyx_v_inc) {
  int __pyx_t_1;

  /* "View.MemoryView":1323
 * 
 * 
 *     if dtype_is_object:             # <<<<<<<<<<<<<<
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape,
 *                                            dst.strides, ndim, inc)
 */
  __pyx_t_1 = (__pyx_v_dtype_is_object != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1324
 * 
 *     if dtype_is_object:
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape,             # <<<<<<<<<<<<<<
 *                                            dst.strides, ndim, inc)
 * 
 */
    __pyx_memoryview_refcount_objects_in_slice_with_gil(__pyx_v_dst->data, __pyx_v_dst->shape, __pyx_v_dst->strides, __pyx_v_ndim, __pyx_v_inc);

    /* "View.MemoryView":1323
 * 
 * 
 *     if dtype_is_object:             # <<<<<<<<<<<<<<
 *         refcount_objects_in_slice_with_gil(dst.data, dst.shape,
 *                                            dst.strides, ndim, inc)
 */
  }

  /* "View.MemoryView":1319
 * 
 * @cname('__pyx_memoryview_refcount_copying')
 * cdef void refcount_copying(__Pyx_memviewslice *dst, bint dtype_is_object,             # <<<<<<<<<<<<<<
 *                            int ndim, bint inc) nogil:
 * 
 */

  /* function exit code */
}

/* "View.MemoryView":1328
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice_with_gil')
 * cdef void refcount_objects_in_slice_with_gil(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                              Py_ssize_t *strides, int ndim,
 *                                              bint inc) with gil:
 */

static void __pyx_memoryview_refcount_objects_in_slice_with_gil(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, int __pyx_v_inc) {
  __Pyx_RefNannyDeclarations
  #ifdef WITH_THREAD
  PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();
  #endif
  __Pyx_RefNannySetupContext("refcount_objects_in_slice_with_gil", 0);

  /* "View.MemoryView":1331
 *                                              Py_ssize_t *strides, int ndim,
 *                                              bint inc) with gil:
 *     refcount_objects_in_slice(data, shape, strides, ndim, inc)             # <<<<<<<<<<<<<<
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')
 */
  __pyx_memoryview_refcount_objects_in_slice(__pyx_v_data, __pyx_v_shape, __pyx_v_strides, __pyx_v_ndim, __pyx_v_inc);

  /* "View.MemoryView":1328
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice_with_gil')
 * cdef void refcount_objects_in_slice_with_gil(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                              Py_ssize_t *strides, int ndim,
 *                                              bint inc) with gil:
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  #ifdef WITH_THREAD
  PyGILState_Release(__pyx_gilstate_save);
  #endif
}

/* "View.MemoryView":1334
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')
 * cdef void refcount_objects_in_slice(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                     Py_ssize_t *strides, int ndim, bint inc):
 *     cdef Py_ssize_t i
 */

static void __pyx_memoryview_refcount_objects_in_slice(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, int __pyx_v_inc) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("refcount_objects_in_slice", 0);

  /* "View.MemoryView":1338
 *     cdef Py_ssize_t i
 * 
 *     for i in range(shape[0]):             # <<<<<<<<<<<<<<
 *         if ndim == 1:
 *             if inc:
 */
  __pyx_t_1 = (__pyx_v_shape[0]);
  for (__pyx_t_2 = 0; __pyx_t_2 < __pyx_t_1; __pyx_t_2+=1) {
    __pyx_v_i = __pyx_t_2;

    /* "View.MemoryView":1339
 * 
 *     for i in range(shape[0]):
 *         if ndim == 1:             # <<<<<<<<<<<<<<
 *             if inc:
 *                 Py_INCREF((<PyObject **> data)[0])
 */
    __pyx_t_3 = ((__pyx_v_ndim == 1) != 0);
    if (__pyx_t_3) {

      /* "View.MemoryView":1340
 *     for i in range(shape[0]):
 *         if ndim == 1:
 *             if inc:             # <<<<<<<<<<<<<<
 *                 Py_INCREF((<PyObject **> data)[0])
 *             else:
 */
      __pyx_t_3 = (__pyx_v_inc != 0);
      if (__pyx_t_3) {

        /* "View.MemoryView":1341
 *         if ndim == 1:
 *             if inc:
 *                 Py_INCREF((<PyObject **> data)[0])             # <<<<<<<<<<<<<<
 *             else:
 *                 Py_DECREF((<PyObject **> data)[0])
 */
        Py_INCREF((((PyObject **)__pyx_v_data)[0]));

        /* "View.MemoryView":1340
 *     for i in range(shape[0]):
 *         if ndim == 1:
 *             if inc:             # <<<<<<<<<<<<<<
 *                 Py_INCREF((<PyObject **> data)[0])
 *             else:
 */
        goto __pyx_L6;
      }

      /* "View.MemoryView":1343
 *                 Py_INCREF((<PyObject **> data)[0])
 *             else:
 *                 Py_DECREF((<PyObject **> data)[0])             # <<<<<<<<<<<<<<
 *         else:
 *             refcount_objects_in_slice(data, shape + 1, strides + 1,
 */
      /*else*/ {
        Py_DECREF((((PyObject **)__pyx_v_data)[0]));
      }
      __pyx_L6:;

      /* "View.MemoryView":1339
 * 
 *     for i in range(shape[0]):
 *         if ndim == 1:             # <<<<<<<<<<<<<<
 *             if inc:
 *                 Py_INCREF((<PyObject **> data)[0])
 */
      goto __pyx_L5;
    }

    /* "View.MemoryView":1345
 *                 Py_DECREF((<PyObject **> data)[0])
 *         else:
 *             refcount_objects_in_slice(data, shape + 1, strides + 1,             # <<<<<<<<<<<<<<
 *                                       ndim - 1, inc)
 * 
 */
    /*else*/ {

      /* "View.MemoryView":1346
 *         else:
 *             refcount_objects_in_slice(data, shape + 1, strides + 1,
 *                                       ndim - 1, inc)             # <<<<<<<<<<<<<<
 * 
 *         data += strides[0]
 */
      __pyx_memoryview_refcount_objects_in_slice(__pyx_v_data, (__pyx_v_shape + 1), (__pyx_v_strides + 1), (__pyx_v_ndim - 1), __pyx_v_inc);
    }
    __pyx_L5:;

    /* "View.MemoryView":1348
 *                                       ndim - 1, inc)
 * 
 *         data += strides[0]             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_data = (__pyx_v_data + (__pyx_v_strides[0]));
  }

  /* "View.MemoryView":1334
 * 
 * @cname('__pyx_memoryview_refcount_objects_in_slice')
 * cdef void refcount_objects_in_slice(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                                     Py_ssize_t *strides, int ndim, bint inc):
 *     cdef Py_ssize_t i
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "View.MemoryView":1354
 * 
 * @cname('__pyx_memoryview_slice_assign_scalar')
 * cdef void slice_assign_scalar(__Pyx_memviewslice *dst, int ndim,             # <<<<<<<<<<<<<<
 *                               size_t itemsize, void *item,
 *                               bint dtype_is_object) nogil:
 */

static void __pyx_memoryview_slice_assign_scalar(__Pyx_memviewslice *__pyx_v_dst, int __pyx_v_ndim, size_t __pyx_v_itemsize, void *__pyx_v_item, int __pyx_v_dtype_is_object) {

  /* "View.MemoryView":1357
 *                               size_t itemsize, void *item,
 *                               bint dtype_is_object) nogil:
 *     refcount_copying(dst, dtype_is_object, ndim, False)             # <<<<<<<<<<<<<<
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim,
 *                          itemsize, item)
 */
  __pyx_memoryview_refcount_copying(__pyx_v_dst, __pyx_v_dtype_is_object, __pyx_v_ndim, 0);

  /* "View.MemoryView":1358
 *                               bint dtype_is_object) nogil:
 *     refcount_copying(dst, dtype_is_object, ndim, False)
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim,             # <<<<<<<<<<<<<<
 *                          itemsize, item)
 *     refcount_copying(dst, dtype_is_object, ndim, True)
 */
  __pyx_memoryview__slice_assign_scalar(__pyx_v_dst->data, __pyx_v_dst->shape, __pyx_v_dst->strides, __pyx_v_ndim, __pyx_v_itemsize, __pyx_v_item);

  /* "View.MemoryView":1360
 *     _slice_assign_scalar(dst.data, dst.shape, dst.strides, ndim,
 *                          itemsize, item)
 *     refcount_copying(dst, dtype_is_object, ndim, True)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_memoryview_refcount_copying(__pyx_v_dst, __pyx_v_dtype_is_object, __pyx_v_ndim, 1);

  /* "View.MemoryView":1354
 * 
 * @cname('__pyx_memoryview_slice_assign_scalar')
 * cdef void slice_assign_scalar(__Pyx_memviewslice *dst, int ndim,             # <<<<<<<<<<<<<<
 *                               size_t itemsize, void *item,
 *                               bint dtype_is_object) nogil:
 */

  /* function exit code */
}

/* "View.MemoryView":1364
 * 
 * @cname('__pyx_memoryview__slice_assign_scalar')
 * cdef void _slice_assign_scalar(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                               Py_ssize_t *strides, int ndim,
 *                               size_t itemsize, void *item) nogil:
 */

static void __pyx_memoryview__slice_assign_scalar(char *__pyx_v_data, Py_ssize_t *__pyx_v_shape, Py_ssize_t *__pyx_v_strides, int __pyx_v_ndim, size_t __pyx_v_itemsize, void *__pyx_v_item) {
  CYTHON_UNUSED Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_stride;
  Py_ssize_t __pyx_v_extent;
  int __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;

  /* "View.MemoryView":1368
 *                               size_t itemsize, void *item) nogil:
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t stride = strides[0]             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t extent = shape[0]
 * 
 */
  __pyx_v_stride = (__pyx_v_strides[0]);

  /* "View.MemoryView":1369
 *     cdef Py_ssize_t i
 *     cdef Py_ssize_t stride = strides[0]
 *     cdef Py_ssize_t extent = shape[0]             # <<<<<<<<<<<<<<
 * 
 *     if ndim == 1:
 */
  __pyx_v_extent = (__pyx_v_shape[0]);

  /* "View.MemoryView":1371
 *     cdef Py_ssize_t extent = shape[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)
 */
  __pyx_t_1 = ((__pyx_v_ndim == 1) != 0);
  if (__pyx_t_1) {

    /* "View.MemoryView":1372
 * 
 *     if ndim == 1:
 *         for i in range(extent):             # <<<<<<<<<<<<<<
 *             memcpy(data, item, itemsize)
 *             data += stride
 */
    __pyx_t_2 = __pyx_v_extent;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "View.MemoryView":1373
 *     if ndim == 1:
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)             # <<<<<<<<<<<<<<
 *             data += stride
 *     else:
 */
      memcpy(__pyx_v_data, __pyx_v_item, __pyx_v_itemsize);

      /* "View.MemoryView":1374
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)
 *             data += stride             # <<<<<<<<<<<<<<
 *     else:
 *         for i in range(extent):
 */
      __pyx_v_data = (__pyx_v_data + __pyx_v_stride);
    }

    /* "View.MemoryView":1371
 *     cdef Py_ssize_t extent = shape[0]
 * 
 *     if ndim == 1:             # <<<<<<<<<<<<<<
 *         for i in range(extent):
 *             memcpy(data, item, itemsize)
 */
    goto __pyx_L3;
  }

  /* "View.MemoryView":1376
 *             data += stride
 *     else:
 *         for i in range(extent):             # <<<<<<<<<<<<<<
 *             _slice_assign_scalar(data, shape + 1, strides + 1,
 *                                 ndim - 1, itemsize, item)
 */
  /*else*/ {
    __pyx_t_2 = __pyx_v_extent;
    for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
      __pyx_v_i = __pyx_t_3;

      /* "View.MemoryView":1377
 *     else:
 *         for i in range(extent):
 *             _slice_assign_scalar(data, shape + 1, strides + 1,             # <<<<<<<<<<<<<<
 *                                 ndim - 1, itemsize, item)
 *             data += stride
 */
      __pyx_memoryview__slice_assign_scalar(__pyx_v_data, (__pyx_v_shape + 1), (__pyx_v_strides + 1), (__pyx_v_ndim - 1), __pyx_v_itemsize, __pyx_v_item);

      /* "View.MemoryView":1379
 *             _slice_assign_scalar(data, shape + 1, strides + 1,
 *                                 ndim - 1, itemsize, item)
 *             data += stride             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __pyx_v_data = (__pyx_v_data + __pyx_v_stride);
    }
  }
  __pyx_L3:;

  /* "View.MemoryView":1364
 * 
 * @cname('__pyx_memoryview__slice_assign_scalar')
 * cdef void _slice_assign_scalar(char *data, Py_ssize_t *shape,             # <<<<<<<<<<<<<<
 *                               Py_ssize_t *strides, int ndim,
 *                               size_t itemsize, void *item) nogil:
 */

  /* function exit code */
}

static PyObject *__pyx_tp_new_array(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_array_obj *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_array_obj *)o);
  p->mode = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_format = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_array___cinit__(o, a, k) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_array(PyObject *o) {
  struct __pyx_array_obj *p = (struct __pyx_array_obj *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_array___dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->mode);
  Py_CLEAR(p->_format);
  (*Py_TYPE(o)->tp_free)(o);
}
static PyObject *__pyx_sq_item_array(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_array(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_array___setitem__(o, i, v);
  }
  else {
    PyErr_Format(PyExc_NotImplementedError,
      "Subscript deletion not supported by %.200s", Py_TYPE(o)->tp_name);
    return -1;
  }
}

static PyObject *__pyx_tp_getattro_array(PyObject *o, PyObject *n) {
  PyObject *v = PyObject_GenericGetAttr(o, n);
  if (!v && PyErr_ExceptionMatches(PyExc_AttributeError)) {
    PyErr_Clear();
    v = __pyx_array___getattr__(o, n);
  }
  return v;
}

static PyObject *__pyx_getprop___pyx_array_memview(PyObject *o, CYTHON_UNUSED void *x) {
  return get_memview(o);
}

static PyMethodDef __pyx_methods_array[] = {
  {"__getattr__", (PyCFunction)__pyx_array___getattr__, METH_O|METH_COEXIST, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_array[] = {
  {(char *)"memview", __pyx_getprop___pyx_array_memview, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_array = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_array, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_array = {
  0, /*mp_length*/
  __pyx_array___getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_array, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_array = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  __pyx_array_getbuffer, /*bf_getbuffer*/
  0, /*bf_releasebuffer*/
};

static PyTypeObject __pyx_type___pyx_array = {
  PyVarObject_HEAD_INIT(0, 0)
  "dipy.align.vector_fields.array", /*tp_name*/
  sizeof(struct __pyx_array_obj), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_array, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_array, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_array, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  __pyx_tp_getattro_array, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_array, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_array, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_array, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_array, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_Enum(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_MemviewEnum_obj *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_MemviewEnum_obj *)o);
  p->name = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_Enum(PyObject *o) {
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->name);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_Enum(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;
  if (p->name) {
    e = (*v)(p->name, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_Enum(PyObject *o) {
  PyObject* tmp;
  struct __pyx_MemviewEnum_obj *p = (struct __pyx_MemviewEnum_obj *)o;
  tmp = ((PyObject*)p->name);
  p->name = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_Enum[] = {
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type___pyx_MemviewEnum = {
  PyVarObject_HEAD_INIT(0, 0)
  "dipy.align.vector_fields.Enum", /*tp_name*/
  sizeof(struct __pyx_MemviewEnum_obj), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_Enum, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_MemviewEnum___repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_Enum, /*tp_traverse*/
  __pyx_tp_clear_Enum, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_Enum, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_MemviewEnum___init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_Enum, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_memoryview __pyx_vtable_memoryview;

static PyObject *__pyx_tp_new_memoryview(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_memoryview_obj *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_memoryview_obj *)o);
  p->__pyx_vtab = __pyx_vtabptr_memoryview;
  p->obj = Py_None; Py_INCREF(Py_None);
  p->_size = Py_None; Py_INCREF(Py_None);
  p->_array_interface = Py_None; Py_INCREF(Py_None);
  p->view.obj = NULL;
  if (unlikely(__pyx_memoryview___cinit__(o, a, k) < 0)) {
    Py_DECREF(o); o = 0;
  }
  return o;
}

static void __pyx_tp_dealloc_memoryview(PyObject *o) {
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_memoryview___dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->obj);
  Py_CLEAR(p->_size);
  Py_CLEAR(p->_array_interface);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_memoryview(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;
  if (p->obj) {
    e = (*v)(p->obj, a); if (e) return e;
  }
  if (p->_size) {
    e = (*v)(p->_size, a); if (e) return e;
  }
  if (p->_array_interface) {
    e = (*v)(p->_array_interface, a); if (e) return e;
  }
  if (p->view.obj) {
    e = (*v)(p->view.obj, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_memoryview(PyObject *o) {
  PyObject* tmp;
  struct __pyx_memoryview_obj *p = (struct __pyx_memoryview_obj *)o;
  tmp = ((PyObject*)p->obj);
  p->obj = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_size);
  p->_size = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_array_interface);
  p->_array_interface = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  Py_CLEAR(p->view.obj);
  return 0;
}
static PyObject *__pyx_sq_item_memoryview(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_memoryview(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_memoryview___setitem__(o, i, v);
  }
  else {
    PyErr_Format(PyExc_NotImplementedError,
      "Subscript deletion not supported by %.200s", Py_TYPE(o)->tp_name);
    return -1;
  }
}

static PyObject *__pyx_getprop___pyx_memoryview_T(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview_transpose(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_base(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview__get__base(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_shape(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview_get_shape(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_strides(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview_get_strides(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_suboffsets(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview_get_suboffsets(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_ndim(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview_get_ndim(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_itemsize(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview_get_itemsize(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_nbytes(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview_get_nbytes(o);
}

static PyObject *__pyx_getprop___pyx_memoryview_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryview_get_size(o);
}

static PyMethodDef __pyx_methods_memoryview[] = {
  {"is_c_contig", (PyCFunction)__pyx_memoryview_is_c_contig, METH_NOARGS, 0},
  {"is_f_contig", (PyCFunction)__pyx_memoryview_is_f_contig, METH_NOARGS, 0},
  {"copy", (PyCFunction)__pyx_memoryview_copy, METH_NOARGS, 0},
  {"copy_fortran", (PyCFunction)__pyx_memoryview_copy_fortran, METH_NOARGS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_memoryview[] = {
  {(char *)"T", __pyx_getprop___pyx_memoryview_T, 0, 0, 0},
  {(char *)"base", __pyx_getprop___pyx_memoryview_base, 0, 0, 0},
  {(char *)"shape", __pyx_getprop___pyx_memoryview_shape, 0, 0, 0},
  {(char *)"strides", __pyx_getprop___pyx_memoryview_strides, 0, 0, 0},
  {(char *)"suboffsets", __pyx_getprop___pyx_memoryview_suboffsets, 0, 0, 0},
  {(char *)"ndim", __pyx_getprop___pyx_memoryview_ndim, 0, 0, 0},
  {(char *)"itemsize", __pyx_getprop___pyx_memoryview_itemsize, 0, 0, 0},
  {(char *)"nbytes", __pyx_getprop___pyx_memoryview_nbytes, 0, 0, 0},
  {(char *)"size", __pyx_getprop___pyx_memoryview_size, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_memoryview = {
  __pyx_memoryview___len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_memoryview, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_memoryview = {
  __pyx_memoryview___len__, /*mp_length*/
  __pyx_memoryview___getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_memoryview, /*mp_ass_subscript*/
};

static PyBufferProcs __pyx_tp_as_buffer_memoryview = {
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getreadbuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getwritebuffer*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getsegcount*/
  #endif
  #if PY_MAJOR_VERSION < 3
  0, /*bf_getcharbuffer*/
  #endif
  __pyx_memoryview_getbuffer, /*bf_getbuffer*/
  0, /*bf_releasebuffer*/
};

static PyTypeObject __pyx_type___pyx_memoryview = {
  PyVarObject_HEAD_INIT(0, 0)
  "dipy.align.vector_fields.memoryview", /*tp_name*/
  sizeof(struct __pyx_memoryview_obj), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_memoryview, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_memoryview___repr__, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_memoryview, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_memoryview, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  __pyx_memoryview___str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  &__pyx_tp_as_buffer_memoryview, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_memoryview, /*tp_traverse*/
  __pyx_tp_clear_memoryview, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_memoryview, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_memoryview, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_memoryview, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct__memoryviewslice __pyx_vtable__memoryviewslice;

static PyObject *__pyx_tp_new__memoryviewslice(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_memoryviewslice_obj *p;
  PyObject *o = __pyx_tp_new_memoryview(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_memoryviewslice_obj *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_memoryview*)__pyx_vtabptr__memoryviewslice;
  p->from_object = Py_None; Py_INCREF(Py_None);
  p->from_slice.memview = NULL;
  return o;
}

static void __pyx_tp_dealloc__memoryviewslice(PyObject *o) {
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_memoryviewslice___dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->from_object);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_memoryview(o);
}

static int __pyx_tp_traverse__memoryviewslice(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;
  e = __pyx_tp_traverse_memoryview(o, v, a); if (e) return e;
  if (p->from_object) {
    e = (*v)(p->from_object, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear__memoryviewslice(PyObject *o) {
  PyObject* tmp;
  struct __pyx_memoryviewslice_obj *p = (struct __pyx_memoryviewslice_obj *)o;
  __pyx_tp_clear_memoryview(o);
  tmp = ((PyObject*)p->from_object);
  p->from_object = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  __PYX_XDEC_MEMVIEW(&p->from_slice, 1);
  return 0;
}

static PyObject *__pyx_getprop___pyx_memoryviewslice_base(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_memoryviewslice__get__base(o);
}

static PyMethodDef __pyx_methods__memoryviewslice[] = {
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets__memoryviewslice[] = {
  {(char *)"base", __pyx_getprop___pyx_memoryviewslice_base, 0, 0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type___pyx_memoryviewslice = {
  PyVarObject_HEAD_INIT(0, 0)
  "dipy.align.vector_fields._memoryviewslice", /*tp_name*/
  sizeof(struct __pyx_memoryviewslice_obj), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc__memoryviewslice, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_memoryview___repr__, /*tp_repr*/
  #else
  0, /*tp_repr*/
  #endif
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  #if CYTHON_COMPILING_IN_PYPY
  __pyx_memoryview___str__, /*tp_str*/
  #else
  0, /*tp_str*/
  #endif
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Internal class for passing memoryview slices to Python", /*tp_doc*/
  __pyx_tp_traverse__memoryviewslice, /*tp_traverse*/
  __pyx_tp_clear__memoryviewslice, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods__memoryviewslice, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets__memoryviewslice, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new__memoryviewslice, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "vector_fields",
    0, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_, __pyx_k_, sizeof(__pyx_k_), 0, 0, 1, 0},
  {&__pyx_n_s_ASCII, __pyx_k_ASCII, sizeof(__pyx_k_ASCII), 0, 0, 1, 1},
  {&__pyx_n_s_AttributeError, __pyx_k_AttributeError, sizeof(__pyx_k_AttributeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Buffer_view_does_not_expose_stri, __pyx_k_Buffer_view_does_not_expose_stri, sizeof(__pyx_k_Buffer_view_does_not_expose_stri), 0, 0, 1, 0},
  {&__pyx_kp_s_Can_only_create_a_buffer_that_is, __pyx_k_Can_only_create_a_buffer_that_is, sizeof(__pyx_k_Can_only_create_a_buffer_that_is), 0, 0, 1, 0},
  {&__pyx_kp_s_Cannot_index_with_type_s, __pyx_k_Cannot_index_with_type_s, sizeof(__pyx_k_Cannot_index_with_type_s), 0, 0, 1, 0},
  {&__pyx_n_s_Ellipsis, __pyx_k_Ellipsis, sizeof(__pyx_k_Ellipsis), 0, 0, 1, 1},
  {&__pyx_kp_s_Empty_shape_tuple_for_cython_arr, __pyx_k_Empty_shape_tuple_for_cython_arr, sizeof(__pyx_k_Empty_shape_tuple_for_cython_arr), 0, 0, 1, 0},
  {&__pyx_kp_s_Expected_at_least_d_arguments, __pyx_k_Expected_at_least_d_arguments, sizeof(__pyx_k_Expected_at_least_d_arguments), 0, 0, 1, 0},
  {&__pyx_kp_u_Format_string_allocated_too_shor, __pyx_k_Format_string_allocated_too_shor, sizeof(__pyx_k_Format_string_allocated_too_shor), 0, 1, 0, 0},
  {&__pyx_kp_u_Format_string_allocated_too_shor_2, __pyx_k_Format_string_allocated_too_shor_2, sizeof(__pyx_k_Format_string_allocated_too_shor_2), 0, 1, 0, 0},
  {&__pyx_kp_s_Function_call_with_ambiguous_arg, __pyx_k_Function_call_with_ambiguous_arg, sizeof(__pyx_k_Function_call_with_ambiguous_arg), 0, 0, 1, 0},
  {&__pyx_n_s_ImportError, __pyx_k_ImportError, sizeof(__pyx_k_ImportError), 0, 0, 1, 1},
  {&__pyx_n_s_IndexError, __pyx_k_IndexError, sizeof(__pyx_k_IndexError), 0, 0, 1, 1},
  {&__pyx_kp_s_Indirect_dimensions_not_supporte, __pyx_k_Indirect_dimensions_not_supporte, sizeof(__pyx_k_Indirect_dimensions_not_supporte), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_affine_transform_matrix, __pyx_k_Invalid_affine_transform_matrix, sizeof(__pyx_k_Invalid_affine_transform_matrix), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_displacement_multiplicat, __pyx_k_Invalid_displacement_multiplicat, sizeof(__pyx_k_Invalid_displacement_multiplicat), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_displacement_pre_multipl, __pyx_k_Invalid_displacement_pre_multipl, sizeof(__pyx_k_Invalid_displacement_pre_multipl), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_from_affine_transform_ma, __pyx_k_Invalid_from_affine_transform_ma, sizeof(__pyx_k_Invalid_from_affine_transform_ma), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_image_affine_transform, __pyx_k_Invalid_image_affine_transform, sizeof(__pyx_k_Invalid_image_affine_transform), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_index_multiplication_mat, __pyx_k_Invalid_index_multiplication_mat, sizeof(__pyx_k_Invalid_index_multiplication_mat), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_index_pre_multiplication, __pyx_k_Invalid_index_pre_multiplication, sizeof(__pyx_k_Invalid_index_pre_multiplication), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_inner_index_multiplicati, __pyx_k_Invalid_inner_index_multiplicati, sizeof(__pyx_k_Invalid_inner_index_multiplicati), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_mode_expected_c_or_fortr, __pyx_k_Invalid_mode_expected_c_or_fortr, sizeof(__pyx_k_Invalid_mode_expected_c_or_fortr), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_outer_index_multiplicati, __pyx_k_Invalid_outer_index_multiplicati, sizeof(__pyx_k_Invalid_outer_index_multiplicati), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_sampling_grid_affine_tra, __pyx_k_Invalid_sampling_grid_affine_tra, sizeof(__pyx_k_Invalid_sampling_grid_affine_tra), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_shape_in_axis_d_d, __pyx_k_Invalid_shape_in_axis_d_d, sizeof(__pyx_k_Invalid_shape_in_axis_d_d), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_spacings, __pyx_k_Invalid_spacings, sizeof(__pyx_k_Invalid_spacings), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_to_affine_transform_matr, __pyx_k_Invalid_to_affine_transform_matr, sizeof(__pyx_k_Invalid_to_affine_transform_matr), 0, 0, 1, 0},
  {&__pyx_kp_s_Invalid_world_to_image_transform, __pyx_k_Invalid_world_to_image_transform, sizeof(__pyx_k_Invalid_world_to_image_transform), 0, 0, 1, 0},
  {&__pyx_n_s_M, __pyx_k_M, sizeof(__pyx_k_M), 0, 0, 1, 1},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_kp_s_MemoryView_of_r_at_0x_x, __pyx_k_MemoryView_of_r_at_0x_x, sizeof(__pyx_k_MemoryView_of_r_at_0x_x), 0, 0, 1, 0},
  {&__pyx_kp_s_MemoryView_of_r_object, __pyx_k_MemoryView_of_r_object, sizeof(__pyx_k_MemoryView_of_r_object), 0, 0, 1, 0},
  {&__pyx_kp_s_No_matching_signature_found, __pyx_k_No_matching_signature_found, sizeof(__pyx_k_No_matching_signature_found), 0, 0, 1, 0},
  {&__pyx_kp_u_Non_native_byte_order_not_suppor, __pyx_k_Non_native_byte_order_not_suppor, sizeof(__pyx_k_Non_native_byte_order_not_suppor), 0, 1, 0, 0},
  {&__pyx_n_b_O, __pyx_k_O, sizeof(__pyx_k_O), 0, 0, 0, 1},
  {&__pyx_kp_s_Out_of_bounds_on_buffer_access_a, __pyx_k_Out_of_bounds_on_buffer_access_a, sizeof(__pyx_k_Out_of_bounds_on_buffer_access_a), 0, 0, 1, 0},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Unable_to_convert_item_to_object, __pyx_k_Unable_to_convert_item_to_object, sizeof(__pyx_k_Unable_to_convert_item_to_object), 0, 0, 1, 0},
  {&__pyx_kp_s_Undefined_gradient_for_image_dim, __pyx_k_Undefined_gradient_for_image_dim, sizeof(__pyx_k_Undefined_gradient_for_image_dim), 0, 0, 1, 0},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_kp_s__3, __pyx_k__3, sizeof(__pyx_k__3), 0, 0, 1, 0},
  {&__pyx_n_s_affine, __pyx_k_affine, sizeof(__pyx_k_affine), 0, 0, 1, 1},
  {&__pyx_n_s_affine_disp, __pyx_k_affine_disp, sizeof(__pyx_k_affine_disp), 0, 0, 1, 1},
  {&__pyx_n_s_affine_idx_in, __pyx_k_affine_idx_in, sizeof(__pyx_k_affine_idx_in), 0, 0, 1, 1},
  {&__pyx_n_s_affine_idx_out, __pyx_k_affine_idx_out, sizeof(__pyx_k_affine_idx_out), 0, 0, 1, 1},
  {&__pyx_n_s_all, __pyx_k_all, sizeof(__pyx_k_all), 0, 0, 1, 1},
  {&__pyx_n_s_allocate_buffer, __pyx_k_allocate_buffer, sizeof(__pyx_k_allocate_buffer), 0, 0, 1, 1},
  {&__pyx_n_s_alpha, __pyx_k_alpha, sizeof(__pyx_k_alpha), 0, 0, 1, 1},
  {&__pyx_n_s_arctan2, __pyx_k_arctan2, sizeof(__pyx_k_arctan2), 0, 0, 1, 1},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_asarray, __pyx_k_asarray, sizeof(__pyx_k_asarray), 0, 0, 1, 1},
  {&__pyx_n_s_astype, __pyx_k_astype, sizeof(__pyx_k_astype), 0, 0, 1, 1},
  {&__pyx_n_s_b, __pyx_k_b, sizeof(__pyx_k_b), 0, 0, 1, 1},
  {&__pyx_n_s_base, __pyx_k_base, sizeof(__pyx_k_base), 0, 0, 1, 1},
  {&__pyx_n_s_beta, __pyx_k_beta, sizeof(__pyx_k_beta), 0, 0, 1, 1},
  {&__pyx_n_s_c, __pyx_k_c, sizeof(__pyx_k_c), 0, 0, 1, 1},
  {&__pyx_n_u_c, __pyx_k_c, sizeof(__pyx_k_c), 0, 1, 0, 1},
  {&__pyx_n_s_calpha, __pyx_k_calpha, sizeof(__pyx_k_calpha), 0, 0, 1, 1},
  {&__pyx_n_s_cbeta, __pyx_k_cbeta, sizeof(__pyx_k_cbeta), 0, 0, 1, 1},
  {&__pyx_n_s_cgamma, __pyx_k_cgamma, sizeof(__pyx_k_cgamma), 0, 0, 1, 1},
  {&__pyx_n_s_class, __pyx_k_class, sizeof(__pyx_k_class), 0, 0, 1, 1},
  {&__pyx_n_s_cnt, __pyx_k_cnt, sizeof(__pyx_k_cnt), 0, 0, 1, 1},
  {&__pyx_n_s_comp, __pyx_k_comp, sizeof(__pyx_k_comp), 0, 0, 1, 1},
  {&__pyx_n_s_compose_vector_fields_2d, __pyx_k_compose_vector_fields_2d, sizeof(__pyx_k_compose_vector_fields_2d), 0, 0, 1, 1},
  {&__pyx_n_s_compose_vector_fields_3d, __pyx_k_compose_vector_fields_3d, sizeof(__pyx_k_compose_vector_fields_3d), 0, 0, 1, 1},
  {&__pyx_kp_s_contiguous_and_direct, __pyx_k_contiguous_and_direct, sizeof(__pyx_k_contiguous_and_direct), 0, 0, 1, 0},
  {&__pyx_kp_s_contiguous_and_indirect, __pyx_k_contiguous_and_indirect, sizeof(__pyx_k_contiguous_and_indirect), 0, 0, 1, 0},
  {&__pyx_n_s_cos, __pyx_k_cos, sizeof(__pyx_k_cos), 0, 0, 1, 1},
  {&__pyx_n_s_create_circle, __pyx_k_create_circle, sizeof(__pyx_k_create_circle), 0, 0, 1, 1},
  {&__pyx_n_s_create_harmonic_fields_2d, __pyx_k_create_harmonic_fields_2d, sizeof(__pyx_k_create_harmonic_fields_2d), 0, 0, 1, 1},
  {&__pyx_n_s_create_harmonic_fields_3d, __pyx_k_create_harmonic_fields_3d, sizeof(__pyx_k_create_harmonic_fields_3d), 0, 0, 1, 1},
  {&__pyx_n_s_create_random_displacement_2d, __pyx_k_create_random_displacement_2d, sizeof(__pyx_k_create_random_displacement_2d), 0, 0, 1, 1},
  {&__pyx_n_s_create_random_displacement_3d, __pyx_k_create_random_displacement_3d, sizeof(__pyx_k_create_random_displacement_3d), 0, 0, 1, 1},
  {&__pyx_n_s_create_sphere, __pyx_k_create_sphere, sizeof(__pyx_k_create_sphere), 0, 0, 1, 1},
  {&__pyx_n_s_current, __pyx_k_current, sizeof(__pyx_k_current), 0, 0, 1, 1},
  {&__pyx_n_s_d, __pyx_k_d, sizeof(__pyx_k_d), 0, 0, 1, 1},
  {&__pyx_n_s_d1, __pyx_k_d1, sizeof(__pyx_k_d1), 0, 0, 1, 1},
  {&__pyx_n_s_d2, __pyx_k_d2, sizeof(__pyx_k_d2), 0, 0, 1, 1},
  {&__pyx_n_s_d_world2grid, __pyx_k_d_world2grid, sizeof(__pyx_k_d_world2grid), 0, 0, 1, 1},
  {&__pyx_n_s_defaults, __pyx_k_defaults, sizeof(__pyx_k_defaults), 0, 0, 1, 1},
  {&__pyx_n_s_di, __pyx_k_di, sizeof(__pyx_k_di), 0, 0, 1, 1},
  {&__pyx_n_s_difmag, __pyx_k_difmag, sizeof(__pyx_k_difmag), 0, 0, 1, 1},
  {&__pyx_n_s_dii, __pyx_k_dii, sizeof(__pyx_k_dii), 0, 0, 1, 1},
  {&__pyx_n_s_dim, __pyx_k_dim, sizeof(__pyx_k_dim), 0, 0, 1, 1},
  {&__pyx_n_s_dipy_align_vector_fields, __pyx_k_dipy_align_vector_fields, sizeof(__pyx_k_dipy_align_vector_fields), 0, 0, 1, 1},
  {&__pyx_n_s_dj, __pyx_k_dj, sizeof(__pyx_k_dj), 0, 0, 1, 1},
  {&__pyx_n_s_djj, __pyx_k_djj, sizeof(__pyx_k_djj), 0, 0, 1, 1},
  {&__pyx_n_s_dk, __pyx_k_dk, sizeof(__pyx_k_dk), 0, 0, 1, 1},
  {&__pyx_n_s_dkk, __pyx_k_dkk, sizeof(__pyx_k_dkk), 0, 0, 1, 1},
  {&__pyx_n_s_dom_size, __pyx_k_dom_size, sizeof(__pyx_k_dom_size), 0, 0, 1, 1},
  {&__pyx_n_s_double, __pyx_k_double, sizeof(__pyx_k_double), 0, 0, 1, 1},
  {&__pyx_kp_s_double_complex, __pyx_k_double_complex, sizeof(__pyx_k_double_complex), 0, 0, 1, 0},
  {&__pyx_kp_s_double_complex_double, __pyx_k_double_complex_double, sizeof(__pyx_k_double_complex_double), 0, 0, 1, 0},
  {&__pyx_kp_s_double_complex_float, __pyx_k_double_complex_float, sizeof(__pyx_k_double_complex_float), 0, 0, 1, 0},
  {&__pyx_kp_s_double_double, __pyx_k_double_double, sizeof(__pyx_k_double_double), 0, 0, 1, 0},
  {&__pyx_kp_s_double_float, __pyx_k_double_float, sizeof(__pyx_k_double_float), 0, 0, 1, 0},
  {&__pyx_n_s_down, __pyx_k_down, sizeof(__pyx_k_down), 0, 0, 1, 1},
  {&__pyx_n_s_downsample_displacement_field_2d, __pyx_k_downsample_displacement_field_2d, sizeof(__pyx_k_downsample_displacement_field_2d), 0, 0, 1, 1},
  {&__pyx_n_s_downsample_displacement_field_3d, __pyx_k_downsample_displacement_field_3d, sizeof(__pyx_k_downsample_displacement_field_3d), 0, 0, 1, 1},
  {&__pyx_n_s_downsample_scalar_field_2d, __pyx_k_downsample_scalar_field_2d, sizeof(__pyx_k_downsample_scalar_field_2d), 0, 0, 1, 1},
  {&__pyx_n_s_downsample_scalar_field_3d, __pyx_k_downsample_scalar_field_3d, sizeof(__pyx_k_downsample_scalar_field_3d), 0, 0, 1, 1},
  {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
  {&__pyx_n_s_dtype_is_object, __pyx_k_dtype_is_object, sizeof(__pyx_k_dtype_is_object), 0, 0, 1, 1},
  {&__pyx_n_s_dx, __pyx_k_dx, sizeof(__pyx_k_dx), 0, 0, 1, 1},
  {&__pyx_n_s_empty, __pyx_k_empty, sizeof(__pyx_k_empty), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_s_enumerate, __pyx_k_enumerate, sizeof(__pyx_k_enumerate), 0, 0, 1, 1},
  {&__pyx_n_s_epsilon, __pyx_k_epsilon, sizeof(__pyx_k_epsilon), 0, 0, 1, 1},
  {&__pyx_n_s_error, __pyx_k_error, sizeof(__pyx_k_error), 0, 0, 1, 1},
  {&__pyx_n_s_expanded, __pyx_k_expanded, sizeof(__pyx_k_expanded), 0, 0, 1, 1},
  {&__pyx_n_s_factors, __pyx_k_factors, sizeof(__pyx_k_factors), 0, 0, 1, 1},
  {&__pyx_n_s_field, __pyx_k_field, sizeof(__pyx_k_field), 0, 0, 1, 1},
  {&__pyx_n_s_flag, __pyx_k_flag, sizeof(__pyx_k_flag), 0, 0, 1, 1},
  {&__pyx_n_s_flags, __pyx_k_flags, sizeof(__pyx_k_flags), 0, 0, 1, 1},
  {&__pyx_n_s_float, __pyx_k_float, sizeof(__pyx_k_float), 0, 0, 1, 1},
  {&__pyx_n_s_float64, __pyx_k_float64, sizeof(__pyx_k_float64), 0, 0, 1, 1},
  {&__pyx_kp_s_float_complex, __pyx_k_float_complex, sizeof(__pyx_k_float_complex), 0, 0, 1, 0},
  {&__pyx_kp_s_float_complex_double, __pyx_k_float_complex_double, sizeof(__pyx_k_float_complex_double), 0, 0, 1, 0},
  {&__pyx_kp_s_float_complex_float, __pyx_k_float_complex_float, sizeof(__pyx_k_float_complex_float), 0, 0, 1, 0},
  {&__pyx_kp_s_float_double, __pyx_k_float_double, sizeof(__pyx_k_float_double), 0, 0, 1, 0},
  {&__pyx_kp_s_float_float, __pyx_k_float_float, sizeof(__pyx_k_float_float), 0, 0, 1, 0},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_fortran, __pyx_k_fortran, sizeof(__pyx_k_fortran), 0, 0, 1, 1},
  {&__pyx_n_u_fortran, __pyx_k_fortran, sizeof(__pyx_k_fortran), 0, 1, 0, 1},
  {&__pyx_n_s_from_grid2world, __pyx_k_from_grid2world, sizeof(__pyx_k_from_grid2world), 0, 0, 1, 1},
  {&__pyx_n_s_from_shape, __pyx_k_from_shape, sizeof(__pyx_k_from_shape), 0, 0, 1, 1},
  {&__pyx_n_s_ftype, __pyx_k_ftype, sizeof(__pyx_k_ftype), 0, 0, 1, 1},
  {&__pyx_n_s_gamma, __pyx_k_gamma, sizeof(__pyx_k_gamma), 0, 0, 1, 1},
  {&__pyx_kp_s_got_differing_extents_in_dimensi, __pyx_k_got_differing_extents_in_dimensi, sizeof(__pyx_k_got_differing_extents_in_dimensi), 0, 0, 1, 0},
  {&__pyx_n_s_gradient, __pyx_k_gradient, sizeof(__pyx_k_gradient), 0, 0, 1, 1},
  {&__pyx_n_s_gradient_2d, __pyx_k_gradient_2d, sizeof(__pyx_k_gradient_2d), 0, 0, 1, 1},
  {&__pyx_n_s_gradient_3d, __pyx_k_gradient_3d, sizeof(__pyx_k_gradient_3d), 0, 0, 1, 1},
  {&__pyx_n_s_h, __pyx_k_h, sizeof(__pyx_k_h), 0, 0, 1, 1},
  {&__pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_k_home_yoh_deb_gits_pkg_exppsy_di, sizeof(__pyx_k_home_yoh_deb_gits_pkg_exppsy_di), 0, 0, 1, 0},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_s_id, __pyx_k_id, sizeof(__pyx_k_id), 0, 0, 1, 1},
  {&__pyx_n_s_ii, __pyx_k_ii, sizeof(__pyx_k_ii), 0, 0, 1, 1},
  {&__pyx_n_s_image, __pyx_k_image, sizeof(__pyx_k_image), 0, 0, 1, 1},
  {&__pyx_n_s_img, __pyx_k_img, sizeof(__pyx_k_img), 0, 0, 1, 1},
  {&__pyx_n_s_img_spacing, __pyx_k_img_spacing, sizeof(__pyx_k_img_spacing), 0, 0, 1, 1},
  {&__pyx_n_s_img_world2grid, __pyx_k_img_world2grid, sizeof(__pyx_k_img_world2grid), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_in_flag, __pyx_k_in_flag, sizeof(__pyx_k_in_flag), 0, 0, 1, 1},
  {&__pyx_n_s_inside, __pyx_k_inside, sizeof(__pyx_k_inside), 0, 0, 1, 1},
  {&__pyx_n_s_int, __pyx_k_int, sizeof(__pyx_k_int), 0, 0, 1, 1},
  {&__pyx_n_s_int32, __pyx_k_int32, sizeof(__pyx_k_int32), 0, 0, 1, 1},
  {&__pyx_kp_s_int_double, __pyx_k_int_double, sizeof(__pyx_k_int_double), 0, 0, 1, 0},
  {&__pyx_n_s_int_field, __pyx_k_int_field, sizeof(__pyx_k_int_field), 0, 0, 1, 1},
  {&__pyx_kp_s_int_float, __pyx_k_int_float, sizeof(__pyx_k_int_float), 0, 0, 1, 0},
  {&__pyx_n_s_interpolate_scalar_2d, __pyx_k_interpolate_scalar_2d, sizeof(__pyx_k_interpolate_scalar_2d), 0, 0, 1, 1},
  {&__pyx_n_s_interpolate_scalar_3d, __pyx_k_interpolate_scalar_3d, sizeof(__pyx_k_interpolate_scalar_3d), 0, 0, 1, 1},
  {&__pyx_n_s_interpolate_scalar_nn_2d, __pyx_k_interpolate_scalar_nn_2d, sizeof(__pyx_k_interpolate_scalar_nn_2d), 0, 0, 1, 1},
  {&__pyx_n_s_interpolate_scalar_nn_3d, __pyx_k_interpolate_scalar_nn_3d, sizeof(__pyx_k_interpolate_scalar_nn_3d), 0, 0, 1, 1},
  {&__pyx_n_s_interpolate_vector_2d, __pyx_k_interpolate_vector_2d, sizeof(__pyx_k_interpolate_vector_2d), 0, 0, 1, 1},
  {&__pyx_n_s_interpolate_vector_3d, __pyx_k_interpolate_vector_3d, sizeof(__pyx_k_interpolate_vector_3d), 0, 0, 1, 1},
  {&__pyx_n_s_inv, __pyx_k_inv, sizeof(__pyx_k_inv), 0, 0, 1, 1},
  {&__pyx_n_s_invert_vector_field_fixed_point, __pyx_k_invert_vector_field_fixed_point, sizeof(__pyx_k_invert_vector_field_fixed_point), 0, 0, 1, 1},
  {&__pyx_n_s_invert_vector_field_fixed_point_2, __pyx_k_invert_vector_field_fixed_point_2, sizeof(__pyx_k_invert_vector_field_fixed_point_2), 0, 0, 1, 1},
  {&__pyx_n_s_is_valid_affine, __pyx_k_is_valid_affine, sizeof(__pyx_k_is_valid_affine), 0, 0, 1, 1},
  {&__pyx_n_s_isfinite, __pyx_k_isfinite, sizeof(__pyx_k_isfinite), 0, 0, 1, 1},
  {&__pyx_n_s_itemsize, __pyx_k_itemsize, sizeof(__pyx_k_itemsize), 0, 0, 1, 1},
  {&__pyx_kp_s_itemsize_0_for_cython_array, __pyx_k_itemsize_0_for_cython_array, sizeof(__pyx_k_itemsize_0_for_cython_array), 0, 0, 1, 0},
  {&__pyx_n_s_iter_count, __pyx_k_iter_count, sizeof(__pyx_k_iter_count), 0, 0, 1, 1},
  {&__pyx_n_s_j, __pyx_k_j, sizeof(__pyx_k_j), 0, 0, 1, 1},
  {&__pyx_n_s_jd_grad, __pyx_k_jd_grad, sizeof(__pyx_k_jd_grad), 0, 0, 1, 1},
  {&__pyx_n_s_jj, __pyx_k_jj, sizeof(__pyx_k_jj), 0, 0, 1, 1},
  {&__pyx_n_s_k, __pyx_k_k, sizeof(__pyx_k_k), 0, 0, 1, 1},
  {&__pyx_n_s_kind, __pyx_k_kind, sizeof(__pyx_k_kind), 0, 0, 1, 1},
  {&__pyx_n_s_kk, __pyx_k_kk, sizeof(__pyx_k_kk), 0, 0, 1, 1},
  {&__pyx_n_s_kwargs, __pyx_k_kwargs, sizeof(__pyx_k_kwargs), 0, 0, 1, 1},
  {&__pyx_n_s_locations, __pyx_k_locations, sizeof(__pyx_k_locations), 0, 0, 1, 1},
  {&__pyx_n_s_long, __pyx_k_long, sizeof(__pyx_k_long), 0, 0, 1, 1},
  {&__pyx_kp_s_long_double, __pyx_k_long_double, sizeof(__pyx_k_long_double), 0, 0, 1, 0},
  {&__pyx_kp_s_long_float, __pyx_k_long_float, sizeof(__pyx_k_long_float), 0, 0, 1, 0},
  {&__pyx_n_s_m, __pyx_k_m, sizeof(__pyx_k_m), 0, 0, 1, 1},
  {&__pyx_n_s_mag, __pyx_k_mag, sizeof(__pyx_k_mag), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_max_iter, __pyx_k_max_iter, sizeof(__pyx_k_max_iter), 0, 0, 1, 1},
  {&__pyx_n_s_maxlen, __pyx_k_maxlen, sizeof(__pyx_k_maxlen), 0, 0, 1, 1},
  {&__pyx_n_s_memview, __pyx_k_memview, sizeof(__pyx_k_memview), 0, 0, 1, 1},
  {&__pyx_n_s_mid_col, __pyx_k_mid_col, sizeof(__pyx_k_mid_col), 0, 0, 1, 1},
  {&__pyx_n_s_mid_row, __pyx_k_mid_row, sizeof(__pyx_k_mid_row), 0, 0, 1, 1},
  {&__pyx_n_s_mid_slice, __pyx_k_mid_slice, sizeof(__pyx_k_mid_slice), 0, 0, 1, 1},
  {&__pyx_n_s_mode, __pyx_k_mode, sizeof(__pyx_k_mode), 0, 0, 1, 1},
  {&__pyx_n_s_n, __pyx_k_n, sizeof(__pyx_k_n), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_nc, __pyx_k_nc, sizeof(__pyx_k_nc), 0, 0, 1, 1},
  {&__pyx_n_s_ncVol, __pyx_k_ncVol, sizeof(__pyx_k_ncVol), 0, 0, 1, 1},
  {&__pyx_n_s_ncols, __pyx_k_ncols, sizeof(__pyx_k_ncols), 0, 0, 1, 1},
  {&__pyx_n_s_ndarray, __pyx_k_ndarray, sizeof(__pyx_k_ndarray), 0, 0, 1, 1},
  {&__pyx_kp_u_ndarray_is_not_C_contiguous, __pyx_k_ndarray_is_not_C_contiguous, sizeof(__pyx_k_ndarray_is_not_C_contiguous), 0, 1, 0, 0},
  {&__pyx_kp_u_ndarray_is_not_Fortran_contiguou, __pyx_k_ndarray_is_not_Fortran_contiguou, sizeof(__pyx_k_ndarray_is_not_Fortran_contiguou), 0, 1, 0, 0},
  {&__pyx_n_s_ndim, __pyx_k_ndim, sizeof(__pyx_k_ndim), 0, 0, 1, 1},
  {&__pyx_n_s_nnc, __pyx_k_nnc, sizeof(__pyx_k_nnc), 0, 0, 1, 1},
  {&__pyx_n_s_nnr, __pyx_k_nnr, sizeof(__pyx_k_nnr), 0, 0, 1, 1},
  {&__pyx_n_s_nns, __pyx_k_nns, sizeof(__pyx_k_nns), 0, 0, 1, 1},
  {&__pyx_n_s_norms, __pyx_k_norms, sizeof(__pyx_k_norms), 0, 0, 1, 1},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_nr, __pyx_k_nr, sizeof(__pyx_k_nr), 0, 0, 1, 1},
  {&__pyx_n_s_nrVol, __pyx_k_nrVol, sizeof(__pyx_k_nrVol), 0, 0, 1, 1},
  {&__pyx_n_s_nrows, __pyx_k_nrows, sizeof(__pyx_k_nrows), 0, 0, 1, 1},
  {&__pyx_n_s_ns, __pyx_k_ns, sizeof(__pyx_k_ns), 0, 0, 1, 1},
  {&__pyx_n_s_nsVol, __pyx_k_nsVol, sizeof(__pyx_k_nsVol), 0, 0, 1, 1},
  {&__pyx_n_s_nslices, __pyx_k_nslices, sizeof(__pyx_k_nslices), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_n_s_obj, __pyx_k_obj, sizeof(__pyx_k_obj), 0, 0, 1, 1},
  {&__pyx_n_s_out, __pyx_k_out, sizeof(__pyx_k_out), 0, 0, 1, 1},
  {&__pyx_n_s_out_grid2world, __pyx_k_out_grid2world, sizeof(__pyx_k_out_grid2world), 0, 0, 1, 1},
  {&__pyx_n_s_out_shape, __pyx_k_out_shape, sizeof(__pyx_k_out_shape), 0, 0, 1, 1},
  {&__pyx_n_s_output, __pyx_k_output, sizeof(__pyx_k_output), 0, 0, 1, 1},
  {&__pyx_n_s_p, __pyx_k_p, sizeof(__pyx_k_p), 0, 0, 1, 1},
  {&__pyx_n_s_pack, __pyx_k_pack, sizeof(__pyx_k_pack), 0, 0, 1, 1},
  {&__pyx_n_s_premult_disp, __pyx_k_premult_disp, sizeof(__pyx_k_premult_disp), 0, 0, 1, 1},
  {&__pyx_n_s_premult_index, __pyx_k_premult_index, sizeof(__pyx_k_premult_index), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_getbuffer, __pyx_k_pyx_getbuffer, sizeof(__pyx_k_pyx_getbuffer), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_q, __pyx_k_q, sizeof(__pyx_k_q), 0, 0, 1, 1},
  {&__pyx_n_s_r, __pyx_k_r, sizeof(__pyx_k_r), 0, 0, 1, 1},
  {&__pyx_n_s_radius, __pyx_k_radius, sizeof(__pyx_k_radius), 0, 0, 1, 1},
  {&__pyx_n_s_randint, __pyx_k_randint, sizeof(__pyx_k_randint), 0, 0, 1, 1},
  {&__pyx_n_s_random, __pyx_k_random, sizeof(__pyx_k_random), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_ref_shape, __pyx_k_ref_shape, sizeof(__pyx_k_ref_shape), 0, 0, 1, 1},
  {&__pyx_n_s_reorient_vector_field_2d, __pyx_k_reorient_vector_field_2d, sizeof(__pyx_k_reorient_vector_field_2d), 0, 0, 1, 1},
  {&__pyx_n_s_reorient_vector_field_3d, __pyx_k_reorient_vector_field_3d, sizeof(__pyx_k_reorient_vector_field_3d), 0, 0, 1, 1},
  {&__pyx_n_s_resample_displacement_field_2d, __pyx_k_resample_displacement_field_2d, sizeof(__pyx_k_resample_displacement_field_2d), 0, 0, 1, 1},
  {&__pyx_n_s_resample_displacement_field_3d, __pyx_k_resample_displacement_field_3d, sizeof(__pyx_k_resample_displacement_field_3d), 0, 0, 1, 1},
  {&__pyx_n_s_ri, __pyx_k_ri, sizeof(__pyx_k_ri), 0, 0, 1, 1},
  {&__pyx_n_s_rj, __pyx_k_rj, sizeof(__pyx_k_rj), 0, 0, 1, 1},
  {&__pyx_n_s_rk, __pyx_k_rk, sizeof(__pyx_k_rk), 0, 0, 1, 1},
  {&__pyx_n_s_s, __pyx_k_s, sizeof(__pyx_k_s), 0, 0, 1, 1},
  {&__pyx_n_s_sample_points, __pyx_k_sample_points, sizeof(__pyx_k_sample_points), 0, 0, 1, 1},
  {&__pyx_n_s_sc, __pyx_k_sc, sizeof(__pyx_k_sc), 0, 0, 1, 1},
  {&__pyx_n_s_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 0, 1, 1},
  {&__pyx_n_s_short, __pyx_k_short, sizeof(__pyx_k_short), 0, 0, 1, 1},
  {&__pyx_kp_s_short_double, __pyx_k_short_double, sizeof(__pyx_k_short_double), 0, 0, 1, 0},
  {&__pyx_kp_s_short_float, __pyx_k_short_float, sizeof(__pyx_k_short_float), 0, 0, 1, 0},
  {&__pyx_n_s_signatures, __pyx_k_signatures, sizeof(__pyx_k_signatures), 0, 0, 1, 1},
  {&__pyx_n_s_simplify_warp_function_2d, __pyx_k_simplify_warp_function_2d, sizeof(__pyx_k_simplify_warp_function_2d), 0, 0, 1, 1},
  {&__pyx_n_s_simplify_warp_function_3d, __pyx_k_simplify_warp_function_3d, sizeof(__pyx_k_simplify_warp_function_3d), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_spacing, __pyx_k_spacing, sizeof(__pyx_k_spacing), 0, 0, 1, 1},
  {&__pyx_n_s_sparse_gradient, __pyx_k_sparse_gradient, sizeof(__pyx_k_sparse_gradient), 0, 0, 1, 1},
  {&__pyx_n_s_sparse_gradient_2d, __pyx_k_sparse_gradient_2d, sizeof(__pyx_k_sparse_gradient_2d), 0, 0, 1, 1},
  {&__pyx_n_s_sparse_gradient_3d, __pyx_k_sparse_gradient_3d, sizeof(__pyx_k_sparse_gradient_3d), 0, 0, 1, 1},
  {&__pyx_n_s_split, __pyx_k_split, sizeof(__pyx_k_split), 0, 0, 1, 1},
  {&__pyx_n_s_sqrt, __pyx_k_sqrt, sizeof(__pyx_k_sqrt), 0, 0, 1, 1},
  {&__pyx_n_s_sr, __pyx_k_sr, sizeof(__pyx_k_sr), 0, 0, 1, 1},
  {&__pyx_n_s_ss, __pyx_k_ss, sizeof(__pyx_k_ss), 0, 0, 1, 1},
  {&__pyx_n_s_start, __pyx_k_start, sizeof(__pyx_k_start), 0, 0, 1, 1},
  {&__pyx_n_s_stats, __pyx_k_stats, sizeof(__pyx_k_stats), 0, 0, 1, 1},
  {&__pyx_n_s_step, __pyx_k_step, sizeof(__pyx_k_step), 0, 0, 1, 1},
  {&__pyx_n_s_step_factor, __pyx_k_step_factor, sizeof(__pyx_k_step_factor), 0, 0, 1, 1},
  {&__pyx_n_s_stop, __pyx_k_stop, sizeof(__pyx_k_stop), 0, 0, 1, 1},
  {&__pyx_kp_s_strided_and_direct, __pyx_k_strided_and_direct, sizeof(__pyx_k_strided_and_direct), 0, 0, 1, 0},
  {&__pyx_kp_s_strided_and_direct_or_indirect, __pyx_k_strided_and_direct_or_indirect, sizeof(__pyx_k_strided_and_direct_or_indirect), 0, 0, 1, 0},
  {&__pyx_kp_s_strided_and_indirect, __pyx_k_strided_and_indirect, sizeof(__pyx_k_strided_and_indirect), 0, 0, 1, 0},
  {&__pyx_n_s_strip, __pyx_k_strip, sizeof(__pyx_k_strip), 0, 0, 1, 1},
  {&__pyx_n_s_struct, __pyx_k_struct, sizeof(__pyx_k_struct), 0, 0, 1, 1},
  {&__pyx_n_s_substats, __pyx_k_substats, sizeof(__pyx_k_substats), 0, 0, 1, 1},
  {&__pyx_n_s_tcols, __pyx_k_tcols, sizeof(__pyx_k_tcols), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_theta, __pyx_k_theta, sizeof(__pyx_k_theta), 0, 0, 1, 1},
  {&__pyx_n_s_time_scaling, __pyx_k_time_scaling, sizeof(__pyx_k_time_scaling), 0, 0, 1, 1},
  {&__pyx_n_s_tmp, __pyx_k_tmp, sizeof(__pyx_k_tmp), 0, 0, 1, 1},
  {&__pyx_n_s_tmp0, __pyx_k_tmp0, sizeof(__pyx_k_tmp0), 0, 0, 1, 1},
  {&__pyx_n_s_tmp1, __pyx_k_tmp1, sizeof(__pyx_k_tmp1), 0, 0, 1, 1},
  {&__pyx_n_s_to_grid2world, __pyx_k_to_grid2world, sizeof(__pyx_k_to_grid2world), 0, 0, 1, 1},
  {&__pyx_n_s_to_shape, __pyx_k_to_shape, sizeof(__pyx_k_to_shape), 0, 0, 1, 1},
  {&__pyx_n_s_tol, __pyx_k_tol, sizeof(__pyx_k_tol), 0, 0, 1, 1},
  {&__pyx_n_s_tolerance, __pyx_k_tolerance, sizeof(__pyx_k_tolerance), 0, 0, 1, 1},
  {&__pyx_n_s_transform_2d_affine, __pyx_k_transform_2d_affine, sizeof(__pyx_k_transform_2d_affine), 0, 0, 1, 1},
  {&__pyx_n_s_transform_2d_affine_nn, __pyx_k_transform_2d_affine_nn, sizeof(__pyx_k_transform_2d_affine_nn), 0, 0, 1, 1},
  {&__pyx_n_s_transform_3d_affine, __pyx_k_transform_3d_affine, sizeof(__pyx_k_transform_3d_affine), 0, 0, 1, 1},
  {&__pyx_n_s_transform_3d_affine_nn, __pyx_k_transform_3d_affine_nn, sizeof(__pyx_k_transform_3d_affine_nn), 0, 0, 1, 1},
  {&__pyx_n_s_trows, __pyx_k_trows, sizeof(__pyx_k_trows), 0, 0, 1, 1},
  {&__pyx_n_s_tslices, __pyx_k_tslices, sizeof(__pyx_k_tslices), 0, 0, 1, 1},
  {&__pyx_n_s_type, __pyx_k_type, sizeof(__pyx_k_type), 0, 0, 1, 1},
  {&__pyx_kp_s_unable_to_allocate_array_data, __pyx_k_unable_to_allocate_array_data, sizeof(__pyx_k_unable_to_allocate_array_data), 0, 0, 1, 0},
  {&__pyx_kp_s_unable_to_allocate_shape_and_str, __pyx_k_unable_to_allocate_shape_and_str, sizeof(__pyx_k_unable_to_allocate_shape_and_str), 0, 0, 1, 0},
  {&__pyx_kp_u_unknown_dtype_code_in_numpy_pxd, __pyx_k_unknown_dtype_code_in_numpy_pxd, sizeof(__pyx_k_unknown_dtype_code_in_numpy_pxd), 0, 1, 0, 0},
  {&__pyx_n_s_unpack, __pyx_k_unpack, sizeof(__pyx_k_unpack), 0, 0, 1, 1},
  {&__pyx_n_s_volume, __pyx_k_volume, sizeof(__pyx_k_volume), 0, 0, 1, 1},
  {&__pyx_n_s_warp_2d, __pyx_k_warp_2d, sizeof(__pyx_k_warp_2d), 0, 0, 1, 1},
  {&__pyx_n_s_warp_2d_nn, __pyx_k_warp_2d_nn, sizeof(__pyx_k_warp_2d_nn), 0, 0, 1, 1},
  {&__pyx_n_s_warp_3d, __pyx_k_warp_3d, sizeof(__pyx_k_warp_3d), 0, 0, 1, 1},
  {&__pyx_n_s_warp_3d_nn, __pyx_k_warp_3d_nn, sizeof(__pyx_k_warp_3d_nn), 0, 0, 1, 1},
  {&__pyx_n_s_warped, __pyx_k_warped, sizeof(__pyx_k_warped), 0, 0, 1, 1},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {&__pyx_n_s_zeros, __pyx_k_zeros, sizeof(__pyx_k_zeros), 0, 0, 1, 1},
  {&__pyx_n_s_zeros_like, __pyx_k_zeros_like, sizeof(__pyx_k_zeros_like), 0, 0, 1, 1},
  {&__pyx_n_s_zip, __pyx_k_zip, sizeof(__pyx_k_zip), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_ImportError = __Pyx_GetBuiltinName(__pyx_n_s_ImportError); if (!__pyx_builtin_ImportError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_AttributeError = __Pyx_GetBuiltinName(__pyx_n_s_AttributeError); if (!__pyx_builtin_AttributeError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_zip = __Pyx_GetBuiltinName(__pyx_n_s_zip); if (!__pyx_builtin_zip) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 56; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 906; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_enumerate = __Pyx_GetBuiltinName(__pyx_n_s_enumerate); if (!__pyx_builtin_enumerate) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 147; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_Ellipsis = __Pyx_GetBuiltinName(__pyx_n_s_Ellipsis); if (!__pyx_builtin_Ellipsis) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 359; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_id = __Pyx_GetBuiltinName(__pyx_n_s_id); if (!__pyx_builtin_id) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 571; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_IndexError = __Pyx_GetBuiltinName(__pyx_n_s_IndexError); if (!__pyx_builtin_IndexError) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 790; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "dipy/align/vector_fields.pyx":25
 * 
 * 
 * def interpolate_vector_2d(floating[:, :, :] field, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Bilinear interpolation of a 2D vector field
 * 
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "dipy/align/vector_fields.pyx":135
 * 
 * 
 * def interpolate_scalar_2d(floating[:, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Bilinear interpolation of a 2D scalar image
 * 
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "dipy/align/vector_fields.pyx":239
 * 
 * 
 * def interpolate_scalar_nn_2d(number[:, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 2D scalar image
 * 
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "dipy/align/vector_fields.pyx":332
 * 
 * 
 * def interpolate_scalar_nn_3d(number[:, :, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 3D scalar image
 * 
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "dipy/align/vector_fields.pyx":434
 * 
 * 
 * def interpolate_scalar_3d(floating[:, :, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Trilinear interpolation of a 3D scalar image
 * 
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__19)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);
  __pyx_tuple__20 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__20)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);
  __pyx_tuple__21 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__21)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);
  __pyx_tuple__22 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__22)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "dipy/align/vector_fields.pyx":567
 * 
 * 
 * def interpolate_vector_3d(floating[:, :, :, :] field, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Trilinear interpolation of a 3D vector field
 * 
 */
  __pyx_tuple__23 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__23)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);
  __pyx_tuple__24 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__24)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);
  __pyx_tuple__25 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__25)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);
  __pyx_tuple__26 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__26)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);

  /* "dipy/align/vector_fields.pyx":847
 * 
 * 
 * def compose_vector_fields_2d(floating[:, :, :] d1, floating[:, :, :] d2,             # <<<<<<<<<<<<<<
 *                              double[:, :] premult_index,
 *                              double[:, :] premult_disp,
 */
  __pyx_tuple__27 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__27)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);
  __pyx_tuple__28 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__28)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);
  __pyx_tuple__29 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__29)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);
  __pyx_tuple__30 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__30)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);

  /* "dipy/align/vector_fields.pyx":900
 *     """
 *     cdef:
 *         double[:] stats = np.zeros(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 * 
 *     if comp is None:
 */
  __pyx_tuple__31 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__31)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);

  /* "dipy/align/vector_fields.pyx":906
 * 
 *     if not is_valid_affine(premult_index, 2):
 *         raise ValueError("Invalid index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(premult_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__32 = PyTuple_Pack(1, __pyx_kp_s_Invalid_index_multiplication_mat); if (unlikely(!__pyx_tuple__32)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 906; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);

  /* "dipy/align/vector_fields.pyx":908
 *         raise ValueError("Invalid index multiplication matrix")
 *     if not is_valid_affine(premult_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     _compose_vector_fields_2d[floating](d1, d2, premult_index, premult_disp,
 */
  __pyx_tuple__33 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__33)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 908; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);

  /* "dipy/align/vector_fields.pyx":900
 *     """
 *     cdef:
 *         double[:] stats = np.zeros(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 * 
 *     if comp is None:
 */
  __pyx_tuple__34 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__34)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 900; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);

  /* "dipy/align/vector_fields.pyx":906
 * 
 *     if not is_valid_affine(premult_index, 2):
 *         raise ValueError("Invalid index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(premult_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__35 = PyTuple_Pack(1, __pyx_kp_s_Invalid_index_multiplication_mat); if (unlikely(!__pyx_tuple__35)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 906; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__35);
  __Pyx_GIVEREF(__pyx_tuple__35);

  /* "dipy/align/vector_fields.pyx":908
 *         raise ValueError("Invalid index multiplication matrix")
 *     if not is_valid_affine(premult_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     _compose_vector_fields_2d[floating](d1, d2, premult_index, premult_disp,
 */
  __pyx_tuple__36 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__36)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 908; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__36);
  __Pyx_GIVEREF(__pyx_tuple__36);

  /* "dipy/align/vector_fields.pyx":1054
 * 
 * 
 * def compose_vector_fields_3d(floating[:, :, :, :] d1, floating[:, :, :, :] d2,             # <<<<<<<<<<<<<<
 *                              double[:, :] premult_index,
 *                              double[:, :] premult_disp,
 */
  __pyx_tuple__37 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__37)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);
  __pyx_tuple__38 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__38)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__38);
  __Pyx_GIVEREF(__pyx_tuple__38);
  __pyx_tuple__39 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__39)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__39);
  __Pyx_GIVEREF(__pyx_tuple__39);
  __pyx_tuple__40 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__40)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__40);
  __Pyx_GIVEREF(__pyx_tuple__40);

  /* "dipy/align/vector_fields.pyx":1112
 *     """
 *     cdef:
 *         double[:] stats = np.zeros(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 * 
 *     if comp is None:
 */
  __pyx_tuple__41 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__41)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__41);
  __Pyx_GIVEREF(__pyx_tuple__41);

  /* "dipy/align/vector_fields.pyx":1118
 * 
 *     if not is_valid_affine(premult_index, 3):
 *         raise ValueError("Invalid index pre-multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(premult_disp, 3):
 *         raise ValueError("Invalid displacement pre-multiplication matrix")
 */
  __pyx_tuple__42 = PyTuple_Pack(1, __pyx_kp_s_Invalid_index_pre_multiplication); if (unlikely(!__pyx_tuple__42)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1118; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__42);
  __Pyx_GIVEREF(__pyx_tuple__42);

  /* "dipy/align/vector_fields.pyx":1120
 *         raise ValueError("Invalid index pre-multiplication matrix")
 *     if not is_valid_affine(premult_disp, 3):
 *         raise ValueError("Invalid displacement pre-multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     _compose_vector_fields_3d[floating](d1, d2, premult_index, premult_disp,
 */
  __pyx_tuple__43 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_pre_multipl); if (unlikely(!__pyx_tuple__43)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1120; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__43);
  __Pyx_GIVEREF(__pyx_tuple__43);

  /* "dipy/align/vector_fields.pyx":1112
 *     """
 *     cdef:
 *         double[:] stats = np.zeros(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 * 
 *     if comp is None:
 */
  __pyx_tuple__44 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__44)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__44);
  __Pyx_GIVEREF(__pyx_tuple__44);

  /* "dipy/align/vector_fields.pyx":1118
 * 
 *     if not is_valid_affine(premult_index, 3):
 *         raise ValueError("Invalid index pre-multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(premult_disp, 3):
 *         raise ValueError("Invalid displacement pre-multiplication matrix")
 */
  __pyx_tuple__45 = PyTuple_Pack(1, __pyx_kp_s_Invalid_index_pre_multiplication); if (unlikely(!__pyx_tuple__45)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1118; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__45);
  __Pyx_GIVEREF(__pyx_tuple__45);

  /* "dipy/align/vector_fields.pyx":1120
 *         raise ValueError("Invalid index pre-multiplication matrix")
 *     if not is_valid_affine(premult_disp, 3):
 *         raise ValueError("Invalid displacement pre-multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     _compose_vector_fields_3d[floating](d1, d2, premult_index, premult_disp,
 */
  __pyx_tuple__46 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_pre_multipl); if (unlikely(!__pyx_tuple__46)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1120; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__46);
  __Pyx_GIVEREF(__pyx_tuple__46);

  /* "dipy/align/vector_fields.pyx":1127
 * 
 * 
 * def invert_vector_field_fixed_point_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                                        double[:, :] d_world2grid,
 *                                        double[:] spacing,
 */
  __pyx_tuple__47 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__47)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__47);
  __Pyx_GIVEREF(__pyx_tuple__47);
  __pyx_tuple__48 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__48)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__48);
  __Pyx_GIVEREF(__pyx_tuple__48);
  __pyx_tuple__49 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__49)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__49);
  __Pyx_GIVEREF(__pyx_tuple__49);
  __pyx_tuple__50 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__50)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__50);
  __Pyx_GIVEREF(__pyx_tuple__50);

  /* "dipy/align/vector_fields.pyx":1185
 *     ftype = np.asarray(d).dtype
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] substats = np.empty(shape=(3,), dtype=np.float64)
 *         double[:, :] norms = np.zeros(shape=(nr, nc), dtype=np.float64)
 */
  __pyx_tuple__51 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__51)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__51);
  __Pyx_GIVEREF(__pyx_tuple__51);

  /* "dipy/align/vector_fields.pyx":1186
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)
 *         double[:] substats = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:, :] norms = np.zeros(shape=(nr, nc), dtype=np.float64)
 *         floating[:, :, :] p = np.zeros(shape=(nr, nc, 2), dtype=ftype)
 */
  __pyx_tuple__52 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__52)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__52);
  __Pyx_GIVEREF(__pyx_tuple__52);

  /* "dipy/align/vector_fields.pyx":1192
 * 
 *     if not is_valid_affine(d_world2grid, 2):
 *         raise ValueError("Invalid world-to-image transform")             # <<<<<<<<<<<<<<
 * 
 *     if start is not None:
 */
  __pyx_tuple__53 = PyTuple_Pack(1, __pyx_kp_s_Invalid_world_to_image_transform); if (unlikely(!__pyx_tuple__53)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1192; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__53);
  __Pyx_GIVEREF(__pyx_tuple__53);

  /* "dipy/align/vector_fields.pyx":1185
 *     ftype = np.asarray(d).dtype
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] substats = np.empty(shape=(3,), dtype=np.float64)
 *         double[:, :] norms = np.zeros(shape=(nr, nc), dtype=np.float64)
 */
  __pyx_tuple__54 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__54)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1185; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__54);
  __Pyx_GIVEREF(__pyx_tuple__54);

  /* "dipy/align/vector_fields.pyx":1186
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)
 *         double[:] substats = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:, :] norms = np.zeros(shape=(nr, nc), dtype=np.float64)
 *         floating[:, :, :] p = np.zeros(shape=(nr, nc, 2), dtype=ftype)
 */
  __pyx_tuple__55 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__55)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1186; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__55);
  __Pyx_GIVEREF(__pyx_tuple__55);

  /* "dipy/align/vector_fields.pyx":1192
 * 
 *     if not is_valid_affine(d_world2grid, 2):
 *         raise ValueError("Invalid world-to-image transform")             # <<<<<<<<<<<<<<
 * 
 *     if start is not None:
 */
  __pyx_tuple__56 = PyTuple_Pack(1, __pyx_kp_s_Invalid_world_to_image_transform); if (unlikely(!__pyx_tuple__56)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1192; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__56);
  __Pyx_GIVEREF(__pyx_tuple__56);

  /* "dipy/align/vector_fields.pyx":1231
 * 
 * 
 * def invert_vector_field_fixed_point_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                                        double[:, :] d_world2grid,
 *                                        double[:] spacing,
 */
  __pyx_tuple__57 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__57)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__57);
  __Pyx_GIVEREF(__pyx_tuple__57);
  __pyx_tuple__58 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__58)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__58);
  __Pyx_GIVEREF(__pyx_tuple__58);
  __pyx_tuple__59 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__59)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__59);
  __Pyx_GIVEREF(__pyx_tuple__59);
  __pyx_tuple__60 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__60)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__60);
  __Pyx_GIVEREF(__pyx_tuple__60);

  /* "dipy/align/vector_fields.pyx":1290
 *     ftype = np.asarray(d).dtype
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] substats = np.zeros(shape=(3,), dtype=np.float64)
 *         double[:, :, :] norms = np.zeros(shape=(ns, nr, nc), dtype=np.float64)
 */
  __pyx_tuple__61 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__61)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__61);
  __Pyx_GIVEREF(__pyx_tuple__61);

  /* "dipy/align/vector_fields.pyx":1291
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)
 *         double[:] substats = np.zeros(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:, :, :] norms = np.zeros(shape=(ns, nr, nc), dtype=np.float64)
 *         floating[:, :, :, :] p = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)
 */
  __pyx_tuple__62 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__62)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__62);
  __Pyx_GIVEREF(__pyx_tuple__62);

  /* "dipy/align/vector_fields.pyx":1297
 * 
 *     if not is_valid_affine(d_world2grid, 3):
 *         raise ValueError("Invalid world-to-image transform")             # <<<<<<<<<<<<<<
 * 
 *     if start is not None:
 */
  __pyx_tuple__63 = PyTuple_Pack(1, __pyx_kp_s_Invalid_world_to_image_transform); if (unlikely(!__pyx_tuple__63)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1297; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__63);
  __Pyx_GIVEREF(__pyx_tuple__63);

  /* "dipy/align/vector_fields.pyx":1290
 *     ftype = np.asarray(d).dtype
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] substats = np.zeros(shape=(3,), dtype=np.float64)
 *         double[:, :, :] norms = np.zeros(shape=(ns, nr, nc), dtype=np.float64)
 */
  __pyx_tuple__64 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1290; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__64);
  __Pyx_GIVEREF(__pyx_tuple__64);

  /* "dipy/align/vector_fields.pyx":1291
 *     cdef:
 *         double[:] stats = np.zeros(shape=(2,), dtype=np.float64)
 *         double[:] substats = np.zeros(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:, :, :] norms = np.zeros(shape=(ns, nr, nc), dtype=np.float64)
 *         floating[:, :, :, :] p = np.zeros(shape=(ns, nr, nc, 3), dtype=ftype)
 */
  __pyx_tuple__65 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__65)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1291; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__65);
  __Pyx_GIVEREF(__pyx_tuple__65);

  /* "dipy/align/vector_fields.pyx":1297
 * 
 *     if not is_valid_affine(d_world2grid, 3):
 *         raise ValueError("Invalid world-to-image transform")             # <<<<<<<<<<<<<<
 * 
 *     if start is not None:
 */
  __pyx_tuple__66 = PyTuple_Pack(1, __pyx_kp_s_Invalid_world_to_image_transform); if (unlikely(!__pyx_tuple__66)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1297; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__66);
  __Pyx_GIVEREF(__pyx_tuple__66);

  /* "dipy/align/vector_fields.pyx":1345
 * 
 * 
 * def simplify_warp_function_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                               double[:, :] affine_idx_in,
 *                               double[:, :] affine_idx_out,
 */
  __pyx_tuple__67 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__67)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__67);
  __Pyx_GIVEREF(__pyx_tuple__67);
  __pyx_tuple__68 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__68)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__68);
  __Pyx_GIVEREF(__pyx_tuple__68);
  __pyx_tuple__69 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__69)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__69);
  __Pyx_GIVEREF(__pyx_tuple__69);
  __pyx_tuple__70 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__70)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__70);
  __Pyx_GIVEREF(__pyx_tuple__70);

  /* "dipy/align/vector_fields.pyx":1411
 *         cnp.npy_intp i, j
 *         double di, dj, dii, djj
 *         floating[:] tmp = np.zeros((2,), dtype=np.asarray(d).dtype)             # <<<<<<<<<<<<<<
 *         floating[:, :, :] out = np.zeros(shape=(nrows, ncols, 2),
 *                                          dtype=np.asarray(d).dtype)
 */
  __pyx_tuple__71 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__71)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__71);
  __Pyx_GIVEREF(__pyx_tuple__71);
  __pyx_tuple__72 = PyTuple_Pack(1, __pyx_tuple__71); if (unlikely(!__pyx_tuple__72)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__72);
  __Pyx_GIVEREF(__pyx_tuple__72);

  /* "dipy/align/vector_fields.pyx":1416
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__73 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__73)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__73);
  __Pyx_GIVEREF(__pyx_tuple__73);

  /* "dipy/align/vector_fields.pyx":1418
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__74 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__74)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__74);
  __Pyx_GIVEREF(__pyx_tuple__74);

  /* "dipy/align/vector_fields.pyx":1420
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__75 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__75)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__75);
  __Pyx_GIVEREF(__pyx_tuple__75);

  /* "dipy/align/vector_fields.pyx":1411
 *         cnp.npy_intp i, j
 *         double di, dj, dii, djj
 *         floating[:] tmp = np.zeros((2,), dtype=np.asarray(d).dtype)             # <<<<<<<<<<<<<<
 *         floating[:, :, :] out = np.zeros(shape=(nrows, ncols, 2),
 *                                          dtype=np.asarray(d).dtype)
 */
  __pyx_tuple__76 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__76)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__76);
  __Pyx_GIVEREF(__pyx_tuple__76);
  __pyx_tuple__77 = PyTuple_Pack(1, __pyx_tuple__76); if (unlikely(!__pyx_tuple__77)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1411; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__77);
  __Pyx_GIVEREF(__pyx_tuple__77);

  /* "dipy/align/vector_fields.pyx":1416
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__78 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__78)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1416; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__78);
  __Pyx_GIVEREF(__pyx_tuple__78);

  /* "dipy/align/vector_fields.pyx":1418
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__79 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__79)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__79);
  __Pyx_GIVEREF(__pyx_tuple__79);

  /* "dipy/align/vector_fields.pyx":1420
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__80 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__80)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__80);
  __Pyx_GIVEREF(__pyx_tuple__80);

  /* "dipy/align/vector_fields.pyx":1461
 * 
 * 
 * def simplify_warp_function_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                               double[:, :] affine_idx_in,
 *                               double[:, :] affine_idx_out,
 */
  __pyx_tuple__81 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__81)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__81);
  __Pyx_GIVEREF(__pyx_tuple__81);
  __pyx_tuple__82 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__82)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__82);
  __Pyx_GIVEREF(__pyx_tuple__82);
  __pyx_tuple__83 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__83)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__83);
  __Pyx_GIVEREF(__pyx_tuple__83);
  __pyx_tuple__84 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__84)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__84);
  __Pyx_GIVEREF(__pyx_tuple__84);

  /* "dipy/align/vector_fields.pyx":1529
 *         cnp.npy_intp i, j, k, inside
 *         double di, dj, dk, dii, djj, dkk
 *         floating[:] tmp = np.zeros((3,), dtype=np.asarray(d).dtype)             # <<<<<<<<<<<<<<
 *         floating[:, :, :, :] out = np.zeros(shape=(nslices, nrows, ncols, 3),
 *                                             dtype=np.asarray(d).dtype)
 */
  __pyx_tuple__85 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__85)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__85);
  __Pyx_GIVEREF(__pyx_tuple__85);
  __pyx_tuple__86 = PyTuple_Pack(1, __pyx_tuple__85); if (unlikely(!__pyx_tuple__86)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__86);
  __Pyx_GIVEREF(__pyx_tuple__86);

  /* "dipy/align/vector_fields.pyx":1534
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__87 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__87)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1534; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__87);
  __Pyx_GIVEREF(__pyx_tuple__87);

  /* "dipy/align/vector_fields.pyx":1536
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__88 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__88)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1536; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__88);
  __Pyx_GIVEREF(__pyx_tuple__88);

  /* "dipy/align/vector_fields.pyx":1538
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__89 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__89)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1538; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__89);
  __Pyx_GIVEREF(__pyx_tuple__89);

  /* "dipy/align/vector_fields.pyx":1529
 *         cnp.npy_intp i, j, k, inside
 *         double di, dj, dk, dii, djj, dkk
 *         floating[:] tmp = np.zeros((3,), dtype=np.asarray(d).dtype)             # <<<<<<<<<<<<<<
 *         floating[:, :, :, :] out = np.zeros(shape=(nslices, nrows, ncols, 3),
 *                                             dtype=np.asarray(d).dtype)
 */
  __pyx_tuple__90 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__90)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__90);
  __Pyx_GIVEREF(__pyx_tuple__90);
  __pyx_tuple__91 = PyTuple_Pack(1, __pyx_tuple__90); if (unlikely(!__pyx_tuple__91)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1529; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__91);
  __Pyx_GIVEREF(__pyx_tuple__91);

  /* "dipy/align/vector_fields.pyx":1534
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__92 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__92)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1534; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__92);
  __Pyx_GIVEREF(__pyx_tuple__92);

  /* "dipy/align/vector_fields.pyx":1536
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__93 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__93)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1536; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__93);
  __Pyx_GIVEREF(__pyx_tuple__93);

  /* "dipy/align/vector_fields.pyx":1538
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__94 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__94)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1538; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__94);
  __Pyx_GIVEREF(__pyx_tuple__94);

  /* "dipy/align/vector_fields.pyx":1588
 * 
 * 
 * def reorient_vector_field_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                              double[:, :] affine):
 *     r"""Linearly transforms all vectors of a 2D displacement field
 */
  __pyx_tuple__95 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__95)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__95);
  __Pyx_GIVEREF(__pyx_tuple__95);
  __pyx_tuple__96 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__96)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__96);
  __Pyx_GIVEREF(__pyx_tuple__96);
  __pyx_tuple__97 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__97)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__97);
  __Pyx_GIVEREF(__pyx_tuple__97);
  __pyx_tuple__98 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__98)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__98);
  __Pyx_GIVEREF(__pyx_tuple__98);

  /* "dipy/align/vector_fields.pyx":1612
 * 
 *     if not is_valid_affine(affine, 2):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     if affine is None:
 */
  __pyx_tuple__99 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__99)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__99);
  __Pyx_GIVEREF(__pyx_tuple__99);
  __pyx_tuple__100 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__100)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1612; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__100);
  __Pyx_GIVEREF(__pyx_tuple__100);

  /* "dipy/align/vector_fields.pyx":1626
 * 
 * 
 * def reorient_vector_field_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                              double[:, :] affine):
 *     r"""Linearly transforms all vectors of a 3D displacement field
 */
  __pyx_tuple__101 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__101)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__101);
  __Pyx_GIVEREF(__pyx_tuple__101);
  __pyx_tuple__102 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__102)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__102);
  __Pyx_GIVEREF(__pyx_tuple__102);
  __pyx_tuple__103 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__103)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__103);
  __Pyx_GIVEREF(__pyx_tuple__103);
  __pyx_tuple__104 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__104)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__104);
  __Pyx_GIVEREF(__pyx_tuple__104);

  /* "dipy/align/vector_fields.pyx":1651
 * 
 *     if not is_valid_affine(affine, 3):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     if affine is None:
 */
  __pyx_tuple__105 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__105)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1651; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__105);
  __Pyx_GIVEREF(__pyx_tuple__105);
  __pyx_tuple__106 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__106)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1651; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__106);
  __Pyx_GIVEREF(__pyx_tuple__106);

  /* "dipy/align/vector_fields.pyx":1668
 * 
 * 
 * def downsample_scalar_field_3d(floating[:, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input volume by a factor of 2
 * 
 */
  __pyx_tuple__107 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__107)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__107);
  __Pyx_GIVEREF(__pyx_tuple__107);
  __pyx_tuple__108 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__108)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__108);
  __Pyx_GIVEREF(__pyx_tuple__108);
  __pyx_tuple__109 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__109)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__109);
  __Pyx_GIVEREF(__pyx_tuple__109);
  __pyx_tuple__110 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__110)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__110);
  __Pyx_GIVEREF(__pyx_tuple__110);

  /* "dipy/align/vector_fields.pyx":1715
 * 
 * 
 * def downsample_displacement_field_3d(floating[:, :, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input 3D vector field by a factor of 2
 * 
 */
  __pyx_tuple__111 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__111)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__111);
  __Pyx_GIVEREF(__pyx_tuple__111);
  __pyx_tuple__112 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__112)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__112);
  __Pyx_GIVEREF(__pyx_tuple__112);
  __pyx_tuple__113 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__113)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__113);
  __Pyx_GIVEREF(__pyx_tuple__113);
  __pyx_tuple__114 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__114)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__114);
  __Pyx_GIVEREF(__pyx_tuple__114);

  /* "dipy/align/vector_fields.pyx":1767
 * 
 * 
 * def downsample_scalar_field_2d(floating[:, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input 2D image by a factor of 2
 * 
 */
  __pyx_tuple__115 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__115)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__115);
  __Pyx_GIVEREF(__pyx_tuple__115);
  __pyx_tuple__116 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__116)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__116);
  __Pyx_GIVEREF(__pyx_tuple__116);
  __pyx_tuple__117 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__117)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__117);
  __Pyx_GIVEREF(__pyx_tuple__117);
  __pyx_tuple__118 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__118)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__118);
  __Pyx_GIVEREF(__pyx_tuple__118);

  /* "dipy/align/vector_fields.pyx":1808
 * 
 * 
 * def downsample_displacement_field_2d(floating[:, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the 2D input vector field by a factor of 2
 *     Down-samples the input vector field by a factor of 2. The value at each
 */
  __pyx_tuple__119 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__119)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__119);
  __Pyx_GIVEREF(__pyx_tuple__119);
  __pyx_tuple__120 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__120)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__120);
  __Pyx_GIVEREF(__pyx_tuple__120);
  __pyx_tuple__121 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__121)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__121);
  __Pyx_GIVEREF(__pyx_tuple__121);
  __pyx_tuple__122 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__122)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__122);
  __Pyx_GIVEREF(__pyx_tuple__122);

  /* "dipy/align/vector_fields.pyx":1851
 * 
 * 
 * def warp_3d(floating[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 */
  __pyx_tuple__125 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__125)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__125);
  __Pyx_GIVEREF(__pyx_tuple__125);
  __pyx_tuple__126 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__126)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__126);
  __Pyx_GIVEREF(__pyx_tuple__126);
  __pyx_tuple__127 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__127)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__127);
  __Pyx_GIVEREF(__pyx_tuple__127);
  __pyx_tuple__128 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__128)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__128);
  __Pyx_GIVEREF(__pyx_tuple__128);

  /* "dipy/align/vector_fields.pyx":1911
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__129 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__129)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1911; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__129);
  __Pyx_GIVEREF(__pyx_tuple__129);

  /* "dipy/align/vector_fields.pyx":1913
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__130 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__130)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1913; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__130);
  __Pyx_GIVEREF(__pyx_tuple__130);

  /* "dipy/align/vector_fields.pyx":1915
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__131 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__131)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__131);
  __Pyx_GIVEREF(__pyx_tuple__131);

  /* "dipy/align/vector_fields.pyx":1928
 *     cdef floating[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                              dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__132 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__132)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__132);
  __Pyx_GIVEREF(__pyx_tuple__132);

  /* "dipy/align/vector_fields.pyx":1911
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__133 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__133)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1911; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__133);
  __Pyx_GIVEREF(__pyx_tuple__133);

  /* "dipy/align/vector_fields.pyx":1913
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__134 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__134)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1913; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__134);
  __Pyx_GIVEREF(__pyx_tuple__134);

  /* "dipy/align/vector_fields.pyx":1915
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__135 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__135)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1915; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__135);
  __Pyx_GIVEREF(__pyx_tuple__135);

  /* "dipy/align/vector_fields.pyx":1928
 *     cdef floating[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                              dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__136 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__136)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1928; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__136);
  __Pyx_GIVEREF(__pyx_tuple__136);

  /* "dipy/align/vector_fields.pyx":1982
 * 
 * 
 * def transform_3d_affine(floating[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                         double[:, :] affine):
 *     r"""Transforms a 3D volume by an affine transform with trilinear interp.
 */
  __pyx_tuple__137 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__137)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__137);
  __Pyx_GIVEREF(__pyx_tuple__137);
  __pyx_tuple__138 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__138)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__138);
  __Pyx_GIVEREF(__pyx_tuple__138);
  __pyx_tuple__139 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__139)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__139);
  __Pyx_GIVEREF(__pyx_tuple__139);
  __pyx_tuple__140 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__140)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__140);
  __Pyx_GIVEREF(__pyx_tuple__140);

  /* "dipy/align/vector_fields.pyx":2028
 * 
 *     if not is_valid_affine(affine, 3):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__141 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__141)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2028; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__141);
  __Pyx_GIVEREF(__pyx_tuple__141);
  __pyx_tuple__142 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__142)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2028; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__142);
  __Pyx_GIVEREF(__pyx_tuple__142);

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_tuple__145 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__145)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__145);
  __Pyx_GIVEREF(__pyx_tuple__145);
  __pyx_tuple__146 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__146)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__146);
  __Pyx_GIVEREF(__pyx_tuple__146);
  __pyx_tuple__147 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__147)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__147);
  __Pyx_GIVEREF(__pyx_tuple__147);
  __pyx_tuple__148 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__148)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__148);
  __Pyx_GIVEREF(__pyx_tuple__148);

  /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__149 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__149)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__149);
  __Pyx_GIVEREF(__pyx_tuple__149);

  /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__150 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__150)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__150);
  __Pyx_GIVEREF(__pyx_tuple__150);

  /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__151 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__151)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__151);
  __Pyx_GIVEREF(__pyx_tuple__151);

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__152 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__152)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__152);
  __Pyx_GIVEREF(__pyx_tuple__152);

  /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__153 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__153)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__153);
  __Pyx_GIVEREF(__pyx_tuple__153);

  /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__154 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__154)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__154);
  __Pyx_GIVEREF(__pyx_tuple__154);

  /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__155 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__155)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__155);
  __Pyx_GIVEREF(__pyx_tuple__155);

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__156 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__156)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__156);
  __Pyx_GIVEREF(__pyx_tuple__156);

  /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__157 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__157)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__157);
  __Pyx_GIVEREF(__pyx_tuple__157);

  /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__158 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__158)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__158);
  __Pyx_GIVEREF(__pyx_tuple__158);

  /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__159 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__159)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__159);
  __Pyx_GIVEREF(__pyx_tuple__159);

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__160 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__160)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__160);
  __Pyx_GIVEREF(__pyx_tuple__160);

  /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__161 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__161)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__161);
  __Pyx_GIVEREF(__pyx_tuple__161);

  /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__162 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__162)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__162);
  __Pyx_GIVEREF(__pyx_tuple__162);

  /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__163 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__163)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__163);
  __Pyx_GIVEREF(__pyx_tuple__163);

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__164 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__164)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__164);
  __Pyx_GIVEREF(__pyx_tuple__164);

  /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__165 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__165)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__165);
  __Pyx_GIVEREF(__pyx_tuple__165);

  /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__166 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__166)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__166);
  __Pyx_GIVEREF(__pyx_tuple__166);

  /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__167 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__167)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__167);
  __Pyx_GIVEREF(__pyx_tuple__167);

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__168 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__168)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__168);
  __Pyx_GIVEREF(__pyx_tuple__168);

  /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__169 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__169)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__169);
  __Pyx_GIVEREF(__pyx_tuple__169);

  /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__170 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__170)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__170);
  __Pyx_GIVEREF(__pyx_tuple__170);

  /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__171 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__171)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__171);
  __Pyx_GIVEREF(__pyx_tuple__171);

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__172 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__172)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__172);
  __Pyx_GIVEREF(__pyx_tuple__172);

  /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__173 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__173)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__173);
  __Pyx_GIVEREF(__pyx_tuple__173);

  /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__174 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__174)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__174);
  __Pyx_GIVEREF(__pyx_tuple__174);

  /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__175 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__175)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__175);
  __Pyx_GIVEREF(__pyx_tuple__175);

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__176 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__176)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__176);
  __Pyx_GIVEREF(__pyx_tuple__176);

  /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__177 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__177)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__177);
  __Pyx_GIVEREF(__pyx_tuple__177);

  /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__178 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__178)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__178);
  __Pyx_GIVEREF(__pyx_tuple__178);

  /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__179 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__179)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__179);
  __Pyx_GIVEREF(__pyx_tuple__179);

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__180 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__180)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__180);
  __Pyx_GIVEREF(__pyx_tuple__180);

  /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__181 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__181)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__181);
  __Pyx_GIVEREF(__pyx_tuple__181);

  /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__182 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__182)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__182);
  __Pyx_GIVEREF(__pyx_tuple__182);

  /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__183 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__183)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__183);
  __Pyx_GIVEREF(__pyx_tuple__183);

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__184 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__184)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__184);
  __Pyx_GIVEREF(__pyx_tuple__184);

  /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__185 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__185)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__185);
  __Pyx_GIVEREF(__pyx_tuple__185);

  /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__186 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__186)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__186);
  __Pyx_GIVEREF(__pyx_tuple__186);

  /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__187 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__187)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__187);
  __Pyx_GIVEREF(__pyx_tuple__187);

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__188 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__188)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__188);
  __Pyx_GIVEREF(__pyx_tuple__188);

  /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__189 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__189)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__189);
  __Pyx_GIVEREF(__pyx_tuple__189);

  /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__190 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__190)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__190);
  __Pyx_GIVEREF(__pyx_tuple__190);

  /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__191 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__191)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__191);
  __Pyx_GIVEREF(__pyx_tuple__191);

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__192 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__192)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__192);
  __Pyx_GIVEREF(__pyx_tuple__192);

  /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__193 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__193)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__193);
  __Pyx_GIVEREF(__pyx_tuple__193);

  /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__194 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__194)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__194);
  __Pyx_GIVEREF(__pyx_tuple__194);

  /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__195 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__195)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__195);
  __Pyx_GIVEREF(__pyx_tuple__195);

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__196 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__196)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__196);
  __Pyx_GIVEREF(__pyx_tuple__196);

  /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__197 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__197)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__197);
  __Pyx_GIVEREF(__pyx_tuple__197);

  /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__198 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__198)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__198);
  __Pyx_GIVEREF(__pyx_tuple__198);

  /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__199 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__199)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__199);
  __Pyx_GIVEREF(__pyx_tuple__199);

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__200 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__200)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__200);
  __Pyx_GIVEREF(__pyx_tuple__200);

  /* "dipy/align/vector_fields.pyx":2108
 * 
 *     if not is_valid_affine(affine_idx_in, 3):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__201 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__201)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2108; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__201);
  __Pyx_GIVEREF(__pyx_tuple__201);

  /* "dipy/align/vector_fields.pyx":2110
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 3):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__202 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__202)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2110; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__202);
  __Pyx_GIVEREF(__pyx_tuple__202);

  /* "dipy/align/vector_fields.pyx":2112
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 3):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__203 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__203)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2112; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__203);
  __Pyx_GIVEREF(__pyx_tuple__203);

  /* "dipy/align/vector_fields.pyx":2125
 *     cdef number[:, :, :] warped = np.zeros(shape=(nslices, nrows, ncols),
 *                                            dtype=np.asarray(volume).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(3,), dtype = np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__204 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__204)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2125; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__204);
  __Pyx_GIVEREF(__pyx_tuple__204);

  /* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */
  __pyx_tuple__206 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__206)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__206);
  __Pyx_GIVEREF(__pyx_tuple__206);
  __pyx_tuple__207 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__207)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__207);
  __Pyx_GIVEREF(__pyx_tuple__207);
  __pyx_tuple__208 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__208)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__208);
  __Pyx_GIVEREF(__pyx_tuple__208);
  __pyx_tuple__209 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__209)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__209);
  __Pyx_GIVEREF(__pyx_tuple__209);

  /* "dipy/align/vector_fields.pyx":2223
 * 
 *     if not is_valid_affine(affine, 3):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__210 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__210)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__210);
  __Pyx_GIVEREF(__pyx_tuple__210);
  __pyx_tuple__211 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__211)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__211);
  __Pyx_GIVEREF(__pyx_tuple__211);
  __pyx_tuple__212 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__212)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__212);
  __Pyx_GIVEREF(__pyx_tuple__212);
  __pyx_tuple__213 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__213)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__213);
  __Pyx_GIVEREF(__pyx_tuple__213);
  __pyx_tuple__214 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__214)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__214);
  __Pyx_GIVEREF(__pyx_tuple__214);
  __pyx_tuple__215 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__215)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__215);
  __Pyx_GIVEREF(__pyx_tuple__215);
  __pyx_tuple__216 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__216)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2223; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__216);
  __Pyx_GIVEREF(__pyx_tuple__216);

  /* "dipy/align/vector_fields.pyx":2243
 * 
 * 
 * def warp_2d(floating[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 */
  __pyx_tuple__219 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__219)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__219);
  __Pyx_GIVEREF(__pyx_tuple__219);
  __pyx_tuple__220 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__220)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__220);
  __Pyx_GIVEREF(__pyx_tuple__220);
  __pyx_tuple__221 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__221)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__221);
  __Pyx_GIVEREF(__pyx_tuple__221);
  __pyx_tuple__222 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__222)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__222);
  __Pyx_GIVEREF(__pyx_tuple__222);

  /* "dipy/align/vector_fields.pyx":2300
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__223 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__223)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2300; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__223);
  __Pyx_GIVEREF(__pyx_tuple__223);

  /* "dipy/align/vector_fields.pyx":2302
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__224 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__224)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2302; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__224);
  __Pyx_GIVEREF(__pyx_tuple__224);

  /* "dipy/align/vector_fields.pyx":2304
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__225 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__225)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__225);
  __Pyx_GIVEREF(__pyx_tuple__225);

  /* "dipy/align/vector_fields.pyx":2314
 *     cdef floating[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                           dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__226 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__226)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__226);
  __Pyx_GIVEREF(__pyx_tuple__226);

  /* "dipy/align/vector_fields.pyx":2300
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__227 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__227)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2300; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__227);
  __Pyx_GIVEREF(__pyx_tuple__227);

  /* "dipy/align/vector_fields.pyx":2302
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__228 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__228)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2302; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__228);
  __Pyx_GIVEREF(__pyx_tuple__228);

  /* "dipy/align/vector_fields.pyx":2304
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__229 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__229)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__229);
  __Pyx_GIVEREF(__pyx_tuple__229);

  /* "dipy/align/vector_fields.pyx":2314
 *     cdef floating[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                           dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__230 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__230)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2314; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__230);
  __Pyx_GIVEREF(__pyx_tuple__230);

  /* "dipy/align/vector_fields.pyx":2357
 * 
 * 
 * def transform_2d_affine(floating[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                         double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with bilinear interp.
 */
  __pyx_tuple__232 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__232)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__232);
  __Pyx_GIVEREF(__pyx_tuple__232);
  __pyx_tuple__233 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__233)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__233);
  __Pyx_GIVEREF(__pyx_tuple__233);
  __pyx_tuple__234 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__234)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__234);
  __Pyx_GIVEREF(__pyx_tuple__234);
  __pyx_tuple__235 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__235)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__235);
  __Pyx_GIVEREF(__pyx_tuple__235);

  /* "dipy/align/vector_fields.pyx":2400
 * 
 *     if not is_valid_affine(affine, 2):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__236 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__236)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2400; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__236);
  __Pyx_GIVEREF(__pyx_tuple__236);
  __pyx_tuple__237 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__237)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2400; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__237);
  __Pyx_GIVEREF(__pyx_tuple__237);

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_tuple__240 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__240)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__240);
  __Pyx_GIVEREF(__pyx_tuple__240);
  __pyx_tuple__241 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__241)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__241);
  __Pyx_GIVEREF(__pyx_tuple__241);
  __pyx_tuple__242 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__242)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__242);
  __Pyx_GIVEREF(__pyx_tuple__242);
  __pyx_tuple__243 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__243)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__243);
  __Pyx_GIVEREF(__pyx_tuple__243);

  /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__244 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__244)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__244);
  __Pyx_GIVEREF(__pyx_tuple__244);

  /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__245 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__245)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__245);
  __Pyx_GIVEREF(__pyx_tuple__245);

  /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__246 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__246)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__246);
  __Pyx_GIVEREF(__pyx_tuple__246);

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__247 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__247)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__247);
  __Pyx_GIVEREF(__pyx_tuple__247);

  /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__248 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__248)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__248);
  __Pyx_GIVEREF(__pyx_tuple__248);

  /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__249 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__249)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__249);
  __Pyx_GIVEREF(__pyx_tuple__249);

  /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__250 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__250)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__250);
  __Pyx_GIVEREF(__pyx_tuple__250);

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__251 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__251)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__251);
  __Pyx_GIVEREF(__pyx_tuple__251);

  /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__252 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__252)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__252);
  __Pyx_GIVEREF(__pyx_tuple__252);

  /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__253 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__253)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__253);
  __Pyx_GIVEREF(__pyx_tuple__253);

  /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__254 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__254)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__254);
  __Pyx_GIVEREF(__pyx_tuple__254);

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__255 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__255)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__255);
  __Pyx_GIVEREF(__pyx_tuple__255);

  /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__256 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__256)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__256);
  __Pyx_GIVEREF(__pyx_tuple__256);

  /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__257 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__257)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__257);
  __Pyx_GIVEREF(__pyx_tuple__257);

  /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__258 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__258)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__258);
  __Pyx_GIVEREF(__pyx_tuple__258);

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__259 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__259)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__259);
  __Pyx_GIVEREF(__pyx_tuple__259);

  /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__260 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__260)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__260);
  __Pyx_GIVEREF(__pyx_tuple__260);

  /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__261 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__261)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__261);
  __Pyx_GIVEREF(__pyx_tuple__261);

  /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__262 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__262)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__262);
  __Pyx_GIVEREF(__pyx_tuple__262);

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__263 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__263)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__263);
  __Pyx_GIVEREF(__pyx_tuple__263);

  /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__264 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__264)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__264);
  __Pyx_GIVEREF(__pyx_tuple__264);

  /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__265 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__265)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__265);
  __Pyx_GIVEREF(__pyx_tuple__265);

  /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__266 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__266)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__266);
  __Pyx_GIVEREF(__pyx_tuple__266);

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__267 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__267)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__267);
  __Pyx_GIVEREF(__pyx_tuple__267);

  /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__268 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__268)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__268);
  __Pyx_GIVEREF(__pyx_tuple__268);

  /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__269 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__269)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__269);
  __Pyx_GIVEREF(__pyx_tuple__269);

  /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__270 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__270)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__270);
  __Pyx_GIVEREF(__pyx_tuple__270);

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__271 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__271)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__271);
  __Pyx_GIVEREF(__pyx_tuple__271);

  /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__272 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__272)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__272);
  __Pyx_GIVEREF(__pyx_tuple__272);

  /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__273 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__273)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__273);
  __Pyx_GIVEREF(__pyx_tuple__273);

  /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__274 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__274)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__274);
  __Pyx_GIVEREF(__pyx_tuple__274);

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__275 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__275)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__275);
  __Pyx_GIVEREF(__pyx_tuple__275);

  /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__276 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__276)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__276);
  __Pyx_GIVEREF(__pyx_tuple__276);

  /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__277 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__277)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__277);
  __Pyx_GIVEREF(__pyx_tuple__277);

  /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__278 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__278)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__278);
  __Pyx_GIVEREF(__pyx_tuple__278);

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__279 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__279)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__279);
  __Pyx_GIVEREF(__pyx_tuple__279);

  /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__280 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__280)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__280);
  __Pyx_GIVEREF(__pyx_tuple__280);

  /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__281 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__281)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__281);
  __Pyx_GIVEREF(__pyx_tuple__281);

  /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__282 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__282)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__282);
  __Pyx_GIVEREF(__pyx_tuple__282);

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__283 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__283)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__283);
  __Pyx_GIVEREF(__pyx_tuple__283);

  /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__284 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__284)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__284);
  __Pyx_GIVEREF(__pyx_tuple__284);

  /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__285 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__285)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__285);
  __Pyx_GIVEREF(__pyx_tuple__285);

  /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__286 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__286)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__286);
  __Pyx_GIVEREF(__pyx_tuple__286);

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__287 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__287)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__287);
  __Pyx_GIVEREF(__pyx_tuple__287);

  /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__288 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__288)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__288);
  __Pyx_GIVEREF(__pyx_tuple__288);

  /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__289 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__289)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__289);
  __Pyx_GIVEREF(__pyx_tuple__289);

  /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__290 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__290)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__290);
  __Pyx_GIVEREF(__pyx_tuple__290);

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__291 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__291)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__291);
  __Pyx_GIVEREF(__pyx_tuple__291);

  /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__292 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__292)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__292);
  __Pyx_GIVEREF(__pyx_tuple__292);

  /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__293 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__293)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__293);
  __Pyx_GIVEREF(__pyx_tuple__293);

  /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__294 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__294)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__294);
  __Pyx_GIVEREF(__pyx_tuple__294);

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__295 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__295)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__295);
  __Pyx_GIVEREF(__pyx_tuple__295);

  /* "dipy/align/vector_fields.pyx":2474
 * 
 *     if not is_valid_affine(affine_idx_in, 2):
 *         raise ValueError("Invalid inner index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")
 */
  __pyx_tuple__296 = PyTuple_Pack(1, __pyx_kp_s_Invalid_inner_index_multiplicati); if (unlikely(!__pyx_tuple__296)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2474; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__296);
  __Pyx_GIVEREF(__pyx_tuple__296);

  /* "dipy/align/vector_fields.pyx":2476
 *         raise ValueError("Invalid inner index multiplication matrix")
 *     if not is_valid_affine(affine_idx_out, 2):
 *         raise ValueError("Invalid outer index multiplication matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")
 */
  __pyx_tuple__297 = PyTuple_Pack(1, __pyx_kp_s_Invalid_outer_index_multiplicati); if (unlikely(!__pyx_tuple__297)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2476; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__297);
  __Pyx_GIVEREF(__pyx_tuple__297);

  /* "dipy/align/vector_fields.pyx":2478
 *         raise ValueError("Invalid outer index multiplication matrix")
 *     if not is_valid_affine(affine_disp, 2):
 *         raise ValueError("Invalid displacement multiplication matrix")             # <<<<<<<<<<<<<<
 * 
 *     if out_shape is not None:
 */
  __pyx_tuple__298 = PyTuple_Pack(1, __pyx_kp_s_Invalid_displacement_multiplicat); if (unlikely(!__pyx_tuple__298)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2478; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__298);
  __Pyx_GIVEREF(__pyx_tuple__298);

  /* "dipy/align/vector_fields.pyx":2488
 *     cdef number[:, :] warped = np.zeros(shape=(nrows, ncols),
 *                                         dtype=np.asarray(image).dtype)
 *     cdef floating[:] tmp = np.zeros(shape=(2,), dtype=np.asarray(d1).dtype)             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__299 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__299)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2488; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__299);
  __Pyx_GIVEREF(__pyx_tuple__299);

  /* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */
  __pyx_tuple__301 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__301)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__301);
  __Pyx_GIVEREF(__pyx_tuple__301);
  __pyx_tuple__302 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__302)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__302);
  __Pyx_GIVEREF(__pyx_tuple__302);
  __pyx_tuple__303 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__303)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__303);
  __Pyx_GIVEREF(__pyx_tuple__303);
  __pyx_tuple__304 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__304)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__304);
  __Pyx_GIVEREF(__pyx_tuple__304);

  /* "dipy/align/vector_fields.pyx":2574
 * 
 *     if not is_valid_affine(affine, 2):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
 */
  __pyx_tuple__305 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__305)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__305);
  __Pyx_GIVEREF(__pyx_tuple__305);
  __pyx_tuple__306 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__306)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__306);
  __Pyx_GIVEREF(__pyx_tuple__306);
  __pyx_tuple__307 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__307)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__307);
  __Pyx_GIVEREF(__pyx_tuple__307);
  __pyx_tuple__308 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__308)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__308);
  __Pyx_GIVEREF(__pyx_tuple__308);
  __pyx_tuple__309 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__309)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__309);
  __Pyx_GIVEREF(__pyx_tuple__309);
  __pyx_tuple__310 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__310)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__310);
  __Pyx_GIVEREF(__pyx_tuple__310);
  __pyx_tuple__311 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__311)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2574; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__311);
  __Pyx_GIVEREF(__pyx_tuple__311);

  /* "dipy/align/vector_fields.pyx":2591
 * 
 * 
 * def resample_displacement_field_3d(floating[:, :, :, :] field,             # <<<<<<<<<<<<<<
 *                                    double[:] factors, int[:] out_shape):
 *     r"""Resamples a 3D vector field to a custom target shape
 */
  __pyx_tuple__312 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__312)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__312);
  __Pyx_GIVEREF(__pyx_tuple__312);
  __pyx_tuple__313 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__313)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__313);
  __Pyx_GIVEREF(__pyx_tuple__313);
  __pyx_tuple__314 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__314)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__314);
  __Pyx_GIVEREF(__pyx_tuple__314);
  __pyx_tuple__315 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__315)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__315);
  __Pyx_GIVEREF(__pyx_tuple__315);

  /* "dipy/align/vector_fields.pyx":2636
 * 
 * 
 * def resample_displacement_field_2d(floating[:, :, :] field, double[:] factors,             # <<<<<<<<<<<<<<
 *                                    int[:] out_shape):
 *     r"""Resamples a 2D vector field to a custom target shape
 */
  __pyx_tuple__316 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__316)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__316);
  __Pyx_GIVEREF(__pyx_tuple__316);
  __pyx_tuple__317 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__317)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__317);
  __Pyx_GIVEREF(__pyx_tuple__317);
  __pyx_tuple__318 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__318)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__318);
  __Pyx_GIVEREF(__pyx_tuple__318);
  __pyx_tuple__319 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__319)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__319);
  __Pyx_GIVEREF(__pyx_tuple__319);

  /* "dipy/align/vector_fields.pyx":2713
 *         cnp.npy_intp i, j, ri, rj
 *         double di, dj, dii, djj
 *         int[:, :, :] int_field = np.empty(tuple(from_shape) + (2,),             # <<<<<<<<<<<<<<
 *                                             dtype=np.int32)
 *         double[:, :, :] output = np.zeros(tuple(from_shape) + (2,),
 */
  __pyx_tuple__320 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__320)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2713; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__320);
  __Pyx_GIVEREF(__pyx_tuple__320);

  /* "dipy/align/vector_fields.pyx":2715
 *         int[:, :, :] int_field = np.empty(tuple(from_shape) + (2,),
 *                                             dtype=np.int32)
 *         double[:, :, :] output = np.zeros(tuple(from_shape) + (2,),             # <<<<<<<<<<<<<<
 *                                           dtype=np.float64)
 *         cnp.npy_intp dom_size = from_shape[0]*from_shape[1]
 */
  __pyx_tuple__321 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__321)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__321);
  __Pyx_GIVEREF(__pyx_tuple__321);

  /* "dipy/align/vector_fields.pyx":2720
 * 
 *     if not is_valid_affine(from_grid2world, 2):
 *         raise ValueError("Invalid 'from' affine transform matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(to_grid2world, 2):
 *         raise ValueError("Invalid 'to' affine transform matrix")
 */
  __pyx_tuple__322 = PyTuple_Pack(1, __pyx_kp_s_Invalid_from_affine_transform_ma); if (unlikely(!__pyx_tuple__322)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2720; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__322);
  __Pyx_GIVEREF(__pyx_tuple__322);

  /* "dipy/align/vector_fields.pyx":2722
 *         raise ValueError("Invalid 'from' affine transform matrix")
 *     if not is_valid_affine(to_grid2world, 2):
 *         raise ValueError("Invalid 'to' affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     # compute the actual displacement field in the physical space
 */
  __pyx_tuple__323 = PyTuple_Pack(1, __pyx_kp_s_Invalid_to_affine_transform_matr); if (unlikely(!__pyx_tuple__323)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2722; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__323);
  __Pyx_GIVEREF(__pyx_tuple__323);

  /* "dipy/align/vector_fields.pyx":2793
 *         cnp.npy_intp i, j, k, ri, rj, rk
 *         double di, dj, dii, djj
 *         int[:, :, :, :] int_field = np.empty(tuple(from_shape) + (3,),             # <<<<<<<<<<<<<<
 *                                                dtype=np.int32)
 *         double[:, :, :, :] output = np.zeros(tuple(from_shape) + (3,),
 */
  __pyx_tuple__324 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__324)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2793; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__324);
  __Pyx_GIVEREF(__pyx_tuple__324);

  /* "dipy/align/vector_fields.pyx":2795
 *         int[:, :, :, :] int_field = np.empty(tuple(from_shape) + (3,),
 *                                                dtype=np.int32)
 *         double[:, :, :, :] output = np.zeros(tuple(from_shape) + (3,),             # <<<<<<<<<<<<<<
 *                                              dtype=np.float64)
 *         cnp.npy_intp dom_size = from_shape[0]*from_shape[1]*from_shape[2]
 */
  __pyx_tuple__325 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__325)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2795; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__325);
  __Pyx_GIVEREF(__pyx_tuple__325);

  /* "dipy/align/vector_fields.pyx":2800
 * 
 *     if not is_valid_affine(from_grid2world, 3):
 *         raise ValueError("Invalid 'from' affine transform matrix")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(to_grid2world, 3):
 *         raise ValueError("Invalid 'to' affine transform matrix")
 */
  __pyx_tuple__326 = PyTuple_Pack(1, __pyx_kp_s_Invalid_from_affine_transform_ma); if (unlikely(!__pyx_tuple__326)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2800; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__326);
  __Pyx_GIVEREF(__pyx_tuple__326);

  /* "dipy/align/vector_fields.pyx":2802
 *         raise ValueError("Invalid 'from' affine transform matrix")
 *     if not is_valid_affine(to_grid2world, 3):
 *         raise ValueError("Invalid 'to' affine transform matrix")             # <<<<<<<<<<<<<<
 * 
 *     # compute the actual displacement field in the physical space
 */
  __pyx_tuple__327 = PyTuple_Pack(1, __pyx_kp_s_Invalid_to_affine_transform_matr); if (unlikely(!__pyx_tuple__327)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2802; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__327);
  __Pyx_GIVEREF(__pyx_tuple__327);

  /* "dipy/align/vector_fields.pyx":3036
 * 
 * 
 * def _gradient_3d(floating[:, :, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                  double[:] img_spacing, double[:, :] out_grid2world,
 *                  floating[:, :, :, :] out, int[:, :, :] inside):
 */
  __pyx_tuple__328 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__328)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__328);
  __Pyx_GIVEREF(__pyx_tuple__328);
  __pyx_tuple__329 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__329)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__329);
  __Pyx_GIVEREF(__pyx_tuple__329);
  __pyx_tuple__330 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__330)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__330);
  __Pyx_GIVEREF(__pyx_tuple__330);
  __pyx_tuple__331 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__331)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__331);
  __Pyx_GIVEREF(__pyx_tuple__331);

  /* "dipy/align/vector_fields.pyx":3079
 *         int i, j, k, in_flag
 *         double tmp
 *         double[:] x = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 */
  __pyx_tuple__332 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__332)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__332);
  __Pyx_GIVEREF(__pyx_tuple__332);

  /* "dipy/align/vector_fields.pyx":3080
 *         double tmp
 *         double[:] x = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 */
  __pyx_tuple__333 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__333)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__333);
  __Pyx_GIVEREF(__pyx_tuple__333);

  /* "dipy/align/vector_fields.pyx":3081
 *         double[:] x = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:
 */
  __pyx_tuple__334 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__334)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__334);
  __Pyx_GIVEREF(__pyx_tuple__334);

  /* "dipy/align/vector_fields.pyx":3082
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 */
  __pyx_tuple__335 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__335)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__335);
  __Pyx_GIVEREF(__pyx_tuple__335);

  /* "dipy/align/vector_fields.pyx":3079
 *         int i, j, k, in_flag
 *         double tmp
 *         double[:] x = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 */
  __pyx_tuple__336 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__336)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3079; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__336);
  __Pyx_GIVEREF(__pyx_tuple__336);

  /* "dipy/align/vector_fields.pyx":3080
 *         double tmp
 *         double[:] x = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 */
  __pyx_tuple__337 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__337)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3080; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__337);
  __Pyx_GIVEREF(__pyx_tuple__337);

  /* "dipy/align/vector_fields.pyx":3081
 *         double[:] x = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:
 */
  __pyx_tuple__338 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__338)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3081; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__338);
  __Pyx_GIVEREF(__pyx_tuple__338);

  /* "dipy/align/vector_fields.pyx":3082
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 */
  __pyx_tuple__339 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__339)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3082; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__339);
  __Pyx_GIVEREF(__pyx_tuple__339);

  /* "dipy/align/vector_fields.pyx":3132
 * 
 * 
 * def _sparse_gradient_3d(floating[:, :, :] img,             # <<<<<<<<<<<<<<
 *                         double[:, :] img_world2grid,
 *                         double[:] img_spacing,
 */
  __pyx_tuple__340 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__340)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__340);
  __Pyx_GIVEREF(__pyx_tuple__340);
  __pyx_tuple__341 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__341)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__341);
  __Pyx_GIVEREF(__pyx_tuple__341);
  __pyx_tuple__342 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__342)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__342);
  __Pyx_GIVEREF(__pyx_tuple__342);
  __pyx_tuple__343 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__343)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__343);
  __Pyx_GIVEREF(__pyx_tuple__343);

  /* "dipy/align/vector_fields.pyx":3169
 *         int i, in_flag
 *         double tmp
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 */
  __pyx_tuple__344 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__344)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__344);
  __Pyx_GIVEREF(__pyx_tuple__344);

  /* "dipy/align/vector_fields.pyx":3170
 *         double tmp
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:
 */
  __pyx_tuple__345 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__345)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__345);
  __Pyx_GIVEREF(__pyx_tuple__345);

  /* "dipy/align/vector_fields.pyx":3171
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 */
  __pyx_tuple__346 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__346)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__346);
  __Pyx_GIVEREF(__pyx_tuple__346);

  /* "dipy/align/vector_fields.pyx":3169
 *         int i, in_flag
 *         double tmp
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 */
  __pyx_tuple__347 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__347)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3169; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__347);
  __Pyx_GIVEREF(__pyx_tuple__347);

  /* "dipy/align/vector_fields.pyx":3170
 *         double tmp
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)
 *     with nogil:
 */
  __pyx_tuple__348 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__348)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3170; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__348);
  __Pyx_GIVEREF(__pyx_tuple__348);

  /* "dipy/align/vector_fields.pyx":3171
 *         double[:] dx = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(3,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(3,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 */
  __pyx_tuple__349 = PyTuple_Pack(1, __pyx_int_3); if (unlikely(!__pyx_tuple__349)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3171; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__349);
  __Pyx_GIVEREF(__pyx_tuple__349);

  /* "dipy/align/vector_fields.pyx":3215
 * 
 * 
 * def _gradient_2d(floating[:, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                  double[:] img_spacing, double[:, :] out_grid2world,
 *                  floating[:, :, :] out, int[:, :] inside):
 */
  __pyx_tuple__350 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__350)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__350);
  __Pyx_GIVEREF(__pyx_tuple__350);
  __pyx_tuple__351 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__351)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__351);
  __Pyx_GIVEREF(__pyx_tuple__351);
  __pyx_tuple__352 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__352)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__352);
  __Pyx_GIVEREF(__pyx_tuple__352);
  __pyx_tuple__353 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__353)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__353);
  __Pyx_GIVEREF(__pyx_tuple__353);

  /* "dipy/align/vector_fields.pyx":3256
 *         int i, j, k, in_flag
 *         double tmp
 *         double[:] x = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 */
  __pyx_tuple__354 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__354)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__354);
  __Pyx_GIVEREF(__pyx_tuple__354);

  /* "dipy/align/vector_fields.pyx":3257
 *         double tmp
 *         double[:] x = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 */
  __pyx_tuple__355 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__355)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__355);
  __Pyx_GIVEREF(__pyx_tuple__355);

  /* "dipy/align/vector_fields.pyx":3258
 *         double[:] x = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:
 */
  __pyx_tuple__356 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__356)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__356);
  __Pyx_GIVEREF(__pyx_tuple__356);

  /* "dipy/align/vector_fields.pyx":3259
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 */
  __pyx_tuple__357 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__357)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__357);
  __Pyx_GIVEREF(__pyx_tuple__357);

  /* "dipy/align/vector_fields.pyx":3256
 *         int i, j, k, in_flag
 *         double tmp
 *         double[:] x = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 */
  __pyx_tuple__358 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__358)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3256; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__358);
  __Pyx_GIVEREF(__pyx_tuple__358);

  /* "dipy/align/vector_fields.pyx":3257
 *         double tmp
 *         double[:] x = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 */
  __pyx_tuple__359 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__359)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3257; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__359);
  __Pyx_GIVEREF(__pyx_tuple__359);

  /* "dipy/align/vector_fields.pyx":3258
 *         double[:] x = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:
 */
  __pyx_tuple__360 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__360)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3258; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__360);
  __Pyx_GIVEREF(__pyx_tuple__360);

  /* "dipy/align/vector_fields.pyx":3259
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 */
  __pyx_tuple__361 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__361)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__361);
  __Pyx_GIVEREF(__pyx_tuple__361);

  /* "dipy/align/vector_fields.pyx":3298
 * 
 * 
 * def _sparse_gradient_2d(floating[:, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                         double[:] img_spacing,
 *                         double[:, :] sample_points,
 */
  __pyx_tuple__362 = PyTuple_Pack(1, __pyx_kp_s_); if (unlikely(!__pyx_tuple__362)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__362);
  __Pyx_GIVEREF(__pyx_tuple__362);
  __pyx_tuple__363 = PyTuple_Pack(1, __pyx_kp_s__3); if (unlikely(!__pyx_tuple__363)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__363);
  __Pyx_GIVEREF(__pyx_tuple__363);
  __pyx_tuple__364 = PyTuple_Pack(1, __pyx_kp_s_No_matching_signature_found); if (unlikely(!__pyx_tuple__364)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__364);
  __Pyx_GIVEREF(__pyx_tuple__364);
  __pyx_tuple__365 = PyTuple_Pack(1, __pyx_kp_s_Function_call_with_ambiguous_arg); if (unlikely(!__pyx_tuple__365)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__365);
  __Pyx_GIVEREF(__pyx_tuple__365);

  /* "dipy/align/vector_fields.pyx":3336
 *         int i, in_flag
 *         double tmp
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 */
  __pyx_tuple__366 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__366)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__366);
  __Pyx_GIVEREF(__pyx_tuple__366);

  /* "dipy/align/vector_fields.pyx":3337
 *         double tmp
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:
 */
  __pyx_tuple__367 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__367)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__367);
  __Pyx_GIVEREF(__pyx_tuple__367);

  /* "dipy/align/vector_fields.pyx":3338
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 */
  __pyx_tuple__368 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__368)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__368);
  __Pyx_GIVEREF(__pyx_tuple__368);

  /* "dipy/align/vector_fields.pyx":3336
 *         int i, in_flag
 *         double tmp
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 */
  __pyx_tuple__369 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__369)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3336; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__369);
  __Pyx_GIVEREF(__pyx_tuple__369);

  /* "dipy/align/vector_fields.pyx":3337
 *         double tmp
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)
 *     with nogil:
 */
  __pyx_tuple__370 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__370)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3337; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__370);
  __Pyx_GIVEREF(__pyx_tuple__370);

  /* "dipy/align/vector_fields.pyx":3338
 *         double[:] dx = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] h = np.empty(shape=(2,), dtype=np.float64)
 *         double[:] q = np.empty(shape=(2,), dtype=np.float64)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         h[0] = 0.5 * img_spacing[0]
 */
  __pyx_tuple__371 = PyTuple_Pack(1, __pyx_int_2); if (unlikely(!__pyx_tuple__371)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__371);
  __Pyx_GIVEREF(__pyx_tuple__371);

  /* "dipy/align/vector_fields.pyx":3399
 *     dim = len(img.shape)
 *     if not is_valid_affine(img_world2grid, dim):
 *         raise ValueError("Invalid image affine transform")             # <<<<<<<<<<<<<<
 *     if not is_valid_affine(out_grid2world, dim):
 *         raise ValueError("Invalid sampling grid affine transform")
 */
  __pyx_tuple__372 = PyTuple_Pack(1, __pyx_kp_s_Invalid_image_affine_transform); if (unlikely(!__pyx_tuple__372)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3399; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__372);
  __Pyx_GIVEREF(__pyx_tuple__372);

  /* "dipy/align/vector_fields.pyx":3401
 *         raise ValueError("Invalid image affine transform")
 *     if not is_valid_affine(out_grid2world, dim):
 *         raise ValueError("Invalid sampling grid affine transform")             # <<<<<<<<<<<<<<
 *     if len(img_spacing) < dim:
 *         raise ValueError("Invalid spacings")
 */
  __pyx_tuple__373 = PyTuple_Pack(1, __pyx_kp_s_Invalid_sampling_grid_affine_tra); if (unlikely(!__pyx_tuple__373)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3401; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__373);
  __Pyx_GIVEREF(__pyx_tuple__373);

  /* "dipy/align/vector_fields.pyx":3403
 *         raise ValueError("Invalid sampling grid affine transform")
 *     if len(img_spacing) < dim:
 *         raise ValueError("Invalid spacings")             # <<<<<<<<<<<<<<
 *     ftype = img.dtype.type
 *     out = np.empty(tuple(out_shape)+(dim,), dtype=ftype)
 */
  __pyx_tuple__374 = PyTuple_Pack(1, __pyx_kp_s_Invalid_spacings); if (unlikely(!__pyx_tuple__374)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3403; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__374);
  __Pyx_GIVEREF(__pyx_tuple__374);

  /* "dipy/align/vector_fields.pyx":3443
 *     dim = len(img.shape)
 *     if not is_valid_affine(img_world2grid, dim):
 *         raise ValueError("Invalid affine transform matrix")             # <<<<<<<<<<<<<<
 *     if len(img_spacing) < dim:
 *         raise ValueError("Invalid spacings")
 */
  __pyx_tuple__375 = PyTuple_Pack(1, __pyx_kp_s_Invalid_affine_transform_matrix); if (unlikely(!__pyx_tuple__375)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3443; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__375);
  __Pyx_GIVEREF(__pyx_tuple__375);

  /* "dipy/align/vector_fields.pyx":3445
 *         raise ValueError("Invalid affine transform matrix")
 *     if len(img_spacing) < dim:
 *         raise ValueError("Invalid spacings")             # <<<<<<<<<<<<<<
 * 
 *     ftype = img.dtype.type
 */
  __pyx_tuple__376 = PyTuple_Pack(1, __pyx_kp_s_Invalid_spacings); if (unlikely(!__pyx_tuple__376)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3445; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__376);
  __Pyx_GIVEREF(__pyx_tuple__376);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":218
 *             if ((flags & pybuf.PyBUF_C_CONTIGUOUS == pybuf.PyBUF_C_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_C_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not C contiguous")             # <<<<<<<<<<<<<<
 * 
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 */
  __pyx_tuple__377 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_C_contiguous); if (unlikely(!__pyx_tuple__377)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 218; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__377);
  __Pyx_GIVEREF(__pyx_tuple__377);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":222
 *             if ((flags & pybuf.PyBUF_F_CONTIGUOUS == pybuf.PyBUF_F_CONTIGUOUS)
 *                 and not PyArray_CHKFLAGS(self, NPY_F_CONTIGUOUS)):
 *                 raise ValueError(u"ndarray is not Fortran contiguous")             # <<<<<<<<<<<<<<
 * 
 *             info.buf = PyArray_DATA(self)
 */
  __pyx_tuple__378 = PyTuple_Pack(1, __pyx_kp_u_ndarray_is_not_Fortran_contiguou); if (unlikely(!__pyx_tuple__378)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 222; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__378);
  __Pyx_GIVEREF(__pyx_tuple__378);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":259
 *                 if ((descr.byteorder == c'>' and little_endian) or
 *                     (descr.byteorder == c'<' and not little_endian)):
 *                     raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *                 if   t == NPY_BYTE:        f = "b"
 *                 elif t == NPY_UBYTE:       f = "B"
 */
  __pyx_tuple__379 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__379)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 259; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__379);
  __Pyx_GIVEREF(__pyx_tuple__379);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":799
 * 
 *         if (end - f) - <int>(new_offset - offset[0]) < 15:
 *             raise RuntimeError(u"Format string allocated too short, see comment in numpy.pxd")             # <<<<<<<<<<<<<<
 * 
 *         if ((child.byteorder == c'>' and little_endian) or
 */
  __pyx_tuple__380 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor); if (unlikely(!__pyx_tuple__380)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 799; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__380);
  __Pyx_GIVEREF(__pyx_tuple__380);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":803
 *         if ((child.byteorder == c'>' and little_endian) or
 *             (child.byteorder == c'<' and not little_endian)):
 *             raise ValueError(u"Non-native byte order not supported")             # <<<<<<<<<<<<<<
 *             # One could encode it in the format string and have Cython
 *             # complain instead, BUT: < and > in format strings also imply
 */
  __pyx_tuple__381 = PyTuple_Pack(1, __pyx_kp_u_Non_native_byte_order_not_suppor); if (unlikely(!__pyx_tuple__381)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 803; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__381);
  __Pyx_GIVEREF(__pyx_tuple__381);

  /* "../../../../../../usr/lib/python2.7/dist-packages/Cython/Includes/numpy/__init__.pxd":823
 *             t = child.type_num
 *             if end - f < 5:
 *                 raise RuntimeError(u"Format string allocated too short.")             # <<<<<<<<<<<<<<
 * 
 *             # Until ticket #99 is fixed, use integers to avoid warnings
 */
  __pyx_tuple__382 = PyTuple_Pack(1, __pyx_kp_u_Format_string_allocated_too_shor_2); if (unlikely(!__pyx_tuple__382)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 823; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__382);
  __Pyx_GIVEREF(__pyx_tuple__382);

  /* "View.MemoryView":129
 * 
 *         if not self.ndim:
 *             raise ValueError("Empty shape tuple for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if itemsize <= 0:
 */
  __pyx_tuple__383 = PyTuple_Pack(1, __pyx_kp_s_Empty_shape_tuple_for_cython_arr); if (unlikely(!__pyx_tuple__383)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 129; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__383);
  __Pyx_GIVEREF(__pyx_tuple__383);

  /* "View.MemoryView":132
 * 
 *         if itemsize <= 0:
 *             raise ValueError("itemsize <= 0 for cython.array")             # <<<<<<<<<<<<<<
 * 
 *         if not isinstance(format, bytes):
 */
  __pyx_tuple__384 = PyTuple_Pack(1, __pyx_kp_s_itemsize_0_for_cython_array); if (unlikely(!__pyx_tuple__384)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__384);
  __Pyx_GIVEREF(__pyx_tuple__384);

  /* "View.MemoryView":135
 * 
 *         if not isinstance(format, bytes):
 *             format = format.encode('ASCII')             # <<<<<<<<<<<<<<
 *         self._format = format  # keep a reference to the byte string
 *         self.format = self._format
 */
  __pyx_tuple__385 = PyTuple_Pack(1, __pyx_n_s_ASCII); if (unlikely(!__pyx_tuple__385)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__385);
  __Pyx_GIVEREF(__pyx_tuple__385);

  /* "View.MemoryView":144
 * 
 *         if not self._shape:
 *             raise MemoryError("unable to allocate shape and strides.")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__386 = PyTuple_Pack(1, __pyx_kp_s_unable_to_allocate_shape_and_str); if (unlikely(!__pyx_tuple__386)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 144; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__386);
  __Pyx_GIVEREF(__pyx_tuple__386);

  /* "View.MemoryView":172
 *             self.data = <char *>malloc(self.len)
 *             if not self.data:
 *                 raise MemoryError("unable to allocate array data.")             # <<<<<<<<<<<<<<
 * 
 *             if self.dtype_is_object:
 */
  __pyx_tuple__387 = PyTuple_Pack(1, __pyx_kp_s_unable_to_allocate_array_data); if (unlikely(!__pyx_tuple__387)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__387);
  __Pyx_GIVEREF(__pyx_tuple__387);

  /* "View.MemoryView":188
 *             bufmode = PyBUF_F_CONTIGUOUS | PyBUF_ANY_CONTIGUOUS
 *         if not (flags & bufmode):
 *             raise ValueError("Can only create a buffer that is contiguous in memory.")             # <<<<<<<<<<<<<<
 *         info.buf = self.data
 *         info.len = self.len
 */
  __pyx_tuple__388 = PyTuple_Pack(1, __pyx_kp_s_Can_only_create_a_buffer_that_is); if (unlikely(!__pyx_tuple__388)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 188; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__388);
  __Pyx_GIVEREF(__pyx_tuple__388);

  /* "View.MemoryView":447
 *             result = struct.unpack(self.view.format, bytesitem)
 *         except struct.error:
 *             raise ValueError("Unable to convert item to object")             # <<<<<<<<<<<<<<
 *         else:
 *             if len(self.view.format) == 1:
 */
  __pyx_tuple__389 = PyTuple_Pack(1, __pyx_kp_s_Unable_to_convert_item_to_object); if (unlikely(!__pyx_tuple__389)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 447; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__389);
  __Pyx_GIVEREF(__pyx_tuple__389);

  /* "View.MemoryView":523
 *             if self.view.strides == NULL:
 * 
 *                 raise ValueError("Buffer view does not expose strides")             # <<<<<<<<<<<<<<
 * 
 *             return tuple([stride for stride in self.view.strides[:self.view.ndim]])
 */
  __pyx_tuple__390 = PyTuple_Pack(1, __pyx_kp_s_Buffer_view_does_not_expose_stri); if (unlikely(!__pyx_tuple__390)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 523; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__390);
  __Pyx_GIVEREF(__pyx_tuple__390);

  /* "View.MemoryView":531
 *         def __get__(self):
 *             if self.view.suboffsets == NULL:
 *                 return (-1,) * self.view.ndim             # <<<<<<<<<<<<<<
 * 
 *             return tuple([suboffset for suboffset in self.view.suboffsets[:self.view.ndim]])
 */
  __pyx_tuple__391 = PyTuple_New(1); if (unlikely(!__pyx_tuple__391)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__391);
  __Pyx_INCREF(__pyx_int_neg_1);
  __Pyx_GIVEREF(__pyx_int_neg_1);
  PyTuple_SET_ITEM(__pyx_tuple__391, 0, __pyx_int_neg_1);
  __Pyx_GIVEREF(__pyx_tuple__391);

  /* "View.MemoryView":640
 *         if item is Ellipsis:
 *             if not seen_ellipsis:
 *                 result.extend([slice(None)] * (ndim - len(tup) + 1))             # <<<<<<<<<<<<<<
 *                 seen_ellipsis = True
 *             else:
 */
  __pyx_slice__392 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__392)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 640; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_slice__392);
  __Pyx_GIVEREF(__pyx_slice__392);

  /* "View.MemoryView":643
 *                 seen_ellipsis = True
 *             else:
 *                 result.append(slice(None))             # <<<<<<<<<<<<<<
 *             have_slices = True
 *         else:
 */
  __pyx_slice__393 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__393)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 643; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_slice__393);
  __Pyx_GIVEREF(__pyx_slice__393);

  /* "View.MemoryView":654
 *     nslices = ndim - len(result)
 *     if nslices:
 *         result.extend([slice(None)] * nslices)             # <<<<<<<<<<<<<<
 * 
 *     return have_slices or nslices, tuple(result)
 */
  __pyx_slice__394 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__394)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 654; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_slice__394);
  __Pyx_GIVEREF(__pyx_slice__394);

  /* "View.MemoryView":661
 *     for suboffset in suboffsets[:ndim]:
 *         if suboffset >= 0:
 *             raise ValueError("Indirect dimensions not supported")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__395 = PyTuple_Pack(1, __pyx_kp_s_Indirect_dimensions_not_supporte); if (unlikely(!__pyx_tuple__395)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 661; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__395);
  __Pyx_GIVEREF(__pyx_tuple__395);

  /* "dipy/align/vector_fields.pyx":15
 * 
 * 
 * def is_valid_affine(double[:, :] M, int dim):             # <<<<<<<<<<<<<<
 *     if M is None:
 *         return True
 */
  __pyx_tuple__396 = PyTuple_Pack(2, __pyx_n_s_M, __pyx_n_s_dim); if (unlikely(!__pyx_tuple__396)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 15; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__396);
  __Pyx_GIVEREF(__pyx_tuple__396);
  __pyx_codeobj__397 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__396, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_is_valid_affine, 15, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__397)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 15; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":25
 * 
 * 
 * def interpolate_vector_2d(floating[:, :, :] field, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Bilinear interpolation of a 2D vector field
 * 
 */
  __pyx_tuple__398 = PyTuple_Pack(7, __pyx_n_s_field, __pyx_n_s_locations, __pyx_n_s_ftype, __pyx_n_s_n, __pyx_n_s_out, __pyx_n_s_inside, __pyx_n_s_i); if (unlikely(!__pyx_tuple__398)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__398);
  __Pyx_GIVEREF(__pyx_tuple__398);
  __pyx_codeobj__399 = (PyObject*)__Pyx_PyCode_New(2, 0, 7, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__398, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_interpolate_vector_2d, 25, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__399)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":135
 * 
 * 
 * def interpolate_scalar_2d(floating[:, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Bilinear interpolation of a 2D scalar image
 * 
 */
  __pyx_tuple__400 = PyTuple_Pack(7, __pyx_n_s_image, __pyx_n_s_locations, __pyx_n_s_ftype, __pyx_n_s_n, __pyx_n_s_out, __pyx_n_s_inside, __pyx_n_s_i); if (unlikely(!__pyx_tuple__400)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__400);
  __Pyx_GIVEREF(__pyx_tuple__400);
  __pyx_codeobj__401 = (PyObject*)__Pyx_PyCode_New(2, 0, 7, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__400, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_interpolate_scalar_2d, 135, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__401)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":239
 * 
 * 
 * def interpolate_scalar_nn_2d(number[:, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 2D scalar image
 * 
 */
  __pyx_tuple__402 = PyTuple_Pack(7, __pyx_n_s_image, __pyx_n_s_locations, __pyx_n_s_ftype, __pyx_n_s_n, __pyx_n_s_out, __pyx_n_s_inside, __pyx_n_s_i); if (unlikely(!__pyx_tuple__402)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__402);
  __Pyx_GIVEREF(__pyx_tuple__402);
  __pyx_codeobj__403 = (PyObject*)__Pyx_PyCode_New(2, 0, 7, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__402, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_interpolate_scalar_nn_2d, 239, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__403)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":332
 * 
 * 
 * def interpolate_scalar_nn_3d(number[:, :, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 3D scalar image
 * 
 */
  __pyx_tuple__404 = PyTuple_Pack(7, __pyx_n_s_image, __pyx_n_s_locations, __pyx_n_s_ftype, __pyx_n_s_n, __pyx_n_s_out, __pyx_n_s_inside, __pyx_n_s_i); if (unlikely(!__pyx_tuple__404)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__404);
  __Pyx_GIVEREF(__pyx_tuple__404);
  __pyx_codeobj__405 = (PyObject*)__Pyx_PyCode_New(2, 0, 7, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__404, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_interpolate_scalar_nn_3d, 332, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__405)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":434
 * 
 * 
 * def interpolate_scalar_3d(floating[:, :, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Trilinear interpolation of a 3D scalar image
 * 
 */
  __pyx_tuple__406 = PyTuple_Pack(7, __pyx_n_s_image, __pyx_n_s_locations, __pyx_n_s_ftype, __pyx_n_s_n, __pyx_n_s_out, __pyx_n_s_inside, __pyx_n_s_i); if (unlikely(!__pyx_tuple__406)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__406);
  __Pyx_GIVEREF(__pyx_tuple__406);
  __pyx_codeobj__407 = (PyObject*)__Pyx_PyCode_New(2, 0, 7, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__406, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_interpolate_scalar_3d, 434, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__407)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":567
 * 
 * 
 * def interpolate_vector_3d(floating[:, :, :, :] field, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Trilinear interpolation of a 3D vector field
 * 
 */
  __pyx_tuple__408 = PyTuple_Pack(7, __pyx_n_s_field, __pyx_n_s_locations, __pyx_n_s_ftype, __pyx_n_s_n, __pyx_n_s_out, __pyx_n_s_inside, __pyx_n_s_i); if (unlikely(!__pyx_tuple__408)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__408);
  __Pyx_GIVEREF(__pyx_tuple__408);
  __pyx_codeobj__409 = (PyObject*)__Pyx_PyCode_New(2, 0, 7, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__408, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_interpolate_vector_3d, 567, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__409)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":847
 * 
 * 
 * def compose_vector_fields_2d(floating[:, :, :] d1, floating[:, :, :] d2,             # <<<<<<<<<<<<<<
 *                              double[:, :] premult_index,
 *                              double[:, :] premult_disp,
 */
  __pyx_tuple__410 = PyTuple_Pack(7, __pyx_n_s_d1, __pyx_n_s_d2, __pyx_n_s_premult_index, __pyx_n_s_premult_disp, __pyx_n_s_time_scaling, __pyx_n_s_comp, __pyx_n_s_stats); if (unlikely(!__pyx_tuple__410)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__410);
  __Pyx_GIVEREF(__pyx_tuple__410);
  __pyx_codeobj__411 = (PyObject*)__Pyx_PyCode_New(6, 0, 7, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__410, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_compose_vector_fields_2d, 847, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__411)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1054
 * 
 * 
 * def compose_vector_fields_3d(floating[:, :, :, :] d1, floating[:, :, :, :] d2,             # <<<<<<<<<<<<<<
 *                              double[:, :] premult_index,
 *                              double[:, :] premult_disp,
 */
  __pyx_tuple__412 = PyTuple_Pack(7, __pyx_n_s_d1, __pyx_n_s_d2, __pyx_n_s_premult_index, __pyx_n_s_premult_disp, __pyx_n_s_time_scaling, __pyx_n_s_comp, __pyx_n_s_stats); if (unlikely(!__pyx_tuple__412)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__412);
  __Pyx_GIVEREF(__pyx_tuple__412);
  __pyx_codeobj__413 = (PyObject*)__Pyx_PyCode_New(6, 0, 7, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__412, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_compose_vector_fields_3d, 1054, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__413)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1127
 * 
 * 
 * def invert_vector_field_fixed_point_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                                        double[:, :] d_world2grid,
 *                                        double[:] spacing,
 */
  __pyx_tuple__414 = PyTuple_Pack(31, __pyx_n_s_d, __pyx_n_s_d_world2grid, __pyx_n_s_spacing, __pyx_n_s_max_iter, __pyx_n_s_tolerance, __pyx_n_s_start, __pyx_n_s_nr, __pyx_n_s_nc, __pyx_n_s_iter_count, __pyx_n_s_current, __pyx_n_s_flag, __pyx_n_s_difmag, __pyx_n_s_mag, __pyx_n_s_maxlen, __pyx_n_s_step_factor, __pyx_n_s_epsilon, __pyx_n_s_error, __pyx_n_s_di, __pyx_n_s_dj, __pyx_n_s_dii, __pyx_n_s_djj, __pyx_n_s_sr, __pyx_n_s_sc, __pyx_n_s_ftype, __pyx_n_s_stats, __pyx_n_s_substats, __pyx_n_s_norms, __pyx_n_s_p, __pyx_n_s_q, __pyx_n_s_i, __pyx_n_s_j); if (unlikely(!__pyx_tuple__414)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__414);
  __Pyx_GIVEREF(__pyx_tuple__414);
  __pyx_codeobj__415 = (PyObject*)__Pyx_PyCode_New(6, 0, 31, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__414, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_invert_vector_field_fixed_point, 1127, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__415)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1231
 * 
 * 
 * def invert_vector_field_fixed_point_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                                        double[:, :] d_world2grid,
 *                                        double[:] spacing,
 */
  __pyx_tuple__416 = PyTuple_Pack(35, __pyx_n_s_d, __pyx_n_s_d_world2grid, __pyx_n_s_spacing, __pyx_n_s_max_iter, __pyx_n_s_tol, __pyx_n_s_start, __pyx_n_s_ns, __pyx_n_s_nr, __pyx_n_s_nc, __pyx_n_s_iter_count, __pyx_n_s_current, __pyx_n_s_dkk, __pyx_n_s_dii, __pyx_n_s_djj, __pyx_n_s_dk, __pyx_n_s_di, __pyx_n_s_dj, __pyx_n_s_difmag, __pyx_n_s_mag, __pyx_n_s_maxlen, __pyx_n_s_step_factor, __pyx_n_s_epsilon, __pyx_n_s_error, __pyx_n_s_ss, __pyx_n_s_sr, __pyx_n_s_sc, __pyx_n_s_ftype, __pyx_n_s_stats, __pyx_n_s_substats, __pyx_n_s_norms, __pyx_n_s_p, __pyx_n_s_q, __pyx_n_s_k, __pyx_n_s_i, __pyx_n_s_j); if (unlikely(!__pyx_tuple__416)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__416);
  __Pyx_GIVEREF(__pyx_tuple__416);
  __pyx_codeobj__417 = (PyObject*)__Pyx_PyCode_New(6, 0, 35, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__416, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_invert_vector_field_fixed_point_2, 1231, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__417)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1345
 * 
 * 
 * def simplify_warp_function_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                               double[:, :] affine_idx_in,
 *                               double[:, :] affine_idx_out,
 */
  __pyx_tuple__418 = PyTuple_Pack(15, __pyx_n_s_d, __pyx_n_s_affine_idx_in, __pyx_n_s_affine_idx_out, __pyx_n_s_affine_disp, __pyx_n_s_out_shape, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_di, __pyx_n_s_dj, __pyx_n_s_dii, __pyx_n_s_djj, __pyx_n_s_tmp, __pyx_n_s_out); if (unlikely(!__pyx_tuple__418)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__418);
  __Pyx_GIVEREF(__pyx_tuple__418);
  __pyx_codeobj__419 = (PyObject*)__Pyx_PyCode_New(5, 0, 15, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__418, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_simplify_warp_function_2d, 1345, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__419)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1461
 * 
 * 
 * def simplify_warp_function_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                               double[:, :] affine_idx_in,
 *                               double[:, :] affine_idx_out,
 */
  __pyx_tuple__420 = PyTuple_Pack(20, __pyx_n_s_d, __pyx_n_s_affine_idx_in, __pyx_n_s_affine_idx_out, __pyx_n_s_affine_disp, __pyx_n_s_out_shape, __pyx_n_s_nslices, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_k, __pyx_n_s_inside, __pyx_n_s_di, __pyx_n_s_dj, __pyx_n_s_dk, __pyx_n_s_dii, __pyx_n_s_djj, __pyx_n_s_dkk, __pyx_n_s_tmp, __pyx_n_s_out); if (unlikely(!__pyx_tuple__420)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__420);
  __Pyx_GIVEREF(__pyx_tuple__420);
  __pyx_codeobj__421 = (PyObject*)__Pyx_PyCode_New(5, 0, 20, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__420, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_simplify_warp_function_3d, 1461, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__421)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1588
 * 
 * 
 * def reorient_vector_field_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                              double[:, :] affine):
 *     r"""Linearly transforms all vectors of a 2D displacement field
 */
  __pyx_tuple__422 = PyTuple_Pack(8, __pyx_n_s_d, __pyx_n_s_affine, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_di, __pyx_n_s_dj); if (unlikely(!__pyx_tuple__422)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__422);
  __Pyx_GIVEREF(__pyx_tuple__422);
  __pyx_codeobj__423 = (PyObject*)__Pyx_PyCode_New(2, 0, 8, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__422, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_reorient_vector_field_2d, 1588, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__423)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1626
 * 
 * 
 * def reorient_vector_field_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                              double[:, :] affine):
 *     r"""Linearly transforms all vectors of a 3D displacement field
 */
  __pyx_tuple__424 = PyTuple_Pack(11, __pyx_n_s_d, __pyx_n_s_affine, __pyx_n_s_nslices, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_k, __pyx_n_s_di, __pyx_n_s_dj, __pyx_n_s_dk); if (unlikely(!__pyx_tuple__424)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__424);
  __Pyx_GIVEREF(__pyx_tuple__424);
  __pyx_codeobj__425 = (PyObject*)__Pyx_PyCode_New(2, 0, 11, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__424, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_reorient_vector_field_3d, 1626, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__425)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1668
 * 
 * 
 * def downsample_scalar_field_3d(floating[:, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input volume by a factor of 2
 * 
 */
  __pyx_tuple__426 = PyTuple_Pack(17, __pyx_n_s_field, __pyx_n_s_field, __pyx_n_s_ftype, __pyx_n_s_ns, __pyx_n_s_nr, __pyx_n_s_nc, __pyx_n_s_nns, __pyx_n_s_nnr, __pyx_n_s_nnc, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_k, __pyx_n_s_ii, __pyx_n_s_jj, __pyx_n_s_kk, __pyx_n_s_down, __pyx_n_s_cnt); if (unlikely(!__pyx_tuple__426)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__426);
  __Pyx_GIVEREF(__pyx_tuple__426);
  __pyx_codeobj__427 = (PyObject*)__Pyx_PyCode_New(1, 0, 17, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__426, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_downsample_scalar_field_3d, 1668, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__427)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1715
 * 
 * 
 * def downsample_displacement_field_3d(floating[:, :, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input 3D vector field by a factor of 2
 * 
 */
  __pyx_tuple__428 = PyTuple_Pack(17, __pyx_n_s_field, __pyx_n_s_field, __pyx_n_s_ftype, __pyx_n_s_ns, __pyx_n_s_nr, __pyx_n_s_nc, __pyx_n_s_nns, __pyx_n_s_nnr, __pyx_n_s_nnc, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_k, __pyx_n_s_ii, __pyx_n_s_jj, __pyx_n_s_kk, __pyx_n_s_down, __pyx_n_s_cnt); if (unlikely(!__pyx_tuple__428)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__428);
  __Pyx_GIVEREF(__pyx_tuple__428);
  __pyx_codeobj__429 = (PyObject*)__Pyx_PyCode_New(1, 0, 17, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__428, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_downsample_displacement_field_3d, 1715, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__429)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1767
 * 
 * 
 * def downsample_scalar_field_2d(floating[:, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input 2D image by a factor of 2
 * 
 */
  __pyx_tuple__430 = PyTuple_Pack(13, __pyx_n_s_field, __pyx_n_s_field, __pyx_n_s_ftype, __pyx_n_s_nr, __pyx_n_s_nc, __pyx_n_s_nnr, __pyx_n_s_nnc, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_ii, __pyx_n_s_jj, __pyx_n_s_down, __pyx_n_s_cnt); if (unlikely(!__pyx_tuple__430)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__430);
  __Pyx_GIVEREF(__pyx_tuple__430);
  __pyx_codeobj__431 = (PyObject*)__Pyx_PyCode_New(1, 0, 13, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__430, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_downsample_scalar_field_2d, 1767, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__431)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1808
 * 
 * 
 * def downsample_displacement_field_2d(floating[:, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the 2D input vector field by a factor of 2
 *     Down-samples the input vector field by a factor of 2. The value at each
 */
  __pyx_tuple__432 = PyTuple_Pack(13, __pyx_n_s_field, __pyx_n_s_field, __pyx_n_s_ftype, __pyx_n_s_nr, __pyx_n_s_nc, __pyx_n_s_nnr, __pyx_n_s_nnc, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_ii, __pyx_n_s_jj, __pyx_n_s_down, __pyx_n_s_cnt); if (unlikely(!__pyx_tuple__432)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__432);
  __Pyx_GIVEREF(__pyx_tuple__432);
  __pyx_codeobj__433 = (PyObject*)__Pyx_PyCode_New(1, 0, 13, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__432, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_downsample_displacement_field_2d, 1808, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__433)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1851
 * 
 * 
 * def warp_3d(floating[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 */
  __pyx_tuple__434 = PyTuple_Pack(24, __pyx_n_s_volume, __pyx_n_s_d1, __pyx_n_s_affine_idx_in, __pyx_n_s_affine_idx_out, __pyx_n_s_affine_disp, __pyx_n_s_out_shape, __pyx_n_s_nslices, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_nsVol, __pyx_n_s_nrVol, __pyx_n_s_ncVol, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_k, __pyx_n_s_inside, __pyx_n_s_dkk, __pyx_n_s_dii, __pyx_n_s_djj, __pyx_n_s_dk, __pyx_n_s_di, __pyx_n_s_dj, __pyx_n_s_warped, __pyx_n_s_tmp); if (unlikely(!__pyx_tuple__434)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__434);
  __Pyx_GIVEREF(__pyx_tuple__434);
  __pyx_codeobj__435 = (PyObject*)__Pyx_PyCode_New(6, 0, 24, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__434, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_warp_3d, 1851, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__435)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1982
 * 
 * 
 * def transform_3d_affine(floating[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                         double[:, :] affine):
 *     r"""Transforms a 3D volume by an affine transform with trilinear interp.
 */
  __pyx_tuple__436 = PyTuple_Pack(28, __pyx_n_s_volume, __pyx_n_s_ref_shape, __pyx_n_s_affine, __pyx_n_s_nslices, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_nsVol, __pyx_n_s_nrVol, __pyx_n_s_ncVol, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_k, __pyx_n_s_ii, __pyx_n_s_jj, __pyx_n_s_kk, __pyx_n_s_inside, __pyx_n_s_dkk, __pyx_n_s_dii, __pyx_n_s_djj, __pyx_n_s_tmp0, __pyx_n_s_tmp1, __pyx_n_s_alpha, __pyx_n_s_beta, __pyx_n_s_gamma, __pyx_n_s_calpha, __pyx_n_s_cbeta, __pyx_n_s_cgamma, __pyx_n_s_out); if (unlikely(!__pyx_tuple__436)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__436);
  __Pyx_GIVEREF(__pyx_tuple__436);
  __pyx_codeobj__437 = (PyObject*)__Pyx_PyCode_New(3, 0, 28, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__436, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_transform_3d_affine, 1982, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__437)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_tuple__438 = PyTuple_Pack(24, __pyx_n_s_volume, __pyx_n_s_d1, __pyx_n_s_affine_idx_in, __pyx_n_s_affine_idx_out, __pyx_n_s_affine_disp, __pyx_n_s_out_shape, __pyx_n_s_nslices, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_nsVol, __pyx_n_s_nrVol, __pyx_n_s_ncVol, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_k, __pyx_n_s_inside, __pyx_n_s_dkk, __pyx_n_s_dii, __pyx_n_s_djj, __pyx_n_s_dk, __pyx_n_s_di, __pyx_n_s_dj, __pyx_n_s_warped, __pyx_n_s_tmp); if (unlikely(!__pyx_tuple__438)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__438);
  __Pyx_GIVEREF(__pyx_tuple__438);
  __pyx_codeobj__439 = (PyObject*)__Pyx_PyCode_New(6, 0, 24, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__438, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_warp_3d_nn, 2048, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__439)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */
  __pyx_tuple__440 = PyTuple_Pack(27, __pyx_n_s_volume, __pyx_n_s_ref_shape, __pyx_n_s_affine, __pyx_n_s_nslices, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_nsVol, __pyx_n_s_nrVol, __pyx_n_s_ncVol, __pyx_n_s_dkk, __pyx_n_s_dii, __pyx_n_s_djj, __pyx_n_s_tmp0, __pyx_n_s_tmp1, __pyx_n_s_alpha, __pyx_n_s_beta, __pyx_n_s_gamma, __pyx_n_s_calpha, __pyx_n_s_cbeta, __pyx_n_s_cgamma, __pyx_n_s_k, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_kk, __pyx_n_s_ii, __pyx_n_s_jj, __pyx_n_s_out); if (unlikely(!__pyx_tuple__440)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__440);
  __Pyx_GIVEREF(__pyx_tuple__440);
  __pyx_codeobj__441 = (PyObject*)__Pyx_PyCode_New(3, 0, 27, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__440, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_transform_3d_affine_nn, 2178, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__441)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2243
 * 
 * 
 * def warp_2d(floating[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 */
  __pyx_tuple__442 = PyTuple_Pack(20, __pyx_n_s_image, __pyx_n_s_d1, __pyx_n_s_affine_idx_in, __pyx_n_s_affine_idx_out, __pyx_n_s_affine_disp, __pyx_n_s_out_shape, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_nrVol, __pyx_n_s_ncVol, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_ii, __pyx_n_s_jj, __pyx_n_s_di, __pyx_n_s_dj, __pyx_n_s_dii, __pyx_n_s_djj, __pyx_n_s_warped, __pyx_n_s_tmp); if (unlikely(!__pyx_tuple__442)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__442);
  __Pyx_GIVEREF(__pyx_tuple__442);
  __pyx_codeobj__443 = (PyObject*)__Pyx_PyCode_New(6, 0, 20, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__442, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_warp_2d, 2243, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__443)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2357
 * 
 * 
 * def transform_2d_affine(floating[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                         double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with bilinear interp.
 */
  __pyx_tuple__444 = PyTuple_Pack(19, __pyx_n_s_image, __pyx_n_s_ref_shape, __pyx_n_s_affine, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_nrVol, __pyx_n_s_ncVol, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_ii, __pyx_n_s_jj, __pyx_n_s_dii, __pyx_n_s_djj, __pyx_n_s_tmp0, __pyx_n_s_alpha, __pyx_n_s_beta, __pyx_n_s_calpha, __pyx_n_s_cbeta, __pyx_n_s_out); if (unlikely(!__pyx_tuple__444)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__444);
  __Pyx_GIVEREF(__pyx_tuple__444);
  __pyx_codeobj__445 = (PyObject*)__Pyx_PyCode_New(3, 0, 19, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__444, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_transform_2d_affine, 2357, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__445)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_tuple__446 = PyTuple_Pack(20, __pyx_n_s_image, __pyx_n_s_d1, __pyx_n_s_affine_idx_in, __pyx_n_s_affine_idx_out, __pyx_n_s_affine_disp, __pyx_n_s_out_shape, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_nrVol, __pyx_n_s_ncVol, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_ii, __pyx_n_s_jj, __pyx_n_s_di, __pyx_n_s_dj, __pyx_n_s_dii, __pyx_n_s_djj, __pyx_n_s_warped, __pyx_n_s_tmp); if (unlikely(!__pyx_tuple__446)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__446);
  __Pyx_GIVEREF(__pyx_tuple__446);
  __pyx_codeobj__447 = (PyObject*)__Pyx_PyCode_New(6, 0, 20, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__446, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_warp_2d_nn, 2417, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__447)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */
  __pyx_tuple__448 = PyTuple_Pack(19, __pyx_n_s_image, __pyx_n_s_ref_shape, __pyx_n_s_affine, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_nrVol, __pyx_n_s_ncVol, __pyx_n_s_dii, __pyx_n_s_djj, __pyx_n_s_tmp0, __pyx_n_s_alpha, __pyx_n_s_beta, __pyx_n_s_calpha, __pyx_n_s_cbeta, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_ii, __pyx_n_s_jj, __pyx_n_s_out); if (unlikely(!__pyx_tuple__448)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__448);
  __Pyx_GIVEREF(__pyx_tuple__448);
  __pyx_codeobj__449 = (PyObject*)__Pyx_PyCode_New(3, 0, 19, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__448, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_transform_2d_affine_nn, 2531, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__449)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2591
 * 
 * 
 * def resample_displacement_field_3d(floating[:, :, :, :] field,             # <<<<<<<<<<<<<<
 *                                    double[:] factors, int[:] out_shape):
 *     r"""Resamples a 3D vector field to a custom target shape
 */
  __pyx_tuple__450 = PyTuple_Pack(15, __pyx_n_s_field, __pyx_n_s_factors, __pyx_n_s_out_shape, __pyx_n_s_ftype, __pyx_n_s_tslices, __pyx_n_s_trows, __pyx_n_s_tcols, __pyx_n_s_k, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_inside, __pyx_n_s_dkk, __pyx_n_s_dii, __pyx_n_s_djj, __pyx_n_s_expanded); if (unlikely(!__pyx_tuple__450)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__450);
  __Pyx_GIVEREF(__pyx_tuple__450);
  __pyx_codeobj__451 = (PyObject*)__Pyx_PyCode_New(3, 0, 15, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__450, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_resample_displacement_field_3d, 2591, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__451)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2636
 * 
 * 
 * def resample_displacement_field_2d(floating[:, :, :] field, double[:] factors,             # <<<<<<<<<<<<<<
 *                                    int[:] out_shape):
 *     r"""Resamples a 2D vector field to a custom target shape
 */
  __pyx_tuple__452 = PyTuple_Pack(12, __pyx_n_s_field, __pyx_n_s_factors, __pyx_n_s_out_shape, __pyx_n_s_ftype, __pyx_n_s_trows, __pyx_n_s_tcols, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_inside, __pyx_n_s_dii, __pyx_n_s_djj, __pyx_n_s_expanded); if (unlikely(!__pyx_tuple__452)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__452);
  __Pyx_GIVEREF(__pyx_tuple__452);
  __pyx_codeobj__453 = (PyObject*)__Pyx_PyCode_New(3, 0, 12, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__452, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_resample_displacement_field_2d, 2636, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__453)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2677
 * 
 * 
 * def create_random_displacement_2d(int[:] from_shape,             # <<<<<<<<<<<<<<
 *                                   double[:, :] from_grid2world,
 *                                   int[:] to_shape,
 */
  __pyx_tuple__454 = PyTuple_Pack(15, __pyx_n_s_from_shape, __pyx_n_s_from_grid2world, __pyx_n_s_to_shape, __pyx_n_s_to_grid2world, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_ri, __pyx_n_s_rj, __pyx_n_s_di, __pyx_n_s_dj, __pyx_n_s_dii, __pyx_n_s_djj, __pyx_n_s_int_field, __pyx_n_s_output, __pyx_n_s_dom_size); if (unlikely(!__pyx_tuple__454)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2677; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__454);
  __Pyx_GIVEREF(__pyx_tuple__454);
  __pyx_codeobj__455 = (PyObject*)__Pyx_PyCode_New(4, 0, 15, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__454, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_create_random_displacement_2d, 2677, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__455)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2677; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2759
 * 
 * 
 * def create_random_displacement_3d(int[:] from_shape, double[:, :] from_grid2world,             # <<<<<<<<<<<<<<
 *                                   int[:] to_shape, double[:, :] to_grid2world):
 *     r"""Creates a random 3D displacement 'exactly' mapping points of two grids
 */
  __pyx_tuple__456 = PyTuple_Pack(19, __pyx_n_s_from_shape, __pyx_n_s_from_grid2world, __pyx_n_s_to_shape, __pyx_n_s_to_grid2world, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_k, __pyx_n_s_ri, __pyx_n_s_rj, __pyx_n_s_rk, __pyx_n_s_di, __pyx_n_s_dj, __pyx_n_s_dii, __pyx_n_s_djj, __pyx_n_s_int_field, __pyx_n_s_output, __pyx_n_s_dom_size, __pyx_n_s_dk, __pyx_n_s_dkk); if (unlikely(!__pyx_tuple__456)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2759; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__456);
  __Pyx_GIVEREF(__pyx_tuple__456);
  __pyx_codeobj__457 = (PyObject*)__Pyx_PyCode_New(4, 0, 19, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__456, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_create_random_displacement_3d, 2759, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__457)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2759; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2846
 * 
 * 
 * def create_harmonic_fields_2d(cnp.npy_intp nrows, cnp.npy_intp ncols,             # <<<<<<<<<<<<<<
 *                               double b, double m):
 *     r"""Creates an invertible 2D displacement field
 */
  __pyx_tuple__458 = PyTuple_Pack(13, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_b, __pyx_n_s_m, __pyx_n_s_mid_row, __pyx_n_s_mid_col, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_ii, __pyx_n_s_jj, __pyx_n_s_theta, __pyx_n_s_d, __pyx_n_s_inv); if (unlikely(!__pyx_tuple__458)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2846; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__458);
  __Pyx_GIVEREF(__pyx_tuple__458);
  __pyx_codeobj__459 = (PyObject*)__Pyx_PyCode_New(4, 0, 13, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__458, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_create_harmonic_fields_2d, 2846, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__459)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2846; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2896
 * 
 * 
 * def create_harmonic_fields_3d(int nslices, cnp.npy_intp nrows,             # <<<<<<<<<<<<<<
 *                               cnp.npy_intp ncols, double b, double m):
 *     r"""Creates an invertible 3D displacement field
 */
  __pyx_tuple__460 = PyTuple_Pack(17, __pyx_n_s_nslices, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_b, __pyx_n_s_m, __pyx_n_s_mid_slice, __pyx_n_s_mid_row, __pyx_n_s_mid_col, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_k, __pyx_n_s_ii, __pyx_n_s_jj, __pyx_n_s_kk, __pyx_n_s_theta, __pyx_n_s_d, __pyx_n_s_inv); if (unlikely(!__pyx_tuple__460)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2896; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__460);
  __Pyx_GIVEREF(__pyx_tuple__460);
  __pyx_codeobj__461 = (PyObject*)__Pyx_PyCode_New(5, 0, 17, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__460, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_create_harmonic_fields_3d, 2896, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__461)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2896; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2955
 * 
 * 
 * def create_circle(cnp.npy_intp nrows, cnp.npy_intp ncols, cnp.npy_intp radius):             # <<<<<<<<<<<<<<
 *     r"""
 *     Create a binary 2D image where pixel values are 1 iff their distance
 */
  __pyx_tuple__462 = PyTuple_Pack(11, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_radius, __pyx_n_s_mid_row, __pyx_n_s_mid_col, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_ii, __pyx_n_s_jj, __pyx_n_s_r, __pyx_n_s_c); if (unlikely(!__pyx_tuple__462)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2955; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__462);
  __Pyx_GIVEREF(__pyx_tuple__462);
  __pyx_codeobj__463 = (PyObject*)__Pyx_PyCode_New(3, 0, 11, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__462, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_create_circle, 2955, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__463)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2955; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2992
 * 
 * 
 * def create_sphere(cnp.npy_intp nslices, cnp.npy_intp nrows,             # <<<<<<<<<<<<<<
 *                   cnp.npy_intp ncols, cnp.npy_intp radius):
 *     r"""
 */
  __pyx_tuple__464 = PyTuple_Pack(15, __pyx_n_s_nslices, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_radius, __pyx_n_s_mid_slice, __pyx_n_s_mid_row, __pyx_n_s_mid_col, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_k, __pyx_n_s_ii, __pyx_n_s_jj, __pyx_n_s_kk, __pyx_n_s_r, __pyx_n_s_s); if (unlikely(!__pyx_tuple__464)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2992; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__464);
  __Pyx_GIVEREF(__pyx_tuple__464);
  __pyx_codeobj__465 = (PyObject*)__Pyx_PyCode_New(4, 0, 15, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__464, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_create_sphere, 2992, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__465)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2992; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":3036
 * 
 * 
 * def _gradient_3d(floating[:, :, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                  double[:] img_spacing, double[:, :] out_grid2world,
 *                  floating[:, :, :, :] out, int[:, :, :] inside):
 */
  __pyx_tuple__466 = PyTuple_Pack(19, __pyx_n_s_img, __pyx_n_s_img_world2grid, __pyx_n_s_img_spacing, __pyx_n_s_out_grid2world, __pyx_n_s_out, __pyx_n_s_inside, __pyx_n_s_nslices, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_k, __pyx_n_s_in_flag, __pyx_n_s_tmp, __pyx_n_s_x, __pyx_n_s_dx, __pyx_n_s_h, __pyx_n_s_q, __pyx_n_s_p); if (unlikely(!__pyx_tuple__466)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__466);
  __Pyx_GIVEREF(__pyx_tuple__466);
  __pyx_codeobj__467 = (PyObject*)__Pyx_PyCode_New(6, 0, 19, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__466, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_gradient_3d, 3036, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__467)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":3132
 * 
 * 
 * def _sparse_gradient_3d(floating[:, :, :] img,             # <<<<<<<<<<<<<<
 *                         double[:, :] img_world2grid,
 *                         double[:] img_spacing,
 */
  __pyx_tuple__468 = PyTuple_Pack(14, __pyx_n_s_img, __pyx_n_s_img_world2grid, __pyx_n_s_img_spacing, __pyx_n_s_sample_points, __pyx_n_s_out, __pyx_n_s_inside, __pyx_n_s_n, __pyx_n_s_i, __pyx_n_s_in_flag, __pyx_n_s_tmp, __pyx_n_s_dx, __pyx_n_s_h, __pyx_n_s_q, __pyx_n_s_p); if (unlikely(!__pyx_tuple__468)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__468);
  __Pyx_GIVEREF(__pyx_tuple__468);
  __pyx_codeobj__469 = (PyObject*)__Pyx_PyCode_New(6, 0, 14, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__468, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_sparse_gradient_3d, 3132, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__469)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":3215
 * 
 * 
 * def _gradient_2d(floating[:, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                  double[:] img_spacing, double[:, :] out_grid2world,
 *                  floating[:, :, :] out, int[:, :] inside):
 */
  __pyx_tuple__470 = PyTuple_Pack(18, __pyx_n_s_img, __pyx_n_s_img_world2grid, __pyx_n_s_img_spacing, __pyx_n_s_out_grid2world, __pyx_n_s_out, __pyx_n_s_inside, __pyx_n_s_nrows, __pyx_n_s_ncols, __pyx_n_s_i, __pyx_n_s_j, __pyx_n_s_k, __pyx_n_s_in_flag, __pyx_n_s_tmp, __pyx_n_s_x, __pyx_n_s_dx, __pyx_n_s_h, __pyx_n_s_q, __pyx_n_s_p); if (unlikely(!__pyx_tuple__470)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__470);
  __Pyx_GIVEREF(__pyx_tuple__470);
  __pyx_codeobj__471 = (PyObject*)__Pyx_PyCode_New(6, 0, 18, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__470, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_gradient_2d, 3215, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__471)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":3298
 * 
 * 
 * def _sparse_gradient_2d(floating[:, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                         double[:] img_spacing,
 *                         double[:, :] sample_points,
 */
  __pyx_tuple__472 = PyTuple_Pack(14, __pyx_n_s_img, __pyx_n_s_img_world2grid, __pyx_n_s_img_spacing, __pyx_n_s_sample_points, __pyx_n_s_out, __pyx_n_s_inside, __pyx_n_s_n, __pyx_n_s_i, __pyx_n_s_in_flag, __pyx_n_s_tmp, __pyx_n_s_dx, __pyx_n_s_h, __pyx_n_s_q, __pyx_n_s_p); if (unlikely(!__pyx_tuple__472)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__472);
  __Pyx_GIVEREF(__pyx_tuple__472);
  __pyx_codeobj__473 = (PyObject*)__Pyx_PyCode_New(6, 0, 14, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__472, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_sparse_gradient_2d, 3298, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__473)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":3373
 * 
 * 
 * def gradient(img, img_world2grid, img_spacing, out_shape,             # <<<<<<<<<<<<<<
 *              out_grid2world):
 *     r""" Gradient of an image in physical space
 */
  __pyx_tuple__474 = PyTuple_Pack(10, __pyx_n_s_img, __pyx_n_s_img_world2grid, __pyx_n_s_img_spacing, __pyx_n_s_out_shape, __pyx_n_s_out_grid2world, __pyx_n_s_dim, __pyx_n_s_ftype, __pyx_n_s_out, __pyx_n_s_inside, __pyx_n_s_jd_grad); if (unlikely(!__pyx_tuple__474)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3373; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__474);
  __Pyx_GIVEREF(__pyx_tuple__474);
  __pyx_codeobj__475 = (PyObject*)__Pyx_PyCode_New(5, 0, 10, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__474, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_gradient, 3373, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__475)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3373; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":3420
 * 
 * 
 * def sparse_gradient(img, img_world2grid, img_spacing, sample_points):             # <<<<<<<<<<<<<<
 *     r""" Gradient of an image in physical space
 * 
 */
  __pyx_tuple__476 = PyTuple_Pack(10, __pyx_n_s_img, __pyx_n_s_img_world2grid, __pyx_n_s_img_spacing, __pyx_n_s_sample_points, __pyx_n_s_dim, __pyx_n_s_ftype, __pyx_n_s_n, __pyx_n_s_out, __pyx_n_s_inside, __pyx_n_s_jd_grad); if (unlikely(!__pyx_tuple__476)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__476);
  __Pyx_GIVEREF(__pyx_tuple__476);
  __pyx_codeobj__477 = (PyObject*)__Pyx_PyCode_New(4, 0, 10, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__476, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_yoh_deb_gits_pkg_exppsy_di, __pyx_n_s_sparse_gradient, 3420, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__477)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "View.MemoryView":278
 *         return self.name
 * 
 * cdef generic = Enum("<strided and direct or indirect>")             # <<<<<<<<<<<<<<
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")
 */
  __pyx_tuple__478 = PyTuple_Pack(1, __pyx_kp_s_strided_and_direct_or_indirect); if (unlikely(!__pyx_tuple__478)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__478);
  __Pyx_GIVEREF(__pyx_tuple__478);

  /* "View.MemoryView":279
 * 
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default             # <<<<<<<<<<<<<<
 * cdef indirect = Enum("<strided and indirect>")
 * 
 */
  __pyx_tuple__479 = PyTuple_Pack(1, __pyx_kp_s_strided_and_direct); if (unlikely(!__pyx_tuple__479)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__479);
  __Pyx_GIVEREF(__pyx_tuple__479);

  /* "View.MemoryView":280
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__480 = PyTuple_Pack(1, __pyx_kp_s_strided_and_indirect); if (unlikely(!__pyx_tuple__480)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 280; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__480);
  __Pyx_GIVEREF(__pyx_tuple__480);

  /* "View.MemoryView":283
 * 
 * 
 * cdef contiguous = Enum("<contiguous and direct>")             # <<<<<<<<<<<<<<
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")
 * 
 */
  __pyx_tuple__481 = PyTuple_Pack(1, __pyx_kp_s_contiguous_and_direct); if (unlikely(!__pyx_tuple__481)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 283; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__481);
  __Pyx_GIVEREF(__pyx_tuple__481);

  /* "View.MemoryView":284
 * 
 * cdef contiguous = Enum("<contiguous and direct>")
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__482 = PyTuple_Pack(1, __pyx_kp_s_contiguous_and_indirect); if (unlikely(!__pyx_tuple__482)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 284; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_tuple__482);
  __Pyx_GIVEREF(__pyx_tuple__482);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  __pyx_float_1_0 = PyFloat_FromDouble(1.0); if (unlikely(!__pyx_float_1_0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initvector_fields(void); /*proto*/
PyMODINIT_FUNC initvector_fields(void)
#else
PyMODINIT_FUNC PyInit_vector_fields(void); /*proto*/
PyMODINIT_FUNC PyInit_vector_fields(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  __Pyx_memviewslice __pyx_t_20 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_21 = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  __Pyx_memviewslice __pyx_t_25 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_26 = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_t_27 = NULL;
  PyObject *__pyx_t_28 = NULL;
  PyObject *__pyx_t_29 = NULL;
  PyObject *__pyx_t_30 = NULL;
  PyObject *__pyx_t_31 = NULL;
  PyObject *__pyx_t_32 = NULL;
  PyObject *__pyx_t_33 = NULL;
  PyObject *__pyx_t_34 = NULL;
  PyObject *__pyx_t_35 = NULL;
  PyObject *__pyx_t_36 = NULL;
  PyObject *__pyx_t_37 = NULL;
  PyObject *__pyx_t_38 = NULL;
  PyObject *__pyx_t_39 = NULL;
  PyObject *__pyx_t_40 = NULL;
  PyObject *__pyx_t_41 = NULL;
  PyObject *__pyx_t_42 = NULL;
  PyObject *__pyx_t_43 = NULL;
  PyObject *__pyx_t_44 = NULL;
  PyObject *__pyx_t_45 = NULL;
  __Pyx_memviewslice __pyx_t_46 = { 0, 0, { 0 }, { 0 }, { 0 } };
  __Pyx_memviewslice __pyx_t_47 = { 0, 0, { 0 }, { 0 }, { 0 } };
  PyObject *__pyx_t_48 = NULL;
  PyObject *__pyx_t_49 = NULL;
  PyObject *__pyx_t_50 = NULL;
  PyObject *__pyx_t_51 = NULL;
  PyObject *__pyx_t_52 = NULL;
  PyObject *__pyx_t_53 = NULL;
  PyObject *__pyx_t_54 = NULL;
  PyObject *__pyx_t_55 = NULL;
  PyObject *__pyx_t_56 = NULL;
  PyObject *__pyx_t_57 = NULL;
  PyObject *__pyx_t_58 = NULL;
  PyObject *__pyx_t_59 = NULL;
  PyObject *__pyx_t_60 = NULL;
  PyObject *__pyx_t_61 = NULL;
  PyObject *__pyx_t_62 = NULL;
  PyObject *__pyx_t_63 = NULL;
  PyObject *__pyx_t_64 = NULL;
  PyObject *__pyx_t_65 = NULL;
  PyObject *__pyx_t_66 = NULL;
  PyObject *__pyx_t_67 = NULL;
  PyObject *__pyx_t_68 = NULL;
  PyObject *__pyx_t_69 = NULL;
  PyObject *__pyx_t_70 = NULL;
  PyObject *__pyx_t_71 = NULL;
  PyObject *__pyx_t_72 = NULL;
  PyObject *__pyx_t_73 = NULL;
  PyObject *__pyx_t_74 = NULL;
  PyObject *__pyx_t_75 = NULL;
  PyObject *__pyx_t_76 = NULL;
  PyObject *__pyx_t_77 = NULL;
  PyObject *__pyx_t_78 = NULL;
  PyObject *__pyx_t_79 = NULL;
  PyObject *__pyx_t_80 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_vector_fields(void)", 0);
  if (__Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("vector_fields", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  if (__pyx_module_is_main_dipy__align__vector_fields) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "dipy.align.vector_fields")) {
      if (unlikely(PyDict_SetItemString(modules, "dipy.align.vector_fields", __pyx_m) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  generic = Py_None; Py_INCREF(Py_None);
  strided = Py_None; Py_INCREF(Py_None);
  indirect = Py_None; Py_INCREF(Py_None);
  contiguous = Py_None; Py_INCREF(Py_None);
  indirect_contiguous = Py_None; Py_INCREF(Py_None);
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  if (PyType_Ready(&__pyx_type___pyx_array) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 101; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type___pyx_array.tp_print = 0;
  __pyx_array_type = &__pyx_type___pyx_array;
  if (PyType_Ready(&__pyx_type___pyx_MemviewEnum) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 271; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type___pyx_MemviewEnum.tp_print = 0;
  __pyx_MemviewEnum_type = &__pyx_type___pyx_MemviewEnum;
  __pyx_vtabptr_memoryview = &__pyx_vtable_memoryview;
  __pyx_vtable_memoryview.get_item_pointer = (char *(*)(struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_get_item_pointer;
  __pyx_vtable_memoryview.is_slice = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_is_slice;
  __pyx_vtable_memoryview.setitem_slice_assignment = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *, PyObject *))__pyx_memoryview_setitem_slice_assignment;
  __pyx_vtable_memoryview.setitem_slice_assign_scalar = (PyObject *(*)(struct __pyx_memoryview_obj *, struct __pyx_memoryview_obj *, PyObject *))__pyx_memoryview_setitem_slice_assign_scalar;
  __pyx_vtable_memoryview.setitem_indexed = (PyObject *(*)(struct __pyx_memoryview_obj *, PyObject *, PyObject *))__pyx_memoryview_setitem_indexed;
  __pyx_vtable_memoryview.convert_item_to_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *))__pyx_memoryview_convert_item_to_object;
  __pyx_vtable_memoryview.assign_item_from_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *, PyObject *))__pyx_memoryview_assign_item_from_object;
  if (PyType_Ready(&__pyx_type___pyx_memoryview) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type___pyx_memoryview.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type___pyx_memoryview.tp_dict, __pyx_vtabptr_memoryview) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_memoryview_type = &__pyx_type___pyx_memoryview;
  __pyx_vtabptr__memoryviewslice = &__pyx_vtable__memoryviewslice;
  __pyx_vtable__memoryviewslice.__pyx_base = *__pyx_vtabptr_memoryview;
  __pyx_vtable__memoryviewslice.__pyx_base.convert_item_to_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *))__pyx_memoryviewslice_convert_item_to_object;
  __pyx_vtable__memoryviewslice.__pyx_base.assign_item_from_object = (PyObject *(*)(struct __pyx_memoryview_obj *, char *, PyObject *))__pyx_memoryviewslice_assign_item_from_object;
  __pyx_type___pyx_memoryviewslice.tp_base = __pyx_memoryview_type;
  if (PyType_Ready(&__pyx_type___pyx_memoryviewslice) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 923; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_type___pyx_memoryviewslice.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type___pyx_memoryviewslice.tp_dict, __pyx_vtabptr__memoryviewslice) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 923; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_memoryviewslice_type = &__pyx_type___pyx_memoryviewslice;
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if CYTHON_COMPILING_IN_PYPY
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) {__pyx_filename = __pyx_f[3]; __pyx_lineno = 9; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_dtype = __Pyx_ImportType("numpy", "dtype", sizeof(PyArray_Descr), 0); if (unlikely(!__pyx_ptype_5numpy_dtype)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_flatiter = __Pyx_ImportType("numpy", "flatiter", sizeof(PyArrayIterObject), 0); if (unlikely(!__pyx_ptype_5numpy_flatiter)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 168; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_broadcast = __Pyx_ImportType("numpy", "broadcast", sizeof(PyArrayMultiIterObject), 0); if (unlikely(!__pyx_ptype_5numpy_broadcast)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 172; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_ndarray = __Pyx_ImportType("numpy", "ndarray", sizeof(PyArrayObject), 0); if (unlikely(!__pyx_ptype_5numpy_ndarray)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 181; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_ptype_5numpy_ufunc = __Pyx_ImportType("numpy", "ufunc", sizeof(PyUFuncObject), 0); if (unlikely(!__pyx_ptype_5numpy_ufunc)) {__pyx_filename = __pyx_f[1]; __pyx_lineno = 861; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif

  /* "dipy/align/vector_fields.pyx":6
 * #cython: cdivision=True
 * 
 * import numpy as np             # <<<<<<<<<<<<<<
 * cimport numpy as cnp
 * cimport cython
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, -1); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 6; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "dipy/align/vector_fields.pyx":15
 * 
 * 
 * def is_valid_affine(double[:, :] M, int dim):             # <<<<<<<<<<<<<<
 *     if M is None:
 *         return True
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_1is_valid_affine, NULL, __pyx_n_s_dipy_align_vector_fields); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 15; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_is_valid_affine, __pyx_t_1) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 15; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "dipy/align/vector_fields.pyx":25
 * 
 * 
 * def interpolate_vector_2d(floating[:, :, :] field, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Bilinear interpolation of a 2D vector field
 * 
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_83interpolate_vector_2d, 0, __pyx_n_s_interpolate_vector_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__399)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_float, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_85interpolate_vector_2d, 0, __pyx_n_s_interpolate_vector_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__399)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_double, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_3interpolate_vector_2d, 0, __pyx_n_s_interpolate_vector_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__399)); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_2)->__signatures__ = __pyx_t_1;
  __Pyx_GIVEREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_interpolate_vector_2d, __pyx_t_2) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 25; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "dipy/align/vector_fields.pyx":135
 * 
 * 
 * def interpolate_scalar_2d(floating[:, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Bilinear interpolation of a 2D scalar image
 * 
 */
  __pyx_t_3 = PyDict_New(); if (unlikely(!__pyx_t_3)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_89interpolate_scalar_2d, 0, __pyx_n_s_interpolate_scalar_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__401)); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_float, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_91interpolate_scalar_2d, 0, __pyx_n_s_interpolate_scalar_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__401)); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_double, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_5interpolate_scalar_2d, 0, __pyx_n_s_interpolate_scalar_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__401)); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_4, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_4)->__signatures__ = __pyx_t_3;
  __Pyx_GIVEREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_interpolate_scalar_2d, __pyx_t_4) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 135; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "dipy/align/vector_fields.pyx":239
 * 
 * 
 * def interpolate_scalar_nn_2d(number[:, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 2D scalar image
 * 
 */
  __pyx_t_5 = PyDict_New(); if (unlikely(!__pyx_t_5)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_95interpolate_scalar_nn_2d, 0, __pyx_n_s_interpolate_scalar_nn_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__403)); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_6, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_short, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_97interpolate_scalar_nn_2d, 0, __pyx_n_s_interpolate_scalar_nn_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__403)); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_6, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_int, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_FusedFunction_NewEx(&__pyx_fuse_2__pyx_mdef_4dipy_5align_13vector_fields_99interpolate_scalar_nn_2d, 0, __pyx_n_s_interpolate_scalar_nn_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__403)); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_6, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_long, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_FusedFunction_NewEx(&__pyx_fuse_3__pyx_mdef_4dipy_5align_13vector_fields_101interpolate_scalar_nn_2d, 0, __pyx_n_s_interpolate_scalar_nn_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__403)); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_6, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_float, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_FusedFunction_NewEx(&__pyx_fuse_4__pyx_mdef_4dipy_5align_13vector_fields_103interpolate_scalar_nn_2d, 0, __pyx_n_s_interpolate_scalar_nn_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__403)); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_6, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_5, __pyx_n_s_double, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_FusedFunction_NewEx(&__pyx_fuse_5__pyx_mdef_4dipy_5align_13vector_fields_105interpolate_scalar_nn_2d, 0, __pyx_n_s_interpolate_scalar_nn_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__403)); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_6, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_5, __pyx_kp_s_float_complex, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_FusedFunction_NewEx(&__pyx_fuse_6__pyx_mdef_4dipy_5align_13vector_fields_107interpolate_scalar_nn_2d, 0, __pyx_n_s_interpolate_scalar_nn_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__403)); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_6, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_5, __pyx_kp_s_double_complex, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_7interpolate_scalar_nn_2d, 0, __pyx_n_s_interpolate_scalar_nn_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__403)); if (unlikely(!__pyx_t_6)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_6, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_6)->__signatures__ = __pyx_t_5;
  __Pyx_GIVEREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_interpolate_scalar_nn_2d, __pyx_t_6) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 239; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "dipy/align/vector_fields.pyx":332
 * 
 * 
 * def interpolate_scalar_nn_3d(number[:, :, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Nearest neighbor interpolation of a 3D scalar image
 * 
 */
  __pyx_t_7 = PyDict_New(); if (unlikely(!__pyx_t_7)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_111interpolate_scalar_nn_3d, 0, __pyx_n_s_interpolate_scalar_nn_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__405)); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_8, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_short, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_113interpolate_scalar_nn_3d, 0, __pyx_n_s_interpolate_scalar_nn_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__405)); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_8, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_int, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __pyx_FusedFunction_NewEx(&__pyx_fuse_2__pyx_mdef_4dipy_5align_13vector_fields_115interpolate_scalar_nn_3d, 0, __pyx_n_s_interpolate_scalar_nn_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__405)); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_8, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_long, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __pyx_FusedFunction_NewEx(&__pyx_fuse_3__pyx_mdef_4dipy_5align_13vector_fields_117interpolate_scalar_nn_3d, 0, __pyx_n_s_interpolate_scalar_nn_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__405)); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_8, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_float, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __pyx_FusedFunction_NewEx(&__pyx_fuse_4__pyx_mdef_4dipy_5align_13vector_fields_119interpolate_scalar_nn_3d, 0, __pyx_n_s_interpolate_scalar_nn_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__405)); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_8, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_7, __pyx_n_s_double, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __pyx_FusedFunction_NewEx(&__pyx_fuse_5__pyx_mdef_4dipy_5align_13vector_fields_121interpolate_scalar_nn_3d, 0, __pyx_n_s_interpolate_scalar_nn_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__405)); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_8, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_7, __pyx_kp_s_float_complex, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __pyx_FusedFunction_NewEx(&__pyx_fuse_6__pyx_mdef_4dipy_5align_13vector_fields_123interpolate_scalar_nn_3d, 0, __pyx_n_s_interpolate_scalar_nn_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__405)); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_8, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_7, __pyx_kp_s_double_complex, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_9interpolate_scalar_nn_3d, 0, __pyx_n_s_interpolate_scalar_nn_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__405)); if (unlikely(!__pyx_t_8)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_8, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_8)->__signatures__ = __pyx_t_7;
  __Pyx_GIVEREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_interpolate_scalar_nn_3d, __pyx_t_8) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "dipy/align/vector_fields.pyx":434
 * 
 * 
 * def interpolate_scalar_3d(floating[:, :, :] image, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Trilinear interpolation of a 3D scalar image
 * 
 */
  __pyx_t_9 = PyDict_New(); if (unlikely(!__pyx_t_9)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_127interpolate_scalar_3d, 0, __pyx_n_s_interpolate_scalar_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__407)); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_10, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_float, __pyx_t_10) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_129interpolate_scalar_3d, 0, __pyx_n_s_interpolate_scalar_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__407)); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_10, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_9, __pyx_n_s_double, __pyx_t_10) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_11interpolate_scalar_3d, 0, __pyx_n_s_interpolate_scalar_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__407)); if (unlikely(!__pyx_t_10)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_10, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_10)->__signatures__ = __pyx_t_9;
  __Pyx_GIVEREF(__pyx_t_9);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_interpolate_scalar_3d, __pyx_t_10) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 434; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "dipy/align/vector_fields.pyx":567
 * 
 * 
 * def interpolate_vector_3d(floating[:, :, :, :] field, double[:, :] locations):             # <<<<<<<<<<<<<<
 *     r"""Trilinear interpolation of a 3D vector field
 * 
 */
  __pyx_t_11 = PyDict_New(); if (unlikely(!__pyx_t_11)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_12 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_133interpolate_vector_3d, 0, __pyx_n_s_interpolate_vector_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__409)); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_12, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_float, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_135interpolate_vector_3d, 0, __pyx_n_s_interpolate_vector_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__409)); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_12, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_double, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_13interpolate_vector_3d, 0, __pyx_n_s_interpolate_vector_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__409)); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_12, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_12)->__signatures__ = __pyx_t_11;
  __Pyx_GIVEREF(__pyx_t_11);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_interpolate_vector_3d, __pyx_t_12) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 567; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "dipy/align/vector_fields.pyx":847
 * 
 * 
 * def compose_vector_fields_2d(floating[:, :, :] d1, floating[:, :, :] d2,             # <<<<<<<<<<<<<<
 *                              double[:, :] premult_index,
 *                              double[:, :] premult_disp,
 */
  __pyx_t_13 = PyDict_New(); if (unlikely(!__pyx_t_13)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_14 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_139compose_vector_fields_2d, 0, __pyx_n_s_compose_vector_fields_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__411)); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_14, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_13, __pyx_n_s_float, __pyx_t_14) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_14 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_141compose_vector_fields_2d, 0, __pyx_n_s_compose_vector_fields_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__411)); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_14, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_13, __pyx_n_s_double, __pyx_t_14) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_14 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_15compose_vector_fields_2d, 0, __pyx_n_s_compose_vector_fields_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__411)); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_14, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_14)->__signatures__ = __pyx_t_13;
  __Pyx_GIVEREF(__pyx_t_13);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_compose_vector_fields_2d, __pyx_t_14) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 847; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

  /* "dipy/align/vector_fields.pyx":1054
 * 
 * 
 * def compose_vector_fields_3d(floating[:, :, :, :] d1, floating[:, :, :, :] d2,             # <<<<<<<<<<<<<<
 *                              double[:, :] premult_index,
 *                              double[:, :] premult_disp,
 */
  __pyx_t_15 = PyDict_New(); if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_15);
  __pyx_t_16 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_145compose_vector_fields_3d, 0, __pyx_n_s_compose_vector_fields_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__413)); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_16, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_15, __pyx_n_s_float, __pyx_t_16) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  __pyx_t_16 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_147compose_vector_fields_3d, 0, __pyx_n_s_compose_vector_fields_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__413)); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_16, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_15, __pyx_n_s_double, __pyx_t_16) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
  __pyx_t_16 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_17compose_vector_fields_3d, 0, __pyx_n_s_compose_vector_fields_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__413)); if (unlikely(!__pyx_t_16)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_16);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_16, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_16)->__signatures__ = __pyx_t_15;
  __Pyx_GIVEREF(__pyx_t_15);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_compose_vector_fields_3d, __pyx_t_16) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1054; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;

  /* "dipy/align/vector_fields.pyx":1127
 * 
 * 
 * def invert_vector_field_fixed_point_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                                        double[:, :] d_world2grid,
 *                                        double[:] spacing,
 */
  __pyx_t_17 = PyTuple_New(1); if (unlikely(!__pyx_t_17)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_17);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_17, 0, Py_None);
  __pyx_t_18 = PyDict_New(); if (unlikely(!__pyx_t_18)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_18);
  __pyx_t_19 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_151invert_vector_field_fixed_point_2d, 0, __pyx_n_s_invert_vector_field_fixed_point, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__415)); if (unlikely(!__pyx_t_19)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_19);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_19, sizeof(__pyx_defaults2), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1131
 *                                        double[:] spacing,
 *                                        int max_iter, double tolerance,
 *                                        floating[:, :, :] start=None):             # <<<<<<<<<<<<<<
 *     r"""Computes the inverse of a 2D displacement fields
 * 
 */
  __pyx_t_20 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(Py_None);
  if (unlikely(!__pyx_t_20.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults2, __pyx_t_19)->__pyx_arg_start = __pyx_t_20;
  __pyx_t_20.memview = NULL;
  __pyx_t_20.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_19, __pyx_t_17);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_19, __pyx_pf_4dipy_5align_13vector_fields_366__defaults__);
  if (PyDict_SetItem(__pyx_t_18, __pyx_n_s_float, __pyx_t_19) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;

  /* "dipy/align/vector_fields.pyx":1127
 * 
 * 
 * def invert_vector_field_fixed_point_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                                        double[:, :] d_world2grid,
 *                                        double[:] spacing,
 */
  __pyx_t_19 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_153invert_vector_field_fixed_point_2d, 0, __pyx_n_s_invert_vector_field_fixed_point, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__415)); if (unlikely(!__pyx_t_19)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_19);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_19, sizeof(__pyx_defaults3), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1131
 *                                        double[:] spacing,
 *                                        int max_iter, double tolerance,
 *                                        floating[:, :, :] start=None):             # <<<<<<<<<<<<<<
 *     r"""Computes the inverse of a 2D displacement fields
 * 
 */
  __pyx_t_21 = __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(Py_None);
  if (unlikely(!__pyx_t_21.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1131; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults3, __pyx_t_19)->__pyx_arg_start = __pyx_t_21;
  __pyx_t_21.memview = NULL;
  __pyx_t_21.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_19, __pyx_t_17);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_19, __pyx_pf_4dipy_5align_13vector_fields_368__defaults__);
  if (PyDict_SetItem(__pyx_t_18, __pyx_n_s_double, __pyx_t_19) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;

  /* "dipy/align/vector_fields.pyx":1127
 * 
 * 
 * def invert_vector_field_fixed_point_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                                        double[:, :] d_world2grid,
 *                                        double[:] spacing,
 */
  __pyx_t_19 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_19invert_vector_field_fixed_point_2d, 0, __pyx_n_s_invert_vector_field_fixed_point, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__415)); if (unlikely(!__pyx_t_19)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_19);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_19, __pyx_t_17);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_19, __pyx_pf_4dipy_5align_13vector_fields_366__defaults__);
  ((__pyx_FusedFunctionObject *) __pyx_t_19)->__signatures__ = __pyx_t_18;
  __Pyx_GIVEREF(__pyx_t_18);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_invert_vector_field_fixed_point, __pyx_t_19) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1127; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
  __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;

  /* "dipy/align/vector_fields.pyx":1231
 * 
 * 
 * def invert_vector_field_fixed_point_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                                        double[:, :] d_world2grid,
 *                                        double[:] spacing,
 */
  __pyx_t_22 = PyTuple_New(1); if (unlikely(!__pyx_t_22)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_22);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_22, 0, Py_None);
  __pyx_t_23 = PyDict_New(); if (unlikely(!__pyx_t_23)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_23);
  __pyx_t_24 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_157invert_vector_field_fixed_point_3d, 0, __pyx_n_s_invert_vector_field_fixed_point_2, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__417)); if (unlikely(!__pyx_t_24)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_24);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_24, sizeof(__pyx_defaults6), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1235
 *                                        double[:] spacing,
 *                                        int max_iter, double tol,
 *                                        floating[:, :, :, :] start=None):             # <<<<<<<<<<<<<<
 *     r"""Computes the inverse of a 3D displacement fields
 * 
 */
  __pyx_t_25 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(Py_None);
  if (unlikely(!__pyx_t_25.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1235; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults6, __pyx_t_24)->__pyx_arg_start = __pyx_t_25;
  __pyx_t_25.memview = NULL;
  __pyx_t_25.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_24, __pyx_t_22);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_24, __pyx_pf_4dipy_5align_13vector_fields_374__defaults__);
  if (PyDict_SetItem(__pyx_t_23, __pyx_n_s_float, __pyx_t_24) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;

  /* "dipy/align/vector_fields.pyx":1231
 * 
 * 
 * def invert_vector_field_fixed_point_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                                        double[:, :] d_world2grid,
 *                                        double[:] spacing,
 */
  __pyx_t_24 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_159invert_vector_field_fixed_point_3d, 0, __pyx_n_s_invert_vector_field_fixed_point_2, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__417)); if (unlikely(!__pyx_t_24)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_24);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_24, sizeof(__pyx_defaults7), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1235
 *                                        double[:] spacing,
 *                                        int max_iter, double tol,
 *                                        floating[:, :, :, :] start=None):             # <<<<<<<<<<<<<<
 *     r"""Computes the inverse of a 3D displacement fields
 * 
 */
  __pyx_t_26 = __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(Py_None);
  if (unlikely(!__pyx_t_26.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1235; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults7, __pyx_t_24)->__pyx_arg_start = __pyx_t_26;
  __pyx_t_26.memview = NULL;
  __pyx_t_26.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_24, __pyx_t_22);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_24, __pyx_pf_4dipy_5align_13vector_fields_376__defaults__);
  if (PyDict_SetItem(__pyx_t_23, __pyx_n_s_double, __pyx_t_24) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;

  /* "dipy/align/vector_fields.pyx":1231
 * 
 * 
 * def invert_vector_field_fixed_point_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                                        double[:, :] d_world2grid,
 *                                        double[:] spacing,
 */
  __pyx_t_24 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_21invert_vector_field_fixed_point_3d, 0, __pyx_n_s_invert_vector_field_fixed_point_2, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__417)); if (unlikely(!__pyx_t_24)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_24);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_24, __pyx_t_22);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_24, __pyx_pf_4dipy_5align_13vector_fields_374__defaults__);
  ((__pyx_FusedFunctionObject *) __pyx_t_24)->__signatures__ = __pyx_t_23;
  __Pyx_GIVEREF(__pyx_t_23);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_invert_vector_field_fixed_point_2, __pyx_t_24) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1231; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_24); __pyx_t_24 = 0;
  __Pyx_DECREF(__pyx_t_22); __pyx_t_22 = 0;

  /* "dipy/align/vector_fields.pyx":1345
 * 
 * 
 * def simplify_warp_function_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                               double[:, :] affine_idx_in,
 *                               double[:, :] affine_idx_out,
 */
  __pyx_t_27 = PyDict_New(); if (unlikely(!__pyx_t_27)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_27);
  __pyx_t_28 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_163simplify_warp_function_2d, 0, __pyx_n_s_simplify_warp_function_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__419)); if (unlikely(!__pyx_t_28)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_28);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_28, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_27, __pyx_n_s_float, __pyx_t_28) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_28); __pyx_t_28 = 0;
  __pyx_t_28 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_165simplify_warp_function_2d, 0, __pyx_n_s_simplify_warp_function_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__419)); if (unlikely(!__pyx_t_28)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_28);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_28, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_27, __pyx_n_s_double, __pyx_t_28) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_28); __pyx_t_28 = 0;
  __pyx_t_28 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_23simplify_warp_function_2d, 0, __pyx_n_s_simplify_warp_function_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__419)); if (unlikely(!__pyx_t_28)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_28);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_28, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_28)->__signatures__ = __pyx_t_27;
  __Pyx_GIVEREF(__pyx_t_27);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_simplify_warp_function_2d, __pyx_t_28) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1345; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_28); __pyx_t_28 = 0;

  /* "dipy/align/vector_fields.pyx":1461
 * 
 * 
 * def simplify_warp_function_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                               double[:, :] affine_idx_in,
 *                               double[:, :] affine_idx_out,
 */
  __pyx_t_29 = PyDict_New(); if (unlikely(!__pyx_t_29)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_29);
  __pyx_t_30 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_169simplify_warp_function_3d, 0, __pyx_n_s_simplify_warp_function_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__421)); if (unlikely(!__pyx_t_30)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_30);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_30, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_29, __pyx_n_s_float, __pyx_t_30) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_30); __pyx_t_30 = 0;
  __pyx_t_30 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_171simplify_warp_function_3d, 0, __pyx_n_s_simplify_warp_function_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__421)); if (unlikely(!__pyx_t_30)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_30);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_30, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_29, __pyx_n_s_double, __pyx_t_30) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_30); __pyx_t_30 = 0;
  __pyx_t_30 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_25simplify_warp_function_3d, 0, __pyx_n_s_simplify_warp_function_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__421)); if (unlikely(!__pyx_t_30)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_30);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_30, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_30)->__signatures__ = __pyx_t_29;
  __Pyx_GIVEREF(__pyx_t_29);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_simplify_warp_function_3d, __pyx_t_30) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1461; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_30); __pyx_t_30 = 0;

  /* "dipy/align/vector_fields.pyx":1588
 * 
 * 
 * def reorient_vector_field_2d(floating[:, :, :] d,             # <<<<<<<<<<<<<<
 *                              double[:, :] affine):
 *     r"""Linearly transforms all vectors of a 2D displacement field
 */
  __pyx_t_31 = PyDict_New(); if (unlikely(!__pyx_t_31)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_31);
  __pyx_t_32 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_175reorient_vector_field_2d, 0, __pyx_n_s_reorient_vector_field_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__423)); if (unlikely(!__pyx_t_32)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_32);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_32, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_31, __pyx_n_s_float, __pyx_t_32) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_32); __pyx_t_32 = 0;
  __pyx_t_32 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_177reorient_vector_field_2d, 0, __pyx_n_s_reorient_vector_field_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__423)); if (unlikely(!__pyx_t_32)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_32);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_32, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_31, __pyx_n_s_double, __pyx_t_32) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_32); __pyx_t_32 = 0;
  __pyx_t_32 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_27reorient_vector_field_2d, 0, __pyx_n_s_reorient_vector_field_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__423)); if (unlikely(!__pyx_t_32)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_32);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_32, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_32)->__signatures__ = __pyx_t_31;
  __Pyx_GIVEREF(__pyx_t_31);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_reorient_vector_field_2d, __pyx_t_32) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1588; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_32); __pyx_t_32 = 0;

  /* "dipy/align/vector_fields.pyx":1626
 * 
 * 
 * def reorient_vector_field_3d(floating[:, :, :, :] d,             # <<<<<<<<<<<<<<
 *                              double[:, :] affine):
 *     r"""Linearly transforms all vectors of a 3D displacement field
 */
  __pyx_t_33 = PyDict_New(); if (unlikely(!__pyx_t_33)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_33);
  __pyx_t_34 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_181reorient_vector_field_3d, 0, __pyx_n_s_reorient_vector_field_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__425)); if (unlikely(!__pyx_t_34)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_34);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_34, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_33, __pyx_n_s_float, __pyx_t_34) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_34); __pyx_t_34 = 0;
  __pyx_t_34 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_183reorient_vector_field_3d, 0, __pyx_n_s_reorient_vector_field_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__425)); if (unlikely(!__pyx_t_34)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_34);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_34, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_33, __pyx_n_s_double, __pyx_t_34) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_34); __pyx_t_34 = 0;
  __pyx_t_34 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_29reorient_vector_field_3d, 0, __pyx_n_s_reorient_vector_field_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__425)); if (unlikely(!__pyx_t_34)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_34);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_34, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_34)->__signatures__ = __pyx_t_33;
  __Pyx_GIVEREF(__pyx_t_33);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_reorient_vector_field_3d, __pyx_t_34) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1626; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_34); __pyx_t_34 = 0;

  /* "dipy/align/vector_fields.pyx":1668
 * 
 * 
 * def downsample_scalar_field_3d(floating[:, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input volume by a factor of 2
 * 
 */
  __pyx_t_35 = PyDict_New(); if (unlikely(!__pyx_t_35)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_35);
  __pyx_t_36 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_187downsample_scalar_field_3d, 0, __pyx_n_s_downsample_scalar_field_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__427)); if (unlikely(!__pyx_t_36)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_36);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_36, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_35, __pyx_n_s_float, __pyx_t_36) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_36); __pyx_t_36 = 0;
  __pyx_t_36 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_189downsample_scalar_field_3d, 0, __pyx_n_s_downsample_scalar_field_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__427)); if (unlikely(!__pyx_t_36)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_36);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_36, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_35, __pyx_n_s_double, __pyx_t_36) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_36); __pyx_t_36 = 0;
  __pyx_t_36 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_31downsample_scalar_field_3d, 0, __pyx_n_s_downsample_scalar_field_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__427)); if (unlikely(!__pyx_t_36)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_36);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_36, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_36)->__signatures__ = __pyx_t_35;
  __Pyx_GIVEREF(__pyx_t_35);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_downsample_scalar_field_3d, __pyx_t_36) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1668; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_36); __pyx_t_36 = 0;

  /* "dipy/align/vector_fields.pyx":1715
 * 
 * 
 * def downsample_displacement_field_3d(floating[:, :, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input 3D vector field by a factor of 2
 * 
 */
  __pyx_t_37 = PyDict_New(); if (unlikely(!__pyx_t_37)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_37);
  __pyx_t_38 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_193downsample_displacement_field_3d, 0, __pyx_n_s_downsample_displacement_field_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__429)); if (unlikely(!__pyx_t_38)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_38);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_38, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_37, __pyx_n_s_float, __pyx_t_38) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_38); __pyx_t_38 = 0;
  __pyx_t_38 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_195downsample_displacement_field_3d, 0, __pyx_n_s_downsample_displacement_field_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__429)); if (unlikely(!__pyx_t_38)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_38);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_38, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_37, __pyx_n_s_double, __pyx_t_38) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_38); __pyx_t_38 = 0;
  __pyx_t_38 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_33downsample_displacement_field_3d, 0, __pyx_n_s_downsample_displacement_field_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__429)); if (unlikely(!__pyx_t_38)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_38);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_38, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_38)->__signatures__ = __pyx_t_37;
  __Pyx_GIVEREF(__pyx_t_37);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_downsample_displacement_field_3d, __pyx_t_38) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1715; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_38); __pyx_t_38 = 0;

  /* "dipy/align/vector_fields.pyx":1767
 * 
 * 
 * def downsample_scalar_field_2d(floating[:, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the input 2D image by a factor of 2
 * 
 */
  __pyx_t_39 = PyDict_New(); if (unlikely(!__pyx_t_39)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_39);
  __pyx_t_40 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_199downsample_scalar_field_2d, 0, __pyx_n_s_downsample_scalar_field_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__431)); if (unlikely(!__pyx_t_40)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_40);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_40, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_39, __pyx_n_s_float, __pyx_t_40) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_40); __pyx_t_40 = 0;
  __pyx_t_40 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_201downsample_scalar_field_2d, 0, __pyx_n_s_downsample_scalar_field_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__431)); if (unlikely(!__pyx_t_40)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_40);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_40, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_39, __pyx_n_s_double, __pyx_t_40) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_40); __pyx_t_40 = 0;
  __pyx_t_40 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_35downsample_scalar_field_2d, 0, __pyx_n_s_downsample_scalar_field_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__431)); if (unlikely(!__pyx_t_40)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_40);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_40, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_40)->__signatures__ = __pyx_t_39;
  __Pyx_GIVEREF(__pyx_t_39);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_downsample_scalar_field_2d, __pyx_t_40) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1767; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_40); __pyx_t_40 = 0;

  /* "dipy/align/vector_fields.pyx":1808
 * 
 * 
 * def downsample_displacement_field_2d(floating[:, :, :] field):             # <<<<<<<<<<<<<<
 *     r"""Down-samples the 2D input vector field by a factor of 2
 *     Down-samples the input vector field by a factor of 2. The value at each
 */
  __pyx_t_41 = PyDict_New(); if (unlikely(!__pyx_t_41)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_41);
  __pyx_t_42 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_205downsample_displacement_field_2d, 0, __pyx_n_s_downsample_displacement_field_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__433)); if (unlikely(!__pyx_t_42)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_42);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_42, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_41, __pyx_n_s_float, __pyx_t_42) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_42); __pyx_t_42 = 0;
  __pyx_t_42 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_207downsample_displacement_field_2d, 0, __pyx_n_s_downsample_displacement_field_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__433)); if (unlikely(!__pyx_t_42)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_42);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_42, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_41, __pyx_n_s_double, __pyx_t_42) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_42); __pyx_t_42 = 0;
  __pyx_t_42 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_37downsample_displacement_field_2d, 0, __pyx_n_s_downsample_displacement_field_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__433)); if (unlikely(!__pyx_t_42)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_42);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_42, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_42)->__signatures__ = __pyx_t_41;
  __Pyx_GIVEREF(__pyx_t_41);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_downsample_displacement_field_2d, __pyx_t_42) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1808; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_42); __pyx_t_42 = 0;

  /* "dipy/align/vector_fields.pyx":1851
 * 
 * 
 * def warp_3d(floating[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 */
  __pyx_t_43 = PyTuple_New(4); if (unlikely(!__pyx_t_43)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_43);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_43, 0, Py_None);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_43, 1, Py_None);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_43, 2, Py_None);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_43, 3, Py_None);
  __Pyx_INCREF(Py_None);
  __pyx_k__123 = Py_None;
  __Pyx_GIVEREF(Py_None);
  __Pyx_INCREF(Py_None);
  __pyx_k__124 = Py_None;
  __Pyx_GIVEREF(Py_None);
  __pyx_t_44 = PyDict_New(); if (unlikely(!__pyx_t_44)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_44);
  __pyx_t_45 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_211warp_3d, 0, __pyx_n_s_warp_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__435)); if (unlikely(!__pyx_t_45)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_45);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_45, sizeof(__pyx_defaults10), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1852
 * 
 * def warp_3d(floating[:, :, :] volume, floating[:, :, :, :] d1,
 *             double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_out=None,
 *             double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1852; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults10, __pyx_t_45)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":1853
 * def warp_3d(floating[:, :, :] volume, floating[:, :, :, :] d1,
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_disp=None,
 *             int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1853; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults10, __pyx_t_45)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":1854
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 *             double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *             int[:] out_shape=None):
 *     r"""Warps a 3D volume using trilinear interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1854; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults10, __pyx_t_45)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":1855
 *             double[:, :] affine_idx_out=None,
 *             double[:, :] affine_disp=None,
 *             int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 3D volume using trilinear interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1855; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults10, __pyx_t_45)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_45, __pyx_t_43);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_45, __pyx_pf_4dipy_5align_13vector_fields_382__defaults__);
  if (PyDict_SetItem(__pyx_t_44, __pyx_n_s_float, __pyx_t_45) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_45); __pyx_t_45 = 0;

  /* "dipy/align/vector_fields.pyx":1851
 * 
 * 
 * def warp_3d(floating[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 */
  __pyx_t_45 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_213warp_3d, 0, __pyx_n_s_warp_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__435)); if (unlikely(!__pyx_t_45)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_45);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_45, sizeof(__pyx_defaults11), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":1852
 * 
 * def warp_3d(floating[:, :, :] volume, floating[:, :, :, :] d1,
 *             double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_out=None,
 *             double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1852; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults11, __pyx_t_45)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":1853
 * def warp_3d(floating[:, :, :] volume, floating[:, :, :, :] d1,
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_disp=None,
 *             int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1853; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults11, __pyx_t_45)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":1854
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 *             double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *             int[:] out_shape=None):
 *     r"""Warps a 3D volume using trilinear interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1854; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults11, __pyx_t_45)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":1855
 *             double[:, :] affine_idx_out=None,
 *             double[:, :] affine_disp=None,
 *             int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 3D volume using trilinear interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1855; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults11, __pyx_t_45)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_45, __pyx_t_43);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_45, __pyx_pf_4dipy_5align_13vector_fields_384__defaults__);
  if (PyDict_SetItem(__pyx_t_44, __pyx_n_s_double, __pyx_t_45) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_45); __pyx_t_45 = 0;

  /* "dipy/align/vector_fields.pyx":1851
 * 
 * 
 * def warp_3d(floating[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 */
  __pyx_t_45 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_39warp_3d, 0, __pyx_n_s_warp_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__435)); if (unlikely(!__pyx_t_45)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_45);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_45, __pyx_t_43);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_45, __pyx_pf_4dipy_5align_13vector_fields_382__defaults__);
  ((__pyx_FusedFunctionObject *) __pyx_t_45)->__signatures__ = __pyx_t_44;
  __Pyx_GIVEREF(__pyx_t_44);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_warp_3d, __pyx_t_45) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1851; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_45); __pyx_t_45 = 0;
  __Pyx_DECREF(__pyx_t_43); __pyx_t_43 = 0;

  /* "dipy/align/vector_fields.pyx":1982
 * 
 * 
 * def transform_3d_affine(floating[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                         double[:, :] affine):
 *     r"""Transforms a 3D volume by an affine transform with trilinear interp.
 */
  __pyx_t_48 = PyDict_New(); if (unlikely(!__pyx_t_48)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_48);
  __pyx_t_49 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_217transform_3d_affine, 0, __pyx_n_s_transform_3d_affine, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__437)); if (unlikely(!__pyx_t_49)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_49);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_49, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_48, __pyx_n_s_float, __pyx_t_49) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_49); __pyx_t_49 = 0;
  __pyx_t_49 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_219transform_3d_affine, 0, __pyx_n_s_transform_3d_affine, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__437)); if (unlikely(!__pyx_t_49)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_49);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_49, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_48, __pyx_n_s_double, __pyx_t_49) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_49); __pyx_t_49 = 0;
  __pyx_t_49 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_41transform_3d_affine, 0, __pyx_n_s_transform_3d_affine, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__437)); if (unlikely(!__pyx_t_49)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_49);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_49, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_49)->__signatures__ = __pyx_t_48;
  __Pyx_GIVEREF(__pyx_t_48);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_transform_3d_affine, __pyx_t_49) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1982; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_49); __pyx_t_49 = 0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_50 = PyTuple_New(4); if (unlikely(!__pyx_t_50)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_50);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_50, 0, Py_None);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_50, 1, Py_None);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_50, 2, Py_None);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_50, 3, Py_None);
  __Pyx_INCREF(Py_None);
  __pyx_k__143 = Py_None;
  __Pyx_GIVEREF(Py_None);
  __Pyx_INCREF(Py_None);
  __pyx_k__144 = Py_None;
  __Pyx_GIVEREF(Py_None);
  __pyx_t_51 = PyDict_New(); if (unlikely(!__pyx_t_51)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_51);
  __pyx_t_52 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0_0__pyx_mdef_4dipy_5align_13vector_fields_223warp_3d_nn, 0, __pyx_n_s_warp_3d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__439)); if (unlikely(!__pyx_t_52)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_52);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_52, sizeof(__pyx_defaults26), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2049
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults26, __pyx_t_52)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2050
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults26, __pyx_t_52)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2051
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults26, __pyx_t_52)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2052
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults26, __pyx_t_52)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_52, __pyx_t_50);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_52, __pyx_pf_4dipy_5align_13vector_fields_414__defaults__);
  if (PyDict_SetItem(__pyx_t_51, __pyx_kp_s_short_float, __pyx_t_52) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_52); __pyx_t_52 = 0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_52 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0_1__pyx_mdef_4dipy_5align_13vector_fields_225warp_3d_nn, 0, __pyx_n_s_warp_3d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__439)); if (unlikely(!__pyx_t_52)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_52);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_52, sizeof(__pyx_defaults27), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2049
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults27, __pyx_t_52)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2050
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults27, __pyx_t_52)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2051
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults27, __pyx_t_52)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2052
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults27, __pyx_t_52)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_52, __pyx_t_50);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_52, __pyx_pf_4dipy_5align_13vector_fields_416__defaults__);
  if (PyDict_SetItem(__pyx_t_51, __pyx_kp_s_short_double, __pyx_t_52) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_52); __pyx_t_52 = 0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_52 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1_0__pyx_mdef_4dipy_5align_13vector_fields_227warp_3d_nn, 0, __pyx_n_s_warp_3d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__439)); if (unlikely(!__pyx_t_52)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_52);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_52, sizeof(__pyx_defaults28), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2049
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults28, __pyx_t_52)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2050
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults28, __pyx_t_52)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2051
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults28, __pyx_t_52)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2052
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults28, __pyx_t_52)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_52, __pyx_t_50);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_52, __pyx_pf_4dipy_5align_13vector_fields_418__defaults__);
  if (PyDict_SetItem(__pyx_t_51, __pyx_kp_s_int_float, __pyx_t_52) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_52); __pyx_t_52 = 0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_52 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1_1__pyx_mdef_4dipy_5align_13vector_fields_229warp_3d_nn, 0, __pyx_n_s_warp_3d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__439)); if (unlikely(!__pyx_t_52)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_52);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_52, sizeof(__pyx_defaults29), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2049
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults29, __pyx_t_52)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2050
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults29, __pyx_t_52)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2051
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults29, __pyx_t_52)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2052
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults29, __pyx_t_52)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_52, __pyx_t_50);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_52, __pyx_pf_4dipy_5align_13vector_fields_420__defaults__);
  if (PyDict_SetItem(__pyx_t_51, __pyx_kp_s_int_double, __pyx_t_52) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_52); __pyx_t_52 = 0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_52 = __pyx_FusedFunction_NewEx(&__pyx_fuse_2_0__pyx_mdef_4dipy_5align_13vector_fields_231warp_3d_nn, 0, __pyx_n_s_warp_3d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__439)); if (unlikely(!__pyx_t_52)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_52);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_52, sizeof(__pyx_defaults30), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2049
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults30, __pyx_t_52)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2050
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults30, __pyx_t_52)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2051
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults30, __pyx_t_52)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2052
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults30, __pyx_t_52)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_52, __pyx_t_50);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_52, __pyx_pf_4dipy_5align_13vector_fields_422__defaults__);
  if (PyDict_SetItem(__pyx_t_51, __pyx_kp_s_long_float, __pyx_t_52) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_52); __pyx_t_52 = 0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_52 = __pyx_FusedFunction_NewEx(&__pyx_fuse_2_1__pyx_mdef_4dipy_5align_13vector_fields_233warp_3d_nn, 0, __pyx_n_s_warp_3d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__439)); if (unlikely(!__pyx_t_52)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_52);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_52, sizeof(__pyx_defaults31), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2049
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults31, __pyx_t_52)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2050
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults31, __pyx_t_52)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2051
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults31, __pyx_t_52)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2052
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults31, __pyx_t_52)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_52, __pyx_t_50);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_52, __pyx_pf_4dipy_5align_13vector_fields_424__defaults__);
  if (PyDict_SetItem(__pyx_t_51, __pyx_kp_s_long_double, __pyx_t_52) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_52); __pyx_t_52 = 0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_52 = __pyx_FusedFunction_NewEx(&__pyx_fuse_3_0__pyx_mdef_4dipy_5align_13vector_fields_235warp_3d_nn, 0, __pyx_n_s_warp_3d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__439)); if (unlikely(!__pyx_t_52)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_52);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_52, sizeof(__pyx_defaults32), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2049
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults32, __pyx_t_52)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2050
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults32, __pyx_t_52)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2051
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults32, __pyx_t_52)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2052
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults32, __pyx_t_52)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_52, __pyx_t_50);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_52, __pyx_pf_4dipy_5align_13vector_fields_426__defaults__);
  if (PyDict_SetItem(__pyx_t_51, __pyx_kp_s_float_float, __pyx_t_52) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_52); __pyx_t_52 = 0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_52 = __pyx_FusedFunction_NewEx(&__pyx_fuse_3_1__pyx_mdef_4dipy_5align_13vector_fields_237warp_3d_nn, 0, __pyx_n_s_warp_3d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__439)); if (unlikely(!__pyx_t_52)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_52);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_52, sizeof(__pyx_defaults33), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2049
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults33, __pyx_t_52)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2050
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults33, __pyx_t_52)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2051
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults33, __pyx_t_52)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2052
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults33, __pyx_t_52)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_52, __pyx_t_50);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_52, __pyx_pf_4dipy_5align_13vector_fields_428__defaults__);
  if (PyDict_SetItem(__pyx_t_51, __pyx_kp_s_float_double, __pyx_t_52) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_52); __pyx_t_52 = 0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_52 = __pyx_FusedFunction_NewEx(&__pyx_fuse_4_0__pyx_mdef_4dipy_5align_13vector_fields_239warp_3d_nn, 0, __pyx_n_s_warp_3d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__439)); if (unlikely(!__pyx_t_52)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_52);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_52, sizeof(__pyx_defaults34), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2049
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults34, __pyx_t_52)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2050
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults34, __pyx_t_52)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2051
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults34, __pyx_t_52)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2052
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults34, __pyx_t_52)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_52, __pyx_t_50);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_52, __pyx_pf_4dipy_5align_13vector_fields_430__defaults__);
  if (PyDict_SetItem(__pyx_t_51, __pyx_kp_s_double_float, __pyx_t_52) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_52); __pyx_t_52 = 0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_52 = __pyx_FusedFunction_NewEx(&__pyx_fuse_4_1__pyx_mdef_4dipy_5align_13vector_fields_241warp_3d_nn, 0, __pyx_n_s_warp_3d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__439)); if (unlikely(!__pyx_t_52)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_52);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_52, sizeof(__pyx_defaults35), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2049
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults35, __pyx_t_52)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2050
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults35, __pyx_t_52)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2051
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults35, __pyx_t_52)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2052
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults35, __pyx_t_52)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_52, __pyx_t_50);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_52, __pyx_pf_4dipy_5align_13vector_fields_432__defaults__);
  if (PyDict_SetItem(__pyx_t_51, __pyx_kp_s_double_double, __pyx_t_52) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_52); __pyx_t_52 = 0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_52 = __pyx_FusedFunction_NewEx(&__pyx_fuse_5_0__pyx_mdef_4dipy_5align_13vector_fields_243warp_3d_nn, 0, __pyx_n_s_warp_3d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__439)); if (unlikely(!__pyx_t_52)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_52);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_52, sizeof(__pyx_defaults36), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2049
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults36, __pyx_t_52)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2050
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults36, __pyx_t_52)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2051
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults36, __pyx_t_52)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2052
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults36, __pyx_t_52)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_52, __pyx_t_50);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_52, __pyx_pf_4dipy_5align_13vector_fields_434__defaults__);
  if (PyDict_SetItem(__pyx_t_51, __pyx_kp_s_float_complex_float, __pyx_t_52) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_52); __pyx_t_52 = 0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_52 = __pyx_FusedFunction_NewEx(&__pyx_fuse_5_1__pyx_mdef_4dipy_5align_13vector_fields_245warp_3d_nn, 0, __pyx_n_s_warp_3d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__439)); if (unlikely(!__pyx_t_52)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_52);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_52, sizeof(__pyx_defaults37), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2049
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults37, __pyx_t_52)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2050
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults37, __pyx_t_52)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2051
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults37, __pyx_t_52)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2052
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults37, __pyx_t_52)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_52, __pyx_t_50);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_52, __pyx_pf_4dipy_5align_13vector_fields_436__defaults__);
  if (PyDict_SetItem(__pyx_t_51, __pyx_kp_s_float_complex_double, __pyx_t_52) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_52); __pyx_t_52 = 0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_52 = __pyx_FusedFunction_NewEx(&__pyx_fuse_6_0__pyx_mdef_4dipy_5align_13vector_fields_247warp_3d_nn, 0, __pyx_n_s_warp_3d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__439)); if (unlikely(!__pyx_t_52)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_52);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_52, sizeof(__pyx_defaults38), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2049
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults38, __pyx_t_52)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2050
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults38, __pyx_t_52)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2051
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults38, __pyx_t_52)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2052
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults38, __pyx_t_52)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_52, __pyx_t_50);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_52, __pyx_pf_4dipy_5align_13vector_fields_438__defaults__);
  if (PyDict_SetItem(__pyx_t_51, __pyx_kp_s_double_complex_float, __pyx_t_52) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_52); __pyx_t_52 = 0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_52 = __pyx_FusedFunction_NewEx(&__pyx_fuse_6_1__pyx_mdef_4dipy_5align_13vector_fields_249warp_3d_nn, 0, __pyx_n_s_warp_3d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__439)); if (unlikely(!__pyx_t_52)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_52);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_52, sizeof(__pyx_defaults39), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2049
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2049; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults39, __pyx_t_52)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2050
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2050; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults39, __pyx_t_52)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2051
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2051; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults39, __pyx_t_52)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2052
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 3D volume using using nearest-neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2052; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults39, __pyx_t_52)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_52, __pyx_t_50);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_52, __pyx_pf_4dipy_5align_13vector_fields_440__defaults__);
  if (PyDict_SetItem(__pyx_t_51, __pyx_kp_s_double_complex_double, __pyx_t_52) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_52); __pyx_t_52 = 0;

  /* "dipy/align/vector_fields.pyx":2048
 * 
 * 
 * def warp_3d_nn(number[:, :, :] volume, floating[:, :, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_52 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_43warp_3d_nn, 0, __pyx_n_s_warp_3d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__439)); if (unlikely(!__pyx_t_52)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_52);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_52, __pyx_t_50);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_52, __pyx_pf_4dipy_5align_13vector_fields_414__defaults__);
  ((__pyx_FusedFunctionObject *) __pyx_t_52)->__signatures__ = __pyx_t_51;
  __Pyx_GIVEREF(__pyx_t_51);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_warp_3d_nn, __pyx_t_52) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2048; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_52); __pyx_t_52 = 0;
  __Pyx_DECREF(__pyx_t_50); __pyx_t_50 = 0;

  /* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */
  __pyx_t_53 = PyTuple_New(1); if (unlikely(!__pyx_t_53)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_53);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_53, 0, Py_None);
  __Pyx_INCREF(Py_None);
  __pyx_k__205 = Py_None;
  __Pyx_GIVEREF(Py_None);
  __pyx_t_54 = PyDict_New(); if (unlikely(!__pyx_t_54)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_54);
  __pyx_t_55 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_253transform_3d_affine_nn, 0, __pyx_n_s_transform_3d_affine_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__441)); if (unlikely(!__pyx_t_55)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_55);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_55, sizeof(__pyx_defaults47), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2179
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,
 *                            double[:, :] affine=None):             # <<<<<<<<<<<<<<
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 * 
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2179; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults47, __pyx_t_55)->__pyx_arg_affine = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_55, __pyx_t_53);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_55, __pyx_pf_4dipy_5align_13vector_fields_456__defaults__);
  if (PyDict_SetItem(__pyx_t_54, __pyx_n_s_short, __pyx_t_55) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_55); __pyx_t_55 = 0;

  /* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */
  __pyx_t_55 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_255transform_3d_affine_nn, 0, __pyx_n_s_transform_3d_affine_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__441)); if (unlikely(!__pyx_t_55)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_55);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_55, sizeof(__pyx_defaults48), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2179
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,
 *                            double[:, :] affine=None):             # <<<<<<<<<<<<<<
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 * 
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2179; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults48, __pyx_t_55)->__pyx_arg_affine = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_55, __pyx_t_53);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_55, __pyx_pf_4dipy_5align_13vector_fields_458__defaults__);
  if (PyDict_SetItem(__pyx_t_54, __pyx_n_s_int, __pyx_t_55) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_55); __pyx_t_55 = 0;

  /* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */
  __pyx_t_55 = __pyx_FusedFunction_NewEx(&__pyx_fuse_2__pyx_mdef_4dipy_5align_13vector_fields_257transform_3d_affine_nn, 0, __pyx_n_s_transform_3d_affine_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__441)); if (unlikely(!__pyx_t_55)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_55);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_55, sizeof(__pyx_defaults49), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2179
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,
 *                            double[:, :] affine=None):             # <<<<<<<<<<<<<<
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 * 
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2179; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults49, __pyx_t_55)->__pyx_arg_affine = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_55, __pyx_t_53);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_55, __pyx_pf_4dipy_5align_13vector_fields_460__defaults__);
  if (PyDict_SetItem(__pyx_t_54, __pyx_n_s_long, __pyx_t_55) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_55); __pyx_t_55 = 0;

  /* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */
  __pyx_t_55 = __pyx_FusedFunction_NewEx(&__pyx_fuse_3__pyx_mdef_4dipy_5align_13vector_fields_259transform_3d_affine_nn, 0, __pyx_n_s_transform_3d_affine_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__441)); if (unlikely(!__pyx_t_55)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_55);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_55, sizeof(__pyx_defaults50), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2179
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,
 *                            double[:, :] affine=None):             # <<<<<<<<<<<<<<
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 * 
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2179; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults50, __pyx_t_55)->__pyx_arg_affine = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_55, __pyx_t_53);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_55, __pyx_pf_4dipy_5align_13vector_fields_462__defaults__);
  if (PyDict_SetItem(__pyx_t_54, __pyx_n_s_float, __pyx_t_55) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_55); __pyx_t_55 = 0;

  /* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */
  __pyx_t_55 = __pyx_FusedFunction_NewEx(&__pyx_fuse_4__pyx_mdef_4dipy_5align_13vector_fields_261transform_3d_affine_nn, 0, __pyx_n_s_transform_3d_affine_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__441)); if (unlikely(!__pyx_t_55)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_55);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_55, sizeof(__pyx_defaults51), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2179
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,
 *                            double[:, :] affine=None):             # <<<<<<<<<<<<<<
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 * 
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2179; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults51, __pyx_t_55)->__pyx_arg_affine = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_55, __pyx_t_53);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_55, __pyx_pf_4dipy_5align_13vector_fields_464__defaults__);
  if (PyDict_SetItem(__pyx_t_54, __pyx_n_s_double, __pyx_t_55) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_55); __pyx_t_55 = 0;

  /* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */
  __pyx_t_55 = __pyx_FusedFunction_NewEx(&__pyx_fuse_5__pyx_mdef_4dipy_5align_13vector_fields_263transform_3d_affine_nn, 0, __pyx_n_s_transform_3d_affine_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__441)); if (unlikely(!__pyx_t_55)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_55);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_55, sizeof(__pyx_defaults52), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2179
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,
 *                            double[:, :] affine=None):             # <<<<<<<<<<<<<<
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 * 
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2179; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults52, __pyx_t_55)->__pyx_arg_affine = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_55, __pyx_t_53);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_55, __pyx_pf_4dipy_5align_13vector_fields_466__defaults__);
  if (PyDict_SetItem(__pyx_t_54, __pyx_kp_s_float_complex, __pyx_t_55) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_55); __pyx_t_55 = 0;

  /* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */
  __pyx_t_55 = __pyx_FusedFunction_NewEx(&__pyx_fuse_6__pyx_mdef_4dipy_5align_13vector_fields_265transform_3d_affine_nn, 0, __pyx_n_s_transform_3d_affine_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__441)); if (unlikely(!__pyx_t_55)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_55);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_55, sizeof(__pyx_defaults53), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2179
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,
 *                            double[:, :] affine=None):             # <<<<<<<<<<<<<<
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 * 
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2179; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults53, __pyx_t_55)->__pyx_arg_affine = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_55, __pyx_t_53);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_55, __pyx_pf_4dipy_5align_13vector_fields_468__defaults__);
  if (PyDict_SetItem(__pyx_t_54, __pyx_kp_s_double_complex, __pyx_t_55) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_55); __pyx_t_55 = 0;

  /* "dipy/align/vector_fields.pyx":2178
 * 
 * 
 * def transform_3d_affine_nn(number[:, :, :] volume, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 3D volume by an affine transform with NN interpolation
 */
  __pyx_t_55 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_45transform_3d_affine_nn, 0, __pyx_n_s_transform_3d_affine_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__441)); if (unlikely(!__pyx_t_55)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_55);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_55, __pyx_t_53);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_55, __pyx_pf_4dipy_5align_13vector_fields_456__defaults__);
  ((__pyx_FusedFunctionObject *) __pyx_t_55)->__signatures__ = __pyx_t_54;
  __Pyx_GIVEREF(__pyx_t_54);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_transform_3d_affine_nn, __pyx_t_55) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2178; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_55); __pyx_t_55 = 0;
  __Pyx_DECREF(__pyx_t_53); __pyx_t_53 = 0;

  /* "dipy/align/vector_fields.pyx":2243
 * 
 * 
 * def warp_2d(floating[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 */
  __pyx_t_56 = PyTuple_New(4); if (unlikely(!__pyx_t_56)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_56);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_56, 0, Py_None);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_56, 1, Py_None);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_56, 2, Py_None);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_56, 3, Py_None);
  __Pyx_INCREF(Py_None);
  __pyx_k__217 = Py_None;
  __Pyx_GIVEREF(Py_None);
  __Pyx_INCREF(Py_None);
  __pyx_k__218 = Py_None;
  __Pyx_GIVEREF(Py_None);
  __pyx_t_57 = PyDict_New(); if (unlikely(!__pyx_t_57)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_57);
  __pyx_t_58 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_269warp_2d, 0, __pyx_n_s_warp_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__443)); if (unlikely(!__pyx_t_58)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_58);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_58, sizeof(__pyx_defaults56), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2244
 * 
 * def warp_2d(floating[:, :] image, floating[:, :, :] d1,
 *             double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_out=None,
 *             double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2244; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults56, __pyx_t_58)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2245
 * def warp_2d(floating[:, :] image, floating[:, :, :] d1,
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_disp=None,
 *             int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2245; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults56, __pyx_t_58)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2246
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 *             double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *             int[:] out_shape=None):
 *     r"""Warps a 2D image using bilinear interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2246; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults56, __pyx_t_58)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2247
 *             double[:, :] affine_idx_out=None,
 *             double[:, :] affine_disp=None,
 *             int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 2D image using bilinear interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2247; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults56, __pyx_t_58)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_58, __pyx_t_56);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_58, __pyx_pf_4dipy_5align_13vector_fields_474__defaults__);
  if (PyDict_SetItem(__pyx_t_57, __pyx_n_s_float, __pyx_t_58) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_58); __pyx_t_58 = 0;

  /* "dipy/align/vector_fields.pyx":2243
 * 
 * 
 * def warp_2d(floating[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 */
  __pyx_t_58 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_271warp_2d, 0, __pyx_n_s_warp_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__443)); if (unlikely(!__pyx_t_58)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_58);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_58, sizeof(__pyx_defaults57), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2244
 * 
 * def warp_2d(floating[:, :] image, floating[:, :, :] d1,
 *             double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_out=None,
 *             double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2244; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults57, __pyx_t_58)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2245
 * def warp_2d(floating[:, :] image, floating[:, :, :] d1,
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_disp=None,
 *             int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2245; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults57, __pyx_t_58)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2246
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 *             double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *             int[:] out_shape=None):
 *     r"""Warps a 2D image using bilinear interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2246; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults57, __pyx_t_58)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2247
 *             double[:, :] affine_idx_out=None,
 *             double[:, :] affine_disp=None,
 *             int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 2D image using bilinear interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2247; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults57, __pyx_t_58)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_58, __pyx_t_56);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_58, __pyx_pf_4dipy_5align_13vector_fields_476__defaults__);
  if (PyDict_SetItem(__pyx_t_57, __pyx_n_s_double, __pyx_t_58) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_58); __pyx_t_58 = 0;

  /* "dipy/align/vector_fields.pyx":2243
 * 
 * 
 * def warp_2d(floating[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *             double[:, :] affine_idx_in=None,
 *             double[:, :] affine_idx_out=None,
 */
  __pyx_t_58 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_47warp_2d, 0, __pyx_n_s_warp_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__443)); if (unlikely(!__pyx_t_58)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_58);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_58, __pyx_t_56);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_58, __pyx_pf_4dipy_5align_13vector_fields_474__defaults__);
  ((__pyx_FusedFunctionObject *) __pyx_t_58)->__signatures__ = __pyx_t_57;
  __Pyx_GIVEREF(__pyx_t_57);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_warp_2d, __pyx_t_58) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2243; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_58); __pyx_t_58 = 0;
  __Pyx_DECREF(__pyx_t_56); __pyx_t_56 = 0;

  /* "dipy/align/vector_fields.pyx":2357
 * 
 * 
 * def transform_2d_affine(floating[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                         double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with bilinear interp.
 */
  __pyx_t_59 = PyTuple_New(1); if (unlikely(!__pyx_t_59)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_59);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_59, 0, Py_None);
  __Pyx_INCREF(Py_None);
  __pyx_k__231 = Py_None;
  __Pyx_GIVEREF(Py_None);
  __pyx_t_60 = PyDict_New(); if (unlikely(!__pyx_t_60)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_60);
  __pyx_t_61 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_275transform_2d_affine, 0, __pyx_n_s_transform_2d_affine, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__445)); if (unlikely(!__pyx_t_61)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_61);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_61, sizeof(__pyx_defaults60), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2358
 * 
 * def transform_2d_affine(floating[:, :] image, int[:] ref_shape,
 *                         double[:, :] affine=None):             # <<<<<<<<<<<<<<
 *     r"""Transforms a 2D image by an affine transform with bilinear interp.
 * 
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2358; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults60, __pyx_t_61)->__pyx_arg_affine = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_61, __pyx_t_59);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_61, __pyx_pf_4dipy_5align_13vector_fields_482__defaults__);
  if (PyDict_SetItem(__pyx_t_60, __pyx_n_s_float, __pyx_t_61) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_61); __pyx_t_61 = 0;

  /* "dipy/align/vector_fields.pyx":2357
 * 
 * 
 * def transform_2d_affine(floating[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                         double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with bilinear interp.
 */
  __pyx_t_61 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_277transform_2d_affine, 0, __pyx_n_s_transform_2d_affine, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__445)); if (unlikely(!__pyx_t_61)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_61);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_61, sizeof(__pyx_defaults61), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2358
 * 
 * def transform_2d_affine(floating[:, :] image, int[:] ref_shape,
 *                         double[:, :] affine=None):             # <<<<<<<<<<<<<<
 *     r"""Transforms a 2D image by an affine transform with bilinear interp.
 * 
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2358; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults61, __pyx_t_61)->__pyx_arg_affine = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_61, __pyx_t_59);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_61, __pyx_pf_4dipy_5align_13vector_fields_484__defaults__);
  if (PyDict_SetItem(__pyx_t_60, __pyx_n_s_double, __pyx_t_61) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_61); __pyx_t_61 = 0;

  /* "dipy/align/vector_fields.pyx":2357
 * 
 * 
 * def transform_2d_affine(floating[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                         double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with bilinear interp.
 */
  __pyx_t_61 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_49transform_2d_affine, 0, __pyx_n_s_transform_2d_affine, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__445)); if (unlikely(!__pyx_t_61)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_61);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_61, __pyx_t_59);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_61, __pyx_pf_4dipy_5align_13vector_fields_482__defaults__);
  ((__pyx_FusedFunctionObject *) __pyx_t_61)->__signatures__ = __pyx_t_60;
  __Pyx_GIVEREF(__pyx_t_60);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_transform_2d_affine, __pyx_t_61) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2357; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_61); __pyx_t_61 = 0;
  __Pyx_DECREF(__pyx_t_59); __pyx_t_59 = 0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_62 = PyTuple_New(4); if (unlikely(!__pyx_t_62)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_62);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_62, 0, Py_None);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_62, 1, Py_None);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_62, 2, Py_None);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_62, 3, Py_None);
  __Pyx_INCREF(Py_None);
  __pyx_k__238 = Py_None;
  __Pyx_GIVEREF(Py_None);
  __Pyx_INCREF(Py_None);
  __pyx_k__239 = Py_None;
  __Pyx_GIVEREF(Py_None);
  __pyx_t_63 = PyDict_New(); if (unlikely(!__pyx_t_63)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_63);
  __pyx_t_64 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0_0__pyx_mdef_4dipy_5align_13vector_fields_281warp_2d_nn, 0, __pyx_n_s_warp_2d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__447)); if (unlikely(!__pyx_t_64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_64);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_64, sizeof(__pyx_defaults76), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2418
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults76, __pyx_t_64)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2419
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults76, __pyx_t_64)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2420
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 2D image using nearest neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults76, __pyx_t_64)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2421
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 2D image using nearest neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults76, __pyx_t_64)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_64, __pyx_t_62);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_64, __pyx_pf_4dipy_5align_13vector_fields_514__defaults__);
  if (PyDict_SetItem(__pyx_t_63, __pyx_kp_s_short_float, __pyx_t_64) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_64); __pyx_t_64 = 0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_64 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0_1__pyx_mdef_4dipy_5align_13vector_fields_283warp_2d_nn, 0, __pyx_n_s_warp_2d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__447)); if (unlikely(!__pyx_t_64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_64);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_64, sizeof(__pyx_defaults77), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2418
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults77, __pyx_t_64)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2419
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults77, __pyx_t_64)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2420
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 2D image using nearest neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults77, __pyx_t_64)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2421
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 2D image using nearest neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults77, __pyx_t_64)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_64, __pyx_t_62);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_64, __pyx_pf_4dipy_5align_13vector_fields_516__defaults__);
  if (PyDict_SetItem(__pyx_t_63, __pyx_kp_s_short_double, __pyx_t_64) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_64); __pyx_t_64 = 0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_64 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1_0__pyx_mdef_4dipy_5align_13vector_fields_285warp_2d_nn, 0, __pyx_n_s_warp_2d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__447)); if (unlikely(!__pyx_t_64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_64);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_64, sizeof(__pyx_defaults78), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2418
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults78, __pyx_t_64)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2419
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults78, __pyx_t_64)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2420
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 2D image using nearest neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults78, __pyx_t_64)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2421
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 2D image using nearest neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults78, __pyx_t_64)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_64, __pyx_t_62);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_64, __pyx_pf_4dipy_5align_13vector_fields_518__defaults__);
  if (PyDict_SetItem(__pyx_t_63, __pyx_kp_s_int_float, __pyx_t_64) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_64); __pyx_t_64 = 0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_64 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1_1__pyx_mdef_4dipy_5align_13vector_fields_287warp_2d_nn, 0, __pyx_n_s_warp_2d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__447)); if (unlikely(!__pyx_t_64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_64);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_64, sizeof(__pyx_defaults79), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2418
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults79, __pyx_t_64)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2419
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults79, __pyx_t_64)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2420
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 2D image using nearest neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults79, __pyx_t_64)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2421
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 2D image using nearest neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults79, __pyx_t_64)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_64, __pyx_t_62);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_64, __pyx_pf_4dipy_5align_13vector_fields_520__defaults__);
  if (PyDict_SetItem(__pyx_t_63, __pyx_kp_s_int_double, __pyx_t_64) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_64); __pyx_t_64 = 0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_64 = __pyx_FusedFunction_NewEx(&__pyx_fuse_2_0__pyx_mdef_4dipy_5align_13vector_fields_289warp_2d_nn, 0, __pyx_n_s_warp_2d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__447)); if (unlikely(!__pyx_t_64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_64);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_64, sizeof(__pyx_defaults80), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2418
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults80, __pyx_t_64)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2419
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults80, __pyx_t_64)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2420
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 2D image using nearest neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults80, __pyx_t_64)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2421
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 2D image using nearest neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults80, __pyx_t_64)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_64, __pyx_t_62);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_64, __pyx_pf_4dipy_5align_13vector_fields_522__defaults__);
  if (PyDict_SetItem(__pyx_t_63, __pyx_kp_s_long_float, __pyx_t_64) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_64); __pyx_t_64 = 0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_64 = __pyx_FusedFunction_NewEx(&__pyx_fuse_2_1__pyx_mdef_4dipy_5align_13vector_fields_291warp_2d_nn, 0, __pyx_n_s_warp_2d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__447)); if (unlikely(!__pyx_t_64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_64);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_64, sizeof(__pyx_defaults81), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2418
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults81, __pyx_t_64)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2419
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults81, __pyx_t_64)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2420
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 2D image using nearest neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults81, __pyx_t_64)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2421
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 2D image using nearest neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults81, __pyx_t_64)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_64, __pyx_t_62);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_64, __pyx_pf_4dipy_5align_13vector_fields_524__defaults__);
  if (PyDict_SetItem(__pyx_t_63, __pyx_kp_s_long_double, __pyx_t_64) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_64); __pyx_t_64 = 0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_64 = __pyx_FusedFunction_NewEx(&__pyx_fuse_3_0__pyx_mdef_4dipy_5align_13vector_fields_293warp_2d_nn, 0, __pyx_n_s_warp_2d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__447)); if (unlikely(!__pyx_t_64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_64);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_64, sizeof(__pyx_defaults82), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2418
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults82, __pyx_t_64)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2419
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults82, __pyx_t_64)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2420
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 2D image using nearest neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults82, __pyx_t_64)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2421
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 2D image using nearest neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults82, __pyx_t_64)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_64, __pyx_t_62);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_64, __pyx_pf_4dipy_5align_13vector_fields_526__defaults__);
  if (PyDict_SetItem(__pyx_t_63, __pyx_kp_s_float_float, __pyx_t_64) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_64); __pyx_t_64 = 0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_64 = __pyx_FusedFunction_NewEx(&__pyx_fuse_3_1__pyx_mdef_4dipy_5align_13vector_fields_295warp_2d_nn, 0, __pyx_n_s_warp_2d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__447)); if (unlikely(!__pyx_t_64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_64);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_64, sizeof(__pyx_defaults83), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2418
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults83, __pyx_t_64)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2419
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults83, __pyx_t_64)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2420
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 2D image using nearest neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults83, __pyx_t_64)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2421
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 2D image using nearest neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults83, __pyx_t_64)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_64, __pyx_t_62);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_64, __pyx_pf_4dipy_5align_13vector_fields_528__defaults__);
  if (PyDict_SetItem(__pyx_t_63, __pyx_kp_s_float_double, __pyx_t_64) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_64); __pyx_t_64 = 0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_64 = __pyx_FusedFunction_NewEx(&__pyx_fuse_4_0__pyx_mdef_4dipy_5align_13vector_fields_297warp_2d_nn, 0, __pyx_n_s_warp_2d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__447)); if (unlikely(!__pyx_t_64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_64);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_64, sizeof(__pyx_defaults84), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2418
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults84, __pyx_t_64)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2419
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults84, __pyx_t_64)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2420
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 2D image using nearest neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults84, __pyx_t_64)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2421
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 2D image using nearest neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults84, __pyx_t_64)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_64, __pyx_t_62);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_64, __pyx_pf_4dipy_5align_13vector_fields_530__defaults__);
  if (PyDict_SetItem(__pyx_t_63, __pyx_kp_s_double_float, __pyx_t_64) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_64); __pyx_t_64 = 0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_64 = __pyx_FusedFunction_NewEx(&__pyx_fuse_4_1__pyx_mdef_4dipy_5align_13vector_fields_299warp_2d_nn, 0, __pyx_n_s_warp_2d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__447)); if (unlikely(!__pyx_t_64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_64);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_64, sizeof(__pyx_defaults85), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2418
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults85, __pyx_t_64)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2419
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults85, __pyx_t_64)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2420
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 2D image using nearest neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults85, __pyx_t_64)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2421
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 2D image using nearest neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults85, __pyx_t_64)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_64, __pyx_t_62);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_64, __pyx_pf_4dipy_5align_13vector_fields_532__defaults__);
  if (PyDict_SetItem(__pyx_t_63, __pyx_kp_s_double_double, __pyx_t_64) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_64); __pyx_t_64 = 0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_64 = __pyx_FusedFunction_NewEx(&__pyx_fuse_5_0__pyx_mdef_4dipy_5align_13vector_fields_301warp_2d_nn, 0, __pyx_n_s_warp_2d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__447)); if (unlikely(!__pyx_t_64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_64);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_64, sizeof(__pyx_defaults86), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2418
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults86, __pyx_t_64)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2419
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults86, __pyx_t_64)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2420
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 2D image using nearest neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults86, __pyx_t_64)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2421
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 2D image using nearest neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults86, __pyx_t_64)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_64, __pyx_t_62);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_64, __pyx_pf_4dipy_5align_13vector_fields_534__defaults__);
  if (PyDict_SetItem(__pyx_t_63, __pyx_kp_s_float_complex_float, __pyx_t_64) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_64); __pyx_t_64 = 0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_64 = __pyx_FusedFunction_NewEx(&__pyx_fuse_5_1__pyx_mdef_4dipy_5align_13vector_fields_303warp_2d_nn, 0, __pyx_n_s_warp_2d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__447)); if (unlikely(!__pyx_t_64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_64);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_64, sizeof(__pyx_defaults87), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2418
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults87, __pyx_t_64)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2419
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults87, __pyx_t_64)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2420
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 2D image using nearest neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults87, __pyx_t_64)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2421
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 2D image using nearest neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults87, __pyx_t_64)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_64, __pyx_t_62);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_64, __pyx_pf_4dipy_5align_13vector_fields_536__defaults__);
  if (PyDict_SetItem(__pyx_t_63, __pyx_kp_s_float_complex_double, __pyx_t_64) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_64); __pyx_t_64 = 0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_64 = __pyx_FusedFunction_NewEx(&__pyx_fuse_6_0__pyx_mdef_4dipy_5align_13vector_fields_305warp_2d_nn, 0, __pyx_n_s_warp_2d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__447)); if (unlikely(!__pyx_t_64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_64);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_64, sizeof(__pyx_defaults88), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2418
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults88, __pyx_t_64)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2419
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults88, __pyx_t_64)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2420
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 2D image using nearest neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults88, __pyx_t_64)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2421
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 2D image using nearest neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults88, __pyx_t_64)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_64, __pyx_t_62);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_64, __pyx_pf_4dipy_5align_13vector_fields_538__defaults__);
  if (PyDict_SetItem(__pyx_t_63, __pyx_kp_s_double_complex_float, __pyx_t_64) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_64); __pyx_t_64 = 0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_64 = __pyx_FusedFunction_NewEx(&__pyx_fuse_6_1__pyx_mdef_4dipy_5align_13vector_fields_307warp_2d_nn, 0, __pyx_n_s_warp_2d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__447)); if (unlikely(!__pyx_t_64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_64);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_64, sizeof(__pyx_defaults89), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2418
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2418; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults89, __pyx_t_64)->__pyx_arg_affine_idx_in = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2419
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2419; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults89, __pyx_t_64)->__pyx_arg_affine_idx_out = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2420
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,             # <<<<<<<<<<<<<<
 *                int[:] out_shape=None):
 *     r"""Warps a 2D image using nearest neighbor interpolation
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults89, __pyx_t_64)->__pyx_arg_affine_disp = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;

  /* "dipy/align/vector_fields.pyx":2421
 *                double[:, :] affine_idx_out=None,
 *                double[:, :] affine_disp=None,
 *                int[:] out_shape=None):             # <<<<<<<<<<<<<<
 *     r"""Warps a 2D image using nearest neighbor interpolation
 * 
 */
  __pyx_t_47 = __Pyx_PyObject_to_MemoryviewSlice_ds_int(Py_None);
  if (unlikely(!__pyx_t_47.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2421; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults89, __pyx_t_64)->__pyx_arg_out_shape = __pyx_t_47;
  __pyx_t_47.memview = NULL;
  __pyx_t_47.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_64, __pyx_t_62);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_64, __pyx_pf_4dipy_5align_13vector_fields_540__defaults__);
  if (PyDict_SetItem(__pyx_t_63, __pyx_kp_s_double_complex_double, __pyx_t_64) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_64); __pyx_t_64 = 0;

  /* "dipy/align/vector_fields.pyx":2417
 * 
 * 
 * def warp_2d_nn(number[:, :] image, floating[:, :, :] d1,             # <<<<<<<<<<<<<<
 *                double[:, :] affine_idx_in=None,
 *                double[:, :] affine_idx_out=None,
 */
  __pyx_t_64 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_51warp_2d_nn, 0, __pyx_n_s_warp_2d_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__447)); if (unlikely(!__pyx_t_64)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_64);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_64, __pyx_t_62);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_64, __pyx_pf_4dipy_5align_13vector_fields_514__defaults__);
  ((__pyx_FusedFunctionObject *) __pyx_t_64)->__signatures__ = __pyx_t_63;
  __Pyx_GIVEREF(__pyx_t_63);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_warp_2d_nn, __pyx_t_64) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2417; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_64); __pyx_t_64 = 0;
  __Pyx_DECREF(__pyx_t_62); __pyx_t_62 = 0;

  /* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */
  __pyx_t_65 = PyTuple_New(1); if (unlikely(!__pyx_t_65)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_65);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  PyTuple_SET_ITEM(__pyx_t_65, 0, Py_None);
  __Pyx_INCREF(Py_None);
  __pyx_k__300 = Py_None;
  __Pyx_GIVEREF(Py_None);
  __pyx_t_66 = PyDict_New(); if (unlikely(!__pyx_t_66)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_66);
  __pyx_t_67 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_311transform_2d_affine_nn, 0, __pyx_n_s_transform_2d_affine_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__449)); if (unlikely(!__pyx_t_67)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_67);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_67, sizeof(__pyx_defaults97), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2532
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,
 *                            double[:, :] affine=None):             # <<<<<<<<<<<<<<
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 * 
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2532; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults97, __pyx_t_67)->__pyx_arg_affine = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_67, __pyx_t_65);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_67, __pyx_pf_4dipy_5align_13vector_fields_556__defaults__);
  if (PyDict_SetItem(__pyx_t_66, __pyx_n_s_short, __pyx_t_67) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_67); __pyx_t_67 = 0;

  /* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */
  __pyx_t_67 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_313transform_2d_affine_nn, 0, __pyx_n_s_transform_2d_affine_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__449)); if (unlikely(!__pyx_t_67)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_67);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_67, sizeof(__pyx_defaults98), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2532
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,
 *                            double[:, :] affine=None):             # <<<<<<<<<<<<<<
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 * 
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2532; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults98, __pyx_t_67)->__pyx_arg_affine = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_67, __pyx_t_65);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_67, __pyx_pf_4dipy_5align_13vector_fields_558__defaults__);
  if (PyDict_SetItem(__pyx_t_66, __pyx_n_s_int, __pyx_t_67) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_67); __pyx_t_67 = 0;

  /* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */
  __pyx_t_67 = __pyx_FusedFunction_NewEx(&__pyx_fuse_2__pyx_mdef_4dipy_5align_13vector_fields_315transform_2d_affine_nn, 0, __pyx_n_s_transform_2d_affine_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__449)); if (unlikely(!__pyx_t_67)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_67);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_67, sizeof(__pyx_defaults99), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2532
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,
 *                            double[:, :] affine=None):             # <<<<<<<<<<<<<<
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 * 
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2532; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults99, __pyx_t_67)->__pyx_arg_affine = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_67, __pyx_t_65);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_67, __pyx_pf_4dipy_5align_13vector_fields_560__defaults__);
  if (PyDict_SetItem(__pyx_t_66, __pyx_n_s_long, __pyx_t_67) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_67); __pyx_t_67 = 0;

  /* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */
  __pyx_t_67 = __pyx_FusedFunction_NewEx(&__pyx_fuse_3__pyx_mdef_4dipy_5align_13vector_fields_317transform_2d_affine_nn, 0, __pyx_n_s_transform_2d_affine_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__449)); if (unlikely(!__pyx_t_67)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_67);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_67, sizeof(__pyx_defaults100), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2532
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,
 *                            double[:, :] affine=None):             # <<<<<<<<<<<<<<
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 * 
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2532; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults100, __pyx_t_67)->__pyx_arg_affine = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_67, __pyx_t_65);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_67, __pyx_pf_4dipy_5align_13vector_fields_562__defaults__);
  if (PyDict_SetItem(__pyx_t_66, __pyx_n_s_float, __pyx_t_67) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_67); __pyx_t_67 = 0;

  /* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */
  __pyx_t_67 = __pyx_FusedFunction_NewEx(&__pyx_fuse_4__pyx_mdef_4dipy_5align_13vector_fields_319transform_2d_affine_nn, 0, __pyx_n_s_transform_2d_affine_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__449)); if (unlikely(!__pyx_t_67)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_67);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_67, sizeof(__pyx_defaults101), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2532
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,
 *                            double[:, :] affine=None):             # <<<<<<<<<<<<<<
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 * 
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2532; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults101, __pyx_t_67)->__pyx_arg_affine = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_67, __pyx_t_65);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_67, __pyx_pf_4dipy_5align_13vector_fields_564__defaults__);
  if (PyDict_SetItem(__pyx_t_66, __pyx_n_s_double, __pyx_t_67) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_67); __pyx_t_67 = 0;

  /* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */
  __pyx_t_67 = __pyx_FusedFunction_NewEx(&__pyx_fuse_5__pyx_mdef_4dipy_5align_13vector_fields_321transform_2d_affine_nn, 0, __pyx_n_s_transform_2d_affine_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__449)); if (unlikely(!__pyx_t_67)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_67);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_67, sizeof(__pyx_defaults102), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2532
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,
 *                            double[:, :] affine=None):             # <<<<<<<<<<<<<<
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 * 
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2532; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults102, __pyx_t_67)->__pyx_arg_affine = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_67, __pyx_t_65);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_67, __pyx_pf_4dipy_5align_13vector_fields_566__defaults__);
  if (PyDict_SetItem(__pyx_t_66, __pyx_kp_s_float_complex, __pyx_t_67) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_67); __pyx_t_67 = 0;

  /* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */
  __pyx_t_67 = __pyx_FusedFunction_NewEx(&__pyx_fuse_6__pyx_mdef_4dipy_5align_13vector_fields_323transform_2d_affine_nn, 0, __pyx_n_s_transform_2d_affine_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__449)); if (unlikely(!__pyx_t_67)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_67);
  if (!__Pyx_CyFunction_InitDefaults(__pyx_t_67, sizeof(__pyx_defaults103), 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

  /* "dipy/align/vector_fields.pyx":2532
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,
 *                            double[:, :] affine=None):             # <<<<<<<<<<<<<<
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 * 
 */
  __pyx_t_46 = __Pyx_PyObject_to_MemoryviewSlice_dsds_double(Py_None);
  if (unlikely(!__pyx_t_46.memview)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2532; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_CyFunction_Defaults(__pyx_defaults103, __pyx_t_67)->__pyx_arg_affine = __pyx_t_46;
  __pyx_t_46.memview = NULL;
  __pyx_t_46.data = NULL;
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_67, __pyx_t_65);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_67, __pyx_pf_4dipy_5align_13vector_fields_568__defaults__);
  if (PyDict_SetItem(__pyx_t_66, __pyx_kp_s_double_complex, __pyx_t_67) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_67); __pyx_t_67 = 0;

  /* "dipy/align/vector_fields.pyx":2531
 * 
 * 
 * def transform_2d_affine_nn(number[:, :] image, int[:] ref_shape,             # <<<<<<<<<<<<<<
 *                            double[:, :] affine=None):
 *     r"""Transforms a 2D image by an affine transform with NN interpolation
 */
  __pyx_t_67 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_53transform_2d_affine_nn, 0, __pyx_n_s_transform_2d_affine_nn, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__449)); if (unlikely(!__pyx_t_67)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_67);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_67, __pyx_t_65);
  __Pyx_CyFunction_SetDefaultsGetter(__pyx_t_67, __pyx_pf_4dipy_5align_13vector_fields_556__defaults__);
  ((__pyx_FusedFunctionObject *) __pyx_t_67)->__signatures__ = __pyx_t_66;
  __Pyx_GIVEREF(__pyx_t_66);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_transform_2d_affine_nn, __pyx_t_67) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2531; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_67); __pyx_t_67 = 0;
  __Pyx_DECREF(__pyx_t_65); __pyx_t_65 = 0;

  /* "dipy/align/vector_fields.pyx":2591
 * 
 * 
 * def resample_displacement_field_3d(floating[:, :, :, :] field,             # <<<<<<<<<<<<<<
 *                                    double[:] factors, int[:] out_shape):
 *     r"""Resamples a 3D vector field to a custom target shape
 */
  __pyx_t_68 = PyDict_New(); if (unlikely(!__pyx_t_68)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_68);
  __pyx_t_69 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_327resample_displacement_field_3d, 0, __pyx_n_s_resample_displacement_field_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__451)); if (unlikely(!__pyx_t_69)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_69);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_69, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_68, __pyx_n_s_float, __pyx_t_69) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_69); __pyx_t_69 = 0;
  __pyx_t_69 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_329resample_displacement_field_3d, 0, __pyx_n_s_resample_displacement_field_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__451)); if (unlikely(!__pyx_t_69)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_69);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_69, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_68, __pyx_n_s_double, __pyx_t_69) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_69); __pyx_t_69 = 0;
  __pyx_t_69 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_55resample_displacement_field_3d, 0, __pyx_n_s_resample_displacement_field_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__451)); if (unlikely(!__pyx_t_69)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_69);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_69, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_69)->__signatures__ = __pyx_t_68;
  __Pyx_GIVEREF(__pyx_t_68);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_resample_displacement_field_3d, __pyx_t_69) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2591; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_69); __pyx_t_69 = 0;

  /* "dipy/align/vector_fields.pyx":2636
 * 
 * 
 * def resample_displacement_field_2d(floating[:, :, :] field, double[:] factors,             # <<<<<<<<<<<<<<
 *                                    int[:] out_shape):
 *     r"""Resamples a 2D vector field to a custom target shape
 */
  __pyx_t_70 = PyDict_New(); if (unlikely(!__pyx_t_70)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_70);
  __pyx_t_71 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_333resample_displacement_field_2d, 0, __pyx_n_s_resample_displacement_field_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__453)); if (unlikely(!__pyx_t_71)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_71);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_71, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_70, __pyx_n_s_float, __pyx_t_71) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_71); __pyx_t_71 = 0;
  __pyx_t_71 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_335resample_displacement_field_2d, 0, __pyx_n_s_resample_displacement_field_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__453)); if (unlikely(!__pyx_t_71)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_71);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_71, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_70, __pyx_n_s_double, __pyx_t_71) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_71); __pyx_t_71 = 0;
  __pyx_t_71 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_57resample_displacement_field_2d, 0, __pyx_n_s_resample_displacement_field_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__453)); if (unlikely(!__pyx_t_71)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_71);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_71, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_71)->__signatures__ = __pyx_t_70;
  __Pyx_GIVEREF(__pyx_t_70);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_resample_displacement_field_2d, __pyx_t_71) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2636; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_71); __pyx_t_71 = 0;

  /* "dipy/align/vector_fields.pyx":2677
 * 
 * 
 * def create_random_displacement_2d(int[:] from_shape,             # <<<<<<<<<<<<<<
 *                                   double[:, :] from_grid2world,
 *                                   int[:] to_shape,
 */
  __pyx_t_72 = PyCFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_59create_random_displacement_2d, NULL, __pyx_n_s_dipy_align_vector_fields); if (unlikely(!__pyx_t_72)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2677; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_72);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_random_displacement_2d, __pyx_t_72) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2677; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_72); __pyx_t_72 = 0;

  /* "dipy/align/vector_fields.pyx":2759
 * 
 * 
 * def create_random_displacement_3d(int[:] from_shape, double[:, :] from_grid2world,             # <<<<<<<<<<<<<<
 *                                   int[:] to_shape, double[:, :] to_grid2world):
 *     r"""Creates a random 3D displacement 'exactly' mapping points of two grids
 */
  __pyx_t_72 = PyCFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_61create_random_displacement_3d, NULL, __pyx_n_s_dipy_align_vector_fields); if (unlikely(!__pyx_t_72)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2759; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_72);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_random_displacement_3d, __pyx_t_72) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2759; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_72); __pyx_t_72 = 0;

  /* "dipy/align/vector_fields.pyx":2846
 * 
 * 
 * def create_harmonic_fields_2d(cnp.npy_intp nrows, cnp.npy_intp ncols,             # <<<<<<<<<<<<<<
 *                               double b, double m):
 *     r"""Creates an invertible 2D displacement field
 */
  __pyx_t_72 = PyCFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_63create_harmonic_fields_2d, NULL, __pyx_n_s_dipy_align_vector_fields); if (unlikely(!__pyx_t_72)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2846; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_72);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_harmonic_fields_2d, __pyx_t_72) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2846; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_72); __pyx_t_72 = 0;

  /* "dipy/align/vector_fields.pyx":2896
 * 
 * 
 * def create_harmonic_fields_3d(int nslices, cnp.npy_intp nrows,             # <<<<<<<<<<<<<<
 *                               cnp.npy_intp ncols, double b, double m):
 *     r"""Creates an invertible 3D displacement field
 */
  __pyx_t_72 = PyCFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_65create_harmonic_fields_3d, NULL, __pyx_n_s_dipy_align_vector_fields); if (unlikely(!__pyx_t_72)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2896; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_72);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_harmonic_fields_3d, __pyx_t_72) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2896; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_72); __pyx_t_72 = 0;

  /* "dipy/align/vector_fields.pyx":2955
 * 
 * 
 * def create_circle(cnp.npy_intp nrows, cnp.npy_intp ncols, cnp.npy_intp radius):             # <<<<<<<<<<<<<<
 *     r"""
 *     Create a binary 2D image where pixel values are 1 iff their distance
 */
  __pyx_t_72 = PyCFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_67create_circle, NULL, __pyx_n_s_dipy_align_vector_fields); if (unlikely(!__pyx_t_72)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2955; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_72);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_circle, __pyx_t_72) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2955; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_72); __pyx_t_72 = 0;

  /* "dipy/align/vector_fields.pyx":2992
 * 
 * 
 * def create_sphere(cnp.npy_intp nslices, cnp.npy_intp nrows,             # <<<<<<<<<<<<<<
 *                   cnp.npy_intp ncols, cnp.npy_intp radius):
 *     r"""
 */
  __pyx_t_72 = PyCFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_69create_sphere, NULL, __pyx_n_s_dipy_align_vector_fields); if (unlikely(!__pyx_t_72)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2992; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_72);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_create_sphere, __pyx_t_72) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 2992; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_72); __pyx_t_72 = 0;

  /* "dipy/align/vector_fields.pyx":3036
 * 
 * 
 * def _gradient_3d(floating[:, :, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                  double[:] img_spacing, double[:, :] out_grid2world,
 *                  floating[:, :, :, :] out, int[:, :, :] inside):
 */
  __pyx_t_72 = PyDict_New(); if (unlikely(!__pyx_t_72)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_72);
  __pyx_t_73 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_339_gradient_3d, 0, __pyx_n_s_gradient_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__467)); if (unlikely(!__pyx_t_73)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_73);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_73, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_72, __pyx_n_s_float, __pyx_t_73) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_73); __pyx_t_73 = 0;
  __pyx_t_73 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_341_gradient_3d, 0, __pyx_n_s_gradient_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__467)); if (unlikely(!__pyx_t_73)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_73);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_73, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_72, __pyx_n_s_double, __pyx_t_73) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_73); __pyx_t_73 = 0;
  __pyx_t_73 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_71_gradient_3d, 0, __pyx_n_s_gradient_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__467)); if (unlikely(!__pyx_t_73)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_73);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_73, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_73)->__signatures__ = __pyx_t_72;
  __Pyx_GIVEREF(__pyx_t_72);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gradient_3d, __pyx_t_73) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3036; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_73); __pyx_t_73 = 0;

  /* "dipy/align/vector_fields.pyx":3132
 * 
 * 
 * def _sparse_gradient_3d(floating[:, :, :] img,             # <<<<<<<<<<<<<<
 *                         double[:, :] img_world2grid,
 *                         double[:] img_spacing,
 */
  __pyx_t_74 = PyDict_New(); if (unlikely(!__pyx_t_74)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_74);
  __pyx_t_75 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_345_sparse_gradient_3d, 0, __pyx_n_s_sparse_gradient_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__469)); if (unlikely(!__pyx_t_75)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_75);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_75, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_74, __pyx_n_s_float, __pyx_t_75) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_75); __pyx_t_75 = 0;
  __pyx_t_75 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_347_sparse_gradient_3d, 0, __pyx_n_s_sparse_gradient_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__469)); if (unlikely(!__pyx_t_75)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_75);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_75, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_74, __pyx_n_s_double, __pyx_t_75) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_75); __pyx_t_75 = 0;
  __pyx_t_75 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_73_sparse_gradient_3d, 0, __pyx_n_s_sparse_gradient_3d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__469)); if (unlikely(!__pyx_t_75)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_75);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_75, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_75)->__signatures__ = __pyx_t_74;
  __Pyx_GIVEREF(__pyx_t_74);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sparse_gradient_3d, __pyx_t_75) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3132; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_75); __pyx_t_75 = 0;

  /* "dipy/align/vector_fields.pyx":3215
 * 
 * 
 * def _gradient_2d(floating[:, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                  double[:] img_spacing, double[:, :] out_grid2world,
 *                  floating[:, :, :] out, int[:, :] inside):
 */
  __pyx_t_76 = PyDict_New(); if (unlikely(!__pyx_t_76)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_76);
  __pyx_t_77 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_351_gradient_2d, 0, __pyx_n_s_gradient_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__471)); if (unlikely(!__pyx_t_77)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_77);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_77, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_76, __pyx_n_s_float, __pyx_t_77) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_77); __pyx_t_77 = 0;
  __pyx_t_77 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_353_gradient_2d, 0, __pyx_n_s_gradient_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__471)); if (unlikely(!__pyx_t_77)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_77);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_77, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_76, __pyx_n_s_double, __pyx_t_77) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_77); __pyx_t_77 = 0;
  __pyx_t_77 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_75_gradient_2d, 0, __pyx_n_s_gradient_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__471)); if (unlikely(!__pyx_t_77)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_77);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_77, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_77)->__signatures__ = __pyx_t_76;
  __Pyx_GIVEREF(__pyx_t_76);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gradient_2d, __pyx_t_77) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_77); __pyx_t_77 = 0;

  /* "dipy/align/vector_fields.pyx":3298
 * 
 * 
 * def _sparse_gradient_2d(floating[:, :] img, double[:, :] img_world2grid,             # <<<<<<<<<<<<<<
 *                         double[:] img_spacing,
 *                         double[:, :] sample_points,
 */
  __pyx_t_78 = PyDict_New(); if (unlikely(!__pyx_t_78)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_78);
  __pyx_t_79 = __pyx_FusedFunction_NewEx(&__pyx_fuse_0__pyx_mdef_4dipy_5align_13vector_fields_357_sparse_gradient_2d, 0, __pyx_n_s_sparse_gradient_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__473)); if (unlikely(!__pyx_t_79)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_79);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_79, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_78, __pyx_n_s_float, __pyx_t_79) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_79); __pyx_t_79 = 0;
  __pyx_t_79 = __pyx_FusedFunction_NewEx(&__pyx_fuse_1__pyx_mdef_4dipy_5align_13vector_fields_359_sparse_gradient_2d, 0, __pyx_n_s_sparse_gradient_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__473)); if (unlikely(!__pyx_t_79)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_79);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_79, __pyx_empty_tuple);
  if (PyDict_SetItem(__pyx_t_78, __pyx_n_s_double, __pyx_t_79) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_79); __pyx_t_79 = 0;
  __pyx_t_79 = __pyx_FusedFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_77_sparse_gradient_2d, 0, __pyx_n_s_sparse_gradient_2d, NULL, __pyx_n_s_dipy_align_vector_fields, __pyx_d, ((PyObject *)__pyx_codeobj__473)); if (unlikely(!__pyx_t_79)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_79);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_79, __pyx_empty_tuple);
  ((__pyx_FusedFunctionObject *) __pyx_t_79)->__signatures__ = __pyx_t_78;
  __Pyx_GIVEREF(__pyx_t_78);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sparse_gradient_2d, __pyx_t_79) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_79); __pyx_t_79 = 0;

  /* "dipy/align/vector_fields.pyx":3373
 * 
 * 
 * def gradient(img, img_world2grid, img_spacing, out_shape,             # <<<<<<<<<<<<<<
 *              out_grid2world):
 *     r""" Gradient of an image in physical space
 */
  __pyx_t_80 = PyCFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_79gradient, NULL, __pyx_n_s_dipy_align_vector_fields); if (unlikely(!__pyx_t_80)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3373; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_80);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_gradient, __pyx_t_80) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3373; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_80); __pyx_t_80 = 0;

  /* "dipy/align/vector_fields.pyx":3420
 * 
 * 
 * def sparse_gradient(img, img_world2grid, img_spacing, sample_points):             # <<<<<<<<<<<<<<
 *     r""" Gradient of an image in physical space
 * 
 */
  __pyx_t_80 = PyCFunction_NewEx(&__pyx_mdef_4dipy_5align_13vector_fields_81sparse_gradient, NULL, __pyx_n_s_dipy_align_vector_fields); if (unlikely(!__pyx_t_80)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_80);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sparse_gradient, __pyx_t_80) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 3420; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_80); __pyx_t_80 = 0;

  /* "dipy/align/vector_fields.pyx":1
 * #!python             # <<<<<<<<<<<<<<
 * #cython: boundscheck=False
 * #cython: wraparound=False
 */
  __pyx_t_80 = PyDict_New(); if (unlikely(!__pyx_t_80)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_80);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_80) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_80); __pyx_t_80 = 0;

  /* "View.MemoryView":205
 *         info.obj = self
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_array_getbuffer, "getbuffer(obj, view, flags)")             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(array self):
 */
  __pyx_t_80 = __pyx_capsule_create(((void *)(&__pyx_array_getbuffer)), __pyx_k_getbuffer_obj_view_flags); if (unlikely(!__pyx_t_80)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 205; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_80);
  if (PyDict_SetItem(__pyx_array_type->tp_dict, __pyx_n_s_pyx_getbuffer, __pyx_t_80) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 205; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_80); __pyx_t_80 = 0;
  PyType_Modified(__pyx_array_type);

  /* "View.MemoryView":278
 *         return self.name
 * 
 * cdef generic = Enum("<strided and direct or indirect>")             # <<<<<<<<<<<<<<
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")
 */
  __pyx_t_80 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__478, NULL); if (unlikely(!__pyx_t_80)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 278; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_80);
  __Pyx_XGOTREF(generic);
  __Pyx_DECREF_SET(generic, __pyx_t_80);
  __Pyx_GIVEREF(__pyx_t_80);
  __pyx_t_80 = 0;

  /* "View.MemoryView":279
 * 
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default             # <<<<<<<<<<<<<<
 * cdef indirect = Enum("<strided and indirect>")
 * 
 */
  __pyx_t_80 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__479, NULL); if (unlikely(!__pyx_t_80)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 279; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_80);
  __Pyx_XGOTREF(strided);
  __Pyx_DECREF_SET(strided, __pyx_t_80);
  __Pyx_GIVEREF(__pyx_t_80);
  __pyx_t_80 = 0;

  /* "View.MemoryView":280
 * cdef generic = Enum("<strided and direct or indirect>")
 * cdef strided = Enum("<strided and direct>") # default
 * cdef indirect = Enum("<strided and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_80 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__480, NULL); if (unlikely(!__pyx_t_80)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 280; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_80);
  __Pyx_XGOTREF(indirect);
  __Pyx_DECREF_SET(indirect, __pyx_t_80);
  __Pyx_GIVEREF(__pyx_t_80);
  __pyx_t_80 = 0;

  /* "View.MemoryView":283
 * 
 * 
 * cdef contiguous = Enum("<contiguous and direct>")             # <<<<<<<<<<<<<<
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")
 * 
 */
  __pyx_t_80 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__481, NULL); if (unlikely(!__pyx_t_80)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 283; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_80);
  __Pyx_XGOTREF(contiguous);
  __Pyx_DECREF_SET(contiguous, __pyx_t_80);
  __Pyx_GIVEREF(__pyx_t_80);
  __pyx_t_80 = 0;

  /* "View.MemoryView":284
 * 
 * cdef contiguous = Enum("<contiguous and direct>")
 * cdef indirect_contiguous = Enum("<contiguous and indirect>")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_80 = __Pyx_PyObject_Call(((PyObject *)__pyx_MemviewEnum_type), __pyx_tuple__482, NULL); if (unlikely(!__pyx_t_80)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 284; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_80);
  __Pyx_XGOTREF(indirect_contiguous);
  __Pyx_DECREF_SET(indirect_contiguous, __pyx_t_80);
  __Pyx_GIVEREF(__pyx_t_80);
  __pyx_t_80 = 0;

  /* "View.MemoryView":498
 *         info.obj = self
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_80 = __pyx_capsule_create(((void *)(&__pyx_memoryview_getbuffer)), __pyx_k_getbuffer_obj_view_flags); if (unlikely(!__pyx_t_80)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 498; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_80);
  if (PyDict_SetItem(__pyx_memoryview_type->tp_dict, __pyx_n_s_pyx_getbuffer, __pyx_t_80) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 498; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_80); __pyx_t_80 = 0;
  PyType_Modified(__pyx_memoryview_type);

  /* "View.MemoryView":954
 *             return self.from_object
 * 
 *     __pyx_getbuffer = capsule(<void *> &__pyx_memoryview_getbuffer, "getbuffer(obj, view, flags)")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_80 = __pyx_capsule_create(((void *)(&__pyx_memoryview_getbuffer)), __pyx_k_getbuffer_obj_view_flags); if (unlikely(!__pyx_t_80)) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 954; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_80);
  if (PyDict_SetItem(__pyx_memoryviewslice_type->tp_dict, __pyx_n_s_pyx_getbuffer, __pyx_t_80) < 0) {__pyx_filename = __pyx_f[2]; __pyx_lineno = 954; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(__pyx_t_80); __pyx_t_80 = 0;
  PyType_Modified(__pyx_memoryviewslice_type);

  /* "__pyxutil":2
 * 
 * cdef extern from *:             # <<<<<<<<<<<<<<
 *     void __pyx_PyErr_Clear "PyErr_Clear" ()
 *     __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(object)
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_XDECREF(__pyx_t_19);
  __PYX_XDEC_MEMVIEW(&__pyx_t_20, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_21, 1);
  __Pyx_XDECREF(__pyx_t_22);
  __Pyx_XDECREF(__pyx_t_23);
  __Pyx_XDECREF(__pyx_t_24);
  __PYX_XDEC_MEMVIEW(&__pyx_t_25, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_26, 1);
  __Pyx_XDECREF(__pyx_t_27);
  __Pyx_XDECREF(__pyx_t_28);
  __Pyx_XDECREF(__pyx_t_29);
  __Pyx_XDECREF(__pyx_t_30);
  __Pyx_XDECREF(__pyx_t_31);
  __Pyx_XDECREF(__pyx_t_32);
  __Pyx_XDECREF(__pyx_t_33);
  __Pyx_XDECREF(__pyx_t_34);
  __Pyx_XDECREF(__pyx_t_35);
  __Pyx_XDECREF(__pyx_t_36);
  __Pyx_XDECREF(__pyx_t_37);
  __Pyx_XDECREF(__pyx_t_38);
  __Pyx_XDECREF(__pyx_t_39);
  __Pyx_XDECREF(__pyx_t_40);
  __Pyx_XDECREF(__pyx_t_41);
  __Pyx_XDECREF(__pyx_t_42);
  __Pyx_XDECREF(__pyx_t_43);
  __Pyx_XDECREF(__pyx_t_44);
  __Pyx_XDECREF(__pyx_t_45);
  __PYX_XDEC_MEMVIEW(&__pyx_t_46, 1);
  __PYX_XDEC_MEMVIEW(&__pyx_t_47, 1);
  __Pyx_XDECREF(__pyx_t_48);
  __Pyx_XDECREF(__pyx_t_49);
  __Pyx_XDECREF(__pyx_t_50);
  __Pyx_XDECREF(__pyx_t_51);
  __Pyx_XDECREF(__pyx_t_52);
  __Pyx_XDECREF(__pyx_t_53);
  __Pyx_XDECREF(__pyx_t_54);
  __Pyx_XDECREF(__pyx_t_55);
  __Pyx_XDECREF(__pyx_t_56);
  __Pyx_XDECREF(__pyx_t_57);
  __Pyx_XDECREF(__pyx_t_58);
  __Pyx_XDECREF(__pyx_t_59);
  __Pyx_XDECREF(__pyx_t_60);
  __Pyx_XDECREF(__pyx_t_61);
  __Pyx_XDECREF(__pyx_t_62);
  __Pyx_XDECREF(__pyx_t_63);
  __Pyx_XDECREF(__pyx_t_64);
  __Pyx_XDECREF(__pyx_t_65);
  __Pyx_XDECREF(__pyx_t_66);
  __Pyx_XDECREF(__pyx_t_67);
  __Pyx_XDECREF(__pyx_t_68);
  __Pyx_XDECREF(__pyx_t_69);
  __Pyx_XDECREF(__pyx_t_70);
  __Pyx_XDECREF(__pyx_t_71);
  __Pyx_XDECREF(__pyx_t_72);
  __Pyx_XDECREF(__pyx_t_73);
  __Pyx_XDECREF(__pyx_t_74);
  __Pyx_XDECREF(__pyx_t_75);
  __Pyx_XDECREF(__pyx_t_76);
  __Pyx_XDECREF(__pyx_t_77);
  __Pyx_XDECREF(__pyx_t_78);
  __Pyx_XDECREF(__pyx_t_79);
  __Pyx_XDECREF(__pyx_t_80);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init dipy.align.vector_fields", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init dipy.align.vector_fields");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* --- Runtime support code --- */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject* args = PyTuple_Pack(1, arg);
    return (likely(args)) ? __Pyx_PyObject_Call(func, args, NULL) : NULL;
}
#endif

static CYTHON_INLINE int __Pyx_IsLittleEndian(void) {
  unsigned int n = 1;
  return *(unsigned char*)(&n) != 0;
}
static void __Pyx_BufFmt_Init(__Pyx_BufFmt_Context* ctx,
                              __Pyx_BufFmt_StackElem* stack,
                              __Pyx_TypeInfo* type) {
  stack[0].field = &ctx->root;
  stack[0].parent_offset = 0;
  ctx->root.type = type;
  ctx->root.name = "buffer dtype";
  ctx->root.offset = 0;
  ctx->head = stack;
  ctx->head->field = &ctx->root;
  ctx->fmt_offset = 0;
  ctx->head->parent_offset = 0;
  ctx->new_packmode = '@';
  ctx->enc_packmode = '@';
  ctx->new_count = 1;
  ctx->enc_count = 0;
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  ctx->is_valid_array = 0;
  ctx->struct_alignment = 0;
  while (type->typegroup == 'S') {
    ++ctx->head;
    ctx->head->field = type->fields;
    ctx->head->parent_offset = 0;
    type = type->fields->type;
  }
}
static int __Pyx_BufFmt_ParseNumber(const char** ts) {
    int count;
    const char* t = *ts;
    if (*t < '0' || *t > '9') {
      return -1;
    } else {
        count = *t++ - '0';
        while (*t >= '0' && *t < '9') {
            count *= 10;
            count += *t++ - '0';
        }
    }
    *ts = t;
    return count;
}
static int __Pyx_BufFmt_ExpectNumber(const char **ts) {
    int number = __Pyx_BufFmt_ParseNumber(ts);
    if (number == -1)
        PyErr_Format(PyExc_ValueError,\
                     "Does not understand character buffer dtype format string ('%c')", **ts);
    return number;
}
static void __Pyx_BufFmt_RaiseUnexpectedChar(char ch) {
  PyErr_Format(PyExc_ValueError,
               "Unexpected format string character: '%c'", ch);
}
static const char* __Pyx_BufFmt_DescribeTypeChar(char ch, int is_complex) {
  switch (ch) {
    case 'c': return "'char'";
    case 'b': return "'signed char'";
    case 'B': return "'unsigned char'";
    case 'h': return "'short'";
    case 'H': return "'unsigned short'";
    case 'i': return "'int'";
    case 'I': return "'unsigned int'";
    case 'l': return "'long'";
    case 'L': return "'unsigned long'";
    case 'q': return "'long long'";
    case 'Q': return "'unsigned long long'";
    case 'f': return (is_complex ? "'complex float'" : "'float'");
    case 'd': return (is_complex ? "'complex double'" : "'double'");
    case 'g': return (is_complex ? "'complex long double'" : "'long double'");
    case 'T': return "a struct";
    case 'O': return "Python object";
    case 'P': return "a pointer";
    case 's': case 'p': return "a string";
    case 0: return "end";
    default: return "unparseable format string";
  }
}
static size_t __Pyx_BufFmt_TypeCharToStandardSize(char ch, int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return 2;
    case 'i': case 'I': case 'l': case 'L': return 4;
    case 'q': case 'Q': return 8;
    case 'f': return (is_complex ? 8 : 4);
    case 'd': return (is_complex ? 16 : 8);
    case 'g': {
      PyErr_SetString(PyExc_ValueError, "Python does not define a standard format string size for long double ('g')..");
      return 0;
    }
    case 'O': case 'P': return sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static size_t __Pyx_BufFmt_TypeCharToNativeSize(char ch, int is_complex) {
  switch (ch) {
    case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(short);
    case 'i': case 'I': return sizeof(int);
    case 'l': case 'L': return sizeof(long);
    #ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(PY_LONG_LONG);
    #endif
    case 'f': return sizeof(float) * (is_complex ? 2 : 1);
    case 'd': return sizeof(double) * (is_complex ? 2 : 1);
    case 'g': return sizeof(long double) * (is_complex ? 2 : 1);
    case 'O': case 'P': return sizeof(void*);
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
typedef struct { char c; short x; } __Pyx_st_short;
typedef struct { char c; int x; } __Pyx_st_int;
typedef struct { char c; long x; } __Pyx_st_long;
typedef struct { char c; float x; } __Pyx_st_float;
typedef struct { char c; double x; } __Pyx_st_double;
typedef struct { char c; long double x; } __Pyx_st_longdouble;
typedef struct { char c; void *x; } __Pyx_st_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { char c; PY_LONG_LONG x; } __Pyx_st_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToAlignment(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_st_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_st_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_st_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_st_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_st_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_st_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_st_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_st_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
/* These are for computing the padding at the end of the struct to align
   on the first member of the struct. This will probably the same as above,
   but we don't have any guarantees.
 */
typedef struct { short x; char c; } __Pyx_pad_short;
typedef struct { int x; char c; } __Pyx_pad_int;
typedef struct { long x; char c; } __Pyx_pad_long;
typedef struct { float x; char c; } __Pyx_pad_float;
typedef struct { double x; char c; } __Pyx_pad_double;
typedef struct { long double x; char c; } __Pyx_pad_longdouble;
typedef struct { void *x; char c; } __Pyx_pad_void_p;
#ifdef HAVE_LONG_LONG
typedef struct { PY_LONG_LONG x; char c; } __Pyx_pad_longlong;
#endif
static size_t __Pyx_BufFmt_TypeCharToPadding(char ch, CYTHON_UNUSED int is_complex) {
  switch (ch) {
    case '?': case 'c': case 'b': case 'B': case 's': case 'p': return 1;
    case 'h': case 'H': return sizeof(__Pyx_pad_short) - sizeof(short);
    case 'i': case 'I': return sizeof(__Pyx_pad_int) - sizeof(int);
    case 'l': case 'L': return sizeof(__Pyx_pad_long) - sizeof(long);
#ifdef HAVE_LONG_LONG
    case 'q': case 'Q': return sizeof(__Pyx_pad_longlong) - sizeof(PY_LONG_LONG);
#endif
    case 'f': return sizeof(__Pyx_pad_float) - sizeof(float);
    case 'd': return sizeof(__Pyx_pad_double) - sizeof(double);
    case 'g': return sizeof(__Pyx_pad_longdouble) - sizeof(long double);
    case 'P': case 'O': return sizeof(__Pyx_pad_void_p) - sizeof(void*);
    default:
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
}
static char __Pyx_BufFmt_TypeCharToGroup(char ch, int is_complex) {
  switch (ch) {
    case 'c':
        return 'H';
    case 'b': case 'h': case 'i':
    case 'l': case 'q': case 's': case 'p':
        return 'I';
    case 'B': case 'H': case 'I': case 'L': case 'Q':
        return 'U';
    case 'f': case 'd': case 'g':
        return (is_complex ? 'C' : 'R');
    case 'O':
        return 'O';
    case 'P':
        return 'P';
    default: {
      __Pyx_BufFmt_RaiseUnexpectedChar(ch);
      return 0;
    }
  }
}
static void __Pyx_BufFmt_RaiseExpected(__Pyx_BufFmt_Context* ctx) {
  if (ctx->head == NULL || ctx->head->field == &ctx->root) {
    const char* expected;
    const char* quote;
    if (ctx->head == NULL) {
      expected = "end";
      quote = "";
    } else {
      expected = ctx->head->field->type->name;
      quote = "'";
    }
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected %s%s%s but got %s",
                 quote, expected, quote,
                 __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex));
  } else {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_StructField* parent = (ctx->head - 1)->field;
    PyErr_Format(PyExc_ValueError,
                 "Buffer dtype mismatch, expected '%s' but got %s in '%s.%s'",
                 field->type->name, __Pyx_BufFmt_DescribeTypeChar(ctx->enc_type, ctx->is_complex),
                 parent->type->name, field->name);
  }
}
static int __Pyx_BufFmt_ProcessTypeChunk(__Pyx_BufFmt_Context* ctx) {
  char group;
  size_t size, offset, arraysize = 1;
  if (ctx->enc_type == 0) return 0;
  if (ctx->head->field->type->arraysize[0]) {
    int i, ndim = 0;
    if (ctx->enc_type == 's' || ctx->enc_type == 'p') {
        ctx->is_valid_array = ctx->head->field->type->ndim == 1;
        ndim = 1;
        if (ctx->enc_count != ctx->head->field->type->arraysize[0]) {
            PyErr_Format(PyExc_ValueError,
                         "Expected a dimension of size %zu, got %zu",
                         ctx->head->field->type->arraysize[0], ctx->enc_count);
            return -1;
        }
    }
    if (!ctx->is_valid_array) {
      PyErr_Format(PyExc_ValueError, "Expected %d dimensions, got %d",
                   ctx->head->field->type->ndim, ndim);
      return -1;
    }
    for (i = 0; i < ctx->head->field->type->ndim; i++) {
      arraysize *= ctx->head->field->type->arraysize[i];
    }
    ctx->is_valid_array = 0;
    ctx->enc_count = 1;
  }
  group = __Pyx_BufFmt_TypeCharToGroup(ctx->enc_type, ctx->is_complex);
  do {
    __Pyx_StructField* field = ctx->head->field;
    __Pyx_TypeInfo* type = field->type;
    if (ctx->enc_packmode == '@' || ctx->enc_packmode == '^') {
      size = __Pyx_BufFmt_TypeCharToNativeSize(ctx->enc_type, ctx->is_complex);
    } else {
      size = __Pyx_BufFmt_TypeCharToStandardSize(ctx->enc_type, ctx->is_complex);
    }
    if (ctx->enc_packmode == '@') {
      size_t align_at = __Pyx_BufFmt_TypeCharToAlignment(ctx->enc_type, ctx->is_complex);
      size_t align_mod_offset;
      if (align_at == 0) return -1;
      align_mod_offset = ctx->fmt_offset % align_at;
      if (align_mod_offset > 0) ctx->fmt_offset += align_at - align_mod_offset;
      if (ctx->struct_alignment == 0)
          ctx->struct_alignment = __Pyx_BufFmt_TypeCharToPadding(ctx->enc_type,
                                                                 ctx->is_complex);
    }
    if (type->size != size || type->typegroup != group) {
      if (type->typegroup == 'C' && type->fields != NULL) {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        ++ctx->head;
        ctx->head->field = type->fields;
        ctx->head->parent_offset = parent_offset;
        continue;
      }
      if ((type->typegroup == 'H' || group == 'H') && type->size == size) {
      } else {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
      }
    }
    offset = ctx->head->parent_offset + field->offset;
    if (ctx->fmt_offset != offset) {
      PyErr_Format(PyExc_ValueError,
                   "Buffer dtype mismatch; next field is at offset %" CYTHON_FORMAT_SSIZE_T "d but %" CYTHON_FORMAT_SSIZE_T "d expected",
                   (Py_ssize_t)ctx->fmt_offset, (Py_ssize_t)offset);
      return -1;
    }
    ctx->fmt_offset += size;
    if (arraysize)
      ctx->fmt_offset += (arraysize - 1) * size;
    --ctx->enc_count;
    while (1) {
      if (field == &ctx->root) {
        ctx->head = NULL;
        if (ctx->enc_count != 0) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return -1;
        }
        break;
      }
      ctx->head->field = ++field;
      if (field->type == NULL) {
        --ctx->head;
        field = ctx->head->field;
        continue;
      } else if (field->type->typegroup == 'S') {
        size_t parent_offset = ctx->head->parent_offset + field->offset;
        if (field->type->fields->type == NULL) continue;
        field = field->type->fields;
        ++ctx->head;
        ctx->head->field = field;
        ctx->head->parent_offset = parent_offset;
        break;
      } else {
        break;
      }
    }
  } while (ctx->enc_count);
  ctx->enc_type = 0;
  ctx->is_complex = 0;
  return 0;
}
static CYTHON_INLINE PyObject *
__pyx_buffmt_parse_array(__Pyx_BufFmt_Context* ctx, const char** tsp)
{
    const char *ts = *tsp;
    int i = 0, number;
    int ndim = ctx->head->field->type->ndim;
;
    ++ts;
    if (ctx->new_count != 1) {
        PyErr_SetString(PyExc_ValueError,
                        "Cannot handle repeated arrays in format string");
        return NULL;
    }
    if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
    while (*ts && *ts != ')') {
        switch (*ts) {
            case ' ': case '\f': case '\r': case '\n': case '\t': case '\v':  continue;
            default:  break;
        }
        number = __Pyx_BufFmt_ExpectNumber(&ts);
        if (number == -1) return NULL;
        if (i < ndim && (size_t) number != ctx->head->field->type->arraysize[i])
            return PyErr_Format(PyExc_ValueError,
                        "Expected a dimension of size %zu, got %d",
                        ctx->head->field->type->arraysize[i], number);
        if (*ts != ',' && *ts != ')')
            return PyErr_Format(PyExc_ValueError,
                                "Expected a comma in format string, got '%c'", *ts);
        if (*ts == ',') ts++;
        i++;
    }
    if (i != ndim)
        return PyErr_Format(PyExc_ValueError, "Expected %d dimension(s), got %d",
                            ctx->head->field->type->ndim, i);
    if (!*ts) {
        PyErr_SetString(PyExc_ValueError,
                        "Unexpected end of format string, expected ')'");
        return NULL;
    }
    ctx->is_valid_array = 1;
    ctx->new_count = 1;
    *tsp = ++ts;
    return Py_None;
}
static const char* __Pyx_BufFmt_CheckString(__Pyx_BufFmt_Context* ctx, const char* ts) {
  int got_Z = 0;
  while (1) {
    switch(*ts) {
      case 0:
        if (ctx->enc_type != 0 && ctx->head == NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        if (ctx->head != NULL) {
          __Pyx_BufFmt_RaiseExpected(ctx);
          return NULL;
        }
        return ts;
      case ' ':
      case '\r':
      case '\n':
        ++ts;
        break;
      case '<':
        if (!__Pyx_IsLittleEndian()) {
          PyErr_SetString(PyExc_ValueError, "Little-endian buffer not supported on big-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '>':
      case '!':
        if (__Pyx_IsLittleEndian()) {
          PyErr_SetString(PyExc_ValueError, "Big-endian buffer not supported on little-endian compiler");
          return NULL;
        }
        ctx->new_packmode = '=';
        ++ts;
        break;
      case '=':
      case '@':
      case '^':
        ctx->new_packmode = *ts++;
        break;
      case 'T':
        {
          const char* ts_after_sub;
          size_t i, struct_count = ctx->new_count;
          size_t struct_alignment = ctx->struct_alignment;
          ctx->new_count = 1;
          ++ts;
          if (*ts != '{') {
            PyErr_SetString(PyExc_ValueError, "Buffer acquisition: Expected '{' after 'T'");
            return NULL;
          }
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          ctx->enc_count = 0;
          ctx->struct_alignment = 0;
          ++ts;
          ts_after_sub = ts;
          for (i = 0; i != struct_count; ++i) {
            ts_after_sub = __Pyx_BufFmt_CheckString(ctx, ts);
            if (!ts_after_sub) return NULL;
          }
          ts = ts_after_sub;
          if (struct_alignment) ctx->struct_alignment = struct_alignment;
        }
        break;
      case '}':
        {
          size_t alignment = ctx->struct_alignment;
          ++ts;
          if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
          ctx->enc_type = 0;
          if (alignment && ctx->fmt_offset % alignment) {
            ctx->fmt_offset += alignment - (ctx->fmt_offset % alignment);
          }
        }
        return ts;
      case 'x':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->fmt_offset += ctx->new_count;
        ctx->new_count = 1;
        ctx->enc_count = 0;
        ctx->enc_type = 0;
        ctx->enc_packmode = ctx->new_packmode;
        ++ts;
        break;
      case 'Z':
        got_Z = 1;
        ++ts;
        if (*ts != 'f' && *ts != 'd' && *ts != 'g') {
          __Pyx_BufFmt_RaiseUnexpectedChar('Z');
          return NULL;
        }
      case 'c': case 'b': case 'B': case 'h': case 'H': case 'i': case 'I':
      case 'l': case 'L': case 'q': case 'Q':
      case 'f': case 'd': case 'g':
      case 'O': case 'p':
        if (ctx->enc_type == *ts && got_Z == ctx->is_complex &&
            ctx->enc_packmode == ctx->new_packmode) {
          ctx->enc_count += ctx->new_count;
          ctx->new_count = 1;
          got_Z = 0;
          ++ts;
          break;
        }
      case 's':
        if (__Pyx_BufFmt_ProcessTypeChunk(ctx) == -1) return NULL;
        ctx->enc_count = ctx->new_count;
        ctx->enc_packmode = ctx->new_packmode;
        ctx->enc_type = *ts;
        ctx->is_complex = got_Z;
        ++ts;
        ctx->new_count = 1;
        got_Z = 0;
        break;
      case ':':
        ++ts;
        while(*ts != ':') ++ts;
        ++ts;
        break;
      case '(':
        if (!__pyx_buffmt_parse_array(ctx, &ts)) return NULL;
        break;
      default:
        {
          int number = __Pyx_BufFmt_ExpectNumber(&ts);
          if (number == -1) return NULL;
          ctx->new_count = (size_t)number;
        }
    }
  }
}
static CYTHON_INLINE void __Pyx_ZeroBuffer(Py_buffer* buf) {
  buf->buf = NULL;
  buf->obj = NULL;
  buf->strides = __Pyx_zeros;
  buf->shape = __Pyx_zeros;
  buf->suboffsets = __Pyx_minusones;
}
static CYTHON_INLINE int __Pyx_GetBufferAndValidate(
        Py_buffer* buf, PyObject* obj,  __Pyx_TypeInfo* dtype, int flags,
        int nd, int cast, __Pyx_BufFmt_StackElem* stack)
{
  if (obj == Py_None || obj == NULL) {
    __Pyx_ZeroBuffer(buf);
    return 0;
  }
  buf->buf = NULL;
  if (__Pyx_GetBuffer(obj, buf, flags) == -1) goto fail;
  if (buf->ndim != nd) {
    PyErr_Format(PyExc_ValueError,
                 "Buffer has wrong number of dimensions (expected %d, got %d)",
                 nd, buf->ndim);
    goto fail;
  }
  if (!cast) {
    __Pyx_BufFmt_Context ctx;
    __Pyx_BufFmt_Init(&ctx, stack, dtype);
    if (!__Pyx_BufFmt_CheckString(&ctx, buf->format)) goto fail;
  }
  if ((unsigned)buf->itemsize != dtype->size) {
    PyErr_Format(PyExc_ValueError,
      "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "d byte%s) does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "d byte%s)",
      buf->itemsize, (buf->itemsize > 1) ? "s" : "",
      dtype->name, (Py_ssize_t)dtype->size, (dtype->size > 1) ? "s" : "");
    goto fail;
  }
  if (buf->suboffsets == NULL) buf->suboffsets = __Pyx_minusones;
  return 0;
fail:;
  __Pyx_ZeroBuffer(buf);
  return -1;
}
static CYTHON_INLINE void __Pyx_SafeReleaseBuffer(Py_buffer* info) {
  if (info->buf == NULL) return;
  if (info->suboffsets == __Pyx_minusones) info->suboffsets = NULL;
  __Pyx_ReleaseBuffer(info);
}

static int
__Pyx_init_memviewslice(struct __pyx_memoryview_obj *memview,
                        int ndim,
                        __Pyx_memviewslice *memviewslice,
                        int memview_is_new_reference)
{
    __Pyx_RefNannyDeclarations
    int i, retval=-1;
    Py_buffer *buf = &memview->view;
    __Pyx_RefNannySetupContext("init_memviewslice", 0);
    if (!buf) {
        PyErr_SetString(PyExc_ValueError,
            "buf is NULL.");
        goto fail;
    } else if (memviewslice->memview || memviewslice->data) {
        PyErr_SetString(PyExc_ValueError,
            "memviewslice is already initialized!");
        goto fail;
    }
    if (buf->strides) {
        for (i = 0; i < ndim; i++) {
            memviewslice->strides[i] = buf->strides[i];
        }
    } else {
        Py_ssize_t stride = buf->itemsize;
        for (i = ndim - 1; i >= 0; i--) {
            memviewslice->strides[i] = stride;
            stride *= buf->shape[i];
        }
    }
    for (i = 0; i < ndim; i++) {
        memviewslice->shape[i]   = buf->shape[i];
        if (buf->suboffsets) {
            memviewslice->suboffsets[i] = buf->suboffsets[i];
        } else {
            memviewslice->suboffsets[i] = -1;
        }
    }
    memviewslice->memview = memview;
    memviewslice->data = (char *)buf->buf;
    if (__pyx_add_acquisition_count(memview) == 0 && !memview_is_new_reference) {
        Py_INCREF(memview);
    }
    retval = 0;
    goto no_fail;
fail:
    memviewslice->memview = 0;
    memviewslice->data = 0;
    retval = -1;
no_fail:
    __Pyx_RefNannyFinishContext();
    return retval;
}
static CYTHON_INLINE void __pyx_fatalerror(const char *fmt, ...) {
    va_list vargs;
    char msg[200];
#ifdef HAVE_STDARG_PROTOTYPES
    va_start(vargs, fmt);
#else
    va_start(vargs);
#endif
    vsnprintf(msg, 200, fmt, vargs);
    Py_FatalError(msg);
    va_end(vargs);
}
static CYTHON_INLINE int
__pyx_add_acquisition_count_locked(__pyx_atomic_int *acquisition_count,
                                   PyThread_type_lock lock)
{
    int result;
    PyThread_acquire_lock(lock, 1);
    result = (*acquisition_count)++;
    PyThread_release_lock(lock);
    return result;
}
static CYTHON_INLINE int
__pyx_sub_acquisition_count_locked(__pyx_atomic_int *acquisition_count,
                                   PyThread_type_lock lock)
{
    int result;
    PyThread_acquire_lock(lock, 1);
    result = (*acquisition_count)--;
    PyThread_release_lock(lock);
    return result;
}
static CYTHON_INLINE void
__Pyx_INC_MEMVIEW(__Pyx_memviewslice *memslice, int have_gil, int lineno)
{
    int first_time;
    struct __pyx_memoryview_obj *memview = memslice->memview;
    if (!memview || (PyObject *) memview == Py_None)
        return;
    if (__pyx_get_slice_count(memview) < 0)
        __pyx_fatalerror("Acquisition count is %d (line %d)",
                         __pyx_get_slice_count(memview), lineno);
    first_time = __pyx_add_acquisition_count(memview) == 0;
    if (first_time) {
        if (have_gil) {
            Py_INCREF((PyObject *) memview);
        } else {
            PyGILState_STATE _gilstate = PyGILState_Ensure();
            Py_INCREF((PyObject *) memview);
            PyGILState_Release(_gilstate);
        }
    }
}
static CYTHON_INLINE void __Pyx_XDEC_MEMVIEW(__Pyx_memviewslice *memslice,
                                             int have_gil, int lineno) {
    int last_time;
    struct __pyx_memoryview_obj *memview = memslice->memview;
    if (!memview ) {
        return;
    } else if ((PyObject *) memview == Py_None) {
        memslice->memview = NULL;
        return;
    }
    if (__pyx_get_slice_count(memview) <= 0)
        __pyx_fatalerror("Acquisition count is %d (line %d)",
                         __pyx_get_slice_count(memview), lineno);
    last_time = __pyx_sub_acquisition_count(memview) == 1;
    memslice->data = NULL;
    if (last_time) {
        if (have_gil) {
            Py_CLEAR(memslice->memview);
        } else {
            PyGILState_STATE _gilstate = PyGILState_Ensure();
            Py_CLEAR(memslice->memview);
            PyGILState_Release(_gilstate);
        }
    } else {
        memslice->memview = NULL;
    }
}

static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
#else
    PyErr_GetExcInfo(type, value, tb);
#endif
}
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(type, value, tb);
#endif
}

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_COMPILING_IN_CPYTHON
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
#if PY_VERSION_HEX >= 0x03030000
    if (cause) {
#else
    if (cause && cause != Py_None) {
#endif
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject* x) {
   Py_ssize_t length;
   #if CYTHON_PEP393_ENABLED
   length = PyUnicode_GET_LENGTH(x);
   if (likely(length == 1)) {
       return PyUnicode_READ_CHAR(x, 0);
   }
   #else
   length = PyUnicode_GET_SIZE(x);
   if (likely(length == 1)) {
       return PyUnicode_AS_UNICODE(x)[0];
   }
   #if Py_UNICODE_SIZE == 2
   else if (PyUnicode_GET_SIZE(x) == 2) {
       Py_UCS4 high_val = PyUnicode_AS_UNICODE(x)[0];
       if (high_val >= 0xD800 && high_val <= 0xDBFF) {
           Py_UCS4 low_val = PyUnicode_AS_UNICODE(x)[1];
           if (low_val >= 0xDC00 && low_val <= 0xDFFF) {
               return 0x10000 + (((high_val & ((1<<10)-1)) << 10) | (low_val & ((1<<10)-1)));
           }
       }
   }
   #endif
   #endif
   PyErr_Format(PyExc_ValueError,
                "only single character unicode strings can be converted to Py_UCS4, "
                "got length %" CYTHON_FORMAT_SSIZE_T "d", length);
   return (Py_UCS4)-1;
}

static long __Pyx__PyObject_Ord(PyObject* c) {
    Py_ssize_t size;
    if (PyBytes_Check(c)) {
        size = PyBytes_GET_SIZE(c);
        if (likely(size == 1)) {
            return (unsigned char) PyBytes_AS_STRING(c)[0];
        }
#if PY_MAJOR_VERSION < 3
    } else if (PyUnicode_Check(c)) {
        return (long)__Pyx_PyUnicode_AsPy_UCS4(c);
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    } else if (PyByteArray_Check(c)) {
        size = PyByteArray_GET_SIZE(c);
        if (likely(size == 1)) {
            return (unsigned char) PyByteArray_AS_STRING(c)[0];
        }
#endif
    } else {
        PyErr_Format(PyExc_TypeError,
            "ord() expected string of length 1, but %.200s found", c->ob_type->tp_name);
        return (long)(Py_UCS4)-1;
    }
    PyErr_Format(PyExc_TypeError,
        "ord() expected a character, but string of length %zd found", size);
    return (long)(Py_UCS4)-1;
}

static CYTHON_INLINE int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (!j) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely((n >= 0) & (n < PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (PyErr_ExceptionMatches(PyExc_OverflowError))
                        PyErr_Clear();
                    else
                        return -1;
                }
            }
            return m->sq_ass_item(o, i, v);
        }
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (is_list || (PySequence_Check(o) && !PyDict_Check(o))) {
#else
    if (is_list || PySequence_Check(o)) {
#endif
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(exc_type == PyExc_StopIteration) || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method, *result = NULL;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) goto bad;
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely(PyMethod_Check(method))) {
        PyObject *self = PyMethod_GET_SELF(method);
        if (likely(self)) {
            PyObject *function = PyMethod_GET_FUNCTION(method);
            result = __Pyx_PyObject_CallOneArg(function, self);
            Py_DECREF(method);
            return result;
        }
    }
#endif
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else if (PyTuple_GET_SIZE(t) < index) {
      __Pyx_RaiseNeedMoreValuesError(PyTuple_GET_SIZE(t));
    } else {
      __Pyx_RaiseTooManyValuesError(index);
    }
}

static CYTHON_INLINE int __Pyx_unpack_tuple2(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                             int is_tuple, int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    if (!is_tuple && unlikely(!PyTuple_Check(tuple))) {
        iternextfunc iternext;
        iter = PyObject_GetIter(tuple);
        if (unlikely(!iter)) goto bad;
        if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
        iternext = Py_TYPE(iter)->tp_iternext;
        value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
        value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
        if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
        Py_DECREF(iter);
    } else {
        if (!has_known_size && unlikely(PyTuple_GET_SIZE(tuple) != 2)) {
            __Pyx_UnpackTupleError(tuple, 2);
            goto bad;
        }
#if CYTHON_COMPILING_IN_PYPY
        value1 = PySequence_ITEM(tuple, 0);
        if (unlikely(!value1)) goto bad;
        value2 = PySequence_ITEM(tuple, 1);
        if (unlikely(!value2)) goto bad;
#else
        value1 = PyTuple_GET_ITEM(tuple, 0);
        value2 = PyTuple_GET_ITEM(tuple, 1);
        Py_INCREF(value1);
        Py_INCREF(value2);
#endif
        if (decref_tuple) { Py_DECREF(tuple); }
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
#if !CYTHON_COMPILING_IN_PYPY
    if (is_dict) {
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
    }
#endif
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_COMPILING_IN_PYPY
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_COMPILING_IN_PYPY
    if (source_is_dict) {
        PyObject *key, *value;
        if (unlikely(orig_length != PyDict_Size(iter_obj))) {
            PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
            return -1;
        }
        if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
            return 0;
        }
        if (pitem) {
            PyObject* tuple = PyTuple_New(2);
            if (unlikely(!tuple)) {
                return -1;
            }
            Py_INCREF(key);
            Py_INCREF(value);
            PyTuple_SET_ITEM(tuple, 0, key);
            PyTuple_SET_ITEM(tuple, 1, value);
            *pitem = tuple;
        } else {
            if (pkey) {
                Py_INCREF(key);
                *pkey = key;
            }
            if (pvalue) {
                Py_INCREF(value);
                *pvalue = value;
            }
        }
        return 1;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyTuple_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyList_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyList_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#endif
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx_PyFloat_SubtractObjC(PyObject *op1, PyObject *op2, double floatval, CYTHON_UNUSED int inplace) {
    const double b = floatval;
    double a, result;
    if (likely(PyFloat_CheckExact(op1))) {
        a = PyFloat_AS_DOUBLE(op1);
    } else
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        a = (double) PyInt_AS_LONG(op1);
    } else
    #endif
    if (likely(PyLong_CheckExact(op1))) {
        #if CYTHON_USE_PYLONG_INTERNALS && PY_MAJOR_VERSION >= 3
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        switch (size) {
            case  0: a = 0.0; break;
            case -1: a = -(double) digits[0]; break;
            case  1: a = (double) digits[0]; break;
            case -2:
            case 2:
                if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (1 * PyLong_SHIFT < 53))) {
                    a = (double) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53) || (a < (double) (1L<<53))) {
                        if (size == -2)
                            a = -a;
                        break;
                    }
                }
            case -3:
            case 3:
                if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53) || (a < (double) (1L<<53))) {
                        if (size == -3)
                            a = -a;
                        break;
                    }
                }
            case -4:
            case 4:
                if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (4 * PyLong_SHIFT < 53) || (a < (double) (1L<<53))) {
                        if (size == -4)
                            a = -a;
                        break;
                    }
                }
            default:
        #else
        {
        #endif
            a = PyLong_AsDouble(op1);
            if (unlikely(a == -1.0 && PyErr_Occurred())) return NULL;
        }
    } else {
        return (inplace ? PyNumber_InPlaceSubtract : PyNumber_Subtract)(op1, op2);
    }
        PyFPE_START_PROTECT("subtract", return NULL)
        result = a - b;
        PyFPE_END_PROTECT(result)
        return PyFloat_FromDouble(result);
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx_PyFloat_DivideCObj(PyObject *op1, PyObject *op2, double floatval, CYTHON_UNUSED int inplace) {
    const double a = floatval;
    double b, result;
    if (likely(PyFloat_CheckExact(op2))) {
        b = PyFloat_AS_DOUBLE(op2);
    } else
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op2))) {
        b = (double) PyInt_AS_LONG(op2);
    } else
    #endif
    if (likely(PyLong_CheckExact(op2))) {
        #if CYTHON_USE_PYLONG_INTERNALS && PY_MAJOR_VERSION >= 3
        const digit* digits = ((PyLongObject*)op2)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op2);
        switch (size) {
            case  0: b = 0.0; break;
            case -1: b = -(double) digits[0]; break;
            case  1: b = (double) digits[0]; break;
            case -2:
            case 2:
                if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (1 * PyLong_SHIFT < 53))) {
                    b = (double) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53) || (b < (double) (1L<<53))) {
                        if (size == -2)
                            b = -b;
                        break;
                    }
                }
            case -3:
            case 3:
                if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53))) {
                    b = (double) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53) || (b < (double) (1L<<53))) {
                        if (size == -3)
                            b = -b;
                        break;
                    }
                }
            case -4:
            case 4:
                if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53))) {
                    b = (double) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (4 * PyLong_SHIFT < 53) || (b < (double) (1L<<53))) {
                        if (size == -4)
                            b = -b;
                        break;
                    }
                }
            default:
        #else
        {
        #endif
            b = PyLong_AsDouble(op2);
            if (unlikely(b == -1.0 && PyErr_Occurred())) return NULL;
        }
    } else {
        return (inplace ? __Pyx_PyNumber_InPlaceDivide(op1, op2) : __Pyx_PyNumber_Divide(op1, op2));
    }
        PyFPE_START_PROTECT("divide", return NULL)
        result = a / b;
        PyFPE_END_PROTECT(result)
        return PyFloat_FromDouble(result);
}
#endif

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx_PyInt_AddCObj(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op2))) {
        const long a = intval;
        long x;
        long b = PyInt_AS_LONG(op2);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS && PY_MAJOR_VERSION >= 3
    if (likely(PyLong_CheckExact(op2))) {
        const long a = intval;
        long b, x;
        const PY_LONG_LONG lla = intval;
        PY_LONG_LONG llb, llx;
        const digit* digits = ((PyLongObject*)op2)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op2);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            b = likely(size) ? digits[0] : 0;
            if (size == -1) b = -b;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        b = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        llb = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        b = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        llb = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        b = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        llb = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        b = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        llb = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        b = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        llb = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        b = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        llb = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
    }
    #endif
    if (PyFloat_CheckExact(op2)) {
        const long a = intval;
        double b = PyFloat_AS_DOUBLE(op2);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (wraparound & unlikely(i < 0)) i += PyList_GET_SIZE(o);
    if ((!boundscheck) || likely((0 <= i) & (i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (wraparound & unlikely(i < 0)) i += PyTuple_GET_SIZE(o);
    if ((!boundscheck) || likely((0 <= i) & (i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_COMPILING_IN_CPYTHON
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (PyErr_ExceptionMatches(PyExc_OverflowError))
                        PyErr_Clear();
                    else
                        return NULL;
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(PyObject_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

static void __Pyx_RaiseArgumentTypeInvalid(const char* name, PyObject *obj, PyTypeObject *type) {
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
}
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (none_allowed && obj == Py_None) return 1;
    else if (exact) {
        if (likely(Py_TYPE(obj) == type)) return 1;
        #if PY_MAJOR_VERSION == 2
        else if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(PyObject_TypeCheck(obj, type))) return 1;
    }
    __Pyx_RaiseArgumentTypeInvalid(name, obj, type);
    return 0;
}

static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_COMPILING_IN_CPYTHON
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

static CYTHON_INLINE PyObject* __Pyx_decode_c_string(
         const char* cstring, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    Py_ssize_t length;
    if (unlikely((start < 0) | (stop < 0))) {
        size_t slen = strlen(cstring);
        if (unlikely(slen > (size_t) PY_SSIZE_T_MAX)) {
            PyErr_SetString(PyExc_OverflowError,
                            "c-string too long to convert to Python");
            return NULL;
        }
        length = (Py_ssize_t) slen;
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    length = stop - start;
    if (unlikely(length <= 0))
        return PyUnicode_FromUnicode(NULL, 0);
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
#else
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
#endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}

#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx_PyInt_AddObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long x;
        long a = PyInt_AS_LONG(op1);
            x = (long)((unsigned long)a + b);
            if (likely((x^a) >= 0 || (x^b) >= 0))
                return PyInt_FromLong(x);
            return PyLong_Type.tp_as_number->nb_add(op1, op2);
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS && PY_MAJOR_VERSION >= 3
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a, x;
        const PY_LONG_LONG llb = intval;
        PY_LONG_LONG lla, llx;
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = -(PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                        lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        goto long_long;
                    }
                default: return PyLong_Type.tp_as_number->nb_add(op1, op2);
            }
        }
                x = a + b;
            return PyLong_FromLong(x);
        long_long:
                llx = lla + llb;
            return PyLong_FromLongLong(llx);
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            double result;
            PyFPE_START_PROTECT("add", return NULL)
            result = ((double)a) + (double)b;
            PyFPE_END_PROTECT(result)
            return PyFloat_FromDouble(result);
    }
    return (inplace ? PyNumber_InPlaceAdd : PyNumber_Add)(op1, op2);
}
#endif

static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_COMPILING_IN_CPYTHON
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(__pyx_CyFunctionObject, func.m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromString(m->func.m_ml->ml_name);
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_New(PyTypeObject *type, PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    __pyx_CyFunctionObject *op = PyObject_GC_New(__pyx_CyFunctionObject, type);
    if (op == NULL)
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    PyObject_GC_Track(op);
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyMem_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
#if CYTHON_COMPILING_IN_PYPY
static PyObject * __Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    PyObject *self = f->m_self;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0 = PySequence_ITEM(arg, 0);
                if (unlikely(!arg0)) return NULL;
                result = (*meth)(self, arg0);
                Py_DECREF(arg0);
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
#else
static PyObject * __Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
	return PyCFunction_Call(func, arg, kw);
}
#endif
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_Call,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
#if !CYTHON_COMPILING_IN_PYPY
    __pyx_CyFunctionType_type.tp_call = PyCFunction_Call;
#endif
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (__pyx_CyFunctionType == NULL) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyMem_Malloc(size);
    if (!m->defaults)
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

static PyObject *
__pyx_FusedFunction_New(PyTypeObject *type, PyMethodDef *ml, int flags,
                        PyObject *qualname, PyObject *self,
                        PyObject *module, PyObject *globals,
                        PyObject *code)
{
    __pyx_FusedFunctionObject *fusedfunc =
        (__pyx_FusedFunctionObject *) __Pyx_CyFunction_New(type, ml, flags, qualname,
                                                           self, module, globals, code);
    if (!fusedfunc)
        return NULL;
    fusedfunc->__signatures__ = NULL;
    fusedfunc->type = NULL;
    fusedfunc->self = NULL;
    return (PyObject *) fusedfunc;
}
static void __pyx_FusedFunction_dealloc(__pyx_FusedFunctionObject *self) {
    __pyx_FusedFunction_clear(self);
    __pyx_FusedFunctionType->tp_free((PyObject *) self);
}
static int
__pyx_FusedFunction_traverse(__pyx_FusedFunctionObject *self,
                             visitproc visit,
                             void *arg)
{
    Py_VISIT(self->self);
    Py_VISIT(self->type);
    Py_VISIT(self->__signatures__);
    return __Pyx_CyFunction_traverse((__pyx_CyFunctionObject *) self, visit, arg);
}
static int
__pyx_FusedFunction_clear(__pyx_FusedFunctionObject *self)
{
    Py_CLEAR(self->self);
    Py_CLEAR(self->type);
    Py_CLEAR(self->__signatures__);
    return __Pyx_CyFunction_clear((__pyx_CyFunctionObject *) self);
}
static PyObject *
__pyx_FusedFunction_descr_get(PyObject *self, PyObject *obj, PyObject *type)
{
    __pyx_FusedFunctionObject *func, *meth;
    func = (__pyx_FusedFunctionObject *) self;
    if (func->self || func->func.flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(self);
        return self;
    }
    if (obj == Py_None)
        obj = NULL;
    meth = (__pyx_FusedFunctionObject *) __pyx_FusedFunction_NewEx(
                    ((PyCFunctionObject *) func)->m_ml,
                    ((__pyx_CyFunctionObject *) func)->flags,
                    ((__pyx_CyFunctionObject *) func)->func_qualname,
                    ((__pyx_CyFunctionObject *) func)->func_closure,
                    ((PyCFunctionObject *) func)->m_module,
                    ((__pyx_CyFunctionObject *) func)->func_globals,
                    ((__pyx_CyFunctionObject *) func)->func_code);
    if (!meth)
        return NULL;
    Py_XINCREF(func->func.func_classobj);
    meth->func.func_classobj = func->func.func_classobj;
    Py_XINCREF(func->__signatures__);
    meth->__signatures__ = func->__signatures__;
    Py_XINCREF(type);
    meth->type = type;
    Py_XINCREF(func->func.defaults_tuple);
    meth->func.defaults_tuple = func->func.defaults_tuple;
    if (func->func.flags & __Pyx_CYFUNCTION_CLASSMETHOD)
        obj = type;
    Py_XINCREF(obj);
    meth->self = obj;
    return (PyObject *) meth;
}
static PyObject *
_obj_to_str(PyObject *obj)
{
    if (PyType_Check(obj))
        return PyObject_GetAttr(obj, __pyx_n_s_name_2);
    else
        return PyObject_Str(obj);
}
static PyObject *
__pyx_FusedFunction_getitem(__pyx_FusedFunctionObject *self, PyObject *idx)
{
    PyObject *signature = NULL;
    PyObject *unbound_result_func;
    PyObject *result_func = NULL;
    if (self->__signatures__ == NULL) {
        PyErr_SetString(PyExc_TypeError, "Function is not fused");
        return NULL;
    }
    if (PyTuple_Check(idx)) {
        PyObject *list = PyList_New(0);
        Py_ssize_t n = PyTuple_GET_SIZE(idx);
        PyObject *string = NULL;
        PyObject *sep = NULL;
        int i;
        if (!list)
            return NULL;
        for (i = 0; i < n; i++) {
#if CYTHON_COMPILING_IN_CPYTHON
            PyObject *item = PyTuple_GET_ITEM(idx, i);
#else
            PyObject *item = PySequence_ITEM(idx, i);
#endif
            string = _obj_to_str(item);
#if !CYTHON_COMPILING_IN_CPYTHON
            Py_DECREF(item);
#endif
            if (!string || PyList_Append(list, string) < 0)
                goto __pyx_err;
            Py_DECREF(string);
        }
        sep = PyUnicode_FromString("|");
        if (sep)
            signature = PyUnicode_Join(sep, list);
__pyx_err:
;
        Py_DECREF(list);
        Py_XDECREF(sep);
    } else {
        signature = _obj_to_str(idx);
    }
    if (!signature)
        return NULL;
    unbound_result_func = PyObject_GetItem(self->__signatures__, signature);
    if (unbound_result_func) {
        if (self->self || self->type) {
            __pyx_FusedFunctionObject *unbound = (__pyx_FusedFunctionObject *) unbound_result_func;
            Py_CLEAR(unbound->func.func_classobj);
            Py_XINCREF(self->func.func_classobj);
            unbound->func.func_classobj = self->func.func_classobj;
            result_func = __pyx_FusedFunction_descr_get(unbound_result_func,
                                                        self->self, self->type);
        } else {
            result_func = unbound_result_func;
            Py_INCREF(result_func);
        }
    }
    Py_DECREF(signature);
    Py_XDECREF(unbound_result_func);
    return result_func;
}
static PyObject *
__pyx_FusedFunction_callfunction(PyObject *func, PyObject *args, PyObject *kw)
{
     __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    PyObject *result;
    int static_specialized = (cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD &&
                              !((__pyx_FusedFunctionObject *) func)->__signatures__);
    if (cyfunc->flags & __Pyx_CYFUNCTION_CCLASS && !static_specialized) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        PyObject *m_self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (!new_args)
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (!self)
            return NULL;
        m_self = cyfunc->func.m_self;
        cyfunc->func.m_self = self;
        result = __Pyx_CyFunction_Call(func, new_args, kw);
        cyfunc->func.m_self = m_self;
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyObject *
__pyx_FusedFunction_call(PyObject *func, PyObject *args, PyObject *kw)
{
    __pyx_FusedFunctionObject *binding_func = (__pyx_FusedFunctionObject *) func;
    Py_ssize_t argc = PyTuple_GET_SIZE(args);
    PyObject *new_args = NULL;
    __pyx_FusedFunctionObject *new_func = NULL;
    PyObject *result = NULL;
    PyObject *self = NULL;
    int is_staticmethod = binding_func->func.flags & __Pyx_CYFUNCTION_STATICMETHOD;
    int is_classmethod = binding_func->func.flags & __Pyx_CYFUNCTION_CLASSMETHOD;
    if (binding_func->self) {
        Py_ssize_t i;
        new_args = PyTuple_New(argc + 1);
        if (!new_args)
            return NULL;
        self = binding_func->self;
#if !CYTHON_COMPILING_IN_CPYTHON
        Py_INCREF(self);
#endif
        Py_INCREF(self);
        PyTuple_SET_ITEM(new_args, 0, self);
        for (i = 0; i < argc; i++) {
#if CYTHON_COMPILING_IN_CPYTHON
            PyObject *item = PyTuple_GET_ITEM(args, i);
            Py_INCREF(item);
#else
            PyObject *item = PySequence_ITEM(args, i);  if (unlikely(!item)) goto bad;
#endif
            PyTuple_SET_ITEM(new_args, i + 1, item);
        }
        args = new_args;
    } else if (binding_func->type) {
        if (argc < 1) {
            PyErr_SetString(PyExc_TypeError, "Need at least one argument, 0 given.");
            return NULL;
        }
#if CYTHON_COMPILING_IN_CPYTHON
        self = PyTuple_GET_ITEM(args, 0);
#else
        self = PySequence_ITEM(args, 0);  if (unlikely(!self)) return NULL;
#endif
    }
    if (self && !is_classmethod && !is_staticmethod) {
        int is_instance = PyObject_IsInstance(self, binding_func->type);
        if (unlikely(!is_instance)) {
            PyErr_Format(PyExc_TypeError,
                         "First argument should be of type %.200s, got %.200s.",
                         ((PyTypeObject *) binding_func->type)->tp_name,
                         self->ob_type->tp_name);
            goto bad;
        } else if (unlikely(is_instance == -1)) {
            goto bad;
        }
    }
#if !CYTHON_COMPILING_IN_CPYTHON
    Py_XDECREF(self);
    self = NULL;
#endif
    if (binding_func->__signatures__) {
        PyObject *tup = PyTuple_Pack(4, binding_func->__signatures__, args,
                                        kw == NULL ? Py_None : kw,
                                        binding_func->func.defaults_tuple);
        if (!tup)
            goto bad;
        new_func = (__pyx_FusedFunctionObject *) __pyx_FusedFunction_callfunction(func, tup, NULL);
        Py_DECREF(tup);
        if (!new_func)
            goto bad;
        Py_XINCREF(binding_func->func.func_classobj);
        Py_CLEAR(new_func->func.func_classobj);
        new_func->func.func_classobj = binding_func->func.func_classobj;
        func = (PyObject *) new_func;
    }
    result = __pyx_FusedFunction_callfunction(func, args, kw);
bad:
#if !CYTHON_COMPILING_IN_CPYTHON
    Py_XDECREF(self);
#endif
    Py_XDECREF(new_args);
    Py_XDECREF((PyObject *) new_func);
    return result;
}
static PyMemberDef __pyx_FusedFunction_members[] = {
    {(char *) "__signatures__",
     T_OBJECT,
     offsetof(__pyx_FusedFunctionObject, __signatures__),
     READONLY,
     0},
    {0, 0, 0, 0, 0},
};
static PyMappingMethods __pyx_FusedFunction_mapping_methods = {
    0,
    (binaryfunc) __pyx_FusedFunction_getitem,
    0,
};
static PyTypeObject __pyx_FusedFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "fused_cython_function",
    sizeof(__pyx_FusedFunctionObject),
    0,
    (destructor) __pyx_FusedFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    0,
    0,
    0,
    &__pyx_FusedFunction_mapping_methods,
    0,
    (ternaryfunc) __pyx_FusedFunction_call,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    0,
    (traverseproc) __pyx_FusedFunction_traverse,
    (inquiry) __pyx_FusedFunction_clear,
    0,
    0,
    0,
    0,
    0,
    __pyx_FusedFunction_members,
    __pyx_CyFunction_getsets,
    &__pyx_CyFunctionType_type,
    0,
    __pyx_FusedFunction_descr_get,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
};
static int __pyx_FusedFunction_init(void) {
    __pyx_FusedFunctionType = __Pyx_FetchCommonType(&__pyx_FusedFunctionType_type);
    if (__pyx_FusedFunctionType == NULL) {
        return -1;
    }
    return 0;
}

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

#if PY_MAJOR_VERSION < 3
static int __Pyx_GetBuffer(PyObject *obj, Py_buffer *view, int flags) {
    if (PyObject_CheckBuffer(obj)) return PyObject_GetBuffer(obj, view, flags);
        if (PyObject_TypeCheck(obj, __pyx_ptype_5numpy_ndarray)) return __pyx_pw_5numpy_7ndarray_1__getbuffer__(obj, view, flags);
        if (PyObject_TypeCheck(obj, __pyx_array_type)) return __pyx_array_getbuffer(obj, view, flags);
        if (PyObject_TypeCheck(obj, __pyx_memoryview_type)) return __pyx_memoryview_getbuffer(obj, view, flags);
    PyErr_Format(PyExc_TypeError, "'%.200s' does not have the buffer interface", Py_TYPE(obj)->tp_name);
    return -1;
}
static void __Pyx_ReleaseBuffer(Py_buffer *view) {
    PyObject *obj = view->obj;
    if (!obj) return;
    if (PyObject_CheckBuffer(obj)) {
        PyBuffer_Release(view);
        return;
    }
        if (PyObject_TypeCheck(obj, __pyx_ptype_5numpy_ndarray)) { __pyx_pw_5numpy_7ndarray_3__releasebuffer__(obj, view); return; }
    Py_DECREF(obj);
    view->obj = NULL;
}
#endif


                    static int
__pyx_typeinfo_cmp(__Pyx_TypeInfo *a, __Pyx_TypeInfo *b)
{
    int i;
    if (!a || !b)
        return 0;
    if (a == b)
        return 1;
    if (a->size != b->size || a->typegroup != b->typegroup ||
            a->is_unsigned != b->is_unsigned || a->ndim != b->ndim) {
        if (a->typegroup == 'H' || b->typegroup == 'H') {
            return a->size == b->size;
        } else {
            return 0;
        }
    }
    if (a->ndim) {
        for (i = 0; i < a->ndim; i++)
            if (a->arraysize[i] != b->arraysize[i])
                return 0;
    }
    if (a->typegroup == 'S') {
        if (a->flags != b->flags)
            return 0;
        if (a->fields || b->fields) {
            if (!(a->fields && b->fields))
                return 0;
            for (i = 0; a->fields[i].type && b->fields[i].type; i++) {
                __Pyx_StructField *field_a = a->fields + i;
                __Pyx_StructField *field_b = b->fields + i;
                if (field_a->offset != field_b->offset ||
                    !__pyx_typeinfo_cmp(field_a->type, field_b->type))
                    return 0;
            }
            return !a->fields[i].type && !b->fields[i].type;
        }
    }
    return 1;
}

static int
__pyx_check_strides(Py_buffer *buf, int dim, int ndim, int spec)
{
    if (buf->shape[dim] <= 1)
        return 1;
    if (buf->strides) {
        if (spec & __Pyx_MEMVIEW_CONTIG) {
            if (spec & (__Pyx_MEMVIEW_PTR|__Pyx_MEMVIEW_FULL)) {
                if (buf->strides[dim] != sizeof(void *)) {
                    PyErr_Format(PyExc_ValueError,
                                 "Buffer is not indirectly contiguous "
                                 "in dimension %d.", dim);
                    goto fail;
                }
            } else if (buf->strides[dim] != buf->itemsize) {
                PyErr_SetString(PyExc_ValueError,
                                "Buffer and memoryview are not contiguous "
                                "in the same dimension.");
                goto fail;
            }
        }
        if (spec & __Pyx_MEMVIEW_FOLLOW) {
            Py_ssize_t stride = buf->strides[dim];
            if (stride < 0)
                stride = -stride;
            if (stride < buf->itemsize) {
                PyErr_SetString(PyExc_ValueError,
                                "Buffer and memoryview are not contiguous "
                                "in the same dimension.");
                goto fail;
            }
        }
    } else {
        if (spec & __Pyx_MEMVIEW_CONTIG && dim != ndim - 1) {
            PyErr_Format(PyExc_ValueError,
                         "C-contiguous buffer is not contiguous in "
                         "dimension %d", dim);
            goto fail;
        } else if (spec & (__Pyx_MEMVIEW_PTR)) {
            PyErr_Format(PyExc_ValueError,
                         "C-contiguous buffer is not indirect in "
                         "dimension %d", dim);
            goto fail;
        } else if (buf->suboffsets) {
            PyErr_SetString(PyExc_ValueError,
                            "Buffer exposes suboffsets but no strides");
            goto fail;
        }
    }
    return 1;
fail:
    return 0;
}
static int
__pyx_check_suboffsets(Py_buffer *buf, int dim, CYTHON_UNUSED int ndim, int spec)
{
    if (spec & __Pyx_MEMVIEW_DIRECT) {
        if (buf->suboffsets && buf->suboffsets[dim] >= 0) {
            PyErr_Format(PyExc_ValueError,
                         "Buffer not compatible with direct access "
                         "in dimension %d.", dim);
            goto fail;
        }
    }
    if (spec & __Pyx_MEMVIEW_PTR) {
        if (!buf->suboffsets || (buf->suboffsets && buf->suboffsets[dim] < 0)) {
            PyErr_Format(PyExc_ValueError,
                         "Buffer is not indirectly accessible "
                         "in dimension %d.", dim);
            goto fail;
        }
    }
    return 1;
fail:
    return 0;
}
static int
__pyx_verify_contig(Py_buffer *buf, int ndim, int c_or_f_flag)
{
    int i;
    if (c_or_f_flag & __Pyx_IS_F_CONTIG) {
        Py_ssize_t stride = 1;
        for (i = 0; i < ndim; i++) {
            if (stride * buf->itemsize != buf->strides[i] &&
                    buf->shape[i] > 1)
            {
                PyErr_SetString(PyExc_ValueError,
                    "Buffer not fortran contiguous.");
                goto fail;
            }
            stride = stride * buf->shape[i];
        }
    } else if (c_or_f_flag & __Pyx_IS_C_CONTIG) {
        Py_ssize_t stride = 1;
        for (i = ndim - 1; i >- 1; i--) {
            if (stride * buf->itemsize != buf->strides[i] &&
                    buf->shape[i] > 1) {
                PyErr_SetString(PyExc_ValueError,
                    "Buffer not C contiguous.");
                goto fail;
            }
            stride = stride * buf->shape[i];
        }
    }
    return 1;
fail:
    return 0;
}
static int __Pyx_ValidateAndInit_memviewslice(
                int *axes_specs,
                int c_or_f_flag,
                int buf_flags,
                int ndim,
                __Pyx_TypeInfo *dtype,
                __Pyx_BufFmt_StackElem stack[],
                __Pyx_memviewslice *memviewslice,
                PyObject *original_obj)
{
    struct __pyx_memoryview_obj *memview, *new_memview;
    __Pyx_RefNannyDeclarations
    Py_buffer *buf;
    int i, spec = 0, retval = -1;
    __Pyx_BufFmt_Context ctx;
    int from_memoryview = __pyx_memoryview_check(original_obj);
    __Pyx_RefNannySetupContext("ValidateAndInit_memviewslice", 0);
    if (from_memoryview && __pyx_typeinfo_cmp(dtype, ((struct __pyx_memoryview_obj *)
                                                            original_obj)->typeinfo)) {
        memview = (struct __pyx_memoryview_obj *) original_obj;
        new_memview = NULL;
    } else {
        memview = (struct __pyx_memoryview_obj *) __pyx_memoryview_new(
                                            original_obj, buf_flags, 0, dtype);
        new_memview = memview;
        if (unlikely(!memview))
            goto fail;
    }
    buf = &memview->view;
    if (buf->ndim != ndim) {
        PyErr_Format(PyExc_ValueError,
                "Buffer has wrong number of dimensions (expected %d, got %d)",
                ndim, buf->ndim);
        goto fail;
    }
    if (new_memview) {
        __Pyx_BufFmt_Init(&ctx, stack, dtype);
        if (!__Pyx_BufFmt_CheckString(&ctx, buf->format)) goto fail;
    }
    if ((unsigned) buf->itemsize != dtype->size) {
        PyErr_Format(PyExc_ValueError,
                     "Item size of buffer (%" CYTHON_FORMAT_SSIZE_T "u byte%s) "
                     "does not match size of '%s' (%" CYTHON_FORMAT_SSIZE_T "u byte%s)",
                     buf->itemsize,
                     (buf->itemsize > 1) ? "s" : "",
                     dtype->name,
                     dtype->size,
                     (dtype->size > 1) ? "s" : "");
        goto fail;
    }
    for (i = 0; i < ndim; i++) {
        spec = axes_specs[i];
        if (!__pyx_check_strides(buf, i, ndim, spec))
            goto fail;
        if (!__pyx_check_suboffsets(buf, i, ndim, spec))
            goto fail;
    }
    if (buf->strides && !__pyx_verify_contig(buf, ndim, c_or_f_flag))
        goto fail;
    if (unlikely(__Pyx_init_memviewslice(memview, ndim, memviewslice,
                                         new_memview != NULL) == -1)) {
        goto fail;
    }
    retval = 0;
    goto no_fail;
fail:
    Py_XDECREF(new_memview);
    retval = -1;
no_fail:
    __Pyx_RefNannyFinishContext();
    return retval;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsds_float(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 3,
                                                 &__Pyx_TypeInfo_float, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsds_double(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 3,
                                                 &__Pyx_TypeInfo_double, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsds_float(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 2,
                                                 &__Pyx_TypeInfo_float, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsds_double(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 2,
                                                 &__Pyx_TypeInfo_double, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsds_short(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 2,
                                                 &__Pyx_TypeInfo_short, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsds_int(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 2,
                                                 &__Pyx_TypeInfo_int, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsds_long(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 2,
                                                 &__Pyx_TypeInfo_long, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_float_complex(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 2,
                                                 &__Pyx_TypeInfo___pyx_t_float_complex, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsds___pyx_t_double_complex(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 2,
                                                 &__Pyx_TypeInfo___pyx_t_double_complex, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsds_short(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 3,
                                                 &__Pyx_TypeInfo_short, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsds_int(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 3,
                                                 &__Pyx_TypeInfo_int, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsds_long(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 3,
                                                 &__Pyx_TypeInfo_long, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_float_complex(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 3,
                                                 &__Pyx_TypeInfo___pyx_t_float_complex, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsds___pyx_t_double_complex(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 3,
                                                 &__Pyx_TypeInfo___pyx_t_double_complex, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_float(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 4,
                                                 &__Pyx_TypeInfo_float, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_double(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 4,
                                                 &__Pyx_TypeInfo_double, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) -(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) -(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) -(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return ::std::complex< float >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      return x + y*(__pyx_t_float_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_float_complex __pyx_t_float_complex_from_parts(float x, float y) {
      __pyx_t_float_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eqf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_sumf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_difff(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_prodf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_quotf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
        __pyx_t_float_complex z;
        float denom = b.real * b.real + b.imag * b.imag;
        z.real = (a.real * b.real + a.imag * b.imag) / denom;
        z.imag = (a.imag * b.real - a.real * b.imag) / denom;
        return z;
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_negf(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zerof(__pyx_t_float_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_conjf(__pyx_t_float_complex a) {
        __pyx_t_float_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE float __Pyx_c_absf(__pyx_t_float_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrtf(z.real*z.real + z.imag*z.imag);
          #else
            return hypotf(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_float_complex __Pyx_c_powf(__pyx_t_float_complex a, __pyx_t_float_complex b) {
            __pyx_t_float_complex z;
            float r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    float denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(a, a);
                    case 3:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(z, a);
                    case 4:
                        z = __Pyx_c_prodf(a, a);
                        return __Pyx_c_prodf(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                }
                r = a.real;
                theta = 0;
            } else {
                r = __Pyx_c_absf(a);
                theta = atan2f(a.imag, a.real);
            }
            lnr = logf(r);
            z_r = expf(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cosf(z_theta);
            z.imag = z_r * sinf(z_theta);
            return z;
        }
    #endif
#endif

#if CYTHON_CCOMPLEX
  #ifdef __cplusplus
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return ::std::complex< double >(x, y);
    }
  #else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      return x + y*(__pyx_t_double_complex)_Complex_I;
    }
  #endif
#else
    static CYTHON_INLINE __pyx_t_double_complex __pyx_t_double_complex_from_parts(double x, double y) {
      __pyx_t_double_complex z;
      z.real = x;
      z.imag = y;
      return z;
    }
#endif

#if CYTHON_CCOMPLEX
#else
    static CYTHON_INLINE int __Pyx_c_eq(__pyx_t_double_complex a, __pyx_t_double_complex b) {
       return (a.real == b.real) && (a.imag == b.imag);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_sum(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real + b.real;
        z.imag = a.imag + b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_diff(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real - b.real;
        z.imag = a.imag - b.imag;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_prod(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        z.real = a.real * b.real - a.imag * b.imag;
        z.imag = a.real * b.imag + a.imag * b.real;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_quot(__pyx_t_double_complex a, __pyx_t_double_complex b) {
        __pyx_t_double_complex z;
        double denom = b.real * b.real + b.imag * b.imag;
        z.real = (a.real * b.real + a.imag * b.imag) / denom;
        z.imag = (a.imag * b.real - a.real * b.imag) / denom;
        return z;
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_neg(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real = -a.real;
        z.imag = -a.imag;
        return z;
    }
    static CYTHON_INLINE int __Pyx_c_is_zero(__pyx_t_double_complex a) {
       return (a.real == 0) && (a.imag == 0);
    }
    static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_conj(__pyx_t_double_complex a) {
        __pyx_t_double_complex z;
        z.real =  a.real;
        z.imag = -a.imag;
        return z;
    }
    #if 1
        static CYTHON_INLINE double __Pyx_c_abs(__pyx_t_double_complex z) {
          #if !defined(HAVE_HYPOT) || defined(_MSC_VER)
            return sqrt(z.real*z.real + z.imag*z.imag);
          #else
            return hypot(z.real, z.imag);
          #endif
        }
        static CYTHON_INLINE __pyx_t_double_complex __Pyx_c_pow(__pyx_t_double_complex a, __pyx_t_double_complex b) {
            __pyx_t_double_complex z;
            double r, lnr, theta, z_r, z_theta;
            if (b.imag == 0 && b.real == (int)b.real) {
                if (b.real < 0) {
                    double denom = a.real * a.real + a.imag * a.imag;
                    a.real = a.real / denom;
                    a.imag = -a.imag / denom;
                    b.real = -b.real;
                }
                switch ((int)b.real) {
                    case 0:
                        z.real = 1;
                        z.imag = 0;
                        return z;
                    case 1:
                        return a;
                    case 2:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(a, a);
                    case 3:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(z, a);
                    case 4:
                        z = __Pyx_c_prod(a, a);
                        return __Pyx_c_prod(z, z);
                }
            }
            if (a.imag == 0) {
                if (a.real == 0) {
                    return a;
                }
                r = a.real;
                theta = 0;
            } else {
                r = __Pyx_c_abs(a);
                theta = atan2(a.imag, a.real);
            }
            lnr = log(r);
            z_r = exp(lnr * b.real - theta * b.imag);
            z_theta = theta * b.real + lnr * b.imag;
            z.real = z_r * cos(z_theta);
            z.imag = z_r * sin(z_theta);
            return z;
        }
    #endif
#endif

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_double(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 1,
                                                 &__Pyx_TypeInfo_double, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static PyObject *__pyx_memview_get_float(const char *itemp) {
    return (PyObject *) PyFloat_FromDouble(*(float *) itemp);
}
static int __pyx_memview_set_float(const char *itemp, PyObject *obj) {
    float value = __pyx_PyFloat_AsFloat(obj);
    if ((value == (float)-1) && PyErr_Occurred())
        return 0;
    *(float *) itemp = value;
    return 1;
}

static PyObject *__pyx_memview_get_double(const char *itemp) {
    return (PyObject *) PyFloat_FromDouble(*(double *) itemp);
}
static int __pyx_memview_set_double(const char *itemp, PyObject *obj) {
    double value = __pyx_PyFloat_AsDouble(obj);
    if ((value == (double)-1) && PyErr_Occurred())
        return 0;
    *(double *) itemp = value;
    return 1;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_int(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 1,
                                                 &__Pyx_TypeInfo_int, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

static PyObject *__pyx_memview_get_int(const char *itemp) {
    return (PyObject *) __Pyx_PyInt_From_int(*(int *) itemp);
}
static int __pyx_memview_set_int(const char *itemp, PyObject *obj) {
    int value = __Pyx_PyInt_As_int(obj);
    if ((value == (int)-1) && PyErr_Occurred())
        return 0;
    *(int *) itemp = value;
    return 1;
}

static CYTHON_INLINE Py_intptr_t __Pyx_PyInt_As_Py_intptr_t(PyObject *x) {
    const Py_intptr_t neg_one = (Py_intptr_t) -1, const_zero = (Py_intptr_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (Py_intptr_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (Py_intptr_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(Py_intptr_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(Py_intptr_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(Py_intptr_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(Py_intptr_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) >= 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (Py_intptr_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(Py_intptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (Py_intptr_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(Py_intptr_t, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(Py_intptr_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(Py_intptr_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) -(((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(Py_intptr_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(Py_intptr_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) -(((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(Py_intptr_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((((Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(Py_intptr_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) -(((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(Py_intptr_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Py_intptr_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Py_intptr_t) - 1 > 4 * PyLong_SHIFT) {
                            return (Py_intptr_t) (((((((((Py_intptr_t)digits[3]) << PyLong_SHIFT) | (Py_intptr_t)digits[2]) << PyLong_SHIFT) | (Py_intptr_t)digits[1]) << PyLong_SHIFT) | (Py_intptr_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(Py_intptr_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, long, PyLong_AsLong(x))
            } else if (sizeof(Py_intptr_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(Py_intptr_t, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            Py_intptr_t val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (Py_intptr_t) -1;
        }
    } else {
        Py_intptr_t val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (Py_intptr_t) -1;
        val = __Pyx_PyInt_As_Py_intptr_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to Py_intptr_t");
    return (Py_intptr_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to Py_intptr_t");
    return (Py_intptr_t) -1;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE int __Pyx_BytesContains(PyObject* bytes, char character) {
    const Py_ssize_t length = PyBytes_GET_SIZE(bytes);
    char* char_start = PyBytes_AS_STRING(bytes);
    char* pos;
    for (pos=char_start; pos < char_start+length; pos++) {
        if (character == pos[0]) return 1;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value) {
    const Py_intptr_t neg_one = (Py_intptr_t) -1, const_zero = (Py_intptr_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(Py_intptr_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(Py_intptr_t),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_short(short value) {
    const short neg_one = (short) -1, const_zero = (short) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(short) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(short) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(short) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(short) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(short) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(short),
                                     little, !is_unsigned);
    }
}

static CYTHON_INLINE short __Pyx_PyInt_As_short(PyObject *x) {
    const short neg_one = (short) -1, const_zero = (short) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(short) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(short, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (short) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (short) 0;
                case  1: __PYX_VERIFY_RETURN_INT(short, digit, digits[0])
                case 2:
                    if (8 * sizeof(short) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) >= 2 * PyLong_SHIFT) {
                            return (short) (((((short)digits[1]) << PyLong_SHIFT) | (short)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(short) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) >= 3 * PyLong_SHIFT) {
                            return (short) (((((((short)digits[2]) << PyLong_SHIFT) | (short)digits[1]) << PyLong_SHIFT) | (short)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(short) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) >= 4 * PyLong_SHIFT) {
                            return (short) (((((((((short)digits[3]) << PyLong_SHIFT) | (short)digits[2]) << PyLong_SHIFT) | (short)digits[1]) << PyLong_SHIFT) | (short)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (short) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(short) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(short, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(short) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(short, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (short) 0;
                case -1: __PYX_VERIFY_RETURN_INT(short, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(short,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(short) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) - 1 > 2 * PyLong_SHIFT) {
                            return (short) -(((((short)digits[1]) << PyLong_SHIFT) | (short)digits[0]));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(short) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) - 1 > 2 * PyLong_SHIFT) {
                            return (short) (((((short)digits[1]) << PyLong_SHIFT) | (short)digits[0]));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(short) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) - 1 > 3 * PyLong_SHIFT) {
                            return (short) -(((((((short)digits[2]) << PyLong_SHIFT) | (short)digits[1]) << PyLong_SHIFT) | (short)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(short) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) - 1 > 3 * PyLong_SHIFT) {
                            return (short) (((((((short)digits[2]) << PyLong_SHIFT) | (short)digits[1]) << PyLong_SHIFT) | (short)digits[0]));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(short) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) - 1 > 4 * PyLong_SHIFT) {
                            return (short) -(((((((((short)digits[3]) << PyLong_SHIFT) | (short)digits[2]) << PyLong_SHIFT) | (short)digits[1]) << PyLong_SHIFT) | (short)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(short) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(short, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(short) - 1 > 4 * PyLong_SHIFT) {
                            return (short) (((((((((short)digits[3]) << PyLong_SHIFT) | (short)digits[2]) << PyLong_SHIFT) | (short)digits[1]) << PyLong_SHIFT) | (short)digits[0]));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(short) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(short, long, PyLong_AsLong(x))
            } else if (sizeof(short) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(short, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            short val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (short) -1;
        }
    } else {
        short val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (short) -1;
        val = __Pyx_PyInt_As_short(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to short");
    return (short) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to short");
    return (short) -1;
}

static PyObject *__pyx_memview_get_short(const char *itemp) {
    return (PyObject *) __Pyx_PyInt_From_short(*(short *) itemp);
}
static int __pyx_memview_set_short(const char *itemp, PyObject *obj) {
    short value = __Pyx_PyInt_As_short(obj);
    if ((value == (short)-1) && PyErr_Occurred())
        return 0;
    *(short *) itemp = value;
    return 1;
}

static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) -(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) -(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) -(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

static PyObject *__pyx_memview_get_long(const char *itemp) {
    return (PyObject *) __Pyx_PyInt_From_long(*(long *) itemp);
}
static int __pyx_memview_set_long(const char *itemp, PyObject *obj) {
    long value = __Pyx_PyInt_As_long(obj);
    if ((value == (long)-1) && PyErr_Occurred())
        return 0;
    *(long *) itemp = value;
    return 1;
}

static __pyx_t_float_complex __Pyx_PyComplex_As___pyx_t_float_complex(PyObject* o) {
    Py_complex cval;
#if CYTHON_COMPILING_IN_CPYTHON
    if (PyComplex_CheckExact(o))
        cval = ((PyComplexObject *)o)->cval;
    else
#endif
        cval = PyComplex_AsCComplex(o);
    return __pyx_t_float_complex_from_parts(
               (float)cval.real,
               (float)cval.imag);
}

static PyObject *__pyx_memview_get___pyx_t_float_complex(const char *itemp) {
    return (PyObject *) __pyx_PyComplex_FromComplex(*(__pyx_t_float_complex *) itemp);
}
static int __pyx_memview_set___pyx_t_float_complex(const char *itemp, PyObject *obj) {
    __pyx_t_float_complex value = __Pyx_PyComplex_As___pyx_t_float_complex(obj);
    if (PyErr_Occurred())
        return 0;
    *(__pyx_t_float_complex *) itemp = value;
    return 1;
}

static __pyx_t_double_complex __Pyx_PyComplex_As___pyx_t_double_complex(PyObject* o) {
    Py_complex cval;
#if CYTHON_COMPILING_IN_CPYTHON
    if (PyComplex_CheckExact(o))
        cval = ((PyComplexObject *)o)->cval;
    else
#endif
        cval = PyComplex_AsCComplex(o);
    return __pyx_t_double_complex_from_parts(
               (double)cval.real,
               (double)cval.imag);
}

static PyObject *__pyx_memview_get___pyx_t_double_complex(const char *itemp) {
    return (PyObject *) __pyx_PyComplex_FromComplex(*(__pyx_t_double_complex *) itemp);
}
static int __pyx_memview_set___pyx_t_double_complex(const char *itemp, PyObject *obj) {
    __pyx_t_double_complex value = __Pyx_PyComplex_As___pyx_t_double_complex(obj);
    if (PyErr_Occurred())
        return 0;
    *(__pyx_t_double_complex *) itemp = value;
    return 1;
}

static CYTHON_INLINE PyObject* __Pyx_PyInt_From_enum__NPY_TYPES(enum NPY_TYPES value) {
    const enum NPY_TYPES neg_one = (enum NPY_TYPES) -1, const_zero = (enum NPY_TYPES) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(enum NPY_TYPES) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
        }
    } else {
        if (sizeof(enum NPY_TYPES) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(enum NPY_TYPES) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(enum NPY_TYPES),
                                     little, !is_unsigned);
    }
}

static int
__pyx_memviewslice_is_contig(const __Pyx_memviewslice *mvs,
                             char order, int ndim)
{
    int i, index, step, start;
    Py_ssize_t itemsize = mvs->memview->view.itemsize;
    if (order == 'F') {
        step = 1;
        start = 0;
    } else {
        step = -1;
        start = ndim - 1;
    }
    for (i = 0; i < ndim; i++) {
        index = start + step * i;
        if (mvs->suboffsets[index] >= 0 || mvs->strides[index] != itemsize)
            return 0;
        itemsize *= mvs->shape[index];
    }
    return 1;
}

static void
__pyx_get_array_memory_extents(__Pyx_memviewslice *slice,
                               void **out_start, void **out_end,
                               int ndim, size_t itemsize)
{
    char *start, *end;
    int i;
    start = end = slice->data;
    for (i = 0; i < ndim; i++) {
        Py_ssize_t stride = slice->strides[i];
        Py_ssize_t extent = slice->shape[i];
        if (extent == 0) {
            *out_start = *out_end = start;
            return;
        } else {
            if (stride > 0)
                end += stride * (extent - 1);
            else
                start += stride * (extent - 1);
        }
    }
    *out_start = start;
    *out_end = end + itemsize;
}
static int
__pyx_slices_overlap(__Pyx_memviewslice *slice1,
                     __Pyx_memviewslice *slice2,
                     int ndim, size_t itemsize)
{
    void *start1, *end1, *start2, *end2;
    __pyx_get_array_memory_extents(slice1, &start1, &end1, ndim, itemsize);
    __pyx_get_array_memory_extents(slice2, &start2, &end2, ndim, itemsize);
    return (start1 < end2) && (start2 < end1);
}

static __Pyx_memviewslice
__pyx_memoryview_copy_new_contig(const __Pyx_memviewslice *from_mvs,
                                 const char *mode, int ndim,
                                 size_t sizeof_dtype, int contig_flag,
                                 int dtype_is_object)
{
    __Pyx_RefNannyDeclarations
    int i;
    __Pyx_memviewslice new_mvs = { 0, 0, { 0 }, { 0 }, { 0 } };
    struct __pyx_memoryview_obj *from_memview = from_mvs->memview;
    Py_buffer *buf = &from_memview->view;
    PyObject *shape_tuple = NULL;
    PyObject *temp_int = NULL;
    struct __pyx_array_obj *array_obj = NULL;
    struct __pyx_memoryview_obj *memview_obj = NULL;
    __Pyx_RefNannySetupContext("__pyx_memoryview_copy_new_contig", 0);
    for (i = 0; i < ndim; i++) {
        if (from_mvs->suboffsets[i] >= 0) {
            PyErr_Format(PyExc_ValueError, "Cannot copy memoryview slice with "
                                           "indirect dimensions (axis %d)", i);
            goto fail;
        }
    }
    shape_tuple = PyTuple_New(ndim);
    if (unlikely(!shape_tuple)) {
        goto fail;
    }
    __Pyx_GOTREF(shape_tuple);
    for(i = 0; i < ndim; i++) {
        temp_int = PyInt_FromSsize_t(from_mvs->shape[i]);
        if(unlikely(!temp_int)) {
            goto fail;
        } else {
            PyTuple_SET_ITEM(shape_tuple, i, temp_int);
            temp_int = NULL;
        }
    }
    array_obj = __pyx_array_new(shape_tuple, sizeof_dtype, buf->format, (char *) mode, NULL);
    if (unlikely(!array_obj)) {
        goto fail;
    }
    __Pyx_GOTREF(array_obj);
    memview_obj = (struct __pyx_memoryview_obj *) __pyx_memoryview_new(
                                    (PyObject *) array_obj, contig_flag,
                                    dtype_is_object,
                                    from_mvs->memview->typeinfo);
    if (unlikely(!memview_obj))
        goto fail;
    if (unlikely(__Pyx_init_memviewslice(memview_obj, ndim, &new_mvs, 1) < 0))
        goto fail;
    if (unlikely(__pyx_memoryview_copy_contents(*from_mvs, new_mvs, ndim, ndim,
                                                dtype_is_object) < 0))
        goto fail;
    goto no_fail;
fail:
    __Pyx_XDECREF(new_mvs.memview);
    new_mvs.memview = NULL;
    new_mvs.data = NULL;
no_fail:
    __Pyx_XDECREF(shape_tuple);
    __Pyx_XDECREF(temp_int);
    __Pyx_XDECREF(array_obj);
    __Pyx_RefNannyFinishContext();
    return new_mvs;
}

static CYTHON_INLINE PyObject *
__pyx_capsule_create(void *p, CYTHON_UNUSED const char *sig)
{
    PyObject *cobj;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(p, sig, NULL);
#else
    cobj = PyCObject_FromVoidPtr(p, NULL);
#endif
    return cobj;
}

static CYTHON_INLINE char __Pyx_PyInt_As_char(PyObject *x) {
    const char neg_one = (char) -1, const_zero = (char) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(char) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(char, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (char) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (char) 0;
                case  1: __PYX_VERIFY_RETURN_INT(char, digit, digits[0])
                case 2:
                    if (8 * sizeof(char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) >= 2 * PyLong_SHIFT) {
                            return (char) (((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) >= 3 * PyLong_SHIFT) {
                            return (char) (((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) >= 4 * PyLong_SHIFT) {
                            return (char) (((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (char) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(char) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, unsigned long, PyLong_AsUnsignedLong(x))
            } else if (sizeof(char) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (char) 0;
                case -1: __PYX_VERIFY_RETURN_INT(char, sdigit, -(sdigit) digits[0])
                case  1: __PYX_VERIFY_RETURN_INT(char,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(char) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 2 * PyLong_SHIFT) {
                            return (char) -(((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 2 * PyLong_SHIFT) {
                            return (char) (((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(char) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 3 * PyLong_SHIFT) {
                            return (char) -(((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 3 * PyLong_SHIFT) {
                            return (char) (((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(char) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 4 * PyLong_SHIFT) {
                            return (char) -(((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 4 * PyLong_SHIFT) {
                            return (char) (((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(char) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, long, PyLong_AsLong(x))
            } else if (sizeof(char) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, PY_LONG_LONG, PyLong_AsLongLong(x))
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            char val;
            PyObject *v = __Pyx_PyNumber_Int(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (char) -1;
        }
    } else {
        char val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (char) -1;
        val = __Pyx_PyInt_As_char(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to char");
    return (char) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to char");
    return (char) -1;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_float(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 1,
                                                 &__Pyx_TypeInfo_float, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_short(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 1,
                                                 &__Pyx_TypeInfo_short, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds_long(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 1,
                                                 &__Pyx_TypeInfo_long, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds___pyx_t_float_complex(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 1,
                                                 &__Pyx_TypeInfo___pyx_t_float_complex, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_ds___pyx_t_double_complex(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 1,
                                                 &__Pyx_TypeInfo___pyx_t_double_complex, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static CYTHON_INLINE __Pyx_memviewslice __Pyx_PyObject_to_MemoryviewSlice_dsdsdsds_int(PyObject *obj) {
    __Pyx_memviewslice result = { 0, 0, { 0 }, { 0 }, { 0 } };
    __Pyx_BufFmt_StackElem stack[1];
    int axes_specs[] = { (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED), (__Pyx_MEMVIEW_DIRECT | __Pyx_MEMVIEW_STRIDED) };
    int retcode;
    if (obj == Py_None) {
        result.memview = (struct __pyx_memoryview_obj *) Py_None;
        return result;
    }
    retcode = __Pyx_ValidateAndInit_memviewslice(axes_specs, 0,
                                                 PyBUF_RECORDS, 4,
                                                 &__Pyx_TypeInfo_int, stack,
                                                 &result, obj);
    if (unlikely(retcode == -1))
        goto __pyx_fail;
    return result;
__pyx_fail:
    result.memview = NULL;
    result.data = NULL;
    return result;
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility",
            module_name, class_name);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling",
            module_name, class_name);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if CYTHON_COMPILING_IN_CPYTHON && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return __Pyx_NewRef(x);
  m = Py_TYPE(x)->tp_as_number;
#if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
